*DECK CREINT
      SUBROUTINE CREINT(IPCPO,NISO,DERIV,NBURN,KBURN,BURN0,BURN1,
     1     NGRP,NL,IMPX,HISO,ITY,CONC,ILEAK,TOTAL,ZNUG,SNUGF,CHI,
     2     OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,BURNUP,DENSIT,UPS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover and interpolate l_compo information according to burnup and
* extracted isotope density.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): A. Hebert
*
*Parameters: input
* IPCPO   pointer to l_compo information.
* NISO    1+number of extracted isotopes.
* DERIV   =.true.: derivative of macrolib info is computed with
*         respect to burn1.
* UPS     =.true.: no upscatering cross sections will be stored.
* NBURN   number of tabulated burnup steps.
* KBURN   =0: no burnup parameters; =1: use mw day/tonne of initial
*         heavy elements).
* BURN0   user defined initial burnup.
* BURN1   user defined final burnup:
*          if burn0=burn1, a simple interpolation is performed;
*          if burn0<burn1, a time-average calculation is performed.
* NGRP    number of energy groups.
* NL      number of legendre orders (=1 for isotropic scattering).
* IMPX    print parameter (=0 for no print).
* HISO    hollerith name information for extracted isotopes.
* ITY     =0: do not process the isotope; =1: use number density
*         stored in conc(i); =2: use number density stored in compo.
* CONC    user defined number density.
*
*Parameters: output
* TOTAL   total macroscopic x-sections.
* ZNUG    nu*fission macroscopic x-sections.
* SNUGF   fission macroscopic x-sections.
* CHI     fission spectrum.
* OVERV   reciprocal neutron velocities.
* DIFFX   x-directed diffusion coefficients.
* DIFFY   y-directed diffusion coefficients.
* DIFFZ   z-directed diffusion coefficients.
* H       h-factors (kappa*fission macroscopic x-sections).
* SCAT    scattering macroscopic x-sections.
* FLUX    integrated fluxes.
*
*Parameters: scratch
* BURNUP
* DENSIT
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPCPO,NISO,NGRP,IMPX,NBURN,KBURN,HISO(3*NISO),ITY(NISO),
     1 ILEAK
      REAL CONC(NISO),TOTAL(NGRP),ZNUG(NGRP),SNUGF(NGRP),CHI(NGRP),
     1 OVERV(NGRP),DIFFX(NGRP),DIFFY(NGRP),DIFFZ(NGRP),H(NGRP),
     2 SCAT(NL,NGRP,NGRP),FLUX(NGRP),BURNUP(NBURN),DENSIT(NISO),
     3 BURN0,BURN1
      LOGICAL DERIV,UPS
*----
*  LOCAL VARIABLES
*----
      INTEGER   IBASE(1)
      CHARACTER TEXT12*12
      COMMON    BASE(1)
      EQUIVALENCE(BASE,IBASE)
*----
*  CASE WITH NO BURNUP
*----
      IF(KBURN.EQ.0)THEN
        CALL LCMSIX(IPCPO,'BURN       1',1)
        CALL LCMGET(IPCPO,'ISOTOPESDENS',DENSIT)
        IF(DENSIT(1).NE.1.)CALL XABORT('@CREINT: DENSIT(1).NE.1.')
        DO I=2,NISO
          IF(ITY(I).EQ.0)THEN
            DENSIT(I)=0.
          ELSEIF(ITY(I).EQ.1)THEN
            DENSIT(I)=CONC(I)
          ELSEIF(ITY(I).NE.2)THEN
            CALL XABORT('@CREINT: INVALID VALUE OF ITY.')
          ENDIF
        ENDDO
        CALL SETARA(IBASE,NGRP,IIJJ)
        CALL SETARA(IBASE,NGRP,INJJ)
        CALL SETARA(IBASE,21+NL,INDXS)
        CALL SETARA(BASE,NISO,IENG)
        CALL SETARA(BASE,3*NGRP,IWORK1)
        CALL SETARA(BASE,NGRP*NGRP,IWORK2)
        CALL CREMAC(IPCPO,NISO,NGRP,NL,IMPX,HISO,DENSIT,ILEAK,TOTAL,
     1       ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,
     2       IBASE(IIJJ),IBASE(INJJ),BASE(IENG),IBASE(INDXS),
     3       BASE(IWORK1),BASE(IWORK2),UPS)
        CALL RLSARA(BASE(IWORK2))
        CALL RLSARA(BASE(IWORK1))
        CALL RLSARA(BASE(IENG))
        CALL RLSARA(IBASE(INDXS))
        CALL RLSARA(IBASE(INJJ))
        CALL RLSARA(IBASE(IIJJ))
        CALL LCMSIX(IPCPO,' ',2)
      ELSE
*----
*  CASE WITH BURNUP
*----
        CALL LCMGET(IPCPO,'BURNUP',BURNUP)
        TEXT12=' '
        IF(BURN0.EQ.BURN1)THEN
          DO I=1,NBURN
            IF(BURN0.EQ.BURNUP(I))THEN
              WRITE(TEXT12,'(4HBURN,4X,I4)') I
              GOTO 30
            ENDIF
          ENDDO
        ENDIF
   30   IF((TEXT12.NE.' ').AND.(.NOT.DERIV))THEN
*         BURN0=BURN1 IS A TABULATION POINT.
          CALL LCMSIX(IPCPO,TEXT12,1)
          CALL LCMGET(IPCPO,'ISOTOPESDENS',DENSIT)
          IF(DENSIT(1).NE.1.)CALL XABORT('@CREINT: DENSIT(1).NE.1.')
          DO I=2,NISO
            IF(ITY(I).EQ.0)THEN
              DENSIT(I)=0.
            ELSEIF(ITY(I).EQ.1)THEN
              DENSIT(I)=CONC(I)
            ELSEIF(ITY(I).NE.2)THEN
              CALL XABORT('@CREINT: INVALID VALUE OF ITY.')
            ENDIF
          ENDDO
          CALL SETARA(BASE,NGRP,IIJJ)
          CALL SETARA(BASE,NGRP,INJJ)
          CALL SETARA(IBASE,21+NL,INDXS)
          CALL SETARA(BASE,NISO,IENG)
          CALL SETARA(BASE,3*NGRP,IWORK1)
          CALL SETARA(BASE,NGRP*NGRP,IWORK2)
          CALL CREMAC(IPCPO,NISO,NGRP,NL,IMPX,HISO,DENSIT,ILEAK,TOTAL,
     1         ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,
     2         IBASE(IIJJ),IBASE(INJJ),BASE(IENG),IBASE(INDXS),
     3         BASE(IWORK1),BASE(IWORK2),UPS)
          CALL RLSARA(BASE(IWORK2))
          CALL RLSARA(BASE(IWORK1))
          CALL RLSARA(BASE(IENG))
          CALL RLSARA(IBASE(INDXS))
          CALL RLSARA(IBASE(INJJ))
          CALL RLSARA(IBASE(IIJJ))
          CALL LCMSIX(IPCPO,' ',2)
        ELSE
*         INTERPOLATION IS REQUIRED.
          CALL SETARA(BASE,NGRP*NBURN,ITOTAL)
          CALL SETARA(BASE,NGRP*NBURN,IZNUG)
          CALL SETARA(BASE,NGRP*NBURN,INUGF)
          CALL SETARA(BASE,NGRP*NBURN,ICHI)
          CALL SETARA(BASE,NGRP*NBURN,IOVERV)
          CALL SETARA(BASE,NGRP*NBURN,IDIFFX)
          CALL SETARA(BASE,NGRP*NBURN,IDIFFY)
          CALL SETARA(BASE,NGRP*NBURN,IDIFFZ)
          CALL SETARA(BASE,NGRP*NBURN,IH)
          CALL SETARA(BASE,NL*NGRP*NGRP*NBURN,ISCAT)
          CALL SETARA(BASE,NGRP*NBURN,IFLUX)
          CALL SETARA(BASE,NBURN,ITERP)
          CALL CREBUR(IPCPO,NISO,NGRP,NL,IMPX,HISO,DERIV,NBURN,
     1         BURN0,BURN1,BURNUP,ITY,CONC,ILEAK,TOTAL,ZNUG,SNUGF,
     2         CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,BASE(ITOTAL),
     3         BASE(IZNUG),BASE(INUGF),BASE(ICHI),BASE(IOVERV),
     4         BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),
     5         BASE(ISCAT),BASE(IFLUX),DENSIT,UPS,BASE(ITERP))
          CALL RLSARA(BASE(ITERP))
          CALL RLSARA(BASE(IFLUX))
          CALL RLSARA(BASE(ISCAT))
          CALL RLSARA(BASE(IH))
          CALL RLSARA(BASE(IDIFFZ))
          CALL RLSARA(BASE(IDIFFY))
          CALL RLSARA(BASE(IDIFFX))
          CALL RLSARA(BASE(IOVERV))
          CALL RLSARA(BASE(ICHI))
          CALL RLSARA(BASE(INUGF))
          CALL RLSARA(BASE(IZNUG))
          CALL RLSARA(BASE(ITOTAL))
        ENDIF
      ENDIF
      RETURN
      END
