*DECK CRERGR
      SUBROUTINE CRERGR(IPCPO,IPMAP,NISO,NGRP,NMIXT,NL,IBM,IMPX,IBTYP,
     1 DERIV,UPS,NBURN,BURNUP,DENSIT,ILEAK,TOTAL,ZNUG,SNUGF,CHI,OVERV,
     2 DIFFX,DIFFY,DIFFZ,H,SCAT,IJJ,NJJ,HISO,ITY,CONC,FMIX,BRN0,BRN1,
     3 NCH,NB,IVARTY,TERP,ZONEDP,TERPW)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform interpolation of fuel properties over the fuel lattice.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): A. Hebert, D. Sekki
*
*Parameters: input
* IPCPO   pointer to L_COMPO information.
* IPMAP   pointer to L_MAP information.
* NISO    1+number of extracted isotopes.
* NGRP    number of energy groups.
* NMIXT   number of material mixtures in the fuel-map macrolib.
* NL      number of legendre orders (=1 for isotropic scattering).
* IBM     mixture number to be treat.
* IMPX    printing index (=0 for no print).
* IBTYP   type of interpolation: =1 time-average; =2 instantaneous;
*         derivative with respect to a single exit burnup.
* DERIV   =.true.: derivative of macrolib info is computed with
*         respect to burn1.
* UPS     =.true.: no upscatering cross sections will be stored.
* NBURN   number of tabulated burnup steps.
* BURNUP  burnup tabulated values from compo file.
* HISO    hollerith name information for extracted isotopes.
* ITY     =0: do not process the isotope; =1: use number density
*         stored in conc(i); =2: use number density stored in compo.
* CONC    user defined number density.
* NCH     number of reactor channels.
* NB      number of fuel bundles per channel.
* FMIX    fuel mixture indices per fuel bundle.
* BRN0    contains either low burnup integration limits or 
*         instantaneous burnups per fuel bundle.
* BRN1    upper burnup integration limits per fuel bundle.
* IVARTY  index of the exit burnup used to compute derivatives. Used
*         if IBTYP=3.
*
*Parameters: output
* ILEAK   diffusion coefficient flag (=1: isotropic; =2: anisotropic).
* TOTAL   total macroscopic x-sections.
* ZNUG    nu*fission macroscopic x-sections.
* SNUGF   fission macroscopic x-sections.
* CHI     fission spectrum.
* OVERV   reciprocal neutron velocities.
* DIFFX   x-directed diffusion coefficients.
* DIFFY   y-directed diffusion coefficients.
* DIFFZ   z-directed diffusion coefficients.
* H       h-factors (kappa*fission macroscopic x-sections).
* SCAT    scattering macroscopic x-sections.
*
*Parameters: scratch
* TERP    interpolation factors.
* ZONEDP
* TERPW
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPCPO,IPMAP,NISO,IBTYP,IBM,NMIXT,NBURN,NGRP,NL,IMPX,
     1     NCH,NB,ILEAK,IJJ(NMIXT,NL,NGRP),NJJ(NMIXT,NL,NGRP),
     2     FMIX(NCH*NB),HISO(3*NISO),ITY(NISO),IVARTY,ZONEDP(NCH,NB)
      REAL CONC(NISO),DENSIT(NISO),TOTAL(NMIXT,NGRP),BURNUP(NBURN),
     1     SNUGF(NMIXT,NGRP),CHI(NMIXT,NGRP),OVERV(NMIXT,NGRP),
     2     DIFFX(NMIXT,NGRP),DIFFY(NMIXT,NGRP),DIFFZ(NMIXT,NGRP),
     3     BRN0(NCH*NB),BRN1(NCH*NB),TERP(NBURN),H(NMIXT,NGRP),
     3     SCAT(NMIXT,NL,NGRP,NGRP),ZNUG(NMIXT,NGRP),TERPW(NBURN)
      LOGICAL DERIV,UPS
*----
*  LOCAL VARIABLES
*----
      INTEGER IBASE(1)
      LOGICAL LCUBIC
      PARAMETER(LCUBIC=.TRUE.)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
      CALL XDRSET(BURNUP,NBURN,0.)
      CALL LCMGET(IPCPO,'BURNUP',BURNUP)
*----
*  FUEL-MAP INFORMATION
*----
      CALL SETARA(IBASE,NCH,IZONE)
      CALL CREGET(IPMAP,NCH,NB,IBTYP,IMPX,BRN0,BRN1,FMIX,
     1 IBASE(IZONE),ZONEDP,IVARTY,VARVAL)
      CALL RLSARA(IBASE(IZONE))
*----
*  CREATE BURNUP-DEPENDENT TABLE
*----
      CALL SETARA(BASE,NGRP,ITOTAL)
      CALL SETARA(BASE,NGRP,IZNUG)
      CALL SETARA(BASE,NGRP,INUGF)
      CALL SETARA(BASE,NGRP,ICHI)
      CALL SETARA(BASE,NGRP,IOVERV)
      CALL SETARA(BASE,NGRP,IDIFFX)
      CALL SETARA(BASE,NGRP,IDIFFY)
      CALL SETARA(BASE,NGRP,IDIFFZ)
      CALL SETARA(BASE,NGRP,IH)
      CALL SETARA(BASE,NL*NGRP*NGRP,ISCAT)
      CALL SETARA(BASE,NGRP,IFLUX)
*
      CALL XDRSET(BASE(ITOTAL),NGRP,0.)
      CALL XDRSET(BASE(IZNUG),NGRP,0.)
      CALL XDRSET(BASE(INUGF),NGRP,0.)
      CALL XDRSET(BASE(ICHI),NGRP,0.)
      CALL XDRSET(BASE(IOVERV),NGRP,0.)
      CALL XDRSET(BASE(IDIFFX),NGRP,0.)
      CALL XDRSET(BASE(IDIFFY),NGRP,0.)
      CALL XDRSET(BASE(IDIFFZ),NGRP,0.)
      CALL XDRSET(BASE(IH),NGRP,0.)
      CALL XDRSET(BASE(ISCAT),NL*NGRP*NGRP,0.)
      CALL XDRSET(BASE(IFLUX),NGRP,0.)
*
      CALL SETARA(BASE,NGRP*NBURN,IZTOTAL)
      CALL SETARA(BASE,NGRP*NBURN,IZZNUG)
      CALL SETARA(BASE,NGRP*NBURN,IZNUGF)
      CALL SETARA(BASE,NGRP*NBURN,IZCHI)
      CALL SETARA(BASE,NGRP*NBURN,IZOVERV)
      CALL SETARA(BASE,NGRP*NBURN,IZDIFX)
      CALL SETARA(BASE,NGRP*NBURN,IZDIFY)
      CALL SETARA(BASE,NGRP*NBURN,IZDIFZ)
      CALL SETARA(BASE,NGRP*NBURN,IZH)
      CALL SETARA(BASE,NL*NGRP*NGRP*NBURN,IZSCAT)
      CALL SETARA(BASE,NGRP*NBURN,IZFLUX)
*
      CALL XDRSET(BASE(IZTOTAL),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZZNUG),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZNUGF),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZCHI),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZOVERV),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZDIFX),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZDIFY),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZDIFZ),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZH),NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZSCAT),NL*NGRP*NGRP*NBURN,0.)
      CALL XDRSET(BASE(IZFLUX),NGRP*NBURN,0.)
*
      CALL CRETAB(IPCPO,NISO,NGRP,NL,IMPX,HISO,NBURN,BURNUP,ITY,CONC,
     1 BASE(ITOTAL),BASE(IZNUG),BASE(INUGF),BASE(ICHI),BASE(IOVERV),
     2 BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),BASE(ISCAT),
     3 BASE(IFLUX),ILEAK,BASE(IZTOTAL),BASE(IZZNUG),BASE(IZNUGF),
     4 BASE(IZCHI),BASE(IZOVERV),BASE(IZDIFX),BASE(IZDIFY),
     5 BASE(IZDIFZ),BASE(IZH),BASE(IZSCAT),BASE(IZFLUX),DENSIT,UPS)
*----
*  PERFORM INTERPOLATION
*----
      CALL SETARA(BASE,3*NBURN,IWK)
      DO 100 ICH=1,NCH
      DO 100 J=1,NB
      IB=(J-1)*NCH+ICH
      IF(FMIX(IB).EQ.IBM)THEN
        IF(IBTYP.EQ.1)THEN
*         TIME-AVERAGE
          BURN0=BRN0(IB)
          BURN1=BRN1(IB)
          IF(BURN0.GE.BURN1) CALL XABORT('@CRERGR: INVALID BURNUP LIMI'
     1    //'TS(1).')
          CALL ALTERI(LCUBIC,NBURN,BURNUP,BURN0,BURN1,TERP,BASE(IWK))
          DO 20 I=1,NBURN
   20     TERP(I)=TERP(I)/(BURN1-BURN0)
        ELSEIF(IBTYP.EQ.2)THEN
*         INSTANTANEOUS
          BURN0=BRN0(IB)
          BURN1=BURN0
          CALL ALTERP(LCUBIC,NBURN,BURNUP,BURN0,DERIV,TERP,BASE(IWK))
        ELSEIF(IBTYP.EQ.3)THEN
*         DERIVATIVE WITH RESPECT TO A SINGLE EXIT BURNUP. USE EQ.(3.3)
*         OF RICHARD CHAMBON'S THESIS.
          IF(ZONEDP(ICH,J).NE.0) THEN
            BURN0=BRN0(IB)
            BURN1=BRN1(IB)
            IF(BURN0.GE.BURN1) CALL XABORT('@CRERGR: INVALID BURNUP LI'
     1      //'MITS(2).')
            CALL ALTERI(LCUBIC,NBURN,BURNUP,BURN0,BURN1,TERPW,BASE(IWK))
            DO 30 I=1,NBURN
   30       TERP(I)=-TERPW(I)
            CALL ALTERP(LCUBIC,NBURN,BURNUP,BURN0,.FALSE.,TERPW,
     1      BASE(IWK))
            DO 40 I=1,NBURN
   40       TERP(I)=TERP(I)-TERPW(I)*BURN0
            CALL ALTERP(LCUBIC,NBURN,BURNUP,BURN1,.FALSE.,TERPW,
     1      BASE(IWK))
            DO 50 I=1,NBURN
   50       TERP(I)=(TERP(I)+TERPW(I)*BURN1)/(VARVAL*(BURN1-BURN0))
          ELSE
            CALL XDRSET(TERP,NBURN,0.0)
          ENDIF
        ENDIF
        IF(BURN1.GT.BURNUP(NBURN))THEN
          WRITE(*,*)'@CRERGR: BURN1 VALUE  :',BURN1
          WRITE(*,*)'@CRERGR: BURNUP LIMIT :',BURNUP(NBURN)
          CALL XABORT('@CRERGR: INTERPOLATION IS OUT OF BURNUP LIMIT.')
        ENDIF
*
        IF((IBTYP.EQ.3).AND.(ZONEDP(ICH,J).EQ.0)) THEN
           DO 60 JGR=0,NGRP-1
           BASE(ITOTAL+JGR)=0.0
           BASE(IZNUG+JGR)=0.0
           BASE(INUGF+JGR)=0.0
           BASE(ICHI+JGR)=0.0
           BASE(IOVERV+JGR)=0.0
           BASE(IDIFFX+JGR)=0.0
           BASE(IDIFFY+JGR)=0.0
           BASE(IDIFFZ+JGR)=0.0
           BASE(IH+JGR)=0.0
           BASE(IFLUX+JGR)=0.0
           DO 60 IGR=1,NGRP
           DO 60 IL=1,NL
   60      BASE(ISCAT+NL*(JGR*NGRP+IGR-1)+IL-1)=0.0
        ELSE
           CALL CREITP(NGRP,NL,NBURN,TERP,BASE(ITOTAL),BASE(IZNUG),
     1     BASE(INUGF),BASE(ICHI),BASE(IOVERV),BASE(IDIFFX),
     2     BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),BASE(ISCAT),BASE(IFLUX),
     3     BASE(IZTOTAL),BASE(IZZNUG),BASE(IZNUGF),BASE(IZCHI),
     4     BASE(IZOVERV),BASE(IZDIFX),BASE(IZDIFY),BASE(IZDIFZ),
     5     BASE(IZH),BASE(IZSCAT),BASE(IZFLUX))
        ENDIF
*       DATA STORAGE
        DO 70 JGR=1,NGRP
        TOTAL(IB,JGR)=BASE(ITOTAL+JGR-1)
        ZNUG(IB,JGR)=BASE(IZNUG+JGR-1)
        SNUGF(IB,JGR)=BASE(INUGF+JGR-1)
        CHI(IB,JGR)=BASE(ICHI+JGR-1)
        OVERV(IB,JGR)=BASE(IOVERV+JGR-1)
        DIFFX(IB,JGR)=BASE(IDIFFX+JGR-1)
        DIFFY(IB,JGR)=BASE(IDIFFY+JGR-1)
        DIFFZ(IB,JGR)=BASE(IDIFFZ+JGR-1)
        H(IB,JGR)=BASE(IH+JGR-1)
        DO 70 IGR=1,NGRP
        DO 70 IL=1,NL
   70   SCAT(IB,IL,IGR,JGR)=BASE(ISCAT+NL*((JGR-1)*NGRP+IGR-1)+IL-1)
*       JGR IS THE SECONDARY GROUP.
        DO 80 JGR=1,NGRP
        DO 80 IL=1,NL
        IGMIN=JGR
        IGMAX=JGR
        DO IGR=NGRP,1,-1
          IF(SCAT(IB,IL,IGR,JGR).NE.0.)THEN
            IGMIN=MIN(IGMIN,IGR)
            IGMAX=MAX(IGMAX,IGR)
          ENDIF
        ENDDO
        IJJ(IB,IL,JGR)=IGMAX
   80   NJJ(IB,IL,JGR)=IGMAX-IGMIN+1
      ENDIF
  100 CONTINUE
      CALL RLSARA(BASE(IWK))
*
      CALL RLSARA(BASE(IFLUX))
      CALL RLSARA(BASE(ISCAT))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(BASE(IDIFFZ))
      CALL RLSARA(BASE(IDIFFY))
      CALL RLSARA(BASE(IDIFFX))
      CALL RLSARA(BASE(IOVERV))
      CALL RLSARA(BASE(ICHI))
      CALL RLSARA(BASE(INUGF))
      CALL RLSARA(BASE(IZNUG))
      CALL RLSARA(BASE(ITOTAL))
*
      CALL RLSARA(BASE(IZFLUX))
      CALL RLSARA(BASE(IZSCAT))
      CALL RLSARA(BASE(IZH))
      CALL RLSARA(BASE(IZDIFZ))
      CALL RLSARA(BASE(IZDIFY))
      CALL RLSARA(BASE(IZDIFX))
      CALL RLSARA(BASE(IZOVERV))
      CALL RLSARA(BASE(IZCHI))
      CALL RLSARA(BASE(IZNUGF))
      CALL RLSARA(BASE(IZZNUG))
      CALL RLSARA(BASE(IZTOTAL))
      RETURN
      END
