*DECK CRETAB
      SUBROUTINE CRETAB(IPCPO,NISO,NGRP,NL,IMPX,HISO,NBURN,BURNUP,
     1 ITY,CONC,TOTAL,ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,
     2 FLUX,ILEAK,ZTOTAL,ZZNUG,ZNUGF,ZCHI,ZOVERV,ZDIFFX,ZDIFFY,ZDIFFZ,
     3 ZH,ZSCAT,ZFLUX,DENSIT,UPS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* create burnup dependent table with the extracted isotope.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): A. Hebert
*
*Parameters: input
* IPCPO   pointer to l_compo information.
* NISO    1+number of extracted isotopes.
* NGRP    number of energy groups.
* NL      number of legendre orders (=1 for isotropic scattering).
* IMPX    print parameter (=0 for no print).
* HISO    hollerith name information for extracted isotopes.
* NBURN   number of tabulated burnup steps
* BURNUP  burnup tabulation points.
* ITY     =0: do not process the isotope; =1: use number density
*         stored in conc(i); =2: use number density stored in compo.
* CONC    user defined number density.
*
*Parameters: output
* ILEAK   diffusion coefficient flag (=1: isotropic; =2: anisotropic).
* ZTOTAL  burnup dependent total macroscopic x-sections
* ZZNUG   burnup dependent nu*fission macroscopic x-sections.
* ZNUGF   burnup dependent fission macroscopic x-sections.
* ZCHI    burnup dependent fission spectrum.
* ZOVERV  burnup dependent reciprocal neutron velocities.
* ZDIFFX  burnup dependent x-directed diffusion coefficients.
* ZDIFFY  burnup dependent y-directed diffusion coefficients.
* ZDIFFZ  burnup dependent z-directed diffusion coefficients.
* ZH      burnup dependent h-factors (kappa*fission macroscopic
*         x-sections).
* ZSCAT   burnup dependent scattering macroscopic x-sections.
* ZFLUX   burnup dependent integrated flux.
*
*Parameters: scratch
* TOTAL   total macroscopic x-sections.
* ZNUG    nu*fission macroscopic x-sections.
* SNUGF   fission macroscopic x-sections.
* CHI     fission spectrum.
* OVERV   reciprocal neutron velocities.
* DIFFX   x-directed diffusion coefficients.
* DIFFY   y-directed diffusion coefficients.
* DIFFZ   z-directed diffusion coefficients.
* H       h-factors (kappa*fission macroscopic x-sections).
* SCAT    scattering macroscopic x-sections.
* FLUX    integrated flux.
* DENSIT  isotopic number densities.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPCPO,NISO,NGRP,NL,IMPX,NBURN,HISO(3*NISO),ITY(NISO),
     1 ILEAK
      REAL BURNUP(NBURN),CONC(NISO),TOTAL(NGRP),ZNUG(NGRP),CHI(NGRP),
     1 OVERV(NGRP),DIFFX(NGRP),DIFFY(NGRP),DIFFZ(NGRP),H(NGRP),
     2 SCAT(NL,NGRP,NGRP),SNUGF(NGRP),FLUX(NGRP),ZTOTAL(NBURN,NGRP),
     3 ZZNUG(NBURN,NGRP),ZCHI(NBURN,NGRP),ZOVERV(NBURN,NGRP),
     4 ZDIFFX(NBURN,NGRP),ZDIFFY(NBURN,NGRP),ZDIFFZ(NBURN,NGRP),
     5 ZH(NBURN,NGRP),ZSCAT(NBURN,NL,NGRP,NGRP),DENSIT(NISO),
     6 ZNUGF(NBURN,NGRP),ZFLUX(NBURN,NGRP)
      LOGICAL UPS
*----
*  LOCAL VARIABLES
*----
      CHARACTER TEXT12*12
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE,IBASE)
*----
*  RECOVER MACROSCOPIC X-SECTION INFO FROM BURNUP DIRECTORIES
*----
      CALL SETARA(IBASE,NGRP,IIJJ)
      CALL SETARA(IBASE,NGRP,INJJ)
      CALL SETARA(BASE,NISO,IENG)
      CALL SETARA(IBASE,(21+NL),INDXS)
      CALL SETARA(BASE,3*NGRP,IWORK1)
      CALL SETARA(BASE,NGRP*NGRP,IWORK2)
      DO 20 IBURN=1,NBURN
      WRITE(TEXT12,'(4HBURN,4X,I4)') IBURN
      CALL LCMSIX(IPCPO,TEXT12,1)
      CALL LCMGET(IPCPO,'ISOTOPESDENS',DENSIT)
      IF(DENSIT(1).NE.1.)CALL XABORT('@CRETAB: DENSIT(1).NE.1.')
      DO I=2,NISO
        IF(ITY(I).EQ.0)THEN
          DENSIT(I)=0.
        ELSEIF(ITY(I).EQ.1)THEN
          DENSIT(I)=CONC(I)
        ELSEIF(ITY(I).NE.2)THEN
          CALL XABORT('@CRETAB: INVALID VALUE OF ITY.')
        ENDIF
      ENDDO
      CALL CREMAC(IPCPO,NISO,NGRP,NL,IMPX,HISO,DENSIT,ILEAK,TOTAL,
     1     ZNUG,SNUGF,CHI,OVERV,DIFFX,DIFFY,DIFFZ,H,SCAT,FLUX,
     2     IBASE(IIJJ),IBASE(INJJ),BASE(IENG),IBASE(INDXS),
     3     BASE(IWORK1),BASE(IWORK2),UPS)
      CALL LCMSIX(IPCPO,' ',2)
      DO 21 JGR=1,NGRP
        ZTOTAL(IBURN,JGR)=TOTAL(JGR)
        ZZNUG(IBURN,JGR)=ZNUG(JGR)
        ZNUGF(IBURN,JGR)=SNUGF(JGR)
        ZCHI(IBURN,JGR)=CHI(JGR)
        ZOVERV(IBURN,JGR)=OVERV(JGR)
        ZDIFFX(IBURN,JGR)=DIFFX(JGR)
        ZDIFFY(IBURN,JGR)=DIFFY(JGR)
        ZDIFFZ(IBURN,JGR)=DIFFZ(JGR)
        ZH(IBURN,JGR)=H(JGR)
        ZFLUX(IBURN,JGR)=FLUX(JGR)
        DO 23 IGR=1,NGRP
        DO 23 IL=1,NL
   23   ZSCAT(IBURN,IL,IGR,JGR)=SCAT(IL,IGR,JGR)
   21 CONTINUE
   20 CONTINUE
*   
      CALL RLSARA(BASE(IWORK2))
      CALL RLSARA(BASE(IWORK1))
      CALL RLSARA(BASE(IENG))
      CALL RLSARA(IBASE(INDXS))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(IBASE(IIJJ))
      RETURN
      END
