*DECK DETCDRV
      SUBROUTINE DETCDRV(IPDET,NGRP,NEL,NUN,NX,NY,NZ,SPEC,MESHX,MESHY,
     1           MESHZ,KEYF,FLUX,IPRT,KC,DT,LHEX,LSIMEX,LNORM,VNORM,
     2           GEOTYP,LPARAB)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for the module DETECT:
*
*Copyright:
* Copyright (C) 2010 Ecole Polytechnique de Montreal.
*
*Author(s): E. Varin, M. Guyot
*
*Parameters:
* IPDET  pointer to the library object
* NGRP   number of energy groups
* NEL    number of finite elements
* NUN    number of unknowns
* NX     number of x mesh-splitted elements 
* NY     number of y mesh-splitted elements 
* NZ     number of z mesh-splitted elements
* SPEC   spectral information
* KEYF   keyflux recover from L_TRACk object
* FLUX   flux for each mesh-splitted elements
* IPRT   printing index
* KC     calculation type reference
* DT     time step
* LHEX   =.TRUE. if hexagonal detectors are present
* LSIMEX =.TRUE. if keyword SIMEX is present
* LNORM  =.TRUE. if keyword NORM is present
* VNORM  real used for normalization
* GEOTYP type of geometry
* LPARAB =.TRUE. if parabolic interpolation is performed
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPDET,IPRT,KC,NGRP,NEL,NX,NY,NZ,SPEC(NGRP),KEYF(NEL),
     1        GEOTYP,NUN
      LOGICAL LHEX,LSIMEX,LNORM,LPARAB
      REAL    FLUX(NUN,NGRP),DT,MESHX(NX+1),MESHY(NY+1),MESHZ(NZ+1),
     1        VNORM
*----
*  LOCAL VARIABLES
*----
      INTEGER   NSTATE,IOUT
      PARAMETER (NSTATE=40,IOUT=6) 
      INTEGER   IBASE(1),ILONG,ITYLCM,INFO(2),NREP,IREP,IFRACT,INVCST,
     1          IPDD,IAPD,IBPD,IHEX,ITHEX,NHEX,J,IXCT,IYCT,IZCT,IDCOR
      REAL      BASE,DEVPOS(6),PLNL,REF,RESP,VLAMDA,XMULT,TLG
      CHARACTER NXTYP*12,FIRST*12,NXDET*12,FIRST1*12
      LOGICAL   LREGUL
      COMMON  BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))

      IF(IPRT.GE.1) THEN
        IF(KC.EQ.0) THEN
          WRITE(IOUT,*) 'DETECT: CALCULATION TYPE REFERENCE, KC = ',KC
        ELSE
          WRITE(IOUT,*) 'DETECT: CALCULATION TYPE NORMAL, KC = ',KC
        ENDIF
        WRITE(IOUT,*) 'DETECT: TIME STEP USED, DT = ',DT
      ENDIF
      CALL LCMSIX(IPDET,' ',0)
      NXTYP = ' '
      CALL LCMNXT(IPDET,NXTYP)
      FIRST = NXTYP
 10   CALL LCMNXT(IPDET,NXTYP)
      CALL LCMLEN(IPDET,NXTYP,ILONG,ITYLCM)
      IF (ITYLCM.EQ.0) THEN
        CALL LCMSIX(IPDET,NXTYP,1)
        CALL LCMGET(IPDET,'INFORMATION',INFO)
        CALL LCMGET(IPDET,'SPECTRAL',SPEC)
        NREP = INFO(2)
        CALL SETARA(BASE,NREP,IREP)
        IF( NXTYP(1:5).EQ.'PLATN' )THEN
          CALL SETARA(BASE,NREP-1,IFRACT)
          CALL LCMGET(IPDET,'FRACTION',BASE(IFRACT))
          CALL SETARA(BASE,NREP-2,INVCST)
          CALL LCMGET(IPDET,'INV-CONST',BASE(INVCST))
          CALL SETARA(BASE,NREP-2,IPDD)
          CALL SETARA(BASE,NREP-2,IAPD)
          CALL SETARA(BASE,NREP-2,IBPD)
        ENDIF
        NXDET = ' '
        CALL LCMNXT(IPDET,NXDET)
        FIRST1 = NXDET
 20     CALL LCMNXT(IPDET,NXDET)
        CALL LCMLEN(IPDET,NXDET,ILONG,ITYLCM)
        IF (ITYLCM.EQ.0) THEN
           IF(IPRT.GT.3) WRITE(IOUT,*) 'NAME DETECTOR ',NXDET
           CALL LCMSIX(IPDET,NXDET,1)
           IHEX = 1
           IF(LHEX) THEN
             CALL LCMLEN(IPDET,'NHEX',NHEX,ITHEX)
             IF(NHEX.EQ.0) CALL XABORT('@DETCDRV: HEXAGON NUMBERS'
     +             //' NOT PRESENT IN DETECT')
             CALL SETARA(IBASE,NHEX,IHEX)
             CALL LCMGET(IPDET,'NHEX',IBASE(IHEX))
           ENDIF
           CALL LCMGET(IPDET,'POSITION',DEVPOS)
           CALL LCMGET(IPDET,'RESPON',BASE(IREP))
            IF(LSIMEX.AND.NXTYP.EQ.'VANAD_REGUL') THEN
              CALL SETARA(BASE,NX,IXCT)
              CALL SETARA(BASE,NY,IYCT)
              CALL SETARA(BASE,NZ,IZCT)
              CALL SETARA(BASE,3,IDCOR)
              CALL DETINT(NX,NY,NZ,NEL,NUN,LPARAB,MESHX,MESHY,MESHZ,
     +             KEYF,FLUX,BASE(IXCT),BASE(IYCT),BASE(IZCT),
     +             BASE(IDCOR),NGRP,DEVPOS,RESP,IPRT)
              CALL RLSARA(BASE(IXCT))
              CALL RLSARA(BASE(IYCT))
              CALL RLSARA(BASE(IZCT))
              CALL RLSARA(BASE(IDCOR))
            ELSE
              CALL DETFLU(GEOTYP,LHEX,NX,NY,NZ,NEL,NUN,MESHX,MESHY,
     +                   MESHZ,KEYF,FLUX,NGRP,SPEC,DEVPOS,NHEX,
     +                   IBASE(IHEX),RESP,IPRT)
            ENDIF
*----
*  DETECTOR RESPONSE CALCULATION
*----
            IF(.NOT.LNORM)THEN
              PLNL = BASE(IREP)
              REF  = BASE(IREP+1)
              IF(NXTYP.EQ.'VANAD_REGUL')THEN
*----
*  VANADIUM RESPONSE CALCULATION
*----
                IF(LSIMEX) THEN
                  REF = RESP
                ELSE
                  IF(KC.EQ.1) THEN
                    VLAMDA = 1./225.
                    XMULT = 1.0 + VLAMDA*DT
                    XMULT = 1.0/XMULT
                    RESP = XMULT*(PLNL+DT*VLAMDA*RESP)
                    REF = PLNL
                  ELSE
                    REF = RESP
                  ENDIF
                ENDIF
              ELSEIF(NXTYP(1:5).EQ.'PLATN')THEN
*----
*  PLATINIUM RESPONSE CALCULATION
*----
                LREGUL = .FALSE.
                DO 30 J=1,NREP-2
                  BASE(IPDD+J-1) = BASE(IREP+J+1)
  30            CONTINUE
                IF(NXTYP.EQ.'PLATN_REGUL')THEN
                  LREGUL = .TRUE.
                ENDIF
                CALL DETPLAT(DT,RESP,REF,KC,BASE(IPDD),LREGUL,
     +            BASE(IFRACT),BASE(INVCST),NREP-2,BASE(IAPD),
     +            BASE(IBPD))
                DO 40 J=1,NREP-2
                  BASE(IREP+J+1) = BASE(IPDD+J-1)
  40            CONTINUE
              ELSEIF (NXTYP(1:5).EQ.'CHION') THEN
*----
*  LECTURE DE CHAMBRES D'ION
*----
                IF(NREP.NE.3)CALL XABORT('@DETCDRV: ION CHAMBERS MUST '
     +            //'HAVE THREE STORED VALUES FOR RESPONSES')
                IF (KC.EQ.1) THEN
                  REF = BASE(IREP+2)
                  RESP = LOG10(RESP/REF)
                  TLG = (RESP-PLNL)/DT
                  REF = TLG
                ELSE
                  BASE(IREP+2) = RESP
                  RESP =LOG10(RESP/BASE(IREP+2))
                ENDIF
              ENDIF
            ELSE
              REF = VNORM/RESP
              RESP = VNORM
            ENDIF
*----
*  DETECTOR RESPONSE STORAGE
*----
           BASE(IREP) = RESP
           BASE(IREP+1)  = REF
           IF(IPRT.GT.4) WRITE(6,*) 'RESP, REF ',RESP, REF
           CALL LCMPUT(IPDET,'RESPON',NREP,2,BASE(IREP))
           CALL LCMSIX(IPDET,' ',2)
        ENDIF
        IF(LHEX) CALL RLSARA(IBASE(IHEX))
        IF(NXDET.EQ.FIRST1) GOTO 45
        GOTO 20
   45   CALL LCMSIX(IPDET,' ',2)
        CALL RLSARA(BASE(IREP))
        IF(NXTYP(1:5).EQ.'PLATN')THEN
          CALL RLSARA(BASE(IFRACT))
          CALL RLSARA(BASE(INVCST))
          CALL RLSARA(BASE(IPDD))
          CALL RLSARA(BASE(IAPD))
          CALL RLSARA(BASE(IBPD))
        ENDIF
      ENDIF
      IF (NXTYP.EQ.FIRST) GOTO 50
        GOTO 10
  50  CONTINUE
      RETURN
      END
