*DECK DEVGET
      SUBROUTINE DEVGET(KPDEV,NEL,NMIX,NTOT,MIX,ID,XMAX,XMIN,YMAX,
     1  YMIN,ZMAX,ZMIN,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read the specification for a given rod from the input file.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input/output
* KPDEV  pointer to DEV_ROD directory for device information.
* NEL    total number of mesh-splitted volumes.
* NMIX   maximum number of material mixtures.
* NTOT   total number of all (material and virtual) mixtures.
* MIX    mixture index of all (material and virtual) mixtures.
* ID     identification number of the current rod.
* XMAX   maximum value of x-coordinate.
* XMIN   minimum value of x-coordinate.
* YMAX   maximum value of y-coordinate.
* YMIN   minimum value of y-coordinate.
* ZMAX   maximum value of z-coordinate.
* ZMIN   minimum value of z-coordinate.
* IMPX   printing index (=0 for no print).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER KPDEV,NEL,NMIX,NTOT,MIX(NTOT),ID,IMPX
      REAL XMAX,XMIN,YMAX,YMIN,ZMAX,ZMIN
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER DMIX(2),INAME(3)
      REAL MAXPOS(6),RODPOS(6),LENGTH,LEVEL
      DOUBLE PRECISION DFLOT
      CHARACTER TEXT*12,RNAME*12,AXIS,FROM*2
*----
*  ROD NAME
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.NE.'ROD-NAME')CALL XABORT('@DEVGET: KEYWORD ROD-NAME'
     1 //' EXPECTED.')
      CALL REDGET(ITYP,NITMA,FLOT,RNAME,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@DEVGET: ROD NAME EXPECTED.')
*----
*  FULL-INSERTION POSITION
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.NE.'MAXPOS')CALL XABORT('@DEVGET: KEYWORD MAXPOS EXP'
     1 //'ECTED.')
      DO I=1,6
        CALL REDGET(ITYP,NITMA,MAXPOS(I),TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@DEVGET: REAL FOR MAXPOS EXPECTED.')
      ENDDO
*----
*  CHECK ROD POSITION
*----
      IF(MAXPOS(2).LT.MAXPOS(1))CALL XABORT('@DEVGET: WRONG X '
     1 //'ROD COORDINATES: X- > X+')
      IF(MAXPOS(1).LT.XMIN)CALL XABORT('@DEVGET: WRONG X- VALUE.')
      IF(MAXPOS(2).GT.XMAX)CALL XABORT('@DEVGET: WRONG X+ VALUE.')
*
      IF(MAXPOS(4).LT.MAXPOS(3))CALL XABORT('@DEVGET: WRONG Y '
     1 //'ROD COORDINATES: Y- > Y+')
      IF(MAXPOS(3).LT.YMIN)CALL XABORT('@DEVGET: WRONG Y- VALUE.')
      IF(MAXPOS(4).GT.YMAX)CALL XABORT('@DEVGET: WRONG Y+ VALUE.')
*
      IF(MAXPOS(6).LT.MAXPOS(5))CALL XABORT('@DEVGET: WRONG Z '
     1 //'ROD COORDINATES: Z- > Z+')
      IF(MAXPOS(5).LT.ZMIN)CALL XABORT('@DEVGET: WRONG Z- VALUE.')
      IF(MAXPOS(6).GT.ZMAX)CALL XABORT('@DEVGET: WRONG Z+ VALUE.')
*----
*  INSERTION LEVEL
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.NE.'LEVEL')CALL XABORT('@DEVGET: KEYWORD LEVEL EXPECTED')
      CALL REDGET(ITYP,NITMA,LEVEL,TEXT,DFLOT)
      IF(ITYP.NE.2)CALL XABORT('@DEVGET: REAL FOR LEVEL EXPECTED.')
      IF(LEVEL.GT.1.)CALL XABORT('@DEVGET: WRONG LEVEL VALUE > 1.')
      IF(LEVEL.LT.0.)CALL XABORT('@DEVGET: WRONG LEVEL VALUE < 0.')
*----
*  MOVEMENT AXIS
*----
      LENGTH=0.
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.NE.'AXIS')CALL XABORT('@DEVGET: KEYWORD AXIS EXPECTED.')
      CALL REDGET(ITYP,NITMA,FLOT,AXIS,DFLOT)
      IF(AXIS.NE.'X')THEN
      IF(AXIS.NE.'Y')THEN
      IF(AXIS.NE.'Z')THEN
       CALL XABORT('@DEVGET: X, Y OR Z EXPECTED FOR AXIS.')
      ELSE
       IAXIS=3
       LENGTH=MAXPOS(6)-MAXPOS(5)
      ENDIF
      ELSE
       IAXIS=2
       LENGTH=MAXPOS(4)-MAXPOS(3)
      ENDIF
      ELSE
       IAXIS=1
       LENGTH=MAXPOS(2)-MAXPOS(1)
      ENDIF
      IF(LENGTH.EQ.0.)CALL XABORT('@DEVGET: WRONG ROD LENGTH = 0.')
*----
*  INSERTION DIRECTION
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.NE.'FROM')CALL XABORT('@DEVGET: KEYWORD FROM EXPECTED.')
      CALL REDGET(ITYP,NITMA,FLOT,FROM,DFLOT)
      IF(FROM.EQ.'H+')THEN
        ITOP=1
      ELSEIF(FROM.EQ.'H-')THEN
        ITOP=-1
      ELSE
        CALL XABORT('@DEVGET: KEYWORD H+ OR H- EXPECTED.')
      ENDIF
*----
*  CURRENT ROD POSITION
*----
      DELH=LEVEL*LENGTH
      DO 10 I=1,6
   10 RODPOS(I)=MAXPOS(I)
      IF(IAXIS.EQ.1)THEN
        IF(ITOP.EQ.1)THEN
          RODPOS(1)=RODPOS(2)-DELH
        ELSE
          RODPOS(2)=RODPOS(1)+DELH
        ENDIF
      ELSEIF(IAXIS.EQ.2)THEN
        IF(ITOP.EQ.1)THEN
          RODPOS(3)=RODPOS(4)-DELH
        ELSE
          RODPOS(4)=RODPOS(3)+DELH
        ENDIF
      ELSEIF(IAXIS.EQ.3)THEN
        IF(ITOP.EQ.1)THEN
          RODPOS(5)=RODPOS(6)-DELH
        ELSE
          RODPOS(6)=RODPOS(5)+DELH
        ENDIF
      ENDIF
*----
*  ROD INSERTION SPEED
*----
      SPEED=0.
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@DEVGET: CHARACTER DATA EXPECTED(1).')
      IF(TEXT.NE.'SPEED')GOTO 20
      CALL REDGET(ITYP,NITMA,SPEED,TEXT,DFLOT)
      IF(ITYP.NE.2)CALL XABORT('@DEVGET: REAL FOR SPEED EXPECTED.')
      IF(SPEED.LT.0.)CALL XABORT('@DEVGET: WRONG SPEED VALUE < 0.')
*----
*  ROD INSERTION TIME
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@DEVGET: CHARACTER DATA EXPECTED(2).')
   20 TIME=0.
      IF(TEXT.NE.'TIME')GOTO 30
      CALL REDGET(ITYP,NITMA,TIME,TEXT,DFLOT)
      IF(ITYP.NE.2)CALL XABORT('@DEVGET: REAL FOR TIME EXPECTED.')
      IF(TIME.LT.0.)CALL XABORT('@DEVGET: WRONG TIME VALUE < 0.')
*----
*  ROD MIXTURES
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
   30 IF(TEXT.NE.'DMIX')CALL XABORT('@DEVGET: KEYWORD DMIX EXPECTED.')
      NMAX=2
      DO I=1,NMAX
        CALL REDGET(ITYP,DMIX(I),FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@DEVGET: INTEGER FOR DMIX EXPECTED.')
        MIX(NEL+(ID-1)*NMAX+I)=DMIX(I)
        DMIX(I)=NMIX+(ID-1)*NMAX+I
      ENDDO
*----
*  STORE ROD DATA
*----
      CALL LCMPUT(KPDEV,'ROD-ID',1,1,ID)
      READ(RNAME,'(3A4)') (INAME(I),I=1,3)
      CALL LCMPUT(KPDEV,'ROD-NAME',3,3,INAME)
      CALL LCMPUT(KPDEV,'MAX-POS',6,2,MAXPOS)
      CALL LCMPUT(KPDEV,'FROM',1,1,ITOP)
      CALL LCMPUT(KPDEV,'AXIS',1,1,IAXIS)
      CALL LCMPUT(KPDEV,'LENGTH',1,2,LENGTH)
      CALL LCMPUT(KPDEV,'LEVEL',1,2,LEVEL)
      CALL LCMPUT(KPDEV,'ROD-POS',6,2,RODPOS)
      CALL LCMPUT(KPDEV,'ROD-MIX',NMAX,1,DMIX)
      CALL LCMPUT(KPDEV,'SPEED',1,2,SPEED)
      CALL LCMPUT(KPDEV,'TIME',1,2,TIME)
*
      IF(IMPX.GT.1)WRITE(IOUT,1000)RNAME,MAXPOS(1),MAXPOS(3),MAXPOS(5),
     1  MAXPOS(2),MAXPOS(4),MAXPOS(6),LENGTH,FROM,AXIS,LEVEL,RODPOS(1),
     2  RODPOS(3),RODPOS(5),RODPOS(2),RODPOS(4),RODPOS(6),SPEED,TIME
      RETURN
*
 1000 FORMAT(/5X,'ROD NAME',1X,'=>',1X,A/
     1  /5X,'FULL-INSERTED ROD POSITION :',
     2   4X,'X-',F10.4,5X,'Y-',F10.4,5X,'Z-',F10.4/
     3  37X,'X+',F10.4,5X,'Y+',F10.4,5X,'Z+',F10.4/
     4  /5X,'ROD LENGTH =',F9.4/
     5  /5X,'INSERTION FROM : ',A2,5X,'MOVEMENT AXIS : ',A1/
     6  /5X,'INSERTION LEVEL =',F8.4/
     7  /5X,'CURRENT ROD POSITION :',
     8   5X,'X-',F10.4,5X,'Y-',F10.4,5X,'Z-',F10.4/
     9  32X,'X+',F10.4,5X,'Y+',F10.4,5X,'Z+',F10.4/
     1  /5X,'INSERTION SPEED =',1X,E10.4,
     2  /5X,'INSERTION TIME  =',1X,E10.4/)
      END
