*DECK FLPOW
      SUBROUTINE FLPOW(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute and print power and flux distributions over the reactor core.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*  Modified 15/07/10 : Creation of L_FLUX object to be used by
*  module DETECT:, M. Guyot
*  
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create type(L_POWER);
*         HENTRY(2): optional read-only type(L_POWER);
*         HENTRY(3): read-only type(L_FLUX);
*         HENTRY(4): read-only type(L_TRACK);
*         HENTRY(5): optional read-only type(L_MAP);
*         HENTRY(6): optional read-only type(L_MATEX);
*         HENTRY(7): optional read-only type(L_MACROLIB).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      INTEGER IBASE(1),KCHAR(3)
      CHARACTER HSIGN*12,TEXT*12
      LOGICAL LNEW,LMAP,LFLX,LRAT,LPOW,LFSTH,LFLU,LNRM,LBUN
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DOUBLE PRECISION DFLOT
*----
*  PARAMETER VALIDATION
*----
      LFLU=.FALSE.
      IF(NENTRY.LT.4)CALL XABORT('@FLPOW: PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))CALL XABORT('@FLPOW'
     1 //': LCM OBJECT FOR L_POWER EXPECTED AT LHS.')
      IF(JENTRY(1).NE.0)CALL XABORT('@FLPOW: CREATE MODE FOR L_POW'
     1 //'ER EXPECTED AT LHS.')
      IPPOW=KENTRY(1)
      IF(JENTRY(2).EQ.0)THEN
        LFLU=.TRUE.
        IPNFX=KENTRY(2)
      ENDIF
      IPFLX=0
      IPTRK=0
      IPMTX=0
      IPMAP=0
      IPMAC=0
      LNEW=.FALSE.
      IF(LFLU)THEN
        NRHS=3
      ELSE
        NRHS=2
        IPNFX=0
      ENDIF
      DO 10 IEN=NRHS,NENTRY
      IF((IENTRY(IEN).NE.1).AND.(IENTRY(IEN).NE.2))CALL XABORT('@F'
     1 //'LPOW: LCM OBJECT EXPECTED AT THE RHS.')
      IF(JENTRY(IEN).NE.2)CALL XABORT('@FLPOW: READ-ONLY MODE EXPE'
     1 //'CTED FOR THE LCM OBJECT AT RHS.')
      CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.EQ.'L_POWER')THEN
        IF(LNEW)CALL XABORT('@FLPOW: L_POWER ALREADY DEFINED AT RHS.')
        CALL LCMEQU(KENTRY(IEN),IPPOW)
        LNEW=.TRUE.
      ELSEIF(HSIGN.EQ.'L_MATEX')THEN
        IF(IPMTX.EQ.0)THEN
          IPMTX=KENTRY(IEN)
        ELSE
          CALL XABORT('@FLPOW: L_MATEX ALREADY DEFINED.')
        ENDIF
      ELSEIF(HSIGN.EQ.'L_FLUX')THEN
        IF(IPFLX.EQ.0)THEN
          IPFLX=KENTRY(IEN)
        ELSE
          CALL XABORT('@FLPOW: L_FLUX ALREADY DEFINED.')
        ENDIF
      ELSEIF(HSIGN.EQ.'L_TRACK')THEN
        IF(IPTRK.EQ.0)THEN
          IPTRK=KENTRY(IEN)
        ELSE
          CALL XABORT('@FLPOW: L_TRACK ALREADY DEFINED.')
        ENDIF
      ELSEIF(HSIGN.EQ.'L_MACROLIB')THEN
        IF(IPMAC.EQ.0)THEN
          IPMAC=KENTRY(IEN)
        ELSE
          CALL XABORT('@FLPOW: L_MACROLIB ALREADY DEFINED.')
        ENDIF
      ELSEIF(HSIGN.EQ.'L_MAP')THEN
        IF(IPMAP.EQ.0)THEN
          IPMAP=KENTRY(IEN)
        ELSE
          CALL XABORT('@FLPOW: L_MAP ALREADY DEFINED.')
        ENDIF
      ENDIF
   10 CONTINUE
      IF(IPFLX.EQ.0)CALL XABORT('@FLPOW: MISSING L_FLUX OBJECT.')
      IF(IPTRK.EQ.0)CALL XABORT('@FLPOW: MISSING L_TRACK OBJECT.')
      IF((IPMAP.NE.0).AND.(IPMTX.EQ.0))CALL XABORT('@FLPOW: MISSI'
     1 //'NG L_MATEX OBJECT.')
      IF((IPMAP.EQ.0).AND.(IPMTX.NE.0))CALL XABORT('@FLPOW: MISSI'
     1 //'NG L_MAP OBJECT.')
      IF((IPMTX.EQ.0).AND.(IPMAC.EQ.0))CALL XABORT('@FLPOW: MISSI'
     1 //'NG EITHER L_MATEX OR L_MACROLIB OBJECT.')
      IF((IPMAP.EQ.0).AND.(IPMAC.EQ.0))CALL XABORT('@FLPOW: MISSI'
     1 //'NG EITHER L_MAP OR L_MACROLIB OBJECT.')
*----
*  READ KEYWORD
*----
      IMPX=1
      PTOT=0.
      LFSTH=.FALSE.
      LNRM=.FALSE.
      LBUN=.FALSE.
      FSTH=0.0
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(1).')
      IF(TEXT.NE.'EDIT')GOTO 20
*     PRINTING INDEX
      CALL REDGET(ITYP,IMPX,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.1)CALL XABORT('@FLPOW: INTEGER DATA EXPECTED(1).')
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(2).')
   20 IF(TEXT.NE.'P-NEW')GOTO 30
      IF(.NOT.LNEW)CALL XABORT('@FLPOW: MISSING READ-ONLY L_POWER'
     1 //' OBJECT AT RHS.')
      GOTO 40
   30 IF(TEXT.NE.'PTOT')CALL XABORT('@FLPOW: KEYWORD P-NEW OR PTO'
     1 //'T EXPECTED.')
      IF(LNEW)CALL XABORT('@FLPOW: ONLY ONE L_POWER OBJECT IN CRE'
     1 //'ATE MODE EXPECTED WITH PTOT OPTION.')
      CALL REDGET(ITYP,NITMA,PTOT,TEXT,DFLOT)
      IF(ITYP.NE.2)CALL XABORT('@FLPOW: REAL FOR PTOT EXPECTED.')
      IF(PTOT.LE.0.)CALL XABORT('@FLPOW: INVALID VALUE PTOT=0.')
   40 LFLX=.FALSE.
      LPOW=.FALSE.
      LMAP=.FALSE.
      LRAT=.FALSE.
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(3).')
*   THERMAL TO FISSION POWER RATIO
      IF(TEXT.NE.'FSTH') GOTO 50
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.2)CALL XABORT('@FLPOW: REAL DATA EXPECTED FOR FSTH.')
      IF((FLOT.GE.1.0).OR.(FLOT.LE.0.0)) CALL XABORT('@FLPOW: FSTH '
     1 //'SHOULD BE BETWEEN 0.0 AND 1.0.')
      FSTH=FLOT
      LFSTH=.TRUE.
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(4).')
 50   IF(TEXT.NE.'NORM') GOTO 60
        LNRM=.TRUE.
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(5).')
 60   IF(TEXT.NE.'BUND') GOTO 70
        LBUN=.TRUE.
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(6).')
 70     IF(TEXT.EQ.'PRINT')THEN
        GOTO 80
      ELSEIF(TEXT.EQ.';')THEN
        GOTO 110
      ELSE
        CALL XABORT('@FLPOW: INVALID KEYWORD '//TEXT//'.')
      ENDIF
   80 CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.EQ.'MAP')THEN
        IF(IPMAP.EQ.0)CALL XABORT('@FLPOW: INVALID KEYWORD MAP.'
     1   //' MISSING L_MAP OBJECT FOR PRINT.')
        LMAP=.TRUE.
        GOTO 100
      ELSEIF(TEXT.EQ.'ALL')THEN
        LFLX=.TRUE.
        LPOW=.TRUE.
        IF(IPMAP.NE.0)LMAP=.TRUE.
        LRAT=.TRUE.
        GOTO 100
      ELSEIF(TEXT.EQ.'DISTR')THEN
   90   CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@FLPOW: CHARACTER DATA EXPECTED(7).')
        IF(TEXT.EQ.'FLUX')THEN
          IF(LFLX)CALL XABORT('@FLPOW: KEYWORD FLUX ALREADY READ.')
          LFLX=.TRUE.
        ELSEIF(TEXT.EQ.'POWER')THEN
          IF(LPOW)CALL XABORT('@FLPOW: KEYWORD POWER ALREADY READ.')
          LPOW=.TRUE.
        ELSEIF(TEXT.EQ.'RATIO')THEN
          IF(LRAT)CALL XABORT('@FLPOW: KEYWORD RATIO ALREADY READ.')
          LRAT=.TRUE.
        ELSEIF(TEXT.EQ.';')THEN
          GOTO 110
        ELSE
          CALL XABORT('@FLPOW: INVALID KEYWORD '//TEXT//'.')
        ENDIF
        GOTO 90
      ELSE
        CALL XABORT('@FLPOW: INVALID KEYWORD '//TEXT//'.')
      ENDIF
  100 CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.NE.';')CALL XABORT('@FLPOW: FINAL ; EXPECTED.')
*     PERFORM CALCULATION
  110 CALL FLPDRV(IPPOW,IPNFX,IPFLX,IPTRK,IPMTX,IPMAP,IPMAC,PTOT,
     1 LNEW,LMAP,LFLX,LPOW,LRAT,IMPX,FSTH,LFSTH,LFLU,LBUN,LNRM)
      RETURN
      END
