*DECK MACCRE
      SUBROUTINE MACCRE(IPOLD,IPNEW,NL,NGRP,NMXOLD,NMXNEW,NTOT,
     1  MIX,LMAP,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover nuclear properties from an initial macrolib and store them
* in a new one containing one mixture per region.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): J. Koclas, E. Varin, D. Sekki
*
*Parameters: input
* IPOLD   pointer to the initial macrolib.
* NL      number of legendre orders (=1 for isotropic scattering).
* NGRP    number of energy groups.
* NMXOLD  number of material mixtures in the initial macrolib.
* NMXNEW  number of material mixtures in the final macrolib.
* NTOT    total number of all (material and virtual) mixtures.
* MIX     index of all (material and virtual) mixtures.
* LMAP    flag for the initial macrolib:
*          =.true. if the fuel map macrolib.
* IMPX    printing index (=0 for no print).
*
*Parameters: output
* IPNEW   pointer to the final macrolib.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPOLD,IPNEW,NL,NGRP,NMXOLD,NMXNEW,NTOT,MIX(NTOT)
      LOGICAL LMAP
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      CHARACTER CM*2,NAME*12,FIRST*12
*
      CALL SETARA(BASE,NMXOLD*NL*NGRP*NGRP,ISCAT)
      CALL SETARA(BASE,NMXOLD*NGRP,IWORK)
      CALL SETARA(IBASE,NMXOLD,IIPOS)
      CALL SETARA(IBASE,NMXOLD*NL*NGRP,IIJJ)
      CALL SETARA(IBASE,NMXOLD*NL*NGRP,INJJ)
*
      CALL SETARA(BASE,NMXNEW*NL*NGRP*NGRP,ISCAT2)
      CALL SETARA(BASE,NMXNEW*NGRP,IWORK2)
      CALL SETARA(IBASE,NMXNEW,IIPOS2)
      CALL SETARA(IBASE,NMXNEW*NL*NGRP,IIJJ2)
      CALL SETARA(IBASE,NMXNEW*NL*NGRP,INJJ2)
*
      CALL XDRSET(BASE(ISCAT),NMXOLD*NL*NGRP*NGRP,0.)
      CALL XDRSET(BASE(ISCAT2),NMXNEW*NL*NGRP*NGRP,0.)
      CALL XDRSET(BASE(IWORK),NMXOLD*NGRP,0.)
      CALL XDRSET(BASE(IWORK2),NMXNEW*NGRP,0.)
*----
*  RECOVER MACROLIB DATA
*----
      JPOLD=LCMGID(IPOLD,'GROUP')
      JPNEW=LCMLID(IPNEW,'GROUP',NGRP)
      DO 100 JGR=1,NGRP
      KPOLD=LCMGIL(JPOLD,JGR)
      KPNEW=LCMDIL(JPNEW,JGR)
      IF(IMPX.GT.3)CALL LCMLIB(KPOLD)
      IF(IMPX.GT.2)WRITE(IOUT,*)'** TREATING ENERGY GROUP #',JGR
      NAME=' '
      CALL LCMNXT(KPOLD,NAME)
      FIRST=NAME
   10 CALL LCMLEN(KPOLD,NAME,LENGT,ITYP)
      IF((INDEX(NAME,'NTO').EQ.1).OR.(INDEX(NAME,'DIF').EQ.1).OR.
     1   (INDEX(NAME,'NUS').EQ.1).OR.(INDEX(NAME,'NFT').EQ.1).OR.
     2   (INDEX(NAME,'CHI').EQ.1).OR.(INDEX(NAME,'OVE').EQ.1).OR.
     3   (INDEX(NAME,'H-F').EQ.1).OR.(INDEX(NAME,'SIG').EQ.1))THEN
*     RECOVER THESE PROPERTIES
        IF(IMPX.GT.2)WRITE(IOUT,*)'PROPERTY NAME : ',NAME
        IF(LENGT.EQ.NMXOLD)THEN
          CALL SETARA(BASE,NMXOLD,IDATA)
          CALL XDRSET(BASE(IDATA),NMXOLD,0.)
          CALL LCMGET(KPOLD,NAME,BASE(IDATA))
          CALL SETARA(BASE,NMXNEW,IDATA2)
          CALL XDRSET(BASE(IDATA2),NMXNEW,0.)
          IF(LMAP)THEN
*         RECOVER EXISTING DATA
            CALL LCMLEN(KPNEW,NAME,LENGT2,ITYP2)
            IF(LENGT2.NE.0)CALL LCMGET(KPNEW,NAME,BASE(IDATA2))
          ENDIF
          ITOT=0
          DO 20 IBM=1,NTOT
          IF(MIX(IBM).EQ.0)GOTO 20
          ITOT=ITOT+1
          IF(LMAP)THEN
*         ONLY FUEL DATA WILL BE COPIED
            IF(MIX(IBM).GT.0)GOTO 20
            J=-MIX(IBM)
          ELSE
*         FUEL DATA WILL NOT BE COPIED
            IF(MIX(IBM).LT.0)GOTO 20
            J=MIX(IBM)
          ENDIF
*         COPY DATA
          BASE(IDATA2+ITOT-1)=BASE(IDATA+J-1)
   20     CONTINUE
*         STORE DATA
          CALL LCMPUT(KPNEW,NAME,NMXNEW,ITYP,BASE(IDATA2))
          CALL RLSARA(BASE(IDATA))
          CALL RLSARA(BASE(IDATA2))
        ELSEIF(LENGT.EQ.-1)THEN
          CALL XABORT('@MACCRE: '//NAME//' IS A DIRECTORY.')
        ELSEIF(LENGT.NE.0)THEN
          CALL XABORT('@MACCRE: INVALID INPUT MACROLIB(1).')
        ENDIF
      ENDIF
      CALL LCMNXT(KPOLD,NAME)
      IF(FIRST.EQ.NAME)GOTO 30
      GOTO 10
*     RECOVER SCAT,IJJ,NJJ,IPOS
   30 IF(IMPX.GT.2)WRITE(IOUT,*)'RECOVERING OF SCAT,IJJ,NJJ,IPOS'
      DO IL=1,NL
        WRITE (CM,'(I2.2)') IL-1
        CALL LCMLEN(KPOLD,'SCAT'//CM,LENGT,ITYP)
        IF(LENGT.GT.NMXOLD*NL*NGRP*NGRP)THEN
          CALL XABORT('@MACCRE: INVALID INPUT MACROLIB(2).')
        ELSEIF(LENGT.GT.0)THEN
          CALL MACSCA(KPOLD,KPNEW,BASE(ISCAT),BASE(ISCAT2),BASE(IWORK),
     1         IBASE(IIJJ),IBASE(INJJ),IBASE(IIPOS),BASE(IWORK2),
     2         IBASE(IIJJ2),IBASE(INJJ2),IBASE(IIPOS2),CM,JGR,IL,MIX,
     3         NMXNEW,NTOT,NMXOLD,NL,NGRP,LMAP)
        ELSE
          CALL XABORT('@MACCRE: OLD FORMAT OF THE MACROLIB.')
        ENDIF
      ENDDO
      IF(IMPX.GT.3)CALL LCMLIB(KPNEW)
  100 CONTINUE
      CALL RLSARA(BASE(ISCAT))
      CALL RLSARA(BASE(ISCAT2))
      CALL RLSARA(BASE(IWORK))
      CALL RLSARA(IBASE(IIPOS))
      CALL RLSARA(IBASE(IIJJ))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(BASE(IWORK2))
      CALL RLSARA(IBASE(IIPOS2))
      CALL RLSARA(IBASE(IIJJ2))
      CALL RLSARA(IBASE(INJJ2))
      RETURN
      END
