*DECK MOVCHK
      SUBROUTINE MOVCHK(ID,AXIS,ITOP,DELH,MAXPOS,RODPOS,LENGTH,LVNEW)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute and check the new rod position and insertion level for
* the moving rod.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input
* ID      rod identification number.
* AXIS    axis of rod movement: =1 for X; =2 for Y; =3 for Z.
* ITOP    rod insertion: = +1 from the top; = -1 from the bottom.
* DELH    rod displacement along the axis of movement .
* MAXPOS  full-inserted rod position.
* LENGTH  full-inserted rod length.
*
*Parameters: output
* RODPOS  new rod position.
* LVNEW   new rod insertion level.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER ID,ITOP,AXIS
      REAL DELH,MAXPOS(6),RODPOS(6),LVNEW,LENGTH
      PARAMETER(IOUT=6)
*----
*  X-AXIS MOVEMENT
*----
      IF(AXIS.EQ.1)THEN
        IF(ITOP.EQ.1)THEN
          RODPOS(1)=RODPOS(1)+DELH
        ELSE
          RODPOS(2)=RODPOS(2)+DELH
        ENDIF
        IF(RODPOS(1).LT.MAXPOS(1).OR.RODPOS(1).GT.MAXPOS(2))THEN
          WRITE(IOUT,1000)ID
          CALL XABORT('@MOVCHK: NEW ROD POSITION IS OU'
     1     //'T OF THE ROD INSERTION LIMITS.')
        ENDIF
        IF(LVNEW.EQ.0.) LVNEW=(RODPOS(2)-RODPOS(1))/LENGTH
*----
*  Y-AXIS MOVEMENT
*----
      ELSEIF(AXIS.EQ.2)THEN
        IF(ITOP.EQ.1)THEN
          RODPOS(3)=RODPOS(3)+DELH
        ELSE
          RODPOS(4)=RODPOS(4)+DELH
        ENDIF
        IF(RODPOS(3).LT.MAXPOS(3).OR.RODPOS(3).GT.MAXPOS(4))THEN
          WRITE(IOUT,1000)ID
          CALL XABORT('@MOVCHK: NEW ROD POSITION IS OU'
     1     //'T OF THE ROD INSERTION LIMITS.')
        ENDIF
        IF(LVNEW.EQ.0.) LVNEW=(RODPOS(4)-RODPOS(3))/LENGTH
*----
*  Z-AXIS MOVEMENT
*----
      ELSEIF(AXIS.EQ.3)THEN
        IF(ITOP.EQ.1)THEN
          RODPOS(5)=RODPOS(6)+DELH
        ELSE
          RODPOS(6)=RODPOS(6)+DELH
        ENDIF
        IF(RODPOS(5).LT.MAXPOS(5).OR.RODPOS(5).GT.MAXPOS(6))THEN
          WRITE(IOUT,1000)ID
          CALL XABORT('@MOVCHK: NEW ROD POSITION IS OU'
     1     //'T OF THE ROD INSERTION LIMITS.')
        ENDIF
        IF(LVNEW.EQ.0.) LVNEW=(RODPOS(6)-RODPOS(5))/LENGTH
      ENDIF
      RETURN
*
 1000 FORMAT(/1X,'@MOVCHK: NEW COMPUTED POS',
     1  'ITION IS WRONG FOR THE ROD #',I3.3)
      END
