*DECK MOVGRP
      SUBROUTINE MOVGRP(IPDEV,IGRP,NDGR,DELT,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* move a group of rod-devices to a new position in the reactor core.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input
* IPDEV  pointer to device information.
* IGRP   current group number.
* DELT   time step increment.
* IMPX   printing index (=0 for no print).
*
*Parameters: output
* NDGR   number of rods in the current group.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPDEV,IGRP,NDGR,IMPX
      REAL DELT
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      REAL MAXPOS(6),RODPOS(6),LENGTH,LVOLD,LVNEW
      CHARACTER TEXT*12
      DOUBLE PRECISION DFLOT
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      LOGICAL EXTR
*----
*  READ MOVEMENT DIRECTION
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.EQ.'INSR')THEN
        EXTR=.FALSE.
      ELSEIF(TEXT.EQ.'EXTR')THEN
        EXTR=.TRUE.
      ELSE
        CALL XABORT('@MOVGRP: KEYWORD INSR OR EXTR EXPECTED.')
      ENDIF
*----
*  READ MOVEMENT OPTION
*----
      LVNEW=0.
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.EQ.'SPEED')THEN
        CALL REDGET(ITYP,NITMA,SPEED,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@MOVGRP: REAL FOR SPEED EXPECTED.')
        IF(SPEED.LE.0.)CALL XABORT('@MOVGRP: SPEED VALUE MUST BE > 0.')
        IOPT=1
      ELSEIF(TEXT.EQ.'DELH')THEN
        CALL REDGET(ITYP,NITMA,DELH,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@MOVGRP: REAL FOR DELH EXPECTED.')
        IF(DELH.LE.0.)CALL XABORT('@MOVGRP: DELH VALUE MUST BE > 0.')
        IOPT=2
      ELSEIF(TEXT.EQ.'LEVEL')THEN
        CALL REDGET(ITYP,NITMA,LVNEW,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@MOVGRP: REAL FOR LEVEL EXPECTED.')
        IF(LVNEW.GT.1.)CALL XABORT('@MOVGRP: WRONG LEVEL VALUE > 1.')
        IF(LVNEW.LT.0.)CALL XABORT('@MOVGRP: WRONG LEVEL VALUE < 0.')
        IOPT=3
      ELSE
        WRITE(IOUT,*)'@MOVGRP: WRONG KEYWORD : ',TEXT
        CALL XABORT('@MOVGRP: ROD MOVEMENT OPTION EXPECTED.')
      ENDIF
*----
*  RECOVER GROUP INFORMATION
*----
      JPDEV=LCMGID(IPDEV,'ROD_GROUP')
      KPDEV=LCMGIL(JPDEV,IGRP)
*     GROUP DATA
      CALL LCMGET(KPDEV,'NUM-ROD',NDGR)
      CALL SETARA(IBASE,NDGR,IDEV)
      CALL XDISET(IBASE(IDEV),NDGR,0)
      CALL LCMGET(KPDEV,'ROD-ID',IBASE(IDEV))
*----
*  MOVE ROD-DEVICES
*----
      DO I=1,NDGR
        ID=IBASE(IDEV+I-1)
*       ROD PARAMETERS
        JPDEV=LCMGID(IPDEV,'DEV_ROD')
        KPDEV=LCMGIL(JPDEV,ID)
        CALL LCMGET(KPDEV,'MAX-POS',MAXPOS)
        CALL LCMGET(KPDEV,'ROD-POS',RODPOS)
        CALL LCMGET(KPDEV,'LENGTH',LENGTH)
        CALL LCMGET(KPDEV,'LEVEL',LVOLD)
        CALL LCMGET(KPDEV,'AXIS',IAXIS)
        CALL LCMGET(KPDEV,'FROM',ITOP)
*       OLD PARAMETERS
        IF(IMPX.GT.1)WRITE(IOUT,1000)ID
        IF(IMPX.GT.2)WRITE(IOUT,1001)LVOLD,RODPOS(1),RODPOS(3),
     1                 RODPOS(5),RODPOS(2),RODPOS(4),RODPOS(6)
*       CHECK MOVEMENT DIRECTION
        IF(EXTR)THEN
          MOVE=ITOP
        ELSE
          MOVE=-ITOP
        ENDIF
*       UPDATE POSITION
        IF(IOPT.EQ.1)THEN
          DELH=MOVE*SPEED*DELT
        ELSEIF(IOPT.EQ.2)THEN
          DELH=MOVE*DELH
        ELSEIF(IOPT.EQ.3)THEN
          DELH=ITOP*(LVOLD-LVNEW)*LENGTH
        ENDIF
*       NEW PARAMETERS
        CALL MOVCHK(ID,IAXIS,ITOP,DELH,MAXPOS,RODPOS,LENGTH,LVNEW)
        CALL LCMPUT(KPDEV,'ROD-POS',6,2,RODPOS)
        CALL LCMPUT(KPDEV,'LEVEL',1,2,LVNEW)
        IF(IMPX.GT.1)WRITE(IOUT,1002)LVNEW,RODPOS(1),RODPOS(3),
     1                 RODPOS(5),RODPOS(2),RODPOS(4),RODPOS(6)
*     PROCEED NEXT ROD
      ENDDO
      CALL RLSARA(IBASE(IDEV))
      RETURN
*
 1000 FORMAT(/5X,' => MOVING ROD #',I3.3)
 1001 FORMAT(
     1 /5X,'PREVIOUS INSERTION LEVEL =',F8.4/
     2  5X,'PREVIOUS ROD POSITION :'/
     3  5X,'X-',F10.4,5X,'Y-',F10.4,5X,'Z-',F10.4/
     4  5X,'X+',F10.4,5X,'Y+',F10.4,5X,'Z+',F10.4)
 1002 FORMAT(
     1 /5X,'NEW INSERTION LEVEL =',F8.4/
     2  5X,'NEW ROD POSITION :'/
     3  5X,'X-',F10.4,5X,'Y-',F10.4,5X,'Z-',F10.4/
     4  5X,'X+',F10.4,5X,'Y+',F10.4,5X,'Z+',F10.4/)
      END
