*DECK MOVPOS
      SUBROUTINE MOVPOS(IPDEV,ID,DELT,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read the movement option and displace an individual rod to a new
* position in the reactor core.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input
* IPDEV  pointer to device information.
* ID     current rod identification number.
* DELT   time step increment.
* IMPX   printing index (=0 for no print).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPDEV,ID,IMPX
      REAL DELT
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      REAL MAXPOS(6),RODPOS(6),LENGTH,LVOLD,LVNEW
      CHARACTER TEXT*12
      DOUBLE PRECISION DFLOT
*----
*  RECOVER INFORMATION
*----
      JPDEV=LCMGID(IPDEV,'DEV_ROD')
      KPDEV=LCMGIL(JPDEV,ID)
*     ROD PARAMETERS
      CALL LCMGET(KPDEV,'MAX-POS',MAXPOS)
      CALL LCMGET(KPDEV,'ROD-POS',RODPOS)
      CALL LCMGET(KPDEV,'LENGTH',LENGTH)
      CALL LCMGET(KPDEV,'LEVEL',LVOLD)
      CALL LCMGET(KPDEV,'AXIS',IAXIS)
      CALL LCMGET(KPDEV,'FROM',ITOP)
*----
*  READ MOVEMENT DIRECTION
*----
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.EQ.'INSR')THEN
        MOVE=-ITOP
      ELSEIF(TEXT.EQ.'EXTR')THEN
        MOVE=ITOP
      ELSE
        CALL XABORT('@MOVPOS: KEYWORD INSR OR EXTR EXPECTED.')
      ENDIF
*----
*  READ MOVEMENT OPTION
*----
      LVNEW=0.
      CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(TEXT.EQ.'SPEED')THEN
        CALL REDGET(ITYP,NITMA,SPEED,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@MOVPOS: REAL FOR SPEED EXPECTED.')
        IF(SPEED.LE.0.)CALL XABORT('@MOVPOS: SPEED VALUE MUST BE > 0.')
        IOPT=1
      ELSEIF(TEXT.EQ.'DELH')THEN
        CALL REDGET(ITYP,NITMA,DELH,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@MOVPOS: REAL FOR DELH EXPECTED.')
        IF(DELH.LE.0.)CALL XABORT('@MOVPOS: DELH VALUE MUST BE > 0.')
        IOPT=2
      ELSEIF(TEXT.EQ.'LEVEL')THEN
        CALL REDGET(ITYP,NITMA,LVNEW,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@MOVPOS: REAL FOR LEVEL EXPECTED.')
        IF(LVNEW.GT.1.)CALL XABORT('@MOVPOS: WRONG LEVEL VALUE > 1.')
        IF(LVNEW.LT.0.)CALL XABORT('@MOVPOS: WRONG LEVEL VALUE < 0.')
        IOPT=3
      ELSE
        WRITE(IOUT,*)'@MOVPOS: WRONG KEYWORD ',TEXT
        CALL XABORT('@MOVPOS: ROD MOVEMENT OPTION EXPECTED.')
      ENDIF
*----
*  NEW ROD POSITION
*----
      IF(IOPT.EQ.1)THEN
        DELH=MOVE*SPEED*DELT
      ELSEIF(IOPT.EQ.2)THEN
        DELH=MOVE*DELH
      ELSEIF(IOPT.EQ.3)THEN
        DELH=ITOP*(LVOLD-LVNEW)*LENGTH
      ENDIF
*     OLD PARAMETERS
      IF(IMPX.GT.2)WRITE(IOUT,1001)LVOLD,RODPOS(1),RODPOS(3),
     1               RODPOS(5),RODPOS(2),RODPOS(4),RODPOS(6)
*     CHECK POSITION
      CALL MOVCHK(ID,IAXIS,ITOP,DELH,MAXPOS,RODPOS,LENGTH,LVNEW)
      CALL LCMPUT(KPDEV,'ROD-POS',6,2,RODPOS)
      CALL LCMPUT(KPDEV,'LEVEL',1,2,LVNEW)
*     UPDATED PARAMETERS
      IF(IMPX.GT.1)WRITE(IOUT,1002)LVNEW,RODPOS(1),RODPOS(3),
     1               RODPOS(5),RODPOS(2),RODPOS(4),RODPOS(6)
      RETURN
*
 1001 FORMAT(
     1 /5X,'PREVIOUS INSERTION LEVEL =',F8.4/
     2  5X,'PREVIOUS ROD POSITION :'/
     3  5X,'X-',F10.4,5X,'Y-',F10.4,5X,'Z-',F10.4/
     4  5X,'X+',F10.4,5X,'Y+',F10.4,5X,'Z+',F10.4)
 1002 FORMAT(
     1 /5X,'NEW INSERTION LEVEL =',F8.4/
     2  5X,'NEW ROD POSITION :'/
     3  5X,'X-',F10.4,5X,'Y-',F10.4,5X,'Z-',F10.4/
     4  5X,'X+',F10.4,5X,'Y+',F10.4,5X,'Z+',F10.4/)
      END
