*DECK RESDRV
      SUBROUTINE RESDRV(IPMAP,IPMTX,NFUEL,LX,LY,LZ,NCH,NB,NCOMB,
     1 IMPX,IGEO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read and validate the fuel-map specification from the input file.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*Modified : 4/06/2010 V. Descotes
*
*Parameters: input
* IPMAP  pointer to fuel-map information.
* IPMTX  pointer to matex information.
* NFUEL  number of fuel types.
* LX     number of elements along x-axis in geometry.
* LY     number of elements along y-axis in geometry.
* LZ     number of elements along z-axis in geometry.
* IMPX   printing index (=0 for no print).
* IGEO   type of geometry (CAR3D=7 or HEXZ=9)
*
*Parameters: output
* NCH    number of reactor channels.
* NB     number of fuel bundles per channel.
* NCOMB  number of combustion zones.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAP,IPMTX,NFUEL,LX,LY,LZ,NCH,NB,NCOMB,IMPX,IGEO
*----
*  LOCAL VARIABLES
*----
      CHARACTER TEXT*12,TEXT4*4
      INTEGER IBASE(1)
      LOGICAL LGEOM,LBUND,LXNAME,LYNAME
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DOUBLE PRECISION DFLOT
      REAL WEIGHT
*
      IMPX=0
      NCH=0
      NB=0
      LGEOM=.TRUE.
      LBUND=.TRUE.
*----
*  TYPE OF GEOMETRY
*----
      IF (IGEO.EQ.7) THEN
        LXNAME=.TRUE.
        LYNAME=.TRUE.
      ELSEIF (IGEO.EQ.9) THEN
        LXNAME=.FALSE.
        LYNAME=.FALSE.
      ELSE
        CALL XABORT('@RESDRV: ONLY 3D-CARTESIAN OR 3D HEXAGONAL' 
     1 //' GEOMETRY EXPECTED')
      ENDIF
   10 CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@RESDRV: CHARACTER DATA EXPECTED.')
      IF(TEXT.EQ.'EDIT')THEN
*----
*  PRINTING INDEX
*----
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@RESDRV: INTEGER DATA EXPECTED(1).')
        IMPX=MAX(0,NITMA)
        IF(IMPX.GT.4)CALL LCMLIB(IPMTX)
      ELSE IF(TEXT.EQ.'NCHAN') THEN
*----
*  NUMBER OF CHANNELS
*----
        CALL REDGET(ITYP,NCH,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@RESDRV: INTEGER DATA EXPECTED(2).')
        IF(NCH.LT.1)CALL XABORT('@RESDRV: WRONG NUMBER FOR NCHAN <1.')
      ELSE IF(TEXT.EQ.'NBUND') THEN
*----
*  NUMBER OF BUNDLES
*----
        CALL REDGET(ITYP,NB,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@RESDRV: INTEGER DATA EXPECTED(3).')
        IF(NB.LT.1)CALL XABORT('@RESDRV: WRONG NUMBER FOR NBUND <1.')
      ELSE IF(TEXT.EQ.'WEIGHT') THEN
*----
*  FUEL WEIGHT
*----
        CALL REDGET(ITYP,NB,WEIGHT,TEXT,DFLOT)
        IF(ITYP.NE.2) CALL XABORT('@RESDRV : REAL DATA '
     +          //'EXPECTED(1).')
        IF(WEIGHT.EQ.0.0 ) CALL XABORT('@RESDRV: INVALID'
     +       //'VALUE FOR FUEL WEIGHT')
        CALL LCMPUT(IPMAP,'FUEL-WEIGHT',1,2,WEIGHT)
        ELSE IF(TEXT.EQ.':::') THEN
*----
*  FUEL-MAP GEOMETRY
*----
        LGEOM=.FALSE.
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@RESDRV: CHARACTER DATA EXPECTED(5).')
        IF(TEXT.NE.'GEO:')CALL XABORT('@RESDRV: EMBEDDED GEO: MODULE '
     1  //'EXPECTED.')
*----
*  CHECK GEOMETRY
*----
        CALL RESGEO(IPMAP,IPMTX,LX,LY,LZ,NX,NY,NZ,NCH,NB,NFUEL,
     1 IMPX,IGEO)
      ELSE IF(TEXT.EQ.'BUND-SIZE') THEN
*----
*  BUNDLE DIMENSIONS
*----
        LBUND=.FALSE.
        IF (IGEO.EQ.7) THEN
         CALL REDGET(ITYP,NITMA,HX,TEXT,DFLOT)
         IF(ITYP.NE.2)CALL XABORT('@RESDRV: REAL DATA FOR HX EXPECTED.')
         IF(HX.LE.0.)CALL XABORT('@RESDRV: WRONG VALUE FOR HX <0.')
         CALL REDGET(ITYP,NITMA,HY,TEXT,DFLOT)
         IF(ITYP.NE.2)CALL XABORT('@RESDRV: REAL DATA FOR HY EXPECTED.')
         IF(HY.LE.0.)CALL XABORT('@RESDRV: WRONG VALUE FOR HY <0.')
         CALL REDGET(ITYP,NITMA,HZ,TEXT,DFLOT)
         IF(ITYP.NE.2)CALL XABORT('@RESDRV: REAL DATA FOR HZ EXPECTED.')
         IF(HZ.LE.0.)CALL XABORT('@RESDRV: WRONG VALUE FOR HZ <0.')
*        COMPUTE BUNDLE VOLUME
         CALL RESVOL(IPMAP,HX,HY,HZ,NX,NY,NZ,NCH,NB,VOL,IMPX)
         CALL LCMPUT(IPMAP,'VOLUME',1,2,VOL)
        ELSEIF (IGEO.EQ.9) THEN
         CALL REDGET(ITYP,NITMA,HX,TEXT,DFLOT)
         IF(ITYP.NE.2)CALL XABORT('@RESDRV: REAL DATA FOR HX EXPECTED.')
         IF(HX.LE.0.)CALL XABORT('@RESDRV: WRONG VALUE FOR HX <0.')
         CALL REDGET(ITYP,NITMA,HZ,TEXT,DFLOT)
         IF(ITYP.NE.2)CALL XABORT('@RESDRV: REAL DATA FOR HZ EXPECTED.')
         IF(HZ.LE.0.)CALL XABORT('@RESDRV: WRONG VALUE FOR HZ <0.')
*        COMPUTE BUNDLE VOLUME
         CALL RESVLH(IPMAP,HX,HZ,NX,NZ,NCH,NB,VOL,IMPX)
         CALL LCMPUT(IPMAP,'VOLUME',1,2,VOL)
        ENDIF        
      ELSEIF(TEXT.EQ.'NXNAME') THEN
*----
*  CHANNEL X-NAMES
*----
        LXNAME=.FALSE.
        CALL SETARA(IBASE,NX,INX)
        DO I=1,NX
          CALL REDGET(ITYP,NITMA,FLOT,TEXT4,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@RESDRV: CHARACTER DATA FOR NXNAME'
     1    //' EXPECTED.')
          READ(TEXT4,'(A4)') IBASE(INX+I-1)
        ENDDO
        CALL LCMPUT(IPMAP,'XNAME',NX,3,IBASE(INX))
        CALL RLSARA(IBASE(INX))
      ELSE IF(TEXT.EQ.'NYNAME') THEN
*----
*  CHANNEL Y-NAMES
*----
        LYNAME=.FALSE.
        CALL SETARA(IBASE,NY,INY)
        DO I=1,NY
          CALL REDGET(ITYP,NITMA,FLOT,TEXT4,DFLOT)
          IF(ITYP.NE.3)CALL XABORT('@RESDRV: CHARACTER DATA FOR NYNAME'
     1    //' EXPECTED.')
          READ(TEXT4,'(A4)') IBASE(INY+I-1)
        ENDDO
        CALL LCMPUT(IPMAP,'YNAME',NY,3,IBASE(INY))
        CALL RLSARA(IBASE(INY))
      ELSE IF(TEXT.EQ.'NCOMB') THEN
*----
*  NUMBER OF COMBUSTION ZONES
*----
        CALL SETARA(IBASE,NCH,IZONE)
        CALL XDISET(IBASE(IZONE),NCH,0)
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF((ITYP.EQ.3).AND.(TEXT.EQ.'ALL'))THEN
          NCOMB=NCH
          DO 20 ICH=1,NCH
   20     IBASE(IZONE+ICH-1)=ICH
        ELSEIF(ITYP.EQ.1)THEN
          IF(NITMA.LT.1)CALL XABORT('@RESDRV: INVALID NCOMB < 1')
          IF(NITMA.GT.NCH)CALL XABORT('@RESDRV: INVALID NCOMB > NCH')
          NCOMB=NITMA
*----
*  COMBUSTION-ZONE INDICES
*----
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(TEXT.NE.'B-ZONE')CALL XABORT('@RESDRV: KEYWORD B-ZONE'
     1    //' EXPECTED.')
          DO 30 ICH=1,NCH
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.1)CALL XABORT('@RESDRV: INTEGER COMBUSTION'
     1    //'-ZONE INDEX EXPECTED.')
          IF(NITMA.LT.1)CALL XABORT('@RESDRV: INVALID COMBUSTION'
     1    //'-ZONE INDEX < 1')
          IF(NITMA.GT.NCOMB)CALL XABORT('@RESDRV: INVALID COMBUSTION'
     1    //'-ZONE INDEX > NCOMB')
          IBASE(IZONE+ICH-1)=NITMA
   30     CONTINUE
        ELSE
          CALL XABORT('@RESDRV: INVALID INPUT FOR NCOMB.')
        ENDIF
        CALL LCMPUT(IPMAP,'B-ZONE',NCH,1,IBASE(IZONE))
        CALL RLSARA(IBASE(IZONE))
	GO TO 40
      ELSE
        CALL XABORT('@RESDRV: INVALID KEYWORD ('//TEXT//').')
      ENDIF
      GO TO 10
*
   40 IF(NCH.EQ.0) CALL XABORT('@RESDRV: KEYWORD NCHAN EXPECTED.')
      IF(NB.EQ.0) CALL XABORT('@RESDRV: KEYWORD NBUND EXPECTED.')
      IF(LGEOM) CALL XABORT('@RESDRV: OPERATOR ::: EXPECTED.')
      IF(LBUND) CALL XABORT('@RESDRV: KEYWORD BUND-SIZE EXPECTED.')
      IF(LXNAME)CALL XABORT('@RESDRV: KEYWORD NXNAME EXPECTED.')
      IF(LYNAME)CALL XABORT('@RESDRV: KEYWORD NYNAME EXPECTED.')
      RETURN
      END
