*DECK RESGEO
      SUBROUTINE RESGEO(IPMAP,IPMTX,LX,LY,LZ,NX,NY,NZ,NCH,NB,NFUEL,
     1 IMPX,IGEO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* create and check the fuel-map geometry.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): E. Varin, D. Sekki
*Modified : 4/06/2010 V. Descotes
*
*Parameters: input
* IPMAP  pointer to fuel-map information.
* IPMTX  pointer to matex information.
* LX     number of elements along x-axis in geometry.
* LY     number of elements along y-axis in geometry.
* LZ     number of elements along z-axis in geometry.
* NCH    number of reactor channels.
* NB     number of fuel bundles per channel.
* NFUEL  number of fuel types.
* IMPX   printing index (=0 for no print).
* IGEO   type of geometry (=7 or =9)
*
*Parameters: output
* NX     number of elements along x-axis in fuel map.
* NY     number of elements along y-axis in fuel map.
* NZ     number of elements along z-axis in fuel map.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAP,IPMTX,LX,LY,LZ,NX,NY,NZ,NCH,NB,NFUEL,IGEO
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6,EPSI=1.0E-4)
      INTEGER ISTATE(NSTATE),JENT(1),IENT(1),KENT(1)
      REAL    GEOXX(LX+1),GEOYY(LY+1),GEOZZ(LZ+1),GEOSI
      COMMON  BASE(1)
      CHARACTER HENT(1)*12
*----
*  FUEL-MAP GEOMETRY
*----
      IF(IMPX.GT.1)WRITE(IOUT,*)'** CREATING FUEL-MAP GEOMETRY **'
      CALL LCMSIX(IPMAP,'GEOMAP',1)
      NENT=1
      JENT(1)=0
      HENT(1)='GEOMAP'
      IENT(1)=1
      KENT(1)=IPMAP
      CALL GEO(NENT,HENT,IENT,JENT,KENT)
      IF(IMPX.GT.2)CALL LCMLIB(IPMAP)
****
      CALL LCMSIX(IPMAP,' ',0)
      CALL LCMSIX(IPMAP,'GEOMAP',1)
****
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
      IF (ISTATE(1).NE.IGEO) CALL XABORT('@RESGEO: THE GEOMETRY '
     1 // 'IN FUEL-MAP MUST HAVE THE SAME TYPE AS IN THE MATEX-OBJECT')
      IGEO=ISTATE(1)
      NX=ISTATE(3)
      NY=ISTATE(4)
      NZ=ISTATE(5)
      IF(IMPX.GT.2)WRITE(IOUT,*)'CHECKING FUEL-MAP GEOMETRY'
      IF((IGEO.NE.7).AND.(IGEO.NE.9))CALL XABORT('@RESGEO: ONLY '
     1 //'3D-CARTESIAN OR 3D-HEXAGONAL GEOMETRY ALLOWED.')
      IF((LX.LT.NX).OR.(LY.LT.NY).OR.(LZ.LT.NZ))
     1  CALL XABORT('@RESGEO: WRONG GEOMETRY DEFINITION.')
      IF(NB.NE.NZ)THEN
        WRITE(IOUT,*)'@RESGEO: FOUND NB=',NB,' NOT EQUAL TO NZ=',NZ
        CALL XABORT('@RESGEO: WRONG FUEL-MAP GEOMETRY DEFINITION.')
      ENDIF
*----
*  CHECK MESHX OR SIDE
*----
      IF (IGEO.EQ.7) THEN
       CALL SETARA(BASE,NX+1,IMAPX)
       CALL XDRSET(BASE(IMAPX),NX+1,0.)
       CALL LCMGET(IPMAP,'MESHX',BASE(IMAPX))
       CALL XDRSET(GEOXX,LX+1,0.)
       CALL LCMGET(IPMTX,'MESHX',GEOXX)
       DO 10 IMP=1,NX+1
       DO IGM=1,LX+1
         IF(ABS(BASE(IMAPX+IMP-1)-GEOXX(IGM)).LT.EPSI)THEN
           GEOXX(IGM)=BASE(IMAPX+IMP-1)
           GOTO 10
         ENDIF
       ENDDO
       WRITE(IOUT,*)'@RESGEO: MESHX IN L_MAP ',BASE(IMAPX+IMP-1)
       CALL XABORT('@RESGEO: UNABLE TO FIND THIS MESHX IN L_GEOM.')
   10  CONTINUE
       CALL RLSARA(BASE(IMAPX))
       CALL LCMPUT(IPMTX,'MESHX',LX+1,2,GEOXX)
      ELSE IF (IGEO.EQ.9) THEN
       CALL SETARA(BASE,NX+1,MAPSI)
       CALL XDRSET(BASE(MAPSI),1,0.)
       CALL LCMGET(IPMAP,'SIDE',BASE(MAPSI))
       CALL XDRSET(GEOSI,1,0.)
       CALL LCMGET(IPMTX,'SIDE',GEOSI)
       IF(ABS(BASE(MAPSI)-GEOSI).LT.EPSI)THEN
           GEOSI=BASE(MAPSI)
           GOTO 20
       ENDIF
       WRITE(IOUT,*)'@RESGEO: SIDE IN L_MAP ',BASE(MAPSI+IMP-1)
       CALL XABORT('@RESGEO: UNABLE TO FIND THIS SIDE IN L_GEOM.')
   20  CONTINUE
       CALL RLSARA(BASE(MAPSI))
       CALL LCMPUT(IPMTX,'SIDE',1,2,GEOSI)
      ENDIF
*----
*  CHECK MESHY (ONLY IF 3D-CARTESIAN GEOMETRY)
*----
      IF (IGEO.EQ.7) THEN
       CALL SETARA(BASE,NY+1,IMAPY)
       CALL XDRSET(BASE(IMAPY),NY+1,0.)
       CALL LCMGET(IPMAP,'MESHY',BASE(IMAPY))
       CALL XDRSET(GEOYY,LY+1,0.)
       CALL LCMGET(IPMTX,'MESHY',GEOYY)
       DO 30 IMP=1,NY+1
       DO IGM=1,LY+1
         IF(ABS(BASE(IMAPY+IMP-1)-GEOYY(IGM)).LT.EPSI)THEN
           GEOYY(IGM)=BASE(IMAPY+IMP-1)
           GOTO 30
         ENDIF
       ENDDO
       WRITE(IOUT,*)'@RESGEO: MESHY IN FUEL MAP ',BASE(IMAPY+IMP-1)
       CALL XABORT('@RESGEO: UNABLE TO FIND THIS MESHY IN L_GEOM.')
   30  CONTINUE
       CALL RLSARA(BASE(IMAPY))
       CALL LCMPUT(IPMTX,'MESHY',LY+1,2,GEOYY)
      ELSE IF (IGEO.EQ.9) THEN
       GOTO 40
   40  CONTINUE
      ENDIF
*----
*  CHECK MESHZ
*----
      CALL SETARA(BASE,NZ+1,IMAPZ)
      CALL XDRSET(BASE(IMAPZ),NZ+1,0.)
      CALL LCMGET(IPMAP,'MESHZ',BASE(IMAPZ))
      CALL XDRSET(GEOZZ,LZ+1,0.)
      CALL LCMGET(IPMTX,'MESHZ',GEOZZ)
      DO 50 IMP=1,NZ+1
      DO IGM=1,LZ+1
        IF(ABS(BASE(IMAPZ+IMP-1)-GEOZZ(IGM)).LT.EPSI)THEN
          GEOZZ(IGM)=BASE(IMAPZ+IMP-1)
          GOTO 50
        ENDIF
      ENDDO
      WRITE(IOUT,*)'@RESGEO: MESHZ IN FUEL MAP ',BASE(IMAPZ+IMP-1)
      CALL XABORT('@RESGEO: UNABLE TO FIND  THIS MESHZ IN L_GEOM.')
   50 CONTINUE
      CALL RLSARA(BASE(IMAPZ))
      CALL LCMPUT(IPMTX,'MESHZ',LZ+1,2,GEOZZ)
*----
*  CHECK FUEL MIXTURES
*----
      IF (IGEO.EQ.7) THEN
        CALL RESPFM(IPMAP,IPMTX,NX,NY,NZ,LX,LY,LZ,NCH,NB,NFUEL,IMPX)
      ELSE IF (IGEO.EQ.9) THEN
        CALL RESHFM(IPMAP,IPMTX,NX,NZ,LX,LZ,NCH,NB,NFUEL,IMPX,IGEO)
      ENDIF
      RETURN
      END
