*DECK RESHFM
      SUBROUTINE RESHFM(IPMAP,IPMTX,NX,NZ,LX,LZ,NCH,NB,NFUEL,
     1 IMPX,IGEO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover, check and store the fuel mixtures for 3D-hexagonal geometry.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): V. Descotes
*
*Parameters: input/output
* IPMAP   pointer to fuel-map information.
* IPMTX   pointer to matex information.
* NX      number of elements along x-axis in fuel map.
* NZ      number of elements along z-axis in fuel map.
* LX      number of elements along x-axis in geometry.
* LZ      number of elements along z-axis in geometry.
* NCH     number of reactor channels.
* NB      number of fuel bundles per channel.
* NFUEL   number of fuel types.
* IMPX    printing index (=0 for no print).
* IGEO    type of geometry
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAP,IPMTX,NX,NZ,LX,LZ,NCH,NB,NFUEL,IMPX,IGEO
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER MIX(NX*NZ),FLMIX(NCH,NB),FMIX(NFUEL),FTOT(NFUEL)
*----
*  CHECK FUEL MIXTURES
*----
      IF (IGEO.NE.9) CALL XABORT('@RESHID: WRONG TYPE OF GEOMETRY,'
     1 // ' 3D-HEXAGONAL GEOMETRY EXPECTED')
      CALL XDISET(MIX,NX*NZ,0)
      CALL LCMGET(IPMAP,'MIX',MIX)
      CALL XDISET(FMIX,NFUEL,0)
      CALL LCMGET(IPMTX,'FMIX',FMIX)
      CALL XDISET(FTOT,NFUEL,0)
      CALL XDISET(FLMIX,NCH*NB,0)
      IF(IMPX.GT.2)WRITE(IOUT,*)'CHECKING FUEL-MAP MIXTURES'
      IEL=0
      NTOT=0
      DO 10 IB=1,NB
      ICH=0
      DO 10 I=1,NX
      IEL=IEL+1
      IF(MIX(IEL).EQ.0)GOTO 10
      ICH=ICH+1
      IF(ICH.GT.NCH)CALL XABORT('@RESHFM: WRONG FUEL-MAP DEFINITION.')
      FLMIX(ICH,IB)=MIX(IEL)
      DO IFUEL=1,NFUEL
        IF(MIX(IEL).EQ.FMIX(IFUEL))THEN
          FTOT(IFUEL)=FTOT(IFUEL)+1
          NTOT=NTOT+1
          IF(NTOT.GT.NCH*NB)THEN
            WRITE(IOUT,*)'@RESHFM: TOTAL NUMBER OF BUNDLES =',NCH*NB
            WRITE(IOUT,*)'@RESHFM: READ TOTAL FUEL MIXTURES ',NTOT
            CALL XABORT('@RESHFM: WRONG FUEL-MAP DEFINITION(1).')
          ENDIF
          GOTO 10
        ENDIF
      ENDDO
      WRITE(IOUT,*)'@RESHFM: READ FUEL MIXTURE NUMBER ',MIX(IEL)
      CALL XABORT('@RESHFM: WRONG FUEL MIXTURE NUMBER.')
   10 CONTINUE
      IF(NTOT.LT.NCH*NB)THEN
        WRITE(IOUT,*)'@RESHFM: TOTAL NUMBER OF BUNDLES =',NCH*NB
        WRITE(IOUT,*)'@RESHFM: READ TOTAL NUMBER OF FUEL MIXTURES ',NTOT
        CALL XABORT('@RESHFM: WRONG FUEL-MAP DEFINITION(2).')
      ENDIF
*----
*  STORE FUEL MIXTURES
*----
      IF(IMPX.GT.2)WRITE(IOUT,*)'STORING FUEL MIXTURES'
*     FUEL DIRECTORIES
      CALL LCMSIX(IPMAP,' ',0)
      JPMAP=LCMLID(IPMAP,'FUEL',NFUEL)
      DO IFUEL=1,NFUEL
        KPMAP=LCMDIL(JPMAP,IFUEL)
        CALL LCMPUT(KPMAP,'MIX',1,1,FMIX(IFUEL))
        CALL LCMPUT(KPMAP,'TOT',1,1,FTOT(IFUEL))
      ENDDO
      CALL LCMPUT(IPMAP,'FLMIX',NCH*NB,1,FLMIX)
*     RENUMBERING
      NMIX=0
      DO IEL=1,NX*NZ
        IF(MIX(IEL).NE.0)THEN
          NMIX=NMIX+1
          MIX(IEL)=NMIX
        ENDIF
      ENDDO
      CALL LCMPUT(IPMAP,'BMIX',NX*NZ,1,MIX)
*     UPDATE MATERIAL INDEX
      CALL RESHID(IPMAP,IPMTX,NX,NZ,LX,LZ,MIX,NFUEL,IMPX,IGEO)
      RETURN
      END
