*DECK RESPAR
      SUBROUTINE RESPRH(IPMAP,NCH,NB,NFUEL,NCOMB,NPARM,NX,NZ,NSTATE,
     1 ISTATE,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read and store the data related to global and local parameters
* for 3D-hexagonal geometry.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): V. Descotes, D. Sekki, R. Chambon
*
*Parameters: input/output
* IPMAP   pointer to fuel-map information.
* NCH     number of reactor channels.
* NB      number of fuel bundles per channel.
* NFUEL   number of fuel types.
* NCOMB   number of combustion zones.
* NX      number of elements along x-axis in fuel map.
* NZ      number of elements along z-axis in fuel map.
* NSTATE  maximum number of state-vector records.
* IMPX    printing index (=0 for no print).
* NPARM   as input, NPARM=0.
*
*Parameters: output
* ISTATE  updated state-vector.
* NPARM   total number of recorded parameters.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER ISTATE(NSTATE)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER IBASE(1),INAME(3)
      REAL VALUE(NCH,NB)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DOUBLE PRECISION DFLOT
      CHARACTER TEXT*12,TEXT12*12,PNAME*12
      LOGICAL LBURN
*----
*  READ INPUT DATA
*----
      LBURN=.FALSE.
   10 CALL REDGET(ITYP,NITMA,FLOT,TEXT12,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@RESPAR: CHARACT'
     1 //'ER DATA EXPECTED.')
      IF(TEXT12.EQ.';')THEN
        GOTO 500
*     PRINTING INDEX
      ELSEIF(TEXT12.EQ.'EDIT')THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT12,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@RESPAR: INTEGER DA'
     1   //'TA FOR EDIT EXPECTED.')
        IMPX=MAX(0,NITMA)
*----
*  ADD NEW PARAMETER
*----
      ELSEIF(TEXT12.EQ.'ADD-PARAM')THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(TEXT.NE.'PNAME')CALL XABORT('@RESPAR: KEY'
     1   //'WORD PNAME EXPECTED.')
*       READ PARAMETER NAME
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@RESPAR: CHARACTER'
     1   //' DATA FOR PARAMETER NAME EXPECTED.')
        IF(IMPX.GT.0)WRITE(IOUT,1000)TEXT
        IF(NPARM.GT.0)THEN
          JPMAP=LCMGID(IPMAP,'PARAM')
          DO IPAR=1,NPARM
            KPMAP=LCMGIL(JPMAP,IPAR)
            CALL LCMGET(KPMAP,'P-NAME',INAME)
            WRITE(PNAME,'(3A4)') (INAME(I),I=1,3)
            IF(PNAME.EQ.TEXT)CALL XABORT('@RESPAR: THE '
     1       //'PARAMETER '//TEXT//' ALREADY EXISTS.')
          ENDDO
        ENDIF
        NPARM=NPARM+1
        JPMAP=LCMLID(IPMAP,'PARAM',NPARM)
        KPMAP=LCMDIL(JPMAP,NPARM)
        READ(TEXT,'(3A4)') (INAME(I),I=1,3)
        CALL LCMPUT(KPMAP,'P-NAME',3,3,INAME)
*       READ PARKEY NAME
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(TEXT.NE.'PARKEY')CALL XABORT('@RESPAR: KEY'
     1   //'WORD PARKEY EXPECTED.')
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@RESPAR: CHARACTER'
     1   //' DATA FOR PARKEY NAME EXPECTED.')
        READ(TEXT,'(3A4)') (INAME(I),I=1,3)
        CALL LCMPUT(KPMAP,'PARKEY',3,3,INAME)
*       READ PARAMETER TYPE
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(TEXT.EQ.'GLOBAL')THEN
          IPTYP=1
        ELSEIF(TEXT.EQ.'LOCAL')THEN
          IPTYP=2
        ELSE
          CALL XABORT('@RESPAR: INVALID KEYWORD '//TEXT)
        ENDIF
        CALL LCMPUT(KPMAP,'P-TYPE',1,1,IPTYP)
        ISTATE(8)=NPARM
*----
*  SET PARAMETER VALUES
*----
      ELSEIF(TEXT12.EQ.'SET-PARAM')THEN
        IF(NPARM.EQ.0)CALL XABORT('@RESPAR: PARAM'
     1   //'ETER NOT YET DEFINED NPARM=0')
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@RESPAR: CHARACT'
     1   //'ER DATA FOR PARAMETER NAME EXPECTED.')
*       RECOVER PARAMETER
        JPMAP=LCMGID(IPMAP,'PARAM')
        DO IPAR=1,NPARM
          KPMAP=LCMGIL(JPMAP,IPAR)
          CALL LCMGET(KPMAP,'P-NAME',INAME)
          WRITE(PNAME,'(3A4)') (INAME(I),I=1,3)
          IF(PNAME.EQ.TEXT)THEN
            CALL LCMGET(KPMAP,'P-TYPE',IPTYP)
            GOTO 20
          ENDIF
        ENDDO
        CALL XABORT('@RESPAR: UNABLE TO FIND PARAME'
     1   //'TER WITH PNAME '//TEXT)
   20   IF(IMPX.GT.0)WRITE(IOUT,1001)TEXT
        IF(IPTYP.EQ.1)THEN
*         GLOBAL PARAMETER
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.2)CALL XABORT('@RESPAR: REAL'
     1     //' DATA FOR VALUE EXPECTED.')
          CALL LCMPUT(KPMAP,'P-VALUE',1,2,FLOT)
        ELSE
*         LOCAL PARAMETER
          CALL XDRSET(VALUE,NCH*NB,0.)
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(TEXT.EQ.'SAME')THEN
            CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
            IF(ITYP.NE.2)CALL XABORT('@RESPAR: REAL'
     1       //' DATA FOR VALUE EXPECTED.')
            DO 30 ICH=1,NCH
            DO 30 IB=1,NB
   30       VALUE(ICH,IB)=FLOT
*
          ELSEIF(TEXT.EQ.'CHAN')THEN
            DO ICH=1,NCH
              CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
              IF(ITYP.NE.2)CALL XABORT('@RESPAR: REAL'
     1         //' DATA FOR VALUE EXPECTED.')
              DO 40 IB=1,NB
   40         VALUE(ICH,IB)=FLOT
            ENDDO
*
          ELSEIF(TEXT.EQ.'BUND')THEN
            DO 50 ICH=1,NCH
            DO 50 IB=1,NB
              CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
              IF(ITYP.NE.2)CALL XABORT('@RESPAR: REAL'
     1         //' DATA FOR VALUE EXPECTED.')
              VALUE(ICH,IB)=FLOT
   50       CONTINUE
          ELSE
            CALL XABORT('@RESPAR: INVALID KEYWORD '//TEXT)
          ENDIF
          CALL LCMPUT(KPMAP,'P-VALUE',NCH*NB,2,VALUE)
        ENDIF
*----
*  CHANNEL REFUELLING SCHEMES
*----
      ELSEIF(TEXT12.EQ.'REF-SHIFT')THEN
        CALL XABORT('@RESPRH: REF-SHIFT OPTION NON AVAILABLE FOR '
     1   //'3D-HEXAGONAL GEOMETRY.')
*----
*  BURNUP INTERPOLATION TYPE
*----
      ELSEIF(TEXT12.EQ.'BTYPE')THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@RESPAR: BURN'
     1   //'UP INTERPOLATION OPTION EXPECTED.')
        IF(TEXT.EQ.'TIMAV-BURN')THEN
          IBTYP=1
        ELSEIF(TEXT.EQ.'INST-BURN')THEN
          IBTYP=2
        ELSE
          CALL XABORT('@RESPAR: INVALID INPUT FOR BTYPE.')
        ENDIF
        ISTATE(5)=IBTYP
*----
*  AVERAGE EXIT BURNUPS
*----
      ELSEIF(TEXT12.EQ.'TIMAV-BVAL')THEN
        CALL XABORT('@RESPRH: INVALID OPTION TIMAV-BVAL FOR '
     1 // '3D-HEXAGONAL GEOMETRY. TIME-AVERAGE MODEL ONLY '
     2 // ' DESIGNED FOR 3D-GEOMETRIES.') 
*----
*  INSTANTANEOUS BURNUPS
*----
      ELSEIF(TEXT12.EQ.'INST-BVAL')THEN
        IF(IMPX.GT.0)WRITE(IOUT,1004)
        CALL XDRSET(VALUE,NCH*NB,0.)
        DO 90 ICH=1,NCH
        DO 90 IB=1,NB
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@RESPAR: REAL DATA'
     1   //' FOR BURNUP VALUE EXPECTED(2).')
        IF(FLOT.LT.0.)CALL XABORT('@RESPAR: INVALID DA'
     1   //'TA FOR BURNUP VALUE <0.')
        VALUE(ICH,IB)=FLOT
   90   CONTINUE
        CALL LCMPUT(IPMAP,'BURN-INST',NCH*NB,2,VALUE)
*----
*  FUEL-TYPE DATA
*----
      ELSEIF(TEXT12.EQ.'FUEL')THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@RESPAR: KEYWO'
     1   //'RD FOR FUEL-TYPE PARAMETER EXPECTED.')
        IF((TEXT.NE.'WEIGHT').AND.(TEXT.NE.'ENRICH').AND.
     1   (TEXT.NE.'POISON'))CALL XABORT('@RESPAR: INVAL'
     2   //'ID INPUT FOR FUEL.')
        IF(IMPX.GT.0)WRITE(IOUT,1005)TEXT
        JPMAP=LCMLID(IPMAP,'FUEL',NFUEL)
        DO IFUEL=1,NFUEL
          CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
          IF(ITYP.NE.2)CALL XABORT('@RESPAR: REAL'
     1     //' DATA PER EACH FUEL-TYPE EXPECTED.')
          KPMAP=LCMDIL(JPMAP,IFUEL)
          CALL LCMPUT(KPMAP,TEXT,1,2,FLOT)
          IF(IMPX.GT.0)CALL LCMLIB(KPMAP)
        ENDDO
      ELSE
        CALL XABORT('@RESPAR: WRONG KEYWORD '//TEXT12)
      ENDIF
      GOTO 10
  500 CONTINUE
      RETURN
*
 1000 FORMAT(/1X,'INPUT OF NEW PARAMETER: ',A12)
 1001 FORMAT(/1X,'READING VALUES FOR PARAMETER: ',A12)
 1002 FORMAT(/1X,'READING INPUT FOR REF-SHIFT')
 1003 FORMAT(/1X,'READING AVERAGE EXIT BURNUPS')
 1004 FORMAT(/1X,'READING INSTANTANEOUS BURNUPS')
 1005 FORMAT(/1X,'READING DATA FOR FUEL-TYPE PARAMETER: ',A12)
      END
