*DECK RESVLH
      SUBROUTINE RESVLH(IPMAP,HX,HZ,NX,NZ,NCH,NB,VOLUME,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* check the mesh widths and compute the fuel bundle volume.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): V. Descotes, D. Sekki
*
*Parameters: input
* IPMAP  pointer to fuel-map information.
* HX     bundle hexagonal side (xOy plane)
* HZ     bundle length along z-axis.
* NX     number of hexagons in a plane of the fuel map.
* NZ     number of elements along z-axis in fuel map.
* NCH    number of reactor channels.
* NB     number of fuel bundles per channel.
* IMPX   printing index (=0 for no print).
*
*Parameters: output
* VOLUME bundle volume.
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAP,NX,NZ,NCH,NB,IMPX
      REAL HX,HZ,VOLUME
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6,EPSI=1.0E-2)
      INTEGER MIX(NX*NZ)
      REAL SIDE,MESHZ(NZ+1),DHZ(NZ)
*----
*  CHECK MESH WIDTHS
*----
      IF(IMPX.GT.2)WRITE(IOUT,*)'CHECKING THE MESH WIDTHS'
      CALL XDRSET(SIDE,1,0.)
      CALL XDRSET(MESHZ,NZ+1,0.)
      JPMAP=LCMGID(IPMAP,'GEOMAP')
      CALL LCMGET(JPMAP,'SIDE',SIDE)
      CALL LCMGET(JPMAP,'MESHZ',MESHZ)
      CALL XDISET(MIX,NX*NZ,0)
      CALL LCMGET(IPMAP,'BMIX',MIX)
*     RECOVER MESHZ WIDTHS
      CALL XDRSET(DHZ,NZ,0.)
      DO 30 K=1,NZ
   30 DHZ(K)=MESHZ(K+1)-MESHZ(K)
*     CHECK MESHZ WIDTHS AND SIDE
      IF(ABS(SIDE-HX).GE.EPSI)THEN
        WRITE(IOUT,1000)HX,SIDE
        CALL XABORT('@RESVOL: FUEL-MAP GEOMETRY NOT WELL DEFINED.')
      ENDIF
      IEL=0
      ITOT=0
      DO 50 K=1,NZ
      DO 50 I=1,NX
      IEL=IEL+1
      IF(MIX(IEL).EQ.0)GOTO 50
      ITOT=ITOT+1
      IF(ABS(DHZ(K)-HZ).GE.EPSI)THEN
        WRITE(IOUT,2000)'Z',HZ,DHZ(K),I,K
        CALL XABORT('@RESVOL: FUEL-MAP GEOMETRY NOT WELL DEFINED.')
      ENDIF
   50 CONTINUE
*     CHECK TOTAL NUMBER
      IF(ITOT.NE.(NB*NCH))THEN
        WRITE(IOUT,*)'@RESVOL: GIVEN TOTAL NUMBER OF BUNDLES =',NCH*NB
        WRITE(IOUT,*)'@RESVOL: FOUND THE TOTAL NUMBER OF BUNDLES ',ITOT
        CALL XABORT('@RESVOL: WRONG FUEL-MAP GEOMETRY DEFINITION.')
      ENDIF
*     BUNDLE VOLUME FOR HEXAGONAL BUNDLES
      VOLUME=3/2*sqrt(3.)*HX*HX*HZ
      IF(IMPX.GT.2)WRITE(IOUT,*)'COMPUTED BUNDLE VOLUME = ',VOLUME
      RETURN
*
 1000 FORMAT(/1X,'BUNDLE SIDE IS GIVEN',1X,
     1 'AS :',F8.4/1X,'** WARNING: FOUND DIFFERENT SIDE WIDTH :',
     2  F8.4/)
 2000 FORMAT(/1X,'BUNDLE LENGTH ALONG AXIS "',A1,'" IS GIVEN',1X,
     1 'AS :',F8.4/1X,'** WARNING: FOUND DIFFERENT MESH WIDTH :',
     2  F8.4,4X,'AT COORD => ','I =',I3,2X,'J =',I3,2X,'K =',I3/)
      END
