*DECK RESVOL
      SUBROUTINE RESVOL(IPMAP,HX,HY,HZ,NX,NY,NZ,NCH,NB,VOLUME,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* check the mesh widths and compute the fuel bundle volume.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input
* IPMAP  pointer to fuel-map information.
* HX     bundle length along x-axis.
* HY     bundle length along y-axis.
* HZ     bundle length along z-axis.
* NX     number of elements along x-axis in fuel map.
* NY     number of elements along y-axis in fuel map.
* NZ     number of elements along z-axis in fuel map.
* NCH    number of reactor channels.
* NB     number of fuel bundles per channel.
* IMPX   printing index (=0 for no print).
*
*Parameters: output
* VOLUME bundle volume.
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAP,NX,NY,NZ,NCH,NB,IMPX
      REAL HX,HY,HZ,VOLUME
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6,EPSI=1.0E-2)
      INTEGER MIX(NX*NY*NZ)
      REAL MESHX(NX+1),MESHY(NY+1),MESHZ(NZ+1),
     1 DHX(NX),DHY(NY),DHZ(NZ)
*----
*  CHECK MESH WIDTHS
*----
      IF(IMPX.GT.2)WRITE(IOUT,*)'CHECKING THE MESH WIDTHS'
      CALL XDRSET(MESHX,NX+1,0.)
      CALL XDRSET(MESHY,NY+1,0.)
      CALL XDRSET(MESHZ,NZ+1,0.)
      JPMAP=LCMGID(IPMAP,'GEOMAP')
      CALL LCMGET(JPMAP,'MESHX',MESHX)
      CALL LCMGET(JPMAP,'MESHY',MESHY)
      CALL LCMGET(JPMAP,'MESHZ',MESHZ)
      CALL XDISET(MIX,NX*NY*NZ,0)
      CALL LCMGET(IPMAP,'BMIX',MIX)
*     RECOVER MESH WIDTHS
      CALL XDRSET(DHX,NX,0.)
      DO 10 I=1,NX
   10 DHX(I)=MESHX(I+1)-MESHX(I)
      CALL XDRSET(DHY,NY,0.)
      DO 20 J=1,NY
   20 DHY(J)=MESHY(J+1)-MESHY(J)
      CALL XDRSET(DHZ,NZ,0.)
      DO 30 K=1,NZ
   30 DHZ(K)=MESHZ(K+1)-MESHZ(K)
*     CHECK MESH WIDTHS
      IEL=0
      ITOT=0
      DO 50 K=1,NZ
      DO 50 J=1,NY
      DO 50 I=1,NX
      IEL=IEL+1
      IF(MIX(IEL).EQ.0)GOTO 50
      ITOT=ITOT+1
      IF(ABS(DHX(I)-HX).GE.EPSI)THEN
        WRITE(IOUT,1000)'X',HX,DHX(I),I,J,K
        CALL XABORT('@RESVOL: FUEL-MAP GEOMETRY NOT WELL DEFINED.')
      ENDIF
      IF(ABS(DHY(J)-HY).GE.EPSI)THEN
        WRITE(IOUT,1000)'Y',HY,DHY(J),I,J,K
        CALL XABORT('@RESVOL: FUEL-MAP GEOMETRY NOT WELL DEFINED.')
      ENDIF
      IF(ABS(DHZ(K)-HZ).GE.EPSI)THEN
        WRITE(IOUT,1000)'Z',HZ,DHZ(K),I,J,K
        CALL XABORT('@RESVOL: FUEL-MAP GEOMETRY NOT WELL DEFINED.')
      ENDIF
   50 CONTINUE
*     CHECK TOTAL NUMBER
      IF(ITOT.NE.(NB*NCH))THEN
        WRITE(IOUT,*)'@RESVOL: GIVEN TOTAL NUMBER OF BUNDLES =',NCH*NB
        WRITE(IOUT,*)'@RESVOL: FOUND THE TOTAL NUMBER OF BUNDLES ',ITOT
        CALL XABORT('@RESVOL: WRONG FUEL-MAP GEOMETRY DEFINITION.')
      ENDIF
*     BUNDLE VOLUME
      VOLUME=HX*HY*HZ
      IF(IMPX.GT.2)WRITE(IOUT,*)'COMPUTED BUNDLE VOLUME = ',VOLUME
      RETURN
*
 1000 FORMAT(/1X,'BUNDLE LENGTH ALONG AXIS "',A1,'" IS GIVEN',1X,
     1 'AS :',F8.4/1X,'** WARNING: FOUND DIFFERENT MESH WIDTH :',
     2  F8.4,4X,'AT COORD => ','I =',I3,2X,'J =',I3,2X,'K =',I3/)
      END
