*DECK TINREF
      SUBROUTINE TINREF(IPRES,IPMIC,IPMIC2,IPMIC3,NCH,NK,NX,NY,NREG,
     +         NAMCHA,NS,MS,WINT,MIX,IXN,IYN,NW,NW2,WORK,BS,PS,ISFT,POW,
     +         IWORK,WORKS,MAXS,NSS,IND,INDEX,IPRT,KRF,LMIC)

*
*-----------------------------------------------------------------------
*
*Purpose:
* refuel a channel according to a refuelling mode
*
*Copyright:
* Copyright (C) 2010 Ecole Polytechnique de Montreal
*
*Author(s): E. Varin, M. Guyot
*
*Parameters: input/output
*
* IPRES  Adress of the map Linked_List or XSM file.
* IPMIC  Adress of the L_LIBRARY in creation mode.
* IPMIC2 Adress of the fuel-map L_LIBRARY in read-only mode.
* IPMIC3 Adress of the L_LIBRARY in read-only mode, containing new 
*        fuel properties.
* NCH    Number of channels
* NK     Number of bundles per channel
* NX     Number of X-Meshes
* NY     Number of Y-Meshes
* NREG   Number of regions in fuel map geometry
* NAMCHA Name of the channel to refuel
* NS     Number of bundles inserted
* MS     Old maximum of shift + 1.
* MIX    Fuel map bundle index
* IXN    Name of the channel according to X
* IYN    Name of the channel according to Y
* POW    Power distribution.
* INDEX  Fuel type indice
* IND    Fuel type indice in the channel to refuel
* MAXS   Maximum number of power shift
* IPRT   Flag for printing level
* KRF    Type of refueling
* LMIC   =.true. for a micro-refueling
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----

      INTEGER    IPRES,NCH,NK,NX,NY,NS,NREG,ILONG,ITYP,IX,IY,IPRT,
     1           MS,IS,MAXS,KS,NSS(NCH),NNS,IPMIC,IPMIC2,IPMIC3
      REAL       WINT(NCH,NK),WORK(NK,2),BS(NCH,NK,MS),
     1           PS(NCH,NK,MS),POW(NCH,NK),WORKS(NK,MS,2)
      CHARACTER  XNAM*4,YNAM*4,NAMCHA*4,TEXT4*4,CS*2
      INTEGER    MIX(NREG),IXN(NX),IYN(NY),NW(NK),NW2(NK),ISFT(NCH,NK),
     1           IWORK(NK,2),INDEX(NCH,NK),IND(*)
      LOGICAL    LMIC,LBASE(1)
      REAL TMPDAY(3)
      DIMENSION  IBASE(1)
      COMMON     BASE(1)
      EQUIVALENCE(BASE(1),LBASE(1),IBASE(1))
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,IOUT=6)
      INTEGER   ISTATE(NSTATE),I,J

* RECOVER SHIFT VECTOR
      CALL LCMLEN(IPRES,'ISHIFT',ILS,ITYLCM)
      IF(ILS.NE.0) THEN
        CALL LCMGET(IPRES,'ISHIFT',ISFT(1,1))
        DO 17 I=1,NK
         DO 17 J=1,NCH
            MAXS=MAX(MAXS,ISFT(J,I))
 17     CONTINUE
      ELSE
        MAXS=0
        DO 15 I=1,NK
          DO 15 J=1,NCH
            ISFT(J,I) = 0
 15     CONTINUE
      ENDIF
      DO 1 I=1,NK
       DO 1 J=1,NCH
         WINT(J,I) = 0.0
         DO 2 K=1,MS
           BS(J,I,K)=0.0
           PS(J,I,K)=0.0
  2      CONTINUE
  1   CONTINUE
* RECOVER FUEL BURNUPS
      CALL LCMLEN(IPRES,'BURN-INST',ILONG,ITYP)
      IF(ILONG.EQ.0) THEN
         CALL XABORT('@TINREF: INITIAL BURNUP REQUIRED')
      ENDIF
      CALL LCMGET(IPRES,'BURN-INST',WINT)
* RECOVER FUEL INDEX
      CALL LCMLEN(IPRES,'FLMIX',ILONG,ITYP)
      IF(ILONG.NE.0) THEN
         CALL LCMGET(IPRES,'FLMIX',INDEX)
      ELSE
        CALL XABORT('@TINREF: FLMIX ARE REQUIRED')
      ENDIF

      IF(MAXS.GT.0) THEN
        DO 16 IS=1,MAXS
          WRITE (CS,'(I2)') IS
          CALL LCMGET(IPRES,'BSHIFT'//CS,BS(1,1,IS))
          CALL LCMGET(IPRES,'PSHIFT'//CS,PS(1,1,IS))
 16     CONTINUE
      ENDIF

      CALL LCMSIX(IPRES,' ',0)
      CALL LCMGET(IPRES,'BMIX',MIX)
*
*     SEARCH FOR THE CHANNEL NUMBER FROM ITS NAME
*
      TEXT4 = NAMCHA(2:3)
      IX = 1
      IY = 1
      DO 10 I=1,NX
        WRITE(XNAM,'(A4)') IXN(I)
        IF (XNAM.EQ.TEXT4) THEN
           IX = I
           GOTO 11
        ENDIF
  10  CONTINUE
  11  TEXT4 = NAMCHA(1:1)
      DO 20 I=1,NY
        WRITE(YNAM,'(A4)') IYN(I)
        IF (YNAM.EQ.TEXT4) THEN
           IY = I
           GOTO 21
        ENDIF
  20  CONTINUE

  21  I = (IY-1)*NX + IX
      ICH = MIX(I)
      IF(ICH.EQ.0) CALL XABORT('@TINREF: WRONG CHANNEL NAME')
      IF(NSS(ICH).NE.0) THEN
        IF(ABS(NSS(ICH)).NE.ABS(NS))
     +       CALL XABORT('@TINREF: WRONG REFUELING SCHEME')
        NS = NSS(ICH)
      ENDIF
      IF( IPRT.GT.3 )THEN
        WRITE(6,*) ' REFUELING CHANNEL ',NAMCHA,' IX IY ',IX,IY
        WRITE(6,*) ' REFUELING CHANNEL ',ICH,' SCHEME ',NS
        WRITE(6,*) ' INITIAL BURNUP ',(WINT(ICH,I),I=1,NK)
      ENDIF

      NNS = ABS(NS)
      CALL TINFL(NNS,NW,NW2,0,NK,ICH)

      II=0
      DO 30 K=1,NK
         KK = K
         IF (NS.LT.0) THEN
            KK = NK - K + 1
         ENDIF
         KA = NW(K)
*----
*  INSERTION OF A NEW BUNDLE OR REPOSITIONNING
*----
         IF (KA.EQ.0) THEN
            II=II+1
            WORK(KK,1) = 0.0
            IWORK(KK,1)=0
            IF( KRF.EQ.1 )THEN
              IWORK(KK,2)=INDEX(ICH,KK)
            ELSE
              IWORK(KK,2)=IND(II)
            ENDIF
            IF(MAXS.GT.0) THEN
              DO 39 IS=1,MAXS
                WORKS(KK,IS,1) = 0.0
                WORKS(KK,IS,2) = 0.0
  39          CONTINUE
            ENDIF
         ELSE
            IF (NS.LT.0) THEN
               KA = NK - KA + 1
            ENDIF
            WORK(KK,1) = WINT(ICH,KA)
            WORK(KK,2) = POW(ICH,KA)
            IWORK(KK,1)= ISFT(ICH,KA)
            IWORK(KK,2)= INDEX(ICH,KA)
            IF(MAXS.GT.0) THEN
              DO 19 IS=1,MAXS
                WORKS(KK,IS,1) = BS(ICH,KA,IS)
                WORKS(KK,IS,2) = PS(ICH,KA,IS)
  19          CONTINUE
            ENDIF
         ENDIF
  30  CONTINUE

      DO 40 K=1,NK
        WINT(ICH,K) = WORK(K,1)
        POW(ICH,K) = WORK(K,2)
        ISFT(ICH,K) = IWORK(K,1)
        INDEX(ICH,K) = IWORK(K,2)
        IF(MAXS.GT.0) THEN
          DO 22 IS=1,MAXS
            BS(ICH,K,IS)=WORKS(K,IS,1)
            PS(ICH,K,IS)=WORKS(K,IS,2)
  22      CONTINUE
        ENDIF
        IF(WORK(K,1).NE.0.0) THEN
          KS=ISFT(ICH,K)+1
          BS(ICH,K,KS)=WINT(ICH,K)
          PS(ICH,K,KS)=WORK(K,2)
          ISFT(ICH,K)=KS
        ENDIF
  40  CONTINUE

      MAXS=0
      DO 12 I=1,NK
        DO 12 J=1,NCH
          MAXS=MAX(MAXS,ISFT(J,I))
  12  CONTINUE

*----
* CALL THE SUBROUTINE FOR A MICROSCOPIC REFUEL
*----
      IF(LMIC) THEN
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPMIC2,'STATE-VECTOR',ISTATE)
        NISO=ISTATE(2)
        NDEP=ISTATE(12)
        IF(NDEP.NE.NK*NCH) CALL XABORT('@TINREF: WRONG NUMBER OF '
     +  //'DEPLETING MIXTURES IN THE LIBRARY.')
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPMIC3,'STATE-VECTOR',ISTATE)
        NISO2=ISTATE(2)
        CALL SETARA(IBASE,NK,INWU)
        DO I=1,NK
          IF(NS.GT.0) THEN
            IBASE(INWU+I-1)=NW(I)
          ELSE
            IBASE(INWU+I-1)=NW2(I)
          ENDIF
        ENDDO

        CALL SETARA(IBASE,NISO,IMIX)
        CALL SETARA(IBASE,3*NISO,INAME)
        CALL SETARA(IBASE,3*NISO,IUSED)
        CALL SETARA(BASE,NISO,IDENS)
        CALL SETARA(IBASE,NISO,ITODO)
        CALL SETARA(IBASE,NISO,ITYP)
        CALL SETARA(IBASE,NISO,ITEMP)
        CALL SETARA(IBASE,NISO,IVOL)

        CALL SETARA(IBASE,NISO2,IMIX2)
        CALL SETARA(IBASE,3*NISO2,INAME2)
        CALL SETARA(IBASE,3*NISO2,IUSED2)
        CALL SETARA(BASE,NISO2,IDENS2)
        CALL SETARA(IBASE,NISO2,ITODO2)
        CALL SETARA(IBASE,NISO2,ITYP2)
        CALL SETARA(IBASE,NISO2,ITEMP2)
        CALL SETARA(IBASE,NISO2,IVOL2)

        CALL SETARA(BASE,NISO,INDENS)

        CALL SETARA(IBASE,NK,ISHF)
        
        CALL TINMIC(IPMIC,IPMIC2,IPMIC3,NK,NCH,IBASE(INWU),NS,ICH,NISO,
     1   NISO2,IWORK,IBASE(IMIX),IBASE(INAME),IBASE(IUSED),BASE(IDENS),
     2   IBASE(ITODO),IBASE(ITYP),IBASE(ITEMP),IBASE(IVOL),IBASE(IMIX2),
     3   IBASE(INAME2),IBASE(IUSED2),BASE(IDENS2),IBASE(ITODO2),
     4   IBASE(ITYP2),IBASE(ITEMP2),IBASE(IVOL2),IBASE(ISHF),
     5   BASE(INDENS))

        CALL LCMPUT(IPMIC,'ISOTOPESDENS',NISO,2,BASE(INDENS))
*----
*  COMPUTE THE MACROSCOPIC X-SECTIONS
*----
      CALL LCMGET(IPMIC,'STATE-VECTOR',ISTATE)
      MAXMIX=ISTATE(1)
      NBISO=ISTATE(2)
      NGRP=ISTATE(3)
      CALL SETARA(LBASE,MAXMIX,IMASK)
      CALL SETARA(LBASE,NGRP,IMASKL)
      CALL LCMIOF(IPMIC,'ISOTOPESUSED',IISONA)
      CALL LCMIOF(IPMIC,'ISOTOPESMIX',IISOMI)
      CALL LCMIOF(IPMIC,'ISOTOPESDENS',IDENIS)
      CALL XDLSET(LBASE(IMASK),MAXMIX,.FALSE.)
      CALL XDLSET(LBASE(IMASKL),NGRP,.TRUE.)
      DO 13 I=1,NBISO
        IBM=IBASE(IISOMI+I-1)
        LBASE(IMASK+IBM-1)=.TRUE.
  13  CONTINUE
      ITSTMP=0
      TMPDAY(1)=0.0
      TMPDAY(2)=0.0
      TMPDAY(3)=0.0
*     CALL THE DRAGON SUBROUTINE FOR THE COMPUTATION OF THE MACROSCOPIC XS
      CALL LIBMIX(IPMIC,MAXMIX,NGRP,NBISO,IBASE(IISONA),IBASE(IISOMI),
     1         BASE(IDENIS),LBASE(IMASK),LBASE(IMASKL),ITSTMP,TMPDAY)
      CALL RLSARA(LBASE(IMASKL))
      CALL RLSARA(LBASE(IMASK))

        CALL RLSARA(IBASE(INWU))
        CALL RLSARA(IBASE(IMIX))
        CALL RLSARA(IBASE(INAME))
        CALL RLSARA(IBASE(IUSED))
        CALL RLSARA(BASE(IDENS))
        CALL RLSARA(IBASE(ITODO))
        CALL RLSARA(IBASE(ITYP))
        CALL RLSARA(IBASE(ITEMP))
        CALL RLSARA(IBASE(IVOL))

        CALL RLSARA(IBASE(IMIX2))
        CALL RLSARA(IBASE(INAME2))
        CALL RLSARA(IBASE(IUSED2))
        CALL RLSARA(BASE(IDENS2))
        CALL RLSARA(IBASE(ITODO2))
        CALL RLSARA(IBASE(ITYP2))
        CALL RLSARA(IBASE(ITEMP2))
        CALL RLSARA(IBASE(IVOL2))

        CALL RLSARA(BASE(INDENS))
        CALL RLSARA(IBASE(ISHF))

      ENDIF

      IF( IPRT.GT.3 )THEN
        WRITE(6,*) ' SHIFTING BURNUP ',(WINT(ICH,I),I=1,NK)
      ENDIF

      CALL LCMSIX(IPRES,' ',0)
      IF(IPRT.GT.3) WRITE(6,*) ' REFUELLING TYPE DIRECT OR HOMOGENOUS'
      CALL LCMPUT(IPRES,'BURN-INST',NCH*NK,2,WINT(1,1))
      CALL LCMPUT(IPRES,'FLMIX',NCH*NK,1,INDEX(1,1))
      CALL LCMPUT(IPRES,'ISHIFT',NCH*NK,1,ISFT(1,1))

      IF(MAXS.GT.0) THEN
        DO 14 IS=1,MAXS
          WRITE (CS,'(I2)') IS
          CALL LCMPUT(IPRES,'BSHIFT'//CS,NCH*NK,2,BS(1,1,IS))
          CALL LCMPUT(IPRES,'PSHIFT'//CS,NCH*NK,2,PS(1,1,IS))
 14    CONTINUE
      ENDIF

      RETURN
      END
