*DECK TINST
      SUBROUTINE TINST(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform computations according to the time-linear model.
*
*Copyright:
* Copyright (C) 2009 Ecole Polytechnique de Montreal
*
*Author(s): B. Toueg, M. Guyot
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): modification type(L_MAP);
*         HENTRY(2): read-only type(L_POWER).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6)
      CHARACTER TEXT*12,HSIGN*12,NAMCHA*4,NAMCHA2*4
      INTEGER KCHAR(3),IMPX,MSHT,NB,NCH,IMOD,NCOMB,
     +         NF,NX,NY,NZ,MAXS,ITYP,NREG,KREF,I,ISTATE(NSTATE),
     +         LENGT,IPPOW,IPMAP,NS,NSS,NITMA,IBUNDMIX,IFLMIX,IBUNDPOW,
     +         IFLWEIGHT,IBURNINST,INSS,ILONG,IXN,IYN,IW,IMIX,IBS,IPS,
     +         IVS,IDX,IPOW,ICDX,IWK,IWKS,INW,IPOOL,IIWK,JPMAP,IPMIC,
     +         IPMIC2,IPMIC3
      DOUBLE PRECISION DFLOT
      LOGICAL LNOTHING,LMIC
      REAL TIME,BURNSTEP,FLOT,BASE
      DIMENSION  IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF((NENTRY.LT.1).AND.(NENTRY.GT.5)) CALL XABORT('@TINST: WRONG '
     +  //'NUMBER OF PARAMETERS')
      IPMAP=0
      IPPOW=0
      IPMIC=0
      IPMIC2=0
      IPMIC3=0
      IF(JENTRY(1).EQ.0) THEN
        IPMIC=KENTRY(1)
        I=2
      ELSE
        I=1
      ENDIF
      DO IEN=I,NENTRY
        CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        IF(HSIGN.EQ.'L_MAP') THEN
          IPMAP=KENTRY(IEN)
          IF(JENTRY(IEN).NE.1) CALL XABORT('@TINST: MODIFICATION MODE '
     +    //'FOR L_MAP EXPECTED')
        ELSEIF(HSIGN.EQ.'L_POWER') THEN
          IPPOW=KENTRY(IEN)
          IF(JENTRY(IEN).NE.2) CALL XABORT('@TINST: READ-ONLY MODE '
     +    //'FOR L_POWER EXPECTED')
        ELSEIF(HSIGN.EQ.'L_LIBRARY') THEN
          IF(IPMIC2.EQ.0) THEN
            IPMIC2=KENTRY(IEN)
            CALL LCMEQU(IPMIC2,IPMIC)
            IF(JENTRY(IEN).NE.2) CALL XABORT('@TINST: READ-ONLY MODE'
     +      //' FOR SECOND L_LIBRARY EXPECTED')
          ELSE
            IPMIC3=KENTRY(IEN)
            IF(JENTRY(IEN).NE.2) CALL XABORT('@TINST: READ-ONLY MODE '
     +      //'FOR THIRD L_LIBRARY EXPECTED')
          ENDIF
        ENDIF          
      ENDDO
*----
*  RECOVER INFORMATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
      NB=ISTATE(1)
      NCH=ISTATE(2)
      NCOMB=ISTATE(3)
      IMOD=ISTATE(5)
      MAXS=ISTATE(6)
      MSHT=MAXS+1
      NF=ISTATE(7)

*----
*  ONLY TIME INSTANTANEOUS CALCULATIONS IN TINST:
*----
      IF(IMOD.NE.2)
     +    CALL XABORT('@TINST: INST-BURN OPTION '
     + //'SHOULD BE USED IN RESINI.')

      JPMAP=LCMGID(IPMAP,'GEOMAP')
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(JPMAP,'STATE-VECTOR',ISTATE)
      NX=ISTATE(3)
      NY=ISTATE(4)
      NZ=ISTATE(5)
      NREG = ISTATE(6)
*     CHECK EXISTING DATA
      IF(IPPOW.EQ.0) THEN
        CALL LCMLEN(IPMAP,'BUND-PW',LENGT,ITYP)
        IF(LENGT.EQ.0)CALL XABORT('@TINST: MISSING BUND-PW DATA I'
     +   //'N L_MAP OBJECT.')
      ELSE
        CALL LCMLEN(IPPOW,'POWER-CHAN',LENGT,ITYP)
        IF(LENGT.EQ.0)CALL XABORT('@TINST: MISSING POWER-CHAN DATA I'
     +   //'N L_POWER OBJECT.')
      ENDIF

*----
*  READ INPUT DATA
*----
      IMPX=0
      LNOTHING=.TRUE.
      LMIC=.FALSE.
   2  TIME=0.0
      BURNSTEP=0.0
*     READ KEYWORD
   1  CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@TINST: CHARACTER DATA EXPECTED(1).')
      IF(TEXT.EQ.'EDIT')THEN
*       PRINTING INDEX
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@TINST: INTEGER DATA EXPECTED.')
        IMPX=MAX(0,NITMA)
        GOTO 1
      ELSEIF(TEXT.EQ.'TIME')THEN
*       TIME VALUE
        IF(TIME.NE.0.0)CALL XABORT('@TINST: TIME ALREADY SPECIFIED(1).')
        IF(BURNSTEP.NE.0.0)CALL XABORT('@TINST: BURNSTEP ALREADY //
     +    //SPECIFIED(1).')
        CALL REDGET(ITYP,NITMA,TIME,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@TINST: REAL DATA EXPECTED(1).')
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.3)CALL XABORT('@TINST: CHARACTER DATA EXPECTED(2).')
        IF(TIME.LE.0.)CALL XABORT('@TINST: EXPECTING REAL > 0 (1).')
        IF(TEXT.EQ.'DAY')THEN
          TIME=TIME
        ELSEIF(TEXT.EQ.'HOUR')THEN
          TIME=TIME/24.
        ELSEIF(TEXT.EQ.'MINUTE')THEN
          TIME=TIME/(24.*60.)
        ELSEIF(TEXT.EQ.'SECOND')THEN
          TIME=TIME/(24.*60.*60.)
        ELSE
          CALL XABORT('@TINST: EXPECTING DAY|HOUR|MINUTE|SECOND.')
        ENDIF
        LNOTHING=.FALSE.
        GOTO 10
      ELSEIF(TEXT.EQ.'BURN-STEP')THEN
*       BURN-STEP
        IF(TIME.NE.0.)CALL XABORT('@TINST: TIME ALREADY SPECIFIED(2).')
        IF(BURNSTEP.NE.0.)CALL XABORT('@TINST: BURNSTEP ALREADY '
     +    //'SPECIFIED(2).')
        CALL REDGET(ITYP,NITMA,BURNSTEP,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@TINST: REAL DATA EXPECTED(2).')
        IF(BURNSTEP.LE.0.)CALL XABORT('@TINST: EXPECTING REAL > 0 (2).')
        LNOTHING=.FALSE.
        GOTO 10
      ELSEIF(TEXT.EQ.'REFUEL')THEN
*      REFUEL
           KREF=1
           LNOTHING=.FALSE.
           CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
           IF(ITYP.NE.3)
     +      CALL XABORT('@TINST: CHARACTER DATA EXPECTED(3).')
           IF(TEXT.EQ.'MICRO') THEN
             LMIC=.TRUE.
             CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
           IF(ITYP.NE.3)
     +      CALL XABORT('@TINST: CHARACTER DATA EXPECTED(4).')
           ENDIF
           IF(TEXT.EQ.'CHAN') THEN
             CALL REDGET(ITYP,NITMA,FLOT,NAMCHA,DFLOT)
             IF(ITYP.NE.3)
     +        CALL XABORT('@TINST: CHARACTER DATA EXPECTED(5).')
             CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
             IF(ITYP.NE.1)
     +         CALL XABORT('@TINST: INTEGER DATA EXPECTED(2).')
             NS = NITMA
           ELSE
            CALL XABORT('@TINST: INVALID KEYWORD '//TEXT)
           ENDIF
       GOTO 20
      ELSEIF(TEXT.EQ.'NEWFUEL')THEN
*      NEWFUEL
           KREF=2
           LNOTHING=.FALSE.
           CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
           IF(ITYP.NE.3)
     +      CALL XABORT('@TINST: CHARACTER DATA EXPECTED(4).')
           IF(TEXT.EQ.'CHAN') THEN
             CALL REDGET(ITYP,NITMA,FLOT,NAMCHA,DFLOT)
             IF(ITYP.NE.3)
     +        CALL XABORT('@TINST: CHARACTER DATA EXPECTED(5).')
             CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
             IF(ITYP.NE.1)
     +         CALL XABORT('@TINST: INTEGER DATA EXPECTED(3).')
             NS = NITMA
             NSS=ABS(NS)
             CALL SETARA(IBASE,NSS,IDX)
             CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
             IF(ITYP.NE.3)
     +         CALL XABORT('@TINST: CHARACTER DATA EXPECTED(6).')
             IF(TEXT.EQ.'SOME')THEN
                DO 11 I=1,NSS
                 CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
                 IF(ITYP.NE.1)
     +             CALL XABORT('@TINST: INTEGER DATA EXPECTED(4).')
                IF (NITMA.GT.NF)
     +             CALL XABORT('@TINST: WRONG NUMBER OF FUEL TYPE. ')
                 IBASE(IDX+I-1) = NITMA
 11             CONTINUE
             ELSEIF(TEXT.EQ.'ALL')THEN
                CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
                IF(ITYP.NE.1)
     +             CALL XABORT('@TINST: INTEGER DATA EXPECTED(5).')
                IF (NITMA.GT.NF)
     +             CALL XABORT('@TINST: WRONG NUMBER OF FUEL TYPE. ')
                DO 12 I=1,NSS
                 IBASE(IDX+I-1) = NITMA
 12             CONTINUE
             ELSE
               CALL XABORT('@TINST: INVALID KEYWORD '//TEXT)
             ENDIF
           ELSE
            CALL XABORT('@TINST: INVALID KEYWORD '//TEXT)
           ENDIF
        GOTO 20
*     SHUFFL
      ELSEIF (TEXT.EQ.'SHUFF') THEN
           KREF = 3
           LNOTHING=.FALSE.
           CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
           IF(ITYP.NE.3)
     +      CALL XABORT('@TINST: CHARACTER DATA EXPECTED(7).')
           IF(TEXT.EQ.'CHAN') THEN
             CALL REDGET(ITYP,NITMA,FLOT,NAMCHA,DFLOT)
             IF(ITYP.NE.3)
     +        CALL XABORT('@TINST: CHARACTER DATA EXPECTED(8).')
             CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
             IF(ITYP.NE.3)
     +        CALL XABORT('@TINST: CHARACTER DATA EXPECTED(8).')
             IF(TEXT.EQ.'TO') THEN
                CALL REDGET(ITYP,NITMA,FLOT,NAMCHA2,DFLOT)
                IF(ITYP.NE.3)
     +            CALL XABORT('@TINST: CHARACTER DATA EXPECTED(9).')
                IF(IMPX.GT.2)
     +            WRITE(6,*) 'TINST : ACTION ',NAMCHA,' TO ',NAMCHA2
             ELSE
               CALL XABORT('@TINST: INVALID KEYWORD '//TEXT)
             ENDIF
           ELSE
             CALL XABORT('@TINST: INVALID KEYWORD '//TEXT)
           ENDIF
        GOTO 20
      ELSEIF(TEXT.EQ.';')THEN
        GOTO 30
      ELSE
*       KEYWORD DOES NOT MATCH
        CALL XABORT('@TINST: WRONG KEYWORD:'//TEXT//'.')
      ENDIF

*----
*  PERFORM CALCULATION
*----
   10 CALL SETARA(IBASE,NCH*NB,IBUNDMIX)
      CALL SETARA(IBASE,NF,IFLMIX)
      CALL SETARA(BASE,NF,IFLWEIGHT)
      CALL SETARA(BASE,NCH*NB,IBUNDPOW)
      IF(IPPOW.EQ.0) THEN
        CALL LCMGET(IPMAP,'BUND-PW',BASE(IBUNDPOW))
      ELSE 
       CALL LCMGET(IPPOW,'POWER-BUND',BASE(IBUNDPOW))
      ENDIF
      IF(LMIC) CALL XABORT('@TINST: NO MICRO-DEPLETION ')
      CALL SETARA(BASE,NCH*NB,IBURNINST)
      CALL TINSTB(IPMAP,IPPOW,TIME,BURNSTEP,NCH,NB,NF,IMPX,
     + IBASE(IBUNDMIX),IBASE(IFLMIX),BASE(IFLWEIGHT),BASE(IBUNDPOW),
     + BASE(IBURNINST))
      CALL RLSARA(IBASE(IBUNDMIX))
      CALL RLSARA(IBASE(IFLMIX))
      CALL RLSARA(BASE(IFLWEIGHT))
      CALL RLSARA(BASE(IBUNDPOW))
      CALL RLSARA(BASE(IBURNINST))
      GOTO 1

   20 CALL LCMSIX(IPMAP,' ',0)
      CALL SETARA(IBASE,NCH,INSS)
      CALL LCMLEN(IPMAP,'REF-SCHEME',ILONG,ITYP)
      IF(ILONG.EQ.0) THEN
         DO 25 I=1,NCH
            IBASE(INSS+I-1) = 0
   25    CONTINUE
      ELSEIF(ILONG.NE.NCH) THEN
         CALL XABORT('@TINST: REF-SCHEME HAS NOT THE CORRECT LENGHT')
      ELSE
         CALL LCMGET(IPMAP,'REF-SCHEME',IBASE(INSS))
      ENDIF
      CALL LCMSIX(IPMAP,' ',0)
      CALL SETARA(IBASE,NX,IXN)
      CALL SETARA(IBASE,NY,IYN)
      CALL LCMSIX(IPMAP,' ',0)
      CALL LCMGET(IPMAP,'XNAME',IBASE(IXN))
      CALL LCMGET(IPMAP,'YNAME',IBASE(IYN))
      CALL SETARA(BASE,NCH*NB,IW)
      CALL SETARA(IBASE,NREG,IMIX)
      CALL SETARA(BASE,NCH*NB*MSHT,IBS)
      CALL SETARA(BASE,NCH*NB*MSHT,IPS)
      CALL SETARA(IBASE,NCH*NB*MSHT,IVS)
      IF(KREF.EQ.1.OR.KREF.EQ.2) THEN
        IF(KREF.EQ.1)
     +     CALL SETARA(IBASE,ABS(NS),IDX)
        CALL SETARA(BASE,NCH*NB,IPOW)
        IF(IPPOW.EQ.0) THEN
        CALL LCMGET(IPMAP,'BUND-PW',BASE(IPOW))
        ELSE 
         CALL LCMGET(IPPOW,'POWER-BUND',BASE(IPOW))
        ENDIF
        CALL SETARA(IBASE,NCH*NB,ICDX)
        CALL SETARA(BASE,NB*2,IWK)
        CALL SETARA(BASE,NB*2*MSHT,IWKS)
        CALL SETARA(IBASE,NB*2,IIWK)
        CALL SETARA(IBASE,NB,INW)
        CALL SETARA(IBASE,NB,INW2)
        CALL TINREF(IPMAP,IPMIC,IPMIC2,IPMIC3,NCH,NB,NX,NY,NREG,NAMCHA,
     +             NS,MSHT,BASE(IW),IBASE(IMIX),IBASE(IXN),IBASE(IYN),
     +             IBASE(INW),IBASE(INW2),BASE(IWK),BASE(IBS),BASE(IPS),
     +             IBASE(IVS),BASE(IPOW),IBASE(IIWK),BASE(IWKS),MAXS,
     +             IBASE(INSS),IBASE(IDX),IBASE(ICDX),IMPX,KREF,LMIC)
        CALL RLSARA(IBASE(IIWK))
        CALL RLSARA(BASE(IWKS))
        CALL RLSARA(BASE(IWK))
        CALL RLSARA(IBASE(INW))
        CALL RLSARA(IBASE(INW2))
        CALL RLSARA(IBASE(ICDX))
        CALL RLSARA(BASE(IPOW))
        CALL RLSARA(IBASE(IDX))
      ELSE
        CALL SETARA(BASE,NCH*NB,IPOOL)
        CALL TINSHU(IPMAP,NCH,NB,NX,NY,NREG,MSHT,NAMCHA,NAMCHA2,
     +     BASE(IW),IBASE(IMIX),BASE(IBS),BASE(IPS),
     +     IBASE(IVS),IBASE(IXN),IBASE(IYN),BASE(IPOOL),IMPX)
        CALL RLSARA(BASE(IPOOL))
      ENDIF
      CALL RLSARA(BASE(IBS))
      CALL RLSARA(BASE(IPS))
      CALL RLSARA(BASE(IVS))
      CALL RLSARA(IBASE(IMIX))
      CALL RLSARA(IBASE(IW))
      CALL RLSARA(IBASE(INSS))
      CALL RLSARA(IBASE(IXN))
      CALL RLSARA(IBASE(IYN))
      MSHT=MAXS+1
      KREF=0
      GOTO 2

  30  IF(LNOTHING)CALL XABORT('@TINST: NO OPTION SPECIFIED.')
      CALL LCMSIX(IPMAP,' ',0)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
      ISTATE(6)=MAXS
      CALL LCMPUT(IPMAP,'STATE-VECTOR',NSTATE,1,ISTATE)

      RETURN
      END
