*DECK TINSTB
      SUBROUTINE TINSTB(IPMAP,IPPOW,TIME,BURNSTEP,NCH,NB,NF,IMPX,
     1             BUNDMIX,FLMIX,FLWEIGHT,BUNDPOW,BURNINST)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute new burnup for each bundle given either an average burnup step
* or a burning time
*
*Copyright:
* Copyright (C) 2009 Ecole Polytechnique de Montreal
*
*Author(s): B. Toueg
*
*Parameters: input/output
* IPMAP    pointer to fuel-map information.
* IPPOW    pointer to power information.
* TIME     time to burn
* BURNSTEP average burnup step
* NCH      number of reactor channels.
* NB       number of fuel bundles.
* NF       number of fuel types
* IMPX     printing index (=0 for no print).
*
*Parameters: scratch
* BUNDMIX
* FLMIX
* FLWEIGHT
* BUNDPOW
* BURNINST
* NENTRY
*-----------------------------------------------------------------------

*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAP,IPPOW,NCH,NB,NF,IMPX
      INTEGER BUNDMIX(NCH,NB), FLMIX(NF)
      REAL FLWEIGHT(NF), BUNDPOW(NCH,NB), BURNINST(NCH,NB)
      REAL TIME,BURNSTEP, BURNAVG, PTOT, MASSTOT, WEIGHT
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6)
      INTEGER JPMAP,KPMAP,IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  RECOVER INFORMATION
*----
      CALL LCMLIB(IPMAP)
*     FUEL MIX
      CALL XDISET(BUNDMIX,NCH*NB,0)
      CALL LCMGET(IPMAP,'FLMIX',BUNDMIX)
*     BURN-INST
      CALL XDRSET(BURNINST,NCH*NB,0.)
      CALL LCMGET(IPMAP,'BURN-INST',BURNINST)
*     BUNDLE POWERS
*      CALL XDRSET(BUNDPOW,NCH*NB,0.)
*      CALL LCMGET(IPPOW,'POWER-BUND',BUNDPOW)
*     FUEL INFORMATION (WEIGHT & MIX)
      JPMAP=LCMLID(IPMAP,'FUEL',NF)
      MAXFL=0 ! maximum fuel mix number
      DO IFL=1,NF
        KPMAP=LCMDIL(JPMAP,IFL)
        CALL LCMGET(KPMAP,'MIX',FLMIX(IFL))
        MAXFL=MAX(MAXFL,FLMIX(IFL))
        CALL LCMGET(KPMAP,'WEIGHT',FLWEIGHT(IFL))
      ENDDO
      IF(MAXFL.LT.NF)THEN
        CALL XABORT('@TINSTB: FOUND MAX FUEL MIX NUMBER : (',MAXFL,
     1  ') THOUGH ',NF,' FUEL MIXES ARE DEFINED')
      ENDIF
* the mix stored in FLMIX field of /FMAP/
* is not the rank of the fuel in FUEL Dir list of /FMAP/
      CALL SETARA(IBASE,MAXFL,IFLRANK)
      CALL XDISET(IBASE(IFLRANK),MAXFL,0)
      DO IFL=1,NF
        IBASE(IFLRANK+FLMIX(IFL)-1)=IFL
      ENDDO
*----
*  COMPUTE BURNAVG, PTOT, MASSTOT, ( TIME if BURNSTEP is specified)
*----
      BURNAVG=0.
      PTOT=0.
      MASSTOT=0.
      DO ICH=1,NCH
        DO IB=1,NB
          IFL=IBASE(IFLRANK+BUNDMIX(ICH,IB)-1)
          WEIGHT = FLWEIGHT(IFL)
          BURNAVG=BURNAVG+BURNINST(ICH,IB)
          PTOT=PTOT+BUNDPOW(ICH,IB)
          MASSTOT=MASSTOT+WEIGHT
        ENDDO
      ENDDO
      BURNAVG=BURNAVG/(NCH*NB)
      IF(TIME.EQ.0.)THEN
        TIME = BURNSTEP*MASSTOT/PTOT
      ENDIF
      IF(IMPX.GT.0)THEN
        WRITE(IOUT,*)'@TINSTB: TOTAL POWER = ',PTOT,' kW'
        WRITE(IOUT,*)'@TINSTB: TOTAL FUEL MASS = ',MASSTOT,' kg'
        WRITE(IOUT,*)'@TINSTB: AVERAGE BURN UP BEFORE = ',
     1  BURNAVG,'MWd/t'
      ENDIF
*----
*  COMPUTE NEW BURN-INST GIVEN TIME
*----
      BURNAVG=0.
      DO ICH=1,NCH
        DO IB=1,NB
          IFL=IBASE(IFLRANK+BUNDMIX(ICH,IB)-1)
          WEIGHT = FLWEIGHT(IFL)
          IF(WEIGHT.GT.0.)THEN
          BURNINST(ICH,IB)=BURNINST(ICH,IB)
     1    +(BUNDPOW(ICH,IB)/WEIGHT)*TIME
          BURNAVG=BURNAVG+BURNINST(ICH,IB)
          ELSE
            IF(IMPX.GT.0)THEN
              WRITE(IOUT,*)'@TINSTB: WARNING MIX ',
     1        BUNDMIX(ICH,IB),' WEIGHS ',WEIGHT,'kg'
            ENDIF
          ENDIF
        ENDDO
      ENDDO
      BURNAVG=BURNAVG/(NCH*NB)
      IF(IMPX.GT.0)THEN
        WRITE(IOUT,*)'@TINSTB: AVERAGE BURN UP AFTER = ',BURNAVG,'MWd/t'
      ENDIF
      CALL LCMPUT(IPMAP,'BURN-INST',NCH*NB,2,BURNINST)
*----
*  RELEASE MEMORY AND RETURN
*----
      CALL RLSARA(IBASE(IFLRANK))
      RETURN
      END
