*DECK B1BETA
      DOUBLE PRECISION FUNCTION B1BETA(IAPROX,B2,SIG,DD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the beta function for a P1 or B1 calculation.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IAPROX  type of beta function calculation:
*         =0: LKRD; =1: P0 or P1; =2: B0 or B1.
* B2      buckling.
* SIG     total macroscopic cross section.
* DD      leakage coefficient (used if IAPROX=0).
*
*Parameters: output
* B1BETA  value of the beta function.
*
*-----------------------------------------------------------------------
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IAPROX
      DOUBLE PRECISION B2,SIG,DD
*----
*  LOCAL VARIABLES
*----
      CHARACTER HSMG*131
*
      SIG2=SIG*SIG
      IF (IAPROX.EQ.0) THEN
*        LKRD APPROXIMATION (IMPOSED DIFFUSION COEFFICIENT).
         B1BETA=DD/(SIG+DD*B2)
      ELSE IF (IAPROX.EQ.1) THEN
*        P0 OR P1 APPROXIMATION
         B1BETA=1.0D0/(3.0D0*SIG2+B2)
      ELSE IF (IAPROX.EQ.2) THEN
*        B0 OR B1 APPROXIMATION
         IF (B2.GT.0.05D0*SIG2) THEN
            TMP=SQRT(B2)/SIG
            B1BETA=(1.0D0-ATAN(TMP)/TMP)/B2
         ELSE IF ((B2.LT.-0.05D0*SIG2).AND.(B2.GE.-0.995D0*SIG2)) THEN
            TMP=SQRT(-B2)
            B1BETA=(1.0D0-0.5D0*SIG*LOG((SIG+TMP)/(SIG-TMP))/TMP)/B2
         ELSE IF((B2.LE.0.05D0*SIG2).AND.(B2.GE.-0.05D0*SIG2)) THEN
            TMP=B2/SIG2
            B1BETA=(1.0D0-3.0D0*TMP*(0.2D0-TMP/7.0D0+TMP*TMP/9.0D0-TMP
     1      *TMP*TMP/11.0D0))/(3.0D0*SIG2)
         ELSE
            WRITE(HSMG,'(22HB1BETA: THE BUCKLING (,1P,E12.4,8H) IS SMA,
     1      9HLLER THAN,E12.4,1H.)') B2,-0.995D0*SIG2
            CALL XABORT(HSMG)
         ENDIF
      ELSE
         CALL XABORT('B1BETA: INVALID VALUE OF IAPROX.')
      ENDIF
      RETURN
      END
