*DECK B1HOM
      SUBROUTINE B1HOM (IPMACR,NUNKNO,OPTION,TYPE,NGRO,IPAS,NBM,NFISSI,
     1                  VOL,MAT,KEYFLX,FLUX,REFKEF,IMPX,D,GAMMA,ALAM1,
     2                  ST,PHI,XSCAT,IJJ,NJJ,IPOS,INORM,B2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* homogenization of the unit cell and solution of the B-n equations.
* The cross section information is found on LCM.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPMACR  pointer to the macrolib LCM object (L_MACROLIB signature).
* NUNKNO  number of flux/current unknowns.
* OPTION  type of leakage coefficients; can be 'LKRD' (imposed leakage
*         coefficients), 'B0' (B-0), 'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
*         or 'B0TR' (B-0 with transport correction).
* TYPE    type of buckling iteration.
*         Can be 'DIFF' (do a B-0 calculation of D(NGRO) and exit);
*                'K' (do a B-n calculation with keff search);
*                'B' (do a B-n calculation with buckling search);
*                'L' (do a B-n calculation with buckling search
*                     for a problem with few or no fission).
* NGRO    number of groups.
* IPAS    number of volumes.
* NBM     number of mixtures.
* NFISSI  maximum number of fission spectrum assigned to a mixture.
* VOL     volumes.
* MAT     mixture number of each volume.
* KEYFLX  position of each flux in the unknown vector.
* FLUX    direct unknown vector.
* REFKEF  target K-effective fot type B or type L calculations
* IMPX    print flag.
* INORM   type of leakage model:
*         =1: Diffon; =2: Ecco; =3: Tibere.
* B2      original direction dependant buckling.
*
*Parameters: output
* D       diffusion coefficients.
* GAMMA   gamma factors.
* ALAM1   effective multiplication factor.
* ST      homogeneized total cross sections.
*
*Parameters: scratch
* PHI
* XSCAT   scattering vector (XSCAT(0)=0.0).
* NJJ     number of scattering group  (NJJ(0)=-NGRO).
* IJJ     last scattering group (IJJ(0)=0).
* IPOS    position self scattering in XSCAT (IPOS(0)=NGRO+1).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER*4 OPTION,TYPE
      INTEGER IPMACR,NUNKNO,NGRO,IPAS,NBM,NFISSI,MAT(IPAS),KEYFLX(IPAS),
     1 IMPX,IJJ(0:NBM),NJJ(0:NBM),IPOS(0:NBM),INORM
      REAL VOL(IPAS),FLUX(NUNKNO,NGRO),D(NGRO),GAMMA(NGRO),ST(NGRO),
     1 IDEL(2),B2(4),XSCAT(0:NBM*NGRO)
      DOUBLE PRECISION REFKEF,ALAM1,PHI(*)
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION B2HOM,CAET,A2,CURN,B2T(3)
      COMMON BASE(1)
*
      IAN=0
      IF ((OPTION.EQ.'B0').OR.(OPTION.EQ.'P0')) THEN
         IAN=0
      ELSE IF ((OPTION.EQ.'B1').OR.(OPTION.EQ.'P1')) THEN
         IAN=1
      ELSE IF (OPTION.EQ.'B0TR') THEN
         IAN=-1
      ENDIF
      CALL SETARA(BASE,NGRO,IIJJ0)
      CALL SETARA(BASE,NGRO,IIJJ1)
      CALL SETARA(BASE,NGRO,INJJ0)
      CALL SETARA(BASE,NGRO,INJJ1)
      CALL B1HXS1(NUNKNO,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,KEYFLX,
     1 FLUX,BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),IDEL,IJJ,
     2 NJJ)
*
      CALL SETARA(BASE,1+NBM,IGAR)
      CALL SETARA(BASE,2*NGRO,IGAF)
      CALL SETARA(BASE,2*NFISSI*(NBM+1),IA14)
      CALL SETARA(BASE,NGRO,ISA)
      CALL SETARA(BASE,NGRO,ISFNU)
      CALL SETARA(BASE,NGRO,IXHI)
      CALL SETARA(BASE,IDEL(1),ISCAT0)
      CALL SETARA(BASE,IDEL(2),ISCAT1)
      CALL SETARA(BASE,2*NGRO,ICUR)
      IF(INORM.EQ.2) THEN
*        ECCO-TYPE ISOTROPIC STREAMING.
         CALL B1HXS3(NUNKNO,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1   KEYFLX,FLUX,BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),
     2   IDEL,XSCAT,IJJ,NJJ,IPOS,BASE(IGAR),BASE(IGAF),PHI,BASE(ISA),
     3   ST,BASE(ISFNU),BASE(IXHI),BASE(ISCAT0),BASE(ISCAT1),BASE(ICUR),
     4   NGROIN)
      ELSE IF(INORM.EQ.3) THEN
*        TIBERE-TYPE ANISOTROPIC STREAMING.
         IF(B2(4).EQ.0.0) THEN
            B2T(1)=0.33333333333333D0
            B2T(2)=B2T(1)
            B2T(3)=B2T(1)
         ELSE
            B2T(1)=DBLE(B2(1))/DBLE(B2(4))
            B2T(2)=DBLE(B2(2))/DBLE(B2(4))
            B2T(3)=DBLE(B2(3))/DBLE(B2(4))
         ENDIF
         CALL SETARA(BASE,2*NUNKNO*NGRO,IFL2)
         IOF=0
         DO 30 IGRO=1,NGRO
           DO 10 IUNK=1,NUNKNO/4
           IOF=IOF+1
   10      BASE(IFL2+IOF-1)=FLUX(IUNK,IGRO)
           DO 20 IUNK=1,NUNKNO/4
           IOF=IOF+1
           CURN=0.0D0
           DO 15 IDIR=1,3
   15      CURN=CURN+B2T(IDIR)*FLUX(NUNKNO/4*IDIR+IUNK,IGRO)
   20      BASE(IFL2+IOF-1)=REAL(CURN)
   30    CONTINUE
         CALL B1HXS3(NUNKNO/2,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,
     1   MAT,KEYFLX,BASE(IFL2),BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),
     2   BASE(INJJ1),IDEL,XSCAT,IJJ,NJJ,IPOS,BASE(IGAR),BASE(IGAF),
     3   PHI,BASE(ISA),ST,BASE(ISFNU),BASE(IXHI),BASE(ISCAT0),
     4   BASE(ISCAT1),BASE(ICUR),NGROIN)
         CALL RLSARA(BASE(IFL2))
      ENDIF
      CALL SETARA(BASE,1+NBM*NFISSI,IGARFI)
      CALL B1HXS2(NUNKNO,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1 KEYFLX,FLUX,BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),
     2 IDEL,XSCAT,IJJ,NJJ,IPOS,BASE(IGAR),BASE(IGARFI),BASE(IGAF),
     3 BASE(IA14),PHI,BASE(ISA),ST,BASE(ISFNU),BASE(IXHI),BASE(ISCAT0),
     4 BASE(ISCAT1),BASE(ICUR),NGROIN,INORM)
      CALL RLSARA(BASE(ICUR))
      CALL RLSARA(BASE(IA14))
      CALL RLSARA(BASE(IGAF))
      CALL RLSARA(BASE(IGARFI))
      CALL RLSARA(BASE(IGAR))
*
      B2HOM=DBLE(B2(4))
      CALL SETARA(BASE,2*NGRO*(NGRO+1),IASTOC)
      CALL SETARA(BASE,2*NGRO*NGRO,IBSTOC)
      CALL SETARA(BASE,2*NGRO,ICSTOC)
      CALL SETARA(BASE,2*NGRO,ISAG)
      CALL B1DIF(OPTION,TYPE,NGRO,ST,BASE(ISFNU),BASE(IXHI),
     1 BASE(IIJJ0),BASE(IIJJ1),BASE(INJJ0),BASE(INJJ1),BASE(ISCAT0),
     2 BASE(ISCAT1),REFKEF,IMPX,D,GAMMA,B2HOM,ALAM1,CAET,A2,PHI,
     3 BASE(IASTOC),BASE(IBSTOC),BASE(ICSTOC),BASE(ISAG))
      CALL RLSARA(BASE(ISAG))
      CALL RLSARA(BASE(ICSTOC))
      CALL RLSARA(BASE(IBSTOC))
      CALL RLSARA(BASE(IASTOC))
      B2(4)=REAL(B2HOM)
*
      IF (TYPE.EQ.'DIFF') GO TO 130
*----
*  NORMALIZE THE DRAGON FLUX USING THE FUNDAMENTAL B1 SOLUTION
*----
      IF(INORM.EQ.1) THEN
        DO 60 I=1,NGRO
          CAET=0.0D0
          DO 40 L=1,IPAS
            CAET=CAET+VOL(L)*FLUX(KEYFLX(L),I)
   40     CONTINUE
          CAET=PHI(I)/CAET
          DO 50 L=1,NUNKNO
            FLUX(L,I)=FLUX(L,I)*CAET
   50     CONTINUE
   60   CONTINUE
      ELSE IF(INORM.EQ.2) THEN
        DO 90 I=1,NGRO
          CAET=0.0D0
          CURN=0.0D0
          DO 70 L=1,IPAS
            CAET=CAET+VOL(L)*FLUX(KEYFLX(L),I)
            CURN=CURN+VOL(L)*FLUX(KEYFLX(L)+NUNKNO/2,I)
   70     CONTINUE
          CAET=PHI(I)/CAET
          CURN=PHI(I)*D(I)/CURN
          DO 80 L=1,NUNKNO/2
            FLUX(L,I)=FLUX(L,I)*CAET
            FLUX(L+NUNKNO/2,I)=FLUX(L+NUNKNO/2,I)*CURN
   80     CONTINUE
   90   CONTINUE
      ELSE IF(INORM.EQ.3) THEN
        IF(B2(4).EQ.0.0.OR.
     >    (B2(1).EQ.0.0.AND.B2(2).EQ.0.0.AND.B2(3).EQ.0.0)) THEN
          B2T(1)=0.33333333333333D0
          B2T(2)=B2T(1)
          B2T(3)=B2T(1)
        ELSE
          B2HOM=1.0D0/(DBLE(B2(1))+DBLE(B2(2))+DBLE(B2(3)))
          B2T(1)=B2HOM*DBLE(B2(1))
          B2T(2)=B2HOM*DBLE(B2(2))
          B2T(3)=B2HOM*DBLE(B2(3))
        ENDIF
        DO 120 I=1,NGRO
          CAET=0.0D0
          CURN=0.0D0
          DO 100 L=1,IPAS
          CAET=CAET+VOL(L)*FLUX(KEYFLX(L),I)
          CURN=CURN+B2T(1)*FLUX(KEYFLX(L)+NUNKNO/4,I)*VOL(L)
     >             +B2T(2)*FLUX(KEYFLX(L)+NUNKNO/2,I)*VOL(L)
     >             +B2T(3)*FLUX(KEYFLX(L)+3*NUNKNO/4,I)*VOL(L)
  100     CONTINUE
          CAET=PHI(I)/CAET
          CURN=PHI(I)*D(I)/CURN
          DO 110 L=1,IPAS
          FLUX(KEYFLX(L),I)=FLUX(KEYFLX(L),I)*CAET
          FLUX(KEYFLX(L)+NUNKNO/4,I)=FLUX(KEYFLX(L)+NUNKNO/4,I)*CURN
          FLUX(KEYFLX(L)+NUNKNO/2,I)=FLUX(KEYFLX(L)+NUNKNO/2,I)*CURN
  110     FLUX(KEYFLX(L)+3*NUNKNO/4,I)=FLUX(KEYFLX(L)+3*NUNKNO/4,I)*CURN
  120   CONTINUE
      ENDIF
*
  130 CALL RLSARA(BASE(ISCAT1))
      CALL RLSARA(BASE(ISCAT0))
      CALL RLSARA(BASE(IXHI))
      CALL RLSARA(BASE(ISFNU))
      CALL RLSARA(BASE(ISA))
      CALL RLSARA(BASE(INJJ1))
      CALL RLSARA(BASE(INJJ0))
      CALL RLSARA(BASE(IIJJ1))
      CALL RLSARA(BASE(IIJJ0))
      RETURN
      END
