*DECK BIVAA
      SUBROUTINE BIVAA(IPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT,VOL,
     1 KNORM,SIGT0,SIGW0,DIFF,ILK,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of one-speed finite-difference or finite-element matrices
* for a discretization of the 2D diffusion equation.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the system matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* NREG    total number of merged regions for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NANI    number of Legendre orders for the scattering cross sections.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   not used.
* SIGT0   total macroscopic cross sections ordered by mixture.
* SIGW0   within-group scattering macroscopic cross section ordered
*         by mixture.
* DIFF    diffusion coefficients ordered by mixture.
* ILK     leakage flag (ILK=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER*72 TITR
      LOGICAL ILK
      INTEGER IPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT(NREG),KNORM
      REAL VOL(NREG),SIGT0(0:NBMIX),SIGW0(0:NBMIX,NANI),DIFF(0:NBMIX)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER JPAR(NSTATE),IGB(8),IBASE(1)
      LOGICAL CYLIND,LBIHET
      CHARACTER NAMP*12
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER BIVAC SPECIFIC TRACKING PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      LBIHET=JPAR(40).NE.0
      IF(LBIHET) THEN
         CALL LCMSIX(IPTRK,'BIHET',1)
         CALL LCMGET(IPTRK,'PARAM',IGB)
         IF(NREG.NE.IGB(3)) CALL XABORT('BIVAA: INVALID VALUE OF NREG('
     1   //'1).')
         CALL LCMSIX(IPTRK,' ',2)
      ELSE
         IF(NREG.NE.JPAR(1)) CALL XABORT('BIVAA: INVALID VALUE OF NREG'
     1   //'(2).')
      ENDIF
      ITYPE=JPAR(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=JPAR(8)
      ICOL=JPAR(9)
      ISPLH=JPAR(10)
      LL4=JPAR(11)
      LX=JPAR(12)
      LY=JPAR(13)
      NLF=JPAR(14)
      ISPN=JPAR(15)
      ISCAT=ABS(JPAR(16))
      NVD=JPAR(17)
*
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY,IXX)
      CALL SETARA(BASE,LX*LY,IYY)
      CALL SETARA(BASE,LX*LY,IDD)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL SETARA(IBASE,LL4,IMU)
      IF(ITYPE.EQ.8) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      CALL LCMGET(IPTRK,'MU',IBASE(IMU))
*----
*  COMPUTE THE WITHIN-GROUP SYSTEM MATRICES (LEAKAGE AND REMOVAL).
*----
      IIMAX=IBASE(IMU+LL4-1)
      IF(NLF.GT.0) IIMAX=IIMAX*NLF/2
      CALL SETARA(BASE,IIMAX,ISYS)
      CALL XDRSET(BASE(ISYS),IIMAX,0.0)
      IF(NLF.EQ.0) THEN
*
*  ++++ DIFFUSION THEORY ++++
*
         IF(NANI.GT.1) THEN
	    CALL XABORT('BIVAA: SPN MACRO-CALCULATION EXPECTED(1).')
	 ENDIF
	 CALL SETARA(BASE,3*NBMIX,ISGD)
	 DO 10 IBM=1,NBMIX
         BASE(ISGD+0*NBMIX+IBM-1)=DIFF(IBM)
         BASE(ISGD+1*NBMIX+IBM-1)=DIFF(IBM)
   10    BASE(ISGD+2*NBMIX+IBM-1)=SIGT0(IBM)-SIGW0(IBM,1)
         IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
            CALL LCMSIX(IPTRK,'BIVCOL',1)
            CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
            CALL SETARA(BASE,LC*LC,IR)
            CALL SETARA(BASE,LC*LC,IRS)
            CALL SETARA(BASE,LC*LC,IQ)
            CALL SETARA(BASE,LC*LC,IQS)
            CALL LCMGET(IPTRK,'R',BASE(IR))
            CALL LCMGET(IPTRK,'RS',BASE(IRS))
            CALL LCMGET(IPTRK,'Q',BASE(IQ))
            CALL LCMGET(IPTRK,'QS',BASE(IQS))
            CALL LCMSIX(IPTRK,' ',2)
            CALL BIVA01(0,MAXKN,BASE(ISGD),CYLIND,NREG,LL4,NBMIX,IIMAX,
     1      BASE(IXX),BASE(IYY),BASE(IDD),MAT,IBASE(IKN),BASE(IQFR),
     2      VOL,IBASE(IMU),LC,BASE(IR),BASE(IRS),BASE(IQ),BASE(IQS),
     3      BASE(ISYS))
            CALL RLSARA(BASE(IQS))
            CALL RLSARA(BASE(IQ))
            CALL RLSARA(BASE(IRS))
            CALL RLSARA(BASE(IR))
         ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
            CALL LCMSIX(IPTRK,'BIVCOL',1)
            CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
            CALL SETARA(BASE,LC*LC,IR)
            CALL SETARA(BASE,LC*(LC-1),IV)
            CALL LCMGET(IPTRK,'R',BASE(IR))
            CALL LCMGET(IPTRK,'V',BASE(IV))
            CALL LCMSIX(IPTRK,' ',2)
            CALL BIVA02(0,BASE(ISGD),CYLIND,IELEM,ICOL,NREG,LL4,
     1      NBMIX,IIMAX,BASE(IXX),BASE(IYY),BASE(IDD),MAT,IBASE(IKN),
     2      BASE(IQFR),VOL,IBASE(IMU),LC,BASE(IR),BASE(IV),BASE(ISYS))
            CALL RLSARA(BASE(IV))
            CALL RLSARA(BASE(IR))
         ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
            CALL LCMSIX(IPTRK,'BIVCOL',1)
            CALL SETARA(BASE,4,IR)
            CALL SETARA(BASE,36,IRH)
            CALL SETARA(BASE,36,IQH)
            CALL SETARA(BASE,9,IRT)
            CALL SETARA(BASE,9,IQT)
            CALL LCMGET(IPTRK,'R',BASE(IR))
            CALL LCMGET(IPTRK,'RH',BASE(IRH))
            CALL LCMGET(IPTRK,'QH',BASE(IQH))
            CALL LCMGET(IPTRK,'RT',BASE(IRT))
            CALL LCMGET(IPTRK,'QT',BASE(IQT))
            CALL LCMSIX(IPTRK,' ',2)
            IF(ISPLH.EQ.1) THEN
               NELEM=MAXKN/7
            ELSE
               NELEM=MAXKN/4
            ENDIF
            CALL BIVA03(0,MAXKN,MAXQF,BASE(ISGD),NREG,LL4,ISPLH,
     1      NELEM,NBMIX,IIMAX,SIDE,MAT,IBASE(IKN),BASE(IQFR),VOL,
     2      IBASE(IMU),BASE(IR),BASE(IRH),BASE(IQH),BASE(IRT),
     3      BASE(IQT),BASE(ISYS))
            CALL RLSARA(BASE(IQT))
            CALL RLSARA(BASE(IRT))
            CALL RLSARA(BASE(IQH))
            CALL RLSARA(BASE(IRH))
            CALL RLSARA(BASE(IR))
         ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(ICOL.EQ.4)) THEN
            CALL BIVA04(0,MAXKN,MAXQF,BASE(ISGD),NREG,LL4,ISPLH,
     1      NBMIX,IIMAX,SIDE,MAT,IBASE(IKN),BASE(IQFR),VOL,IBASE(IMU),
     2      BASE(ISYS))
         ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
            CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
            CALL LCMSIX(IPTRK,'BIVCOL',1)
            CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
            CALL SETARA(BASE,LC**2,IR)
            CALL SETARA(BASE,LC*(LC-1),IV)
            CALL SETARA(BASE,LC*(LC-1),IH)
            CALL LCMGET(IPTRK,'R',BASE(IR))
            CALL LCMGET(IPTRK,'V',BASE(IV))
            CALL LCMGET(IPTRK,'H',BASE(IH))
            CALL LCMSIX(IPTRK,' ',2)
            CALL BIVA05(0,BASE(ISGD),IELEM,ICOL,ISPLH,LX,LL4,NBMIX,
     1      IIMAX,SIDE,MAT,IBASE(IZGLOB),IBASE(IKN),BASE(IQFR),
     2      IBASE(IMU),LC,BASE(IR),BASE(IV),BASE(IH),BASE(ISYS))
            CALL RLSARA(BASE(IH))
            CALL RLSARA(BASE(IV))
            CALL RLSARA(BASE(IR))
         ELSE
            CALL XABORT('BIVAA: UNKNOWN TYPE OF GEOMETRY(1).')
         ENDIF
	 CALL RLSARA(BASE(ISGD))
         CALL ALLDLF(LL4,BASE(ISYS),IBASE(IMU))
      ELSE
*
*  ++++ PN OR SPN THEORY ++++
*
         IF(NLF.LT.2) THEN
	    CALL XABORT('BIVAA: PN OR SPN KEYWORD EXPECTED.')
	 ENDIF
         NAN=MIN(ISCAT,NANI)+1
         CALL SETARA(BASE,NBMIX*NAN,ISGD)
         CALL SETARA(BASE,NBMIX*NAN,ISGDI)
         DO 30 IL=0,NAN-1
         DO 20 IBM=1,NBMIX
         GARS=SIGT0(IBM)
         IF(IL.LE.NAN-2) GARS=GARS-SIGW0(IBM,IL+1)
         BASE(ISGD+IL*NBMIX+IBM-1)=GARS
         IF(GARS.NE.0.0) THEN
            BASE(ISGDI+IL*NBMIX+IBM-1)=1.0/GARS
         ELSE
            BASE(ISGDI+IL*NBMIX+IBM-1)=1.0E10
         ENDIF
   20    CONTINUE
         WRITE(NAMP,'(4HSCAR,I2.2,6H001001)') IL
         CALL LCMPUT(IPSYS,NAMP,NBMIX,2,BASE(ISGD+IL*NBMIX))
         WRITE(NAMP,'(4HSCAI,I2.2,6H001001)') IL
         CALL LCMPUT(IPSYS,NAMP,NBMIX,2,BASE(ISGDI+IL*NBMIX))
   30    CONTINUE
         CALL XDISET(JPAR,NSTATE,0)
         JPAR(7)=NBMIX
         JPAR(8)=NAN
         CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,JPAR)
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         IF(ITYPE.EQ.3) THEN
            CALL SETARA(BASE,LC*LC,IRC)
            CALL LCMGET(IPTRK,'RS',BASE(IRC))
         ELSE IF(ITYPE.EQ.4) THEN
            CALL SETARA(BASE,LC*LC,IRS)
            CALL SETARA(BASE,LC*LC,IRSS)
            CALL LCMGET(IPTRK,'RSH1',BASE(IRS))
            CALL LCMGET(IPTRK,'RSH2',BASE(IRSS))
         ENDIF
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL LCMGET(IPTRK,'V',BASE(IV))
         CALL LCMSIX(IPTRK,' ',2)
*----
*  ASSEMBLY OF THE SYSTEM MATRICES USING MIXED-DUAL FINITE ELEMENTS.
*----
         IF(((ITYPE.EQ.2).OR.((ITYPE.EQ.5).AND.(ISPN.EQ.1)))) THEN
*           2D CARTESIAN GEOMETRY/SPN.
            CALL PNDM2E(0,NREG,LL4,IELEM,ICOL,MAT,VOL,NBMIX,NLF,NVD,
     1      NAN,BASE(ISGD),BASE(ISGDI),BASE(IXX),BASE(IYY),IBASE(IKN),
     2      BASE(IQFR),IBASE(IMU),IIMAX,LC,BASE(IR),BASE(IV),BASE(ISYS))
            IOF=0
            DO 40 IL=0,NLF-2,2
            CALL ALLDLF(LL4,BASE(ISYS+IOF),IBASE(IMU))
   40       IOF=IOF+IBASE(IMU+LL4-1)
         ELSE
            CALL XABORT('BIVAA: UNKNOWN TYPE OF GEOMETRY(2).')
         ENDIF
         CALL RLSARA(BASE(IV))
         IF(ITYPE.EQ.3) CALL RLSARA(BASE(IRC))
         IF(ITYPE.EQ.4) CALL RLSARA(BASE(IRS))
         IF(ITYPE.EQ.4) CALL RLSARA(BASE(IRSS))
         CALL RLSARA(BASE(IR))
         CALL RLSARA(BASE(ISGDI))
         CALL RLSARA(BASE(ISGD))
      ENDIF
      CALL LCMPOF(IPSYS,'IA001001',IIMAX,2,ISYS)
*----
*  RELEASE BIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IMU))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      IF(IMPX.GT.2) CALL LCMLIB(IPSYS)
      IF(IMPX.GT.10) CALL LCMVAL(IPSYS,' ')
      RETURN
      END
