*DECK BIVAF
      SUBROUTINE BIVAF(KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,MAT,
     1 VOL,KEYFLX,FUNKNO,SUNKNO,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one-group transport equation for fluxes using the diffusion
* approximation.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* KPSYS   pointer to the assembly matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* IDIR    not used.
* NREG    total number of regions for which specific values of the
*         neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NUN     total number of unknowns in vectors SUNKNO and FUNKNO.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  position of averaged flux elements in FUNKNO vector.
* SUNKNO  input source vector.
* TITR    title.
*
*Parameters: input/output
* FUNKNO  unknown vector.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   TITR*72
      INTEGER     KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,MAT(NREG),
     >            KEYFLX(NREG)
      REAL        VOL(NREG),FUNKNO(NUN),SUNKNO(NUN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40)
      INTEGER     IPAR(NSTATE),IBASE(1)
      LOGICAL     CYLIND,LBIHET
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER BIVAC SPECIFIC PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      IF(NREG.NE.IPAR(1)) CALL XABORT('BIVAF: INVALID VALUE OF NREG.')
      IF(NUN.NE.IPAR(2)) CALL XABORT('BIVAF: INVALID VALUE OF NUN.')
      ITYPE=IPAR(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=IPAR(8)
      ICOL=IPAR(9)
      ISPLH=IPAR(10)
      LL4=IPAR(11)
      LX=IPAR(12)
      LY=IPAR(13)
      NLF=IPAR(14)
      ISPN=IPAR(15)
      ISCAT=IPAR(16)
      LBIHET=IPAR(40).NE.0
      IF(NLF.GT.0) CALL XABORT('BIVAF: LIMITED TO DIFFUSION THEORY.')
      IF(IMPX.GT.2) THEN
         WRITE(IUNOUT,'(//33H N E U T R O N    S O U R C E S :)')
         WRITE(IUNOUT,'(1P,6(5X,E15.7))') (SUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL SETARA(BASE,NUN,ISUNKN)
         DO 10 I=1,NUN
   10    BASE(ISUNKN+I-1)=SUNKNO(I)
         NREGAR=NREG
         NBMIXG=NBMIX
         CALL DOORFB2(KPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX,SUNKNO)
         CALL LCMSIX(KPSYS,'BIHET',1)
      ENDIF
*
      CALL SETARA(BASE,LX*LY,IXX)
      CALL SETARA(BASE,LX*LY,IYY)
      CALL SETARA(BASE,LX*LY,IDD)
      IF(ITYPE.EQ.8) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      CALL SETARA(IBASE,LL4,IMU)
      CALL LCMGET(IPTRK,'MU',IBASE(IMU))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL SETARA(BASE,MAXQF,IBFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      CALL LCMGET(IPTRK,'BFR',BASE(IBFR))
*----
*  MULTIPLICATION OF THE EVEN-PARITY SOURCES BY THE VOLUMES.
*----
      DO 20 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 20
      JND1=KEYFLX(K)
      SUNKNO(JND1)=SUNKNO(JND1)*VOL(K)
   20 CONTINUE
*----
*  COMPUTE THE NEUTRON FLUX
*----
      IIMAX=IBASE(IMU+LL4-1)
      CALL SETARA(BASE,IIMAX,ISYS)
      CALL LCMGET(KPSYS,'IA001001',BASE(ISYS))
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC*LC,IQ)
         CALL SETARA(BASE,LC*LC,IQS)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL BIVS01(MAXKN,CYLIND,NREG,NUN,LL4,IIMAX,BASE(IXX),
     1   BASE(IYY),BASE(IDD),MAT,IBASE(IKN),BASE(IBFR),VOL,KEYFLX,
     2   IBASE(IMU),SUNKNO,LC,BASE(IT),BASE(ITS),BASE(ISYS),
     3   FUNKNO)
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
         CALL BIVS02(CYLIND,IELEM,ICOL,NREG,NUN,LL4,IIMAX,MAT,
     1   IBASE(IKN),BASE(IBFR),VOL,IBASE(IMU),SUNKNO,BASE(ISYS),
     2   FUNKNO)
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK,'RH',BASE(IRH))
         CALL LCMGET(IPTRK,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL BIVS03(MAXKN,MAXQF,NREG,NUN,LL4,ISPLH,NELEM,IIMAX,SIDE,
     1   MAT,IBASE(IKN),BASE(IQFR),BASE(IBFR),VOL,KEYFLX,IBASE(IMU),
     2   SUNKNO,BASE(IRH),BASE(IRT),BASE(ISYS),FUNKNO)
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(ICOL.EQ.4)) THEN
         CALL BIVS04(MAXKN,MAXQF,NREG,NUN,LL4,ISPLH,IIMAX,SIDE,MAT,
     1   IBASE(IKN),BASE(IQFR),BASE(IBFR),VOL,KEYFLX,IBASE(IMU),SUNKNO,
     2   BASE(ISYS),FUNKNO)
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL BIVS05(IELEM,ISPLH,LX,NUN,LL4,IIMAX,IBASE(IZGLOB),
     1   IBASE(IKN),BASE(IBFR),KEYFLX,IBASE(IMU),SUNKNO,BASE(ISYS),
     2   FUNKNO)
      ELSE
         CALL XABORT('BIVAF: UNKNOWN TYPE OF GEOMETRY.')
      ENDIF
      CALL RLSARA(BASE(ISYS))
      CALL RLSARA(BASE(IBFR))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(IBASE(IMU))
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
*----
*  DIVISION OF THE EVEN-PARITY SOURCES BY THE VOLUMES.
*----
      DO 30 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 30
      JND1=KEYFLX(K)
      SUNKNO(JND1)=SUNKNO(JND1)/VOL(K)
   30 CONTINUE
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL LCMSIX(KPSYS,' ',2)
         NREG=NREGAR
         NBMIX=NBMIXG
         DO 40 I=1,NUN
   40    SUNKNO(I)=BASE(ISUNKN+I-1)
         CALL RLSARA(BASE(ISUNKN))
         CALL DOORFB3(KPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX,SUNKNO,
     1   FUNKNO)
      ENDIF
*
      IF(IMPX.GT.2) THEN
         WRITE(IUNOUT,'(//33H N E U T R O N    F L U X E S   :)')
         WRITE(IUNOUT,'(1P,6(5X,E15.7))') (FUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
      RETURN
      END
