*DECK BIVGSH
      SUBROUTINE BIVGSH (IEX,MAXKN,MAXQF,NGCOND,NMERGE,FLXMER,SUNMER,
     1 NREG,NUN,LL4,ISPLH,NBMIX,SIDE,MAT,KN,QFR,VOL,DIFF0,MERG,COUR,
     2 FUNKNO,SOURCE,ZGAR1,ZGAR2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* source term calculation for mesh centered finite differences in
* hexagonal geometry. Used in DRAGON.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IEX     iteration number.
* MAXKN   dimension of array KN.
* MAXQF   dimension of array QFR.
* NGCOND  number of groups condensed.
* NMERGE  number of merged regions.
* FLXMER  flux estimate per mixture.
* SUNMER  incoming source (scattering+fission) cross sections.
* NREG    number of hexagons in BIVAC.
* NUN     number of unknowns per group in BIVAC.
* LL4     order of the system matrices in BIVAC. Equal to the number
*         of finite elements (hexagons or triangles) excluding the
*         virtual elements.
* ISPLH   hexagonal geometry flag:
*         =1: hexagonal elements; >1: triangular elements
* NBMIX   number of macro-mixtures.
* SIDE    side of the hexagons.
* MAT     mixture index per hexagon.
* KN      element-ordered unknown list.
* QFR     element-ordered information.
* VOL     volume of hexagons.
* DIFF0   diffusion coefficients.
* MERG    index of merged hexagons.
* COUR    four times the incoming current per unit surface.
* FUNKNO  previously calculated fluxes.
*
*Parameters: output
* SOURCE  fission and diffusion sources.
*
*Parameters: scratch
* ZGAR1
* ZGAR2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IEX,MAXKN,MAXQF,NGCOND,NMERGE,NREG,NUN,LL4,ISPLH,NBMIX,
     1 MAT(NREG),KN(MAXKN),MERG(NREG)
      REAL FLXMER(NMERGE,NGCOND),SUNMER(NMERGE,NGCOND),SIDE,QFR(MAXQF),
     1 VOL(NREG),DIFF0(NBMIX),COUR,FUNKNO(NUN,NGCOND),SOURCE(NREG),
     2 ZGAR1(NBMIX),ZGAR2(NBMIX)
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION A,DHARM
      DHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
*
      IF(ISPLH.EQ.1) THEN
         DS=SQRT(3.0)*SIDE
         FACT=2.0/(3.0*DS)
         NSURF=6
      ELSE
         DS=SIDE/(SQRT(3.0)*REAL(ISPLH-1))
         FACT=4.0/(3.0*DS)
         NSURF=3
      ENDIF
*
      CALL XDRSET(SOURCE,LL4,0.0)
*----
*  INCOMING CURRENT SOURCE.
*----
      IF(COUR.NE.0.0) THEN
         NUM1=0
         DO 40 IND1=1,LL4
         KHEX=KN(NUM1+NSURF+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 40
         L=MAT(KHEX)
         VOL0=QFR(NUM1+NSURF+1)
         SIDEB=FACT*VOL0
         DO 30 IC=1,NSURF
         IF(KN(NUM1+IC).EQ.-1) THEN
            A=DHARM(DS,DS,DIFF0(L),0.5*DS*QFR(NUM1+IC))
            SOURCE(IND1)=SOURCE(IND1)+A*COUR*SIDEB
         ENDIF
   30    CONTINUE
   40    NUM1=NUM1+NSURF+1
      ENDIF
*----
*  DIFFUSION AND FISSION SOURCE.
*----
      IF(IEX.EQ.1) GO TO 95
      DO 90 NL=1,NGCOND
      DO 70 L=1,NBMIX
      ZGAR1(L)=0.0
   70 ZGAR2(L)=0.0
      NUM1=0
      DO 80 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 80
      L=MAT(KHEX)
      VOL0=QFR(NUM1+NSURF+1)
      ZGAR1(L)=ZGAR1(L)+VOL0
      ZGAR2(L)=ZGAR2(L)+FUNKNO(IND1,NL)*VOL0
   80 NUM1=NUM1+NSURF+1
      NUM1=0
      DO 90 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 90
      L=MAT(KHEX)
      VOL0=QFR(NUM1+NSURF+1)
      PV=SUNMER(MERG(KHEX),NL)*FLXMER(MERG(KHEX),NL)*ZGAR1(L)/ZGAR2(L)
      SOURCE(IND1)=SOURCE(IND1)+FUNKNO(IND1,NL)*VOL0*PV
   90 NUM1=NUM1+NSURF+1
      RETURN
   95 NUM1=0
      DO 110 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 110
      VOL0=QFR(NUM1+NSURF+1)
      PV=0.0
      DO 100 NL=1,NGCOND
  100 PV=PV+SUNMER(MERG(KHEX),NL)*FLXMER(MERG(KHEX),NL)
      SOURCE(IND1)=SOURCE(IND1)+VOL0*PV
  110 NUM1=NUM1+NSURF+1
      RETURN
      END
