*DECK BIVGSR
      SUBROUTINE BIVGSR (IEX,NGCOND,NMERGE,FLXMER,SUNMER,IELEM,ISPLH,
     1 LX,NUN,NBMIX,SIDE,MAT,IZGLOB,KN,QFR,MERG,COUR,FUNKNO,SOURCE,
     2 ZGAR1,ZGAR2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* source term calculation for a Thomas-Raviart-Schneider formulation of
* the finite element technique in a 2-D hexagonal geometry. Used in
* DRAGON.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IEX     iteration number.
* NGCOND  number of groups condensed.
* NMERGE  number of merged regions.
* FLXMER  flux estimate per mixture.
* SUNMER  incoming source (scattering+fission) cross sections.
* IELEM   degree of the Lagrangian finite elements: =1 (linear);
*         =2 (parabolic); =3 (cubic); =4 (quartic).
* ISPLH   mesh-splitting index. Each hexagon is splitted into 3*ISPLH**2
*         lozenges.
* LX      number of hexagons in the W-X-Y plane.
* NUN     number of unknowns per group in BIVAC.
* NBMIX   number of macro-mixtures.
* SIDE    side of the hexagons.
* MAT     mixture index per region.
* IZGLOB  hexagon index assigned to each lozenge.
* KN      element-ordered unknown list.
* QFR     element-ordered boundary conditions.
* MERG    index of merged regions.
* COUR    four times the incoming current per unit surface.
* FUNKNO  previously calculated fluxes.
*
*Parameters: output
* SOURCE  fission and diffusion sources.
*
*Parameters: scratch
* ZGAR1
* ZGAR2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IEX,NGCOND,NMERGE,IELEM,ISPLH,LX,NUN,NBMIX,MAT(LX),
     1 IZGLOB(LX*ISPLH**2),KN(LX*ISPLH**2,4+6*IELEM*(IELEM+1)),MERG(LX)
      REAL FLXMER(NMERGE,NGCOND),SUNMER(NMERGE,NGCOND),SIDE,
     1 QFR(LX*ISPLH**2,6),COUR,FUNKNO(NUN,NGCOND),SOURCE(NUN),
     2 ZGAR1(NBMIX),ZGAR2(NBMIX)
*
      CALL XDRSET(SOURCE,NUN,0.0)
      NBLOS=LX*ISPLH**2
      NELEM=IELEM*(IELEM+1)
      TTTT=0.5D0*SQRT(3.D00)*SIDE*SIDE
*----
*  INCOMING CURRENT SOURCE.
*----
      IF(COUR.NE.0.0) THEN
         NUM=0
         DO 20 KEL=1,NBLOS
         IF(IZGLOB(KEL).EQ.0) GO TO 20
         NUM=NUM+1
         DO 10 K4=0,1
         DO 10 K2=1,IELEM+1,IELEM
         KNW1=KN(NUM,4+K4*NELEM+K2)
         KNX1=KN(NUM,4+(K4+2)*NELEM+K2)
         KNY1=KN(NUM,4+(K4+4)*NELEM+K2)
         IF((KNW1.NE.0).AND.(K2.EQ.1).AND.(K4.EQ.0)) THEN
            IND1=ABS(KNW1)
            SG=REAL(SIGN(1,KNW1))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*QFR(NUM,1)
         ENDIF
         IF((KNW1.NE.0).AND.(K2.EQ.IELEM+1).AND.(K4.EQ.1)) THEN
            IND1=ABS(KNW1)
            SG=REAL(SIGN(1,KNW1))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*QFR(NUM,2)
         ENDIF
         IF((KNX1.NE.0).AND.(K2.EQ.1).AND.(K4.EQ.0)) THEN
            IND1=ABS(KNX1)
            SG=REAL(SIGN(1,KNX1))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*QFR(NUM,3)
         ENDIF
         IF((KNX1.NE.0).AND.(K2.EQ.IELEM+1).AND.(K4.EQ.1)) THEN
            IND1=ABS(KNW1)
            SG=REAL(SIGN(1,KNW1))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*QFR(NUM,4)
         ENDIF
         IF((KNY1.NE.0).AND.(K2.EQ.1).AND.(K4.EQ.0)) THEN
            IND1=ABS(KNY1)
            SG=REAL(SIGN(1,KNY1))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*QFR(NUM,5)
         ENDIF
         IF((KNY1.NE.0).AND.(K2.EQ.IELEM+1).AND.(K4.EQ.1)) THEN
            IND1=ABS(KNY1)
            SG=REAL(SIGN(1,KNY1))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*QFR(NUM,6)
         ENDIF
   10    CONTINUE
   20    CONTINUE      
      ENDIF
*----
*  DIFFUSION AND FISSION SOURCE.
*----
      IF(IEX.EQ.1) GO TO 60
      DO 50 NL=1,NGCOND
      CALL XDRSET(ZGAR1,NBMIX,0.0)
      CALL XDRSET(ZGAR2,NBMIX,0.0)
      NUM=0
      DO 30 KEL=1,NBLOS
      IHEX=IZGLOB(KEL)
      IF(IHEX.EQ.0) GO TO 30
      NUM=NUM+1
      L=MAT(IHEX)
      IF(L.EQ.0) GO TO 30
      JND1=KN(NUM,1)
      JND2=KN(NUM,2)
      JND3=KN(NUM,3)
      ZGAR1(L)=ZGAR1(L)+3.0*TTTT
      ZGAR2(L)=ZGAR2(L)+TTTT*(FUNKNO(JND1,NL)+FUNKNO(JND2,NL)+
     1                        FUNKNO(JND3,NL))
   30 CONTINUE
      NUM=0
      DO 50 KEL=1,NBLOS
      IHEX=IZGLOB(KEL)
      IF(IHEX.EQ.0) GO TO 50
      NUM=NUM+1
      L=MAT(IHEX)
      IF(L.EQ.0) GO TO 50
      DO 40 K2=0,IELEM-1
      DO 40 K1=0,IELEM-1
      JND1=KN(NUM,1)+K2*IELEM+K1
      JND2=KN(NUM,2)+K2*IELEM+K1
      JND3=KN(NUM,3)+K2*IELEM+K1
      PV=SUNMER(MERG(IHEX),NL)*FLXMER(MERG(IHEX),NL)*ZGAR1(L)/ZGAR2(L)
      SOURCE(JND1)=SOURCE(JND1)+FUNKNO(JND1,NL)*TTTT*PV
      SOURCE(JND2)=SOURCE(JND2)+FUNKNO(JND2,NL)*TTTT*PV
   40 SOURCE(JND3)=SOURCE(JND3)+FUNKNO(JND3,NL)*TTTT*PV
   50 CONTINUE
      RETURN
   60 NUM=0
      DO 80 KEL=1,NBLOS
      IHEX=IZGLOB(KEL)
      IF(IHEX.EQ.0) GO TO 80
      NUM=NUM+1
      IF(MAT(IHEX).EQ.0) GO TO 80
      JND1=KN(NUM,1)
      JND2=KN(NUM,2)
      JND3=KN(NUM,3)
      PV=0.0
      DO 70 NL=1,NGCOND
   70 PV=PV+SUNMER(MERG(IHEX),NL)*FLXMER(MERG(IHEX),NL)
      SOURCE(JND1)=SOURCE(JND1)+TTTT*PV
      SOURCE(JND2)=SOURCE(JND2)+TTTT*PV
      SOURCE(JND3)=SOURCE(JND3)+TTTT*PV
   80 CONTINUE
      RETURN
      END
