*DECK CHAB
      SUBROUTINE CHAB(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
*  modify data contained in the microlib and renormalize the fission
*  and scattering information.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_MICROLIB);
*         HENTRY(2): optional read-only type(L_MICROLIB).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER TEXT4*4,TYPSEC*8,HISOT*12,HSIGN*12,FNAM1*12,NAM1*12
      DOUBLE PRECISION DFLOTT
      INTEGER KCHAR(3),ISTATE(NSTATE),IMOD
      COMMON BASE(1)
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LT.1) CALL XABORT('CHAB: MIN OF 1 OBJECT EXPECTED.')
      IF(NENTRY.GT.2) CALL XABORT('CHAB: MAX OF 2 OBJECTS EXPECTED.')
      IPMIC=KENTRY(1)
      IF(NENTRY.EQ.1) THEN
         IF(JENTRY(1).NE.1) CALL XABORT('CHAB: OBJECT IN MODIFICATION '
     1   //'MODE EXPECTED.')
         CALL LCMGET(IPMIC,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_LIBRARY') CALL XABORT('CHAB: MICROLIB OBJECT E'
     1   //'XPECTED AT RHS.')
      ELSE IF(NENTRY.EQ.2) THEN
         IF(JENTRY(1).NE.0) CALL XABORT('CHAB: OBJECT IN CREATE MODE E'
     1   //'XPECTED.')
         IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('CHAB: '
     1   //'LCM OBJECT EXPECTED AT RHS.')
         IF(JENTRY(2).NE.2) CALL XABORT('CHAB: LCM OBJECT IN READ-ONLY'
     1   //'MODE EXPECTED AT RHS.')
         CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_LIBRARY') CALL XABORT('CHAB: MICROLIB OBJECT E'
     1   //'XPECTED AT RHS.')
         CALL LCMEQU(KENTRY(2),IPMIC)
      ENDIF
      CALL LCMGET(IPMIC,'STATE-VECTOR',ISTATE)
      NGRP=ISTATE(3)
      CALL SETARA(BASE,NGRP,IVAL)
      CALL SETARA(BASE,NGRP,IGAR1)
      CALL SETARA(BASE,NGRP,IGAR2)
      CALL SETARA(BASE,NGRP*NGRP,IGAR3)
      CALL SETARA(BASE,NGRP,IDELTA)
*----
*  READ THE INPUT DATA
*----
      IMPX=1
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('CHAB: CHARACTER DATA EXPECTED(1).')
      IF(TEXT4.EQ.'EDIT') THEN
*        READ THE PRINT INDEX.
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('CHAB: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'MODI') THEN
*        MODIFY/ADD AN ENTRY (CROSS SECTION, SPECTRA, ETC).
         CALL REDGET(INDIC,NITMA,FLOTT,TYPSEC,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CHAB: CHARACTER DATA EXPECTED(2).')
         CALL REDGET(INDIC,IGM,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('CHAB: INTEGER DATA EXPECTED(2).')
         IF((IGM.LT.1).OR.(IGM.GT.NGRP)) CALL XABORT('CHAB: WRONG IGM.')
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CHAB: CHARACTER DATA EXPECTED(3).')
         IF(TEXT4.NE.'TO') CALL XABORT('CHAB: TO KEYWORD EXPECTED.')
         CALL REDGET(INDIC,IGP,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('CHAB: INTEGER DATA EXPECTED(3).')
         IF((IGP.LT.1).OR.(IGP.GT.NGRP)) CALL XABORT('CHAB: WRONG IGP.')
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CHAB: CHARACTER DATA EXPECTED(4).')
         IF(TEXT4.EQ.'VALE') THEN
            DO 20 IGR=1,IGP-IGM+1
            CALL REDGET(INDIC,NITMA,BASE(IVAL+IGR-1),TEXT4,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('CHAB: REAL DATA EXPECTED(1).')
   20       CONTINUE
            IMOD=1
         ELSE IF(TEXT4.EQ.'CONS') THEN
            CALL REDGET(INDIC,NITMA,VALUE,TEXT4,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('CHAB: REAL DATA EXPECTED(2).')
            IMOD=2
         ELSE IF(TEXT4.EQ.'PLUS') THEN
            CALL REDGET(INDIC,NITMA,VALUE,TEXT4,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('CHAB: REAL DATA EXPECTED(3).')
            IMOD=3
         ELSE IF(TEXT4.EQ.'MULT') THEN
            CALL REDGET(INDIC,NITMA,VALUE,TEXT4,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('CHAB: REAL DATA EXPECTED(4).')
            IMOD=4
         ELSE
            CALL XABORT('CHAB: VALE/CONS/PLUS/MULT KEYWORD EXPECTED.')
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,HISOT,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('CHAB: CHARACTER DATA EXPECTED(5).')
         IF(HISOT(9:).EQ.' ') THEN
            NAM1=' '
            CALL LCMNXT(IPMIC,NAM1)
            FNAM1=NAM1
   30       IF(NAM1(:8).EQ.HISOT) THEN
               CALL CHAB01(IPMIC,IMPX,NGRP,IMOD,TYPSEC,NAM1,VALUE,IGM,
     1         IGP,BASE(IVAL),BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),
     2         BASE(IDELTA))
            ENDIF
            CALL LCMNXT(IPMIC,NAM1)
            IF(NAM1.EQ.FNAM1) GO TO 10
            GO TO 30
         ELSE
            CALL CHAB01(IPMIC,IMPX,NGRP,IMOD,TYPSEC,HISOT,VALUE,IGM,IGP,
     1      BASE(IVAL),BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IDELTA))
         ENDIF
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 40
      ELSE
         CALL XABORT('CHAB: '//TEXT4//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 10
*----
*  RECOVER INFORMATION
*----
   40 CALL RLSARA(BASE(IDELTA))
      CALL RLSARA(BASE(IGAR3))
      CALL RLSARA(BASE(IGAR2))
      CALL RLSARA(BASE(IGAR1))
      CALL RLSARA(BASE(IVAL))
      RETURN
      END
