*DECK CPO
      SUBROUTINE CPO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* creation and construction of a Compo database object.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): G. Marleau
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): Create or modification L_COMPO database object;
*         HENTRY(2): Read-only type(L_EDIT);
*         HENTRY(3): Read-only type(L_BURNUP).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
      IMPLICIT      NONE
      INTEGER       IOUT,NSTATE,NDPROC,MAXNED
      CHARACTER     NAMSBR*6
      PARAMETER    (IOUT=6,NSTATE=40,NDPROC=20,MAXNED=50,
     >              NAMSBR='CPO   ')
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER       NENTRY
      CHARACTER     HENTRY(NENTRY)*12
      INTEGER       IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  MEMORY ALLOCATION PARAMETERS
*----
      INTEGER       IBASE(1)
      REAL          RBASE
      COMMON        RBASE(1)
      EQUIVALENCE  (RBASE(1),IBASE(1))
      INTEGER       IDBS,ISOCPO,ISOEXT,ISOORD,IVOLME,IENERG,INAMAD,
     >              NBIMRG,IDIMIX,ICOMIX,ISOTMP,
     >              IMXTMP,IDETMP,ITIME,IBURN,IWIR,IDNTMP,IMJMAC,
     >              IDNCPO,IMJCPO,INDPRO,ITYPRO,IXSMIC,ISCMIC,IXSMAC,
     >              ISCMAC,IRVALO
      INTEGER       IADF
*----
*  INPUT DATA
*----
      INTEGER       ITYPLU,INTLIR
      CHARACTER     CARLIR*12
      REAL          REALIR
      DOUBLE PRECISION DBLLIR
*----
*  LOCAL PARAMETERS
*----
      CHARACTER     HVECT(MAXNED)*6,CURNAM*12,CDIRO*12,TEXT12*12,
     >              TEXT4*4,HSIGN*12,NAMCPO*8,TEXT6*6
      LOGICAL       LB2,LBURN
      INTEGER       CTITRE(18),KCHAR(3),ISTATE(NSTATE),
     >              ISTATM(NSTATE)
      INTEGER       NBMICR,IED,NXXXZ,NL,NIFISS,NGCOND,NMERGE,NEDMAC,
     >              ILOCED,IST,NPROC,IEN,ITC,IPLIB,IKLIB,MXBURN,
     >              LENGT,MAXMRG,ITYLCM,ITEXT4,IPCPO,
     >              I,IPDEPL,IKDEPL,IPEDIT,IKEDIT,
     >              MAXISM,ILEAKS,ITRANC,NOLD,
     >              ILCMLN,IBR,IPBR,MAXISO,
     >              IPRINT,NISCPO,NSBS,IEXTRC,NISEXT
*----
* PARAMETER VALIDATION.
*----
      TEXT4='    '
      READ(TEXT4,'(A4)') ITEXT4
      IF(NENTRY.LT.2) CALL XABORT(NAMSBR//
     >': AT LEAST TWO DATA STRUCTURES EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT(NAMSBR//
     >': LINKED LIST OR XSM FILE EXPECTED AT LHS.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT(NAMSBR//
     >': LINKED LIST OR XSM FILE EXPECTED AT LHS.')
      IPCPO=KENTRY(1)
      IF(JENTRY(1).EQ.0) THEN
        HSIGN='L_COMPO'
        READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        CALL LCMPUT(IPCPO,'SIGNATURE',3,3,KCHAR)
      ELSE
        CALL LCMGET(IPCPO,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        IF(HSIGN.NE.'L_COMPO') THEN
          TEXT12=HENTRY(1)
          CALL XABORT(NAMSBR//
     >    ': SIGNATURE OF '//TEXT12//' IS '//HSIGN//' L_COMPO EXPECTED')
        ENDIF
      ENDIF
*----
*  SCAN ENTRY FOR EDIT, BURNUP AND LIB
*----
      IPEDIT=0
      IKEDIT=0
      IPDEPL=0
      IKDEPL=0
      IPLIB=0
      IKLIB=0
      DO 100 IEN=2,NENTRY
        TEXT12=HENTRY(IEN)
        IF(JENTRY(IEN).NE.2) CALL XABORT(NAMSBR//
     >  ': DATA STRUCTURE '//TEXT12//' NOT IN READ-ONLY MODE')
        IF(IENTRY(IEN).EQ.1.OR.IENTRY(IEN).EQ.2) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF(HSIGN.EQ.'L_EDIT'.AND.IKEDIT.EQ.0) THEN
            IPEDIT=KENTRY(IEN)
            IKEDIT=IEN
          ELSE IF(HSIGN.EQ.'L_BURNUP'.AND.IKDEPL.EQ.0) THEN
            IPDEPL=KENTRY(IEN)
            IKDEPL=IEN
          ENDIF
        ENDIF
 100  CONTINUE
      IF(IKEDIT.EQ.0) CALL XABORT(NAMSBR//
     >': NO DATA STRUCTURE WITH SIGNATURE L_EDIT FOUND.')
      IF(IKDEPL.EQ.0) THEN
        MXBURN=1
      ELSE
        CALL LCMLEN(IPDEPL,'DEPL-TIMES',MXBURN,ITYLCM)
        IF(MXBURN.EQ.0) CALL XABORT(NAMSBR//
     >  ': NO DEPL-TIMES DIRECTORY ON BURNUP DATA STRUCTURE')
      ENDIF
      CALL SETARA(IBASE,MXBURN,IDBS)
*----
*   RECOVER THE TITLE.
*----
      CALL LCMLEN(IPEDIT,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
        CALL LCMGET(IPEDIT,'TITLE',CTITRE)
      ELSE
        DO 101 I=1,18
          CTITRE(I)=ITEXT4
 101    CONTINUE
      ENDIF
*----
*  GET EDIT INFORMATION FOR MEMORY ALLOCATION OF
*  NUMBER OF ISOTOPES
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
      MAXMRG=ISTATE(1)
      MAXISM=ISTATE(13)
      MAXISO=MAXMRG*MAXISM
      CALL SETARA(IBASE,3*MAXISO,ISOCPO)
      CALL SETARA(IBASE,3*MAXISO,ISOEXT)
      CALL SETARA(IBASE,MAXISO,ISOORD)
      CALL XDISET(IBASE(ISOCPO),3*MAXISO,ITEXT4)
      CALL XDISET(IBASE(ISOEXT),3*MAXISO,ITEXT4)
      CALL XDISET(IBASE(ISOORD),MAXISO,0)
*----
*  READ CPO DATA.
*----
      IPRINT=1
      IEXTRC=0
      NISEXT=0
      NISCPO=0
      NSBS=-1
      LBURN=.FALSE.
      LB2=.FALSE.
      ILEAKS=0
      ITRANC=1
      CURNAM='REF-CASE   1'
      NAMCPO='COMPO'
 110  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 120  CONTINUE
      IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >': KEYWORD EXPECTED')
      IF(CARLIR.EQ.';') THEN
        GO TO 115
      ELSE IF(CARLIR.EQ.'EDIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >  ': EDIT LEVEL EXPECTED')
        IPRINT=INTLIR
      ELSE IF(CARLIR.EQ.'B2') THEN
        LB2=.TRUE.
      ELSE IF(CARLIR.EQ.'STEP') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >  ': STEP NAME EXPECTED')
        CURNAM=CARLIR
        NSBS=0
      ELSE IF(CARLIR.EQ.'NOTR') THEN
        ITRANC=0
      ELSE IF(CARLIR.EQ.'BURNUP') THEN
        IF(IKDEPL.EQ.0) CALL XABORT(NAMSBR//
     >  ': A BURNUP DATA STRUCTURE IS REQUIRED ')
        LBURN=.TRUE.
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >  ': BURNUP NAME EXPECTED')
        CURNAM=CARLIR
        NSBS=MXBURN
        DO 111 IBR=1,NSBS
          IBASE(IBR+IDBS-1)=IBR
 111    CONTINUE
      ELSE IF(CARLIR.EQ.'EXTRACT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >  ': ISOTOPE EXTRACTION NAME EXPECTED')
        IF(CARLIR.EQ.'ALL') THEN
*----
*  FOR EXTRACT ALL, RECOVER ISOTOPE NAMES FROM EDIT STRUCTURE
*----
          IEXTRC=2
          NISEXT=MAXISO
          NISCPO=MAXISO
        ELSE
          IEXTRC=1
          NISCPO=NISCPO+1
          IF(NISCPO.GT.MAXISO) CALL XABORT(NAMSBR//
     >    ': TOO MANY EXTRACTION ISOTOPES')
          READ(CARLIR,'(3A4)')
     >      (IBASE(I),I=ISOCPO+3*(NISCPO-1),ISOCPO+3*NISCPO-1)
 130      CONTINUE
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >    ': ISOTOPE NAME TO EXTRACT EXPECTED')
          IF((CARLIR.EQ.'EXTRACT').OR.(CARLIR.EQ.'EXPORT').OR.
     >       (CARLIR.EQ.'NAME')   .OR.(CARLIR.EQ.'ESBS')  .OR.
     >       (CARLIR.EQ.';')) GO TO 120
          NISEXT=NISEXT+1
          IF(NISEXT.GT.MAXISO) CALL XABORT(NAMSBR//
     >    ': TOO MANY ISOTOPES TO EXTRACT')
          READ(CARLIR,'(3A4)')
     >      (IBASE(I),I=ISOEXT+3*(NISEXT-1),ISOEXT+3*NISEXT-1)
          IBASE(ISOORD+NISEXT-1)=NISCPO
          GO TO 130
        ENDIF
      ELSE IF(CARLIR.EQ.'ESBS') THEN
        IF(.NOT.LBURN) CALL XABORT(NAMSBR//
     >  ': OPTION ESBS VALID ONLY WITH BURNUP OPTION.')
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >  ': KEYWORD FOLLOWING ESBS MISSING')
        IF(CARLIR.EQ.'NBUR') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >    ': INTEGER EXPECTED(2).')
          NSBS=INTLIR
          IPBR=0
          NOLD=0
          DO 112 IBR=1,NSBS
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >      ': INTEGER  EXPECTED(3).')
            IF(INTLIR.GT.MXBURN.OR.INTLIR.LT.1) THEN
              WRITE(IOUT,7000) NAMSBR,INTLIR,MXBURN
            ELSE IF(INTLIR.LE.NOLD) THEN
              WRITE(IOUT,7001) NAMSBR,NOLD,INTLIR
            ELSE
              IBASE(IPBR+IDBS)=INTLIR
              NOLD=INTLIR
              IPBR=IPBR+1
            ENDIF
 112      CONTINUE
          NSBS=IPBR
        ELSE
          CALL XABORT(NAMSBR//': NBUR KEY WORD EXPECTED.')
        ENDIF
      ELSE IF(CARLIR.EQ.'NAME') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT(NAMSBR//
     >  ': CPO NAME EXPECTED')
        NAMCPO=CARLIR
      ELSE
        CALL XABORT(NAMSBR//
     >  ': '//CARLIR//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 110
*----
*  CREATE THE COMPO
*----
 115  CONTINUE
      IF(LBURN) THEN
        WRITE(CDIRO,'(A8,I4)') CURNAM(1:8),IBASE(IDBS)
      ELSE
        CDIRO=CURNAM
      ENDIF
      CALL LCMLEN(IPEDIT,CDIRO,ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.0) THEN
        CALL XABORT(NAMSBR//': MISSING '//CDIRO//' DIRECTORY')
      ENDIF
      CALL LCMSIX(IPEDIT,CDIRO,1)
      CALL LCMSIX(IPEDIT,'MACROLIB',1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
      NGCOND=ISTATE(1)
      NMERGE=ISTATE(2)
      NL=ISTATE(3)
      NIFISS=ISTATE(4)
      NEDMAC=ISTATE(5)
      IF(ITRANC.EQ.1) THEN
        ITRANC=ISTATE(6)
      ENDIF
      IF((ITRANC.EQ.1).AND.(IPRINT.GT.0)) THEN
        WRITE(IOUT,6020)
      ENDIF
      ILEAKS=ISTATE(9)
      IF(LB2.AND.ILEAKS.EQ.0) CALL XABORT(NAMSBR//
     >': MISSING B2 INFO.')
      CALL SETARA(RBASE,NMERGE,IVOLME)
      CALL SETARA(RBASE,NGCOND+1,IENERG)
      CALL LCMGET(IPEDIT,'ENERGY',RBASE(IENERG))
      CALL LCMGET(IPEDIT,'VOLUME',RBASE(IVOLME))
      IF(NEDMAC.GT.0) THEN
        CALL SETARA(IBASE,2*NEDMAC,INAMAD)
        CALL LCMGET(IPEDIT,'ADDXSNAME-P0',IBASE(INAMAD))
        ILOCED=INAMAD
        DO 140 IED=1,NEDMAC
          WRITE(TEXT6,'(A4,A2)') IBASE(ILOCED),IBASE(ILOCED+1)
          HVECT(IED)=TEXT6
          ILOCED=ILOCED+2
 140    CONTINUE
        CALL RLSARA(IBASE(INAMAD))
      ENDIF
      CALL LCMSIX(IPEDIT,' ',2)
      IF(IPRINT.GE.1) THEN
        IF(ILEAKS.EQ.1) THEN
          WRITE(IOUT,6000) NAMSBR,CURNAM,NGCOND,NMERGE,NSBS,
     >                     NL,ILEAKS
        ELSE IF(ILEAKS.EQ.2) THEN
          WRITE(IOUT,6000) NAMSBR,CURNAM,NGCOND,NMERGE,NSBS,
     >                     NL,ILEAKS
        ELSE
          WRITE(IOUT,6000) NAMSBR,CURNAM,NGCOND,NMERGE,NSBS,
     >                     NL,ILEAKS
        ENDIF
      ENDIF
*----
*  PREPARE ISOTOPES
*----
      CALL LCMLEN(IPEDIT,'ISOTOPESMIX',NBMICR,ITYLCM)
      CALL SETARA(IBASE,NMERGE,NBIMRG)
      IF(IEXTRC.GE.1.AND.NBMICR.GT.0) THEN
        NXXXZ=MAX(NBMICR,1)
        CALL SETARA(IBASE,NMERGE*NXXXZ,IDIMIX)
        CALL SETARA(IBASE,NMERGE*MAXISM,ICOMIX)
        CALL SETARA(IBASE,3*NXXXZ,ISOTMP)
        CALL SETARA(IBASE,NXXXZ,IMXTMP)
        CALL SETARA(IBASE,NXXXZ,IDETMP)
        CALL LCMGET(IPEDIT,'ISOTOPESUSED',IBASE(ISOTMP))
        CALL LCMGET(IPEDIT,'ISOTOPESMIX',IBASE(IMXTMP))
        CALL XDISET(IBASE(IDETMP),NBMICR,0)
        CALL XDISET(IBASE(IDIMIX),NMERGE*NBMICR,0)
        CALL CPOISO(IPEDIT,IPRINT,IEXTRC,NMERGE,MAXISO,MAXISM,
     >              NBMICR,NIFISS,NISCPO,NISEXT,IBASE(ISOCPO),
     >              IBASE(ISOEXT),IBASE(ISOORD),IBASE(ISOTMP),
     >              IBASE(IMXTMP),IBASE(IDETMP),IBASE(IDIMIX),
     >              IBASE(NBIMRG),IBASE(ICOMIX))
        CALL RLSARA(IBASE(IDETMP))
      ELSE
        NXXXZ=MAX(NBMICR,1)
        CALL XDISET(IBASE(NBIMRG),NMERGE,0)
        CALL SETARA(IBASE,NMERGE*NXXXZ,IDIMIX)
        CALL SETARA(IBASE,NMERGE*NXXXZ,ICOMIX)
        CALL SETARA(IBASE,3*NXXXZ,ISOTMP)
        CALL SETARA(IBASE,1*NXXXZ,IMXTMP)
        NISCPO=0
      ENDIF
      CALL RLSARA(IBASE(ISOORD))
      CALL RLSARA(IBASE(ISOEXT))
      CALL LCMSIX(IPEDIT,' ',2)
*----
*  TEST IF OTHER BURNUP STEP CONSISTENT WITH FIRST BURNUP STEP
*----
      DO 160 IBURN=2,NSBS
        WRITE(CDIRO,'(A8,I4)') CURNAM(1:8),IBASE(IDBS+IBURN-1)
        CALL LCMLEN(IPEDIT,CDIRO,ILCMLN,ITYLCM)
        IF(ILCMLN.EQ.0) THEN
          WRITE(IOUT,7002) NAMSBR,CDIRO
          IBASE(IDBS+IBURN-1)=0
        ELSE
          CALL LCMSIX(IPEDIT,CDIRO,1)
          CALL LCMSIX(IPEDIT,'MACROLIB',1)
          CALL XDISET(ISTATM,NSTATE,0)
          CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATM)
          CALL LCMSIX(IPEDIT,'MACROLIB',2)
          CALL LCMSIX(IPEDIT,CDIRO,2)
          DO 170 IST=1,NSTATE
            IF(ISTATE(IST).NE.ISTATM(IST)) THEN
             WRITE(IOUT,7003) NAMSBR,CURNAM(1:8),
     >                        IBASE(IDBS),CDIRO
              IBASE(IDBS+IBURN-1)=0
              GO TO 175
            ENDIF
 170      CONTINUE
 175      CONTINUE
        ENDIF
 160  CONTINUE
      CALL SETARA(RBASE,MXBURN,ITIME)
      CALL SETARA(RBASE,MXBURN,IBURN)
      CALL SETARA(RBASE,MXBURN,IWIR)
      IF(LBURN) THEN
        CALL LCMGET(IPDEPL,'DEPL-TIMES',RBASE(ITIME))
      ELSE
        RBASE(ITIME)=0.0
      ENDIF
*----
*  ALLOCATE MEMORY FOR CPODRV
*----
      NPROC=NDPROC+NL+1
      CALL SETARA(RBASE,NXXXZ,IDNTMP)
      CALL SETARA(RBASE,2*NMERGE,IMJMAC)
      CALL SETARA(RBASE,2*NXXXZ,IDNCPO)
      CALL SETARA(RBASE,4*NXXXZ,IMJCPO)
      CALL SETARA(IBASE,NPROC,INDPRO)
      CALL SETARA(IBASE,NPROC,ITYPRO)
      CALL SETARA(RBASE,2*NGCOND*NPROC,IXSMIC)
      CALL SETARA(RBASE,2*NGCOND*NGCOND*NL,ISCMIC)
      CALL SETARA(RBASE,2*NGCOND*NPROC*NMERGE,IXSMAC)
      CALL SETARA(RBASE,2*NGCOND*NGCOND*NL*NMERGE,ISCMAC)
      CALL SETARA(RBASE,2*2*NGCOND*3,IADF)
      CALL SETARA(RBASE,2*NMERGE*MXBURN,IRVALO)
      CALL CPODRV(IPCPO ,IPEDIT,IPDEPL,IPRINT,CURNAM,CTITRE,
     >            NAMCPO,NGCOND,NMERGE,NBMICR,NIFISS,MXBURN,
     >            NL    ,NISCPO,NPROC ,ILEAKS,NXXXZ ,
     >            ITRANC,NEDMAC,HVECT ,IEXTRC,NSBS  ,
     >            IBASE(ISOCPO),IBASE(ISOTMP),IBASE(IMXTMP),
     >            IBASE(IDIMIX),IBASE(NBIMRG),IBASE(ICOMIX),
     >            RBASE(IVOLME),RBASE(IENERG),RBASE(ITIME) ,
     >            RBASE(IBURN) ,RBASE(IWIR)  ,IBASE(IDBS)  ,
     >            RBASE(IDNTMP),RBASE(IMJMAC),RBASE(IDNCPO),
     >            RBASE(IMJCPO),IBASE(INDPRO),IBASE(ITYPRO),
     >            RBASE(IXSMIC),RBASE(ISCMIC),RBASE(IXSMAC),
     >            RBASE(ISCMAC),RBASE(IADF)  ,RBASE(IRVALO))
*----
*  RELEASE MEMORY FOR CPODRV
*----
      CALL RLSARA(RBASE(IRVALO))
      CALL RLSARA(RBASE(IADF))
      CALL RLSARA(RBASE(ISCMAC))
      CALL RLSARA(RBASE(IXSMAC))
      CALL RLSARA(RBASE(ISCMIC))
      CALL RLSARA(RBASE(IXSMIC))
      CALL RLSARA(IBASE(ITYPRO))
      CALL RLSARA(IBASE(INDPRO))
      CALL RLSARA(RBASE(IMJCPO))
      CALL RLSARA(RBASE(IDNCPO))
      CALL RLSARA(RBASE(IMJMAC))
      CALL RLSARA(RBASE(IDNTMP))
*----
*  RELEASE MAIN MEMORY
*----
      CALL RLSARA(RBASE(IWIR))
      CALL RLSARA(RBASE(IBURN))
      CALL RLSARA(RBASE(ITIME))
      CALL RLSARA(IBASE(IMXTMP))
      CALL RLSARA(IBASE(ISOTMP))
      CALL RLSARA(IBASE(ICOMIX))
      CALL RLSARA(IBASE(NBIMRG))
      CALL RLSARA(IBASE(IDIMIX))
      CALL RLSARA(RBASE(IENERG))
      CALL RLSARA(RBASE(IVOLME))
      CALL RLSARA(IBASE(ISOCPO))
      CALL RLSARA(IBASE(IDBS))
      RETURN
*----
*  PRINT FORMAT
*----
 6000 FORMAT(1X,A6,
     >     ': RECOVER INFORMATION FROM DIRECTORY = ',A12/
     > 10X,'             NUMBER OF GROUPS  =',I5/
     > 10X,'             NUMBER OF COMPOS  =',I5/
     > 10X,'             NUMBER OF BURNUPS =',I5/
     > 10X,'             LEGENDRE ORDERS   =',I5/
     > 10X,'             LEAKAGE OPTION    =',I5)
 6010 FORMAT(1X,A6,': LIST OF EXTRACTED ISOTOPES:'/
     > 10X,'DRAGON NAME ',10X,'EXPORT NAME ')
 6011 FORMAT(10X,3A4,10X,3A4)
 6020 FORMAT(' TRANSPORT CORRECTED CROSS SECTIONS')
*----
*  WARNING FORMAT
*----
 7000 FORMAT(1X,A6,': ****** WARNING ******'/
     >       ' ILLEGAL BURNUP STEP NUMBER'/
     >       ' CURRENT BURNUP STEP - SKIPPED     = ',I10/
     >       ' NUMBER OF BURNUP STEP AVAILABLE   = ',I10/
     >       ' **************************')
 7001 FORMAT(1X,A6,': ****** WARNING ******'/
     >       ' BURNUP STEPS MUST BE ORDERED INCREASINGLY '/
     >       ' PREVIOUS BURNUP STEP REQUESTED    = ',I10/
     >       ' CURRENT BURNUP STEP - SKIPPED     = ',I10/
     >       ' **************************')
 7002 FORMAT(1X,A6,': ****** WARNING ******'/
     >       ' BURNUP STEP DOES NOT EXISTS '/
     >       ' CURRENT BURNUP STEP - SKIPPED     = ',A12/
     >       ' **************************')
 7003 FORMAT(1X,A6,': ****** WARNING ******'/
     >       ' INCONSISTENT BURNUP STEP '/
     >       ' REFERENCE BURNUP STEP             = ',A8,I4/
     >       ' CURRENT BURNUP STEP - SKIPPED     = ',A12/
     >       ' **************************')
      END
