*DECK CPODRV
      SUBROUTINE CPODRV(IPCPO ,IPEDIT,IPDEPL,IPRINT,CURNAM,CTITRE,
     >                  NAMCPO,NGROUP,NMERGE,NBMICR,NIFISS,MXBURN,
     >                  NL    ,NISCPO,NPROC ,ILEAKS,NXXXZ ,
     >                  ITRANC,NEDMAC,HVECT ,IEXTRC,NSBS  ,
     >                  ISOCPO,ISOTMP,IMXTMP,IDIMIX, NBIMRG,ICOMIX,
     >                  VOLMER,ENERGY,TIME  ,BURN  ,WIRRAD, IBSTEP,              
     >                  DENTMP,EMJMAC,DENCPO,DMJCPO,INDPRO,ITYPRO,
     >                  DXSMIC,DSCMIC,DXSMAC,DSCMAC,DISFAC,RVALOC)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover cross section information located on directory CURNAM or on
* directory family with prefix CURNAM.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): G. Marleau
*
*Parameters: input
* IPCPO   pointer to the compo (L_COMPO signature).
* IPEDIT  pointer to edit information (L_EDIT signature).
* IPDEPL  pointer to depletion information (L_BURNUP signature).
* IPRINT  print parameter. Equal to zero for no print.
* CURNAM  name of the output directory (or prefix of output
*         directory in burnup cases).
* CTITRE  character*72 title.
* NAMCPO  character*8 name of the material mixture sub-directory.
* NGROUP  number of energy groups in output data.
* NMERGE  number of output regions.
* NBMICR  maximum number of isotopes.
* NIFISS  number of fissile isotopes.
* MXBURN  maximum number of output burnup sets.
* NL      number of Legendre orders. =1 for isotropic scattering.
* NISCPO  number of Compo isotopes treated.
* NPROC   number of microscopic xs to process.
* ILEAKS  leak option -> 0 no leakage ; 1 homogeneous leakage ;
*         2 heterogeneous leakage.
* NXXXZ   maximum dimension of ISO dependent vector = max(nbmicr,1).
* ITRANC  type of transport correction in the Compo.
* NEDMAC  number of edit xs.
* HVECT   name of edit xs.
* IEXTRC  type of extraction: 0 for none, 1 for part 2 for all.
* NSBS    number of sub-burnup step considered.
* ISOCPO  Compo name of isotopes.
* ISOTMP  name of isotopes in EDIT.
* IMXTMP  mixture of isotopes in EDIT.
* IDIMIX  isotopes identifier in each Compo material.
* NBIMRG  final number of isotope per region.
* ICOMIX  pointer to Compo isotope for region.
* VOLMER  merge volume.
* ENERGY  energy.
* TIME    time steps.
* BURN    burnup.
* WIRRAD  irradiation.
* IBSTEP  sub-burnup step considered.
*
*Parameters: scratch
* DENTMP  density of EDI isotopes.
* EMJMAC  fission energy for macroscopic data.
* DENCPO  density of Compo isotopes.
* DMJCPO  fission energy for macroscopic data.
* INDPRO  identifier for xs processing.
* ITYPRO  identifier for xs processed.
* DXSMIC  micro vector xs.
* DSCMIC  micro scattering matrix xs.
* DXSMAC  macro vector xs.
* DSCMAC  macro scattering matrix xs.
* DISFAC  discontinuity factors.
* RVALOC  local burnup and irradiation values.
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER          IPCPO ,IPEDIT,IPDEPL,IPRINT,NGROUP,NMERGE,
     >                 CTITRE(18),NBMICR,NIFISS,MXBURN,NL,NISCPO,
     >                 NPROC ,ILEAKS,NXXXZ,ITRANC,NEDMAC,
     >                 IEXTRC,NSBS  ,ISOCPO(3,NXXXZ),
     >                 ISOTMP(3,NXXXZ),IMXTMP(NXXXZ),
     >                 IDIMIX(NMERGE,NXXXZ),NBIMRG(NMERGE),
     >                 ICOMIX(NMERGE,NXXXZ),INDPRO(NPROC),
     >                 ITYPRO(NPROC),IBSTEP(MXBURN)
      CHARACTER        CURNAM*12,NAMCPO*8,HVECT(NEDMAC)*6
      REAL             VOLMER(NMERGE),ENERGY(NGROUP+1),
     >                 TIME(MXBURN),BURN(MXBURN),WIRRAD(MXBURN),
     >                 DENTMP(NXXXZ),EMJMAC(NMERGE),
     >                 RVALOC(2,NMERGE,MXBURN)
      DOUBLE PRECISION DENCPO(NXXXZ),DXSMIC(NGROUP,NPROC),
     >                 DSCMIC(NGROUP,NGROUP,NL),
     >                 DXSMAC(NGROUP,NPROC,NMERGE),
     >                 DSCMAC(NGROUP,NGROUP,NL,NMERGE),
     >                 DMJCPO(2,NXXXZ)
      DOUBLE PRECISION DISFAC(2,NGROUP,3)
*----
*  MEMORY ALLOCATION PARAMETERS
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          DISTMP
*----
*  LOCAL PARAMETERS
*----
      INTEGER          IOUT,NSTATE,NDPROC,NPARAM
      REAL             CUTOFF
      PARAMETER       (IOUT=6,NSTATE=40,NDPROC=20,NPARAM=4,
     >                 CUTOFF=1.0E5)
      INTEGER          NEFBRN,IBR,IBR2,IBURN,IVECT,IXSREC,IXSCAT,
     >                 NDUM1,NDUM2,MAXDM,IDISFC,ISCATC,IIJJ,INJJ,
     >                 IMRG,ISOC,ISOR,ITC,IDFLU,IDNUFI,INAMI,IDENSI,
     >                 ISTATE(NSTATE),IPARAM(NPARAM),MXISOS,IEMJI,
     >                 IXSREM,ISCREM,NBISO,NREAC,NVAR,NBMIX,NREG,NLOC,
     >                 IDEN,ITIME,IBURNU,IVPHV,IVX,IJM,IWORK,ISIG,
     >                 IVOLR,IMATR,IMERG,IVOLIB,IIPARM
      CHARACTER        NAMMIX*12,NAMBRN*12,NAMISO*12,NAMMAC*12
      REAL             DELTA(2),TMPDAY(3),DELERR(3)
      DOUBLE PRECISION DMJMAC
      INTEGER          IFCDIS
*----
*  GET GLOBAL BURNUP AND IRRADIATION
*----
      IFCDIS=1
      NAMMAC='MACR       '
      IF(NSBS.EQ.0) THEN
        BURN(1)=0.0
        WIRRAD(1)=0.0
        NEFBRN=1
        IBSTEP(NEFBRN)=0
      ELSE
        DO 100 IBR=1,NSBS
          IBURN=IBSTEP(IBR)
          IF(IBURN.GT.0.AND. IBURN.LE.MXBURN) THEN
            WRITE(NAMBRN,'(8HDEPL-DAT,I4.4)') IBURN
            CALL LCMSIX(IPDEPL,NAMBRN,1)
            CALL LCMGET(IPDEPL,'BURNUP-IRRAD',DELTA)
            BURN(IBR)=DELTA(1)
            WIRRAD(IBR)=DELTA(2)
            TIME(IBR)=TIME(IBURN)
            CALL LCMSIX(IPDEPL,NAMBRN,2)
          ENDIF
 100    CONTINUE
        NEFBRN=NSBS
      ENDIF
*----
*  GET LOCAL BURNUP AND IRRADIATION
*----
      IF(NSBS.EQ.0) THEN
         CALL XDRSET(RVALOC,2*NMERGE*NEFBRN,0.0)
      ELSE
         NLOC=2
         CALL LCMGET(IPDEPL,'STATE-VECTOR',ISTATE)
         IF(ISTATE(3).NE.MXBURN) CALL XABORT('CPODRV: INVALID STATE-VE'
     1   //'CTOR.')
         NBISO=ISTATE(4)
         NREAC=ISTATE(6)
         NVAR=ISTATE(7)
         NBMIX=ISTATE(8)
         IF(IPDEPL.EQ.0) CALL XABORT('CPODRV: NO DEPLETION OBJECT AVAI'
     1   //'LABLE AMONG THE RHS LCM OBJECTS.')
         CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
         NREG=ISTATE(17)
         CALL SETARA(RBASE,NBISO,IDEN)
         CALL SETARA(RBASE,MXBURN,ITIME)
         CALL SETARA(RBASE,MXBURN,IBURNU)
         CALL SETARA(RBASE,NBMIX*2,IIPARM)
         CALL SETARA(RBASE,NBMIX*2,IVPHV)
         CALL SETARA(RBASE,NBMIX,IVX)
         CALL SETARA(RBASE,NBMIX*NVAR,IJM)
         CALL SETARA(RBASE,NBMIX,IWORK)
         CALL SETARA(RBASE,(NVAR+1)*(NREAC+1)*NBMIX,ISIG)
         CALL SETARA(RBASE,NREG,IVOLR)
         CALL SETARA(RBASE,NREG,IMATR)
         CALL SETARA(RBASE,NREG,IMERG)
         CALL SETARA(RBASE,NMERGE,IVOLIB)
         DO 105 IBR=1,NSBS
           IBURN=IBSTEP(IBR)
           CALL COMGEN(IPDEPL,IPEDIT,NREG,NMERGE,IBURN,'IRRA',MXBURN,
     1     NBMIX,NBISO,NREAC,NVAR,1,NLOC,RVALOC(1,1,IBR),RBASE(IDEN),
     2     RBASE(ITIME),RBASE(IBURNU),RBASE(IIPARM),RBASE(IVPHV),
     3     RBASE(IVX),RBASE(IJM),RBASE(IWORK),RBASE(ISIG),RBASE(IVOLR),
     4     RBASE(IMATR),RBASE(IMERG),RBASE(IVOLIB))
           CALL COMGEN(IPDEPL,IPEDIT,NREG,NMERGE,IBURN,'FLUB',MXBURN,
     1     NBMIX,NBISO,NREAC,NVAR,2,NLOC,RVALOC(1,1,IBR),RBASE(IDEN),
     2     RBASE(ITIME),RBASE(IBURNU),RBASE(IIPARM),RBASE(IVPHV),
     3     RBASE(IVX),RBASE(IJM),RBASE(IWORK),RBASE(ISIG),RBASE(IVOLR),
     4     RBASE(IMATR),RBASE(IMERG),RBASE(IVOLIB))
  105    CONTINUE
         CALL RLSARA(RBASE(IVOLIB))
         CALL RLSARA(RBASE(IMERG))
         CALL RLSARA(RBASE(IMATR))
         CALL RLSARA(RBASE(IVOLR))
         CALL RLSARA(RBASE(ISIG))
         CALL RLSARA(RBASE(IWORK))
         CALL RLSARA(RBASE(IJM))
         CALL RLSARA(RBASE(IVX))
         CALL RLSARA(RBASE(IVPHV))
         CALL RLSARA(RBASE(IIPARM))
         CALL RLSARA(RBASE(IBURNU))
         CALL RLSARA(RBASE(ITIME))
         CALL RLSARA(RBASE(IDEN))
      ENDIF
*----
*  INITIALIZE INDPRO FOR MICROSCOPIC XS TO PROCESS
*----
      CALL SETARA(IBASE,NEDMAC,IVECT)
      CALL XDISET(IBASE(IVECT),NEDMAC,0)
      CALL CPONED(NPROC ,0,NL-1,MAX(1,ILEAKS),NEDMAC,HVECT,
     >            IBASE(IVECT),INDPRO)
      INDPRO(6)=0
      INDPRO(16)=0
      CALL SETARA(RBASE,NGROUP*NPROC,IXSREC)
      CALL SETARA(RBASE,NGROUP*NGROUP*NL,IXSCAT)
*----
*  LOOP OVER BURNUP STEPS
*----
      NDUM1=NMERGE*MAX(NIFISS,NGROUP)
      NDUM2=NGROUP*NGROUP
      MAXDM=MAX(NDUM1,NDUM2)
      CALL SETARA(RBASE,NGROUP,IDISFC)
      CALL SETARA(RBASE,MAXDM,ISCATC)
      CALL SETARA(IBASE,MAX(NMERGE,NGROUP),IIJJ)
      CALL SETARA(IBASE,MAX(NMERGE,NGROUP),INJJ)
      CALL SETARA(RBASE,2*NGROUP*NPROC,IXSREM)
      CALL SETARA(RBASE,2*NGROUP*NGROUP*NL,ISCREM)
      IDFLU=16
      DO 110 IBR=1,NEFBRN
        WRITE(NAMBRN,'(A8,I4)') 'BURN    ',IBR
        IBURN=IBSTEP(IBR)
        IF(IBURN.GT.0) WRITE(CURNAM(9:12),'(I4)') IBURN
        CALL LCMSIX(IPEDIT,CURNAM,1)
        IF(NISCPO.GT.0) CALL LCMGET(IPEDIT,'ISOTOPESDENS',DENTMP)
        IF(IPRINT.GE.10) WRITE(IOUT,6000) CURNAM
        CALL LCMSIX(IPEDIT,'MACROLIB',1)
        CALL LCMGET(IPEDIT,'TIMESTAMP',TMPDAY)
        DELERR(1)=CUTOFF*ABS(TMPDAY(1)-TIME(IBR)/8.64E-4)
        DELERR(2)=CUTOFF*ABS(TMPDAY(2)-BURN(IBR))
        DELERR(3)=CUTOFF*ABS(TMPDAY(3)-WIRRAD(IBR))
        IF( (DELERR(1).GT.TMPDAY(1)) .OR.
     >      (DELERR(2).GT.TMPDAY(2)) .OR.
     >      (DELERR(3).GT.TMPDAY(3)) ) THEN
          IF(TIME(IBR)   .EQ. 0.0 .AND.
     >       BURN(IBR)   .EQ. 0.0 .AND.
     >       WIRRAD(IBR) .EQ. 0.0) THEN
            WRITE(IOUT,7001)
          ELSE
            WRITE(IOUT,7000)
     >        TMPDAY(1),TIME(IBR)/8.64E-4,DELERR(1)/CUTOFF,
     >        TMPDAY(2),BURN(IBR),DELERR(2)/CUTOFF,
     >        TMPDAY(3),WIRRAD(IBR)      ,DELERR(3)/CUTOFF
          ENDIF
          TIME(IBR)=TMPDAY(1)*8.64E-4
          BURN(IBR)=TMPDAY(2)
          WIRRAD(IBR)=TMPDAY(3)
        ENDIF
*----
*  READ MACROSCOPIC XS FOR ALL GROUP AND ALL REGIONS
*----
        CALL SETARA(RBASE,2*NMERGE*(NIFISS+1),IDNUFI)
        CALL CPOMAR(IPEDIT,NGROUP,NMERGE,NL    ,NIFISS,NEDMAC,
     >              HVECT ,IBASE(IVECT) ,NPROC ,ILEAKS,DXSMAC,
     >              DSCMAC,EMJMAC,RBASE(IDISFC),RBASE(ISCATC),
     >              IBASE(IIJJ),IBASE(INJJ),RBASE(IDNUFI),
     >              IFCDIS,DISFAC)
        CALL RLSARA(RBASE(IDNUFI))
        CALL LCMSIX(IPEDIT,'MACROLIB',2)
        CALL XDDSET(DENCPO,NISCPO,0.0D0)
        MXISOS=0
        DO 120 IMRG=1,NMERGE
          DMJMAC=DBLE(EMJMAC(IMRG))
          WRITE(NAMMIX,'(A8,I4)') NAMCPO,IMRG
          CALL LCMSIX(IPCPO,NAMMIX,1)
          CALL LCMSIX(IPCPO,NAMBRN,1)
          IF(IPRINT.GE.10) WRITE(IOUT,6001) NAMMIX
          CALL XDDSET(DMJCPO,2*NBMICR,0.0D0)
          CALL XDDSET(RBASE(IXSREM),NGROUP*NPROC,0.0D0)
          CALL XDDSET(RBASE(ISCREM),NGROUP*NGROUP*NL,0.0D0)
          DO 130 ISOC=1,NBIMRG(IMRG)
            ISOR=ICOMIX(IMRG,ISOC)
            WRITE(NAMISO,'(3A4)') (ISOCPO(ITC,ISOR),ITC=1,3)
            IF(IPRINT.GE.10) WRITE(IOUT,6002) NAMISO
*----
*   CREATE AND SAVE XS FOR A CPO ISOTOPE IN CURRENT REGION
*----
            CALL LCMSIX(IPCPO,NAMISO,1)
            CALL XDDSET(DXSMIC,NGROUP*NPROC,0.0D0)
            CALL XDDSET(DSCMIC,NGROUP*NGROUP*NL,0.0D0)
            CALL CPOMIC(IPCPO ,IPEDIT,IPRINT,NGROUP,NMERGE,NBMICR,
     >                  NL    ,IMRG  ,ISOR  ,NPROC ,ISOTMP,IDIMIX,
     >                  INDPRO,ITYPRO,DENCPO,DENTMP,DXSMIC,DSCMIC,
     >                  DMJCPO,DXSMAC(1,IDFLU,IMRG),RBASE(IXSREC),
     >                  RBASE(IXSCAT),RBASE(ISCATC),IBASE(IIJJ)  ,
     >                  IBASE(INJJ))
            CALL LCMSIX(IPCPO,NAMISO,2)
*----
*  REMOVE CONTRIBUTION OF CPO ISOTOPE FROM MACROSCOPIC.
*----
            IF(DENCPO(ISOR).GT.0.0D0) THEN
              CALL CPOREM(NGROUP,NL    ,NPROC ,INDPRO,
     >                    DENCPO(ISOR),DXSMIC,DSCMIC,
     >                    RBASE(IXSREM),RBASE(ISCREM))
            ENDIF
            DMJMAC=DMJMAC-DMJCPO(1,ISOR)
 130      CONTINUE
*----
*  WRITE MACROSCOPIC XS FOR ALL GROUP IN THIS REGION REGIONS
*----
          CALL SETARA(RBASE,2*NGROUP,DISTMP)
          CALL CPOMAW(IPCPO ,IPRINT,NGROUP,NL    ,NPROC ,INDPRO,
     >                ITYPRO,DXSMAC(1,1,IMRG),DSCMAC(1,1,1,IMRG),
     >                RBASE(IXSREM),RBASE(ISCREM),
     >                RBASE(IDISFC),RBASE(IXSREC),RBASE(IXSCAT),
     >                RBASE(ISCATC),IBASE(IIJJ),IBASE(INJJ),DMJMAC,
     >                IFCDIS,DISFAC,RBASE(DISTMP))
          CALL RLSARA(RBASE(DISTMP))
          CALL SETARA(RBASE,(NBIMRG(IMRG)+1),IDENSI)
          CALL SETARA(RBASE,(NBIMRG(IMRG)+1),IEMJI)
          RBASE(IDENSI)=1.0
          RBASE(IEMJI)=DMJMAC*1.0E-18
          DO 140 ISOC=1,NBIMRG(IMRG)
            ISOR=ICOMIX(IMRG,ISOC)
            RBASE(IDENSI+ISOC)=REAL(DENCPO(ISOR))
            IF(DMJCPO(2,ISOR).GT.0.0D0) THEN
              RBASE(IEMJI+ISOC)=1.0E-18*
     >                          REAL(DMJCPO(1,ISOR)/DMJCPO(2,ISOR))
            ELSE
              RBASE(IEMJI+ISOC)=0.0
            ENDIF
 140      CONTINUE
          CALL LCMPUT(IPCPO,'ISOTOPESDENS',(NBIMRG(IMRG)+1),2,
     >                RBASE(IDENSI))
          CALL LCMPUT(IPCPO,'ISOTOPES-EFJ',(NBIMRG(IMRG)+1),2,
     >                RBASE(IEMJI))
          CALL RLSARA(RBASE(IEMJI))
          CALL RLSARA(RBASE(IDENSI))
          CALL LCMSIX(IPCPO,NAMBRN,2)
*----
*  PUT REMAINING INFORMATION ON CPO FOR THIS MIXTURE
*----
          CALL LCMPUT(IPCPO,'TITLE',18,3,CTITRE)
          CALL LCMPUT(IPCPO,'VOLUME',1,2,VOLMER(IMRG))
          CALL LCMPUT(IPCPO,'ENERGY',NGROUP+1,2,ENERGY)
          IF(IBR.EQ.NEFBRN) THEN
            DO 145 IBR2=1,NEFBRN
              BURN(IBR2)=RVALOC(1,IMRG,IBR2)
              WIRRAD(IBR2)=RVALOC(2,IMRG,IBR2)
 145        CONTINUE
            CALL LCMPUT(IPCPO,'BURNUP',NEFBRN,2,BURN)
            CALL LCMPUT(IPCPO,'N/KB  ',NEFBRN,2,WIRRAD)
            CALL SETARA(IBASE,3*(NBIMRG(IMRG)+1),INAMI)
            READ(NAMMAC,'(3A4)') (IBASE(ITC),ITC=INAMI,INAMI+2)
            ITC=INAMI+3
            DO 150 ISOC=1,NBIMRG(IMRG)
              ISOR=ICOMIX(IMRG,ISOC)
              IBASE(ITC)=ISOCPO(1,ISOR)
              IBASE(ITC+1)=ISOCPO(2,ISOR)
              IBASE(ITC+2)=ISOCPO(3,ISOR)
              ITC=ITC+3
 150        CONTINUE
            CALL LCMPUT(IPCPO,'ISOTOPESNAME',3*(NBIMRG(IMRG)+1),3,
     >                  IBASE(INAMI))
            CALL XDISET(IBASE(INAMI),(NBIMRG(IMRG)+1),0)
            CALL LCMPUT(IPCPO,'JTAB',(NBIMRG(IMRG)+1),1,IBASE(INAMI))
            CALL RLSARA(IBASE(INAMI))
            CALL XDISET(IPARAM,NPARAM,0)
            IPARAM(1)=NGROUP
            IPARAM(2)=NBIMRG(IMRG)+1
            IPARAM(3)=NL
            IPARAM(4)=NEFBRN
            MXISOS=MAX(MXISOS,NBIMRG(IMRG)+1)
            CALL LCMPUT(IPCPO,'PARAM',NPARAM,1,IPARAM)
          ENDIF
          CALL LCMSIX(IPCPO,NAMMIX,2)
 120    CONTINUE
        CALL LCMSIX(IPEDIT,CURNAM,2)
 110  CONTINUE
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NMERGE
      ISTATE(2)=NGROUP
      ISTATE(3)=MXISOS
      ISTATE(4)=NL
      ISTATE(5)=NEFBRN
      ISTATE(6)=NPARAM
      ISTATE(7)=IFCDIS
      CALL LCMPUT(IPCPO,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL RLSARA(RBASE(ISCREM))
      CALL RLSARA(RBASE(IXSREM))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(IBASE(IIJJ))
      CALL RLSARA(RBASE(ISCATC))
      CALL RLSARA(RBASE(IDISFC))
      IF(NISCPO.GT.0) THEN
        CALL RLSARA(RBASE(IXSCAT))
        CALL RLSARA(RBASE(IXSREC))
        CALL RLSARA(IBASE(IVECT))
      ENDIF
*----
*  PRINT FORMAT
*----
 6000 FORMAT(' CPODRV: STEPPING UP ON DIRECTORY = ',A12)
 6001 FORMAT(' CPODRV: CREATING MIXTURE         = ',A12)
 6002 FORMAT(' CPODRV: CREATING ISOTOPE         = ',A12)
*----
*  WARNING FORMAT
*----
 7000 FORMAT(
     > ' CPODRV: WARNING -> BURNUP AND EDIT DATA DIFFER',1P/
     > ' TIME: EDIT=',E15.7,5X,' BURNUP=',E15.7,' DIFF=',E15.7/
     > ' BURN: EDIT=',E15.7,5X,' BURNUP=',E15.7,' DIFF=',E15.7/
     > ' WIRR: EDIT=',E15.7,5X,' BURNUP=',E15.7,' DIFF=',E15.7/
     > ' USE EDIT DATA   ')
 7001 FORMAT(
     > ' CPODRV: WARNING -> 0 BURNUP STEP, USE EDIT DATA')
      RETURN
      END
