*DECK DMA
      SUBROUTINE DMA(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* set the source of an adjoint fixed source eigenvalue problem. The
* source is the gradient of a macrolib.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation type(L_GPT);
*         HENTRY(2): read-only type(L_FLUX);
*         HENTRY(3): read-only type(L_MACROLIB);
*         HENTRY(4): read-only type(L_TRACKING).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER HSIGN*12,TEXT12*12
      INTEGER KCHAR(3),ISTATE(NSTATE)
      COMMON IBASE(1)
*----
*  PARAMETER VALIDATION.
*----
      IF(NENTRY.NE.4) CALL XABORT('DMA: FOUR PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('DMA: LI'
     1 //'NKED LIST OR XSM FILE EXPECTED AT LHS.')
      IF(JENTRY(1).NE.0) CALL XABORT('DMA: ENTRY IN CREATE OR MODE E'
     1 //'XPECTED.')
      DO I=2,4
         IF((JENTRY(I).NE.2).OR.((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2)))
     1   CALL XABORT('DMA: LINKED LIST OR XSM FILE IN READ-ONLY MODE E'
     2   //'XPECTED AT RHS.')
      ENDDO
      IPDMA=KENTRY(1)
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_FLUX') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('DMA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_FLUX EXPECTED.')
      ENDIF
      IPFLX=KENTRY(2)
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.EQ.'L_MACROLIB') THEN
         IPMAC=KENTRY(3)
      ELSE IF(HSIGN.EQ.'L_LIBRARY') THEN
         IPMAC=LCMGID(KENTRY(3),'MACROLIB')
      ELSE
         TEXT12=HENTRY(3)
         CALL XABORT('DMA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_MACROLIB OR L_LIBRARY EXPECTED.')
      ENDIF
      CALL LCMGET(KENTRY(4),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_TRACK') THEN
         TEXT12=HENTRY(4)
         CALL XABORT('DMA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_TRACK EXPECTED.')
      ENDIF
      IPTRK=KENTRY(4)
*----
*  RECOVER STATE VECTOR INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NREG=ISTATE(1)
      NBMIX=ISTATE(4)
      CALL LCMGET(IPFLX,'STATE-VECTOR',ISTATE)
      NG=ISTATE(1)
      NUN=ISTATE(2)
      CALL LCMGET(IPMAC,'STATE-VECTOR',ISTATE)
      IF(ISTATE(1).NE.NG) CALL XABORT('DMA: INVALID NUMBER OF GROUPS.')
      NMIL=ISTATE(2)
      NL=ISTATE(3)
      NFM=ISTATE(4)
      NED=ISTATE(5)
      NDEL=ISTATE(7)
      CALL SETARA(IBASE,2*NED,INAMEAD)
      IF(NED.GT.0) CALL LCMGET(IPMAC,'ADDXSNAME-P0',IBASE(INAMEAD))
*----
*  READ INPUT PARAMETERS
*----
      CALL LCMIOF(IPTRK,'MATCOD',INMIX)
      CALL SETARA(IBASE,NREG,IIMERGE)
      CALL SETARA(IBASE,1+NBMIX,IMIXMER)
      CALL SETARA(IBASE,NG,IIGCR)
      CALL DMAGET(IPDMA,NG,NREG,NBMIX,IBASE(INMIX),IPRINT,NMERGE,
     1 NGCOND,IBASE(IIMERGE),IBASE(IMIXMER),IBASE(IIGCR))
      CALL RLSARA(IBASE(IIGCR))
      CALL RLSARA(IBASE(IMIXMER))
      CALL RLSARA(IBASE(IIMERGE))
      NCST=(5+NGCOND*NL+2*NFM*(1+NDEL)+NED)*NMERGE*NGCOND
*----
*  COMPUTE THE GPT SOURCE
*----
      CALL SETARA(IBASE,NG,INDGRP)
      CALL XDISET(IBASE(INDGRP),NG,0)
      CALL LCMIOF(IPDMA,'REF:IMERGE',IMERGE)
      CALL LCMIOF(IPDMA,'REF:IGCOND',IGCR)
      CALL LCMIOF(IPTRK,'MATCOD',IMAT)
      CALL LCMIOF(IPTRK,'KEYFLX',IKEY)
      CALL LCMIOF(IPTRK,'VOLUME',IVOL)
      IOF=1
      JJ=IBASE(IGCR)
      DO IND=1,NG
	IF(IND.GT.JJ) THEN
	  IOF=IOF+1
	  IF(IOF.GT.NGCOND) CALL XABORT('DMA: NGCOND OVERFLOW.')
	  JJ=IBASE(IGCR+IOF-1)
	ENDIF
	IBASE(INDGRP+IND-1)=IOF
      ENDDO
      CALL SETARA(IBASE,2*NMERGE*NGCOND,IGAR1)
      CALL SETARA(IBASE,2*NMERGE*NGCOND,IGAR2)
      CALL SETARA(IBASE,2*NMERGE*NGCOND*NGCOND,IGAR3)
      CALL SETARA(IBASE,NUN,IFLUX)
      CALL SETARA(IBASE,NMIL,ISIGT)
      CALL SETARA(IBASE,NMIL,ICHI)
      CALL SETARA(IBASE,NMIL*NG*NG,IXSSNN)
      CALL SETARA(IBASE,NG,IIKEP)
      CALL SETARA(IBASE,NCST,IEPS)
      CALL DMASOU(IPRINT,IPDMA,IPMAC,IPFLX,NG,NREG,NMIL,NL,NF,NDEL,
     1 NED,IBASE(INAMEAD),NUN,NMERGE,NGCOND,NCST,IBASE(IMERGE),
     2 IBASE(INDGRP),IBASE(IMAT),IBASE(IKEY),IBASE(IVOL),IBASE(IGAR1),
     3 IBASE(IGAR2),IBASE(IGAR3),IBASE(IFLUX),IBASE(ISIGT),IBASE(ICHI),
     4 IBASE(IXSSNN),IBASE(IIKEP),IBASE(IEPS))
      CALL RLSARA(IBASE(IEPS))
      CALL RLSARA(IBASE(IIKEP))
      CALL RLSARA(IBASE(IXSSNN))
      CALL RLSARA(IBASE(ICHI))
      CALL RLSARA(IBASE(ISIGT))
      CALL RLSARA(IBASE(IFLUX))
      CALL RLSARA(IBASE(IGAR3))
      CALL RLSARA(IBASE(IGAR2))
      CALL RLSARA(IBASE(IGAR1))
      CALL RLSARA(IBASE(INDGRP))
      CALL RLSARA(IBASE(INAMEAD))
*----
*  SAVE THE SIGNATURE AND STATE VECTOR
*----
      HSIGN='L_GPT'
      READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPDMA,'SIGNATURE',3,3,KCHAR)
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NG
      ISTATE(2)=NUN
      ISTATE(3)=0
      ISTATE(4)=NCST
      ISTATE(5)=NMERGE
      ISTATE(6)=NGCOND
      IF(IPRINT.GT.0) WRITE(6,100) (ISTATE(I),I=1,6)
      CALL LCMPUT(IPDMA,'STATE-VECTOR',NSTATE,1,ISTATE)
      RETURN
*
  100 FORMAT(/8H OPTIONS/8H -------/
     1 7H NG    ,I8,28H   (NUMBER OF ENERGY GROUPS)/
     2 7H NUN   ,I8,40H   (NUMBER OF UNKNOWNS PER ENERGY GROUP)/
     3 7H NDIR  ,I8,35H   (NUMBER OF DIRECT FIXED SOURCES)/
     4 7H NCST  ,I8,36H   (NUMBER OF ADJOINT FIXED SOURCES)/
     5 7H NMERGE,I8,34H   (NUMBER OF HOMOGENIZED REGIONS)/
     6 7H NGCOND,I8,38H   (NUMBER OF CONDENSED ENERGY GROUPS))
      END
