*DECK DMASOU
      SUBROUTINE DMASOU(IPRINT,IPGPT,IPMAC,IPFLX,NG,NREG,NMIL,NL,NF,
     1 NDEL,NED,NAMEAD,NUN,NMERGE,NGCOND,NCST,IMERGE,INDGRP,MATCOD,
     2 KEYFLX,VOL,GAR1,GAR2,GAR3,FLUX,SIGT,CHI,XSSNN,IKEP,EPS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the GPT sources corresponding to the gradient of a macrolib.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPRINT  print parameter
* IPGPT   pointer to the DMA data structure.
* IPMAC   pointer to the macrolib structure.
* IPFLX   pointer to the multIG flux.
* NG      number of energy groups.
* NREG    number of regions.
* NMIL    number of material mixtures.
* NL      number of Legendre orders.
* NF      number of fissile isotopes.
* NDEL    number of delayed precursors.
* NED     number of extra edit vectors.
* NAMEAD  names of these extra edits.
* NUN     number of unknowns per energy group.
* NMERGE  number of merged regions.
* NGCOND  number of condensed energy groups.
* NCST    number of DMA fixed sources.
* IMERGE  merging indices.
* INDGRP  condensation indices.
* MATCOD  material mixture indices per region.
* KEYFLX  position of averaged fluxes in unknown vector.
* VOL     volumes.
*
*Parameters: scratch
* GAR1
* GAR2
* GAR3
* FLUX
* SIGT
* CHI
* XSSNN
* IKEP
* EPS
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPRINT,IPGPT,IPMAC,IPFLX,NG,NREG,NMIL,NL,NF,NDEL,NED,
     1 NAMEAD(2,NED),NMERGE,NGCOND,NCST,IMERGE(NREG),INDGRP(NG),
     2 MATCOD(NREG),KEYFLX(NREG),IKEP(NG)
      REAL VOL(NREG),FLUX(NUN),SIGT(NMIL),CHI(NMIL),XSSNN(NMIL,NG,NG),
     1 EPS(NCST)
      DOUBLE PRECISION GAR1(NMERGE,NGCOND),GAR2(NMERGE,NGCOND),
     1 GAR3(NMERGE,NGCOND,NGCOND)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSECT=4,EPSMAX=1.0E-7)
      CHARACTER TEXT12*12,TEXB12*12,HSECT(NSECT)*12,CM*2
      DOUBLE PRECISION WW,SUM
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
      DATA HSECT/'NTOT0','SIGS00','N2N','N3N'/
*
      IOF=0
      CALL XDRSET(EPS,NCST,0.0)
      JPFLX=LCMGID(IPFLX,'FLUX')
      JPGPT=LCMLID(IPGPT,'ASOUR',NCST)
*----
*  NWT0 INFORMATION
*----
      SUM=0.0D0
      CALL XDDSET(GAR2,NMERGE*NGCOND,0.0D0)
      DO IG=1,NG
        IGCND=INDGRP(IG)
        CALL LCMGDL(JPFLX,IG,FLUX)
        DO IR=1,NREG
          IF(KEYFLX(IR).EQ.0) CYCLE
          IMERG=IMERGE(IR)
          WW=FLUX(KEYFLX(IR))*VOL(IR)
          SUM=SUM+WW
          IF((IGCND.NE.0).AND.(IMERG.NE.0)) THEN            
            GAR2(IMERG,IGCND)=GAR2(IMERG,IGCND)+WW
          ENDIF
        ENDDO
      ENDDO
      DO IGCND=1,NGCOND
        DO IMERG=1,NMERGE
          IOF=IOF+1
          IF(IOF.GT.NCST) CALL XABORT('DMASOU: NCST OVERFLOW(1).')
          DO IG=1,NG
            CALL SETARA(BASE,NUN,IKEP(IG))
            ISUNK=IKEP(IG)
            CALL XDRSET(BASE(ISUNK),NUN,0.0)
            DO IR=1,NREG
              IF(KEYFLX(IR).EQ.0) CYCLE
              IUNK=KEYFLX(IR)
              FUNC=VOL(IR)*GAR2(IMERG,IGCND)/SUM
              IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND))
     1        THEN
                BASE(ISUNK+IUNK-1)=FUNC/GAR2(IMERG,IGCND)
              ENDIF
              BASE(ISUNK+IUNK-1)=BASE(ISUNK+IUNK-1)-FUNC/SUM
              ZN=SUM/FUNC
              EPS(IOF)=MAX(EPS(IOF),ABS(BASE(ISUNK+IUNK-1)*ZN))
            ENDDO
          ENDDO
          IF(EPS(IOF).GT.EPSMAX) THEN
            KPGPT=LCMLIL(JPGPT,IOF,NG)
            DO IG=1,NG
              CALL LCMPSL(KPGPT,IG,NUN,2,IKEP(IG))
            ENDDO
          ELSE
            DO IG=1,NG
              CALL RLSARA(BASE(IKEP(IG)))
            ENDDO
          ENDIF
        ENDDO
      ENDDO
*----
*  SET OF NSECT BASIC CROSS SECTIONS
*----
      JPMAC=LCMGID(IPMAC,'GROUP')
      DO ISECT=1,NSECT
        TEXT12=HSECT(ISECT)
        CALL XDDSET(GAR1,NMERGE*NGCOND,0.0D0)
        CALL XDDSET(GAR2,NMERGE*NGCOND,0.0D0)
        DO IG=1,NG
          KPMAC=LCMGIL(JPMAC,IG)
          CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
          IF(ILONG.GT.0) THEN
            CALL LCMGET(KPMAC,TEXT12,SIGT)
            IGCND=INDGRP(IG)
            CALL LCMGDL(JPFLX,IG,FLUX)
            DO IR=1,NREG
              IF(KEYFLX(IR).EQ.0) CYCLE
              IMERG=IMERGE(IR)
              WW=FLUX(KEYFLX(IR))*VOL(IR)
              IF((IGCND.NE.0).AND.(IMERG.NE.0)) THEN            
                IBM=MATCOD(IR)
                GAR1(IMERG,IGCND)=GAR1(IMERG,IGCND)+WW
                IF(IBM.GT.0) THEN
                  GAR2(IMERG,IGCND)=GAR2(IMERG,IGCND)+SIGT(IBM)*WW
                ENDIF
              ENDIF
            ENDDO
          ENDIF
        ENDDO
        DO IGCND=1,NGCOND
          DO IMERG=1,NMERGE
            IOF=IOF+1
            IF(IOF.GT.NCST) CALL XABORT('DMASOU: NCST OVERFLOW(2).')
            IF(GAR2(IMERG,IGCND).NE.0.0) THEN
              DO IG=1,NG
                KPMAC=LCMGIL(JPMAC,IG)
                CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
                CALL SETARA(BASE,NUN,IKEP(IG))
                ISUNK=IKEP(IG)
                CALL XDRSET(BASE(ISUNK),NUN,0.0)
                IF(ILONG.GT.0) THEN
                  CALL LCMGET(KPMAC,TEXT12,SIGT)
                  DO IR=1,NREG
                    IF(KEYFLX(IR).EQ.0) CYCLE
                    IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND))
     1              THEN
                      IUNK=KEYFLX(IR)
                      IBM=MATCOD(IR)
                      FUNC=VOL(IR)*GAR2(IMERG,IGCND)/GAR1(IMERG,IGCND)
                      IF(IBM.EQ.0) THEN
                        BASE(ISUNK+IUNK-1)=-FUNC/GAR1(IMERG,IGCND)
                      ELSE
                        BASE(ISUNK+IUNK-1)=FUNC*(SIGT(IBM)/
     1                  GAR2(IMERG,IGCND)-1.0/GAR1(IMERG,IGCND))
                      ENDIF
                      ZN=SUM/FUNC
                      EPS(IOF)=MAX(EPS(IOF),ABS(BASE(ISUNK+IUNK-1)*ZN))
                    ENDIF
                  ENDDO
                ENDIF
              ENDDO
              IF(EPS(IOF).GT.EPSMAX) THEN
                KPGPT=LCMLIL(JPGPT,IOF,NG)
                DO IG=1,NG
                  CALL LCMPSL(KPGPT,IG,NUN,2,IKEP(IG))
                ENDDO
              ELSE
                DO IG=1,NG
                  CALL RLSARA(BASE(IKEP(IG)))
                ENDDO
              ENDIF
            ENDIF
          ENDDO
        ENDDO
      ENDDO
*----
*  SCATTERING CROSS SECTION INFORMATION
*----
      CALL SETARA(IBASE,NMIL,IJJS00)
      CALL SETARA(IBASE,NMIL,NJJS00)
      CALL SETARA(IBASE,NMIL,IPOS00)
      DO IL=1,NL
        WRITE(CM,'(I2.2)') IL-1
        CALL XDRSET(XSSNN,NMIL*NG*NG,0.0)
        DO JG=1,NG
          KPMAC=LCMGIL(JPMAC,JG)
          CALL LCMGET(KPMAC,'IJJS'//CM,IBASE(IJJS00))
          CALL LCMGET(KPMAC,'NJJS'//CM,IBASE(NJJS00))
          CALL LCMGET(KPMAC,'IPOS'//CM,IBASE(IPOS00))
          IMAX=0
          DO IBM=1,NMIL
            IMAX=IMAX+IBASE(NJJS00+IBM-1)
          ENDDO
          CALL SETARA(BASE,IMAX,ISCAT)
          CALL LCMGET(KPMAC,'SCAT'//CM,BASE(ISCAT))
          DO IBM=1,NMIL
            IPOS=IBASE(IPOS00+IBM-1)
            IG=IBASE(IJJS00+IBM-1)
            IENBR=IBASE(NJJS00+IBM-1)
            DO WHILE (IENBR.GE.1) 
              XSSNN(IBM,JG,IG)=BASE(ISCAT+IPOS-1) ! JG <-- IG
              IPOS=IPOS+1
              IENBR=IENBR-1
              IG=IG-1
            ENDDO
          ENDDO
          CALL RLSARA(BASE(ISCAT))
        ENDDO
        CALL XDDSET(GAR1,NMERGE*NGCOND,0.0D0)
        CALL XDDSET(GAR3,NMERGE*NGCOND*NGCOND,0.0D0)
        DO IG=1,NG
          IGCND=INDGRP(IG)
          CALL LCMGDL(JPFLX,IG,FLUX)
          DO IR=1,NREG
            IF(KEYFLX(IR).EQ.0) CYCLE
            IMERG=IMERGE(IR)
            WW=FLUX(KEYFLX(IR))*VOL(IR)
            IF((IGCND.NE.0).AND.(IMERG.NE.0)) THEN            
              IBM=MATCOD(IR)
              GAR1(IMERG,IGCND)=GAR1(IMERG,IGCND)+WW
              IF(IBM.GT.0) THEN
                DO JG=1,NG
                  JGCND=INDGRP(JG)
                  GAR3(IMERG,JGCND,IGCND)=GAR3(IMERG,JGCND,IGCND)
     1                                   +XSSNN(IBM,JG,IG)*WW
                ENDDO
              ENDIF
            ENDIF
          ENDDO
        ENDDO
        DO JGCND=1,NGCOND
          DO IGCND=1,NGCOND
            DO IMERG=1,NMERGE
              IOF=IOF+1
              IF(IOF.GT.NCST) CALL XABORT('DMASOU: NCST OVERFLOW(3).')
              IF(GAR3(IMERG,JGCND,IGCND).NE.0.0) THEN
                DO IG=1,NG
                  CALL SETARA(BASE,NUN,IKEP(IG))
                  ISUNK=IKEP(IG)
                  CALL XDRSET(BASE(ISUNK),NUN,0.0)
                  DO JG=1,NG
                    DO IR=1,NREG
                      IF(KEYFLX(IR).EQ.0) CYCLE
                      IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND)
     1                .AND.(INDGRP(JG).EQ.JGCND)) THEN
                        IBM=MATCOD(IR)
                        IF(IBM.NE.0) THEN
                          IUNK=KEYFLX(IR)
                          FUNC=VOL(IR)*GAR3(IMERG,JGCND,IGCND)/
     1                                 GAR1(IMERG,IGCND)
                          BASE(ISUNK+IUNK-1)=BASE(ISUNK+IUNK-1)+FUNC*
     1                    XSSNN(IBM,JG,IG)/GAR3(IMERG,JGCND,IGCND)
                        ENDIF
                      ENDIF
                    ENDDO
                  ENDDO
                  DO IR=1,NREG
                    IF(KEYFLX(IR).EQ.0) CYCLE
                    IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND))
     1              THEN
                      IUNK=KEYFLX(IR)
                      FUNC=VOL(IR)*GAR3(IMERG,JGCND,IGCND)/
     1                             GAR1(IMERG,IGCND)
                      BASE(ISUNK+IUNK-1)=BASE(ISUNK+IUNK-1)-FUNC/
     1                GAR1(IMERG,IGCND)
                      ZN=SUM/FUNC
                      EPS(IOF)=MAX(EPS(IOF),ABS(BASE(ISUNK+IUNK-1)*ZN))
                    ENDIF
                  ENDDO
                ENDDO
                IF(EPS(IOF).GT.EPSMAX) THEN
                  KPGPT=LCMLIL(JPGPT,IOF,NG)
                  DO IG=1,NG
                    CALL LCMPSL(KPGPT,IG,NUN,2,IKEP(IG))
                  ENDDO
                ELSE
                  DO IG=1,NG
                    CALL RLSARA(BASE(IKEP(IG)))
                  ENDDO
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDDO
      ENDDO
      CALL RLSARA(IBASE(IPOS00))
      CALL RLSARA(IBASE(NJJS00))
      CALL RLSARA(IBASE(IJJS00))
*----
*  FISSION INFORMATION
*----
      DO IDEL=1,1+NDEL
        IF(IDEL.EQ.1) THEN
          TEXT12='NUSIGF'
          TEXB12='CHI'
        ELSE
          WRITE(TEXT12,'(6HNUSIGF,I2.2)') IDEL-1
          WRITE(TEXB12,'(3HCHI,I2.2)') IDEL-1
        ENDIF
        CALL XDDSET(GAR1,NMERGE*NGCOND,0.0D0)
        CALL XDDSET(GAR2,NMERGE*NGCOND,0.0D0)
        CALL XDDSET(GAR3(1,1,1),NMERGE*NGCOND,0.0D0)
        DO IG=1,NG
          KPMAC=LCMGIL(JPMAC,IG)
          CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
          IF(ILONG.GT.0) THEN
            CALL LCMGET(KPMAC,TEXT12,SIGT)
            CALL LCMGET(KPMAC,TEXB12,CHI)
            IGCND=INDGRP(IG)
            CALL LCMGDL(JPFLX,IG,FLUX)
            DO IR=1,NREG
              IF(KEYFLX(IR).EQ.0) CYCLE
              IMERG=IMERGE(IR)
              WW=FLUX(KEYFLX(IR))*VOL(IR)
              IF((IGCND.NE.0).AND.(IMERG.NE.0)) THEN            
                IBM=MATCOD(IR)
                GAR1(IMERG,IGCND)=GAR1(IMERG,IGCND)+WW
                IF(IBM.GT.0) THEN
                  GAR2(IMERG,IGCND)=GAR2(IMERG,IGCND)+SIGT(IBM)*WW
                  GAR3(IMERG,IGCND,1)=GAR3(IMERG,IGCND,1)+CHI(IBM)*
     1            SIGT(IBM)*WW
                ENDIF
              ENDIF
            ENDDO
          ENDIF
        ENDDO
        DO IGCND=1,NGCOND
          DO IMERG=1,NMERGE
            IOF=IOF+1
            IF(IOF.GT.NCST) CALL XABORT('DMASOU: NCST OVERFLOW(4).')
            IF(GAR2(IMERG,IGCND).NE.0.0) THEN
              DO IG=1,NG
                KPMAC=LCMGIL(JPMAC,IG)
                CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
                CALL SETARA(BASE,NUN,IKEP(IG))
                ISUNK=IKEP(IG)
                CALL XDRSET(BASE(ISUNK),NUN,0.0)
                IF(ILONG.GT.0) THEN
                  CALL LCMGET(KPMAC,TEXT12,SIGT)
                  DO IR=1,NREG
                    IF(KEYFLX(IR).EQ.0) CYCLE
                    IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND))
     1              THEN
                      IUNK=KEYFLX(IR)
                      IBM=MATCOD(IR)
                      FUNC=VOL(IR)*GAR2(IMERG,IGCND)/GAR1(IMERG,IGCND)
                      IF(IBM.EQ.0) THEN
                        BASE(ISUNK+IUNK-1)=-FUNC/GAR1(IMERG,IGCND)
                      ELSE
                        BASE(ISUNK+IUNK-1)=FUNC*(SIGT(IBM)/
     1                  GAR2(IMERG,IGCND)-1.0/GAR1(IMERG,IGCND))
                      ENDIF
                      ZN=SUM/FUNC
                      EPS(IOF)=MAX(EPS(IOF),ABS(BASE(ISUNK+IUNK-1)*ZN))
                    ENDIF
                  ENDDO
                ENDIF
              ENDDO
              IF(EPS(IOF).GT.EPSMAX) THEN
                KPGPT=LCMLIL(JPGPT,IOF,NG)
                DO IG=1,NG
                  CALL LCMPSL(KPGPT,IG,NUN,2,IKEP(IG))
                ENDDO
              ELSE
                DO IG=1,NG
                  CALL RLSARA(BASE(IKEP(IG)))
                ENDDO
              ENDIF
            ENDIF
            IOF=IOF+1
            IF(IOF.GT.NCST) CALL XABORT('DMASOU: NCST OVERFLOW(5).')
            IF(GAR3(IMERG,IGCND,1).NE.0.0) THEN
              DO IG=1,NG
                KPMAC=LCMGIL(JPMAC,IG)
                CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
                CALL SETARA(BASE,NUN,IKEP(IG))
                ISUNK=IKEP(IG)
                CALL XDRSET(BASE(ISUNK),NUN,0.0)
                IF(ILONG.GT.0) THEN
                  CALL LCMGET(KPMAC,TEXT12,SIGT)
                  CALL LCMGET(KPMAC,TEXB12,CHI)
                  DO IR=1,NREG
                    IF(KEYFLX(IR).EQ.0) CYCLE
                    IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND))
     1              THEN
                      IBM=MATCOD(IR)
                      IF(IBM.NE.0) THEN
                        IUNK=KEYFLX(IR)
                        FUNC=VOL(IR)*SIGT(IBM)*GAR3(IMERG,IGCND,1)/
     1                  GAR2(IMERG,IGCND)
                        BASE(ISUNK+IUNK-1)=FUNC*(CHI(IBM)/
     1                  GAR3(IMERG,IGCND,1)-1.0/GAR2(IMERG,IGCND))
                        ZN=SUM/FUNC
                        EPS(IOF)=MAX(EPS(IOF),ABS(BASE(ISUNK+IUNK-1)*
     1                  ZN))
                      ENDIF
                    ENDIF
                  ENDDO
                ENDIF
              ENDDO
              IF(EPS(IOF).GT.EPSMAX) THEN
                KPGPT=LCMLIL(JPGPT,IOF,NG)
                DO IG=1,NG
                  CALL LCMPSL(KPGPT,IG,NUN,2,IKEP(IG))
                ENDDO
              ELSE
                DO IG=1,NG
                  CALL RLSARA(BASE(IKEP(IG)))
                ENDDO
              ENDIF
            ENDIF
          ENDDO
        ENDDO
      ENDDO
*----
*  ADDITIONAL CROSS SECTION INFORMATION
*----
      DO IED=1,NED
        WRITE(TEXT12,'(2A4)') NAMEAD(1,IED),NAMEAD(2,IED)
        CALL XDDSET(GAR1,NMERGE*NGCOND,0.0D0)
        CALL XDDSET(GAR2,NMERGE*NGCOND,0.0D0)
        DO IG=1,NG
          KPMAC=LCMGIL(JPMAC,IG)
          CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
          IF(ILONG.GT.0) THEN
            CALL LCMGET(KPMAC,TEXT12,SIGT)
            IGCND=INDGRP(IG)
            CALL LCMGDL(JPFLX,IG,FLUX)
            DO IR=1,NREG
              IF(KEYFLX(IR).EQ.0) CYCLE
              IMERG=IMERGE(IR)
              WW=FLUX(KEYFLX(IR))*VOL(IR)
              IF((IGCND.NE.0).AND.(IMERG.NE.0)) THEN            
                IBM=MATCOD(IR)
                GAR1(IMERG,IGCND)=GAR1(IMERG,IGCND)+WW
                IF(IBM.GT.0) THEN
                  GAR2(IMERG,IGCND)=GAR2(IMERG,IGCND)+SIGT(IBM)*WW
                ENDIF
              ENDIF
            ENDDO
          ENDIF
        ENDDO
        DO IGCND=1,NGCOND
          DO IMERG=1,NMERGE
            IOF=IOF+1
            IF(IOF.GT.NCST) CALL XABORT('DMASOU: NCST OVERFLOW(6).')
            IF(GAR2(IMERG,IGCND).NE.0.0) THEN
              DO IG=1,NG
                KPMAC=LCMGIL(JPMAC,IG)
                CALL LCMLEN(KPMAC,TEXT12,ILONG,ITYLCM)
                CALL SETARA(BASE,NUN,IKEP(IG))
                ISUNK=IKEP(IG)
                CALL XDRSET(BASE(ISUNK),NUN,0.0)
                IF(ILONG.GT.0) THEN
                  CALL LCMGET(KPMAC,TEXT12,SIGT)
                  DO IR=1,NREG
                    IF(KEYFLX(IR).EQ.0) CYCLE
                    IF((IMERGE(IR).EQ.IMERG).AND.(INDGRP(IG).EQ.IGCND))
     1              THEN
                      IUNK=KEYFLX(IR)
                      IBM=MATCOD(IR)
                      FUNC=VOL(IR)*GAR2(IMERG,IGCND)/GAR1(IMERG,IGCND)
                      IF(IBM.EQ.0) THEN
                        BASE(ISUNK+IUNK-1)=-FUNC/GAR1(IMERG,IGCND)
                      ELSE
                        BASE(ISUNK+IUNK-1)=FUNC*(SIGT(IBM)/
     1                  GAR2(IMERG,IGCND)-1.0/GAR1(IMERG,IGCND))
                      ENDIF
                      ZN=SUM/FUNC
                      EPS(IOF)=MAX(EPS(IOF),ABS(BASE(ISUNK+IUNK-1)*ZN))
                    ENDIF
                  ENDDO
                ENDIF
              ENDDO
              IF(EPS(IOF).GT.EPSMAX) THEN
                KPGPT=LCMLIL(JPGPT,IOF,NG)
                DO IG=1,NG
                  CALL LCMPSL(KPGPT,IG,NUN,2,IKEP(IG))
                ENDDO
              ELSE
                DO IG=1,NG
                  CALL RLSARA(BASE(IKEP(IG)))
                ENDDO
              ENDIF
            ENDIF
          ENDDO
        ENDDO
      ENDDO
*----
*  CHECK SOURCE ORTHOGONALITY
*----
      DO IOF=1,NCST
        CALL LCMLEL(JPGPT,IOF,ILONG,ITYLCM)
        IF(ILONG.GT.0) THEN
          KPGPT=LCMGIL(JPGPT,IOF)
          SUM=0.0D0
          DO IG=1,NG
            CALL LCMGSL(KPGPT,IG,ISUNK)
            CALL LCMGDL(JPFLX,IG,FLUX)
            DO IR=1,NREG
              IUNK=KEYFLX(IR)
              IF(IUNK.GT.0) SUM=SUM+BASE(ISUNK+IUNK-1)*FLUX(IUNK)
            ENDDO
          ENDDO
          IF(IPRINT.GT.0) THEN
            WRITE(6,'(14H SOURCE INDEX=,I5,14H  DOT PRODUCT=,1P,E11.4,
     1      19H  SOURCE INTENSITY=,E11.4)') IOF,ABS(SUM),EPS(IOF)
          ENDIF
          IF(ABS(SUM).GT.1.0E-5) THEN
            WRITE(TEXT12,'(I5,7X)') IOF
            CALL XABORT('DMASOU: NON ORTHOGONAL SOURCE (IOF='//
     1      TEXT12(:5)//').')
          ENDIF
        ENDIF
      ENDDO
      RETURN
      END
