*DECK DOORAB
      SUBROUTINE DOORAB (CDOOR,JPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,
     1 NBMIX,NANI,MAT,VOL,KNORM,IPIJK,LEAKSW,LNORM,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* double heterogeneity treatment (part 1). Vectorial version.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* CDOOR   name of the geometry/solution operator.
* JPSYS   pointer to the PIJ LCM object (L_PIJ signature). JPSYS is a
*         list of NGRP directories.
* NPSYS   index array pointing to the JPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IMPX    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NREG    total number of regions for which specific values of the
*         neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MAT(i))).
* NANI    number of Legendre orders (usually equal to one).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   normalization scheme for PIJ matrices.
* IPIJK   pij option (=1 pij, =4 pijk).
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* LNORM   logical switch for removing leakage from collision
*         probabilities and keeping the PIS information.
* TITR    title.
*
*Parameters: output
* NREG   number of regions in the macro-geometry.
* MAT    index-number of the mixture type assigned to each macro-volume.
* VOL    macro-volumes.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW,LNORM
      INTEGER JPSYS,NPSYS(NGRP),IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,NANI,
     > MAT(NREG),KNORM,IPIJK
      REAL VOL(NREG)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER IPAR(8)
      REAL BASE(1)
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  RECOVER DOUBLE HETEROGENEITY DATA
*----
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMGET(IPTRK,'PARAM',IPAR)
      IR1=IPAR(1)
      IR2=IPAR(2)
      NREG2=IPAR(3)
      NG=IPAR(4)
      NSMAX=IPAR(5)
      IBIHET=IPAR(6)
      MICRO=IPAR(7)
      IQUA10=IPAR(8)
      IF(IR1.NE.NBMIX) CALL XABORT('DOORAB: INVALID DATA IN TRACKING.')
      IF(IBIHET.EQ.0) CALL XABORT('DOORAB: BIHET MODEL NOT SET.')
      NMILG=IR2-IR1
      CALL SETARA(IBASE,NG,INS)
      CALL SETARA(BASE,NG*(1+NSMAX),IRS)
      CALL SETARA(BASE,NG*IR2,IFRACT)
      CALL SETARA(BASE,NG*NSMAX,IVOLK)
      CALL SETARA(BASE,NMILG,IIDIL)
      CALL SETARA(BASE,NSMAX*NG*NMILG,IMIXGR)
      CALL SETARA(BASE,NREG2,IVOL2)
      CALL SETARA(IBASE,NREG2,IIBI)
      CALL LCMGET(IPTRK,'NS',IBASE(INS))
      CALL LCMGET(IPTRK,'RS',BASE(IRS))
      CALL LCMGET(IPTRK,'FRACT',BASE(IFRACT))
      CALL LCMGET(IPTRK,'VOLK',BASE(IVOLK))
      CALL LCMGET(IPTRK,'IDIL',IBASE(IIDIL))
      CALL LCMGET(IPTRK,'MIXGR',IBASE(IMIXGR))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL2))
      CALL LCMGET(IPTRK,'IBI',IBASE(IIBI))
      CALL LCMSIX(IPTRK,' ',2)
      IF(IMPX.GT.0) THEN
        WRITE(IOUT,'(/43H DOORAB: RECOVER DOUBLE-HETEROGENEITY DATA.)')
        WRITE(IOUT,'(35H   NUMBER OF ORDINARY MIXTURES     ,I4)') IR1
        WRITE(IOUT,'(35H   NUMBER OF COMPOSITE MIXTURES    ,I4)') NMILG
        WRITE(IOUT,'(35H   NUMBER OF KIND OF GRAINS        ,I4)') NG
        WRITE(IOUT,'(35H   NUMBER OF ORDINARY VOLUMES      ,I4)') NREG2
        WRITE(IOUT,'(35H   NUMBER OF COMPOSITE VOLUMES     ,I4)') NREG
        WRITE(IOUT,'(35H   TYPE OF MICRO VOLUMES (3 OR 4)  ,I4)') MICRO
        WRITE(IOUT,'(35H   MAX. NUMBER OF VOLUMES PER GRAIN,I4)') NSMAX
        WRITE(IOUT,'(35H   QUADRATURE PARAMETER FOR GRAINS ,I4)') IQUA10
        WRITE(IOUT,'(35H   DOUBLE HETEROGENEITY MODEL      ,I4)') IBIHET
      ENDIF
*----
*  COMPUTE THE EQUIVALENT CROSS SECTIONS IN COMPOSITE REGIONS
*----
      NB1=NBMIX+1
      CALL SETARA(BASE,(NB1+NMILG),ISGAR)
      CALL SETARA(BASE,(NB1+NMILG)*NANI,ISGAS)
      CALL XDRSET(BASE(ISGAS),(NB1+NMILG)*NANI,0.0)
      DO 100 IGR=1,NGRP
      IOFSET=NPSYS(IGR)
      IF(IOFSET.NE.0) THEN
         IF(IMPX.GT.10) WRITE(IOUT,'(/25H DOORAB: PROCESSING GROUP,I5,
     >   6H WITH ,A,1H.)') IGR,CDOOR
         KPSYS=LCMGIL(JPSYS,IOFSET)
         CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
         CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
*----
*   MORE MEMORY ALLOCATION
*----
         CALL SETARA(IBASE,NMILG,INCO)
         CALL SETARA(BASE,NMILG,IRRRR)
         CALL SETARA(BASE,NG*NSMAX*NMILG,IQKDEL)
         CALL SETARA(BASE,NG*NSMAX*NMILG,IQKOLD)
         CALL SETARA(BASE,NG*NSMAX*NSMAX*NMILG,IPKL)
         CALL SETARA(BASE,2*NMILG*(1+NG*NSMAX)**2,ICOEF)
*----
*   DOUBLE HETEROGENEITY TREATMENT -- PART 1
*----
         CALL SETARA(IBASE,NSMAX*NSMAX,IQKN)
         CALL SETARA(BASE,2*(1+NG*NSMAX),IRHS)
         IF(IBIHET.EQ.1) THEN
*           SANCHEZ-POMRANING MODEL.
            CALL XDRHT1(NBMIX,NMILG,NG,NSMAX,MICRO,IQUA10,IBASE(INS),
     >      IBASE(IIDIL),IBASE(IMIXGR),BASE(IRS),BASE(IFRACT),
     >      BASE(IVOLK),BASE(ISGAR),BASE(ISGAS),IBASE(INCO),BASE(IRRRR),
     >      BASE(IQKOLD),BASE(IQKDEL),BASE(IPKL),BASE(ICOEF),BASE(IQKN),
     >      BASE(IRHS))
         ELSE IF(IBIHET.EQ.2) THEN
*           HEBERT MODEL.
            CALL XDRHT2(NBMIX,NMILG,NG,NSMAX,MICRO,IQUA10,IBASE(INS),
     >      IBASE(IIDIL),IBASE(IMIXGR),BASE(IRS),BASE(IFRACT),
     >      BASE(IVOLK),BASE(ISGAR),BASE(ISGAS),IBASE(INCO),BASE(IRRRR),
     >      BASE(IQKDEL),BASE(IPKL),BASE(ICOEF),BASE(IQKN),BASE(IRHS))
         ELSE
            CALL XABORT('DOORAB: INVALID DOUBLE HETEROGENEITY MODEL.')
         ENDIF
         CALL RLSARA(BASE(IRHS))
         CALL RLSARA(IBASE(IQKN))
*
         KPBIH=LCMDID(KPSYS,'BIHET')
         CALL LCMPUT(KPBIH,'DRAGON-TXSC',1+IR2,2,BASE(ISGAR))
         CALL LCMPUT(KPBIH,'DRAGON-S0XSC',(1+IR2)*NANI,2,BASE(ISGAS))
*
         CALL LCMPOF(KPSYS,'NCO',NMILG,1,INCO)
         CALL LCMPOF(KPSYS,'RRRR',NMILG,3,IRRRR)
         CALL LCMPOF(KPSYS,'QKOLD',NG*NSMAX*NMILG,3,IQKOLD)
         CALL LCMPOF(KPSYS,'QKDEL',NG*NSMAX*NMILG,3,IQKDEL)
         CALL LCMPOF(KPSYS,'PKL',NG*NSMAX*NSMAX*NMILG,3,IPKL)
         CALL LCMPOF(KPSYS,'COEF',2*NMILG*(1+NG*NSMAX)**2,4,ICOEF)
      ENDIF
  100 CONTINUE
      CALL RLSARA(BASE(ISGAS))
      CALL RLSARA(BASE(ISGAR))
*----
*  SET MACRO-GEOMETRY.
*----
      NREG=NREG2
      NBMIX=NBMIX+NMILG
      DO I=1,NREG2
         MAT(I)=IBASE(IIBI+I-1)
         VOL(I)=BASE(IVOL2+I-1)
      ENDDO
      RETURN
      END
