*DECK DOORFB2
      SUBROUTINE DOORFB2(IPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX,NBMIX2,
     1 NREG2,SUNKNO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* double heterogeneity treatment (part 2). One-group version.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the assembly LCM object (L_PIJ signature). IPSYS is
*         a list of directories.
* NPSYS   index array pointing to the IPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IMPX    print flag (equal to zero for no print).
* NBMIX   number of composite mixtures in the domain. Equal to the
*         number of mixtures in the internal library.
* NREG    number of volumes in the composite geometry.
* NUN     total number of unknowns in vector SUNKNO.
* KEYFLX  index of flux components in unknown vector.
* SUNKNO  sources defined in the composite geometry.
*
*Parameters: output
* NBMIX2  number of mixtures in the internal library, including double-
*         heterogeneity mixtures.
* NREG2   number of volumes in the macro geometry.
* SUNKNO  equivalent macro-sources.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX(NREG),NBMIX2,NREG2
      REAL SUNKNO(NUN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER IPAR(8)
      REAL BASE(1)
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  RECOVER DOUBLE HETEROGENEITY DATA
*----
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMGET(IPTRK,'PARAM',IPAR)
      IR1=IPAR(1)
      IR2=IPAR(2)
      NREG2=IPAR(3)
      NG=IPAR(4)
      NSMAX=IPAR(5)
      IBIHET=IPAR(6)
      IF(IR1.NE.NBMIX) THEN
         WRITE(6,'(/13HDOORFB2: IR1=,I6,7H NBMIX=,I6)') IR1,NBMIX
         CALL XABORT('DOORFB2: INVALID DATA IN TRACKING.')
      ENDIF
      IF(NREG2.GT.NUN) CALL XABORT('DOORFB2: NUN OVERFLOW.')
      NMILG=IR2-IR1
      CALL LCMIOF(IPTRK,'NS',INS)
      CALL LCMIOF(IPTRK,'IBI',IIBI)
      CALL LCMIOF(IPTRK,'FRACT',IFRACT)
      CALL LCMIOF(IPTRK,'VOLK',IVOLK)
      CALL LCMIOF(IPTRK,'IDIL',IIDIL)
      CALL LCMIOF(IPTRK,'MIXGR',IMIXGR)
      CALL LCMSIX(IPTRK,' ',2)
*----
*   RECOVER GROUP-DEPENDENT BIHET INFORMATION
*----
      CALL LCMIOF(IPSYS,'NCO',INCO)
      CALL LCMIOF(IPSYS,'RRRR',IRRRR)
      CALL LCMIOF(IPSYS,'QKOLD',IQKOLD)
      CALL LCMIOF(IPSYS,'QKDEL',IQKDEL)
      CALL LCMIOF(IPSYS,'PKL',IPKL)
      CALL LCMIOF(IPSYS,'COEF',ICOEF)
*----
*  COMPUTE THE EQUIVALENT CROSS SECTIONS IN COMPOSITE REGIONS
*----
      CALL LCMSIX(IPSYS,'BIHET',1)
      NB1=NBMIX+1
      CALL SETARA(BASE,NB1+NMILG,ISGAR)
      CALL LCMGET(IPSYS,'DRAGON-TXSC',BASE(ISGAR))
      CALL LCMSIX(IPSYS,' ',2)
*----
*   DOUBLE HETEROGENEITY TREATMENT -- PART 2
*----
      CALL SETARA(BASE,NREG2,IFLUAS)
      CALL SETARA(BASE,2*(1+NG*NSMAX),IRHS)
      CALL XDRHT3(IBIHET,NUN,NBMIX,NMILG,NREG,NREG2,NG,NSMAX,KEYFLX,
     > IBASE(INS),IBASE(IIDIL),IBASE(IMIXGR),IBASE(IIBI),BASE(IFRACT),
     > BASE(IVOLK),BASE(ISGAR),IBASE(INCO),BASE(IRRRR),BASE(IQKOLD),
     > BASE(IQKDEL),BASE(IPKL),BASE(ICOEF),SUNKNO,BASE(IFLUAS),
     > BASE(IRHS))
      CALL RLSARA(BASE(IRHS))
*----
*   SET MACRO-INFORMATION
*----
      CALL XDRSET(SUNKNO,NUN,0.0)
      DO 10 I=1,NREG2
   10 SUNKNO(KEYFLX(I))=BASE(IFLUAS+I-1)
      CALL RLSARA(BASE(IFLUAS))
      NBMIX2=NBMIX+NMILG
*----
*   MEMORY RELEASE
*----
      CALL RLSARA(BASE(ISGAR))
      RETURN
      END
