*DECK DOORFB3
      SUBROUTINE DOORFB3(IPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX,SUNKNO,
     1 FUNKNO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* double heterogeneity treatment (part 3). One-group version.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the assembly LCM object (L_PIJ signature). IPSYS is
*         a list of directories.
* NPSYS   index array pointing to the IPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IMPX    print flag (equal to zero for no print).
* NBMIX   number of composite mixtures in the domain. Equal to the
*         number of mixtures in the internal library.
* NREG    number of volumes in the composite geometry.
* NUN     total number of unknowns in vector SUNKNO.
* KEYFLX  index of flux components in unknown vector.
* SUNKNO  equivalent macro-sources.
* FUNKNO  equivalent macro-fluxes.
*
*Parameters: output
* FUNKNO  composite fluxes.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX(NREG)
      REAL SUNKNO(NUN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER IPAR(8)
      REAL BASE(1)
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  RECOVER DOUBLE HETEROGENEITY DATA
*----
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMGET(IPTRK,'PARAM',IPAR)
      IR1=IPAR(1)
      IR2=IPAR(2)
      NREG2=IPAR(3)
      NG=IPAR(4)
      NSMAX=IPAR(5)
      IBIHET=IPAR(6)
      IF(IR1.NE.NBMIX) CALL XABORT('DOORFB3: INVALID DATA IN TRACKING.')
      IF(NREG2.GT.NUN) CALL XABORT('DOORFB3: NUN OVERFLOW.')
      NMILG=IR2-IR1
      CALL LCMIOF(IPTRK,'NS',INS)
      CALL LCMIOF(IPTRK,'IBI',IIBI)
      CALL LCMIOF(IPTRK,'FRACT',IFRACT)
      CALL LCMIOF(IPTRK,'VOLK',IVOLK)
      CALL LCMIOF(IPTRK,'IDIL',IIDIL)
      CALL LCMIOF(IPTRK,'MIXGR',IMIXGR)
      CALL LCMSIX(IPTRK,' ',2)
*----
*   RECOVER GROUP-DEPENDENT BIHET INFORMATION
*----
      CALL LCMIOF(IPSYS,'NCO',INCO)
      CALL LCMIOF(IPSYS,'RRRR',IRRRR)
      CALL LCMIOF(IPSYS,'QKOLD',IQKOLD)
      CALL LCMIOF(IPSYS,'QKDEL',IQKDEL)
      CALL LCMIOF(IPSYS,'PKL',IPKL)
      CALL LCMIOF(IPSYS,'COEF',ICOEF)
*----
*  COMPUTE THE EQUIVALENT CROSS SECTIONS IN COMPOSITE REGIONS
*----
      CALL LCMSIX(IPSYS,'BIHET',1)
      NB1=NBMIX+1
      CALL LCMLEN(IPSYS,'DRAGON-S0XSC',ILONG,ITYLCM)
      NANI=ILONG/(NB1+NMILG)
      CALL SETARA(BASE,(NB1+NMILG),ISGAR)
      CALL SETARA(BASE,(NB1+NMILG)*NANI,ISGAS)
      CALL XDRSET(BASE(ISGAS),(NB1+NMILG)*NANI,0.0)
      CALL LCMGET(IPSYS,'DRAGON-TXSC',BASE(ISGAR))
      CALL LCMGET(IPSYS,'DRAGON-S0XSC',BASE(ISGAS))
      CALL LCMSIX(IPSYS,' ',2)
*----
*   DOUBLE HETEROGENEITY TREATMENT -- PART 2
*----
      CALL SETARA(BASE,NREG2,IFLUAS)
      CALL SETARA(BASE,2*(1+NG*NSMAX),IRHS)
      CALL XDRHT4(IBIHET,NUN,NBMIX,NMILG,NREG,NREG2,NG,NSMAX,KEYFLX,
     > IBASE(INS),IBASE(IIDIL),IBASE(IMIXGR),IBASE(IIBI),BASE(IFRACT),
     > BASE(IVOLK),BASE(ISGAR),BASE(ISGAS),IBASE(INCO),BASE(IRRRR),
     > BASE(IQKOLD),BASE(IQKDEL),BASE(IPKL),BASE(ICOEF),SUNKNO,
     > FUNKNO,BASE(IFLUAS),BASE(IRHS))
      CALL RLSARA(BASE(IRHS))
*----
*   MEMORY RELEASE
*----
      CALL RLSARA(BASE(ISGAS))
      CALL RLSARA(BASE(ISGAR))
      RETURN
      END
