*DECK DOORPV
      SUBROUTINE DOORPV (CDOOR,JPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,
     1 NBMIX,NANI,MAT,VOL,KNORM,IPIJK,LEAKSW,LNORM,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities. Vectorial version.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* CDOOR   name of the geometry/solution operator.
* JPSYS   pointer to the PIJ LCM object (L_PIJ signature). JPSYS is
*         a list of directories.
* NPSYS   index array pointing to the JPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IMPX    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MAT(i))).
* NANI    number of Legendre orders (usually equal to one).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   normalization scheme for PIJ matrices.
* IPIJK   pij option (=1 pij, =4 pijk).
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* LNORM   logical switch for removing leakage from collision
*         probabilities and keeping the PIS information.
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW,LNORM
      INTEGER JPSYS,NPSYS(NGRP),IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,NANI,
     > MAT(NREG),KNORM,IPIJK
      REAL VOL(NREG)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6)
      CHARACTER CMS(3)*1
      INTEGER PROBKS,ISTATE(NSTATE)
      LOGICAL LBIHET,LNXT
      COMMON BASE(1)
      SAVE CMS
      DATA CMS/'1','2','3'/
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      LBIHET=ISTATE(40).NE.0
      IF(LBIHET) THEN
         CALL SETARA(BASE,NREG,IMAT)
         CALL SETARA(BASE,NREG,IVOL)
         DO I=1,NREG
            BASE(IMAT+I-1)=MAT(I)
            BASE(IVOL+I-1)=VOL(I)
         ENDDO
         NREGAR=NREG
         NBMIXG=NBMIX
         CALL DOORAB(CDOOR,JPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,
     1   NBMIX,NANI,MAT,VOL,KNORM,IPIJK,LEAKSW,LNORM,TITR)
      ENDIF
*
      NELPIJ=NREG*(NREG+1)/2
      NB1=NBMIX+1
      IF(CDOOR.EQ.'EXCELL') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         INSB=ISTATE(18)
         LNXT=(ISTATE(7).EQ.4)
         IF(INSB.GE.1) GO TO 110
      ENDIF
*----
*  COMPUTE THE REDUCED PIJ MATRIX -- NON-VECTORIAL ALGORITHM.
*----
      CALL SETARA(BASE,IPIJK*NELPIJ,IPGAR)
      CALL SETARA(BASE,NB1,ISGAR)
      CALL SETARA(BASE,NB1*NANI,ISGAS)
      DO 100 IGR=1,NGRP
      IOFSET=NPSYS(IGR)
      IF(IOFSET.NE.0) THEN
         IF(IMPX.GT.10) WRITE(IOUT,'(/25H DOORPV: PROCESSING GROUP,I5,
     >   6H WITH ,A,1H.)') IGR,CDOOR
         KPSYS=LCMGIL(JPSYS,IOFSET)
         IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
         CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
         CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
         IF(CDOOR.EQ.'EXCELL') THEN
            IF(IPIJK.EQ.4) THEN
               NNREG=NREG*NREG
               NPST=3*NNREG
               CALL SETARA(BASE,NPST,PROBKS)
            ELSE
               PROBKS=0
               NPST=1
            ENDIF
            CALL EXCELP(IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT,VOL,
     >      KNORM,BASE(ISGAR),BASE(ISGAS),NELPIJ,IPIJK,BASE(IPGAR),
     >      LEAKSW,1,1,NPST,BASE(PROBKS),TITR)
            IF(IPIJK.EQ.4) THEN
               CALL LCMPUT(KPSYS,'DRAGON1P*SCT',NNREG,2,BASE(PROBKS))
               CALL LCMPUT(KPSYS,'DRAGON2P*SCT',NNREG,2,
     >         BASE(PROBKS+NNREG))
               CALL LCMPUT(KPSYS,'DRAGON3P*SCT',NNREG,2,
     >         BASE(PROBKS+2*NNREG))
            ENDIF
            IF(IPIJK.EQ.4) CALL RLSARA(BASE(PROBKS))
         ELSE IF(CDOOR(:5).EQ.'SYBIL') THEN
            CALL SYBILP(IPTRK,IMPX,NREG,NBMIX,MAT,VOL,KNORM,BASE(ISGAR),
     >      BASE(ISGAS),NELPIJ,IPIJK,BASE(IPGAR),LEAKSW,TITR)
         ELSE
            CALL XABORT('DOORPV: UNKNOWN PIJ DOOR NAMED '//CDOOR//'.')
         ENDIF
*----
*  REMOVE LEAKAGE FROM THE SCATTERING-REDUCED CP MATRIX.
*----
         IF(LNORM) THEN
            CALL SETARA(BASE,NREG,IPIS)
            CALL XDRNRM(NREG,NBMIX,MAT,VOL,BASE(ISGAR+1),BASE(ISGAS+1),
     >      BASE(IPGAR),BASE(IPIS))
            IF(LEAKSW) THEN
               CALL LCMPUT(KPSYS,'DRAGON-WIS',NREG,2,BASE(IPIS))
            ENDIF
            CALL RLSARA(BASE(IPIS))
         ENDIF
*----
*  FORMAT THE REDUCED PIJ MATRIX AS A SQUARE MATRIX.
*----
         CALL SETARA(BASE,NREG*NREG,IPREG)
         DO 10 I=1,NREG
         FACT=1.0/VOL(I)
         DO 10 J=1,NREG
   10    BASE(IPREG+(J-1)*NREG+I-1)=BASE(IPGAR+INDPOS(I,J)-1)*FACT
         CALL LCMPOF(KPSYS,'DRAGON-PCSCT',NREG*NREG,2,IPREG)
         IF(IPIJK.EQ.4) THEN
            DO 30 IJKS=1,3
            CALL SETARA(BASE,NREG*NREG,IPREG)
            DO 20 I=1,NREG
            FACT=1.0/VOL(I)
            DO 20 J=1,NREG
            KS=NELPIJ*IJKS+INDPOS(I,J)
   20       BASE(IPREG+(J-1)*NREG+I-1)=BASE(IPGAR+KS-1)*FACT
            CALL LCMPOF(KPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NREG*NREG,2,
     >      IPREG)
   30       CONTINUE
         ENDIF
         IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
      ENDIF
  100 CONTINUE
      CALL RLSARA(BASE(ISGAS))
      CALL RLSARA(BASE(ISGAR))
      CALL RLSARA(BASE(IPGAR))
      GO TO 210
*----
*  COMPUTE THE REDUCED PIJ MATRIX -- VECTORIAL ALGORITHM FOR EXCELP.
*----
  110 CALL SETARA(BASE,NB1*NGRP,ISGAR)
      CALL SETARA(BASE,NB1*NANI*NGRP,ISGAS)
      DO 120 IGR=1,NGRP
      IOFSET=NPSYS(IGR)
      IF(IOFSET.NE.0) THEN
         KPSYS=LCMGIL(JPSYS,IOFSET)
         IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
         CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR+(IGR-1)*NB1))
         CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS+(IGR-1)*NB1*NANI))
         IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
      ENDIF
  120 CONTINUE
      CALL SETARA(BASE,IPIJK*NELPIJ*NGRP,IPGAR)
      IF(LNXT.OR.(INSB.EQ.1)) THEN
*        --- ALLG VECTORIZATION
         IF(IPIJK.EQ.4) THEN
            NNREG=NREG*NREG
            NPST=3*NNREG
            CALL SETARA(BASE,NPST*NGRP,PROBKS)
         ELSE
            PROBKS=0
            NPST=1
         ENDIF
         CALL EXCELP(IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT,VOL,KNORM,
     >   BASE(ISGAR),BASE(ISGAS),NELPIJ,IPIJK,BASE(IPGAR),LEAKSW,
     >   NGRP,NPSYS,NPST,BASE(PROBKS),TITR)
      ELSE IF(INSB.EQ.2) THEN
*        --- XCLL VECTORIZATION
         IF(IPIJK.NE.1) CALL XABORT('DOORPV: INVALID VALUE OF IPIJK')
         CALL PIJXL3(IPTRK,IMPX,NGRP,NANI,NBMIX,NPSYS,KNORM,LNORM,
     >   LEAKSW,BASE(ISGAR),BASE(ISGAS),NELPIJ,BASE(IPGAR))
      ELSE
         CALL XABORT('DOORPV: INVALID VALUE OF INSB')
      ENDIF
      KPIJSY=IPGAR
      KPST=PROBKS
      DO 200 IGR=1,NGRP
      IOFSET=NPSYS(IGR)
      IF(IOFSET.NE.0) THEN
         KPSYS=LCMGIL(JPSYS,IOFSET)
         IF(LBIHET) CALL LCMSIX(KPSYS,'BIHET',1)
         IF(IPIJK.EQ.4) THEN
            CALL LCMPUT(KPSYS,'DRAGON1P*SCT',NNREG,2,BASE(KPST))
            CALL LCMPUT(KPSYS,'DRAGON2P*SCT',NNREG,2,BASE(KPST+NNREG))
            CALL LCMPUT(KPSYS,'DRAGON3P*SCT',NNREG,2,BASE(KPST+2*NNREG))
         ENDIF
*----
*  REMOVE LEAKAGE FROM THE SCATTERING-REDUCED CP MATRIX.
*----
         IF(LNORM) THEN
            CALL SETARA(BASE,NREG,IPIS)
            CALL XDRNRM(NREG,NBMIX,MAT,VOL,BASE(ISGAR+(IGR-1)*NB1+1),
     >      BASE(ISGAS+(IGR-1)*NB1*NANI+1),BASE(KPIJSY),BASE(IPIS))
            IF(LEAKSW) THEN
               CALL LCMPUT(KPSYS,'DRAGON-WIS',NREG,2,BASE(IPIS))
            ENDIF
            CALL RLSARA(BASE(IPIS))
         ENDIF
*----
*  FORMAT THE REDUCED PIJ MATRIX AS A SQUARE MATRIX.
*----
         CALL SETARA(BASE,NREG*NREG,IPREG)
         DO 130 I=1,NREG
         FACT=1.0/VOL(I)
         DO 130 J=1,NREG
  130    BASE(IPREG+(J-1)*NREG+I-1)=BASE(KPIJSY+INDPOS(I,J)-1)*FACT
         CALL LCMPOF(KPSYS,'DRAGON-PCSCT',NREG*NREG,2,IPREG)
         IF(IPIJK.EQ.4) THEN
            DO 150 IJKS=1,3
            CALL SETARA(BASE,NREG*NREG,IPREG)
            DO 140 I=1,NREG
            FACT=1.0/VOL(I)
            DO 140 J=1,NREG
            KS=NELPIJ*IJKS+INDPOS(I,J)
  140       BASE(IPREG+(J-1)*NREG+I-1)=BASE(KPIJSY+KS-1)*FACT
            CALL LCMPOF(KPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NREG*NREG,2,
     >      IPREG)
  150       CONTINUE
         ENDIF
         IF(LBIHET) CALL LCMSIX(KPSYS,' ',2)
      ENDIF
      KPST=KPST+NPST
      KPIJSY=KPIJSY+NELPIJ*IPIJK
  200 CONTINUE
      IF(IPIJK.EQ.4) CALL RLSARA(BASE(PROBKS))
      CALL RLSARA(BASE(ISGAS))
      CALL RLSARA(BASE(ISGAR))
      CALL RLSARA(BASE(IPGAR))
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
  210 IF(LBIHET) THEN
         NREG=NREGAR
         NBMIX=NBMIXG
         DO I=1,NREG
            MAT(I)=BASE(IMAT+I-1)
            VOL(I)=BASE(IVOL+I-1)
         ENDDO
         CALL RLSARA(BASE(IMAT))
         CALL RLSARA(BASE(IVOL))
      ENDIF
      RETURN
      END
