*DECK EDI
      SUBROUTINE EDI(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* edition operator for Dragon.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and G. Marleau
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file (order is arbitrary for
*         objects 2,3,4):
*         HENTRY(1): create or modification type(L_EDIT);
*         HENTRY(2): read-only type(L_FLUX);
*         HENTRY(3): read-only type(L_MACROLIB OR L_LIBRARY);
*         HENTRY(4): read-only type(L_TRACK);
*         The following three LCM objects are required with a SPH
*         calculation (order is arbitrary for objects 5,6,7). The
*         object 5 is required if the "MERG CELL" option is used.
*         HENTRY(5): optional read-only type(L_GEOM) containing the
*         original geometry;
*         HENTRY(6): optional read-only type(L_GEOM) containing the SPH
*         macrogeometry;
*         HENTRY(7): optional read-only type(L_SYS) containing the
*         L_PIJ object of the original geometry in cases where a
*         Selengut normalization is required.
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MAXED=100,NSTATE=40,IOUT=6)
      CHARACTER*12 TEXT12,CDOOR,CNDOOR,OLDGEO,MACGEO,CARISO(MAXED),
     > CURNAM,OLDNAM,HSIGN
      CHARACTER TITLE*72,HSMG*131
      INTEGER IGP(NSTATE),KCHAR(18),IDATA(NSTATE),ISTATE(NSTATE),
     > IBASE(1)
      LOGICAL LEAKSW,LNEWGE,LSPH,LISO
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
* PARAMETER VALIDATION.
*----
      IF(NENTRY.LT.4) CALL XABORT('EDI: FOUR LCM OBJECTS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('EDI: LC'
     > //'M OBJECT EXPECTED AT LHS.')

      IPEDIT=KENTRY(1)
      IF(JENTRY(1) .EQ. 0) THEN
        HSIGN='L_EDIT'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPEDIT,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPEDIT,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN .NE. 'L_EDIT') THEN
          TEXT12=HENTRY(1)
          CALL XABORT('EDI: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >    '. L_EDIT EXPECTED.')
        ENDIF
      ELSE
        CALL XABORT('EDI: EDITING LCM OBJECT IN CREATE OR MODIFY MODE '
     >  //'EXPECTED.')
      ENDIF
*----
*  SCAN READ-ONLY MODE DATA STRUCTURE ENTRY(2) TO ENTRY(4)
*  FOR FLUX, TRACK AND LIB
*----
      IF(JENTRY(2).NE.2) CALL XABORT('EDI: LCM OBJECT IN READ-ONLY MOD'
     > //'E EXPECTED AT RHS.')
      IPFLUX=0
      IKFLUX=0
      IPTRK1=0
      IKTRK1=0
      IPLIB=0
      IKLIB=0
      DO 60 IEN=2,4
        IF((IENTRY(IEN).EQ.1).OR.(IENTRY(IEN).EQ.2)) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF((HSIGN.EQ.'L_FLUX').AND.(IKFLUX.EQ.0)) THEN
            IPFLUX=KENTRY(IEN)
            IKFLUX=IEN
          ELSE IF((HSIGN.EQ.'L_TRACK').AND.(IKTRK1.EQ.0)) THEN
            IPTRK1=KENTRY(IEN)
            IKTRK1=IEN
          ELSE IF((HSIGN.EQ.'L_LIBRARY').AND.(IKLIB.EQ.0)) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=IEN
          ELSE IF((HSIGN.EQ.'L_MACROLIB').AND.(IKLIB.EQ.0)) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=-IEN
          ENDIF
        ENDIF
   60 CONTINUE
      IF(IKFLUX.EQ.0) CALL XABORT('EDI: NO FLUXUNK LCM OBJECT FOUND.')
      IF(IKTRK1.EQ.0) CALL XABORT('EDI: NO TRACKING LCM OBJECT FOUND.')
      IF(IKLIB.EQ.0) CALL XABORT('EDI: NO MACROLIB OR MICROLIB LCM OBJ'
     > //'ECT FOUND.')
      CALL LCMGET(IPFLUX,'STATE-VECTOR',ISTATE)
      ILEAKC=ISTATE(7)
*----
*  READ MACROLIB INFORMATION
*----
      IF(IKLIB.GT.0) THEN
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        MAXISM=ISTATE(22)
        CALL LCMSIX(IPLIB,'MACROLIB',1)
      ELSE
        MAXISM=1
      ENDIF
      CALL LCMGET(IPLIB,'STATE-VECTOR',IDATA)
      NGRP=IDATA(1)
      NBMIX=IDATA(2)
      NL=IDATA(3)
      NIFISS=IDATA(4)
      NEDMAC=IDATA(5)
      ITRANC=IDATA(6)
      NDEL=IDATA(7)
      IF(IKLIB.GT.0) CALL LCMSIX(IPLIB,' ',2)
*----
*  READ GEOMETRIES AND SYSTEM (SPH ONLY)
*----
      IPGEO1=0
      IKGEO1=0
      IPGEO2=0
      IKGEO2=0
      IPTRK2=0
      IKTRK2=0
      IFTRK2=0
      IPSYS=0
      IKSYS=0
      OLDGEO=' '
      IF(NENTRY.GT.4) THEN
        DO 70 IEN=5,NENTRY
          IF((IENTRY(IEN).EQ.1).OR.(IENTRY(IEN).EQ.2)) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
            IF(HSIGN.EQ.'L_GEOM') THEN
              IF(IKGEO1.EQ.0) THEN
                IPGEO1=KENTRY(IEN)
                OLDGEO=HENTRY(IEN)
                IKGEO1=IEN
              ELSE IF(IKGEO2.EQ.0) THEN
                IPGEO2=KENTRY(IEN)
                IKGEO2=IEN
              ENDIF
            ELSE IF((HSIGN.EQ.'L_TRACK').AND.(IKTRK2.EQ.0)) THEN
              IPTRK2=KENTRY(IEN)
              IKTRK2=IEN
            ELSE IF((HSIGN.EQ.'L_PIJ').AND.(IKSYS.EQ.0)) THEN
              IPSYS=KENTRY(IEN)
              IKSYS=IEN
            ENDIF
          ELSE IF(IENTRY(IEN).EQ.3) THEN
            IF(IKTRK2.EQ.0) CALL XABORT('EDI: MACRO-TRACKING LCM STRU'
     >      //'CTURE MUST BE PROVIDED BEFORE MACRO-TRACKING FILE.')
            IFTRK2=KENTRY(IEN)
          ENDIF
   70   CONTINUE
      ENDIF
*----
*  RECOVER GENERAL TRACKING INFORMATION
*----
   80 CALL LCMGET(IPTRK1,'STATE-VECTOR',IGP)
      NREG=IGP(1)
      LEAKSW=IGP(3).EQ.0
      CALL LCMGET(IPTRK1,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
      IF(CDOOR.EQ.'MCCG') THEN
         CALL LCMLEN(IPTRK1,'KEYFLX',LKFL,ITYLCM)
         NFUNL=LKFL/NREG
      ELSE
         NFUNL=1
      ENDIF
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG*NFUNL,IIDL)
      CALL LCMGET(IPTRK1,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK1,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK1,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK1,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK1,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
         CALL LCMPUT(IPEDIT,'TITLE',18,3,KCHAR)
      ELSE
         TITLE='*** NO TITLE PROVIDED FOR THE REFERENCE CASE ***'
      ENDIF
*----
*  READ GROUP STRUCTURE
*----
      CALL SETARA(BASE,2*NGRP+1,IENERG)
      CALL SETARA(BASE,NGRP,IENERV)
      CALL LCMLEN(IPLIB,'ENERGY',NTENER,ITYLCM)
      IF(NTENER.EQ.NGRP+1) THEN
        CALL LCMGET(IPLIB,'ENERGY',BASE(IENERG))
      ELSE IF(NTENER.NE.0) THEN
        CALL XABORT('EDI: INVALID NUMBER OF GROUP ON MACROLIB.')
      ENDIF
*----
*  READ EDITION OPTIONS PARAMETERS
*----
      CALL SETARA(IBASE,NGRP,IGCOND)
      CALL SETARA(IBASE,NREG,IMERGE)
      CALL SETARA(IBASE,NBMIX,IACTI)
      CALL SETARA(IBASE,NBMIX+1,IMIXME)
      ICALL=0
      CURNAM=' '
      IPSPH=0
      IF(JENTRY(1).EQ.0) THEN
         HSIGN='L_EDIT'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPEDIT,'SIGNATURE',3,3,KCHAR)
         OLDNAM=' '
         MACGEO=' '
         NMERGE=NREG
         NGCOND=NGRP
         IHF=1
         IFFAC=0
         ILUPS=0
         NACTI=0
         NSTATS=0
         JSPH=0
         NBMICR=0
         IPRINT=1
         NSAVES=0
         NWGTH=0
         IF(ILEAKC.GE.5) NWGTH=1
         MAXISK=0
         MAXCND=0
         MAXMRG=0
         IXEDI=0
         MAXPTS=NREG
         DO 90 IGROUP=1,NGRP
   90    IBASE(IGCOND+IGROUP-1)=IGROUP
         DO 100 IREGIO=1,NREG
  100    IBASE(IMERGE+IREGIO-1)=IREGIO
         IF(IPTRK2.NE.0) THEN
            CALL LCMGET(IPTRK2,'STATE-VECTOR',IDATA)
            IF(IDATA(1).NE.NREG) CALL XABORT('EDI: INVALID MACRO-TRACK'
     1      //'ING.')
            NMERGE=IDATA(4)
            CALL SETARA(IBASE,NREG,IMAT2)
            CALL LCMGET(IPTRK2,'MATCOD',IBASE(IMAT2))
            DO 104 IREGIO=1,NREG
  104       IBASE(IMERGE+IREGIO-1)=IBASE(IMAT2+IREGIO-1)
            CALL RLSARA(IBASE(IMAT2))
         ENDIF
      ELSE IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(IPEDIT,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_EDIT') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('EDI: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >      '. L_EDIT EXPECTED.')
         ENDIF
         CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
         NMERGE=ISTATE(1)
         NGCOND=ISTATE(2)
         IFFAC=ISTATE(3)
         ILUPS=ISTATE(4)
         NACTI=ISTATE(5)
         NSTATS=ISTATE(6)
         JSPH=ISTATE(7)
         NBMICR=ISTATE(9)
         IPRINT=ISTATE(10)
         NSAVES=ISTATE(11)
         NWGTH=ISTATE(12)
         MAXISK=ISTATE(13)
         MAXCND=ISTATE(14)
         MAXMRG=ISTATE(15)
         IXEDI=ISTATE(16)
         MAXPTS=ISTATE(17)
         IHF=ISTATE(18)
         IF(ISTATE(19).NE.NDEL) CALL XABORT('EDI: BAD VALUE OF NDEL')
         CALL LCMLEN(IPEDIT,'LAST-EDIT',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPEDIT,'LAST-EDIT',KCHAR)
            WRITE(OLDNAM,'(3A4)') (KCHAR(I1),I1=1,3)
            INTLIR=0
            READ(OLDNAM,'(8X,I4)',ERR=105) INTLIR
  105       ICALL=MAX(ICALL,INTLIR)
         ENDIF
         CALL LCMLEN(IPEDIT,'REF:IMERGE',LENGT,ITYLCM)
         IF(LENGT.EQ.NREG) THEN
            CALL LCMGET(IPEDIT,'REF:IMERGE',IBASE(IMERGE))
         ELSE
            DO 106 IREGIO=1,NREG
  106       IBASE(IMERGE+IREGIO-1)=IREGIO
         ENDIF
         CALL LCMLEN(IPEDIT,'REF:IGCOND',LENGT,ITYLCM)
         IF(LENGT.EQ.NGCOND) THEN
            CALL LCMGET(IPEDIT,'REF:IGCOND',IBASE(IGCOND))
         ELSE
            DO 107 IGROUP=1,NGRP
  107       IBASE(IGCOND+IGROUP-1)=IGROUP
         ENDIF
         CALL LCMLEN(IPEDIT,'LINK.MACGEOM',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET(IPEDIT,'LINK.MACGEOM',KCHAR)
            WRITE(MACGEO,'(3A4)') (KCHAR(I),I=1,3)
         ELSE
            MACGEO=' '
         ENDIF
         IF(NBMICR.GT.0) THEN
            CALL SETARA(BASE,3*NBMICR,IJCEDM)
            CALL LCMGET(IPEDIT,'CARISO',BASE(IJCEDM))
            IDEB=IJCEDM
            DO 110 J=1,NBMICR
              WRITE(CARISO(J),'(3A4)') (BASE(IDEB+I1),I1=0,2)
              IDEB=IDEB+3
  110       CONTINUE
            CALL RLSARA(BASE(IJCEDM))
         ENDIF
         IF(NACTI.GT.0) CALL LCMGET(IPEDIT,'IACTI',IBASE(IACTI))
      ENDIF
      NGCR=0
      LISO=.FALSE.
      MAXISK=MAX(MAXISK,MAXISM)
      CALL SETARA(IBASE,NGRP+1,IGCR)
      CALL SETARA(BASE,NGRP+1,IECR)
      CALL XDISET(IBASE(IGCR),NGRP+1,NGRP)
      CALL XDRSET(BASE(IECR),NGRP+1,0.0)
      CALL EDIGET(NGRP,NGCR,NREG,NBMIX,NMERGE,IHF,IFFAC,ILUPS,NSAVES,
     1 NSTATS,IBASE(IGCR),BASE(IECR),IBASE(IMERGE),CURNAM,OLDNAM,
     2 LSPH,NWGTH,IBASE(IMAT),NBMICR,CARISO,NACTI,IBASE(IACTI),IPRINT,
     3 MAXPTS,ICALL,ISOTXS,LISO,IBASE(IMIXME))
      CALL RLSARA(IBASE(IMIXME))
*----
*  RECOVER SPH OPTIONS
*----
      IF(LSPH) THEN
        CALL SPHGET(IPEDIT,IPSPH,NREG,NBMIX,IBASE(IMAT),CDOOR,CNDOOR,
     1  MACGEO,JSPH)
      ELSE
        IPSPH=0
        CNDOOR=' '
      ENDIF
*----
*  CALL EDIMRG TO FIND MERGE INDEX ASSOCIATED WITH THE SECOND GEOMETRY
*  OR TRACK FILE (EQUIGEOM CAPABILITIES)
*----
      TEXT12=' '
      LNEWGE=.FALSE.
      IF(NMERGE.EQ.-1) THEN
        IF(IKGEO2.GT.0) THEN
          ITM=-1
          IPMRG=IPGEO2
          TEXT12=HENTRY(IKGEO2)
        ELSE IF(IKGEO1.GT.0) THEN
          ITM=-1
          IPMRG=IPGEO1
          TEXT12=HENTRY(IKGEO1)
        ELSE IF(IKTRK2.GT.0) THEN
          ITM=1
          IPMRG=IPTRK2
        ELSE
          ITM=0
          IPMRG=IPTRK1
        ENDIF
        CALL EDIMRG(IPTRK1,IPMRG,IPRINT,TEXT12,ITM,NREG,NMERGE,
     >              IBASE(IMERGE))
*----
*  BUILD A MACRO-GEOMETRY FROM REFERENCE GEOMETRY OLDGEO (CELL OPTION)
*----
      ELSE IF(NMERGE.EQ.-2) THEN
        IF(IPGEO1.EQ.0) CALL XABORT('EDI: NO REFERENCE GEOMETRY AVAILA'
     >  //'BLE.')
        IF(IPGEO2.NE.0) CALL XABORT('EDI: INPUT MACRO-GEOMETRY NOT EXP'
     >  //'ECTED WITH CELL OPTION.')
        IF(IPRINT.GT.0) WRITE(IOUT,190) OLDGEO,CDOOR
        CALL LCMGET(IPGEO1,'STATE-VECTOR',ISTATE)
        MAXGEO=MAX(MAXPTS,ISTATE(6))
        LNEWGE=.TRUE.
        IF(JSPH.GT.0) JSPH=3
        MACGEO='MACRO$GEO'
        CALL LCMOP(IPGEO2,'MACRO$GEO',0,1,9)
        IF((CNDOOR.EQ.'SYBIL').OR.(CNDOOR.EQ.'EXCELL')) THEN
           IEUR=1
        ELSE IF(CNDOOR.EQ.'NXT') THEN
           IEUR=2
        ELSE
           IEUR=3
        ENDIF
        MAXMER=MIN(NREG,MAXGEO)
        CALL SETARA(BASE,MAXGEO,IXX)
        CALL SETARA(BASE,MAXGEO,IYY)
        CALL SETARA(BASE,2*MAXGEO,IINUM)
        CALL SETARA(BASE,MAXGEO,IIGEN)
        CALL SETARA(BASE,2*MAXGEO,IIORI)
        CALL SETARA(BASE,2*MAXGEO,IIMAT)
        CALL SETARA(BASE,MAXGEO+1,INMC)
        CALL EDIGEO(MAXGEO,MAXMER,IPGEO1,IPGEO2,IPRINT,NREG,IEUR,
     1  NMERGE,IBASE(IMERGE),BASE(IXX),BASE(IYY),BASE(IINUM),
     2  BASE(IIGEN),BASE(IIORI),BASE(IIMAT),BASE(INMC))
        CALL RLSARA(BASE(INMC))
        CALL RLSARA(BASE(IIMAT))
        CALL RLSARA(BASE(IIORI))
        CALL RLSARA(BASE(IIGEN))
        CALL RLSARA(BASE(IINUM))
        CALL RLSARA(BASE(IYY))
        CALL RLSARA(BASE(IXX))
*
*       COPY THE MACRO-GEOMETRY INTO THE EDITION OBJECT.
        CALL LCMSIX(IPEDIT,'MACRO-GEOM',1)
        CALL LCMEQU(IPGEO2,IPEDIT)
        CALL LCMSIX(IPEDIT,' ',2)
      ENDIF
*----
*  TEST ENERGY CONDENSATION INPUT
*----
      CALL EDIENE(NGRP,NGCR,NGCOND,NTENER,IBASE(IGCR),BASE(IECR),
     > IBASE(IGCOND),BASE(IENERG),BASE(IENERV))
*
      IF(IPRINT.GT.0) THEN
        WRITE(IOUT,200) NMERGE,NGCOND,IFFAC,ILUPS,NACTI,NSTATS,NBMICR,
     >  JSPH,NSAVES,NWGTH,MAXPTS,IHF
        WRITE(IOUT,'(//15H MERGING INDEX:/(1X,14I5))')
     >    (IBASE(IMERGE+I-1),I=1,NREG)
        IF(CURNAM.NE.' ') WRITE(IOUT,'(/27H EDI: SAVE MICROLIB INFO ON,
     >  12H DIRECTORY '',A12,2H''.)') CURNAM
      ENDIF
      CALL RLSARA(BASE(IECR))
      CALL RLSARA(IBASE(IGCR))
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NMERGE
      ISTATE(2)=NGCOND
      ISTATE(3)=IFFAC
      ISTATE(4)=ILUPS
      ISTATE(5)=NACTI
      ISTATE(6)=NSTATS
      ISTATE(7)=JSPH
      ISTATE(9)=NBMICR
      ISTATE(10)=IPRINT
      ISTATE(11)=NSAVES
      ISTATE(12)=NWGTH
      ISTATE(13)=MAXISK
      ISTATE(14)=MAX(NGCOND,MAXCND)
      ISTATE(15)=MAX(NMERGE,MAXMRG)
      ISTATE(16)=IXEDI+ISOTXS*NMERGE
      ISTATE(17)=MAXPTS
      ISTATE(18)=IHF
      ISTATE(19)=NDEL
      CALL LCMLEN(IPEDIT,'MACRO-GEOM',ILONG,ITYLCM)
      IF(ILONG.NE.0) ISTATE(20)=1
      CALL LCMPUT(IPEDIT,'STATE-VECTOR',NSTATE,1,ISTATE)
      IF(OLDGEO.NE.' ') THEN
         READ(OLDGEO,'(3A4)') (KCHAR(I),I=1,3)
         CALL LCMPUT(IPEDIT,'LINK.GEOM',3,3,KCHAR)
      ENDIF
      IF(NSAVES.GE.2) THEN
         READ(CURNAM,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPEDIT,'LAST-EDIT',3,3,KCHAR)
      ENDIF
      IF(NBMICR.GT.0) THEN
         CALL SETARA(BASE,3*NBMICR,IJCEDM)
         IDEB=IJCEDM
         DO 120 J=1,NBMICR
           READ(CARISO(J),'(3A4)') (BASE(IDEB+I1),I1=0,2)
           IDEB=IDEB+3
  120    CONTINUE
         CALL LCMPUT(IPEDIT,'CARISO',3*NBMICR,3,BASE(IJCEDM))
         CALL RLSARA(BASE(IJCEDM))
      ENDIF
      IF(NACTI.GT.0) CALL LCMPUT(IPEDIT,'IACTI',NACTI,1,IBASE(IACTI))
      IF(MACGEO.NE.' ') THEN
         IF(HENTRY(IKGEO1).EQ.MACGEO) THEN
            IPGEO2=IPGEO1
            IKGEO2=IKGEO1
         ENDIF
         IF(IPGEO2.EQ.0) CALL XABORT('EDI: MISSING LCM OBJECT FOR THE '
     >   //'SPH MACRO-GEOMETRY.')
         IF(IKGEO2.NE.0) THEN
           IF(HENTRY(IKGEO2).NE.MACGEO) THEN
             WRITE(HSMG,'(37HEDI: WRONG SPH MACRO-GEOMETRY NAMED '',
     >       A12,17H'' FOUND ON RHS. '',A12,11H'' EXPECTED.)')
     >       HENTRY(IKGEO2),MACGEO
             CALL XABORT(HSMG)
           ENDIF
         ENDIF
         CALL LCMGET(IPGEO2,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_GEOM') THEN
            CALL XABORT('EDI: SIGNATURE OF '//MACGEO//' IS '//HSIGN//
     >      '. L_GEOM EXPECTED.')
         ENDIF
*
*        COPY THE MACRO-GEOMETRY INTO THE EDITION OBJECT.
         CALL LCMSIX(IPEDIT,'MACRO-GEOM',1)
         CALL LCMEQU(IPGEO2,IPEDIT)
         CALL LCMSIX(IPEDIT,' ',2)
      ENDIF
*----
*  STORE SPH-RELATED INFORMATION
*----
      IF((JSPH.EQ.0).OR.(JSPH.EQ.1)) THEN
         IPTRK2=0
	 IFTRK2=0
      ELSE IF(JSPH.EQ.2) THEN
         IF(IPTRK2.EQ.0) CALL XABORT('EDI: NO MACRO-TRACKING.')
         CALL LCMGET(IPTRK2,'TRACK-TYPE',KCHAR)
         WRITE(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
         IF((CNDOOR.EQ.'EXCELL').AND.(IFTRK2.EQ.0)) CALL XABORT('EDI: '
     >   //'TRACKING FILE MISSING FOR SPH WITH '//CNDOOR)
         IF((CNDOOR.EQ.'NXT').AND.(IFTRK2.EQ.0)) CALL XABORT('EDI: '
     >   //'TRACKING FILE MISSING FOR SPH WITH '//CNDOOR)
         IF((CNDOOR.EQ.'MCCG').AND.(IFTRK2.EQ.0)) CALL XABORT('EDI: '
     >   //'TRACKING FILE MISSING FOR SPH WITH '//CNDOOR)
	 NSPH=4
	 IF((CNDOOR.EQ.'SYBIL').OR.(CNDOOR.EQ.'EXCELL').OR.
     >   (CNDOOR.EQ.'NXT')) NSPH=3
      ELSE IF(JSPH.EQ.3) THEN
         IF(IPGEO2.EQ.0) CALL XABORT('EDI: NO MACRO-GEOMETRY.')
         IPTRK2=0
	 IFTRK2=0
      ENDIF
*----
*  EDITION
*----
      IF(NREG.EQ.0) CALL XABORT('EDI: NREG = 0.')
      IF(NGRP.EQ.0) CALL XABORT('EDI: NGRP = 0.')
      CALL SETARA(BASE,NREG*NGRP*(NWGTH+1),JFLUXE)
      CALL EDIDRV(IPEDIT,IPFLUX,IPLIB,IPSYS,IPGEO2,IPTRK2,IFTRK2,NGRP,
     1 NBMIX,NREG,LEAKSW,IBASE(IMAT),BASE(IVOL),IBASE(IIDL),NIFISS,
     2 NEDMAC,TITLE,NL,NDEL,ITRANC,NGCOND,NMERGE,NWGTH,IHF,IFFAC,ILUPS,
     3 NSAVES,NSTATS,IXEDI,ISOTXS,IBASE(IGCOND),IBASE(IMERGE),CURNAM,
     4 OLDNAM,IPSPH,NBMICR,CARISO,NACTI,IBASE(IACTI),IPRINT,LISO,
     5 BASE(JFLUXE))
      CALL RLSARA(BASE(JFLUXE))
*----
*  DESTROY THE TEMPORARY MACRO-GEOMETRY
*----
      IF(LNEWGE) THEN
         CALL LCMCL(IPGEO2,1)
         TEXT12='MACRO$GEO'
         CALL LCMOP(IPGEO2,TEXT12,1,1,0)
         CALL LCMCL(IPGEO2,2)
      ENDIF
*----
*  COMPLETE THE EDITION LCM OBJECT
*----
      CALL LCMPUT(IPEDIT,'REF:IMERGE',NREG,1,IBASE(IMERGE))
      CALL LCMPUT(IPEDIT,'REF:MATCOD',NREG,1,IBASE(IMAT))
      CALL LCMPUT(IPEDIT,'REF:VOLUME',NREG,2,BASE(IVOL))
      CALL LCMPUT(IPEDIT,'REF:IGCOND',NGCOND,1,IBASE(IGCOND))
*
      CALL RLSARA(IBASE(IACTI))
      CALL RLSARA(IBASE(IMERGE))
      CALL RLSARA(IBASE(IGCOND))
*----
*  RELEASE GENERAL TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      CALL RLSARA(BASE(IENERV))
      CALL RLSARA(BASE(IENERG))
      IF(IPRINT.GT.2) CALL LCMLIB(IPEDIT)
      RETURN
*
  190 FORMAT (/16H EDI: GEOMETRY ',A12,28H' WAS PREVIOUSLY TRACKED BY ,
     > 7HMODULE ,A12,1H.)
  200 FORMAT(/8H OPTIONS/1X,7(1H-)/
     1  7H NMERGE,I8,29H   (NUMBER OF MERGED REGIONS)/
     2  7H NGCOND,I8,38H   (NUMBER OF CONDENSED ENERGY GROUPS)/
     3  7H IFFAC ,I8,40H   (=1: 4 FACTORS CALCULATION REQUESTED)/
     4  7H ILUPS ,I8,43H   (=1: REMOVE UP-SCATTERING CONTRIBUTIONS)/
     5  7H NACTI ,I8,45H   (NUMBER OF MIXTURES WITH ACTIVATION EDITS)/
     6  7H NSTATS,I8,35H   (TYPE OF STATISTIC CALCULATIONS)/
     7  7H NBMICR,I8,47H   (=-1: PROCESS ALL ISOTOPES; >1 NUMBER OF ISO,
     8  17HTOPES TO PROCESS)/
     9  7H JSPH  ,I8,47H   (=0: DO NOT USE SPH FRAMEWORK; =1: USE WITHO,
     1  60HUT MACRO-GEOMETRY NOR MACRO-TRACKING; =2: USE WITH A MACRO-T,
     2  8HRACKING;/19X,30H=3: USE WITH A MACRO-GEOMETRY)/
     3  7H NSAVES,I8,47H   (=0: NO COMPUTE/NO SAVE; =1: COMPUTE/NO SAVE,
     4  19H; =2: COMPUTE/SAVE)/
     5  7H NWGTH ,I8,47H   (=0: FLUX WEIGHTING FOR P1 INFO; =1: CURRENT,
     6  23H WEIGHTING FOR P1 INFO)/
     7  7H MAXPTS,I8,47H   (ALLOCATED STORAGE LENGTH FOR REGION-DEPENDE,
     8  10HNT ARRAYS)/
     9  7H IHF   ,I8,39H   (=1: H-FACTOR CALCULATION REQUESTED))
      END
