*DECK EDIDRV
      SUBROUTINE EDIDRV(IPEDIT,IPFLUX,IPLIB,IPSYS,IPGEO2,IPTRK2,IFTRK2,
     >                  NGROUP,NBMIX,NREGIO,LEAKSW,MATCOD,VOLUME,KEYFLX,
     >                  NIFISS,NEDMAC,CTITRE,NL,NDEL,ITRANC,NGCOND,
     >                  NMERGE,NWGTH,IHF,IFFAC,ILUPS,NSAVES,NSTATS,
     >                  IXEDI,ISOTXS,IGCOND,IMERGE,CURNAM,OLDNAM,IPSPH,
     >                  NBMICR,CARISO,NACTI,IACTI,IPRINT,LISO,FLUXES)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for edition operations.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): G. Marleau
*
*Parameters: input
* IPEDIT  pointer to the edition LCM object.
* IPFLUX  pointer to the flux LCM object.
* IPLIB   pointer to the internal library or macrolib LCM object.
* IPSYS   pointer to the pij LCM object (only used with Selengut
*         normalization).
* IPGEO2  pointer to the macro geometry LCM object.
* IPTRK2  pointer to the macro-tracking LCM object.
* IFTRK2  pointer to the macro-tracking binary sequential file.
* NGROUP  number of energy groups.
* NBMIX   number of mixtures.
* NREGIO  number of regions.
* LEAKSW  logical for leakage.
* MATCOD  mixture index in region.
* VOLUME  volume of region.
* KEYFLX  average flux position per region.
* NIFISS  number fissile isotopes.
* NEDMAC  number of extra macroscopic cross section types.
* CTITRE  execution title.
* NL      number of Legendre orders of the scattering cross sections.
* NDEL    number of delayed precursor groups.
* ITRANC  type of transport correction.
* NMERGE  number of regions merged.
* NWGTH   type of weighting for P1 cross section information:
*         = 0 P0; = 1 P1.
* IHF     H-factor calculation flag:
*         = 0 no; = 1 yes.
* IFFAC   four factor calculation flag:
*         = 0 no four factors (defaut);
*         = 1 four factor evaluation.
* ILUPS   flag to remove up-scattering from output.
* NSAVES  homogenized cross section computation and saving:
*         = 0  no compute no save;
*         = 1  compute, no save;
*         = 2  compute, save.
* NSTATS  statistics level:
*         = 0  no stats;
*         = 1  statistics on fluxes
*         = 2  statistics on reaction rates;
*         = 3  statistics on fluxes and reaction rates;
*         =-1  delta sigma ('MERG COMP' only).
* IXEDI   first ISOTX mixture record number.
* ISOTXS  ISOTX file enabling flag (0: off; 1: binary; 2: ascii).
* IGCOND  condensed groups limits.
* IMERGE  merged regions position.
* CURNAM  name of LCM directory where the current rates are to be
*         stored.
* OLDNAM  name of LCM directory where old rates were stored.
* IPSPH   pointer to the LCM directory containing SPH-related info.
*         IPSPH=0 to avoid the SPH calculation.
* NBMICR  type of microlib edition:
*         =-2: process only macroscopic residue;
*         =-1: process each isotope;
*         =0: process no isotope;
*         >0 number of isotopes to process.
* CARISO  names of the isotopes to process.
* NACTI   number of activation editions.
* IACTI   activation mixtures.
* IPRINT  print index.
* LISO    =.TRUE. if we want to keep all the isotopes after 
*         homogeneization
*
*Parameters: scratch
* FLUXES  fluxes.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      LOGICAL       LEAKSW,LISO
      INTEGER       IPEDIT,IPFLUX,IPLIB,IPSYS,IPGEO2,IPTRK2,IFTRK2,
     >              NGROUP,NBMIX,NREGIO,MATCOD(NREGIO),KEYFLX(NREGIO),
     >              NIFISS,NEDMAC,NL,NDEL,ITRANC,NGCOND,NMERGE,NWGTH,
     >              IHF,IFFAC,ILUPS,NSAVES,NSTATS,IXEDI,ISOTXS,
     >              IGCOND(NGROUP),IMERGE(NREGIO),IPSPH,NBMICR,NACTI,
     >              IACTI(NBMIX),IPRINT
      REAL          VOLUME(NREGIO),FLUXES(NREGIO,NGROUP,NWGTH+1)
      CHARACTER     CTITRE*(*),CURNAM*(*),OLDNAM*(*),CARISO(NBMICR)*(*)
*----
*  LOCAL VARIABLES
*----
      PARAMETER    (IUNOUT=6,MAXED=100,NSTATE=40,IOUT=6)
      CHARACTER     HSIGN*12,TEXT8*8,HVECT(MAXED)*8,NISEXT*6,NISOTX*12,
     >              CTITLE*72,HNEW*12,NAMSBR*12
      INTEGER       IBASE(1),IFPAR(NSTATE),IPAR(NSTATE),KCHAR(3),
     >              IDIM(NSTATE)
      REAL          B2(4),B2T(3),TIMEF(3)
      COMMON        RBASE(1)
      EQUIVALENCE  (RBASE(1),IBASE(1))
*----
*  FIND THE SIGNATURE OF IPLIB
*----
      CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
*----
*  RECOVER NEUTRON FLUXES AND CURRENTS (IF ILEAKC.GE.5)
*----
      CALL LCMGET(IPFLUX,'STATE-VECTOR',IFPAR)
      ITYPEC=IFPAR(6)
      ILEAKC=IFPAR(7)
      IF(ILEAKC.EQ.0) THEN
*       NO LEAKAGE
        ILEAKS=0
      ELSE IF(ILEAKC.LE.4) THEN
*       DIFFON-TYPE LEAKAGE
        ILEAKS=1
      ELSE IF(ILEAKC.EQ.5) THEN
*       ECCO-TYPE LEAKAGE (WITH ISOTROPIC STREAMING EFFECTS)
        ILEAKS=2
      ELSE IF(ILEAKC.GE.6) THEN
*       TIBERE-TYPE LEAKAGE (WITH ANISOTROPIC STREAMING EFFECTS)
        ILEAKS=3
      ENDIF
      IF(ITYPEC.GT.0) CALL LCMGET(IPFLUX,'K-INFINITY',CUREIN)
      IF(ITYPEC.GT.2) THEN
        CALL LCMGET(IPFLUX,'B2  B1HOM',B2(4))
        IF(ILEAKS.EQ.3) THEN
          CALL LCMGET(IPFLUX,'B2  HETE',B2)
          IF(B2(4).EQ.0.0) THEN
            B2T(1)=1.0/3.0
            B2T(2)=B2T(1)
            B2T(3)=B2T(1)
          ELSE
            B2T(1)=B2(1)/B2(4)
            B2T(2)=B2(2)/B2(4)
            B2T(3)=B2(3)/B2(4)
          ENDIF
        ENDIF
      ENDIF
      IF((NWGTH.GE.1).AND.(ILEAKC.LE.4)) THEN
        CALL XABORT('EDIDRV: CURRENT WEIHTING OF P1 XS INFO (NWGTH=1) '
     >  //'IS ONLY AVAILABLE WITH A STREAMING-ENABLED LEAKAGE MODEL.')
      ENDIF
      CALL LCMLEN(IPFLUX,'FLUX',ILON,ITYLCM)
      IF(ILON.EQ.0) CALL XABORT('EDIDRV: MISSING FLUX INFO.')
      JPFLUX=LCMGID(IPFLUX,'FLUX')
      CALL LCMLEL(JPFLUX,1,NUN,ITYLCM)
      CALL SETARA(RBASE,NUN,IWORK)
      DO 120 IGR=1,NGROUP
        CALL LCMGDL(JPFLUX,IGR,RBASE(IWORK))
        DO 80 IREG=1,NREGIO
          FLUXES(IREG,IGR,1)=RBASE(IWORK+KEYFLX(IREG)-1)
   80   CONTINUE
        IF((NWGTH.GE.1).AND.(ILEAKS.EQ.2)) THEN
*         ISOTROPIC STREAMING (ECCO)
          DO 90 IREG=1,NREGIO
            FLUXES(IREG,IGR,2)=RBASE(IWORK+NUN/2+KEYFLX(IREG)-1)
   90     CONTINUE
        ELSE IF((NWGTH.GE.1).AND.(ILEAKS.EQ.3)) THEN
*         ANISOTROPIC STREAMING
          DO 110 IREG=1,NREGIO
            CURN=0.0
            DO 100 IDIR=1,3
              CURN=CURN+B2T(IDIR)*RBASE(IWORK+IDIR*NUN/4+KEYFLX(IREG)-1)
  100       CONTINUE
            FLUXES(IREG,IGR,2)=CURN
  110     CONTINUE
        ENDIF
  120 CONTINUE
      CALL RLSARA(RBASE(IWORK))
*----
*  ALLOCATE MEMORY FOR GROUP CONDENSATION AND MERGE
*----
      CALL SETARA(RBASE,NMERGE,IVOLME)
      CALL SETARA(RBASE,NGCOND,IWLETY)
      CALL SETARA(RBASE,NGCOND+1,IWE)
      NELEMT=NMERGE*NGCOND
*----
*  USE SPH FRAMEWORK AND SET SPH FACTORS
*----
      CALL SETARA(RBASE,NELEMT*(NWGTH+1),ISPH)
      IF(IPSPH.EQ.0) THEN
        CALL XDRSET(RBASE(ISPH),NELEMT*(NWGTH+1),1.0)
      ELSE
        IF(HSIGN.EQ.'L_LIBRARY') CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL SPHDRV(IPEDIT,IPGEO2,IPFLUX,IPLIB,IPSYS,IPRINT,ITRANC,
     >              NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,ILEAKS,NWGTH,
     >              NL,NIFISS,LEAKSW,MATCOD,VOLUME,KEYFLX,IGCOND,
     >              IMERGE,FLUXES,IPSPH,IPTRK2,IFTRK2,CURNAM,
     >              CTITRE,RBASE(ISPH))
        IF(HSIGN.EQ.'L_LIBRARY') CALL LCMSIX(IPLIB,' ',2)
      ENDIF
*----
*  COMPUTE REACTION RATES FOR THE EDITION MACROLIB
*----
      NTAUXT=15+2*NDEL
      CALL SETARA(RBASE,NTAUXT*NELEMT,ITAUXT)
      CALL SETARA(RBASE,NL*NELEMT,ISIGS)
      NBISO=0
      CALL SETARA(RBASE,NELEMT*NGCOND*NL,ISCATS)
      CALL SETARA(RBASE,NREGIO*NGROUP*(NWGTH+1),IFLINT)
      CALL SETARA(RBASE,2*NELEMT*NGCOND*NL,ISCATD)
      CALL LCMLEN(IPFLUX,'K-EFFECTIVE',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.1) THEN
        CALL LCMGET(IPFLUX,'K-EFFECTIVE',EIGENK)
      ELSE
        EIGENK=1.0
      ENDIF
      CALL SETARA(RBASE,NGCOND,IDISFC)
      CALL SETARA(RBASE,NGCOND*NMERGE,IFLDMC)
      CALL SETARA(RBASE,NELEMT*NEDMAC,ITAUXE)
      CALL SETARA(IBASE,2*NEDMAC,INAMEA)
      CALL SETARA(RBASE,NREGIO*NIFISS,IFFUEL)
      CALL SETARA(IBASE,NREGIO,IFUELR)
      CALL SETARA(RBASE,NGROUP,IENERV)
      CALL SETARA(RBASE,NGCOND*NMERGE,IOVERV)
      CALL SETARA(RBASE,NGCOND*NMERGE,IHFACT)
      CALL SETARA(RBASE,NDEL*NIFISS,IDECAY)
      TIMEF(1)=0.0
      TIMEF(2)=0.0
      TIMEF(3)=0.0
      IF(HSIGN.EQ.'L_LIBRARY') THEN
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMLEN(IPLIB,'TIMESTAMP',ILCMLN,ILCMTY)
        IF((ILCMLN.GE.1).AND.(ILCMLN.LE.3)) THEN
           CALL LCMGET(IPLIB,'TIMESTAMP',TIMEF)
        ENDIF
      ENDIF
      CALL EDIDTX(IPEDIT,IPFLUX,IPLIB,IPRINT,NL,NDEL,ITRANC,NGROUP,
     >            NGCOND,NBMIX,NREGIO,NMERGE,ILEAKS,ILUPS,NWGTH,MATCOD,
     >            VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,EIGENK,RBASE(ISPH),
     >            RBASE(IVOLME),RBASE(IWLETY),RBASE(IWE),RBASE(ITAUXT),
     >            RBASE(IFLINT),RBASE(ISCATD),RBASE(ISCATS),NIFISS,
     >            RBASE(IDISFC),NSAVES,CURNAM,NEDMAC,RBASE(ISIGS),
     >            RBASE(ITAUXE),IBASE(INAMEA),RBASE(IFFUEL),
     >            IBASE(IFUELR),RBASE(IFLDMC),RBASE(IENERV),
     >            RBASE(IOVERV),RBASE(IHFACT),B2,CUREIN,TIMEF,NTAUXT,
     >            RBASE(IDECAY))
      IF(HSIGN.EQ.'L_LIBRARY') CALL LCMSIX(IPLIB,' ',2)
      CALL RLSARA(RBASE(IDECAY))
      CALL RLSARA(RBASE(IHFACT))
      CALL RLSARA(RBASE(IOVERV))
      CALL RLSARA(RBASE(IENERV))
      CALL RLSARA(IBASE(IFUELR))
      CALL RLSARA(RBASE(IFFUEL))
      CALL RLSARA(IBASE(INAMEA))
      CALL RLSARA(RBASE(ITAUXE))
      CALL RLSARA(RBASE(IFLDMC))
      CALL RLSARA(RBASE(IDISFC))
      CALL RLSARA(RBASE(ISCATD))
      CALL RLSARA(RBASE(IFLINT))
*----
*  RECOVER ISOTOPIC INFORMATION FROM THE MICROLIB
*----
      IF(HSIGN.EQ.'L_LIBRARY') THEN
        CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
        NBISO=IPAR(2)
        NED=IPAR(13)
        IF(NBISO.EQ.0) CALL XABORT('EDIDRV: NO ISOTOPES FOUND.')
        CALL SETARA(RBASE,NBISO,IDEN)
        CALL LCMGET(IPLIB,'ISOTOPESDENS',RBASE(IDEN))
        CALL SETARA(IBASE,NBISO,IITYPE)
        CALL LCMGET(IPLIB,'ISOTOPESTYPE',IBASE(IITYPE))
        CALL SETARA(IBASE,NBISO,IMIX)
        CALL LCMGET(IPLIB,'ISOTOPESMIX',IBASE(IMIX))
        CALL SETARA(RBASE,NBISO,ITN)
        CALL LCMGET(IPLIB,'ISOTOPESTEMP',RBASE(ITN))
        CALL SETARA(IBASE,NBISO,IDEPL)
        CALL LCMGET(IPLIB,'ISOTOPESTODO',IBASE(IDEPL))
        CALL SETARA(IBASE,2*NED,IJCEDM)
        CALL LCMGET(IPLIB,'ADDXSNAME-P0',IBASE(IJCEDM))
        IDEB=IJCEDM
        DO 130 J=1,NED
          WRITE(HVECT(J),'(2A4)') IBASE(IDEB),IBASE(IDEB+1)
          IDEB=IDEB+2
  130   CONTINUE
        CALL RLSARA(IBASE(IJCEDM))
        CALL SETARA(IBASE,3*NBISO,IISONA)
        CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(IISONA))
        CALL SETARA(IBASE,3*NBISO,IISONR)
        CALL LCMGET(IPLIB,'ISOTOPERNAME',IBASE(IISONR))
        CALL SETARA(IBASE,NBISO,ILSISO)
        CALL XDISET(IBASE(ILSISO),NBISO,0)
        IF(NBMICR.EQ.-2) THEN
          CALL XDISET(IBASE(ILSISO),NBISO,0)
        ELSE IF(NBMICR.EQ.-1) THEN
          CALL XDISET(IBASE(ILSISO),NBISO,1)
        ELSE IF(NBMICR.GT.0) THEN
          DO 160 IISO=1,NBISO
            WRITE(TEXT8,'(2A4)') (IBASE(IISONA+(IISO-1)*3+I0),I0=0,1)
            DO 150 IIII=1,NBMICR
              IF(CARISO(IIII)(1:8).EQ.TEXT8) IBASE(ILSISO+IISO-1)=1
  150       CONTINUE
  160     CONTINUE
        ENDIF
      ENDIF
*----
*  EVALUATE H-FACTOR IF REQUIRED FOR THE EDITION MACROLIB
*----
      CALL SETARA(RBASE,NBISO,JEMEVF)
      CALL XDRSET(RBASE(JEMEVF),NBISO,0.0)
      CALL SETARA(RBASE,NBISO,JEMEVG)
      CALL XDRSET(RBASE(JEMEVG),NBISO,0.0)
      IF((NSAVES.GE.2).AND.(IHF.NE.0)) THEN
        CALL LCMLEN(IPLIB,'DEPL-CHAIN',ILLCM,ITLCM)
        IF(ILLCM.NE.0) THEN
          CALL LCMSIX(IPEDIT,CURNAM,1)
          CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
          CALL LCMGET(IPLIB,'STATE-VECTOR',IDIM)
          NDEPL=IDIM(1)
          NREAC=IDIM(8)
*
          CALL SETARA(IBASE,3*NDEPL,INADPL)
          CALL LCMGET(IPLIB,'ISOTOPESDEPL',IBASE(INADPL))
          CALL SETARA(RBASE,NREAC*NDEPL,IRER)
          CALL LCMGET(IPLIB,'DEPLETE-ENER',RBASE(IRER))
          CALL LCMSIX(IPLIB,' ',2)
*
          CALL SETARA(RBASE,NGROUP,ISIG)
          CALL SETARA(RBASE,NMERGE,IHFACT)
          CALL SETARA(RBASE,2*NGCOND*NMERGE,IFLXME)
          CALL SETARA(RBASE,2*NGCOND*NMERGE,IWORK)
          CALL SETARA(IBASE,NBISO,IINDX)
          CALL EDIHFC(IPEDIT,IPLIB,NGROUP,NGCOND,NREGIO,NMERGE,NBISO,
     >                NDEPL,NREAC,MATCOD,VOLUME,IBASE(INADPL),
     >                IBASE(IISONA),IBASE(IISONR),IBASE(IMIX),
     >                FLUXES(1,1,1),RBASE(ISPH),RBASE(IDEN),
     >                IGCOND,IMERGE,RBASE(IRER),RBASE(JEMEVF),
     >                RBASE(JEMEVG),RBASE(ISIG),RBASE(IHFACT),
     >                RBASE(IFLXME),RBASE(IWORK),IBASE(IINDX))
          CALL RLSARA(IBASE(IINDX))
          CALL RLSARA(RBASE(IWORK))
          CALL RLSARA(RBASE(IFLXME))
          CALL RLSARA(RBASE(IHFACT))
          CALL RLSARA(RBASE(ISIG))
*
          CALL RLSARA(RBASE(IRER))
          CALL RLSARA(IBASE(INADPL))
          CALL LCMSIX(IPEDIT,' ',2)
        ENDIF
      ENDIF
*----
*  LUMP THE DEPLETION CHAIN
*----
      CALL SETARA(RBASE,NBISO,JDECAY)
      CALL XDRSET(RBASE(JDECAY),NBISO,0.0)
      CALL SETARA(RBASE,(NGCOND+1)*NBISO*NMERGE,JYIELD)
      CALL XDRSET(RBASE(JYIELD),(NGCOND+1)*NBISO*NMERGE,0.0)
      NDEPL=0
      NDFI=0
      JPIFI=JYIELD
      JPYIEL=JYIELD
      IF((NBMICR.NE.0).AND.(NBISO.NE.0)) THEN
        CALL LCMSIX(IPEDIT,CURNAM,1)
        CALL LCMLEN(IPLIB,'DEPL-CHAIN',ILCMLN,ITYLCM)
        IF((ILCMLN.NE.0).AND.(CURNAM.NE.' ')) THEN
          CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
          CALL EDIDEP(IPRINT,IPLIB,IPEDIT,NBISO,IBASE(IISONR),
     1    IBASE(ILSISO),LISO,NBCH)
          CALL LCMSIX(IPLIB,' ',2)
        ENDIF
*----
*  RECOVER DEPLETION INFORMATION FROM THE INTERNAL LIBRARY
*----
        CALL LCMLEN(IPEDIT,'DEPL-CHAIN',ILLCM,ITLCM)
        IF(ILLCM.NE.0) THEN
          CALL LCMSIX(IPEDIT,'DEPL-CHAIN',1)
          CALL LCMGET(IPEDIT,'STATE-VECTOR',IDIM)
          NDEPL=IDIM(1)
          NDFI=IDIM(2)
          NDFP=IDIM(3)
          NREAC=IDIM(8)
          CALL SETARA(IBASE,NDFI*NMERGE,JPIFI)
          CALL SETARA(RBASE,NDFI*NBISO*NMERGE,JPYIEL)
*
          CALL SETARA(IBASE,3*NDEPL,INADPL)
          CALL LCMGET(IPEDIT,'ISOTOPESDEPL',IBASE(INADPL))
          CALL SETARA(IBASE,NREAC*NDEPL,IKDRI)
          CALL LCMGET(IPEDIT,'DEPLETE-REAC',IBASE(IKDRI))
          CALL SETARA(RBASE,NREAC*NDEPL,IRER)
          CALL LCMGET(IPEDIT,'DEPLETE-ENER',RBASE(IRER))
          CALL SETARA(RBASE,NDEPL,IRRD)
          CALL LCMGET(IPEDIT,'DEPLETE-DECA',RBASE(IRRD))
          CALL SETARA(RBASE,NDFI*NDFP,IFIYI)
          IF(NDFI*NDFP.GT.0) THEN
             CALL LCMGET(IPEDIT,'FISSIONYIELD',RBASE(IFIYI))
          ENDIF
          CALL LCMSIX(IPEDIT,' ',2)
*
          CALL SETARA(RBASE,NGROUP,ISIG)
          CALL SETARA(RBASE,(NGCOND+1)*NMERGE,IFIRA)
          CALL SETARA(IBASE,NBISO,IINDX)
          CALL EDIHFD(IPLIB,NGROUP,NGCOND,NREGIO,NMERGE,NBISO,NDEPL,
     >                NDFI,NDFP,NREAC,MATCOD,VOLUME,IBASE(INADPL),
     >                IBASE(IISONA),IBASE(IISONR),IBASE(IMIX),
     >                FLUXES(1,1,1),RBASE(IDEN),IBASE(IDEPL),
     >                IGCOND,IMERGE,IBASE(IKDRI),RBASE(IRER),
     >                RBASE(IRRD),RBASE(IFIYI),RBASE(JDECAY),
     >                RBASE(JYIELD),IBASE(JPIFI),RBASE(JPYIEL),
     >                RBASE(ISIG),RBASE(IFIRA),IBASE(IINDX))
          CALL RLSARA(IBASE(IINDX))
          CALL RLSARA(RBASE(IFIRA))
          CALL RLSARA(RBASE(ISIG))
*
          CALL RLSARA(RBASE(IFIYI))
          CALL RLSARA(RBASE(IRRD))
          CALL RLSARA(RBASE(IRER))
          CALL RLSARA(IBASE(IKDRI))
          CALL RLSARA(IBASE(INADPL))
        ENDIF
        CALL LCMSIX(IPEDIT,' ',2)
*----
*  COMPUTE MICROSCOPIC CROSS SECTIONS
*----
        CALL SETARA(RBASE,NGCOND**2,IGAF)
        CALL SETARA(RBASE,NGROUP*(10+NL+NED+2*NDEL),IGAR)
        CALL SETARA(RBASE,2*NGCOND*(10+NL+NED+2*NDEL),IGAS)
        CALL SETARA(RBASE,NGROUP*3*NL,IIGAR)
        CALL SETARA(RBASE,NGCOND,INJJ)
        CALL SETARA(RBASE,NGCOND,IIJJ)
        CALL SETARA(RBASE,NGROUP*NGROUP*NL,IWGAR)
        CALL SETARA(RBASE,2*NGCOND*NGCOND*NL,IWSCAT)
        CALL SETARA(RBASE,NBMIX+1,IXSECT)
        CALL SETARA(RBASE,NGCOND,IWSTRD)
        CALL SETARA(IBASE,3*NBISO*NMERGE,IHNISO)
        CALL SETARA(RBASE,NBISO*NMERGE,ISDEN)
        CALL SETARA(IBASE,NBISO*NMERGE,ISMIX)
        CALL SETARA(IBASE,NBISO*NMERGE,ISTYP)
        CALL SETARA(IBASE,NBISO*NMERGE,ISTOD)
        CALL SETARA(RBASE,NBISO*NMERGE,IVREL)
        CALL SETARA(RBASE,NBISO*NMERGE,ITNISO)
        CALL SETARA(RBASE,NGCOND,JTMPXS)
        CALL SETARA(RBASE,NBISO,IMASK)
        CALL SETARA(IBASE,NL,IITYPR)
        CALL SETARA(RBASE,NDEL,IWDLA)
        CALL SETARA(IBASE,NDFI,IJPIFI)
        CALL SETARA(RBASE,NGROUP,IWORK)
        CALL SETARA(IBASE,NMERGE,IMILVO)
        CALL EDIMIC(IPEDIT,IPFLUX,IPLIB,NL,NDEL,NBISO,NDEPL,
     >             IBASE(IISONA),IBASE(IISONR),IBASE(IMIX),RBASE(ITN),
     >             NED,HVECT,IPRINT,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,
     >             NDFI,ILEAKS,ILUPS,NWGTH,MATCOD,VOLUME,KEYFLX,CURNAM,
     >             IGCOND,IMERGE,FLUXES,RBASE(ISPH),RBASE(IDEN),
     >             IBASE(IITYPE),IBASE(IDEPL),IBASE(ILSISO),
     >             RBASE(JEMEVF),RBASE(JEMEVG),RBASE(JDECAY),
     >             RBASE(JYIELD),IBASE(JPIFI),RBASE(JPYIEL),ITRANC,LISO,
     >             RBASE(IGAF),RBASE(IGAR),RBASE(IGAS),RBASE(IIGAR),
     >             RBASE(INJJ),RBASE(IIJJ),RBASE(IWGAR),RBASE(IWSCAT),
     >             RBASE(IXSECT),RBASE(IWSTRD),IBASE(IHNISO),
     >             RBASE(ISDEN),IBASE(ISMIX),IBASE(ISTYP),IBASE(ISTOD),
     >             RBASE(IVREL),RBASE(ITNISO),RBASE(JTMPXS),
     >             RBASE(IMASK),IBASE(IITYPR),RBASE(IWDLA),
     >             IBASE(IJPIFI),RBASE(IWORK),IBASE(IMILVO))
        CALL RLSARA(IBASE(IMILVO))
        CALL RLSARA(RBASE(IWORK))
        CALL RLSARA(IBASE(IJPIFI))
        CALL RLSARA(RBASE(IWDLA))
        CALL RLSARA(IBASE(IITYPR))
        CALL RLSARA(RBASE(IMASK))
        CALL RLSARA(RBASE(JTMPXS))
        CALL RLSARA(RBASE(ITNISO))
        CALL RLSARA(RBASE(IVREL))
        CALL RLSARA(IBASE(ISTOD))
        CALL RLSARA(IBASE(ISTYP))
        CALL RLSARA(IBASE(ISMIX))
        CALL RLSARA(RBASE(ISDEN))
        CALL RLSARA(IBASE(IHNISO))
        CALL RLSARA(RBASE(IWSTRD))
        CALL RLSARA(RBASE(IXSECT))
        CALL RLSARA(RBASE(IWSCAT))
        CALL RLSARA(RBASE(IWGAR))
        CALL RLSARA(RBASE(IIJJ))
        CALL RLSARA(RBASE(INJJ))
        CALL RLSARA(RBASE(IIGAR))
        CALL RLSARA(RBASE(IGAS))
        CALL RLSARA(RBASE(IGAR))
        CALL RLSARA(RBASE(IGAF))
*----
*  ISOTX FILE PROCESSING
*----
        IF(ISOTXS.GE.1) THEN
          CALL LCMSIX(IPEDIT,CURNAM,1)
          CALL LCMGET(IPEDIT,'STATE-VECTOR',IPAR)
          NBNISO=IPAR(2)
          NAMSBR='EDIDRV'
          IF(IPRINT.GE.1) WRITE(IOUT,6000) NAMSBR
          CALL SETARA(RBASE,NGCOND+1,IENERG)
          CALL SETARA(IBASE,3*NBNISO,INNAM)
          CALL SETARA(IBASE,3*NBNISO,INNRF)
          CALL SETARA(IBASE,NBNISO,INMIX)
          CALL SETARA(RBASE,NBNISO,INAWR)
          CALL SETARA(RBASE,NBNISO,INDEN)
          CALL SETARA(RBASE,NBNISO,INTMP)
          CALL SETARA(RBASE,NBNISO,INVOL)
          CALL SETARA(RBASE,NBNISO,ISNEJ)
          CALL LCMGET(IPEDIT,'ENERGY',RBASE(IENERG))
          CALL LCMGET(IPEDIT,'ISOTOPESUSED',IBASE(INNAM))
          CALL LCMGET(IPEDIT,'ISOTOPERNAME',IBASE(INNRF))
          CALL LCMGET(IPEDIT,'ISOTOPESMIX',IBASE(INMIX))
          CALL LCMGET(IPEDIT,'ISOTOPESDENS',RBASE(INDEN))
          CALL LCMGET(IPEDIT,'ISOTOPESTEMP',RBASE(INTMP))
          CALL LCMGET(IPEDIT,'ISOTOPESVOL',RBASE(INVOL))
          DO 170 ISO=1,NBNISO
            WRITE(HNEW,'(3A4)') (IBASE(INNAM+(ISO-1)*3+I0),I0=0,2)
            CALL LCMSIX(IPEDIT,HNEW,1)
            CALL LCMGET(IPEDIT,'AWR',AWR)
            EMEVF=0.0
            EMEVG=0.0
            CALL LCMLEN(IPEDIT,'MEVF',ILENF,ITYLCM)
            CALL LCMLEN(IPEDIT,'MEVG',ILENG,ITYLCM)
            IF(ILENF.EQ.1) CALL LCMGET(IPEDIT,'MEVF',EMEVF)
            IF(ILENG.EQ.1) CALL LCMGET(IPEDIT,'MEVG',EMEVG)
            CALL LCMSIX(IPEDIT,' ',2)
            RBASE(INAWR+ISO-1)=AWR
            RBASE(ISNEJ+ISO-1)=EMEVF+EMEVG
  170     CONTINUE
*
          CALL SETARA(RBASE,NGCOND,IANVEL)
          CALL SETARA(IBASE,NBNISO,LOCISO)
          CALL SETARA(IBASE,NGCOND,IIJJ)
          CALL SETARA(IBASE,NGCOND,INJJ)
          NBIXS=IXEDI
          DO 190 IMRG=1,NMERGE
            NBIXS=NBIXS+1
            WRITE(NISEXT,'(I6)') NBIXS
            DO 180 ICAR=1,6
              IF(NISEXT(ICAR:ICAR) .EQ. ' ' .OR.
     >           NISEXT(ICAR:ICAR) .EQ. '*') THEN
                 NISEXT(ICAR:ICAR)='0'
              ENDIF
 180        CONTINUE
            NISOTX='ISOTXS'//NISEXT
*----
*  GENERATE ONE ISOTXS FILE FOR EACH MERGED REGION IN EACH MIXTURE
*----
           WRITE(CTITLE,9000) NAMSBR,CURNAM,
     >                        'MICR        ','MIX',IMRG,NISOTX
           IF(IPRINT.GE.1) WRITE(IOUT,6002) IMRG,NISOTX
           IUTYPE=ISOTXS+1
           IWGOXS=KDROPN(NISOTX,0,IUTYPE,0,0)
           CALL EDITXS(IPEDIT,IWGOXS,IUTYPE,IPRINT,NGCOND,NL    ,
     >                 NBNISO,CTITLE,IMRG  ,RBASE(IENERG),IBASE(INNAM) ,
     >                 IBASE(INNRF) ,IBASE(INMIX) ,RBASE(INAWR) ,
     >                 RBASE(INDEN) ,RBASE(INTMP) ,RBASE(INVOL) ,
     >                 RBASE(ISNEJ) ,RBASE(IANVEL),IBASE(LOCISO),
     >                 IBASE(IIJJ)  ,IBASE(INJJ)  )
            IRETRN=KDRCLS(IWGOXS,1)
 190      CONTINUE
          CALL RLSARA(IBASE(INJJ))
          CALL RLSARA(IBASE(IIJJ))
          CALL RLSARA(IBASE(LOCISO))
          CALL RLSARA(RBASE(IANVEL))
*
          CALL RLSARA(RBASE(ISNEJ))
          CALL RLSARA(RBASE(INVOL))
          CALL RLSARA(RBASE(INTMP))
          CALL RLSARA(RBASE(INDEN))
          CALL RLSARA(RBASE(INAWR))
          CALL RLSARA(IBASE(INMIX))
          CALL RLSARA(IBASE(INNRF))
          CALL RLSARA(IBASE(INNAM))
          CALL RLSARA(RBASE(IENERG))
          CALL LCMSIX(IPEDIT,' ',2)
        ENDIF
      ENDIF
*----
*  COMPUTE MACROSCOPIC RESIDUAL CROSS SECTIONS
*----
      IF((NBMICR.NE.0).AND.(NBMICR.NE.-1).AND.(NBISO.NE.0)) THEN
        IPRIN2=IPRINT-1
        CALL SETARA(IBASE,NBISO,ILSIS2)
        CALL SETARA(RBASE,NDEL,IWDLA)
        CALL SETARA(IBASE,3*NBISO*NMERGE,IHNISO)
        CALL SETARA(IBASE,3*NBISO*NMERGE,IHNIRF)
        CALL SETARA(RBASE,NBISO*NMERGE,ISDEN)
        CALL SETARA(IBASE,NBISO*NMERGE,ISMIX)
        CALL SETARA(IBASE,NBISO*NMERGE,ISTYP)
        CALL SETARA(IBASE,NBISO*NMERGE,ISTOD)
        CALL SETARA(RBASE,NBISO*NMERGE,IVREL)
        CALL SETARA(RBASE,NBISO*NMERGE,ITNISO)
        CALL SETARA(RBASE,NGCOND*NGCOND*NL,IWSCAT)
        CALL SETARA(RBASE,NGCOND*(10+NL+NED+2*NDEL),IGAS)
        CALL SETARA(IBASE,NDFI,IJPIFI)
        CALL SETARA(RBASE,NGCOND+1,IWORK)
        CALL SETARA(RBASE,NDFI,IWPY)
        CALL SETARA(RBASE,2*NGCOND*(NDEL+1),IPNFIR)
        CALL SETARA(IBASE,NL,IITYPR)
        CALL SETARA(RBASE,NGCOND*NGCOND*NL,IWORK2)
        CALL SETARA(RBASE,NMERGE,IDENTO)
        CALL SETARA(RBASE,NMERGE,IDAWR)
        CALL SETARA(RBASE,NMERGE,ITNTOT)
        CALL EDIRES(IPEDIT,IPFLUX,IPLIB,NL,NDEL,NBISO,NDEPL,
     >              IBASE(IISONA),IBASE(IISONR),IBASE(IMIX),RBASE(ITN),
     >              NED,HVECT,IPRIN2,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,
     >              NDFI,ILEAKS,ILUPS,NWGTH,MATCOD,VOLUME,KEYFLX,CURNAM,
     >              IGCOND,IMERGE,FLUXES,RBASE(ISPH),RBASE(IDEN),
     >              IBASE(IITYPE),IBASE(IDEPL),IBASE(ILSISO),
     >              RBASE(JEMEVF),RBASE(JEMEVG),RBASE(JDECAY),
     >              RBASE(JYIELD),IBASE(JPIFI),RBASE(JPYIEL),ITRANC,
     >              LISO,IBASE(ILSIS2),RBASE(IWDLA),IBASE(IHNISO),
     >              IBASE(IHNIRF),RBASE(ISDEN),IBASE(ISMIX),
     >              IBASE(ISTYP),IBASE(ISTOD),RBASE(IVREL),
     >              RBASE(ITNISO),RBASE(IWSCAT),RBASE(IGAS),
     >              IBASE(IJPIFI),RBASE(IWORK),RBASE(IWPY),
     >              RBASE(IPNFIR),IBASE(IITYPR),RBASE(IWORK2),
     >              RBASE(IDENTO),RBASE(IDAWR),RBASE(ITNTOT))
        CALL RLSARA(RBASE(ITNTOT))
        CALL RLSARA(RBASE(IDAWR))
        CALL RLSARA(RBASE(IDENTO))
        CALL RLSARA(RBASE(IWORK2))
        CALL RLSARA(IBASE(IITYPR))
        CALL RLSARA(RBASE(IPNFIR))
        CALL RLSARA(RBASE(IWPY))
        CALL RLSARA(RBASE(IWORK))
        CALL RLSARA(IBASE(IJPIFI))
        CALL RLSARA(RBASE(IGAS))
        CALL RLSARA(RBASE(IWSCAT))
        CALL RLSARA(RBASE(ITNISO))
        CALL RLSARA(RBASE(IVREL))
        CALL RLSARA(IBASE(ISTOD))
        CALL RLSARA(IBASE(ISTYP))
        CALL RLSARA(IBASE(ISMIX))
        CALL RLSARA(RBASE(ISDEN))
        CALL RLSARA(IBASE(IHNIRF))
        CALL RLSARA(IBASE(IHNISO))
        CALL RLSARA(RBASE(IWDLA))
        CALL RLSARA(IBASE(ILSIS2))
      ENDIF
*----
*  EDIT MICROSCOPIC ACTIVATION XS
*----
      IF(NACTI.GT.0) THEN
        CALL SETARA(RBASE,NGROUP*(4+NL+NED),IRXSV)
        CALL SETARA(IBASE,NGROUP*3*NL,IKFJJ)
        CALL SETARA(RBASE,NGROUP*NGROUP*NL,IRSCAT)
        CALL SETARA(RBASE,2*NGCOND*NMERGE,IDFLX)
        CALL SETARA(RBASE,2*NGCOND*(4+NL+NED),IDXSV)
        CALL SETARA(RBASE,2*NGCOND*NGCOND*NL,IDSCAT)
        CALL SETARA(RBASE,NGCOND,ICXSV)
        CALL SETARA(IBASE,NGCOND*2,IKCJJ)
        CALL SETARA(RBASE,NGCOND*NGCOND,ICSCAT)
        CALL SETARA(IBASE,3*NBISO*NMERGE*NACTI,IHNISO)
        CALL SETARA(IBASE,NACTI*NBISO*NMERGE,ISOMIX)
        CALL EDIACT(IPEDIT,IPLIB,IPRINT,NGROUP,NGCOND,NREGIO,NMERGE,
     >              NL,NBISO,NED,VOLUME,IBASE(IMIX),IGCOND,IMERGE,
     >              FLUXES(1,1,1),RBASE(ISPH),ITRANC,IBASE(IISONA),
     >              HVECT,CURNAM,NACTI,IACTI,RBASE(JEMEVF),
     >              RBASE(JEMEVG),RBASE(IRXSV),IBASE(IKFJJ),
     >              RBASE(IRSCAT),RBASE(IDFLX),RBASE(IDXSV),
     >              RBASE(IDSCAT),RBASE(ICXSV),IBASE(IKCJJ),
     >              RBASE(ICSCAT),IBASE(IHNISO),IBASE(ISOMIX))
        CALL RLSARA(IBASE(ISOMIX))
        CALL RLSARA(IBASE(IHNISO))
        CALL RLSARA(RBASE(ICSCAT))
        CALL RLSARA(IBASE(IKCJJ))
        CALL RLSARA(RBASE(ICXSV))
        CALL RLSARA(RBASE(IDSCAT))
        CALL RLSARA(RBASE(IDXSV))
        CALL RLSARA(RBASE(IDFLX))
        CALL RLSARA(RBASE(IRSCAT))
        CALL RLSARA(IBASE(IKFJJ))
        CALL RLSARA(RBASE(IRXSV))
      ENDIF
*----
*  STATISTICS AND DELTA SIGMAS
*----
      IF(NSTATS.NE.0) THEN
        CALL SETARA(RBASE,NMERGE,INEWFL)
        CALL SETARA(RBASE,NMERGE,IOLDFL)
        CALL SETARA(RBASE,NMERGE+NGROUP,IOLDVA)
        CALL EDIDST(IPEDIT,IPRINT,NL,NGCOND,NMERGE,NSTATS,ILEAKS,
     >              RBASE(IVOLME),RBASE(IWLETY),RBASE(ITAUXT),
     >              RBASE(ISCATS),RBASE(INEWFL),RBASE(IOLDFL),
     >              RBASE(IOLDVA),OLDNAM,NTAUXT)
        CALL RLSARA(RBASE(IOLDVA))
        CALL RLSARA(RBASE(IOLDFL))
        CALL RLSARA(RBASE(INEWFL))
      ENDIF
*----
*  FOUR FACTORS
*----
      IF(IFFAC.NE.0) THEN
        CALL SETARA(IBASE,NMERGE,ITYPER)
        CALL SETARA(RBASE,NMERGE*NGCOND,IFLXT)
        CALL SETARA(RBASE,8*(NGCOND+1),IDRATE)
        CALL SETARA(RBASE,4*(NGCOND+1),IDLEAK)
        CALL EDIBAL(IPEDIT,IPFLUX,IPRINT,NL,IFFAC,NGCOND,NMERGE,EIGENK,
     >              RBASE(ITAUXT),RBASE(ISCATS),ILEAKS,IBASE(ITYPER),
     >              RBASE(IFLXT),RBASE(IDRATE),RBASE(IDLEAK),B2,NTAUXT)
        CALL RLSARA(RBASE(IDLEAK))
        CALL RLSARA(RBASE(IDRATE))
        CALL RLSARA(RBASE(IFLXT))
        CALL RLSARA(IBASE(ITYPER))
      ENDIF
*
      CALL RLSARA(RBASE(ISCATS))
      IF(NDFI.GT.0) THEN
         CALL RLSARA(RBASE(JPYIEL))
         CALL RLSARA(IBASE(JPIFI))
      ENDIF
      CALL RLSARA(RBASE(JYIELD))
      CALL RLSARA(RBASE(JDECAY))
      CALL RLSARA(RBASE(JEMEVG))
      CALL RLSARA(RBASE(JEMEVF))
      CALL RLSARA(RBASE(ISIGS))
      CALL RLSARA(RBASE(ITAUXT))
      CALL RLSARA(RBASE(ISPH))
      CALL RLSARA(RBASE(IWE))
      CALL RLSARA(RBASE(IWLETY))
      CALL RLSARA(RBASE(IVOLME))
      IF(HSIGN.EQ.'L_LIBRARY') THEN
         CALL RLSARA(IBASE(IISONR))
         CALL RLSARA(IBASE(IISONA))
         CALL RLSARA(IBASE(IDEPL))
         CALL RLSARA(RBASE(ITN))
         CALL RLSARA(IBASE(IMIX))
         CALL RLSARA(RBASE(IITYPE))
         CALL RLSARA(RBASE(IDEN))
         CALL RLSARA(IBASE(ILSISO))
      ENDIF
      RETURN
*----
*  FORMAT
*----
 6000 FORMAT(1X,A6,': GENERATING ISOTXS FILE ')
 6002 FORMAT(8X,' FOR EDITING MIXTURE    = ',I6,
     >          ' INFORMATION STORED ON FILE = ',A12)
 9000 FORMAT(1X,A6,3X,A12,3X,A12,3X,A4,I6,5X,A12)
      END
