*DECK EDIGEO
      SUBROUTINE EDIGEO(MAXPTS,MAXMER,IPGEO1,IPGEO2,IPRINT,NREGIO,IEUR,
     1 NMERGE,IMERGE,XX,YY,INUM,IGEN,IORI,IMAT,NMC)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover the geometry information from LCM (stored by operator GEO) and
* compute a corresponding macro-geometry (i.e. a pure geometry that can
* be treated by Bivac or Trivac or a two-level Eurydice geometry)
* together with NMERGE and array IMERGE.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXPTS  allocated storage for arrays of dimension NMCEL.
* MAXMER  allocated storage for arrays of dimension NMERGE.
* IPGEO1  pointer to the original geometry (L_GEOM signature).
* IPGEO2  pointer to the macro-geometry (L_GEOM signature).
* IPRINT  print flag (equal to 0 for no print).
* NREGIO  number of regions.
* IEUR    type of tracking operator for the macro-geometry:
*         =1: 2-level macro-geometry with MERGE;
*         =2: 2-level NXT-type macro-geometry without MERGE;
*         =3: 1-level TRIVAC-type macro-geometry without MERGE.
* NMERGE  macro-calculation merging flag (=1 to indicate a homogeneous
*         macro-calculation).
*
*Parameters: output
* NMERGE  number of merged regions (equal to the number of physical
*         cells in Eurydice).
* IMERGE  index of merged regions.
*
*Parameters: scratch
* XX
* YY
* INUM
* IGEN
* IORI
* IMAT
* NMC
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      MAXPTS,MAXMER,IPGEO1,IPGEO2,IPRINT,NREGIO,IEUR,
     >             NMERGE,IMERGE(NREGIO),INUM(2*MAXPTS),IGEN(MAXPTS),
     >             IORI(2*MAXPTS),IMAT(2*MAXPTS),NMC(MAXPTS+1)
      REAL         XX(MAXPTS),YY(MAXPTS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER   (NSTATE=40)
      CHARACTER    HSMG*131,TYPE(0:9)*16,HSIGN*12,TEXT12*12
      LOGICAL      ILK,LL1,LL2,LHOM
      INTEGER      NCODE(6),ICODE(6),ISTATE(NSTATE),JSTATE(NSTATE),
     1             KCHAR(3),IBASE(1)
      REAL         ZCODE(6),ALBEDO(6),XXX(2),YYY(2)
      COMMON       BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
      DATA TYPE/'VIRTUAL','HOMOGENEOUS','CARTESIAN 1-D','TUBE 1-D',
     1 'SPHERE 1-D','CARTESIAN 2-D','TUBE 2-D','CARTESIAN 3-D',
     2 'HEXAGONAL 2-D','HEXAGONAL 3-D'/
*----
*  RECOVER ASSEMBLY INFORMATION FROM THE ORIGINAL GEOMETRY
*----
      LHOM=(NMERGE.EQ.1)
      CALL LCMGET(IPGEO1,'STATE-VECTOR',ISTATE)
      IF( ((ISTATE(1).EQ.5).AND.(ISTATE(8).EQ.1)).OR.
     1     ((ISTATE(1).EQ.8).AND.(ISTATE(8).EQ.1)).OR.
     2     ((ISTATE(1).EQ.20).AND.(ISTATE(13).EQ.0)).OR.
     3     ((ISTATE(1).EQ.24).AND.(ISTATE(13).EQ.0)) ) THEN
         ICLS=0
         LR=0
         CALL SETARA(BASE,NREGIO,IISS)
         CALL SETARA(BASE,MAXPTS,IRAYRE)
         CALL SETARA(BASE,MAXPTS,ILSECT)
         CALL SETARA(BASE,MAXPTS,INMCR)
         CALL SETARA(BASE,MAXPTS,IMIL)
         CALL SETARA(BASE,MAXPTS,INBREG)
         CALL SETARA(BASE,MAXPTS,IMILIE)
         CALL SETARA(BASE,MAXPTS,IRAYON)
         CALL SETARA(BASE,3*MAXPTS,ICELL)
         CALL SETARA(BASE,MAXPTS+1,IXXX)
         CALL SETARA(BASE,MAXPTS+1,IYYY)
         CALL READEU(MAXPTS,MAXPTS,IPGEO1,IR,BASE(IISS),ILK,NMCEL,
     1   NMERGE,NGEN,INUM,IGEN,IPAS,LX,LY,XX,YY,BASE(ILSECT),
     2   BASE(IRAYRE),NMC,BASE(INMCR),IORI,NCODE,ZCODE,IHEX,IPRINT-5,
     3   BASE(IMIL),BASE(INBREG),BASE(IMILIE),BASE(IRAYON),BASE(ICELL),
     4   BASE(IXXX),BASE(IYYY))
         CALL RLSARA(BASE(IYYY))
         CALL RLSARA(BASE(IXXX))
         CALL RLSARA(BASE(ICELL))
         CALL RLSARA(BASE(IRAYON))
         CALL RLSARA(BASE(IMILIE))
         CALL RLSARA(BASE(INBREG))
         CALL RLSARA(BASE(IMIL))
         CALL RLSARA(BASE(INMCR))
         CALL RLSARA(BASE(ILSECT))
         CALL RLSARA(BASE(IRAYRE))
         CALL RLSARA(BASE(IISS))
      ELSE IF( ((ISTATE(1).EQ.3).OR.(ISTATE(1).EQ.20).OR.
     1          (ISTATE(1).EQ.24)).AND.(ISTATE(13).GE.1)) THEN
        IROT=3
        LX=0
        LY=0
        IF(ISTATE(1).EQ.3) THEN
          NSURW=1
          ICLS=1
        ELSE IF(ISTATE(1).EQ.20) THEN
          NSURW=4
          ICLS=2
          LX=1
          LY=1
        ELSE IF(ISTATE(1).EQ.24) THEN
          NSURW=6
          ICLS=3
        ENDIF
        MAXJ=1
        CALL XCGDIM(IPGEO1,MAXPTS,NSURW,IROT,IAPP,MAXJ,NVOLW,
     1              NANW,MNANW,NRTW,MSRODW,MARODW,NSURFW)
*----
*  ALLOCATE BLOCK FOR READING GEOMETRY INFORMATION
*----
        CALL SETARA(IBASE,2*NANW,NRINFO)
        CALL SETARA(IBASE,NANW,MATANN)
        CALL SETARA(BASE,NANW,JRAN)
        CALL SETARA(IBASE,NSURW+NVOLW+1,KEYMRG)
        CALL SETARA(IBASE,NSURW+NVOLW+1,MATALB)
        CALL SETARA(BASE,NSURW+NVOLW+1,JVS)
        CALL SETARA(IBASE,3*NRTW,NRODS)
        CALL SETARA(BASE,2*NRTW,JRODS)
        CALL SETARA(IBASE,NRTW,NRODR)
        CALL SETARA(BASE,MSRODW*NRTW,JRODR)
        CALL SETARA(IBASE,MSRODW*NRTW,MATROD)
        CALL SETARA(IBASE,MNANW,ISPLIT)
        CALL SETARA(BASE,MNANW,JRAD)
        CALL SETARA(IBASE,NRT,NXRS)
        CALL SETARA(IBASE,NRT*NANW,NXRI)
        CALL SETARA(BASE,2*NRT,JVRG)
        CALL XCGGEO(IPGEO1,IROT,NSURW,NVOLW,NANW,MNANW,NRTW,MSRODW,
     1  IPRINT,ILK,NMATW,BASE(JRAN),IBASE(MATANN),IBASE(NRODS),
     2  BASE(JRODS),IBASE(MATROD),IBASE(NRODR),BASE(JRODR),
     3  IBASE(NRINFO),IBASE(MATALB),BASE(JVS),IBASE(ISPLIT),BASE(JRAD),
     4  COTE,RADMIN,NCODE,ICODE,ZCODE,ALBEDO,IBASE(KEYMRG),IBASE(NXRS),
     5  IBASE(NXRI),BASE(JVRG))
        CALL RLSARA(BASE(JVRG))
        CALL RLSARA(IBASE(NXRI))
        CALL RLSARA(IBASE(NXRS))
        CALL RLSARA(BASE(JRAD))
        CALL RLSARA(IBASE(ISPLIT))
        CALL RLSARA(IBASE(MATROD))
        CALL RLSARA(BASE(JRODR))
        CALL RLSARA(IBASE(NRODR))
        CALL RLSARA(BASE(JRODS))
        CALL RLSARA(IBASE(NRODS))
        CALL RLSARA(BASE(JVS))
        CALL RLSARA(IBASE(MATALB))
        CALL RLSARA(IBASE(KEYMRG))
*----
*  STORE INFORMATION FOR NEW GEOMETRY
*----
        IPAS=NVOLW
        NMCEL=NANW
        NMERGE=NANW
        IHEX=0
        XX(1)=0.0
        NMC(1)=0
        DO 10 II=1,NANW
          XX(II+1)=BASE(JRAN-1+II)
          INUM(II)=II
          IGEN(II)=II
          NMC(II+1)=IBASE(NRINFO+2*(II-1))
  10    CONTINUE
        IF(NSURW.EQ.1) THEN
          LR=NANW
        ELSE IF(NSURW.EQ.4) THEN
          LR=NANW-1
          XXX(1)=0.0
          XXX(2)=XX(NANW+1)
          YYY(1)=0.0
          YYY(2)=COTE
        ELSE
          LR=NANW-1
          SIDE=XX(NANW+1)
        ENDIF
*
        CALL RLSARA(BASE(JRAN))
        CALL RLSARA(IBASE(MATANN))
        CALL RLSARA(IBASE(NRINFO))
      ELSE
         CALL XABORT('EDIGEO: INVALID TYPE OF INPUT GEOMETRY.')
      ENDIF
      IF(IPAS.NE.NREGIO) THEN
         WRITE(HSMG,'(45HEDIGEO: INCONSISTENT NUMBER OF REGIONS. IPAS=,
     1   I6,8H NREGIO=,I6)') IPAS,NREGIO
         CALL XABORT(HSMG)
      ENDIF
*----
*  COMPUTE IMERGE ARRAY
*----
      IF(LHOM) THEN
*        HOMOGENEOUS MACRO-CALCULATION.
         DO 20 I=1,NREGIO
   20    IMERGE(I)=1
         NMERGE=1
         DO 30 I=1,NMCEL
   30    IMAT(I)=1
      ELSE
*        CELL-WISE HETEROGENEOUS MACRO-CALCULATION.
         IPAS=0
         DO 50 IKK=1,NMERGE
         IKG=IGEN(IKK)
         I2=NMC(IKG+1)-NMC(IKG)
         DO 40 I=1,I2
   40    IMERGE(IPAS+I)=IKK
   50    IPAS=IPAS+I2
         IF(IPAS.NE.NREGIO) THEN
            CALL XABORT('EDIGEO: INCONSISTENT ARRAY NMC.')
         ENDIF
         DO 60 I=1,NMCEL
   60    IMAT(I)=INUM(I)
      ENDIF
      IF(NMERGE.GT.MAXMER) THEN
         WRITE(HSMG,'(37HEDIGEO: INSUFFICIENT STORAGE (MAXMER=,I5,
     1   47H). THE MERGE OPTION SHOULD NOT BE USED IN EDI:.)') MAXMER
         CALL XABORT(HSMG)
      ENDIF
      IF(IPRINT.GT.0) THEN
         WRITE (6,'(/23H EDIGEO: MERGING INDEX:/(1X,14I5))') (IMERGE(I),
     1   I=1,NREGIO)
      ENDIF
*----
*  COMPUTE THE MACRO-GEOMETRY
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL XDISET(ICODE,6,0)
      CALL LCMPUT(IPGEO2,'NCODE',6,1,NCODE)
      CALL LCMPUT(IPGEO2,'ZCODE',6,2,ZCODE)
      CALL LCMPUT(IPGEO2,'ICODE',6,1,ICODE)
      IF((ICLS.EQ.0).AND.(IEUR.LE.2)) THEN
*       TWO-LEVEL MACRO-GEOMETRY.
        CALL SETARA(BASE,3*NMERGE,IHCEL)
        CALL XDISET(JSTATE,NSTATE,0)
        JSTATE(6)=1
        HSIGN='L_GEOM'
        READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        IF(IHEX.EQ.0) THEN
          ISTATE(1)=5
          JSTATE(1)=20
          JSTATE(3)=1
          JSTATE(4)=1
          DO 70 IKK=1,NMERGE
          WRITE(TEXT12,'(4HCELL,I8.8)') IKK
          READ(TEXT12,'(3A4)') (BASE(IHCEL+(IKK-1)*3+I0),I0=0,2)
          CALL LCMSIX(IPGEO2,TEXT12,1)
          CALL LCMPUT(IPGEO2,'MIX',1,1,IKK)
          XXX(1)=0.0
          XXX(2)=XX(IGEN(IKK))
          YYY(1)=0.0
          YYY(2)=YY(IGEN(IKK))
          CALL LCMPUT(IPGEO2,'MESHX',2,2,XXX)
          CALL LCMPUT(IPGEO2,'MESHY',2,2,YYY)
          JSTATE(7)=IKK
          CALL LCMPUT(IPGEO2,'SIGNATURE',3,3,KCHAR)
          CALL LCMPUT(IPGEO2,'STATE-VECTOR',NSTATE,1,JSTATE)
          CALL LCMSIX(IPGEO2,' ',2)
   70     CONTINUE
        ELSE
          ISTATE(1)=8
          JSTATE(1)=24
          JSTATE(3)=1
          DO 80 IKK=1,NMERGE
          WRITE(TEXT12,'(4HCELL,I8.8)') IKK
          READ(TEXT12,'(3A4)') (BASE(IHCEL+(IKK-1)*3+I0),I0=0,2)
          CALL LCMSIX(IPGEO2,TEXT12,1)
          CALL LCMPUT(IPGEO2,'MIX',1,1,IKK)
          CALL LCMPUT(IPGEO2,'SIDE',1,2,XX(1))
          JSTATE(7)=IKK
          CALL LCMPUT(IPGEO2,'SIGNATURE',3,3,KCHAR)
          CALL LCMPUT(IPGEO2,'STATE-VECTOR',NSTATE,1,JSTATE)
          CALL LCMSIX(IPGEO2,' ',2)
   80     CONTINUE
          CALL LCMPUT(IPGEO2,'IHEX',1,1,IHEX)
        ENDIF
        CALL LCMPUT(IPGEO2,'CELL',3*NMERGE,3,BASE(IHCEL))
        IF(IEUR.EQ.1) CALL LCMPUT(IPGEO2,'MERGE',NMCEL,1,INUM)
        CALL LCMPUT(IPGEO2,'TURN',NMCEL,1,IORI)
        DO 90 IBLK=1,NMCEL
   90   INUM(IBLK)=-INUM(IBLK)
        CALL LCMPUT(IPGEO2,'MIX',NMCEL,1,INUM)
        CALL RLSARA(BASE(IHCEL))
        ISTATE(8)=1
        ISTATE(9)=NMERGE
        ISTATE(10)=1
      ELSE IF(ICLS.EQ.0) THEN
*       ONE-LEVEL (TRIVAC-TYPE) MACRO-GEOMETRY.
        CALL LCMPUT(IPGEO2,'MIX',NMCEL,1,IMAT)
        IF(IHEX.EQ.0) THEN
          ISTATE(1)=5
          CALL SETARA(BASE,LX+1,IMESHX)
          CALL SETARA(BASE,LY+1,IMESHY)
          BASE(IMESHX)=0.0
          BASE(IMESHY)=0.0
          LL1=((NCODE(2).EQ.3).AND.(NCODE(3).EQ.3))
          LL2=((NCODE(1).EQ.3).AND.(NCODE(4).EQ.3))
          IBLK=0
          DO 100 K1=1,LY
            LXM=1
            LXP=LX
            IF(LL1) LXP=K1
            IF(LL2) LXM=K1
            DO 100 K2=LXM,LXP
              IBLK=IBLK+1
              IKK=INUM(IBLK)
              IF(IKK.EQ.0) GO TO 100
              IF(MOD(IORI(IBLK)-1,2).EQ.0) THEN
                BASE(IMESHX+K2)=BASE(IMESHX+K2-1)+XX(IGEN(IKK))
                BASE(IMESHY+K1)=BASE(IMESHY+K1-1)+YY(IGEN(IKK))
              ELSE
                BASE(IMESHX+K2)=BASE(IMESHX+K2-1)+YY(IGEN(IKK))
                BASE(IMESHY+K1)=BASE(IMESHY+K1-1)+XX(IGEN(IKK))
              ENDIF
  100     CONTINUE
          CALL LCMPUT(IPGEO2,'MESHX',LX+1,2,BASE(IMESHX))
          CALL LCMPUT(IPGEO2,'MESHY',LY+1,2,BASE(IMESHY))
          CALL RLSARA(BASE(IMESHY))
          CALL RLSARA(BASE(IMESHX))
        ELSE
          ISTATE(1)=8
          IF(IHEX.EQ.10) THEN
            IHEX=1
          ELSE IF(IHEX.EQ.11) THEN
            IHEX=2
          ENDIF
          CALL LCMPUT(IPGEO2,'IHEX',1,1,IHEX)
          CALL LCMPUT(IPGEO2,'SIDE',1,2,XX(1))
        ENDIF
      ELSE
        CALL LCMPUT(IPGEO2,'MIX',NMCEL,1,IMAT)
        CALL LCMPUT(IPGEO2,'RADIUS',LR+1,2,XX)
        IF(ICLS.EQ.1) THEN
          ISTATE(1)=3
        ELSE IF(ICLS.EQ.2) THEN
          ISTATE(1)=20
          CALL LCMPUT(IPGEO2,'MESHX',LX+1,2,XXX)
          CALL LCMPUT(IPGEO2,'MESHY',LY+1,2,YYY)
        ELSE
          ISTATE(1)=24
          IHEX=9
          CALL LCMPUT(IPGEO2,'IHEX',1,1,IHEX)
          CALL LCMPUT(IPGEO2,'SIDE',1,2,SIDE)
        ENDIF
      ENDIF
      ISTATE(2)=LR
      ISTATE(3)=LX
      ISTATE(4)=LY
      ISTATE(6)=NMCEL
      ISTATE(7)=NMERGE
      CALL LCMPUT(IPGEO2,'STATE-VECTOR',NSTATE,1,ISTATE)
      HSIGN='L_GEOM'
      READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(IPGEO2,'SIGNATURE',3,3,KCHAR)
      IF(IPRINT.GT.0) THEN
         HSIGN='MACRO$GEO'
         WRITE (6,510) HSIGN,1,TYPE(ISTATE(1))
         IF(IPRINT.GT.1) THEN
            WRITE (6,520) ISTATE(1),TYPE(ISTATE(1)),(ISTATE(I),I=2,7)
         ENDIF
      ENDIF
      RETURN
*
  510 FORMAT(/39H EDIGEO: CREATION OF A GEOMETRY NAMED ',A12,7H' ON LE,
     1 3HVEL,I3,11H WITH TYPE ,A16,1H.)
  520 FORMAT(/31H STATE VECTOR (MACRO GEOMETRY):/
     1 7H ITYPE ,I6, 4H   (,A16,1H)/
     2 7H LR    ,I6,20H   (NUMBER OF TUBES)/
     3 7H LX    ,I6,22H   (X-DIMENSION INDEX)/
     4 7H LY    ,I6,22H   (Y-DIMENSION INDEX)/
     5 7H LZ    ,I6,22H   (Z-DIMENSION INDEX)/
     6 7H LREG  ,I6,22H   (NUMBER OF REGIONS)/
     7 7H MAXMIX,I6,25H   (MAX. NB. OF MIXTURES)/1X,60(1H-)/)
      END
