*DECK EDIGET
      SUBROUTINE EDIGET(NGROUP,NGCOND,NREGIO,NBMIX,NMERGE,IHF,IFFAC,
     >                  ILUPS,NSAVES,NSTATS,IGCR,EGCR,IMERGE,CURNAM,
     >                  OLDNAM,LSPH,NWGTH,MATCOD,NBMICR,CARISO,NACTI,
     >                  IACTI,IPRINT,MAXPTS,ICALL,ISOTXS,LISO,MIXMER)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read edition option parameters.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): G. Marleau
*
*Parameters: input
* NGROUP  number of groups.
* NREGIO  number of regions.
* NBMIX   maximum number of mixtures.
*
*Parameters: output
* NGCOND  number of groups condensed.
* NMERGE  number of regions merged.
* IHF     H-factor calculation (= 0 no ; =1 yes).
* IFFAC   four factor calculation flag:
*         = 0 no four factors (default);
*         = 1 four factor evaluation.
* ILUPS   remove up-scattering from output.
* NSAVES  homogenized x-s computation+save:
*         = 0 no compute no save;
*         = 1 compute, no save;
*         = 2 compute, save.
* NSTATS  statistics level:
*         = 0 no statistics;
*         = 1 statistics on fluxes;
*         = 2 statistics on reaction rates;
*         = 3 statistics on fluxes and reaction rates;
*         =-1 delta sigma (MERG COMP only).
* IGCR    condensed groups limits.
* EGCR    condensed energy limits.
* IMERGE  merged regions position.
* CURNAM  name of LCM directory where the current rates are to be
*         stored.
* OLDNAM  name of LCM directory where old reaction rates were stored.
* LSPH    flag for SPH input data:
*         =.TRUE/.FALSE.: do/do not read additional SPH-related data.
* NWGTH   type of weighting for P1 cross section info:
*         =0 use flux to merge/condense P1 matrices;
*         =1 use current to merge/condense P1 matrices.
* MATCOD  mixture associated to each region.
* NBMICR  type of microlib edition:
*         =-2: process only macroscopic residue;
*         =-1: process each isotope;
*         =0: process no isotope;
*         >0 number of isotopes to process.
* CARISO  names of the isotopes to process.
* NACTI   number of activation edit.
* IACTI   activation mixtures.
* IPRINT  print index.
* MAXPTS  maximum number of macro-regions.
* ICALL   maximum directory index in IPEDIT.
* ISOTXS  ISOTX file enabling flag (0: off; 1: binary; 2: ascii).
* LISO    =.TRUE. if we want to keep all the isotopes after 
*         homogeneization.
*
*Parameters: scratch
* MIXMER  merge mixture indices.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      PARAMETER    (MAXED=100)
      INTEGER       NGROUP,NGCOND,NREGIO,NBMIX,NMERGE,IHF,IFFAC,ILUPS,
     >              NSAVES,NSTATS,IGCR(NGROUP),IMERGE(NREGIO),NWGTH,
     >              MATCOD(NREGIO),NBMICR,NACTI,IACTI(NBMIX),IPRINT,
     >              MAXPTS,ICALL,ISOTXS,MIXMER(0:NBMIX)
      REAL          EGCR(NGROUP)
      LOGICAL       LSPH,LISO
      CHARACTER     CURNAM*12,OLDNAM*12,CARISO(MAXED)*12
*----
*  LOCAL VARIABLES
*----
      CHARACTER     CARLIR*4
      REAL          REALIR
      DOUBLE PRECISION DBLLIR
*----
*  INITIALIZE MIXMER
*----
      DO 90 IMATER=0,NBMIX
        MIXMER(IMATER)=IMATER
  90  CONTINUE
*----
*  READ OPTION NAME
*----
      ISOTXS=0
 100  CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  IF(ITYPLU.EQ.10) GO TO 250
      IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTER VARI'
     > //'ABLE EXPECTED')
 102  IF(CARLIR.EQ.';') THEN
        LSPH=.FALSE.
        GO TO 250
      ELSE IF(CARLIR.EQ.'SPH') THEN
        LSPH=.TRUE.
        GO TO 250
      ELSE IF(CARLIR.EQ.'EDIT') THEN
        CALL REDGET(ITYPLU,IPRINT,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT('EDIGET: READ ERROR - INTEGER VARI'
     >  //'ABLE EXPECTED')
      ELSE IF(CARLIR.EQ.'UPS') THEN
        ILUPS=1
      ELSE IF(CARLIR.EQ.'P0W') THEN
*       FLUX WEIGHTING OF THE P1 MATRICES.
        NWGTH=0
      ELSE IF(CARLIR.EQ.'P1W') THEN
*       CURRENT WEIGHTING OF THE P1 MATRICES.
        NWGTH=1
      ELSE IF(CARLIR.EQ.'ALLX') THEN
*       TO REGISTER ALL ISOTOPES CROSS SECTION IN THE MERGED REGIONS
        LISO=.TRUE.
      ELSE IF(CARLIR.EQ.'MICR') THEN
        CALL REDGET(ITYPLU,NBMICR,REALIR,CARLIR,DBLLIR)
        IF((ITYPLU.EQ.3).AND.(CARLIR(1:4).EQ.'ISOT')) THEN
          ISOTXS=1
          CALL REDGET(ITYPLU,NBMICR,REALIR,CARLIR,DBLLIR)
          IF((ITYPLU.EQ.3).AND.(CARLIR(1:4).EQ.'ASCI')) THEN
            ISOTXS=2
            CALL REDGET(ITYPLU,NBMICR,REALIR,CARLIR,DBLLIR)
          ENDIF
        ENDIF
        IF((ITYPLU.EQ.3).AND.(CARLIR.EQ.'RES')) THEN
          NBMICR=-2
        ELSE IF((ITYPLU.EQ.3).AND.(CARLIR.EQ.'ALL')) THEN
          NBMICR=-1
        ELSE IF(ITYPLU.EQ.1) THEN
          IF(NBMICR.GT.MAXED) CALL XABORT('EDIGET: TOO MANY MICR')
          DO 200 IIII=1,NBMICR
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARISO(IIII),DBLLIR)
            IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTE'
     >      //'R VARIABLE EXPECTED')
 200      CONTINUE
        ELSE
          CALL XABORT('EDIGET: READ ERROR - KEY ISOTXS, ALL, NONE OR I'
     >    //'NTEGER VARIABLE EXPECTED AFTER MICR')
        ENDIF
      ELSE IF(CARLIR.EQ.'ACTI') THEN
        IF((ITYPLU.EQ.3).AND.(CARLIR(1:4).EQ.'ISOT')) THEN
          ISOTXS=1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF((ITYPLU.EQ.3).AND.(CARLIR(1:4).EQ.'ASCI')) THEN
            ISOTXS=2
            CALL REDGET(ITYPLU,NBMICR,REALIR,CARLIR,DBLLIR)
          ENDIF
        ENDIF
        IF((ITYPLU.EQ.3).AND.(CARLIR.EQ.'NONE')) THEN
          NACTI=0
        ELSE
          DO 211 IREGIO=1,NBMIX
            IF(ITYPLU.EQ.1) THEN
              IF(INTLIR.GT.NBMIX) CALL XABORT('EDIGET: INVALID ACTIVAT'
     >        //'ION INDEX')
              NACTI=NACTI+1
              IACTI(NACTI)=INTLIR
            ELSE
              GO TO 101
            ENDIF
            IF(IREGIO.LT.NBMIX) THEN
              CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            ENDIF
  211     CONTINUE
        ENDIF
      ELSE IF(CARLIR.EQ.'COND') THEN
*----
*  GROUP CONDENSATION DIRECTIVE ANALYSIS
*----
        DO 108 IGROUP=1,NGROUP+1
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.EQ.3) THEN
            IF(IGROUP.EQ.1) THEN
              IF(CARLIR.EQ.'NONE') THEN
                NGCOND=NGROUP
                DO 107 JGROUP=1,NGROUP
                  IGCR(JGROUP)=JGROUP
 107            CONTINUE
                CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
                GO TO 101
              ELSE
                NGCOND=1
                IGCR(NGCOND)=NGROUP
              ENDIF
            ENDIF
            GO TO 101
          ELSE IF(ITYPLU.EQ.1) THEN
            IF(INTLIR.GT.NGROUP) INTLIR=NGROUP
            IF(NGCOND.GT.0) THEN
              IF(INTLIR.GT.IGCR(NGCOND)) THEN
                NGCOND=NGCOND+1
                IGCR(NGCOND)=INTLIR
              ENDIF
            ELSE
              NGCOND=NGCOND+1
              IGCR(NGCOND)=INTLIR
            ENDIF
          ELSE
            IF(NGCOND.GT.0) THEN
              IF(REALIR.LT.EGCR(NGCOND)) THEN
                NGCOND=NGCOND+1
                EGCR(NGCOND)=REALIR
              ENDIF
            ELSE
              NGCOND=NGCOND+1
              EGCR(NGCOND)=REALIR
            ENDIF
          ENDIF
 108    CONTINUE
      ELSE IF(CARLIR.EQ.'MERG') THEN
*----
*  MERGING DIRECTIVE ANALYSIS
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTER VA'
     >  //'RIABLE EXPECTED')
        IF(CARLIR.EQ.'COMP') THEN
*----
*  COMPLETE MERGE
*----
          CALL XDISET(IMERGE,NREGIO,1)
          NMERGE=1
          GO TO 100
        ELSE IF(CARLIR.EQ.'GEO') THEN
*----
*  MERGE BY GEOMETRY
*----
          NMERGE=-1
          GO TO 100
        ELSE IF(CARLIR.EQ.'CELL') THEN
*----
*  CELL-BY-CELL MERGE
*----
          NMERGE=-2
          GO TO 100
        ELSE IF(CARLIR.EQ.'MIX') THEN
*----
*  MERGE BY MIXTURES
*----
          NMIXME=0
          DO 114 IREGIO=1,NREGIO
            NMIXME=MAX(NMIXME,MATCOD(IREGIO))
            IMERGE(IREGIO)=MIXMER(MATCOD(IREGIO))
 114      CONTINUE
          NMERGE=NMIXME
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.EQ.1) THEN
*----
*  SPECIFY MIXTURES TO BE MERGED
*----
            NMERGE=MAX(0,INTLIR)
            MIXMER(1)=INTLIR
            DO 115 IMATER=2,NMIXME
              CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
              IF(ITYPLU.NE.1) CALL XABORT('EDIGET: READ ERROR - INTEGE'
     >        //'R VARIABLE EXPECTED')
              NMERGE=MAX(NMERGE,INTLIR)
              MIXMER(IMATER)=INTLIR
 115        CONTINUE
            DO 116 IREGIO=1,NREGIO
              IMERGE(IREGIO)=MIXMER(MATCOD(IREGIO))
 116        CONTINUE
          ELSE IF(ITYPLU.EQ.3) THEN
*----
*  ASSOCIATE ONE REGION BY MIXTURE
*----
            GO TO 102
          ELSE
            CALL XABORT('EDIGET: READ ERROR - INVALID TYPE READ')
          ENDIF
        ELSE IF(CARLIR.EQ.'REGI') THEN
*----
*  MERGE BY REGIONS
*----
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.NE.1) CALL XABORT('EDIGET: READ ERROR - INTEGE'
     >    //'R VARIABLE EXPECTED')
          NMERGE=MAX(0,INTLIR)
          IMERGE(1)=INTLIR
          DO 118 IREGIO=2,NREGIO
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.NE.1) THEN
	       WRITE(CARLIR,'(I4)') NREGIO
	       CALL XABORT('EDIGET: READ ERROR - INTEGER VARIABLE EXPE'
     >         //'CTED NREGIO='//CARLIR)
            ENDIF
            NMERGE=MAX(NMERGE,INTLIR)
            IMERGE(IREGIO)=INTLIR
 118      CONTINUE
        ELSE IF(CARLIR.EQ.'NONE') THEN
*----
*  NO MERGING
*----
          NMERGE=NREGIO
          DO 106 IREGIO=1,NREGIO
            IMERGE(IREGIO)=IREGIO
 106      CONTINUE
        ELSE
          CALL XABORT('EDIGET: READ ERROR - ILLEGAL KEYWORD '//
     >    'FOLLOWING MERG -- ALLOWED : COMP, MIX REGI, READ : '//
     >     CARLIR)
        ENDIF
      ELSE IF(CARLIR.EQ.'TAKE') THEN
*----
*  TAKE DIRECTIVE ANALYSIS
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTER VA'
     >  //'RIABLE EXPECTED')
        IF(CARLIR.EQ.'MIX') THEN
*----
*  TAKE PER MIXTURE
*----
          NMIXME=0
          DO 120 IREGIO=1,NREGIO
            NMIXME=MAX(NMIXME,MATCOD(IREGIO))
 120      CONTINUE
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.EQ.1) THEN
            DO 121 IMX=1,NMIXME
              MIXMER(IMX)=0
 121        CONTINUE
*----
*  SPECIFY MIXTURES TO BE SELECTED
*----
            IF(INTLIR.LE.NMIXME.AND.INTLIR.GT.0) MIXMER(INTLIR)=1
            NMERGE=1
            DO 122 IMATER=2,NBMIX
              CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
              IF(ITYPLU.NE.1) GO TO 123
              IF(INTLIR.LE.NMIXME.AND.INTLIR.GT.0) MIXMER(INTLIR)=IMATER
              NMERGE=NMERGE+1
 122        CONTINUE
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          ENDIF
 123      CONTINUE
          WRITE(6,'(1X,A6,2X,2I10)') 'MIXMER',NMIXME,NMERGE
          WRITE(6,'(5I10)') (MIXMER(JJJ),JJJ=1,NMIXME)
          DO 124 IREGIO=1,NREGIO
            IMERGE(IREGIO)=MIXMER(MATCOD(IREGIO))
 124      CONTINUE
          GO TO 101
        ELSE IF(CARLIR.EQ.'REGI') THEN
*----
*  TAKE PER REGIONS
*----
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU.NE.1) CALL XABORT('EDIGET: AT LEAST ONE REGION'
     >      //' MUST BE SELECTED')
          DO 125 IREGIO=1,NREGIO
            IMERGE(IREGIO)=0
 125      CONTINUE
          NMERGE=1
          IMERGE(INTLIR)=1
          DO 126 IREGIO=2,NREGIO
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
            IF(ITYPLU.NE.1) GO TO 101
            NMERGE=NMERGE+1
            IMERGE(INTLIR)=IREGIO
 126      CONTINUE
        ELSE
          CALL XABORT('EDIGET: READ ERROR - ILLEGAL KEYWORD '//
     >    'FOLLOWING TAKE -- ALLOWED : MIX REGI, READ : '// CARLIR)
        ENDIF
      ELSE IF(CARLIR.EQ.'SAVE') THEN
*----
*  SAVE DIRECTIVE ANALYSIS
*----
        NSAVES=2
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTER VA'
     >  //'RIABLE EXPECTED')
        IF(CARLIR.EQ.'ON') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,CURNAM,DBLLIR)
          IF(ITYPLU.EQ.2) CALL XABORT('EDIGET: READ ERROR - REAL VARIA'
     >    //'BLE FORBIDDEN')
          IF(ITYPLU.EQ.1) THEN
            WRITE(CURNAM,'(8HREF-CASE,I4)') INTLIR
            ICALL=MAX(ICALL,INTLIR)
          ENDIF
        ELSE
          GO TO 102
        ENDIF
      ELSE IF(CARLIR.EQ.'STAT') THEN
*----
*  STAT DIRECTIVE ANALYSIS
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTER VA'
     >  //'RIABLE EXPECTED')
        IF(CARLIR.EQ.'FLUX') THEN
          NSTATS=1
        ELSE IF(CARLIR.EQ.'RATE') THEN
          NSTATS=2
        ELSE IF(CARLIR.EQ.'ALL ') THEN
          NSTATS=3
        ELSE IF(CARLIR.EQ.'DELS') THEN
          NSTATS=-1
        ELSE
          CALL XABORT('EDIGET: READ ERROR - ILLEGAL KEYWORD '//
     >                 CARLIR)
        ENDIF
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.3) CALL XABORT('EDIGET: READ ERROR - CHARACTER VA'
     >  //'RIABLE EXPECTED')
        IF(CARLIR.EQ.'REFE') THEN
          CALL REDGET(ITYPLU,INTLIR,REALIR,OLDNAM,DBLLIR)
          IF(ITYPLU.EQ.2) CALL XABORT('EDIGET: READ ERROR - REAL VARIA'
     >    //'BLE FORBIDDEN')
          IF(ITYPLU.EQ.1) WRITE(OLDNAM,'(8HREF-CASE,I4)') INTLIR
        ELSE
          GO TO 102
        ENDIF
      ELSE IF(CARLIR.EQ.'NOHF') THEN
        IHF=0
      ELSE IF(CARLIR.EQ.'NBAL') THEN
        IFFAC=1000
      ELSE IF(CARLIR.EQ.'MAXR') THEN
        CALL REDGET(ITYPLU,MAXPTS,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT('EDIGET: READ ERROR - INTEGER VARI'
     >  //'ABLE EXPECTED')
      ELSE
        CALL XABORT('EDIGET:ILLEGAL KEYWORD '//CARLIR)
      ENDIF
      GO TO 100
*----
*  RETURN
*----
  250 IF(IPRINT.GE.2) NSAVES=MAX(1,NSAVES)
      IF((NSAVES.EQ.0).AND.((NSTATS.NE.0).OR.(IFFAC.NE.0))) NSAVES=1
      IF((NSAVES.GE.2).AND.(CURNAM.EQ.' ')) THEN
        ICALL=ICALL+1
        WRITE(CURNAM,'(8HREF-CASE,I4)') ICALL
      ENDIF
      RETURN
      END
