*DECK EPCRMD
      SUBROUTINE EPCRMD(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY,
     >                  IPRINT,NOPT,IOPT,CARRET)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Driver for Error Propagation Module with option "RNDMicXS".
*
*Copyright:
* Copyright (C) 2009 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Parameters: input
* NENTRY  number of data structures transfered to this module.
* HENTRY  name of the data structures.
* IENTRY  data structure type where:
*         \begin{itemize} 
*         \item IENTRY=1 for LCM memory object; 
*         \item IENTRY=2 for XSM file; 
*         \item IENTRY=3 for sequential binary file;
*         \item IENTRY=4 for sequential ASCII file.  
*         \end{itemize}
* JENTRY  access permission for the data structure where:
*         \begin{itemize} 
*         \item JENTRY=0 for a data structure in creation mode;
*         \item JENTRY=1 for a data structure in modifications mode;
*         \item JENTRY=2 for a data structure in read-only mode.
*         \end{itemize} 
* KENTRY  data structure pointer.
* IPRINT  print level
* NOPT    length of array IOPT containing the procession option flags.
* IOPT    processing option with
*         IOPT(1)  type of processing (3 for current option)
*         IOPT(2)  entry number for L_EPC structure
*         IOPT(3)  entry number for read only SEQ_ASCII or BINARY file
*         IOPT(4)  entry number for update microlib
* CARRET  last input option read
* 
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          NENTRY                  	
      CHARACTER        HENTRY(NENTRY)*12
      INTEGER          IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
      INTEGER          IPRINT,NOPT,IOPT(NOPT)
      CHARACTER*12     CARRET
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='EPCRMD')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
      INTEGER          NSTATE,NTC
      PARAMETER       (NSTATE=40,NTC=3)
      INTEGER          NDPROC
      PARAMETER       (NDPROC=20)      
*----
*  Input and output parameters
*----
      INTEGER          ITYPLU,INTLIR
      CHARACTER        CARLIR*12
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
*----
*  Memory allocation
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          IDIS,IDXS,VAR,ICOV,COV,NAMISO,IDNXS
      INTEGER          NISOU,NISON,ISOMIX,DENS,IDVAR,DRVAR
      INTEGER          FID,FNM
      INTEGER          IDVF,IDMF
*----
*  Local variables
*----
      INTEGER          IPEPC,IPCOV,IPMIC,IFMT,IEN,IPRTL,IXS
      INTEGER          ISTATE(NSTATE),ISTATM(NSTATE)
      CHARACTER*6      NAMDXS(NDPROC+1)
      CHARACTER*8      XSN
      INTEGER          NGR,NIS,NXS,NCV,ISOREC
      INTEGER          NMIXT,NBISO,NGROUP,NIFISS,NFI,NISL
      INTEGER          ITOTL,ISCAT
*----
*  Read instruction for RNDMicXS
*----
      IPRTL=IPRINT
      IF(IPRTL .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      IF(CARRET .EQ. ';') GO TO 105
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  CONTINUE
      IF(ITYPLU .EQ. 10) GO TO 105
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': Read error -- Character variable expected')
      CARRET=CARLIR
      IF(CARLIR .EQ. ';') THEN
        GO TO 105 
      ELSE
        CALL XABORT(NAMSBR//': Read error -- keyword'//CARLIR//
     >'for processing option is invalid.')      
      ENDIF
      GO TO 100
 105  CONTINUE
*----
*  Output structure
*----
      IEN=ABS(IOPT(3))
      IPEPC=KENTRY(ABS(IOPT(2)))
      IPCOV=KENTRY(IEN)
      IPMIC=KENTRY(ABS(IOPT(4)))
      IF(IOPT(3) .NE. 0) THEN
*----
*  Transfer Variance and covariance data on EPC data structure
*----
        IFMT=IOPT(3)/IEN
        IF(IFMT .GT. 0) THEN
          READ(IPCOV,'(3I8)') NGR,NIS,NXS
        ELSE
          READ(IPCOV) NGR,NIS,NXS
        ENDIF
        IF(IPRTL .GE. 10) THEN
          WRITE(IOUT,6010) NGR,NIS,NXS
        ENDIF
        NCV=(NGR*(NGR+1))/2
        CALL SETARA(IBASE,NIS,IDIS)
        CALL SETARA(IBASE,3*NIS,NAMISO)
        CALL SETARA(IBASE,2*NXS,IDXS)
        CALL SETARA(RBASE,NGR,VAR)
        CALL SETARA(IBASE,NGR,ICOV)
        CALL SETARA(RBASE,NCV,COV)
        CALL EPCRMV(IPEPC,IPCOV,IPRINT,IFMT,NGR,NIS,NXS,NCV,
     >              IBASE(IDIS),IBASE(NAMISO),IBASE(IDXS),
     >              RBASE(VAR),IBASE(ICOV),RBASE(COV))        
        CALL RLSARA(RBASE(COV))
        CALL RLSARA(IBASE(ICOV))
        CALL RLSARA(RBASE(VAR))
        CALL RLSARA(IBASE(IDXS))
        CALL RLSARA(IBASE(NAMISO))
        CALL RLSARA(IBASE(IDIS))
        CALL XDISET(ISTATE,NSTATE,0)        
        ISTATE(1)=IOPT(1)
        ISTATE(2)=NGR
        ISTATE(3)=NIS
        ISTATE(4)=NXS
        CALL LCMPUT(IPEPC,'STATE-VECTOR',NSTATE,1,ISTATE)
      ENDIF
      IF(IOPT(4) .NE. 0) THEN
*----
*  Get info for EPC data structure
*----
        CALL XDISET(ISTATE,NSTATE,0)        
        CALL LCMGET(IPEPC,'STATE-VECTOR',ISTATE)
        IF(ISTATE(1) .NE. 3) CALL XABORT(NAMSBR//
     >': Invalid format for EPC data structure')
        NGR=ISTATE(2)
        NIS=ISTATE(3)
        NXS=ISTATE(4)
        ISOREC=ISTATE(5)
        NFI=ISTATE(6)
        NISL=ISTATE(7)
        IF(IPRTL .GE. 10) THEN
          WRITE(IOUT,6011) NGR,NIS,NXS
        ENDIF
        NCV=(NGR*(NGR+1))/2
*----
*  Get info for microlib data structure
*----
        CALL XDISET(ISTATM,NSTATE,0)        
        CALL LCMGET(IPMIC,'STATE-VECTOR',ISTATM)
        NMIXT=ISTATM(1)
        NBISO=ISTATM(2)
        NGROUP=ISTATM(3)
        IF(NGROUP .NE. NGR) CALL XABORT(NAMSBR//
     >': Number of groups in MICROLIB and EPC incoherent.')
        CALL LCMSIX(IPMIC,'MACROLIB    ',ILCMUP)
        CALL XDISET(ISTATM,NSTATE,0)        
        CALL LCMGET(IPMIC,'STATE-VECTOR',ISTATM)
        NIFISS=ISTATM(4)
        CALL LCMSIX(IPMIC,'MACROLIB    ',ILCMDN)
*----
*  Allocate memory
*----
        CALL SETARA(IBASE,3*NIS,NAMISO)
        CALL SETARA(IBASE,3*NBISO,NISOU)
        CALL SETARA(IBASE,NBISO,ISOMIX)
        CALL SETARA(IBASE,2*NIS,IDVF)
        CALL SETARA(IBASE,2*NBISO,IDMF)
        CALL LCMGET(IPMIC,'ISOTOPESUSED',IBASE(NISOU))
        CALL LCMGET(IPMIC,'ISOTOPESMIX ',IBASE(ISOMIX))
        CALL LCMSIX(IPEPC,'XSVariances ',ILCMUP)
        CALL LCMGET(IPEPC,'NAMEISO     ',IBASE(NAMISO))
*----
*  Variance XS
*----
        CALL SETARA(IBASE,2*NXS,IDXS)
        CALL LCMGET(IPEPC,'NAMEXS      ',IBASE(IDXS))
        ISCAT=0
        ITOTL=0
        DO IXS=1,NXS
          WRITE(XSN,'(2A4)') IBASE(IDXS+2*IXS-2),IBASE(IDXS+2*IXS-1)
          NAMDXS(IXS)=XSN(1:6)
          IF(NAMDXS(IXS).EQ.'TOTAL ') THEN
            ITOTL=IXS
          ELSE IF(NAMDXS(IXS).EQ.'SIGS00') THEN
            ISCAT=IXS
          ENDIF
        ENDDO
        CALL RLSARA(IBASE(IDXS))
        IF(ISOREC .EQ. 0) THEN
          CALL SETARA(IBASE,3*NBISO,NISON)
          CALL SETARA(IBASE,NMIXT*NIFISS,FID)
          CALL SETARA(IBASE,2*NIFISS,FNM)
          CALL EPCRMI(IPMIC,IPRINT,NIS,NXS,NBISO,NMIXT,NIFISS,
     >                IBASE(NAMISO),IBASE(NISOU),
     >                IBASE(IDVF),IBASE(IDMF),IBASE(NISON),
     >                IBASE(FID),IBASE(FNM))
          CALL RLSARA(IBASE(FNM))
          CALL RLSARA(IBASE(FID))
          CALL RLSARA(IBASE(NISON))
          CALL LCMPUT(IPEPC,'INDEXISOV',2*NIS,1,IBASE(IDVF))
          CALL LCMPUT(IPEPC,'INDEXISOM',2*NBISO,1,IBASE(IDMF))
          ISOREC=1
          NFI=NIFISS
          NISL=NBISO
          ISTATE(5)=ISOREC
          ISTATE(6)=NFI
          ISTATE(7)=NISL
          CALL LCMPUT(IPEPC,'STATE-VECTOR',NSTATE,1,ISTATE)
        ELSE
          IF(NFI .NE. NIFISS) CALL XABORT(NAMSBR//
     >': Number of fissile isotopes in MICROLIB and EPC incoherent.')
          IF(NISL .NE. NBISO) CALL XABORT(NAMSBR//
     >': Number of isotopes in MICROLIB and EPC incoherent.')
          CALL LCMGET(IPEPC,'INDEXISOV',IBASE(IDVF))
          CALL LCMGET(IPEPC,'INDEXISOM',IBASE(IDMF))
        ENDIF
*----
*  Update macrolib from random
*  error distribution
*----
        CALL SETARA(IBASE,NGR*NXS,ICOV)
        CALL SETARA(RBASE,NCV*NXS,COV)
        CALL SETARA(RBASE,NBISO,DENS)
        CALL SETARA(RBASE,2*NGR*NXS,DRVAR)
        CALL EPCRMU(IPEPC,IPMIC,IPRINT,NGR,NIS,NXS,NCV,
     >              NBISO,NMIXT,NIFISS,ITOTL,ISCAT,NAMDXS,
     >              IBASE(NAMISO),IBASE(NISOU),IBASE(ISOMIX),
     >              IBASE(IDVF),IBASE(IDMF),IBASE(ICOV),
     >              RBASE(COV),RBASE(DENS),RBASE(DRVAR))
        CALL RLSARA(RBASE(DRVAR))
        CALL RLSARA(RBASE(DENS))
        CALL RLSARA(RBASE(COV))
        CALL RLSARA(IBASE(ICOV))
        CALL RLSARA(IBASE(IDMF))
        CALL RLSARA(IBASE(IDVF))
        CALL RLSARA(IBASE(ISOMIX))
        CALL RLSARA(IBASE(NISOU))
        CALL RLSARA(IBASE(NAMISO))
        CALL LCMSIX(IPEPC,'XSVariances ',ILCMDN)
      ENDIF
      IF(IPRTL .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT('  Creating EPC data structure with:'/
     >       '  NGROUP  = ',I8/
     >       '  NISO    = ',I8/
     >       '  NXS     = ',I8)
 6011 FORMAT('  Updating MICROLIB from EPC with:'/
     >       '  NGROUP  = ',I8/
     >       '  NISO    = ',I8/
     >       '  NXS     = ',I8)
      END
