*DECK EPCRMU
      SUBROUTINE EPCRMU(IPEPC,IPMIC,IPRINT,NGR,NIS,NXS,NCV,
     >                  NBISO,NMIXT,NIFISS,ITOTL,ISCAT,NAMDXS,
     >                  NAMISO,NISOU,ISOMIX,IDVF,IDMF,
     >                  ICOV,COV,DENS,DRVAR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Update macrolib from random error distribution.
*
*Copyright:
* Copyright (C) 2009 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Parameters: input
* IPEPC   pointer to EPC data structure.
* IPMIC   pointer to microlib.
* IPRINT  print level
* NGR     number of groups.
* NIS     number of isotopes on EPC.
* NXS     number of cross section types on EPC.
* NCV     maximum dimension of symmetrized covariance matrix.
* NBISO   number of isotopes on MICROLIB.
* NMIXT   number of MIXTURES on MICROLIB.
* NIFISS  number of fissiles isotopes on MICROLIB.
* ITOTL   Position of total XS in NAMDXS
* ISCAT   Position of scattering XS in NAMDXS
* NAMDXS  names of XS
* NAMISO  array containing the isotope names.
* NISOU   MICROLIB isotopes used
* ISOMIX  MICROLIB isotopes mixtures
* IDVF    Variance isotopes to analyze and fission id
* IDMF    Microlib isotopes to analyze and fission id
*Parameters: work
* ICOV    array to store indices to reconstructe full covariance
*         matrix from compressed covariance matrix.
* COV     array to store compressed covariance matrix.
* DENS    MICROLIB isotopes densities
* DRVAR   Random variance distribution (width=1)
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPEPC,IPMIC,IPRINT,NGR,NIS,NXS,NCV,
     >                 NBISO,NMIXT,NIFISS,ITOTL,ISCAT
      CHARACTER*6      NAMDXS(NXS)
      INTEGER          NAMISO(3,NIS),NISOU(3,NBISO),ISOMIX(NBISO),
     >                 IDVF(2,NIS),IDMF(2,NBISO),ICOV(NGR,NXS)
      REAL             COV(NCV,NXS),DENS(NBISO)
      DOUBLE PRECISION DRVAR(NGR,NXS)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='EPCRMU')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
*----
*  Memory allocation
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          XSREC,XSSCMP,NJJ,IJJ,VAR,VAROLD,
     >                 XSMAC,IPO,XSFIS,RST
*----
*  Local variables
*----
      INTEGER          ILCMLN,ILCMTY,IPRTL
      INTEGER          ISO,IXS,JSO,NUSED,IGR,ISOF
      CHARACTER        ISONAM*12,RECNAM*12,XSN*8,NAMMIC*12
      INTEGER          ITC,ITYPRS,IMIX,NRN
      REAL             DENSI
      INTEGER          ISEED,IGS,IORD,MINLEG,MAXLEG
      SAVE             ISEED,IGS,IORD,MINLEG,MAXLEG
      DATA             ISEED,IGS,IORD,MINLEG,MAXLEG
     >                /0,    -1, 1,   0,     0/
*----
*  Write header
*----
      IPRTL=IPRINT
      IF(IPRTL .GE. 2) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
*----
*  Isotope densities
*----
      CALL SETARA(RBASE,NGR*NXS*NMIXT*NIFISS,XSMAC)
      CALL XDRSET(RBASE(XSMAC),NGR*NXS*NMIXT*NIFISS,0.0)
      CALL SETARA(RBASE,NGR*(NXS+1),XSREC)
      CALL SETARA(RBASE,NGR,VAR)
      CALL SETARA(RBASE,NGR,VAROLD)
      CALL SETARA(RBASE,NGR,RST)
      CALL LCMGET(IPMIC,'ISOTOPESDENS',DENS)
      DO ISO=1,NIS
*----
*  Test if isotope used in Microlib
*----
        IF(IDVF(1,ISO) .GT .0) THEN
*----
*  Isotope is used
*  read covariance matrices
*----
          CALL XDISET(ICOV,NGR*NXS,0)
          CALL XDRSET(COV,NCV*NXS,0)
          WRITE(ISONAM,'(3A4)') (NAMISO(ITC,ISO),ITC=1,3)
          IF(IPRTL .GE. 2) THEN
            WRITE(IOUT,*) ISONAM
          ENDIF
          CALL LCMSIX(IPEPC,ISONAM,ILCMUP)
          DO IXS=1,NXS
*----
*  Get covariance matrices
*----
            RECNAM='INDX'//NAMDXS(IXS)//'  '
            CALL LCMLEN(IPEPC,RECNAM,ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. NGR) THEN
              CALL LCMGET(IPEPC,RECNAM,ICOV(1,IXS))
              RECNAM=NAMDXS(IXS)//'      '
              CALL LCMGET(IPEPC,RECNAM,COV(1,IXS))
*----
*  Generate random numbers from normal distribution
*----
              CALL RANDDN(ISEED,NGR,DRVAR(1,IXS))
*               CALL EPCRMT(ISEED,NGR,NCV,IXS,ICOV,COV,NRN,DRVAR)
*              CALL XDDSET(DRVAR(1,IXS),NGR,0.001D0)
              IF(IPRTL .GE. 5) THEN
                WRITE(IOUT,*) NAMDXS(IXS),'DRVAR=['
                WRITE(IOUT,6010) (DRVAR(IGR,IXS),IGR=1,NGR)
                WRITE(IOUT,*) '];'
              ENDIF
            ENDIF
          ENDDO
          CALL LCMSIX(IPEPC,ISONAM,ILCMDN)
*----
*  Scan over Microlib isotopes associated with this variance isotope   
*----
          DO JSO=1,NBISO
            IF(IDMF(1,JSO) .EQ. ISO) THEN
              ISOF=IDMF(2,JSO)
*----
*  Read microlib for isotope
*----
              WRITE(NAMMIC,'(3A4)') (NISOU(ITC,JSO),ITC=1,3)
              CALL LCMSIX(IPMIC,NAMMIC,ILCMUP)
*----
*  Get microscopic xs
*----
              CALL XDRLXS(IPMIC ,IGS   ,IPRINT,NXS   ,NAMDXS,IORD  ,
     >                    NGR,RBASE(XSREC))
*----
*  Add contribution to macrolib
*---- 
              DENSI=DENS(JSO)  
              IMIX=ISOMIX(JSO)           
              CALL LCMSIX(IPMIC,'VARIANCES   ',ILCMUP)
              CALL EPCRMA(IPMIC,IPRINT,NGR,NXS,NCV,NMIXT,NIFISS,
     >                    IMIX,ISOF,ITOTL,ISCAT,NAMDXS,
     >                    DENSI,ICOV,COV,DRVAR,
     >                    RBASE(XSREC),RBASE(XSMAC),RBASE(VAR),
     >                    RBASE(VAROLD),RBASE(RST))
              CALL LCMSIX(IPMIC,'VARIANCES   ',ILCMDN)
              CALL LCMSIX(IPMIC,NAMMIC,ILCMDN)
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      IF(IPRTL .GE. 2) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      CALL RLSARA(RBASE(RST))
      CALL RLSARA(RBASE(VAROLD))
      CALL RLSARA(RBASE(VAR))
      CALL RLSARA(RBASE(XSREC))
*----
*  Update macrolib
*----
      CALL LCMSIX(IPMIC,'MACROLIB    ',ILCMUP)
      CALL SETARA(RBASE,NMIXT,XSREC)
      CALL SETARA(RBASE,NMIXT*NIFISS,XSFIS)
      CALL SETARA(RBASE,NGR*NMIXT,XSSCMP)
      CALL SETARA(IBASE,NMIXT,IJJ)
      CALL SETARA(IBASE,NMIXT,IPO)
      CALL EPCRMS(IPMIC,IPRINT,NGR,NXS,NMIXT,NIFISS,
     >            NAMDXS,RBASE(XSMAC),
     >            RBASE(XSREC),RBASE(XSFIS),RBASE(XSSCMP),
     >            IBASE(IJJ),IBASE(IPO))
      CALL LCMSIX(IPMIC,'MACROLIB    ',ILCMDN)
      CALL RLSARA(IBASE(IPO))
      CALL RLSARA(IBASE(IJJ))
      CALL RLSARA(RBASE(XSSCMP))
      CALL RLSARA(RBASE(XSFIS))
      CALL RLSARA(RBASE(XSREC))
      CALL RLSARA(RBASE(XSMAC))
*----
*  Return
*----
      RETURN
*----
*  Formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT(10F10.4,:,' ...')
      END
