*DECK EPCRPC
      SUBROUTINE EPCRPC(NBUNIT,IUNIT,NOREC,NTREC,NFREC,MAXD,
     >                  NBREN,NAMNEW,NRENEW,
     >                  NAMOLD,NREOLD,REFOLD,NXTOLD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Store normal distributions on EPC data structure.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*
*Parameters: input
* NBUNIT  number of data structures
* IUNIT   unit associated data structures
* NOREC   old total number of independent parameters
* NTREC   new total number of parameters
* NBREN   number of parameters for each data structure
* NAMNEW  name of new parameters
* NRENEW  number of elements for new parameters
*
*Parameters: input/output
* NAMOLD  name of old (input) and updated (output) parameters
* NREOLD  number of elements for old (input) and 
*         updated (output) parameters
* REFOLD  reference value of elements for old (input) and 
*         0.0 for updated (output) parameters
* NXTOLD  last of elements for old (input) and 
*         element 0 of updated (output) parameters
* 
*Parameters: output
* NFREC   final total number of independent parameters
* MAXD    maximum number of elements in distributions
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          NBUNIT                  	
      INTEGER          IUNIT(NBUNIT)
      INTEGER          NOREC,NTREC,NFREC,MAXD
      INTEGER          NBREN(NBUNIT)
      INTEGER          NAMNEW(3,NTREC+1),NRENEW(NTREC+1)
      INTEGER          NAMOLD(3,NOREC+NTREC+1),NREOLD(NOREC+NTREC+1)
      REAL             REFOLD(NOREC+NTREC+1)
      INTEGER          NXTOLD(NOREC+NTREC+1)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='EPCRPC')
      INTEGER          NTC
      PARAMETER       (NTC=3)
*----
*  Memory allocation
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          RECD
*----
*  Local variables
*----
*      INTEGER          KCHAR(NTC)
      INTEGER          IPEPC,IU,ILU,ITC,NPAR,IPAR,
     >                 INREC,IOREC,IBR,NBR
      CHARACTER        NAME*12,NAMED*12
*----
*  Process current EPC data structure and
*  extract new parameters name from distribution files.
*----
      IU=1
      IPEPC=IUNIT(IU)
      IF(NOREC .GT. 0) THEN
        CALL LCMGET(IPEPC,'ParametreNom',NAMOLD)
        CALL LCMGET(IPEPC,'ParametreNbr',NREOLD)
        CALL LCMGET(IPEPC,'ParametreNxt',NXTOLD)
        CALL LCMGET(IPEPC,'ParametreRef',REFOLD)
      ENDIF
      INREC=0
      DO IU=2,NBUNIT
        NPAR=NBREN(IU)
        DO IPAR=1,NPAR
          INREC=INREC+1
          ILU=IUNIT(IU)
          READ(ILU,*) NAME,NRENEW(INREC)
          READ(NAME,'(3A4)') (NAMNEW(ITC,INREC),ITC=1,NTC)
          MAXD=MAX(MAXD,NRENEW(INREC))
        ENDDO
      ENDDO
*----
*  Scan new parameter names and compare with old names      
*----
      NFREC=NOREC
      DO INREC=1,NTREC
        DO IOREC=1,NFREC
          IF(NAMNEW(1,INREC) .EQ. NAMOLD(1,IOREC) .AND. 
     >       NAMNEW(2,INREC) .EQ. NAMOLD(2,IOREC) .AND.
     >       NAMNEW(3,INREC) .EQ. NAMOLD(3,IOREC) ) THEN
            IF(NREOLD(IOREC) .GT. 0) THEN
              NREOLD(IOREC)=-INREC
              NXTOLD(IOREC)=0
              REFOLD(IOREC)=0.0
              GO TO 1000
            ENDIF
          ENDIF
        ENDDO
        NFREC=NFREC+1
        NAMOLD(1,NFREC)=NAMNEW(1,INREC)
        NAMOLD(2,NFREC)=NAMNEW(2,INREC)
        NAMOLD(3,NFREC)=NAMNEW(3,INREC)
        NREOLD(NFREC)=-INREC
        NXTOLD(IOREC)=0
        REFOLD(IOREC)=0.0
 1000   CONTINUE
      ENDDO
      CALL SETARA(RBASE,MAXD,RECD)
      INREC=0
      DO IU=2,NBUNIT
        NPAR=NBREN(IU)
        DO IPAR=1,NPAR
          INREC=INREC+1
          NBR=NRENEW(INREC)
          ILU=IUNIT(IU)
          WRITE(NAME,'(3A4)') (NAMNEW(ITC,INREC),ITC=1,NTC)
          READ(ILU,*) NAMED
          IF(NAMED .NE. NAME) CALL XABORT(NAMSBR//': '//
     >    NAMED//' and '//NAME//' should be identical.')
          READ(ILU,*) (RBASE(RECD-1+IBR),IBR=1,NBR)
          DO IOREC=1,NFREC
            IF(NREOLD(IOREC).EQ. -INREC) THEN
              NREOLD(IOREC)=NBR
              CALL LCMPUT(IPEPC,NAME,NBR,2,RBASE(RECD))
            ENDIF
          ENDDO   
        ENDDO
      ENDDO
      CALL LCMPUT(IPEPC,'ParametreNom',3*NFREC,3,NAMOLD)
      CALL LCMPUT(IPEPC,'ParametreNbr',NFREC,1,NREOLD)
      CALL LCMPUT(IPEPC,'ParametreNxt',NFREC,1,NXTOLD)
      CALL LCMPUT(IPEPC,'ParametreRef',NFREC,2,REFOLD)
      CALL RLSARA(RBASE(RECD))
      RETURN
*----
*  Warning formats
*----
      END
