*DECK EPCRPD
      SUBROUTINE EPCRPD(NENTRY,KENTRY,IPRINT,NOPT,IOPT,CARRET)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To extract library parameters with normal distribution 
* around the average.
*
*Copyright:
* Copyright (C) 2009 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Parameters: input
* NENTRY  number of data structures transfered to this module.
* KENTRY  data structure pointer.
* IPRINT  print level
* NOPT    number of options
* IOPT    processing option with
*         IOPT(1)  type of processing (=0 for current option)
*         IOPT(2)  entry position for L_EPC structure
*         IOPT(3)  number of parameters
*         IOPT(4)  entry for normal distribution file
*         IOPT(5)  number of records on normal distribution file
* CARRET  last input option read
* 
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          NENTRY                  	
      INTEGER          KENTRY(NENTRY)
      INTEGER          IPRINT,NOPT,IOPT(NOPT)
      CHARACTER*12     CARRET
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='EPCRPD')
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
*      INTEGER          NTC
*      PARAMETER       (NTC=3)
*----
*  Input and output parameters
*----
      INTEGER          ITYPLU,INTLIR
      CHARACTER        CARLIR*12
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
*----
*  Memory allocation
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          NBREN,NAMNEW,NRENEW,NAMOLD,NREOLD,NXTOLD,
     >                 REFOLD,RECD
*----
*  Local variables
*----
      INTEGER          IKNDI,IPNDI
*      INTEGER          IEN,ITC,KCHAR(NTC)
*      CHARACTER        HSIGN*12
      INTEGER          ISTATE(NSTATE)
      INTEGER          NOREC,NTREC,NFREC,MAXD
*----
*  Output structure
*----
      IKNDI=1
      IF(IOPT(3) .EQ. 0) IKNDI=-1
      IPNDI=KENTRY(IOPT(2))
      CALL LCMGET(IPNDI,'STATE-VECTOR',ISTATE)
*----
*  Input structure
*----
      NOREC=ISTATE(1)
      MAXD=ISTATE(2)
      NTREC=IOPT(5)
*----
*  Recover parameter names from output and input structures      
*----
      NFREC=NTREC+NOREC
      CALL SETARA(IBASE,3*(NFREC+1),NAMOLD)
      CALL SETARA(IBASE,(NFREC+1),NREOLD)
      CALL SETARA(RBASE,(NFREC+1),REFOLD)
      CALL SETARA(IBASE,(NFREC+1),NXTOLD)
      CALL SETARA(IBASE,3*(NTREC+1),NBREN)
      CALL SETARA(IBASE,3*(NTREC+1),NAMNEW)
      CALL SETARA(IBASE,(NTREC+1),NRENEW)
*      CALL NDICRE(NENTRY,KENTRY,NOREC,NTREC,NFREC,MAXD,
*     >            IBASE(NBREN),IBASE(NAMNEW),IBASE(NRENEW),
*     >            IBASE(NAMOLD),IBASE(NREOLD),
*     >            RBASE(REFOLD),IBASE(NXTOLD)) 
      ISTATE(1)=NFREC
      ISTATE(2)=MAXD
      CALL LCMPUT(IPNDI,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL RLSARA(IBASE(NRENEW))
      CALL RLSARA(IBASE(NAMNEW))
      CALL RLSARA(IBASE(NBREN))
*----
*  INPUT/OUTPUT VARIABLES 
*  Input data is of the form
*    [ EDIT iprint ]
*    [ SET (dataSET) ]
*    [ GET (dataGET) ]
*  where
*  EDIT               = keyword for print level 
*  iprint             = integer print level 
*  SET                = keyword to set reference value 
*                       and extract number of parameters
*  GET                = keyword to set next value  
*                       and to extract parameter
*  (dataSET)          = SET data processed by NDISET
*  (dataGET)          = GET data processed by NDIGET
*----
      CALL SETARA(RBASE,MAXD,RECD)
      IPRINT=1    
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  CONTINUE
      IF(ITYPLU .EQ. 10) GO TO 105
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': Read error -- Character variable expected')
      IF(CARLIR .EQ. ';') THEN
        GO TO 105 
      ELSE IF(CARLIR .EQ. 'EDIT') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': Read error -- integer value for EDIT expected.')
        IPRINT=INTLIR
      ELSE IF(CARLIR .EQ. 'SET') THEN
        CARRET=CARLIR
*        CALL NDISET(IPNDI,IPRINT,CARRET,NFREC,
*     >              IBASE(NAMOLD),IBASE(NREOLD),
*     >              RBASE(REFOLD),IBASE(NXTOLD))
        CARLIR=CARRET
        GO TO 101
      ELSE IF(CARLIR .EQ. 'GET') THEN
        CARRET=CARLIR
*        CALL NDIGET(IPNDI,IPRINT,CARRET,NFREC,MAXD,
*     >              IBASE(NAMOLD),IBASE(NREOLD),
*     >              RBASE(REFOLD),IBASE(NXTOLD),
*     >              RBASE(RECD))
        CARLIR=CARRET
        GO TO 101
      ELSE
        CALL XABORT(NAMSBR//
     >  ': '//CARLIR//' is an invalid keyword.')
      ENDIF
      GO TO 100
 105  CONTINUE
*----
*  Release memory and return
*----
      CALL RLSARA(RBASE(RECD))
      CALL RLSARA(RBASE(REFOLD))
      CALL RLSARA(IBASE(NXTOLD))
      CALL RLSARA(IBASE(NREOLD))
      CALL RLSARA(IBASE(NAMOLD))
      RETURN
*----
*  Warning formats
*----
      END
