*DECK EPCRPG
      SUBROUTINE EPCRPG(IPEPC,IPRINT,CARRET,NFREC,MAXD,
     >                  NAMOLD,NREOLD,REFOLD,NXTOLD,RECD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* GET EPC data from normal distributions.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*
*Parameters: input
* IPEPC   pointer to the EPC data structure
* IPRINT  print level
* NFREC   final total number of independent parameters
* MAXD    maximum number of elements in distributions
* NAMOLD  name of old (input) and updated (output) parameters
* NREOLD  number of elements for old (input) and 
*         updated (output) parameters
* REFOLD  reference value of parameters
*Parameters: input/output
* CARRET  last character variable read
* NXTOLD  last of parameter extracted
*Parameters: dummy
* RECD    parameter list
* 
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPEPC,IPRINT                  	
      CHARACTER        CARRET*12
      INTEGER          NFREC,MAXD
      INTEGER          NAMOLD(3,NFREC+1),NREOLD(NFREC+1)
      REAL             REFOLD(NFREC+1)
      INTEGER          NXTOLD(NFREC+1)
      REAL             RECD(MAXD)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='EPCRPG')
      INTEGER          NTC
      PARAMETER       (NTC=3)
*----
*  Input and output parameters
*----
      INTEGER          ITYPLU,INTLIR
      CHARACTER        CARLIR*12
      REAL             REALIR
      DOUBLE PRECISION DBLLIR
      INTEGER          ITYPUT,INTPUT
      CHARACTER        CARPUT*12
      REAL             REAPUT
      DOUBLE PRECISION DBLPUT
*----
*  Local variables
*----
      INTEGER          KCHAR(NTC)
      INTEGER          ITC,IUPD,IREC,JREC,JPAR
      CHARACTER        NAMREC*12
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      IUPD=0       
*----
*  Input data is of the form (dataGET)
*    [[ cparam [ <<inext>> ] >>pvalue<<  ]]
*  where
*  cparam             = parameter name
*  inext              = parameters number tabulated
*  pvalue             = value of parametre
*----
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  CONTINUE
      IF(ITYPLU .EQ. 10) THEN
        CARLIR=';'
        GO TO 105
      ENDIF
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': Read error -- Character variable expected')
      IF(CARLIR .EQ. ';') THEN
        GO TO 105 
      ELSE IF(CARLIR .EQ. 'SET' .OR. CARLIR .EQ. 'GET') THEN
        GO TO 105
      ELSE
*----
*  Search if name of parameter exists
*----
        NAMREC=CARLIR
        JREC=0
        READ(NAMREC,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        DO IREC=1,NFREC
          IF(KCHAR(1) .EQ. NAMOLD(1,IREC) .AND. 
     >       KCHAR(2) .EQ. NAMOLD(2,IREC) .AND.
     >       KCHAR(3) .EQ. NAMOLD(3,IREC) ) THEN
            JREC=IREC
            GO TO 110
          ENDIF
        ENDDO
        CALL XABORT(NAMSBR//': '//CARLIR//' is an invalid parameter.')
 110    CONTINUE
        JPAR=NXTOLD(JREC)+1
        IUPD=IUPD+1
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .EQ. 3) GO TO 101
        IF(ITYPLU .EQ. 1) THEN
          IF(INTLIR .GT. NREOLD(JREC)) THEN  
            IUPD=IUPD-1
            INTLIR=1
          ELSE IF(INTLIR .LE. 0) THEN
            INTLIR=1
          ELSE
            IUPD=IUPD-1
          ENDIF  
          JPAR=INTLIR
          IF(IPRINT .GE. 10) THEN
            WRITE(IOUT,6010) CARLIR,JPAR,JREC
          ENDIF
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .EQ. 3) GO TO 101
        ELSE
          IF(JPAR .GT. NREOLD(JREC)) THEN  
            JPAR=1
          ENDIF  
          NXTOLD(JREC)=JPAR
          IF(IPRINT .GE. 10) THEN
            WRITE(IOUT,6010) CARLIR,JPAR,JREC
          ENDIF
        ENDIF
        ITYPUT=2
        IF(ITYPLU .NE. -ITYPUT) CALL XABORT(NAMSBR//
     >': Real output variable for steps value expected')
        CALL LCMGET(IPEPC,NAMREC,RECD)
        REAPUT=RECD(JPAR)+REFOLD(JREC)
        CALL REDPUT(ITYPUT,INTPUT,REAPUT,CARPUT,DBLPUT)
        IF(IPRINT .GE. 10) THEN
          WRITE(IOUT,6011) NAMREC,RECD(JPAR),REFOLD(JREC),REAPUT
        ENDIF
      ENDIF
      GO TO 100
 105  CONTINUE
      CARRET=CARLIR
      IF(IUPD .GT. 0) THEN
        CALL LCMPUT(IPEPC,'ParametreNxt',NFREC,1,NXTOLD)
      ENDIF
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT(' Index for Parameter ',A12,' is ',2I20)
 6011 FORMAT(' Value for parameter ',A12,' is ',1P,3E20.10)
      END 
