*DECK EVO
      SUBROUTINE EVO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for an isotopic depletion calculation.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation or modification type(L_BURNUP);
*         HENTRY(2): modification type(L_LIBRARY);
*         HENTRY(3): read-only type(L_TRACK);
*         HENTRY(4): optional read-only type(L_FLUX)
*         or
*         HENTRY(1): creation or modification type(L_BURNUP);
*         HENTRY(2): creation type(L_LIBRARY);
*         HENTRY(3): read-only type(L_LIBRARY);
*         HENTRY(4): read-only type(L_TRACK);
*         HENTRY(5): optional read-only type(L_FLUX).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER HSMG*131,TEXT12*12,CDOOR*12,HSIGN*12
      INTEGER IPAR(NSTATE),KCHAR(3),IGP(NSTATE),IBASE(1)
      INTEGER          ITYPE,ITIXS,IEXTR,IGLOB,ISAT,
     >                 IDIRAC,ISAVE,ISET,INR
      REAL             RPAR(5),XT(5)
      LOGICAL LOG,LMACRO
      COMMON RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.2) CALL XABORT('EVO: THREE PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('EVO: LCM O'
     1 //'BJECT EXPECTED AT FIRST LHS.')
      IF(JENTRY(1).EQ.2) CALL XABORT('EVO: BURNUP HISTORY STORAGE IN CR'
     1 //'EATION OR MODIFICATION MODE EXPECTED.')
      IPDEPL=KENTRY(1)
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('EVO: LCM O'
     1 //'BJECT EXPECTED AT SECOND LHS.')
      IF(JENTRY(2).EQ.0) THEN
*        INTERNAL LIBRARY CREATION. COPY THE FIRST RHS ON THIS LHS.
         IF(NENTRY.LE.3) CALL XABORT('EVO: FOUR PARAMETERS EXPECTED.')
         IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.
     1   (IENTRY(3).NE.2)))  CALL XABORT('EVO: LCM OBJECT IN READ-ONLY '
     2   //'MODE EXPECTED AT FIRST RHS.')
         CALL LCMEQU(KENTRY(3),KENTRY(2))
         IENT=4
      ELSE IF(JENTRY(2).EQ.1) THEN
*        INTERNAL LIBRARY MODIFICATION.
         IENT=3
      ELSE
         CALL XABORT('EVO: INTERNAL LIBRARY IN CREATE OR MODIFICATION M'
     1   //'ODE EXPECTED.')
      ENDIF
      IPLIB=KENTRY(2)
      TEXT12=HENTRY(2)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPDEPL,'LINK.LIB',3,3,KCHAR)
*----
*  RECOVER IPTRK AND IPFLUX POINTERS
*----
      IPFLUX=0
      IPTRK=0
      DO 10 I=IENT,NENTRY
      IF((JENTRY(I).EQ.2).AND.((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)))
     1 THEN
         CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.EQ.'L_FLUX') THEN
            IPFLUX=KENTRY(I)
         ELSE IF(HSIGN.EQ.'L_TRACK') THEN
            IPTRK=KENTRY(I)
         ELSE
            WRITE(HSMG,'(35HEVO: UNEXPECTED SIGNATURE AT RHS ('',A12,
     1      3H'').)') HSIGN
            CALL XABORT(HSMG)
         ENDIF
      ELSE
         CALL XABORT('EVO: LCM OBJECT IN READ-ONLY MODE EXPECTED AT '
     1   //'RHS.')
      ENDIF
   10 CONTINUE
      IF(IPTRK.EQ.0) CALL XABORT('EVO: UNABLE TO FIND A POINTER TO A TR'
     1 //'ACKING LCM OBJECT.')
*----
*  RECOVER GENERAL TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      NREG=IGP(1)
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
      IF(CDOOR.EQ.'MCCG') THEN
         CALL LCMLEN(IPTRK,'KEYFLX',LKFL,ITYLCM)
         NFUNL=LKFL/NREG
      ELSE
         NFUNL=1
      ENDIF
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG*NFUNL,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
*----
*  RECOVER GENERAL INTERNAL LIBRARY INFORMATION
*----
      CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_LIBRARY') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('EVO: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_LIBRARY EXPECTED.')
      ENDIF
      CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
      NBMIX=IPAR(1)
      NBISO=IPAR(2)
      NGRO=IPAR(3)
      NDEPL=IPAR(11)
      NCOMB=IPAR(12)
      LMACRO=(IPAR(17).GE.0).AND.(IPAR(18).EQ.1)
      IF(IGP(4).GT.NBMIX) THEN
         WRITE(HSMG,'(43HEVO: THE NUMBER OF MIXTURES IN THE TRACKING,
     1   2H (,I5,51H) IS GREATER THAN THE NUMBER OF MIXTURES IN THE INT,
     2   15HERNAL LIBRARY (,I5,2H).)') IGP(4),NBMIX
         CALL XABORT(HSMG)
      ENDIF
      CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
         CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
         IF(IPAR(1).NE.NDEPL) CALL XABORT('EVO: INVALID NUMBER OF D'
     1   //'EPLETING ISOTOPES.')
         NSUPS=IPAR(7)
         NREAC=IPAR(8)
      CALL LCMSIX(IPLIB,' ',2)
      CALL SETARA(IBASE,NBISO,JJMIX)
      CALL SETARA(RBASE,NBISO,JJDEN)
      CALL SETARA(RBASE,NBISO,JIEVOL)
      CALL SETARA(IBASE,NBMIX,MIXBRN)
      CALL SETARA(IBASE,NBMIX,MIXPWR)
      CALL LCMGET(IPLIB,'ISOTOPESMIX',IBASE(JJMIX))
      CALL LCMGET(IPLIB,'ISOTOPESDENS',RBASE(JJDEN))
      CALL LCMGET(IPLIB,'ISOTOPESTODO',IBASE(JIEVOL))
*----
*  RECOVER LOCAL ISOTOPES NAMES
*----
      CALL SETARA(IBASE,3*NBISO,IISONA)
      CALL SETARA(IBASE,3*NBISO,IISONR)
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(IISONA))
      CALL LCMGET(IPLIB,'ISOTOPERNAME',IBASE(IISONR))
*----
*  RECOVER THE AVERAGE NEUTRON FLUXES IN THE L_FLUX SPECIFICATION
*----
      CALL SETARA(RBASE,NREG*NGRO,JFLUXE)
      IF(IPFLUX.NE.0) THEN
         JPFLUX=LCMGID(IPFLUX,'FLUX')
         CALL LCMLEL(JPFLUX,1,ILONG,ITYLCM)
         CALL SETARA(RBASE,ILONG,IFUNKN)
         DO 40 IGR=1,NGRO
         CALL LCMGDL(JPFLUX,IGR,RBASE(IFUNKN))
         DO 40 IREG=1,NREG
         KEYFLX=IBASE(IIDL+IREG-1)
   40    RBASE(JFLUXE+(IGR-1)*NREG+IREG-1)=RBASE(IFUNKN+KEYFLX-1)
         CALL RLSARA(RBASE(IFUNKN))
      ELSE
         CALL XDRSET(RBASE(JFLUXE),NREG*NGRO,0.0)
      ENDIF
*----
*  READ THE INPUT DATA
*----
*     DEFAULT OPTIONS:
      IMPX=1
      IDEPL=0
      INR=-1
      XT(4)=0.0
      IF(JENTRY(1).EQ.0) THEN
         INDREC=1
         RPAR(1)=1.0E-5
         RPAR(2)=1.0E-4
         RPAR(3)=80.0
         RPAR(4)=1.0E-4
         RPAR(5)=0.0
         ITYPE=2
         IEXTR=1
         IGLOB=0
         ISAT=0
         IDIRAC=0
         ITIXS=0
         HSIGN='L_BURNUP'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPDEPL,'SIGNATURE',3,3,KCHAR)
         CALL LCMPUT(IPDEPL,'ISOTOPESMIX',NBISO,1,IBASE(JJMIX))
         CALL LCMPUT(IPDEPL,'ISOTOPESUSED',3*NBISO,3,IBASE(IISONA))
         CALL XDISET(IBASE(MIXBRN),NBMIX,1)
         CALL XDISET(IBASE(MIXPWR),NBMIX,1)
         CALL LCMPUT(IPDEPL,'MIXTURESBurn',NBMIX,1,IBASE(MIXBRN))
         CALL LCMPUT(IPDEPL,'MIXTURESPowr',NBMIX,1,IBASE(MIXPWR))
      ELSE IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(IPDEPL,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_BURNUP') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('LIB: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_BURNUP EXPECTED.')
         ENDIF
         INDREC=2
         CALL LCMGET(IPDEPL,'STATE-VECTOR',IPAR)
         ITYPE=IPAR(1)
         NTIME=IPAR(3)
         IEXTR=IPAR(9)
         IGLOB=IPAR(10)
         ISAT=IPAR(11)
         IDIRAC=IPAR(12)
         ITIXS=IPAR(13)
         IF(IPAR(4).NE.NBISO) CALL XABORT('EVO: INVALID NUMBER OF ISOTO'
     1   //'PES.')
         IF(IPAR(5).NE.NCOMB) CALL XABORT('EVO: INVALID NUMBER OF DEPLE'
     1   //'TING MIXTURES.')
         IF(IPAR(6).NE.NREAC) CALL XABORT('EVO: INVALID NUMBER OF DEPLE'
     1   //'TING REACTIONS.')
         IF(IPAR(7).NE.NDEPL-NSUPS) CALL XABORT('EVO: INVALID NUMBER OF'
     1   //' DEPLETING ISOTOPES.')
         IF(IPAR(8).NE.NBMIX) CALL XABORT('EVO: INVALID NUMBER OF MIXTU'
     1   //'RES.')
         CALL LCMGET(IPDEPL,'EVOLUTION-R',RPAR)
         IF(NTIME.GT.0) THEN
            CALL SETARA(RBASE,NTIME+2,ITIMES)
            CALL LCMGET(IPDEPL,'DEPL-TIMES',RBASE(ITIMES))
            XT(4)=RBASE(ITIMES+NTIME-1)
            CALL RLSARA(RBASE(ITIMES))
         ENDIF
         CALL LCMGET(IPDEPL,'MIXTURESBurn',IBASE(MIXBRN))
         CALL LCMGET(IPDEPL,'MIXTURESPowr',IBASE(MIXPWR))
      ENDIF
*----
*  read input options
*----
      IF(NBMIX.EQ.0) CALL XABORT('EVO: NBMIX NOT YET DEFINED.')
      CALL EVOGET(IMPX,ITYPE,ITIXS,IEXTR,IGLOB,ISAT,
     >            IDIRAC,ISAVE,ISET,INR,IDEPL,RPAR,XT,
     >            NBMIX,IBASE(MIXBRN),IBASE(MIXPWR))
*
      XTI=XT(3)
      XTF=XT(5)
      EPS1=RPAR(1)
      EPS2=RPAR(2)
      EXPMAX=RPAR(3)
      H1=RPAR(4)
      FIT=RPAR(5)
      LOG=(ISAVE.GE.0).OR.(IDEPL.GT.0)
      IF(LOG.AND.(INR.EQ.-1)) CALL XABORT('EVO: TYPE OF DEPLETION NO'//
     1 'T DEFINED.')
      IF(NREG.EQ.0) CALL XABORT('EVO: NREG = 0')
      IF(NDEPL.EQ.0) CALL XABORT('EVO: NO DEPLETING ISOTOPES.')
      IF(NCOMB.EQ.0) CALL XABORT('EVO: NO DEPLETING MIXTURES.')
      CALL SETARA(IBASE,NBMIX,LMASKM)
      CALL SETARA(IBASE,NGRO,LMASKG)
      CALL SETARA(IBASE,NDEPL*NBMIX,IJM)
      CALL SETARA(IBASE,NCOMB,IMILVO)
      CALL SETARA(RBASE,(NDEPL-NSUPS+1)*NCOMB*2,IYDPL)
      CALL SETARA(RBASE,NBMIX,IVX)
      CALL SETARA(IBASE,NBISO,ISOCMB)
      CALL SETARA(IBASE,3*NDEPL,IINADP)
      CALL SETARA(IBASE,NDEPL,IEVOL2)
      CALL SETARA(RBASE,3*(NDEPL-NSUPS+1)*NBMIX*(NREAC+1),ISIG)
      CALL SETARA(RBASE,3*NBMIX,IVPHV)
      CALL SETARA(RBASE,NBMIX,IENERG)
      CALL SETARA(RBASE,NDEPL,IAWR)
      CALL EVODRV(IPDEPL,IPLIB,INDREC,IMPX,NBISO,NGRO,NBMIX,NREG,
     1 IBASE(IISONA),IBASE(IISONR),IBASE(JJMIX),RBASE(JJDEN),
     2 IBASE(JIEVOL),IBASE(IMAT),RBASE(IVOL),NDEPL,NSUPS,NREAC,NCOMB,
     3 EPS1,EPS2,EXPMAX,H1,ITYPE,INR,IEXTR,IGLOB,ISAT,IDIRAC,ITIXS,FIT,
     4 ISAVE,ISET,IDEPL,XTI,XTF,XT,LMACRO,RBASE(JFLUXE),IBASE(MIXBRN),
     5 IBASE(MIXPWR),IBASE(LMASKM),IBASE(LMASKG),IBASE(IJM),
     6 IBASE(IMILVO),RBASE(IYDPL),RBASE(IVX),IBASE(ISOCMB),
     7 IBASE(IINADP),IBASE(IEVOL2),RBASE(ISIG),RBASE(IVPHV),
     8 RBASE(IENERG),RBASE(IAWR))
      CALL RLSARA(RBASE(IAWR))
      CALL RLSARA(RBASE(IENERG))
      CALL RLSARA(RBASE(IVPHV))
      CALL RLSARA(RBASE(ISIG))
      CALL RLSARA(IBASE(IEVOL2))
      CALL RLSARA(IBASE(IINADP))
      CALL RLSARA(IBASE(ISOCMB))
      CALL RLSARA(RBASE(IVX))
      CALL RLSARA(RBASE(IYDPL))
      CALL RLSARA(IBASE(IMILVO))
      CALL RLSARA(IBASE(IJM))
      CALL RLSARA(IBASE(LMASKG))
      CALL RLSARA(IBASE(LMASKM))
      CALL RLSARA(RBASE(JFLUXE))
      CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO,2,RBASE(JJDEN))
*----
*  RELEASE GENERAL TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
*----
*  RELEASE GENERAL INTERNAL LIBRARY INFORMATION
*----
      CALL RLSARA(IBASE(IISONR))
      CALL RLSARA(IBASE(IISONA))
      CALL RLSARA(IBASE(JIEVOL))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
      RETURN
      END
