*DECK EXCELP
      SUBROUTINE EXCELP(  IPTRK, IFTRAK, IPRNTP, NREGIO,  NBMIX,   NANI,
     >                   MATCOD, VOLUME, NRENOR, XSSIGT, XSSIGW, NELPIJ,
     >                    IPIJK,    PIJ, LEAKSW,  NSBG,   NPSYS,   NPST,
     >                   PROBKS, TITREC)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities for Excell. All surfaces
* will disappear from the system using external boundary conditions.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Roy
*
*Parameters: input
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IPRNTP  print flag (equal to zero for no print).
* NREGIO  total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MATCOD(i))).
* NANI    number of Legendre orders.
* MATCOD  index-number of the mixture type assigned to each volume.
* VOLUME  volumes.
* NRENOR  normalization scheme for PIJ matrices.
* XSSIGT  total macroscopic cross sections ordered by mixture.
* XSSIGW  P0 within-group scattering macroscopic cross sections
*         ordered by mixture.
* NELPIJ  number of elements in symmetrized pij matrix.
* IPIJK   pij option (=1 pij, =4 pijk).
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* NSBG    number of energy groups.
* NPSYS   non-converged energy group indices.
* NPST    first dimension of matrix PROBKS.
* TITREC  title.
*
*Parameters: output
* PIJ     reduced and symmetrized collision probabilities.
* PROBKS  directional collision probabilities.
*
*-----------------------------------------------------------------------
*--------+---------------- R O U T I N E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* PIJABC / TO ELIMINATE SURFACES USING B.C. OF THE SYSTEM              *
* PIJAAA / TO ELIMINATE SURFACES FOR PIJKS USING B.C. OF THE SYSTEM    *
* PIJI2D / TO INTEGRATE CP IN 2D GEOMETRIES (ISOTROPIC B.C.)           *
* PIJI3D / TO INTEGRATE CP IN 3D GEOMETRIES (ISOTROPIC B.C.)           *
* PIJRDG / TO RENORMALIZE CP USING DIAGONAL COEFFICIENTS               *
* PIJRGL / TO RENORMALIZE CP USING GELBARD HOMOGENEOUS SCHEME          *
* PIJRNL / TO RENORMALIZE CP USING NON-LINEAR FACTORS                  *
* PIJS2D / TO INTEGRATE CP IN 2D GEOMETRIES (SPECULAR  B.C.)           *
* PIJS3D / TO INTEGRATE CP IN 3D GEOMETRIES (SPECULAR  B.C.)           *
* PIJWPR / TO PRINT CP MATRICES IN SUM FORMAT                          *
* PIJSMD / TO EVALUATE SCATTERING-MODIFIED CP MATRIX                   *
*--------+-------------------------------------------------------------*
*
      IMPLICIT          NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER         TITREC*72
      LOGICAL           LEAKSW
      INTEGER           IPTRK, IFTRAK, IPRNTP, NREGIO,  NBMIX,   NANI,
     >                  MATCOD(NREGIO),NRENOR, NELPIJ,  IPIJK,   NSBG,
     >                  NPSYS(NSBG), NPST
      REAL              VOLUME(NREGIO), XSSIGT(0:NBMIX,NSBG),
     >                  XSSIGW(0:NBMIX,NANI,NSBG),
     >                  PIJ(NELPIJ,IPIJK,NSBG), PROBKS(NPST,NSBG)
*----
*  LOCAL VARIABLES
*----
      INTEGER           IOUT, ICPALL, ICPEND, MXGAUS, NSTATE
      PARAMETER       ( IOUT=6, ICPALL=4, ICPEND=3, MXGAUS=64,
     >                  NSTATE=40 )
      CHARACTER         NAMSBR*6
      PARAMETER       ( NAMSBR='EXCELP')
      INTEGER           MKI1, MKI2, MKI3, MKI4, MKI5
      PARAMETER (MKI1=600,MKI2=600,MKI3=600,MKI4=600,MKI5=600)
      INTEGER           IDL,JPSST,JPSVT, STAYIN, GOSOUT,LENGHT,
     >                  NUMERO, MATALB, VOLSUR, SIGTAL, SIGT00,  IPRT,
     >                   ISPEC,  KSPEC,  NSOUT,   NREG,  NNREG,
     >                  NUNKNO, NCOMNT,   NTRK,   NDIM,  NANGL,   NCOR,
     >                   MXSEG,  IUN,    JUN,  ISIGT,
     >                  LOPATH, SIGANG,     RI,    CIJ,  NALBG,
     >                  WSPACE,   WEIG,    FSP,   LOPT,  NSCRP,
     >                  IC,       NGSS,    IVV,   JGSS,  FFACT,  
     >                  ILONG,MATRT,ITYPE,IIVOL,IPCSCT, ISTATE(NSTATE)
      INTEGER           NPROB,N2PROB,DPROB,IPRB,DPROBX,IPRBX,ISBG,KSBG,
     >                  WORK
      INTEGER           ISSIGT,ILSIGA
      REAL              ALBEDO(6), EXTKOP(NSTATE), CUTOF, RCUTOF, ASCRP,
     >                  YGSS, XGSS(MXGAUS), WGSS(MXGAUS), WGSSX(MXGAUS),
     >                  ANG0, FACT
      LOGICAL           SWNZBC, SWVOID, LPIJK, LSKIP
      CHARACTER         CTRKT*4, COMENT*80
      REAL              WEIGHT
*
      INTEGER           JJ,MSYM,IU,IL,ITEMP,ISOUT,IIN,I,J,IBM,IOP,
     >                  INDPIJ,IJKS,NALLOC,ITRAK,IANG,NBSEG
*----
*  Variables for NXT: inline tracking
*----
      INTEGER           ILCMUP,ILCMDN
      PARAMETER        (ILCMUP=1,ILCMDN=2)
      DOUBLE PRECISION DZERO,DONE,DTWO
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
      INTEGER          IEDIMG(NSTATE),NPOINT,NBUCEL,MXMSH,MAXPIN,
     >                 MXGSUR,MXGREG,MAXMSH,NPLANE,NUCELL(3)
      CHARACTER        NAMREC*12
      INTEGER          DSV,DNSANG,IUNFLD,DANGLT,DORITR,DWGTRK,DVNOR,
     >                 DLENGT,ICINT,DCINT,IDSUR,IDREG,DCMESH,ICSINT,
     >                 DCSINT,ITPIN,DRAPIN,DGMESH,NCORT
*
      INTEGER          L1, L2, L3, L4, L5
      REAL             PAS1,XLIM1,PAS2,XLIM2,PAS3,XLIM3,
     >                 PAS4,XLIM4,PAS5,XLIM5,BI1,BI2,BI3,BI4,BI5
      COMMON /BICKL1/  BI1(0:MKI1,3),PAS1,XLIM1,L1
      COMMON /BICKL2/  BI2(0:MKI2,3),PAS2,XLIM2,L2
      COMMON /BICKL3/  BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/  BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/  BI5(0:MKI5,3),PAS5,XLIM5,L5
      DOUBLE PRECISION ABSC(3,2)
*
      INTEGER           ID(1)
      REAL              RD
      COMMON            RD(1)
      EQUIVALENCE      (RD(1),ID(1))
*----
*  INTRINSIC FUNCTION FOR POSITION IN CONDENSE PIJ MATRIX
*----
      INTEGER INDPOS
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
*----
* RECOVER EXCELL SPECIFIC TRACKING INFORMATION.
*             ALBEDO: SURFACE ALBEDOS (REAL(6))
*             KSPEC : KIND OF PIJ INTEGRATION (0:ISOTROPE,1:SPECULAR)
*             CUTOF : MFP CUTOFF FOR SPECULAR INTEGRATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      KSPEC=ISTATE(10)
      CALL LCMGET(IPTRK,'EXCELTRACKOP',EXTKOP)
      CUTOF=EXTKOP(1)
      CALL LCMGET(IPTRK,'ALBEDO',ALBEDO)
*
      IPRT  = IPRNTP
      IF( IPRT.GE.ICPEND ) WRITE(IOUT,'(1X,A72//)') TITREC
      IF(IFTRAK .NE. 0) THEN
        REWIND IFTRAK
        READ(IFTRAK) CTRKT,NCOMNT,NTRK
        IF( CTRKT .NE.'$TRK' ) GO TO 998
        IF( NCOMNT.LT.0      ) GO TO 998
        IF( NTRK  .EQ.0      ) GO TO 998
        DO 10 IC= 1,NCOMNT
           READ(IFTRAK) COMENT
   10   CONTINUE
        READ(IFTRAK) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL,MXSEG
        IF(NREG.NE.NREGIO )THEN
           CALL XABORT(NAMSBR//': TRACKING FILE HAS INVALID # OF ZONES')
        ENDIF
        NCORT=NCOR
      ELSE
        IF(ISTATE(7) .NE. 4) CALL XABORT(NAMSBR//
     >  ': Tracking file required unless NXT: tracking provided')
        NREG=ISTATE(1)
        NSOUT=ISTATE(5)
        ISPEC=ISTATE(9)
        NPOINT=ISTATE(14)
        MXSEG=ISTATE(15)
        NANGL=ISTATE(17)
        CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMUP)
        CALL LCMGET(IPTRK,'G00000001DIM',IEDIMG)
        NDIM=IEDIMG(1)
        NBUCEL=IEDIMG( 5)
        NUCELL(1)=IEDIMG(13)
        NUCELL(2)=IEDIMG(14)
        NUCELL(3)=IEDIMG(15)
        MXMSH=IEDIMG(16)
        MAXPIN=IEDIMG(19)
        MXGSUR=IEDIMG(24)
        MXGREG=IEDIMG(25)
        NPLANE=1
        NCOR=1
        NCORT=NCOR
        IF(NDIM .EQ.3) NPLANE=3
        NTRK=NANGL*NPLANE*NPOINT**(NDIM-1)
        IF(MXSEG .LE. 1) THEN
          IF(ISPEC .EQ. 0) THEN
            MXSEG=NBUCEL*
     >           ((MAXPIN+1)*(2*MXGREG+2)+MXGSUR+16)
          ELSE
            MXSEG=8*NANGL*NBUCEL*
     >            ((MAXPIN+1)*(2*MXGREG+2)+MXGSUR+16)
          ENDIF
        ENDIF
        MAXMSH=MAX(MXMSH,IEDIMG(17),IEDIMG(20))
      ENDIF
      NNREG = NREGIO*NREGIO
      NUNKNO= NREG+NSOUT+1
      CALL SETARA( RD, NUNKNO*NSBG, VOLSUR )
      IIVOL=VOLSUR+NSOUT+1
      CALL SETARA( ID, NUNKNO, MATALB )
      IF(IFTRAK .NE. 0) THEN
        READ(IFTRAK) (RD(VOLSUR+JUN),JUN=0,NUNKNO-1)
        READ(IFTRAK) (ID(MATALB+JUN),JUN=0,NUNKNO-1)
        READ(IFTRAK) ( NSCRP,JUN=1,NALBG)
        READ(IFTRAK) ( ASCRP,JUN=1,NALBG)
        READ(IFTRAK) ANG0,(ASCRP,IUN=2,NDIM),
     >               ((ASCRP,IUN=1,NDIM),JUN=2,NANGL)
        READ(IFTRAK) ( ASCRP,JUN=1,NANGL)
      ELSE
        CALL LCMGET(IPTRK,'MATALB      ',ID(MATALB))
        CALL SETARA(RD,2*NUNKNO,DSV)
        CALL LCMGET(IPTRK,'SAreaRvolume',RD(DSV))
        CALL XDRSDB(NUNKNO,RD(VOLSUR),RD(DSV),1)
        DO JJ=0,NSOUT
          RD(VOLSUR+JJ)=RD(VOLSUR+JJ)/4.0
        ENDDO
*----
*  Allocate memory for NXT tracking
*----
        CALL SETARA(RD,2*(MAXMSH+2)*4,DGMESH)
        CALL NXTXYZ(IPTRK ,IPRNTP,NDIM  ,MAXMSH,NUCELL,
     >              ABSC,RD(DGMESH))
        IF(ISPEC .EQ. 1) THEN
          CALL SETARA(RD,2*NANGL,DNSANG)
          CALL LCMGET(IPTRK,'TrackingDnsA',RD(DNSANG))
        ENDIF
        CALL SETARA(ID,2*NBUCEL,IUNFLD)
        CALL SETARA(RD,2*NDIM*NANGL,DANGLT)
        CALL SETARA(RD,2*NDIM*(NDIM+1)*NPLANE*NANGL,DORITR)
        CALL SETARA(RD,2*NANGL,DWGTRK)
        CALL SETARA(RD,2*NREG,DVNOR)
        NAMREC='G00000001CUF'
        CALL LCMGET(IPTRK,NAMREC,ID(IUNFLD))
        CALL LCMGET(IPTRK,'TrackingDirc',RD(DANGLT))
        CALL LCMGET(IPTRK,'TrackingOrig',RD(DORITR))
        CALL LCMGET(IPTRK,'TrackingWgtD',RD(DWGTRK))
        CALL LCMGET(IPTRK,'VTNormalize ',RD(DVNOR))
        CALL SETARA(RD,2*MXSEG,DLENGT)
        CALL SETARA(ID,5*2*(NBUCEL+4),ICINT)
        CALL SETARA(RD,2*2*(NBUCEL+4),DCINT)
        CALL SETARA(ID,MXGSUR,IDSUR)
        CALL SETARA(ID,MXGREG,IDREG)
        CALL SETARA(RD,2*4*(MAXMSH+2),DCMESH)
        CALL SETARA(ID,5*4*(MXGREG+4)*3,ICSINT)
        CALL SETARA(RD,2*4*(MXGREG+4)*3,DCSINT)
        CALL SETARA(ID,3*(MAXPIN),ITPIN)
        CALL SETARA(RD,6*2*(MAXPIN),DRAPIN)
      ENDIF
      CALL SETARA( RD, NUNKNO*NSBG, SIGTAL )
      CALL SETARA( RD, NUNKNO*NSBG, SIGT00 )
*
      SWNZBC= .FALSE.
      SWVOID= .FALSE.
      LPIJK= IPIJK.EQ.4
*----
*  PREPARE FOR MULTIGROUP CALCULATION
*----
      DO 2000 ISBG=1,NSBG
      IF(NPSYS(ISBG).EQ.0) GO TO 2000
      KSBG=(ISBG-1)*NUNKNO
      JUN   = 0
      IF(ISBG .GT. 1) THEN
        DO 2001 IUN= -NSOUT, NREG
           RD(VOLSUR+JUN+KSBG)= RD(VOLSUR+JUN)
           JUN= JUN+1
 2001   CONTINUE
      ENDIF
 2000 CONTINUE
      DO 2002 ISBG=1,NSBG
      IF(NPSYS(ISBG).EQ.0) GO TO 2002
      KSBG=(ISBG-1)*NUNKNO
      JUN   = 0
      DO 20 IUN= -NSOUT, NREG
         IF( IUN.LT.0 )THEN
            RD(SIGT00+JUN+KSBG)= 0.0
            RD(SIGTAL+JUN+KSBG)= ALBEDO(-ID(MATALB+JUN))
            SWNZBC= SWNZBC.OR.(RD(SIGTAL+JUN+KSBG).NE.0.0)
         ELSEIF( IUN.EQ.0 )THEN
            RD(SIGT00+JUN+KSBG)= 0.0
            RD(SIGTAL+JUN+KSBG)= 0.0
         ELSE
            RD(SIGT00+JUN+KSBG)= XSSIGT(MATCOD(IUN),ISBG)
            RD(SIGTAL+JUN+KSBG)= XSSIGT(MATCOD(IUN),ISBG)
            IF( RD(SIGTAL+JUN+KSBG).EQ.0.0 )THEN
               SWVOID= .TRUE.
            ELSE
               RD(VOLSUR+JUN+KSBG)= RD(VOLSUR+JUN+KSBG)*
     >                              RD(SIGTAL+JUN+KSBG)
            ENDIF
         ENDIF
         JUN= JUN+1
   20 CONTINUE
 2002 CONTINUE
*----
*  CHOOSE ISOTROPIC OR SPECULAR B.C.
*----
      IF( KSPEC.EQ.0 )THEN
         ISIGT = SIGT00
      ELSE
         ISIGT = SIGTAL
      ENDIF
*
      NPROB = (NUNKNO*(NUNKNO+1))/2
      N2PROB= NUNKNO*NUNKNO
      IF(IPRNTP.GT.1) THEN
         NALLOC=(2*N2PROB*NSBG)
         IF(LPIJK) NALLOC=NALLOC+(2*N2PROB*NSBG)
         WRITE(IOUT,6000) NALLOC/256
      ENDIF
      CALL SETARA( RD,2*N2PROB*NSBG,  DPROB )
      CALL XDDSET(RD(DPROB),N2PROB*NSBG,0.0D0)
      IF(LPIJK)THEN
        CALL SETARA( RD,2*N2PROB*NSBG,  DPROBX )
        CALL XDDSET(RD(DPROBX),N2PROB*NSBG,0.0D0)
      ENDIF
      IF(IPRNTP.GT.1) WRITE(IOUT,6001)
*
      IF(IPRNTP .GE. 10) THEN
        WRITE(IOUT,6010) MXSEG
      ENDIF
      CALL SETARA( ID,  MXSEG, NUMERO )
      CALL SETARA( RD,  MXSEG, LENGHT )
      IF( ISPEC.EQ.0 )THEN
*----
*  Standard tracking
*----
         IF( NDIM.EQ.2 )THEN
           CALL SETARA( RD,  MXSEG, LOPATH )
           DO ITRAK= 1, NTRK
             IF(IFTRAK .NE. 0) THEN
*----
*  Read tracks from file
*----
               READ(IFTRAK)  IANG,NBSEG,WEIGHT,
     >         (ID(NUMERO+IL),IL=0,NBSEG-1),
     >         (RD(LENGHT+IL),IL= 0,NBSEG-1)
             ELSE
*----
*  Generate selected track
*----
               CALL NXTTGS(IPTRK ,IPRNTP,NDIM  ,NANGL ,NPOINT,NTRK  ,
     >                     ITRAK ,MAXMSH,NSOUT ,NREG  ,NUCELL,NBUCEL,
     >                     MXGSUR,MXGREG,MAXPIN,MXSEG ,
     >                     ID(IUNFLD),ID(MATALB),RD(DSV)   ,
     >                     RD(DGMESH),RD(DANGLT),RD(DVNOR) ,
     >                     RD(DLENGT),ID(ICINT) ,RD(DCINT) ,
     >                     RD(DWGTRK),RD(DORITR),ID(IDSUR) ,
     >                     ID(IDREG) ,ID(ITPIN) ,RD(DCMESH),
     >                     RD(DRAPIN),ID(ICSINT),RD(DCSINT),
     >                     NBSEG ,NCORT ,WEIGHT,
     >                     ID(NUMERO),RD(LENGHT))
               IF(NCORT .EQ. 0) GO TO 1005
             ENDIF
             CALL PIJI2D(NREG,NSOUT,NBSEG,NCOR,NSBG,
     >                   SWVOID,RD(ISIGT),NPSYS,WEIGHT,
     >                   RD(LENGHT),ID(NUMERO),
     >                   RD(LOPATH),RD(DPROB),
     >                   MKI1,BI1,PAS1,XLIM1,L1,
     >                   MKI2,BI2,PAS2,XLIM2,L2,
     >                   MKI3,BI3,PAS3,XLIM3,L3)
             IF(LPIJK)THEN
               CALL PIJI2D(NREG,NSOUT,NBSEG,NCOR,NSBG,
     >                     SWVOID,RD(ISIGT),NPSYS,WEIGHT,
     >                     RD(LENGHT),ID(NUMERO),
     >                     RD(LOPATH),RD(DPROBX),
     >                     MKI3,BI3,PAS3,XLIM3,L3,
     >                     MKI4,BI4,PAS4,XLIM4,L4,
     >                     MKI5,BI5,PAS5,XLIM5,L5)
             ENDIF
 1005        CONTINUE
           ENDDO
           CALL RLSARA( RD( LOPATH ) )
         ELSE
           CALL SETARA( RD,  MXSEG, STAYIN )
           CALL SETARA( RD,  MXSEG, GOSOUT )
           DO ITRAK= 1, NTRK
             IF(IFTRAK .NE. 0) THEN
               READ(IFTRAK)  IANG,NBSEG,WEIGHT,
     >         (ID(NUMERO+IL),IL=0,NBSEG-1),
     >         (RD(LENGHT+IL),IL= 0,NBSEG-1)
             ELSE
               CALL NXTTGS(IPTRK ,IPRNTP,NDIM  ,NANGL ,NPOINT,NTRK  ,
     >                     ITRAK ,MAXMSH,NSOUT ,NREG  ,NUCELL,NBUCEL,
     >                     MXGSUR,MXGREG,MAXPIN,MXSEG ,
     >                     ID(IUNFLD),ID(MATALB),RD(DSV)   ,
     >                     RD(DGMESH),RD(DANGLT),RD(DVNOR) ,
     >                     RD(DLENGT),ID(ICINT) ,RD(DCINT) ,
     >                     RD(DWGTRK),RD(DORITR),ID(IDSUR) ,
     >                     ID(IDREG) ,ID(ITPIN) ,RD(DCMESH),
     >                     RD(DRAPIN),ID(ICSINT),RD(DCSINT),
     >                     NBSEG ,NCORT,WEIGHT,
     >                     ID(NUMERO),RD(LENGHT))
               IF(NCORT .EQ. 0) GO TO 1015
             ENDIF
             CALL PIJI3D(NREG,NSOUT,NBSEG,NCOR,NSBG,
     >                   SWVOID,RD(ISIGT),NPSYS,WEIGHT,
     >                   RD(LENGHT),ID(NUMERO),
     >                   RD(STAYIN),RD(GOSOUT),RD(DPROB))
 1015        CONTINUE
           ENDDO
           CALL RLSARA( RD( GOSOUT ) )
           CALL RLSARA( RD( STAYIN ) )
           IF(LPIJK)THEN
             CALL XABORT(NAMSBR//': 3D PIJK NOT SUPPORTED')
           ENDIF
         ENDIF
      ELSEIF( ISPEC.EQ.1 )THEN
*----
*  CYCLIC TRACKING
*----
         RCUTOF= CUTOF
         IF( NDIM.EQ.2 )THEN
            IF( ANG0.EQ.0.0 )THEN
               NGSS= NANGL/2
            ELSE
               NGSS= (NANGL+1)/2
            ENDIF
            CALL ALGPT( NGSS,0.0,1.0,XGSS,WGSS)
            CALL SETARA( RD, NGSS*NUNKNO*NSBG, SIGANG )
            CALL SETARA( RD, NGSS*MXSEG , STAYIN )
            CALL SETARA( RD, NGSS*MXSEG , GOSOUT )
            DO 110 JGSS= 1, NGSS
               YGSS= SQRT(1.0 - XGSS(JGSS)**2)
               WGSS(JGSS)= WGSS(JGSS) * YGSS
               XGSS(JGSS)= 1.0/YGSS
               WGSSX(JGSS)= WGSS(JGSS) / (XGSS(JGSS)**2)
               DO 111 ISBG=1,NSBG
               IF(NPSYS(ISBG).EQ.0) GO TO 111
               DO 112 IUN= 1,NUNKNO
                 ILSIGA=SIGANG-1+JGSS+NGSS*(IUN-1+(ISBG-1)*NUNKNO)
                 ISSIGT=ISIGT+IUN-1+(ISBG-1)*NUNKNO
                  IF( ID(MATALB+IUN-1).LE.0 )THEN
                     RD(ILSIGA)= RD(ISSIGT)
                  ELSE
                     RD(ILSIGA)= RD(ISSIGT)*XGSS(JGSS)
                  ENDIF
  112          CONTINUE
  111          CONTINUE
  110       CONTINUE
*----
*  Loop over tracks
*  then loop over groups
*----
            DO ITRAK= 1, NTRK
              IF(IFTRAK .NE. 0) THEN
                READ(IFTRAK)  IANG,NBSEG,WEIGHT,
     >          (ID(NUMERO+IL),IL=0,NBSEG-1),
     >          (RD(LENGHT+IL),IL= 0,NBSEG-1)
              ELSE
               CALL NXTTGC(IPTRK ,IPRNTP,NDIM  ,NANGL ,NPOINT,NTRK  ,
     >                     ITRAK ,MAXMSH,NSOUT ,NREG  ,NUCELL,NBUCEL,
     >                     MXGSUR,MXGREG,MAXPIN,MXSEG ,
     >                     ID(IUNFLD),ID(MATALB),RD(DSV)   ,
     >                     RD(DGMESH),RD(DANGLT),RD(DVNOR) ,
     >                     RD(DLENGT),ID(ICINT) ,RD(DCINT) ,
     >                     RD(DWGTRK),RD(DORITR),ID(IDSUR) ,
     >                     ID(IDREG) ,ID(ITPIN) ,RD(DCMESH),
     >                     RD(DRAPIN),ID(ICSINT),RD(DCSINT),
     >                     NBSEG ,NCORT,WEIGHT,
     >                     ID(NUMERO),RD(LENGHT))
                IF(NCORT .EQ. 0) GO TO 1025
              ENDIF
              CALL PIJS2D(NREG,NSOUT,NBSEG,NSBG,WEIGHT,
     >                    RCUTOF,NGSS,RD(SIGANG),XGSS,WGSS,NPSYS,
     >                    RD(LENGHT),ID(NUMERO),RD(STAYIN),
     >                    RD(GOSOUT),RD(DPROB))
              IF(LPIJK)THEN
*               X-DIRECTION  PROBABILITIES CALCULATIONS ( PX=PY )
                CALL PIJS2D(NREG,NSOUT,NBSEG,NSBG,WEIGHT,
     >                      RCUTOF,NGSS,RD(SIGANG),XGSS,WGSSX,NPSYS,
     >                      RD(LENGHT),ID(NUMERO),RD(STAYIN),
     >                      RD(GOSOUT),RD(DPROBX))
              ENDIF
 1025         CONTINUE
            ENDDO
            CALL RLSARA( RD( GOSOUT ))
            CALL RLSARA( RD( STAYIN ))
            CALL RLSARA( RD( SIGANG ))
         ELSE
            CALL SETARA( RD,  MXSEG, STAYIN )
            CALL SETARA( RD,  MXSEG, GOSOUT )
            DO ITRAK= 1, NTRK
              IF(IFTRAK .NE. 0) THEN
                READ(IFTRAK)  IANG,NBSEG,WEIGHT,
     >          (ID(NUMERO+IL),IL=0,NBSEG-1),
     >          (RD(LENGHT+IL),IL= 0,NBSEG-1)
              ELSE
               CALL NXTTGC(IPTRK ,IPRNTP,NDIM  ,NANGL ,NPOINT,NTRK  ,
     >                     ITRAK ,MAXMSH,NSOUT ,NREG  ,NUCELL,NBUCEL,
     >                     MXGSUR,MXGREG,MAXPIN,MXSEG ,
     >                     ID(IUNFLD),ID(MATALB),RD(DSV)   ,
     >                     RD(DGMESH),RD(DANGLT),RD(DVNOR) ,
     >                     RD(DLENGT),ID(ICINT) ,RD(DCINT) ,
     >                     RD(DWGTRK),RD(DORITR),ID(IDSUR) ,
     >                     ID(IDREG) ,ID(ITPIN) ,RD(DCMESH),
     >                     RD(DRAPIN),ID(ICSINT),RD(DCSINT),
     >                     NBSEG ,NCORT,WEIGHT,
     >                     ID(NUMERO),RD(LENGHT))
                IF(NBSEG .EQ. 0) GO TO 1035
              ENDIF
              CALL PIJS3D(NREG,NSOUT,NBSEG,NSBG,WEIGHT,
     >                    RCUTOF,RD(ISIGT),NPSYS,RD(LENGHT),ID(NUMERO),
     >                    RD(STAYIN),RD(GOSOUT),RD(DPROBX))
 1035         CONTINUE
            ENDDO
            CALL RLSARA( RD( GOSOUT ))
            CALL RLSARA( RD( STAYIN ))
         ENDIF
      ENDIF
      IF(IFTRAK .EQ. 0) THEN
        CALL RLSARA(RD(DRAPIN))
        CALL RLSARA(ID(ITPIN))
        CALL RLSARA(RD(DCSINT))
        CALL RLSARA(ID(ICSINT))
        CALL RLSARA(RD(DCMESH))
        CALL RLSARA(ID(IDREG))
        CALL RLSARA(ID(IDSUR))
        CALL RLSARA(RD(DCINT))
        CALL RLSARA(ID(ICINT))
        CALL RLSARA(RD(DLENGT))
        CALL RLSARA(RD(DVNOR))
        CALL RLSARA(RD(DWGTRK))
        CALL RLSARA(RD(DORITR))
        CALL RLSARA(RD(DANGLT))
        CALL RLSARA(ID(IUNFLD))
        IF(ISPEC .EQ. 1) THEN
          CALL RLSARA(RD(DNSANG))
        ENDIF
        CALL RLSARA(RD(DGMESH))
        CALL RLSARA(RD(DSV))
        CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMDN)
      ENDIF
*
      CALL SETARA( RD,  2*NPROB, WORK )
      DO 2050 ISBG=1,NSBG
         IF(NPSYS(ISBG).EQ.0) GO TO 2050
         IPRB=DPROB+(ISBG-1)*2*N2PROB
         IPRBX=DPROBX+(ISBG-1)*2*N2PROB
         KSBG=VOLSUR+(ISBG-1)*NUNKNO
         CALL PIJCMP(NREG,NSOUT,NCOR,RD(IPRB),RD(KSBG),.FALSE.,
     >               RD(IPRB),RD(WORK))
         IF(LPIJK)THEN
           CALL PIJCMP(NREG,NSOUT,NCOR,RD(IPRBX),RD(KSBG),.TRUE.,
     >                 RD(IPRBX),RD(WORK))
         ENDIF
 2050 CONTINUE
      CALL RLSARA( RD( WORK   ) )
      CALL RLSARA( RD( LENGHT ) )
      CALL RLSARA( ID( NUMERO ) )
*----
*  RENORMALIZE ALL ISOTROPIC PROBS WITH VARIOUS OPTIONS
*----
      IF( KSPEC.EQ.0 )THEN
         IF( NRENOR.EQ.1 )THEN
            CALL SETARA( RD, 2*NUNKNO, RI)
         ELSEIF( NRENOR.EQ.3 )THEN
            CALL SETARA( RD,  2*NPROB,    CIJ )
            CALL SETARA( RD,  2*NPROB, WSPACE )
            CALL SETARA( RD, 2*NUNKNO,   WEIG )
            CALL SETARA( RD, 2*NUNKNO,    FSP )
            CALL SETARA( ID,   NUNKNO,    IDL )
         ELSEIF( NRENOR .EQ. 4 )THEN
            CALL SETARA( RD, 2*NUNKNO*3, WEIG )
            CALL SETARA( RD, 2*NUNKNO, FSP )
         ENDIF
      ENDIF
      DO 2060 ISBG=1,NSBG
      IF(NPSYS(ISBG).EQ.0) GO TO 2060
      IPRB=DPROB+(ISBG-1)*2*N2PROB
      IPRBX=DPROBX+(ISBG-1)*2*N2PROB
      IIVOL=VOLSUR+NSOUT+1+(ISBG-1)*NUNKNO
      ILSIGA=SIGTAL+(ISBG-1)*NUNKNO
      IF( KSPEC.EQ.0 )THEN
         IF( NRENOR.EQ.1 )THEN
*
*           NORMALIZATION USING GELBARD SCHEME
            CALL PIJRGL( IPRT, NREG, NSOUT, RD(ILSIGA), RD(IPRB),
     >                   RD(RI) )
            IF(LPIJK)CALL PIJRGL( IPRT, NREG, NSOUT, RD(ILSIGA),
     >                            RD(IPRBX), RD(RI) )
         ELSEIF( NRENOR.EQ.2 )THEN
*
*           NORMALIZATION WORKING ON DIAGONAL COEFFICIENTS
            CALL PIJRDG( IPRT, NREG, NSOUT, RD(ILSIGA), RD(IPRB) )
            IF(LPIJK)CALL PIJRDG( IPRT, NREG, NSOUT, RD(ILSIGA),
     >                            RD(IPRBX) )
         ELSEIF( NRENOR.EQ.3 )THEN
*
*           NORMALIZATION WORKING ON WEIGHT FACTORS TO KEEP DIAG = 0.0
            CALL PIJRNL( IPRT,  NREG,   NSOUT, RD(ILSIGA), RD(IPRB),
     >                   RD(CIJ), RD(WSPACE), RD(FSP),
     >                   ID(IDL), RD(WEIG) )
            IF(LPIJK)CALL PIJRNL( IPRT,  NREG,   NSOUT, RD(ILSIGA),
     >                            RD(IPRBX), RD(CIJ), RD(WSPACE),
     >                            RD(FSP), ID(IDL), RD(WEIG) )
         ELSEIF( NRENOR .EQ. 4 )THEN  ! ATTENTION
*
*           NORMALIZATION WORKING ON WEIGHT FACTORS ADDITIVE (HELIOS)
            CALL PIJRHL( IPRT, NREG, NSOUT, RD(ILSIGA), RD(IPRB),
     >                RD(FSP), RD(WEIG) )
            IF(LPIJK)CALL PIJRHL( IPRT,  NREG,   NSOUT, RD(ILSIGA),
     >                            RD(IPRBX), RD(FSP), RD(WEIG) )
         ENDIF
         IF( IPRT.GE.ICPALL )THEN
            LOPT= -1
            MSYM=1
            WRITE(IOUT,'(1H )')
            WRITE(IOUT,'(35H   COLLISION PROBABILITIES OUTPUT: ,
     >                   35H *BEFORE* ALBEDO REDUCTION          )')
            CALL PIJWPR(LOPT,NREG,NSOUT,RD(ILSIGA),RD(IPRB),
     >                  RD(IIVOL),MSYM)
*
            IF(LPIJK)THEN
              WRITE(IOUT,'(35H   X-DIRECT. COLL. PROBAB. OUTPUT: ,
     >                     35H *BEFORE* ALBEDO REDUCTION          )')
              CALL PIJWPR(LOPT,NREG,NSOUT,RD(ILSIGA),RD(IPRBX),
     >                    RD(IIVOL),MSYM)
            ENDIF
*
         ENDIF
      ENDIF
 2060 CONTINUE
      IF( KSPEC.EQ.0 )THEN
         IF( NRENOR.EQ.1 )THEN
            CALL RLSARA( RD(    RI ))
         ELSEIF( NRENOR.EQ.3 )THEN
            CALL RLSARA( ID(    IDL ))
            CALL RLSARA( RD(    FSP ))
            CALL RLSARA( RD(   WEIG ))
            CALL RLSARA( RD( WSPACE ))
            CALL RLSARA( RD(    CIJ ))
         ELSEIF( NRENOR .EQ. 4 )THEN
            CALL RLSARA( RD(    FSP ))
            CALL RLSARA( RD(   WEIG ))
         ENDIF
      ENDIF
      IF(LPIJK)THEN
         DO 2070 ISBG=1,NSBG
         IF(NPSYS(ISBG).EQ.0) GO TO 2070
         IPRBX=DPROBX+(ISBG-1)*2*N2PROB
         CALL PIJD2S(NREG,NSOUT,RD(IPRBX),PROBKS(1,ISBG))
 2070    CONTINUE
      ENDIF
      IF( KSPEC.EQ.0 )THEN
*----
*  ELIMINATION OF SURFACES FOR PIJ
*----
         IF( SWNZBC )THEN
            CALL SETARA( RD, 2*NSOUT*NSOUT , JPSST )
            CALL SETARA( RD,  2*NSOUT*NREG , JPSVT )
            CALL SETARA( ID,         NSOUT ,   IDL )
            CALL SETARA( ID,         NSOUT , MATRT )
            CALL LCMLEN(IPTRK,'BC-REFL+TRAN',ILONG,ITYPE)
            IF(ILONG.EQ.NSOUT) THEN
              CALL LCMGET(IPTRK,'BC-REFL+TRAN',ID(MATRT))
            ELSE
               WRITE(IOUT,9000) NAMSBR
               DO 130 ISOUT=0,NSOUT-1
                 ID(MATRT+ISOUT)=ISOUT+1
 130           CONTINUE
            ENDIF
            DO 2080 ISBG=1,NSBG
              IF(NPSYS(ISBG).EQ.0) GO TO 2080
              IPRB=DPROB+(ISBG-1)*2*N2PROB
              IPRBX=DPROBX+(ISBG-1)*2*N2PROB
              ILSIGA=SIGTAL+(ISBG-1)*NUNKNO
              CALL PIJABC(NREG,NSOUT,NPROB,RD(ILSIGA),ID(MATRT),
     >                    RD(IPRB),ID(IDL),RD(JPSST),RD(JPSVT))
*----
*  ELIMINATION OF SURFACES FOR PIJX AND CREATION OF PIJXX
*----
            IF(LPIJK)THEN
               CALL PIJAAA(NREG,NSOUT,RD(ILSIGA),RD(IPRBX),
     >                     RD(JPSVT),PROBKS(1,ISBG))
               CALL PIJABC(NREG,NSOUT,NPROB,RD(ILSIGA),ID(MATRT),
     >                     RD(IPRBX),ID(IDL),RD(JPSST),RD(JPSVT))
            ENDIF
 2080     CONTINUE
*
            CALL RLSARA( ID( MATRT ))
            CALL RLSARA( ID(   IDL ))
            CALL RLSARA( RD( JPSVT ))
            CALL RLSARA( RD( JPSST ))
         ENDIF
      ENDIF
      IF(LPIJK) CALL SETARA(RD,4*NNREG,ITEMP)
*
      CALL SETARA( RD, NREG,   FFACT )
      DO 2090 ISBG=1,NSBG
      IF(NPSYS(ISBG).EQ.0) GO TO 2090
      IPRB=DPROB+(ISBG-1)*2*N2PROB
      IPRBX=DPROBX+(ISBG-1)*2*N2PROB
      IIVOL=VOLSUR+NSOUT+1+(ISBG-1)*NUNKNO
      ILSIGA=SIGTAL+(ISBG-1)*NUNKNO
      IF( IPRT.GE.ICPEND )THEN
         LOPT= +1
         MSYM=1
         WRITE(IOUT,'(1H )')
         WRITE(IOUT,'(35H   COLLISION PROBABILITIES OUTPUT: ,
     >                35H *AFTER* ALBEDO REDUCTION          )')
         CALL PIJWPR( LOPT,NREG,NSOUT,RD(ILSIGA),RD(IPRB),
     >                RD(IIVOL),MSYM)
*
         IF(LPIJK)THEN
           WRITE(IOUT,'(35H   X-DIRECT. COLL. PROBAB. OUTPUT: ,
     >                  35H *AFTER* ALBEDO REDUCTION          )')
           CALL PIJWPR( LOPT,NREG,NSOUT,RD(ILSIGA),RD(IPRBX),
     >                  RD(IIVOL),MSYM)
           WRITE(IOUT,'(35H0 X-DIRECT. COLL. PROBAB." OUTPUT: ,
     >                  35H PIJX"=PIJX+PISX*(1/(1-PSS))*PSJ   )')
           MSYM=0
           CALL PIJWPR( LOPT,NREG,NSOUT,RD(ILSIGA),PROBKS(1,ISBG),
     >                  RD(IIVOL),MSYM)
         ENDIF
*
      ENDIF
*----
*  CHARGE PIJ MATRIX IN THE DRAGON SYMMETRIZED FORMAT
*----
      DO 160 IIN=1,NREG
         IF(RD(ILSIGA+NSOUT+IIN).EQ.0.0) THEN
            RD(FFACT+IIN-1)=1.0
         ELSE
            RD(FFACT+IIN-1)=1.0/RD(ILSIGA+NSOUT+IIN)
         ENDIF
  160 CONTINUE
      CALL PIJD2R(NREG,NSOUT,RD(IPRB),RD(FFACT),.FALSE.,NELPIJ,
     >            PIJ(1,1,ISBG))
*----
*  CHARGE PIJX AND PIJY MATRICES IN THE DRAGON SYMMETRIZED FORMAT
*  ( PIJX=PIJY ), AND PIJZ CALCULATION ( PIJZ=3*PIJ-PIJX-PIJY )
*  AND THE SAME FOR FULL MATRICES OF PIJX", PIJY" AND PIJZ"
*----
      IF(LPIJK)THEN
        CALL PIJD2R(NREG,NSOUT,RD(IPRBX),RD(FFACT),.TRUE.,NELPIJ,
     >              PIJ(1,2,ISBG))
        IVV=0
        DO 181 IUN=1,NREG
          IU=IUN
          IL=(IUN-1)*NREG+1
          DO 191 JUN=1,IUN
            IVV=IVV+1
            PROBKS(IL,ISBG)=1.5*PROBKS(IL,ISBG)*RD(FFACT+IUN-1)*
     >                                          RD(FFACT+JUN-1)
            IF(IL.NE.IU)PROBKS(IU,ISBG)=1.5*PROBKS(IU,ISBG)*
     >                      RD(FFACT+IUN-1)*RD(FFACT+JUN-1)
            PIJ(IVV,3,ISBG)=PIJ(IVV,2,ISBG)
            PROBKS(NNREG+IL,ISBG)=PROBKS(IL,ISBG)
            PROBKS(NNREG+IU,ISBG)=PROBKS(IU,ISBG)
            PIJ(IVV,4,ISBG)=3*PIJ(IVV,1,ISBG)-PIJ(IVV,2,ISBG)
     >                                       -PIJ(IVV,3,ISBG)
            PROBKS(2*NNREG+IL,ISBG)=3*PIJ(IVV,1,ISBG)
     >      -PROBKS(IL,ISBG)-PROBKS(NNREG+IL,ISBG)
            PROBKS(2*NNREG+IU,ISBG)=3*PIJ(IVV,1,ISBG)
     >      -PROBKS(IU,ISBG)-PROBKS(NNREG+IU,ISBG)
            IU=IUN+JUN*NREG
            IL=IL+1
  191     CONTINUE
  181   CONTINUE
*----
*  COMPUTE PIJ**(-1)*PIJK*
*----
        CALL PIJKST(IPRNTP,NREGIO,PIJ(1,1,ISBG),
     >              PROBKS(1,ISBG),RD(ITEMP))
      ENDIF
 2090 CONTINUE
      CALL RLSARA( RD( FFACT ) )
      IF(LPIJK)THEN
        CALL RLSARA(RD(ITEMP))
        CALL RLSARA( RD( DPROBX) )
      ENDIF
*
      CALL RLSARA( RD( DPROB  ) )
      CALL RLSARA( RD( SIGT00 ) )
      CALL RLSARA( RD( SIGTAL ) )
      CALL RLSARA( ID( MATALB ) )
      CALL RLSARA( RD( VOLSUR ) )
*----
*  CHECK IF SCATTERING REDUCTION IS REQUIRED
*----
      CALL SETARA(RD,4*NREGIO*NREGIO,IPCSCT)
      DO 3000 ISBG=1,NSBG
      IF(NPSYS(ISBG).EQ.0) GO TO 3000
      LSKIP=.TRUE.
      DO 200 IBM=1,NBMIX
        LSKIP=LSKIP.AND.(XSSIGW(IBM,1,ISBG).EQ.0.0)
  200 CONTINUE
*----
*  COMPUTE THE SCATTERING-REDUCED CP MATRICES
*----
      IOP=1
      IF(.NOT.LSKIP) THEN
        CALL PIJSMD(IPRNTP,NBMIX,NREGIO,MATCOD,VOLUME,XSSIGW(0,1,ISBG),
     >              XSSIGT(0,ISBG),LEAKSW,PIJ(1,1,ISBG),RD(IPCSCT),IOP)
        CALL PIJSDB(NREGIO*NREGIO,RD(IPCSCT),RD(IPCSCT),1)
        DO 220 I=1,NREGIO
          FACT=VOLUME(I)
          DO 210 J=1,NREGIO
            INDPIJ=INDPOS(I,J)
            PIJ(INDPIJ,1,ISBG)=RD(IPCSCT+(J-1)*NREGIO+I-1)*FACT
  210     CONTINUE
  220   CONTINUE
      ENDIF
*-------
      IF(IPIJK.EQ.4) THEN
        IOP=4
        IF(.NOT.LSKIP) THEN
*         P1 SCATTERING REDUCTION OF THE DIRECTIONNAL CP MATRICES.
          IF(NANI.LT.2) CALL XABORT('EXCELP: ANISOTROPIC SCAT MISSING.')
          DO 250 IJKS=1,3
            CALL PIJSMD(IPRNTP,NBMIX,NREGIO,MATCOD,VOLUME,
     >                  XSSIGW(0,2,ISBG),XSSIGT(0,ISBG),LEAKSW,
     >                  PIJ(1,IJKS+1,ISBG),RD(IPCSCT),IOP)
            CALL PIJSDB(NREGIO*NREGIO,RD(IPCSCT),RD(IPCSCT),1)
            DO 240 I=1,NREGIO
              FACT=VOLUME(I)
              DO 230 J=1,NREGIO
                INDPIJ=INDPOS(I,J)
                PIJ(INDPIJ,IJKS+1,ISBG)=RD(IPCSCT+(J-1)*NREGIO+I-1)*FACT
  230         CONTINUE
  240       CONTINUE
  250     CONTINUE
        ENDIF
      ENDIF
 3000 CONTINUE
      CALL RLSARA( RD(IPCSCT) )
      RETURN
*
 6010 FORMAT(' Maximum length of a line =',I10)
  998 WRITE(IOUT,'(30H ECHO = FIRST RECORD DESTROYED      )')
      CALL XABORT(NAMSBR//': PROBLEM WITH THE TRACKING FILE')
 6000 FORMAT(' *** SPACE REQUIRED FOR CP MATRICES = ',I10,' K ***')
 6001 FORMAT(' *** CP MATRICES ALLOCATED            ',10X,'   ***')
 9000 FORMAT(1X,A6,': *** WARNING *** '/
     >       ' REFLECTION/TRANSMISSION MATRIX MISSING'/
     >       ' USE IDENTITY REFLECTION MATRIX')
      END
