*DECK FLU2DR
      SUBROUTINE FLU2DR(IPRT,IPMACR,IPFLUX,IPSYS,IPTRK,IPFLUP,IPGPT,
     1 IGPT,IFTRAK,CXDOOR,TITLE,NUNKNO,NREG,NSOUT,NANIS,NLF,NFUNL,NGRP,
     2 NMAT,NIFIS,LFORW,LEAKSW,MAXINR,EPSINR,MAXOUT,EPSUNK,EPSOUT,NCPTL,
     3 NCPTA,ITYPEC,IPHASE,ITPIJ,ILEAK,OPTION,REFKEF,MATCOD,KEYFLX,VOL,
     4 XSTOT,XSTRC,XSDIA,XSNUF,XSCHI,LREBAL,DIFF,FLUX,IJJ,NJJ,IPOS,
     5 XSCAT,XCSOU,GAMMA,NPSYS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* fixed source problem or inverse power method for K-effective or
* buckling iteration. Perform thermal iterations.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Roy
*
*Parameters: input
* IPRT    print flag.
* IPMACR  pointer to the macrolib LCM object.
* IPFLUX  pointer to the flux LCM object.
* IPSYS   pointer to the system LCM object.
* IPTRK   pointer to the tracking LCM object.
* IPFLUP  pointer to the unperturbed flux LCM object (if ITYPEC=1).
* IPGPT   pointer to the GPT fixed source LCM object (if ITYPEC=1).
* IGPT    index of the fixed source eigenvalue problem to solve.
* IFTRAK  tracking file unit number.
* CXDOOR  name of the flux solution door.
* TITLE   title.
* NUNKNO  number of unknowns per energy group including spherical
*         harmonic terms, interface currents and fundamental
*         currents.
* NREG    number of regions.
* NSOUT   number of outer surfaces.
* NANIS   maximum cross section Legendre order.
* NLF     number of Legendre orders for the flux.
* NFUNL   second dimension of matrix KEYFLX.
* NGRP    number of energy groups.
* NMAT    number of mixtures.
* NIFIS   number of fissile isotopes.
* LFORW   flag set to .false. to solve an adjoint problem.
* LEAKSW  leakage flag (=.true. if leakage is present on the outer
*         surface).
* MAXINR  maximum number of thermal iterations.
* EPSINR  thermal iterations epsilon.
* MAXOUT  maximum number of outer iterations.
* EPSUNK  outer iterations eigenvector epsilon.
* EPSOUT  outer iterations eigenvalue epsilon.
* NCPTL   number of free iterations in an acceleration cycle.
* NCPTA   number of accelerated iterations in an acceleration cycle.
* ITYPEC  type of flux evaluation:
*         =-1: skip the flux calculation;
*         =0: fixed sources;
*         =1: fixed source eigenvalue problem (GPT type).
*         =2: fission sources/k effective convergence;
*         =3: fission sources/k effective convergence/
*             db2 buckling evaluation;
*         =4: fission sources/db2 buckling convergence;
*         =5: b2 sources/db2 buckling convergence.
* IPHASE  type of flux solution door (1 for asm 2 for pij).
* ITPIJ   type of cp available:
*         =1: scatt mod pij (wij)
*         =2: stand. pij
*         =3: scatt mod pij+pijk (wij,wijk)
*         =4: stand. pij+pijk.
* ILEAK   method used to include DB2 effect
*         =1: the scattering modified cp matrix is multiplied by PNLR;
*         =2: the reduced cp matrix is multiplied by PNL;
*         =3: sigs0-db2 approximation;
*         =4: albedo approximation;
*         =5: Ecco-type isotropic streaming model
*         >5: Tibere-type anisotropic streaming model.
* OPTION  type of leakage coefficients; can be 'LKRD' (imposed leakage
*         coefficients), 'B0' (B-0), 'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
*         or 'B0TR' (B-0 with transport correction).
* REFKEF  target effective multiplication factor.
* MATCOD  mixture indices.
* KEYFLX  index of L-th order flux components in unknown vector.
* VOL     volumes.
* XSTOT   non-transport-corrected macroscopic total cross sections.
* XSTRC   transport-corrected macroscopic total cross sections.
* XSDIA   transport-corrected macroscopic within-group scattering cross
*         sections.
* XSNUF   nu*macroscopic fission cross sections or fixed sources if
*         ITYPEC=0.
* XSCHI   fission spectrum.
* LREBAL  thermal iteration rebalancing flag (=.true. if thermal
*         rebalancing required).
*
*Parameters: scratch
* DIFF    homogeneous leakage coefficients.
* FLUX    iteration flux:
*         FLUX(:,:,1) <=old     outer;
*         FLUX(:,:,2) <=present outer;
*         FLUX(:,:,3) <=new     outer;
*         FLUX(:,:,4) <=source  outer;
*         FLUX(:,:,5) <=old     inner;
*         FLUX(:,:,6) <=present inner;
*         FLUX(:,:,7) <=new     inner;
*         FLUX(:,:,8) <=source  inner.
* IJJ
* NJJ
* IPOS
* XSCAT
* XCSOU
* GAMMA
* NPSYS
*
*-----------------------------------------------------------------------
*
*----
*  INTERNAL PARAMETERS:
*   SYBILF : SYBIL FLUX SOLUTION DOOR                 EXT ROUTINE
*   TRFICF : DEFAULT CP FLUX SOLUTION DOOR            EXT ROUTINE
*   BIVAF  : DEFAULT 2D DIFFUSION FLUX SOLUTION DOOR  EXT ROUTINE
*   TRIVAF : DEFAULT 3D DIFFUSION FLUX SOLUTION DOOR  EXT ROUTINE
*   PNF    : DEFAULT PN/SPN FLUX SOLUTION DOOR        EXT ROUTINE
*   SNF    : DEFAULT SN FLUX SOLUTION DOOR            EXT ROUTINE
*----
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPRT,IPMACR,IPFLUX,IPSYS,IPTRK,IPFLUP,IPGPT,IGPT,IFTRAK,
     1 NUNKNO,NREG,NSOUT,NANIS,NLF,NFUNL,NGRP,NMAT,NIFIS,MAXINR,MAXOUT,
     2 NCPTL,NCPTA,ITYPEC,IPHASE,ITPIJ,ILEAK,MATCOD(NREG),
     3 KEYFLX(NREG,NFUNL),IJJ(0:NMAT),NJJ(0:NMAT),IPOS(0:NMAT),
     4 NPSYS(NGRP)
      REAL EPSINR,EPSUNK,EPSOUT,VOL(NREG),XSTOT(0:NMAT,NGRP),
     1 XSTRC(0:NMAT,NGRP),XSDIA(0:NMAT,0:NANIS,NGRP),
     2 XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),DIFF(NGRP),
     3 FLUX(NUNKNO,NGRP,8),XSCAT(0:NMAT*NGRP),GAMMA(NGRP),B2(4)
      CHARACTER CXDOOR*12,TITLE*72,OPTION*4
      LOGICAL LFORW,LEAKSW,LREBAL,CFLI,CEXE
      DOUBLE PRECISION REFKEF,XCSOU(NGRP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER JPAR(NSTATE)
      CHARACTER CAN(0:9)*2,MESSIN*5,MESSOU*5,TEXT12*12,HTYPE(0:5)*4
      INTEGER INDD(3)
      DOUBLE PRECISION AKEEP(8),FISOUR,OLDBIL,AKEFF,AKEFFO,AFLNOR,
     1 BFLNOR,DDELN1,DDELD1
      LOGICAL LSCAL,LEXAC
      REAL ALBEDO(6)
*----
*  SETARA POINTERS
*----
      INTEGER REBAL,XCTOT,FLUN,SIGT0,SIGS0,QTOTL,KEYCUR,IV,MATALB
*
      INTEGER     I(1)
      COMMON      A(1)
      EQUIVALENCE (A(1),I(1))
*
      EXTERNAL TRFICF,SYBILF,BIVAF,TRIVAF,PNF,SNF
      LOGICAL REBFLG
*
      SAVE CAN,HTYPE
      DATA CAN /'00','01','02','03','04','05','06','07','08','09'/
      DATA (HTYPE(JJ),JJ=0,5)/'S   ','P   ',2*'K   ','B   ','L   '/
*
      REBFLG=.TRUE.
      IPREB=IPMACR
*
      ICHAR=0
      ICTOT=0
*----
*  RECOVER INDEX FOR THE CURRENTS IN FLUX, NUMERICAL SURFACES,
*  ALBEDO IF NEEDED BY THE REBALANCING.
*  RECOVER THE SPATIAL APPROXIMATION ORDER (IELEM).
*----
      IELEM=1
      ICREB=0
      IF(CXDOOR.EQ.'MCCG') THEN
         CALL LCMLEN(IPTRK,'KEYCUR$MCCG',ICREB,ITYLCM)
         IF (ICREB.GT.0) THEN
            CALL LCMLEN(IPTRK,'NZON$MCCG',ILONG,ITYLCM)
            NNN=ILONG-ICREB
            CALL SETARA(I,NSOUT,KEYCUR)
            CALL SETARA(A,ILONG,IV)
            CALL SETARA(I,ILONG,MATALB)
            CALL LCMGET(IPTRK,'KEYCUR$MCCG',I(KEYCUR))
            CALL LCMGET(IPTRK,'V$MCCG',A(IV))
            CALL LCMGET(IPTRK,'ALBEDO',ALBEDO)
            CALL LCMGET(IPTRK,'NZON$MCCG',I(MATALB))
         ENDIF
      ELSE IF(CXDOOR.EQ.'SN') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
         ITYPE=JPAR(6)
         IELEM=JPAR(8)
         IF(ITYPE.EQ.5) IELEM=IELEM**2
         IF(ITYPE.EQ.7) IELEM=IELEM**3
      ELSE IF((CXDOOR.EQ.'BIVAC').OR.(CXDOOR.EQ.'TRIVAC')) THEN
         IF(IELEM.GT.1) CALL XABORT('FLU2DR: ONLY IELEM=1 AVAILABLE.')
      ENDIF
*----
*  SELECT THE CALCULATION DOORS FOR WHICH A GROUP-BY-GROUP SCALAR
*  PROCEDURE WILL BE USED. A VECTORIAL APPROACH WILL BE USED WITH
*  OTHER DOORS.
*----
      LSCAL=(CXDOOR.EQ.'TRAFIC').OR.(CXDOOR.EQ.'SYBIL').OR.
     > (CXDOOR.EQ.'BIVAC').OR.(CXDOOR.EQ.'TRIVAC').OR.(CXDOOR.EQ.'SN')
*
      CALL KDRCPU(CPU0)
      IF(ILEAK.LT.5) THEN
        INORM=1
      ELSE IF(ILEAK.EQ.5) THEN
        INORM=2
      ELSE IF(ILEAK.GT.5) THEN
        INORM=3
      ENDIF
      LEXAC=.FALSE.
      AKEEP(5)=1.0D0
      AKEEP(6)=1.0D0
      AKEEP(7)=1.0D0
      CALL XDRSET(DIFF,NGRP,0.0)
      CALL XDRSET(GAMMA,NGRP,1.0)
*----
*  EXTERNAL FLUX(:,:,2) INITIALISATION AND FIXED-EXTERNAL SOURCE IN
*  FLUX(:,:,4)
*----
      IF(ITYPEC.GE.3) THEN
         CALL LCMGET(IPFLUX,'B2  B1HOM',B2(4))
         IF(ILEAK.GE.6) CALL LCMGET(IPFLUX,'B2  HETE',B2)
      ELSE
         CALL XDRSET(B2,4,0.0)
      ENDIF
      AKEFFO=0.0D0
      IF(LFORW) THEN
         IF(ITYPEC.EQ.1) THEN
           J1=LCMGID(IPGPT,'DSOUR')
           JPGPT=LCMGIL(J1,IGPT)
           J1=LCMGID(IPFLUX,'DFLUX')
           JPFLUX=LCMGIL(J1,IGPT)
         ELSE
           JPFLUX=LCMGID(IPFLUX,'FLUX')
         ENDIF
      ELSE
         IF(ITYPEC.EQ.1) THEN
           J1=LCMGID(IPGPT,'ASOUR')
           JPGPT=LCMGIL(J1,IGPT)
           J1=LCMGID(IPFLUX,'ADFLUX')
           JPFLUX=LCMGIL(J1,IGPT)
         ELSE
           JPFLUX=LCMGID(IPFLUX,'AFLUX')
         ENDIF
      ENDIF
      DO 20 IG=1,NGRP
      CALL XDRSET(FLUX(1,IG,2),NUNKNO,0.0)
      CALL XDRSET(FLUX(1,IG,4),NUNKNO,0.0)
      XCSOU(IG)=0.0D0
      IF(LFORW) THEN
         CALL LCMGDL(JPFLUX,IG,FLUX(1,IG,2))
      ELSE
         CALL LCMGDL(JPFLUX,NGRP-IG+1,FLUX(1,IG,2))
      ENDIF
      IF(ITYPEC.EQ.0) THEN
         DO 10 IR=1,NREG
         IND=KEYFLX(IR,1)
         IF(IND.EQ.0) GO TO 10
         FLUX(IND,IG,4)=XSNUF(MATCOD(IR),1,IG)
         XCSOU(IG)=XCSOU(IG)+FLUX(IND,IG,4)*VOL(IR)
   10    CONTINUE
      ELSE IF(ITYPEC.EQ.1) THEN
         IF(LFORW) THEN
            CALL LCMGDL(JPGPT,IG,FLUX(1,IG,4))
         ELSE
            CALL LCMGDL(JPGPT,NGRP-IG+1,FLUX(1,IG,4))
         ENDIF
         DO 15 IR=1,NREG
         IND=KEYFLX(IR,1)
         IF(IND.GT.0) XCSOU(IG)=XCSOU(IG)+FLUX(IND,IG,4)*VOL(IR)
   15    CONTINUE
      ENDIF
   20 CONTINUE
*----
*  COMPUTE FIRST K-EFFECTIVE
*----
      IF((ITYPEC.EQ.0).OR.(ITYPEC.EQ.5)) THEN
         AKEFFO=1.0D0
         AKEFF=1.0D0
         AFLNOR=1.0D0
      ELSE IF(ITYPEC.EQ.1) THEN
         CALL LCMGET(IPFLUP,'STATE-VECTOR',JPAR)
         IF(JPAR(6).GE.2) THEN
            CALL LCMGET(IPFLUP,'K-EFFECTIVE',RKEFF)
            CALL LCMGET(IPFLUP,'K-INFINITY',CUREIN)
         ENDIF
         AKEFF=RKEFF
         IF(JPAR(6).GE.3) THEN
            CALL XDRSET(B2,4,0.0)
            CALL LCMGET(IPFLUP,'B2  B1HOM',B2(4))
            CALL LCMGET(IPFLUP,'DIFFB1HOM',DIFF)
         ENDIF
         IF((JPAR(6).GT.2).AND.(JPAR(7).GE.6)) THEN
            CALL LCMGET(IPFLUP,'B2  HETE',B2)
         ENDIF
         IF((JPAR(6).GT.2).AND.(JPAR(7).GE.5)) THEN
            CALL LCMGET(IPFLUP,'GAMMA',GAMMA)
         ENDIF
         AKEFFO=AKEFF
         AKEEP(2)=AKEFF
         AFLNOR=1.0D0/RKEFF
      ELSE
         OLDBIL=0.0D0
         CALL FLUKEF(IPRT,IPMACR,NGRP,NREG,NUNKNO,NANIS,NMAT,NIFIS,
     1   MATCOD(1),VOL,KEYFLX(1,1),XSTOT,XSNUF,XSCHI,DIFF,FLUX(1,1,2),
     2   B2,ILEAK,LEAKSW,OLDBIL,AKEFF,AFLNOR,IJJ,NJJ,IPOS,XSCAT)
         AKEFFO=AKEFF
         AKEEP(2)=AKEFF
      ENDIF
      B2VALO=B2(4)
*
      NCTOT=NCPTA+NCPTL
      IF(NCPTA.EQ.0) THEN
         NCPTM=NCTOT+1
      ELSE
         NCPTM=NCPTA
      ENDIF
      MESSOU='     '
      IF(IPRT.GT.0) WRITE(6,1090) 0,1.0,EPSOUT,AKEFFO,B2(4)
*----
*  CALCULATION OF THE INITIAL LEAKAGE COEFFICIENTS
*----
      IF(ITYPEC.GT.2) THEN
         IF(LEAKSW)
     >   CALL XABORT('FLU2DR: THE ALBEDO SHOULD BE EQUAL TO 1.')
         CALL XDRSET(DIFF,NGRP,0.0)
         IF(OPTION.EQ.'LKRD') THEN
            CALL LCMGET(IPMACR,'DIFFB1HOM',DIFF)
            CALL XDRSET(GAMMA,NGRP,1.0)
         ELSE
            CALL SETARA(A,NGRP,XCTOT)
            CALL SETARA(A,2*NGRP,FLUN)
            CALL B1HOM(IPMACR,NUNKNO,OPTION,'DIFF',NGRP,NREG,NMAT,
     1      NIFIS,VOL,MATCOD,KEYFLX(1,1),FLUX(1,1,2),REFKEF,0,DIFF,
     2      GAMMA,AKEFF,A(XCTOT),A(FLUN),XSCAT,IJJ,NJJ,IPOS,INORM,B2)
            CALL RLSARA(A(FLUN))
            CALL RLSARA(A(XCTOT))
            CALL LCMPUT(IPFLUX,'B2  B1HOM',1,2,B2(4))
         ENDIF
         CALL LCMPUT(IPFLUX,'DIFFB1HOM',NGRP,2,DIFF)
      ENDIF
*
****  OUTER LOOP  ******************************************
      IGDEB=1
      CFLI=.FALSE.
      CEXE=.FALSE.
      DO 400 IT=1,MAXOUT
      CALL KDRCPU(CPU1)
      MESSIN='     '
*----
*  FISSION SOURCE CALCULATION IN FLUX(:,:,4)
*----
      IF(ITYPEC.GT.0) THEN
         IF(ITYPEC.EQ.1) THEN
            DO 24 IG=1,NGRP
            IF(LFORW) THEN
               CALL LCMGDL(JPGPT,IG,FLUX(1,IG,4))
            ELSE
               CALL LCMGDL(JPGPT,NGRP-IG+1,FLUX(1,IG,4))
            ENDIF
            DO 23 IUN=1,NUNKNO
   23       FLUX(IUN,IG,4)=-FLUX(IUN,IG,4)
            DO 24 IR=1,NREG
            IUN=KEYFLX(IR,1)
            IF(IUN.GT.0) FLUX(IUN,IG,4)=FLUX(IUN,IG,4)/VOL(IR)
   24       CONTINUE
         ELSE
            CALL XDRSET(FLUX(1,1,4),NUNKNO*NGRP,0.0)
         ENDIF
         DO 35 IR=1,NREG
         IBM=MATCOD(IR)
         IF(IBM.GT.0) THEN
            DO 30 IEL=0,IELEM-1
            IND=KEYFLX(IR,1)+IEL
            DO 30 IS=1,NIFIS
            FISOUR=0.0D0
            DO 25 IG=1,NGRP
   25       FISOUR=FISOUR+XSNUF(IBM,IS,IG)*FLUX(IND,IG,2)
            FISOUR=FISOUR*AFLNOR
            DO 30 IG=1,NGRP
   30       FLUX(IND,IG,4)=FLUX(IND,IG,4)+XSCHI(IBM,IS,IG)*FISOUR
         ENDIF
   35    CONTINUE
         DO 40 IG=1,NGRP
         XCSOU(IG)=0.0D0
         DO 40 IR=1,NREG
         IND=KEYFLX(IR,1)
         IF(IND.GT.0) XCSOU(IG)=XCSOU(IG)+FLUX(IND,IG,4)*VOL(IR)
   40    CONTINUE
      ENDIF
*----
*  SET THE STARTING ENERGY GROUP
*----
      DO 45 IG=1,NGRP
      IGDEB=IG
      IF(XCSOU(IG).NE.0.0) GO TO 46
   45 CONTINUE
*----
*  DOWNLOAD FROM EXTERNAL FLUX(:,:,2) TO PRESENT INTERNAL FLUX(:,:,6)
*----
   46 DO 50 IG=1,NGRP
      DO 50 IND=1,NUNKNO
   50 FLUX(IND,IG,6)=FLUX(IND,IG,2)
*
****  INNER LOOP  ******************************************
      DO 270 JT=1,MAXINR
      JPMACR=LCMGID(IPMACR,'GROUP')
      DO 140 IG=IGDEB,NGRP
      DO 55 IND=1,NUNKNO
   55 FLUX(IND,IG,8)=FLUX(IND,IG,4)
*----
*  PROCESS SELF-SCATTERING REDUCTION IN INNER SOURCES.
*----
      IF((ITPIJ.EQ.2).OR.(ITPIJ.EQ.4)) THEN
         IF((CXDOOR.EQ.'BIVAC').OR.(CXDOOR.EQ.'TRIVAC')) THEN
            CALL XABORT('FLU2DR: SCATTERING REDUCTION IS MANDATORY.')
         ENDIF
         DO 60 IR=1,NREG
         IBM=MATCOD(IR)
         DO 60 IAL=0,MIN(NLF-1,NANIS)
         XXS=XSDIA(IBM,IAL,IG)*REAL(2*IAL+1)
         DO 60 IAM=0,IAL
         IND=KEYFLX(IR,1+IAL*(IAL+1)/2+IAM)
         IF(IND.GT.0) FLUX(IND,IG,8)=FLUX(IND,IG,8)+XXS*FLUX(IND,IG,6)
   60    CONTINUE
      ENDIF
      IF(ILEAK.EQ.5) THEN
*        ECCO ISOTROPIC STREAMING MODEL.
         CCLBD=0.0
         IF((ITPIJ.EQ.1).AND.(OPTION.EQ.'B1')) CCLBD=1.0-GAMMA(IG)
         DO 70 IR=1,NREG
         IBM=MATCOD(IR)
         IND=NUNKNO/2+KEYFLX(IR,1)
         IF(IND.EQ.0) GO TO 70
         IF(OPTION(2:2).EQ.'1') THEN
*           B1 OR P1 CASE.
            IF(ITPIJ.EQ.2) THEN
               FLUX(IND,IG,8)=FLUX(IND,IG,8)+XSDIA(IBM,1,IG)*
     >         FLUX(IND,IG,6)
            ENDIF
         ELSE IF(ITPIJ.EQ.1) THEN
*           B0, P0 OR B0TR CASE.
            FLUX(IND,IG,8)=FLUX(IND,IG,8)-XSDIA(IBM,1,IG)*
     >      FLUX(IND,IG,6)*GAMMA(IG)
         ENDIF
         FLUX(IND,IG,8)=FLUX(IND,IG,8)+CCLBD*XSDIA(IBM,1,IG)*
     >   FLUX(IND,IG,6)
  70     CONTINUE
      ELSE IF(ILEAK.GE.6) THEN
*        TIBERE ANISOTROPIC STREAMING MODEL.
         CCLBD=0.0
         IF((ITPIJ.EQ.3).AND.(OPTION.EQ.'B1')) CCLBD=1.0-GAMMA(IG)
         DO 85 IR=1,NREG
         IND0=KEYFLX(IR,1)
         IF(IND0.EQ.0) GO TO 85
         IBM=MATCOD(IR)
         INDD(1)=NUNKNO/4+IND0
         INDD(2)=NUNKNO/2+IND0
         INDD(3)=3*NUNKNO/4+IND0
         DO 80 IDIR=1,3
         IND=INDD(IDIR)
         IF(OPTION(2:2).EQ.'1') THEN
*           B1 OR P1 CASE.
            IF(ITPIJ.EQ.4) THEN
               FLUX(IND,IG,8)=FLUX(IND,IG,8)+XSDIA(IBM,1,IG)*
     >         FLUX(IND,IG,6)
            ENDIF
         ELSE IF(ITPIJ.EQ.3) THEN
*           B0, P0 OR B0TR CASE.
            FLUX(IND,IG,8)=FLUX(IND,IG,8)-XSDIA(IBM,1,IG)*
     >      FLUX(IND,IG,6)*GAMMA(IG)
         ENDIF
         FLUX(IND,IG,8)=FLUX(IND,IG,8)+CCLBD*XSDIA(IBM,1,IG)*
     >   FLUX(IND,IG,6)
  80     CONTINUE
  85     CONTINUE
      ENDIF
*----
*  COMPUTE INNER SOURCES ASSUMING SELF-SCATTERING REDUCTION.
*----
      IF(.NOT.LSCAL) THEN
         KPMACR=LCMGIL(JPMACR,IG)
         DO 100 IAL=0,MIN(NLF-1,NANIS)
         CALL LCMGET(KPMACR,'NJJS'//CAN(IAL),NJJ(1))
         CALL LCMGET(KPMACR,'IJJS'//CAN(IAL),IJJ(1))
         CALL LCMGET(KPMACR,'IPOS'//CAN(IAL),IPOS(1))
         CALL LCMGET(KPMACR,'SCAT'//CAN(IAL),XSCAT(1))
         DO 100 IR=1,NREG
         IBM=MATCOD(IR)
         IF(IBM.GT.0) THEN
            DO 90 IAM=0,IAL
            IND=KEYFLX(IR,1+IAL*(IAL+1)/2+IAM)
            JG=IJJ(IBM)
            DO 90 JND=1,NJJ(IBM)
            IF(JG.NE.IG) THEN
               FLUX(IND,IG,8)=FLUX(IND,IG,8)+REAL(2*IAL+1)*
     >         XSCAT(IPOS(IBM)+JND-1)*FLUX(IND,JG,6)
            ENDIF
  90        JG=JG-1
         ENDIF
 100     CONTINUE
         IF((ILEAK.EQ.5).AND.(OPTION(2:2).EQ.'1')) THEN
*           ECCO ISOTROPIC STREAMING MODEL.
            CALL LCMGET(KPMACR,'NJJS01',NJJ(1))
            CALL LCMGET(KPMACR,'IJJS01',IJJ(1))
            CALL LCMGET(KPMACR,'IPOS01',IPOS(1))
            CALL LCMGET(KPMACR,'SCAT01',XSCAT(1))
            DO 120 IR=1,NREG
            IBM=MATCOD(IR)
            IF(IBM.GT.0) THEN
               IND=NUNKNO/2+KEYFLX(IR,1)
               JG=IJJ(IBM)
               DO 110 JND=1,NJJ(IBM)
               IF(JG.NE.IG) THEN
                  FLUX(IND,IG,8)=FLUX(IND,IG,8)+XSCAT(IPOS(IBM)+JND-1)*
     >            FLUX(IND,JG,6)
               ENDIF
  110          JG=JG-1
            ENDIF
  120       CONTINUE
         ENDIF
         DO 130 IND=1,NUNKNO
  130    FLUX(IND,IG,7)=FLUX(IND,IG,6)
      ENDIF
  140 CONTINUE
*----
*  FLUX COMPUTATION
*----
      CALL XDISET(NPSYS,NGRP,0)
      DO 150 IG=IGDEB,NGRP
  150 NPSYS(IG)=IG
      JPSTR=0
      IF(IPSYS.NE.0) THEN
         JPSYS=LCMGID(IPSYS,'GROUP')
         IF(ILEAK.EQ.5) THEN
            IPSTR=LCMGID(IPSYS,'STREAMING')
            JPSTR=LCMGID(IPSTR,'GROUP')
         ENDIF
      ENDIF
      IF((.NOT.LSCAL).AND.(ILEAK.EQ.0)) THEN
         IDIR=0
         CALL DOORFVR(CXDOOR,JPSYS,NPSYS,IPTRK,IFTRAK,IPRT,NGRP,
     1   NMAT,NLF,IDIR,NREG,NUNKNO,IPHASE,LEXAC,MATCOD,VOL,KEYFLX,
     2   TITLE,FLUX(1,1,8),FLUX(1,1,7),IPREB,REBFLG)
      ELSE IF(.NOT.LSCAL) THEN
         CALL FLUDBV(CXDOOR,IPHASE,JPSYS,JPSTR,NPSYS,IPTRK,IFTRAK,IPRT,
     1   NREG,NUNKNO,NANIS,NLF,NFUNL,NGRP,NMAT,LEXAC,MATCOD,VOL,KEYFLX,
     2   TITLE,ILEAK,B2,DIFF,GAMMA,LEAKSW,FLUX(1,1,6),IPREB,REBFLG)
      ELSE
*        A GROUP-BY-GROUP SCALAR PROCEDURE IS BEEN USED.
         IF(IPSYS.EQ.0) CALL XABORT('FLU2DR: MISSING L_PIJ OBJECT.')
         DO 160 IG=1,NGRP
         DO 160 IND=1,NUNKNO
  160    FLUX(IND,IG,7)=FLUX(IND,IG,6)
         KPSTR=0
         DO 230 IG=IGDEB,NGRP
         IF(IPRT.GT.10) WRITE(6,'(/25H FLU2DR: PROCESSING GROUP,I5,
     >   6H WITH ,A,1H.)') IG,CXDOOR
         KPMACR=LCMGIL(JPMACR,IG)
         IF(CXDOOR.EQ.'BIVAC') THEN
            CALL BIVSOU(IG,IPTRK,KPMACR,NANIS,NREG,NMAT,NUNKNO,NGRP,
     1      MATCOD,KEYFLX(1,1),VOL,FLUX(1,1,7),FLUX(1,1,8),IJJ,NJJ,
     2      IPOS,XSCAT)
         ELSE IF(CXDOOR.EQ.'TRIVAC') THEN
            CALL TRIVSO(IG,IPTRK,KPMACR,NANIS,NREG,NMAT,NUNKNO,NGRP,
     1      MATCOD,KEYFLX(1,1),VOL,FLUX(1,1,7),FLUX(1,1,8),IJJ,NJJ,
     2      IPOS,XSCAT)
         ELSE IF(CXDOOR.EQ.'SN') THEN
            CALL SNSOUR(IG,IPTRK,KPMACR,NANIS,NREG,NMAT,NUNKNO,NGRP,
     1      MATCOD,KEYFLX(1,1),VOL,FLUX(1,1,7),FLUX(1,1,8),IJJ,NJJ,
     2      IPOS,XSCAT)
         ELSE
            DO 180 IAL=0,MIN(NLF-1,NANIS)
            CALL LCMGET(KPMACR,'NJJS'//CAN(IAL),NJJ(1))
            CALL LCMGET(KPMACR,'IJJS'//CAN(IAL),IJJ(1))
            CALL LCMGET(KPMACR,'IPOS'//CAN(IAL),IPOS(1))
            CALL LCMGET(KPMACR,'SCAT'//CAN(IAL),XSCAT(1))
            DO 180 IR=1,NREG
            IBM=MATCOD(IR)
            IF(IBM.GT.0) THEN
               DO 170 IAM=0,IAL
               IND=KEYFLX(IR,1+IAL*(IAL+1)/2+IAM)
               JG=IJJ(IBM)
               DO 170 JND=1,NJJ(IBM)
               IF(JG.NE.IG) THEN
                  FLUX(IND,IG,8)=FLUX(IND,IG,8)+REAL(2*IAL+1)*
     >            XSCAT(IPOS(IBM)+JND-1)*FLUX(IND,JG,7)
               ENDIF
  170          JG=JG-1
            ENDIF
  180       CONTINUE
         ENDIF
         IF((ILEAK.GE.5).AND.(OPTION(2:2).EQ.'1')) THEN
            CALL LCMGET(KPMACR,'NJJS01',NJJ(1))
            CALL LCMGET(KPMACR,'IJJS01',IJJ(1))
            CALL LCMGET(KPMACR,'IPOS01',IPOS(1))
            CALL LCMGET(KPMACR,'SCAT01',XSCAT(1))
            IF(ILEAK.EQ.5) THEN
*              ECCO ISOTROPIC STREAMING MODEL.
               KPSTR=LCMGIL(JPSTR,IG)
               DO 200 IR=1,NREG
               IBM=MATCOD(IR)
               IF(IBM.GT.0) THEN
                  IND=NUNKNO/2+KEYFLX(IR,1)
                  JG=IJJ(IBM)
                  DO 190 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     FLUX(IND,IG,8)=FLUX(IND,IG,8)+
     >               XSCAT(IPOS(IBM)+JND-1)*FLUX(IND,JG,7)
                  ENDIF
  190             JG=JG-1
               ENDIF
  200          CONTINUE
            ELSE IF(ILEAK.GE.6) THEN
*              TIBERE ANISOTROPIC STREAMING MODEL.
               DO 220 IR=1,NREG
               IBM=MATCOD(IR)
               IF(IBM.GT.0) THEN
                  INDD(1)=NUNKNO/4+KEYFLX(IR,1)
                  INDD(2)=NUNKNO/2+KEYFLX(IR,1)
                  INDD(3)=3*NUNKNO/4+KEYFLX(IR,1)
                  DO 210 IDIR=1,3
                  IND=INDD(IDIR)
                  JG=IJJ(IBM)
                  DO 210 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     FLUX(IND,IG,8)=FLUX(IND,IG,8)+
     >               XSCAT(IPOS(IBM)+JND-1)*FLUX(IND,JG,7)
                  ENDIF
  210             JG=JG-1
               ENDIF
  220          CONTINUE
            ENDIF
         ENDIF
*
         KPSYS=LCMGIL(JPSYS,IG)
         IF(JPSTR.NE.0) KPSTR=LCMGIL(JPSTR,IG)
         IF((CXDOOR.EQ.'TRAFIC').AND.(ILEAK.EQ.0)) THEN
             CALL TRFICF(KPSYS,IPTRK,IFTRAK,IPRT,0,NREG,NMAT,NUNKNO,
     1       MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),FLUX(1,IG,8),
     2       TITLE)
         ELSE IF(CXDOOR.EQ.'TRAFIC') THEN
             CALL FLUDB2(TRFICF,KPSYS,KPSTR,IPTRK,IFTRAK,IPRT,NREG,
     1       NUNKNO,NMAT,MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),
     2       FLUX(1,IG,8),ILEAK,B2,DIFF(IG),GAMMA(IG),TITLE)
         ELSE IF((CXDOOR.EQ.'SYBIL').AND.(ILEAK.EQ.0)) THEN
             CALL SYBILF(KPSYS,IPTRK,IFTRAK,IPRT,0,NREG,NMAT,NUNKNO,
     1       MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),FLUX(1,IG,8),
     2       TITLE)
         ELSE IF(CXDOOR.EQ.'SYBIL') THEN
             CALL FLUDB2(SYBILF,KPSYS,KPSTR,IPTRK,IFTRAK,IPRT,NREG,
     1       NUNKNO,NMAT,MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),
     2       FLUX(1,IG,8),ILEAK,B2,DIFF(IG),GAMMA(IG),TITLE)
         ELSE IF((CXDOOR.EQ.'BIVAC').AND.(ILEAK.EQ.0).AND.(NLF.EQ.1))
     1   THEN
             CALL BIVAF(KPSYS,IPTRK,IFTRAK,IPRT,0,NREG,NMAT,NUNKNO,
     1       MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),FLUX(1,IG,8),
     2       TITLE)
         ELSE IF((CXDOOR.EQ.'BIVAC').AND.(ILEAK.EQ.0)) THEN
             CALL PNF(KPSYS,IPTRK,IFTRAK,IPRT,0,NREG,NMAT,NUNKNO,
     1       MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),FLUX(1,IG,8),
     2       TITLE)
         ELSE IF((CXDOOR.EQ.'TRIVAC').AND.(ILEAK.EQ.0)) THEN
             CALL TRIVAF(KPSYS,IPTRK,IFTRAK,IPRT,0,NREG,NMAT,NUNKNO,
     1       MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),FLUX(1,IG,8),
     2       TITLE)
         ELSE IF((CXDOOR.EQ.'SN').AND.(ILEAK.EQ.0)) THEN
             CALL SNF(KPSYS,IPTRK,IFTRAK,IPRT,0,NREG,NMAT,NUNKNO,
     1       MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),FLUX(1,IG,8),
     2       TITLE)
         ELSE IF((CXDOOR.EQ.'BIVAC').AND.(NLF.EQ.1)) THEN
             CALL FLUDB2(BIVAF,KPSYS,KPSTR,IPTRK,IFTRAK,IPRT,NREG,
     1       NUNKNO,NMAT,MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),
     2       FLUX(1,IG,8),ILEAK,B2,DIFF(IG),GAMMA(IG),TITLE)
         ELSE IF(CXDOOR.EQ.'BIVAC') THEN
             CALL FLUDB2(PNF,KPSYS,KPSTR,IPTRK,IFTRAK,IPRT,NREG,
     1       NUNKNO,NMAT,MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),
     2       FLUX(1,IG,8),ILEAK,B2,DIFF(IG),GAMMA(IG),TITLE)
         ELSE IF(CXDOOR.EQ.'TRIVAC') THEN
             CALL FLUDB2(TRIVAF,KPSYS,KPSTR,IPTRK,IFTRAK,IPRT,NREG,
     1       NUNKNO,NMAT,MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),
     2       FLUX(1,IG,8),ILEAK,B2,DIFF(IG),GAMMA(IG),TITLE)
         ELSE IF(CXDOOR.EQ.'SN') THEN
             CALL FLUDB2(SNF,KPSYS,KPSTR,IPTRK,IFTRAK,IPRT,NREG,
     1       NUNKNO,NMAT,MATCOD(1),VOL,KEYFLX(1,1),FLUX(1,IG,7),
     2       FLUX(1,IG,8),ILEAK,B2,DIFF(IG),GAMMA(IG),TITLE)
         ELSE
             TEXT12=CXDOOR
             CALL XABORT('FLU2DR: DOOR '//TEXT12//' NOT AVAILABLE.')
         ENDIF
  230    CONTINUE
      ENDIF
      IF(LREBAL.AND.(ITYPEC.NE.5)) THEN
         CALL SETARA(A,NGRP*(NGRP+1),REBAL)
         CALL FLUBAL(IPMACR,NGRP,ILEAK,NMAT,NREG,ICREB,NUNKNO,NANIS,
     1   MATCOD,VOL,KEYFLX(1,1),XSTRC,XSDIA,XCSOU,IGDEB,B2,DIFF,GAMMA,
     2   I(KEYCUR),I(MATALB+NNN),ALBEDO,A(IV+NNN),FLUX(1,1,7),A(REBAL),
     3   IJJ,NJJ,IPOS,XSCAT)
         CALL RLSARA(A(REBAL))
      ENDIF
*
      IF(MOD(JT-1,NCTOT).GE.NCPTM) THEN
*        ACCELERATING INNER ITERATIONS
         CALL FLU2AC(IPRT,NGRP,NUNKNO,IGDEB,FLUX(1,1,5),AKEEP(5),ZMU)
      ELSE
         ZMU=1.0
      ENDIF
      EINN=0.0
      ICHAR=ICHAR+1
      ICTOT=ICTOT+NGRP-IGDEB+1
      IGDEBO=IGDEB
      DO 260 IG=IGDEBO,NGRP
      GINN=0.0
      FINN=0.0
      DO 240 IR=1,NREG
      IND=KEYFLX(IR,1)
      IF(IND.EQ.0) GO TO 240
      GINN=MAX(GINN,ABS(FLUX(IND,IG,6)-FLUX(IND,IG,7)))
      FINN=MAX(FINN,ABS(FLUX(IND,IG,7)))
  240 CONTINUE
      DO 250 IND=1,NUNKNO
      FLUX(IND,IG,5)=FLUX(IND,IG,6)
  250 FLUX(IND,IG,6)=FLUX(IND,IG,7)
      GINN=GINN/FINN
      IF((GINN.LT.EPSINR).AND.(IGDEB.EQ.IG)) THEN
         IGDEB=IGDEB+1
      ELSEIF((IGDEB.EQ.IG).AND.(IG.LT.NGRP)) THEN
         ERRDEB1=GINN
      ENDIF
  260 EINN=MAX(EINN,GINN)
      ITERF=JT
      IF(IPRT.GT.0) WRITE(6,1080) JT,EINN,EPSINR,IGDEB,ZMU
      IF((IPRT.GT.0).AND.(IGDEB.GT.1).AND.(IGDEB.LE.NGRP)) THEN
         WRITE(6,1082) ERRDEB1 
      ENDIF
*     thermal convergence is reached
      IF(EINN.LT.EPSINR) THEN
         CFLI=CEXE
         GOTO 280
      ENDIF
*     near convergence (eps < 10.0 criterion)
*      -> a new outer iteration is started
      IF ((IGDEB.GT.1).AND.(EINN.LT.10.*EPSINR)) GOTO 281
  270 CONTINUE
****  END OF INNER LOOP  ******************************************
*
  281 MESSIN='*NOT*'
  280 IF(LREBAL) THEN
         IF (LEAKSW) THEN
            IF (ICREB.EQ.0) THEN
               WRITE(6,*) ' *** INCOMPATIBILITY ON LEAKAGE SWITCH ***'
               CALL XABORT('FLU2DR: ERROR ON LEAKAGE SWITCH')
            ELSE
               IF(IPRT.GT.0) 
     &            WRITE(6,*) 'FLU2DR: LEAKAGE & ICREB -> REBALANCING ON'
            ENDIF
         ELSE
            IF(IPRT.GT.0) 
     &         WRITE(6,*) 'FLU2DR: NO LEAKAGE-> REBALANCING ON'
         ENDIF
      ELSE
         IF(IPRT.GT.0) WRITE(6,*) 'FLU2DR:    LEAKAGE-> REBALANCING OFF'
      ENDIF
      CALL KDRCPU(CPU2)
      IF(IPRT.GT.0) WRITE(6,1040) CPU2-CPU1,'INTERNAL',MESSIN,ITERF
*----
*  PROMOTE FROM NEW INTERNAL FLUX(,,,7) TO NEW EXTERNAL FLUX(,,,3)
*----
      DO 290 IG=1, NGRP
      DO 290 IND=1,NUNKNO
  290 FLUX(IND,IG,3)=FLUX(IND,IG,7)
*----
*  HOTELLING DEFLATION IN GPT CASES.
*----
      IF(ITYPEC.EQ.1) THEN
         JPFLUP1=LCMGID(IPFLUP,'FLUX')
         JPFLUP2=LCMGID(IPFLUP,'AFLUX')
         DDELN1=0.0D0
         DDELD1=0.0D0
         DO 300 IG=1,NGRP
         IF(LFORW) THEN
            CALL LCMGDL(JPFLUP1,IG,FLUX(1,IG,5)) ! EVECT
            CALL LCMGDL(JPFLUP2,IG,FLUX(1,IG,6)) ! ADECT
         ELSE
            CALL LCMGDL(JPFLUP2,NGRP-IG+1,FLUX(1,IG,5)) ! ADECT
            CALL LCMGDL(JPFLUP1,NGRP-IG+1,FLUX(1,IG,6)) ! EVECT
         ENDIF
  300    CONTINUE
         DO 330 IG=1,NGRP
         CALL XDRSET(FLUX(1,IG,7),NUNKNO,0.0)
         DO 320 IR=1,NREG
         IBM=MATCOD(IR)
         IF(IBM.GT.0) THEN
           IND=KEYFLX(IR,1)
           DO 310 IS=1,NIFIS
           DO 310 JG=1,NGRP
  310      FLUX(IND,IG,7)=FLUX(IND,IG,7)+VOL(IR)*XSNUF(IBM,IS,IG)*
     1     XSCHI(IBM,IS,JG)*FLUX(IND,JG,6)
         ENDIF
  320    CONTINUE
         DO 330 IND=1,NUNKNO
         DDELN1=DDELN1+FLUX(IND,IG,7)*FLUX(IND,IG,3)
  330    DDELD1=DDELD1+FLUX(IND,IG,7)*FLUX(IND,IG,5)
         DO 340 IG=1,NGRP
         DO 340 IND=1,NUNKNO
  340    FLUX(IND,IG,3)=FLUX(IND,IG,3)-(DDELN1/DDELD1)*FLUX(IND,IG,5)
      ENDIF
*
      IF(ITYPEC.EQ.2) THEN
*        NO B-N LEAKAGE CALCULATION REQUIRED
         IF(B2(4).NE.0.0) CALL XABORT('FLU2DR: NON ZERO BUCKLING.')
         CALL FLUKEF(IPRT,IPMACR,NGRP,NREG,NUNKNO,NANIS,NMAT,NIFIS,
     1   MATCOD(1),VOL,KEYFLX(1,1),XSTOT,XSNUF,XSCHI,DIFF,FLUX(1,1,3),
     2   B2,ILEAK,LEAKSW,OLDBIL,AKEFF,AFLNOR,IJJ,NJJ,IPOS,XSCAT)
      ELSE IF(ITYPEC.GT.2) THEN
*        PERFORM THE B-N LEAKAGE CALCULATION.
         CALL LCMGET(IPFLUX,'DIFFB1HOM',DIFF)
         CALL SETARA(A,NGRP,XCTOT)
         CALL SETARA(A,2*NGRP,FLUN)
         CALL B1HOM(IPMACR,NUNKNO,OPTION,HTYPE(ITYPEC),NGRP,NREG,NMAT,
     1   NIFIS,VOL,MATCOD,KEYFLX(1,1),FLUX(1,1,3),REFKEF,IPRT,DIFF,
     2   GAMMA,AKEFF,A(XCTOT),A(FLUN),XSCAT,IJJ,NJJ,IPOS,INORM,B2)
         CALL RLSARA(A(FLUN))
         CALL RLSARA(A(XCTOT))
         IF(ILEAK.GE.6) THEN
*           COMPUTE THE DIRECTIONNAL BUCKLING COMPONENTS FOR TIBERE.
            IHETL=ILEAK/10-1
            IF(IHETL.GT.0) THEN
               CALL SETARA(A,(NMAT+1)*NIFIS,SIGT0)
               CALL SETARA(A,NMAT+1,SIGS0)
               CALL SETARA(A,NREG*NIFIS,QTOTL)
               CALL FLUBLN(IPMACR,IPRT,NGRP,NMAT,NREG,NUNKNO,NIFIS,
     1         MATCOD,VOL,KEYFLX(1,1),FLUX(1,1,3),IHETL,REFKEF,B2,
     2         A(SIGT0),A(SIGS0),A(QTOTL))
               CALL RLSARA(A(QTOTL))
               CALL RLSARA(A(SIGS0))
               CALL RLSARA(A(SIGT0))
            ENDIF
         ENDIF
         CALL LCMPUT(IPFLUX,'DIFFB1HOM',NGRP,2,DIFF)
         CALL LCMPUT(IPFLUX,'B2  B1HOM',1,2,B2(4))
      ENDIF
      IF(ITYPEC.GE.3) THEN
         IF(B2(4).EQ.0.0) THEN
            BFLNOR=1.0D0
         ELSE
            BFLNOR=1.0D0/ABS(B2(4))
         ENDIF
         EEXT=ABS(B2(4)-B2VALO)*BFLNOR
         B2VALO=B2(4)
      ENDIF
      IEXTF=IT
      IF((ITYPEC.GT.1).AND.(ITYPEC.LT.5)) THEN
         AFLNOR=1.0D0/AKEFF
         EEXT=ABS(AKEFF-AKEFFO)/AKEFF
      ELSE
         EEXT=0.0D0
      ENDIF
      AKEEP(3)=AKEFF
      IF(MOD(IT-1,NCTOT).GE.NCPTM) THEN
*        ACCELERATING OUTER ITERATIONS
         CALL FLU2AC(IPRT,NGRP,NUNKNO,1,FLUX(1,1,1),AKEEP(1),ZMU)
      ELSE
         ZMU=1.0
      ENDIF
      EINN=0.0
      IF(ITYPEC.GE.1) THEN
         IF(IPRT.GT.0) WRITE(6,1090) IT,EEXT,EPSOUT,AKEFF,B2(4)
         IF(EEXT.LT.EPSOUT) THEN
*           COMPARE FLUX FOR OUTER ITERATIONS
            DO 370 IG=1,NGRP
            GINN=0.0
            FINN=0.0
            DO 350 IR=1,NREG
            IND=KEYFLX(IR,1)
            IF(IND.EQ.0) GO TO 350
            GINN=MAX(GINN,ABS(FLUX(IND,IG,2)-FLUX(IND,IG,3)))
            FINN=MAX(FINN,ABS(FLUX(IND,IG,3)))
  350       CONTINUE
            DO 360 IND=1,NUNKNO
            FLUX(IND,IG,1)=FLUX(IND,IG,2)
  360       FLUX(IND,IG,2)=FLUX(IND,IG,3)
            GINN=GINN/FINN
  370       EINN=MAX(EINN,GINN)
            IF(IPRT.GT.0) WRITE(6,1100) IT,EINN,EPSUNK,AFLNOR,ZMU
            CEXE=.TRUE.
         ELSE
            DO 380 IG=1,NGRP
            DO 380 IND=1,NUNKNO
            FLUX(IND,IG,1)=FLUX(IND,IG,2)
  380       FLUX(IND,IG,2)=FLUX(IND,IG,3)
            IF(IPRT.GT.0) WRITE(6,1110) IT,AFLNOR,ZMU
         ENDIF
         IF(ITYPEC.GE.2) AFLNOR=(AKEFF/AKEEP(3))*AFLNOR
      ELSE
         DO 390 IG=1,NGRP
         DO 390 IND=1,NUNKNO
         FLUX(IND,IG,1)=FLUX(IND,IG,2)
  390    FLUX(IND,IG,2)=FLUX(IND,IG,3)
      ENDIF
      AKEEP(1)=AKEEP(2)
      AKEEP(2)=AKEEP(3)
*----
*  UPDATE KEFF
*----
      AKEFFO=AKEFF
      IF((EEXT.LT.EPSOUT).AND.(EINN.LT.EPSUNK)) GO TO 410
  400 CONTINUE
      WRITE(6,*) '*** FLU2DR: CONVERGENCE NOT REACHED ***'
      WRITE(6,*) '*** FLU2DR: CONVERGENCE NOT REACHED ***'
      WRITE(6,*) '*** FLU2DR: CONVERGENCE NOT REACHED ***'
      MESSOU='*NOT*'
*
****  CONVERGENCE REACHED  ******************************************
  410 RKEFF=AKEFF
      IF(IPRT.GE.3) THEN
         WRITE(6,1010) (IR,IR=1,NREG)
         CALL SETARA(A,NREG,IFL)
         DO 440 IG=1,NGRP
         WRITE(6,1070) IG
         CALL XDRSET(A(IFL),NREG,0.0)
         DO 420 IR=1,NREG
         IND=KEYFLX(IR,1)
         IF(IND.GT.0) A(IFL+IR-1)=FLUX(IND,IG,3)*AFLNOR
  420    CONTINUE
         WRITE(6,1020) (A(IFL+IR-1),IR=1,NREG)
         DO 440 IA=2,NFUNL
         CALL XDRSET(A(IFL),NREG,0.0)
         DO 430 IR=1,NREG
         IND=KEYFLX(IR,IA)
         IF(IND.GT.0) A(IFL+IR-1)=FLUX(IND,IG,3)*AFLNOR
  430    CONTINUE
         WRITE(6,1030) IA,(A(IFL+IR-1),IR=1,NREG)
  440    CONTINUE
         CALL RLSARA(A(IFL))
      ENDIF
*----
*  COMPUTE K-INF
*----
      IF(ITYPEC.GE.2) THEN
         FISOUR=0.0D0
         OLDBIL=0.0D0
         DO 490 IG=1,NGRP
         DO 460 IR=1,NREG
         IND=KEYFLX(IR,1)
         IF(IND.EQ.0) GO TO 460
         DO 450 IS=1,NIFIS
  450    FISOUR=FISOUR+XSNUF(MATCOD(IR),IS,IG)*FLUX(IND,IG,3)*VOL(IR)
         OLDBIL=OLDBIL+XSTOT(MATCOD(IR),IG)*FLUX(IND,IG,3)*VOL(IR)
  460    CONTINUE
         KPMACR=LCMGIL(JPMACR,IG)
         CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
         CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
         CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
         CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
         DO 480 IR=1,NREG
         IBM=MATCOD(IR)
         IF(IBM.GT.0) THEN
            IND=KEYFLX(IR,1)
            JG=IJJ(IBM)
            DO 470 JND=1,NJJ(IBM)
            OLDBIL=OLDBIL-XSCAT(IPOS(IBM)+JND-1)*FLUX(IND,JG,3)*VOL(IR)
  470       JG=JG-1
         ENDIF
  480    CONTINUE
  490    CONTINUE
         CUREIN=REAL(FISOUR/OLDBIL)
*
*        FLUX NORMALIZATION TO KEFF.
         IF(ITYPEC.LT.5) THEN
            DO 500 IG=1,NGRP
            DO 500 IND=1,NUNKNO
  500       FLUX(IND,IG,3)=FLUX(IND,IG,3)*AKEFF/FISOUR
         ENDIF
      ENDIF
      CALL KDRCPU(CPU1)
      IF(IPRT.GE.1) THEN
         WRITE(6,1040) CPU1-CPU0,'EXTERNAL',MESSOU,IEXTF
         IF(ITYPEC.EQ.0) THEN
            WRITE(6,1050) ICHAR,EEXT
         ELSE
            WRITE(6,1060) ICHAR,CUREIN,AKEFF,B2(4),EEXT
         ENDIF
         WRITE(6,1120) ICTOT
      ENDIF
*----
*  RELEASE ARRAYS
*----
      IF (CXDOOR.EQ.'MCCG') THEN
         IF (ICREB.GT.0) THEN
            CALL RLSARA(I(MATALB))
            CALL RLSARA(A(IV))
            CALL RLSARA(I(KEYCUR))
         ENDIF
      ENDIF
*----
*  SAVE THE SOLUTION
*----
      DO 510 IG=1,NGRP
      IF(LFORW) THEN
         CALL LCMPDL(JPFLUX,IG,NUNKNO,2,FLUX(1,IG,3))
      ELSE
         CALL LCMPDL(JPFLUX,NGRP-IG+1,NUNKNO,2,FLUX(1,IG,3))
      ENDIF
  510 CONTINUE
      IF(ITYPEC.GE.2) THEN
         CALL LCMPUT(IPFLUX,'K-EFFECTIVE',1,2,RKEFF)
         CALL LCMPUT(IPFLUX,'K-INFINITY',1,2,CUREIN)
      ENDIF
      IF(ITYPEC.GE.3) THEN
         CALL LCMPUT(IPFLUX,'B2  B1HOM',1,2,B2(4))
         CALL LCMPUT(IPFLUX,'DIFFB1HOM',NGRP,2,DIFF)
      ENDIF
      IF((ITYPEC.GT.2).AND.(ILEAK.GE.6)) THEN
         CALL LCMPUT(IPFLUX,'B2  HETE',3,2,B2)
      ENDIF
      IF((ITYPEC.GT.2).AND.(ILEAK.GE.5)) THEN
         CALL LCMPUT(IPFLUX,'GAMMA',NGRP,2,GAMMA)
      ENDIF
      RETURN
*
 1010 FORMAT (/28H FLUXES AVERAGED           :/
     1 (9X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,
     2 I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,2H/=,I4,:,6X,
     3 2H/=,I4))
 1020 FORMAT (7H FLUX  ,2H: ,1P,10E12.5/(9X,10E12.5))
 1030 FORMAT (5H CUR ,I2,2H: ,1P,10E12.5/(9X,10E12.5))
 1040 FORMAT (18H FLU2DR: CPU TIME=, F10.0,1X,A8,
     1        13H CONVERGENCE ,A5,14H REACHED AFTER ,I6,
     2        12H ITERATIONS.  )
 1050 FORMAT (/20H ++ TRACKING CALLED=,I4,6H TIMES ,
     1         11H PRECISION=,E9.2)
 1060 FORMAT (/20H ++ TRACKING CALLED=,I4,6H TIMES ,
     1         12H FINAL KINF=,1P,E12.6,
     2         12H FINAL KEFF=,E12.6,4H B2=,E12.5,
     3         11H PRECISION=,E9.2)
 1070 FORMAT (/14H ENERGY GROUP ,I6)
 1080 FORMAT (10X,3HIN(,I3,6H) FLX:,5H PRC=,1P,E9.2,5H TAR=,E9.2,
     1 7H IGDEB=, I13,6H ACCE=,0P,F12.5)
 1081 FORMAT (10X,3HIN(,I3,6H) FLX:,5H PRC=,1P,E9.2,5H TAR=,E9.2,
     1 7H IGDEB=, I7,2H (,I3,1H),6H ACCE=,0P,F12.5)
 1082 FORMAT (18X,28HFIRST UNCONVERGED GROUP PRC=,E9.2)
 1090 FORMAT (5H OUT(,I3,6H) EIG:,5H PRC=,1P,E9.2,5H TAR=,E9.2,
     1 6H KEFF=,E13.6,6H BUCK=,E12.5)
 1100 FORMAT (5H OUT(,I3,6H) FLX:,5H PRC=,1P,E9.2,5H TAR=,E9.2,
     1 6H FNOR=,E13.6,6H ACCE=,0P,F12.5)
 1110 FORMAT (5H OUT(,I3,6H) FLX:,28X,6H FNOR=,1P,E13.6,6H ACCE=,
     1 0P,F12.5)
 1120 FORMAT (38H ++ TOTAL NUMBER OF FLUX CALCULATIONS=,I10)
      END
