*DECK LIB
      SUBROUTINE LIB(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* interpolation of nuclear properties in an internal library.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): A. Hebert and G. Marleau
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_LIBRARY)
*         HENTRY(2): optional read-only type(L_MACROLIB) used to
*                    initialize a new lattice code library.
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
      IMPLICIT     NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      INTEGER      IOUT,NSTATE,NTC,ILCMUP,ILCMDN,MAXED,MAXISD
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NSTATE=40,NTC=3,ILCMUP=1,ILCMDN=2,MAXED=50,
     >             MAXISD=300,NAMSBR='LIB   ')
*----
*  MEMORY ALLOCATION
*----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IENER,IJCEDM,IBSTD,JTMPMX,JMASKI,ILSHI,JJNTFG,
     >             JJTN,LMASKG,JDENMD,LISM,LOCUPD,JDENIX,JIMIX,LMASKM,
     >             JDENMX,JJDEN,JJMIX,JJNRF,JJNAM,IADNAM,IISHIN,ISNISO,
     >             ISBISO,IGIR,INIR,IIHLIB,IILLIB,IIEVOL,IITYP
*----
*  INPUT
*----
      INTEGER      ITYPLU,INTLIR
      CHARACTER    CARLIR*12
      REAL         REALIR
      DOUBLE PRECISION DBLLIR
*----
*  LOCAL PARAMETERS
*----
      CHARACTER    TEXT12*12,HSIGN*12,TEXT4*4,HVECT(MAXED)*8,HADD*8,
     >             HLIBX*12
      INTEGER      ISTATE(NSTATE),KCHAR(NTC),IPRINT,NBISOX,NBMIXX,
     >             IPLIBX,MAXMIX,INDREC,NBISO,NGRO,NGT,NGF,NGFR,
     >             NL,ITRANC,ITIME,NLIB,NIDEPL,NCOMB,
     >             NEDMAC,NBMIX,NRES,MAXISM,IEN,ILCMLN,ILCMTY,
     >             ITC,IED,JED,KED,IDP,ITEXT4,IBSTEP,MAXISO,
     >             IPLIB,NDEPL,NEDMA0,ITPROC,ISOADD,NADDXS,IPROB,
     >             IPROC,IMAC,NDEL,NFISS,I,IPRECI
      REAL         TMPDAY(3),DELT,TIMBRN 
      INTEGER      IKSTEP
*----
*  PARAMETER VALIDATION.
*----
      IF(NENTRY .EQ. 0) CALL XABORT(NAMSBR//': PARAMETER EXPECTED.')
      IF(IENTRY(1) .NE. 1 .AND.
     >   IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >': LCM OBJECT OR XSM FILE EXPECTED AT LHS.')
      IF(JENTRY(1) .NE. 0 .AND.
     >   JENTRY(1) .NE. 1) CALL XABORT(NAMSBR//': ENTRY'
     1 //' IN CREATE OR MODIFICATION MODE EXPECTED.')
      IPLIB=KENTRY(1)
*----
*  READ THE INPUT DATA.
*  DEFAULT OPTIONS:
*----
      IPRINT=1
      NBISOX=0
      NBMIXX=0
      IPLIBX=0
      IBSTEP=0
      HLIBX=' '
      IF(JENTRY(1) .EQ. 0) THEN
        MAXMIX=0
        INDREC=1
        NBISO=0
        NGRO=0
        NGT=0
        NGF=9999999
        NGFR=0
        NL=2
        ITRANC=0
        IPROB=0
        ITIME=1
        NLIB=0
        NIDEPL=0
        NCOMB=0
        NEDMAC=0
        NBMIX=0
        NRES=0
        IPROC=0
        IMAC=1
        NDEL=0
        NFISS=0
        ISOADD=0
        MAXISM=MAXISD
        IPRECI=4
*----
*  TRY TO FIND A READ-ONLY MACROLIB TO INCLUDE IN THE LIBRARY.
*----
        DO 130 IEN=2,NENTRY
          IF(IENTRY(IEN) .LE. 2 .AND. JENTRY(IEN) .EQ. 2) THEN
            CALL LCMLEN(KENTRY(IEN),'MACROLIB',ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. -1) THEN
              CALL LCMSIX(KENTRY(IEN),'MACROLIB',ILCMUP)
              CALL LCMLEN(KENTRY(IEN),'SIGNATURE',ILCMLN,ILCMTY)
              IF(ILCMLN .LE. 0) CALL XABORT(NAMSBR//
     >        ': NO SIGNATURE FOUND FOR MACROLIB DATA STRUCTURE')
              CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
              WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
              IF(HSIGN.NE.'L_MACROLIB') CALL XABORT(NAMSBR//
     >        ': WRONG SIGNATURE FOR MACROLIB DATA STRUCTURE')
              INDREC=3
              CALL LCMGET(KENTRY(IEN),'STATE-VECTOR',ISTATE)
              NGRO=ISTATE(1)
              NGT=NGRO
              MAXMIX=ISTATE(2)
              NL=ISTATE(3)
              NADDXS=ISTATE(5)
              ITRANC=ISTATE(6)
              NDEL=ISTATE(7)
              IF(NGT .GT. 0) THEN
                CALL SETARA(RBASE,2*NGT+1,IENER)
                CALL LCMGET(KENTRY(IEN),'ENERGY',RBASE(IENER))
                CALL LCMGET(KENTRY(IEN),'DELTAU',RBASE(IENER+NGT+1))
              ENDIF
              CALL LCMSIX(IPLIB,'MACROLIB',ILCMUP)
              CALL LCMEQU(KENTRY(IEN),IPLIB)
              IF(NADDXS .NE. 0) THEN
                IF(NADDXS .GT. MAXED-NEDMAC) CALL XABORT(NAMSBR//
     >          ': TOO MANY EXTRA EDITS REQUESTED')
                CALL SETARA(IBASE,2*NADDXS,IADNAM)
                CALL LCMGET(IPLIB,'ADDXSNAME-P0',IBASE(IADNAM))
                JED=IADNAM
                DO 120 IED=1,NADDXS
                  WRITE(HADD,'(2A4)') IBASE(JED),IBASE(JED+1)
                  DO 100 KED=1,NEDMAC
                    IF(HADD.EQ.HVECT(KED)) GO TO 110
 100              CONTINUE
                  NEDMAC=NEDMAC+1
                  HVECT(NEDMAC)=HADD
 110              CONTINUE
                  JED=JED+2
 120            CONTINUE
                CALL RLSARA(IBASE(IADNAM))
              ENDIF
*----
*  WRITE ENERGY AND DELTAU ON MACROLIB
*----
              IF(NGT .GT. 0) THEN
                CALL LCMPUT(IPLIB,'ENERGY',NGT+1,2,RBASE(IENER))
                CALL LCMPUT(IPLIB,'DELTAU',NGT,2,RBASE(IENER+NGT+1))
              ENDIF
              CALL LCMSIX(IPLIB,'MACROLIB',ILCMDN)
              IF(NGT .GT. 0) THEN
                CALL LCMPUT(IPLIB,'ENERGY',NGT+1,2,RBASE(IENER))
                CALL LCMPUT(IPLIB,'DELTAU',NGT,2,RBASE(IENER+NGT+1))
                CALL RLSARA(RBASE(IENER))
              ENDIF
              CALL LCMSIX(KENTRY(IEN),'MACROLIB',ILCMDN)
              GO TO 150
            ENDIF
          ENDIF
 130    CONTINUE
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        IF(HSIGN.NE.'L_LIBRARY') THEN
          TEXT12=HENTRY(1)
          CALL XABORT(NAMSBR//
     >    ': SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     >    '. L_LIBRARY EXPECTED.')
        ENDIF
        INDREC=2
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        MAXMIX=ISTATE(1)
        NBISO=ISTATE(2)
        NGRO=ISTATE(3)
        NGT=NGRO
        NL=ISTATE(4)
        ITRANC=ISTATE(5)
        IPROB=ISTATE(6)
        ITIME=ISTATE(7)
        NLIB=ISTATE(8)
        NGF=ISTATE(9)
        NGFR=ISTATE(10)
        NIDEPL=ISTATE(11)
        NCOMB=ISTATE(12)
        NEDMAC=ISTATE(13)
        NBMIX=ISTATE(14)
        NRES=ISTATE(15)
        IPROC=ISTATE(17)
        IMAC=ISTATE(18)
        NDEL=ISTATE(19)
        NFISS=ISTATE(20)
        ISOADD=ISTATE(21)
        MAXISM=ISTATE(22)
        IPRECI=ISTATE(23)
        IF(NEDMAC.GT.0) THEN
          CALL SETARA(IBASE,2*NEDMAC,IJCEDM)
          CALL LCMGET(IPLIB,'ADDXSNAME-P0',IBASE(IJCEDM))
          IED=IJCEDM
          DO 140 JED=1,NEDMAC
            WRITE(HVECT(JED),'(2A4)') IBASE(IED),IBASE(IED+1)
            IED=IED+2
 140      CONTINUE
          CALL RLSARA(IBASE(IJCEDM))
        ENDIF
      ENDIF
*----
*  READ LIBRARY DATA
*----
 150  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': KEYWORD EXPECTED')
      IF(CARLIR(1:4) .EQ. 'EDIT') THEN
*---
*  READ THE PRINT INDEX
*----
        CALL REDGET(ITYPLU,IPRINT,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU.NE.1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR IPRINT EXPECTED')
      ELSE IF(CARLIR(1:4) .EQ. 'NGRO') THEN
*----
*  READ THE NUMBER OF ENERGY GROUPS.
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR NGRO EXPECTED')
        IF(INDREC .EQ. 2) THEN
          IF(NGRO .NE. INTLIR) CALL XABORT(NAMSBR//
     >    ': INCOMPATIBLE VALUE OF NGRO')
        ELSE
          NGRO=INTLIR
        ENDIF
      ELSE IF(CARLIR(1:4) .EQ. 'MXIS') THEN
*----
*  CHANGE MAXIMUM NUMBER OF ISOTOPES
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR MXIS EXPECTED')
        MAXISM=MAX(MAXISM,INTLIR)
      ELSE IF(CARLIR(1:4) .EQ. 'NMIX') THEN
*----
*  READ THE MAXIMUM NUMBER OF MATERIAL MIXTURES
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR NMIX EXPECTED')
        MAXMIX=MAX(MAXMIX,INTLIR)
      ELSE IF(CARLIR(1:4) .EQ. 'CTRA') THEN
*----
*  READ TRANSPORT CORRECTION TYPE
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >  ': CHARACTER CTRA TYPE EXPECTED')
        IF(CARLIR(1:4) .EQ. 'NONE') THEN
           ITRANC=0
        ELSE IF(CARLIR(1:4) .EQ. 'APOL') THEN
           ITRANC=1
        ELSE IF(CARLIR(1:4) .EQ. 'WIMS') THEN
           ITRANC=2
        ELSE IF(CARLIR(1:4) .EQ. 'OLDW') THEN
           ITRANC=3
        ELSE IF(CARLIR(1:4) .EQ. 'LEAK') THEN
           ITRANC=4
        ELSE
           CALL XABORT(NAMSBR//
     >     ': CTRA TYPE NONE, APOL, WIMS, OLDW OR LEAK EXPECTED.')
        ENDIF
      ELSE IF(CARLIR(1:4) .EQ. 'ANIS') THEN
*----
*  READ THE SCATTERING ANISOTROPY FOR TRANSPORT THEORY CASES
*----
        CALL REDGET(ITYPLU,NL,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR ANIS EXPECTED')
      ELSE IF(CARLIR(1:3) .EQ. 'ADJ') THEN
        IPROB=1
      ELSE IF(CARLIR(1:4) .EQ. 'PROM') THEN
        ITIME=2
      ELSE IF(CARLIR(1:7) .EQ. 'RDEPCHN') THEN
        ISOADD=1
      ELSE IF(CARLIR(1:7) .EQ. 'CDEPCHN') THEN
        ISOADD=0
      ELSE IF(CARLIR(1:4) .EQ. 'SKIP') THEN
        IPROC=-1
        IMAC=0
      ELSE IF(CARLIR(1:4) .EQ. 'INTR') THEN
        IPROC=0
        IMAC=0
      ELSE IF(CARLIR(1:4) .EQ. 'SUBG') THEN
        IPROC=1
        IMAC=0
      ELSE IF(CARLIR(1:4) .EQ. 'NEWL') THEN
        IPROC=2
        IMAC=0
      ELSE IF(CARLIR(1:4) .EQ. 'PTSL') THEN
        IPROC=4
        IMAC=0
      ELSE IF(CARLIR(1:4) .EQ. 'PTMC') THEN
        IPROC=5
        IMAC=0
      ELSE IF(CARLIR(1:2) .EQ. 'PT') THEN
        IPROC=3
        IMAC=0
      ELSE IF(CARLIR(1:4) .EQ. 'MACR') THEN
        IMAC=1
      ELSE IF(CARLIR(1:4) .EQ. 'CALE') THEN
        CALL REDGET(ITYPLU,IPRECI,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': INTEGER VALUE EXPECTED FOR CALENDF ACCURACY')
      ELSE IF(CARLIR(1:4) .EQ. 'DEPL') THEN
        CALL LIBDEP(IPLIB,IPRINT,NIDEPL)
      ELSE IF(CARLIR.EQ.'ADED') THEN
        CALL REDGET(ITYPLU,NEDMA0,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': VALUE FOR ADED EXPECTED')
        DO 170 IED=1,NEDMA0
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >    ': TYPE FOR ADED EXPECTED')
          DO 160 JED=1,NEDMAC
            IF(CARLIR(:8) .EQ. HVECT(JED)) GO TO 170
 160      CONTINUE
          NEDMAC=NEDMAC+1
          IF(NEDMAC .GT. MAXED) CALL XABORT(NAMSBR//
     >    ': TOO MANY EXTRA EDITS REQUESTED')
          HVECT(NEDMAC)=CARLIR(:8)
 170    CONTINUE
      ELSE IF(CARLIR(1:4) .EQ. 'MIXS') THEN
         ITPROC=1
         GO TO 240
      ELSE IF(CARLIR(1:4) .EQ. 'MAXS') THEN
         ITPROC=2
         IF(INDREC .NE. 2)  CALL XABORT(NAMSBR//
     >   ': MAXS CAN ONLY BE USE TO UPDATE '//
     >   'A LIBRARY - IT CANNOT CREATE A NEW LIBRARY')
*----
*  TRY TO FIND A SECOND READ-ONLY LIBRARY TO MODIFY
*  ORIGINAL LIBRARY
*----
         DO 180 IEN=2,NENTRY
           IF(IENTRY(IEN) .LE. 2 .AND.
     >        JENTRY(IEN) .EQ.2 ) THEN
             CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
             WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
             IF(HSIGN.EQ.'L_LIBRARY') THEN
               IPLIBX=KENTRY(IEN)
               HLIBX=HENTRY(IEN)
               CALL LCMGET(IPLIBX,'STATE-VECTOR',ISTATE)
               NBMIXX=ISTATE(1)
               NBISOX=ISTATE(2)
             ENDIF
           ENDIF
 180    CONTINUE
        TMPDAY(1)=0.0
        TMPDAY(2)=0.0
        TMPDAY(3)=0.0
        CALL LCMLEN(IPLIB,'MACROLIB',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. -1) THEN
          CALL LCMSIX(IPLIB,'MACROLIB',ILCMUP)
          CALL LCMLEN(IPLIB,'TIMESTAMP',ILCMLN,ILCMTY)
          IF(ILCMLN .GT. 0 .AND. ILCMLN .LE. 3) THEN
            CALL LCMGET(IPLIB,'TIMESTAMP',TMPDAY)
          ENDIF
          CALL LCMSIX(IPLIB,'MACROLIB',ILCMDN)
        ENDIF
        GO TO 240
      ELSE IF(CARLIR(1:4) .EQ. 'BURN') THEN
        ITPROC=2
        IF(INDREC .NE. 2) CALL XABORT(NAMSBR//
     >  ': MAXS CAN ONLY BE USE TO UPDATE '//
     >  'A LIBRARY - IT CANNOT CREATE A NEW LIBRARY')
        DO 190 IEN=2,NENTRY
          IF(IENTRY(IEN) .LE. 2 .AND.
     >       JENTRY(IEN) .EQ. 2) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
            IF(HSIGN.EQ.'L_BURNUP') THEN
              IPLIBX=KENTRY(IEN)
              HLIBX=HENTRY(IEN)
              CALL LCMGET(IPLIBX,'STATE-VECTOR',ISTATE)
              NDEPL=ISTATE(3)
              NBISOX=ISTATE(4)
              NBMIXX=ISTATE(8)
              GO TO 200
            ENDIF
          ENDIF
 190    CONTINUE
        CALL XABORT(NAMSBR//': BURNUP FILE MISSING')
 200    CONTINUE
        CALL SETARA(RBASE,NDEPL,IBSTD)
        CALL LCMGET(IPLIBX,'DEPL-TIMES  ',RBASE(IBSTD))
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .EQ. 3) CALL XABORT(NAMSBR//': INVALID BURNUP STEP')
        IF(ITYPLU .EQ. 2) THEN
          TMPDAY(1)=REALIR
          TIMBRN=0.000864*TMPDAY(1) 
          IF(TIMBRN .LE. 0.0) THEN  
            IBSTEP=1
            TMPDAY(1)=0.0
          ELSE
            IBSTEP=1
            IKSTEP=0
            DO 210 IDP=1,NDEPL
              DELT=ABS(TIMBRN-RBASE(IBSTD+IDP-1))
              IF(DELT .LT. 1.0E-6) THEN
                IBSTEP=IDP
                GO TO 220
              ELSE IF(TIMBRN .GT. RBASE(IBSTD+IDP-1)) THEN
                IKSTEP=IDP  
              ENDIF
 210        CONTINUE
            WRITE(IOUT,9000) TMPDAY
            WRITE(IOUT,9001) (RBASE(IBSTD+IDP-1)/0.000864,IDP=1,NDEPL)
            IBSTEP=MIN(IKSTEP+1,NDEPL)
            WRITE(IOUT,9002) RBASE(IBSTD+IBSTEP-1)/0.000864
 220        CONTINUE
          ENDIF
        ELSE IF(ITYPLU .EQ. 1) THEN
          IBSTEP=INTLIR
          IF(IBSTEP .LE. 0 ) THEN 
            WRITE(IOUT,9010) 
            IBSTEP=1
            WRITE(IOUT,9010) RBASE(IBSTD+IBSTEP-1)/0.000864
          ELSE IF(IBSTEP .GT. NDEPL) THEN 
            IBSTEP=NDEPL 
            WRITE(IOUT,9011) RBASE(IBSTD+IBSTEP-1)/0.000864
          ENDIF
          TMPDAY(1)=RBASE(IBSTD+IBSTEP-1)/0.000864
        ENDIF
        CALL RLSARA(RBASE(IBSTD)) 
        TMPDAY(2)=0.0
        TMPDAY(3)=0.0
        IF(IPRINT .GE. 1) WRITE(IOUT,6000) IBSTEP,TMPDAY(1)
        GO TO 240
      ELSE IF(CARLIR(1:1).EQ.';') THEN
*       SAVE THE LIBRARY SPECIFIC INFORMATION.
        TEXT12='L_LIBRARY'
        READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
        CALL LCMPUT(IPLIB,'SIGNATURE',3,3,KCHAR)
        CALL XDISET(ISTATE,NSTATE,0)
        ISTATE(1)=MAXMIX
        ISTATE(2)=NBISO
        ISTATE(3)=NGRO
        ISTATE(4)=NL
        ISTATE(5)=ITRANC
        ISTATE(6)=IPROB
        ISTATE(7)=ITIME
        ISTATE(8)=NLIB
        ISTATE(9)=NGF
        ISTATE(10)=NGFR
        ISTATE(11)=NIDEPL
        ISTATE(12)=NCOMB
        ISTATE(13)=NEDMAC
        ISTATE(14)=NBMIX
        ISTATE(15)=NRES
        ISTATE(17)=IPROC
        ISTATE(18)=IMAC
        ISTATE(19)=NDEL
        ISTATE(20)=NFISS
        ISTATE(21)=ISOADD
        ISTATE(22)=MAXISM
        ISTATE(23)=IPRECI
        CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,ISTATE)
        GO TO 250
      ELSE
        CALL XABORT(NAMSBR//': '//CARLIR//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 150
*----
*  PROCESS THE LIB: MODULE INPUT DATA.
*----
 240  CONTINUE
      IF(MAXMIX .EQ. 0) CALL XABORT(NAMSBR//
     >': MAXMIX NOT YET DEFINED.')
      MAXISO=MAXISM*MAXMIX
*----
*  ALLOCATE
*----
      CALL SETARA(IBASE,NTC*MAXISO,JJNAM)
      CALL SETARA(IBASE,NTC*MAXISO,JJNRF)
      CALL SETARA(IBASE,MAXISO,JJMIX)
      CALL SETARA(RBASE,MAXISO,JJDEN)
      CALL SETARA(RBASE,MAXMIX,JDENMX)
      CALL SETARA(IBASE,MAXMIX,LMASKM)
*----
*  INITIALIZE
*----
      TEXT4='    '
      READ(TEXT4,'(A4)') ITEXT4
      CALL XDISET(IBASE(JJNAM),NTC*MAXISO,ITEXT4)
      CALL XDISET(IBASE(JJNRF),NTC*MAXISO,ITEXT4)
      CALL XDISET(IBASE(JJMIX),MAXISO,0)
      CALL XDRSET(RBASE(JJDEN),MAXISO,0.0)
      CALL XDRSET(RBASE(JDENMX),MAXMIX,-1.0)
      IF(ITPROC .EQ. 1) THEN
*----
*  ALLOCATE
*----
        CALL SETARA(RBASE,MAXISO,JJTN)
        CALL SETARA(IBASE,3*MAXISO,IISHIN)
        CALL SETARA(RBASE,MAXISO,ISNISO)
        CALL SETARA(RBASE,MAXISO,ISBISO)
        CALL SETARA(IBASE,MAXISO,JJNTFG)
        CALL SETARA(IBASE,MAXISO,ILSHI)
        CALL SETARA(IBASE,MAXISO,IGIR)
        CALL SETARA(IBASE,MAXISO,INIR)
        CALL SETARA(IBASE,MAXISO,JMASKI)
        CALL SETARA(RBASE,MAXMIX,JTMPMX)
        CALL SETARA(IBASE,6*MAXISO,IIHLIB)
        CALL SETARA(IBASE,MAXISO,IILLIB)
        CALL SETARA(IBASE,MAXISO,IIEVOL)
        CALL SETARA(IBASE,MAXISO,IITYP)
        CALL LIBINP(MAXMIX,MAXED ,MAXISO,IPLIB ,INDREC,IPRINT,
     >              NBISO ,NGRO  ,NGT   ,NL    ,ITRANC,IPROB ,
     >              ITIME ,NLIB  ,NGF   ,NGFR  ,NIDEPL,NCOMB ,
     >              NEDMAC,NBMIX ,NRES  ,IPROC ,IMAC  ,NDEL  ,
     >              ISOADD,MAXISM,HVECT ,IPRECI,
     >              IBASE(JJNAM) ,IBASE(JJNRF)  ,IBASE(JJMIX) ,
     >              RBASE(JJDEN) ,RBASE(JDENMX) ,IBASE(LMASKM),
     >              RBASE(JJTN)  ,IBASE(IISHIN) ,RBASE(ISNISO),
     >              RBASE(ISBISO),IBASE(JJNTFG) ,IBASE(ILSHI) ,
     >              RBASE(IGIR)  ,IBASE(INIR)   ,IBASE(JMASKI),
     >              RBASE(JTMPMX),IBASE(IIHLIB) ,IBASE(IILLIB),
     >              IBASE(IIEVOL),IBASE(IITYP))
*----
*  RELEASE
*----
        CALL RLSARA(IBASE(IITYP))
        CALL RLSARA(IBASE(IIEVOL))
        CALL RLSARA(IBASE(IILLIB))
        CALL RLSARA(IBASE(IIHLIB))
        CALL RLSARA(RBASE(JTMPMX))
        CALL RLSARA(IBASE(JMASKI))
        CALL RLSARA(IBASE(INIR))
        CALL RLSARA(IBASE(IGIR))
        CALL RLSARA(IBASE(ILSHI))
        CALL RLSARA(IBASE(JJNTFG))
        CALL RLSARA(RBASE(ISBISO))
        CALL RLSARA(RBASE(ISNISO))
        CALL RLSARA(IBASE(IISHIN))
        CALL RLSARA(RBASE(JJTN))
      ELSE IF(ITPROC .GE. 2) THEN
*----
*  ALLOCATE
*----
        IF(NGRO .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NUMBER OF GROUP REQUIRED FOR MAXS OF BURN')
        CALL SETARA(IBASE,MAXISO,JIMIX)
        CALL SETARA(RBASE,MAXISO,JDENIX)
        CALL SETARA(IBASE,MAXMIX,LOCUPD)
        CALL SETARA(IBASE,MAXISO,LISM)
        CALL SETARA(RBASE,MAXISO,JDENMD)
        CALL SETARA(IBASE,NGRO,LMASKG)
        CALL LIBMAC(IPLIB ,IPLIBX,IPRINT,MAXISO,NBISO ,NBISOX,
     >              IBSTEP,NBMIX ,NBMIXX,NGRO  ,TMPDAY,
     >              IBASE(JJNAM) ,IBASE(JJNRF) ,IBASE(JJMIX) ,
     >              RBASE(JJDEN) ,RBASE(JDENMX),IBASE(JIMIX) ,
     >              RBASE(JDENIX),IBASE(LOCUPD),IBASE(LISM)  ,
     >              RBASE(JDENMD),IBASE(LMASKM),IBASE(LMASKG))
*----
* RELEASE
*----
        CALL RLSARA(IBASE(LMASKG))
        CALL RLSARA(RBASE(JDENMD))
        CALL RLSARA(IBASE(LISM))
        CALL RLSARA(IBASE(LOCUPD))
        CALL RLSARA(RBASE(JDENIX))
        CALL RLSARA(IBASE(JIMIX))
      ENDIF
*----
* RELEASE
*----
      CALL RLSARA(IBASE(LMASKM))
      CALL RLSARA(RBASE(JDENMX))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
      CALL RLSARA(IBASE(JJNRF))
      CALL RLSARA(IBASE(JJNAM))
  250 IF(IPRINT .GE. 5) CALL LCMLIB(IPLIB)
      RETURN
*----
*  FORMATS
*----
 6000 FORMAT(' LIBRARY UPDATE AT BURNUP STEP : ',I5,
     >       ' BURNUP TIME = ',F20.7,' DAYS') 
 9000 FORMAT(' **** WARNING  *****'/
     >       ' INVALID BURNUP TIME =',F20.7,' DAYS'/
     >       ' BURNUP TABULATION (DAYS) ')
 9001 FORMAT(6F20.7)
 9002 FORMAT(' BURNUP STEP SELECTED =',F20.7,' DAYS')
 9010 FORMAT(' **** WARNING  *****'/
     >       ' BURNUP STEP NEGATIVE '/
     >       ' USE FIRST BURNUP STEP AT ',F20.7,' DAYS')
 9011 FORMAT(' **** WARNING  *****'/
     >       ' BURNUP STEP TOO LARGE '/
     >       ' USE LAST BURNUP STEP AT ',F20.7,' DAYS')
      END
