*DECK LIBA20
      SUBROUTINE LIBA20 (IPLIB,NAMFIL,NGRO,NBISO,NL,IPROC,ISONAM,
     1 ISONRF,ISHINA,MASKI,TN,SN,SB,IMPX,NGF,NGFR,NDEL,AWR,IPR,ENERG,
     2 DELTA,SECT,SIGS,SCAT,ITYPRO,XSTOT,NFS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transcription of the useful interpolated microscopic cross section
* data from APOLIB-2 to LCM data structures
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature)
* NAMFIL  name of the APOLIB-2 file
* NGRO    number of energy groups
* NBISO   number of isotopes present in the calculation domain
* NL      number of Legendre orders required in the calculation
*         NL=1 or higher
* IPROC   type of library processing
* ISONAM  alias name of isotopes
* ISONRF  library reference name of isotopes
* ISHINA  self shielding names
* MASKI   isotopic mask. Isotope with index I is processed if
*         MASKI(I)=.true.
* TN      temperature of each isotope
* SN      dilution cross section in each energy group of each
*         isotope. a value of 1.0E10 is used for infinite dilution
* SB      dilution cross section as used by Livolant and Jeanpierre
*         normalization
* IMPX    print flag
*
*Parameters: output
* NGF     number of fast groups without self-shielding
* NGFR    number of fast and resonance groups
* NDEL    number of precursor groups for delayed neutrons
*
*Parameters: scratch
* AWR
* IPR
* ENERG
* DELTA
* SECT
* SIGS
* SCAT
* ITYPRO
* XSTOT
* NFS
*
*Reference:
*   A. Hebert, P. Bellier, M. Coste, R. Sanchez, Z. Stankovski et
*   I. Zmijarevic, "APOLLO2: Notice informatique Version 2.4",
*   Commissariat a l'Energie Atomique, Rapport SERMA/LENR/RT/98-2477/A,
*   1998.
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NGRO,NBISO,NL,IPROC,ISONAM(3,NBISO),ISONRF(3,NBISO),
     1 ISHINA(3,NBISO),IMPX,NGF,NGFR,NDEL,IPR(7+2*(NL-1),NBISO),
     2 ITYPRO(NL),NFS(NGRO)
      REAL TN(NBISO),SN(NGRO,NBISO),SB(NGRO,NBISO),AWR(NBISO),
     1 ENERG(NGRO+1),DELTA(NGRO),SECT(NGRO),SIGS(NGRO,NL),
     2 SCAT(NGRO,NGRO,NL),XSTOT(NGRO)
      CHARACTER NAMFIL*(*)
      LOGICAL MASKI(NBISO)
*----
*  LOCAL VARIABLES
*----
      LOGICAL LSACO
      PARAMETER (NXSMAX=10,IOUT=6,MAXHOM=9,LSACO=.FALSE.)
*     NOTE: LSACO MUST BE SET TO .TRUE. WITH THE SANCHEZ-COSTE METHOD.
      EXTERNAL LIBA21
      CHARACTER TEXT8*8,TEXT20*20,TEXT80*80,NOMOBJ*20,TYPOBJ*8,
     1 TYPSEG*8,HNAMIS*12,HNISOR*12,HNISSS*12,HSMG*131,TEXT2*2,
     2 TEXT12*12
      LOGICAL LBASE(1),LPFIX,LTRAN,LGPROB,LGTDIF,LGTTRA,LN2N,LPTHOM,
     1 L104,LABS,LDIF,LFIS
      INTEGER ZFISS,FGTD,FGHOMO,FGRESO,FAGG,FDGG,WGAL,FAG
      DOUBLE PRECISION UU,XDRCST
      INTEGER ITHOMO(MAXHOM),ITEXT(20),ISFICH(3),IPAR(3),IBASE(1)
      REAL TKT(5)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1),LBASE(1))
*
      INTEGER TKCARO(31)
      SAVE TKCARO
      DATA TKCARO /
     &   0,   1,   2,   3,  4,   5,  6,  30,   7,  -8,
     &   9, -10,  11, -12, 13, -14, 15,  16, -17,  18,
     & -19,  20, -21,  22, 23, -24, 25, -26,  27, -28,
     &  29   /
*
      ANEUT=XDRCST('Neutron mass','amu')
      NGF=NGRO+1
      NGFR=0
      NDEL=0
      IF(IMPX.GT.0) WRITE (IOUT,800) NAMFIL
*----
*  OPEN AND PROBE THE APOLIB-2 FILE.
*----
      CALL XDRSET(TKT,5,0.0)
      CALL KDRCPU(TK1)
      CALL AEXTPA(NAMFIL,ISFICH)
      IADRES=ISFICH(1)
      NBOBJ=ISFICH(2)
      LBLOC=ISFICH(3)
      IUNIT=KDROPN(NAMFIL,2,4,LBLOC,IHANDL)
      IF(IUNIT.LE.0) THEN
         TEXT12=NAMFIL
         CALL XABORT('LIBA20: APOLLO-2 LIBRARY '//TEXT12//' CANNOT B'//
     1   'E OPENED')
      ENDIF
*----
*  INDEX THE APOLIB-2 FILE.
*----
      IDKNO=1-TKCARO(14)
      IDKTY=1-TKCARO(21)
      IDKDS=1-TKCARO(10)
      IDKTS=1-TKCARO(23)
      IDKDA=1-TKCARO(26)
      IDKNS=TKCARO(2)+1
      IDKLS=TKCARO(8)
      CALL SETARA(IBASE,NBOBJ-3,IKDS)
      CALL SETARA(IBASE,NBOBJ-3,ILGS)
      CALL SETARA(IBASE,7*(NBOBJ-3),INOMOB)
      CALL XDISET(IBASE(IKDS),NBOBJ-3,0)
      CALL XDISET(IBASE(ILGS),NBOBJ-3,0)
      CALL XDISET(IBASE(INOMOB),7*(NBOBJ-3),0)
      CALL LCMSIX(IPLIB,'INDEX',1)
      TEXT12=NAMFIL
      CALL LCMLEN(IPLIB,TEXT12,ILENG,ITYLCM)
      CALL LCMSIX(IPLIB,TEXT12,1)
      IF(ILENG.NE.0) THEN
*        RECOVER AN EXISTING INDEX.
         CALL LCMGET(IPLIB,'IPAR',IPAR)
         CALL LCMGET(IPLIB,'KDS',IBASE(IKDS))
         CALL LCMGET(IPLIB,'LGS',IBASE(ILGS))
         CALL LCMGET(IPLIB,'NOMOB',IBASE(INOMOB))
      ELSE
*        CREATE A NEW INDEX.
         CALL SETARA(IBASE,2*NBOBJ,IVINTE)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(IVINTE),IADRES,2*NBOBJ)
         NSEGM=0
         DO 10 IOBJ=3,NBOBJ
         IDKOBJ=IBASE(IVINTE+2*IOBJ-2)
         LGSEG=IBASE(IVINTE+2*IOBJ-1)+1
         CALL SETARA(IBASE,LGSEG,ITCARO)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
         IDK=IBASE(ITCARO+IDKNO-1)
         CALL AEXCPC(IDK,20,IBASE(ITCARO),NOMOBJ)
         IDK=IBASE(ITCARO+IDKTY-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPOBJ)
         JDKDS=IBASE(ITCARO+IDKDS-1)
         JDKTS=IBASE(ITCARO+IDKTS-1)
         NS=IBASE(ITCARO+IDKNS-1)
         IDK=IBASE(ITCARO+IDKDA-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TEXT8)
         IF(TYPOBJ.EQ.'APOLIB') THEN
            IPAR(2)=IDKOBJ
            IPAR(3)=LGSEG
         ELSE IF(TYPOBJ.EQ.'APOLIBE') THEN
            NSEGM=NSEGM+1
            ISO2=(NSEGM-1)*7+1
            CALL LCMCAR(NOMOBJ,.TRUE.,IBASE(INOMOB+ISO2-1))
            CALL LCMCAR(TEXT8,.TRUE.,IBASE(INOMOB+ISO2+4))
            IBASE(IKDS+NSEGM-1)=IDKOBJ
            IBASE(ILGS+NSEGM-1)=LGSEG
         ELSE
            CALL XABORT('LIBA20: WEIRD SEGMENT TYPE: '//TYPOBJ//'.')
         ENDIF
         CALL RLSARA(IBASE(ITCARO))
   10    CONTINUE
         CALL RLSARA(IBASE(IVINTE))
         IPAR(1)=NSEGM
*
*        SAVE THE INDEX.
         CALL LCMPUT(IPLIB,'IPAR',3,1,IPAR)
         CALL LCMPUT(IPLIB,'NOMOB',7*(NBOBJ-3),1,IBASE(INOMOB))
         CALL LCMPUT(IPLIB,'KDS',NBOBJ-3,1,IBASE(IKDS))
         CALL LCMPUT(IPLIB,'LGS',NBOBJ-3,1,IBASE(ILGS))
      ENDIF
      CALL LCMSIX(IPLIB,' ',2)
      CALL LCMSIX(IPLIB,' ',2)
*----
*  RECOVER GENERIC INFORMATION FROM THE APOLIB SEGMENT.
*----
      NSEGM=IPAR(1)
      IDKOBJ=IPAR(2)
      LGSEG=IPAR(3)
      NISOT=0
      NISOTS=0
      NAMASS=0
      IDKCOM=0
      CALL SETARA(IBASE,LGSEG,ITCARO)
      CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
      IDK=IBASE(ITCARO+IDKNO-1)
      CALL AEXCPC(IDK,20,IBASE(ITCARO),NOMOBJ)
      IDK=IBASE(ITCARO+IDKTY-1)
      CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPOBJ)
      JDKDS=IBASE(ITCARO+IDKDS-1)
      JDKTS=IBASE(ITCARO+IDKTS-1)
      NS=IBASE(ITCARO+IDKNS-1)
      IDK=IBASE(ITCARO+IDKDA-1)
      CALL AEXCPC(IDK,8,IBASE(ITCARO),TEXT8)
      IF(TYPOBJ.NE.'APOLIB') CALL XABORT('LIBA20: UNABLE TO FIND TH'//
     1 'E APOLIB SEGMENT.')
      DO 80 IS=1,NS
      IDK=JDKTS+8*(IS-1)
      CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
      LNGS=IBASE(ITCARO+IDKLS+IS-1)
      IF(LNGS.LE.0) GO TO 80
      JDKS=IBASE(ITCARO+JDKDS+IS-1)
      CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
      CALL SETARA(IBASE,LNGS+1,ITSEGM)
      CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
      IF(TYPSEG.EQ.'PHEAD') THEN
         CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         CALL AEXCPC(0,NV,IBASE(ITSEGM+IDK-1),TEXT80)
         IF((IMPX.GT.0).AND.(NV.GT.0)) WRITE (IOUT,810) TEXT80
         CALL AEXGNV(3,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         IF(NV.EQ.0) THEN
            TEXT12=NAMFIL
            CALL XABORT('LIBA20: NO ISOTOPES PRESENT ON APOLIB-2 FIL'//
     1      'E NAMED: '//TEXT12)
         ENDIF
         NISOT=NV/20
         CALL SETARA(IBASE,5*NISOT,INOM)
         IF(IMPX.GE.10) THEN
            WRITE(IOUT,'(/41H LIBA20: STANDARD ISOTOPE NAMES PRESENT I,
     1      10HN LIBRARY:)')
         ENDIF
         DO 20 ISO=1,NISOT
         ISO2=(ISO-1)*5+1
         CALL AEXCPC(0,20,IBASE(ITSEGM+IDK+ISO2-2),TEXT20)
         IF(IMPX.GE.10) WRITE(IOUT,'(8H -----> ,A20)') TEXT20
         CALL LCMCAR(TEXT20,.TRUE.,IBASE(INOM+ISO2-1))
   20    CONTINUE
         CALL AEXGNV(6,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         NISOTS=NV/20
         IF(NISOTS.GT.0) THEN
            CALL SETARA(IBASE,5*NISOTS,INOMS)
            IF(IMPX.GE.10) THEN
               WRITE(IOUT,'(/38H LIBA20: SELF-SHIELDED ISOTOPE NAMES P,
     1         18HRESENT IN LIBRARY:)')
            ENDIF
            DO 30 ISO=1,NISOTS
            ISO2=(ISO-1)*5+1
            CALL AEXCPC(0,20,IBASE(ITSEGM+IDK+ISO2-2),TEXT20)
            IF(IMPX.GE.10) WRITE(IOUT,'(8H -----> ,A20)') TEXT20
            CALL LCMCAR(TEXT20,.TRUE.,IBASE(INOMS+ISO2-1))
   30       CONTINUE
         ENDIF
      ELSE IF(TYPSEG.EQ.'PMAIL') THEN
         CALL AEXGNV(3,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         IF(NV-1.NE.NGRO) CALL XABORT('LIBA20: BAD GROUP STRUCTURE.')
         DO 40 IG=1,NV
   40    ENERG(IG)=BASE(ITSEGM+IDK+IG-2)*1.0E6
         CALL AEXGNV(5,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         IF(NV.EQ.NGRO) THEN
            DO 50 IG=1,NGRO
   50       DELTA(IG)=BASE(ITSEGM+IDK+IG-2)
         ELSE
            DO 60 IG=1,NGRO
   60       DELTA(IG)=LOG(ENERG(IG)/ENERG(IG+1))
         ENDIF
         CALL LCMPUT(IPLIB,'ENERGY',NGRO+1,2,ENERG)
         CALL LCMPUT(IPLIB,'DELTAU',NGRO,2,DELTA)
      ELSE IF(TYPSEG.EQ.'PCONST') THEN
         CALL AEXGNV(13,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NAMASS)
         CALL SETARA(BASE,NAMASS,IAMASS)
         DO 70 IA=1,NAMASS
   70    BASE(IAMASS+IA-1)=BASE(ITSEGM+IDK+IA-2)/ANEUT
      ELSE IF(TYPSEG.EQ.'PCOM') THEN
*        ISOTOPE-DEPENDENT COMMENTS ARE AVAILABLE.
         IDKCOM=IDKOBJ
         LGCOM=LGSEG
         ISCOM=IS
      ENDIF
      CALL RLSARA(IBASE(ITSEGM))
      CALL RLSARA(IBASE(ICHDIM))
      CALL RLSARA(IBASE(ICHTYP))
      CALL RLSARA(IBASE(ICHDKL))
   80 CONTINUE
      CALL RLSARA(IBASE(ITCARO))
      IF(NAMASS.NE.NISOT) CALL XABORT('LIBA20: INVALID AWR INFO.')
*----
*  SET THE CORRESPONDANCE BETWEEN THE APOLIB AND THE LIST OF ISOTOPES.
*----
      IF(IMPX.GT.1) WRITE(IOUT,820) NISOT,NISOTS,NSEGM
      CALL LIBA27(NAMFIL,NBISO,NISOT,NISOTS,NSEGM,NL,ISONRF,ISHINA,
     1 MASKI,IBASE(INOM),IBASE(INOMOB),IPR)
      CALL RLSARA(IBASE(INOM))
      IF(NISOTS.GT.0) CALL RLSARA(IBASE(INOMS))
      CALL KDRCPU(TK2)
      TKT(1)=TK2-TK1
*----
*  READ THROUGH APOLIB-2 FILE AND ACCUMULATE CROSS SECTIONS FOR THIS
*  RANGE OF MATS, LEGENDRE ORDERS, AND GROUPS.
*----
      CALL LCMGET(IPLIB,'ENERGY',ENERG)
      DO 560 IMX=1,NBISO
*----
*  PROCESS INFINITE DILUTION INFORMATION.
*----
      KISEG=IPR(2,IMX)
      IF(KISEG.GT.0) THEN
         IF(IMPX.GT.1) WRITE(IOUT,'(/29H LIBA20: PROCESSING ISOTOPE '',
     1   3A4,2H''.)') (ISONRF(I0,IMX),I0=1,3)
*
*        RECOVER THE ISOTOPE TITLE.
         CALL KDRCPU(TK1)
         IF(IDKCOM.EQ.0) THEN
*           MAKE A NEW TITLE.
            ISO2=(KISEG-1)*7+1
            CALL LCMCAR(NOMOBJ,.FALSE.,IBASE(INOMOB+ISO2-1))
            CALL LCMCAR(TEXT8,.FALSE.,IBASE(INOMOB+ISO2+4))
            TEXT80='APOLIB-2 ISOTOPE:'//NOMOBJ(7:)//TEXT8
         ELSE
*           RECOVER THE TITLE FROM THE PCOM SEGMENT.
            IF(IPR(1,IMX).LE.0) CALL XABORT('LIBA20: BAD TITLE.')
            CALL SETARA(IBASE,LGCOM,ITITLE)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITITLE),IDKCOM,LGCOM)
            JDKDS=IBASE(ITITLE+IDKDS-1)
            JDKTS=IBASE(ITITLE+IDKTS-1)
            NS=IBASE(ITITLE+IDKNS-1)
            IDK=JDKTS+8*(ISCOM-1)
            CALL AEXCPC(IDK,8,IBASE(ITITLE),TYPSEG)
            IF(TYPSEG.NE.'PCOM') CALL XABORT('LIBA20: SEGMENT ERROR.')
            LNGS=IBASE(ITITLE+IDKLS+ISCOM-1)
            IF(LNGS.LE.0) CALL XABORT('LIBA20: LENGTH ERROR.')
            JDKS=IBASE(ITITLE+JDKDS+ISCOM-1)
            CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
            CALL SETARA(IBASE,LNGS+1,ITSEGM)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            ISO2=(IPR(1,IMX)-1)*20+1
            CALL AEXCPC(0,NV,IBASE(ITSEGM+IDK+ISO2-2),TEXT80)
            CALL RLSARA(IBASE(ITSEGM))
            CALL RLSARA(IBASE(ICHDIM))
            CALL RLSARA(IBASE(ICHTYP))
            CALL RLSARA(IBASE(ICHDKL))
            CALL RLSARA(IBASE(ITITLE))
         ENDIF
         READ(TEXT80,'(20A4)') (ITEXT(I),I=1,20)
         IF(IMPX.GT.2) WRITE(IOUT,870) TEXT80
*
         IDKOBJ=IBASE(IKDS+KISEG-1)
         LGSEG=IBASE(ILGS+KISEG-1)
         CALL SETARA(IBASE,LGSEG,ITCARO)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
         IDK=IBASE(ITCARO+IDKNO-1)
         CALL AEXCPC(IDK,20,IBASE(ITCARO),NOMOBJ)
         IDK=IBASE(ITCARO+IDKTY-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPOBJ)
         JDKDS=IBASE(ITCARO+IDKDS-1)
         JDKTS=IBASE(ITCARO+IDKTS-1)
         NS=IBASE(ITCARO+IDKNS-1)
*----
*  RECOVER THE INFINITE DILUTION CROSS SECTION NUMEROTATION.
*----
         LPFIX=.FALSE.
         LTRAN=.FALSE.
         DO 160 IS=1,NS
         IDK=JDKTS+8*(IS-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
         LNGS=IBASE(ITCARO+IDKLS+IS-1)
         IF(LNGS.LE.0) GO TO 160
         JDKS=IBASE(ITCARO+JDKDS+IS-1)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
         IF(TYPSEG.EQ.'PFIX') THEN
            LPFIX=.TRUE.
            CALL AEXGNV(2,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            ZFISS=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(4,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            LGPROB=LBASE(ITSEGM+IDK-1)
            CALL AEXGNV(5,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            LGTDIF=LBASE(ITSEGM+IDK-1)
            CALL AEXGNV(6,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            LGTTRA=LBASE(ITSEGM+IDK-1)
            CALL AEXGNV(7,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            FGTD=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(8,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            ID2=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(12,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NSECTT)
            CALL SETARA(IBASE,NSECTT,IZSECT)
            NSETOT=0
            NPHY=MAX(0,NSECTT-5)
            DO 90 I=1,NSECTT
            IBASE(IZSECT+I-1)=IBASE(ITSEGM+IDK+I-2)
            IF((IBASE(IZSECT+I-1).NE.0).AND.(I.LE.5)) NSETOT=NSETOT+1
   90       CONTINUE
            IF(IMPX.GT.2) WRITE(IOUT,875) (IBASE(IZSECT+I-1),I=1,NSECTT)
            CALL AEXGNV(14,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NANISD)
            CALL AEXGNV(16,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NANIST)
            CALL AEXGNV(18,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NSECTT)
            CALL SETARA(LBASE,NSECTT,ILGTRE)
            DO 100 I=1,NSECTT
  100       LBASE(ILGTRE+I-1)=LBASE(ITSEGM+IDK+I-2)
            CALL AEXGNV(24,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NTEMP)
            CALL SETARA(BASE,NTEMP,ITEMP)
            DO 110 I=1,NTEMP
  110       BASE(ITEMP+I-1)=BASE(ITSEGM+IDK+I-2)
            IF(IMPX.GT.2) WRITE(IOUT,880) (BASE(ITEMP+I-1),I=1,NTEMP)
            CALL AEXGNV(26,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            IF(NV/8.NE.NSECTT) CALL XABORT('LIBA20: INVALID TYPSECT.')
            CALL SETARA(IBASE,2*NSECTT,ISECTT)
            III=0
            DO 120 I=1,NSECTT
            I2=(I-1)*2+1
            IF(IBASE(IZSECT+I-1).NE.0) THEN
               III=III+1
               I3=(III-1)*2+1
               CALL AEXCPC(0,8,IBASE(ITSEGM+IDK+I2-2),TEXT8)
               CALL LCMCAR(TEXT8,.TRUE.,IBASE(ISECTT+I3-1))
            ENDIF
  120       CONTINUE
            IF(IMPX.GT.2) WRITE(IOUT,890) ZFISS,LGPROB,LGTDIF,LGTTRA,
     1      FGTD,ID2,NSECTT,NSETOT,NPHY,NANISD,NANIST,
     2      (LBASE(ILGTRE+I-1),I=1,NSECTT)
            IF(NANIST.GT.NANISD) CALL XABORT('LIBA20: NANIST.GT.NANISD')
         ELSE IF(TYPSEG.EQ.'PPPSN') THEN
            LTRAN=.TRUE.
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            FAGG=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(2,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            LAGG=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(3,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            FDGG=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(4,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            WGAL=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(5,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            FAG=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(6,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            LAG=IBASE(ITSEGM+IDK-1)
            CALL AEXGNV(7,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            IF(NV.NE.NGRO) CALL XABORT('LIBA20: INVALID LIBRARY(1).')
            CALL SETARA(IBASE,NV,IFDG)
            DO 130 I=1,NV
  130       IBASE(IFDG+I-1)=IBASE(ITSEGM+IDK+I-2)
            CALL AEXGNV(9,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            IF(NV.NE.NGRO+1) CALL XABORT('LIBA20: INVALID LIBRARY(2).')
            CALL SETARA(IBASE,NV,IIAD)
            DO 140 I=1,NV
  140       IBASE(IIAD+I-1)=IBASE(ITSEGM+IDK+I-2)
            CALL AEXGNV(11,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NGTD)
            CALL SETARA(IBASE,NGTD,IDEPL)
            DO 150 I=1,NGTD
  150       IBASE(IDEPL+I-1)=IBASE(ITSEGM+IDK+I-2)
            IF(IMPX.GT.2) WRITE(IOUT,900) FAGG,LAGG,FDGG,WGAL,FAG,LAG,
     1      NGTD
         ENDIF
         CALL RLSARA(IBASE(ITSEGM))
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
  160    CONTINUE
         IF(.NOT.LPFIX) CALL XABORT('LIBA20: NO PFIX SEGMENT.')
*----
*  RECOVER THE INFINITE DILUTION CROSS SECTIONS.
*----
         ITSEC=0
         NDIFG=0
         DO 220 IS=1,NS
         IDK=JDKTS+8*(IS-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
         LNGS=IBASE(ITCARO+IDKLS+IS-1)
         IF(LNGS.LE.0) GO TO 220
         JDKS=IBASE(ITCARO+JDKDS+IS-1)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
         CALL SETARA(BASE,NTEMP,IWEIJH)
         IF(TYPSEG.EQ.'PSECT') THEN
*           RECOVER A VECTOR CROSS SECTION.
            ITSEC=ITSEC+1
            IF(ITSEC.GT.NXSMAX) THEN
               CALL XABORT('LIBA20: SECT OVERFLOW.')
            ELSE IF(ITSEC.LE.NSETOT) THEN
               I3=(ITSEC-1)*2+1
               CALL LCMCAR(TEXT8,.FALSE.,IBASE(ISECTT+I3-1))
            ELSE IF(ITSEC.EQ.NSETOT+1) THEN
               TEXT8='SIGS00'
            ELSE IF(ITSEC.GT.NSETOT+1) THEN
               CALL XABORT('LIBA20: UNKNOWN CROSS SECTION TYPE(1).')
            ENDIF
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            DO 190 JMX=IMX,NBISO
            IF(IPR(2,JMX).EQ.KISEG) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
               CALL LCMLEN(IPLIB,HNAMIS,ILENG,ITYLCM)
               CALL LCMSIX(IPLIB,HNAMIS,1)
               IF(ILENG.EQ.0) THEN
                  IF(IPR(1,JMX).LE.0) CALL XABORT('LIBA20: BAD AWR.')
                  CALL LCMPUT(IPLIB,'AWR',1,2,BASE(IAMASS+IPR(1,JMX)-1))
                  CALL LCMPUT(IPLIB,'README',20,3,ITEXT)
               ENDIF
               IF(ITSEC.EQ.1) THEN
                  CALL XDRSET(SIGS,NGRO,0.0)
                  CALL LCMPUT(IPLIB,'SIGS00',NGRO,2,SIGS)
                  CALL LCMPUT(IPLIB,'NTOT0',NGRO,2,SIGS)
               ENDIF
               CALL LIBA22(NGRO,TN(JMX),NTEMP,NV,FGTD,BASE(ITEMP),
     2         BASE(ITSEGM+IDK-1),SECT,BASE(IWEIJH))
               IF(TEXT8.EQ.'SIGA') THEN
                  CALL LCMPUT(IPLIB,'NTOT0',NGRO,2,SECT)
               ELSE IF(TEXT8.EQ.'NEXCESS') THEN
                  LN2N=.FALSE.
                  DO 170 IG=1,NGRO
  170             LN2N=LN2N.OR.(SECT(IG).NE.0.0)
                  IF(LN2N) THEN
                     CALL LCMPUT(IPLIB,'N2N',NGRO,2,SECT)
                     CALL LCMPUT(IPLIB,'SIGS00',NGRO,2,SECT)
                  ENDIF
               ELSE IF(TEXT8.EQ.'SIGF') THEN
                  CALL LCMPUT(IPLIB,'NFTOT',NGRO,2,SECT)
               ELSE IF(TEXT8.EQ.'NUSIGF') THEN
                  CALL LCMPUT(IPLIB,'NUSIGF',NGRO,2,SECT)
               ELSE IF(TEXT8.EQ.'CHI') THEN
                  CALL LCMPUT(IPLIB,'CHI',NGRO,2,SECT)
               ELSE IF(TEXT8.EQ.'SIGS00') THEN
                  CALL LCMGET(IPLIB,'NTOT0',XSTOT)
                  CALL LCMGET(IPLIB,'SIGS00',SIGS)
                  NDIFG=NV
                  DO 180 IG=1,NGRO
                  XSTOT(IG)=XSTOT(IG)+SECT(IG)
  180             SIGS(IG,1)=SIGS(IG,1)+SECT(IG)
                  CALL LCMPUT(IPLIB,'NTOT0',NGRO,2,XSTOT)
                  CALL LCMPUT(IPLIB,'SIGS00',NGRO,2,SIGS)
               ELSE
                  CALL XABORT('LIBA20: UNKNOWN X-S TYPE:'//TEXT8)
               ENDIF
               CALL LCMSIX(IPLIB,' ',2)
            ENDIF
  190       CONTINUE
         ELSE IF(TYPSEG.EQ.'PPSN') THEN
*           RECOVER A MATRIX CROSS SECTION.
            IF(.NOT.LTRAN) CALL XABORT('LIBA20: PPPSN MISSING.')
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            NPSN=NV
            DO 210 JMX=IMX,NBISO
            IF(IPR(2,JMX).EQ.KISEG) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
               CALL LCMSIX(IPLIB,HNAMIS,1)
               CALL XDRSET(SCAT,NGRO*NGRO,0.0)
               CALL LIBA23(NGRO,1,TN(JMX),NTEMP,NGTD,NV,BASE(ITEMP),
     1         FGTD,ID2,FAGG,LAGG,FDGG,WGAL,FAG,LAG,IBASE(IFDG),
     2         IBASE(IIAD),IBASE(IDEPL),BASE(ITSEGM+IDK-1),SCAT,
     3         BASE(IWEIJH))
               CALL LCMGET(IPLIB,'SIGS00',SIGS)
               IF(LGPROB) THEN
                  DO 200 IG=1,NGRO
                  DO 200 JG=1,NGRO
  200             SCAT(JG,IG,1)=SCAT(JG,IG,1)*SIGS(IG,1)
               ENDIF
               CALL SETARA(BASE,NGRO*NGRO,IXSSCM)
               CALL SETARA(BASE,NGRO,INJJ)
               CALL SETARA(BASE,NGRO,IIJJ)
               CALL XDRLGS(IPLIB,1,IMPX,0,0,1,NGRO,SIGS,SCAT,ITYPRO,
     1         BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
               CALL RLSARA(BASE(IIJJ))
               CALL RLSARA(BASE(INJJ))
               CALL RLSARA(BASE(IXSSCM))
               CALL LCMSIX(IPLIB,' ',2)
            ENDIF
  210       CONTINUE
         ENDIF
         CALL RLSARA(BASE(IWEIJH))
         CALL RLSARA(IBASE(ITSEGM))
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
  220    CONTINUE
         CALL RLSARA(IBASE(ITCARO))
         DO 240 JMX=IMX,NBISO
         IF(IPR(2,JMX).EQ.KISEG) THEN
            IF(.NOT.LTRAN) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
               CALL LCMSIX(IPLIB,HNAMIS,1)
               CALL LCMGET(IPLIB,'SIGS00',SIGS)
               CALL XDRSET(SCAT,NGRO*NGRO,0.0)
               DO 230 IG=1,NGRO
  230          SCAT(IG,IG,1)=SIGS(IG,1)
               CALL SETARA(BASE,NGRO*NGRO,IXSSCM)
               CALL SETARA(BASE,NGRO,INJJ)
               CALL SETARA(BASE,NGRO,IIJJ)
               CALL XDRLGS(IPLIB,1,IMPX,0,0,1,NGRO,SIGS,SCAT,ITYPRO,
     1         BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
               CALL RLSARA(BASE(IIJJ))
               CALL RLSARA(BASE(INJJ))
               CALL RLSARA(BASE(IXSSCM))
               CALL LCMSIX(IPLIB,' ',2)
            ENDIF
            IPR(2,JMX)=0
         ENDIF
  240    CONTINUE
         CALL KDRCPU(TK2)
         TKT(2)=TKT(2)+(TK2-TK1)
*----
*  RECOVER SCATTERING X-S FOR HIGHER LEGENDRE ORDERS.
*----
         CALL KDRCPU(TK1)
         DO 270 IL=2,MIN(NANISD,NL)
         WRITE(TEXT2,'(I2.2)') IL-1
         KISEG=IPR(7+(IL-1),IMX)
         IF(KISEG.EQ.0) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
            WRITE(HNISOR,'(3A4)') (ISONRF(I0,IMX),I0=1,3)
            WRITE(HSMG,830) IL-1,HNAMIS,HNISOR,NAMFIL
            CALL XABORT(HSMG)
         ENDIF
         IDKOBJ=IBASE(IKDS+KISEG-1)
         LGSEG=IBASE(ILGS+KISEG-1)
         CALL SETARA(IBASE,LGSEG,ITCARO)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
         LDKDS=IBASE(ITCARO+IDKDS-1)
         LDKTS=IBASE(ITCARO+IDKTS-1)
         IF(IBASE(ITCARO+IDKNS-1).NE.1) CALL XABORT('LIBA20: INVALID DI'
     1   //'FF(1).')
         CALL AEXCPC(LDKTS,8,IBASE(ITCARO),TYPSEG)
         IF(TYPSEG.NE.'PSECT') CALL XABORT('LIBA20: INVALID DIFF(2).')
         LNGS=IBASE(ITCARO+IDKLS)
         IF(LNGS.LE.0) CALL XABORT('LIBA20: INVALID DIFF(3).')
         LDKS=IBASE(ITCARO+LDKDS)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),LDKS,LNGS+1)
         CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         IF(NV.NE.NDIFG) CALL XABORT('LIBA20: INVALID DIFF(4).')
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
         CALL RLSARA(IBASE(ITCARO))
         DO 260 JMX=IMX,NBISO
         IF(IPR(7+(IL-1),JMX).EQ.KISEG) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
            CALL LCMSIX(IPLIB,HNAMIS,1)
            CALL SETARA(BASE,NTEMP,IWEIJH)
            CALL LIBA22(NGRO,TN(JMX),NTEMP,NV,FGTD,BASE(ITEMP),
     1      BASE(ITSEGM+IDK-1),SIGS,BASE(IWEIJH))
            CALL RLSARA(BASE(IWEIJH))
            CALL LCMPUT(IPLIB,'SIGS'//TEXT2,NGRO,2,SIGS)
            IF(IL.GT.NANIST) THEN
               CALL XDRSET(SCAT,NGRO*NGRO,0.0)
               DO 250 IG=1,NGRO
  250          SCAT(IG,IG,1)=SIGS(IG,1)
               CALL SETARA(BASE,NGRO*NGRO,IXSSCM)
               CALL SETARA(BASE,NGRO,INJJ)
               CALL SETARA(BASE,NGRO,IIJJ)
               CALL XDRLGS(IPLIB,1,IMPX,IL-1,IL-1,1,NGRO,SIGS,SCAT,
     1         ITYPRO,BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
               CALL RLSARA(BASE(IIJJ))
               CALL RLSARA(BASE(INJJ))
               CALL RLSARA(BASE(IXSSCM))
            ENDIF
            CALL LCMSIX(IPLIB,' ',2)
            IPR(7+(IL-1),JMX)=0
         ENDIF
  260    CONTINUE
         CALL RLSARA(BASE(ITSEGM))
  270    CONTINUE
*----
*  RECOVER TRANSFER MATRICES FOR HIGHER LEGENDRE ORDERS.
*----
         DO 300 IL=2,MIN(NANIST,NL)
         WRITE(TEXT2,'(I2.2)') IL-1
         KISEG=IPR(7+(NL-1)+(IL-1),IMX)
         IF(KISEG.EQ.0) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
            WRITE(HNISOR,'(3A4)') (ISONRF(I0,IMX),I0=1,3)
            WRITE(HSMG,830) IL-1,HNAMIS,HNISOR,NAMFIL
            CALL XABORT(HSMG)
         ENDIF
         IDKOBJ=IBASE(IKDS+KISEG-1)
         LGSEG=IBASE(ILGS+KISEG-1)
         CALL SETARA(IBASE,LGSEG,ITCARO)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
         LDKDS=IBASE(ITCARO+IDKDS-1)
         LDKTS=IBASE(ITCARO+IDKTS-1)
         IF(IBASE(ITCARO+IDKNS-1).NE.1) CALL XABORT('LIBA20: INVALID TR'
     1   //'AN(1).')
         CALL AEXCPC(LDKTS,8,IBASE(ITCARO),TYPSEG)
         IF(TYPSEG.NE.'PPSN') CALL XABORT('LIBA20: INVALID TRAN(2).')
         LNGS=IBASE(ITCARO+IDKLS)
         IF(LNGS.LE.0) CALL XABORT('LIBA20: INVALID TRAN(3).')
         LDKS=IBASE(ITCARO+LDKDS)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),LDKS,LNGS+1)
         CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         IF(NV.NE.NPSN) CALL XABORT('LIBA20: INVALID TRAN(4).')
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
         CALL RLSARA(IBASE(ITCARO))
         DO 290 JMX=IMX,NBISO
         IF(IPR(7+(NL-1)+(IL-1),JMX).EQ.KISEG) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
            CALL LCMSIX(IPLIB,HNAMIS,1)
            CALL SETARA(BASE,NTEMP,IWEIJH)
            CALL LIBA23(NGRO,IL,TN(JMX),NTEMP,NGTD,NV,BASE(ITEMP),FGTD,
     1      ID2,FAGG,LAGG,FDGG,WGAL,FAG,LAG,IBASE(IFDG),IBASE(IIAD),
     2      IBASE(IDEPL),BASE(ITSEGM+IDK-1),SCAT,BASE(IWEIJH))
            CALL RLSARA(BASE(IWEIJH))
            CALL LCMGET(IPLIB,'SIGS'//TEXT2,SIGS)
            IF(LGPROB) THEN
               DO 280 IG=1,NGRO
               DO 280 JG=1,NGRO
  280          SCAT(JG,IG,1)=SCAT(JG,IG,1)*SIGS(IG,1)
            ENDIF
            CALL SETARA(BASE,NGRO*NGRO,IXSSCM)
            CALL SETARA(BASE,NGRO,INJJ)
            CALL SETARA(BASE,NGRO,IIJJ)
            CALL XDRLGS(IPLIB,1,IMPX,IL-1,IL-1,1,NGRO,SIGS,SCAT,ITYPRO,
     1      BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
            CALL RLSARA(BASE(IIJJ))
            CALL RLSARA(BASE(INJJ))
            CALL RLSARA(BASE(IXSSCM))
            CALL LCMSIX(IPLIB,' ',2)
            IPR(7+(NL-1)+(IL-1),JMX)=0
         ENDIF
  290    CONTINUE
         CALL RLSARA(BASE(ITSEGM))
  300    CONTINUE
         CALL KDRCPU(TK2)
         TKT(3)=TKT(3)+(TK2-TK1)
*----
*  RECOVER A PRODUCTION X-S.
*----
         CALL KDRCPU(TK1)
         IF(NPHY.GE.1) THEN
            KISEG=IPR(3,IMX)
            IF(KISEG.EQ.0) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
               WRITE(HNISOR,'(3A4)') (ISONRF(I0,IMX),I0=1,3)
               WRITE(HSMG,840) HNAMIS,HNISOR,NAMFIL
               CALL XABORT(HSMG)
            ENDIF
            IDKOBJ=IBASE(IKDS+KISEG-1)
            LGSEG=IBASE(ILGS+KISEG-1)
            CALL SETARA(IBASE,LGSEG,ITCARO)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
            LDKDS=IBASE(ITCARO+IDKDS-1)
            LDKTS=IBASE(ITCARO+IDKTS-1)
            NS=IBASE(ITCARO+IDKNS-1)
            IF(NS.NE.NPHY) CALL XABORT('LIBA20: INVALID PRODUCTION X-S('
     1      //'1).')
         ENDIF
         DO 320 IPHY=1,NPHY
         IDK=LDKTS+8*(IPHY-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
         IF(TYPSEG.NE.'PSECT') CALL XABORT('LIBA20: INVALID PRODUCTI'
     1   //'ON X-S(2).')
         LNGS=IBASE(ITCARO+IDKLS+IPHY-1)
         IF(LNGS.LE.0) CALL XABORT('LIBA20: INVALID PRODUCTION X-S(3).')
         LDKS=IBASE(ITCARO+LDKDS+IPHY-1)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),LDKS,LNGS+1)
         CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1   IBASE(ICHDKL),IDK,NV)
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
         I3=(NSETOT+IPHY-1)*2+1
         CALL LCMCAR(TEXT8,.FALSE.,IBASE(ISECTT+I3-1))
         DO 310 JMX=IMX,NBISO
         IF(IPR(3,JMX).EQ.KISEG) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
            CALL LCMSIX(IPLIB,HNAMIS,1)
            CALL SETARA(BASE,NTEMP,IWEIJH)
            CALL LIBA22(NGRO,TN(JMX),NTEMP,NV,FGTD,BASE(ITEMP),
     1      BASE(ITSEGM+IDK-1),SECT,BASE(IWEIJH))
            CALL RLSARA(BASE(IWEIJH))
            IF(TEXT8.EQ.'CREA-P') THEN
               TEXT8='NP'
            ELSE IF (TEXT8.EQ.'CREA-H2') THEN
               TEXT8='ND'
            ELSE IF (TEXT8.EQ.'CREA-H3') THEN
               TEXT8='NT'
            ENDIF
            CALL LCMPUT(IPLIB,TEXT8,NGRO,2,SECT)
            CALL LCMSIX(IPLIB,' ',2)
         ENDIF
  310    CONTINUE
         CALL RLSARA(BASE(ITSEGM))
  320    CONTINUE
         DO 330 JMX=IMX,NBISO
         IF(IPR(3,JMX).EQ.KISEG) IPR(3,JMX)=0
  330    CONTINUE
         IF(NPHY.GE.1) CALL RLSARA(IBASE(ITCARO))
         CALL RLSARA(IBASE(ISECTT))
*----
*  RECOVER DELAYED NEUTRON DATA.
*----
         KISEG=IPR(4,IMX)
         IF(KISEG.GT.0) THEN
            IDKOBJ=IBASE(IKDS+KISEG-1)
            LGSEG=IBASE(ILGS+KISEG-1)
            CALL SETARA(IBASE,LGSEG,ITCARO)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
            IDK=IBASE(ITCARO+IDKNO-1)
            CALL AEXCPC(IDK,20,IBASE(ITCARO),NOMOBJ)
            IDK=IBASE(ITCARO+IDKTY-1)
            CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPOBJ)
            JDKDS=IBASE(ITCARO+IDKDS-1)
            JDKTS=IBASE(ITCARO+IDKTS-1)
            NS=IBASE(ITCARO+IDKNS-1)
            ICHI=0
            IPWD=0
            IPED=0
            DO 350 IS=1,NS
            IDK=JDKTS+8*(IS-1)
            CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
            LNGS=IBASE(ITCARO+IDKLS+IS-1)
            IF(LNGS.LE.0) GO TO 350
            JDKS=IBASE(ITCARO+JDKDS+IS-1)
            CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
            CALL SETARA(IBASE,LNGS+1,ITSEGM)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,ILENG)
            CALL RLSARA(IBASE(ICHDIM))
            CALL RLSARA(IBASE(ICHTYP))
            CALL RLSARA(IBASE(ICHDKL))
            IF(TYPSEG.EQ.'.R1 RCHI') THEN
               ICHI=ICHI+1
            ELSE IF(TYPSEG.EQ.'.R1 RRBE') THEN
               NDEL0=ILENG
               NDEL=MAX(NDEL,NDEL0)
               IPWD0=ITSEGM
               IPWD=ITSEGM+IDK-1
               GO TO 350
            ELSE IF(TYPSEG.EQ.'.R1 RBET') THEN
               IPED0=ITSEGM
               IPED=ITSEGM+IDK-1
               GO TO 350
            ENDIF
            DO 340 JMX=IMX,NBISO
            IF(IPR(4,JMX).EQ.KISEG) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
               CALL LCMSIX(IPLIB,HNAMIS,1)
               IF(TYPSEG.EQ.'.R1 RLAM') THEN
                  CALL LCMPUT(IPLIB,'LAMBDA-D',ILENG,2,
     1            BASE(ITSEGM+IDK-1))
                  NDEL0=ILENG
                  NDEL=MAX(NDEL,NDEL0)
               ELSE IF(TYPSEG.EQ.'.R1 RCHI') THEN
                  WRITE(TEXT2,'(I2.2)') ICHI
                  CALL LCMPUT(IPLIB,'CHI'//TEXT2,ILENG,2,
     1            BASE(ITSEGM+IDK-1))
               ENDIF
               CALL LCMSIX(IPLIB,' ',2)
            ENDIF
  340       CONTINUE
            CALL RLSARA(IBASE(ITSEGM))
  350       CONTINUE
            CALL RLSARA(IBASE(ITCARO))
            IF((IPWD0.NE.0).AND.(IPED0.NE.0)) THEN
               DO 390 JMX=IMX,NBISO
               IF(IPR(4,JMX).EQ.KISEG) THEN
                  WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
                  CALL LCMSIX(IPLIB,HNAMIS,1)
                  DO 380 IDEL=1,NDEL0
                  WRITE(TEXT2,'(I2.2)') IDEL
                  CALL LCMGET(IPLIB,'NUSIGF',SECT)
                  DO 370 IGR=1,NGRO
  370             SECT(IGR)=SECT(IGR)*BASE(IPWD+IDEL-1)*BASE(IPED+IGR-1)
                  CALL LCMPUT(IPLIB,'NUSIGF'//TEXT2,NGRO,2,SECT)
  380             CONTINUE
                  CALL LCMSIX(IPLIB,' ',2)
               ENDIF
  390          CONTINUE
            ENDIF
            IF(IPWD0.NE.0) CALL RLSARA(IBASE(IPWD0))
            IF(IPED0.NE.0) CALL RLSARA(IBASE(IPED0))
            DO 400 JMX=IMX,NBISO
            IF(IPR(4,JMX).EQ.KISEG) IPR(4,JMX)=0
  400       CONTINUE
         ENDIF
*----
*  RELEASE ALLOCATED MEMORY FOR THE CURRENT ISOTOPE.
*----
         IF(LTRAN) CALL RLSARA(IBASE(IDEPL))
         IF(LTRAN) CALL RLSARA(IBASE(IIAD))
         IF(LTRAN) CALL RLSARA(IBASE(IFDG))
         CALL RLSARA(BASE(ITEMP))
         CALL RLSARA(IBASE(IZSECT))
         CALL RLSARA(LBASE(ILGTRE))
         CALL KDRCPU(TK2)
         TKT(2)=TKT(2)+(TK2-TK1)
         IF((IMPX.GT.9).AND.(IPR(5,IMX).EQ.0)) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
            CALL LCMSIX(IPLIB,HNAMIS,1)
            CALL LCMLIB(IPLIB)
            CALL LCMSIX(IPLIB,' ',2)
         ENDIF
      ENDIF
*----
*  PROCESS SELF-SHIELDING INFORMATION.
*----
      KISEG=IPR(5,IMX)
      IF(KISEG.GT.0) THEN
         CALL KDRCPU(TK1)
         IDKOBJ=IBASE(IKDS+KISEG-1)
         LGSEG=IBASE(ILGS+KISEG-1)
         CALL SETARA(IBASE,LGSEG,ITCARO)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
         IDK=IBASE(ITCARO+IDKNO-1)
         CALL AEXCPC(IDK,20,IBASE(ITCARO),NOMOBJ)
         IDK=IBASE(ITCARO+IDKTY-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPOBJ)
         JDKDS=IBASE(ITCARO+IDKDS-1)
         JDKTS=IBASE(ITCARO+IDKTS-1)
         NS=IBASE(ITCARO+IDKNS-1)
*----
*  RECOVER THE SELF-SHIELDED CROSS SECTION NUMEROTATION.
*----
         LPTHOM=.FALSE.
         DO 440 IS=1,NS
         IDK=JDKTS+8*(IS-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
         LNGS=IBASE(ITCARO+IDKLS+IS-1)
         IF(LNGS.LE.0) GO TO 440
         JDKS=IBASE(ITCARO+JDKDS+IS-1)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
         IF(TYPSEG.EQ.'PTHOM1') THEN
            LPTHOM=.TRUE.
            CALL AEXGNV(4,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NTHOMO)
            IF(NTHOMO.GT.MAXHOM) CALL XABORT('LIBA20: ITHOMO OVERFLOW.')
            DO 410 I=1,NTHOMO
  410       ITHOMO(I)=IBASE(ITSEGM+IDK+I-2)
            FGHOMO=ITHOMO(1)
            LGHOMO=ITHOMO(2)
            FGRESO=ITHOMO(3)
            NGHOMO=LGHOMO-FGHOMO+1
            NGF=MIN(NGF,FGHOMO)
            NGFR=MAX(NGFR,LGHOMO)
            L104=.FALSE.
            IF(NTHOMO.GE.9) L104=ITHOMO(9).NE.0
            CALL AEXGNV(13,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NBIN)
            CALL AEXGNV(16,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NSEQHO)
            CALL SETARA(BASE,NSEQHO,ISEQHO)
            DO 420 I=1,NSEQHO
  420       BASE(ISEQHO+I-1)=BASE(ITSEGM+IDK+I-2)
            CALL AEXGNV(22,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NTEMPS)
            CALL SETARA(IBASE,NTEMPS,ITEMPS)
            DO 430 I=1,NTEMPS
  430       BASE(ITEMPS+I-1)=BASE(ITSEGM+IDK+I-2)
            IF(IMPX.GT.1) THEN
               WRITE(IOUT,910) (BASE(ISEQHO+I-1),I=1,NSEQHO)
               WRITE(IOUT,920) (BASE(ITEMPS+I-1),I=1,NTEMPS)
               WRITE(IOUT,930) FGHOMO,FGRESO,NGHOMO,NSEQHO,NTEMPS,
     1         L104,NBIN
            ENDIF
         ENDIF
         CALL RLSARA(IBASE(ITSEGM))
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
  440    CONTINUE
         IF(.NOT.LPTHOM) CALL XABORT('LIBA20: NO PTHOM1 SEGMENT.')
         LENGTH=NGHOMO*NSEQHO*NTEMPS
         IF(LENGTH.EQ.0) THEN
            CALL RLSARA(BASE(ISEQHO))
            CALL RLSARA(IBASE(ITCARO))
            DO 450 JMX=IMX,NBISO
            IF(IPR(5,JMX).EQ.KISEG) IPR(5,JMX)=0
  450       CONTINUE
            GO TO 550
         ENDIF
         CALL SETARA(BASE,6*NGHOMO,ITAUX)
*----
*  RECOVER THE SELF-SHIELDED FLUX (REACTION 104).
*----
         IF(L104) THEN
            KISEG=IPR(6,IMX)
            IF(KISEG.EQ.0) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
               WRITE(HNISSS,'(3A4)') (ISHINA(I0,IMX),I0=1,3)
               WRITE(HSMG,850) HNAMIS,HNISSS,NAMFIL,1
               CALL XABORT(HSMG)
            ENDIF
            IDKOBJ=IBASE(IKDS+KISEG-1)
            LGSEG=IBASE(ILGS+KISEG-1)
            CALL SETARA(IBASE,LGSEG,ITC104)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITC104),IDKOBJ,LGSEG)
            LDKDS=IBASE(ITC104+IDKDS-1)
            LDKTS=IBASE(ITC104+IDKTS-1)
            IF(IBASE(ITC104+IDKNS-1).NE.1) CALL XABORT('LIBA20: IN'
     1      //'VALID FL104(1).')
            CALL AEXCPC(LDKTS,8,IBASE(ITC104),TYPSEG)
            IF(TYPSEG.NE.'.R3 TXSS') CALL XABORT('LIBA20: INVALID '
     1      //'FL104(2).')
            LNGS=IBASE(ITC104+IDKLS)
            IF(LNGS.LE.0) CALL XABORT('LIBA20: INVALID FL104(3).')
            LDKS=IBASE(ITC104+LDKDS)
            CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
            CALL SETARA(IBASE,LNGS+1,ITS104)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITS104),LDKS,LNGS+1)
            CALL AEXGNV(1,IBASE(ITS104),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDK,NV)
            IF(NV.NE.LENGTH) CALL XABORT('LIBA20: INVALID FL104(4).')
            IFL104=ITS104+IDK-1
            CALL RLSARA(IBASE(ICHDIM))
            CALL RLSARA(IBASE(ICHTYP))
            CALL RLSARA(IBASE(ICHDKL))
            CALL RLSARA(IBASE(ITC104))
         ENDIF
*----
*  RECOVER THE SELF-SHIELDED EFFECTIVE RATES.
*----
         LPTHOM=.FALSE.
         KISEG=IPR(5,IMX)
         DO 470 IS=1,NS
         IDK=JDKTS+8*(IS-1)
         CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
         LNGS=IBASE(ITCARO+IDKLS+IS-1)
         IF(LNGS.LE.0) GO TO 470
         JDKS=IBASE(ITCARO+JDKDS+IS-1)
         CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
         CALL SETARA(IBASE,LNGS+1,ITSEGM)
         CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
         IF(TYPSEG.EQ.'PTHOM2') THEN
            LPTHOM=.TRUE.
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDKA,NV)
            LABS=NV.EQ.LENGTH
            CALL AEXGNV(5,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDKD,NV)
            LDIF=NV.EQ.LENGTH
            CALL AEXGNV(9,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDKF,NV)
            LFIS=NV.EQ.LENGTH
            DO 460 JMX=IMX,NBISO
            IF(IPR(5,JMX).EQ.KISEG) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
               CALL LCMSIX(IPLIB,HNAMIS,1)
               CALL SETARA(BASE,NTEMPS,IWEIJH)
               CALL SETARA(BASE,NTEMPS,ISQRTE)
               CALL SETARA(BASE,NGHOMO*NSEQHO,IABSOH)
               CALL SETARA(BASE,NGHOMO*NSEQHO,IDIFFH)
               CALL SETARA(BASE,NGHOMO*NSEQHO,IFISSH)
               CALL SETARA(BASE,NGHOMO*NSEQHO,IFLUXH)
               CALL LIBA24(NGRO,FGHOMO,NGHOMO,NSEQHO,NTEMPS,LDIF,
     1         LFIS,L104,BASE(ISEQHO),BASE(ITEMPS),TN(JMX),SN(1,JMX),
     2         BASE(ITSEGM+IDKA-1),BASE(ITSEGM+IDKD-1),
     3         BASE(ITSEGM+IDKF-1),BASE(IFL104),IMPX,BASE(ITAUX),
     4         BASE(IWEIJH),BASE(ISQRTE),BASE(IABSOH),BASE(IDIFFH),
     5         BASE(IFISSH),BASE(IFLUXH))
               CALL RLSARA(BASE(IFLUXH))
               CALL RLSARA(BASE(IFISSH))
               CALL RLSARA(BASE(IDIFFH))
               CALL RLSARA(BASE(IABSOH))
               CALL RLSARA(BASE(ISQRTE))
               CALL RLSARA(BASE(IWEIJH))
*
*              COMPUTE THE SELF-SHIELDED FLUX AND CROSS SECTIONS.
               CALL SETARA(BASE,2*NGRO,ISS1)
               CALL LIBA25(IPLIB,LABS,LDIF,LFIS,L104,NGRO,FGHOMO,
     1         NGHOMO,NSEQHO,NL,BASE(ISEQHO),SN(1,JMX),SB(1,JMX),
     2         DELTA,ISONAM(1,JMX),BASE(ITAUX),IMPX,SECT,SIGS,SCAT,
     3         ITYPRO,XSTOT,BASE(ISS1),BASE(ISS1+NGRO))
               CALL RLSARA(BASE(ISS1))
               CALL LCMSIX(IPLIB,' ',2)
               IPR(5,JMX)=0
               IPR(6,JMX)=0
            ENDIF
  460       CONTINUE
         ENDIF
         CALL RLSARA(IBASE(ITSEGM))
         CALL RLSARA(IBASE(ICHDIM))
         CALL RLSARA(IBASE(ICHTYP))
         CALL RLSARA(IBASE(ICHDKL))
  470    CONTINUE
         IF(.NOT.LPTHOM) CALL XABORT('LIBA20: NO PTHOM2 SEGMENT.')
         IF(L104) CALL RLSARA(BASE(ITS104))
         CALL RLSARA(BASE(ISEQHO))
         CALL RLSARA(IBASE(ITCARO))
         CALL KDRCPU(TK2)
         TKT(4)=TKT(4)+(TK2-TK1)
*----
*  RECOVER THE AUTOLIB (BIN CROSS SECTIONS) INFORMATION.
*----
         IF((NBIN.GT.0).AND.(IPROC.GE.3)) THEN
            CALL KDRCPU(TK1)
            KISEG=IPR(7,IMX)
            IF(KISEG.EQ.0) THEN
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
               WRITE(HNISSS,'(3A4)') (ISHINA(I0,IMX),I0=1,3)
               WRITE(HSMG,850) HNAMIS,HNISSS,NAMFIL,2
               CALL XABORT(HSMG)
            ENDIF
*----
*  PROCESS THE RESOLVED ENERGY DOMAIN.
*----
            IDKOBJ=IBASE(IKDS+KISEG-1)
            LGSEG=IBASE(ILGS+KISEG-1)
            CALL SETARA(IBASE,LGSEG,ITCARO)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITCARO),IDKOBJ,LGSEG)
            JDKDS=IBASE(ITCARO+IDKDS-1)
            JDKTS=IBASE(ITCARO+IDKTS-1)
            NS=IBASE(ITCARO+IDKNS-1)
            FGRESO=MAX(FGRESO,FGHOMO)
            IF(NS.EQ.(LGHOMO-FGRESO+1)*NTEMPS) THEN
               NGBIN=LGHOMO-FGRESO+1
            ELSE IF(NS.EQ.NGHOMO*NTEMPS) THEN
               NGBIN=NGHOMO
            ELSE
               CALL XABORT('LIBA20: INVALID PTHOM5(1).')
            ENDIF
            LBIN=0
            CALL XDISET(NFS,NGRO,0)
            DO 480 IG=1,NGBIN
            IDK=JDKTS+8*(IG-1)
            CALL AEXCPC(IDK,8,IBASE(ITCARO),TYPSEG)
            IF(TYPSEG.NE.'PTHOM5') CALL XABORT('LIBA20: INVALID PTH'
     1      //'OM5(2).')
            LNGS=IBASE(ITCARO+IDKLS+IG-1)
            IF(LNGS.LE.0) CALL XABORT('LIBA20: INVALID PTHOM5(3).')
            JDKS=IBASE(ITCARO+JDKDS+IG-1)
            CALL AEXTRT(LIBA21,TYPSEG,NBRTYP,ICHDIM,ICHTYP,ICHDKL)
            CALL SETARA(IBASE,LNGS+1,ITSEGM)
            CALL AEXDIR(IUNIT,LBLOC,IBASE(ITSEGM),JDKS,LNGS+1)
            CALL AEXGNV(1,IBASE(ITSEGM),IBASE(ICHDIM),IBASE(ICHTYP),
     1      IBASE(ICHDKL),IDKD,NV)
            LBIN=LBIN+NV
            NFS(FGRESO+IG-1)=NV
            CALL RLSARA(IBASE(ITSEGM))
            CALL RLSARA(IBASE(ICHDIM))
            CALL RLSARA(IBASE(ICHTYP))
            CALL RLSARA(IBASE(ICHDKL))
  480       CONTINUE
            IF(LSACO) THEN
               NFSBIN=NFS(FGRESO)
               LBIN=LBIN+(FGRESO-FGHOMO)*NFSBIN
            ELSE
               NFSBIN=0
            ENDIF
            DO 530 JMX=IMX,NBISO
            IF(IPR(7,JMX).EQ.KISEG) THEN
               CALL SETARA(BASE,LBIN,IDELTF)
               CALL SETARA(BASE,LBIN,ISIGTF)
               CALL SETARA(BASE,LBIN,ISIGAF)
               IOF=(FGRESO-FGHOMO)*NFSBIN
               CALL SETARA(BASE,NTEMPS,IWEIJH)
               CALL SETARA(BASE,NTEMPS,ISQRTE)
               WRITE(HNAMIS,'(3A4)') (ISONAM(I0,JMX),I0=1,3)
               CALL LCMSIX(IPLIB,HNAMIS,1)
               DO 500 IG=1,NGBIN
               IGG=FGRESO+IG-1
               CALL LIBA26(LGSEG,IG,NGBIN,IUNIT,LBLOC,TKCARO,
     1         IBASE(ITCARO),NFS(IGG),TN(JMX),NTEMPS,BASE(ITEMPS),
     2         BASE(IDELTF+IOF),BASE(ISIGTF+IOF),BASE(ISIGAF+IOF),
     3         DELINF,SGTINF,SGAINF,BASE(IWEIJH),BASE(ISQRTE))
               IG2=IG+FGRESO-FGHOMO
               F1=DELTA(IGG)/DELINF
               F2=(BASE(ITAUX+(4*NGHOMO+IG2)-1)+
     1             BASE(ITAUX+(5*NGHOMO+IG2)-1))/(SGTINF*DELTA(IGG))
               F3=BASE(ITAUX+(4*NGHOMO+IG2)-1)/(SGAINF*DELTA(IGG))
               DO 490 I=1,NFS(IGG)
               BASE(IDELTF+IOF+I-1)=BASE(IDELTF+IOF+I-1)*F1
               BASE(ISIGTF+IOF+I-1)=BASE(ISIGTF+IOF+I-1)*F2
  490          BASE(ISIGAF+IOF+I-1)=BASE(ISIGAF+IOF+I-1)*F3
  500          IOF=IOF+NFS(IGG)
               CALL RLSARA(BASE(ISQRTE))
               CALL RLSARA(BASE(IWEIJH))
*----
*  PROCESS THE UNRESOLVED ENERGY DOMAIN. THE AUTOLIB OF THE FIRST
*  RESOLVED ENERGY GROUP IS USED AND NORMALIZED TO THE CORRECT
*  INFINITE DILUTION VALUES. USED WITH THE SANCHEZ-COSTE METHOD.
*----
               IF(LSACO) THEN
                  E0=ENERG(FGHOMO)
                  IG2=FGRESO-FGHOMO+1
                  E1=DELTA(FGRESO)
                  E2=(BASE(ITAUX+(4*NGHOMO+IG2)-1)+
     1                BASE(ITAUX+(5*NGHOMO+IG2)-1))
                  E3=BASE(ITAUX+(4*NGHOMO+IG2)-1)
                  IBIN=0
                  DO 510 IGG=FGHOMO,FGRESO-1
                  NFS(IGG)=NFSBIN
                  IG2=IGG-FGHOMO+1
                  F1=DELTA(IGG)/E1
                  F2=(BASE(ITAUX+(4*NGHOMO+IG2)-1)+
     1                BASE(ITAUX+(5*NGHOMO+IG2)-1))/E2
                  F3=BASE(ITAUX+(4*NGHOMO+IG2)-1)/E3
                  JBIN=(FGRESO-FGHOMO)*NFSBIN
                  DO 510 I=1,NFSBIN
                  IBIN=IBIN+1
                  JBIN=JBIN+1
                  BASE(IDELTF+IBIN-1)=BASE(IDELTF+JBIN-1)*F1
                  BASE(ISIGTF+IBIN-1)=BASE(ISIGTF+JBIN-1)*F2/F1
  510             BASE(ISIGAF+IBIN-1)=BASE(ISIGAF+JBIN-1)*F3/F1
               ELSE
                  E0=ENERG(FGRESO)
               ENDIF
*
               CALL SETARA(BASE,LBIN+1,IENER)
               BASE(IENER)=E0
               UU=0.0D0
               DO 520 I=1,LBIN
               UU=UU+BASE(IDELTF+I-1)
               BASE(IENER+I)=E0*EXP(-UU)
  520          BASE(ISIGAF+I-1)=BASE(ISIGTF+I-1)-BASE(ISIGAF+I-1)
               CALL RLSARA(BASE(IDELTF))
               CALL LCMPUT(IPLIB,'BIN-NFS',NGRO,1,NFS)
               CALL LCMPOF(IPLIB,'BIN-ENERGY',LBIN+1,2,IENER)
               CALL LCMPOF(IPLIB,'BIN-NTOT0',LBIN,2,ISIGTF)
               CALL LCMPOF(IPLIB,'BIN-SIGS00',LBIN,2,ISIGAF)
               CALL LCMSIX(IPLIB,' ',2)
               IPR(7,JMX)=0
            ENDIF
  530       CONTINUE
            CALL RLSARA(IBASE(ITCARO))
            CALL KDRCPU(TK2)
            TKT(5)=TKT(5)+(TK2-TK1)
         ELSE
            KISEG=IPR(7,IMX)
            DO 540 JMX=IMX,NBISO
            IF(IPR(7,JMX).EQ.KISEG) IPR(7,JMX)=0
  540       CONTINUE
         ENDIF
         CALL RLSARA(BASE(ITEMPS))
         CALL RLSARA(BASE(ITAUX))
*
  550    IF(IMPX.GT.9) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
            CALL LCMSIX(IPLIB,HNAMIS,1)
            CALL LCMLIB(IPLIB)
            CALL LCMSIX(IPLIB,' ',2)
         ENDIF
      ENDIF
  560 CONTINUE
*
      CALL RLSARA(IBASE(ILGS))
      CALL RLSARA(IBASE(IKDS))
      CALL RLSARA(IBASE(INOMOB))
      CALL RLSARA(BASE(IAMASS))
      IERR=KDRCLS(IUNIT,1)
      IF(IERR.LT.0) THEN
         TEXT12=NAMFIL
         CALL XABORT('LIBA20: APOLLO-2 LIBRARY '//TEXT12//' CANNOT B'//
     1   'E CLOSED')
      ENDIF
*----
*  CHECK IF ALL REACTIONS HAVE BEEN PROCESSED.
*----
      DO 570 IMX=1,NBISO
      DO 570 I=2,7+2*(NL-1)
      IF(IPR(I,IMX).NE.0) THEN
         WRITE(HSMG,950) I,(ISONAM(I0,IMX),I0=1,3)
         CALL XABORT(HSMG)
      ENDIF
  570 CONTINUE
      IF(IMPX.GT.2) WRITE(IOUT,940) (TKT(I),I=1,5)
*----
*  ADD NG CROSS SECTIONS.
*----
      DO 610 IMX=1,NBISO
      IF(MASKI(IMX)) THEN
         WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
         CALL LCMSIX(IPLIB,HNAMIS,1)
         CALL LCMGET(IPLIB,'NTOT0',SECT)
         CALL LCMLEN(IPLIB,'SIGS00',LENGT,ITYLCM)
         IF(LENGT.EQ.NGRO) THEN
            CALL LCMGET(IPLIB,'SIGS00',XSTOT)
            DO 580 IU=1,NGRO
  580       SECT(IU)=SECT(IU)-XSTOT(IU)
         ENDIF
         CALL LCMLEN(IPLIB,'NFTOT',LENGT,ITYLCM)
         IF(LENGT.EQ.NGRO) THEN
            CALL LCMGET(IPLIB,'NFTOT',XSTOT)
            DO 590 IU=1,NGRO
  590       SECT(IU)=SECT(IU)-XSTOT(IU)
         ENDIF
         CALL LCMLEN(IPLIB,'N2N',LENGT,ITYLCM)
         IF(LENGT.EQ.NGRO) THEN
            CALL LCMGET(IPLIB,'N2N',XSTOT)
            DO 600 IU=1,NGRO
  600       SECT(IU)=SECT(IU)+XSTOT(IU)
         ENDIF
         CALL LCMPUT(IPLIB,'NG',NGRO,2,SECT)
         CALL LCMSIX(IPLIB,' ',2)
      ENDIF
  610 CONTINUE
      RETURN
*
  800 FORMAT(/43H LIBA20: PROCESSING APOLIB-2 LIBRARY NAME: ,A12,1H.)
  810 FORMAT(/32H LIBA20: X-SECTION LIBRARY INFO:/9X,A80/)
  820 FORMAT(/35H LIBA20: PROBING THE APOLIB-2 FILE./9X,11HNUMBER OF I,
     1 29HSOTOPES AT INFINITE DILUTION=,I8/9X,21HNUMBER OF SELF-SHIELD,
     2 12HED ISOTOPES=,I8/9X,27HNUMBER OF APOLIBE SEGMENTS=,I8)
  830 FORMAT(9HLIBA20: P,I2,27H INFO OF MATERIAL/ISOTOPE ',A12,5H' = ',
     1 A12,35H' IS MISSING ON APOLIB-2 FILE NAME ,A12,1H.)
  840 FORMAT(45HLIBA20: PRODUCTION INFO OF MATERIAL/ISOTOPE ',A12,
     1 5H' = ',A12,35H' IS MISSING ON APOLIB-2 FILE NAME ,A12,1H.)
  850 FORMAT(49HLIBA20: SELF-SHIELDING DATA OF MATERIAL/ISOTOPE ',A12,
     1 5H' = ',A12,35H' IS MISSING ON APOLIB-2 FILE NAME ,A12,1H(,I1,
     2 2H).)
  860 FORMAT(/38H LIBA20: PROCESSING ISOTOPE/MATERIAL ',A12,2H'.)
  870 FORMAT(/9X,15HISOTOPE TITLE: ,A80)
  875 FORMAT(/9X,6HZSECT=,10I10/(15X,10I10))
  880 FORMAT(/9X,13HTEMPERATURES=,1P,9E12.4/(22X,9E12.4))
  890 FORMAT(/9X,6HZFISS=,I2,8H LGPROB=,L2,8H LGTDIF=,L2,8H LGTTRA=,L2,
     1 6H FGTD=,I5,5H ID2=,I5,8H NSECTT=,I3,8H NSETOT=,I3,6H NPHY=,I3/
     2 9X,7HNANISD=,I3,8H NANIST=,I3,8H LGTREA=,10L2)
  900 FORMAT(/9X,5HFAGG=,I5,6H LAGG=,I5,6H FDGG=,I5,6H WGAL=,I5,5H FAG=,
     1 I5,5H LAG=,I5,6H NGTD=,I5)
  910 FORMAT(/9X,10HDILUTIONS=,1P,9E12.4/(19X,9E12.4))
  920 FORMAT(/9X,28HSELF-SHIELDING TEMPERATURES=,1P,7E12.4/(37X,7E12.4))
  930 FORMAT(/9X,7HFGHOMO=,I4,8H FGRESO=,I4,8H NGHOMO=,I4,8H NSEQHO=,
     1 I4,8H NTEMPS=,I4,6H L104=,L2,6H NBIN=,I5)
  940 FORMAT(/26H LIBA20: CPU TIME USAGE --,F10.2,9H INDEXING/26X,
     1 F10.2,24H INFINITE DILUTION P0 XS/26X,F10.2,11H PN XS DATA/
     2 26X,F10.2,27H DILUTION-DEPENDENT XS DATA/26X,F10.2,5H AUTO,
     3 12HLIB XS DATA.)
  950 FORMAT(26HLIBA20: REMAINING REACTION,I3,14H FOR ISOTOPE ',3A4,
     1 2H'.)
      END
