*DECK LIBCOR
      SUBROUTINE LIBCOR (IPLIB,NGRO,HNAMIS1,HNAMIS2,NFS1,NFS2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the correlation information between a pair of resonant
* isotopes.
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the internal library (L_LIBRARY signature).
* NGRO    number of energy groups.
* HNAMIS1 local name of the first isotope:
*         HNAMIS1(1:8)  is the local isotope name;
*         HNAMIS1(9:12) is a suffix function of the mixture index.
* HNAMIS2 local name of the second isotope
*         HNAMIS2(1:8)  is the local isotope name;
*         HNAMIS2(9:12) is a suffix function of the mixture index.
*
*Parameters: scratch
* NFS1
* NFS2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NGRO,NFS1(NGRO),NFS2(NGRO)
      CHARACTER HNAMIS1*12,HNAMIS2*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER(MAXNPT=12)
      REAL SIGQT1(MAXNPT),SIGQT2(MAXNPT),WSLD1(MAXNPT**2),
     1 WSLD2(MAXNPT**2)
      DOUBLE PRECISION SUMA1,SUMB1,SUMA2,SUMB2
      COMMON BASE(1)
*
      IP1=LCMGID(IPLIB,HNAMIS1)
      IP2=LCMGID(IPLIB,HNAMIS2)
      CALL LCMLEN(IP1,'BIN-NFS',LENGT1,ITYLCM)
      CALL LCMLEN(IP2,'BIN-NFS',LENGT2,ITYLCM)
      IF((LENGT1.EQ.0).OR.(LENGT1.NE.LENGT2)) CALL XABORT('LIBCOR: UNA'
     1 //'BLE TO FIND CONSISTENT BIN TYPE INFORMATION.')
      CALL LCMGET(IP1,'BIN-NFS',NFS1)
      CALL LCMGET(IP2,'BIN-NFS',NFS2)
      LBIN=0
      IGRMIN=1
      IGRMAX=NGRO
      DO 10 IGRP=NGRO,1,-1
      IF(NFS1(IGRP).NE.NFS2(IGRP)) CALL XABORT('INVALID BIN INFO.')
      IF((IGRMAX.EQ.IGRP).AND.(NFS1(IGRP).EQ.0)) IGRMAX=IGRP-1
   10 LBIN=LBIN+NFS1(IGRP)
      DO 20 IGRP=1,NGRO
      IF((IGRMIN.EQ.IGRP).AND.(NFS1(IGRP).EQ.0)) IGRMIN=IGRP+1
   20 CONTINUE
      CALL SETARA(BASE,LBIN,ITBIN1)
      CALL SETARA(BASE,LBIN,ITBIN2)
      CALL SETARA(BASE,LBIN+1,IEBIN)
      CALL SETARA(BASE,LBIN,IDBIN)
      CALL LCMGET(IP1,'BIN-ENERGY',BASE(IEBIN))
      CALL LCMGET(IP1,'BIN-NTOT0',BASE(ITBIN1))
      CALL LCMGET(IP2,'BIN-NTOT0',BASE(ITBIN2))
      CALL LCMSIX(IP1,'PT-PHYS',1)
      CALL LCMSIX(IP2,'PT-PHYS',1)
*---
*  LOOP OVER THE RESONANT ENERGY GROUPS.
*---
      LBIN=0
      JP1=LCMGID(IP1,'GROUP')
      JP2=LCMGID(IP2,'GROUP')
      DO 120 IGRP=IGRMIN,IGRMAX
      SUMA1=0.0D0
      SUMB1=0.0D0
      SUMA2=0.0D0
      SUMB2=0.0D0
      DO 30 IGF=1,NFS1(IGRP)
      SIGTA=MAX(0.002,BASE(ITBIN1+LBIN+IGF-1))
      SIGTB=MAX(0.002,BASE(ITBIN2+LBIN+IGF-1))
      DELM=LOG(BASE(IEBIN+LBIN+IGF-1)/BASE(IEBIN+LBIN+IGF))
      SUMA1=SUMA1+BASE(ITBIN1+LBIN+IGF-1)*DELM
      SUMB1=SUMB1+SIGTA*DELM
      SUMA2=SUMA2+BASE(ITBIN2+LBIN+IGF-1)*DELM
      SUMB2=SUMB2+SIGTB*DELM
      BASE(ITBIN1+LBIN+IGF-1)=SIGTA
      BASE(ITBIN2+LBIN+IGF-1)=SIGTB
   30 BASE(IDBIN+LBIN+IGF-1)=DELM
      DO 40 IGF=1,NFS1(IGRP)
      BASE(ITBIN1+LBIN+IGF-1)=BASE(ITBIN1+LBIN+IGF-1)*(SUMA1/SUMB1)
   40 BASE(ITBIN2+LBIN+IGF-1)=BASE(ITBIN2+LBIN+IGF-1)*(SUMA2/SUMB2)
*
      CALL LCMLEL(JP1,IGRP,N1,ITYLCM)
      CALL LCMLEL(JP2,IGRP,N2,ITYLCM)
      IF((N1.EQ.0).OR.(N2.EQ.0)) GO TO 120
      KP1=LCMGIL(JP1,IGRP)
      KP2=LCMGIL(JP2,IGRP)
      CALL LCMLEN(KP1,'SIGQT-SIGS',NQT1,ITYLCM)
      CALL LCMLEN(KP2,'SIGQT-SIGS',NQT2,ITYLCM)
      CALL LCMLEN(KP1,'PROB-TABLE',NQT10,ITYLCM)
      CALL LCMLEN(KP2,'PROB-TABLE',NQT20,ITYLCM)
      CALL SETARA(BASE,NQT10,IPROB1)
      CALL SETARA(BASE,NQT20,IPROB2)
      CALL LCMGET(KP1,'PROB-TABLE',BASE(IPROB1))
      CALL LCMGET(KP2,'PROB-TABLE',BASE(IPROB2))
      DO 50 I=1,NQT1
   50 SIGQT1(I)=BASE(IPROB1+MAXNPT+I-1)
      DO 60 I=1,NQT2
   60 SIGQT2(I)=BASE(IPROB2+MAXNPT+I-1)
*
      CALL SETARA(BASE,2*NQT1*NQT2,ICOMOM)
      CALL LIBCOM(NFS1(IGRP),BASE(LBIN+IDBIN),BASE(LBIN+ITBIN1),
     1 BASE(LBIN+ITBIN2),NQT1,NQT2,BASE(ICOMOM))
      CALL LIBOMG(NQT1,1-NQT1/2,SIGQT1,NQT2,1-NQT2/2,SIGQT2,
     1 BASE(ICOMOM),WSLD1)
      CALL RLSARA(BASE(ICOMOM))
*---
*  CHECK NORMALIZATION OF THE CORRELATED WEIGHT MATRIX.
*---
      DO 80 I=1,NQT1
      SUM=0.0
      DO 70 J=1,NQT2
   70 SUM=SUM+WSLD1((J-1)*NQT1+I)
      IF(ABS(SUM-BASE(IPROB1+I-1)).GT.1.0E-4) THEN
         CALL XABORT('LIBCOR: BAD NORMALIZATION EXCEPTION(1).')
      ENDIF
   80 CONTINUE
      DO 100 I=1,NQT2
      SUM=0.0
      DO 90 J=1,NQT1
   90 SUM=SUM+WSLD1((I-1)*NQT1+J)
      IF(ABS(SUM-BASE(IPROB2+I-1)).GT.1.0E-4) THEN
         CALL XABORT('LIBCOR: BAD NORMALIZATION EXCEPTION(2).')
      ENDIF
  100 CONTINUE
      CALL RLSARA(BASE(IPROB2))
      CALL RLSARA(BASE(IPROB1))
*
      CALL LCMPUT(KP1,HNAMIS2,NQT1*NQT2,2,WSLD1)
      DO 110 I=1,NQT1
      DO 110 J=1,NQT2
  110 WSLD2((I-1)*NQT2+J)=WSLD1((J-1)*NQT1+I)
      CALL LCMPUT(KP2,HNAMIS1,NQT2*NQT1,2,WSLD2)
  120 LBIN=LBIN+NFS1(IGRP)
*
      CALL LCMSIX(IP2,' ',2)
      CALL LCMSIX(IP1,' ',2)
      CALL RLSARA(BASE(IDBIN))
      CALL RLSARA(BASE(IEBIN))
      CALL RLSARA(BASE(ITBIN2))
      CALL RLSARA(BASE(ITBIN1))
      RETURN
      END
