*DECK LIBDRA
      SUBROUTINE LIBDRA (IPLIB,IPDRL,NAMFIL,NGRO,NBISO,NL,ISONAM,
     1 ISONRF,TN,SN,SB,MASKI,NED,HVECT,IMPX,NGF,NGFR,NDEL,AWR,DELTA,
     2 SIGS,SCAT,TOTAL,SIGF,CHI,SADD,LSCAT,LADD,GOLD,NFS,ZNPHI,ITYPRO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transcription of the useful interpolated microscopic cross section
* data from a microscopic x-section library (draglib format) to LCM
* data structures.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature).
* IPDRL   pointer to the draglib (L_DRAGLIB signature).
* NAMFIL  name of the Dragon library file.
* NGRO    number of energy groups.
* NBISO   number of isotopes present in the calculation domain.
* NL      number of Legendre orders required in the calculation
*         NL=1 or higher.
* ISONAM  alias name of isotopes.
* ISONRF  library name of isotopes.
* TN      temperature of each isotope.
* SN      dilution cross section in each energy group of each
*         isotope. a value of 1.0e10 is used for infinite dilution.
* SB      dilution cross section as used in livolant and jeanpierre
*         normalization.
* MASKI   isotopic mask. Isotope with index I is processed if
*         MASKI(I)=.true.
* NED     number of extra vector edits.
* HVECT   names of the extra vector edits.
* IMPX    print flag.
*
*Parameters: output
* NGF     number of fast groups without self-shielding.
* NGFR    number of fast and resonance groups.
* NDEL    number of precursor groups for delayed neutrons.
*
*Parameters: scratch
* AWR
* DELTA
* SIGS
* SCAT
* TOTAL
* SIGF
* CHI
* SADD
* LSCAT
* LADD
* GOLD
* NFS
* ZNPHI
* ITYPRO
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      PARAMETER(MAXDEL=10)
      CHARACTER*(*) HVECT(NED),NAMFIL
      INTEGER IPLIB,IPDRL,NGRO,NBISO,NL,ISONAM(3,NBISO),ISONRF(3,NBISO),
     1 NED,IMPX,NGF,NGFR,NDEL,NFS(NGRO),ITYPRO(NL)
      REAL TN(NBISO),SN(NGRO,NBISO),SB(NGRO,NBISO),AWR(NBISO),
     1 DELTA(NGRO),SIGS(NGRO,NL),SCAT(NGRO,NGRO,NL),TOTAL(NGRO),
     2 SIGF(NGRO,0:MAXDEL),CHI(NGRO,0:MAXDEL),SADD(NGRO,NED),
     3 GOLD(NGRO),ZNPHI(NGRO)
      LOGICAL MASKI(NBISO),LSCAT(NL),LADD(NED)
*----
*  LOCAL VARIABLES
*----
      CHARACTER CD*4,HSMG*131,HTITLE*80,HNISOR*12,HNAMIS*12,HNUSIG*12,
     1 HCHI*12
      PARAMETER (IOUT=6,MAXTMP=50,NOTX=3)
      LOGICAL LSIGF,LGOLD,LOGT
      DOUBLE PRECISION FACTOR,TERP(MAXTMP),TERPM
      REAL TEMP(MAXTMP),ZLAMB(MAXDEL)
      COMMON BASE(1)
*----
*  RECOVER THE GROUP STRUCTURE.
*----
      NGF=NGRO+1
      NGFR=0
      NDEL=0
      IF(IMPX.GT.0) WRITE (IOUT,900) NAMFIL
      CALL LCMLEN(IPDRL,'README',LENGT,ITYLCM)
      IF((IMPX.GT.0).AND.(LENGT.GT.0)) THEN
         CALL SETARA(BASE,LENGT,ITITLE)
         CALL LCMGET(IPDRL,'README',BASE(ITITLE))
         WRITE (IOUT,940)
         I2=0
         DO 10 J=0,LENGT/20
         I1=I2+1
         I2=MIN(I1+19,LENGT)
         WRITE (HTITLE,'(20A4)') (BASE(ITITLE+I-1),I=I1,I2)
         WRITE (IOUT,'(1X,A80)') HTITLE
   10    CONTINUE
         CALL RLSARA(BASE(ITITLE))
      ENDIF
      CALL SETARA(BASE,NGRO+1,IENER)
      CALL LCMLEN(IPDRL,'ENERGY',LENGT,ITYLCM)
      LENGT=LENGT-1
      IF(LENGT.NE.NGRO) CALL XABORT('LIBDRA: INVALID GROUP STRUCTURE.')
      CALL LCMGET(IPDRL,'ENERGY',BASE(IENER))
      CALL LCMLEN(IPDRL,'DELTAU',LENGT,ITYLCM)
      IF(LENGT.EQ.NGRO) THEN
         CALL LCMGET(IPDRL,'DELTAU',DELTA)
      ELSE IF(LENGT.EQ.0) THEN
         IF(BASE(IENER+NGRO).EQ.0.0) BASE(IENER+NGRO)=1.0E-5
         DO 15 J=1,NGRO
   15    DELTA(J)=LOG(BASE(IENER+J-1)/BASE(IENER+J))
      ENDIF
      CALL LCMPUT(IPLIB,'ENERGY',NGRO+1,2,BASE(IENER))
      CALL LCMPUT(IPLIB,'DELTAU',NGRO,2,DELTA)
      CALL RLSARA(BASE(IENER))
*----
*  READ THROUGH DRAGON FILE AND ACCUMULATE CROSS SECTIONS FOR THIS RANGE
*  OF MATS, LEGENDRE ORDERS, AND GROUPS.
*----
      DO 400 IMX=1,NBISO
      IF(MASKI(IMX)) THEN
         WRITE(HNAMIS,'(3A4)') (ISONAM(I0,IMX),I0=1,3)
         WRITE(HNISOR,'(3A4)') (ISONRF(I0,IMX),I0=1,3)
         CALL LCMLEN(IPDRL,HNISOR,LENGT,ITYLCM)
         IF(LENGT.EQ.0) THEN
            CALL LCMLIB(IPDRL)
            WRITE (HSMG,910) HNAMIS,HNISOR,NAMFIL
            CALL XABORT(HSMG)
         ENDIF
         IF(IMPX.GT.0) WRITE (IOUT,920) HNAMIS,HNISOR
         CALL LCMSIX(IPDRL,HNISOR,1)
*
         CALL LCMGET(IPDRL,'AWR',AWR(IMX))
         CALL LCMLEN(IPDRL,'README',LTITLE,ITYLCM)
         IF(LTITLE.GT.0) THEN
            CALL SETARA(BASE,LTITLE,ITITLE)
            CALL LCMGET(IPDRL,'README',BASE(ITITLE))
            IF(IMPX.GT.0) THEN
               WRITE (IOUT,930)
               I2=0
               DO 20 J=0,LTITLE/20
               I1=I2+1
               I2=MIN(I1+19,LTITLE)
               WRITE (HTITLE,'(20A4)') (BASE(ITITLE+I-1),I=I1,I2)
               WRITE (IOUT,'(1X,A80)') HTITLE
   20          CONTINUE
            ENDIF
         ENDIF
*----
*  RECOVER BIN TYPE INFORMATION (IF AVAILABLE).
*----
         LBIN=0
         CALL LCMLEN (IPDRL,'BIN-NFS',LENGT,ITYXSM)
         IF(LENGT.GT.0) THEN
            CALL LCMGET (IPDRL,'BIN-NFS',NFS)
            DO 30 I=1,NGRO
   30       LBIN=LBIN+NFS(I)
            CALL SETARA (BASE,3*LBIN,IBIN)
            CALL SETARA (BASE,LBIN+1,IEBIN)
            CALL LCMGET (IPDRL,'BIN-ENERGY',BASE(IEBIN))
         ENDIF
*
         CALL SETARA (BASE,NGRO*NGRO,IGAR)
         CALL SETARA (BASE,NGRO,INJJ)
         CALL SETARA (BASE,NGRO,IIJJ)
         CALL SETARA (BASE,NL,ILSDIL)
         CALL SETARA (BASE,NL,ILPDIL)
         CALL SETARA (BASE,50*NGRO,ITERP)
         CALL SETARA (BASE,NED,IKADDR)
         CALL SETARA (BASE,NGRO*50,ISIGT)
         CALL SETARA (BASE,NGRO,ILINF)
         CALL LCMLEN (IPDRL,'TEMPERATURE',NTMP,ITYLCM)
         IF(NTMP.GT.MAXTMP) CALL XABORT('LIBDRA: MAXTMP OVERFLOW.')
         IF(NTMP.EQ.0) THEN
            CALL LCMLEN (IPDRL,'LAMBDA-D',NDEL0,ITYLCM)
            NDEL=MAX(NDEL,NDEL0)
            IF(NDEL0.GT.MAXDEL) CALL XABORT('LIBDRA: MAXDEL OVERFLOW.')
            IF(NDEL0.GT.0) CALL LCMGET (IPDRL,'LAMBDA-D',ZLAMB)
            CALL LIBDRB (IPDRL,NGRO,NL,NDEL0,SN(1,IMX),SB(1,IMX),NED,
     1      HVECT,DELTA,LBIN,NFS,BASE(IEBIN),IMPX,NGF,NGFR,LSCAT,LSIGF,
     2      LADD,LGOLD,SIGS,SCAT,TOTAL,SIGF,CHI,SADD,GOLD,BASE(IBIN),
     3      BASE(IGAR),BASE(INJJ),BASE(IIJJ),BASE(ILSDIL),BASE(ILPDIL),
     4      BASE(ITERP),ZNPHI,BASE(IKADDR),BASE(ISIGT),BASE(ILINF))
         ELSE
*----
*  PERFORM TEMPERATURE LAGRANGIAN INTERPOLATION (ORDER ABS(NOTX)).
*----
            CALL LCMSIX (IPDRL,'SUBTMP0001',1)
            CALL LCMLEN (IPDRL,'LAMBDA-D',NDEL0,ITYLCM)
            NDEL=MAX(NDEL,NDEL0)
            IF(NDEL0.GT.MAXDEL) CALL XABORT('LIBDRA: MAXDEL OVERFLOW.')
            IF(NDEL0.GT.0) CALL LCMGET (IPDRL,'LAMBDA-D',ZLAMB)
            CALL LCMSIX (IPDRL,' ',2)
            CALL LCMGET (IPDRL,'TEMPERATURE',TEMP)
            CALL LIBLEX(NTMP,TN(IMX),TEMP,NOTX,TERP)
            DO 120 IG1=1,NGRO
            TOTAL(IG1)=0.0
            ZNPHI(IG1)=0.0
            DO 100 IDEL=0,NDEL0
            SIGF(IG1,IDEL)=0.0
  100       CHI(IG1,IDEL)=0.0
            GOLD(IG1)=0.0
            DO 110 IL=1,NL
            SIGS(IG1,IL)=0.0
            DO 110 IG2=1,NGRO
  110       SCAT(IG1,IG2,IL)=0.0
            DO 120 IED=1,NED
  120       SADD(IG1,IED)=0.0
            DO 125 IG=1,3*LBIN
  125       BASE(IBIN+IG-1)=0.0
            CALL SETARA (BASE,NGRO*NL,ISIGS)
            CALL SETARA (BASE,NGRO*NGRO*NL,ISCAT)
            CALL SETARA (BASE,NGRO,ITOTAL)
            CALL SETARA (BASE,NGRO*(NDEL0+1),ISIGF)
            CALL SETARA (BASE,NGRO*(NDEL0+1),ICHI)
            CALL SETARA (BASE,NGRO*NED,ISADD)
            CALL SETARA (BASE,NGRO,IGOLD)
            CALL SETARA (BASE,3*LBIN,IBIN2)
            CALL SETARA (BASE,NGRO,IZNPHI)
            FACTOR=1.0D0
            DO 210 ITM=1,NTMP
            TERPM=TERP(ITM)
            FACTOR=FACTOR-TERPM
            IF(TERPM.EQ.0.0) GO TO 210
            IF(IMPX.GT.4) WRITE(6,'(/30H DRAGLIB ACCESS AT TEMPERATURE,
     >      1P,E12.4,18H KELVIN. FACTOR = ,E12.4)')
     >      TEMP(ITM),TERPM
            WRITE (CD,'(I4.4)') ITM
            CALL LCMSIX (IPDRL,'SUBTMP'//CD,1)
            CALL LIBDRB (IPDRL,NGRO,NL,NDEL0,SN(1,IMX),SB(1,IMX),NED,
     1      HVECT,DELTA,LBIN,NFS,BASE(IEBIN),IMPX,NGF,NGFR,LSCAT,LSIGF,
     2      LADD,LGOLD,BASE(ISIGS),BASE(ISCAT),BASE(ITOTAL),BASE(ISIGF),
     3      BASE(ICHI),BASE(ISADD),BASE(IGOLD),BASE(IBIN2),BASE(IGAR),
     4      BASE(INJJ),BASE(IIJJ),BASE(ILSDIL),BASE(ILPDIL),BASE(ITERP),
     5      BASE(IZNPHI),BASE(IKADDR),BASE(ISIGT),BASE(ILINF))
            CALL LCMSIX (IPDRL,' ',2)
            DO 130 IG=1,NGRO
              TOTAL(IG)=TOTAL(IG)+TERPM*BASE(ITOTAL+IG-1)
              ZNPHI(IG)=ZNPHI(IG)+TERPM*BASE(IZNPHI+IG-1)
  130       CONTINUE
            IF(LSIGF) THEN
              DO 140 IDEL=0,NDEL0
              DO 140 IG=1,NGRO
                IOFSET=IDEL*NGRO+IG-1
                SIGF(IG,IDEL)=SIGF(IG,IDEL)+TERPM*BASE(ISIGF+IOFSET)
                CHI(IG,IDEL)=CHI(IG,IDEL)+TERPM*BASE(ICHI+IOFSET)
  140         CONTINUE
            ENDIF
            DO 160 IL=1,NL
              IF(LSCAT(IL)) THEN
                DO 150 IG2=1,NGRO
                  SIGS(IG2,IL)=SIGS(IG2,IL)+
     >                         TERPM*BASE(ISIGS+(IL-1)*NGRO+IG2-1)
                  IOF=(IL-1)*NGRO*NGRO+(IG2-1)*NGRO
                  DO 151 IG1=1,NGRO
                    SCAT(IG1,IG2,IL)=SCAT(IG1,IG2,IL)+TERPM*
     >              BASE(ISCAT+IOF+IG1-1)
  151             CONTINUE
  150           CONTINUE
              ENDIF
  160       CONTINUE
            DO 180 IED=1,NED
            IF(LADD(IED)) THEN
               DO 170 IG=1,NGRO
                 SADD(IG,IED)=SADD(IG,IED)+
     >             TERPM*BASE(ISADD+(IED-1)*NGRO+IG-1)
  170          CONTINUE
            ENDIF
  180       CONTINUE
            IF(LGOLD) THEN
               DO 190 IG=1,NGRO
  190          GOLD(IG)=GOLD(IG)+TERPM*BASE(IGOLD+IG-1)
            ENDIF
            DO 200 IG=1,3*LBIN
  200       BASE(IBIN+IG-1)=BASE(IBIN+IG-1)+TERPM*BASE(IBIN2+IG-1)
  210       CONTINUE
            CALL RLSARA (BASE(IZNPHI))
            CALL RLSARA (BASE(IBIN2))
            CALL RLSARA (BASE(IGOLD))
            CALL RLSARA (BASE(ISADD))
            CALL RLSARA (BASE(ICHI))
            CALL RLSARA (BASE(ISIGF))
            CALL RLSARA (BASE(ITOTAL))
            CALL RLSARA (BASE(ISCAT))
            CALL RLSARA (BASE(ISIGS))
            IF(ABS(FACTOR).GT.1.0D-4) CALL XABORT('LIBDRA: TERP ERROR')
         ENDIF
         CALL RLSARA (BASE(ILINF))
         CALL RLSARA (BASE(ISIGT))
         CALL RLSARA (BASE(IKADDR))
         CALL RLSARA (BASE(ITERP))
         CALL RLSARA (BASE(ILPDIL))
         CALL RLSARA (BASE(ILSDIL))
         CALL RLSARA (BASE(IIJJ))
         CALL RLSARA (BASE(INJJ))
         CALL RLSARA (BASE(IGAR))
         CALL LCMSIX(IPDRL,' ',2)
*----
*        SAVE CROSS SECTION DATA ON LCM.
*----
         CALL LCMSIX(IPLIB,HNAMIS,1)
         CALL LCMPUT(IPLIB,'AWR',1,2,AWR(IMX))
         IF(LTITLE.GT.0) THEN
            CALL LCMPUT(IPLIB,'README',LTITLE,3,BASE(ITITLE))
            CALL RLSARA(BASE(ITITLE))
         ENDIF
         DO 220 IG=1,NGRO
         IF(TOTAL(IG).LT.0.0) THEN
            WRITE(HSMG,'(42HLIBDRA: NEGATIVE TOTAL CROSS SECTION IN GR,
     1      3HOUP,I4,14H FOR ISOTOPE '',A12,2H''.)') IG,HNAMIS
            CALL XABORT(HSMG)
         ELSE IF(ZNPHI(IG).LT.0.0) THEN
            WRITE(HSMG,'(41HLIBDRA: NEGATIVE INTEGRATED FLUX IN GROUP,
     1      I4,14H FOR ISOTOPE '',A12,2H''.)') IG,HNAMIS
            CALL XABORT(HSMG)
         ENDIF
  220    CONTINUE
         CALL LCMPUT(IPLIB,'NTOT0',NGRO,2,TOTAL)
         CALL LCMPUT(IPLIB,'NWT0',NGRO,2,ZNPHI)
         IF(NDEL0.GT.0) CALL LCMPUT (IPLIB,'LAMBDA-D',NDEL0,2,ZLAMB)
         IF(LSIGF) THEN
            DO 250 IDEL=0,NDEL0
            IF(IDEL.EQ.0) THEN
               HNUSIG='NUSIGF'
               HCHI='CHI'
            ELSE
               WRITE(HNUSIG,'(6HNUSIGF,I2.2)') IDEL
               WRITE(HCHI,'(3HCHI,I2.2)') IDEL
            ENDIF
            CALL LCMPUT(IPLIB,HNUSIG,NGRO,2,SIGF(1,IDEL))
            CALL LCMPUT(IPLIB,HCHI,NGRO,2,CHI(1,IDEL))
  250       CONTINUE
         ENDIF
         CALL SETARA(BASE,NGRO*NGRO,IGAR)
         CALL SETARA(BASE,NGRO,INJJ)
         CALL SETARA(BASE,NGRO,IIJJ)
         CALL XDRLGS(IPLIB,1,0,0,NL-1,1,NGRO,SIGS,SCAT,ITYPRO,
     1   BASE(IGAR),BASE(INJJ),BASE(IIJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IGAR))
         DO 340 IED=1,NED
         IF(LADD(IED).AND.(HVECT(IED)(:3).NE.'CHI')
     1               .AND.(HVECT(IED)(:2).NE.'NU')
     2               .AND.(HVECT(IED).NE.'NTOT0')
     3               .AND.(HVECT(IED)(:3).NE.'NWT')) THEN
            CALL LCMPUT(IPLIB,HVECT(IED),NGRO,2,SADD(1,IED))
         ENDIF
  340    CONTINUE
         IF(LGOLD) CALL LCMPUT(IPLIB,'NGOLD',NGRO,2,GOLD)
         IF(LBIN.GT.0) THEN
            CALL LCMPUT(IPLIB,'BIN-NFS',NGRO,1,NFS)
            CALL LCMPUT(IPLIB,'BIN-ENERGY',LBIN+1,2,BASE(IEBIN))
            CALL LCMPUT(IPLIB,'BIN-NTOT0',LBIN,2,BASE(IBIN))
            CALL LCMPUT(IPLIB,'BIN-SIGS00',LBIN,2,BASE(IBIN+LBIN))
            LOGT=.FALSE.
            DO 350 I=1,LBIN
  350       LOGT=LOGT.OR.(BASE(IBIN+2*LBIN+I-1).NE.0.0)
            IF(LOGT) THEN
               CALL LCMPUT(IPLIB,'BIN-NUSIGF',LBIN,2,BASE(IBIN+2*LBIN))
            ENDIF
            CALL RLSARA (BASE(IEBIN))
            CALL RLSARA (BASE(IBIN))
         ENDIF
         IF(IMPX.GT.9) CALL LCMLIB(IPLIB)
         CALL LCMSIX(IPLIB,' ',2)
      ENDIF
  400 CONTINUE
      RETURN
*
  900 FORMAT(/33H PROCESSING DRAGON LIBRARY NAMED ,A12,1H.)
  910 FORMAT(26HLIBDRA: MATERIAL/ISOTOPE ',A12,5H' = ',A12,9H' IS MISS,
     1 25HING ON DRAGON FILE NAMED ,A12,1H.)
  920 FORMAT(/30H PROCESSING ISOTOPE/MATERIAL ',A12,11H' (HNISOR=',A12,
     1 3H').)
  930 FORMAT(/23H ISOTOPE/MATERIAL INFO:)
  940 FORMAT(/24H X-SECTION LIBRARY INFO:)
      END
