*DECK LIBLIB
      SUBROUTINE LIBLIB (IPLIB,NBISO,MASKI,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transcription of the useful interpolated microscopic cross section
* data from various format of libraries to LCM. A two dimensional
* interpolation in temperature and dilution is performed. Part A
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature)
* NBISO   number of isotopes present in the calculation domain
* MASKI   isotopic masks. An isotope with index I is process if
*         MASKI(I)=.true.
* IMPX    print flag
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NBISO,IMPX
      LOGICAL MASKI(*)
*----
*  INTERNAL PARAMETERS
*----
      INTEGER IOUT,MAXED,NSTATE
      PARAMETER (IOUT=6,MAXED=50,NSTATE=40)
*----
*  LOCAL VARIABLES
*----
      INTEGER IPAR(NSTATE),NGRO,NL,ITRANC,ITIME,NLIB,NGF,IGRMAX,NED,
     > NDEL,IPROC,IJCEDM,IDEB,J,IISONA,IISONR,ITMPIS,IIHLIB,IILLIB,
     > INAME,INTFG,ILENG,ITYLCM,IISHIN,IVOID,ISN,ISB,INIR,IGIR
      REAL BASE
      CHARACTER HVECT(MAXED)*8,TEXT4*4
      COMMON BASE(1)
*----
*  RECOVER INFORMATION FROM THE /MICROLIB/ DIRECTORY.
*----
      CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
      IF(NBISO.NE.IPAR(2)) CALL XABORT('LIBLIB: INCONSISTENT LIBRARY.')
      NGRO=IPAR(3)
      NL=IPAR(4)
      ITRANC=IPAR(5)
      ITIME=IPAR(7)
      NLIB=IPAR(8)
      NGF=IPAR(9)
      IGRMAX=IPAR(10)
      NED=IPAR(13)
      IF(NED.GT.MAXED) CALL XABORT('LIBLIB: MAXED OVERFLOW.')
      IPROC=IPAR(17)
      NDEL=IPAR(19)
      CALL SETARA(BASE,2*NED,IJCEDM)
      CALL LCMGET(IPLIB,'ADDXSNAME-P0',BASE(IJCEDM))
      IDEB=IJCEDM
      DO 10 J=1,NED
        WRITE(HVECT(J),'(2A4)') BASE(IDEB),BASE(IDEB+1)
        IDEB=IDEB+2
   10 CONTINUE
      CALL RLSARA(BASE(IJCEDM))
*
      CALL SETARA(BASE,3*NBISO,IISONA)
      CALL LCMGET(IPLIB,'ISOTOPESUSED',BASE(IISONA))
*
      CALL SETARA(BASE,3*NBISO,IISONR)
      CALL LCMGET(IPLIB,'ISOTOPERNAME',BASE(IISONR))
*
      CALL SETARA(BASE,NBISO,ITMPIS)
      CALL LCMGET(IPLIB,'ISOTOPESTEMP',BASE(ITMPIS))
*
      CALL SETARA(BASE,6*NBISO,IIHLIB)
      CALL LCMGET(IPLIB,'ILIBRARYTYPE',BASE(IIHLIB))
*
      CALL SETARA(BASE,6*NBISO,IILLIB)
      CALL LCMGET(IPLIB,'ILIBRARYINDX',BASE(IILLIB))
*
      CALL SETARA(BASE,16*NLIB,INAME)
      CALL LCMGET(IPLIB,'ILIBRARYNAME',BASE(INAME))
*
      CALL SETARA(BASE,NBISO,INTFG)
      CALL LCMLEN(IPLIB,'ISOTOPESNTFG',ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         CALL LCMGET(IPLIB,'ISOTOPESNTFG',BASE(INTFG))
         CALL LCMGET(IPLIB,'ISOTOPESCOH',BASE(IIHLIB+2*NBISO))
         CALL LCMGET(IPLIB,'ISOTOPESINC',BASE(IIHLIB+4*NBISO))
      ELSE
         CALL XDISET(BASE(INTFG),NBISO,0)
      ENDIF
*
      CALL SETARA(BASE,3*NBISO,IISHIN)
      CALL LCMLEN(IPLIB,'ISOTOPESHIN',ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         CALL LCMGET(IPLIB,'ISOTOPESHIN',BASE(IISHIN))
      ELSE
         TEXT4=' '
         READ(TEXT4,'(A4)') IVOID
         CALL XDISET(BASE(IISHIN),2*NBISO,IVOID)
      ENDIF
*
      CALL SETARA(BASE,NGRO*NBISO,ISN)
      CALL SETARA(BASE,NGRO*NBISO,ISB)
      CALL LCMLEN(IPLIB,'ISOTOPESDSN',ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         CALL LCMGET(IPLIB,'ISOTOPESDSN',BASE(ISN))
         CALL LCMGET(IPLIB,'ISOTOPESDSB',BASE(ISB))
      ELSE
         CALL XDRSET(BASE(ISN),NGRO*NBISO,1.0E10)
         CALL XDRSET(BASE(ISB),NGRO*NBISO,1.0E10)
      ENDIF
*
      CALL SETARA(BASE,NBISO,INIR)
      CALL SETARA(BASE,NBISO,IGIR)
      CALL LCMLEN(IPLIB,'ISOTOPESNIR',ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         CALL LCMGET(IPLIB,'ISOTOPESNIR',BASE(INIR))
         CALL LCMGET(IPLIB,'ISOTOPESGIR',BASE(IGIR))
      ELSE
         CALL XDISET(BASE(INIR),NBISO,0)
         CALL XDRSET(BASE(IGIR),NBISO,1.0)
      ENDIF
*----
*  RECOVER AND INTERPOLATE MICROSCOPIC CROSS SECTIONS.
*----
      CALL LIBLIC (IPLIB,NBISO,MASKI,IMPX,NGRO,NL,ITRANC,ITIME,NLIB,
     1 NED,HVECT,BASE(IISONA),BASE(IISONR),BASE(IISHIN),BASE(ITMPIS),
     2 BASE(IIHLIB),BASE(IILLIB),BASE(INAME),BASE(INTFG),BASE(ISN),
     3 BASE(ISB),BASE(INIR),BASE(IGIR),NGF,IGRMAX,NDEL,IPROC)
*
      CALL RLSARA(BASE(IGIR))
      CALL RLSARA(BASE(INIR))
      CALL RLSARA(BASE(ISB))
      CALL RLSARA(BASE(ISN))
      CALL RLSARA(BASE(INTFG))
      CALL RLSARA(BASE(INAME))
      CALL RLSARA(BASE(IILLIB))
      CALL RLSARA(BASE(IIHLIB))
      CALL RLSARA(BASE(ITMPIS))
      CALL RLSARA(BASE(IISHIN))
      CALL RLSARA(BASE(IISONR))
      CALL RLSARA(BASE(IISONA))
*
      IPAR(9)=NGF
      IPAR(10)=IGRMAX
      IPAR(19)=NDEL
      CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,IPAR)
      IF(IMPX.GT.9) THEN
         WRITE (IOUT,'(36H LIBLIB: VALIDATION OF MICROLIB DATA)')
         CALL LCMVAL(IPLIB,' ')
      ENDIF
      RETURN
      END
