*DECK LIBLIC
      SUBROUTINE LIBLIC (IPLIB,NBISO,MASKI,IMPX,NGRO,NL,ITRANC,ITIME,
     1 NLIB,NED,HVECT,ISONAM,ISONRF,ISHINA,TMPISO,IHLIB,ILLIB,INAME,
     2 NTFG,SN,SB,NIR,GIR,NGF,IGRMAX,NDEL,IPROC)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transcription of the useful interpolated microscopic cross section
* data from various format of libraries to lcm. A two dimensional
* interpolation in temperature and dilution is performed. Part B
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature)
* NBISO   number of isotopes present in the calculation domain
* MASKI   isotopic masks. An isotope with index I is process if
*         MASKI(I)=.true.
* IMPX    print flag
* NGRO    number of energy groups
* NL      number of Legendre orders required in the calculation
*         NL=1 (for isotropic scattering) or higher
* ITRANC  type of transport correction: =0: no transport correction
*         =1: Apollo type transport correction; =2: recover from
*         library; =3: Wims-D type; =4: leakage correction alone.
* ITIME   MATXS type of fission spectrum:
*         =1 steady-state; =2 prompt
* NLIB    number of independent libraries
* NED     number of requested vector edits
* HVECT   names of the requested vector edits
* ISONAM  alias name of each isotope
* ISONRF  library reference name of each isotope
* ISHINA  self-shielding name of each isotope
* TMPISO  temperature of each isotope
* IHLIB   isotope options
* ILLIB   xs library index for each isotope (.le.NLIB)
* INAME   names of the NLIB xs libraries
* NTFG    number of thermal groups where the thermal inelastic
*         correction is applied
* SN      dilution cross section in each energy group of each
*         isotope. a value of 1.0E10 is used for infinite dilution
* SB      dilution cross section as used in Livolant and Jeanpierre
*         normalization
* NIR     first group index with an imposed IR slowing-down model;
*         =0 for no IR model
* GIR     value of the imposed Goldstein-Cohen parameter for groups
*         with an IR model
* NGF     number of fast groups without self-shielding
* IGRMAX  maximum group index with self-shielding
* NDEL    number of precursor groups for delayed neutrons
* IPROC   type of library processing
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NBISO,IMPX,NGRO,NL,ITRANC,ITIME,NLIB,NED,NGF,
     > IGRMAX,NDEL,IPROC,ISONAM(3,NBISO),ISONRF(3,NBISO),
     > ISHINA(3,NBISO),IHLIB(2,NBISO,3),ILLIB(NBISO),INAME(16,NLIB),
     > NTFG(NBISO),NIR(NBISO)
      LOGICAL MASKI(NBISO)
      CHARACTER*(*) HVECT(NED)
      REAL TMPISO(NBISO),SN(NGRO,NBISO),SB(NGRO,NBISO),GIR(NBISO)
*----
*  INTERNAL PARAMETERS
*----
      INTEGER MAXDIL
      PARAMETER (MAXDIL=65)
*----
*  LOCAL VARIABLES
*----
      INTEGER I,J,IND1,NBIS,NDEL0,IPDRL,IAWR,IDELTA,ISIGS,ISCAT,ITOTAL,
     > ISIGF,ICHI,ISADD,ILSCAT,ILADD,IGOLD,INFS,IZNPHI,IITYPR,NGF0,
     > NGFR0,ITYPRO,IXSREC,IXSCMP,INJJ,IIJJ,IXSOUT,ISORD,IGAR,ICNORM,
     > ISNORM,IDNORM,ISFIS,ISAVE,IFLUX,IVECT,IIPR,ILOGIE,ILCMAT,MAXTRA,
     > ISIG1,ISIGA,IPRI,IVTHER,IPHI,IPP,IPP1,ISSS,ISSS1,ISS1,ISS11,
     > IUUU,ISEFF,INISB,INISBE,ILINF,IENERG,ISECT,IXSTOT,IWORK,IWR2,
     > ILIB,ILONG
      CHARACTER NAMLBT*8,NAMFIL*64,HSMG*131,NAMLCM*12,NAMMY*12
      REAL BASE
      LOGICAL LTEST,EMPTY,LCM
      COMMON BASE(1)
*----
*  CHECK FOR DUPLICATE ISOTOPE NAMES.
*----
      DO 40 I=1,NBISO
      IF(MASKI(I).AND.(ILLIB(I).NE.0)) THEN
         DO 30 J=I+1,NBISO
         IF(MASKI(J).AND.(ISONAM(1,I).EQ.ISONAM(1,J)).AND.
     1                   (ISONAM(2,I).EQ.ISONAM(2,J)).AND.
     2                   (ISONAM(3,I).EQ.ISONAM(3,J))) THEN
            WRITE (HSMG,200) ISONAM(1,I),ISONAM(2,I),ISONAM(3,I)
            CALL XABORT(HSMG)
         ENDIF
   30    CONTINUE
      ENDIF
   40 CONTINUE
*
      IND1=1
   50 NBIS=1
      LTEST=MASKI(IND1)
      DO 60 I=IND1+1,NBISO
        IF((IHLIB(1,I,1).EQ.IHLIB(1,IND1,1)).AND.
     1     (IHLIB(2,I,1).EQ.IHLIB(2,IND1,1)).AND.
     2     (ILLIB(I).EQ.ILLIB(IND1))) THEN
          NBIS=NBIS+1
          LTEST=LTEST.OR.MASKI(I)
        ELSE
          GO TO 70
        ENDIF
   60 CONTINUE
   70 WRITE(NAMLBT,'(2A4)') IHLIB(1,IND1,1),IHLIB(2,IND1,1)
      ILIB=ILLIB(IND1)
      IF(ILIB.EQ.0) THEN
         NAMFIL=' '
      ELSE
         WRITE(NAMFIL,'(16A4)') (INAME(I,ILIB),I=1,16)
      ENDIF
      NDEL0=0
      IF(LTEST.AND.(NAMLBT.EQ.'DRAGON')) THEN
*        TRANSFER INFORMATION FROM DRAGON LIBRARY TO LCM.
         CALL LCMINF(IPLIB,NAMLCM,NAMMY,EMPTY,ILONG,LCM)
         IF(NAMFIL.EQ.NAMLCM) THEN
           IPDRL=IPLIB
         ELSE
           CALL LCMOP(IPDRL,NAMFIL(:12),2,2,0)
         ENDIF
         CALL SETARA (BASE,NBIS,IAWR)
         CALL SETARA (BASE,NGRO,IDELTA)
         CALL SETARA (BASE,NGRO*NL,ISIGS)
         CALL SETARA (BASE,NGRO*NGRO*NL,ISCAT)
         CALL SETARA (BASE,NGRO,ITOTAL)
         CALL SETARA (BASE,NGRO*11,ISIGF)
         CALL SETARA (BASE,NGRO*11,ICHI)
         CALL SETARA (BASE,NGRO*NED,ISADD)
         CALL SETARA (BASE,NL,ILSCAT)
         CALL SETARA (BASE,NED,ILADD)
         CALL SETARA (BASE,NGRO,IGOLD)
         CALL SETARA (BASE,NGRO,INFS)
         CALL SETARA (BASE,NGRO,IZNPHI)
         CALL SETARA (BASE,NL,IITYPR)
         CALL LIBDRA (IPLIB,IPDRL,NAMFIL,NGRO,NBIS,NL,ISONAM(1,IND1),
     1   ISONRF(1,IND1),TMPISO(IND1),SN(1,IND1),SB(1,IND1),MASKI(IND1),
     2   NED,HVECT,IMPX,NGF0,NGFR0,NDEL0,BASE(IAWR),BASE(IDELTA),
     3   BASE(ISIGS),BASE(ISCAT),BASE(ITOTAL),BASE(ISIGF),BASE(ICHI),
     4   BASE(ISADD),BASE(ILSCAT),BASE(ILADD),BASE(IGOLD),BASE(INFS),
     5   BASE(IZNPHI),BASE(IITYPR))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(IZNPHI))
         CALL RLSARA (BASE(INFS))
         CALL RLSARA (BASE(IGOLD))
         CALL RLSARA (BASE(ILADD))
         CALL RLSARA (BASE(ILSCAT))
         CALL RLSARA (BASE(ISADD))
         CALL RLSARA (BASE(ICHI))
         CALL RLSARA (BASE(ISIGF))
         CALL RLSARA (BASE(ITOTAL))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(IDELTA))
         CALL RLSARA (BASE(IAWR))
         IF(NAMFIL.NE.NAMLCM) CALL LCMCL(IPDRL,1)
      ELSE IF(LTEST.AND.(NAMLBT(1:4).EQ.'WIMS')) THEN
*        TRANSFER INFORMATION FROM WIMS LIBRARY FILE TO LCM.
         CALL SETARA(BASE,NL,ITYPRO)
         CALL SETARA(BASE,NGRO,IDELTA)
         CALL SETARA(BASE,NGRO*(10+NL),IXSREC)
         CALL SETARA(BASE,NGRO*NGRO*NL,ISCAT)
         CALL SETARA(BASE,NGRO*(NGRO+2),IXSCMP)
         CALL SETARA(BASE,NGRO,INJJ)
         CALL SETARA(BASE,NGRO,IIJJ)
         CALL SETARA(BASE,7*NGRO,IXSOUT)
         IF(NAMLBT.EQ.'WIMSD4') THEN
*           WIMS-D4 FORMAT
            CALL SETARA(BASE,NBIS,ISORD)
            CALL SETARA(BASE,NBIS,IAWR)
            CALL SETARA(BASE,NGRO*4,IGAR)
            CALL LIBWD4(IPLIB,IMPX,NAMFIL,NGRO,NBIS,NL,ISONAM(1,IND1),
     1      ISONRF(1,IND1),ISHINA(1,IND1),TMPISO(IND1),SN(1,IND1),
     2      SB(1,IND1),MASKI(IND1),NGF0,NGFR0,BASE(ITYPRO),BASE(IDELTA),
     3      BASE(IXSREC),BASE(ISCAT),BASE(IXSCMP),BASE(INJJ),
     4      BASE(IIJJ),BASE(IXSOUT),BASE(ISORD),BASE(IAWR),BASE(IGAR))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IAWR))
            CALL RLSARA(BASE(ISORD))
         ELSE
*           WIMS-AECL FORMAT
            CALL SETARA(BASE,NGRO*2,IGAR)
            CALL LIBWIM(IPLIB,IMPX,NAMFIL,NGRO,NBIS,NL,ISONAM(1,IND1),
     1      ISONRF(1,IND1),ISHINA(1,IND1),TMPISO(IND1),SN(1,IND1),
     2      SB(1,IND1),MASKI(IND1),NGF0,NGFR0,BASE(ITYPRO),BASE(IDELTA),
     3      BASE(IXSREC),BASE(ISCAT),BASE(IXSCMP),BASE(INJJ),BASE(IIJJ),
     4      BASE(IXSOUT),BASE(IGAR))
            CALL RLSARA(BASE(IGAR))
         ENDIF
         CALL RLSARA(BASE(IXSOUT))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IXSCMP))
         CALL RLSARA(BASE(ISCAT))
         CALL RLSARA(BASE(IXSREC))
         CALL RLSARA(BASE(IDELTA))
         CALL RLSARA(BASE(ITYPRO))
      ELSE IF(LTEST.AND.(NAMLBT.EQ.'MATXS')) THEN
*        TRANSFER INFORMATION FROM MATXS (NJOY-89) TO LCM.
         CALL SETARA (BASE,NBIS,IAWR)
         CALL SETARA (BASE,NBIS,ICNORM)
         CALL SETARA (BASE,NBIS,ISNORM)
         CALL SETARA (BASE,NBIS,IDNORM)
         CALL SETARA (BASE,NGRO,ISFIS)
         CALL SETARA (BASE,NGRO,ISAVE)
         CALL SETARA (BASE,NGRO*NBIS,ICHI)
         CALL SETARA (BASE,NGRO*NL*NBIS,ISIGS)
         CALL SETARA (BASE,NGRO*NBIS,ISIGF)
         CALL SETARA (BASE,NGRO*NBIS,ITOTAL)
         CALL SETARA (BASE,NGRO*NGRO*NL,ISCAT)
         CALL SETARA (BASE,NGRO*NBIS,IFLUX)
         CALL SETARA (BASE,NGRO,IVECT)
         CALL SETARA (BASE,2*NBIS,IIPR)
         CALL SETARA (BASE,NGRO,IGAR)
         CALL SETARA (BASE,NL,IITYPR)
         CALL SETARA (BASE,NED*NBIS,ILOGIE)
         CALL LIBTR1 (IPLIB,NAMFIL,NGRO,NBIS,NL,ISONAM(1,IND1),
     1   ISONRF(1,IND1),IHLIB(1,IND1,2),IHLIB(1,IND1,3),NTFG(IND1),
     2   TMPISO(IND1),SN(1,IND1),SB(1,IND1),MASKI(IND1),NED,HVECT,
     3   ITIME,IMPX,NGF0,NGFR0,BASE(IAWR),BASE(ICNORM),BASE(ISNORM),
     4   BASE(IDNORM),BASE(ISFIS),BASE(ISAVE),BASE(ICHI),BASE(ISIGS),
     5   BASE(ISIGF),BASE(ITOTAL),BASE(ISCAT),BASE(IFLUX),BASE(IVECT),
     6   BASE(IIPR),BASE(IGAR),BASE(IITYPR),BASE(ILOGIE))
         CALL RLSARA (BASE(ILOGIE))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(IGAR))
         CALL RLSARA (BASE(IIPR))
         CALL RLSARA (BASE(IVECT))
         CALL RLSARA (BASE(IFLUX))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ITOTAL))
         CALL RLSARA (BASE(ISIGF))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(ICHI))
         CALL RLSARA (BASE(ISAVE))
         CALL RLSARA (BASE(ISFIS))
         CALL RLSARA (BASE(IDNORM))
         CALL RLSARA (BASE(ISNORM))
         CALL RLSARA (BASE(ICNORM))
         CALL RLSARA (BASE(IAWR))
      ELSE IF(LTEST.AND.(NAMLBT.EQ.'MATXS2')) THEN
*        TRANSFER INFORMATION FROM MATXS (NJOY-91) TO LCM.
         CALL SETARA (BASE,NGRO,ISFIS)
         CALL SETARA (BASE,NGRO,ISAVE)
         CALL SETARA (BASE,NGRO,ICHI)
         CALL SETARA (BASE,NGRO*NL,ISIGS)
         CALL SETARA (BASE,NGRO,ISIGF)
         CALL SETARA (BASE,NGRO,ITOTAL)
         CALL SETARA (BASE,NGRO*NGRO*NL,ISCAT)
         CALL SETARA (BASE,NGRO,IFLUX)
         CALL SETARA (BASE,NGRO,IVECT)
         CALL SETARA (BASE,NBIS,IIPR)
         CALL SETARA (BASE,NGRO,IGAR)
         CALL SETARA (BASE,NGRO*NGRO*NL,ILCMAT)
         CALL SETARA (BASE,NL,IITYPR)
         CALL SETARA (BASE,NED,ILOGIE)
         CALL LIBTR2 (IPLIB,NAMFIL,NGRO,NBIS,NL,ISONAM(1,IND1),
     1   ISONRF(1,IND1),IHLIB(1,IND1,2),IHLIB(1,IND1,3),NTFG(IND1),
     2   TMPISO(IND1),SN(1,IND1),SB(1,IND1),MASKI(IND1),NED,HVECT,
     3   ITIME,IMPX,NGF0,NGFR0,BASE(ISFIS),BASE(ISAVE),BASE(ICHI),
     4   BASE(ISIGS),BASE(ISIGF),BASE(ITOTAL),BASE(ISCAT),BASE(IFLUX),
     5   BASE(IVECT),BASE(IIPR),BASE(IGAR),BASE(ILCMAT),BASE(IITYPR),
     6   BASE(ILOGIE))
         CALL RLSARA (BASE(ILOGIE))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ILCMAT))
         CALL RLSARA (BASE(IGAR))
         CALL RLSARA (BASE(IIPR))
         CALL RLSARA (BASE(IVECT))
         CALL RLSARA (BASE(IFLUX))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ITOTAL))
         CALL RLSARA (BASE(ISIGF))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(ICHI))
         CALL RLSARA (BASE(ISAVE))
         CALL RLSARA (BASE(ISFIS))
      ELSE IF(LTEST.AND.(NAMLBT.EQ.'APLIB1')) THEN
*        TRANSFER INFORMATION FROM APOLIB-1 TO LCM.
         MAXTRA=NL*NGRO**2
         CALL SETARA (BASE,NBIS,IAWR)
         CALL SETARA (BASE,4*NBIS,IIPR)
         CALL SETARA (BASE,NGRO+1,IVECT)
         CALL SETARA (BASE,NGRO*NL,ISIGS)
         CALL SETARA (BASE,NGRO,ISIG1)
         CALL SETARA (BASE,NGRO,ISIGA)
         CALL SETARA (BASE,NGRO,ISIGF)
         CALL SETARA (BASE,MAXTRA,IPRI)
         CALL SETARA (BASE,NGRO,IVTHER)
         CALL SETARA (BASE,11*NGRO,IPHI)
         CALL SETARA (BASE,NGRO*(NGRO+1),IPP)
         CALL SETARA (BASE,NGRO*(NGRO+1),IPP1)
         CALL SETARA (BASE,NGRO,ISSS)
         CALL SETARA (BASE,NGRO,ISSS1)
         CALL SETARA (BASE,NGRO,ISS1)
         CALL SETARA (BASE,NGRO,ISS11)
         CALL SETARA (BASE,NGRO,IUUU)
         CALL SETARA (BASE,NGRO,IDELTA)
         CALL SETARA (BASE,(NGRO**2)*NL,ISCAT)
         CALL SETARA (BASE,4*MAXDIL*NGRO,ISEFF)
         CALL SETARA (BASE,NBIS,INISB)
         CALL SETARA (BASE,NBIS,INISBE)
         CALL SETARA (BASE,NGRO,ILINF)
         CALL SETARA (BASE,NL,IITYPR)
         CALL LIBAPL (IPLIB,NAMFIL,MAXTRA,NGRO,NBIS,NL,ISONAM(1,IND1),
     1   ISONRF(1,IND1),ISHINA(1,IND1),MASKI(IND1),TMPISO(IND1),
     2   SN(1,IND1),SB(1,IND1),IMPX,NGF0,NGFR0,BASE(IAWR),BASE(IIPR),
     3   BASE(IVECT),BASE(ISIGS),BASE(ISIG1),BASE(ISIGA),BASE(ISIGF),
     4   BASE(IPRI),BASE(IVTHER),BASE(IPHI),BASE(IPP),BASE(IPP1),
     5   BASE(ISSS),BASE(ISSS1),BASE(ISS1),BASE(ISS11),BASE(IUUU),
     6   BASE(IDELTA),BASE(ISCAT),BASE(ISEFF),BASE(INISB),BASE(INISBE),
     7   BASE(ILINF),BASE(IITYPR))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ILINF))
         CALL RLSARA (BASE(INISBE))
         CALL RLSARA (BASE(INISB))
         CALL RLSARA (BASE(ISEFF))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(IDELTA))
         CALL RLSARA (BASE(IUUU))
         CALL RLSARA (BASE(ISS11))
         CALL RLSARA (BASE(ISS1))
         CALL RLSARA (BASE(ISSS1))
         CALL RLSARA (BASE(ISSS))
         CALL RLSARA (BASE(IPP1))
         CALL RLSARA (BASE(IPP))
         CALL RLSARA (BASE(IPHI))
         CALL RLSARA (BASE(IVTHER))
         CALL RLSARA (BASE(IPRI))
         CALL RLSARA (BASE(ISIGF))
         CALL RLSARA (BASE(ISIGA))
         CALL RLSARA (BASE(ISIG1))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(IVECT))
         CALL RLSARA (BASE(IIPR))
         CALL RLSARA (BASE(IAWR))
      ELSE IF(LTEST.AND.(NAMLBT.EQ.'APLIB2')) THEN
*        TRANSFER INFORMATION FROM APOLIB-2 TO LCM.
         CALL SETARA (BASE,NBIS,IAWR)
         CALL SETARA (BASE,(7+2*(NL-1))*NBIS,IIPR)
         CALL SETARA (BASE,NGRO+1,IENERG)
         CALL SETARA (BASE,NGRO,IDELTA)
         CALL SETARA (BASE,NGRO,ISECT)
         CALL SETARA (BASE,NL*NGRO,ISIGS)
         CALL SETARA (BASE,NL*NGRO**2,ISCAT)
         CALL SETARA (BASE,NL,IITYPR)
         CALL SETARA (BASE,NGRO,IXSTOT)
         CALL SETARA (BASE,NGRO,INFS)
         CALL LIBA20 (IPLIB,NAMFIL,NGRO,NBIS,NL,IPROC,ISONAM(1,IND1),
     1   ISONRF(1,IND1),ISHINA(1,IND1),MASKI(IND1),TMPISO(IND1),
     2   SN(1,IND1),SB(1,IND1),IMPX,NGF0,NGFR0,NDEL0,BASE(IAWR),
     3   BASE(IIPR),BASE(IENERG),BASE(IDELTA),BASE(ISECT),BASE(ISIGS),
     4   BASE(ISCAT),BASE(IITYPR),BASE(IXSTOT),BASE(INFS))
         CALL RLSARA (BASE(INFS))
         CALL RLSARA (BASE(IXSTOT))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(ISECT))
         CALL RLSARA (BASE(IDELTA))
         CALL RLSARA (BASE(IENERG))
         CALL RLSARA (BASE(IIPR))
         CALL RLSARA (BASE(IAWR))
      ELSE IF(LTEST.AND.(NAMLBT.EQ.'NDAS')) THEN
         CALL LIBND1 (IPLIB,NAMFIL,NGRO,NBIS,NL,ISONAM(1,IND1),
     1   ISONRF(1,IND1),ISHINA(1,IND1),MASKI(IND1),TMPISO(IND1),
     2   SN(1,IND1),SB(1,IND1),IMPX,NGF0,NGFR0,NDEL0)
      ENDIF
      IF(LTEST) THEN
         NGF=MIN(NGF,NGF0)
         IGRMAX=MAX(IGRMAX,NGFR0)
         IF(NDEL.EQ.0) THEN
            NDEL=NDEL0
         ELSE IF((NDEL0.NE.NDEL).AND.(NDEL0.NE.0)) THEN
            ILIB=ILLIB(IND1)
            IF(ILIB.GT.0) WRITE(6,210) (INAME(I,ILIB),I=1,4),NDEL0,NDEL
            NDEL=MAX(NDEL,NDEL0)
         ENDIF
*
*        COMPUTE THE TRANSPORT XS AND ADD COMPLEMENTARY INFORMATION.
         CALL SETARA (BASE,NGRO,IWORK)
         CALL SETARA (BASE,NGRO,IWR2)
         CALL SETARA (BASE,NGRO*NGRO,ISCAT)
         CALL SETARA (BASE,NGRO+1,IDELTA)
         CALL LIBADD (IPLIB,NBIS,MASKI(IND1),IMPX,NGRO,NL,ITRANC,
     1   ISONAM(1,IND1),NIR(IND1),GIR(IND1),BASE(IWORK),BASE(IWR2),
     2   BASE(ISCAT),BASE(IDELTA))
         CALL RLSARA (BASE(IDELTA))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(IWR2))
         CALL RLSARA (BASE(IWORK))
      ENDIF
*
      IND1=IND1+NBIS
      IF(IND1.LE.NBISO) GO TO 50
      RETURN
*
  200 FORMAT(8HLIBLIC: ,3A4,34H IS A DUPLICATE ISOTOPE/MATERIAL N,
     1 4HAME.)
  210 FORMAT(/51H LIBLIC: INVALID NB OF PRECURSOR GROUPS IN LIBRARY ,
     1 4A4,8H (NDEL0=,I3,6H NDEL=,I3,2H).)
      END
