*DECK LIBMIX
      SUBROUTINE LIBMIX(IPLIB,NBMIX,NGROUP,NBISO,ISONAM,MIX,DEN,MASK,
     1 MASKL,ITSTMP,TMPDAY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transformation of the isotope ordered microscopic cross sections to
* group ordered macroscopic cross sections (part 1).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature).
* NBMIX   number of material mixtures.
* NGROUP  number of energy groups.
* NBISO   number of isotopes present in the calculation domain.
* ISONAM  alias name of isotopes.
* MIX     mixture number of each isotope (can be zero).
* DEN     density of each isotope.
* MASK    mixture mask (=.true. if a mixture is to be made).
* MASKL   group mask (=.true. if an energy group is to be treated).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NBMIX,NGROUP,NBISO,ISONAM(3,NBISO),MIX(NBISO),ITSTMP
      REAL DEN(NBISO),TMPDAY(3)
      LOGICAL MASK(NBMIX),MASKL(NGROUP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NBLK=50,NSTATE=40)
      LOGICAL LSAME
      INTEGER ISTATE(NSTATE),KCHAR(3)
      CHARACTER TEXT12*12
      REAL OLDTIM(3)
      COMMON BASE(1)
*----
*  RECOVER SOME LIBRARY PARAMETERS.
*----
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NL=ISTATE(4)
      ITRANC=ISTATE(5)
      IPROB=ISTATE(6)
      NED=ISTATE(13)
      NDEL=ISTATE(19)
      CALL SETARA(BASE,2*NED,IJNED)
      CALL LCMGET(IPLIB,'ADDXSNAME-P0',BASE(IJNED))
*----
*  LOOK FOR OLD LIBRARY DATA
*----
      CALL LCMLEN(IPLIB,'MACROLIB',LENGTH,ITYLCM)
      IF(LENGTH.EQ.-1) THEN
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
        IF(TEXT12.NE.'L_MACROLIB') THEN
          CALL XABORT('LIBMIX: INVALID SIGNATURE ON THE MACROLIB.')
        ENDIF
        CALL LCMLEN(IPLIB,'TIMESTAMP',LENGTH,ITYLCM)
        IF((LENGTH.GT.0).AND.(LENGTH.LE.3)) THEN
          CALL LCMGET(IPLIB,'TIMESTAMP',OLDTIM)
          IF(ITSTMP.EQ.0) THEN
            TMPDAY(1)=OLDTIM(1)
            TMPDAY(2)=OLDTIM(2)
            TMPDAY(3)=OLDTIM(3)
          ENDIF
        ENDIF
        CALL LCMSIX(IPLIB,' ',2)
      ENDIF
*----
*  TRANSPOSE THE MICROSCOPIC CROSS SECTIONS TO ADJOINT ORDERING.
*----
      IF(IPROB.EQ.1) THEN
         CALL SETARA (BASE,NGROUP*2,IGA1)
         CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
         CALL SETARA (BASE,NGROUP*NL,ISIGS)
         CALL SETARA (BASE,NGROUP*NGROUP*NL,ISCAT)
         CALL SETARA (BASE,NL,IITYPR)
         CALL LIBADJ (IPLIB,NGROUP,NBISO,NL,NDEL,ISONAM,NED,BASE(IJNED),
     1   BASE(IGA1),BASE(IGA2),BASE(ISIGS),BASE(ISCAT),BASE(IITYPR))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(IGA2))
         CALL RLSARA (BASE(IGA1))
      ENDIF
*----
*  COMPUTE THE MAXIMUM NUMBER OF FISSIONABLE ISOTOPES IN A MIXTURE.
*----
      DO 10 I=1,NBISO
      IF(MIX(I).GT.NBMIX) CALL XABORT('LIBMIX: NBMIX OVERFLOW.')
   10 CONTINUE
      MAXNFI=MIN(NBISO,200)
      CALL SETARA (BASE,NBMIX*MAXNFI,IINDFI)
      CALL SETARA (BASE,NGROUP,ICHI1)
      CALL LIBNFI (IPLIB,NGROUP,NBISO,NBMIX,NDEL,ISONAM,MIX,MAXNFI,
     1 NFISSI,LSAME,BASE(IINDFI),BASE(ICHI1))
      CALL RLSARA (BASE(ICHI1))
      CALL RLSARA (BASE(IINDFI))
*----
*  BUILD THE MACROSCOPIC CROSS SECTIONS.
*----
      CALL SETARA (BASE,NGROUP+1,IGA1)
      CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
      CALL SETARA (BASE,NDEL*NFISSI,IGA3)
      CALL SETARA (BASE,NBMIX*(NBLK+1),IGAR)
      CALL SETARA (BASE,NBMIX*NGROUP*NBLK,IGAF)
      CALL SETARA (BASE,NGROUP*NBMIX,ISCAT)
      CALL SETARA (BASE,NBMIX*NBLK,INJJM)
      CALL SETARA (BASE,NBMIX*NBLK,IIJJM)
      CALL SETARA (BASE,NBMIX,IIPOS)
      CALL SETARA (BASE,NBMIX*NGROUP*NL,ICHECK)
      CALL SETARA (BASE,NBMIX*NFISSI,IINDFI)
      CALL SETARA (BASE,NBMIX*NFISSI*NGROUP*(NDEL+1),IZNUS)
      CALL SETARA (BASE,NBMIX*NFISSI*NGROUP*(NDEL+1),IZCHI)
      CALL SETARA (BASE,NGROUP,IIJJ)
      CALL SETARA (BASE,NGROUP,INJJ)
      CALL SETARA (BASE,NBISO,ILMADE)
      CALL SETARA (BASE,NGROUP,IIPGRP)
      CALL LIBDEN (IPLIB,NGROUP,NBISO,NBMIX,NL,NDEL,ISONAM,MIX,DEN,
     1 MASK,MASKL,NED,BASE(IJNED),ITRANC,NFISSI,LSAME,ITSTMP,TMPDAY,
     2 BASE(IGA1),BASE(IGA2),BASE(IGA3),BASE(IGAR),BASE(IGAF),
     3 BASE(ISCAT),BASE(INJJM),BASE(IIJJM),BASE(IIPOS),BASE(ICHECK),
     4 BASE(IINDFI),BASE(IZNUS),BASE(IZCHI),BASE(IIJJ),BASE(INJJ),
     5 BASE(ILMADE),BASE(IIPGRP))
      CALL RLSARA (BASE(IIPGRP))
      CALL RLSARA (BASE(ILMADE))
      CALL RLSARA (BASE(INJJ))
      CALL RLSARA (BASE(IIJJ))
      CALL RLSARA (BASE(IZCHI))
      CALL RLSARA (BASE(IZNUS))
      CALL RLSARA (BASE(IINDFI))
      CALL RLSARA (BASE(ICHECK))
      CALL RLSARA (BASE(IIPOS))
      CALL RLSARA (BASE(IIJJM))
      CALL RLSARA (BASE(INJJM))
      CALL RLSARA (BASE(ISCAT))
      CALL RLSARA (BASE(IGAF))
      CALL RLSARA (BASE(IGAR))
      CALL RLSARA (BASE(IGA3))
      CALL RLSARA (BASE(IGA2))
      CALL RLSARA (BASE(IGA1))
*----
*  TRANSPOSE THE MICROSCOPIC CROSS SECTIONS BACK TO FORWARD ORDERING.
*----
      IF(IPROB.EQ.1) THEN
         CALL SETARA (BASE,NGROUP*2,IGA1)
         CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
         CALL SETARA (BASE,NGROUP*NL,ISIGS)
         CALL SETARA (BASE,NGROUP*NGROUP*NL,ISCAT)
         CALL SETARA (BASE,NL,IITYPR)
         CALL LIBADJ (IPLIB,NGROUP,NBISO,NL,NDEL,ISONAM,NED,BASE(IJNED),
     1   BASE(IGA1),BASE(IGA2),BASE(ISIGS),BASE(ISCAT),BASE(IITYPR))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(IGA2))
         CALL RLSARA (BASE(IGA1))
      ENDIF
      CALL RLSARA(BASE(IJNED))
      RETURN
      END
