*DECK LIBNFI
      SUBROUTINE LIBNFI(IPLIB,NGRO,NBISO,NBMIX,NDEL,ISONAM,MIX,MAXNFI,
     1 NFISSI,LSAME,INDFIS,CHI1)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the maximum number of fissionable isotopes in a mixture
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature)
* NGRO    number of energy groups
* NBISO   number of isotopes present in the calculation domain
* NBMIX   number of mixtures present in the calculation domain
* NDEL    number of delayed precursor groups
* ISONAM  alias name of isotopes
* MIX     mixture number of each isotope. can be zero
* MAXNFI  second dimension of array INDFIS
*
*Parameters: output
* NFISSI  maximum number of fissionable isotopes in a mixture
* LSAME   fission spectrum mask (=.true. if all the isotopes have the
*         same fission spectrum and the same precursor group decay
*         constants
*
*Parameters: scratch
* INDFIS
* CHI1
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NGRO,NBISO,NBMIX,NDEL,ISONAM(3,NBISO),MIX(NBISO),
     1 MAXNFI,NFISSI,INDFIS(NBMIX,MAXNFI)
      REAL CHI1(NGRO)
      LOGICAL LSAME
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MAXGRO=50,NSTATE=40)
      CHARACTER HSMG*131,TEXT12*12
      REAL CHI2(MAXGRO),LAM1(MAXGRO),LAM2(MAXGRO)
      INTEGER IDATA(NSTATE),KCHAR(3)
      LOGICAL LFISS
      COMMON IBASE(1)
*
      NFISSI=0
      CALL LCMLEN(IPLIB,'MACROLIB',ILONG,ITYLCM)
      LSAME=(NGRO.LE.MAXGRO).AND.(NDEL.LE.MAXGRO)
      IF(ILONG.EQ.-1) THEN
         CALL LCMSIX(IPLIB,'MACROLIB',1)
         CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
         WRITE(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
         IF(TEXT12.NE.'L_MACROLIB') THEN
            CALL XABORT('LIBNFI: INVALID SIGNATURE ON THE MACROLIB.')
         ENDIF
         CALL LCMGET(IPLIB,'STATE-VECTOR',IDATA)
         IF(IDATA(1).NE.NGRO) THEN
            WRITE(HSMG,'(38HLIBNFI: EXISTING MACROLIB HAVE NGROUP=,I4,
     1      26H NEW MACROLIB HAVE NGROUP=,I4,1H.)') IDATA(1),NGRO
            CALL XABORT(HSMG)
         ELSE IF(IDATA(2).GT.NBMIX) THEN
            WRITE(HSMG,'(37HLIBNFI: EXISTING MACROLIB HAVE NBMIX=,I4,
     1      25H NEW MACROLIB HAVE NBMIX=,I4,1H.)') IDATA(2),NBMIX
            CALL XABORT(HSMG)
         ELSE IF(IDATA(4).GT.NBISO) THEN
            WRITE(HSMG,'(38HLIBNFI: EXISTING MACROLIB HAVE NFISSI=,I4,
     1      13H GREATER THAN,I5,1H.)') IDATA(4),NBISO
            CALL XABORT(HSMG)
         ENDIF
         NFISSI=IDATA(4)
         LSAME=LSAME.AND.(NFISSI.LE.1)
         IF(NFISSI.GT.0) THEN
            CALL LCMLEN(IPLIB,'FISSIONINDEX',LENGTH,ITYLCM)
            IF(LENGTH.EQ.0) THEN
*              THE NAMES ARE NOT DEFINED.
               DO 10 IFIS=1,NFISSI
               DO 10 IBM=1,NBMIX
   10          INDFIS(IBM,IFIS)=0
            ELSE IF(LENGTH.EQ.NFISSI*NBMIX) THEN
               CALL LCMGET(IPLIB,'FISSIONINDEX',INDFIS)
            ELSE IF(LENGTH.LT.NFISSI*NBMIX) THEN
*              REORDER THE 'FISSIONINDEX' MATRIX.
               CALL SETARA(IBASE,LENGTH,IWRK)
               CALL LCMGET(IPLIB,'FISSIONINDEX',IBASE(IWRK))
               DO 30 IFIS=1,NFISSI
               DO 20 IBM=1,IDATA(2)
   20          INDFIS(IBM,IFIS)=IBASE(IWRK+(IFIS-1)*IDATA(2)+IBM-1)
               DO 30 IBM=IDATA(2)+1,NBMIX
   30          INDFIS(IBM,IFIS)=0
               CALL RLSARA(IBASE(IWRK))
            ELSE
               CALL XABORT('LIBNFI: INVALID NUMBER OF MIXTURES.')
            ENDIF
         ENDIF
         CALL LCMSIX(IPLIB,' ',2)
      ENDIF
      DO 100 ISOT=1,NBISO
      IBM=MIX(ISOT)
      IF(IBM.GT.0) THEN
         WRITE(TEXT12,'(3A4)') (ISONAM(I0,ISOT),I0=1,3)
         CALL LCMLEN(IPLIB,TEXT12,LENGTH,ITYLCM)
         IF(LENGTH.NE.0) THEN
            CALL LCMSIX(IPLIB,TEXT12,1)
            CALL LCMLEN(IPLIB,'NUSIGF',LENGTH,ITYLCM)
            CALL LCMLEN(IPLIB,'CHI',LENGTZ,ITYLCM)
            IF((LENGTH.GT.0).AND.(LENGTZ.GT.0)) THEN
               CALL LCMGET(IPLIB,'CHI',CHI1)
               LFISS=.FALSE.
               DO 35 IGR=1,NGRO
               LFISS=LFISS.OR.(CHI1(IGR).GT.0.0)
   35          CONTINUE
               IF(.NOT.LFISS) THEN
                  CALL LCMSIX(IPLIB,' ',2)
                  GO TO 100
               ENDIF
               IF(LSAME) THEN
                  CALL LCMLEN(IPLIB,'LAMBDA-D',LENGT1,ITYLCM)
                  IF((LENGT1.EQ.NDEL).AND.(NDEL.GT.0)) THEN
                     CALL LCMGET(IPLIB,'LAMBDA-D',LAM1)
                  ENDIF
               ENDIF
               CALL LCMSIX(IPLIB,' ',2)
               DO 40 IFIS=1,NFISSI
               IWFIS=INDFIS(IBM,IFIS)
               IF((IWFIS.EQ.ISOT).OR.(IWFIS.EQ.0)) THEN
                  KFIS=IFIS
                  GO TO 90
               ENDIF
   40          CONTINUE
               IF(LSAME) THEN
                  DO 70 IFIS=1,NFISSI
                  IWFIS=INDFIS(IBM,IFIS)
                  WRITE(TEXT12,'(3A4)') (ISONAM(I0,IWFIS),I0=1,3)
                  CALL LCMSIX(IPLIB,TEXT12,1)
                  CALL LCMGET(IPLIB,'CHI',CHI2)
                  DO 50 IGR=1,NGRO
   50             LSAME=LSAME.AND.(ABS(CHI1(IGR)-CHI2(IGR)).LE.1.0E-3)
                  CALL LCMLEN(IPLIB,'LAMBDA-D',LENGT2,ITYLCM)
                  IF((LENGT1.EQ.NDEL).AND.(LENGT2.EQ.NDEL)
     1                               .AND.(NDEL.GT.0)) THEN
                     CALL LCMGET(IPLIB,'LAMBDA-D',LAM2)
                     DO 60 IGR=1,NDEL
   60                LSAME=LSAME.AND.(LAM1(IGR).EQ.LAM2(IGR))
                  ENDIF
                  CALL LCMSIX(IPLIB,' ',2)
   70             CONTINUE
               ENDIF
               NFISSI=NFISSI+1
               IF(NFISSI.GT.MAXNFI) CALL XABORT('LIBNFI: INDFIS OVERFL'
     1         //'OW.')
               KFIS=NFISSI
               DO 80 JBM=1,NBMIX
   80          INDFIS(JBM,KFIS)=0
   90          INDFIS(IBM,KFIS)=ISOT
            ELSE
               CALL LCMSIX(IPLIB,' ',2)
            ENDIF
         ENDIF
      ENDIF
  100 CONTINUE
      RETURN
      END
