*DECK LIBPTW
      SUBROUTINE LIBPTW (IPLIB,IPTMP,IPROC,NGRO,NL,HNAMIS,NED,HVECT,
     1 NDEL,NDIL,DILUT,AWR,IPRECI,IMPX,ISMIN,ISMAX,LSCAT,LADD,GOLD,NFS,
     2 NOR,NSIGQA,NSIGTA,ISM)
*
*-----------------------------------------------------------------------
*
*Purpose:
* build a temperature-independant draglib (IPROC=2) on the internal
* library or write the probability table information (IPROC=3/4).
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature).
* IPTMP   pointer to the multi-dilution internal library.
* IPROC   type of microlib processing:
*         =1: perform temperature interpolation and compute physical
*             probability tables.
*         =2: perform temperature interpolation and build a
*             temperature-independent draglib;
*         =3: perform temperature interpolation and compute calendf-
*             type mathematical probability tables based on bin-type
*             cross-sections for total cross sections;
*         =4: perform temperature interpolation and compute physical
*             probability tables or slowing-down correlated probability
*             tables.
*         =5: perform temperature interpolation and compute calendf-
*             type mathematical probability tables based on bin-type
*             cross-sections for all available cross-sections types.
* NGRO    number of energy groups.
* NL      number of Legendre orders required in the calculation
*         (NL=1 or higher).
* HNAMIS  local name of the isotope:
*         HNAMIS(1:8)  is the local isotope name;
*         HNAMIS(9:12) is a suffix function of the mix number.
* NED     number of extra vector edits.
* HVECT   names of the extra vector edits.
* NDEL    number of delayed neutron precursor groups.
* NDIL    number of finite dilutions.
* DILUT   dilutions.
* AWR     mass ratio for current isotope.
* IPRECI  accuracy index for probability tables in CALENDF.
* IMPX    print flag.
*
*Parameters: scratch
* ISMIN
* ISMAX
* LSCAT
* LADD
* GOLD
* NFS
* NOR
* NSIGQA
* NSIGTA
* ISM
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IPTMP,IPROC,NGRO,NL,NED,NDEL,NDIL,IPRECI,IMPX,
     1 ISMIN(NL,NGRO),ISMAX(NL,NGRO),NFS(NGRO),NOR(NGRO),NSIGQA(NGRO),
     2 NSIGTA(NGRO),ISM(2,NL)
      REAL DILUT(NDIL+1),AWR,GOLD(NGRO)
      LOGICAL LSCAT(NL),LADD(NED)
      CHARACTER HNAMIS*12,HVECT(NED)*(6)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MAXNPT=12,MAXTRA=10000)
      CHARACTER TEXT12*12,TEXX12*12
      LOGICAL LSIGF,LGOLD,EMPTY,LCM,LBSIGF
      COMMON BASE(1),IBASE(1)
*
      CALL LCMSIX(IPLIB,HNAMIS,1)
*----
*  COPY INFINITE DILUTION DATA FROM IPTMP TO IPLIB.
*----
      TEXT12=HNAMIS(1:8)
      WRITE(TEXT12(9:12),'(I4.4)') NDIL+1
      CALL LCMSIX(IPTMP,TEXT12,1)
      CALL LCMEQU(IPTMP,IPLIB)
      CALL LCMSIX(IPTMP,' ',2)
*
      IF(NDIL.GT.0) THEN
*        RECOVER INFORMATION FROM IPTMP AND PUT NEW INFORMATION INTO
*        IPLIB.
         CALL SETARA(BASE,NGRO*(NDIL+1),IFLUX)
         CALL SETARA(BASE,NGRO*(NDIL+1),ITOTAL)
         CALL SETARA(BASE,NGRO*(NDIL+1),ISIGF)
         CALL SETARA(BASE,NGRO*NL*(NDIL+1),ISIGS)
         CALL SETARA(BASE,NGRO*NGRO*NL*(NDIL+1),ISCAT)
         CALL SETARA(BASE,NGRO*NED*(NDIL+1),ISADD)
         CALL SETARA(BASE,NGRO*NDEL*(NDIL+1),IZDEL)
         CALL SETARA(BASE,NGRO,IDELTG)
*
         CALL SETARA(BASE,NL,IITYPR)
         CALL SETARA(BASE,NDIL+1,IIPDIL)
         CALL LIBEXT(IPTMP,NGRO,NL,NDIL,HNAMIS,NED,HVECT,NDEL,IMPX,
     1   DILUT,MDIL,LSCAT,LSIGF,LADD,LGOLD,BASE(IFLUX),BASE(ITOTAL),
     2   BASE(ISIGF),BASE(ISIGS),BASE(ISCAT),BASE(ISADD),BASE(IZDEL),
     3   BASE(IDELTG),GOLD,ISMIN,ISMAX,BASE(IITYPR),BASE(IIPDIL))
         CALL RLSARA(BASE(IIPDIL))
         CALL RLSARA(BASE(IITYPR))
*
*        DESTROY THE MULTI-DILUTION INTERNAL LIBRARY.
         CALL LCMCL(IPTMP,2)
*
         IF(IPROC.EQ.1) THEN
*           COMPUTE THE PHYSICAL PROBABILITY TABLES.
            MAXNOR=MAXNPT
            CALL LCMINF(IPLIB,TEXT12,TEXX12,EMPTY,ILONG,LCM)
            CALL LCMSIX(IPLIB,'PT-PHYS',1)
            JPLIB=LCMLID(IPLIB,'GROUP',NGRO)
            DO 20 IGRP=1,NGRO
            NPART=3+NL+NED+NDEL
            DO 10 IL=1,NL
   10       NPART=NPART+MAX(ISMAX(IL,IGRP)-ISMIN(IL,IGRP)+1,0)
            IF(LGOLD) THEN
               GOLD0=GOLD(IGRP)
            ELSE
               GOLD0=1.0
            ENDIF
            CALL SETARA(BASE,MAXNOR*NPART,ISIGP)
            CALL XDRSET(BASE(ISIGP),MAXNOR*NPART,0.0)
            CALL SETARA(BASE,MDIL+1,ILDIL)
            CALL SETARA(BASE,MDIL+1,IDIL2)
            CALL LIBTAB(IGRP,NGRO,NL,MDIL,NPART,NED,NDEL,HNAMIS,IMPX,
     1      LSCAT,LSIGF,LADD,DILUT,BASE(ITOTAL),BASE(ISIGF),BASE(ISIGS),
     2      BASE(ISCAT),BASE(ISADD),BASE(IZDEL),GOLD0,ISMIN,ISMAX,
     3      NOR(IGRP),BASE(ISIGP),BASE(ILDIL),BASE(IDIL2))
            CALL RLSARA(BASE(IDIL2))
            CALL RLSARA(BASE(ILDIL))
*
            IF(NOR(IGRP).GT.1) THEN
*              SAVE THE PROBABILITY TABLE INTO IPLIB.
               KPLIB=LCMDIL(JPLIB,IGRP)
               IF(LCM) THEN
                  CALL LCMPOF(KPLIB,'PROB-TABLE',MAXNOR*NPART,2,ISIGP)
               ELSE
                  CALL LCMPUT(KPLIB,'PROB-TABLE',MAXNOR*NPART,2,
     1            BASE(ISIGP))
                  CALL RLSARA(BASE(ISIGP))
               ENDIF
               DO 15 IL=1,NL
               ISM(1,IL)=ISMIN(IL,IGRP)
   15          ISM(2,IL)=ISMAX(IL,IGRP)
               CALL LCMPUT(KPLIB,'ISM-LIMITS',2*NL,1,ISM)
            ELSE
               CALL RLSARA(BASE(ISIGP))
            ENDIF
   20       CONTINUE
            CALL LCMPUT(IPLIB,'NOR',NGRO,1,NOR)
            CALL LCMSIX(IPLIB,' ',2)
         ELSE IF(IPROC.EQ.2) THEN
*           BUILD A TEMPERATURE-INDEPENDENT DRAGLIB.
            CALL SETARA(BASE,NGRO,IGAS)
            CALL SETARA(BASE,NGRO*NL,IGA1)
            CALL SETARA(BASE,NGRO*NGRO*NL,IGA2)
            CALL SETARA(BASE,NL,IITYPR)
            CALL LIBNOT(IPLIB,NGRO,NL,MDIL,NED,NDEL,IMPX,LSCAT,LSIGF,
     1      LADD,DILUT,BASE(IFLUX),BASE(ITOTAL),BASE(ISIGF),BASE(ISIGS),
     2      BASE(ISCAT),BASE(ISADD),BASE(IZDEL),HVECT,BASE(IGAS),
     3      BASE(IGA1),BASE(IGA2),BASE(IITYPR))
            CALL RLSARA(BASE(IITYPR))
            CALL RLSARA(BASE(IGA2))
            CALL RLSARA(BASE(IGA1))
            CALL RLSARA(BASE(IGAS))
         ELSE IF((IPROC.EQ.3).OR.(IPROC.EQ.4).OR.(IPROC.EQ.5)) THEN
*           COMPUTE PHYSICAL PROBABILITY TABLES OR PROBABILITY TABLES
*           TAKING INTO ACCOUNT SLOWING-DOWN EFFECTS.
*
*           RECOVER BIN TYPE INFORMATION (IF AVAILABLE).
            LBSIGF=.FALSE.
            CALL LCMLEN(IPLIB,'BIN-NFS',LENGT,ITYLCM)
            LBIN=0
            IF(LENGT.GT.0) THEN
               CALL LCMGET(IPLIB,'BIN-NFS',NFS)
               IGRMIN=1
               IGRMAX=NGRO
               DO 30 IGRP=NGRO,1,-1
               IF((IGRMAX.EQ.IGRP).AND.(NFS(IGRP).EQ.0)) IGRMAX=IGRP-1
   30          LBIN=LBIN+NFS(IGRP)
               DO 40 IGRP=1,NGRO
               IF((IGRMIN.EQ.IGRP).AND.(NFS(IGRP).EQ.0)) IGRMIN=IGRP+1
   40          CONTINUE
               CALL SETARA(IBASE,NGRO,INBIN)
               CALL SETARA(BASE,LBIN,ITBIN)
               CALL SETARA(BASE,LBIN,ISBIN)
               CALL SETARA(BASE,LBIN,IFBIN) 
               CALL SETARA(BASE,LBIN+1,IEBIN)
               CALL LCMGET(IPLIB,'BIN-ENERGY',BASE(IEBIN))
               CALL LCMGET(IPLIB,'BIN-NTOT0',BASE(ITBIN))
               CALL LCMGET(IPLIB,'BIN-SIGS00',BASE(ISBIN))
               CALL LCMLEN(IPLIB,'BIN-NUSIGF',LENGT,ITYLCM)
               IF(LENGT.GT.0) THEN 
                  CALL LCMGET(IPLIB,'BIN-NUSIGF',BASE(IFBIN))
                  LBSIGF=.TRUE.
               ENDIF
            ELSE
               CALL XDISET(NFS,NGRO,0)
               IGRMIN=1
               IGRMAX=0
            ENDIF
*
*           RECOVER SCATTERING MATRIX PROFILE.
            LPART=0
            DO 50 IL=1,NL
            DO 50 IG1=1,NGRO
   50       LPART=MAX(LPART,ISMAX(IL,IG1)-ISMIN(IL,IG1)+1)
*
            CALL SETARA(BASE,MAXNPT*(3+NL+NL*LPART+NED+NDEL),IPRTS)
            CALL SETARA(BASE,MAXNPT*(3+NL+NL*LPART+NED+NDEL),IPRTS1)
            CALL SETARA(BASE,MAXNPT*(3+NL+NL*LPART+NED+NDEL),IPRTS2)
            CALL SETARA(BASE,LBIN*NGRO,ISCAT00)
            CALL SETARA(BASE,NGRO*NGRO,IWSCAT)
            CALL SETARA(BASE,LBIN,IGAR)
            CALL SETARA(BASE,MAXNPT*MAXNPT,IWSLD)
            CALL SETARA(BASE,LBIN,IDELTA)
            CALL SETARA(BASE,LBIN,IUUU)
            CALL SETARA(BASE,LBIN,ISTIS)
            CALL SETARA(BASE,LBIN,ISIGAF)
            CALL SETARA(BASE,LBIN*LPART*NL,ISIGPF)
            CALL SETARA(BASE,2*MAXNPT,IPHIMT)
            CALL SETARA(BASE,2*MAXNPT*(MAXNPT+1),IMATRI)
            CALL SETARA(BASE,MAXTRA*NL,IPRI)
            CALL SETARA(BASE,2*LBIN*MDIL*NL,IPHI)
            CALL SETARA(BASE,MAXNPT*MDIL*NL,IPRTPH)
            CALL SETARA(BASE,2*(MDIL+1)*MAXNPT,IWORK)
            CALL SETARA(BASE,2*MAXNPT,ICC)
            CALL SETARA(BASE,MAXNPT,IPRTW)
            CALL SETARA(BASE,MAXNPT,IPRTA)
            CALL SETARA(BASE,2*LBIN*(MDIL+1),IRSTAR)
            CALL SETARA(BASE,MAXNPT*(MDIL+1),IPRTRS)
            CALL LIBFQD(MAXNPT,LPART,MAXTRA,HNAMIS,IPLIB,NGRO,NL,NED,
     1      NDEL,MDIL,IGRMIN,IGRMAX,LBIN,NFS,IMPX,LSCAT,LSIGF,LADD,
     2      DILUT,BASE(IFLUX),BASE(ITOTAL),BASE(ISIGF),BASE(ISIGS),
     3      BASE(ISCAT),BASE(ISADD),BASE(IZDEL),BASE(IDELTG),
     4      BASE(IEBIN),BASE(ITBIN),BASE(ISBIN),BASE(IFBIN),AWR,ISMIN,
     5      ISMAX,GOLD,IPRECI,NOR,BASE(IPRTS),BASE(IWSLD),BASE(IDELTA),
     6      BASE(IUUU),BASE(ISTIS),BASE(ISIGAF),BASE(ISIGPF),
     7      BASE(IPHIMT),BASE(IMATRI),BASE(IPRI),BASE(IPHI),
     8      BASE(IPRTPH),BASE(IWORK),BASE(ICC),BASE(IPRTW),BASE(IPRTA),
     9      BASE(IRSTAR),BASE(IPRTRS),ISM,BASE(IPRTS1),BASE(IPRTS2),
     1      LBSIGF,BASE(ISCAT00),BASE(IWSCAT),BASE(IGAR))
            CALL RLSARA(BASE(IPRTRS))
            CALL RLSARA(BASE(IRSTAR))
            CALL RLSARA(BASE(IPRTA))
            CALL RLSARA(BASE(IPRTW))
            CALL RLSARA(BASE(ICC))
            CALL RLSARA(BASE(IWORK))
            CALL RLSARA(BASE(IPRTPH))
            CALL RLSARA(BASE(IPHI))
            CALL RLSARA(BASE(IPRI))
            CALL RLSARA(BASE(IMATRI))
            CALL RLSARA(BASE(IPHIMT))
            CALL RLSARA(BASE(ISIGPF))
            CALL RLSARA(BASE(ISIGAF))
            CALL RLSARA(BASE(ISTIS))
            CALL RLSARA(BASE(IUUU))
            CALL RLSARA(BASE(IDELTA))
            CALL RLSARA(BASE(IWSLD))
            CALL RLSARA(BASE(IPRTS))
            CALL RLSARA(BASE(IPRTS1))     
            CALL RLSARA(BASE(IPRTS2))
            CALL RLSARA(BASE(ISCAT00))
            CALL RLSARA(BASE(IWSCAT))
            CALL RLSARA(BASE(IGAR))
*           
            IF(LBIN.GT.0) THEN
               CALL RLSARA(BASE(IEBIN))
               CALL RLSARA(BASE(ISBIN))
               CALL RLSARA(BASE(ITBIN))
               CALL RLSARA(IBASE(INBIN))
               IF(LBSIGF) CALL RLSARA(BASE(IFBIN))
            ENDIF
         ELSE
            CALL XABORT('LIBPTW: INVALID VALUE OF IPROC.')
         ENDIF
         CALL RLSARA(BASE(IDELTG))
         CALL RLSARA(BASE(IZDEL))
         CALL RLSARA(BASE(ISADD))
         CALL RLSARA(BASE(ISCAT))
         CALL RLSARA(BASE(ISIGS))
         CALL RLSARA(BASE(ISIGF))
         CALL RLSARA(BASE(ITOTAL))
         CALL RLSARA(BASE(IFLUX))
      ELSE
*        DESTROY THE MULTI-DILUTION INTERNAL LIBRARY.
         CALL LCMCL(IPTMP,2)
      ENDIF
      CALL LCMSIX(IPLIB,' ',2)
*
      RETURN
      END
