*DECK M2T
      SUBROUTINE M2T(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
*  recover information from a macrolib and translate the requested data
*  towards an Apotrim interface file.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification ascii file containing
*         Apotrim data;
*         HENTRY(2): read-only type(L_MACROLIB).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,IOUT=6)
      CHARACTER TEXT12*12,TEXT20*20,HSIGN*12
      DOUBLE PRECISION DFLOTT
      INTEGER KCHAR(3),ISTATE(NSTATE),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.1) CALL XABORT('M2T: MINIMUM OF 2 OBJECTS EXPECTED.')
      IF(IENTRY(1).NE.4) CALL XABORT('M2T: ASCII FILE EXPECTED AT LHS.')
      IF(JENTRY(1).EQ.2) CALL XABORT('M2T: ASCII FILE IN CREATE OR MOD'
     1 //'IFICATION MODE EXPECTED.')
      LOUT=KENTRY(1)
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('M2T: LCM '
     1 //'OBJECT EXPECTED AT RHS.')
      IF(JENTRY(2).NE.2) CALL XABORT('M2T: LCM OBJECTS IN READ-ONLY MO'
     1 //'DE EXPECTED AT RHS.')
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_MACROLIB') CALL XABORT('M2T: MACROLIB OBJECT EXPE'
     1 //'CTED AT RHS.')
      IPMAC=KENTRY(2)
      CALL LCMGET(IPMAC,'STATE-VECTOR',ISTATE)
      NGRP=ISTATE(1)
      MAXMIX=ISTATE(2)
      MAXNL=ISTATE(3)
      NBFIS=ISTATE(4)
      IF(NBFIS.GT.1) CALL XABORT('M2T: THE CAPABILITY TO MERGE MANY FI'
     1 //'SSION SPECTRA IS NOT IMPLEMENTED.')
*----
*  ALLOCATE MEMORY
*----
      CALL SETARA(IBASE,MAXMIX,INBM)
      CALL SETARA(IBASE,5*MAXMIX,IHBM)
      CALL SETARA(BASE,MAXMIX,IBUP)
      CALL SETARA(BASE,MAXMIX,ITEMP)
      CALL XDISET(IBASE(INBM),MAXMIX,1)
      CALL XDRSET(BASE(IBUP),MAXMIX,0.0)
      CALL XDRSET(BASE(ITEMP),MAXMIX,0.0)
*----
*  READ THE INPUT DATA
*----
      NL=1
      NBMIX=0
      ICTR=0
      IGMAIL=0
   20 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('M2T: CHARACTER DATA EXPECTED(1).')
      IF(TEXT12.EQ.'EDIT') THEN
*        READ THE PRINT INDEX.
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('M2T: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT12.EQ.'MIX') THEN
*        READ A MATERIAL MIXTURE.
         TEXT20=' '
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT20,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('M2T: CHARACTER DATA EXPECTED(2).')
         NBMIX=NBMIX+1
         IF(NBMIX.GT.MAXMIX) CALL XABORT('M2T: MAXMIX OVERFLOW.')
         READ(TEXT20,'(5A4)') (IBASE(IHBM+5*(NBMIX-1)+I0),I0=0,4)
   30    CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('M2T: CHARACTER DATA EXPECTED(3).')
         IF(TEXT12.EQ.'FROM') THEN
            CALL REDGET(INDIC,IBASE(INBM+NBMIX-1),FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.1) CALL XABORT('M2T: INTEGER DATA EXPECTED(2).')
            GO TO 30
         ELSE IF(TEXT12.EQ.'BURN') THEN
*           READ A BURNUP.
            CALL REDGET(INDIC,NITMA,BASE(IBUP+NBMIX-1),TEXT12,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('M2T: REAL DATA EXPECTED(1).')
            GO TO 30
         ELSE IF(TEXT12.EQ.'TEMP') THEN
*           READ A TEMPERATURE.
            CALL REDGET(INDIC,NITMA,BASE(ITEMP+NBMIX-1),TEXT12,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('M2T: REAL DATA EXPECTED(2).')
            GO TO 30
         ELSE IF(TEXT12.NE.'ENDMIX') THEN
            CALL XABORT('M2T: FROM, BURN, TEMP OR ENDMIX EXPECTED.')
         ENDIF
      ELSE IF(TEXT12.EQ.'PN') THEN
*        READ THE ANISOTROPY ORDER
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT20,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('M2T: INTEGER DATA EXPECTED(3).')
         NL=MIN(NITMA+1,MAXNL)
      ELSE IF(TEXT12.EQ.'TRAN') THEN
*        PERFORM TRANSPORT CORRECTION
         ICTR=1
      ELSE IF(TEXT12.EQ.'NOMA') THEN
*        DO NOT WRITE ENERGY MESH ON APOTRIM FILE
         IGMAIL=1
      ELSE IF(TEXT12.EQ.';') THEN
         GO TO 40
      ELSE
         CALL XABORT('M2T: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20
*----
*  RECOVER INFORMATION
*----
   40 CALL SETARA(IBASE,NGRP+1,IGAR1)
      CALL SETARA(IBASE,5*NGRP,IGAR2)
      CALL SETARA(IBASE,NGRP*NGRP,IXTRAN)
      CALL SETARA(IBASE,MAXMIX,IIND)
      CALL SETARA(IBASE,MAXMIX,ISIG)
      CALL SETARA(IBASE,NGRP,IIFDG)
      CALL SETARA(IBASE,NGRP+1,IIADR)
      CALL SETARA(IBASE,MAXMIX,IIJJ)
      CALL SETARA(IBASE,MAXMIX,INJJ)
      CALL SETARA(IBASE,MAXMIX,IIPOS)
      CALL SETARA(IBASE,NGRP*MAXMIX,IWORK)
      CALL SETARA(IBASE,NGRP,ITRAN)
      CALL M2TDRV(IMPX,LOUT,IPMAC,NGRP,NBMIX,MAXMIX,NL,NBFIS,ICTR,
     1 IGMAIL,BASE(IBUP),BASE(ITEMP),IBASE(IHBM),IBASE(INBM),
     2 IBASE(IGAR1),IBASE(IGAR2),IBASE(IXTRAN),IBASE(IIND),IBASE(ISIG),
     3 IBASE(IIFDG),IBASE(IIADR),IBASE(IIJJ),IBASE(INJJ),IBASE(IIPOS),
     4 IBASE(IWORK),IBASE(ITRAN))
      CALL RLSARA(IBASE(ITRAN))
      CALL RLSARA(IBASE(IWORK))
      CALL RLSARA(IBASE(IIPOS))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(IBASE(IIJJ))
      CALL RLSARA(IBASE(IIADR))
      CALL RLSARA(IBASE(IIFDG))
      CALL RLSARA(IBASE(ISIG))
      CALL RLSARA(IBASE(IIND))
      CALL RLSARA(IBASE(IXTRAN))
      CALL RLSARA(IBASE(IGAR2))
      CALL RLSARA(IBASE(IGAR1))
*----
*  RELEASE MEMORY
*----
      CALL RLSARA(BASE(ITEMP))
      CALL RLSARA(BASE(IBUP))
      CALL RLSARA(IBASE(IHBM))
      CALL RLSARA(IBASE(INBM))
      RETURN
      END
