*DECK MACDRV
      SUBROUTINE MACDRV(IPLIST,INDREC,IPRINT,NBMIX,NGROUP,NANISO,
     >                  NIFISS,NEDMAC,ITRANC,NDELG,NSTEP)
*
*-----------------------------------------------------------------------
*
*Purpose:
* input macroscopic cross sections in Dragon/Donjon.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): G. Marleau
*
*Parameters: input
* IPLIST  LCM pointer to the macrolib.
* INDREC  =1: the macrolib is created;
*         =2: an existing macrolib is modified.
* IPRINT  print level.
*
*Parameters: output
* NBMIX   number of mixtures.
* NGROUP  maximum number of groups (default = 1).
* NANISO  maximum scattering anisotropy (default = 1 corresponding to
*         isotropic collision in laboratory).
* NIFISS  number of fissile isotopes per mixtures (default = 0).
* NEDMAC  number of aditional edition cross section types.
* ITRANC  type of transport correction: =0: no transport correction
*         =1: Apollo type transport correction; =2: recover from
*         library; =4: leakage correction alone.
* NDELG   number of precursor groups for delayed neutrons.
* NSTEP   number of delta cross-section sets used for generalized
*         perturbation theory (GPT) or kinetics calculations.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER     IPLIST,INDREC,IPRINT,NBMIX,NGROUP,NANISO,NIFISS,
     >            NEDMAC,ITRANC,NDELG,NSTEP
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NCXST=16)
      CHARACTER   CARLIR*8,CARLU*4,CGOXSN*7
      LOGICAL     LIREAD,LOLDXS(NCXST),LNEWXS(NCXST),LNORM,LADD,LUPD
      INTEGER     ITYPLU,INTLIR,IMATER,IBASE(1)
      REAL        REALIR
      DOUBLE PRECISION DBLINP
      DOUBLE PRECISION SQFMAS,XDRCST,NMASS,EVJ
      COMMON      RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*----
*  FOR AVERAGED NEUTRON VELOCITY
*  V=SQRT(2*ENER/M)=SQRT(2/M)*SQRT(ENER)
*  SQFMAS=SQRT(2/M) IN CM/S/SQRT(EV) FOR V IN CM/S AND E IN EV
*        =SQRT(2*1.602189E-19(J/EV)* 1.0E4(CM2/M2) /1.67495E-27 (KG))
*        =1383155.30602 CM/S/SQRT(EV)
*----
      EVJ=XDRCST('eV','J')
      NMASS=XDRCST('Neutron mass','kg')
      SQFMAS=SQRT(2.0D4*EVJ/NMASS)
*----
*  INITIALIZE USEFUL PARAMETERS
*----
      LIREAD=.TRUE.
      LNORM=.FALSE.
      LADD=.FALSE.
      LUPD=(INDREC.EQ.2)
      NEXTRE=1
      NEXTMI=1
      MAXFIS=MAX(1,NIFISS)
      CALL SETARA(IBASE,NANISO,ISCATA)
      CALL XDISET(IBASE(ISCATA),NANISO,0)
      DO 120 IL=1,NCXST
        LOLDXS(IL)=.FALSE.
        LNEWXS(IL)=.FALSE.
 120  CONTINUE
      LPLIST=IPLIST
      ISTEP=0
*----
*  READ A MAIN OPTION
*----
 1000 CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLINP)
      IF(ITYPLU.NE.3)
     >  CALL XABORT('MACDRV: READ ERROR 1 CHARACTER VARIABLE EXPECTED')
*----
*  CHECK FOR STOP/RETURN
*----
 1002 IF(CARLIR.EQ.';') THEN
        GO TO 2000
      ELSE IF(CARLIR.EQ.'EDIT') THEN
*----
*  READ THE PRINT INDEX
*----
        CALL REDGET(ITYPLU,IPRINT,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
      ELSE IF(CARLIR.EQ.'DELP') THEN
*----
*  READ THE NUMBER OF PRECURSOR GROUPS FOR DELAYED NEUTRONS
*----
        CALL REDGET(ITYPLU,NDELG,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
      ELSE IF(CARLIR.EQ.'STEP') THEN
*----
*  STEP TO A GPT SUB-DIRECTORY
*----
        CARLIR=' '
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLINP)
        IF((ITYPLU.NE.3).OR.(CARLIR.NE.'INPUT'))
     >  CALL XABORT('MACDRV: READ ERROR - INPUT STRING EXPECTED')
        CALL REDGET(ITYPLU,ISTEP,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
        IF(INDREC.EQ.1) THEN
          IF(ISTEP.LT.NSTEP)
     >    CALL XABORT('MACDRV: THIS DIRECTORY STEP ALREADY EXISTS.')
        ENDIF
        NSTEP=MAX(NSTEP,ISTEP)
        JPLIST=LCMLID(IPLIST,'STEP',NSTEP)
        KPLIST=LCMDIL(JPLIST,ISTEP)
        LPLIST=KPLIST
      ELSE IF(CARLIR.EQ.'READ') THEN
        IF(NBMIX.EQ.0) CALL XABORT('MACDRV: NBMIX NOT YET DEFINED')
        CALL SETARA(IBASE,NBMIX,IPERMU)
        NBELEM=0
*----
*  IDENTIFY MIXTURES TO READ
*----
        NUMMAT=0
        DO 100 IMATER=1,NBMIX
          CALL REDGET(ITYPLU,JPERMU,REALIR,CGOXSN,DBLINP)
          IF(ITYPLU.NE.1) GO TO 1001
          IF(JPERMU.GT.NBMIX) CALL XABORT('MACDRV: MATERIAL NUMBER IS'
     >      //' LARGER THAN NBMIX')
          IBASE(IPERMU+IMATER-1)=JPERMU
          NUMMAT=MAX(NUMMAT,JPERMU)
          IF(IMATER.EQ.NBMIX)
     >      CALL REDGET(ITYPLU,INTLIR,REALIR,CGOXSN,DBLINP)
          NBELEM=NBELEM+1
 100    CONTINUE
 1001   IF(ITYPLU.NE.3)
     >  CALL XABORT('MACDRV: READ ERROR 2 CHARACTER VARIABLE EXPECTED')
        IF(LIREAD) THEN
          CALL SETARA(RBASE,NBMIX*(NGROUP+2),JWORK)
          CALL SETARA(RBASE,NBMIX*NGROUP,JTOTL)
          CALL SETARA(RBASE,NBMIX*NGROUP,JTOT1)
          CALL SETARA(RBASE,NBMIX*NGROUP*MAXFIS,JFISS)
          CALL SETARA(RBASE,NBMIX*NGROUP*MAXFIS,JSPEC)
          CALL SETARA(RBASE,NBMIX*NGROUP,JFIXE)
          CALL SETARA(RBASE,NBMIX*NGROUP,JTRANC)
          CALL SETARA(RBASE,NBMIX*NGROUP,JDIFF)
          CALL SETARA(RBASE,NBMIX*NGROUP,JNFTOT)
          CALL SETARA(RBASE,NBMIX*NGROUP,JH)
          CALL SETARA(RBASE,NBMIX*NGROUP*NGROUP*NANISO,JSCAT)
          CALL SETARA(IBASE,NBMIX,INBSCT)
          CALL SETARA(IBASE,NBMIX,INGSCT)
          IF(NDELG.GT.0) THEN
             CALL SETARA(RBASE,NBMIX*NGROUP*MAXFIS*NDELG,JNUDL)
             CALL SETARA(RBASE,NBMIX*NGROUP*MAXFIS*NDELG,JCHDL)
          ENDIF
          CALL SETARA(RBASE,NBMIX*NGROUP,JOVERV)
          CALL SETARA(RBASE,NBMIX*NGROUP,JDIFFX)
          CALL SETARA(RBASE,NBMIX*NGROUP,JDIFFY)
          CALL SETARA(RBASE,NBMIX*NGROUP,JDIFFZ)
          CALL MACIXS(LPLIST,MAXFIS,NGROUP,NBMIX,NIFISS,NANISO,NDELG,
     >                RBASE(JTOTL),RBASE(JTOT1),RBASE(JFISS),
     >                RBASE(JSPEC),RBASE(JFIXE),RBASE(JTRANC),
     >                RBASE(JDIFF),RBASE(JNFTOT),RBASE(JH),RBASE(JSCAT),
     >                RBASE(JWORK),IBASE(INBSCT),IBASE(INGSCT),LOLDXS,
     >                IBASE(ISCATA),RBASE(JNUDL),RBASE(JCHDL),
     >                RBASE(JDIFFX),RBASE(JDIFFY),RBASE(JDIFFZ),
     >                RBASE(JOVERV))
          CALL RLSARA(IBASE(INGSCT))
          CALL RLSARA(IBASE(INBSCT))
          LIREAD=.FALSE.
        ENDIF
        IF(CGOXSN.EQ.'INPUT  ') THEN
          CALL SETARA(IBASE,NCXST*NBMIX,LINIXS)
          CALL MACXSR(MAXFIS,NGROUP,NBMIX,NIFISS,NANISO,NDELG,
     >                RBASE(JTOTL),RBASE(JTOT1),RBASE(JFISS),
     >                RBASE(JSPEC),RBASE(JFIXE),RBASE(JTRANC),
     >                RBASE(JDIFF),RBASE(JNFTOT),RBASE(JH),RBASE(JSCAT),
     >                LOLDXS,LNEWXS,IBASE(LINIXS),CARLIR,LADD,LUPD,
     >                IPRINT,IBASE(ISCATA),RBASE(JNUDL),RBASE(JCHDL),
     >                RBASE(JDIFFX),RBASE(JDIFFY),RBASE(JDIFFZ),
     >                RBASE(JOVERV))
          CALL RLSARA(IBASE(LINIXS))
        ENDIF
        CALL RLSARA(IBASE(IPERMU))
        GO TO 1002
      ELSE IF(CARLIR.EQ.'NGRO') THEN
        CALL REDGET(ITYPLU,NGROUP,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
      ELSE IF(CARLIR.EQ.'NMIX') THEN
        CALL REDGET(ITYPLU,NBMIX,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
      ELSE IF(CARLIR.EQ.'ANIS') THEN
        NANISC=NANISO
        CALL REDGET(ITYPLU,NANISO,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
        IF(NANISO.GT.NANISC) THEN
          CALL RLSARA(IBASE(ISCATA))
          CALL SETARA(IBASE,NANISO,ISCATA)
          CALL XDISET(IBASE(ISCATA),NANISO,0)
        ENDIF
      ELSE IF(CARLIR.EQ.'CTRA') THEN
*----
*  READ TRANSPORT CORRECTION TYPE
*----
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.3) CALL XABORT('MACDRV: READ ERROR - CHARACTER CT'
     >  //'RA TYPE EXPECTED')
        IF(CARLU.EQ.'NONE') THEN
          ITRANC=0
        ELSE IF(CARLU.EQ.'APOL') THEN
          ITRANC=1
        ELSE IF(CARLU.EQ.'WIMS') THEN
          ITRANC=2
        ELSE IF(CARLU.EQ.'LEAK') THEN
          ITRANC=4
        ELSE
          CALL XABORT('MACDRV: NONE, APOL, WIMS OR LEAK EXPECTED')
        ENDIF
      ELSE IF(CARLIR.EQ.'NIFI') THEN
        CALL REDGET(ITYPLU,NIFISS,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1)
     >  CALL XABORT('MACDRV: READ ERROR - INTEGER VARIABLE EXPECTED')
        MAXFIS=MAX(1,NIFISS)
      ELSE IF(CARLIR.EQ.'NORM') THEN
        LNORM=.TRUE.
      ELSE IF(CARLIR.EQ.'ADD') THEN
        IF(.NOT.LUPD) CALL XABORT('MACDRV: CANNOT USE THE ADD OPTION ON'
     >  //' A MACROLIB IN CREATION MODE')
        LADD=.TRUE.
      ELSE IF(CARLIR.EQ.'ENER') THEN
*----
*  READ ENERGY GROUPS
*----
        IF(NGROUP.GT.0) THEN
          CALL SETARA(RBASE,NGROUP+1,IENE)
          CALL SETARA(RBASE,NGROUP,IVEL)
          CALL REDGET(ITYPLU,INTLU,ENEMAX,CARLU,DBLINP)
          IF(ITYPLU.NE.2) CALL XABORT('MACDRV: READ ERROR - REAL VAR'
     >    //'IABLE EXPECTED')
          DO 179 IGR=IENE,IENE+NGROUP-1
            RBASE(IGR)=ENEMAX
            CALL REDGET(ITYPLU,INTLU,ENECUR,CARLU,DBLINP)
            IF(ITYPLU.NE.2) CALL XABORT('MACDRV: READ ERROR - REAL V'
     >      //'ARIABLE EXPECTED')
            IF(ENECUR.GT.ENEMAX) CALL XABORT('MACDRV: READ ERROR - E'
     >      //'NERGY GOES FROM MAX TO MIN')
            ENEMAX=ENECUR
 179      CONTINUE
          IF(ENEMAX.LE.0.0) THEN
            RBASE(IENE+NGROUP)=1.0E-5
          ELSE
            RBASE(IENE+NGROUP)=ENEMAX
          ENDIF
          CALL LCMPUT(IPLIST,'ENERGY',NGROUP+1,2,RBASE(IENE))
          VELG1=SQRT(RBASE(IENE))
          JVEL=IVEL
          DO 178 IGR=IENE,IENE+NGROUP-1
            VELG2=SQRT(RBASE(IGR+1))
            RBASE(JVEL)=SQFMAS*SQRT(VELG1*VELG2)
            VELG1=VELG2
            RBASE(IGR)=LOG(RBASE(IGR)/RBASE(IGR+1))
            JVEL=JVEL+1
 178      CONTINUE
          CALL LCMPUT(IPLIST,'AVGVEL',NGROUP,2,RBASE(IVEL))
          CALL LCMPUT(IPLIST,'DELTAU',NGROUP,2,RBASE(IENE))
          CALL RLSARA(RBASE(IVEL))
          CALL RLSARA(RBASE(IENE))
        ENDIF
      ELSE IF(CARLIR.EQ.'ALBP') THEN
*----
*  READ GROUP INDEPENDENT PHYSICAL ALBEDOS
*----
        CALL REDGET(ITYPLU,NALBD,REALIR,CARLU,DBLINP)
        IF(ITYPLU.NE.1) CALL XABORT('MACDRV: INTEGER DATA EXPECTED.')
        IF(NALBD.GT.0) THEN
           CALL SETARA(RBASE,NALBD,IALB)
           DO 180 IAL=1,NALBD
           CALL REDGET(ITYPLU,INTLIR,RBASE(IALB+IAL-1),CARLU,DBLINP)
           IF(ITYPLU.NE.2) CALL XABORT('MACDRV: REAL DATA EXPECTED.')
 180       CONTINUE
           CALL LCMPUT(IPLIST,'ALBEDO',NALBD,2,RBASE(IALB))
           CALL RLSARA(RBASE(IALB))
        ELSE
           CALL XABORT('MACDRV: INVALID NUMBER OF ALBEDOS.')
        ENDIF
      ELSE
        CALL XABORT('MACDRV: '//CARLIR//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 1000
*----
*  TRANSFER MODIFIED X-S ON THE MACROLIB
*----
 2000 IF(.NOT.LIREAD) THEN
        CALL SETARA(IBASE,NBMIX,INGSCT)
        CALL SETARA(IBASE,NBMIX,IFGSCT)
        CALL SETARA(IBASE,NBMIX,IPOSCT)
        CALL MACPXS(LPLIST,MAXFIS,NGROUP,NBMIX,NIFISS,NANISO,
     >              NDELG,ITRANC,LNEWXS,RBASE(JTOTL),RBASE(JTOT1),
     >              RBASE(JFISS),RBASE(JSPEC),RBASE(JFIXE),
     >              RBASE(JTRANC),RBASE(JDIFF),RBASE(JNFTOT),RBASE(JH),
     >              RBASE(JSCAT),RBASE(JWORK),IBASE(INGSCT),
     >              IBASE(IFGSCT),IBASE(IPOSCT),NEDMAC,IBASE(ISCATA),
     >              RBASE(JNUDL),RBASE(JCHDL),RBASE(JDIFFX),
     >              RBASE(JDIFFY),RBASE(JDIFFZ),RBASE(JOVERV))
        CALL RLSARA(IBASE(IPOSCT))
        CALL RLSARA(IBASE(IFGSCT))
        CALL RLSARA(IBASE(INGSCT))
        CALL RLSARA(RBASE(JDIFFZ))
        CALL RLSARA(RBASE(JDIFFY))
        CALL RLSARA(RBASE(JDIFFX))
        CALL RLSARA(RBASE(JOVERV))
        IF(NDELG.GT.0) THEN
           CALL RLSARA(RBASE(JCHDL))
           CALL RLSARA(RBASE(JNUDL))
        ENDIF
        CALL RLSARA(RBASE(JSCAT))
        CALL RLSARA(RBASE(JH))
        CALL RLSARA(RBASE(JNFTOT))
        CALL RLSARA(RBASE(JDIFF))
        CALL RLSARA(RBASE(JTRANC))
        CALL RLSARA(RBASE(JFIXE))
        CALL RLSARA(RBASE(JSPEC))
        CALL RLSARA(RBASE(JFISS))
        CALL RLSARA(RBASE(JTOT1))
        CALL RLSARA(RBASE(JTOTL))
        CALL RLSARA(RBASE(JWORK))
      ENDIF
*----
*  NORMALIZATION OF X-S INFORMATION
*----
      IF(LNORM) THEN
        CALL SETARA(IBASE,NBMIX,INGSCT)
        CALL SETARA(IBASE,NBMIX,IFGSCT)
        CALL SETARA(RBASE,NBMIX*NGROUP,JWORK)
        CALL SETARA(RBASE,NBMIX*NIFISS,JWOR2)
        CALL SETARA(RBASE,2*NBMIX*NANISO*NGROUP,ISCWOR)
        CALL SETARA(RBASE,NBMIX*MAXFIS,ICHWOR)
        CALL MACNXS(IPLIST,MAXFIS,NGROUP,NBMIX,NIFISS,NANISO,
     >              IBASE(INGSCT),IBASE(IFGSCT),RBASE(JWORK),
     >              RBASE(JWOR2),RBASE(ISCWOR),RBASE(ICHWOR))
        CALL RLSARA(RBASE(ICHWOR))
        CALL RLSARA(RBASE(ISCWOR))
        CALL RLSARA(RBASE(JWOR2))
        CALL RLSARA(RBASE(JWORK))
        CALL RLSARA(IBASE(IFGSCT))
        CALL RLSARA(IBASE(INGSCT))
      ENDIF
*----
*  PRINT/CHECK X-S INFORMATION
*----
      IF((IPRINT.NE.0).AND.(IPRINT.NE.1)) THEN
        CALL SETARA(IBASE,NBMIX,INGSCT)
        CALL SETARA(IBASE,NBMIX,IFGSCT)
        CALL SETARA(IBASE,NBMIX,IPOSCT)
        CALL SETARA(RBASE,NBMIX*(NGROUP+2),JWORK)
        CALL SETARA(RBASE,NBMIX*NIFISS,JWOR2)
        CALL SETARA(RBASE,NBMIX*NANISO*NGROUP,ISCWOR)
        CALL SETARA(RBASE,NBMIX,JCTRAN)
        CALL SETARA(IBASE,2*(NEDMAC+1),IADNAM)
        CALL MACWXS(IPLIST,IPRINT,NGROUP,NBMIX,NIFISS,NANISO,
     >              IBASE(INGSCT),IBASE(IFGSCT),IBASE(IPOSCT),
     >              RBASE(JWORK),RBASE(JWOR2),RBASE(ISCWOR),ITRANC,
     >              NEDMAC,RBASE(JCTRAN),IBASE(IADNAM))
        CALL RLSARA(IBASE(IADNAM))
        CALL RLSARA(RBASE(JCTRAN))
        CALL RLSARA(RBASE(ISCWOR))
        CALL RLSARA(RBASE(JWOR2))
        CALL RLSARA(RBASE(JWORK))
        CALL RLSARA(IBASE(IPOSCT))
        CALL RLSARA(IBASE(IFGSCT))
        CALL RLSARA(IBASE(INGSCT))
      ENDIF
      RETURN
      END
