*DECK MCCGA
      SUBROUTINE MCCGA(IPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NBREG,NBMIX,
     1                 NANI,MAT,VOL,KNORM,ILK,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of PJJ for flux integration when isotropic scat. is considered
* calculation of preconditioning matrices for Algebraic Collapsing
* Acceleration or Self-Collision Probability acceleration of inner
* iterations. Vectorial version.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and R. Le Tellier
*
*Parameters: input/output
* IPSYS   pointer to the PIJ LCM object (L_PIJ signature). IPSYS is a
*         list of NGRP directories.
* NPSYS   index array pointing to the IPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  tracking file unit number.
* IMPX    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NBREG   total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NANI    number of Legendre orders.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   not used.
* SIGT    total macroscopic cross sections ordered by mixture.
* ILK     leakage flag (ILK=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER*72 TITR
      LOGICAL ILK
      INTEGER IPSYS,IPTRK,IFTRAK,IMPX,NGRP,NBREG,NBMIX,NANI,
     1 MAT(NBREG),KNORM,NPSYS(NGRP)
      REAL VOL(NBREG)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER JPAR(NSTATE),IBASE(1),TRTY,PACA,STIS,IGB(8)
      CHARACTER*4 TEXT4
      REAL ALBEDO(6),ZREAL(4),DELU,FACSYM
      LOGICAL LEXA,LEXF,CYCLIC,LTMT,LACA,LPJJ,LPJJAN,LVOID,LPRISM,
     1 LBIHET
      EXTERNAL MCGDSCA,MCGDSCE,MCGDDDF,MCGDS2,MOCDS2,MCGDSP,MOCDSP,
     1 MCGDS2A,MCGDS2E
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER MCCG3D SPECIFIC PARAMETERS
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      IF(JPAR(4).GT.NBMIX) CALL XABORT('MCCGA: INVALID NBMIX.')
      IF(IFTRAK.LE.0) CALL XABORT('MCCGA: INVALID TRACKING FILE.')
*     recover state-vector information
      LBIHET=JPAR(40).NE.0
      IF(LBIHET) THEN
         CALL LCMSIX(IPTRK,'BIHET',1)
         CALL LCMGET(IPTRK,'PARAM',IGB)
         NREG=IGB(3)
         CALL LCMSIX(IPTRK,' ',2)
      ELSE
         NREG=JPAR(1)
      ENDIF
      NFI=NREG+JPAR(5)
      IF(JPAR(6).NE.NANI) CALL XABORT('MCCGA: INVALID NANI.')
      TRTY=JPAR(9)
      IF(TRTY.EQ.1) THEN
         CYCLIC=.TRUE.
         NLONG=NREG
      ELSE
         CYCLIC=.FALSE.
         NLONG=NFI
      ENDIF
      NMU=JPAR(22)
      NMAX=JPAR(25)
      IAAC=JPAR(27)
      STIS=JPAR(35)
      ISCR=JPAR(28)
      LC=JPAR(26)
      LPS=JPAR(29)
      PACA=JPAR(30)
      LC0=JPAR(37)
      LTMT=(JPAR(34).EQ.1)
      LEXA=(JPAR(31).EQ.1)
      LEXF=(JPAR(32).EQ.1)
      NPJJM=JPAR(36)
      NZP=JPAR(39)
      LPRISM=(NZP.NE.0)
*     recover real parameters
      CALL LCMGET(IPTRK,'REAL-PARAM',ZREAL)
      HDD=ZREAL(2)
      DELU=ZREAL(3)
      FACSYM=ZREAL(4)
*     recover tracking file information
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR
      DO ICOM=1,NCOMNT
         READ(IFTRAK)
      ENDDO
      READ(IFTRAK) NDIM,ISPEC,N2REG,N2SOU,NALBG,NCOR,NANGL,MXSEG
      IF (NCOR.NE.1) 
     1 CALL XABORT('MCCGA: INVALID TRACKING FILE: NCOR.NE.1')
      READ(IFTRAK)
      READ(IFTRAK)
      READ(IFTRAK)
      READ(IFTRAK)
      CALL SETARA(BASE,NANGL,ICAZ)
      IF (NDIM.EQ.2) THEN
         CALL SETARA(BASE,NMU,ICPO)
         CALL LCMGET(IPTRK,'XMU$MCCG',BASE(ICPO))
         READ(IFTRAK) (BASE(ICAZ-1+JJ),DUMMY,JJ=1,NANGL)
      ELSE ! NDIM.EQ.3
         CALL SETARA(BASE,NANGL,ICPO)
         READ(IFTRAK) (BASE(ICPO-1+JJ),BASE(ICAZ-1+JJ),DUMMY,
     1                 JJ=1,NANGL)
         DO JJ=1,NANGL
            BASE(ICAZ-1+JJ)=BASE(ICAZ-1+JJ)/
     1                      SQRT(1.0-BASE(ICPO-1+JJ)*BASE(ICPO-1+JJ))
         ENDDO
      ENDIF
*     recover polar quadrature
      CALL LCMIOF(IPTRK,'ZMU$MCCG',IZMU)
      CALL LCMIOF(IPTRK,'WZMU$MCCG',IWZMU)
*     recover MATALB and VOLSUR
      CALL LCMIOF(IPTRK,'NZON$MCCG',NZON)
      CALL LCMIOF(IPTRK,'V$MCCG',IV)
      IF (NANI.GT.1) THEN
*        recover spherical harmonics paremeters
         CALL LCMIOF(IPTRK,'ISGNR$MCCG',ISGNR)
         CALL LCMIOF(IPTRK,'KEYANI$MCCG',IKEYAN)
      ENDIF
      IF ((.NOT.LPRISM).AND.(NDIM.EQ.2)) THEN
         NFUNL=NANI*(NANI+1)/2
         NMOD=4
      ELSE ! NDIM.EQ.3
         NFUNL=NANI*NANI
         NMOD=8
      ENDIF
*---
* DETERMINE THE NUMBER OF GROUPS TO BE PROCESSED
* RECOVER POINTERS TO EACH GROUP PROPERTIES
* CREATE AN INDEX FOR THE GROUPS TO BE PROCESSED
*---
      NGEFF=0
      DO IG=1,NGRP
         IOFSET=NPSYS(IG)
         IF(IOFSET.NE.0) NGEFF=NGEFF+1
      ENDDO
      CALL SETARA(IBASE,NGEFF,NGIND)
      CALL SETARA(IBASE,NGEFF,KPSYS)
      II=1
      DO IG=1,NGRP
         IOFSET=NPSYS(IG)
         IF(IOFSET.NE.0) THEN
            IBASE(NGIND-1+II)=IG
            IF(LBIHET) THEN
               JPSYS=LCMGIL(IPSYS,IOFSET)
               IBASE(KPSYS-1+II)=LCMGID(JPSYS,'BIHET')
            ELSE
               IBASE(KPSYS-1+II)=LCMGIL(IPSYS,IOFSET)
            ENDIF
            II=II+1
         ENDIF
      ENDDO
*----
*  CONSTRUCT TOTAL CROSS SECTIONS ARRAY AND CHECK FOR ZERO CROSS SECTION
*----
      CALL SETARA(BASE,(NBMIX+7)*NGEFF,ISIGAL)
      CALL XDRSET(ALBEDO,6,0.0)
      CALL MCGSIG(NBMIX,NGEFF,IBASE(KPSYS),ALBEDO,BASE(ISIGAL),LVOID)
      IF ((LVOID).AND.(STIS.EQ.-1)) THEN
         IF (IMPX.GT.0) 
     1       WRITE(6,*) 'VOID EXISTS -> STIS SET TO 1 INSTEAD OF -1'
         STIS=1
      ENDIF
*---
* IS THERE SOMETHING TO DO ?
*---
      LACA=(IAAC.GT.0)
      LPJJ=((STIS.EQ.1).OR.(ISCR.GT.0))
      IF (.NOT.(LACA.OR.LPJJ)) GOTO 10
      LPJJAN=(LPJJ.AND.(NANI.GT.1))
      IF (LACA) THEN
*        recover connection matrices
         CALL LCMIOF(IPTRK,'KM$MCCG',IKM)
         CALL LCMIOF(IPTRK,'IM$MCCG',IM)
         CALL LCMIOF(IPTRK,'MCU$MCCG',IMCU)
*        recover modified MATALB and VOLSUR
         CALL LCMIOF(IPTRK,'NZONA$MCCG',NZONA)
         CALL LCMIOF(IPTRK,'VA$MCCG',IVA)
*        recover permutation array
         CALL LCMIOF(IPTRK,'INVPI$MCCG',IPERM)
         IF (PACA.GE.2) THEN
            CALL LCMIOF(IPTRK,'JU$MCCG',JU)
            IF (PACA.EQ.3) THEN
               CALL LCMIOF(IPTRK,'IM0$MCCG',IM0)
               CALL LCMIOF(IPTRK,'MCU0$MCCG',IMCU0)
            ENDIF
         ENDIF
      ENDIF
      IF ((.NOT.CYCLIC).AND.(ISCR.GT.0)) THEN
*        recover (IS,JS) array for surfaces neighbors identification
         CALL LCMIOF(IPTRK,'IS$MCCG',IS)
         CALL LCMIOF(IPTRK,'JS$MCCG',JS)
      ENDIF
      IF (LPJJAN) CALL LCMIOF(IPTRK,'PJJIND$MCCG',IPJJM)
!!! temporary
      IF (HDD.GT.0.0) THEN
         ISCH=0
      ELSEIF (LEXF) THEN
         ISCH=-1
      ELSE
         ISCH=1
      ENDIF
*----
*  PRECONDITIONING MATRICES CALCULATION
*----
      CALL SETARA(BASE,2*NLONG*NGEFF,IDIAGF)
      CALL SETARA(BASE,2*LC*NGEFF,ICF)
      IF (ISCH.EQ.1) THEN
*     PJJ/SCR: Step-Characteristics Scheme with Tabulated Exponentials
         IF (CYCLIC) THEN
*        ACA: cyclic tracking
            IF (LEXA) THEN
*           ACA: Exact Exponentials
               CALL MCGASM(MCGDSCA,MOCDS2,MOCDSP,MCGDS2E,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ELSE
*           ACA: Tabulated Exponentials
               CALL MCGASM(MCGDSCA,MOCDS2,MOCDSP,MCGDS2A,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ENDIF
         ELSE
*        ACA: non-cyclic tracking
            IF (LEXA) THEN
*           ACA: Exact Exponentials
               CALL MCGASM(MCGDSCA,MCGDS2,MCGDSP,MCGDS2E,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ELSE
*           ACA: Tabulated Exponentials
               CALL MCGASM(MCGDSCA,MCGDS2,MCGDSP,MCGDS2A,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ENDIF
         ENDIF        
      ELSEIF (ISCH.EQ.0) THEN
*     PJJ/SCR: Diamond-Differencing Scheme
         IF (CYCLIC) THEN
*        ACA: cyclic tracking
            IF (LEXA) THEN
*           ACA: Exact Exponentials
               CALL MCGASM(MCGDDDF,MOCDS2,MOCDSP,MCGDS2E,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ELSE
*           ACA: Tabulated Exponentials
               CALL MCGASM(MCGDDDF,MOCDS2,MOCDSP,MCGDS2A,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ENDIF
         ELSE
*        ACA: non-cyclic tracking
            IF (LEXA) THEN
*           ACA: Exact Exponentials
               CALL MCGASM(MCGDDDF,MCGDS2,MCGDSP,MCGDS2E,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ELSE
*           ACA: Tabulated Exponentials
               CALL MCGASM(MCGDDDF,MCGDS2,MCGDSP,MCGDS2A,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ENDIF
         ENDIF
      ELSEIF (ISCH.EQ.-1) THEN
*     PJJ/SCR: Step-Characteristics Scheme with Exact Exponentials
         IF (CYCLIC) THEN
*        ACA: cyclic tracking
            IF (LEXA) THEN
*           ACA: Exact Exponentials
               CALL MCGASM(MCGDSCE,MOCDS2,MOCDSP,MCGDS2E,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ELSE
*           ACA: Tabulated Exponentials
               CALL MCGASM(MCGDSCE,MOCDS2,MOCDSP,MCGDS2A,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ENDIF
         ELSE
*        ACA: non-cyclic tracking
            IF (LEXA) THEN
*           ACA: Exact Exponentials
               CALL MCGASM(MCGDSCE,MCGDS2,MCGDSP,MCGDS2E,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ELSE
*           ACA: Tabulated Exponentials
               CALL MCGASM(MCGDSCE,MCGDS2,MCGDSP,MCGDS2A,IPTRK,
     1              IBASE(KPSYS),IMPX,IFTRAK,NANI,NGRP,NGEFF,NFI,NREG,
     2              NLONG,NBMIX,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,
     3              IBASE(NGIND),IBASE(NZON),IBASE(NZONA),IBASE(ISGNR),
     4              IBASE(IKEYAN),BASE(IV),BASE(IVA),BASE(ICAZ),
     5              BASE(ICPO),BASE(IZMU),BASE(IWZMU),IBASE(IKM),
     6              IBASE(IMCU),IBASE(IM),IBASE(JU),IBASE(IPERM),LC0,
     7              IBASE(IMCU0),IBASE(IM0),IBASE(IS),IBASE(JS),PACA,
     8              LPS,LTMT,NPJJM,IBASE(IPJJM),LACA,LPJJ,LPJJAN,
     9              BASE(ISIGAL),LPRISM,N2REG,N2SOU,NZP,DELU,FACSYM,
     1              BASE(IDIAGF),BASE(ICF))
            ENDIF
         ENDIF
      ENDIF
      CALL RLSARA(BASE(ICF))
      CALL RLSARA(BASE(IDIAGF))
*
 10   CALL RLSARA(BASE(ISIGAL))
      CALL RLSARA(IBASE(KPSYS))
      CALL RLSARA(IBASE(NGIND))
      CALL RLSARA(BASE(ICPO))
      CALL RLSARA(BASE(ICAZ))
*
      RETURN
      END
