*DECK MCCGT
      SUBROUTINE MCCGT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* adapt EXCELL tracking to MCCG requirements.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and R. Le Tellier
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): modification type(L_TRACK);
*         HENTRY(2): sequential binary tracking file.
*         HENTRY(3): read-only type(L_GEOM).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER    NENTRY                  
      CHARACTER  HENTRY(NENTRY)*12
      INTEGER    IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      INTEGER NSTATE,MXGAUS,IOUT,IBCV
      PARAMETER (NSTATE=40,MXGAUS=64,IOUT=6,IBCV=-7)
      INTEGER ITRK,IFTR,IGEO,IPTRK,IPGEO,IFTRAK,J,I1,NCOMNT,NBTR,ICOM,
     1 NDIM,ISPEC,N2REG,N2SOU,NALBG,NCOR,NANGL,MXSEG,NREG,NSOU,NANIS,
     2 ISYMM,IMPX,LCACT,NMU,MAXI,IAAC,ISCR,KRYL,IDIFC,ILEXA,ILEXF,INDIC,
     3 NITMA,NZP,N2RS,DIMKEYF,TYPOR1,TYPOR2,LTMT,STIS,LMXMCU,TRTY,PACA,
     4 SSYM,H,IMU,NFI,LMCU,N3MAX,ILINE,IANGL,N2SEG,NR2SE,NSEG,LMCU0,
     5 NLONG,LPS,NFIRST,NLEV,IK,JK,K,ILAST,IH,KJ,IPOS,IJEND,IJ,NFUNL,
     6 NUN,IA,IR,IKEY,ICUR,NRNL,NPJJM,IFORW,II,I
      REAL EPSI,HDD,TMUIM,FACSYM,DELU,FLOTT,WEI2D,DUM
      DOUBLE PRECISION DFLOTT,CMU
      CHARACTER TEXT4*4,TEXT12*12,TITLE*72,HSIGN*12,CFTRAK*12,
     1 COMNT(10)*80
      LOGICAL LPRISM,ACFLAG,LACA,LSCR,CYCLIC,LBIHET
      INTEGER IGP(NSTATE),ISTATE(NSTATE),KCHAR(3),KTITL(18),NCODE(6),
     1 IGB(8)
      REAL ZREAL(4),ZMU(MXGAUS),WZMU(MXGAUS),XMU(MXGAUS),ALBEDO(6),
     1 EXTKOP(NSTATE)
      DOUBLE PRECISION CMUV(MXGAUS),CMUIV(MXGAUS),SMUV(MXGAUS),
     1 SMUIV(MXGAUS),TMUV(MXGAUS),TMUIV(MXGAUS)
      INTEGER IV,NZON,NZONA,ITEMP,SEGLEN,NRSEG,MCUW,MCUI,KM,MCU,IZ,
     1 IM,LEV,LEVPT,KMROR,MCUROR,JU,ISGNR,KEYANI,KEYFLX,KEYCUR,IVA,PI,
     2 INVPI,IS,JS,IM0,MCU0,INDREG,IRTEMP,ISURFD,IT2D,INOM3D,IH3D,
     3 IVGLNUM,IMROR,IDENSTY,IVNUM,IWORK,IQUA10,IBIHET,IMAT,IVOL,IR2,
     4 NREG2
      INTEGER FACMCU(3)
      DATA FACMCU / 2,8,12 /
*
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      ITRK=0
      IFTR=0
      IGEO=0
      IF(NENTRY.LE.2) CALL XABORT('MCCGT: THREE PARAMETERS EXPECTED.')
*     tracking table in modification mode
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))
     1  CALL XABORT('MCCGT: LINKED LIST EXPECTED AT LHS.')
      ITRK=1
      IF(JENTRY(ITRK).NE.1) 
     1  CALL XABORT('MCCGT: ENTRY IN MODIFICATION MODE EXPECTED(1).')
      IF(IENTRY(2).EQ.3) THEN
*     tracking file in read-only mode
         IFTR=2
         IF (JENTRY(IFTR).NE.2) 
     1     CALL XABORT('MCCGT: ENTRY IN READ-ONLY MODE EXPECTED(1).')
*     geometry table in read-only mode
         IF((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2))
     1     CALL XABORT('MCCGT: LINKED LIST EXPECTED AT RHS.')
         IGEO=3
         IF(JENTRY(IGEO).NE.2) 
     1     CALL XABORT('MCCGT: ENTRY IN READ-ONLY MODE EXPECTED(2).')
      ELSE
         CALL XABORT('MCCGT: INVALID OR MISSING ENTRY')
      ENDIF
*
      IPTRK=KENTRY(ITRK)
      IPGEO=KENTRY(IGEO)
      IFTRAK=KENTRY(IFTR)
*      
      CALL LCMGET(IPTRK,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(J),J=1,3)
      IF(HSIGN.NE.'L_TRACK') THEN
            TEXT12=HENTRY(ITRK)
            CALL XABORT('MCCGT: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_TRACK EXPECTED.')
      ENDIF
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(J),J=1,3)
      IF(HSIGN.NE.'EXCELL') THEN
            TEXT12=HENTRY(ITRK)
            CALL XABORT('MCCGT: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. EXCELL EXPECTED.')
      ENDIF
*----
*  RECOVER GEOMETRY
*----
      CALL LCMGET(IPGEO,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(J),J=1,3)
      IF(HSIGN.NE.'L_GEOM') THEN
         TEXT12=HENTRY(IGEO)
         CALL XABORT('MCCGT: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1        '. L_GEOM EXPECTED.')
      ENDIF
      CALL LCMGET(IPGEO,'STATE-VECTOR',ISTATE)
      TEXT12=HENTRY(IGEO)
      READ(TEXT12,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(IPTRK,'LINK.GEOM',3,3,KCHAR)
*----
*  RECOVER SEQUENTIAL BINARY TRACKING FILE CHARACTERISTICS
*----
      CFTRAK=HENTRY(IFTR)
      READ(CFTRAK,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(IPTRK,'LINK.FTRACK',3,3,KCHAR)
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR
      DO ICOM=1,NCOMNT
         READ(IFTRAK) COMNT(ICOM)
      ENDDO
      READ(IFTRAK) NDIM,ISPEC,N2REG,N2SOU,NALBG,NCOR,NANGL,MXSEG
      IF((NDIM.NE.2).AND.(NDIM.NE.3)) 
     & CALL XABORT('2D OR 3D EXCELT TRACKING EXPECTED')
*----
*  RECOVER TRACKING STATE-VECTOR AND USER INPUT INFORMATION
*----
      CALL XDISET(IGP,NSTATE,0) 
      CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      CALL LCMGET(IPTRK,'ALBEDO',ALBEDO)
      NREG=IGP(1)
      NSOU=IGP(5)
      NANIS=IGP(6)
      TRTY=IGP(9)
      CYCLIC=(TRTY.EQ.1)
      ISYMM=IGP(12)
*
      IMPX=1
      LCACT=IGP(21)
      NMU=IGP(22)
      LBIHET=(IGP(40).NE.0)
      MAXI=20
      IAAC=1
      ISCR=0
      KRYL=10
      IDIFC=0
      EPSI=1.0E-4
      HDD=0.0
      PACA=3
      ILEXA=0
      LTMT=0
      ILEXF=0
      STIS=0
      LMXMCU=0
      IFORW=0
      DELU=0.0
      FACSYM=0.0
      IF(NANIS.LE.4) STIS=1
*----
*  PROCESS DOUBLE HETEROGENEITY (BIHET) DATA (IF AVAILABLE)
*----
      IF(LBIHET) THEN
         CALL LCMSIX(IPTRK,'BIHET',1)
         CALL LCMGET(IPTRK,'PARAM',IGB)
         IR2=IGB(2)
         NREG2=IGB(3)
         IBIHET=IGB(6)
         IQUA10=IGB(8)
         CALL LCMIOF(IPTRK,'IBI',IMAT)
         CALL LCMIOF(IPTRK,'VOLUME',IVOL)
         CALL LCMSIX(IPTRK,' ',2)
         CALL LCMPUT(IPTRK,'MATCOD',NREG,1,BASE(IMAT))
         CALL LCMPUT(IPTRK,'VOLUME',NREG,2,BASE(IVOL))
         IGP(1)=NREG2
         IGP(2)=IGP(2)-(NREG-NREG2)
         IGP(4)=IR2
         CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,IGP)
         NREG=NREG2
      ENDIF
*
   50 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
   55 IF(INDIC.EQ.10) GO TO 60
      IF(INDIC.NE.3) CALL XABORT('MCCGT: CHARACTER DATA EXPECTED(1).')
      IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'GAUS') THEN
         LCACT=0
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'CACA') THEN
         LCACT=1
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'CACB') THEN
         LCACT=2
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'LCMD') THEN
         LCACT=3
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'OPP1') THEN
         LCACT=4
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'OGAU') THEN
         LCACT=5
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'EPSI') THEN
*        CONVERGENCE CRITERION FOR INNER ITERATIONS.
         CALL REDGET(INDIC,NITMA,EPSI,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('MCCGT: REAL DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'SCR') THEN
*        SCR ACCELERATION FLAG.
         CALL REDGET(INDIC,ISCR,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(2).')
      ELSE IF(TEXT4.EQ.'KRYL') THEN
*        GMRES FLAG.
         CALL REDGET(INDIC,KRYL,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(3).')
      ELSE IF(TEXT4.EQ.'AAC') THEN
*        ACA ACCELERATION FLAG.
         CALL REDGET(INDIC,IAAC,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(4).')
      ELSE IF(TEXT4.EQ.'BICG') THEN
*        ACA SYSTEM RESOLUTION TYPE (obsolete because it is the only option!)
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: BICG ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'ILU0') THEN
*        ILU0 PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=3
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: ILU0 ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'DIAG') THEN
*        DIAGONAL PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=1
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: DIAG ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'FULL') THEN
*        FULL MATRIX PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=2
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: FULL ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'NONE') THEN
*        NO PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=0
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: NONE ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'TMT') THEN
*        TO USE A TRACK MERGING TECHNIQUE IN ACA CALCULATION
         LTMT=1
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: LTMT ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'LEXA') THEN
*        TO FORCE EXACT EXPONENTIALS IN PRECONDITIONER CALCULATIONS
         ILEXA=1
      ELSE IF(TEXT4.EQ.'DIFC') THEN
*        TRANSPORT/DIFFUSION SOLUTION FLAG.
         IDIFC=1
         IAAC=1
      ELSE IF(TEXT4.EQ.'MCU') THEN
*        MAXIMUM DIMENSION OF MCU FOR MEMORY ALLOCATION.
         CALL REDGET(INDIC,LMXMCU,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(5).')
      ELSE IF(TEXT4.EQ.'HDD') THEN
*        SELECTION OD STEP CHARACTERISTICS METHOD.
         CALL REDGET(INDIC,NITMA,HDD,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('MCCGT: REAL DATA EXPECTED(2).')
      ELSE IF(TEXT4.EQ.'STIS') THEN
*        'SOURCE TERM ISOLATION' FLAG
         CALL REDGET(INDIC,STIS,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(6).')
         IF(ABS(STIS).GT.1) THEN
            CALL XABORT('MCCGT: STIS MUST BE SET TO -1, 0 OR 1.')
         ENDIF
      ELSE IF(TEXT4.EQ.'LEXF') THEN
*        TO FORCE EXACT EXPONENTIALS IN FLUX CALCULATIONS
         ILEXF=1
      ELSE IF(TEXT4.EQ.'ADJ') THEN
*        ADJOINT FLUX CALCULATION
         IFORW=1
      ELSE IF(TEXT4.EQ.'MAXI') THEN
*        MAXIMUM NUMBER OF INNER ITERATIONS.
         CALL REDGET(INDIC,MAXI,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(7).')
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 60
      ELSE 
         CALL XABORT('MCCGT: '//TEXT4//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 50
*
 60   IF(NMU.EQ.0) THEN
         IF(ISYMM.LE.1) THEN
            NMU=(NANGL+1)/2
         ELSE IF(ISYMM.GE.2) THEN
            NMU=NANGL
         ENDIF
      ENDIF
      IGP(21)=LCACT
      LACA=(IAAC.GT.0)
      IGP(23)=KRYL
      IGP(24)=IDIFC
      IGP(27)=IAAC
      IGP(28)=ISCR
      LSCR=((ISCR.GT.0).AND.(.NOT.CYCLIC))
      IGP(31)=ILEXA
      IGP(32)=ILEXF
      IGP(33)=MAXI
      IGP(34)=LTMT
      IGP(38)=IFORW
      ZREAL(1)=EPSI
      ZREAL(2)=HDD
      ACFLAG=((LSCR).OR.(LACA))
      NZP=IGP(39)
      LPRISM=(NZP.NE.0)
      IF (LPRISM) THEN
*     3D PRISMATIC GEOMETRY
         CALL LCMGET(IPTRK,'NCODE',NCODE)
         IF (NCODE(6).EQ.30) THEN
            IF (NCODE(5).EQ.30) THEN
*           Z- and Z+ surfaces symmetry
               SSYM=2
               FACSYM=0.0
            ELSE
*           Z+ symmetry
               SSYM=1
               FACSYM=1.0
            ENDIF
         ELSE
            SSYM=0
            FACSYM=0.0
         ENDIF
         N2RS=N2SOU+N2REG+1
         CALL SETARA(BASE,(NZP+1),IZ)
         CALL SETARA(IBASE,(N2RS*(NZP+2)),INDREG)
         CALL LCMSIX(IPTRK,'PROJECTION',1)
         CALL LCMGET(IPTRK,'IND2T3',IBASE(INDREG))
         CALL LCMGET(IPTRK,'ZCOORD',BASE(IZ))
         CALL LCMSIX(IPTRK,'PROJECTION',2)
         CALL LCMGET(IPTRK,'EXCELTRACKOP',EXTKOP)
         DELU=EXTKOP(40)
         ZREAL(3)=DELU
      ELSE
         ZREAL(3)=0.0
      ENDIF 
*
      IF(IMPX.GT.1) THEN
         CALL LCMGET(IPTRK,'TITLE',KTITL)
         WRITE(TITLE,'(3A4)') (KTITL(J),J=1,3)
         WRITE(IOUT,100) TITLE
         IF (LPRISM) WRITE(IOUT,*) '3D PRISMATIC EXTENDED TRACKING'
      ENDIF
*---
* CALCULATE POLAR QUADRATURE IF REQUIRED
*--- 
      IF(NDIM.EQ.2) THEN
         IF(LCACT.EQ.0) THEN   
            CALL ALGPT ( NMU, 0.0, 1.0, XMU, WZMU)
         ELSE
            IF(LCACT.GE.3) THEN
               IF(NMU.GT.4) NMU=4
               IF(NMU.LT.2) NMU=2
            ENDIF
            CALL ALCACT( LCACT, NMU, XMU, WZMU)
         ENDIF
         IF (LPRISM) THEN
            TMUIM=0.0
            DO IMU=1,NMU
               ZMU(IMU)=1.0
               WZMU(IMU)=0.5*WZMU(IMU)
               CMU=DBLE(XMU(IMU))
               CMUV(IMU)=CMU
               CMUIV(IMU)=1.D0/CMU
               SMUV(IMU)=SQRT(1.D0-CMU**2)
               SMUIV(IMU)=1.D0/SMUV(IMU)
               TMUV(IMU)=SMUV(IMU)/CMU
               TMUIV(IMU)=1.D0/TMUV(IMU)
               TMUIM=MAX(REAL(TMUIM),REAL(TMUIV(IMU)))
            ENDDO
         ELSE
            DO IMU=1,NMU
               DUM=SQRT(1.-XMU(IMU)*XMU(IMU))
               ZMU(IMU)=1./DUM
               WZMU(IMU)=WZMU(IMU)*DUM
            ENDDO
         ENDIF
         IF(IMPX.GT.3) THEN
            CALL PRINAM('XMU   ',XMU,NMU)
            CALL PRINAM('ZMU   ',ZMU,NMU)
            CALL PRINAM('WZMU  ',WZMU,NMU)
         ENDIF
      ELSE ! NDIM.EQ.3
         NMU=1
         WZMU(1)=1.0
         XMU(1)=0.0
         ZMU(1)=1.0
      ENDIF
      IF (LPRISM) THEN
         CALL LCMSIX(IPTRK,'PROJECTION',1)
         CALL LCMPUT(IPTRK,'CMU',2*NMU,4,CMUV)
         CALL LCMPUT(IPTRK,'CMUI',2*NMU,4,CMUIV)
         CALL LCMPUT(IPTRK,'SMU',2*NMU,4,SMUV)
         CALL LCMPUT(IPTRK,'SMUI',2*NMU,4,SMUIV)
         CALL LCMPUT(IPTRK,'TMU',2*NMU,4,TMUV)
         CALL LCMPUT(IPTRK,'TMUI',2*NMU,4,TMUIV)
         CALL LCMSIX(IPTRK,'PROJECTION',2)
      ENDIF
      CALL LCMPUT(IPTRK,'ZMU$MCCG',NMU,2,ZMU)
      CALL LCMPUT(IPTRK,'XMU$MCCG',NMU,2,XMU)
      CALL LCMPUT(IPTRK,'WZMU$MCCG',NMU,2,WZMU)

      NFI=NREG+NSOU
      CALL SETARA(BASE,NFI+1,IV)
      CALL SETARA(IBASE,NFI+1,NZON)
      IF(ACFLAG) CALL SETARA(IBASE,NFI+1,NZONA)
      CALL SETARA(IBASE,NSOU,ITEMP)
      CALL SETARA(BASE,NSOU,IRTEMP)
*---
* RECOVER VOLUME AND MATALB ARRAYS
*---
      IF (LPRISM) THEN
*     3D PRISMATIC GEOMETRY
         NDIM=3
         READ(IFTRAK)
         READ(IFTRAK)
         CALL LCMSIX(IPTRK,'PROJECTION',1)
         CALL LCMGET(IPTRK,'VOLSUR',BASE(IV))
         CALL LCMGET(IPTRK,'MATALB',IBASE(NZON))
         CALL LCMSIX(IPTRK,'PROJECTION',2)
      ELSE
*     REGULAR 2D OR 3D GEOMETRY
         READ(IFTRAK) (BASE(IV+NSOU+II),II=-NSOU,NREG)
         READ(IFTRAK) (IBASE(NZON+NSOU+II),II=-NSOU,NREG)
      ENDIF
*---
* REORDER VOLUME AND MATALB ARRAYS (SURFACE -j BECOMES NV+j)
*     [S(-NS) S(-NS+1) ... S(-1) 0 V(1) ... V(NV-1) V(NV)]
*     ->[V(1)... V(NV-1) V(NV) S(-1) ... S(-NS+1) S(-NS)]
*---
      DO I=0,NSOU-1
         IBASE(ITEMP+I)=IBASE(NZON+NSOU-1-I)
         BASE(IRTEMP+I)=BASE(IV+NSOU-1-I)
      ENDDO
      DO I=0,NREG-1
         BASE(IV+I)=BASE(IV+NSOU+I+1)
         IBASE(NZON+I)=IBASE(NZON+NSOU+I+1)
         IF(ACFLAG) IBASE(NZONA+I)=IBASE(NZON+I)
      ENDDO
      DO I=0,NSOU-1
         IBASE(NZON+NREG+I)=IBASE(ITEMP+I)
         IF(ACFLAG) THEN
            IF(ALBEDO(-IBASE(ITEMP+I)).GT.(0.0)) THEN
               IBASE(NZONA+NREG+I)=IBASE(ITEMP+I)
            ELSE
               IBASE(NZONA+NREG+I)=IBCV
            ENDIF
         ENDIF
         BASE(IV+NREG+I)=BASE(IRTEMP+I)
      ENDDO
*
      CALL RLSARA(BASE(IRTEMP))
      CALL RLSARA(IBASE(ITEMP))
*
      CALL SETARA(BASE,NANGL,IDENSTY)
      READ(IFTRAK)
      READ(IFTRAK)
      READ(IFTRAK)
      READ(IFTRAK) (BASE(IDENSTY+II),II=0,NANGL-1)
*---
* CALCULATE NUMERICAL SURFACES FOR NON CYCLIC TRACKING
* IF AAC OR SCR USED, CALCULATE CONNECTION MATRICES
*---
      IF(ACFLAG) THEN
         IF (LMXMCU.EQ.0) LMXMCU=FACMCU(NDIM)*NFI
         CALL SETARA(IBASE,LMXMCU,MCUW)
         CALL SETARA(IBASE,LMXMCU,MCUI)
         CALL XDISET(IBASE(MCUW),LMXMCU,0)
         CALL XDISET(IBASE(MCUI),LMXMCU,0)
      ENDIF
      CALL SETARA(BASE,MXSEG,SEGLEN)
      CALL SETARA(IBASE,MXSEG,NRSEG)
      IF(.NOT.CYCLIC) THEN
         CALL SETARA(BASE,2*NSOU,ISURFD)
         CALL XDDSET(BASE(ISURFD),NSOU,0.D0)
      ENDIF
      LMCU=NFI
      IF (LPRISM) THEN
*     3D PRISMATIC GEOMETRY: 3D TRACKS ARE RECONSTRUCTED
      CALL SETARA(BASE,(4*NREG*NMU*NANGL),IVNUM)
      CALL XDDSET(BASE(IVNUM),2*NREG*NANGL*NMU,0.D0)
      CALL SETARA(BASE,MXSEG,IT2D)
      N3MAX=(INT(FACSYM)+1)*MXSEG*(NZP+2)
      IF (SSYM.LT.2) THEN
         CALL SETARA(IBASE,N3MAX,INOM3D)
         CALL SETARA(BASE,N3MAX,IH3D)
      ELSE
         TMUIM=TMUIM/BASE(IZ+NZP)
      ENDIF
      DO ILINE=1,NBTR
         READ(IFTRAK) IANGL,N2SEG,WEI2D,
     1        (IBASE(NRSEG-1+II),II=1,N2SEG),
     2        (BASE(SEGLEN-1+II),II=1,N2SEG)
         IF(N2SEG.GT.0) THEN
            NR2SE=N2SEG-2
            BASE(IT2D)=0.0
            DO II=0,NR2SE
               BASE(IT2D+II+1)=BASE(IT2D+II)+BASE(SEGLEN+II+1)
            ENDDO
            IF (SSYM.EQ.2) THEN
               FACSYM=MAX(TMUIM*BASE(IT2D+NR2SE+1),FACSYM)
               CALL SETARA(IBASE,(INT(FACSYM)+1)*N3MAX,INOM3D)
               CALL SETARA(BASE,(INT(FACSYM)+1)*N3MAX,IH3D)
            ENDIF
!!!!            if (NR2SE.GE.3) then
!!!!            do IZP=0,NZP
!!!!            if (IZP.lt.NZP) write(8,900) BASE(IT2D),BASE(IT2D),
!!!!     1                                   BASE(IZ+IZP),BASE(IZ+IZP+1)
!!!!            do II=1,NR2SE
!!!!            write(8,900) BASE(IT2D+II-1),BASE(IT2D+II),
!!!!     1                   BASE(IZ+IZP),BASE(IZ+IZP)
!!!!            if (IZP.lt.NZP) write(8,900) BASE(IT2D+II),BASE(IT2D+II)
!!!!     1                                  ,BASE(IZ+IZP),BASE(IZ+IZP+1)
!!!!            enddo
!!!!            enddo
!!!! 900        FORMAT(
!!!!     1  7H line([,E16.8,1H,,E16.8,3H],[,E16.8,1H,,E16.8,2H],,
!!!!     2 26H 'Color','r','Marker','o')/)
            CALL MCGPTV(N2SOU,N2REG,NZP,SSYM,NFI,NREG,NSOU,N2SEG,NR2SE,
     1           NANGL,NMU,LMCU,LMXMCU,IANGL,IBASE(NZONA),IBASE(INDREG),
     2           IBASE(NRSEG),IBASE(MCUW),IBASE(MCUI),BASE(IZ),
     3           BASE(SEGLEN),BASE(IT2D),WEI2D,CMUV,CMUIV,SMUV,SMUIV,
     4           TMUV,TMUIV,WZMU,DELU,IBASE(INOM3D),BASE(IH3D),
     5           BASE(ISURFD),BASE(IVNUM),ACFLAG)
!!!!            stop
!!!!            endif
            IF (SSYM.EQ.2) THEN
               CALL RLSARA(BASE(IH3D))
               CALL RLSARA(IBASE(INOM3D))
            ENDIF
         ENDIF
      ENDDO
      IF (SSYM.LT.2) THEN
         CALL RLSARA(BASE(IH3D))
         CALL RLSARA(IBASE(INOM3D))
      ENDIF
      CALL RLSARA(BASE(IT2D))
      CALL SETARA(BASE,2*NREG,IVGLNUM)
      CALL XDDSET(BASE(IVGLNUM),NREG,0.D0)
      CALL MCGPTN(IMPX,NREG,NSOU,NANGL,NMU,BASE(IV),BASE(IVNUM),
     1     BASE(ISURFD),BASE(IDENSTY),BASE(IVGLNUM),WZMU)
      CALL RLSARA(BASE(IVGLNUM))
      IF (IMPX.GT.4) CALL PRINDM('VNORF',BASE(IVNUM),2*NREG*NANGL*NMU)
      CALL LCMSIX(IPTRK,'PROJECTION',1)
      CALL LCMPUT(IPTRK,'VNORF',4*NREG*NANGL*NMU,4,BASE(IVNUM))
      CALL LCMSIX(IPTRK,'PROJECTION',2)
      CALL RLSARA(BASE(IVNUM))
      ELSE
*     REGULAR 2D OR 3D GEOMETRY
      DO ILINE=1,NBTR
         READ(IFTRAK) IANGL,NSEG,WEI2D,
     1        (IBASE(NRSEG-1+II),II=1,NSEG),
     2        (BASE(SEGLEN-1+II),II=1,NSEG)
         IF(NSEG.GT.0) THEN
            CALL MCGDTV(NDIM,NFI,NREG,NSOU,NSEG,NMU,LMCU,LMXMCU,
     1           IBASE(NZONA),IBASE(NRSEG),IBASE(MCUW),IBASE(MCUI),
     2           WEI2D,BASE(SEGLEN),WZMU,BASE(ISURFD),CYCLIC,ACFLAG)
         ENDIF
      ENDDO
      IF (.NOT.CYCLIC) CALL XDRSDB(NSOU,BASE(IV+NREG),BASE(ISURFD),1)
      ENDIF
*
      IF (.NOT.CYCLIC) CALL RLSARA(BASE(ISURFD))
      CALL RLSARA(IBASE(NRSEG))
      CALL RLSARA(BASE(SEGLEN))
      CALL RLSARA(BASE(IDENSTY))
      IF (LPRISM) THEN  
         CALL RLSARA(IBASE(INDREG))
         CALL RLSARA(BASE(IZ))
      ENDIF
*---
* CREATE CONNECTION MATRICES IN KM/MCU FORMAT
*---
      LMCU0=0
      IF(ACFLAG) THEN
*        KM(i) is the number of non-diagonal element on row i
*        MCU gives the column indexes.
         CALL SETARA(IBASE,NFI,KM)
         CALL SETARA(IBASE,LMXMCU,MCU)
         CALL MCGREC(NFI,IBASE(KM),IBASE(MCUW),IBASE(MCUI),
     1        IBASE(MCU),LMCU,LMXMCU,0)
         CALL RLSARA(IBASE(MCUI))
         CALL RLSARA(IBASE(MCUW))
         NLONG=NFI
         IF(CYCLIC) THEN
*        if cyclic tracking, only the volume related data are stored
            LMCU=0
            DO I=1,NREG
               LMCU=LMCU+IBASE(KM-1+I)
            ENDDO
            NLONG=NREG
         ENDIF
         CALL SETARA(IBASE,NLONG+1,IM)
*        construct IM
*        containing number of sparse matrix elements in rows before I
*        so location of J-th non-zero in row # I is IM(I)+J i.e.
*        IM(K+1)=sum_i=1^K KM(i) where K in [1,NLONG]
         IBASE(IM)=0
         DO I=1,NLONG
            IBASE(IM+I)=IBASE(IM+I-1)+IBASE(KM-1+I)
         ENDDO
         IF(LSCR) THEN
*---
* SCR ACCELERATION : CREATE INDEX FOR THE SURFACES NEIGHBORS
*---
            LPS=IBASE(IM+NFI)-IBASE(IM+NREG)
            CALL SETARA(IBASE,(NSOU+1),IS)
            CALL SETARA(IBASE,LPS,JS)
            LPS=0
            DO I=NREG+1,NFI
               IBASE(IS-1+I-NREG)=LPS
               DO J=IBASE(IM-1+I)+1,IBASE(IM+I)
                  IF(IBASE(MCU-1+J).GT.0) THEN
                     LPS=LPS+1
                     IBASE(JS-1+LPS)=IBASE(MCU-1+J)
                  ENDIF
               ENDDO
            ENDDO
            IBASE(IS+NSOU)=LPS
         ENDIF
         IF(LACA) THEN
*---
* ACA ACCELERATION : 
*---
            CALL SETARA(IBASE,NFI,PI)
            CALL SETARA(IBASE,NFI,INVPI)
            IF(PACA.GE.2) THEN
               LMCU0=LMCU
               CALL SETARA(IBASE,NLONG,LEV)
               CALL SETARA(IBASE,NLONG+1,LEVPT)
               CALL SETARA(IBASE,NLONG,KMROR)
               CALL SETARA(IBASE,LMCU,MCUROR)
               CALL SETARA(IBASE,NLONG+1,IMROR)
               CALL SETARA(IBASE,NLONG,JU)
               CALL SETARA(BASE,NFI,IVA)
               IF(PACA.EQ.3) THEN
                  CALL SETARA(IBASE,NFI,IWORK)
                  CALL SETARA(IBASE,(NFI+1),IM0)
                  CALL SETARA(IBASE,LMCU0,MCU0)
               ENDIF
*              construct PI permutation : old_index=PI(new_index) or F_new=F_old(PI)
*              reordering of the unknowns of the corrective system for ilu0 preconditioner.
               NFIRST=1
               TYPOR1=0
               TYPOR2=0
               CALL RENUM(NLONG,LMCU,NFIRST,IBASE(IM),IBASE(MCU),TYPOR1,
     1             TYPOR2,NLEV,IBASE(LEV),IBASE(LEVPT),IBASE(PI))
               IF(CYCLIC) THEN
                  DO I=NLONG+1,NFI
                     IBASE(PI-1+I)=I
                  ENDDO
               ENDIF
*              reorder everything according to PI
*              construct INVPI permutation : new_index=INVPI(old_index) or F_old=F_new(INVPI)
               DO I=1,NFI
                  J=IBASE(PI-1+I)
                  IBASE(INVPI-1+J)=I
               ENDDO
               DO I=1,NLONG
                  J=IBASE(PI-1+I)               
                  IBASE(KMROR-1+I)=IBASE(KM-1+J)
               ENDDO
               IBASE(IMROR)=0
               DO I=1,NLONG
                  IBASE(IMROR+I)=IBASE(IMROR-1+I)+IBASE(KMROR-1+I)
               ENDDO
               DO I=1,NLONG
                  J=IBASE(PI-1+I)
                  IK=IBASE(IMROR-1+I)
                  DO JK=IBASE(IM-1+J)+1,IBASE(IM+J)
                     IK=IK+1
                     IF(IBASE(MCU-1+JK).GT.0) THEN
                       IBASE(MCUROR-1+IK)=IBASE(INVPI-1+IBASE(MCU-1+JK))
                     ELSE
                        IBASE(MCUROR-1+IK)=IBASE(MCU-1+JK)
                     ENDIF
                  ENDDO
               ENDDO
*              sort each line by increasing column index
               DO I=1,NLONG
                  K=IBASE(IMROR-1+I)+1
                  CALL SORTIN(IBASE(KMROR-1+I),IBASE(MCUROR-1+K))
               ENDDO
               DO I=1,NFI
                  J=IBASE(PI-1+I)
                  IBASE(NZONA-1+I)=IBASE(NZON-1+J)
                  BASE(IVA-1+I)=BASE(IV-1+J)
                  IF(J.GT.NREG) THEN
                     IF(ALBEDO(-IBASE(NZON-1+J)).EQ.(0.0)) THEN
                        IBASE(NZONA-1+I)=IBCV
                     ENDIF
                  ENDIF
               ENDDO
               CALL XDISET(IBASE(JU),NLONG,0)
               IF(PACA.EQ.3) THEN
                  CALL XDISET(IBASE(IM0),(NLONG+1),LMCU)
                  CALL XDISET(IBASE(IWORK),NLONG,0)
                  ILAST=0
                  LMCU0=0
               ENDIF
               DO 30 I=1,NLONG
*              construct JU (and IM0/MCU0 for optimized storage)
*              MCUROR(JU(i):IMROR(i+1)) corresponds to the upper triangular part of line i.
*              MCUROR(IMROR(i)+1:JU(i)-1) correspond to the lower triangular part of line i.
                  DO IH=IBASE(IMROR-1+I)+1,IBASE(IMROR+I)
                     H=IBASE(MCUROR-1+IH)
                     IF(H.GT.0) THEN
                        IF((H.GT.I).AND.(IBASE(JU-1+I).EQ.0)) THEN
                           IBASE(JU-1+I)=IH
                        ENDIF
                        IF(PACA.EQ.3) IBASE(IWORK-1+H)=IH
                     ENDIF
                  ENDDO
                  IF(IBASE(JU-1+I).EQ.0) THEN
                     IBASE(JU-1+I)=IBASE(IMROR+I)+1
                  ENDIF
                  IF(PACA.EQ.3) THEN
                  DO IK=IBASE(IMROR-1+I)+1,IBASE(JU-1+I)-1
                     K=IBASE(MCUROR-1+IK)
                     IF(K.GT.0) THEN
                     DO KJ=IBASE(JU-1+K),IBASE(IMROR+K)
                        J=IBASE(MCUROR-1+KJ)
                        IF(IBASE(IWORK-1+J).GT.0) THEN
                           IPOS=0
                           IJEND=MIN(IBASE(IM0+I),LMCU0)
                           DO IJ=IBASE(IM0-1+I)+1,IJEND
                           IF(IBASE(MCU0-1+IJ).EQ.J) THEN
                              IPOS=IJ
                              GOTO 40
                           ENDIF
                           ENDDO
 40                        CONTINUE
                           IF(IPOS.EQ.0) THEN
                           IF(ILAST.NE.I) THEN
                           CALL XDISET(IBASE(IM0+ILAST),(I-ILAST),LMCU0)
                           ILAST=I
                           ENDIF
                           LMCU0=LMCU0+1
                           IBASE(MCU0-1+LMCU0)=J
                           ENDIF
                        ENDIF
                     ENDDO
                     ENDIF
                  ENDDO
                  DO IH=IBASE(IMROR-1+I)+1,IBASE(IMROR+I)
                     H=IBASE(MCUROR-1+IH)
                     IF(H.GT.0) THEN
                        IBASE(IWORK-1+H)=0
                     ENDIF
                  ENDDO
                  ENDIF
 30            CONTINUE
               IF(PACA.EQ.3) THEN
               IF(LMCU0.EQ.0) THEN
                  PACA=4 ! SPECIAL CASE WHEN THERE IS NO EXTRA-STORAGE FOR ILU0-ACA
               ELSE
                  CALL XDISET(IBASE(IM0+ILAST),(NLONG+1-ILAST),LMCU0)
               ENDIF
               ENDIF
            ELSE
               DO I=1,NFI
                  IBASE(PI-1+I)=I
                  IBASE(INVPI-1+I)=I
               ENDDO
            ENDIF
         ENDIF
      ENDIF
      IF(IMPX.GT.3) THEN
         CALL PRINIM('MATALB',IBASE(NZON),NFI)
         CALL PRINAM('VOLSUR',BASE(IV),NFI)
         IF(ACFLAG) THEN
            CALL PRINIM('MATALA',IBASE(NZONA),NFI)
            WRITE(IOUT,11) LMCU
 11         FORMAT(1X,'MCGREC : LMCU =',I6)
            CALL PRINIM('KM    ',IBASE(KM),NLONG)
            CALL PRINIM('MCU   ',IBASE(MCU),LMCU)
            IF((LACA).AND.(PACA.GE.2)) THEN
               CALL PRINIM('IPERM ',IBASE(INVPI),NFI)
               CALL PRINIM('KMROR ',IBASE(KMROR),NLONG)
               CALL PRINIM('MCUROR',IBASE(MCUROR),LMCU)
               CALL PRINIM('JU    ',IBASE(JU),NLONG)
               IF(PACA.GE.3) THEN
                  WRITE(IOUT,12) LMCU0
 12               FORMAT(1X,'MCCGT : LMCU0 =',I6)
                  IF(LMCU0.GT.0) THEN
                     CALL PRINIM('IM0    ',IBASE(IM0),NLONG+1)
                     CALL PRINIM('MCU0   ',IBASE(MCU0),LMCU0)
                  ENDIF
               ENDIF
            ENDIF
            IF(LSCR) THEN
               CALL PRINIM('IS    ',IBASE(IS),NSOU+1)
               CALL PRINIM('JS    ',IBASE(JS),LPS)
            ENDIF
         ENDIF
      ENDIF
*     
      CALL LCMPUT(IPTRK,'NZON$MCCG',NFI,1,IBASE(NZON))
      CALL LCMPUT(IPTRK,'MATCOD',NREG,1,IBASE(NZON))
      CALL LCMPUT(IPTRK,'V$MCCG',NFI,2,BASE(IV))
      CALL LCMPUT(IPTRK,'VOLUME',NREG,2,BASE(IV))
      IF(ACFLAG) THEN
         IF(LACA) THEN
            CALL LCMPUT(IPTRK,'NZONA$MCCG',NFI,1,IBASE(NZONA))
            IF(PACA.GE.2) THEN
               CALL LCMPUT(IPTRK,'VA$MCCG',NFI,2,BASE(IVA))
               CALL LCMPUT(IPTRK,'KM$MCCG',NLONG,1,IBASE(KMROR))
               CALL LCMPUT(IPTRK,'IM$MCCG',NLONG+1,1,IBASE(IMROR))
               CALL LCMPUT(IPTRK,'MCU$MCCG',LMCU,1,IBASE(MCUROR))
               CALL LCMPUT(IPTRK,'JU$MCCG',NLONG,1,IBASE(JU))
               IF(PACA.EQ.3) THEN
                  CALL LCMPUT(IPTRK,'IM0$MCCG',NLONG+1,1,IBASE(IM0))
                  CALL LCMPUT(IPTRK,'MCU0$MCCG',LMCU0,1,IBASE(MCU0))    
               ENDIF
               IF(PACA.GE.3) THEN
                  CALL RLSARA(IBASE(MCU0))
                  CALL RLSARA(IBASE(IM0))
                  CALL RLSARA(IBASE(IWORK))
               ENDIF
               CALL RLSARA(BASE(IVA))
               CALL RLSARA(IBASE(JU))
               CALL RLSARA(IBASE(IMROR))
               CALL RLSARA(IBASE(MCUROR))
               CALL RLSARA(IBASE(KMROR))
               CALL RLSARA(IBASE(LEVPT))
               CALL RLSARA(IBASE(LEV))
            ELSE
               CALL LCMPUT(IPTRK,'KM$MCCG',NLONG,1,IBASE(KM))
               CALL LCMPUT(IPTRK,'IM$MCCG',NLONG+1,1,IBASE(IM))
               CALL LCMPUT(IPTRK,'MCU$MCCG',LMCU,1,IBASE(MCU)) 
               CALL LCMPUT(IPTRK,'VA$MCCG',NLONG,2,BASE(IV))           
            ENDIF
            CALL LCMPUT(IPTRK,'INVPI$MCCG',NFI,1,IBASE(INVPI))
            CALL LCMPUT(IPTRK,'PI$MCCG',NLONG,1,IBASE(PI))
            CALL RLSARA(IBASE(INVPI))
            CALL RLSARA(IBASE(PI))
         ENDIF
         IF(LSCR) THEN
            CALL LCMPUT(IPTRK,'IS$MCCG',NSOU+1,1,IBASE(IS))
            CALL LCMPUT(IPTRK,'JS$MCCG',LPS,1,IBASE(JS))
            CALL RLSARA(IBASE(JS))
            CALL RLSARA(IBASE(IS))
         ENDIF
         CALL RLSARA(IBASE(IM))
         CALL RLSARA(IBASE(MCU))
         CALL RLSARA(IBASE(KM))
         CALL RLSARA(IBASE(NZONA))
      ENDIF
      CALL RLSARA(IBASE(NZON))
      CALL RLSARA(BASE(IV))  
      IF(.NOT.LACA) LMCU=0
      IF(.NOT.LSCR) LPS=0
*---
* MODIFY KEYFLX FOR ANISOTROPIC SCATTERING
* CREATE KEYCUR
*---
      IF(NDIM.EQ.2) THEN
         NFUNL=NANIS*(NANIS+1)/2
      ELSE ! NDIM.EQ.3
         NFUNL=NANIS*NANIS
      ENDIF
      DIMKEYF=NREG*NFUNL

      IGP(2)=DIMKEYF
      TEXT12='MCCG'
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPTRK,'TRACK-TYPE',3,3,KCHAR)
      IF(.NOT.CYCLIC) THEN
*     non-cyclic tracking -> MCCG used (else MOCC)
         IGP(2)=IGP(2)+IGP(5)
      ENDIF

      NUN=IGP(2)
      CALL SETARA(IBASE,DIMKEYF,KEYFLX)
      DO 20 IA=1,NFUNL
         DO 20 IR=1,NREG
 20         IBASE(KEYFLX+(IA-1)*NREG+IR-1)=IR+(IA-1)
     1                                     *NUN/NFUNL
      IF(.NOT.CYCLIC) THEN
         CALL SETARA(IBASE,NSOU,KEYCUR)
         IKEY=1
         ICUR=0
         DO I=1,NUN
            IF((IBASE(KEYFLX+IKEY-1).NE.I).OR.(IKEY.GT.DIMKEYF)) THEN
               ICUR=ICUR+1
               IF (ICUR.GT.NSOU)
     1         CALL XABORT('MCCGT: INCORRECT NUMBER OF UNKNOWNS')
               IBASE(KEYCUR-1+ICUR)=I
            ELSE
               IKEY=IKEY+1
            ENDIF
         ENDDO
         CALL LCMPUT(IPTRK,'KEYCUR$MCCG',NSOU,1,IBASE(KEYCUR))
         CALL RLSARA(IBASE(KEYCUR))      
      ENDIF
      CALL LCMPUT(IPTRK,'KEYFLX$MCCG',DIMKEYF,1,IBASE(KEYFLX))
      CALL LCMPUT(IPTRK,'KEYFLX',DIMKEYF,1,IBASE(KEYFLX))
      CALL RLSARA(IBASE(KEYFLX))
*---
* GENERATE ALL SIGNS FOR SPHERICAL HARMONICS
*---
      NRNL=NANIS*NANIS
      CALL SETARA(IBASE,NRNL,KEYANI)
      CALL SETARA(IBASE,8*NRNL,ISGNR)
      CALL XDISET(IBASE(KEYANI),NRNL,0)
      CALL MOCIK3(NDIM,NANIS-1,IBASE(ISGNR),IBASE(KEYANI)) 
      CALL LCMPUT(IPTRK,'ISGNR$MCCG',8*NRNL,1,IBASE(ISGNR))
      CALL LCMPUT(IPTRK,'KEYANI$MCCG',NRNL,1,IBASE(KEYANI))
      CALL RLSARA(IBASE(ISGNR))
*---
* GENERATE INDEX FOR PJJ(NU'->NU) STORAGE
* IF 'SOURCE TERM ISOLATION' OPTION IS ON
*---
      IF((STIS.NE.0).OR.(ISCR.GT.0)) THEN
         CALL MCGPJJ(IPTRK,IMPX,NDIM,NANIS,NFUNL,NPJJM,
     1        IBASE(KEYANI))
      ELSE
         NPJJM=1
      ENDIF
      CALL RLSARA(IBASE(KEYANI))
*     
      IGP(22)=NMU
      IGP(25)=MXSEG
      IGP(26)=LMCU
      IGP(29)=LPS
      IGP(30)=PACA
      IGP(35)=STIS
      IGP(36)=NPJJM
      IGP(37)=LMCU0
      IGP(38)=IFORW
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,IGP)
      ZREAL(4)=FACSYM
      CALL LCMPUT(IPTRK,'REAL-PARAM',4,2,ZREAL)
*
      IF(IMPX.GT.1) THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
         WRITE(IOUT,120) (IGP(I),I=21,38)
         CALL LCMGET(IPTRK,'REAL-PARAM',ZREAL)
         WRITE(IOUT,130) (ZREAL(I),I=1,4)
      ENDIF
*----
*  PROCESS DOUBLE HETEROGENEITY (BIHET) DATA (IF AVAILABLE)
*----
      IF(LBIHET) CALL XDRTBH(IPGEO,IPTRK,IQUA10,IBIHET,IMPX)
*
      IF(IMPX.GT.2) CALL LCMLIB(IPTRK)
*
  100 FORMAT(/
     1 44H MM      MM  CCCCC   CCCCC   GGGGG  TTTTTTTT/
     2 41H MMM    MMM CCCCCCC CCCCCCC GGGGGGG    TT/
     4 41H MMMM  MMMM CC   CC CC   CC GG         TT/
     5 41H MM  MM  MM CC      CC      GG  GGG    TT/
     6 41H MM      MM CC      CC      GG  GGG    TT/
     7 41H MM      MM CC   CC CC   CC GG   GG    TT/
     8 41H MM      MM CCCCCCC CCCCCCC GGGGGGG    TT/
     9 41H MM      MM  CCCCC   CCCCC   GGGGG     TT/
     1 17H TRACKING TITLE: ,A72/) 
  120 FORMAT(/
     1 55H STATE VECTOR RELATED TO THE METHOD OF CHARACTERISTICS:/
     2 7H LCACT ,I8,29H   (TYPE OF POLAR QUADRATURE)/
     1 7H NMU   ,I8,48H   (ORDER OF THE POLAR QUADRATURE IN 2D/1 IN 3D)/
     5 7H KRYL  ,I8,48H   (<0 Bi-CGSTAB SCHEME USED /0=KRYLOV SCHEMES N,
     6 30HOT USED/ >0=GMRES SCHEME USED)/
     7 7H IDIFC ,I8,39H   (0=TRANSPORT/1=CDD SOLUTION OF FLUX)/
     8 7H NMAX  ,I8,42H   (MAXIMUM NUMBER OF ELEMENTS IN A TRACK)/
     9 7H LMCU  ,I8,42H   (DIMENSION OF MCU FOR ACA ACCELERATION)/
     3 7H IAAC  ,I8,48H   (0=NO ACCELERATION/1=CDD ACCELERATION OF INNE,
     4 13HR ITERATIONS)/
     2 7H SCR   ,I8,48H   (0=NO ACCELERATION/1=SCR ACCELERATION OF INNE,
     3 13HR ITERATIONS)/
     4 7H LPS   ,I8,42H   (DIMENSION OF PSJ FOR SCR ACCELERATION)/
     5 7H PACA  ,I8,48H   (PRECONDITIONER FOR SOLVING THE ACA SYSTEM WI,
     6 38HTH BICGSTAB (>2=ILU0, 1=DIAG, 0=NONE))/
     7 7H LEXA  ,I8,48H   (TO FORCE EXACT EXPONENTIAL USAGE IN PRECONDI,
     8 19HTIONER CALCULATION)/
     9 7H LEXF  ,I8,48H   (TO FORCE EXACT EXPONENTIAL USAGE IN FLUX CAL,
     1 9HCULATION)/
     2 7H MAXI  ,I8,39H   (MAXIMUM NUMBER OF INNER ITERATIONS)/
     3 7H LTMT  ,I8,48H   (TO USE TRACK MERGING FOR ACA SYSTEM CALCULAT,
     4 4HION)/
     5 7H STIS  ,I8,48H   (1=SOURCE TERM ISOLATION FOR FLUX INTEGRATION,
     6 1H)/
     7 7H NPJJM ,I8,48H   (NUMBER OF PJJ MODES TO STORE FOR STIS OPTION,
     8 1H)/
     9 7H LMCU0 ,I8,48H   (DIMENSION OF MCU0 FOR ILU0-ACA ACCELERATION)/
     1 7H IFORW ,I8,40H   (0/1=DIRECT/ADJOINT FLUX CALCULATION))
  130 FORMAT(/
     1 12H REAL PARAM:/
     2 7H EPSI  ,1P,E12.4,33H   (TOLERANCE ON INNER ITERATION)/
     3 7H HDD   ,1P,E12.4,41H   (0.0=STEP CHARACTERISTICS SOLUTION/>>0,
     4 33H.0=DIAMOND DIFFERENCING SOLUTION)/
     5 7H DELU  ,1P,E12.4,42H   (TRACK SPACING FOR 3D PRISMATIC GEOMETR,
     6 2HY)/
     7 7H FACSYM,1P,E12.4,42H   (TRACKING SYMMETRY FACTOR FOR MAXIMUM T,
     8 38HRACK LENGTH FOR 3D PRISMATIC GEOMETRY)/)
*
      RETURN
      END
