*DECK MCGACA
      SUBROUTINE MCGACA(LFORW,PACA,N,NG,NFIRST,NGEFF,M,LC,NGIND,NGINDV,
     1                  NCONV,KPSYS,JPMACR,NZON,IPERM,IM,MCU,JU,NJJ,
     2                  IJJ,IPOS,XSCAT,XIN,XOUT,TEMP,LC0,IM0,MCU0)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Compute the product of the left-hand side ACA matrix in its multigroup
* form with a vector and apply group per group left preconditioner.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Le Tellier
*
*Parameters: input
* LFORW   flag set to .false. to transpose the coefficient matrix.
* PACA    type of preconditioner to solve the ACA corrective system.
* N       number of unknowns per group.
* NG      total number of groups.
* NFIRST  first group to proceed.
* NGEFF   number of  unconverged groups.
* M       number of material mixtures.
* LC      dimension of profiled matrices MCU and CQ.
* NGIND   index of the groups to process.
* NGINDV  index to pass from "NGEFF format" to "NG format".
* NCONV   logical array of convergence status for each group (.TRUE. : not converged).
* KPSYS   pointer array for each group properties.
* JPMACR  pointer to the macrolib LCM object ('GROUP' directory).
* NZON    index-number of the mixture type assigned to each volume.
* IPERM   permutation array for ACA.
* IM      connection matrix.
* MCU     connection matrix.
* JU      used for ilu0 preconditioner.
* XIN    
* LC0     used in ILU0-ACA acceleration.
* IM0     used in ILU0-ACA acceleration.
* MCU0    used in ILU0-ACA acceleration.
*
*Parameters: output
* XOUT
*
*Parameters: scratch
* NJJ
* IJJ
* IPOS
* XSCAT
* TEMP
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER PACA,N,NFIRST,NGEFF,NG,M,LC,NGIND(NGEFF),NGINDV(NG),
     1 KPSYS(NGEFF),JPMACR,NZON(N),IPERM(N),IM(N+1),MCU(LC),JU(N),
     2 NJJ(0:M),IJJ(0:M),IPOS(0:M),LC0,IM0(*),MCU0(*)
      REAL XSCAT(0:M*NG)
      DOUBLE PRECISION XIN(N,NGEFF),XOUT(N,NGEFF),TEMP(N)
      LOGICAL LFORW,NCONV(NGEFF)
*----
* LOCAL VARIABLES
*----
      INTEGER I,J,II,IG,JG,JJ,JND,IBM,JPSYS,KPMACR
      INTEGER IDIAGF,ICF,ILUDF,ILUCF,IDIAGQ,ICQ
      INTEGER LCMGIL
      EXTERNAL LCMGIL
      REAL BASE
      COMMON BASE(1)
*
      DO II=NFIRST,NGEFF
         IF (NCONV(II)) THEN
*           compute temp=sum_{g' ne g} Sigma_s^{g<-g'} XIN^{g'}
            IG=NGIND(II)
            JPSYS=KPSYS(II)
            CALL LCMIOF(JPSYS,'DIAGF$MCCG',IDIAGF)
            CALL LCMIOF(JPSYS,'CF$MCCG',ICF)
            IF (PACA.GE.2) THEN
               CALL LCMIOF(JPSYS,'ILUDF$MCCG',ILUDF)
               IF (PACA.LT.4) CALL LCMIOF(JPSYS,'ILUCF$MCCG',ILUCF)
            ENDIF
            KPMACR=LCMGIL(JPMACR,IG)
            CALL LCMIOF(JPSYS,'DIAGQ$MCCG',IDIAGQ)
            CALL LCMIOF(JPSYS,'CQ$MCCG',ICQ)
            CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
            CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
            CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
            CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
            DO I=1,N
               TEMP(I)=0.0
               J=IPERM(I)
               IBM=NZON(J)
               IF (IBM.GT.0) THEN
                  JG=IJJ(IBM)
                  DO 10 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     JJ=NGINDV(JG)
                     IF (JJ.GE.NFIRST) THEN
                        TEMP(I)=TEMP(I)+XSCAT(IPOS(IBM)+JND-1)*XIN(I,JJ)
                     ENDIF
                  ENDIF
 10               JG=JG-1
               ENDIF
            ENDDO
*           compute E^{g}*temp
            CALL MCGPRA(LFORW,1,PACA,.FALSE.,N,LC,IM,MCU,JU,
     1           BASE(IDIAGQ),BASE(ICQ),BASE(ILUDF),BASE(ILUCF),
     2           BASE(IDIAGF),TEMP,XOUT(1,II),LC0,IM0,MCU0,BASE(ICF))
*           compute D^{g}*XIN^{g}
            CALL MCGPRA(LFORW,1,PACA,.FALSE.,N,LC,IM,MCU,JU,
     1           BASE(IDIAGF),BASE(ICF),BASE(ILUDF),BASE(ILUCF),
     2           BASE(IDIAGF),XIN(1,II),TEMP,LC0,IM0,MCU0,BASE(ICF))
*           temp=D^{g}*XIN^{g}-E^{g}*sum_{g' ne g} Sigma_s^{g<-g'} XIN^{g'}
            DO I=1,N
               TEMP(I)=TEMP(I)-XOUT(I,II)
            ENDDO
*           apply single-group preconditioner XOUT^{g}=P^{g}*temp
            CALL MCGPRA(LFORW,2,PACA,.TRUE.,N,LC,IM,MCU,JU,BASE(IDIAGF),
     1           BASE(ICF),BASE(ILUDF),BASE(ILUCF),BASE(IDIAGF),
     2           XOUT(1,II),TEMP,LC0,IM0,MCU0,BASE(ICF))
         ENDIF
      ENDDO
*
      RETURN
      END
