*DECK MCGASM
      SUBROUTINE MCGASM(SUBPJJ,SUBDS2,SUBDSP,SUBDSC,IPTRK,KPSYS,IPRINT,
     1                  IFTRAK,NANI,NG,NGEFF,NFI,NREG,NLONG,M,NMU,NANGL,
     2                  N2MAX,LC,NDIM,NFUNL,NMOD,NGIND,NZON,NZONA,ISGNR,
     3                  KEYANI,V,VA,CAZ,CPO,ZMU,WZMU,KM,MCU,IM,JU,IPERM,
     4                  LC0,MCU0,IM0,IS,JS,PACA,LPS,LTMT,NPJJM,PJJIND,
     5                  LACA,LPJJ,LPJJAN,SIGAL,LPRISM,N2REG,N2SOU,NZP,
     6                  DELU,FACSYM,DIAGF,CF)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Preconditioning matrices calculation based on the Algebraic Collapsing
* Acceleration developed by I. R. Suslov and R. Le Tellier 
* or Self-Collision Probabilities acceleration developed by G.J. Wu 
* and R. Roy.
*
*  Igor R. Suslov, "An Algebraic Collapsing Acceleration in Long 
*                   Characteristics Transport Theory"
*   Proc. of 11-th Symposium of AER,178\9-188,Csopak,September 2001.
*  Igor R. Suslov, "Solution of Transport Equation in 2- and 3-
*                   dimensional Irregular Geometry by the Method 
*                   of Characteristics"
*   Int. Conf. Mathematical. Methods and Supercomputing in Nuclear
*   Applications,Karlsruhe,1994.
*  G.J. Wu and R. Roy, "Acceleration Techniques for Trajectory-based
*                        Deterministic 3D Transport Solvers",
*   Ann. Nucl. Energy, 30, 567-583 (2003).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): I. Suslov and R. Le Tellier
*
*Parameters: input/output
* SUBPJJ  PJJ calculation subroutine.
* SUBDS2  ACA coefficients summation subroutine.
* SUBDSP  ACA coefficients position subroutine.
* SUBDSC  ACA coefficients calculation subroutine.
* IPTRK   pointer to the tracking (L_TRACK signature).
* KPSYS   pointer array for each group properties.
* IPRINT  print parameter (equal to zero for no print).
* IFTRAK  tracking file unit number if IOFSET=0.
* NANI    number of Legendre orders.
* NG      number of energy groups.
* NGEFF   number of groups to process.
* NFI     total number of volumes and surfaces for which specific values
*         of the neutron flux and reactions rates are required.
* NREG    number of volumes for which specific values
*         of the neutron flux and reactions rates are required.
* NLONG   order of the corrective system.
* M       number of material mixtures.
* NMU     order of the polar quadrature in 2D / 1 in 3D.
* NANGL   number of tracking angles in the plan.
* N2MAX   maximum number of elements in a track
* LC      dimension of vector MCU.
* NDIM    number of dimensions for the geometry.
* NFUNL   number of moments of the flux (in 2D : NFUNL=NANI*(NANI+1)/2).
* NMOD    second dimension of ISGNR.
* NGIND   index of the groups to process.
* NZON    index-number of the mixture type assigned to each volume.
* NZONA   index-number of the mixture type assigned to each volume for ACA.
* ISGNR   array of the spherical harmonics signs for the different reflections.
* KEYANI  index for anisotropy.
* V       volumes and surfaces.
* VA      volumes and surfaces reordered for ACA.
* CAZ     cosines of the different tracking azimuthal angles.
* CPO     cosines of the different tracking polar angles.
*         (polar quadrature in 2D / tracking angles in 3D).
* ZMU     polar quadrature set in 2D.
* WZMU    polar quadrature set in 2D.
* KM      used in ACA acceleration.
* MCU     used in ACA acceleration.
* IM      used in ACA acceleration.
* JU      used in ACA  acceleration for ilu0.
* IPERM   permutation array for the unknowns of the corrective system for ilu0.
* LC0     used in ILU0-ACA acceleration.
* IM0     used in ILU0-ACA acceleration.
* MCU0    used in ILU0-ACA acceleration.
* IS      arrays for surfaces neighbors
* JS      JS(IS(ISOUT)+1:IS(ISOUT+1)) give the neighboring regions to surface ISOUT.
* PACA    type of preconditioner to solve the ACA corrective system.
* LPS     dimension of JS.
* LTMT    tracking merging flag.
* NPJJM   number of pjj modes to store for STIS option.
* PJJIND  index of the modes for STIS option.
* LACA    ACA flag.
* LPJJ    PJJ flag.
* LPJJAN  anisotropic PJJ flag.
* SIGAL   albedos and total cross sections array.
* LPRISM  3D prismatic extended tracking flag.
* N2REG   number of regions in the 2D tracking if LPRISM.
* N2SOU   number of external surfaces in the 2D tracking if LPRISM.
* NZP     number of z-plans if LPRISM.
* DELU    input track spacing for 3D track reconstruction if LPRISM.
* FACSYM  tracking symmetry factor for maximum track length if LPRISM.
*
*Parameters: scratch
* DIAGF
* CF
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER KPSYS(NGEFF),IPRINT,IFTRAK,NG,NGEFF,NFI,NREG,NLONG,
     1 M,NMU,NANGL,N2MAX,LC,NDIM,NFUNL,NMOD,NGIND(NGEFF),NZON(NFI),
     2 NZONA(NFI),ISGNR(NFUNL,NMOD),KEYANI(NFUNL),KM(NFI),MCU(LC),
     3 IM(NLONG+1),JU(NLONG),IPERM(NFI),LC0,MCU0(*),IM0(*),
     4 IS(NFI-NREG+1),JS(LPS),PACA,LPS,NPJJM,PJJIND(NPJJM,2),N2REG,
     5 N2SOU,NZP
      REAL V(NFI),VA(NFI),CAZ(NANGL),CPO(*),ZMU(NMU),WZMU(NMU),
     1 SIGAL(-6:M,NGEFF),DELU,FACSYM
      DOUBLE PRECISION DIAGF(NLONG,NGEFF),CF(LC,NGEFF)
      LOGICAL LTMT,LACA,LPJJ,LPJJAN,LPRISM,LFORC
      EXTERNAL SUBPJJ,SUBDS2,SUBDSP,SUBDSC
*----
*  LOCAL VARIABLES
*----
      INTEGER IBASE(1)
      INTEGER NOM,IH,IHH,IZMUA,IWZMUA,PREV,NEXT,CQ,DIAGQ,DIAGFR,CFR,
     1 ILUDF,ILUCF,PJJ,PJJD,PSJ,NOM0,IH0,IXSW,IWORK,IH3D,IH3D0,NOM3D,
     2 NOM3D0,NCODE(6),SSYM
      REAL T1,T2,T3
      DOUBLE PRECISION WEIGHT0
      CHARACTER TEXT4*4
      EXTERNAL MCGDS2,MOCDS2,MCGDSP,MOCDSP
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  MEMORY ALLOCATION
*---- 
      IF (LPRISM) THEN
         CALL LCMGET(IPTRK,'NCODE',NCODE)
         IF (NCODE(6).EQ.30) THEN
            IF (NCODE(5).EQ.30) THEN
*           Z- and Z+ surfaces symmetry
               SSYM=2
            ELSE
*           Z+ symmetry
               SSYM=1
            ENDIF
         ELSE
            SSYM=0
         ENDIF
         NMAX=(INT(FACSYM)+1)*N2MAX*(NZP+2)
      ELSE
         NMAX=N2MAX
      ENDIF
      CALL SETARA(IBASE,N2MAX,NOM)
      CALL SETARA(BASE,N2MAX,IH)
      CALL SETARA(BASE,N2MAX,IHH)
      CALL SETARA(BASE,NMU,IZMUA)
      CALL SETARA(BASE,NMU,IWZMUA)      
      IF (LPJJ) THEN
*     Self-Collision Probabilities
         IF (LPJJAN) THEN
            NRNL=NANI*NANI
            CALL SETARA(BASE,NMU*8*NRNL,ITRHAR)
            CALL SETARA(BASE,NMU*NRNL,IRHARM)
         ENDIF
         CALL SETARA(BASE,NREG*NPJJM*NGEFF,PJJ)
         IF (LPS.GT.0) THEN
            CALL SETARA(BASE,LPS*NGEFF,PSJ)
            CALL XDRSET(BASE(PSJ),LPS*NGEFF,0.0)
         ENDIF
         CALL SETARA(BASE,2*NREG*NPJJM*NGEFF,PJJD)
         CALL XDDSET(BASE(PJJD),NREG*NPJJM*NGEFF,0.D0)
      ENDIF
      IF (LACA) THEN
*     Algebraic Collapsing Acceleration
         CALL SETARA(BASE,(M+1)*NANI*NGEFF,IXSW)
         IF (LTMT) THEN
            CALL SETARA(IBASE,N2MAX,NOM0)
            CALL SETARA(BASE,N2MAX,IH0)
         ENDIF
         CALL SETARA(IBASE,NMAX,PREV)
         CALL SETARA(IBASE,NMAX,NEXT)
         CALL SETARA(BASE,LC*NGEFF,CQ)
         CALL SETARA(BASE,NLONG*NGEFF,DIAGQ)
         CALL SETARA(BASE,NLONG,DIAGFR)
         CALL SETARA(BASE,LC,CFR)
         CALL SETARA(BASE,6*NMAX,IWORK)
         IF (PACA.GE.2) THEN
            CALL SETARA(BASE,NLONG,ILUDF)
            IF (LC0.GT.0) CALL SETARA(BASE,LC0,ILUCF)
         ENDIF
         DO II=1,NGEFF
            JPSYS=KPSYS(II)
            CALL LCMGET(JPSYS,'DRAGON-S0XSC',BASE(IXSW+(II-1)*(M+1)))
         ENDDO
         CALL XDRSET(BASE(DIAGQ),NLONG*NGEFF,0.0)
         CALL XDRSET(BASE(CQ),LC*NGEFF,0.0)
         CALL XDDSET(DIAGF,NLONG*NGEFF,0.D0)
         CALL XDDSET(CF,LC*NGEFF,0.D0)
      ENDIF 
*
      NMUA=NMU
      DO IE=1,NMUA
         BASE(IZMUA-1+IE)=ZMU(IE)
         BASE(IWZMUA-1+IE)=WZMU(IE)       
      ENDDO
      IF ((.NOT.LPRISM).AND.(NDIM.EQ.2).AND.LTMT) THEN
         NMUA=1
         ZMUI=BASE(IZMUA)
         W=BASE(IWZMUA)               
         TEMP=ZMUI*W
         DO IE=2,NMU
            ZMUI=BASE(IZMUA-1+IE)
            WZMUI=BASE(IWZMUA-1+IE)
            W=W+WZMUI
            TEMP=TEMP+ZMUI*WZMUI
         ENDDO               
         BASE(IZMUA)=TEMP/W
         BASE(IWZMUA)=W
      ENDIF
*---
* SUMMATION UPON THE TRACKING
*---
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR  
      DO ICOM=1,NCOMNT+7
         READ(IFTRAK)
      ENDDO
      CALL KDRCPU(T1)
      IANGL0=0
      IPANG=1
      NMERG=0
      NTR=0
      NTRTMT=0
      NSE=0
      NSETMT=0
      LFORC=.FALSE.
      NTPROC=1
*      
      IF (LPRISM) THEN
*     3D PRISMATIC GEOMETRY CONSTRUCTED FROM A 2D TRACKING
      N3TR=0
      N3TRTMT=0
      N3SE=0
      N3SETMT=0
      CALL SETARA(IBASE,NMAX,NOM3D)
      CALL SETARA(BASE,NMAX,IH3D)
      IF (LTMT) THEN
         CALL SETARA(IBASE,2*NMAX,NOM3D0)
         CALL SETARA(BASE,2*NMAX,IH3D0)
      ENDIF
      CALL LCMSIX(IPTRK,'PROJECTION',1)
      CALL LCMIOF(IPTRK,'IND2T3',INDREG)
      CALL LCMIOF(IPTRK,'ZCOORD',IZ)
      CALL LCMIOF(IPTRK,'VNORF',IVNORF)
      CALL LCMIOF(IPTRK,'CMU',ICMU)
      CALL LCMIOF(IPTRK,'CMUI',ICMUI)
      CALL LCMIOF(IPTRK,'SMU',ISMU)
      CALL LCMIOF(IPTRK,'SMUI',ISMUI)
      CALL LCMIOF(IPTRK,'TMU',ITMU)
      CALL LCMIOF(IPTRK,'TMUI',ITMUI)
      CALL LCMSIX(IPTRK,'PROJECTION',2)
      DO 10 ILINE=1,NBTR
         READ(IFTRAK) IANGL,NSEG,WEIGHT,
     1        (IBASE(NOM-1+II),II=1,NSEG),
     2        (BASE(IH-1+II),II=1,NSEG)
         IF (LPJJ) THEN
*        ----------------------------
*        Self-Collision Probabilities
*        ----------------------------
            NR2SE=NSEG-2
            BASE(IHH)=0.0
            DO II=0,NR2SE
               BASE(IHH+II+1)=BASE(IHH+II)+BASE(IH+II+1)
            ENDDO
            CALL MCGPTS(SUBPJJ,NFI,NREG,M,NANI,NFUNL,NANGL,NMU,
     1           NMOD,LPS,NPJJM,NG,NGEFF,IANGL,IANGL0,NSEG,ISGNR,NZON,
     2           IBASE(NOM),IS,JS,PJJIND,WEIGHT,CPO,CAZ,BASE(IRHARM),
     3           ZMU,WZMU,BASE(ITRHAR),SIGAL,BASE(IHH),BASE(PSJ),
     4           BASE(PJJD),LPJJAN,NR2SE,NMAX,NZP,N2REG,N2SOU,DELU,
     5           IBASE(INDREG),IBASE(NOM3D),BASE(IH3D),BASE(IZ),
     6           BASE(IVNORF),BASE(ICMU),BASE(ICMUI),BASE(ISMU),
     7           BASE(ISMUI),BASE(ITMU),BASE(ITMUI),SSYM)
         ENDIF
         IF (LTMT) THEN
*        ----------------
*        Tracking Merging (angle by angle) for ACA
*        ----------------
            NTR=NTR+1
            NSE=NSE+NSEG
            LFORC=(IPANG.NE.IANGL)
            IF (LFORC) THEN
               ITEMP=IANGL
               IANGL=IPANG
               IPANG=ITEMP
            ENDIF
            IF (ILINE.EQ.NBTR) LFORC=.TRUE.
            CALL MCGTMT(NMERG,NTRTMT,NSETMT,NSEG,NSEG0,IBASE(NOM),
     1           IBASE(NOM0),WEIGHT,WEIGHT0,BASE(IH),BASE(IH0),LFORC,
     2           NTPROC)
            IF (NTPROC.EQ.0) GOTO 10
         ENDIF
         IF (LACA) THEN
*        ---------------------------------
*        Algebraic Collapsing Acceleration
*        ---------------------------------
            NR2SE=NSEG-2
            BASE(IHH)=0.0
            DO II=0,NR2SE
               BASE(IHH+II+1)=BASE(IHH+II)+BASE(IH+II+1)
            ENDDO
            CALL MCGPTA(NFI,NREG,NLONG,M,NANI,NANGL,NMU,LC,NG,NGEFF,
     1           IANGL,NSEG,IBASE(NOM),NZONA,IPERM,KM,IM,MCU,
     2           IBASE(PREV),IBASE(NEXT),WEIGHT,CPO,ZMU,WZMU,SIGAL,
     3           BASE(IXSW),BASE(IHH),BASE(DIAGQ),BASE(CQ),DIAGF,
     4           CF,BASE(IWORK),LTMT,SUBDS2,SUBDSP,SUBDSC,NR2SE,
     5           NMAX,NZP,N2REG,N2SOU,DELU,IBASE(INDREG),IBASE(NOM3D),
     6           IBASE(NOM3D0),BASE(IH3D),BASE(IH3D0),BASE(IZ),
     7           BASE(IVNORF),BASE(ICMU),BASE(ICMUI),BASE(ISMU),
     8           BASE(ISMUI),BASE(ITMU),BASE(ITMUI),N3TR,N3TRTMT,
     9           N3SE,N3SETMT,NTPROC,SSYM)
         ENDIF
 10   CONTINUE
      IF (LTMT) THEN
*     process last integration line for ACA TMT
         CALL MCGTMT(NMERG,NTRTMT,NSETMT,NSEG,NSEG0,IBASE(NOM),
     1        IBASE(NOM0),WEIGHT,WEIGHT0,BASE(IH),BASE(IH0),LFORC,
     2        NTPROC) 
         NR2SE=NSEG-2
         BASE(IHH)=0.0
         DO II=0,NR2SE
            BASE(IHH+II+1)=BASE(IHH+II)+BASE(IH+II+1)
         ENDDO
         CALL MCGPTA(NFI,NREG,NLONG,M,NANI,NANGL,NMU,LC,NG,NGEFF,
     1        IANGL,NSEG,IBASE(NOM),NZONA,IPERM,KM,IM,MCU,
     2        IBASE(PREV),IBASE(NEXT),WEIGHT,CPO,ZMU,WZMU,SIGAL,
     3        BASE(IXSW),BASE(IHH),BASE(DIAGQ),BASE(CQ),DIAGF,
     4        CF,BASE(IWORK),LTMT,SUBDS2,SUBDSP,SUBDSC,NR2SE,
     5        NMAX,NZP,N2REG,N2SOU,DELU,IBASE(INDREG),IBASE(NOM3D),
     6        IBASE(NOM3D0),BASE(IH3D),BASE(IH3D0),BASE(IZ),
     7        BASE(IVNORF),BASE(ICMU),BASE(ICMUI),BASE(ISMU),
     8        BASE(ISMUI),BASE(ITMU),BASE(ITMUI),N3TR,N3TRTMT,
     9        N3SE,N3SETMT,NTPROC,SSYM)
         CALL RLSARA(BASE(IH3D0))
         CALL RLSARA(IBASE(NOM3D0))
      ENDIF
      CALL RLSARA(BASE(IH3D))
      CALL RLSARA(IBASE(NOM3D))
      ELSE
*     REGULAR 2D OR 3D GEOMETRY
      DO 20 ILINE=1,NBTR
         READ(IFTRAK) IANGL,NSEG,WEIGHT,
     1        (IBASE(NOM-1+II),II=1,NSEG),
     2        (BASE(IH-1+II),II=1,NSEG)
         DO II=1,NSEG
            IF (IBASE(NOM-1+II).LT.0) THEN
               IBASE(NOM-1+II)=NREG-IBASE(NOM-1+II)
            ENDIF
         ENDDO
         IF (LPJJ) THEN
*        ----------------------------
*        Self-Collision Probabilities
*        ----------------------------
            IF (LPJJAN) THEN
            IF (IANGL.NE.IANGL0) THEN
               IANGL0=IANGL
               IF (NDIM.EQ.2) THEN
                  CALL MOCCHR(NDIM,NANI-1,NMU,CPO,CAZ(IANGL),
     1                 BASE(IRHARM))
               ELSE
                  CALL MOCCHR(NDIM,NANI-1,NMU,CPO(IANGL),CAZ(IANGL),
     1                 BASE(IRHARM))
               ENDIF
               DO 21 JM=1,NMOD
               DO 21 JF=1,NFUNL
               DO 21 IE=1,NMU
                  IND2=IE+(JF-1)*NMU
                  IND1=IND2+(JM-1)*NFUNL*NMU
                  BASE(ITRHAR-1+IND1)=ISGNR(JF,JM)*BASE(IRHARM-1+IND2)
*                     (IE,JF,JM)     =     (JF,JM)     (IE,JF)
 21            CONTINUE
            ENDIF
            ENDIF
            CALL MCGDS4(SUBPJJ,NSEG,NMU,LPS,NANI,NFUNL,NMOD,NG,NGEFF,
     1           WEIGHT,BASE(ITRHAR),BASE(IH),ZMU,WZMU,IBASE(NOM),
     2           BASE(IHH),NZON,NFI,NREG,NDIM,M,IS,JS,BASE(PJJD),
     3           BASE(PSJ),LPJJAN,NPJJM,PJJIND,SIGAL,1,1)
         ENDIF
         IF (LTMT) THEN
*        ----------------
*        Tracking Merging (angle by angle) for ACA
*        ----------------
            NTR=NTR+1
            NSE=NSE+NSEG
            IF (ILINE.EQ.NBTR) LFORC=.TRUE.
            CALL MCGTMT(NMERG,NTRTMT,NSETMT,NSEG,NSEG0,IBASE(NOM),
     1           IBASE(NOM0),WEIGHT,WEIGHT0,BASE(IH),BASE(IH0),LFORC,
     2           NTPROC)
            IF (NTPROC.EQ.0) GOTO 20
         ENDIF
         IF (LACA) THEN
*        ---------------------------------
*        Algebraic Collapsing Acceleration
*        ---------------------------------
            DO II=1,NSEG
               IBASE(NOM-1+II)=IPERM(IBASE(NOM-1+II))
            ENDDO
            CALL MCGDS1(SUBDS2,SUBDSP,SUBDSC,NSEG,NMUA,NANI,NG,NGEFF,
     1           WEIGHT,BASE(IH),BASE(IZMUA),BASE(IWZMUA),IBASE(NOM),
     2           BASE(IHH),NZONA,NLONG,NFI,NDIM,LC,M,KM,IM,MCU,
     3           DIAGF,BASE(DIAGQ),CF,BASE(CQ),IBASE(PREV),
     4           IBASE(NEXT),SIGAL,BASE(IXSW),BASE(IWORK))
         ENDIF
 20   CONTINUE
      IF (LTMT) THEN
*     process last integration line
         CALL MCGTMT(NMERG,NTRTMT,NSETMT,NSEG,NSEG0,IBASE(NOM),
     1        IBASE(NOM0),WEIGHT,WEIGHT0,BASE(IH),BASE(IH0),LFORC,
     2        NTPROC)
         DO II=1,NSEG
            IBASE(NOM-1+II)=IPERM(IBASE(NOM-1+II))
         ENDDO
         CALL MCGDS1(SUBDS2,SUBDSP,SUBDSC,NSEG,NMUA,NANI,NG,NGEFF,
     1        WEIGHT,BASE(IH),BASE(IZMUA),BASE(IWZMUA),IBASE(NOM),
     2        BASE(IHH),NZONA,NLONG,NFI,NDIM,LC,M,KM,IM,MCU,
     3        DIAGF,BASE(DIAGQ),CF,BASE(CQ),IBASE(PREV),
     4        IBASE(NEXT),SIGAL,BASE(IXSW),BASE(IWORK))
      ENDIF
      ENDIF
*
      IF ((LTMT).AND.(IPRINT.GT.1)) THEN
         WRITE(6,*) 'TRACKING MERGING: FROM TRACKING FILE'
         WRITE(6,*) '                 ',
     1              NTR,' TRACKS ->',
     2              NTRTMT,' EQUIVALENT TRACKS'
         WRITE(6,*) '                 ',
     1              NSE,' SEGMENTS ->',
     2              NSETMT, ' SEGMENTS'
         IF ((.NOT.LPRISM).AND.(NDIM.EQ.2))
     1    WRITE(6,*) '                 ',
     2              NMU,' POLAR ANGLES ->',
     3              ' 1 EQUIVALENT POLAR ANGLE'
         IF (LPRISM) THEN
            WRITE(6,*) 'TRACKING MERGING: 3D PRISMATIC EXTENSION'
            WRITE(6,*) '                 ',
     1                 N3TR,' TRACKS ->',
     2                 N3TRTMT,' EQUIVALENT TRACKS'
            WRITE(6,*) '                 ',
     1                 N3SE,' SEGMENTS ->',
     2                 N3SETMT, ' SEGMENTS'
         ENDIF
      ENDIF
      CALL KDRCPU(T2)
      IF (IPRINT.GT.0) THEN
         WRITE(6,100) 'SUMMATION UPON THE TRACKING FOR ACA/SCR ',(T2-T1)
      ENDIF
*---
*  FOR ACA: (IF PACA.GE.2)
*  CALCULATION OF ILU0 PRECONDITIONER FOR BICGSTAB ITERATIONS
*---
      IF (LACA) THEN
         DO II=1,NGEFF
            IG=NGIND(II)
            JPSYS=KPSYS(II)
            IF (IPRINT.GT.2) THEN
               WRITE(6,200) 'GROUP',IG
            ENDIF
            CALL MCGDS3(IPRINT,NLONG,PACA,M,SIGAL(0,II),
     1           BASE(IXSW+(II-1)*(M+1)),VA,NZONA,LC,KM,MCU,IM,JU,LC0,
     2           IM0,MCU0,DIAGF(1,II),CF(1,II),BASE(DIAGQ+(II-1)*NLONG),
     3           BASE(DIAGFR),BASE(CFR),BASE(ILUDF),BASE(ILUCF))
*     
            IF(IPRINT.GT.3) THEN
               CALL PRINAM('DIAGF ',BASE(DIAGFR),NLONG)
               CALL PRINAM('CF    ',BASE(CFR),LC)
            ENDIF
            CALL LCMPUT(JPSYS,'DIAGF$MCCG',NLONG,2,BASE(DIAGFR))
            CALL LCMPUT(JPSYS,'CF$MCCG',LC,2,BASE(CFR))
            IF (PACA.GE.2) THEN
               CALL LCMPUT(JPSYS,'ILUDF$MCCG',NLONG,2,BASE(ILUDF))
               IF (LC0.GT.0)
     1            CALL LCMPUT(JPSYS,'ILUCF$MCCG',LC0,2,BASE(ILUCF))
            ENDIF
            IF(IPRINT.GT.3) THEN
               CALL PRINAM('DIAGQ ',BASE(DIAGQ+(II-1)*NLONG),NLONG)
               CALL PRINAM('CQ    ',BASE(CQ+(II-1)*LC),LC)
            ENDIF
            CALL LCMPUT(JPSYS,'CQ$MCCG',LC,2,BASE(CQ+(II-1)*LC))
            CALL LCMPUT(JPSYS,'DIAGQ$MCCG',NLONG,2,
     1           BASE(DIAGQ+(II-1)*NLONG))
         ENDDO
         CALL KDRCPU(T3)
         IF (IPRINT.GT.0) THEN
            WRITE(6,100) 'CALCULATION OF ACA PRECONDITIONER       ',
     1                   (T3-T2)
         ENDIF
*
         IF (PACA.GE.2) THEN
            IF (LC0.GT.0) CALL RLSARA(BASE(ILUCF))
            CALL RLSARA(BASE(ILUDF))
         ENDIF
         CALL RLSARA(BASE(IWORK))
         CALL RLSARA(BASE(CFR))
         CALL RLSARA(BASE(DIAGFR))
         CALL RLSARA(BASE(DIAGQ))
         CALL RLSARA(BASE(CQ))
         CALL RLSARA(IBASE(NEXT))
         CALL RLSARA(IBASE(PREV))
         IF (LTMT) THEN
            CALL RLSARA(BASE(IH0))
            CALL RLSARA(IBASE(NOM0))
         ENDIF
         CALL RLSARA(BASE(IXSW))
      ENDIF
*----
*  FOR SCR/STIS:
*  VECTORS NORMALIZATION
*----
      IF (LPJJ) THEN
         CALL MCGDS6(NGEFF,NPJJM,NREG,BASE(PJJD),V,BASE(PJJ))
         CALL RLSARA(BASE(PJJD))
*
         DO II=1,NGEFF
            JPSYS=KPSYS(II)
            IF(IPRINT.GT.3) THEN
               CALL PRINAM('PJJ    ',BASE(PJJ+(II-1)*NREG*NPJJM),
     1                               NREG*NPJJM)
               IF (LPS.GT.0) THEN
                  CALL PRINAM('PSJ    ',BASE(PSJ+(II-1)*LPS),LPS)
               ENDIF
            ENDIF
            CALL LCMPUT(JPSYS,'PJJ$MCCG',NREG*NPJJM,2,
     1                  BASE(PJJ+(II-1)*NREG*NPJJM))
            IF (LPS.GT.0) THEN
               CALL LCMPUT(JPSYS,'PSJ$MCCG',LPS,2,BASE(PSJ+(II-1)*LPS))
            ENDIF
         ENDDO
*
         IF (LPS.GT.0) CALL RLSARA(BASE(PSJ))
         CALL RLSARA(BASE(PJJ))
         IF (LPJJAN) THEN
            CALL RLSARA(BASE(IRHARM))
            CALL RLSARA(BASE(ITRHAR))
         ENDIF
      ENDIF
*
      CALL RLSARA(BASE(IWZMUA))
      CALL RLSARA(BASE(IZMUA))
      CALL RLSARA(BASE(IHH))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(IBASE(NOM))
*
      RETURN
*
 100  FORMAT('   -->>TIME SPENT IN: ',A40,': ',F10.3,' s.')
 200  FORMAT(1X,A6,1X,I4)
      END
