*DECK MCGFCA
      SUBROUTINE MCGFCA(KPSYS,JPMACR,IPRINT,N1,NG,NGEFF,KPN,K,NREG,NANI,
     1                  NFUNL,M,LC,LFORW,PACA,KEYFLX,KEYCUR,NZON,NGIND,
     2                  NGINDV,NCONV,IPERM,IM,MCU,JU,FORM,MXACA,EPSACA,
     3                  MACFLG,REBFLG,N2,PHIOUT,PHIIN,AR,PSI,XSW,NJJ,
     4                  IJJ,IPOS,XSCAT,PI,RI,SI,ROT,API,ASI,LC0,IM0,
     5                  MCU0,COMBFLG,NPJJM,PJJ,KEYANI,PJJIND,ARSCR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Acceleration of iterations (ACA method)
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Le Tellier
*
*Parameters: input
* KPSYS   pointer array for each group properties.
* JPMACR  pointer to the macrolib LCM object ('GROUP' directory)
* IPRINT  print parameter (equal to zero for no print).
* N1      number of unknowns per group of the corrective system.
* NG      number of groups.
* NGEFF   number of groups to process.
* KPN     total number of unknowns in vectors SUNKNO and FUNKNO.
* K       total number of volumes for which specific values
*         of the neutron flux and reactions rates are required.
* NREG    number of volumes.
* NANI    scattering anisotropy (=1 for isotropic scattering).
* NFUNL   number of moments of the flux (in 2D : NFUNL=NANI*(NANI+1)/2).
* M       number of material mixtures.
* LC      dimension of profiled matrices MCU and CQ.
* LFORW   flag set to .false. to transpose the coefficient matrix.
* PACA    type of preconditioner to solve the ACA corrective system.
* KEYFLX  position of flux elements in FI vector.
* KEYCUR  position of current elements in FI vector.
* NZON    index-number of the mixture type assigned to each volume.
* NGIND   index of the groups to process.
* NGINDV  index to pass from "NGEFF format" to "NG format".
* NCONV   logical array of convergence status for each group (.TRUE. : not converged).
* IPERM   permutation array for ACA.
* IM      connection matrices.
* MCU     
* JU      used in ACA  acceleration for ilu0.
* FORM    input flux format flag (.TRUE. : same format as output flux ; .FALSE. same format as input source).
* MXACA   maximum number of iterations.
* EPSACA  convergence criterion.
* N2      number of groups to treat at the same time.
* MACFLG  multigroup cross section flag.
* REBFLG  rebalancing form flag for ACA.
* PHIIN   initial guess (for this iteration) of zonal scalar flux.
* LC0     used in ILU0-ACA acceleration.
* IM0     used in ILU0-ACA acceleration.
* MCU0    used in ILU0-ACA acceleration.
* COMBFLG flag for three-step scheme in combination wih SCR.
* NPJJM   second dimension of PJJ.
* KEYANI  'mode to l' index: l=KEYANI(nu).
* PJJIND  index for pjj(nu <- nu') modes.
*
*Parameters: input/output
* PHIOUT  zonal scalar flux.
*
*Parameters: scratch
* AR
* PSI
* XSW
* NJJ
* IJJ
* IPOS
* XSCAT
* PI
* RI
* SI
* ROT
* API
* ASI
* ARSCR
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER N1,N2,NGEFF,NG,KPSYS(NGEFF),JPMACR,IPRINT,KPN,K,NREG,NANI,
     1 NFUNL,M,LC,PACA,KEYFLX(NREG,NFUNL),KEYCUR(*),NZON(N1),
     2 NGIND(NGEFF),NGINDV(NG),IPERM(N1),IM(N1+1),MCU(LC),JU(N1),MXACA,
     3 NJJ(0:M),IJJ(0:M),IPOS(0:M),LC0,IM0(*),MCU0(*),NPJJM,
     4 KEYANI(NFUNL),PJJIND(NPJJM,2)
      REAL EPSACA,PHIIN(KPN,NG),XSCAT(0:M*NG),PJJ(NREG,NPJJM),
     1 XSW(0:M,NANI)
      DOUBLE PRECISION AR(N1,NGEFF),PSI(N1,NGEFF),PI(N1,N2),RI(N1,N2),
     1 SI(N1,N2),ROT(N1,N2),API(N1,N2),ASI(N1,N2),PHIOUT(KPN,NGEFF),
     2 ARSCR(KPN,NGEFF)
      LOGICAL LFORW,FORM,NCONV(NGEFF),MACFLG,REBFLG,COMBFLG
*---
*  LOCAL VARIABLES
*---
      REAL FLXN
      CHARACTER*12 NGTYP
      COMMON BASE(1)
*
*---
*  CONSTRUCT NGINDV (THE INVERSE OF NGIND)
*---
      CALL XDISET(NGINDV,NG,0)
      DO II=1,NGEFF
         IF (NCONV(II)) THEN
            IG=NGIND(II)
            NGINDV(IG)=II
         ENDIF
      ENDDO
*---
*  COMPUTE RESIDUAL OF THE PREVIOUS FREE ITERATION FOR RHS
*---
      DO II=1,NGEFF
         IF (NCONV(II)) THEN
            IG=NGIND(II)
            JPSYS=KPSYS(II)
            CALL LCMGET(JPSYS,'DRAGON-S0XSC',XSW)
            IF (MACFLG) THEN
               KPMACR=LCMGIL(JPMACR,IG)
               CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
               CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
               CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
               CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
            ENDIF
*           residual for ACA system
            CALL MCGFCR(IPRINT,IG,II,NG,NGEFF,KPN,N1,NREG,NANI,NFUNL,
     1           M,.TRUE.,KEYFLX,KEYCUR,NZON,NGINDV,FORM,MACFLG,PHIOUT,
     2           PHIIN,AR(1,II),XSW,NJJ,IJJ,IPOS,XSCAT,IPERM)
            IF (COMBFLG) THEN
*           residual for SCR-combined scheme
               CALL MCGFCR(IPRINT,IG,II,NG,NGEFF,KPN,N1,NREG,NANI,NFUNL,
     1              M,.FALSE.,KEYFLX,KEYCUR,NZON,NGINDV,FORM,MACFLG,
     2              PHIOUT,PHIIN,ARSCR(1,II),XSW,NJJ,IJJ,IPOS,
     3              XSCAT,KEYANI)
               IF (NANI.GT.1) THEN
               DO I=1,N1
                  J=IPERM(I)
                  IBM=NZON(J)
                  IF(IBM.GE.0) THEN
                     DO IMOD=1,NPJJM
                        INU1=PJJIND(IMOD,1)
                        INU2=PJJIND(IMOD,2)
                        IF ((INU1.EQ.1).AND.(INU2.NE.1)) THEN
                           IND2=KEYFLX(J,INU2)
                           AR(I,II)=AR(I,II)+PJJ(J,IMOD)*ARSCR(IND2,II)
                        ELSEIF ((INU2.EQ.1).AND.(INU1.NE.1)) THEN
                           IND1=KEYFLX(J,INU1)
                           AR(I,II)=AR(I,II)+PJJ(J,IMOD)*ARSCR(IND1,II)
                        ENDIF
                     ENDDO
                  ENDIF
               ENDDO
               ENDIF
            ENDIF
         ENDIF
      ENDDO
*---
*  ITERATIVE APPROACH TO SOLVE THE PRECONDITIONING SYSTEM
*---
*     ---
*     GROUP PER GROUP PROCEDURE
*     ---
      IF (MACFLG) THEN
*     MULTIGROUP REBALANCING (GAUSS-SEIDEL SCHEME)
         NGTYP='GAUSS-SEIDEL'
         CALL XDDSET(PSI,(N1*NGEFF),0.D0)
         NGFAST=NGEFF
         IF (REBFLG) THEN
*        ONLY FOR FAST GROUPS (thermal group will be treated iteratively)
         DO II=1,NGEFF
            IF (NCONV(II)) THEN
               IG=NGIND(II)
               KPMACR=LCMGIL(JPMACR,IG)
               CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
               DO IBM=1,M
                  IF (IJJ(IBM).GT.IG) THEN
                     NGFAST=II-1 ! last fast group index in NGEFF format
                     GOTO 5
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
         ENDIF
      ELSE
*     INNER ITERATION ACCELERATION
         NGTYP='   ONE-GROUP'
         NGFAST=NGEFF
      ENDIF
 5    CONTINUE
      DO II=1,NGFAST
         IF (NCONV(II)) THEN
*        infinite norm of group scalar flux
            FLXN=0.0
            DO I=1,NREG
               IND=KEYFLX(I,1)
               TEMP=REAL(ABS(PHIOUT(IND,II)))
               FLXN=MAX(TEMP,FLXN)
            ENDDO
            IF (MACFLG) THEN
*           contribution from other groups (Gauss-Seidel multigroup scheme without iterations)
               IG=NGIND(II)
               KPMACR=LCMGIL(JPMACR,IG)
               CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
               CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
               CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
               CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
               DO I=1,N1
                  J=IPERM(I)
                  IBM=NZON(J)
                  IF (IBM.GT.0) THEN
                     JG=IJJ(IBM)
                     DO 10 JND=1,NJJ(IBM)
                     IF(JG.NE.IG) THEN
                        JJ=NGINDV(JG)
                        IF(JJ.GT.0) THEN
                           AR(I,II)=AR(I,II)+XSCAT(IPOS(IBM)+JND-1)*
     1                              PSI(I,JJ)
                        ENDIF
                     ENDIF
 10                  JG=JG-1
                  ENDIF
               ENDDO
            ENDIF
*           apply preconditioner to RHS            
            IG=NGIND(II)
            JPSYS=KPSYS(II)
            CALL LCMIOF(JPSYS,'DIAGQ$MCCG',IDIAGQ)
            CALL LCMIOF(JPSYS,'CQ$MCCG',ICQ)
            IF (PACA.GE.2) THEN
               CALL LCMIOF(JPSYS,'ILUDF$MCCG',ILUDF)
               IF (PACA.LT.4) CALL LCMIOF(JPSYS,'ILUCF$MCCG',ILUCF)
               IF (PACA.GE.3) CALL LCMIOF(JPSYS,'CF$MCCG',ICF)
            ELSEIF (PACA.EQ.1) THEN
               CALL LCMIOF(JPSYS,'DIAGF$MCCG',IDIAGF)
            ENDIF
            CALL MCGPRA(LFORW,3,PACA,.TRUE.,N1,LC,IM,MCU,JU,
     1           BASE(IDIAGQ),BASE(ICQ),BASE(ILUDF),BASE(ILUCF),
     2           BASE(IDIAGF),AR(1,II),PSI(1,II),LC0,IM0,MCU0,BASE(ICF))
*           group per group BICGSTAB
            JPSYS=KPSYS(II)
            CALL LCMIOF(JPSYS,'DIAGF$MCCG',IDIAGF)
            CALL LCMIOF(JPSYS,'CF$MCCG',ICF)
            IF (PACA.GE.2) THEN
               CALL LCMIOF(JPSYS,'ILUDF$MCCG',ILUDF)
               IF (PACA.LT.4) CALL LCMIOF(JPSYS,'ILUCF$MCCG',ILUCF)
            ENDIF        
            CALL MCGABG(IPRINT,LFORW,PACA,N1,LC,EPSACA,MXACA,IM,MCU,
     1           JU,BASE(IDIAGF),BASE(ICF),BASE(ILUDF),BASE(ILUCF),
     2           AR(1,II),PSI(1,II),PI,RI,SI,ROT,API,ASI,FLXN,
     3           LC0,IM0,MCU0)
         ENDIF
      ENDDO
*
      IF ((REBFLG).AND.(IPRINT.GT.0)) THEN
         IF (NGFAST.GT.0) WRITE(6,100) NGIND(1),NGIND(NGFAST),NGTYP
      ELSE
         IF (IPRINT.GT.1) WRITE(6,100) NGIND(1),NGIND(NGFAST),NGTYP
      ENDIF
*
      IF ((REBFLG).AND.(NGFAST.LT.NGEFF)) THEN
*     ---
*     MULTIGROUP PROCEDURE
*     ---
*     THERMAL GROUPS REBALANCING
         FLXN=0.0
         NFIRST=NGFAST+1
         DO II=NFIRST,NGEFF
            IF (NCONV(II)) THEN
*              infinite norm of multigroup (thermal groups) scalar flux
               DO I=1,NREG
                  IND=KEYFLX(I,1)
                  TEMP=REAL(ABS(PHIOUT(IND,II)))
                  FLXN=MAX(TEMP,FLXN)
               ENDDO
*              contribution from fast groups to rhs
               IG=NGIND(II)
               KPMACR=LCMGIL(JPMACR,IG)
               CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
               CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
               CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
               CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
               DO I=1,N1
                  J=IPERM(I)
                  IBM=NZON(J)
                  IF (IBM.GT.0) THEN
                     JG=IJJ(IBM)
                     DO 20 JND=1,NJJ(IBM)
                        IF(JG.NE.IG) THEN
                        JJ=NGINDV(JG)
                        IF((JJ.GT.0).AND.(JJ.LE.NGFAST)) THEN
                           AR(I,II)=AR(I,II)+XSCAT(IPOS(IBM)+JND-1)*
     1                          PSI(I,JJ)
                        ENDIF
                        ENDIF
 20                  JG=JG-1
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
*        apply preconditioner to RHS
         DO II=NFIRST,NGEFF
         IF (NCONV(II)) THEN
            JPSYS=KPSYS(II)
            CALL LCMIOF(JPSYS,'DIAGQ$MCCG',IDIAGQ)
            CALL LCMIOF(JPSYS,'CQ$MCCG',ICQ)
            IF (PACA.GE.2) THEN
               CALL LCMIOF(JPSYS,'ILUDF$MCCG',ILUDF)
               IF (PACA.LT.4) CALL LCMIOF(JPSYS,'ILUCF$MCCG',ILUCF)
               IF (PACA.GE.3) CALL LCMIOF(JPSYS,'CF$MCCG',ICF)
            ELSEIF (PACA.EQ.1) THEN
               CALL LCMIOF(JPSYS,'DIAGF$MCCG',IDIAGF)
            ENDIF
            CALL MCGPRA(LFORW,3,PACA,.TRUE.,N1,LC,IM,MCU,JU,
     1           BASE(IDIAGQ),BASE(ICQ),BASE(ILUDF),BASE(ILUCF),
     2           BASE(IDIAGF),AR(1,II),PSI(1,II),LC0,IM0,MCU0,BASE(ICF))
         ENDIF
         ENDDO
*        multigroup BICGSTAB
         CALL MCGABGR(IPRINT,LFORW,PACA,N1,NG,NFIRST,NGEFF,M,LC,NGIND,
     1        NGINDV,NCONV,KPSYS,JPMACR,NZON,IPERM,IM,MCU,JU,EPSACA,
     2        MXACA,AR,PSI,PI,RI,SI,ROT,API,ASI,NJJ,IJJ,IPOS,XSCAT,FLXN,
     3        LC0,IM0,MCU0)
      ENDIF
*----
* PERFORM THE CORRECTION
*----
      IF (COMBFLG) THEN
*     -----------------------------------------------
*     ACA is combined in a three-step scheme with SCR
*     -----------------------------------------------
         DO II=1,NGEFF
         IF (NCONV(II)) THEN
            IG=NGIND(II)
            JPSYS=KPSYS(II)
            CALL LCMGET(JPSYS,'DRAGON-S0XSC',XSW)
            IF (MACFLG) THEN
               KPMACR=LCMGIL(JPMACR,IG)
               CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
               CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
               CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
               CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
            ENDIF
            CALL LCMGET(JPSYS,'PJJ$MCCG',PJJ)
            DO I=1,N1
            J=IPERM(I)
            IBM=NZON(J)
            IF(IBM.GE.0) THEN
*           Flux Correction
               IND=KEYFLX(J,1)
               PHIOUT(IND,II)=PHIOUT(IND,II)
     1                       +(1.0-PJJ(J,1)*XSW(IBM,1))*PSI(I,II)
               DO IMOD=1,NPJJM
                  INU1=PJJIND(IMOD,1)
                  INU2=PJJIND(IMOD,2)
                  IF (INU1.EQ.1) THEN
                     IND2=KEYFLX(J,INU2)
                     PHIOUT(IND,II)=PHIOUT(IND,II)
     1                    -PJJ(J,IMOD)*ARSCR(IND2,II)
                  ELSEIF (INU2.EQ.1) THEN
                     IND1=KEYFLX(J,INU1)
                     PHIOUT(IND,II)=PHIOUT(IND,II)
     1                    -PJJ(J,IMOD)*ARSCR(IND1,II)
                  ENDIF
               ENDDO
               IF (MACFLG) THEN
                  JG=IJJ(IBM)
                  DO 30 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     JJ=NGINDV(JG)
                     IF(JJ.GT.0) THEN
                        PHIOUT(IND,II)=PHIOUT(IND,II)-PJJ(J,1)*
     1                       XSCAT(IPOS(IBM)+JND-1)*PSI(I,JJ)
                     ENDIF
                  ENDIF
 30               JG=JG-1
               ENDIF
            ELSE
*           Current Correction
               IND=KEYCUR(J-NREG)
               PHIOUT(IND,II)=PHIOUT(IND,II)+PSI(I,II)
            ENDIF
            ENDDO
         ENDIF
         ENDDO
      ELSE
*     -----------------
*     ACA is used alone
*     -----------------
         DO II=1,NGEFF
         IF (NCONV(II)) THEN
            DO I=1,N1
               J=IPERM(I)
               IF(NZON(J).GE.0) THEN
*              Flux Correction
                  IND=KEYFLX(J,1)
                  PHIOUT(IND,II)=PHIOUT(IND,II)+PSI(I,II)
               ELSE
*              Current Correction
                  IND=KEYCUR(J-NREG)
                  PHIOUT(IND,II)=PHIOUT(IND,II)+PSI(I,II)
               ENDIF
            ENDDO
         ENDIF
         ENDDO
      ENDIF
*
      RETURN
*
 100  FORMAT(10X,11HACA: GROUPS,I4,3H TO,I4,2H: ,A12,7H SCHEME)
      END
