*DECK MCGFL1
      SUBROUTINE MCGFL1(SUBFFI,SUBFFA,SUBSCH,CYCLIC,KPSYS,IPRINT,IPTRK,
     1           IFTRAK,IPMACR,NDIM,K,KPN,NLONG,PHIOUT,NZON,MATALB,M,
     2           NANI,NMU,N2MAX,NANGL,NREG,NSOUT,NG,NGEFF,NGIND,S,
     3           IAAC,ISCR,LC,LFORW,PACA,EPSACC,MAXACC,NFUNL,NMOD,ISGNR,
     4           KEYFLX,KEYCUR,QFR,PHIIN,CAZ,CPO,ZMU,WZMU,V,SIGAL,LPS,
     5           NCONV,FORM,LAST,STIS,NPJJM,REBFLG,LPRISM,N2REG,N2SOU,
     6           NZP,DELU,FACSYM)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform one inner iteration of the characteristics method.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Le Tellier
*
*Parameters: input/output
* SUBFFI  isotropic flux integration subroutine.
* SUBFFA  anisotropic flux integration subroutine.
* SUBSCH  track coefficients calculation subroutine.
* CYCLIC  cyclic tracking flag.
* KPSYS   pointer array for each group properties.
* IPRINT  print parameter (equal to zero for no print).
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  tracking file unit number.
* IPMACR  pointer to the macrolib LCM object.
* NDIM    number of dimensions for the geometry.
* K       total number of volumes for which specific values
*         of the neutron flux and reactions rates are required.
* KPN     total number of unknowns per group in vector PHIIN.
* NLONG   number of spatial unknowns.
* PHIOUT  output flux vector.
* NZON    mixture-albedo index array in MCCG format.
* MATALB  albedo-mixture index array in MOCC format.
* M       number of material mixtures.
* NANI    scattering anisotropy (=1 for isotropic scattering).
* NMU     order of the polar quadrature in 2D / 1 in 3D.
* N2MAX   maximum number of elements in a track.
* NANGL   number of tracking angles in the plan.
* NREG    number of volumes.
* NG      number of groups.
* NGEFF   number of groups to process.
* NGIND   index of the groups to process.
* S       scratch.
* IAAC    no acceleration / CDD acceleration of inner iterations (0/1).
* ISCR    no acceleration / SCR acceleration of inner iterations (0/1).
* LC      dimension of profiled matrices MCU and CQ.
* LFORW   flag set to .false. to transpose the coefficient matrix.
* PACA    type of preconditioner to solve the ACA corrective system.
* EPSACC  stopping criterion for BICGSTAB in ACA resolution.
* MAXACC  maximum number of iterations allowed for BICGSTAB in ACA resolution.
* NFUNL   number of moments of the flux (in 2D : NFUNL=NANI*(NANI+1)/2).
* NMOD    second dimension of ISGNR.
* ISGNR   array of the spherical harmonics signs for the different reflections.
* KEYFLX  position of flux elements in PHIIN vector.
* KEYCUR  position of current elements in PHIIN vector.
* QFR     input source vector.
* PHIIN   input flux vector.
* CAZ     cosines of the different tracking azimuthal angles.
* CPO     cosines of the different tracking polar angles.
*         (polar quadrature in 2D / tracking angles in 3D).
* ZMU     polar quadrature set in 2D.
* WZMU    polar quadrature set in 2D.
* V       volumes.
* SIGAL   total cross-section and albedo array.
* LPS     used in scr acceleration.
* NCONV   logical array of convergence status for each group (.TRUE. : not converged).
* FORM    input flux format flag (.TRUE. : same format as output flux ; .FALSE. same format as input source).
* LAST    flag for SCR and ACA rebalancing.
* STIS    'Source term isolation' option for flux integration.
* NPJJM   number of pjj modes to store for STIS option.
* REBFLG  ACA or SCR rebalancing flag.
* LPRISM  3D prismatic extended tracking flag.
* N2REG   number of regions in the 2D tracking if LPRISM.
* N2SOU   number of external surfaces in the 2D tracking if LPRISM.
* NZP     number of z-plans if LPRISM.
* DELU    input track spacing for 3D track reconstruction if LPRISM.
* FACSYM  tracking symmetry factor for maximum track length if LPRISM.
*
*-----------------------------------------------------------------------
*
*---
* SUBROUTINES ARGUMENTS
*---
      INTEGER NGEFF,KPSYS(NGEFF),IPTRK,IPRINT,IFTRAK,IPMACR,NDIM,K,KPN,
     1 NLONG,NG,NGIND(NGEFF),NZON(NLONG),M,NANI,NMU,N2MAX,NANGL,IAAC,LC,
     2 PACA,NREG,NSOUT,ISCR,LPS,NFUNL,NMOD,ISGNR(NFUNL,NMOD),
     3 KEYFLX(NREG,NFUNL),KEYCUR(NLONG-NREG),MAXACC,STIS,NPJJM,
     4 MATALB(-NSOUT:NREG),N2REG,N2SOU,NZP
      REAL QFR(KPN,NG),PHIIN(KPN,*),CAZ(NANGL),CPO(*),ZMU(NMU),
     1 WZMU(NMU),V(NLONG),SIGAL(-6:M,NGEFF),EPSACC,DELU,FACSYM
      DOUBLE PRECISION PHIOUT(KPN,NGEFF),S(KPN,NGEFF)
      LOGICAL LFORW,CYCLIC,NCONV(NGEFF),FORM,LAST,REBFLG,LPRISM
      EXTERNAL SUBFFI,SUBFFA,SUBSCH
*---
* LOCAL VARIABLES
*---
      REAL T1,T2,T3
      INTEGER NCODE(6),SIZE,EXPT,EXP2,FLM,FLP,CYM,CYP,SSYM
      CHARACTER TEXT4*4
      LOGICAL MACFLG,COMBFLG
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
      NRNL=NANI*NANI
      CALL XDDSET(PHIOUT,KPN*NGEFF,0.D0)
*
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR
      DO ICOM=1,NCOMNT+7
         READ(IFTRAK)
      ENDDO
*
      IF (LPRISM) THEN
         CALL LCMGET(IPTRK,'NCODE',NCODE)
         IF (NCODE(6).EQ.30) THEN
            IF (NCODE(5).EQ.30) THEN
*           Z- and Z+ surfaces symmetry
               SSYM=2
            ELSE
*           Z+ symmetry
               SSYM=1
            ENDIF
         ELSE
            SSYM=0
         ENDIF
         NDIMB=3
         NMAX=(INT(FACSYM)+1)*N2MAX*(NZP+2)
      ELSE
         NDIMB=NDIM
         NMAX=N2MAX
      ENDIF
*---
* SOURCE ELEMENTS CALCULATION
*---
      CALL LCMIOF(IPTRK,'BC-REFL+TRAN',IBC)
      DO II=1,NGEFF
         IF (NCONV(II)) THEN
         IG=NGIND(II)
         IGG=IG
         IF (FORM) IGG=II
         JPSYS=KPSYS(II)
         CALL LCMIOF(JPSYS,'DRAGON-S0XSC',IXSSC)
         CALL MCGFCS(NLONG,NDIMB,NZON,QFR(1,IG),PHIIN(1,IGG),M,NANI,
     1        NFUNL,BASE(IXSSC),S(1,II),KPN,NREG,IPRINT,KEYFLX,KEYCUR,
     2        IBASE(IBC),SIGAL(-6,II),STIS)
         ENDIF
      ENDDO
*---
* FLUX INTEGRATION UPON THE TRACKING FILE
*---
      CALL KDRCPU(T1)
      IF (NANI.GT.1) THEN
         CALL SETARA(BASE,NMU*8*NRNL,ITRHAR)
         CALL SETARA(BASE,NMU*NRNL,IRHARM)
      ENDIF
      CALL SETARA(BASE,N2MAX,IH)
      CALL SETARA(IBASE,N2MAX,INOM)
      IF (CYCLIC) THEN
*     --------------------------------
*     Method of Cyclic Characteristics
*     --------------------------------
         IF (STIS.NE.-1) CALL SETARA(BASE,(2*NMU*N2MAX),EXPT)
         IF (STIS.EQ.0) THEN
            CALL SETARA(BASE,(4*NMU*N2MAX),EXP2)
         ELSE
            CALL SETARA(BASE,(2*NMU*N2MAX),EXP2)
         ENDIF
         CALL SETARA(BASE,(2*NMU*N2MAX),FLM)
         CALL SETARA(BASE,(2*NMU*N2MAX),FLP)
         CALL SETARA(BASE,(2*NMU*N2MAX),CYM)
         CALL SETARA(BASE,(2*NMU*N2MAX),CYP)
         CALL MOCFCF(SUBFFI,SUBFFA,SUBSCH,IFTRAK,NBTR,N2MAX,KPN,NREG,
     1        NSOUT,M,6,NGEFF,NANGL,NMU,NANI,NFUNL,NMOD,KEYFLX,MATALB,
     2        NCONV,SIGAL,IBASE(INOM),BASE(IH),CAZ,CPO,ZMU,WZMU,PHIOUT,
     3        S,BASE(EXPT),BASE(EXP2),ISGNR,BASE(IRHARM),BASE(ITRHAR),
     4        BASE(FLM),BASE(FLP),BASE(CYM),BASE(CYP))
         CALL RLSARA(BASE(CYP))
         CALL RLSARA(BASE(CYM))
         CALL RLSARA(BASE(FLP))
         CALL RLSARA(BASE(FLM))
         CALL RLSARA(BASE(EXP2))
         IF (STIS.NE.-1) CALL RLSARA(BASE(EXPT))
      ELSE
*     ------------------------------------
*     Method of Non-Cyclic Characteristics
*     ------------------------------------
         IF (NANI.GT.1) THEN
            CALL SETARA(BASE,(4*NMU*NMAX*NGEFF),ISTOT)
            CALL XDRSET(BASE(ISTOT),4*NMU*NMAX*NGEFF,0.0)
         ENDIF
         IF (NDIM.EQ.2) CALL SETARA(BASE,N2MAX,IHH)
         IF (STIS.EQ.0) THEN
            CALL SETARA(BASE,8*NMAX,IB)
         ELSE
            CALL SETARA(BASE,2*NMAX,IB)
         ENDIF
         IF (LPRISM) THEN
*        3D PRISMATIC GEOMETRY CONSTRUCTED FROM A 2D TRACKING
            CALL SETARA(IBASE,NMAX,INOM3D)
            CALL SETARA(BASE,NMAX,IH3D)
            CALL LCMSIX(IPTRK,'PROJECTION',1)
            CALL LCMIOF(IPTRK,'IND2T3',INDREG)
            CALL LCMIOF(IPTRK,'ZCOORD',IZ)
            CALL LCMIOF(IPTRK,'VNORF',IVNORF)
            CALL LCMIOF(IPTRK,'CMU',ICMU)
            CALL LCMIOF(IPTRK,'CMUI',ICMUI)
            CALL LCMIOF(IPTRK,'SMU',ISMU)
            CALL LCMIOF(IPTRK,'SMUI',ISMUI)
            CALL LCMIOF(IPTRK,'TMU',ITMU)
            CALL LCMIOF(IPTRK,'TMUI',ITMUI)
            CALL LCMSIX(IPTRK,'PROJECTION',2)
            CALL MCGPTF(SUBFFI,SUBFFA,SUBSCH,IFTRAK,NBTR,N2MAX,KPN,
     1           K,NREG,M,NG,NGEFF,NANGL,NMU,NANI,NFUNL,NMOD,KEYFLX,
     2           KEYCUR,NZON,NGIND,NCONV,IBASE(INOM),BASE(IH),CAZ,CPO,
     3           ZMU,WZMU,PHIOUT,S,SIGAL,BASE(IB),ISGNR,
     4           BASE(IRHARM),BASE(ITRHAR),BASE(ISTOT),BASE(IHH),NMAX,
     5           NZP,N2REG,N2SOU,DELU,IBASE(INDREG),IBASE(INOM3D),
     6           BASE(IH3D),BASE(IZ),BASE(IVNORF),BASE(ICMU),
     7           BASE(ICMUI),BASE(ISMU),BASE(ISMUI),BASE(ITMU),
     8           BASE(ITMUI),SSYM)
            CALL RLSARA(BASE(IH3D))
            CALL RLSARA(IBASE(INOM3D))
         ELSE
*        REGULAR 2D OR 3D GEOMETRY       
            CALL MCGFCF(SUBFFI,SUBFFA,SUBSCH,IFTRAK,NBTR,N2MAX,NDIM,KPN,
     1           K,NREG,M,NG,NGEFF,NANGL,NMU,NANI,NFUNL,NMOD,KEYFLX,
     2           KEYCUR,NZON,NGIND,NCONV,IBASE(INOM),BASE(IH),CAZ,CPO,
     3           ZMU,WZMU,PHIOUT,S,SIGAL,BASE(IB),ISGNR,
     4           BASE(IRHARM),BASE(ITRHAR),BASE(ISTOT),BASE(IHH))
         ENDIF
         CALL RLSARA(BASE(IB))
         IF (NDIM.EQ.2) CALL RLSARA(BASE(IHH))
         IF (NANI.GT.1) CALL RLSARA(BASE(ISTOT))
      ENDIF
      CALL RLSARA(IBASE(INOM))
      CALL RLSARA(BASE(IH))
      IF (NANI.GT.1) THEN
         CALL RLSARA(BASE(IRHARM))
         CALL RLSARA(BASE(ITRHAR))
      ENDIF
*
      IF (STIS.EQ.1) THEN
         CALL SETARA(BASE,(NREG*NPJJM),IPJJ)
         CALL LCMIOF(IPTRK,'PJJIND$MCCG',IPJJM)
         CALL MCGFST(NGEFF,KPSYS,NCONV,KPN,NLONG,NREG,NANI,NFUNL,NPJJM,
     1        KEYFLX,KEYCUR,IBASE(IPJJM),NZON,V,S,PHIOUT,BASE(IPJJ))
         CALL RLSARA(BASE(IPJJ))
      ELSEIF (STIS.EQ.-1) THEN
         CALL LCMIOF(IPTRK,'KEYANI$MCCG',IKEYAN)
         DO II=1,NGEFF
         IF (NCONV(II)) THEN
            CALL MCGFMC(KPN,NLONG,NREG,M,NANI,NFUNL,NZON,KEYFLX,KEYCUR,
     1           PHIOUT(1,II),V,S(1,II),SIGAL(0,II),IBASE(IKEYAN))
         ENDIF
         ENDDO
      ELSE
         DO II=1,NGEFF
         IF (NCONV(II)) THEN
            DO I=1,NLONG
            IF(V(I).GT.0.) THEN 
               IF (NZON(I).LT.0) THEN
                  IND=KEYCUR(I-NREG)
                  PHIOUT(IND,II)=PHIOUT(IND,II)/V(I)
               ELSE
                  DO IL=1,NFUNL
                     IND=KEYFLX(I,IL)
                     PHIOUT(IND,II)=PHIOUT(IND,II)/V(I)
                  ENDDO
               ENDIF
            ENDIF
            ENDDO
         ENDIF
         ENDDO
      ENDIF
      CALL KDRCPU(T2)
*---
* PRECONDITIONING TECHNIQUES
*---
      MACFLG=((LAST).AND.(IPMACR.GT.0).AND.((IAAC.GT.1).OR.(ISCR.GT.1)))
      REBFLG=(REBFLG.AND.MACFLG)
      COMBFLG=((IAAC.GT.0).AND.(ISCR.GT.0))
      IF (IAAC.GT.0) THEN
*     ---------------------------------
*     Algebraic Collapsing Acceleration
*     ---------------------------------
         CALL SETARA(IBASE,NG,INGINDV)
         CALL SETARA(BASE,(2*NLONG*NGEFF),ITEMP)
         CALL SETARA(BASE,(2*NLONG*NGEFF),IPSI)
         CALL SETARA(BASE,((M+1)*NANI),IXSW)
         IF (COMBFLG) THEN
            CALL SETARA(BASE,(NREG*NPJJM),IPJJ)
            CALL SETARA(BASE,(2*KPN*NGEFF),IARSCR)
            CALL LCMIOF(IPTRK,'KEYANI$MCCG',IKEYAN)
            CALL LCMIOF(IPTRK,'PJJIND$MCCG',IPJJM)
         ENDIF
*        recover connection matrices
         CALL LCMIOF(IPTRK,'IM$MCCG',IM)
         CALL LCMIOF(IPTRK,'MCU$MCCG',IMCU)
*        recover permutation array
         CALL LCMIOF(IPTRK,'PI$MCCG',IPERM)
         IF (PACA.GE.2) CALL LCMIOF(IPTRK,'JU$MCCG',JU)
         IF (PACA.EQ.3) THEN
            CALL LCMLEN(IPTRK,'MCU0$MCCG',LMCU0,ITYLCM)
            CALL LCMIOF(IPTRK,'IM0$MCCG',IM0)
            CALL LCMIOF(IPTRK,'MCU0$MCCG',MCU0)
         ELSE
            LMCU0=0
         ENDIF
         IF (MACFLG) THEN
            JPMACR=LCMGID(IPMACR,'GROUP')
            CALL SETARA(IBASE,(M+1),INJJ)
            CALL SETARA(IBASE,(M+1),IIJJ)
            CALL SETARA(IBASE,(M+1),IIPOS)
            CALL SETARA(BASE,(M*NG+1),IXSCAT)
         ENDIF
         IF (REBFLG) THEN
*        rebalancing
            MAXACC=IAAC
            SIZE=NGEFF      
         ELSE
*        inner iterations acceleration
            SIZE=1
         ENDIF
         CALL SETARA(BASE,(2*NLONG*SIZE),IPI)
         CALL SETARA(BASE,(2*NLONG*SIZE),IRI)
         CALL SETARA(BASE,(2*NLONG*SIZE),ISI)
         CALL SETARA(BASE,(2*NLONG*SIZE),IROT)
         CALL SETARA(BASE,(2*NLONG*SIZE),IAPI)
         CALL SETARA(BASE,(2*NLONG*SIZE),IASI)   
         CALL MCGFCA(KPSYS,JPMACR,IPRINT,NLONG,NG,NGEFF,KPN,K,NREG,
     1        NANI,NFUNL,M,LC,LFORW,PACA,KEYFLX,KEYCUR,NZON,NGIND,
     2        IBASE(INGINDV),NCONV,IBASE(IPERM),IBASE(IM),IBASE(IMCU),
     3        IBASE(JU),FORM,MAXACC,EPSACC,MACFLG,REBFLG,SIZE,PHIOUT,
     4        PHIIN,BASE(ITEMP),BASE(IPSI),BASE(IXSW),IBASE(INJJ),
     5        IBASE(IIJJ),IBASE(IIPOS),BASE(IXSCAT),BASE(IPI),BASE(IRI),
     6        BASE(ISI),BASE(IROT),BASE(IAPI),BASE(IASI),LMCU0,
     7        IBASE(IM0),IBASE(MCU0),COMBFLG,NPJJM,BASE(IPJJ),
     8        IBASE(IKEYAN),IBASE(IPJJM),BASE(IARSCR))
         CALL RLSARA(BASE(IASI))
         CALL RLSARA(BASE(IAPI))
         CALL RLSARA(BASE(IROT))
         CALL RLSARA(BASE(ISI))
         CALL RLSARA(BASE(IRI))
         CALL RLSARA(BASE(IPI))
         IF (MACFLG) THEN
            CALL RLSARA(BASE(IXSCAT))
            CALL RLSARA(IBASE(IIPOS))
            CALL RLSARA(IBASE(IIJJ))
            CALL RLSARA(IBASE(INJJ))
         ENDIF
         IF (COMBFLG) THEN
            CALL RLSARA(BASE(IARSCR))
            CALL RLSARA(BASE(IPJJ))
         ENDIF
         CALL RLSARA(BASE(IXSW))
         CALL RLSARA(BASE(IPSI))
         CALL RLSARA(BASE(ITEMP))
         CALL RLSARA(IBASE(INGINDV))
         CALL KDRCPU(T3)
*     ---------------------------------
      ENDIF
      NLON2=NLONG
      IF (COMBFLG) NLON2=NREG
      IF (ISCR.GT.0) THEN
*     ---------------------------------
*     Self-Collision Rebalancing Method
*     ---------------------------------
         CALL SETARA(IBASE,NG,INGINDV)
         CALL SETARA(BASE,((M+1)*NANI),IXSW)
         CALL SETARA(BASE,(4*KPN*NGEFF),ITEMP)
         CALL SETARA(BASE,(4*KPN*NGEFF),IPSI)
         CALL SETARA(BASE,(NREG*NPJJM),IPJJ)
         CALL SETARA(BASE,LPS,IPSJ)
         CALL XDDSET(BASE(IPSI),(2*KPN*NGEFF),0.D0)
         CALL LCMIOF(IPTRK,'KEYANI$MCCG',IKEYAN)
         CALL LCMIOF(IPTRK,'PJJIND$MCCG',IPJJM)
         IF (NLON2.GT.NREG) THEN
*        recover (IS,JS) arrays
            CALL LCMIOF(IPTRK,'IS$MCCG',IS)
            CALL LCMIOF(IPTRK,'JS$MCCG',JS)
         ENDIF
         IF (MACFLG) THEN
            JPMACR=LCMGID(IPMACR,'GROUP')
            CALL SETARA(IBASE,(M+1),INJJ)
            CALL SETARA(IBASE,(M+1),IIJJ)
            CALL SETARA(IBASE,(M+1),IIPOS)
            CALL SETARA(BASE,(M*NG+1),IXSCAT)
         ENDIF
         IF (REBFLG) THEN
*        rebalancing            
            MAXSCR=ISCR
         ELSE
            MAXSCR=1
         ENDIF
         CALL MCGSCR(KPSYS,JPMACR,IPRINT,NLON2,NG,NGEFF,KPN,K,NREG,NANI,
     1        NFUNL,M,LPS,KEYFLX,KEYCUR,NZON,NGIND,IBASE(INGINDV),NCONV,
     2        IBASE(IS),IBASE(JS),FORM,MAXSCR,EPSACC,MACFLG,PHIOUT,
     3        PHIIN,BASE(ITEMP),V,BASE(IPSI),BASE(IPJJ),BASE(IPSJ),
     4        BASE(IXSW),IBASE(INJJ),IBASE(IIJJ),IBASE(IIPOS),
     5        BASE(IXSCAT),NPJJM,IBASE(IKEYAN),IBASE(IPJJM))
         IF (MACFLG) THEN
            CALL RLSARA(BASE(IXSCAT))
            CALL RLSARA(IBASE(IIPOS))
            CALL RLSARA(IBASE(IIJJ))
            CALL RLSARA(IBASE(INJJ))
         ENDIF
         CALL RLSARA(BASE(IPSJ))
         CALL RLSARA(BASE(IPJJ))
         CALL RLSARA(BASE(IPSI))
         CALL RLSARA(BASE(ITEMP))
         CALL RLSARA(BASE(IXSW))
         CALL RLSARA(IBASE(INGINDV))
         CALL KDRCPU(T3)
*     ---------------------------------
      ENDIF
      IF ((IPRINT.GT.1).AND.(LAST)) THEN
         WRITE(6,100) ' FLUX INTEGRATION       ',(T2-T1)
         IF ((IAAC.GT.0).OR.(ISCR.GT.0)) THEN
            WRITE(6,100) ' ACCELERATION           ',(T3-T2)
         ENDIF
      ENDIF
      RETURN
*
 100  FORMAT('   -->>TIME SPENT IN ',A24,': ',F10.3)
      END
