*DECK MCGFLS
      SUBROUTINE MCGFLS(IMPX,IPTRK,IPMACR,NUN,K,NREG,NLONG,M,NG,NGEFF,
     1                  LC,LFORW,PACA,NZON,KEYFLX,KEYCUR,NGIND,KPSYS,
     2                  NCONV,EPSI,MAXI,FIMEM,QFR)

*
*-----------------------------------------------------------------------
*
*Purpose:
* Synthetic diffusion (ACA) flux calculation.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): I. Suslov and R. Le Tellier
*
*Parameters: input
* IMPX   print flag (equal to zero for no print).
* IPTRK   pointer to the tracking (L_TRACK signature).
* IPMACR  pointer to the macrolib LCM object.
* NUN     number of unknowns per group.
* K       number of volumes and outer surfaces.
* NREG    number of volume regions.
* NLONG   size of the corrective system.
* M       number of material mixtures.
* NG      number of groups.
* NGEFF   number of groups to process.
* LC      dimension of profiled matrices MCU and CQ.
* LFORW   flag set to .false. to transpose the coefficient matrix.
* PACA    type of preconditioner to solve the ACA corrective system.
* NZON    index-number of the mixture type assigned to each volume.
* KEYFLX  position of flux elements in FIMEM vector.
* KEYCUR  position of current elements in FIMEM vector.
* NGIND   index of the groups to process.
* NGIND   index of the groups to process.
* NCONV   array of convergence flag for each group.
* EPSI    stopping criterion for BICGSTAB in ACA resolution.
* MAXI    maximum number of iterations allowed for BICGSTAB in ACA resolution.
* QFR     input source vector.
*
*Parameters: input/output
* FIMEM   unknown vector.
*
*-----------------------------------------------------------------------
*
*----
* SUBROUTINE ARGUMENTS
*----
      INTEGER IMPX,IPTRK,IPMACR,NUN,K,NREG,M,NG,NGEFF,LC,PACA,
     1 NZON(NLONG),KEYFLX(NREG),KEYCUR(NLONG-NREG),NGIND(NGEFF),
     2 KPSYS(NGEFF),MAXI
      REAL EPSI,FIMEM(NUN,NG),QFR(NUN,NG)
      LOGICAL LFORW,NCONV(NGEFF)
*----
*  LOCAL VARIABLES
*----
      INTEGER K2,IAR,IPHI,IM,IMCU,JU,LMCU0,ITYLCM,IM0,MCU0,JPMACR,M1,
     1 INJJ,IIJJ,IIPOS,IXSCAT,II,IG,JPSYS,IDIAGF,ICF,ILUDF,ILUCF,IDIAGQ,
     2 ICQ
      INTEGER IBASE(1)
      REAL BASE
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      K2=2*NLONG
      M1=M+1
      CALL SETARA(BASE,K2,IAR)
      CALL SETARA(BASE,K2,IPHI)
*     recover connection matrices
      CALL LCMIOF(IPTRK,'IM$MCCG',IM)
      CALL LCMIOF(IPTRK,'MCU$MCCG',IMCU)
*     recover permutation array
      CALL LCMIOF(IPTRK,'PI$MCCG',IPERM)
      IF (PACA.GE.2) CALL LCMIOF(IPTRK,'JU$MCCG',JU)
      IF (PACA.EQ.3) THEN
         CALL LCMLEN(IPTRK,'MCU0$MCCG',LMCU0,ITYLCM)
         CALL LCMIOF(IPTRK,'IM0$MCCG',IM0)
         CALL LCMIOF(IPTRK,'MCU0$MCCG',MCU0)
      ELSE
         LMCU0=0
      ENDIF
      IF (IPMACR.GT.0) THEN
         JPMACR=LCMGID(IPMACR,'GROUP')
         CALL SETARA(IBASE,M1,INJJ)
         CALL SETARA(IBASE,M1,IIJJ)
         CALL SETARA(IBASE,M1,IIPOS)
         CALL SETARA(BASE,(M*NG+1),IXSCAT)
      ELSE
         JPMACR=0
      ENDIF
      DO II=1,NGEFF
      IF (NCONV(II)) THEN
         IG=NGIND(II)
         JPSYS=KPSYS(II)
         CALL LCMIOF(JPSYS,'DIAGF$MCCG',IDIAGF)
         CALL LCMIOF(JPSYS,'CF$MCCG',ICF)
         IF (PACA.GE.2) THEN
            CALL LCMIOF(JPSYS,'ILUDF$MCCG',ILUDF)
            IF (PACA.LT.4) CALL LCMIOF(JPSYS,'ILUCF$MCCG',ILUCF)
         ENDIF
         CALL LCMIOF(JPSYS,'DIAGQ$MCCG',IDIAGQ)
         CALL LCMIOF(JPSYS,'CQ$MCCG',ICQ)
         CALL MCGCDD(IMPX,JPMACR,IG,NG,NGEFF,NGIND,NCONV,M,NLONG,NUN,
     1        NREG,LC,LFORW,PACA,NZON,KEYFLX,KEYCUR,IBASE(IPERM),
     2        IBASE(IM),IBASE(IMCU),IBASE(JU),EPSI,MAXI,FIMEM(1,IG),
     3        BASE(IPHI),QFR,BASE(IAR),BASE(IDIAGQ),BASE(ICQ),
     4        BASE(IDIAGF),BASE(ICF),BASE(ILUDF),BASE(ILUCF),
     5        IBASE(INJJ),IBASE(IIJJ),IBASE(IIPOS),BASE(IXSCAT),
     6        LMCU0,IBASE(IM0),IBASE(MCU0))  
      ENDIF
      ENDDO
      IF (IPMACR.GT.0) THEN
         CALL RLSARA(BASE(IXSCAT))
         CALL RLSARA(IBASE(IIPOS))
         CALL RLSARA(IBASE(IIJJ))
         CALL RLSARA(IBASE(INJJ))
      ENDIF
      CALL RLSARA(BASE(IPHI))
      CALL RLSARA(BASE(IAR))
*     
      RETURN
      END
