*DECK MCT
      SUBROUTINE MCT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Monte-Carlo method based on NXT geometry analysis.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Le Tellier and B. Arsenault
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation type(L_MC);
*         HENTRY(2): read-only or modification type(L_TRACK);
*         HENTRY(3): read-only type(L_LIBRARY) or type(L_MACROLIB).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER    NENTRY                  
      CHARACTER  HENTRY(NENTRY)*12
      INTEGER    IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      INTEGER    IPOUT,IPTRK,IPLIB,NSTATE,IOUT,NTC,ITC,NSRCK,IKZ,KCT,
     1           NGRP,NFREG,NBMIX,NMIX,NFM,NL,NDEL,NED,ISEED,IPRINT,
     2           NBSCO,NMERGE,NGCOND
      PARAMETER (NSTATE=40,IOUT=6,NTC=3)
      INTEGER    ISTATE(NSTATE),GSTATE(NSTATE),KCHAR(3)
      DOUBLE PRECISION XYZL(2,3),KEFF,REKEFF
      CHARACTER  NAMREC*12,HSIGN*12
      LOGICAL    MODIF,LN2N
*----
*  SETARA POINTERS
*----
      INTEGER    IXST,IXSS,IXSSNN,IXNUFI,IXCHI,IXSN2N,IXSN3N,INMIX,
     1           IIMERGE,IMIXMER,IIGCR,ISCORE2,IASCORE2,IBSCORE2,
     2           INAMEAD,IXSEDI
*----
*  DYNAMIC MEMORY
*----
      INTEGER    IBASE(1)
      REAL       BASE
      COMMON     BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.NE.3) CALL XABORT('MCT: THREE PARAMETERS EXPECTED.')
*     output table in creation or modification mode
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))
     1  CALL XABORT('MCT: LCM OBJECT EXPECTED AT LHS (1).')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) 
     1  CALL XABORT('MCT: ENTRY IN CREATION '//
     2              'OR MODIFICATION MODE EXPECTED.')
      IPOUT=KENTRY(1)
      MODIF=(JENTRY(1).EQ.1)
*     tracking table in read-only mode
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2))
     1  CALL XABORT('MCT: LCM OBJECT EXPECTED AT LHS (2).')
      IF((JENTRY(2).NE.1).AND.(JENTRY(2).NE.2)) CALL XABORT('MCT: ENTR'
     1  //'Y IN READ-ONLY OR MODIFICATION MODE EXPECTED.')
      IPTRK=KENTRY(2)
      CALL LCMGET(IPTRK,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
      IF(HSIGN.NE.'L_TRACK') THEN
	 NAMREC=HENTRY(2)
         CALL XABORT('MCT: INVALID SIGNATURE FOR '//NAMREC)
      ENDIF
*     xs library in read-only mode      
      IF((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2))
     1  CALL XABORT('MCT: LCM OBJECT EXPECTED AT LHS (3).')
      IF(JENTRY(3).NE.2) 
     1  CALL XABORT('MCT: ENTRY IN READ-ONLY MODE EXPECTED (2).')
      IPLIB=KENTRY(3)
      CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
      IF(HSIGN.EQ.'L_LIBRARY') THEN
         CALL LCMSIX(IPLIB,'MACROLIB',1)
         CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
         IF(HSIGN.NE.'L_MACROLIB') THEN
	    NAMREC=HENTRY(3)
            CALL XABORT('MCT: INVALID SIGNATURE FOR '//NAMREC)
	 ENDIF
      ELSE IF(HSIGN.NE.'L_MACROLIB') THEN
	 NAMREC=HENTRY(3)
         CALL XABORT('MCT: INVALID SIGNATURE FOR '//NAMREC)
      ENDIF
*----
*  INITIALIZE OUTPUT TABLE 
*----
      IF(MODIF) THEN
         CALL LCMGET(IPOUT,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
         IF(HSIGN.NE.'L_MC') THEN
	    NAMREC=HENTRY(1)
            CALL XABORT('MCT: INVALID SIGNATURE FOR '//NAMREC)
	 ENDIF
         CALL LCMGET(IPOUT,'STATE-VECTOR',ISTATE)
      ELSE
         HSIGN='L_MC'
         READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
         CALL LCMPUT(IPOUT,'SIGNATURE',NTC,3,KCHAR)
         CALL XDISET(ISTATE,NSTATE,0)
      ENDIF
*----
*  READ INPUT PARAMETERS
*----
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',GSTATE)
      NGRP = GSTATE(1)
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',GSTATE)
      NFREG= GSTATE(1)
      NBMIX= GSTATE(4)
      CALL LCMIOF(IPTRK,'MATCOD',INMIX)
      CALL SETARA(IBASE,NFREG,IIMERGE)
      CALL SETARA(IBASE,1+NBMIX,IMIXMER)
      CALL SETARA(IBASE,NGRP,IIGCR)
      CALL MCTGET(IPOUT,NGRP,NFREG,NBMIX,IBASE(INMIX),IPRINT,
     < IBASE(IIMERGE),IBASE(IMIXMER),IBASE(IIGCR))
      CALL RLSARA(IBASE(IIGCR))
      CALL RLSARA(IBASE(IMIXMER))
      CALL RLSARA(IBASE(IIMERGE))
      CALL LCMGET(IPOUT,'STATE-VECTOR',ISTATE)
      ISEED=ISTATE(4)
      LN2N=(ISTATE(5).EQ.1)
*----
*  VALIDATE THE OPTIONS SPECIFIED FOR THE KCODE CARD
*----
      IF ((ISTATE(1).GT.0).AND.
     <    (ISTATE(2).GT.0).AND.
     <    (ISTATE(3).GT.0).AND.
     <    (ISTATE(2).LE.ISTATE(3))) THEN
         NSRCK=ISTATE(1)
         IKZ  =ISTATE(2)
         KCT  =ISTATE(3)
       ELSE
         CALL XABORT('MCT: INVALID PARAMETERS SPECIFIED FOR KCODE CARD')
      ENDIF
*----
*  RECOVER MACROSCOPIC CROSS SECTIONS.
*----
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',GSTATE)
      NGRP = GSTATE(1)
      NMIX = GSTATE(2)
      NL   = GSTATE(3)
      NFM  = GSTATE(4)
      NED  = GSTATE(5)
      NDEL = GSTATE(7)
      CALL SETARA(IBASE,2*NED,INAMEAD)
      IF(NED.GT.0) CALL LCMGET(IPLIB,'ADDXSNAME-P0',IBASE(INAMEAD))
      CALL SETARA(BASE,NMIX*NGRP,IXST)
      CALL SETARA(BASE,NMIX*NGRP*NL,IXSS)
      CALL SETARA(BASE,NGRP*NGRP*NMIX*NL,IXSSNN)
      CALL SETARA(BASE,NFM*NMIX*NGRP*(1+NDEL),IXNUFI)
      CALL SETARA(BASE,NFM*NMIX*NGRP*(1+NDEL),IXCHI)
      CALL SETARA(BASE,NMIX*NGRP,IXSN2N)
      CALL SETARA(BASE,NMIX*NGRP,IXSN3N)
      CALL SETARA(BASE,NMIX*NGRP*NED,IXSEDI)
      CALL MCTLIB(IPLIB,NMIX,NGRP,NL,NFM,NDEL,NED,IBASE(INAMEAD),LN2N,
     <            BASE(IXST),BASE(IXSS),BASE(IXSSNN),BASE(IXNUFI),
     <            BASE(IXCHI),BASE(IXSN2N),BASE(IXSN3N),BASE(IXSEDI))
*----
*  POWER ITERATION WITH THE MONTE-CARLO METHOD IN 1D/2D/3D CARTESIAN
*  GEOMETRY.
*----
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPOUT,'STATE-VECTOR',GSTATE)
      NMERGE=GSTATE(7)
      NGCOND=GSTATE(8)
      NBSCO=5+NGCOND*NL+2*NFM*(1+NDEL)+NED
      CALL SETARA(BASE,NBSCO*NMERGE*NGCOND,ISCORE2)
      CALL SETARA(BASE,2*NBSCO*NMERGE*NGCOND,IASCORE2)
      CALL SETARA(BASE,2*NBSCO*NMERGE*NGCOND,IBSCORE2)
      CALL MCTFLX(IPTRK,IPOUT,IPRINT,NMIX,NGRP,NL,NFM,NDEL,NED,
     <            IBASE(INAMEAD),BASE(IXST),BASE(IXSS),BASE(IXSSNN),
     <            BASE(IXNUFI),BASE(IXCHI),BASE(IXSN2N),BASE(IXSN3N),
     <            BASE(IXSEDI),NSRCK,IKZ,KCT,ISEED,XYZL,NBSCO,NMERGE,
     <            NGCOND,KEFF,REKEFF,BASE(ISCORE2),BASE(IASCORE2),
     <            BASE(IBSCORE2))
      CALL RLSARA(BASE(IBSCORE2))
      CALL RLSARA(BASE(IASCORE2))
      CALL RLSARA(BASE(ISCORE2))
*
      CALL RLSARA(BASE(IXSEDI))
      CALL RLSARA(BASE(IXSN3N))
      CALL RLSARA(BASE(IXSN2N))
      CALL RLSARA(BASE(IXCHI))
      CALL RLSARA(BASE(IXNUFI))
      CALL RLSARA(BASE(IXSSNN))
      CALL RLSARA(BASE(IXSS))
      CALL RLSARA(BASE(IXST))
      CALL RLSARA(IBASE(INAMEAD))
*----
*  RESET LIBRARY ON ROOT LEVEL
*----
      CALL LCMSIX(IPLIB,' ',0)
*----
*  SAVE KEFF INFORMATION ON MC OBJECT
*----
      CALL LCMPUT(IPOUT,'K-EFFECTIVE',1,2,REAL(KEFF))
      CALL LCMPUT(IPOUT,'K-EFFECTI-SD',1,2,REAL(REKEFF))
      IF(IPRINT.GT.0) WRITE(6,100) (ISTATE(ITC),ITC=1,9)
      RETURN
*
  100 FORMAT(/8H OPTIONS/8H -------/
     1 7H NSRCK ,I8,34H   (NUMBER OF HISTORIES PER CYCLE)/
     2 7H IKZ   ,I8,29H   (NUMBER OF CYCLES TO SKIP)/
     3 7H KCT   ,I8,27H   (TOTAL NUMBER OF CYCLES)/
     4 7H ISEED ,I8,45H   (INITIAL SEED FOR RANDOM NUMBER GENERATOR)/
     5 7H IN2N  ,I8,24H   (N2N PROCESSING FLAG)/
     6 7H ITALLY,I8,20H   (TYPE OF TALLIES)/
     7 7H NMERGE,I8,44H   (NUMBER OF HOMOGENIZED MIXTURES IN TALLY)/
     8 7H NGCOND,I8,40H   (NUMBER OF CONDENSED GROUPS IN TALLY)/
     9 7H NREG  ,I8,34H   (NUMBER OF REGIONS IN GEOMETRY))
      END
