*DECK MCTFLX
      SUBROUTINE MCTFLX(IPTRK,IPOUT,IPRINT,NMIX,NGRP,NL,NFM,NDEL,NED,
     <                  NAMEAD,XSTOT,XSS,XSSNN,XSNUSI,XSCHI,XSN2N,
     <                  XSN3N,XSEDI,NSRCK,IKZ,KCT,ISEED,XYZL,NBSCO,
     <                  NMERGE,NGCOND,KEFF,REKEFF,SCORE2,ASCORE2,
     <                  BSCORE2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* power iteration with the Monte Carlo method in 1D/2D/3D Cartesian
* geometry.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): B. Arsenault
*
*Parameters: input
* IPTRK   pointer to the TRACKING data structure.
* IPOUT   pointer to the MC data structure.
* IPRINT  print flag
* NMIX    number of mixtures in the geometry.
* NGRP    number of energy groups
* NL      number of Legendre orders required in the estimations
*         (NL=1 or higher).
* NFM     number of fissile isotopes
* NDEL    number of delayed precursor groups.
* NED     number of extra edit vectors.
* NAMEAD  names of these extra edits.
* XSTOT   total macroscopic cross sections for each mixture and energy
*         group
* XSS     total scattering cross sections for each mixture and energy
*         group
* XSSNN   in-group and out-of-group macroscopic transfert cross sections
*         for each mixture
* XSNUSI  the values of Nu time the fission cross sections for each
*         isotope per mixture and energy group
* XSCHI   the values of fission spectrum per isotope per mixture for
*         each energy group
* XSN2N   N2N macroscopic cross sections for each mixture and energy
*         group
* XSN3N   N3N macroscopic cross sections for each mixture and energy
*         group
* XSEDI   extra edit cross sections for each mixture and energy group
* NSRCK   number of neutrons generated per cycle
* IKZ     number of inactive cycles
* KCT     number of active cycles
* ISEED   the seed for the generation of random numbers
* NDIM    number of dimensions
* XYZL    Cartesian boundary coordinates
* NBSCO   number of macrolib-related scores
* NMERGE  number of homogenized regions
* NGCOND  number of condensed energy groups
*
*Parameters: output
* KEFF    effective multiplication factor
* REKEFF  standard deviation on the effective multiplication factor
*
*Parameters: scratch
* SCORE2
* ASCORE2
* BSCORE2
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IPOUT,IPRINT,NMIX,NGRP,NL,NFM,NDEL,NED,
     <        NAMEAD(2,NED),NSRCK,IKZ,KCT,ISEED,NBSCO,NMERGE,NGCOND
      REAL    XSTOT(NMIX,NGRP),XSS(NMIX,NGRP,NL),XSN2N(NMIX,NGRP),
     <        XSN3N(NMIX,NGRP),XSSNN(NGRP,NGRP,NMIX,NL),
     <        XSCHI(NMIX,NFM,NGRP,1+NDEL),XSNUSI(NMIX,NFM,NGRP,1+NDEL),
     <        XSEDI(NMIX,NGRP,NED),SCORE2(NBSCO,NMERGE,NGCOND)
      DOUBLE PRECISION XYZL(2,3),KEFF,REKEFF,
     <                 ASCORE2(NBSCO,NMERGE,NGCOND),
     <                 BSCORE2(NBSCO,NMERGE,NGCOND)
      CHARACTER NAMREC*12
*----
*  LOCAL VARIABLES 
*----
      INTEGER NSTATE
      PARAMETER(NSTATE=40)
      INTEGER IPARAM,IBANK1,IBANK2,ICYCLE,ILOOP,NLOOP,IDIR,IFIRST,T1,
     <        T2,MIX,ISONBR,NFREG,NFSUR,ANIS,ANGBC,NTRK,JJ,ITYPBC,ITRK,
     <        IDIRG,NBOCEL,NBUCEL,IDIAG,ISAXIS(3),NOCELL(3),NUCELL(3),
     <        ICODE(6),MXMSH,MAXREG,NBTCLS,MAXPIN,MAXMSP,MAXRSP,MXGSUR,
     <        MXGREG,NUNK,MAXMSH,NDIM,ESTATE(NSTATE),GSTATE(NSTATE),
     <        ITALLY,IGCR,IND,IOF,IGR
      REAL    ALBEDO(6),RAND,NUCALL,NULIMIT,SCORE1(3),FACT1,FACT2
      LOGICAL LKEEP
      DOUBLE PRECISION ABSC(3,2),POS(3),KCYCLE,WEIGHT,NU,SUM1,SUM2,
     <        ASCORE1(3),BSCORE1(3)
      CHARACTER HSMG*131
*----
*  SETARA POINTERS
*----
      INTEGER INGEN1,INGEN2,INMIX,IBCRT,MATALB,SURVOL,IUNFLD,DGMESH,
     <        INDEX,IDREG,IDSUR,DCMESH,ITPIN,DRAPIN,INGAR,NUCYCLE,
     <        INDGRP,IMERGE,IGAR1,IGAR2,IGAR3,IIJJ,INJJ,IIPOS
*----
*  DYNAMIC MEMORY
*----
      INTEGER IBASE(1)
      REAL BASE
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  SET THE RANDOM NUMBER GENERATOR
*----
      IFIRST=1
      IF(ISEED.EQ.0) THEN
         CALL CLETIM(T1,T2)
         ISEED=T1+T2
         DO JJ=0,MOD(ISEED,10)
           CALL RANDF(ISEED,IFIRST,RAND)
         ENDDO
      ENDIF
*----
*  RECOVER SOME BASIC NXT GEOMETRY ANALYSIS INFO AND ALLOCATE RELATED
*  MEMORY
*----     
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',GSTATE)
      NFREG    =GSTATE( 1)
      NMIX     =GSTATE( 4)
      NFSUR    =GSTATE( 5)
      IF(GSTATE(7).NE.4)
     1 CALL XABORT('MCTFLX: ONLY NXT: GEOMETRY ANALYSIS IS PERMITTED')
      ANIS     =GSTATE( 6)
      ANGBC    =ABS(GSTATE(10))
*----
*  READ THE MATERIAL NUMBER ASSOCIATED TO EACH REGION NUMBER 
*----
      CALL SETARA(IBASE,NFREG,INMIX)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(INMIX))
      CALL SETARA(IBASE,NFSUR,IBCRT)
      CALL LCMGET(IPTRK,'BC-REFL+TRAN',IBASE(IBCRT))
      CALL LCMGET(IPTRK,'ALBEDO',ALBEDO)
      CALL LCMGET(IPTRK,'ICODE',ICODE)
      CALL LCMSIX(IPTRK,'NXTRecords',1)
      CALL XDISET(ESTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'G00000001DIM',ESTATE)
      NDIM     =ESTATE( 1)
      ITYPBC   =ESTATE( 2)
      IDIRG    =ESTATE( 3)
      NBOCEL   =ESTATE( 4)
      NBUCEL   =ESTATE( 5)
      IDIAG    =ESTATE( 6)
      ISAXIS(1)=ESTATE( 7)
      ISAXIS(2)=ESTATE( 8)
      ISAXIS(3)=ESTATE( 9)
      NOCELL(1)=ESTATE(10)
      NOCELL(2)=ESTATE(11)
      NOCELL(3)=ESTATE(12)
      NUCELL(1)=ESTATE(13)
      NUCELL(2)=ESTATE(14)
      NUCELL(3)=ESTATE(15)
      MXMSH    =ESTATE(16)
      MAXREG   =ESTATE(17)
      NBTCLS   =ESTATE(18)
      MAXPIN   =ESTATE(19)
      MAXMSP   =ESTATE(20)
      MAXRSP   =ESTATE(21)
      IF(NFSUR.NE.ESTATE(22))
     1 CALL XABORT('MCTFLX: INCONSISTENT NUMBER OF OUTER SURFACES')
      IF(NFREG.NE.ESTATE(23))
     1 CALL XABORT('MCTFLX: INCONSISTENT NUMBER OF REGIONS')
      MXGSUR   =ESTATE(24)
      MXGREG   =ESTATE(25)
      NUNK=NFSUR+NFREG+1
      MAXMSH=MAX(MXMSH,MAXMSP,MAXREG)
*     surface-volumes and mixture indexes
      CALL SETARA(IBASE,NUNK  ,MATALB)
      CALL SETARA(BASE,2*NUNK,SURVOL)
      CALL LCMGET(IPTRK,'MATALB      ',IBASE(MATALB))
      CALL LCMGET(IPTRK,'SAreaRvolume',BASE(SURVOL))
*     cell index and orientation for the cells filling the geometry
      CALL SETARA(IBASE,2*NBUCEL,IUNFLD)
      NAMREC='G00000001CUF'
      CALL LCMGET(IPTRK,NAMREC,IBASE(IUNFLD))
*     global mesh for geometry
      CALL SETARA(BASE,2*(MAXMSH+2)*4,DGMESH)
      CALL XDDSET(BASE(DGMESH),(MAXMSH+2)*4,0.D0)
*
*     An offset of 2 has been used to be compatible with the definition
*     of the pin geometries where the first two values give the offset
*     of the pin.  With a cartesian geometry the array doesn't contain
*     the offests. 
      CALL NXTXYZ(IPTRK,IPRINT,NDIM,MAXMSH,NUCELL,ABSC,BASE(DGMESH))
      DO IDIR=1,NDIM
        XYZL(1,IDIR)=ABSC(IDIR,2)-ABSC(IDIR,1)
        XYZL(2,IDIR)=ABSC(IDIR,2)
      ENDDO
      CALL SETARA(IBASE,2*4*(MXGSUR+MXGREG+1),INDEX)
      CALL SETARA(IBASE,2*MXGREG,IDREG)
      CALL SETARA(IBASE,2*MXGSUR,IDSUR)
      CALL SETARA(BASE,2*2*4*(MAXMSH+2),DCMESH)
      CALL SETARA(IBASE,3*(MAXPIN+1),ITPIN)
      CALL SETARA(BASE,6*2*(MAXPIN+1),DRAPIN)
*----
*  TALLY INITIALIZATION
*----
      CALL LCMGET(IPOUT,'STATE-VECTOR',GSTATE)
      ITALLY=GSTATE(6)
      IF(ITALLY.EQ.2) THEN
	CALL SETARA(IBASE,NGRP,INDGRP)
	CALL XDISET(IBASE(INDGRP),NGRP,0)
        CALL LCMIOF(IPOUT,'REF:IMERGE',IMERGE)
        CALL LCMIOF(IPOUT,'REF:IGCOND',IGCR)
	IOF=1
	JJ=IBASE(IGCR)
	DO IND=1,NGRP
	  IF(IND.GT.JJ) THEN
	    IOF=IOF+1
	    IF(IOF.GT.NGCOND) CALL XABORT('MCTFLX: NGCOND OVERFLOW.')
	    JJ=IBASE(IGCR+IOF-1)
	  ENDIF
	  IBASE(INDGRP+IND-1)=IOF
	ENDDO
      ENDIF
*----
*  MEMORY ALLOCATION FOR THE POWER ITERATION
*----
      IPARAM = 6
      CALL SETARA(BASE,2*NSRCK*IPARAM,INGEN1)
      CALL SETARA(BASE,2*NSRCK*IPARAM,INGEN2)
      CALL SETARA(BASE,2*NSRCK,NUCYCLE)
      CALL XDRSET(BASE(INGEN1),2*NSRCK*IPARAM,0.0)
      CALL XDRSET(BASE(INGEN2),2*NSRCK*IPARAM,0.0)
*----
*  UNIFORM INITIAL ESTIMATE OF SOURCE NEUTRONS
*----
      DO ILOOP=1,NSRCK
        DO IDIR=1,NDIM
          CALL RANDF(ISEED,IFIRST,RAND)
          POS(IDIR)=RAND*(XYZL(2,IDIR)-XYZL(1,IDIR))+XYZL(1,IDIR)
        ENDDO
        IBASE(INGEN1+(ILOOP-1)*IPARAM+0) = -1
        IBASE(INGEN1+(ILOOP-1)*IPARAM+1) = -1
        BASE(INGEN1+(ILOOP-1)*IPARAM+2) = 1.0
        BASE(INGEN1+(ILOOP-1)*IPARAM+3) = POS(1)
        BASE(INGEN1+(ILOOP-1)*IPARAM+4) = POS(2)
        BASE(INGEN1+(ILOOP-1)*IPARAM+5) = POS(3)
      ENDDO
      IBANK1=NSRCK
*----
*  POWER ITERATION
*----
      KCYCLE=1.D0
      SUM1=0.0D0
      SUM2=0.0D0
      IF(ITALLY.GT.0) THEN
        CALL XDDSET(ASCORE1,3,0.0D0)
        CALL XDDSET(BSCORE1,3,0.0D0)
        IF(ITALLY.EQ.2) THEN
          CALL XDDSET(ASCORE2,NBSCO*NMERGE*NGCOND,0.0D0)
          CALL XDDSET(BSCORE2,NBSCO*NMERGE*NGCOND,0.0D0)
        ENDIF
      ENDIF
      DO ICYCLE=1,KCT
        IBANK2=0
        CALL XDRSET(SCORE1,3,0.0)
        CALL XDRSET(SCORE2,NBSCO*NMERGE*NGCOND,0.0)
        WEIGHT=KCYCLE
        KCYCLE=0.D0
        DO NTRK=1,IBANK1
          NU = BASE(INGEN1+(NTRK-1)*IPARAM  + 2)
          NLOOP=MAX(1,INT(NU/WEIGHT))
	  BASE(INGEN1+(NTRK-1)*IPARAM+2)=NU/(DBLE(NLOOP)*WEIGHT)
          DO ILOOP=1,NLOOP
*----
*  TRACK EACH NEUTRON INDIVIDUALLY
*----
            MIX    = IBASE(INGEN1+(NTRK-1)*IPARAM + 0)
            ISONBR = IBASE(INGEN1+(NTRK-1)*IPARAM + 1)
            NUCALL = BASE(INGEN1+(NTRK-1)*IPARAM + 2)
            POS(1) = BASE(INGEN1+(NTRK-1)*IPARAM + 3)
            POS(2) = BASE(INGEN1+(NTRK-1)*IPARAM + 4)
            POS(3) = BASE(INGEN1+(NTRK-1)*IPARAM + 5)
            CALL MCTRK(IPTRK,IPRINT,NFREG,NFSUR,NDIM,NMIX,ANIS,ANGBC,
     1      ITYPBC,MAXMSH,NUCELL,NBUCEL,MXGSUR,MXGREG,MAXPIN,
     2      IBASE(MATALB),BASE(SURVOL),IBASE(IBCRT),ICODE,ALBEDO,
     3      IBASE(IUNFLD),BASE(DGMESH),XYZL,IBASE(INDEX),IBASE(IDSUR),
     4      IBASE(IDREG),BASE(DCMESH),IBASE(ITPIN),BASE(DRAPIN),ISEED,
     5      NGRP,NL,NFM,NDEL,NED,IBASE(INMIX),XSTOT,XSS,XSN2N,XSN3N,
     6      XSSNN,XSNUSI,XSCHI,XSEDI,MIX,ISONBR,NUCALL,POS,ITALLY,NBSCO,
     7      NMERGE,NGCOND,IBASE(IMERGE),IBASE(INDGRP),SCORE1,SCORE2)
            IF(ISONBR.GT.0) THEN
              IBANK2=IBANK2+1
              IF(IBANK2.GT.2*NSRCK) THEN
                CALL XABORT('MCTFLX: TOO MANY NEUTRON TRACKS BEING'//
     1          ' BANKED.')
              ENDIF
              IBASE(INGEN2+(IBANK2-1)*IPARAM + 0) = MIX
              IBASE(INGEN2+(IBANK2-1)*IPARAM + 1) = ISONBR
              BASE(INGEN2+(IBANK2-1)*IPARAM + 2)  = NUCALL
              BASE(INGEN2+(IBANK2-1)*IPARAM + 3)  = POS(1)
              BASE(INGEN2+(IBANK2-1)*IPARAM + 4)  = POS(2)
              BASE(INGEN2+(IBANK2-1)*IPARAM + 5)  = POS(3)
	      BASE(NUCYCLE+IBANK2-1) = NUCALL
              KCYCLE=KCYCLE+NUCALL
            ENDIF
          ENDDO
*         END OF THE NTRK CYCLE
        ENDDO
        KCYCLE=KCYCLE/DBLE(NSRCK)
*----
*  RUSSIAN ROULETTE
*----
	IF(IBANK2.GT.NSRCK) THEN
	  CALL SORTRE(IBANK2,BASE(NUCYCLE))
	  NULIMIT=BASE(NUCYCLE+IBANK2-NSRCK)
	  IBANK1=0
          DO ITRK=1,IBANK2
            MIX    = IBASE(INGEN2+(ITRK-1)*IPARAM + 0)
            ISONBR = IBASE(INGEN2+(ITRK-1)*IPARAM + 1)
            NUCALL = BASE(INGEN2+(ITRK-1)*IPARAM + 2)
            POS(1) = BASE(INGEN2+(ITRK-1)*IPARAM + 3)
            POS(2) = BASE(INGEN2+(ITRK-1)*IPARAM + 4)
            POS(3) = BASE(INGEN2+(ITRK-1)*IPARAM + 5)
	    LKEEP=(NUCALL.GE.NULIMIT)
	    IF(.NOT.LKEEP) THEN
              CALL RANDF(ISEED,IFIRST,RAND)
	      LKEEP=RAND.LE.(NUCALL/NULIMIT)
	      NUCALL=NULIMIT
	    ENDIF
	    IF(LKEEP) THEN
	      IBANK1=IBANK1+1
              IBASE(INGEN1+(IBANK1-1)*IPARAM + 0) = MIX
              IBASE(INGEN1+(IBANK1-1)*IPARAM + 1) = ISONBR
              BASE(INGEN1+(IBANK1-1)*IPARAM + 2)  = NUCALL
              BASE(INGEN1+(IBANK1-1)*IPARAM + 3)  = POS(1)
              BASE(INGEN1+(IBANK1-1)*IPARAM + 4)  = POS(2)
              BASE(INGEN1+(IBANK1-1)*IPARAM + 5)  = POS(3)
	    ENDIF
          ENDDO
	ELSE
          IBANK1 = IBANK2
	  INGAR  = INGEN2
	  INGEN2 = INGEN1
	  INGEN1 = INGAR
	ENDIF
*----
*  K-EFFECTIVE OF THE PROBLEM WITH THE RELATIVE ERROR
*----
        IF(ICYCLE.GT.IKZ) THEN
          SUM1=SUM1+KCYCLE
          SUM2=SUM2+KCYCLE**2
          IF(ITALLY.GT.0) THEN
            DO IND=1,3
              ASCORE1(IND)=ASCORE1(IND)+SCORE1(IND)
              BSCORE1(IND)=BSCORE1(IND)+SCORE1(IND)**2
            ENDDO
            IF(ITALLY.EQ.2) THEN
              DO IND=1,NBSCO
                DO MIX=1,NMERGE
                  DO IGR=1,NGCOND
                    ASCORE2(IND,MIX,IGR)=ASCORE2(IND,MIX,IGR)+
     1              SCORE2(IND,MIX,IGR)
                    BSCORE2(IND,MIX,IGR)=BSCORE2(IND,MIX,IGR)+
     1              SCORE2(IND,MIX,IGR)**2
                  ENDDO
                ENDDO
              ENDDO
            ENDIF
          ENDIF
        ENDIF
        IF(IPRINT.GT.0) WRITE(6,1000) ICYCLE,KCYCLE
      ENDDO
      FACT1=REAL(KCT-IKZ)
      FACT2=REAL(KCT-IKZ-1)
      KEFF=SUM1/FACT1
      REKEFF=SQRT((SUM2-FACT1*KEFF**2)/FACT1/FACT2)
      WRITE(6,2000) KCT,KCYCLE,KEFF,REKEFF
      IF(ITALLY.GT.0) THEN
        DO IND=1,3
          ASCORE1(IND)=ASCORE1(IND)/FACT1
          BSCORE1(IND)=SQRT((BSCORE1(IND)-FACT1*ASCORE1(IND)**2)/FACT1/
     1    FACT2)
        ENDDO
        KEFF=ASCORE1(2)/ASCORE1(3)
        REKEFF=KEFF*(BSCORE1(2)/ASCORE1(2)-BSCORE1(3)/ASCORE1(3))
        WRITE(6,3000) KEFF,REKEFF
      ENDIF
      IF(ITALLY.EQ.2) THEN
        DO IND=1,NBSCO
          DO MIX=1,NMERGE
            DO IGR=1,NGCOND
*---- 
*     CHECK FIRST FOR 0-TALLIES IN TOTAL CROSS-SECTIONS PER MIXTURE
*---- 
              IF(ASCORE2(1,MIX,IGR).EQ.0.0D0) THEN
                WRITE(HSMG,'(28HMCPTFLX: ZERO TALLY FOR MIX ,I5,
     1          10H IN GROUP ,I5,28H. INCREASE KCODE PARAMETERS.)')
     2          MIX,IGR
                CALL XABORT(HSMG)
              ENDIF
              ASCORE2(IND,MIX,IGR)=ASCORE2(IND,MIX,IGR)/FACT1
              BSCORE2(IND,MIX,IGR)=SQRT((BSCORE2(IND,MIX,IGR)-FACT1*
     1        ASCORE2(IND,MIX,IGR)**2)/FACT1/FACT2)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
*----
*  RECONSTRUCT THE TALLY-GENERATED MACROLIB
*----
      IF(ITALLY.EQ.2) THEN
        CALL SETARA(BASE,NMERGE,IGAR1)
        CALL SETARA(BASE,NMERGE*NGCOND,IGAR2)
        CALL SETARA(BASE,NMERGE*NFM,IGAR3)
        CALL SETARA(IBASE,NMERGE,IIJJ)
        CALL SETARA(IBASE,NMERGE,INJJ)
        CALL SETARA(IBASE,NMERGE,IIPOS)
        CALL MCTOUT(IPOUT,NL,NFM,NDEL,NED,NAMEAD,NBSCO,NMERGE,NGCOND,
     1  ASCORE1,BSCORE1,ASCORE2,BSCORE2,BASE(IGAR1),BASE(IGAR2),
     2  BASE(IGAR3),IBASE(IIJJ),IBASE(INJJ),IBASE(IIPOS))
        CALL RLSARA(IBASE(IIPOS))
        CALL RLSARA(IBASE(INJJ))
        CALL RLSARA(IBASE(IIJJ))
        CALL RLSARA(BASE(IGAR3))
        CALL RLSARA(BASE(IGAR2))
        CALL RLSARA(BASE(IGAR1))
      ENDIF
*----
*  DEALLOCATE MEMORY
*----
*     POWER ITERATION RELATED
      CALL RLSARA(BASE(NUCYCLE))
      CALL RLSARA(BASE(INGEN2))
      CALL RLSARA(BASE(INGEN1))
      IF(ITALLY.EQ.2) CALL RLSARA(IBASE(INDGRP))
*
*     TRACKING RELATED
      CALL LCMSIX(IPTRK,' ',2)
      CALL RLSARA(BASE(DRAPIN))
      CALL RLSARA(IBASE(ITPIN))
      CALL RLSARA(BASE(DCMESH))
      CALL RLSARA(IBASE(IDSUR))
      CALL RLSARA(IBASE(IDREG))
      CALL RLSARA(IBASE(INDEX))
      CALL RLSARA(BASE(DGMESH))
      CALL RLSARA(IBASE(IUNFLD))
      CALL RLSARA(BASE(SURVOL))
      CALL RLSARA(IBASE(MATALB))
      CALL RLSARA(IBASE(IBCRT))
      CALL RLSARA(IBASE(INMIX))
      RETURN
*
 1000 FORMAT(' CYCLE NUMBER: ',I5,' K-EFFECTIVE CYCLE: ',F8.6)
 2000 FORMAT(/' CYCLE NUMBER: ',I5,' K-EFFECTIVE CYCLE: ',F8.6,
     <        ' K-EFFECTIVE AVERAGE: ',F8.6,
     <        ' SIGMA: ',F8.6)
 3000 FORMAT(/' VIRTUAL COLLISION ESTIMATION:  K-EFFECTIVE AVERAGE: ',
     <        F8.6,' SIGMA: ',F8.6)
      END
