*DECK MRGTRK
      SUBROUTINE MRGTRK(IPTRKR,IPTRKM,IPRINT,ITYPE ,NREGR ,
     >                  IMERGE) 
C
C----
C  1- PROGRAMME STATISTICS:
C      NAME     : MRGTRK
C      USE      : SELECT MERGE REGION FROM GEOMETRY 
C      AUTHOR   : G.MARLEAU
C      CREATED  : 2001-07-19
C
C      MODIFICATION LOG
C      --------------------------------------------------------------
C      | DATE AND INITIALS  | MOTIVATIONS
C      --------------------------------------------------------------
C      | 2001-07-19 G.M.    | MRGTRK IS FOR GENERATION OF MERGE
C      |                    | VECTOR FOR 2 EXCELL TYPE GEOMETRIES
C      |                    | BASED ON XELDRV FOR EXCELL
C      |                    | GEOMETRY ANALYSIS
C      ______________________________________________________________
C
C  2- ROUTINE PARAMETERS:
C     INPUT
C       IPTRKR : DATA STRUCTURES POINTER                  
C                TO TRACKING FILE FOR INITIAL GEOMETRY        
C               ***> INTEGER IPTRKR
C       IPTRKM : DATA STRUCTURES POINTER                  
C                TO MERGED TRACKING
C               ***> INTEGER IPTRKM
C       IPRINT : PRINT LEVEL                              
C               ***> INTEGER IPRINT
C       NANFIL : GEOMETRY FILE NAME                       
C               ***> CHARACTER*12  NAMFIL
C       ITYPE  : TYPE OF MERGE                            
C                = 0 MERGE TO NEW REGIONS           
C                = 1 MERGE TO NEW MIXTURES
C               ***> INTEGER ITYPE
C       NREGR : NUMBER OF REGION FOR WHICH SOLUTION IS 
C               CONSIDERED       
C               ***> INTEGER NREG
C     OUTPUT 
C       IMERGE : MERGING VECTOR                          
C               -> INTEGER IMERGE(0:NREGR)
C----
C
      IMPLICIT     NONE
      INTEGER      IOUT,NSTATE,NALB,MXDIM
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NSTATE=40,NALB=6,MXDIM=3,
     >             NAMSBR='MRGTRK')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      IPTRKR
      INTEGER      IPTRKM
      INTEGER      IPRINT
      INTEGER      ITYPE
      INTEGER      NREGR
      INTEGER      IMERGE(0:NREGR)
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      INDEX,ICORD,MAXDIM,MINDIM,JREMSH,
     >             MATALB,KEYMRG,KEYTR,MATAR,MINDR,
     >             MAXDR,INDXR,JREMR,ICORR,IMRGT
      INTEGER      RAN   ,NRODS ,RODS  ,NRODR ,RODR  ,NRINFO,NXRI  ,
     >             RANR  ,NRODSR,RODSR ,NRODRR,RODRR ,NRINFR,NXRIR 
C----
C  LOCAL VARIABLES
C----
      INTEGER      ISTATE(NSTATE)
      INTEGER      NDIM  ,NVOL  ,NSUR  ,NUNK  ,
     >             NDIMR ,NSURR ,NVOLR ,NUNKR
      INTEGER      NTOTCL,MAXR  ,NTOTR ,MAXRR 
      INTEGER      NSURX ,NBAN  ,NRT   ,MSROD ,MAROD ,
     >             NSURXR,NBANR ,NRTR  ,MSRODR,MARODR
      INTEGER      ITROPM,ITROPR,IST
      REAL         COTE,COTER
C----
C  READ TRACKING DATA STRUCTURE FOR MERGED GEOMETRY
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRKM,'STATE-VECTOR',ISTATE)
      ITROPM=ISTATE(7)
      IF(ITROPM .EQ. 2 ) THEN
        WRITE(IOUT,9000) (ISTATE(IST),IST=1,NSTATE)
        CALL XABORT(NAMSBR//': Hexagonal geometry is invalid')
      ENDIF
      CALL LCMSIX(IPTRKM,'EXCELL      ',1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRKM,'STATE-VECTOR',ISTATE)
      NDIM=ISTATE(1)
      NSUR=-ISTATE(2)
      NVOL=ISTATE(3)
      NUNK=ISTATE(6)
      IF(ITROPM .EQ. 1 ) THEN
        NTOTCL=ISTATE(4)
        MAXR=ISTATE(5)
        CALL SETARA(IBASE,  NUNK  ,KEYMRG)
        CALL SETARA(IBASE,  NUNK  ,MATALB)
        CALL SETARA(IBASE,  NTOTCL,MINDIM)
        CALL SETARA(IBASE,  NTOTCL,MAXDIM)
        CALL SETARA(IBASE,  NTOTCL,ICORD)
        CALL SETARA(IBASE,4*NUNK  ,INDEX)
        CALL SETARA(RBASE,  MAXR  ,JREMSH)
        CALL LCMGET(IPTRKM,'MATALB      ',IBASE(MATALB))
        CALL LCMGET(IPTRKM,'KEYMRG      ',IBASE(KEYMRG))
        CALL LCMGET(IPTRKM,'MINDIM      ',IBASE(MINDIM))
        CALL LCMGET(IPTRKM,'MAXDIM      ',IBASE(MAXDIM))
        CALL LCMGET(IPTRKM,'ICORD       ',IBASE(ICORD))
        CALL LCMGET(IPTRKM,'INDEX       ',IBASE(INDEX))
        CALL LCMGET(IPTRKM,'REMESH      ',RBASE(JREMSH))
      ELSE IF(ITROPM .EQ. 3) THEN
C----
C  CLUSTER GEOMETRY
C---- 
        NSURX=ISTATE(4)
        NBAN=ISTATE(5)
        NRT=ISTATE(7)
        MSROD=ISTATE(8)
        MAROD=ISTATE(9)
        CALL SETARA(RBASE,NBAN     ,RAN   )
        CALL SETARA(IBASE,3*NRT    ,NRODS )
        CALL SETARA(RBASE,2*NRT    ,RODS  )
        CALL SETARA(IBASE,NRT      ,NRODR )
        CALL SETARA(RBASE,MSROD*NRT,RODR  )
        CALL SETARA(IBASE,2*NBAN   ,NRINFO)
        CALL SETARA(IBASE,NRT*NBAN ,NXRI  )
        CALL LCMGET(IPTRKM,'RAN         ',RBASE(RAN)   )
        IF(NSURX .EQ. 4)
     >  CALL LCMGET(IPTRKM,'COTE        ',COTE         )
        CALL LCMGET(IPTRKM,'NRODS       ',IBASE(NRODS) )
        CALL LCMGET(IPTRKM,'RODS        ',RBASE(RODS)  )
        CALL LCMGET(IPTRKM,'NRODR       ',IBASE(NRODR) )
        CALL LCMGET(IPTRKM,'RODR        ',RBASE(RODR)  )
        CALL LCMGET(IPTRKM,'NRINFO      ',IBASE(NRINFO))
        CALL LCMGET(IPTRKM,'NXRI        ',IBASE(NXRI)  )
      ENDIF
      CALL LCMSIX(IPTRKM,'EXCELL      ',2)
C----
C  READ TRACKING DATA STRUCTURE FOR GEOMETRY TO MERGE
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRKR,'STATE-VECTOR',ISTATE)
      ITROPR=ISTATE(7)
      IF(ITROPR .EQ. 2 ) THEN
        WRITE(IOUT,9000) (ISTATE(IST),IST=1,NSTATE)
        CALL XABORT(NAMSBR//': Hexagonal geometry is invalid')
      ELSE IF(ITROPR .EQ. 3 ) THEN
        WRITE(IOUT,9000) (ISTATE(IST),IST=1,NSTATE)
        CALL XABORT(NAMSBR//': Cluster geometry is invalid')
      ENDIF
      CALL LCMSIX(IPTRKR,'EXCELL      ',1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRKR,'STATE-VECTOR',ISTATE)
      NDIMR=ISTATE(1)
      NSURR=-ISTATE(2)
      NVOLR=ISTATE(3)
      NUNKR=ISTATE(6)
      IF(ITROPR .EQ. 1 ) THEN
        NTOTR=ISTATE(4)
        MAXRR=ISTATE(5)
        CALL SETARA(IBASE,  NUNKR,KEYTR)
        CALL SETARA(IBASE,  NUNKR,MATAR)
        CALL SETARA(IBASE,  NTOTR,MINDR)
        CALL SETARA(IBASE,  NTOTR,MAXDR)
        CALL SETARA(IBASE,  NTOTR,ICORR)
        CALL SETARA(IBASE,4*NUNKR,INDXR)
        CALL SETARA(RBASE,  MAXRR,JREMR)
        CALL LCMGET(IPTRKR,'MATALB      ',IBASE(MATAR))
        CALL LCMGET(IPTRKR,'KEYMRG      ',IBASE(KEYTR))
        CALL LCMGET(IPTRKR,'MINDIM      ',IBASE(MINDR))
        CALL LCMGET(IPTRKR,'MAXDIM      ',IBASE(MAXDR))
        CALL LCMGET(IPTRKR,'ICORD       ',IBASE(ICORR))
        CALL LCMGET(IPTRKR,'INDEX       ',IBASE(INDXR))
        CALL LCMGET(IPTRKR,'REMESH      ',RBASE(JREMR))
      ELSE IF(ITROPR .EQ. 3) THEN
C----
C  CLUSTER GEOMETRY
C  IMPOSSIBLE TO MERGE WITH EXCELL CARTESIAN ASSEMBLY
C  RETURN
C---- 
        NSURXR=ISTATE(4)
        NBANR=ISTATE(5)
        NRTR=ISTATE(7)
        MSRODR=ISTATE(8)
        MARODR=ISTATE(9)
        CALL SETARA(RBASE,NBANR      ,RANR  )
        CALL SETARA(IBASE,3*NRTR     ,NRODSR)
        CALL SETARA(RBASE,2*NRTR     ,RODSR )
        CALL SETARA(IBASE,NRTR       ,NRODRR)
        CALL SETARA(RBASE,MSRODR*NRTR,RODRR )
        CALL SETARA(IBASE,2*NBANR    ,NRINFR)
        CALL SETARA(IBASE,NRTR*NBANR ,NXRIR )
        CALL LCMGET(IPTRKR,'RAN         ',RBASE(RANR)  )
        IF(NSURXR .EQ. 4)
     >  CALL LCMGET(IPTRKR,'COTE        ',COTER        )
        CALL LCMGET(IPTRKR,'NRODS       ',IBASE(NRODSR))
        CALL LCMGET(IPTRKR,'RODS        ',RBASE(RODSR) )
        CALL LCMGET(IPTRKR,'NRODR       ',IBASE(NRODRR))
        CALL LCMGET(IPTRKR,'RODR        ',RBASE(RODRR) )
        CALL LCMGET(IPTRKR,'NRINFO      ',IBASE(NRINFR))
        CALL LCMGET(IPTRKR,'NXRI        ',IBASE(NXRIR) )
      ENDIF
      CALL LCMSIX(IPTRKR,'EXCELL      ',2)
      CALL SETARA(IBASE,NVOLR,IMRGT)
      IF(ITROPR .EQ. 1) THEN
        IF(ITROPM .EQ. 1) THEN
C----
C  GENERATE MERGE INDEX BY COMPARING CURRENT GEOMETRY
C  WITH NEW GEOMETRY
C----
          CALL MRGXEL(IPRINT,ITYPE ,NREGR ,NDIM  ,NSUR  ,NVOL  ,
     >                NTOTCL,MAXR  ,NDIMR ,NSURR ,NVOLR ,NTOTR ,
     >                MAXRR ,IMERGE,
     >                IBASE(MINDIM),IBASE(MAXDIM),IBASE(KEYMRG),
     >                IBASE(INDEX) ,RBASE(JREMSH),RBASE(MATALB),
     >                IBASE(ICORD) ,IBASE(MINDR) ,IBASE(MAXDR) ,
     >                IBASE(KEYTR) ,IBASE(INDXR) ,RBASE(JREMR) ,
     >                RBASE(MATAR) ,IBASE(ICORR) ,IBASE(IMRGT) )
        ELSE IF(ITROPM .EQ. 2) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE CARTESIAN TO HEXAGONAL GEOMETRY')
        ELSE IF(ITROPM .EQ. 3) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE CARTESIAN TO CLUSTER GEOMETRY')
        ENDIF
      ELSE IF(ITROPR .EQ. 2) THEN
        IF(ITROPM .EQ. 1) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE HEXAGONAL TO CARTESIAN GEOMETRY')
        ELSE IF(ITROPM .EQ. 2) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE HEXAGONAL TO HEXAGONAL GEOMETRY')
        ELSE IF(ITROPM .EQ. 3) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE CARTESIAN TO CLUSTER GEOMETRY')
        ENDIF
      ELSE IF(ITROPR .EQ. 3) THEN
        IF(ITROPM .EQ. 1) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE CLUSTER TO CARTESIAN GEOMETRY')
        ELSE IF(ITROPM .EQ. 2) THEN
          CALL XABORT(NAMSBR//
     >    ': CANNOT MERGE CLUSTER TO HEXAGONAL GEOMETRY')
        ELSE IF(ITROPM .EQ. 3) THEN
          CALL MRGXCW(IPRINT,ITYPE ,NREGR ,NDIM  ,NSUR  ,NVOL  ,
     >                NTOTCL,MAXR  ,NDIMR ,NSURR ,NVOLR ,NTOTR ,
     >                MAXRR ,IMERGE,
     >                IBASE(MINDIM),IBASE(MAXDIM),IBASE(KEYMRG),
     >                IBASE(INDEX) ,RBASE(JREMSH),RBASE(MATALB),
     >                IBASE(ICORD) ,IBASE(MINDR) ,IBASE(MAXDR) ,
     >                IBASE(KEYTR) ,IBASE(INDXR) ,RBASE(JREMR) ,
     >                RBASE(MATAR) ,IBASE(ICORR) ,IBASE(IMRGT) )
        ENDIF
      ENDIF
      CALL RLSARA(IBASE(IMRGT))
C----
C  RELEASE REMAINING MEMORY
C----
      IF(ITROPR .EQ. 1) THEN
        CALL RLSARA(RBASE(JREMR))
        CALL RLSARA(IBASE(INDXR))
        CALL RLSARA(IBASE(ICORR))
        CALL RLSARA(IBASE(MAXDR))
        CALL RLSARA(IBASE(MINDR))
        CALL RLSARA(IBASE(MATAR))
        CALL RLSARA(IBASE(KEYTR))
      ELSE IF(ITROPR .EQ. 3) THEN
        CALL RLSARA(IBASE(NXRIR ))
        CALL RLSARA(IBASE(NRINFR))
        CALL RLSARA(RBASE(RODRR ))
        CALL RLSARA(IBASE(NRODRR))
        CALL RLSARA(RBASE(RODSR ))
        CALL RLSARA(IBASE(NRODSR))
        CALL RLSARA(RBASE(RANR  ))
      ENDIF
      IF(ITROPM .EQ. 1) THEN
        CALL RLSARA(RBASE(JREMSH))
        CALL RLSARA(IBASE(INDEX ))
        CALL RLSARA(IBASE(ICORD ))
        CALL RLSARA(IBASE(MAXDIM))
        CALL RLSARA(IBASE(MINDIM))
        CALL RLSARA(IBASE(MATALB))
        CALL RLSARA(IBASE(KEYMRG))
      ELSE IF(ITROPM .EQ. 3) THEN
        CALL RLSARA(IBASE(NXRI  ))
        CALL RLSARA(IBASE(NRINFO))
        CALL RLSARA(RBASE(RODR  ))
        CALL RLSARA(IBASE(NRODR ))
        CALL RLSARA(RBASE(RODS  ))
        CALL RLSARA(IBASE(NRODS ))
        CALL RLSARA(RBASE(RAN   ))
      ENDIF
      RETURN
C----
C  FORMAT
C----
 9000 FORMAT(' CONTENTS OF STATE VECTOR',/40I5)
 9001 FORMAT(' Type of geometry =',I10)
      END
