*DECK MRGXEL
      SUBROUTINE MRGXEL(IPRINT,ITYPE ,NREGR ,NDIMN ,NSURN ,NVOLN ,
     >                  NTOTN ,MAXRN ,NDIMR ,NSURR ,NVOLR ,NTOTR ,
     >                  MAXRR ,IMERGE,
     >                  MINDIN,MAXDIN,KEYMRN,INDEXN,REMESN,MATALN,
     >                  ICORDN,MINDIR,MAXDIR,KEYMRR,INDEXR,REMESR,
     >                  MATALR,ICORDR,IMRGTR)
C
C-------------------------    MRGXEL    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : MRGXEL
C     USE      : COMPARE 2 CARTESIAN GEOMETRIES AND FIND MERGE
C                VECTOR
C     MODIFIED : 2001-07-19
C     AUTHOR   : G. MARLEAU
C
C 2- PARAMETERS:
C    INPUT
C       IPRINT : PRINT LEVEL                              
C                -> INTEGER IPRINT
C       ITYPE  : TYPE OF MERGE                            
C                = 0 MERGE TO NEW REGIONS           
C                = 1 MERGE TO NEW MIXTURES
C                -> INTEGER ITYPE
C       NREGR  : NUMBER OF FLUX REGION IN REGERENCE GEOMETRY                       
C                -> INTEGER NREGR
C       NDINN  : NUMBER OF DIMENSIONS NEW GEOMETRY                        
C                -> INTEGER NDIMN
C       NSURN   : NUMBER OF SURFACES NEW GEOMETRY                         
C                -> INTEGER NSURN
C       NVOLN   : NUMBER OF ZONES NEW GEOMETRY                          
C                -> INTEGER NVOLN
C       NTOTN  : NUMBER OF CYLINDERS NEW GEOMETRY                        
C                -> INTEGER NTOTN
C       MAXRN  : DIMENSION OF MESH VECTOR NEW GEOMETRY                 
C                -> INTEGER MAXRN 
C       NDINR   : NUMBER OF DIMENSIONS REFERENCE GEOMETRY                        
C                -> INTEGER NDIMR
C       NSURR   : NUMBER OF SURFACES REFERENCE GEOMETRY                         
C                -> INTEGER NSURR
C       NVOLR   : NUMBER OF ZONES REFERENCE GEOMETRY                          
C                -> INTEGER NVOLR
C       NTOTR  : NUMBER OF CYLINDERS REFERENCE GEOMETRY                        
C                -> INTEGER NTOTR
C       MAXRR  : DIMENSION OF MESH VECTOR REFERENCE GEOMETRY                 
C                -> INTEGER MAXRR 
C       IMERGE : FINAL MERGING INDEX NEW GEOMETRY
C                -> INTEGER IMERGE(0:NREGR)
C       MINDIN : MIN INDEX VALUES FOR AXES NEW GEOMETRY
C                -> INTEGER MINDIN(NTOTN)
C       MAXDIN : MAX INDEX VALUES FOR AXES NEW GEOMETRY
C                -> INTEGER MAXDIN(NTOTN)
C       KEYMRN : MERGE INDEX NEW GEOMETRY
C                -> INTEGER KEYMRN(NSURN:NVOLN)
C       INDEXN : #ING OF SURFACES & ZONES NEW GEOMETRY
C                -> INTEGER INDEXN(4,NSURN:NVOLN)
C       REMESN : MESHING NEW GEOMETRY
C                -> REAL   REMESN(MAXRN)
C       MATALN : REGION MIXTURE NEW GEOMETRY
C                -> INTEGER MATALN(NSURN:NVOLN)
C       ICORDN : PRINCIPAL AXES DIRECTION FOR MESH NEW GEOMETRY
C                -> INTEGER ICORDN(NTOTN)
C       MINDIR : MIN INDEX VALUES FOR AXES REFERENCE GEOMETRY
C                -> INTEGER MINDIR(NTOTR)
C       MAXDIR : MAX INDEX VALUES FOR AXES REFERENCE GEOMETRY
C                -> INTEGER MAXDIR(NTOTR)
C       KEYMRR : MERGE INDEX REFERENCE GEOMETRY
C                -> INTEGER KEYMRR(NSURR:NVOLR)
C       INDEXR : #ING OF SURFACES & ZONES REFERENCE GEOMETRY
C                -> INTEGER INDEXR(4,NSURR:NVOLR)
C       REMESR : MESHING REFERENCE GEOMETRY
C                -> REAL    REMESR(MAXRR)
C       MATALR : REGION MIXTURE REFERENCE GEOMETRY
C                -> INTEGER MATALR(NSURR:NVOLR)
C       ICORDR : PRINCIPAL AXES DIRECTION FOR MESH REFERENCE GEOMETRY
C                -> INTEGER ICORDR(NTOTR)
C       IMRGTR : TEMPORARY MERGE VECTOR FOR REFERENCE GEOMETRY
C                -> INTEGER IMRGTR(NVOLR)
C----------------------------------------------------------------------
C
      IMPLICIT         NONE
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='MRGXEL') 
      REAL             CUTOFF
      PARAMETER       (CUTOFF=1.0E-6)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPRINT
      INTEGER          ITYPE
      INTEGER          NREGR
      INTEGER          NDIMN
      INTEGER          NSURN
      INTEGER          NVOLN
      INTEGER          NTOTN
      INTEGER          MAXRN 
      INTEGER          NDIMR
      INTEGER          NSURR
      INTEGER          NVOLR
      INTEGER          NTOTR
      INTEGER          MAXRR 
      INTEGER          IMERGE(0:NREGR)
      INTEGER          MINDIN(NTOTN)
      INTEGER          MAXDIN(NTOTN)
      INTEGER          KEYMRN(NSURN:NVOLN)
      INTEGER          INDEXN(4,NSURN:NVOLN)
      REAL             REMESN(MAXRN)
      INTEGER          MATALN(NSURN:NVOLN)
      INTEGER          ICORDN(NTOTN)
      INTEGER          MINDIR(NTOTR)
      INTEGER          MAXDIR(NTOTR)
      INTEGER          KEYMRR(NSURR:NVOLR)
      INTEGER          INDEXR(4,NSURR:NVOLR)
      REAL             REMESR(MAXRR)
      INTEGER          MATALR(NSURR:NVOLR)
      INTEGER          ICORDR(NTOTR)
      INTEGER          IMRGTR(NVOLR)
C----
C  LOCAL VARIABLES
C----
      INTEGER          IDIR,MNN,MXN,MNR,MXR,ILN,ILR,IRR,IRN,
     >                 ICLN,ICLR,IUNR,IAR,IAN,IAR1,IAR2
      REAL             POSN,POSR,CENTRN(2),CENTRR(2)
      REAL             POSOFF(3)
      REAL             RELERR
C----
C  TEST IF OUTER ASSMEBLY LIMITS ARE COMPATIBLE
C  AND SCAN CARTESIAN MESH TO FIND COMMON SURFACES
C----
      IF(NDIMN .NE. NDIMR) THEN 
        ITYPE = -1
        WRITE(IOUT,8000) NAMSBR
        WRITE(IOUT,8001) NDIMN,NDIMR
        RETURN 
      ELSE IF(NDIMR .EQ. 2) THEN 
        DO 10 IRR=1,NVOLR
          INDEXR(3,IRR)=0
 10     CONTINUE
        DO 11 IRN=1,NVOLN
          INDEXN(3,IRN)=0
 11     CONTINUE
      ENDIF
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
        IF(IPRINT .GE. 20) THEN
          WRITE(IOUT,6001) 'REFERENCE INDEX VECTOR  '
          WRITE(IOUT,6010) ((INDEXR(IDIR,IRR),IDIR=1,4),IRR=1,NVOLR)
          WRITE(IOUT,6001) 'NEW INDEX VECTOR        '
          WRITE(IOUT,6010) ((INDEXN(IDIR,IRN),IDIR=1,4),IRN=1,NVOLN)
          WRITE(IOUT,6001) 'REFERENCE MINDIN        '
          WRITE(IOUT,6011) (MINDIR(IRN),IRN=1,NTOTR)
          WRITE(IOUT,6001) 'NEW MINDIN               '
          WRITE(IOUT,6011) (MINDIN(IRN),IRN=1,NTOTN)
          WRITE(IOUT,6001) 'REFERENCE MAXDIN         '
          WRITE(IOUT,6011) (MAXDIR(IRN),IRN=1,NTOTR)
          WRITE(IOUT,6001) 'NEW MAXDIN               '
          WRITE(IOUT,6011) (MAXDIN(IRN),IRN=1,NTOTN)
          WRITE(IOUT,6001) 'REFERENCE KEYMRG         '
          WRITE(IOUT,6011) (KEYMRR(IRN),IRN=1,NVOLR)
          WRITE(IOUT,6001) 'NEW KEYMRG               '
          WRITE(IOUT,6011) (KEYMRN(IRN),IRN=1,NVOLN)
          WRITE(IOUT,6001) 'REFERENCE REMESH         '
          WRITE(IOUT,6012) (REMESR(IRN),IRN=1,MAXRN)
          WRITE(IOUT,6001) 'NEW REMESH               '
          WRITE(IOUT,6012) (REMESN(IRN),IRN=1,MAXRN)
        ENDIF
      ENDIF
      DO 100 IDIR=1,NDIMR
        MNN=MINDIN(IDIR)
        MXN=MAXDIN(IDIR)
        MNR=MINDIR(IDIR)
        MXR=MAXDIR(IDIR)
        POSOFF(IDIR)=REMESR(MNR)-REMESN(MNN) 
        RELERR=ABS(REMESN(MXN)-(REMESR(MXR)-POSOFF(IDIR)))/REMESN(MXN)
        IF(RELERR .GT. CUTOFF) THEN
          ITYPE = -2
          WRITE(IOUT,8000) NAMSBR
          WRITE(IOUT,8010) REMESN(MXN),REMESR(MXR)-POSOFF(IDIR)
        ENDIF
        REMESN(MXN)=REMESR(MXR)-POSOFF(IDIR)
        DO 110 ILN=MNN+1,MXN
          POSN=REMESN(ILN)
          DO 111 ILR=MNR+1,MXR
            POSR=REMESR(ILR)-POSOFF(IDIR)
            IF(POSR .LE. POSN*(1.0+CUTOFF)) THEN
              DO 120 IRR=1,NVOLR
                IF(INDEXR(IDIR,IRR) .EQ. ILR-1) THEN
                  INDEXR(IDIR,IRR)=-(ILN-1)
                ENDIF
 120          CONTINUE
              RELERR=ABS(POSR-POSN)/POSR
              IF (RELERR .LT. CUTOFF) GO TO 115 
            ELSE
              ITYPE = -2
              WRITE(IOUT,8000) NAMSBR
              WRITE(IOUT,8011) POSR
              RETURN
            ENDIF
 111      CONTINUE
 115      CONTINUE
 110    CONTINUE
 100  CONTINUE
      IF(NTOTN .EQ. 3) THEN 
C----
C  NO ANNULAR REGION IN NEW GEOMETRY
C  ASSOCIATE ANNULUS WITH CARTESIAN REGION
C----
        DO 130 IRR=1,NVOLR
          INDEXR(4,IRR)=0
 130    CONTINUE
      ELSE
        IF(NTOTR .EQ. 3) THEN
C----
C  NO ANNULAR REGION IN REFERENCE GEOMETRY
C  WARNING AND RETURN
C----
          ITYPE = -3
          WRITE(IOUT,8000) NAMSBR
          WRITE(IOUT,8002) 
          RETURN
        ENDIF
C----
C  ANNULAR REGION IN BOTH GEOMETRY
C  FIND ANNULUS AT IDENTICAL LOCATION
C----
        DO 140 ICLN=4,NTOTN
          IAN=ICORDN(ICLN)
          MNN=MINDIN(ICLN)
          MXN=MAXDIN(ICLN)
          CENTRN(1)=REMESN(MNN-2)
          CENTRN(2)=REMESN(MNN-1) 
          DO 141 ICLR=4,NTOTR
            IAR=ICORDR(ICLR) 
            IAR1=MOD(IAR,3)+1
            IAR2=MOD(IAR1,3)+1
            MNR=MINDIR(ICLR)
            MXR=MAXDIR(ICLR)
            CENTRR(1)=MAX(ABS(REMESR(MNR-2)),ABS(POSOFF(IAR1)),
     >                    ABS(CENTRN(1)))
            IF(CENTRR(1) .LT. CUTOFF) 
     >        CENTRR(1)=MAX(CENTRR(1),ABS(REMESN(MNN)),CUTOFF)
            CENTRR(2)=MAX(ABS(REMESR(MNR-1)),ABS(POSOFF(IAR2)),
     >                    ABS(CENTRN(2)),REMESN(MNN))
            IF(CENTRR(2) .LT. CUTOFF) 
     >        CENTRR(2)=MAX(CENTRR(2),ABS(REMESN(MNN)),CUTOFF)
            CENTRR(1)=ABS(REMESR(MNR-2)-POSOFF(IAR1)-CENTRN(1))/
     >        CENTRR(1)
            CENTRR(2)=ABS(REMESR(MNR-1)-POSOFF(IAR2)-CENTRN(2))/
     >        CENTRR(2)
            IF((IAR .EQ. IAN)          .AND.
     >         (CENTRR(1) .LE. CUTOFF) .AND.
     >         (CENTRR(2) .LE. CUTOFF)) THEN 
              DO 150 ILN=MNN,MXN
                POSN=REMESN(ILN)
                DO 151 ILR=MNR,MXR
                  POSR=REMESR(ILR)
                  IF(POSR .LE. POSN*(1.0+CUTOFF)) THEN
                    DO 152 IRR=1,NVOLR
                      IF(INDEXR(4,IRR) .EQ. ILR-1) THEN
                        INDEXR(4,IRR)=-(ILN-1)
                      ENDIF
 152                CONTINUE
                    RELERR=ABS(POSR-POSN)/POSR
                    IF (RELERR .LT. CUTOFF) GO TO 155 
                  ELSE
                    ITYPE = -3
                    WRITE(IOUT,8000) NAMSBR
                    WRITE(IOUT,8012) POSR
                   RETURN
                  ENDIF
 151            CONTINUE
 155            CONTINUE
 150          CONTINUE
              GO TO 145
            ENDIF
 141      CONTINUE
          ITYPE = -3
          WRITE(IOUT,8000) NAMSBR
          WRITE(IOUT,8003) ICLN-3
 145      CONTINUE
 140    CONTINUE
C----
C  OTHER ANNULAR REGION ASSOCIATED WITH CARTESIAN
C----
        DO 160 IRR=1,NVOLR
          IF(INDEXR(4,IRR) .GT. 0) INDEXR(4,IRR) = 0
 160    CONTINUE
      ENDIF
C----
C  COMPARE INDEXR AND INDEXN FOR NEW REGION IDENTIFICATION
C----
      IF(IPRINT .GE. 20) THEN
        WRITE(IOUT,6001) 'MODIFIED INDEX VECTOR   '
        WRITE(IOUT,6010) ((INDEXR(IDIR,IRR),IDIR=1,4),IRR=1,NVOLR)
      ENDIF
      DO 170 IRR=1,NVOLR
        IMRGTR(IRR)=0
        DO 171 IRN=1,NVOLN
          IF(-INDEXR(1,IRR) .EQ. INDEXN(1,IRN) .AND.
     >       -INDEXR(2,IRR) .EQ. INDEXN(2,IRN) .AND.
     >       -INDEXR(3,IRR) .EQ. INDEXN(3,IRN) .AND.
     >       -INDEXR(4,IRR) .EQ. INDEXN(4,IRN) ) THEN
            IMRGTR(IRR)=IRN
          ENDIF
 171    CONTINUE
 170  CONTINUE
C----
C  PRINT NEW REGION FOR REFERENCE REGION NUMBER
C----
      IF(IPRINT .GE. 20) THEN
        WRITE(IOUT,6001) 'COUPLING INDEX VECTOR   '
        WRITE(IOUT,6013) (IRR,IMRGTR(IRR),IRR=1,NVOLR)
      ENDIF
      IF(ITYPE .EQ. 0) THEN   
        DO 180 IRR=1,NVOLR
          IRN=IMRGTR(IRR)
          IMRGTR(IRR)=KEYMRN(IRN)
 180    CONTINUE
      ELSE IF(ITYPE .EQ. 1) THEN   
        DO 190 IRR=1,NVOLR
          IRN=IMRGTR(IRR)
          IMRGTR(IRR)=MATALN(IRN)
 190    CONTINUE
      ENDIF
      DO 200 IRR=1,NVOLR
        IUNR=KEYMRR(IRR)
        IF(IUNR .GE. 0 .AND. IUNR .LE. NREGR) THEN
          IMERGE(IUNR)=IMRGTR(IRR)
        ELSE
          ITYPE = -4
          WRITE(IOUT,8000) NAMSBR
          WRITE(IOUT,8004) IUNR,NREGR
          RETURN
        ENDIF
 200  CONTINUE
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) 'MERGE VECTOR            '
        WRITE(IOUT,6013) (IUNR,IMERGE(IUNR),IUNR=1,NREGR)
      ENDIF
      RETURN 
C----
C  PRINT FORMAT
C----
 6000 FORMAT(1X,'***** OUTPUT FROM ROUTINE ',A6,1X,'*****')
 6001 FORMAT(1X,A24)
 6010 FORMAT(4(2X,I8))
 6011 FORMAT(10(2X,I8))
 6012 FORMAT(5F20.10)
 6013 FORMAT(5(I8,'->',I8,:,2X,'::'))
C----
C  WARNING FORMATS
C----
 8000 FORMAT(1X,'***** WARNING IN ROUTINE ',A6,1X,'*****')
 8001 FORMAT(1X,'DIFFERENT GEOMETRY DIMENSIONS '/
     >       1X,'MERGING GEOMETRY   =',I10/ 
     >       1X,'REFERENCE GEOMETRY =',I10)
 8002 FORMAT(1X,'MERGING GEOMETRY HAS ANNULAR REGION',
     >       1X,'WHILE REFERENCE GEOMETRY IS CARTESIAN') 
 8003 FORMAT(1X,'MERGING GEOMETRY INCOMPATIBLE ANNULAR REGION',
     >       1X,'MERGING ANNULUS   =',I10) 
 8004 FORMAT(1X,'MERGING REGION    =',I10/
     >       1X,'MAXIMUM REGION    =',I10) 
 8010 FORMAT(1X,'DIFFERENT GEOMETRY UPPER LIMITS '/
     >       1X,'MERGING GEOMETRY   =',F15.5/ 
     >       1X,'REFERENCE GEOMETRY =',F15.5)
 8011 FORMAT(1X,'INVALID MERGING CARTESIAN MESH '/
     >       1X,'MERGING GEOMETRY   =',F15.5) 
 8012 FORMAT(1X,'INVALID MERGING ANNULAR MESH '/
     >       1X,'MERGING GEOMETRY   =',F15.5) 
      END
