*DECK NCR
      SUBROUTINE NCR(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
*  recover and interpolate microlib and macrolib information from one
*  or many multicompo database objects (part 1).
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert and R. Chambon
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_MICROLIB);
*         HENTRY(I): I>1 read-only type(L_MULTICOMPO) multicompo object;
*         HENTRY(NENTRY): read-only type(L_MAP) fuel-map object.
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,IOUT=6)
      CHARACTER TEXT12*12,HSMG*131,HSIGN*12,NAMDIR*12
      LOGICAL LMACRO,LCUBIC,LBASE(1),LISO,LRES
      DOUBLE PRECISION DFLOTT
      INTEGER KCHAR(3),ISTATE(NSTATE),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),LBASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.1) CALL XABORT('NCR: MINIMUM OF 2 OBJECTS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('NCR: MICRO'
     1 //'LIB LCM OBJECT EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('NCR: MICRO'
     1 //'LIB IN CREATE OR MODIFICATION MODE EXPECTED.')
      IACCS=JENTRY(1)
      IPLIB=KENTRY(1)
      IPLIB2=0
      IPMAP=0
      NGRP=0
      NMIX=0
      IF(IACCS.EQ.1) THEN
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        IF(HSIGN.NE.'L_LIBRARY') THEN
          TEXT12=HENTRY(1)
          CALL XABORT('NCR: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1    '. L_LIBRARY EXPECTED.')
        ENDIF
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NGRP=ISTATE(3)
        NMIX=ISTATE(1)
      ENDIF
      DO 10 I=2,NENTRY
        IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2)) CALL XABORT('NCR: '
     1 //'LCM OBJECTS EXPECTED AT RHS.')
        IF(JENTRY(I).NE.2) CALL XABORT('NCR: LCM OBJECTS IN READ-ONLY '
     1 //'MODE EXPECTED AT RHS.')
        CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
        IF(HSIGN.EQ.'L_LIBRARY') THEN
          IF(IPLIB2.NE.0) CALL XABORT('NCR: ONLY ONE MICROLIB EXPECTED '
     1   //'AT RHS.')
          IPLIB2=KENTRY(I)
	      GO TO 10
        ELSE IF(HSIGN.EQ.'L_MAP') THEN
          IF(I.NE.NENTRY)CALL XABORT('NCR: FUEL-MAP EXPECTED TO BE THE '
     1    //'LAST OBJECT.')
          IF(NENTRY.LT.3)CALL XABORT('NCR: MISSING MULTI-COMPO OBJECT.')
          IPMAP=KENTRY(NENTRY)
          CALL LCMLEN(IPMAP,'FLMIX',NMIX,ITYP)
        ELSE IF(HSIGN.NE.'L_MULTICOMPO') THEN
          TEXT12=HENTRY(I)
          CALL XABORT('NCR: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_MULTICOMPO EXPECTED.')
        ENDIF
   10 CONTINUE
*----
*  READ THE INPUT DATA
*----
      LMACRO=.FALSE.
      LISO=.FALSE.
      LCUBIC=.TRUE.
      LRES=.FALSE.
      B2=0.0
   20 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(1).')
   30 IF(TEXT12.EQ.'EDIT') THEN
*       READ THE PRINT INDEX.
        CALL REDGET(INDIC,IMPX,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('NCR: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT12.EQ.'NMIX') THEN
*       READ THE MAXIMUM NUMBER OF MATERIAL MIXTURES.
        IF(NMIX.NE.0) CALL XABORT('NCR: NMIX IS ALREADY DEFINED.')
        CALL REDGET(INDIC,NMIX,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT('NCR: INTEGER DATA EXPECTED(2).')
      ELSE IF(TEXT12.EQ.'MACRO') THEN
        IF(LMACRO) CALL XABORT('NCR: ONLY ONE MACRO KEYWORD EXPECTED.')
        LMACRO=.TRUE.
        IF(IACCS.NE.0) CALL XABORT('NCR: LHS EXPECTED IN CREATION.')
        IPMAC=IPLIB
        CALL LCMOP(IPLIB,'*TEMPORARY*',0,1,0)
      ELSE IF(TEXT12.EQ.'LINEAR') THEN
        LCUBIC=.FALSE.
      ELSE IF(TEXT12.EQ.'CUBIC') THEN
        LCUBIC=.TRUE.
      ELSE IF(TEXT12.EQ.'ALLX') THEN
        LISO=.TRUE.
      ELSE IF(TEXT12.EQ.'RES') THEN
        LRES=.TRUE.
      ELSE IF(TEXT12.EQ.'COMPO') THEN
        IF(NMIX.EQ.0) CALL XABORT('NCR: ZERO NUMBER OF MIXTURES.')
        IF((IACCS.EQ.0).AND.(IPLIB2.NE.0)) THEN
          CALL LCMEQU(IPLIB2,IPLIB)
          IACCS=1
        ENDIF
   40   CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(2).')
        CALL REDGET(INDIC,NITMA,FLOTT,NAMDIR,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(3).')
        I0=0
        DO 50 I=2,NENTRY
          IF(KENTRY(I).EQ.IPLIB2) GO TO 50
          IF(TEXT12.EQ.HENTRY(I)) THEN
            IPCPO=KENTRY(I)
            ITH=I
            GO TO 60
          ENDIF
   50   CONTINUE
        CALL XABORT('NCR: COMPO '//TEXT12//' NOT FOUND.')
   60   IF(IMPX.GT.0) WRITE(IOUT,310) TEXT12,NAMDIR
        CALL LCMLEN(IPCPO,NAMDIR,ILENG,ITYLCM)
        IF((ILENG.EQ.0).OR.(ITYLCM.NE.0)) THEN
          CALL LCMLIB(IPCPO)
          CALL XABORT('NCR: NO '//NAMDIR//' DIRECTORY TO STEP.')
        ENDIF
        JPCPO=LCMGID(IPCPO,NAMDIR)
        CALL LCMGET(JPCPO,'STATE-VECTOR',ISTATE)
        IF(NGRP.EQ.0) THEN
          NGRP=ISTATE(2)
        ELSE IF(NGRP.NE.ISTATE(2)) THEN
          WRITE(HSMG,'(9H NCR: THE,I4,27H-TH COMPO HAS AN INVALID NU,
     1      22HMBER OF ENERGY GROUPS.)') ITH
          CALL XABORT(HSMG)
        ENDIF
        IF(ISTATE(12).NE.2006) CALL XABORT('NCR: 2006 COMPO SPECIF EX'
     1   //'PECTED.')
        NMIL=ISTATE(1)
        NCAL=ISTATE(3)
        CALL SETARA(IBASE,NMIX,IMIXUP)
        CALL XDISET(IBASE(IMIXUP),NMIX,0)
        CALL SETARA(IBASE,NMIX,IMIXC)
        CALL SETARA(BASE,NCAL*NMIX,ITERP)
        CALL NCRDRV(IPLIB,JPCPO,LCUBIC,IACCS,NMIX,NGRP,IMPX,NMIL,NCAL,
     1   LISO,LRES,ITER,IBASE(IMIXC),BASE(ITERP),IBASE(IMIXUP))
        CALL RLSARA(BASE(ITERP))
        CALL RLSARA(IBASE(IMIXC))
        IACCS=1
        CALL NCRMAC(IPLIB,IPCPO,NGRP,NMIX,IBASE(IMIXUP),B2)
        CALL RLSARA(IBASE(IMIXUP))
        IF(ITER.EQ.0) GO TO 100
        IF(ITER.EQ.1) GO TO 40
      ELSE IF(TEXT12.EQ.'TABLE') THEN
        IF(IPMAP.EQ.0) CALL XABORT('NCR: MISSING FUEL-MAP OBJECT.')
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPMAP,'STATE-VECTOR',ISTATE)
        NB=ISTATE(1)
        NCH=ISTATE(2)
        NCOMB=ISTATE(3)
        NGRP=ISTATE(4)
        NFUEL=ISTATE(7)
        NPARMP=ISTATE(8)
        NPARM=MAX(1,NPARMP)
        IF(NCOMB.EQ.0)CALL XABORT('NCR: NUMBER OF COMBUSTION ZONES NO'
     1   //'T YET DEFINED IN THE FUEL MAP NCOMB=0.')
        IF((IACCS.EQ.0).AND.(IPLIB2.NE.0)) THEN
          CALL LCMEQU(IPLIB2,IPLIB)
          IACCS=1
        ENDIF
   70   CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(2).')
        CALL REDGET(INDIC,NITMA,FLOTT,NAMDIR,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT('NCR: CHARACTER DATA EXPECTED(3).')
        I0=0
        DO 80 I=2,NENTRY
          IF((KENTRY(I).EQ.IPLIB2).OR.(KENTRY(I).EQ.IPMAP)) GO TO 80
          IF(TEXT12.EQ.HENTRY(I)) THEN
            IPCPO=KENTRY(I)
            GO TO 90
          ENDIF
   80   CONTINUE
        CALL XABORT('NCR: COMPO '//TEXT12//' NOT FOUND.')
   90   IF(IMPX.GT.0) WRITE(IOUT,310) TEXT12,NAMDIR
        CALL LCMLEN(IPCPO,NAMDIR,ILENG,ITYLCM)
        IF((ILENG.EQ.0).OR.(ITYLCM.NE.0)) THEN
          CALL LCMLIB(IPCPO)
          CALL XABORT('NCR: NO '//NAMDIR//' DIRECTORY TO STEP.')
        ENDIF
        JPCPO=LCMGID(IPCPO,NAMDIR)
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(JPCPO,'STATE-VECTOR',ISTATE)
        IF(ISTATE(12).NE.2006) CALL XABORT('NCR: 2006 COMPO SPECIF EX'
     1   //'PECTED.')
        NMIL=ISTATE(1)
        NCAL=ISTATE(3)
        IF(ISTATE(2).NE.NGRP) CALL XABORT('NCR: WRONG NUMBER OF ENERG'
     1   //'Y GROUPS BETWEEN COMPO AND FUEL MAP.')
        CALL SETARA(IBASE,NMIX,IMIXUP)
        CALL XDISET(IBASE(IMIXUP),NMIX,0)
        CALL SETARA(IBASE,NMIX,IMIXC)
        CALL SETARA(BASE,NCAL*NMIX,ITERP)
        CALL SETARA(BASE,NCH*NB,IBURN0)
        CALL SETARA(BASE,NCH*NB,IBURN1)
        CALL SETARA(IBASE,NCH*NB,IFMIX)
        CALL SETARA(BASE,NPARM*NCH*NB,IWPAR)
        CALL SETARA(IBASE,(NPARM+1)*3,IHPAR)
        CALL SETARA(IBASE,NCH*NB,IZONED)
        CALL SETARA(IBASE,NCH,IZONEC)
        CALL NCRRGR(IPLIB,JPCPO,IPMAP,LCUBIC,IACCS,NMIX,NGRP,IMPX,NMIL,
     1         NCAL,NCH,NB,NFUEL,NCOMB,ITER,IBASE(IMIXC),BASE(ITERP),
     2         IBASE(IFMIX),BASE(IBURN0),BASE(IBURN1),BASE(IWPAR),
     3         IBASE(IZONED),IBASE(IZONEC),NPARMP,NPARM,LISO,LRES,
     4         IBASE(IHPAR),IBASE(IMIXUP))
        CALL RLSARA(IBASE(IZONEC))
        CALL RLSARA(IBASE(IZONED))
        CALL RLSARA(IBASE(IHPAR))
        CALL RLSARA(BASE(IWPAR))
        CALL RLSARA(IBASE(IFMIX))
        CALL RLSARA(BASE(IBURN1))
        CALL RLSARA(BASE(IBURN0))
        CALL RLSARA(BASE(ITERP))
        CALL RLSARA(IBASE(IMIXC))
        IACCS=1
        CALL NCRMAC(IPLIB,IPCPO,NGRP,NMIX,IBASE(IMIXUP),B2)
        CALL RLSARA(IBASE(IMIXUP))
        IF(ITER.EQ.0) GO TO 100
        IF(ITER.EQ.1) GO TO 70
      ELSE IF(TEXT12.EQ.'LEAK') THEN
        CALL REDGET(INDIC,NITMA,B2,TEXT12,DFLOTT)
        IF(INDIC.NE.2) CALL XABORT('NCR: REAL DATA EXPECTED.')
      ELSE
        CALL XABORT('NCR: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20
*----
*  PRINT THE MICROLIB STATE-VECTOR
*----
  100 IF(IMPX.GT.0) THEN
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        WRITE(IOUT,300) IMPX,(ISTATE(I),I=1,15),(ISTATE(I),I=17,23)
      ENDIF
      IF(LMACRO) THEN
        CALL LCMVAL(IPLIB,' ')
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMEQU(IPLIB,IPMAC)
        CALL LCMSIX(IPLIB,' ',2)
        CALL LCMCL(IPLIB,2)
      ENDIF
      RETURN
*
  300 FORMAT(/8H OPTIONS/8H -------/
     1 7H IMPX  ,I6,30H   (0=NO PRINT/1=SHORT/2=MORE)/
     2 7H MAXMIX,I6,31H   (MAXIMUM NUMBER OF MIXTURES)/
     3 7H NBISO ,I6,36H   (NUMBER OF ISOTOPES OR MATERIALS)/
     4 7H NGRP  ,I6,28H   (NUMBER OF ENERGY GROUPS)/
     5 7H NL    ,I6,30H   (NUMBER OF LEGENDRE ORDERS)/
     6 7H ITRANC,I6,45H   (0=NO TRANSPORT CORRECTION/1=APOLLO TYPE/2,
     7 57H=RECOVER FROM LIBRARY/3=WIMS-D TYPE/4=LEAKAGE CORRECTION)/
     8 7H IPROB ,I6,23H   (0=DIRECT/1=ADJOINT)/
     9 7H ITIME ,I6,28H   (1=STEADY-STATE/2=PROMPT)/
     1 7H NLIB  ,I6,32H   (NUMBER OF SETS OF LIBRARIES)/
     2 7H NGF   ,I6,48H   (NUMBER OF FAST GROUP WITHOUT SELF-SHIELDING)/
     3 7H IGRMAX,I6,41H   (LAST GROUP INDEX WITH SELF-SHIELDING)/
     4 7H NDEPL ,I6,33H   (NUMBER OF DEPLETING ISOTOPES)/
     5 7H NCOMB ,I6,33H   (NUMBER OF DEPLETING MIXTURES)/
     6 7H NEDMAC,I6,34H   (NUMBER OF CROSS SECTION EDITS)/
     7 7H NBMIX ,I6,23H   (NUMBER OF MIXTURES)/
     8 7H NRES  ,I6,40H   (NUMBER OF SETS OF RESONANT MIXTURES)/
     9 7H IPROC ,I6,48H   (-1=SKIP LIBRARY PROCESSING/0=DILUTION INTERP,
     1 48HOLATION/1=USE PHYSICAL TABLES/2=BUILD A DRAGLIB//17X,
     2 55H3=COMPUTE CALENDF TABLES/4=COMPUTE SLOWING-DOWN TABLES)/
     3 7H IMAC  ,I6,45H   (0=DO NOT/1=DO BUILD AN EMBEDDED MACROLIB)/
     4 7H NDEL  ,I6,31H   (NUMBER OF PRECURSOR GROUPS)/
     5 7H NFISS ,I6,31H   (NUMBER OF FISSILE ISOTOPES)/
     6 7H ISOADD,I6,37H   (0=COMPLETE BURNUP CHAIN/1=DO NOT)/
     7 7H MAXISM,I6,40H   (MAX. NUMBER OF ISOTOPES PER MIXTURE)/
     8 7H IPRECI,I6,34H   (CALENDF ACCURACY FLAG:1/2/3/4))
  310 FORMAT(/27H NCR: INTERPOLATING COMPO ',A12,18H' FROM DIRECTORY ',
     1 A12,2H'.)
      END
