*DECK NCRDRV
      SUBROUTINE NCRDRV(IPLIB,IPCPO,LCUBIC,IACCS,NMIX,NGRP,IMPX,NMIL,
     1 NCAL,LXS,LRES,ITER,MIXC,TERP,MIXUPD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover and/or interpolate multicompo information and store them
* in a new or existing microlib. Use user-defined global and local
* parameters.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert and R. Chambon
*
*Parameters: input
* IPLIB   address of the microlib LCM object.
* IPCPO   address of the multidimensional compo object.
* LCUBIC  =.TRUE.: cubic Ceschino interpolation; =.FALSE: linear
*         Lagrange interpolation.
* IACCS   =0 microlib is created; =2 ... is updated.
* NMIX    maximum number of material mixtures in the microlib.
* NGRP    number of energy groups.
* IMPX    print parameter (equal to zero for no print).
* NMIL    number of material mixtures in the compo.
* NCAL    number of elementary calculations in the compo.
* LXS     =.true. if keyword 'ALLX' is specified
* LRES    =.true. if the interpolation is done for the residual
*         isotope only
*
*Parameters: output
* ITER    completion flag (=0: all over; =1: use another compo).
* MIXUPD  tag for mixture which will be updated
*
*Parameters: scratch
* MIXC
* TERP
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IPCPO,IACCS,NMIX,NGRP,IMPX,NMIL,NCAL,ITER
      INTEGER MIXC(NMIX),MIXUPD(NMIX)
      REAL TERP(NCAL,NMIX)
      LOGICAL LCUBIC,LXS,LRES
*----
*  LOCAL VARIABLES
*----
      PARAMETER (REPS=1.0E-4,IOUT=6,MAXPAR=50,MAXLIN=50,MAXVAL=200,
     1 NSTATE=40,MAXISD=500)
      CHARACTER TEXT12*12,PARKEY(MAXPAR)*12,PARFMT(MAXPAR)*8,
     1 PARKEL(MAXPAR)*12,HSMG*131,COMMEN(MAXLIN)*80,VALH(MAXPAR)*12,
     2 RECNAM*12,VCHAR(MAXVAL)*12,HISO(MAXISD)*12
      INTEGER ISTATE(NSTATE),VALI(MAXPAR),NVALUE(MAXPAR),VINTE(MAXVAL),
     1 MUPLET(2*MAXPAR),MUTYPE(2*MAXPAR)
      DOUBLE PRECISION DFLOTT
      REAL VALR(2*MAXPAR,2),VREAL(MAXVAL),CONC(MAXISD)
      LOGICAL LISO,LDELTA
      COMMON BASE(1)
*----
*  RECOVER TABLE-OF-CONTENT INFORMATION FOR THE COMPO.
*----
      CALL LCMGET(IPCPO,'STATE-VECTOR',ISTATE)
      NPAR=ISTATE(5)
      NLOC=ISTATE(6)
      NCOMLI=ISTATE(10)
      CALL LCMGTC(IPCPO,'COMMENT',80,NCOMLI,COMMEN)
      IF(NPAR.GT.0) THEN
         CALL LCMSIX(IPCPO,'GLOBAL',1)
         CALL LCMGTC(IPCPO,'PARKEY',12,NPAR,PARKEY)
         CALL LCMGTC(IPCPO,'PARFMT',8,NPAR,PARFMT)
         CALL LCMSIX(IPCPO,' ',2)
      ENDIF
      IF(NLOC.GT.0) THEN
         CALL LCMSIX(IPCPO,'LOCAL',1)
         CALL LCMGTC(IPCPO,'PARKEY',12,NLOC,PARKEL)
         CALL LCMSIX(IPCPO,' ',2)
      ENDIF
      IF(IMPX.GT.0) WRITE(IOUT,'(1X,A)') (COMMEN(I),I=1,NCOMLI)
      CALL XDRSET(TERP,NCAL*NMIX,0.0)
      CALL XDISET(MIXC,NMIX,0)
*----
*  READ (INTERP_DATA) AND SET VALI, VALR AND VALH PARAMETERS
*  CORRESPONDING TO THE INTERPOLATION POINT. FILL MUPLET FOR
*  PARAMETERS SET WITHOUT INTERPOLATION.
*----
      NISO=0
      LISO=.TRUE.
      LDELTA=.FALSE.
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('NCRDRV: CHARACTER DATA EXPECTED.')
   20 IF(TEXT12.EQ.'MIX') THEN
         CALL XDISET(MUPLET,NPAR+NLOC,0)
         CALL XDISET(MUTYPE,NPAR+NLOC,0)
         CALL XDISET(VALI,NPAR,0)
         CALL XDRSET(VALR(1,1),NPAR+NLOC,0.0)
         CALL XDRSET(VALR(1,2),NPAR+NLOC,0.0)
         DO 30 I=1,NPAR
   30    VALH(I)=' '
         CALL REDGET(INDIC,IBM,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('NCRDRV: INTEGER DATA EXPECTED.')
         MIXUPD(IBM)=1
         IBMOLD=1
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRDRV: CHARACTER DATA EXPECTED.')
         IF(TEXT12.EQ.'FROM') THEN
            CALL REDGET(INDIC,IBMOLD,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.1) CALL XABORT('NCRDRV: INTEGER DATA EXPECTED.')
            MIXC(IBM)=IBMOLD
            GO TO 10
         ENDIF
         MIXC(IBM)=IBMOLD
	 GO TO 20
      ELSE IF(TEXT12.EQ.'MICRO') THEN
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRDRV: CHARACTER DATA EXPECTED.')
         IF(TEXT12.EQ.'ALL') THEN
            LISO=.TRUE.
         ELSE IF(TEXT12.EQ.'ONLY') THEN
            LISO=.FALSE.
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRDRV: CHARACTER DATA EXPECTED.')
   40    IF(TEXT12.EQ.'ENDMIX') THEN
            GO TO 20
         ELSE
            NISO=NISO+1
            IF(NISO.GT.MAXISD) CALL XABORT('NCRDRV: MAXISD OVERFLOW.')
            HISO(NISO)=TEXT12
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.EQ.2) THEN
               CONC(NISO)=FLOTT
            ELSE IF((INDIC.EQ.3).AND.(TEXT12.EQ.'*')) THEN
               CONC(NISO)=-99.99
            ELSE
               CALL XABORT('NCRDRV: INVALID HISO DATA.')
            ENDIF
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.3) CALL XABORT('NCRDRV: CHARACTER DATA EXPECTE'
     1      //'D.')
            GO TO 40
         ENDIF
      ELSE IF((TEXT12.EQ.'SET').OR.(TEXT12.EQ.'DELTA')) THEN
         IF(TEXT12.EQ.'SET') THEN
            ITYPE=1
         ELSE IF(TEXT12.EQ.'DELTA') THEN
            ITYPE=2
            LDELTA=.TRUE.
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRDRV: CHARACTER DATA EXPECTED.')
         DO 50 I=1,NPAR
         IF(TEXT12.EQ.PARKEY(I)) THEN
            IPAR=I
            GO TO 60
         ENDIF
   50    CONTINUE
         GO TO 100
   60    LPCPO=LCMGID(IPCPO,'GLOBAL')
         CALL LCMGET(LPCPO,'NVALUE',NVALUE)
         IF(NVALUE(IPAR).GT.MAXVAL) CALL XABORT('NCRDRV: MAXVAL OVERFL'
     1   //'OW.')
         WRITE(RECNAM,'(''pval'',I8.8)') IPAR
         CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
         IF(LENGTH.EQ.0) THEN
            WRITE(HSMG,'(25HNCRDRV: GLOBAL PARAMETER ,A,9H NOT SET.)')
     1      PARKEY(IPAR)
            CALL XABORT(HSMG)
         ENDIF
         IF(PARFMT(IPAR).EQ.'INTEGER') THEN
            IF(ITYPE.NE.1) CALL XABORT('NCRDRV: SET MANDATORY WITH INT'
     1      //'EGER PARAMETERS.')
            CALL REDGET(INDIC,VALI(IPAR),FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.1) CALL XABORT('NCRDRV: INTEGER DATA EXPECTED.')
            CALL LCMGET(LPCPO,RECNAM,VINTE)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRDRV: KEY=,A,18H TABULATED POINTS=,
     1         1P,6I12/(35X,6I12))') PARKEY(IPAR),(VINTE(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            DO 70 J=1,NVALUE(IPAR)
            IF(VALI(IPAR).EQ.VINTE(J)) THEN
               MUPLET(IPAR)=J
               MUTYPE(IPAR)=ITYPE
               GO TO 10
            ENDIF
   70       CONTINUE
            WRITE(HSMG,'(26HNCRDRV: INTEGER PARAMETER ,A,9H WITH VAL,
     1      2HUE,I5,29H NOT FOUND IN COMPO DATABASE.)') PARKEY(IPAR),
     2      VALI(IPAR)
            CALL XABORT(HSMG)
         ELSE IF(PARFMT(IPAR).EQ.'REAL') THEN
            CALL REDGET(INDIC,NITMA,VALR(IPAR,1),TEXT12,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('NCRDRV: REAL DATA EXPECTED.')
            VALR(IPAR,2)=VALR(IPAR,1)
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.EQ.2) THEN
               VALR(IPAR,2)=FLOTT
               CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            ENDIF
            CALL LCMGET(LPCPO,RECNAM,VREAL)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRDRV: KEY=,A,18H TABULATED POINTS=,
     1         1P,6E12.4/(35X,6E12.4))') PARKEY(IPAR),(VREAL(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) THEN
               DO 80 J=1,NVALUE(IPAR)
               IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J))) THEN
                  MUPLET(IPAR)=J
                  IF(ITYPE.NE.1) MUPLET(IPAR)=-1
                  MUTYPE(IPAR)=ITYPE
                  GO TO 20
               ENDIF
   80          CONTINUE
            ENDIF
            IF(VALR(IPAR,1).LT.VREAL(1)) THEN
               WRITE(HSMG,'(23HNCRDRV: REAL PARAMETER ,A,10H WITH VALU,
     1         1HE,1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEY(IPAR),
     2         VALR(IPAR,1)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,2).GT.VREAL(NVALUE(IPAR))) THEN
               WRITE(HSMG,'(23HNCRDRV: REAL PARAMETER ,A,10H WITH VALU,
     1         1HE,1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEY(IPAR),
     2         VALR(IPAR,2)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,1).GT.VALR(IPAR,2)) THEN
               WRITE(HSMG,'(23HNCRDRV: REAL PARAMETER ,A,9H IS DEFIN,
     1         7HED WITH,1P,E12.4,2H >,E12.4,1H.)') PARKEY(IPAR),
     2         VALR(IPAR,1),VALR(IPAR,2)
               CALL XABORT(HSMG)
            ENDIF
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=ITYPE
            GO TO 20
         ELSE IF(PARFMT(IPAR).EQ.'STRING') THEN
            IF(ITYPE.NE.1) CALL XABORT('NCRDRV: SET MANDATORY WITH STR'
     1      //'ING PARAMETERS.')
            CALL REDGET(INDIC,NITMA,FLOTT,VALH(IPAR),DFLOTT)
            IF(INDIC.NE.3) CALL XABORT('NCRDRV: STRING DATA EXPECTED.')
            CALL LCMGTC(LPCPO,RECNAM,12,NVALUE(IPAR),VCHAR)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRDRV: KEY=,A,18H TABULATED POINTS=,
     1         1P,6A12/(35X,6A12))') PARKEY(IPAR),(VCHAR(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            DO 90 J=1,NVALUE(IPAR)
            IF(VALH(IPAR).EQ.VCHAR(J)) THEN
               MUPLET(IPAR)=J
               MUTYPE(IPAR)=ITYPE
               GO TO 10
            ENDIF
   90       CONTINUE
            WRITE(HSMG,'(25HNCRDRV: STRING PARAMETER ,A,10H WITH VALU,
     1      2HE ,A12,29H NOT FOUND IN COMPO DATABASE.)') PARKEY(IPAR),
     2      VALH(IPAR)
            CALL XABORT(HSMG)
         ENDIF
  100    DO 110 I=1,NLOC
         IF(TEXT12.EQ.PARKEL(I)) THEN
            IPAR=NPAR+I
            GO TO 120
         ENDIF
  110    CONTINUE
         CALL XABORT('NCRDRV: PARAMETER '//TEXT12//' NOT FOUND.')
  120    JPCPO=LCMGID(IPCPO,'MIXTURES')
         KPCPO=LCMGIL(JPCPO,IBMOLD)
         LPCPO=LCMGID(KPCPO,'TREE')
         CALL LCMGET(LPCPO,'NVALUE',NVALUE)
         CALL REDGET(INDIC,NITMA,VALR(IPAR,1),TEXT12,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('NCRDRV: REAL DATA EXPECTED.')
         VALR(IPAR,2)=VALR(IPAR,1)
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.EQ.2) THEN
            VALR(IPAR,2)=FLOTT
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         ENDIF
         WRITE(RECNAM,'(''pval'',I8.8)') IPAR-NPAR
         CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
         IF(LENGTH.EQ.0) THEN
            WRITE(HSMG,'(24HNCRDRV: LOCAL PARAMETER ,A,9H NOT SET.)')
     1      PARKEL(IPAR-NPAR)
            CALL XABORT(HSMG)
         ELSE IF(LENGTH.GT.MAXVAL) THEN
            CALL XABORT('NCRDRV: MAXVAL OVERFLOW.')
         ENDIF
         CALL LCMGET(LPCPO,RECNAM,VREAL)
         IF(IMPX.GT.0) THEN
            WRITE(IOUT,'(13H NCRDRV: KEY=,A,18H TABULATED POINTS=,
     1      1P,6E12.4/(35X,6E12.4))') PARKEL(IPAR-NPAR),(VREAL(I),I=1,
     2      NVALUE(IPAR-NPAR))
         ENDIF
         IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) THEN
            DO 130 J=1,NVALUE(IPAR-NPAR)
            IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J))) THEN
               MUPLET(IPAR)=J
               IF(ITYPE.NE.1) MUPLET(IPAR)=-1
               MUTYPE(IPAR)=ITYPE
               GO TO 20
            ENDIF
  130       CONTINUE
         ENDIF
         IF(VALR(IPAR,1).LT.VREAL(1)) THEN
            WRITE(HSMG,'(23HNCRDRV: REAL PARAMETER ,A,11H WITH VALUE,
     1      1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEL(IPAR-NPAR),
     2      VALR(IPAR,1)
            CALL XABORT(HSMG)
         ELSE IF(VALR(IPAR,2).GT.VREAL(NVALUE(IPAR-NPAR))) THEN
            WRITE(HSMG,'(23HNCRDRV: REAL PARAMETER ,A,11H WITH VALUE,
     1      1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEL(IPAR-NPAR),
     2      VALR(IPAR,2)
            CALL XABORT(HSMG)
         ELSE IF(VALR(IPAR,1).GT.VALR(IPAR,2)) THEN
            WRITE(HSMG,'(23HNCRDRV: REAL PARAMETER ,A,9H IS DEFIN,
     1      7HED WITH,1P,E12.4,2H >,E12.4,1H.)') PARKEL(IPAR-NPAR),
     2      VALR(IPAR,1),VALR(IPAR,2)
            CALL XABORT(HSMG)
         ENDIF
         MUPLET(IPAR)=-1
         MUTYPE(IPAR)=ITYPE
         GO TO 20
      ELSE IF(TEXT12.EQ.'ENDMIX') THEN
*----
*  COMPUTE THE TERP FACTORS USING TABLE-OF-CONTENT INFORMATION.
*----
         IF(IBMOLD.GT.NMIL) CALL XABORT('NCRDRV: MIX OVERFLOW (COMPO).')
         IF(IBM.GT.NMIX) CALL XABORT('NCRDRV: MIX OVERFLOW (MICROLIB).')
         IF(NCAL.EQ.1) THEN
           TERP(1,IBM)=1.0
         ELSE
           CALL NCRTRP(IPCPO,LCUBIC,IMPX,IBMOLD,NPAR,NLOC,NCAL,MUPLET,
     1     MUTYPE,VALR,0.0,TERP(1,IBM))
         ENDIF
      ELSE IF((TEXT12.EQ.'COMPO').OR.(TEXT12.EQ.';')) THEN
*----
*  BUILD THE MICROLIB.
*----
         IF(TEXT12.EQ.';') ITER=0
         IF(TEXT12.EQ.'COMPO') ITER=1
         IF(IACCS.EQ.0) THEN
            MAXISO=MAXISD*NMIX
         ELSE
            CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
            MAXISO=MAX(MAXISD*NMIX,ISTATE(2))
         ENDIF
         CALL SETARA(BASE,3*MAXISO,IHUSE1)
         CALL SETARA(BASE,3*MAXISO,IHNAM1)
         CALL SETARA(BASE,MAXISO,IDENS1)
         CALL SETARA(BASE,MAXISO,IITYP1)
         CALL SETARA(BASE,MAXISO,IITOD1)
         CALL SETARA(BASE,MAXISO,ITEMP1)
         CALL SETARA(BASE,MAXISO,IVOL1)
         CALL SETARA(BASE,3*MAXISO,IHUSE2)
         CALL SETARA(BASE,3*MAXISO,IHNAM2)
         CALL SETARA(BASE,MAXISO,IDENS2)
         CALL SETARA(BASE,MAXISO,IIMIX2)
         CALL SETARA(BASE,MAXISO,IITYP2)
         CALL SETARA(BASE,MAXISO,IITOD2)
         CALL SETARA(BASE,MAXISO,ITEMP2)
         CALL SETARA(BASE,MAXISO,IVOL2)
         CALL SETARA(BASE,NGRP,IWORK)
         CALL SETARA(BASE,MAXISO,IPIFI)
         CALL SETARA(BASE,MAXISO,ISW)
         CALL SETARA(BASE,NGRP+1,IENER)
         CALL SETARA(BASE,NGRP,IDELT)
         CALL SETARA(BASE,NCAL,IFACT)
         CALL SETARA(BASE,NMIX,IMILVO)
         CALL NCRLIB(MAXISO,IPLIB,IPCPO,IACCS,NMIX,NGRP,IMPX,NCAL,
     1   TERP,NISO,LISO,HISO,CONC,MIXC,LDELTA,LXS,LRES,BASE(IHUSE1),
     2   BASE(IHNAM1),BASE(IDENS1),BASE(IITYP1),BASE(IITOD1),
     3   BASE(ITEMP1),BASE(IVOL1),BASE(IHUSE2),BASE(IHNAM2),
     4   BASE(IDENS2),BASE(IIMIX2),BASE(IITYP2),BASE(IITOD2),
     5   BASE(ITEMP2),BASE(IVOL2),BASE(IWORK),BASE(IPIFI),BASE(ISW),
     6   BASE(IENER),BASE(IDELT),BASE(IFACT),BASE(IMILVO))
         CALL RLSARA(BASE(IMILVO))
         CALL RLSARA(BASE(IFACT))
         CALL RLSARA(BASE(IDELT))
         CALL RLSARA(BASE(IENER))
         CALL RLSARA(BASE(ISW))
         CALL RLSARA(BASE(IPIFI))
         CALL RLSARA(BASE(IWORK))
         CALL RLSARA(BASE(IVOL2))
         CALL RLSARA(BASE(ITEMP2))
         CALL RLSARA(BASE(IITOD2))
         CALL RLSARA(BASE(IITYP2))
         CALL RLSARA(BASE(IIMIX2))
         CALL RLSARA(BASE(IHNAM2))
         CALL RLSARA(BASE(IDENS2))
         CALL RLSARA(BASE(IHUSE2))
         CALL RLSARA(BASE(IVOL1))
         CALL RLSARA(BASE(ITEMP1))
         CALL RLSARA(BASE(IITOD1))
         CALL RLSARA(BASE(IITYP1))
         CALL RLSARA(BASE(IDENS1))
         CALL RLSARA(BASE(IHNAM1))
         CALL RLSARA(BASE(IHUSE1))
         RETURN
      ELSE
         CALL XABORT('NCRDRV: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 10
      END
