*DECK NCRLIB
      SUBROUTINE NCRLIB(MAXISO,IPLIB,IPCPO,IACCS,NMIX,NGRP,IMPX,NCAL,
     1 TERP,NISO,LISO,HISO,CONC,MIXC,LDELTA,LXS,LRES,HUSE1,HNAM1,DENS1,
     2 ITYP1,ITOD1,TEMP1,VOL1,HUSE2,HNAM2,DENS2,IMIX2,ITYP2,ITOD2,TEMP2,
     3 VOL2,WORK,IPIFI,ISW,ENER,DELT,FACT,MILVO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* build the microlib by scanning the NCAL elementary calculations and
* weighting them with TERP factors.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXISO  maximum allocated space for microlib TOC information.
* IPLIB   address of the microlib LCM object.
* IPCPO   address of the multidimensional compo object.
* IACCS   =0 microlib is created; =2 ... is updated.
* NMIX    maximum number of material mixtures in the microlib.
* NGRP    number of energy groups.
* IMPX    print parameter (equal to zero for no print).
* NCAL    number of elementary calculations in the compo.
* TERP    interpolation factors.
* NISO    number of user-selected isotopes.
* LISO    type of treatment (=.true.: ALL; =.false.: ONLY).
* HISO    name of the user-selected isotopes.
* CONC    user-defined number density of the user-selected isotopes. A
*         value of -99.99 is set to indicate that the compo value is
*         used.
* MIXC    mixture index in the compo corresponding to each microlib
*         mixture.
* LDELTA  =.false. for Sigma, =.true. for Delta Sigma
* LXS     =.true. if keyword 'ALLX' is specified
* LRES    =.true. if the interpolation is done for the residual
*         isotope only
*
*Parameters: scratch
* HUSE1
* HNAM1
* DENS1
* ITYP1
* ITOD1
* TEMP1
* VOL1
* HUSE2
* HNAM2
* DENS2
* IMIX2
* ITYP2
* ITOD2
* TEMP2
* VOL2
* WORK
* IPIFI
* ISW
* ENER
* DELT
* FACT
* MILVO
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MAXISO,IPLIB,IPCPO,IACCS,NMIX,NGRP,IMPX,NCAL,NISO,
     1 MIXC(NMIX),HUSE1(3,MAXISO),HNAM1(3,MAXISO),ITYP1(MAXISO),
     2 ITOD1(MAXISO),IMIX2(MAXISO),ITYP2(MAXISO),ITOD2(MAXISO),
     3 HUSE2(3,MAXISO),HNAM2(3,MAXISO),IPIFI(MAXISO),ISW(MAXISO),
     4 MILVO(NMIX)
      REAL TERP(NCAL,NMIX),CONC(NISO),DENS1(MAXISO),TEMP1(MAXISO),
     1 VOL1(MAXISO),DENS2(MAXISO),TEMP2(MAXISO),VOL2(MAXISO),WORK(NGRP),
     2 ENER(NGRP+1),DELT(NGRP),FACT(NCAL)
      LOGICAL LISO,LDELTA,LXS,LRES
      CHARACTER HISO(NISO)*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MAXED=50,IOUT=6,MAXISD=2)
      CHARACTER TEXT12*12,HNAME*12,HSMG*131,HVECT1(MAXED)*8,
     1 HVECT2(MAXED)*8,CHAR*4,CHAR2*4
      INTEGER ISTATE(NSTATE),KCHAR(3)
      LOGICAL LUSER
      COMMON BASE(1)
*----
*  MICROLIB INITIALIZATION
*----
      IF(IACCS.EQ.0) THEN
         NBISO2=0
         NCOMB2=0
         NED2=0
         TEXT12='L_LIBRARY'
         READ(TEXT12,'(3A4)') (KCHAR(I0),I0=1,3)
         CALL LCMPUT(IPLIB,'SIGNATURE',3,3,KCHAR)
         CALL XDRSET(DENS2,MAXISO,0.0)
         CALL XDRSET(VOL2,MAXISO,0.0)
         CALL XDRSET(TEMP2,MAXISO,0.0)
         CALL XDISET(IMIX2,MAXISO,0)
         CALL XDISET(ITYP2,MAXISO,0)
         CALL XDISET(ITOD2,MAXISO,0)
      ELSE
         CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
         IF(ISTATE(1).NE.NMIX) CALL XABORT('NCRLIB: INVALID NUMBER OF '
     1   //'MATERIAL MIXTURES IN THE MICROLIB.')
         IF(ISTATE(3).NE.NGRP) CALL XABORT('NCRLIB: INVALID NUMBER OF '
     1   //'ENERGY GROUPS IN THE MICROLIB.')
         NBISO2=ISTATE(2)
         NCOMB2=ISTATE(12)
         IF(NBISO2.GT.MAXISO) CALL XABORT('NCRLIB: MAXISO OVERFLOW(1).')
         NED2=ISTATE(13)
         IF(NED2.GT.MAXED) CALL XABORT('NCRLIB: MAXED OVERFLOW.')
         CALL XDISET(IMIX2,MAXISO,0)
         CALL XDISET(ITYP2,MAXISO,0)
         CALL XDISET(ITOD2,MAXISO,0)
         CALL XDRSET(DENS2,MAXISO,0.0)
         CALL XDRSET(VOL2,MAXISO,0.0)
         CALL XDRSET(TEMP2,MAXISO,0.0)
         CALL LCMGET(IPLIB,'ISOTOPESUSED',HUSE2)
         CALL LCMGET(IPLIB,'ISOTOPERNAME',HNAM2)
         CALL LCMGET(IPLIB,'ISOTOPESMIX',IMIX2)
         CALL LCMGET(IPLIB,'ISOTOPESTYPE',ITYP2)
         CALL LCMGET(IPLIB,'ISOTOPESTODO',ITOD2)
         IF(LRES) THEN
           CALL LCMGET(IPLIB,'ISOTOPESDENS',DENS2)
           CALL LCMGET(IPLIB,'ISOTOPESVOL',VOL2)   
           CALL LCMGET(IPLIB,'ISOTOPESTEMP',TEMP2)        
         ENDIF
         IF(NED2.GT.0) CALL LCMGTC(IPLIB,'ADDXSNAME-P0',8,NED2,HVECT2)
         CALL LCMGET(IPLIB,'ENERGY',ENER)
         CALL LCMGET(IPLIB,'DELTAU',DELT)
      ENDIF
*----
*  LOOP OVER MICROLIB MIXTURES
*----
      CALL XDISET(MILVO,NMIX,0)
      NDEPL=0
      NCOMB=0
      JPCPO=LCMGID(IPCPO,'MIXTURES')
      DO 190 IBM=1,NMIX
      IBMOLD=MIXC(IBM)
      IF(IBMOLD.EQ.0) GO TO 190
      IF(LDELTA) THEN
        SUM=0.0
      ELSE
        SUM=1.0
      ENDIF
      DO 10 ICAL=1,NCAL
   10 SUM=SUM-TERP(ICAL,IBM)
      IF(ABS(SUM).GT.1.0E-4) THEN
         WRITE(HSMG,'(45HNCRLIB: INVALID INTERPOLATION FACTORS IN MIXT,
     1   3HURE,I4,1H.)') IBM
         CALL XABORT(HSMG)
      ENDIF
      KPCPO=LCMGIL(JPCPO,IBMOLD)
      LPCPO=LCMGID(KPCPO,'CALCULATIONS')
*----
*  LOOP OVER ELEMENTARY CALCULATIONS
*----
      DO 110 ICAL=1,NCAL
      WEIGHT=TERP(ICAL,IBM)
      IF(WEIGHT.EQ.0.0) GO TO 110
      MPCPO=LCMGIL(LPCPO,ICAL)
      IF(IMPX.GT.0) THEN
         WRITE(IOUT,'(33H NCRLIB: COMPO ACCESS FOR MIXTURE,I4,6H AND C,
     1   10HALCULATION,I5,1H.)') IBMOLD,ICAL
         IF(IMPX.GT.50) CALL LCMLIB(MPCPO)
      ENDIF
      CALL LCMGET(MPCPO,'STATE-VECTOR',ISTATE)
      NL=ISTATE(4)
      ITRANC=ISTATE(5)
      NDEPL=MAX(ISTATE(11),NDEPL)
      NDEL=ISTATE(19)
      NDFI=ISTATE(20)
      IF(ISTATE(1).NE.1) CALL XABORT('NCRLIB: INVALID NUMBER OF MATERI'
     1 //'AL MIXTURES IN THE COMPO.')
      IF(ISTATE(3).NE.NGRP) CALL XABORT('NCRLIB: INVALID NUMBER OF ENE'
     1 //'RGY GROUPS IN THE COMPO.')
      NBISO1=ISTATE(2)
      IF(NBISO1.GT.MAXISO) CALL XABORT('NCRLIB: MAXISO OVERFLOW(2).')
      NED1=ISTATE(13)
      IF(NED1.GT.MAXED) CALL XABORT('NCRLIB: MAXED OVERFLOW.')
      CALL LCMGET(MPCPO,'ISOTOPESUSED',HUSE1)
      CALL LCMGET(MPCPO,'ISOTOPERNAME',HNAM1)
      CALL LCMGET(MPCPO,'ISOTOPESDENS',DENS1)
      CALL LCMGET(MPCPO,'ISOTOPESTYPE',ITYP1)
      CALL LCMGET(MPCPO,'ISOTOPESTODO',ITOD1)
      CALL LCMGET(MPCPO,'ISOTOPESVOL',VOL1)
      CALL LCMGET(MPCPO,'ISOTOPESTEMP',TEMP1)
      IF(NED1.GT.0) CALL LCMGTC(MPCPO,'ADDXSNAME-P0',8,NED1,HVECT1)
      DO 15 IED=1,NED1
        IF(HVECT1(IED).EQ.'NWT0') GO TO 20
   15 CONTINUE
      NED1=NED1+1
      HVECT1(NED1)='NWT0'
   20 CALL LCMGET(MPCPO,'ENERGY',ENER)
      CALL LCMGET(MPCPO,'DELTAU',DELT)
      DO 30 IED1=1,NED1
      DO 25 IED2=1,NED2
      IF(HVECT1(IED1).EQ.HVECT2(IED2)) GO TO 30
   25 CONTINUE
      NED2=NED2+1
      HVECT2(NED2)=HVECT1(IED1)
   30 CONTINUE
      DO 100 ISO=1,NBISO1 ! compo isotope
      WRITE(TEXT12,'(2A4,I4.4)') (HUSE1(I,ISO),I=1,2),IBM
      IF(LXS) THEN
        WRITE(CHAR,'(A4)') HUSE1(3,ISO)
        READ(CHAR,'(I4)') NBRG
        NBISOT=NBRG+MAXISD*(IBM-1)
        WRITE(TEXT12,'(2A4,I4.4)') (HUSE1(I,ISO),I=1,2),NBISOT
      ENDIF
      KSO1=0
      DO 40 KSO=1,NISO ! user-selected isotope
      IF(TEXT12(:8).EQ.HISO(KSO)) THEN
         KSO1=KSO
         GO TO 50
      ENDIF
   40 CONTINUE
      IF(.NOT.LISO) GO TO 100
   50 DO 60 JSO=1,NBISO2 ! microlib isotope
      IF((HUSE1(1,ISO).EQ.HUSE2(1,JSO)).AND.(HUSE1(2,ISO).EQ.
     1 HUSE2(2,JSO)).AND.(IMIX2(JSO).EQ.IBM)) THEN
         IF(ITYP1(ISO).NE.ITYP2(JSO)) THEN
            WRITE(HSMG,500) 'ITYP',ISO,ITYP1(ISO),ITYP2(JSO)
            CALL XABORT(HSMG)
         ENDIF
         IF(LXS) THEN
          WRITE(CHAR,'(A4)') HUSE1(3,ISO)
          WRITE(CHAR2,'(A4)') HUSE2(3,JSO)
          READ(CHAR2,'(I4.4)') NBISOT2
          NBISOT2=NBISOT2-MAXISD*(IBM-1)
          WRITE(CHAR2,'(I4.4)') NBISOT2
           IF(CHAR.EQ.CHAR2) THEN
             JSO1=JSO
             GO TO 90
           ENDIF
         ELSE
           JSO1=JSO
           GO TO 90
         ENDIF
      ENDIF
   60 CONTINUE
      NBISO2=NBISO2+1
      IF(NBISO2.GT.MAXISO) CALL XABORT('NCRLIB: MAXISO OVERFLOW(3).')
      READ(TEXT12,'(3A4)') (HUSE2(I0,NBISO2),I0=1,3)
      DO 70 I0=1,3
   70 HNAM2(I0,NBISO2)=HNAM1(I0,ISO)
      IMIX2(NBISO2)=IBM
      ITYP2(NBISO2)=ITYP1(ISO)
      ITOD2(NBISO2)=ITOD1(ISO)
      IF(ITYP2(NBISO2).EQ.1) ITOD2(NBISO2)=1
      DENS2(NBISO2)=0.0
      JSO1=NBISO2
      IF(ITOD2(NBISO2).NE.1) THEN
         DO 80 J=1,NCOMB
         IF(IBM.EQ.MILVO(J)) GO TO 90
   80    CONTINUE
         NCOMB=NCOMB+1
         IF(NCOMB.GT.NMIX) CALL XABORT('NCRLIB: MILVO OVERFLOW.')
         MILVO(NCOMB)=IBM
      ENDIF
   90 LUSER=.FALSE.
      IF(KSO1.GT.0) LUSER=(CONC(KSO1).NE.-99.99)
      IF(LUSER) DENS1(ISO)=CONC(KSO1)
      IF(.NOT.LRES) THEN
         DENS2(JSO1)=DENS2(JSO1)+WEIGHT*DENS1(ISO)
         VOL2(JSO1)=VOL2(JSO1)+WEIGHT*VOL1(ISO)
         TEMP2(JSO1)=TEMP2(JSO1)+WEIGHT*TEMP1(ISO)
      ENDIF
  100 CONTINUE
  110 CONTINUE
*----
*  PROCESS ISOTOPE DIRECTORIES FOR MICROLIB MIXTURE IBM
*----
      DO 180 ISO=1,NBISO2 ! microlib isotope
      IF(IMIX2(ISO).NE.IBM) GO TO 180
      WRITE(TEXT12,'(3A4)') (HUSE2(I0,ISO),I0=1,3)
      IF(LRES) THEN
        IF(TEXT12(:8).NE.'*MAC*RES') GOTO 180
      ENDIF
      DO 120 JSO=1,NBISO1 ! compo isotope
      IF((HUSE1(1,JSO).EQ.HUSE2(1,ISO)).AND.(HUSE1(2,JSO).EQ.
     1 HUSE2(2,ISO))) THEN
        IF(LXS) THEN
          WRITE(CHAR,'(A4)') HUSE1(3,JSO)
          WRITE(CHAR2,'(A4)') HUSE2(3,ISO)
          READ(CHAR,'(I4)') NBRG
          NBISOT=NBRG+MAXISD*(IBM-1)
          READ(CHAR2,'(I4)') NBISOT2
          IF(NBISOT.EQ.NBISOT2) THEN
            WRITE(HNAME,'(3A4)') (HUSE1(I0,JSO),I0=1,3)
            JSO1=JSO
            GO TO 130
          ENDIF
        ELSE
          WRITE(HNAME,'(3A4)') (HUSE1(I0,JSO),I0=1,3)
          JSO1=JSO
          GO TO 130
        ENDIF
      ENDIF
  120 CONTINUE
      CALL XABORT('NCRLIB: UNABLE TO FIND '//TEXT12//'.')
  130 CALL XDRSET(FACT,NCAL,0.0)
      KSO1=0
      DO 140 KSO=1,NISO ! user-selected isotope
      IF(HNAME(:8).EQ.HISO(KSO)) THEN
         KSO1=KSO
         GO TO 150
      ENDIF
  140 CONTINUE
      IF(.NOT.LISO) GO TO 170
  150 DO 160 ICAL=1,NCAL
      WEIGHT=TERP(ICAL,IBM)
      IF(WEIGHT.EQ.0.0) GO TO 160
      LUSER=.FALSE.
      IF(KSO1.GT.0) LUSER=(CONC(KSO1).NE.-99.99)
      IF(LUSER) THEN
         FACT(ICAL)=CONC(KSO1)
      ELSE
         MPCPO=LCMGIL(LPCPO,ICAL)
         CALL LCMGET(MPCPO,'ISOTOPESDENS',DENS1)
         FACT(ICAL)=DENS1(JSO1)
      ENDIF
      IF(DENS2(ISO).GT.FACT(ICAL)*1.0E-9) THEN
         FACT(ICAL)=FACT(ICAL)/DENS2(ISO)
      ELSE
         FACT(ICAL)=1.0
      ENDIF
  160 CONTINUE
  170 CALL LCMSIX(IPLIB,TEXT12,1)
      CALL SETARA(BASE,NGRP*(10+NL+NED2+2*NDEL),IGAR1)
      CALL SETARA(BASE,NGRP+1,IYIEL1)
      CALL SETARA(BASE,NDFI,IPYIE1)
      CALL SETARA(BASE,NDFI,IJPIF1)
      CALL SETARA(BASE,NGRP*NGRP*NL,IWSCA1)
      CALL SETARA(BASE,NGRP*(10+NL+NED2+2*NDEL),IGAR2)
      CALL SETARA(BASE,NGRP*NGRP*NL,IWSCA2)
      CALL SETARA(BASE,NL,ITYPR)
      CALL SETARA(BASE,NDEL,IWDLA)
      CALL NCRISO(IPLIB,LPCPO,HNAME,IBMOLD,NCAL,NGRP,NL,NED2,HVECT2,
     1 NDEL,NDFI,IMPX,FACT,TERP(1,IBM),BASE(IGAR1),BASE(IYIEL1),
     2 BASE(IPYIE1),BASE(IJPIF1),BASE(IWSCA1),BASE(IGAR2),BASE(IWSCA2),
     3 BASE(ITYPR),BASE(IWDLA))
      CALL RLSARA(BASE(IWDLA))
      CALL RLSARA(BASE(ITYPR))
      CALL RLSARA(BASE(IWSCA2))
      CALL RLSARA(BASE(IGAR2))
      CALL RLSARA(BASE(IWSCA1))
      CALL RLSARA(BASE(IJPIF1))
      CALL RLSARA(BASE(IPYIE1))
      CALL RLSARA(BASE(IYIEL1))
      CALL RLSARA(BASE(IGAR1))
      CALL LCMSIX(IPLIB,' ',2)
  180 CONTINUE
  190 CONTINUE
*----
*  MICROLIB FINALIZATION
*----
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NMIX
      ISTATE(2)=NBISO2
      ISTATE(3)=NGRP
      ISTATE(4)=NL
      ISTATE(5)=ITRANC
      ISTATE(7)=1
      ISTATE(11)=NDEPL
      ISTATE(12)=NCOMB+NCOMB2
      ISTATE(13)=NED2
      ISTATE(14)=NMIX
      ISTATE(18)=1
      ISTATE(19)=NDEL
      ISTATE(20)=NDFI
      ISTATE(22)=MAXISO/NMIX
      CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPLIB,'ISOTOPESUSED',3*NBISO2,3,HUSE2)
      CALL LCMPUT(IPLIB,'ISOTOPERNAME',3*NBISO2,3,HNAM2)
      CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO2,2,DENS2)
      CALL LCMPUT(IPLIB,'ISOTOPESMIX',NBISO2,1,IMIX2)
      CALL LCMPUT(IPLIB,'ISOTOPESTYPE',NBISO2,1,ITYP2)
      CALL LCMPUT(IPLIB,'ISOTOPESTODO',NBISO2,1,ITOD2)
      CALL LCMPUT(IPLIB,'ISOTOPESVOL',NBISO2,2,VOL2)
      CALL LCMPUT(IPLIB,'ISOTOPESTEMP',NBISO2,2,TEMP2)
      IF(NED2.GT.0) CALL LCMPTC(IPLIB,'ADDXSNAME-P0',8,NED2,HVECT2)
      CALL LCMPUT(IPLIB,'ENERGY',NGRP+1,2,ENER)
      CALL LCMPUT(IPLIB,'DELTAU',NGRP,2,DELT)
      IF(IMPX.GT.5) CALL LCMLIB(IPLIB)
      RETURN
  500 FORMAT(8HNCRLIB: ,A,1H(,I4,2H)=,2I5)
      END
