*DECK NCRMAC
      SUBROUTINE NCRMAC(IPLIB,IPCPO,NGRP,NMIX,MIXUPD,B2)
*
*-----------------------------------------------------------------------
*
*Purpose:
*  recover and interpolate microlib and macrolib information from one
*  or many multi-parameter compo database objects (part 2).
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert, R. Chambon
*
*Parameters: input/output
* IPLIB   address of the microlib LCM object.
* IPCPO   address of the multicompo object.
* NGRP    number of energy groups.
* NMIX    maximum number of material mixtures in the microlib.
* MIXUPD  tag for mixture which will be updated
* B2      buckling
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IPCPO,NGRP,NMIX
      INTEGER MIXUPD(NMIX)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,IOUT=6)
      CHARACTER HSMG*131
      LOGICAL LBASE(1)
      INTEGER ISTATE(NSTATE),IST1(NSTATE),IST2(NSTATE),IBASE(1)
      REAL TMPDAY(3)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),LBASE(1),IBASE(1))
*----
*  RECOVER THE DEPLETION CHAIN
*----
      CALL LCMLEN(IPLIB,'DEPL-CHAIN',ILENG1,ITYLCM)
      CALL LCMLEN(IPCPO,'DEPL-CHAIN',ILENG2,ITYLCM)
      IF((ILENG1.NE.0).AND.(ILENG2.NE.0)) THEN
        CALL LCMSIX(IPCPO,'DEPL-CHAIN',1)
        CALL LCMGET(IPCPO,'STATE-VECTOR',IST1)
        CALL LCMSIX(IPCPO,' ',2)
        CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
        CALL LCMGET(IPLIB,'STATE-VECTOR',IST2)
        CALL LCMSIX(IPLIB,' ',2)
        DO 110 I=1,NSTATE
        IF(IST1(I).NE.IST2(I)) THEN
          WRITE(HSMG,'(40HNCRMAC: INVALID STATE-VECTOR COMPONENT (,I2,
     1    36H) FOR DEPL-CHAIN DATA IN MULTICOMPO ,1H.)') I
          CALL XABORT(HSMG)
        ENDIF
  110   CONTINUE
      ELSE IF((ILENG1.EQ.0).AND.(ILENG2.NE.0)) THEN
        CALL LCMSIX(IPCPO,'DEPL-CHAIN',1)
        CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
        CALL LCMEQU(IPCPO,IPLIB)
        CALL LCMSIX(IPLIB,' ',2)
        CALL LCMSIX(IPCPO,' ',2)
      ENDIF
*----
*  COMPUTE THE MACROSCOPIC X-SECTIONS
*----
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      MAXMIX=ISTATE(1)
      NBISO=ISTATE(2)
      CALL SETARA(LBASE,MAXMIX,IMASK)
      CALL SETARA(LBASE,NGRP,IMASKL)
      CALL LCMIOF(IPLIB,'ISOTOPESUSED',IISONA)
      CALL LCMIOF(IPLIB,'ISOTOPESMIX',IISOMI)
      CALL LCMIOF(IPLIB,'ISOTOPESDENS',IDENIS)
      CALL XDLSET(LBASE(IMASK),MAXMIX,.FALSE.)
      CALL XDLSET(LBASE(IMASKL),NGRP,.TRUE.)
      DO 130 I=1,NBISO
        IBM=IBASE(IISOMI+I-1)
        IF(MIXUPD(IBM).NE.0) LBASE(IMASK+IBM-1)=.TRUE.
  130 CONTINUE
      ITSTMP=0
      TMPDAY(1)=0.0
      TMPDAY(2)=0.0
      TMPDAY(3)=0.0
      CALL LIBMIX(IPLIB,MAXMIX,NGRP,NBISO,IBASE(IISONA),IBASE(IISOMI),
     1         BASE(IDENIS),LBASE(IMASK),LBASE(IMASKL),ITSTMP,TMPDAY)
      CALL RLSARA(LBASE(IMASKL))
      CALL RLSARA(LBASE(IMASK))
*----
*  INCLUDE LEAKAGE IN THE MACROLIB (USED ONLY FOR NON-REGRESSION TESTS)
*----
      IF(B2.NE.0.0) THEN
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        JPLIB=LCMGID(IPLIB,'GROUP')
        CALL SETARA(BASE,MAXMIX,IGAR1)
        CALL SETARA(BASE,MAXMIX,IGAR2)
        DO 160 IGR=1,NGRP
          KPLIB=LCMGIL(JPLIB,IGR)
          CALL LCMGET(KPLIB,'NTOT0',BASE(IGAR1))
          CALL LCMGET(KPLIB,'DIFF',BASE(IGAR2))
          DO 150 IBM=1,MAXMIX
            IF(MIXUPD(IBM).NE.0) 
     >        BASE(IGAR1+IBM-1)=BASE(IGAR1+IBM-1)+B2*BASE(IGAR2+IBM-1)
  150     CONTINUE
          CALL LCMPUT(KPLIB,'NTOT0',MAXMIX,2,BASE(IGAR1))
  160   CONTINUE
        CALL RLSARA(BASE(IGAR2))
        CALL RLSARA(BASE(IGAR1))
        CALL LCMSIX(IPLIB,' ',2)
      ENDIF
      RETURN
      END
