*DECK NCRRGR
      SUBROUTINE NCRRGR(IPLIB,IPCPO,IPMAP,LCUBIC,IACCS,NMIX,NGRP,IMPX,
     1 NMIL,NCAL,NCH,NB,NFUEL,NCOMB,ITER,MIXC,TERP,FMIX,BRN0,BRN1,WPAR,
     2 ZONEDP,ZONEC,NPARMP,NPARM,LXS,LRES,HPAR,MIXUPD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover and/or interpolate multicompo information and store them in a
* new microlib. Use global and local parameters from a fuel-map object
* and optional user-defined values.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert, D. Sekki, R. Chambon
*
*Parameters: input
* IPLIB   address of the microlib to be created.
* IPCPO   address of the multicompo object.
* IPMAP   address of the fuel-map object.
* LCUBIC  =.TRUE.: cubic Ceschino interpolation; =.FALSE: linear
*         Lagrange interpolation.
* NMIX    number of material mixtures in the fuel-map macrolib.
* IACCS   =0 microlib is created; =1 ... is updated.
* NGRP    number of energy groups.
* IMPX    printing index (=0 for no print).
* NMIL    number of material mixtures in the compo.
* NCAL    number of elementary calculations in the compo.
* NCH     number of reactor channels.
* NB      number of fuel bundles per channel.
* NFUEL   number of fuel types.
* NCOMB   total number of combustion zones.
* NPARMP  number of additional parameters defined in FMAP object
* NPARM   =NPARMP, or =1 if NPARMP=0 (used for SETARA)
* LXS     =.true. if keyword 'ALLX' is specified
* LRES    =.true. if the interpolation is done for the residual
*         isotope only
*
*Parameters: output
* ITER    completion flag (=0: all over; =1: use another compo).
* MIXUPD  tag for mixture which will be updated
*
*Parameters: scratch
* MIXC
* TERP    interpolation factors.
* FMIX    fuel mixture indices per fuel bundle.
* BRN0    contains either low burnup integration limits or
*         instantaneous burnups per fuel bundle.
* BRN1    upper burnup integration limits per fuel bundle.
* WPAR    other parameter distributions.
* HPAR    'PARKEY' name of the other parameters.
* ZONEDP
* ZONEC
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IPCPO,IPMAP,IACCS,NMIX,NGRP,IMPX,NMIL,NCAL,
     1        NFUEL,NCH,NB,NCOMB,ITER,MIXC(NMIX),FMIX(NCH*NB),
     2        ZONEDP(NCH,NB),ZONEC(NCH),NPARMP,NPARM,MIXUPD(NMIX)
      REAL TERP(NCAL,NMIX),BRN0(NCH*NB),BRN1(NCH*NB),WPAR(NCH*NB,NPARM)
      CHARACTER HPAR(NPARM+1)*12
      LOGICAL LCUBIC,LXS,LRES
*----
*  LOCAL VARIABLES
*----
      PARAMETER (REPS=1.0E-4,IOUT=6,MAXPAR=50,MAXLIN=50,MAXVAL=200,
     1 NSTATE=40,MAXISD=200,MAXADD=10)
      CHARACTER TEXT12*12,PARKEY(MAXPAR)*12,PARFMT(MAXPAR)*8,
     1 PARKEL(MAXPAR)*12,HSMG*131,COMMEN(MAXLIN)*80,VALH(MAXPAR)*12,
     2 RECNAM*12,VCHAR(MAXVAL)*12,HISO(MAXISD)*12,PARNAM*12
      INTEGER ISTATE(NSTATE),VALI(MAXPAR),NVALUE(MAXPAR),VINTE(MAXVAL),
     1 MUPLET(2*MAXPAR),MUTYPE(2*MAXPAR),IBASE(1),
     2 MAPLET(2*MAXPAR,MAXADD),MATYPE(2*MAXPAR,MAXADD),IDLTA(2*MAXPAR),
     3 IDLTA1,MUPLT2(2*MAXPAR),MUTYP2(2*MAXPAR)
      DOUBLE PRECISION DFLOTT
      REAL VALR(2*MAXPAR,2),VREAL(MAXVAL),CONC(MAXISD),VALREF(2*MAXPAR),
     1 VALRA(2*MAXPAR,2,MAXADD),VALRFA(2*MAXPAR,MAXADD)
      LOGICAL LISO,LPARM(NPARM+1),LDELT(2*MAXPAR),LDELT1,LDELTA,
     1        LSET(2*MAXPAR),LSET1,LADD(2*MAXPAR),LADD1,LDMAP(2*MAXPAR),
     2        LAMAP(2*MAXPAR),LTST
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  READ THE PARKEY NAME OF THE BURNUP FOR THIS COMPO.
*----
      CALL REDGET(INDIC,NITMA,FLOTT,HPAR(NPARMP+1),DFLOTT)
      IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
*----
*  RECOVER TABLE-OF-CONTENT INFORMATION FOR THE COMPO.
*----
      CALL LCMGET(IPCPO,'STATE-VECTOR',ISTATE)
      NPAR=ISTATE(5)
      NLOC=ISTATE(6)
      NCOMLI=ISTATE(10)
      CALL LCMGTC(IPCPO,'COMMENT',80,NCOMLI,COMMEN)
      IF(NPAR.GT.0)THEN
        CALL LCMSIX(IPCPO,'GLOBAL',1)
        CALL LCMGTC(IPCPO,'PARKEY',12,NPAR,PARKEY)
        CALL LCMGTC(IPCPO,'PARFMT',8,NPAR,PARFMT)
        CALL LCMSIX(IPCPO,' ',2)
      ENDIF
      IF(NLOC.GT.0)THEN
        CALL LCMSIX(IPCPO,'LOCAL',1)
        CALL LCMGTC(IPCPO,'PARKEY',12,NLOC,PARKEL)
        CALL LCMSIX(IPCPO,' ',2)
      ENDIF
      IF(IMPX.GT.0)WRITE(IOUT,'(1X,A)') (COMMEN(I),I=1,NCOMLI)
      CALL XDRSET(TERP,NCAL*NMIX,0.0)
      CALL XDISET(MIXC,NMIX,0)
*----
*  READ (INTERP_DATA) AND SET VALI, VALR AND VALH PARAMETERS
*  CORRESPONDING TO THE INTERPOLATION POINT. FILL MUPLET FOR
*  PARAMETERS SET WITHOUT INTERPOLATION.
*----
      NISO=0
      LISO=.TRUE.
      LDELTA=.FALSE.
      IDLTA1=0
      DO I=1,2*MAXPAR
         LSET(I)=.FALSE.
         LDELT(I)=.FALSE.
         LADD(I)=.FALSE.
         LDMAP(I)=.FALSE.
         LAMAP(I)=.FALSE.
      ENDDO
*----
*  MAIN LOOP OF THE SUBROUTINE (UNTIL THE END)
*----
   20 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
   30 IF(TEXT12.EQ.'MIX')THEN
        IVARTY=0
        IBTYP=0
        CALL XDISET(MUPLET,NPAR+NLOC,0)
        CALL XDISET(MUTYPE,NPAR+NLOC,0)
        CALL XDISET(VALI,NPAR,0)
        CALL XDRSET(VALR(1,1),NPAR+NLOC,0.0)
        CALL XDRSET(VALR(1,2),NPAR+NLOC,0.0)
        DO 35 I=1,MAXADD
          CALL XDISET(MAPLET(1,I),NPAR+NLOC,0)
          CALL XDISET(MATYPE(1,I),NPAR+NLOC,0)
          CALL XDRSET(VALRA(1,1,I),NPAR+NLOC,0.0)
          CALL XDRSET(VALRA(1,2,I),NPAR+NLOC,0.0)
   35   CONTINUE
        DO I=1,2*MAXPAR
          LSET(I)=.FALSE.
          LDELT(I)=.FALSE.
          LADD(I)=.FALSE.
          LDMAP(I)=.FALSE.
          LAMAP(I)=.FALSE.
        ENDDO
        DO 40 I=1,NPAR
   40   VALH(I)=' '
        CALL REDGET(INDIC,IBM,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1)CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
*       CHECK FUEL MIXTURE
        JPMAP=LCMGID(IPMAP,'FUEL')
        DO IFUEL=1,NFUEL
          KPMAP=LCMGIL(JPMAP,IFUEL)
          CALL LCMGET(KPMAP,'MIX',IMIX)
          IF(IMIX.EQ.IBM)GOTO 50
        ENDDO
        WRITE(IOUT,*)'NCRRGR: UNABLE TO FIND FUEL MIXTURE ',IBM
        CALL XABORT('NCRRGR: WRONG MIXTURE NUMBER.')
   50   IBMOLD=1
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
        IF(TEXT12.EQ.'FROM')THEN
          CALL REDGET(INDIC,IBMOLD,FLOTT,TEXT12,DFLOTT)
          IF(INDIC.NE.1)CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
          CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTE'
     1     //'D.')
        ENDIF
        GOTO 30
      ELSEIF(TEXT12.EQ.'MICRO')THEN
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
        IF(TEXT12.EQ.'ALL')THEN
          LISO=.TRUE.
        ELSEIF(TEXT12.EQ.'ONLY')THEN
          LISO=.FALSE.
        ENDIF
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
   60   IF(TEXT12.EQ.'ENDMIX')THEN
          GOTO 30
        ELSE
          NISO=NISO+1
          IF(NISO.GT.MAXISD)CALL XABORT('NCRRGR: MAXISD OVERFLOW.')
          HISO(NISO)=TEXT12
          CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          IF(INDIC.EQ.2)THEN
            CONC(NISO)=FLOTT
          ELSEIF((INDIC.EQ.3).AND.(TEXT12.EQ.'*'))THEN
            CONC(NISO)=-99.99
          ELSE
            CALL XABORT('NCRRGR: INVALID HISO DATA.')
          ENDIF
          CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
          GOTO 60
        ENDIF
      ELSEIF((TEXT12.EQ.'SET').OR.(TEXT12.EQ.'DELTA').OR.
     1       (TEXT12.EQ.'ADD'))THEN
        LSET1=.FALSE.
        LDELT1=.FALSE.
        LADD1=.FALSE.
        IF(TEXT12.EQ.'SET')THEN
          ITYPE=1
          LSET1=.TRUE.
        ELSEIF(TEXT12.EQ.'DELTA')THEN
          ITYPE=2
          LDELT1=.TRUE.
          LDELTA=.TRUE.
        ELSEIF(TEXT12.EQ.'ADD')THEN
          ITYPE=2
          LADD1=.TRUE.
          IDLTA1=IDLTA1+1
          DO 65 JPAR=1,NPAR+NLOC
            MAPLET(JPAR,IDLTA1)=MUPLET(JPAR)
            MATYPE(JPAR,IDLTA1)=MUTYPE(JPAR)
   65     CONTINUE
        ENDIF
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.3)CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
* check if parameter is global
        DO I=1,NPAR
          IF(TEXT12.EQ.PARKEY(I))THEN
            IPAR=I
            LPCPO=LCMGID(IPCPO,'GLOBAL')
            IPARTM=IPAR
            PARNAM=TEXT12
            GOTO 70
          ENDIF
        ENDDO
* check if parameter is local
        DO I=1,NLOC
          IF(TEXT12.EQ.PARKEL(I))THEN
            IPAR=NPAR+I
            JPCPO=LCMGID(IPCPO,'MIXTURES')
            KPCPO=LCMGIL(JPCPO,IBMOLD)
            LPCPO=LCMGID(KPCPO,'TREE')
            IPARTM=IPAR-NAR
            PARNAM=TEXT12
            GOTO 70
          ENDIF
        ENDDO
        WRITE(HSMG,'(18HNCRRGR: PARAMETER ,A,14H NOT FOUND(1).)') TEXT12
        CALL XABORT(HSMG)
   70   CONTINUE
		WRITE(RECNAM,'(''pval'',I8.8)') IPARTM
        CALL LCMGET(LPCPO,'NVALUE',NVALUE)
        IF(NVALUE(IPARTM).GT.MAXVAL)CALL XABORT('NCRRGR: MAXVAL OVERFL'
     1   //'OW.')
        CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
        IF(LENGTH.EQ.0)THEN
          WRITE(HSMG,'(25HNCRRGR: GLOBAL PARAMETER ,A,9H NOT SET.)')
     1     PARNAM
          CALL XABORT(HSMG)
        ENDIF
        IF((IPAR.GT.NPAR).OR.
     1   ((IPAR.LE.NPAR).AND.(PARFMT(IPAR).EQ.'REAL')))THEN
          CALL REDGET(INDIC,NITMA,VALR1,TEXT12,DFLOTT)
          IF(INDIC.NE.2)CALL XABORT('NCRRGR: REAL DATA EXPECTED.')
          VALR2=VALR1
          IF(LSET1) THEN
            LSET(IPAR)=.TRUE.
            VALR(IPAR,1)=VALR1
            VALR(IPAR,2)=VALR1
            VALREF(IPAR)=VALR1
          ENDIF
          IF(LDELT1) THEN
            LDELT(IPAR)=.TRUE.
            VALR(IPAR,1)=VALR1
            VALR(IPAR,2)=VALR1
            VALREF(IPAR)=VALR1
          ENDIF
          IF(LADD1) THEN
            LADD(IPAR)=.TRUE.
            VALRA(IPAR,1,IDLTA1)=VALR1
            VALRA(IPAR,2,IDLTA1)=VALR1
            VALRFA(IPAR,IDLTA1)=VALR1
            IDLTA(IPAR)=IDLTA1
          ENDIF
          CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          IF(ITYPE.GE.2)THEN
            IF(INDIC.EQ.2)THEN
              VALR2=FLOTT
              IF(LDELT1)THEN
                VALR(IPAR,2)=VALR2
              ELSEIF(LADD1)THEN
                VALRA(IPAR,2,IDLTA1)=VALR2
              ENDIF
            ELSEIF(TEXT12.EQ.'MAP')THEN
              IF(LDELT1)THEN
                LDMAP(IPAR)=.TRUE.
              ELSEIF(LADD1)THEN
                LAMAP(IPAR)=.TRUE.
              ENDIF
            ELSE
              CALL XABORT('NCRRGR: real value or "MAP" expected')
            ENDIF
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          ENDIF
          CALL LCMGET(LPCPO,RECNAM,VREAL)
          IF(IMPX.GT.0)THEN
            WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1       1P,6E12.4/(35X,6E12.4))') PARNAM,(VREAL(I),I=1,
     2       NVALUE(IPARTM))
          ENDIF
          LTST=.FALSE.
          IF(.NOT.LADD1)THEN
            IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) LTST=.TRUE.
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=ITYPE
          ELSE
            MAPLET(IPAR,IDLTA1)=-1
            MATYPE(IPAR,IDLTA1)=2
          ENDIF
          IF((LTST).AND.(ITYPE.EQ.1))THEN
            DO J=1,NVALUE(IPARTM)
              IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J)))THEN
                MUPLET(IPAR)=J
                GOTO 30
              ENDIF
            ENDDO
          ENDIF
*----
*  ERRORS HANDLING
*----
          IF(VALR1.LT.VREAL(1))THEN
*           OUTSIDE OF THE DOMAIN (1)
            WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1       1HE,1P,E12.4,26H IS OUTSIDE THE DOMAIN.(1))') PARNAM,VALR1
             WRITE(6,*)'Domain:',VREAL(1),' <-> ',VREAL(NVALUE(IPAR))
            CALL XABORT(HSMG)
          ELSEIF(VALR2.GT.VREAL(NVALUE(IPARTM)))THEN
*           OUTSIDE OF THE DOMAIN (2)
            WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1       1HE,1P,E12.4,26H IS OUTSIDE THE DOMAIN.(2))') PARNAM,VALR2
             WRITE(6,*)'Domain:',VREAL(1),' <-> ',VREAL(NVALUE(IPAR))
            CALL XABORT(HSMG)
          ELSEIF((VALR1.GT.VALR2).AND.(ITYPE.EQ.1))THEN
*           ITYPE=1 correspond to an integral between VALR1 and VALR2
*           otherwise it is a simple difference
            WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,9H IS DEFIN,
     1       7HED WITH,1P,E12.4,2H >,E12.4,4H.(1))') PARNAM,
     2       VALR1,VALR2
            CALL XABORT(HSMG)
          ENDIF
          IF((LADD1).AND.(TEXT12.EQ.'REF'))THEN
  120       CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(TEXT12.EQ.'ENDREF') GOTO 170
            DO I=1,NPAR
              IF(TEXT12.EQ.PARKEY(I))THEN
                IPAR=I
                GOTO 125
              ENDIF
            ENDDO
            DO I=1,NLOC
              IF(TEXT12.EQ.PARKEL(I))THEN
                IPAR=NPAR+I
                GOTO 125
              ENDIF
            ENDDO
            CALL XABORT('NCRRGR: PARAMETER '//TEXT12//' NOT FOUND (2).')
  125       CONTINUE
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.EQ.2)THEN
              VALRA(IPAR,1,IDLTA1)=FLOTT
              VALRA(IPAR,2,IDLTA1)=FLOTT
              IF(IPAR.LE.NPAR)THEN
                LPCPO=LCMGID(IPCPO,'GLOBAL')
                IPARTM=IPAR
              ELSE
                JPCPO=LCMGID(IPCPO,'MIXTURES')
                KPCPO=LCMGIL(JPCPO,IBMOLD)
                LPCPO=LCMGID(KPCPO,'TREE')
                IPARTM=IPAR-NPAR
              ENDIF
              WRITE(RECNAM,'(''pval'',I8.8)') IPARTM
              CALL LCMGET(LPCPO,RECNAM,VREAL)
              CALL LCMGET(LPCPO,'NVALUE',NVALUE)
              MAPLET(IPAR,IDLTA1)=-1
              MATYPE(IPAR,IDLTA1)=1
              DO J=1,NVALUE(IPARTM)
                IF(ABS(VALRA(IPAR,1,IDLTA1)-VREAL(J)).LE.
     1             REPS*ABS(VREAL(J)))THEN
                  MAPLET(IPAR,IDLTA1)=J
                  GOTO 120
                ENDIF
              ENDDO
            ELSEIF(TEXT12.EQ.'SAMEASREF')THEN
              MAPLET(IPAR,IDLTA1)=-1
              MATYPE(IPAR,IDLTA1)=-1
            ELSE
              CALL XABORT('NCRRGR: REAL or "SAMEASREF" expected')
            ENDIF
            GOTO 120
  170       CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
          ENDIF
          GOTO 30
        ELSEIF(PARFMT(IPAR).EQ.'INTEGER')THEN
          IF(ITYPE.NE.1)CALL XABORT('NCRRGR: SET MANDATORY WITH INT'
     1     //'EGER PARAMETERS.')
          CALL REDGET(INDIC,VALI(IPAR),FLOTT,TEXT12,DFLOTT)
          IF(INDIC.NE.1)CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
          CALL LCMGET(LPCPO,RECNAM,VINTE)
          IF(IMPX.GT.0) THEN
            WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1       1P,6I12/(35X,6I12))') PARKEY(IPAR),(VINTE(I),I=1,
     2       NVALUE(IPAR))
          ENDIF
          DO 90 J=1,NVALUE(IPAR)
            IF(VALI(IPAR).EQ.VINTE(J))THEN
              MUPLET(IPAR)=J
              MUTYPE(IPAR)=ITYPE
              GOTO 20
            ENDIF
   90     CONTINUE
          WRITE(HSMG,'(26HNCRRGR: INTEGER PARAMETER ,A,9H WITH VAL,
     1     2HUE,I5,29H NOT FOUND IN COMPO DATABASE.)') PARKEY(IPAR),
     2     VALI(IPAR)
           CALL XABORT(HSMG)
        ELSEIF(PARFMT(IPAR).EQ.'STRING')THEN
          IF(ITYPE.NE.1)CALL XABORT('NCRRGR: SET MANDATORY WITH STR'
     1     //'ING PARAMETERS.')
          CALL REDGET(INDIC,NITMA,FLOTT,VALH(IPAR),DFLOTT)
          IF(INDIC.NE.3)CALL XABORT('NCRRGR: STRING DATA EXPECTED.')
          CALL LCMGTC(LPCPO,RECNAM,12,NVALUE(IPAR),VCHAR)
          IF(IMPX.GT.0)THEN
            WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1       1P,6A12/(35X,6A12))') PARKEY(IPAR),(VCHAR(I),I=1,
     2       NVALUE(IPAR))
          ENDIF
          DO 110 J=1,NVALUE(IPAR)
            IF(VALH(IPAR).EQ.VCHAR(J))THEN
              MUPLET(IPAR)=J
              MUTYPE(IPAR)=ITYPE
              GOTO 20
            ENDIF
  110     CONTINUE
          WRITE(HSMG,'(25HNCRRGR: STRING PARAMETER ,A,10H WITH VALU,
     1     1HE,A12,29H NOT FOUND IN COMPO DATABASE.)') PARKEY(IPAR),
     2     VALH(IPAR)
          CALL XABORT(HSMG)
        ENDIF
      ELSEIF(TEXT12.EQ.'TIMAV-BURN')THEN
        IBTYP=1
      ELSEIF(TEXT12.EQ.'INST-BURN')THEN
        IBTYP=2
      ELSEIF(TEXT12.EQ.'AVG-EX-BURN')THEN
        IBTYP=3
        CALL REDGET(INDIC,IVARTY,FLOTT,TEXT12,DFLOTT)
        IF(INDIC.NE.1)CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
      ELSEIF(TEXT12.EQ.'ENDMIX')THEN
*----
*  RECOVER FUEL-MAP INFORMATION.
*----
        CALL XDISET(FMIX,NCH*NB,0)
        CALL LCMGET(IPMAP,'FLMIX',FMIX)
        CALL SETARA(BASE,NCH,IBZONE)
        CALL SETARA(BASE,NCH*NB,IBURNB)
        CALL NCRMAP(IPMAP,NPARM,NPARMP,HPAR,NCH,NB,IBTYP,NCOMB,IMPX,
     1              BRN0,BRN1,WPAR,LPARM,BASE(IBZONE),BASE(IBURNB))
        CALL RLSARA(BASE(IBURNB))
        CALL RLSARA(BASE(IBZONE))
        IF(IBTYP.EQ.3) THEN
          IF(IVARTY.EQ.0) CALL XABORT('NCRRGR: IVARTY NOT SET.')
          CALL LCMGET(IPMAP,'B-ZONE',ZONEC)
          DO ICH=1,NCH
            DO J=1,NB
              IF(ZONEC(ICH).EQ.IVARTY) THEN
                ZONEDP(ICH,J)=1
              ELSE
                ZONEDP(ICH,J)=0
              ENDIF
            ENDDO
          ENDDO
          CALL LCMLEN(IPMAP,'B-VALUE',ILONG,ITYP)
          IF (ILONG.EQ.0) CALL XABORT('NCRRGR: NO SAVED VALUES FOR '
     1    //'THIS TYPE OF VARIABLE IN L_MAP')
          CALL SETARA(BASE,ILONG,IVARC)
          CALL LCMGET(IPMAP,'B-VALUE',BASE(IVARC))
          VARVAL=BASE(IVARC+IVARTY-1)
          CALL RLSARA(BASE(IVARC))
        ENDIF
*----
*  PERFORM INTERPOLATION OVER THE FUEL MAP.
*----
        DO 185 JPARM=1,NPARMP+1
          DO I=1,NPAR
            IF(HPAR(JPARM).EQ.PARKEY(I))THEN
              IPAR=I
              IF(LSET(IPAR)) THEN
                WRITE(6,*) 'L_MAP values overwritten by the SET option'
                IF(.NOT.LADD(IPAR)) LPARM(JPARM)=.FALSE.
              ENDIF
              GOTO 185
            ENDIF
          ENDDO
          DO I=1,NLOC
            IF(HPAR(JPARM).EQ.PARKEL(I))THEN
              IPAR=NPAR+I
              IF(LSET(IPAR)) THEN
                WRITE(6,*) 'L_MAP values overwritten by the SET option'
                IF(.NOT.LAMAP(IPAR)) LPARM(JPARM)=.FALSE.
              ENDIF
              GOTO 185
            ENDIF
          ENDDO
  185   CONTINUE
*----
*  COMPUTE ALL THE MUPLETS FOR EACH BUNDLE
*----
        IMPY=MAX(0,IMPX-1)
        DO 280 ICH=1,NCH
        DO 280 JB=1,NB
        IB=(JB-1)*NCH+ICH
        IF(FMIX(IB).EQ.IBM)THEN
          MIXUPD(IB)=1
          DO 260 JPARM=1,NPARMP+1
	  IF(.NOT.LPARM(JPARM))GOTO 260
* check if parameter is global
        DO I=1,NPAR
          IF(HPAR(JPARM).EQ.PARKEY(I))THEN
            IPAR=I
            LPCPO=LCMGID(IPCPO,'GLOBAL')
            IPARTM=IPAR
            PARNAM=HPAR(JPARM)
            GOTO 190
          ENDIF
        ENDDO
* check if parameter is local
        DO I=1,NLOC
          IF(HPAR(JPARM).EQ.PARKEL(I))THEN
            IPAR=NPAR+I
            JPCPO=LCMGID(IPCPO,'MIXTURES')
            KPCPO=LCMGIL(JPCPO,IBMOLD)
            LPCPO=LCMGID(KPCPO,'TREE')
            IPARTM=IPAR-NAR
            PARNAM=HPAR(JPARM)
            GOTO 190
          ENDIF
        ENDDO
        WRITE(HSMG,'(18HNCRRGR: PARAMETER ,A,14H NOT FOUND(3).)')
     1  HPAR(JPARM)
        CALL XABORT(HSMG)
  190   CONTINUE
        WRITE(RECNAM,'(''pval'',I8.8)') IPARTM
        CALL LCMGET(LPCPO,'NVALUE',NVALUE)
        IF(NVALUE(IPARTM).GT.MAXVAL)CALL XABORT('NCRRGR: MAXVAL OVERFL'
     1   //'OW.')
        CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
        IF(LENGTH.EQ.0)THEN
          WRITE(HSMG,'(25HNCRRGR: GLOBAL PARAMETER ,A,9H NOT SET.)')
     1     PARNAM
          CALL XABORT(HSMG)
        ENDIF

  200   IF(JPARM.EQ.(NPARMP+1))THEN
*         parameter JPARAM is burn-up
          ITYPE=1
          MUTYPE(IPAR)=1
          MUPLET(IPAR)=-1
          IF(IBTYP.EQ.1)THEN
*           TIME-AVERAGE
            BURN0=BRN0(IB)
            BURN1=BRN1(IB)
          ELSEIF(IBTYP.EQ.2)THEN
*           INSTANTANEOUS
            BURN0=BRN0(IB)
            BURN1=BURN0
          ELSEIF(IBTYP.EQ.3)THEN
*           DIFFERENCIATION RELATIVE TO EXIT BURNUP
            ITYPE=3
            BURN0=BRN0(IB)
            BURN1=BRN1(IB)
          ENDIF
          VALR(IPAR,1)=BURN0
          VALR1=BURN0
          VALR(IPAR,2)=BURN1
          VALR2=BURN1
	ELSE
          IF(LDMAP(IPAR))THEN
            VALR(IPAR,1)=VALREF(IPAR)
            VALR1=VALREF(IPAR)
            VALR(IPAR,2)=WPAR(IB,JPARM)
            VALR2=WPAR(IB,JPARM)
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=2
            ITYPE=2
          ELSEIF(.NOT.LSET(IPAR))THEN
            VALR(IPAR,1)=WPAR(IB,JPARM)
            VALR(IPAR,2)=WPAR(IB,JPARM)
            VALR1=WPAR(IB,JPARM)
            VALR2=WPAR(IB,JPARM)
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=1
            ITYPE=1
          ENDIF
          IF(LAMAP(IPAR))THEN
            VALRA(IPAR,1,IDLTA(IPAR))=VALRFA(IPAR,IDLTA(IPAR))
            VALR1=VALRFA(IPAR,IDLTA(IPAR))
            VALRA(IPAR,2,IDLTA(IPAR))=WPAR(IB,JPARM)
            VALR2=WPAR(IB,JPARM)
            MAPLET(IPAR,IDLTA(IPAR))=-1
            MATYPE(IPAR,IDLTA(IPAR))=2
            ITYPE=2
          ENDIF
        ENDIF
        LPCPO=LCMGID(IPCPO,'GLOBAL')
        CALL LCMGET(LPCPO,'NVALUE',NVALUE)
        IF(NVALUE(IPARTM).GT.MAXVAL) CALL XABORT('NCRRGR: MAXVAL OVE'
     1  //'RFLOW.')
        WRITE(RECNAM,'(''pval'',I8.8)') IPARTM
        CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
        IF(LENGTH.EQ.0)THEN
          WRITE(HSMG,'(25HNCRRGR: GLOBAL PARAMETER ,A,9H NOT SET.)')
     1    PARNAM
          CALL XABORT(HSMG)
        ENDIF
        IF(LENGTH.GT.MAXVAL) CALL XABORT('NCRRGR: MAXVAL OVERFLOW.')
        CALL LCMGET(LPCPO,RECNAM,VREAL)
        IF(IMPY.GT.2)THEN
          WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1    1P,6E12.4/(35X,6E12.4))') PARNAM,(VREAL(I),I=1,
     2    NVALUE(IPAR))
        ENDIF
        IF(ITYPE.EQ.1)THEN
          IF(VALR1.EQ.VALR2)THEN
            DO J=1,NVALUE(IPARTM)
              IF(ABS(VALR1-VREAL(J)).LE.REPS*ABS(VREAL(J)))THEN
                MUPLET(IPAR)=J
                IF(ITYPE.NE.1) MUPLET(IPAR)=-1
                MUTYPE(IPAR)=ITYPE
                GOTO 260
              ENDIF
            ENDDO
          ENDIF
        ENDIF
*----
*  ERRORS HANDLING
*----
        IF(VALR1.LT.VREAL(1))THEN
*         OUTSIDE OF THE DOMAIN (1)
          WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1    1HE,1P,E12.4,26H IS OUTSIDE THE DOMAIN(3).)') PARNAM,VALR1
          WRITE(6,*)'Domain:',VREAL(1),' <-> ',VREAL(NVALUE(IPAR))
          CALL XABORT(HSMG)
        ELSEIF(VALR2.GT.VREAL(NVALUE(IPAR)))THEN
*         OUTSIDE OF THE DOMAIN (2)
          WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1    1HE,1P,E12.4,26H IS OUTSIDE THE DOMAIN(4).)') PARNAM,VALR2
          WRITE(6,*)'Domain:',VREAL(1),' <-> ',VREAL(NVALUE(IPAR))
          CALL XABORT(HSMG)
        ELSEIF((ITYPE.EQ.1).AND.(VALR1.GT.VALR2))THEN
*         VALR1 > VALR2
          WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,9H IS DEFIN,
     1    7HED WITH,1P,E12.4,2H >,E12.4,4H.(2))') PARNAM,
     2    VALR1,VALR2
          CALL XABORT(HSMG)
        ENDIF
*----
*  COMPUTE THE TERP FACTORS USING TABLE-OF-CONTENT INFORMATION.
*----
  260     CONTINUE
          MIXC(IB)=IBMOLD
          IF(IBMOLD.GT.NMIL)
     1       CALL XABORT('NCRRGR: MIX OVERFLOW (COMPO).')
          IF(IMPY.GT.2) WRITE(6,'(32H NCRRGR: COMPUTE TERP FACTORS IN,
     1    12H NEW MIXTURE,I5,1H.)') IB
          DO JPAR=1,NPAR+NLOC
            MUPLT2(JPAR)=MUPLET(JPAR)
          ENDDO
          IF(IBTYP.EQ.3)THEN
             IF(ZONEDP(ICH,JB).NE.0) THEN
                CALL NCRTRP(IPCPO,LCUBIC,IMPY,IBMOLD,NPAR,NLOC,NCAL,
     1                      MUPLT2,MUTYPE,VALR,VARVAL,TERP(1,IB))
             ELSE
                CALL XDRSET(TERP(1,IB),NCAL,0.0)
             ENDIF
          ELSE
             CALL NCRTRP(IPCPO,LCUBIC,IMPY,IBMOLD,NPAR,NLOC,NCAL,
     1                   MUPLT2,MUTYPE,VALR,VARVAL,TERP(1,IB))
          ENDIF
*         DELTA-ADD
          DO 270 IPAR=1,NPAR+NLOC
            IF(LADD(IPAR))THEN
              DO JPAR=1,NPAR+NLOC
                MUPLT2(JPAR)=MAPLET(JPAR,IDLTA(IPAR))
                MUTYP2(JPAR)=MATYPE(JPAR,IDLTA(IPAR))
              ENDDO
              DO JPAR=1,NPAR+NLOC
                IF(MUTYP2(JPAR).LT.0)THEN
                  MUPLT2(JPAR)=MUPLET(JPAR)
                  MUTYP2(JPAR)=MUTYPE(JPAR)
                  VALRA(JPAR,1,IDLTA(IPAR))=VALR(JPAR,1)
                  VALRA(JPAR,2,IDLTA(IPAR))=VALR(JPAR,2)
                ENDIF
              ENDDO
              CALL SETARA(BASE,NCAL,ITERPA)
              CALL NCRTRP(IPCPO,LCUBIC,IMPY,IBMOLD,NPAR,NLOC,NCAL,
     1        MUPLT2,MUTYP2,VALRA(1,1,IDLTA(IPAR)),VARVAL,BASE(ITERPA))
              DO 275 JCAL=1,NCAL
  275         TERP(JCAL,IB)=TERP(JCAL,IB)+BASE(ITERPA+JCAL-1)
              CALL RLSARA(BASE(ITERPA))
            ENDIF
  270     CONTINUE
        ENDIF
  280   CONTINUE
        GOTO 20
      ELSEIF((TEXT12.EQ.'TABLE').OR.(TEXT12.EQ.';'))THEN
*----
*  BUILD THE MICROLIB AND RETURN
*----
        IF(TEXT12.EQ.';') ITER=0
        IF(TEXT12.EQ.'TABLE') ITER=1
        IF(IACCS.EQ.0)THEN
          MAXISO=MAXISD*NMIX
        ELSE
          CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
          MAXISO=MAX(MAXISD*NMIX,ISTATE(2))
        ENDIF
        CALL SETARA(IBASE,3*MAXISO,IHUSE1)
        CALL SETARA(IBASE,3*MAXISO,IHNAM1)
        CALL SETARA(BASE,MAXISO,IDENS1)
        CALL SETARA(IBASE,MAXISO,IITYP1)
        CALL SETARA(IBASE,MAXISO,IITOD1)
        CALL SETARA(BASE,MAXISO,ITEMP1)
        CALL SETARA(BASE,MAXISO,IVOL1)
        CALL SETARA(IBASE,3*MAXISO,IHUSE2)
        CALL SETARA(IBASE,3*MAXISO,IHNAM2)
        CALL SETARA(BASE,MAXISO,IDENS2)
        CALL SETARA(IBASE,MAXISO,IIMIX2)
        CALL SETARA(IBASE,MAXISO,IITYP2)
        CALL SETARA(IBASE,MAXISO,IITOD2)
        CALL SETARA(BASE,MAXISO,ITEMP2)
        CALL SETARA(BASE,MAXISO,IVOL2)
        CALL SETARA(BASE,NGRP,IWORK)
        CALL SETARA(IBASE,MAXISO,IPIFI)
        CALL SETARA(IBASE,MAXISO,ISW)
        CALL SETARA(BASE,NGRP+1,IENER)
        CALL SETARA(BASE,NGRP,IDELT)
        CALL SETARA(BASE,NCAL,IFACT)
        CALL SETARA(IBASE,NMIX,IMILVO)
        CALL NCRLIB(MAXISO,IPLIB,IPCPO,IACCS,NMIX,NGRP,IMPY,NCAL,
     1   TERP,NISO,LISO,HISO,CONC,MIXC,LDELTA,LXS,LRES,BASE(IHUSE1),
     2   BASE(IHNAM1),BASE(IDENS1),BASE(IITYP1),BASE(IITOD1),
     3   BASE(ITEMP1),BASE(IVOL1),BASE(IHUSE2),BASE(IHNAM2),
     4   BASE(IDENS2),BASE(IIMIX2),BASE(IITYP2),BASE(IITOD2),
     5   BASE(ITEMP2),BASE(IVOL2),BASE(IWORK),BASE(IPIFI),BASE(ISW),
     6   BASE(IENER),BASE(IDELT),BASE(IFACT),BASE(IMILVO))
        CALL RLSARA(IBASE(IMILVO))
        CALL RLSARA(BASE(IFACT))
        CALL RLSARA(BASE(IDELT))
        CALL RLSARA(BASE(IENER))
        CALL RLSARA(IBASE(ISW))
        CALL RLSARA(IBASE(IPIFI))
        CALL RLSARA(BASE(IWORK))
        CALL RLSARA(BASE(IVOL2))
        CALL RLSARA(BASE(ITEMP2))
        CALL RLSARA(IBASE(IITOD2))
        CALL RLSARA(IBASE(IITYP2))
        CALL RLSARA(IBASE(IIMIX2))
        CALL RLSARA(BASE(IDENS2))
        CALL RLSARA(IBASE(IHNAM2))
        CALL RLSARA(IBASE(IHUSE2))
        CALL RLSARA(BASE(IVOL1))
        CALL RLSARA(BASE(ITEMP1))
        CALL RLSARA(IBASE(IITOD1))
        CALL RLSARA(IBASE(IITYP1))
        CALL RLSARA(BASE(IDENS1))
        CALL RLSARA(IBASE(IHNAM1))
        CALL RLSARA(IBASE(IHUSE1))
*----
*  EXIT MAIN LOOP OF THE SUBROUTINE
*----
        RETURN
      ELSE
        CALL XABORT('NCRRGR: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GOTO 20
      END
