*DECK NXTACG
      SUBROUTINE NXTACG(IPGEO ,IPTRK ,IPRINT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To analyze an assembly of cells containing
* clusters using the new EXCELL type procedure.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPGEO   pointer to the GEOMETRY data structure in 
*         read only mode.
* IPTRK   pointer to the TRACKING data structure in 
*         update or creation mode.
* IPRINT  print level.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPGEO,IPTRK
      INTEGER          IPRINT
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTACG')
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          ITSYM,IDFEX,IDFRT,IUNFLD,
     >                 NAGGEO,ITURN,MERGE,DCMESH,DGMESH,
     >                 ITGEO,IMGEO,ITPIN,ISPLT,MIX,
     >                 DAMESH,DRAPIN,RMESH,RPIN,APIN,
     >                 KEYMRG,MATALB,SURVOL,
     >                 MATCOD,VOLUME,MATRT,KEYFLX
      INTEGER          IDIRR
*----
*  Local variables
*----
      INTEGER          NDIM,ITYPBC,ITYGEO,IDIRG
      INTEGER          ISTATG(NSTATE),ISTATT(NSTATE),IEDIMG(NSTATE)
      INTEGER          ILEAK,IDIAG,ISAXIS(3),NBOCEL,NBUCEL,
     >                 NOCELL(3),NUCELL(3),MAXCEL
      INTEGER          IDIR,IMCELL,ILCELL,MAXPIN,MAXMSP,
     >                 MAXMSH,MAXREG,NBTCLS
      INTEGER          NFSUR,NFREG,MXGSUR,MXGREG,IANIS,NBUNK,IUNK,IKEY
      CHARACTER        NAMREC*12
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) NAMSBR
        WRITE(IOUT,6010)
      ENDIF
*----
*  Get state vectors
*----
      CALL XDISET(ISTATG,NSTATE,0)
      CALL LCMGET(IPGEO,'STATE-VECTOR',ISTATG)
      CALL XDISET(ISTATT,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATT)
*----
*  Get geometry state vector and test if geometry
*  is valid for NXT.
*  Valid geometries are:
*  Cartesian Boundary (ITYPBC=0):
*    CAR2D (5), CAR3D (7),
*    CARCEL (20), CARCELX (21), CARCELY (22), CARCELZ (23)
*  Annular Boundary (ITYPBC=1):  
*    TUBE (3), TUBEX (10), TUBEY (11), TUBEZ (6)
*----
      NDIM=2
      IDIRG=3
      ITYPBC=0
      ITYGEO=ISTATG(1)
      IF(ITYGEO .EQ. 5) THEN
        IDIRG=1
      ELSE IF(ITYGEO .EQ. 7) THEN
        NDIM=3
        IDIRG=1
      ELSE IF(ITYGEO .EQ. 20) THEN
        IDIRG=1
      ELSE IF(ITYGEO .EQ. 21) THEN
        NDIM=3
        IDIRG=2
      ELSE IF(ITYGEO .EQ. 22) THEN
        NDIM=3
      ELSE IF(ITYGEO .EQ. 23) THEN
        NDIM=3
        IDIRG=1
      ELSE IF(ITYGEO .EQ. 3) THEN
        IDIRG=1
        ITYPBC=1
      ELSE IF(ITYGEO .EQ. 6) THEN
        NDIM=3
        IDIRG=1
        ITYPBC=1
      ELSE IF(ITYGEO .EQ. 10) THEN
        NDIM=3
        IDIRG=2
        ITYPBC=1
      ELSE IF(ITYGEO .EQ. 11) THEN
        NDIM=3
        ITYPBC=1
      ELSE
        CALL XABORT(NAMSBR//
     >  ': Module cannot analyze this geometry')
      ENDIF
      IMCELL=ISTATG(8)
      IF(IMCELL .EQ. 0) THEN
        NOCELL(1)=1
        NOCELL(2)=1
        NOCELL(3)=1
        IF(NDIM .LT. 3) NOCELL(3)=0
        ILCELL=0
      ELSE
        NOCELL(1)=ISTATG(3)
        NOCELL(2)=ISTATG(4)
        NOCELL(3)=ISTATG(5)
        ILCELL=1
      ENDIF
*----
*  Read and analyze boundary conditions
*----
      CALL NXTBCG(IPGEO ,IPTRK ,IPRINT,NDIM  ,ITYPBC,IDIRG ,
     >            IDIAG ,ISAXIS,ILEAK ,ISTATT(39))
      IF(IDIAG .NE. 0) THEN
        IF(NOCELL(1) .NE. NOCELL(2)) CALL XABORT(NAMSBR//
     >  ': DIAG requires symmetric X-Y mesh.')
      ENDIF
      CALL XDISET(IEDIMG,NSTATE,0)
      IEDIMG(1)=NDIM
      IEDIMG(2)=ITYPBC
      IEDIMG(3)=IDIRG
*----
*  Compute global mesh in each direction and
*  total number of cell after unfolding of geometry
*----
      IF(IDIAG .EQ. 0) THEN
        NBOCEL=1
        DO IDIR=1,3
          NBOCEL=NBOCEL*MAX(NOCELL(IDIR),1)
        ENDDO
      ELSE
        NBOCEL=MAX(NOCELL(1),1)
        NBOCEL=(NBOCEL*(NBOCEL+1))/2
        NBOCEL=NBOCEL*MAX(NOCELL(3),1)
      ENDIF
      NBUCEL=1
      MAXCEL=1
      DO IDIR =1,3
        IF(ABS(ISAXIS(IDIR)) .EQ. 1) THEN
          NUCELL(IDIR)=2*NOCELL(IDIR)-1
        ELSE IF (ABS(ISAXIS(IDIR)) .EQ. 2) THEN
          NUCELL(IDIR)=2*NOCELL(IDIR)
        ELSE
          NUCELL(IDIR)=NOCELL(IDIR)
        ENDIF
        NBUCEL=NBUCEL*MAX(NUCELL(IDIR),1)
        MAXCEL=MAX(MAXCEL,NUCELL(IDIR))
      ENDDO
*----
*  Create Array for testing symmetry and unfolding the
*  geometry according to symmetries
*----
      IEDIMG(4)=NBOCEL
      IEDIMG(5)=NBUCEL
      IEDIMG(6)=IDIAG
      IEDIMG(7)=ISAXIS(1)
      IEDIMG(8)=ISAXIS(2)
      IEDIMG(9)=ISAXIS(3)
      CALL SETARA(IBASE,7*NBOCEL,IDFEX)
      CALL SETARA(IBASE,6*NBOCEL,IDFRT)
      CALL SETARA(IBASE,4*NBOCEL,ITSYM)
      CALL SETARA(IBASE,2*NBUCEL,IUNFLD)
      CALL NXTCUA(IPRINT,NDIM  ,IDIAG ,ISAXIS,
     >            NBOCEL,NBUCEL,NOCELL,NUCELL,IBASE(ITSYM) ,
     >            IBASE(IDFEX) ,IBASE(IDFRT) ,IBASE(IUNFLD))
*----
*  Save cell unfolding and internal symmetrization
*  vectors 
*----
      CALL LCMSIX(IPTRK,'NXTRecords  ',1)
      NAMREC='G00000001CUF'
      CALL LCMPUT(IPTRK,NAMREC,2*NBUCEL,1,IBASE(IUNFLD))
      NAMREC='G00000001CIS'
      CALL LCMPUT(IPTRK,NAMREC,4*NBOCEL,1,IBASE(ITSYM))
      NAMREC='G00000001CFE'
      CALL LCMPUT(IPTRK,NAMREC,7*NBOCEL,1,IBASE(IDFEX))
      IEDIMG(10)=NOCELL(1)
      IEDIMG(11)=NOCELL(2)
      IEDIMG(12)=NOCELL(3)
      IEDIMG(13)=NUCELL(1)
      IEDIMG(14)=NUCELL(2)
      IEDIMG(15)=NUCELL(3)
      IEDIMG(16)=MAXCEL
*----
*  Get maximum dimensions and geometry limits
*  Test if assembly can be built and save global mesh for geometry.
*  Test if cluster pins are valid.
*----
      CALL SETARA(IBASE,3*NBOCEL,NAGGEO)
      CALL SETARA(IBASE,NBOCEL,IDIRR)
      CALL SETARA(IBASE,NBOCEL,ITURN)
      CALL SETARA(IBASE,NBOCEL,MERGE)
      CALL SETARA(RBASE,2*3*2*NBOCEL,DCMESH)
      CALL SETARA(RBASE,2*(MAXCEL+1)*3,DGMESH)
      CALL NXTGMD(IPGEO ,IPTRK ,IPRINT,ILCELL,NSTATE,
     >            NBOCEL,NBUCEL,MAXCEL,NUCELL,IBASE(IUNFLD),
     >            IEDIMG,IBASE(NAGGEO),IBASE(ITURN),
     >            IBASE(MERGE),IBASE(IDIRR),RBASE(DCMESH),
     >            RBASE(DGMESH))
      CALL RLSARA(RBASE(DGMESH))
*----
*  Allocate memory to read global information
*  for each geometry and subgeometry
*----
      MAXMSH=IEDIMG(16)
      MAXREG=IEDIMG(17)
      NBTCLS=IEDIMG(18)
      MAXPIN=IEDIMG(19)
      MAXMSP=MAX(IEDIMG(20),IEDIMG(16),1)
*----
*  Create multicell description of geometry
*----
      CALL SETARA(IBASE,NBOCEL,ITGEO)
      CALL SETARA(IBASE,NBOCEL,IMGEO)
      CALL SETARA(IBASE,MAXPIN*3,ITPIN)
      CALL SETARA(IBASE,MAXMSH*4,ISPLT)
      CALL SETARA(IBASE,MAXREG,MIX)
      CALL SETARA(RBASE,2*(MAXMSH+2)*4,DAMESH)
      CALL SETARA(RBASE,2*6*MAXPIN,DRAPIN)
      CALL SETARA(RBASE,MAXMSH+1,RMESH)
      CALL SETARA(RBASE,MAXPIN,RPIN)
      CALL SETARA(RBASE,MAXPIN,APIN)
      CALL NXTMCD(IPGEO ,IPTRK ,IPRINT,NDIM  ,ITYPBC,
     >            IDIAG ,ISAXIS,ILCELL,NBOCEL,NBUCEL,NOCELL,
     >            NUCELL,MAXMSH,MAXREG,MAXPIN,NBTCLS,
     >            IBASE(ITSYM) ,
     >            IBASE(IDFEX) ,IBASE(IUNFLD),RBASE(DCMESH),
     >            IBASE(NAGGEO),IBASE(ITURN) ,IBASE(MERGE) ,
     >            IBASE(IDIRR),IBASE(ITPIN) ,
     >            NFSUR ,NFREG ,MXGSUR,MXGREG,
     >            IBASE(ISPLT) ,IBASE(MIX)   ,
     >            RBASE(DAMESH),RBASE(DRAPIN),
     >            RBASE(RMESH) ,RBASE(RPIN) ,RBASE(APIN))
      IEDIMG(22)=NFSUR
      IEDIMG(23)=NFREG
      IEDIMG(24)=MXGSUR
      IEDIMG(25)=MXGREG
      CALL RLSARA(RBASE(APIN))
      CALL RLSARA(RBASE(RPIN))
      CALL RLSARA(RBASE(RMESH))
      CALL RLSARA(RBASE(DRAPIN))
      CALL RLSARA(RBASE(DAMESH))
      CALL RLSARA(IBASE(MIX))
      CALL RLSARA(IBASE(ISPLT))
      CALL RLSARA(IBASE(ITPIN))
      CALL RLSARA(IBASE(IMGEO))
      CALL RLSARA(IBASE(ITGEO))
*----
*  Compute surfaces and volumes and find define MATALB
*----
      CALL SETARA(IBASE,NFREG,MATCOD)
      CALL SETARA(RBASE,NFREG,VOLUME)
      CALL SETARA(IBASE,NFSUR+NFREG+1,KEYMRG)
      CALL SETARA(IBASE,NFSUR+NFREG+1,MATALB)
      CALL SETARA(RBASE,2*(NFSUR+NFREG+1),SURVOL)
      CALL NXTCVS(IPTRK ,IPRINT,NDIM  ,NBOCEL,
     >            NFSUR ,NFREG ,MXGSUR,MXGREG,
     >            IBASE(KEYMRG),IBASE(MATALB),RBASE(SURVOL),
     >            IBASE(MATCOD),RBASE(VOLUME))
      CALL RLSARA(RBASE(SURVOL))
      CALL RLSARA(IBASE(MATALB))
      CALL RLSARA(IBASE(KEYMRG))
*----
*  Create BC-REFL+TRAN vector
*----
      CALL SETARA(IBASE,NFSUR,MATRT)
      CALL SETARA(RBASE,2*6*MAXPIN*2,DRAPIN)
      CALL SETARA(RBASE,2*(MAXMSP+2)*4*2,DAMESH)
      CALL NXTBRT(IPTRK ,IPRINT,NDIM  ,ISAXIS,NBOCEL,NOCELL,
     >            MAXMSP,MAXPIN,NBTCLS,NFSUR ,MXGSUR,MXGREG,
     >            IBASE(IDFRT) ,IBASE(MATRT) ,
     >            RBASE(DAMESH),RBASE(DRAPIN))
      CALL RLSARA(RBASE(DAMESH))
      CALL RLSARA(RBASE(DRAPIN))
*----
*  Save global tracking information
*----
      CALL LCMPUT(IPTRK,'G00000001DIM',NSTATE,1,IEDIMG)
      CALL LCMSIX(IPTRK,'NXTRecords  ',2)
      CALL LCMPUT(IPTRK,'BC-REFL+TRAN',NFSUR,1,IBASE(MATRT))
      CALL LCMPUT(IPTRK,'MATCOD      ',NFREG,1,IBASE(MATCOD))
      CALL LCMPUT(IPTRK,'VOLUME      ',NFREG,2,RBASE(VOLUME))
      CALL RLSARA(IBASE(MATRT))
      CALL RLSARA(RBASE(VOLUME))
      CALL RLSARA(RBASE(MATCOD))
      NBUNK=0
      DO IANIS=0,ISTATT(6)-1
        NBUNK=NBUNK+2*IANIS+1
      ENDDO      
      ISTATT(2)=NFREG*NBUNK
      CALL SETARA(IBASE,NFREG,KEYFLX)
      IUNK=1
      DO IKEY=0,NFREG-1
        IBASE(KEYFLX+IKEY)=IUNK
        IUNK=IUNK+1
      ENDDO
      CALL LCMPUT(IPTRK ,'KEYFLX      ',NFREG,1,IBASE(KEYFLX))
      ISTATT(3)=ILEAK
      ISTATT(1)=NFREG
      ISTATT(4)=ISTATG(7)
      ISTATT(5)=NFSUR
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,ISTATT)
*----
*  Release memory
*----
      CALL RLSARA(RBASE(DCMESH))
      CALL RLSARA(IBASE(MERGE))
      CALL RLSARA(IBASE(ITURN))
      CALL RLSARA(IBASE(IDIRR))
      CALL RLSARA(IBASE(NAGGEO))
      CALL RLSARA(IBASE(IUNFLD))
      CALL RLSARA(IBASE(ITSYM))
      CALL RLSARA(IBASE(IDFRT))
      CALL RLSARA(IBASE(IDFEX))
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6011) NFREG,NFSUR
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT(' Analysis of geometry begins')
 6011 FORMAT(' Analysis of geometry completed:'/
     >       ' Number of regions  =',I10/
     >       ' Number of surfaces =',I10)
      END 
