*DECK NXTBRT
      SUBROUTINE NXTBRT(IPTRK ,IPRINT,NDIM  ,ISAXIS,NBOCEL,NOCELL,
     >                  MAXMSP,MAXPIN,NBTCLS,NFSUR ,MXGSUR,MXGREG,
     >                  IDFRT ,MATRT ,DAMESH,DRAPIN)
*
*----------
*
*Purpose:
* To built the surface reflection/transmission coupling 
* array.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau.
*
*Update(s): 
* 2005/09/29: Validated with DRAGON and documented in
*             Report IGE-260.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPTRK   pointer to the TRACKING data structure.
* IPRINT  print level.
* NDIM    problem dimensions.
* ISAXIS  symmetry vector for each direction.
* NBOCEL  number of cells in original geometry.
* NOCELL  number of cell before unfolding in 
*         $X$, $Y$ and $Z$ directions.
* MAXMSP  maximum number of elements in MESH array.
* MAXPIN  maximum number of pins in clusters.
* NBTCLS  total number of cluster geometry.
* IDFRT   identify reflection/transmission faces.
* NFSUR   final number of surfaces.
* MXGSUR  maximum number of surfaces for any geometry.
* MXGREG  maximum number of region for any geometry.
*
*Parameters: output
* MATRT   reflection/transmission surface coupling array.
*
*Parameters: temporary storage
* DRAPIN  temporary vector for storing global pin positions.
* DAMESH  temporary vector for storing global mesh array.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPTRK
      INTEGER          IPRINT,NDIM,ISAXIS(3),NBOCEL,NOCELL(3),
     >                 MAXMSP,MAXPIN,NBTCLS,IDFRT(6,NBOCEL),
     >                 NFSUR,MXGSUR,MXGREG
      INTEGER          MATRT(NFSUR)
      DOUBLE PRECISION DAMESH(-1:MAXMSP,4,2),DRAPIN(-1:4,MAXPIN,2)     
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTBRT')
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          ID1,IX1,SV1,ID2,IX2,SV2
*----
*  Local variables
*----
      INTEGER          ISV,IDT,IDIR,ICEL,IGEN(2),ILEV,IG,NBSD,NBST,
     >                 NR1,NS1,NUNK1,IG1,ICL1,IPIN1,IFPIN1,ILPIN1,
     >                 NR2,NS2,NUNK2,IG2,ICL2,IPIN2,IFPIN2,ILPIN2,
     >                 MXRUNK,IDO
      INTEGER          IEDIMG(NSTATE,2),IEDIMC(NSTATE,2)
      CHARACTER        NAMREC*12 
      INTEGER          ILCMLN,ILCMTY
*----
*  Data
*----
      CHARACTER        CDIR(1:4)*1,CLEV(2)*1
      SAVE             CDIR,CLEV
      DATA             CDIR /'X','Y','Z','R'/
      DATA             CLEV /'C','P'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      MXRUNK=MXGSUR+MXGREG+1
      CALL SETARA(IBASE,MXGSUR,ID1)
      CALL SETARA(IBASE,4*MXRUNK,IX1)
      CALL SETARA(RBASE,2*MXRUNK,SV1)
      CALL SETARA(IBASE,MXGSUR,ID2)
      CALL SETARA(IBASE,4*MXRUNK,IX2)
      CALL SETARA(RBASE,2*MXRUNK,SV2)
*----
*  Initialize MATRT assuming all surfaces are reflective
*----
      DO ISV=1,NFSUR
        MATRT(ISV)=ISV
      ENDDO
*----
*  X, Y, and Z translation
*  Scan over cells and locate those with X- surface boundary
*  Find X+ cell from which neutrons are generated
*----
      DO IDT=1,3
        IDO=2*IDT-1
        IF(ISAXIS(IDT) .EQ. 3) THEN
          DO ICEL=1,NBOCEL
            IGEN(1)=ICEL
            ILEV=1
            IGEN(2)=IDFRT(IDO,ICEL)
            IF(IGEN(2) .GT. 0) THEN
*----
*  Cells are identified:
*  Extract dimensioning vectors.
*----
              CALL XDISET(IEDIMG,NSTATE*2,0)
              DO IG=1,2
                WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(IG),'DIM'
                CALL LCMGET(IPTRK,NAMREC,IEDIMG(1,IG))
*----
*  Read meshes
*----
                DO IDIR=1,4
                  WRITE(NAMREC,'(A1,I8.8,A3)') 
     >            CLEV(ILEV),IGEN(IG),'SM'//CDIR(IDIR)
                  CALL LCMLEN(IPTRK,NAMREC,ILCMLN,ILCMTY)
                  IF(ILCMLN .GT. 0) 
     >            CALL LCMGET(IPTRK,NAMREC,DAMESH(-1,IDIR,IG))
                ENDDO
                WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(IG),'PIN'
                CALL LCMLEN(IPTRK,NAMREC,ILCMLN,ILCMTY)
                IF(ILCMLN .GT. 0) 
     >          CALL LCMGET(IPTRK,NAMREC,DRAPIN(-1,1,IG))
              ENDDO
*----
*  Find maximum surfaces and regions
*  Allocate memory for MESH, DRAPIN, INDXSR, IDSUR and SURVOL 
*  and extract
*----
              NR1=IEDIMG(8,1)
              NS1=IEDIMG(9,1)
              NUNK1=NR1+NS1+1
              WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(1),'VSE'
              CALL LCMGET(IPTRK,NAMREC,RBASE(SV1))
              WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(1),'VSI'
              CALL LCMGET(IPTRK,NAMREC,IBASE(IX1))
              WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(1),'SID'
              CALL LCMGET(IPTRK,NAMREC,IBASE(ID1))
              NR2=IEDIMG(8,1)
              NS2=IEDIMG(9,1)
              NUNK2=NR2+NS2+1
              WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(2),'VSE'
              CALL LCMGET(IPTRK,NAMREC,RBASE(SV2))
              WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(2),'VSI'
              CALL LCMGET(IPTRK,NAMREC,IBASE(IX2))
              WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(2),'SID'
              CALL LCMGET(IPTRK,NAMREC,IBASE(ID2))
*----
*  Find equivalent translated surface
*----
              CALL NXTETS(IPRINT,IDT   ,ILEV  ,NFSUR ,MAXMSP,
     >                    NS1   ,NR1   ,NS2   ,NR2   ,IEDIMG,DAMESH,
     >                    IBASE(IX1),IBASE(ID1) ,RBASE(SV1),
     >                    IBASE(IX2),IBASE(ID2) ,RBASE(SV2),
     >                    MATRT ,NBSD  ,NBST  )
*----
*  For EACH pin in first geometry, find if a pin at an equivalent position
*  in second geometry can be found.
*----
              ILEV=2
              CALL XDISET(IEDIMC,NSTATE*2,0)
              IG1=1
              IG2=2
              IGEN(IG1)=IEDIMG(17,IG1)-1
              DO ICL1=1,IEDIMG(16,IG1)
                IGEN(IG1)=IGEN(IG1)+1
                IG=IG1
                WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(IG),'DIM'
                CALL LCMGET(IPTRK,NAMREC,IEDIMC(1,IG))
                IFPIN1=IEDIMC(16,IG)
                ILPIN1=IFPIN1+IEDIMC(17,IG)-1
                DO IDIR=1,4
                  WRITE(NAMREC,'(A1,I8.8,A3)') 
     >            CLEV(ILEV),IGEN(IG),'SM'//CDIR(IDIR)
                  CALL LCMLEN(IPTRK,NAMREC,ILCMLN,ILCMTY)
                  IF(ILCMLN .GT. 0) 
     >            CALL LCMGET(IPTRK,NAMREC,DAMESH(-1,IDIR,IG))
                ENDDO
                NR1=IEDIMC(8,1)
                NS1=IEDIMC(9,1)
                NUNK1=NR1+NS1+1
                WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(1),'VSE'
                CALL LCMGET(IPTRK,NAMREC,RBASE(SV1))
                WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(1),'VSI'
                CALL LCMGET(IPTRK,NAMREC,IBASE(IX1))
                WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(1),'SID'
                CALL LCMGET(IPTRK,NAMREC,IBASE(ID1))
                IGEN(IG2)=IEDIMG(17,IG2)-1
                DO ICL2=1,IEDIMG(16,IG2)
                  IGEN(IG2)=IGEN(IG2)+1
                  IG=IG2
                  WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(IG),'DIM'
                  CALL LCMGET(IPTRK,NAMREC,IEDIMC(1,IG))
                  IFPIN2=IEDIMC(16,IG)
                  ILPIN2=IFPIN2+IEDIMC(17,IG)-1
                  DO IDIR=1,4
                    WRITE(NAMREC,'(A1,I8.8,A3)') 
     >              CLEV(ILEV),IGEN(IG),'SM'//CDIR(IDIR)
                    CALL LCMLEN(IPTRK,NAMREC,ILCMLN,ILCMTY)
                    IF(ILCMLN .GT. 0) 
     >              CALL LCMGET(IPTRK,NAMREC,DAMESH(-1,IDIR,IG))
                  ENDDO
                  NR2=IEDIMC(8,IG2)
                  NS2=IEDIMC(9,IG2)
                  NUNK2=NR2+NS2+1
                  WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(2),'VSE'
                  CALL LCMGET(IPTRK,NAMREC,RBASE(SV2))
                  WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(2),'VSI'
                  CALL LCMGET(IPTRK,NAMREC,IBASE(IX2))
                  WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEN(2),'SID'
                  CALL LCMGET(IPTRK,NAMREC,IBASE(ID2))
*----
*  Find equivalent translated surface
*----
                  CALL NXTETS(IPRINT,IDT   ,ILEV  ,NFSUR ,MAXMSP,
     >                        NS1   ,NR1   ,NS2   ,NR2   ,IEDIMC,DAMESH,
     >                        IBASE(IX1),IBASE(ID1) ,RBASE(SV1),
     >                        IBASE(IX2),IBASE(ID2) ,RBASE(SV2),
     >                        MATRT ,NBSD  ,NBST  )
*----
*  No IDT directed face for direct cluster
*  go to next direct cluster
*----
                  IF(NBSD .EQ. 0) GO TO 105 
                  IF(NBSD .EQ. NBST) THEN
*----
*  Test if pin position locations are adequate
*----
                    DO IPIN1=IFPIN1,ILPIN1
                      DO IPIN2=IFPIN2,ILPIN2
                        IF(DRAPIN(-1,IPIN1,1) .EQ. DRAPIN(-1,IPIN2,2) 
     >               .AND. DRAPIN( 0,IPIN1,1) .EQ. DRAPIN( 0,IPIN2,2)
     >               .AND. DRAPIN( 4,IPIN1,1) .EQ. DRAPIN( 4,IPIN2,2)
     >                     ) THEN
*----
*   Pin positions are identical, select next pin
*----
                          GO TO 125
                        ENDIF
                      ENDDO
*----
*  Pin positions are not compatible
*  go to next translated cluster 
*----
                      GO TO 115
 125                  CONTINUE
                    ENDDO
*----
*   Translation surfaces found here go to next direct cluster
*----
                    GO TO 105
                  ENDIF
 115              CONTINUE
                ENDDO
*----
*  Translated surfaces for directed pin not found
*  send warning signal and continue
*----
                WRITE(IOUT,9000) ICEL,ICL1
 105            CONTINUE
              ENDDO
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      CALL RLSARA(IBASE(ID2))
      CALL RLSARA(IBASE(IX2))
      CALL RLSARA(RBASE(SV2))
      CALL RLSARA(IBASE(ID1))
      CALL RLSARA(IBASE(IX1))
      CALL RLSARA(RBASE(SV1))
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 9000 FORMAT(' ***** Warning ***** '/
     >       '       Translated surface for CELL ',I5,1X,
     >       ' and PIN :',I5,' is absent')
      END
