*DECK NXTEGI
      SUBROUTINE NXTEGI(IPGEO ,IPRINT,ITYPG ,MAXMSH,NMIX  ,NM    ,
     >                  MAXMSS,NMS   ,NREG  ,NREGS ,NSUR  ,NSURS ,
     >                  MIX   ,ISPLT ,DAMESH,
     >                  RMESH ,MIXC  )
*
*-----------------------------------------------------------------------
*
*Purpose:
* To extract cell or pin geometry information.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* 2005/09/29: Validated with DRAGON and documented in
*             Report IGE-260.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPGEO   pointer to the reference geometry data structure.
* IPRINT  intermediate printing level for output.
* ITYPG   geometry type.
* MAXMSH  maximum number of elements in MESH array.
* NMIX    number of elements in MIX array.
* NM      mesh size in all directions ($X$, $Y$, $Z$ and $R$)
*
*Parameters: output
* MAXMSS  maximum number of elements in MESH array after split.
* NMS     mesh size in all directions ($X$, $Y$, $Z$ and $R$)
*         after split.
* NREG    number of regions.
* NREGS   number of regions after split.
* NSUR    number of surfaces.
* NSURS   number of surfaces after split.
* MIX     final mixture description vector for geometry.
* ISPLT   final split desctiption for geometry.
* DAMESH  final mesh description for geometry.
*
*Parameters: temporary storage
* RMESH   temporary vector for reading cell mesh array.
* MIXC    temporary mixture vector for cell rotation.
*
*-----------------------------------------------------------------------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPGEO
      INTEGER          IPRINT,ITYPG,MAXMSH,NMIX,NM(4),MAXMSS,
     >                 NMS(4),NREG,NREGS,NSUR,NSURS
      INTEGER          MIX(NMIX),ISPLT(MAXMSH,4)
      DOUBLE PRECISION DAMESH(-1:MAXMSH,4)
      REAL             RMESH(0:MAXMSH)
      INTEGER          MIXC(NMIX)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTEGI')
      DOUBLE PRECISION DZERO,DONE
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0)
*----
*  Local variables
*----
      INTEGER          IDIR,IR,IX,IY,IZ,IMTN,IMTO
      INTEGER          ILCMLN,ILCMTY
      INTEGER          NX,NY,NZ,NR,NRM,NXS,NYS,NZS,NRS,NRMS,
     >                 NMREAD(4)
      CHARACTER        NAMREC*12
      REAL             OFFCEN(3)
*----
*  Data
*----
      CHARACTER        CDIR(4)*1
      SAVE             CDIR
      DATA             CDIR /'X','Y','Z','R'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      NX=MAX(1,NM(1))
      NY=MAX(1,NM(2))
      NZ=MAX(1,NM(3))
      NR=MAX(1,NM(4))
*----
*  Read geometry information
*  1- Cartesian MESH 
*----
      DO IDIR=1,3
        NMREAD(IDIR)=0
        IF(NM(IDIR) .GT. 0) THEN 
          NAMREC='MESH'//CDIR(IDIR)//'       '
          CALL LCMLEN(IPGEO,NAMREC,NMREAD(IDIR),ILCMTY)
          IF(NMREAD(IDIR) .EQ. NM(IDIR)+1) THEN
            CALL LCMGET(IPGEO,NAMREC,RMESH)
            DO IX=0,NM(IDIR)
              DAMESH(IX,IDIR)=DBLE(RMESH(IX))
            ENDDO
            CALL XDISET(ISPLT(1,IDIR),NM(IDIR),1)
          ENDIF
        ELSE
          DAMESH(0,IDIR)=DZERO
          IX=1
          DAMESH(IX,IDIR)=DONE
        ENDIF
      ENDDO
*----
*  2- Get off center
*----
      CALL XDRSET(OFFCEN,3,0.0)
      CALL LCMLEN(IPGEO,'OFFCENTER   ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0)
     >  CALL LCMGET(IPGEO,'OFFCENTER   ',OFFCEN)
      DO IDIR=1,3
        DAMESH(-1,IDIR)=DBLE(OFFCEN(IDIR))
      ENDDO
*----
*  3- Radial mesh
*----
      IDIR=4
      NMREAD(IDIR)=0
      NAMREC='RADIUS      '
      CALL LCMLEN(IPGEO,NAMREC,NMREAD(IDIR),ILCMTY)
      IF(NMREAD(IDIR) .EQ. NM(IDIR)+1) THEN
        CALL LCMGET(IPGEO,NAMREC,RMESH)
        DO IX=0,NM(IDIR)
          DAMESH(IX,IDIR)=DBLE(RMESH(IX))
        ENDDO
        CALL XDISET(ISPLT(1,IDIR),NM(IDIR),1)
      ENDIF
*----
*  4- Cartesian and radial split
*----
      DO IDIR=1,4
        IF(NM(IDIR) .GT. 0) THEN 
          NAMREC='SPLIT'//CDIR(IDIR)//'      '
          CALL LCMLEN(IPGEO,NAMREC,ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. NM(IDIR))
     >      CALL LCMGET(IPGEO,NAMREC,ISPLT(1,IDIR))
          NMS(IDIR)=0
          DO IR=1,NM(IDIR)
            NMS(IDIR)=NMS(IDIR)+ABS(ISPLT(IR,IDIR))
          ENDDO
        ELSE
          CALL XDISET(ISPLT(1,IDIR),NM(IDIR),1)
          NMS(IDIR)=NM(IDIR)
        ENDIF
      ENDDO
      NXS=MAX(1,NMS(1))
      NYS=MAX(1,NMS(2))
      NZS=MAX(1,NMS(3))
      NRS=MAX(1,NMS(4))
      IF(IPRINT .GE. 100) THEN 
        WRITE(IOUT,6010) ITYPG,(CDIR(IDIR),NM(IDIR),IDIR=1,4),NMIX
        DO IDIR=1,3
          IF(NMREAD(IDIR) .GT. 0) THEN
            WRITE(IOUT,6011) 'MESH'//CDIR(IDIR)//' ='
            WRITE(IOUT,6012) (DAMESH(IX,IDIR),IX=-1,NM(IDIR))
            WRITE(IOUT,6011) 'SPLT'//CDIR(IDIR)//' ='
            WRITE(IOUT,6013) (ISPLT(IX,IDIR),IX=1,NM(IDIR))
          ENDIF
        ENDDO 
        IDIR=4
        IF(NMREAD(IDIR) .GT. 0) THEN
          WRITE(IOUT,6011) 'RADIUS='
          WRITE(IOUT,6012) (DAMESH(IX,IDIR),IX=0,NM(IDIR))
          WRITE(IOUT,6011) 'SPLT'//CDIR(IDIR)//' ='
          WRITE(IOUT,6013) (ISPLT(IX,IDIR),IX=1,NM(IDIR))
        ENDIF
      ENDIF
*----
*  5- Get MIX 
*----
      NAMREC='MIX         '
      CALL LCMLEN(IPGEO,NAMREC,ILCMLN,ILCMTY)
      IF(ILCMLN .LT. 0 .OR. ILCMLN .GT. NMIX) CALL XABORT(NAMSBR//
     >': Size of MIX vector is invalid')
      IF(ILCMLN .GT. 0) THEN
        IF     (ITYPG .EQ.  3 ) THEN
          NRM=NR
          NSUR=NZ         
          NRMS=NRS
          NSURS=NZS 
          IF(ILCMLN .LT. NY*NX*NRM) THEN
            WRITE(IOUT,9000) NAMSBR,ILCMLN,NY*NX*NRM
            CALL XABORT(NAMSBR//
     >    ': Invalid number of mixtures provided')
          ENDIF 
          CALL LCMGET(IPGEO,NAMREC,MIX)
        ELSE IF(ITYPG .EQ.  6 ) THEN
          NRM=NR
          NSUR=2*(NRM*NX*NY)+NZ         
          NRMS=NRS
          NSURS=2*(NRMS*NXS*NYS)+NZS 
          IF(ILCMLN .LT. NZ*NY*NX*NRM) THEN
            WRITE(IOUT,9000) NAMSBR,ILCMLN,NZ*NY*NX*NRM
            CALL XABORT(NAMSBR//
     >    ': Invalid number of mixtures provided')
          ENDIF 
          CALL LCMGET(IPGEO,NAMREC,MIX)
        ELSE IF(ITYPG .EQ.  5) THEN
          NRM=1
          NSUR=2*(NY+NX)
          NRMS=1
          NSURS=2*(NYS+NXS)
          CALL LCMGET(IPGEO,NAMREC,MIX)
        ELSE IF(ITYPG .EQ.  7 ) THEN
          NRM=1
          NSUR=2*(NX*NY+NY*NZ+NZ*NX)
          NRMS=1
          NSURS=2*(NXS*NYS+NYS*NZS+NZS*NXS)
          CALL LCMGET(IPGEO,NAMREC,MIX)
        ELSE IF(ITYPG .EQ. 20) THEN
          NRM=NR+1
          NSUR=2*(NY+NX)
          NRMS=NRS+1
          NSURS=2*(NYS+NXS)
          IF(ILCMLN .LT. NY*NX*NRM) THEN
            WRITE(IOUT,9000) NAMSBR,ILCMLN,NY*NX*NRM
            CALL XABORT(NAMSBR//
     >    ': Invalid number of mixtures provided')
          ENDIF 
          CALL LCMGET(IPGEO,NAMREC,MIX)
        ELSE IF(ITYPG .EQ. 23 ) THEN
          NRM=NR+1
          NSUR=2*(NX*NY*NRM+NY*NZ+NZ*NX)
          NRMS=NRS+1
          NSURS=2*(NXS*NYS*NRMS+NYS*NZS+NZS*NXS)
          IF(ILCMLN .LT. NZ*NY*NX*NRM) THEN
            WRITE(IOUT,9000) NAMSBR,ILCMLN,NZ*NY*NX*NRM
            CALL XABORT(NAMSBR//
     >    ': Invalid number of mixtures provided')
          ENDIF 
          CALL LCMGET(IPGEO,NAMREC,MIX)
        ELSE IF(ITYPG .EQ. 10 .OR. ITYPG .EQ. 21) THEN
          IF(ITYPG .EQ.21) THEN
            NRM=NR+1
            NSUR=2*(NX*NY+NY*NZ*NRM+NZ*NX)
            NRMS=NRS+1
            NSURS=2*(NXS*NYS+NYS*NZS*NRMS+NZS*NXS)
          ELSE
            NRM=NR
            NSUR=NX+2*NY*NZ*NRM
            NRMS=NRS
            NSURS=NXS+2*NYS*NZS*NRMS
          ENDIF
*----
*  For CARCELX reorder mixtures from $(R,Y,Z,X)$ to $(R,X,Y,Z)$
*----
          IF(ILCMLN .LT. NZ*NY*NX*NRM) THEN
            WRITE(IOUT,9000) NAMSBR,ILCMLN,NZ*NY*NX*NRM
            CALL XABORT(NAMSBR//
     >    ': Invalid number of mixtures provided')
          ENDIF 
          CALL LCMGET(IPGEO,NAMREC,MIXC)
          IMTN=0
          DO 20 IZ=1,NZ
            DO 21 IY=1,NY
              DO 22 IX=1,NX
                DO 23 IR=1,NRM
                  IMTN=IMTN+1 
                  IMTO=(IX-1)*NY*NZ*NRM
     >                +(IZ-1)*NY*NRM
     >                +(IY-1)*NRM+IR
                  MIX(IMTN)=MIXC(IMTO)
 23             CONTINUE
 22           CONTINUE
 21         CONTINUE
 20       CONTINUE
        ELSE IF(ITYPG .EQ. 11 .OR. ITYPG .EQ. 22) THEN
          IF(ITYPG .EQ.22) THEN
            NRM=NR+1
            NSUR=2*(NX*NY+NY*NZ+NZ*NX*NRM)
            NRMS=NRS+1
            NSURS=2*(NXS*NYS+NYS*NZS+NZS*NXS*NRMS)
          ELSE
            NRM=NR
            NSUR=NY+2*NZ*NX*NRM
            NRMS=NRS
            NSURS=NYS+2*NZS*NXS*NRMS
          ENDIF
*----
*  For CARCELX reorder mixtures from $(R,Z,X,Y)$ to $(R,X,Y,Z)$
*----
          IF(ILCMLN .LT. NZ*NY*NX*NRM) THEN
            WRITE(IOUT,9000) NAMSBR,ILCMLN,NZ*NY*NX*NRM
            CALL XABORT(NAMSBR//
     >    ': Invalid number of mixtures provided')
          ENDIF 
          CALL LCMGET(IPGEO,NAMREC,MIXC)
          IMTN=0
          DO 30 IZ=1,NZ
            DO 31 IY=1,NY
              DO 32 IX=1,NX
                DO 33 IR=1,NRM
                  IMTN=IMTN+1 
                  IMTO=(IY-1)*NZ*NX*NRM
     >                +(IX-1)*NZ*NRM
     >                +(IZ-1)*NRM+IR
                  MIX(IMTN)=MIXC(IMTO)
 33             CONTINUE
 32           CONTINUE
 31         CONTINUE
 30       CONTINUE
        ELSE
          CALL XABORT(NAMSBR//
     >    ': Geometry type invalid for cell or pin') 
        ENDIF
      ENDIF
      NREG=NRM*NX*NY*NZ
      NREGS=NRMS*NXS*NYS*NZS
      MAXMSS=MAX(NRMS,NXS,NYS,NZS,MAXMSH)+1
*----
*  Print mesh if required
*----
      IF(IPRINT .GE. 100) THEN 
        WRITE(IOUT,6011) 'MIXT  ='
        WRITE(IOUT,6013) (MIX(IX),IX=1,NMIX)
        WRITE(IOUT,6001) NAMSBR
      ENDIF
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT(1X,'Geometry type =',I10/
     >       1X,'Original mesh dimensions ='/
     >       4(1X,A1,'=',1X,I8)/
     >       1X,'Number of regions =',i8)
 6011 FORMAT(1X,A7)
 6012 FORMAT(5F15.9)
 6013 FORMAT(5I15) 
 9000 FORMAT(' ***** Error in ',A6,' *****'/
     >       ' Number of mixtures provided = ',I10/
     >       ' Number of mixtures required = ',I10)
      END
