*DECK NXTGMD
      SUBROUTINE NXTGMD(IPGEO ,IPTRK ,IPRINT,ILCELL,NEDIM,
     >                  NBOCEL,NBUCEL,MAXCEL,NUCELL,IUNFLD,
     >                  IEDIMG,NAGGEO,ITURN ,MERGE ,IDIRR ,
     >                  DCMESH,DGMESH)
*
*----------
*
*Purpose:
* To obtain general dimension vector for geometry.
* To evaluate global mesh for assembly.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau.
*
*Update(s): 
* 2005/09/29: Validated with DRAGON and documented in
*             Report IGE-260.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*  This routine is based on the XELDCL routine written by
*  R. Roy for the EXCELT: module. It contains an additional
*  level for cluster subgeometry analysis.
*
*Parameters: input
* IPGEO   pointer to the GEOMETRY data structure.
* IPTRK   pointer to the TRACKING data structure.
* IPRINT  print level.
* ILCELL  cell level. 
* NEDIM  number of elements for general dimension vector.
* NBOCEL  number of cells in original geometry.
* NBUCEL  number of cells in unfolded geometry.
* MAXCEL  maximum number of elements in mesh vector for
*         each directions.
* NUCELL  number of cell after unfolding in 
*         $X$, $Y$ and $Z$ directions.
* IUNFLD  array to identify the generating cell (\moc{IUNFLD(1,*)})
*         and the rotation associated with this region in space.
*
*Parameters: input/output
* IEDIMG  general dimension vector for problem where
*         \begin{itemize} 
*         \item \moc{IEDIMG(1)}.
*         \end{itemize}
* NAGGEO  geometry names.
* ITURN   geometry turns.
* MERGE   geometry merge.
* IDIRR   direction of cell (1 for XYZ, 2 for YZX and 3 for ZXY)
*         Note: for CAR3D without pins IDIRR=1
*         for CAR3D with pins IDIRR specified by pins direction
* DCMESH  global mesh for each cell.
* DGMESH  meshing vector for global geometry.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPGEO,IPTRK
      INTEGER          IPRINT,ILCELL,NEDIM,NBOCEL,NBUCEL,MAXCEL,
     >                 NUCELL(3),IUNFLD(2,NBUCEL)
      INTEGER          IEDIMG(NEDIM),NAGGEO(3,NBOCEL),
     >                 ITURN(NBOCEL),MERGE(NBOCEL),IDIRR(NBOCEL)
      DOUBLE PRECISION DCMESH(3,2,NBOCEL),DGMESH(0:MAXCEL,3)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTGMD')
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
      DOUBLE PRECISION DCUT
      PARAMETER       (DCUT=1.0D-6)
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          IGEN,ISPLT,NAGCLS,RMESH,
     >                 PINDIM,APIN,RPIN,PINPOS,NAGCEL
*----
*  Local variables
*----
      INTEGER          ISTATG(NSTATE)
      INTEGER          NDIM,NSDIM,NCDIM,IDIRC
      INTEGER          MAXMSH,MAXREG,NBTCLS,
     >                 MAXPIN,MAXMSP,MAXRSP,NBGCLS,
     >                 NTPIN,NPIN,NBSP,NBRSP,NZU
      INTEGER          ITC,IDIR,ICEL,ICN,ISPL,ICLS,
     >                 IX,IY,IZ,IPOS,INTRN,IRTRN
      INTEGER          ILCMLN,ILCMTY
      CHARACTER        NAMGG*12,NAMREC*12,NAMCL*12
      REAL             OFFCEN(3) 
      DOUBLE PRECISION DRW(3),DNW(3)        
*----
*  Data
*----
      CHARACTER        CDIR(4)*1
      SAVE             CDIR
      DATA             CDIR /'X','Y','Z','R'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      NDIM=IEDIMG(1)
      MAXMSH=IEDIMG(16)
      MAXREG=1
      NBTCLS=0
      MAXPIN=0 
      MAXMSP=0
      MAXRSP=0
      CALL XDDSET(DGMESH,3*(MAXCEL+1),0.0D0)
      IF(ILCELL .EQ. 1) THEN
        CALL SETARA(IBASE,NBOCEL,IGEN)
        CALL SETARA(IBASE,3*NBOCEL,NAGCEL)
        CALL LCMGET(IPGEO,'CELL',IBASE(NAGCEL))
        CALL LCMGET(IPGEO,'MIX',IBASE(IGEN))
        DO ICEL=1,NBOCEL
          ICN=-IBASE(IGEN+ICEL-1)
          ICN=NAGCEL+3*(ICN-1)-1
          DO ITC=1,3
            NAGGEO(ITC,ICEL)=IBASE(ICN+ITC)
          ENDDO
        ENDDO
        CALL RLSARA(IBASE(NAGCEL))
        CALL RLSARA(IBASE(IGEN))
        CALL LCMLEN(IPGEO,'TURN',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          CALL LCMGET(IPGEO,'TURN',ITURN)
        ELSE
          CALL XDISET(ITURN,NBOCEL,1)
        ENDIF
        CALL LCMLEN(IPGEO,'MERGE',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
*----
*  Treatment of MERGE not yet implemented
*----
          CALL XABORT(NAMSBR//
     >    ': the module NXT cannot process geometry MERGE option')
*          CALL LCMGET(IPGEO,'MERGE',MERGE)
        ELSE
          DO ICEL=1,NBOCEL
            MERGE(ICEL)=ICEL
          ENDDO
        ENDIF
      ELSE
        NAMGG='            '
        READ(NAMGG,'(3A4)') (NAGGEO(ITC,1),ITC=1,3)
        ITURN(1)=1
        MERGE(1)=1
      ENDIF
      DO ICEL=1,NBOCEL
        IF(ILCELL .EQ. 1) THEN
          WRITE(NAMGG,'(3A4)') (NAGGEO(ITC,ICEL),ITC=1,3)
          CALL LCMSIX(IPGEO,NAMGG,1)
        ENDIF
        CALL XDISET(ISTATG,NSTATE,0)
        CALL LCMGET(IPGEO,'STATE-VECTOR',ISTATG)
        IRTRN=ITURN(ICEL)
*----
*  Find cell dimension and direction
*  Default is CAR3D
*----
        IDIRR(ICEL)=0
        NSDIM=3
        IF(ISTATG(1) .EQ. 5) THEN
          IDIRR(ICEL)=1
          NSDIM=2
        ELSE IF(ISTATG(1) .EQ. 20) THEN
          IDIRR(ICEL)=1
          NSDIM=2
        ELSE IF(ISTATG(1) .EQ. 21) THEN
          IDIRR(ICEL)=2
        ELSE IF(ISTATG(1) .EQ. 22) THEN
          IDIRR(ICEL)=3
        ELSE IF(ISTATG(1) .EQ. 23) THEN
          IDIRR(ICEL)=1
        ENDIF
        IF(NDIM .NE. NSDIM) CALL XABORT(NAMSBR//
     >  ': Geometry and sub-geometry must have the same dimensions')
        MAXMSH=MAX(MAXMSH,ISTATG(2),ISTATG(3),ISTATG(4),ISTATG(5))
        MAXREG=MAX(MAXREG,(ISTATG(2)+1)*
     >         MAX(ISTATG(3),1)*MAX(ISTATG(4),1)*MAX(ISTATG(5),1))
        CALL SETARA(IBASE,MAXMSH,ISPLT)
        NBRSP=1
        DO IDIR=1,4
          NAMREC='SPLIT'//CDIR(IDIR)//'      '
          CALL LCMLEN(IPGEO,NAMREC,ILCMLN,ILCMTY)
          IF(ILCMLN .GT. 0) THEN
            CALL LCMGET(IPGEO,NAMREC,IBASE(ISPLT))
            NBSP=0
            DO ISPL=1,ILCMLN
              NBSP=NBSP+ABS(IBASE(ISPLT+ISPL-1))
            ENDDO
*            NBRSP=NBRSP*(NBSP+1)
            IF(IDIR .EQ. 4) THEN
              NBRSP=NBRSP*(NBSP+1)
            ELSE
              NBRSP=NBRSP*NBSP
            ENDIF
            MAXMSP=MAX(MAXMSP,NBSP+1)
          ENDIF
        ENDDO 
        MAXRSP=MAX(MAXRSP,NBRSP)           
        CALL RLSARA(IBASE(ISPLT))
*----
*  Get off center
*----
        INTRN=1
        CALL XDRSET(OFFCEN,3,0.0)
        CALL LCMLEN(IPGEO,'OFFCENTER   ',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0)
     >  CALL LCMGET(IPGEO,'OFFCENTER   ',OFFCEN)
        DO IDIR=1,3
          DRW(IDIR)=OFFCEN(IDIR)
        ENDDO
        CALL NXTTRM(IRTRN,INTRN,DRW,DCMESH(1,2,ICEL))
*----
*  Get Cartesian mesh
*----
        CALL SETARA(RBASE,MAXMSH+1,RMESH)
        DO IDIR=1,3
          NAMREC='MESH'//CDIR(IDIR)//'       '
          CALL LCMLEN(IPGEO,NAMREC,ILCMLN,ILCMTY)
          IF(ILCMLN .GT. 0) THEN
            CALL LCMGET(IPGEO,NAMREC,RBASE(RMESH))
            DRW(IDIR)=DBLE(RBASE(RMESH+ILCMLN-1))
     >                  -DBLE(RBASE(RMESH))
          ENDIF
        ENDDO
        CALL RLSARA(RBASE(RMESH))
        CALL NXTTRM(IRTRN,INTRN,DRW,DCMESH(1,1,ICEL))
*----
*  Fill DGMESH according to IUNFLD
*----
        IPOS=0
        NZU=MAX(NUCELL(3),1)
        DO IZ=1,MAX(NUCELL(3),1)
          DO IY=1,NUCELL(2)
            DO IX=1,NUCELL(1)
              IPOS=IPOS+1
              IF(IUNFLD(1,IPOS) .EQ. ICEL) THEN
*----
*  cell is located at this position
*  Turn width according to symmetry
*----
                INTRN=IUNFLD(2,IPOS)
                CALL NXTTRM(IRTRN,INTRN,DRW,DNW)
                IF(DGMESH(IX,1) .EQ. 0.0D0) THEN
                  DGMESH(IX,1)=DNW(1)
                ELSE IF(ABS(DGMESH(IX,1)-DNW(1)) .GT. DCUT) THEN
                  CALL XABORT(NAMSBR//
     >            ': X Mesh in assembly is not uniform')
                ENDIF
                IF(DGMESH(IY,2) .EQ. 0.0D0) THEN
                  DGMESH(IY,2)=DNW(2)
                ELSE IF(ABS(DGMESH(IY,2)-DNW(2)) .GT. DCUT) THEN
                  CALL XABORT(NAMSBR//
     >            ': Y Mesh in assembly is not uniform')
                ENDIF
                IF(DGMESH(IZ,3) .EQ. 0.0D0) THEN
                  DGMESH(IZ,3)=DNW(3)
                ELSE IF(ABS(DGMESH(IZ,3)-DNW(3)) .GT. DCUT) THEN
                  CALL XABORT(NAMSBR//
     >            ': Z Mesh in assembly is not uniform')
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDDO
        IF(ISTATG(13) .GT. 0) THEN
          NBGCLS=ISTATG(13)
          NBTCLS=NBTCLS+NBGCLS
          CALL SETARA(IBASE,3*NBGCLS,NAGCLS)
          CALL LCMGET(IPGEO,'CLUSTER',IBASE(NAGCLS))
          NTPIN=0
          DO ICLS=1,NBGCLS
            ICN=NAGCLS+3*(ICLS-1)
            WRITE(NAMCL,'(3A4)') (IBASE(ICN+ITC),ITC=0,2)
            CALL LCMSIX(IPGEO,NAMCL,1)
            CALL XDISET(ISTATG,NSTATE,0)
            CALL LCMGET(IPGEO,'STATE-VECTOR',ISTATG)
*----
*  Find cluster dimension and direction
*----
            IDIRC=1
            NCDIM=3
            IF(ISTATG(1) .EQ. 3) THEN
              NCDIM=2
            ELSE IF(ISTATG(1) .EQ. 10) THEN
              IDIRC=2
            ELSE IF(ISTATG(1) .EQ. 11) THEN
              IDIRC=3
            ELSE IF(ISTATG(1) .EQ. 6) THEN
              IDIRC=1
            ELSE
              CALL XABORT(NAMSBR//': Invalid pin geometry'//
     >' -- Only TUBE* permitted')
            ENDIF 
            IF(NSDIM .NE. NCDIM) CALL XABORT(NAMSBR//
     >': Sub-geometry and clusters must have the same dimensions')
            IF(IDIRC .NE. IDIRR(ICEL)) THEN
              IF(IDIRR(ICEL) .NE. 0) CALL XABORT(NAMSBR//
     >': Pin and geometry direction are incompatible')
              IDIRR(ICEL)=IDIRC
            ENDIF
            MAXMSH=MAX(MAXMSH,ISTATG(2),ISTATG(3),ISTATG(4),ISTATG(5))
            MAXREG=MAX(MAXREG,ISTATG(6))
            CALL LCMGET(IPGEO,'NPIN',NPIN) 
            NTPIN=NTPIN+NPIN
            CALL SETARA(IBASE,MAXMSH,ISPLT)
            NBRSP=1
            DO IDIR=1,4
              NAMREC='SPLIT'//CDIR(IDIR)//'      '
              CALL LCMLEN(IPGEO,NAMREC,ILCMLN,ILCMTY)
              IF(ILCMLN .GT. 0) THEN
                CALL LCMGET(IPGEO,NAMREC,IBASE(ISPLT))
                NBSP=0
                DO ISPL=1,ILCMLN
                  NBSP=NBSP+ABS(IBASE(ISPLT+ISPL-1))
                ENDDO
                IF(IDIR .EQ. 4) THEN
                  NBRSP=NBRSP*(NBSP+1)
                ELSE
                  NBRSP=NBRSP*NBSP
                ENDIF
                MAXMSP=MAX(MAXMSP,NBSP+1)
              ENDIF
            ENDDO            
            MAXRSP=MAX(MAXRSP,NBRSP)           
            CALL RLSARA(IBASE(ISPLT))
            CALL LCMSIX(IPGEO,NAMCL,2)
          ENDDO
          MAXPIN=MAX(MAXPIN,NTPIN)           
*----
*  Now test if pin overlapp
*----
          CALL SETARA(RBASE,MAXMSH+1,PINDIM)
          CALL SETARA(RBASE,NTPIN,RPIN)
          CALL SETARA(RBASE,NTPIN,APIN)
          CALL SETARA(RBASE,2*3*NTPIN,PINPOS)
          CALL NXTTPO(IPGEO ,IPRINT,NBGCLS,NTPIN ,MAXMSH,NCDIM ,
     >                IDIRR(ICEL) ,DRW   ,OFFCEN,IBASE(NAGCLS),
     >                RBASE(PINDIM),RBASE(RPIN),RBASE(APIN),
     >                RBASE(PINPOS))          
          CALL RLSARA(RBASE(PINPOS))
          CALL RLSARA(RBASE(APIN))
          CALL RLSARA(RBASE(RPIN))
          CALL RLSARA(RBASE(PINDIM))
          CALL RLSARA(IBASE(NAGCLS))
        ENDIF
        IF(IDIRR(ICEL) .EQ. 0) IDIRR(ICEL)=1
        IF(ILCELL .EQ. 1) THEN
          CALL LCMSIX(IPGEO,NAMGG,2)
        ENDIF
      ENDDO
      IEDIMG(16)=MAXMSH
      IEDIMG(17)=MAXREG
      IEDIMG(18)=NBTCLS
      IEDIMG(19)=MAXPIN
      IEDIMG(20)=MAXMSP+1
      IEDIMG(21)=MAXRSP
      DO IDIR=1,3
        IF(NUCELL(IDIR) .GT. 0) THEN
          DO IX=1,NUCELL(IDIR)
            DGMESH(IX,IDIR)=DGMESH(IX-1,IDIR)+DGMESH(IX,IDIR)
          ENDDO
          NAMREC='G00000001SM'//CDIR(IDIR)
          CALL LCMPUT(IPTRK,NAMREC,2*(NUCELL(IDIR)+1),4,DGMESH(0,IDIR))
        ENDIF
      ENDDO
      CALL LCMPUT(IPTRK,'G00000001DIM',NEDIM,1,IEDIMG)
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
      END
