*DECK NXTMCA
      SUBROUTINE NXTMCA(IPTRK,IPRINT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* add MC: specific geometry analysis info to NXTRecords.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): Romain Le Tellier
*
*Parameters: input
* IPTRK   pointer to the Tracking data structure.
* IPRINT  print level.
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IPRINT
*----
*  LOCAL VARIABLES
*----
      INTEGER NSTATE
      PARAMETER(NSTATE=40)
      INTEGER NFREG,NMIX,NFSUR,NDIM,NBUCEL,NUCELL(3),MAXMSH,MAXMDH,
     1 MAXREG,NBTCLS,MAXPIN,MAXMSP,MAXRSP,MXGSUR,MXGREG,NUNK,JJ,IDIR
      INTEGER GSTATE(NSTATE),ESTATE(NSTATE)
      CHARACTER NAMREC*12,CDIR(4)*1
      DATA CDIR /'X','Y','Z','R'/
*----
*  SETARA POINTERS
*----
      INTEGER IUNFLD,DGMESH,INDEX,IDREG,IDSUR,DCMESH,ITPIN,DRAPIN
*----
*  DYNAMIC MEMORY
*----
      INTEGER IBASE(1)
      REAL BASE
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  RECOVER SOME BASIC NXT GEOMETRY ANALYSIS INFO AND ALLOCATE RELATED
*  MEMORY
*----     
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',GSTATE)
      NFREG    =GSTATE( 1)
      NMIX     =GSTATE( 4)
      NFSUR    =GSTATE( 5)
      IF (GSTATE(7).NE.4)
     1 CALL XABORT('NXTMCA: ONLY NXT: GEOMETRY ANALYSIS IS PERMITTED')
      CALL LCMSIX(IPTRK,'NXTRecords',1)
      CALL XDISET(ESTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'G00000001DIM',ESTATE)
      NDIM     =ESTATE( 1)
      NBUCEL   =ESTATE( 5)
      NUCELL(1)=ESTATE(13)
      NUCELL(2)=ESTATE(14)
      NUCELL(3)=ESTATE(15)
      MAXMSH   =ESTATE(16)
      MAXREG   =ESTATE(17)
      NBTCLS   =ESTATE(18)
      MAXPIN   =ESTATE(19)
      MAXMSP   =ESTATE(20)
      MAXRSP   =ESTATE(21)
      IF (NFSUR.NE.ESTATE(22))
     1 CALL XABORT('NXTMCA: INCONSISTENT NUMBER OF OUTER SURFACES')
      IF (NFREG.NE.ESTATE(23))
     1 CALL XABORT('NXTMCA: INCONSISTENT NUMBER OF REGIONS')
      MXGSUR   =ESTATE(24)
      MXGREG   =ESTATE(25)
      NUNK=NFSUR+NFREG+1
      MAXMDH=MAX(MAXMSH,MAXMSP,MAXREG)
*     cell index and orientation for the cells filling the geometry
      CALL SETARA(IBASE,2*NBUCEL,IUNFLD)
      NAMREC='G00000001CUF'
      CALL LCMGET(IPTRK,NAMREC,IBASE(IUNFLD))
*     global mesh for geometry
      CALL SETARA(BASE,2*(MAXMDH+2)*4,DGMESH)
      CALL XDDSET(BASE(DGMESH),(MAXMDH+2)*4,0.D0)
*     An offset of 2 has been used to be compatible with the definition
*     of the pin geometries where the first two values give the offset
*     of the pin.  With a cartesian geometry the array doesn't contain
*     the offests. 
      JJ=2
      DO IDIR=1,NDIM
        NAMREC='G00000001SM'//CDIR(IDIR)
        CALL LCMGET(IPTRK,NAMREC,BASE(DGMESH+JJ))
      ENDDO
      CALL SETARA(IBASE,2*4*(MXGSUR+MXGREG+1),INDEX)
      CALL SETARA(IBASE,2*MXGSUR,IDSUR)
      CALL SETARA(IBASE,2*MXGREG,IDREG)
      CALL SETARA(BASE,2*2*4*(MAXMDH+2),DCMESH)
      CALL SETARA(IBASE,3*(MAXPIN+1),ITPIN)
      CALL SETARA(BASE,6*2*(MAXPIN+1),DRAPIN)
*----
*  ADD MCA: SPECIFIC GEOMETRY ANALYSIS INFO TO NXTRecords
*----
      CALL NXTMCB(IPTRK,IPRINT,NFREG,NFSUR,NDIM,MAXMDH,NUCELL,NBUCEL,
     1     MXGSUR,MXGREG,MAXPIN,IBASE(IUNFLD),BASE(DGMESH),IBASE(INDEX),
     2     IBASE(IDSUR),IBASE(IDREG),BASE(DCMESH),IBASE(ITPIN),
     3     BASE(DRAPIN))
*----
*  RELEASE MEMORY
*----
      CALL RLSARA(BASE(DRAPIN))
      CALL RLSARA(IBASE(ITPIN))
      CALL RLSARA(BASE(DCMESH))
      CALL RLSARA(IBASE(IDREG))
      CALL RLSARA(IBASE(IDSUR))
      CALL RLSARA(IBASE(INDEX))
      CALL RLSARA(BASE(DGMESH))
      CALL RLSARA(IBASE(IUNFLD))
*     
      CALL LCMSIX(IPTRK,' ',2)
      
      RETURN
      END
