*DECK NXTMVS
      SUBROUTINE NXTMVS(IPTRK ,IPRINT,NDIM  ,NBOCEL,
     >                  NFSUR ,NFREG ,MXGSUR,MXGREG,IMERGE,
     >                  KEYMRG,MATALB,SURVOL,MATCOD,VOLUME)
*
*----------
*
*Purpose:
* To merge final surfaces and volumes for geometry
* and to create the EXCELL type MATALB and KEYMRG vector.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau.
*
*Update(s): 
*
*Reference: 
*
*Parameters: input
* IPTRK   pointer to the TRACKING data structure.
* IPRINT  print level.
* NDIM    problem dimensions.
* NBOCEL  number of cells in original geometry.
* NFSUR   final number of surfaces.
* NFREG   final number of regions.
* MXGSUR  maximum number of surfaces for any geometry.
* MXGREG  maximum number of region for any geometry.
* IMERGE  array of indices for merging regions and surfaces.
*
*Parameters: output
* KEYMRG  global merging vector.
* MATALB  global mixture/albedo identification vector.
* SURVOL  global surface volume vector.
* MATCOD  final mixture vector.
* VOLUME  final volume vector.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPTRK
      INTEGER          IPRINT,NDIM,NBOCEL,NFSUR,NFREG,MXGSUR,MXGREG
      INTEGER          IMERGE(-NFSUR:NFREG)
      INTEGER          KEYMRG(-NFSUR:NFREG),MATALB(-NFSUR:NFREG)
      DOUBLE PRECISION SURVOL(-NFSUR:NFREG)
      INTEGER          MATCOD(NFREG)
      REAL             VOLUME(NFREG)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTMVS')
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
      DOUBLE PRECISION DZERO,DONE,DTWO
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          MIX,INDXSR,SVSGEO,IDREG,IDSUR
*----
*  Local variables
*----
      INTEGER          ICEL,ICLS,ILEV,ISV,IGEO
      INTEGER          NREG,NSUR,NBGCLS,IGCLS,NUNK,MXRUNK
      INTEGER          IEDIMX(NSTATE),IEDIMP(NSTATE)
      CHARACTER        NAMREC*12 
      DOUBLE PRECISION DFACC,DFACP
*----
*  Data
*----
      CHARACTER        CLEV(2)*1
      SAVE             CLEV
      DATA             CLEV /'C','P'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      NUNK=NFSUR+NFREG+1
      MXRUNK=MXGSUR+MXGREG+1
      CALL XDDSET(SURVOL,NUNK,DZERO)
      CALL XDISET(MATALB,NUNK,0)
      CALL SETARA(IBASE,MXGREG,MIX)
      CALL SETARA(RBASE,4*MXRUNK,INDXSR)
      CALL SETARA(RBASE,2*4*MXRUNK,SVSGEO)
      CALL SETARA(IBASE,MXGREG,IDREG)
      CALL SETARA(IBASE,MXGSUR,IDSUR)
*----
*  Here there are no merge
*----
      DO ISV=-NFSUR,NFREG
        KEYMRG(ISV)=ISV
      ENDDO
      DO ICEL=1,NBOCEL
        ILEV=1
        IGEO=ICEL
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'DIM'
        CALL XDISET(IEDIMX,NSTATE,0)
        CALL LCMGET(IPTRK,NAMREC,IEDIMX)
        NREG=IEDIMX(8)
        NSUR=IEDIMX(9)
        NBGCLS=IEDIMX(16)
        IGCLS=IEDIMX(17)-1
*----
*   Get MIXTURE
*----
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'MIX'
        CALL LCMGET(IPTRK,NAMREC,IBASE(MIX))
*----
*  Get INDEX and SURVOL for pin
*----
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'VSI'
        CALL LCMGET(IPTRK,NAMREC,IBASE(INDXSR))
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'VSE'
        CALL LCMGET(IPTRK,NAMREC,RBASE(SVSGEO))
*----
*   Get IDREG and IDSUR
*----
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'RID'
        CALL LCMGET(IPTRK,NAMREC,IBASE(IDREG))
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'SID'
        CALL LCMGET(IPTRK,NAMREC,IBASE(IDSUR))
        DFACC=DBLE(IEDIMX(19))
        CALL NXTAVS(IPRINT,NDIM  ,NBOCEL,
     >              NFSUR ,NFREG ,NSUR  ,NREG  ,
     >              IBASE(MIX)   ,IBASE(INDXSR),IBASE(IDSUR) ,
     >              IBASE(IDREG) ,RBASE(SVSGEO),DFACC ,
     >              MATALB,SURVOL)
        IF(NBGCLS .NE. 0) THEN
          ILEV=2
          DO ICLS=1,NBGCLS
            IGEO=IGCLS+ICLS
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'DIM'
            CALL XDISET(IEDIMP,NSTATE,0)
            CALL LCMGET(IPTRK,NAMREC,IEDIMP)
            NREG=IEDIMP(8)
            NSUR=IEDIMP(9)
*----
*   Get MIXTURE
*----
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'MIX'
            CALL LCMGET(IPTRK,NAMREC,IBASE(MIX))
*----
*   Get INDEX and SURVOL for cell
*----
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'VSI'
            CALL LCMGET(IPTRK,NAMREC,IBASE(INDXSR))
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'VSE'
            CALL LCMGET(IPTRK,NAMREC,RBASE(SVSGEO))
*----
*   Get IDREG and IDSUR
*----
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'RID'
            CALL LCMGET(IPTRK,NAMREC,IBASE(IDREG))
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGEO,'SID'
            CALL LCMGET(IPTRK,NAMREC,IBASE(IDSUR))
            DFACP=DFACC*DBLE(IEDIMP(17))
            CALL NXTAVS(IPRINT,NDIM  ,NBOCEL,
     >                  NFSUR ,NFREG ,NSUR  ,NREG  ,
     >                  IBASE(MIX)   ,IBASE(INDXSR),IBASE(IDSUR) ,
     >                  IBASE(IDREG) ,RBASE(SVSGEO),DFACP ,
     >                  MATALB,SURVOL)
          ENDDO
        ENDIF
      ENDDO
*----
*  Save records on IPTRK
*----
      CALL LCMPUT(IPTRK ,'KEYMRG      ',NUNK  ,1,KEYMRG)
      CALL LCMPUT(IPTRK ,'MATALB      ',NUNK  ,1,MATALB)
      CALL LCMPUT(IPTRK ,'SAreaRvolume',2*NUNK,4,SURVOL)
      DO ISV=1,NFREG
        MATCOD(ISV)=MATALB(ISV)
        VOLUME(ISV)=SURVOL(ISV)
      ENDDO
      CALL RLSARA(IBASE(IDSUR))
      CALL RLSARA(IBASE(IDREG))
      CALL RLSARA(RBASE(SVSGEO))
      CALL RLSARA(IBASE(INDXSR))
      CALL RLSARA(IBASE(MIX))
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
      END
