*DECK NXTTCG
      SUBROUTINE NXTTCG(IPTRK ,IFTRK ,IPRINT,IGTRK ,NBSLIN)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To track an assembly of cells containing
* clusters using the new EXCELL type procedure.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPTRK   pointer to the TRACKING data structure in 
*         update or creation mode.
* IFTRK   pointer to the TRACKING file in 
*         creation mode.
* IPRINT  print level.
* IGTRK   flag to generate the tracking file. In the case where 
*         IGTRK=1, the tracking is performed and
*         used to evaluate the track normalisation factor and the 
*         tracking file is generated. When IGTRK=0, the tracking is
*         still performed and used to evaluate the 
*         track normalisation factor but the tracking file is not 
*         generated. 
* NBSLIN  maximum number of segments in a single tracking line.
*         computed by default in NXTTCG but limited to 100000
*         elements. This default value can be bypassed using
*         keyword NBSLIN.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPTRK,IFTRK
      INTEGER          IPRINT,IGTRK
      INTEGER          NBSLIN
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTTCG')
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
      DOUBLE PRECISION DZERO,DONE,DTWO
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
      INTEGER          MXGAUS
      PARAMETER       (MXGAUS=64)
*----
*  Functions
*----
      INTEGER          KDROPN,IFTEMP,KDRCLS,ICLS
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          DANGLT,DDENWT,MATALB,SURVOL,VOLSUR,DGMESH,
     >                 ANGLT,DENWT,DVNOR,DSNOR,NUMERO,LENGTH,DLENGT,
     >                 ICINT,DCINT,IUNFLD,DDANG,DNSANG,NBSANG,
     >                 DEPART
      INTEGER          DDIRET,DWGTRK,DORITR
*----
*  Local variables
*----
      INTEGER          ISTATE(NSTATE),IEDIMG(NSTATE),ICODE(6)
      REAL             RSTATT(NSTATE),ALBEDO(6)
      INTEGER          RENO,LTRK,AZMOAQ,ISYMM,POLQUA,POLOAQ,AZMQUA
      DOUBLE PRECISION DENUSR,RCUTOF,DENLIN,SPACLN
      REAL             WEIGHT
      DOUBLE PRECISION RADIUS,CENTER(3)
      INTEGER          NDIM,ITYPBC,IDIRG,NBOCEL,NBUCEL,IDIAG,
     >                 ISAXIS(3),NOCELL(3),NUCELL(3),MXMSH,MAXMSH,
     >                 MAXREG,NBTCLS,MAXPIN,MAXMSP,MAXRSP,NFSUR,
     >                 NFREG,MXGSUR,MXGREG,NUNK,
     >                 NPLANE,NPOINT,
     >                 NTLINE,NBTDIR,MAXSGL,NBDR
      INTEGER          IPER(3)
      INTEGER          JJ,NCOR,NQUAD,NANGL,NBANGL,LINMAX
      DOUBLE PRECISION DQUAD(4),ABSC(3,2),RCIRC
      CHARACTER        CTRK*4,COMENT*80,NAMREC*12
      REAL             XGSS(MXGAUS),WGSS(MXGAUS)
      INTEGER          IFMT
*----
*  Processing starts:
*  print routine opening header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
*----
*  Open temporary tracking file if required
*----
      IF(IGTRK .EQ. 1) THEN
        IFTEMP= KDROPN('DUMMYSQ',0,2,0,0)
        IF(IFTEMP .LE. 0) WRITE(IOUT,9010) NAMSBR
      ENDIF
*----
*  Get state vectors
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      CALL LCMGET(IPTRK,'EXCELTRACKOP',RSTATT)
      RENO  =ISTATE(8)
      LTRK  =ISTATE(9)
      AZMOAQ=ISTATE(11)
      ISYMM =ISTATE(12)
      AZMQUA=ISTATE(13)
      POLQUA=ISTATE(21)
      POLOAQ=ISTATE(22)
      DENUSR=DBLE(RSTATT(2))
      RCUTOF=DBLE(RSTATT(3))
      DENLIN=DBLE(RSTATT(4))
      SPACLN=DBLE(RSTATT(5))
      WEIGHT=RSTATT(6)
*----
*  Get main tracking records
*----
      CALL LCMGET(IPTRK,'ICODE       ',ICODE )
      CALL LCMGET(IPTRK,'ALBEDO      ',ALBEDO) 
      CALL LCMSIX(IPTRK,'NXTRecords  ',1)
*----
*  Get general dimensioning vector for geometry tracking
*----
      CALL XDISET(IEDIMG,NSTATE,0)
      CALL LCMGET(IPTRK,'G00000001DIM',IEDIMG)
      NDIM     =IEDIMG( 1)
      ITYPBC   =IEDIMG( 2)
      IDIRG    =IEDIMG( 3)
      NBOCEL   =IEDIMG( 4)
      NBUCEL   =IEDIMG( 5)
      IDIAG    =IEDIMG( 6)
      ISAXIS(1)=IEDIMG( 7)
      ISAXIS(2)=IEDIMG( 8)
      ISAXIS(3)=IEDIMG( 9)
      NOCELL(1)=IEDIMG(10)
      NOCELL(2)=IEDIMG(11)
      NOCELL(3)=IEDIMG(12)
      NUCELL(1)=IEDIMG(13)
      NUCELL(2)=IEDIMG(14)
      NUCELL(3)=IEDIMG(15)
      MXMSH    =IEDIMG(16)
      MAXREG   =IEDIMG(17)
      NBTCLS   =IEDIMG(18)
      MAXPIN   =IEDIMG(19)
      MAXMSP   =IEDIMG(20)
      MAXRSP   =IEDIMG(21)
      NFSUR    =IEDIMG(22)
      NFREG    =IEDIMG(23)
      MXGSUR   =IEDIMG(24)
      MXGREG   =IEDIMG(25)
      NUNK=NFSUR+NFREG+1
      MAXMSH=MAX(MXMSH,MAXMSP,MAXREG)
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6011) NFREG,NFSUR
      ENDIF
*----
*  Define Cell periodicity
*----
      IPER(1)=2
      IF(ABS(ISAXIS(1)) .EQ. 3) IPER(1)=1
      IPER(2)=2
      IF(ABS(ISAXIS(2)) .EQ. 3) IPER(2)=1
      IPER(3)=2
      IF(ABS(ISAXIS(3)) .EQ. 3) IPER(3)=1
*----
*  Use intrinsic geometry symmetries
*  to simplify tracking unless 
*  NOSY tracking option activated
*----
      IF(ISYMM .NE. 0) THEN
        ISYMM=0
        IF(ABS(ISAXIS(1)) .EQ. 1 .OR. ABS(ISAXIS(1)) .EQ. 2) THEN
*----
*  X SYMMETRY
*----
          ISYMM=2
        ENDIF
        IF(ABS(ISAXIS(2)) .EQ. 1 .OR. ABS(ISAXIS(2)) .EQ. 2) THEN
*----
*  Y SYMMETRY
*----
          ISYMM=4+2*ISYMM
        ENDIF
        IF(NDIM .EQ. 3) THEN
          IF(ABS(ISAXIS(3)) .EQ. 1 .OR. ABS(ISAXIS(3)) .EQ. 2) THEN
*----
*  Z SYMMETRY
*----
            ISYMM=16+ISYMM
          ENDIF
        ENDIF
        IF(ISYMM .EQ. 0) ISYMM=1
      ENDIF
*----
*  Get cell description of geometry
*----
      CALL SETARA(IBASE,2*NBUCEL,IUNFLD)
      NAMREC='G00000001CUF'
      CALL LCMGET(IPTRK,NAMREC,IBASE(IUNFLD))
      CALL SETARA(IBASE,NUNK  ,MATALB)
      CALL SETARA(RBASE,2*NUNK,SURVOL)
      CALL LCMGET(IPTRK,'MATALB      ',IBASE(MATALB))
      CALL LCMGET(IPTRK,'SAreaRvolume',RBASE(SURVOL))
*----
*  Read global mesh for geometry
*----
      CALL SETARA(RBASE,2*(MAXMSH+2)*4,DGMESH)
      CALL NXTXYZ(IPTRK ,IPRINT,NDIM  ,MAXMSH,NUCELL,
     >            ABSC,RBASE(DGMESH))
*----
*  Verify tracking parameters and compute number of angles
*  associated with angular order 
*  and spatial quadrature parameters
*  1. Isotropic tracking
*----
      NCOR= 1
      NPLANE=1
      IF(LTRK .EQ. 0) THEN
        IF(NDIM .EQ. 2) THEN
          NQUAD=2
          DQUAD(1)=DONE
          DQUAD(2)=DONE
          NANGL=AZMOAQ
          NBANGL=NANGL
          IF(ISYMM .EQ. 2 .OR. ISYMM .EQ. 8) THEN
            DQUAD(1)=DONE/DTWO
            DQUAD(2)=DZERO
          ENDIF
          CALL SETARA(RBASE,2*NDIM*NBANGL*NQUAD,DANGLT)
          CALL SETARA(RBASE,2*NBANGL*NQUAD,DDENWT)
          CALL NXTQAS(IPRINT,NDIM  ,AZMQUA,NANGL ,NQUAD ,NBANGL,
     >                DQUAD ,RBASE(DANGLT),RBASE(DDENWT))
        ELSE IF(NDIM .EQ. 3) THEN
          NQUAD=4
          NANGL=AZMOAQ
          IF(MOD(AZMOAQ,2) .EQ. 1)THEN
            WRITE(IOUT,9000) NAMSBR,NANGL,NANGL+1
            NANGL=NANGL+1
          ENDIF
          IF(AZMQUA .EQ. 1) THEN
            IF(AZMOAQ .GT. 16) THEN
              WRITE(IOUT,9001) NAMSBR
              AZMQUA=4
            ENDIF
            NBANGL=(NANGL * (NANGL+2)) / 8
          ENDIF
          IF(AZMQUA .EQ. 4) THEN
            IF(NANGL .GT. 44) THEN
              WRITE(IOUT,9004) NAMSBR,NANGL
              NANGL=44
            ENDIF
            NBANGL=3*(NANGL * (NANGL+2)) / 8
          ELSE IF(AZMQUA .EQ. 5) THEN
            IF(NANGL .GT. 44) THEN
              WRITE(IOUT,9004) NAMSBR,NANGL
              NANGL=44
            ENDIF
            NBANGL=3*(NANGL*NANGL)/2
          ELSE IF(AZMQUA .EQ. 6) THEN
            IF(NANGL .GT. 20) THEN
              WRITE(IOUT,9005) NAMSBR,NANGL
              NANGL=20
            ENDIF
            NBANGL=(NANGL * (NANGL+2)) / 8
          ELSE IF(AZMQUA .EQ. 7) THEN
            IF(NANGL .GT. 74) THEN
              WRITE(IOUT,9006) NAMSBR,NANGL
              NANGL=74
            ENDIF
            NBANGL=3*(NANGL * (NANGL+2)) / 8
          ENDIF
          AZMOAQ=NANGL
          DQUAD(1)=DONE
          DQUAD(2)=DONE
          DQUAD(3)=DONE
          DQUAD(4)=DONE
          IF( ISYMM .EQ. 8 .OR. ISYMM .EQ. 24 )THEN
            DQUAD(1)=DQUAD(1)/(DTWO*DTWO)
            DQUAD(2)=DZERO
            DQUAD(3)=DZERO
            DQUAD(4)=DZERO
          ELSE IF( ISYMM .EQ. 4 .OR. ISYMM .EQ. 20 )THEN
            DQUAD(1)=DQUAD(1)/DTWO
            DQUAD(2)=DZERO
            DQUAD(3)=DQUAD(3)/DTWO
            DQUAD(4)=DZERO
          ELSE IF( ISYMM .EQ. 2 .OR. ISYMM .EQ. 18 )THEN
            DQUAD(1)=DQUAD(1)/DTWO
            DQUAD(2)=DQUAD(2)/DTWO
            DQUAD(3)=DZERO
            DQUAD(4)=DZERO
          ENDIF
          CALL SETARA(RBASE,2*NDIM*NQUAD*NBANGL,DANGLT)
          CALL SETARA(RBASE,2*NQUAD*NBANGL,DDENWT)
          CALL NXTQAS(IPRINT,NDIM  ,AZMQUA,NANGL ,NQUAD ,NBANGL,
     >                DQUAD ,RBASE(DANGLT),RBASE(DDENWT))
        ENDIF
        LINMAX=NBUCEL*
     >         ((MAXPIN+1)*(2*MXGREG+2)+MXGSUR+16)
*----
*  Select standard spatial tracking parameters
*----
        CALL NXTQSS(IPRINT,NDIM  ,MAXMSH,NUCELL,DENUSR,
     >              RBASE(DGMESH),NPLANE,NPOINT,DENLIN,SPACLN,
     >              WEIGHT,RADIUS,CENTER)
      ELSE IF( LTRK.EQ.1 )THEN
*----
*  2. Specular tracking
*----
        NQUAD=1
        IF(NDIM .EQ. 3) CALL XABORT(NAMSBR//
     >  ': TSPC option not valid for 3-D geometries')
        IF(AZMOAQ .GT. 24) THEN
          WRITE(IOUT,9002) NAMSBR,AZMOAQ,24,30
          AZMOAQ=30
        ELSE IF(AZMOAQ .GT. 20) THEN
          IF(AZMOAQ .NE. 24) THEN
            WRITE(IOUT,9003) NAMSBR,AZMOAQ,20,24,24
            AZMOAQ=24
          ENDIF
        ELSE IF(AZMOAQ .GT. 18) THEN
          IF(AZMOAQ .NE. 20) THEN
            WRITE(IOUT,9003) NAMSBR,AZMOAQ,18,20,20
            AZMOAQ=20
          ENDIF
        ELSE IF(AZMOAQ .GT. 14) THEN
          IF(AZMOAQ .NE. 18) THEN
            WRITE(IOUT,9003) NAMSBR,AZMOAQ,14,18,18
            AZMOAQ=18
          ENDIF
        ELSE IF(AZMOAQ .GT. 12) THEN
          IF(AZMOAQ .NE. 14) THEN
            WRITE(IOUT,9003) NAMSBR,AZMOAQ,12,14,14
            AZMOAQ=14
          ENDIF
        ELSE IF(AZMOAQ .GT. 8) THEN
          IF(AZMOAQ .NE. 12) THEN
            WRITE(IOUT,9003) NAMSBR,AZMOAQ,8,12,12
            AZMOAQ=12
          ENDIF
        ELSE IF(AZMOAQ .GE. 0) THEN
          IF(AZMOAQ .NE. 8) THEN
            WRITE(IOUT,9003) NAMSBR,AZMOAQ,-1,8,8
            AZMOAQ=8
          ENDIF
        ENDIF  
        NBANGL=AZMOAQ
        NANGL =AZMOAQ
        CALL SETARA(RBASE,2*NDIM*NBANGL,DANGLT)
        CALL SETARA(RBASE,2*NBANGL,DDENWT)
        CALL SETARA(RBASE,2*NBANGL,DNSANG)
        CALL SETARA(IBASE,5*NBANGL,NBSANG)
        CALL SETARA(RBASE,2*NBANGL,DDANG)
        LINMAX=8*NANGL*NBUCEL*
     >         ((MAXPIN+1)*(2*MXGREG+2)+MXGSUR+16)
        RCIRC=SQRT(ABSC(1,1)**2+ABSC(2,1)**2)
        ABSC(1,1)= ABSC(1,1)/RCIRC
        ABSC(2,1)= ABSC(2,1)/RCIRC
        CALL NXTQAC(IPRINT,NDIM  ,NANGL ,NBANGL,DENUSR,ABSC  ,
     >              RCIRC ,AZMQUA,IPER  ,
     >              RBASE(DANGLT),RBASE(DDENWT),RBASE(DNSANG),
     >              IBASE(NBSANG),RBASE(DDANG))
        CALL LCMPUT(IPTRK,'TrackingDirc',2*NDIM*NBANGL,
     >              4,RBASE(DANGLT))
        CALL LCMPUT(IPTRK,'TrackingTrkW',2*NBANGL,
     >              4,RBASE(DDENWT))
        CALL LCMPUT(IPTRK,'TrackingSpaD',2*NBANGL,
     >              4,RBASE(DNSANG))
        CALL RLSARA(RBASE(DDANG))
*----
*  Select cyclic spatial tracking parameters
*----
        CALL SETARA(RBASE,2*NDIM*2*NBANGL,DEPART)
        CALL NXTQSC(IPRINT,NDIM  ,NBANGL,MAXMSH,NUCELL,
     >              RBASE(DGMESH),RBASE(DANGLT),RBASE(DDENWT),
     >              RBASE(DNSANG),IBASE(NBSANG),RBASE(DEPART))
        CALL LCMPUT(IPTRK,'TrackingNbST',5*NBANGL,
     >              1,IBASE(NBSANG))
        CALL LCMPUT(IPTRK,'TrackinDepT',2*NDIM*2*NBANGL,
     >              4,RBASE(DEPART))
        IF(POLQUA .GT. 0) THEN
          IF(POLOAQ .LE. 0) THEN
            IF(ISYMM.LE.1) THEN
              POLOAQ=(AZMOAQ+1)/2
            ELSE IF(ISYMM.GE.2) THEN
              POLOAQ=AZMOAQ
            ENDIF
          ENDIF
          IF( POLQUA.EQ.0 )THEN
            CALL ALGPT(POLOAQ, 0.0, 1.0, XGSS, WGSS)
          ELSE
            CALL ALCACT(POLQUA, POLOAQ, XGSS, WGSS)
          ENDIF
          CALL LCMPUT(IPTRK,'POLAR MU    ',POLOAQ,2,XGSS)
          CALL LCMPUT(IPTRK,'POLAR WEIGHT',POLOAQ,2,WGSS)
        ENDIF
      ENDIF
      RSTATT(4)=REAL(DENLIN)
      RSTATT(5)=REAL(SPACLN)
      RSTATT(6)=REAL(WEIGHT)
      RSTATT(7)=REAL(RADIUS)
      RSTATT(8)=REAL(CENTER(1))
      RSTATT(9)=REAL(CENTER(2))
      IF(NDIM .EQ. 3) RSTATT(10)=REAL(CENTER(3))
*----
*  Track
*----
      LINMAX=MIN(LINMAX,NBSLIN)
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6010) LINMAX
      ENDIF
      CALL SETARA(IBASE,LINMAX,NUMERO)
      CALL SETARA(RBASE,LINMAX,LENGTH)
      NBDR=1
      IF(RENO .EQ. -1) NBDR=NQUAD*NBANGL+1
      CALL SETARA(RBASE,2*NFREG*NBDR,DVNOR)
      CALL SETARA(RBASE,2*NFSUR*NQUAD*NBANGL,DSNOR)
      CALL SETARA(RBASE,2*LINMAX,DLENGT)
      CALL SETARA(IBASE,5*2*(NBUCEL+4),ICINT)
      CALL SETARA(RBASE,2*2*(NBUCEL+4),DCINT)
      IF(LTRK .EQ. 0) THEN      
*----
*  Standard (isotropic) tracking (white boundary conditions)
*----
        CALL SETARA(RBASE,2*NDIM*NBANGL*NQUAD,DDIRET)
        CALL SETARA(RBASE,2*NBANGL*NQUAD,DWGTRK)
        CALL SETARA(RBASE,2*NDIM*(NDIM+1)*NPLANE*NQUAD*NBANGL,
     >              DORITR)
        IF(IPRINT .GE. 1) THEN
          IF(NDIM .EQ.2 ) THEN
            WRITE(IOUT,6030) NBANGL*NQUAD,NPOINT
          ELSE
            WRITE(IOUT,6030) NBANGL*NQUAD,NPOINT*NPOINT*NPLANE
          ENDIF
        ENDIF
        CALL NXTTLS(IPTRK ,IFTEMP,IPRINT,IGTRK ,NDIM  ,MAXMSH,
     >              NFSUR ,NFREG ,NUCELL,NBUCEL,NBANGL,NQUAD ,
     >              NPLANE,NPOINT,LINMAX,MXGSUR,MXGREG,RENO  ,
     >              MAXPIN,NBTDIR,NBDR  ,
     >              RCUTOF,SPACLN,WEIGHT,RADIUS,CENTER,
     >              IBASE(IUNFLD),RBASE(SURVOL),
     >              RBASE(DGMESH),RBASE(DANGLT),RBASE(DDENWT),
     >              MAXSGL,NTLINE,RBASE(DVNOR) ,RBASE(DSNOR) ,
     >              IBASE(NUMERO),RBASE(LENGTH),
     >              RBASE(DLENGT),IBASE(ICINT) ,RBASE(DCINT) ,
     >              RBASE(DDIRET),RBASE(DWGTRK),RBASE(DORITR))
         CALL RLSARA(RBASE(DORITR))
         CALL RLSARA(RBASE(DWGTRK))
         CALL RLSARA(RBASE(DDIRET))
      ELSE
*----
*  Cyclic (specular) tracking (mirror like boundary conditions)
*----
        CALL NXTTLC(IPTRK ,IFTEMP,IPRINT,IGTRK ,NDIM  ,MAXMSH,
     >              NFSUR ,NFREG ,NUCELL,NBUCEL,NBANGL,
     >              LINMAX,MXGSUR,MXGREG,RENO  ,NBDR  ,
     >              MAXPIN,AZMQUA,IPER  ,
     >              IBASE(IUNFLD),RBASE(SURVOL),RBASE(DGMESH),
     >              RBASE(DANGLT),RBASE(DDENWT),
     >              RBASE(DNSANG),IBASE(NBSANG),RBASE(DEPART),
     >              MAXSGL,NTLINE,RBASE(DVNOR) ,RBASE(DSNOR) ,
     >              IBASE(NUMERO),RBASE(LENGTH),
     >              RBASE(DLENGT),IBASE(ICINT) ,RBASE(DCINT) )
      ENDIF
*----
*  Save track normalisation vector
*----
      CALL LCMPUT(IPTRK,'VTNormalize ',2*NFREG,
     >            4,RBASE(DVNOR))
      IF(NBDR .GT. 1) THEN
        CALL LCMPUT(IPTRK,'VTNormalizeD',2*NFREG*(NBDR-1),
     >              4,RBASE(DVNOR+2*NFREG))
      ENDIF
      CALL LCMSIX(IPTRK,'NXTRecords  ',2)
      ISTATE(12)=ISYMM
      ISTATE(14)=NPOINT
      ISTATE(15)=LINMAX
      ISTATE(16)=NTLINE
      ISTATE(17)=NBTDIR
      ISTATE(22)=POLOAQ
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPTRK,'EXCELTRACKOP',NSTATE,2,RSTATT)
      CALL RLSARA(RBASE(DCINT))
      CALL RLSARA(IBASE(ICINT))
      CALL RLSARA(RBASE(DLENGT))
*----
*  Renormalize tracks if required and transfer to final tracking file
*----
      IFMT=0
      IF(LTRK .EQ. 0 .AND. ABS(IPRINT) .GE. 1000) IFMT=1      
      IF(IGTRK .EQ. 1) THEN
        CTRK  = '$TRK'
        WRITE(IFTRK) CTRK,5,NTLINE
        COMENT='CREATOR     : DRAGON'
        WRITE(IFTRK) COMENT
        COMENT='MODULE      : NXTTCG'
        WRITE(IFTRK) COMENT
        COMENT='TYPE        : CARTESIAN'
        WRITE(IFTRK) COMENT
        IF(RENO .EQ. -1) THEN
          COMENT='TRKNOR      : Directional '
        ELSE IF(RENO .EQ. 0) THEN
          COMENT='TRKNOR      : Global      '
        ELSE
          COMENT='TRKNOR      : Off         '
        ENDIF
        WRITE(IFTRK) COMENT
        IF(IFMT .EQ. 1) THEN      
          COMENT='OPTION      : Extended    '
          WRITE(IFTRK) COMENT
        ELSE
          COMENT='OPTION      : Short       '
          WRITE(IFTRK) COMENT
        ENDIF
        WRITE(IFTRK) NDIM,LTRK,NFREG,NFSUR,6,NCOR,NQUAD*NBANGL,MAXSGL
        CALL SETARA(RBASE,NUNK,VOLSUR)
        CALL XDRSDB(NUNK,RBASE(VOLSUR),RBASE(SURVOL),1)
        DO JJ=0,NUNK-1
          IF(JJ .LE. NFSUR) THEN
            RBASE(VOLSUR+JJ)=REAL(RBASE(VOLSUR+JJ))/4.0
          ENDIF
        ENDDO
        WRITE(IFTRK) (RBASE(VOLSUR+JJ),JJ=0,NUNK-1)
        CALL RLSARA(RBASE(VOLSUR))
        WRITE(IFTRK) (IBASE(MATALB+JJ),JJ=0,NUNK-1)
        WRITE(IFTRK) ( ICODE(JJ),JJ=1,6)
        WRITE(IFTRK) (ALBEDO(JJ),JJ=1,6)
        CALL SETARA(RBASE,NDIM*NQUAD*NBANGL,ANGLT)
        CALL SETARA(RBASE,NQUAD*NBANGL,DENWT)
        CALL XDRSDB(NDIM*NQUAD*NBANGL,RBASE(ANGLT),RBASE(DANGLT),1)
        CALL XDRSDB(NQUAD*NBANGL,RBASE(DENWT),RBASE(DDENWT),1)
        WRITE(IFTRK) (RBASE(ANGLT+JJ),JJ=0,NQUAD*NBANGL*NDIM-1)
        WRITE(IFTRK) (RBASE(DENWT+JJ),JJ=0,NQUAD*NBANGL-1)
        CALL RLSARA(RBASE(DENWT))
        CALL RLSARA(RBASE(ANGLT))
        REWIND IFTEMP
        CALL NXTTNS(IFTRK ,IFTEMP,IPRINT,RENO  ,NFSUR ,NFREG ,
     >              NDIM  ,MAXSGL,NTLINE,NBTDIR,NBDR  ,IFMT  ,
     >              RBASE(DVNOR) ,IBASE(NUMERO),RBASE(LENGTH))
*----
*  Close temporary tracking file if required
*----
        ICLS=KDRCLS(IFTEMP,2)
        IF(ICLS .NE. 0) WRITE(IOUT,9011) NAMSBR
      ENDIF
*----
*  Save ISTATE
*----
      CALL RLSARA(RBASE(DSNOR))
      CALL RLSARA(RBASE(DVNOR))
      CALL RLSARA(RBASE(LENGTH))
      CALL RLSARA(IBASE(NUMERO))
      CALL RLSARA(RBASE(DGMESH))
      CALL RLSARA(RBASE(SURVOL))
      CALL RLSARA(IBASE(MATALB))
      CALL RLSARA(IBASE(IUNFLD))
      IF(LTRK .EQ. 1) THEN
        CALL RLSARA(RBASE(DEPART))
        CALL RLSARA(IBASE(NBSANG))
        CALL RLSARA(RBASE(DNSANG))
      ENDIF
      CALL RLSARA(RBASE(DDENWT))
      CALL RLSARA(RBASE(DANGLT))
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6012)
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT(' Maximum length of a line =',I10)
 6011 FORMAT(' Tracking of geometry begins:'/
     >       ' Number of regions  =',I10/
     >       ' Number of surfaces =',I10)
 6012 FORMAT(' Tracking of geometry completed')
 6030 FORMAT(' Number of directions for tracking = ',I10/
     >       ' Number of lines per direction     = ',I10)
 9000 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Order of quadrature must be even '/
     >       '       Order ',I10,' replaced by ',I10)
 9001 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Order of quadrature for EQW limited to 16'/
     >       '       Use PNTN quadrature instead')
 9002 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Number of speculular angles requested :',I10/
     >       '       For values > ',I10,' use ',I10)
 9003 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Number of speculular angles requested :',I10/
     >       '       For values > ',I10,' and < ',I10,' use ',I10)
 9004 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Order of quadrature for PNTN limited to 44'/
     >       '       Order ',I10,' replaced by 44')
 9005 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Order of quadrature for LSN limited to 20'/
     >       '       Order ',I10,' replaced by 20')
 9006 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Order of quadrature for QRN limited to 74'/
     >       '       Order ',I10,' replaced by 74')
 9010 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Impossible to open temporary tracking file ')
 9011 FORMAT(' ***** Warning in ',A6,' *****'/
     >       '       Impossible to close temporary tracking file ')
      END 
