*DECK NXTTIS
      SUBROUTINE NXTTIS(IPTRK ,IFLINE,IPRINT,NDIM  ,MAXMSH,
     >                  NUCELL,NBUCEL,NBANGL,NQUAD ,NPLANE,NPOINT,
     >                  RCUTOF,SPACLN,WEIGHT,RADIUS,CENTER,
     >                  DGMESH,DANGLT,DDENWT,
     >                  NTLINE,ICINT ,DCINT )
*
*-----------------------------------------------------------------------
*
*Purpose:
* To identify and save the standard tracking lines
* for a geometry.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*  Based on the XELTI2 and XELTI3 routines.
*
*Parameters: input
* IPTRK   pointer to the TRACKING data structure in 
*         update or creation mode.
* IFLINE  pointer to a binary file comtaining the tracking lines 
*         for this geometry.
* IPRINT  print level.
* NDIM    problem dimensions.
* MAXMSH  maximum number of elements in MESH array.
* NUCELL  number of cell after unfolding in 
*         $X$, $Y$ and $Z$ directions.
* NBUCEL  number of cells in unfolded geometry.
* NBANGL  number of angles.
* NQUAD   number of quadrant (in 3-D) and quarter (in 2-D).
* NPLANE  number of normal planes considered.
* NPOINT  number of integration points along each axis
*         in a plane mormal to track direction.
* RCUTOF  corner cutoff.
* SPACLN  linear track spacing in the plane.
* WEIGHT  weight associated with each line in the plane.
* RADIUS  radius of circle (2-D) or sphere (3-D) surrounding
*         the geometry.
* CENTER  center of circle (2-D) or sphere (3-D) surrounding
*         the geometry.
* DGMESH  meshing vector for global geometry.
* DANGLT  angles.
* DDENWT  angular density for each angle.
*
*Parameters: output
* NTLINE  total number of lines generated.
*
*Parameters: temporary storage
* ICINT   identification of spatial position for each
*         line segment in cell description of geometry.
* DCINT   position of each intersection point for each
*         line segment in cell description of geometry.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPTRK,IFLINE
      INTEGER          IPRINT,NDIM,MAXMSH,NUCELL(3),NBUCEL
      INTEGER          NBANGL,NQUAD,NPLANE,NPOINT,NTLINE
      DOUBLE PRECISION RCUTOF,SPACLN
      REAL             WEIGHT
      DOUBLE PRECISION RADIUS,CENTER(NDIM)
      DOUBLE PRECISION DGMESH(-1:MAXMSH,4),
     >                 DANGLT(NDIM,NQUAD,NBANGL),DDENWT(NQUAD,NBANGL)
      INTEGER          ICINT(0:4,2*(NBUCEL+4))
      DOUBLE PRECISION DCINT(2*(NBUCEL+4))
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTTIS')
      DOUBLE PRECISION DCUTOF,DZERO,DONE,DTWO
      PARAMETER       (DCUTOF=1.0D-10,DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
*----
*  Functions
*----
      DOUBLE PRECISION XDRCST,PI
      INTEGER          NXTLCA,IRLCA
*----
*  Local variables
*----
      INTEGER          LMAXT,ITST,NPTA2,NPTA3,NPO2,NCUTOF,
     >                 ITDIR,IANGL,IQUAD,IDIR,IPLANE,JDIR,
     >                 IPTA2,IPTA3,NBCOR(2),NBSINT
      DOUBLE PRECISION RAD2G,RAD2T,DWGT,ANGLES(3,3),DNPDIR(3,2,3),
     >                 TRKORI(3),TRKOR2(3),TCUTOF(3,2,2)
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      PI=XDRCST('Pi',' ')
      LMAXT=2*(NBUCEL+4)
      ITST=1
      NPTA2=NPOINT
      NPTA3=1
      RAD2G=RADIUS*RADIUS
      IF(NDIM .EQ. 3) NPTA3=NPOINT
      NPO2=NPOINT/2
      IF(RCUTOF .EQ. DZERO) THEN
        NCUTOF= 1
      ELSE
        NCUTOF= 4
      ENDIF
*----
*  Loop over angles in a quarter (2-D) or a quadrant (3-D)
*----
      ITDIR=0
      NTLINE=0
      DO IANGL=1,NBANGL
*----
*  Loop over 2 quarters (2-D) or 4 quadrants (3-D)
*----
        DO IQUAD=1,NQUAD
*----
*  Do not track angle with 0 density 
*  because of the problem symmetry
*----
          IF(DDENWT(IQUAD,IANGL) .EQ. DZERO) GO TO 105
          DWGT=DDENWT(IQUAD,IANGL)*WEIGHT
*----
*  Track this angle
*----
          ITDIR=ITDIR+1
*----
*  Find planes mormal to selected direction
*----
          DO IDIR=1,NDIM
            ANGLES(IDIR,1)=DANGLT(IDIR,IQUAD,IANGL)
            DO JDIR=2,NDIM
              ANGLES(IDIR,JDIR)=DZERO
            ENDDO
          ENDDO
          CALL NXTQPS(NDIM,ANGLES,DNPDIR)
*----
*  Loop over planes normal to direction
*  1 in 2-D and 3 in 3-D
*----
          DO IPLANE=1,NPLANE
            DO IDIR=1,NDIM
              DO JDIR=2,NDIM
                ANGLES(IDIR,JDIR)=SPACLN*DNPDIR(IDIR,JDIR-1,IPLANE)
              ENDDO
              IF( NCUTOF.NE.1 )THEN
                TCUTOF(IDIR,1,1)=RCUTOF*(ANGLES(IDIR,2)+ANGLES(IDIR,3))
                TCUTOF(IDIR,1,2)=RCUTOF*(ANGLES(IDIR,2)-ANGLES(IDIR,3))
                TCUTOF(IDIR,2,1)=-TCUTOF(IDIR,1,2)
                TCUTOF(IDIR,2,2)=-TCUTOF(IDIR,1,1)
              ENDIF
              TRKOR2(IDIR)=CENTER(IDIR)
     >                    -DBLE(NPO2+1)*(ANGLES(IDIR,2)+ANGLES(IDIR,3))
            ENDDO
*----
*  Loop over lines on first normal axis (direction 2)
*----
            DO IPTA2=1,NPTA2
*----
*  Displace starting point by an additional value of SPALLN
*  along first normal axis (direction 2)
*----
              DO IDIR=1,NDIM
                TRKOR2(IDIR)=TRKOR2(IDIR)+ANGLES(IDIR,2)
                TRKORI(IDIR)=TRKOR2(IDIR)
              ENDDO
*----
*  For 3-D models:
*  Loop over lines on second normal axis (direction 3)
*  For 2-D models:
*  NPTA3=1
*----
              DO IPTA3=1,NPTA3
                RAD2T=0.0
*----
*  For 3-D models:
*  Displace starting point by an additional value of SPALLN
*  along second normal axis (direction 3)
*  For 2-D models:
*  No displacement since ANGLES(IDIR,3)=0
*----
                DO IDIR=1,NDIM
                  TRKORI(IDIR)=TRKORI(IDIR)+ANGLES(IDIR,3)
                  RAD2T=RAD2T+(TRKORI(IDIR)-CENTER(IDIR))**2
                ENDDO
*----
*  Eliminate tracks outside circle or sphere surrounding geometry
*----
                IF(RAD2T .GT. RAD2G) GO TO 115
*----
*  Find cells crossed by track
*----
                IRLCA=NXTLCA(IPRINT,ITST  ,NDIM  ,MAXMSH,LMAXT,
     >                       NUCELL,TRKORI,ANGLES,DGMESH,
     >                       NBCOR ,NBSINT,ICINT ,DCINT)
*----
*  When no external face crossed go to next line
*----
                IF(IRLCA .EQ. 2) THEN
                  NTLINE=NTLINE+1
                  IF(IPRINT .GE. 500) THEN 
                    WRITE(IOUT,6040) NTLINE,IANGL,IQUAD,IPLANE,
     >                IPTA2,IPTA3,WEIGHT,SPACLN
                    WRITE(IOUT,6041) NTLINE,TRKORI
                    WRITE(IOUT,6042) NTLINE,ANGLES
                    WRITE(IOUT,6043) NTLINE,DCINT(1)
                  ENDIF
                  WRITE(IFLINE) IANGL,IQUAD,IPLANE,
     >              IPTA2,IPTA3,WEIGHT,SPACLN,DCINT(1),
     >              (TRKORI(IDIR),IDIR=1,NDIM),
     >              ((ANGLES(IDIR,JDIR),IDIR=1,NDIM),JDIR=1,NDIM)
                ENDIF
 115            CONTINUE
*----
*  END loop over points on second normal axis
*----
              ENDDO
*----
*  END loop over points on first normal axis
*----
            ENDDO
*----
*  END loop over planes
*----
          ENDDO
*----
*  Exit because angle with 0 density not tracked
*----
 105      CONTINUE
*----
*  END loop over quarter or quadrant
*----
        ENDDO
*----
*  END loop over angles
*----
      ENDDO
*----
*  Save track normalisation vector
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6040 FORMAT(' Track',I10.10,'=',5I10,2F15.8)
 6041 FORMAT(' Torig',I10.10,'={',2(F15.8,','),F15.8,'};')
 6042 FORMAT(' Tdire',I10.10,'={',8(F15.8,','),F15.8,'};')
 6043 FORMAT(' Tstrt',I10.10,'={',F15.8,'};')
      END 
