*DECK PIJXL3
      SUBROUTINE PIJXL3(IPTRK,IPRT,NGRP,NANI,NBMIX,NPSYS,NRENOR,LNORM,
     > LEAKSW,XSSIGT,XSSIGW,NELPIJ,PIJ)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities in EXCELL without producing
* a tracking file. Based on subroutine XL3TRK in DRAGON 3.4.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Roy and A. Hebert
*
*Parameters: input
* IPTRK   pointer to the tracking (L_TRACK signature).
* IPRT    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NANI    number of Legendre orders (usually equal to one).
* NBMIX   number of mixtures.
* NPSYS   index set to zero if a group is not to be processed. Usually,
*         NPSYS(I)=I.
* NRENOR  normalization scheme for PIJ matrices.
* LNORM   logical switch for removing leakage from collision
*         probabilities and keeping the PIS information.
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* XSSIGT  total macroscopic cross sections ordered by mixture.
* XSSIGW  P0 within-group scattering macroscopic cross sections
*         ordered by mixture.
* NELPIJ  number of elements in symmetrized pij matrix.
*
*Parameters: output
* PIJ     reduced and symmetrized collision probabilities.
*
*-----------------------------------------------------------------------
*
      IMPLICIT        NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      LOGICAL         LEAKSW,LNORM
      INTEGER         IPTRK,IPRT,NGRP,NANI,NBMIX,NPSYS(NGRP),NRENOR,
     >                NELPIJ
      REAL            XSSIGT(0:NBMIX,NGRP),XSSIGW(0:NBMIX,NANI,NGRP),
     >                PIJ(NELPIJ,NGRP)
*----
*  LOCAL VARIABLES
*----
      INTEGER         IOUT,NALB,NSTATE,ICPALL,ICPEND
      PARAMETER      (IOUT=6,NALB=6,NSTATE=40,ICPALL=4,ICPEND=3)
      INTEGER         NDIM  ,ISPEC ,NANGLE,NANGL ,ISYMM, NORE
      INTEGER         MINDIM,MAXDIM,ICORD ,INDEL ,KEYMRG,MATALB,
     >                INAM  ,MATMRG,ICUR  ,INCR  ,NUMERO,NRSEG ,
     >                IDL   ,MATRT
      INTEGER         REMESH,VOLSUR,VOLMRG,SIGTAL,SIGVOL,CONV  ,
     >                TRKBEG,TRKDIR,ANGLES,DENSTY,SEGLEN,STAYIN,
     >                GOSOUT,PRODUC,DSCBEG,DSCEND,ILSIGA,JPSST ,
     >                JPSVT ,VOLTRK,DBLPIJ,NUNKMR,NREG  ,NPIJ  ,
     >                NALBG ,NC    ,NTR   ,NTZ   ,ICL   ,NSOUT ,
     >                ITGEO ,LENGHT,NRMV  ,NTY   ,LTRK  ,LINMAX,
     >                NUNK  ,MAXR  ,NEXTGE,NTX   ,NCOR  ,NSUR  ,
     >                NTOTCL,NVOL  ,NV    ,NS    ,RI    ,CIJ   ,
     >                WSPACE,WEIG  ,FSP   ,IGRP  ,IPRB  ,ISOUT ,
     >                ILONG ,ITYPE ,FFACT ,INDPIJ,IIN   ,IPCSCT,
     >                IBM   ,IIVOL ,I     ,J
      INTEGER         ISTATE(NSTATE),LCLSYM(3)
      INTEGER         MXANGL,ICODE(NALB)
      LOGICAL         SWVOID,SWNZBC,LSKIP
      REAL            ALBOLD(NALB),EXTKOP(NSTATE),DENUSR, RCUTOF,
     >                CUTOFX, FACT
*----
*  PARAMETERS FOR MEMORY ALLOCATION
*----
      REAL            RD
      INTEGER         ID(1)
      COMMON          RD(1)
      EQUIVALENCE    (RD(1),ID(1)) 
*----
*  INTRINSIC FUNCTION FOR POSITION IN CONDENSE PIJ MATRIX
*----
      INTEGER INDPOS
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
*----
*  READ THE GEOMETRY INFORMATION STORED ON IPTRK
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NORE=ISTATE(8)
      LTRK=ISTATE(9)+1
      NANGLE=ISTATE(11)
      ISYMM=ISTATE(12)
      CALL LCMGET(IPTRK,'EXCELTRACKOP',EXTKOP)
      CUTOFX=EXTKOP(1)
      DENUSR=EXTKOP(2)
      RCUTOF=EXTKOP(3)
      CALL LCMSIX(IPTRK,'EXCELL',1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NDIM     =ISTATE(1)
      NSUR    =-ISTATE(2)
      NVOL     =ISTATE(3)
      NTOTCL   =ISTATE(4)
      MAXR     =ISTATE(5)
      NUNK     =ISTATE(6)
      NEXTGE   =ISTATE(7)
*----
*  Intrinsic symmetries used in geometry
*  Use these to simplify tracking unless 
*  NOSYMM tracking option activated
*----
      LCLSYM(1) =ISTATE(8)
      LCLSYM(2) =ISTATE(9)
      LCLSYM(3) =ISTATE(10)
      IF(ISYMM .NE. 0) THEN
        ISYMM=0
        IF(NDIM .EQ. 2) THEN
          IF(LCLSYM(1) .NE. 0) THEN
*----
*  X SYMMETRY
*----
            ISYMM=2
          ENDIF
          IF(LCLSYM(2) .NE. 0) THEN
            IF(ISYMM .EQ. 0) THEN
*----
*  Y SYMMETRY
*----
              ISYMM=4
            ELSE
*----
*  X AND Y SYMMETRY
*----
              ISYMM=8 
            ENDIF
          ENDIF
*----
*  X-Y DIAGONAL SYMMETRY
*---- 
        ELSE
          IF(LCLSYM(1) .NE. 0) THEN
*----
*  X SYMMETRY
*----
            ISYMM=2
          ENDIF
          IF(LCLSYM(2) .NE. 0) THEN
            IF(ISYMM .EQ. 0) THEN
*----
*  Y SYMMETRY
*----
              ISYMM=4
            ELSE
*----
*  X AND Y SYMMETRY
*----
              ISYMM=8 
            ENDIF
          ENDIF 
          IF(LCLSYM(3) .NE. 0) THEN
*----
*  Z SYMMETRY
*----
            ISYMM=ISYMM+16
          ENDIF
        ENDIF
        IF(ISYMM .EQ. 0) ISYMM=1
      ENDIF
      CALL SETARA(ID,NTOTCL,MINDIM)
      CALL SETARA(ID,NTOTCL,MAXDIM)
      CALL SETARA(ID,NTOTCL,ICORD )
      CALL SETARA(ID,4*NUNK,INDEL )
      CALL SETARA(RD,MAXR  ,REMESH)
      CALL SETARA(ID,NUNK  ,KEYMRG)
      CALL SETARA(ID,NUNK  ,MATALB)
      CALL SETARA(RD,NUNK  ,VOLSUR)
      CALL LCMGET(IPTRK,'MINDIM      ',ID(MINDIM))
      CALL LCMGET(IPTRK,'MAXDIM      ',ID(MAXDIM))
      CALL LCMGET(IPTRK,'ICORD       ',ID(ICORD) )
      CALL LCMGET(IPTRK,'INDEX       ',ID(INDEL) )
      CALL LCMGET(IPTRK,'REMESH      ',RD(REMESH))
      CALL LCMGET(IPTRK,'KEYMRG      ',ID(KEYMRG))
      CALL LCMGET(IPTRK,'MATALB      ',ID(MATALB))
      CALL LCMGET(IPTRK,'VOLSUR      ',RD(VOLSUR))
      CALL LCMSIX(IPTRK,'EXCELL      ',2)
      CALL LCMGET(IPTRK,'ICODE       ',ICODE )
      CALL LCMGET(IPTRK,'ALBEDO      ',ALBOLD)
*----
*  VERIFY SYMMETRY AND STUDY TRACKING PARAMETERS. ARE THEY BASICALLY
*  POSSIBLE ?
*----
      IF(LTRK .EQ. 1)THEN
        NCOR= 1
        IF(NDIM .EQ. 2) THEN
          MXANGL=NANGLE
          IF(ISYMM .GE. 2) THEN
            NANGL = (NANGLE+1)/2
          ELSE 
            NANGL = NANGLE
          ENDIF
          IF( RCUTOF.GT.0.0 ) NCOR= 2
        ELSE IF(NDIM .EQ. 3) THEN
          IF(MOD(NANGLE,2) .EQ. 1)THEN
            NANGLE=NANGLE+1
            WRITE(IOUT,'(/31H MESS = ONLY EVEN # EQN ANGLES )')
          ENDIF
          IF(NANGLE .GT. 16)THEN
            NANGLE=16
            WRITE(IOUT,'(/31H MESS = 16 IS MAX # EQN ANGLES )')
          ENDIF
          MXANGL=(NANGLE * (NANGLE+2)) / 2 
          IF(NEXTGE .EQ. 1) THEN
            NANGL = (NANGLE * (NANGLE+2)) / 8
          ELSE
            IF(ISYMM .EQ. 8 .OR. ISYMM .EQ. 24) THEN
              NANGL = (NANGLE * (NANGLE+2)) / 8 
            ELSE IF(ISYMM .EQ. 2  .OR. ISYMM .EQ. 4 .OR. 
     >              ISYMM .EQ. 18 .OR. ISYMM .EQ. 20 ) THEN
              NANGL = (NANGLE * (NANGLE+2)) / 4 
            ELSE
              NANGL = (NANGLE * (NANGLE+2)) / 2 
            ENDIF
          ENDIF
          IF(RCUTOF .GT. 0.0) NCOR= 4
        ENDIF
      ELSEIF( LTRK.EQ.2 )THEN
        NCOR  = 1
        MXANGL=NANGLE
        IF( NDIM.EQ.2 )THEN
           NANGL = NANGLE
        ELSEIF( NDIM.EQ.3 )THEN
           CALL XABORT('PIJXL3: *TSPC* NOT AVAILABLE FOR 3-D GEOMETRY')
        ENDIF
        CUTOFX= RCUTOF
      ENDIF
      IF( IPRT.GE.1 ) THEN
         WRITE(IOUT,6002) NANGL,ISYMM,CUTOFX,DENUSR,RCUTOF
      ENDIF
      IF( IPRT.GT.1 .AND. NEXTGE.EQ.0 )THEN
*
*        IF PRINT REQUIRED AND OVERALL CARTESIAN GEOMETRY
*        PRINT CARTESIAN REGION MAP
         NTX= ID(MAXDIM  )-ID(MINDIM  )
         NTY= ID(MAXDIM+1)-ID(MINDIM+1)
         NTZ= ID(MAXDIM+2)-ID(MINDIM+2)
         NTR=0
         DO 103 ICL=3,NTOTCL-1
            NTR= MAX(NTR,ID(MAXDIM+ICL)-ID(MINDIM+ICL)+1)
  103    CONTINUE
         CALL SETARA(ID,4*(NTR+1)*(NTX+2)*(NTY+2)*(NTZ+2),INAM)
         CALL XELGPR(NDIM,NTX,NTY,NTZ,NTR,ISYMM,
     >               NSUR,NVOL,NTOTCL,ID(MINDIM),ID(MAXDIM),
     >               ID(KEYMRG),ID(INDEL),ID(MATALB),ID(INAM))
         CALL RLSARA(ID(INAM))
      ENDIF
      CALL SETARA( RD, 2*(NANGL+1)*NUNK, VOLTRK)
*
      NV=  NVOL
      NS= -NSUR
      CALL SETARA( RD,     NUNK, VOLMRG)
      CALL SETARA( ID,     NUNK, MATMRG)
      ITGEO=3
      CALL XELCMP(   NS,    NV, RD(VOLSUR), ID(MATALB), ID(KEYMRG),
     >            NSOUT,  NREG, RD(VOLMRG), ID(MATMRG), ITGEO,ICODE)
      NUNKMR= NREG+NSOUT+1
      NPIJ= (NUNKMR*(NUNKMR+1))/2
      IF( IPRT .GT. 1 ) WRITE(IOUT,6000) (NGRP*NPIJ/128)
      CALL SETARA( RD, 2*NGRP*NPIJ, DBLPIJ)
      IF( IPRT .GT. 1 ) WRITE(IOUT,6001)
*
*     ALLOCATE AND CHARGE TOTAL XS PER REGION
      CALL SETARA( RD, NGRP*NUNKMR, SIGTAL)
      CALL SETARA( RD, NGRP*NREG, SIGVOL)
*
*  3) DO THE TRACKING OF THE EXACT GEOMETRY FOR *NEWT* OPTION.
      IF( LTRK.NE.0 )THEN
         NC= NTOTCL - 3
         IF( IPRT.GE.1 )THEN
            WRITE(IOUT,'(1H )')
            IF( NC.EQ.0 )THEN
               WRITE(IOUT,'(/38H NOW, TRACKING GEOMETRY WITH NO CYLIND,
     >         2HER/)')
            ELSEIF( NC.EQ.1 )THEN
               WRITE(IOUT,'(/38H NOW, TRACKING GEOMETRY WITH ONE CYLIN,
     >         3HDER/)')
            ELSE
               WRITE(IOUT,'(/28H NOW, TRACKING GEOMETRY WITH,I4,
     >         10H CYLINDERS/)') NC
            ENDIF
         ENDIF
         CALL SETARA( ID,  NTOTCL,   ICUR )
         CALL SETARA( ID,  NTOTCL,   INCR )
         CALL SETARA( RD,  NTOTCL,   CONV )
         CALL SETARA( RD,  NTOTCL, TRKBEG )
         CALL SETARA( RD,  NTOTCL, TRKDIR )
*
*  3.0)  WRITE FIRST RECORDS OF THE UNNORMALIZED TRACKING FILE
         IF( LTRK.EQ.1 )THEN
            LINMAX= 2*NVOL + 10
         ELSE
            LINMAX= 8*NANGL*(2*NVOL + 8)
         ENDIF
         ISPEC = LTRK-1
         NALBG = 6
         CALL SETARA( ID,  LINMAX, NUMERO )
         CALL SETARA( RD,  LINMAX, LENGHT )
         CALL SETARA( RD,3*MXANGL, ANGLES )
         CALL SETARA( RD,  MXANGL, DENSTY )
         CALL SETARA( RD,  LINMAX, SEGLEN )
         CALL SETARA( ID,  LINMAX, NRSEG  )
         CALL SETARA( RD,  NGRP*LINMAX, STAYIN )
         CALL SETARA( RD,  NGRP*LINMAX, GOSOUT )
         CALL SETARA( RD,  NGRP, PRODUC )
         CALL SETARA( RD,  NGRP, DSCBEG )
         CALL SETARA( RD,  NGRP, DSCEND )
*
         NRMV=1
         CALL XL3TI3( IPRT, NANGLE, DENUSR,ISYMM,
     >                RD(ANGLES), RD(DENSTY), NTOTCL,
     >                NEXTGE, MAXR, RD(REMESH), LINMAX, RCUTOF,
     >                NSUR,   NVOL, ID(INDEL),
     >                ID(MINDIM), ID(MAXDIM), ID(ICORD),
     >                ID(INCR), ID(ICUR),
     >                RD(TRKBEG), RD(CONV), RD(TRKDIR),
     >                RD(LENGHT), ID(NUMERO),
     >                NPIJ, NGRP, RD(SIGTAL), SWVOID,
     >                ID(NRSEG), RD(SEGLEN),
     >                RD(STAYIN),RD(GOSOUT),
     >                RD(PRODUC),RD(DSCBEG),RD(DSCEND),
     >                NORE, NRMV, RD(VOLTRK),
     >                ID(KEYMRG),-NSOUT,NREG,NPSYS,RD(DBLPIJ) )
*
         CALL XL3NTR( IPRT, NDIM, ISPEC, NS, NV, NORE,
     >                RD(VOLSUR), ID(KEYMRG), ID(MATALB),
     >                NANGL, RD(VOLTRK), RD(DENSTY) )
*
         CALL XL3SIG( NGRP, NBMIX, NANI, XSSIGT, XSSIGW, ALBOLD,
     >                NPSYS, NGRP, -NSOUT, NREG, ID(MATMRG),
     >                RD(VOLMRG+NSOUT+1), RD(SIGTAL), RD(SIGVOL),
     >                SWVOID, SWNZBC)
*
         NRMV=0
         CALL XL3TI3( IPRT, NANGLE, DENUSR,ISYMM,
     >                RD(ANGLES), RD(DENSTY), NTOTCL,
     >                NEXTGE, MAXR, RD(REMESH), LINMAX, RCUTOF,
     >                NSUR,   NVOL, ID(INDEL),
     >                ID(MINDIM), ID(MAXDIM), ID(ICORD),
     >                ID(INCR), ID(ICUR),
     >                RD(TRKBEG), RD(CONV), RD(TRKDIR),
     >                RD(LENGHT), ID(NUMERO),
     >                NPIJ, NGRP, RD(SIGTAL), SWVOID,
     >                ID(NRSEG), RD(SEGLEN),
     >                RD(STAYIN),RD(GOSOUT),
     >                RD(PRODUC),RD(DSCBEG),RD(DSCEND),
     >                NORE, NRMV, RD(VOLTRK),
     >                ID(KEYMRG),-NSOUT,NREG,NPSYS,RD(DBLPIJ) )
*
         CALL QIJCMP(NREG,-NSOUT,NPIJ,NGRP,NCOR,
     >               RD(VOLMRG),RD(SIGTAL),RD(DBLPIJ),NPSYS)
*----
*  RENORMALIZE ALL ISOTROPIC PROBS WITH VARIOUS OPTIONS
*----
         IF( NRENOR.EQ.1 )THEN
            CALL SETARA( RD, 2*NUNKMR, RI)
         ELSEIF( NRENOR.EQ.3 )THEN
            CALL SETARA( RD,  2*NPIJ,    CIJ )
            CALL SETARA( RD,  2*NPIJ, WSPACE )
            CALL SETARA( RD, 2*NUNKMR,   WEIG )
            CALL SETARA( RD, 2*NUNKMR,    FSP )
            CALL SETARA( ID,   NUNKMR,    IDL )
         ELSEIF( NRENOR .EQ. 4 )THEN
            CALL SETARA( RD, 2*NUNKMR*3, WEIG )
            CALL SETARA( RD, 2*NUNKMR, FSP )
         ENDIF
         DO 2060 IGRP=1,NGRP
         IF(NPSYS(IGRP).EQ.0) GO TO 2060
         IPRB=DBLPIJ+(IGRP-1)*2*NPIJ
         IIVOL=SIGVOL+(IGRP-1)*NREG
         ILSIGA=SIGTAL+(IGRP-1)*NUNKMR
         IF( NRENOR.EQ.1 )THEN
*
*           NORMALIZATION USING GELBARD SCHEME
            CALL PIJRGL( IPRT, NREG, NSOUT, RD(ILSIGA), RD(IPRB),
     >                   RD(RI) )
         ELSEIF( NRENOR.EQ.2 )THEN
*
*           NORMALIZATION WORKING ON DIAGONAL COEFFICIENTS
            CALL PIJRDG( IPRT, NREG, NSOUT, RD(ILSIGA), RD(IPRB) )
         ELSEIF( NRENOR.EQ.3 )THEN
*
*           NORMALIZATION WORKING ON WEIGHT FACTORS TO KEEP DIAG = 0.0
            CALL PIJRNL( IPRT,  NREG,   NSOUT, RD(ILSIGA), RD(IPRB),
     >                   RD(CIJ), RD(WSPACE), RD(FSP),
     >                   ID(IDL), RD(WEIG) )
         ELSEIF( NRENOR .EQ. 4 )THEN  ! ATTENTION
*
*           NORMALIZATION WORKING ON WEIGHT FACTORS ADDITIVE (HELIOS)
            CALL PIJRHL( IPRT, NREG, NSOUT, RD(ILSIGA), RD(IPRB),
     >                   RD(FSP), RD(WEIG) )
         ENDIF
         IF( IPRT.GE.ICPALL )THEN
            WRITE(IOUT,'(1H )')
            WRITE(IOUT,'(35H   COLLISION PROBABILITIES OUTPUT: ,
     >                   35H *BEFORE* ALBEDO REDUCTION          )')
            CALL PIJWPR(0,NREG,NSOUT,RD(ILSIGA),RD(IPRB),RD(IIVOL),1)
         ENDIF
 2060    CONTINUE
         IF( NRENOR.EQ.1 )THEN
            CALL RLSARA( RD(    RI ))
         ELSEIF( NRENOR.EQ.3 )THEN
            CALL RLSARA( ID(    IDL ))
            CALL RLSARA( RD(    FSP ))
            CALL RLSARA( RD(   WEIG ))
            CALL RLSARA( RD( WSPACE ))
            CALL RLSARA( RD(    CIJ ))
         ELSEIF( NRENOR .EQ. 4 )THEN
            CALL RLSARA( RD(    FSP ))
            CALL RLSARA( RD(   WEIG ))
         ENDIF
*----
*  ELIMINATION OF SURFACES FOR PIJ
*----
         IF( SWNZBC )THEN
            CALL SETARA( RD, 2*NSOUT*NSOUT , JPSST )
            CALL SETARA( RD,  2*NSOUT*NREG , JPSVT )
            CALL SETARA( ID,         NSOUT ,   IDL )
            CALL SETARA( ID,         NSOUT , MATRT )
            CALL LCMLEN(IPTRK,'BC-REFL+TRAN',ILONG,ITYPE)
            IF(ILONG.EQ.NSOUT) THEN
              CALL LCMGET(IPTRK,'BC-REFL+TRAN',ID(MATRT))
            ELSE
               DO 130 ISOUT=0,NSOUT-1
                 ID(MATRT+ISOUT)=ISOUT+1
 130           CONTINUE
            ENDIF
            DO 2080 IGRP=1,NGRP
              IF(NPSYS(IGRP).EQ.0) GO TO 2080
              IPRB=DBLPIJ+(IGRP-1)*2*NPIJ
              ILSIGA=SIGTAL+(IGRP-1)*NUNKMR
              CALL PIJABC(NREG,NSOUT,NPIJ,RD(ILSIGA),ID(MATRT),
     >                    RD(IPRB),ID(IDL),RD(JPSST),RD(JPSVT))
 2080       CONTINUE
*
            CALL RLSARA( ID( MATRT ))
            CALL RLSARA( ID(   IDL ))
            CALL RLSARA( RD( JPSVT ))
            CALL RLSARA( RD( JPSST ))
         ENDIF
*
         CALL SETARA( RD, NREG,   FFACT )
         DO 2090 IGRP=1,NGRP
         IF(NPSYS(IGRP).EQ.0) GO TO 2090
         IPRB=DBLPIJ+(IGRP-1)*2*NPIJ
         IIVOL=SIGVOL+(IGRP-1)*NREG
         ILSIGA=SIGTAL+(IGRP-1)*NUNKMR
         IF( IPRT.GE.ICPEND )THEN
            WRITE(IOUT,'(1H )')
            WRITE(IOUT,'(35H   COLLISION PROBABILITIES OUTPUT: ,
     >                   35H *AFTER* ALBEDO REDUCTION          )')
            CALL PIJWPR(1,NREG,NSOUT,RD(ILSIGA),RD(IPRB),RD(IIVOL),1)
         ENDIF
*----
*  CHARGE PIJ MATRIX IN THE DRAGON SYMMETRIZED FORMAT
*----
         DO 160 IIN=1,NREG
            IF(RD(ILSIGA+NSOUT+IIN).EQ.0.0) THEN
               RD(FFACT+IIN-1)=1.0
            ELSE
               RD(FFACT+IIN-1)=1.0/RD(ILSIGA+NSOUT+IIN)
            ENDIF
  160    CONTINUE
         CALL PIJD2R(NREG,NSOUT,RD(IPRB),RD(FFACT),.FALSE.,NELPIJ,
     >               PIJ(1,IGRP))
 2090    CONTINUE
         CALL RLSARA( RD(FFACT ) )
*
         CALL RLSARA( RD(DSCEND) )
         CALL RLSARA( RD(DSCBEG) )
         CALL RLSARA( RD(PRODUC) )
         CALL RLSARA( RD(GOSOUT) )
         CALL RLSARA( RD(STAYIN) )
         CALL RLSARA( ID(NRSEG ) )
         CALL RLSARA( RD(SEGLEN) )
         CALL RLSARA( RD(DENSTY) )
         CALL RLSARA( RD(ANGLES) )
         CALL RLSARA( RD(LENGHT) )
         CALL RLSARA( ID(NUMERO) )
         CALL RLSARA( RD(TRKDIR) )
         CALL RLSARA( RD(TRKBEG) )
         CALL RLSARA( RD(  CONV) )
         CALL RLSARA( ID(  INCR) )
         CALL RLSARA( ID(  ICUR) )
      ENDIF
      CALL RLSARA( ID( INDEL) )
      CALL RLSARA( ID( ICORD) )
      CALL RLSARA( ID(MAXDIM) )
      CALL RLSARA( ID(MINDIM) )
      CALL RLSARA( RD(REMESH) )
      CALL RLSARA( RD(DBLPIJ) )
      CALL RLSARA( RD(SIGTAL) )
      CALL RLSARA( RD(SIGVOL) )
      CALL RLSARA( RD(VOLSUR) )
      CALL RLSARA( RD(VOLTRK) )
      CALL RLSARA( ID(KEYMRG) )
      CALL RLSARA( ID(MATALB) )
*----
*  CHECK IF SCATTERING REDUCTION IS REQUIRED
*----
      CALL SETARA(RD,4*NREG*NREG,IPCSCT)
      DO 3000 IGRP=1,NGRP
      IF(NPSYS(IGRP).EQ.0) GO TO 3000
      LSKIP=.TRUE.
      DO 200 IBM=1,NBMIX
        LSKIP=LSKIP.AND.(XSSIGW(IBM,1,IGRP).EQ.0.0)
  200 CONTINUE
*----
*  COMPUTE THE SCATTERING-REDUCED CP MATRICES
*----
      IF(.NOT.LSKIP) THEN
        CALL PIJSMD(IPRT,NBMIX,NREG,ID(MATMRG+NSOUT+1),
     >              RD(VOLMRG+NSOUT+1),XSSIGW(0,1,IGRP),XSSIGT(0,IGRP),
     >              LEAKSW,PIJ(1,IGRP),RD(IPCSCT),1)
        CALL PIJSDB(NREG*NREG,RD(IPCSCT),RD(IPCSCT),1)
        DO 220 I=1,NREG
          FACT=RD(VOLMRG+NSOUT+I)
          DO 210 J=1,NREG
            INDPIJ=INDPOS(I,J)
            PIJ(INDPIJ,IGRP)=RD(IPCSCT+(J-1)*NREG+I-1)*FACT
  210     CONTINUE
  220   CONTINUE
      ENDIF
 3000 CONTINUE
      CALL RLSARA( RD(IPCSCT) )
      CALL RLSARA( RD(VOLMRG) )
      CALL RLSARA( ID(MATMRG) )
      RETURN 
*----
*  FORMAT
*----
 6000 FORMAT(' *** SPACE REQUIRED FOR CP MATRICES = ',I10,' K ***')
 6001 FORMAT(' *** CP MATRICES ALLOCATED            ',10X,'   ***')
 6002 FORMAT(
     > ' -----------------'/' RECOMPUTED PARAMETERS '/
     > ' NANGL  =',I10  ,' (NUMBER OF TRACKING ANGLES)'/
     > ' ISYMM  =',I10  ,' (TRACKING SYMMETRY FACTOR)'/
     > ' CUTOFX =',F10.5,' (CUTOFF FOR TRACK LENGTH)'/
     > ' DENS   =',F10.5,' (TRACK DENSITY)'/
     > ' PCORN  =',F10.5,' (CORNER DUPLICATION DISTANCE)'/
     > ' -----------------'/)
      END
