*DECK PNF
      SUBROUTINE PNF(KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,MAT,
     1 VOL,KEYFLX,FUNKNO,SUNKNO,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one-group transport equation for fluxes using the spherical
* harmonics (PN) method.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* KPSYS   pointer to the assembly matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* IDIR    not used.
* NREG    total number of regions for which specific values of the
*         neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NUN     total number of unknowns in vectors SUNKNO and FUNKNO.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  position of averaged flux elements in FUNKNO vector.
* SUNKNO  input source vector.
* TITR    title.
*
*Parameters: input/output
* FUNKNO  unknown vector.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   TITR*72
      INTEGER     KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,MAT(NREG),
     >            KEYFLX(NREG)
      REAL        VOL(NREG),FUNKNO(NUN),SUNKNO(NUN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40,MAXIT=20,EPSINR=1.0E-5,ICL1=3,
     1            ICL2=3)
      INTEGER     IPAR(NSTATE),IBASE(1)
      DOUBLE PRECISION F1,F2,R1,R2,DMU
      CHARACTER   NAMP*12
      LOGICAL     LBIHET
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER PN SPECIFIC PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      IF(NREG.NE.IPAR(1)) CALL XABORT('PNF: INVALID VALUE OF NREG.')
      IF(NUN.NE.IPAR(2)) CALL XABORT('PNF: INVALID VALUE OF NUN.')
      ITYPE=IPAR(6)
      IELEM=IPAR(8)
      ICOL=IPAR(9)
      L4=IPAR(11)
      NLF=IPAR(14)
      ISPN=IPAR(15)
      ISCAT=IPAR(16)
      NVD=IPAR(17)
      LBIHET=IPAR(40).NE.0
      IF(IMPX.GT.2) THEN
         WRITE(IUNOUT,'(//33H N E U T R O N    S O U R C E S :)')
         WRITE(IUNOUT,'(1P,6(5X,E15.7))') (SUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL SETARA(BASE,NUN,ISUNKN)
         DO 5 I=1,NUN
    5    BASE(ISUNKN+I-1)=SUNKNO(I)
         NREGAR=NREG
         NBMIXG=NBMIX
         CALL DOORFB2(KPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX,NBMIX2,
     1   NREG2,SUNKNO)
         NREG=NREG2
         NBMIX=NBMIX2
         CALL LCMSIX(KPSYS,'BIHET',1)
      ENDIF
*
      CALL SETARA(IBASE,L4,IMU)
      CALL LCMGET(IPTRK,'MU',IBASE(IMU))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
*----
*  RECOVER CROSS SECTIONS.
*----
      CALL LCMGET(KPSYS,'STATE-VECTOR',IPAR)
      NAN=IPAR(8)
      CALL SETARA(BASE,NBMIX*NAN,ISGDI)
      DO 10 IL=0,NAN-1
      WRITE(NAMP,'(4HSCAI,I2.2,6H001001)') IL
      CALL LCMGET(KPSYS,NAMP,BASE(ISGDI+IL*NBMIX))
   10 CONTINUE
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  MULTIPLICATION OF THE EVEN-PARITY SOURCES BY THE VOLUMES.
*----
      DO 15 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 15
      JND1=KEYFLX(K)
      SUNKNO(JND1)=SUNKNO(JND1)*VOL(K)
   15 CONTINUE
*----
*  INNER ITERATION LOOP FOR ONE-GROUP TRANSPORT EQUATION.
*----
      IIMAX=IBASE(IMU+L4-1)*(NLF/2)
      CALL SETARA(BASE,IIMAX,ISYS)
      CALL LCMGET(KPSYS,'IA001001',BASE(ISYS))
      CALL SETARA(BASE,NUN,IOLD1)
      CALL SETARA(BASE,NUN,IOLD2)
      CALL XDRSET(BASE(IOLD2),NUN,0.0)
      ITER=0
   20 ITER=ITER+1
      IF(ITER.GT.MAXIT) THEN
         WRITE(IUNOUT,'(43H PNF: MAXIMUM NUMBER OF ONE-SPEED ITERATION,
     1   9H REACHED.)')
         GO TO 100
      ENDIF
      DO 30 I=1,NUN
      BASE(IOLD1+I-1)=BASE(IOLD2+I-1)
   30 BASE(IOLD2+I-1)=FUNKNO(I)
      IF((ITYPE.EQ.2).OR.((ITYPE.EQ.5).AND.(ISPN.EQ.1))) THEN
         CALL SETARA(BASE,NREG,IXX)
         CALL SETARA(BASE,NREG,IYY)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL PNFL2E(NREG,IELEM,ICOL,BASE(IXX),BASE(IYY),MAT,VOL,NBMIX,
     1   NLF,NVD,NAN,BASE(ISGDI),L4,IBASE(IKN),BASE(IQFR),IBASE(IMU),
     2   IIMAX,LC,BASE(IR),BASE(IV),BASE(ISYS),SUNKNO,FUNKNO)
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ELSE
         CALL XABORT('PNF: TYPE OF DISCRETIZATION NOT IMPLEMENTED.')
      ENDIF
      IF(NLF.EQ.2) GO TO 100
      IF(MOD(ITER-1,ICL1+ICL2).GE.ICL1) THEN
*        VARIATIONAL ACCELERATION.
         F1=0.0
         F2=0.0
         DO  40 I=1,NUN
         R1=BASE(IOLD2+I-1)-BASE(IOLD1+I-1)
         R2=FUNKNO(I)-BASE(IOLD2+I-1)
         F1=F1+R1*(R2-R1)
   40    F2=F2+(R2-R1)*(R2-R1)
         DMU=-F1/F2
         IF(DMU.GT.0.0) THEN
            DO  50 I=1,NUN
            FUNKNO(I)=BASE(IOLD2+I-1)+DMU*(FUNKNO(I)-BASE(IOLD2+I-1))
            BASE(IOLD2+I-1)=BASE(IOLD1+I-1)+DMU*(BASE(IOLD2+I-1)-
     1      BASE(IOLD1+I-1))
   50       CONTINUE
         ENDIF
      ENDIF      
      AAA=0.0
      BBB=0.0
      DO 60 I=1,NREG
      IF(KEYFLX(I).EQ.0) GO TO 60
      AAA=MAX(AAA,ABS(FUNKNO(KEYFLX(I))-BASE(IOLD2+KEYFLX(I)-1)))
      BBB=MAX(BBB,ABS(FUNKNO(KEYFLX(I))))
   60 CONTINUE
      IF(IMPX.GT.1) WRITE(IUNOUT,300) ITER,AAA,BBB,DMU
      IF(AAA.LE.EPSINR*BBB) GO TO 100
      IF(ITER.EQ.1) TEST=AAA
      IF((ITER.GE.10).AND.(AAA.GT.TEST)) THEN
         WRITE(IUNOUT,'(43H PNF: UNABLE TO CONVERGE ONE-SPEED ITERATIO,
     1   3HNS.)')
         GO TO 100
      ENDIF
      GO TO 20
  100 CALL RLSARA(BASE(IOLD2))
      CALL RLSARA(BASE(IOLD1))
      CALL RLSARA(BASE(ISYS))
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(ISGDI))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(IBASE(IMU))
*----
*  DIVISION OF THE EVEN-PARITY SOURCES BY THE VOLUMES.
*----
      DO 110 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 110
      JND1=KEYFLX(K)
      SUNKNO(JND1)=SUNKNO(JND1)/VOL(K)
  110 CONTINUE
*
      IF((IMPX.GT.0).AND.(NLF.GT.2)) WRITE(IUNOUT,'(15H PNF: NUMBER OF,
     1 22H ONE-SPEED ITERATIONS=,I5,1H.)') ITER
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL LCMSIX(KPSYS,' ',2)
         NREG=NREGAR
         NBMIX=NBMIXG
         DO 120 I=1,NUN
  120    SUNKNO(I)=BASE(ISUNKN+I-1)
         CALL RLSARA(BASE(ISUNKN))
         CALL DOORFB3(KPSYS,IPTRK,IMPX,NBMIX,NREG,NUN,KEYFLX,SUNKNO,
     1   FUNKNO)
      ENDIF
*
      IF(IMPX.GT.2) THEN
         WRITE(IUNOUT,'(//33H N E U T R O N    F L U X E S   :)')
         WRITE(IUNOUT,'(1P,6(5X,E15.7))') (FUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
      RETURN
  300 FORMAT(25H PNF: ONE-SPEED ITERATION,I3,8H  ERROR=,1P,E11.4,
     1 5H OVER,E11.4,22H  ACCELERATION FACTOR=,0P,F7.3)
      END
