*DECK PSP
      SUBROUTINE PSP(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
C
C----
C  1- PROGRAMME STATISTICS:
C      NAME     : PSP
C      USE      : POSTSCRIPT PLOT UTILITY MODULE
C      AUTHOR   : G.MARLEAU
C      CREATED  : 99-01-21
C
C      MODIFICATION LOG
C      --------------------------------------------------------------
C      | DATE AND INITIALS  | MOTIVATIONS
C      --------------------------------------------------------------
C      | 99-01-21 G.M.      | GENERATION OF A POSTSCRIPT FILE
C      |                    | CONTAINING A SPECIFIC EXCELL 2-D
C      |                    | GEOMETRIES
C      ______________________________________________________________
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      NENTRY : NUMBER OF DATA STRUCTURES USED.          I
C      HENTRY : NAME OF DATA STRUCTURES.                 C(NENTRY)*12
C      IENTRY : TYPE OF DATA STRUCTURES.                 I(NENTRY)
C               = 1 FOR LINKED LIST;
C               = 2 FOR XSM FILE;
C               = 3 FOR SEQUENTIAL BINARY FILE;
C               = 4 FOR SEQUENTIAL ASCII FILE.
C      JENTRY : MODE OF DATA STRUCTURES.                 I(NENTRY)
C               = 0 CREATION MODE;
C               = 1 UPDATE MODE;
C               = 2 READ-ONLY MODE.
C      KENTRY : ADDRESS OF DATA STRUCTURES.              I(NENTRY)
C               FOR IENTRY = 1  LINKED LIST ADDRESS;
C               FOR IENTRY > 1  FILE UNIT NUMBER.
C  3- INPUT REQUIREMENTS
C      NENTRY  >= 2
C        IEN = 1 : STRUCTURE IS A SEQUENTIAL ASCII FILE
C                  CONTAINING THE OUTPUT POSTSCRIPT. THIS CAN BE A
C                  NEW FILE OR A FILE TO UPDATE.
C                  IENTRY(IEN) = 4, JENTRY(IEN) <= 1
C      FOR PSP BY MIXTURE OR REGION
C        IEN > 1 : STRUCTURE IS A VALID DRAGON GEOMETRY
C                  FOR EXCELT STORED IN A LINKED LIST OR XSM FILE.
C                  THIS STRUCTURE MUST BE IN READ-ONLY MODE
C                  IENTRY(IEN)<= 2, JENTRY(IEN) = 2
C      FOR PSP BY FLUX
C        IEN = 2 : STRUCTURE IS A VALID DRAGON GEOMETRY
C                  FOR EXCELT STORED IN A LINKED LIST OR XSM FILE.
C                  THIS STRUCTURE MUST BE IN READ-ONLY MODE
C                  IENTRY(IEN)<= 2, JENTRY(IEN) = 2
C        IEN = 3 : STRUCTURE IS A VALID FLUX STRUCTURE
C
C----
C
      IMPLICIT     NONE
      INTEGER      IOUT,NSTATE,NTC,ILCMUP,ILCMDN
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NSTATE=40,NTC=3,ILCMUP=1,ILCMDN=2,
     >             NAMSBR='PSP   ')
C----
C  ROUTINE PARAMTERS
C----
      INTEGER      NENTRY,IENTRY(NENTRY),JENTRY(NENTRY),
     >             KENTRY(NENTRY)
      CHARACTER    HENTRY(NENTRY)*12
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IFLUX,ITFLX
C----
C  LOCAL VARIABLES
C----
      INTEGER      ISTATE(NSTATE),KCHAR(NTC),IPRINT,
     >             IENGT,IPGT,IENFL,IPFL,JPFL,ITC,ISPSP,ITYPE,ICOLR,
     >             NPAGE,ITRK,NGROUP,NUNKNO,IUN,IGR,NGT,LCMGID
      CHARACTER    HSIGN*12,NAMGT*12,NAMLEG*24
      REAL         XYPOS(2)
      CHARACTER    NAMTR2*12,NAMGEO*12
      INTEGER      IPTRK2,IMODT2,IMEDT2,ICLST2,IPRIN2
      LOGICAL      LASS,LDRASS
      NAMTR2='PSPGEOIPTRK2'
      IMODT2=0
      IMEDT2=1
      IPRIN2=0
      ICLST2=2
C----
C  INPUT PARAMETER VALIDATION
C----
      XYPOS(1)=0.5
      XYPOS(2)=0.5
      IF(NENTRY    .LT. 2 ) CALL XABORT(NAMSBR//
     >  ': AT LEAST TWO DATA STRUCTURES REQUIRED')
      ISPSP=KENTRY(1)
      IF(IENTRY(1) .NE. 4 ) CALL XABORT(NAMSBR//
     >  ': POSTSCRIPT DATA STRUCTURE NOT AN ASCII FILE')
      IF(JENTRY(1) .NE. 0   .AND.
     >   JENTRY(1) .NE. 1 ) CALL XABORT(NAMSBR//
     >  ': POSTSCRIPT DATA STRUCTURE NOT IN CREATE OR MODIFY MODE')
C----
C  READ PSP OPTIONS
C  IPRINT : EDIT LEVEL
c           = 0 NO EDIT
C           = 1 NORMAL EDIT  (DEFAULT)
C           > 1 EDIT FOR DEBUG
C----
      CALL PSPGET(IPRINT,ITYPE,ICOLR)
C----
C  OPEN POSTSCRIPT OUTPUT FILE
C    1) IF THE FIRST DATA STRUCTURE IS IN UPDATE
C       TEST IF IT IS A POSTSCRIPT FILE CREATED BY DRAGON
C       AND PREPARE FILE FOR OUTPUT
C----
      CALL PSPFIL(ISPSP,JENTRY(1),HENTRY(1),NPAGE)
C----
C  SCAN OVER DATA STRUCTURES AND PROCESS STRUCTURE ONE AFTER THE OTHER
C----
      IF(ITYPE .EQ. 0 .OR. ITYPE .EQ. 1) THEN
        NUNKNO=1
        IF(ITYPE .EQ. 0) THEN
          NAMLEG='Region'
        ELSE
          NAMLEG='Mixture'
        ENDIF
        CALL SETARA(RBASE,NUNKNO,IFLUX)
        RBASE(IFLUX)=0.0
        DO 100 IENGT=2,NENTRY
C----
C  READ SIGNATURE OF NEXT DATA STRUCTURE AND TEST IF
C  PSP CAN BE USED TO PROCESS THIS DATA STRUCTURE
C----
          IF(IENTRY(IENGT) .NE. 1   .AND.
     >       IENTRY(IENGT) .NE. 2 ) CALL XABORT(NAMSBR//
     >      ': NEXT DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
          IF(JENTRY(IENGT) .NE. 2 ) CALL XABORT(NAMSBR//
     >      ': NEXT DATA STRUCTURE NOT IN READ-ONLY MODE')
          IPGT=KENTRY(IENGT)
          NAMGT=HENTRY(IENGT)
          CALL LCMGET(IPGT,'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          ITRK=1
C----
C  TEST IF GEOMETRY OR EXCELL TRACK DATA STRUCTURE
C----
          IF(HSIGN .EQ. 'L_GEOM      ') THEN
            ITRK=0
            NAMGEO=HENTRY(IENGT)
          ELSE IF(HSIGN .EQ. 'L_TRACK     ') THEN
            CALL LCMGET(IPGT,'TRACK-TYPE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
            IF(HSIGN .NE. 'EXCELL') ITRK=-1
          ELSE 
            GO TO 115 
          ENDIF
C----
C  FOR GEOMETRY OPTION CALL AXGGEO
C  TO GENERATE TEMPORARY TRACKING STRUCTURE
C---- 
          IF(ITRK .EQ. 0) THEN 
            LASS=LDRASS(IPGT,1,IPRINT)
            CALL  LCMOP(IPTRK2,NAMTR2,IMODT2,IMEDT2,IPRIN2)
            CALL AXGGEO(IPGT  ,IPTRK2,IPRINT,NAMGEO) 
            IPGT=IPTRK2
          ENDIF
C----
C  CALL PSPTRK TO GENERATE POSTSCRIPT
C----
          CALL PSPTRK(IPRINT,ISPSP ,ITYPE ,ICOLR ,IPGT  ,NAMGT ,
     >                NAMLEG,NUNKNO,RBASE(IFLUX) )
          IF(ITRK .EQ. 0) THEN 
            CALL LCMCL(IPTRK2,ICLST2)
          ENDIF
          CALL PSCUTP(ISPSP)
          IF(IENGT .NE. NENTRY) THEN
            NPAGE=NPAGE+1
            CALL PSPAGE(ISPSP,NPAGE,XYPOS)
          ENDIF
 115      CONTINUE
 100    CONTINUE
        CALL RLSARA(RBASE(IFLUX))
      ELSE IF(ITYPE .EQ. 2 .OR. ITYPE .EQ. 3) THEN
C----
C  TEST SECOND DATA STRUCTURE
C----
        IENGT=2
        IF(IENTRY(IENGT) .NE. 1   .AND.
     >     IENTRY(IENGT) .NE. 2 ) CALL XABORT(NAMSBR//
     >    ': SECOND DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
        IF(JENTRY(IENGT) .NE. 2 ) CALL XABORT(NAMSBR//
     >    ': SECOND DATA STRUCTURE NOT IN READ-ONLY MODE')
        IPGT=KENTRY(IENGT)
        NAMGT=HENTRY(IENGT)
        CALL LCMGET(IPGT,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        ITRK=1
C----
C  TEST IF GEOMETRY OR EXCELL TRACK DATA STRUCTURE
C----
        IF(HSIGN .EQ. 'L_GEOM      ') THEN
          ITRK=0
          NAMGEO=HENTRY(IENGT)
        ELSE IF(HSIGN .EQ. 'L_TRACK     ') THEN
          CALL LCMGET(IPGT,'TRACK-TYPE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
          IF(HSIGN .NE. 'EXCELL') ITRK=-1
        ENDIF
C----
C  TEST IF THIRD DATA STRUCTURE IS A FLUX
C----
        IENFL=3
        IF(IENTRY(IENFL) .NE. 1   .AND.
     >     IENTRY(IENFL) .NE. 2 ) CALL XABORT(NAMSBR//
     >    ': THIRD DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
        IF(JENTRY(IENFL) .NE. 2 ) CALL XABORT(NAMSBR//
     >    ': THIRD DATA STRUCTURE NOT IN READ-ONLY MODE')
        IPFL=KENTRY(IENFL)
        CALL LCMGET(IPFL,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,NTC)
        IF(HSIGN .NE. 'L_FLUX      ') CALL XABORT(NAMSBR//
     >    ': THIRD DATA STRUCTURE NOT A FLUX AS EXPECTED')
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPFL,'STATE-VECTOR',ISTATE)
        NGROUP=ISTATE(9)
        NUNKNO=ISTATE(10)
        JPFL=LCMGID(IPFL,'FLUX')
        CALL SETARA(RBASE,NUNKNO,IFLUX)
        IF(ITYPE .EQ. 2) THEN
          CALL XDRSET(RBASE(IFLUX),NUNKNO,0.0)
          CALL SETARA(RBASE,NUNKNO,ITFLX)
          DO 110 IGR=1,NGROUP
            CALL LCMGDL(JPFL,IGR,RBASE(ITFLX))
            DO 111 IUN=0,NUNKNO-1
              RBASE(IFLUX+IUN)=RBASE(IFLUX+IUN)+RBASE(ITFLX+IUN)
 111        CONTINUE
 110      CONTINUE
          CALL RLSARA(RBASE(ITFLX))
          NGT=1
          NAMLEG='Group integrated flux'
        ELSE
          NGT=NGROUP
        ENDIF
        DO 120 IGR=1,NGT
          IF(ITYPE .EQ. 3) THEN
            WRITE(NAMLEG,'(A14,I3)') 'Flux in group ',IGR
            CALL LCMGDL(JPFL,IGR,RBASE(IFLUX))
          ENDIF
C----
C  FOR GEOMETRY OPTION CALL AXGGEO
C  TO GENERATE TEMPORARY TRACKING STRUCTURE
C---- 
          IF(ITRK .EQ. 0) THEN 
            CALL  LCMOP(IPTRK2,NAMTR2,IMODT2,IMEDT2,IPRIN2)
            CALL AXGGEO(IPGT  ,IPTRK2,IPRINT,NAMGEO) 
            IPGT=IPTRK2
          ENDIF
C----
C  CALL PSPTRK TO GENERATE POSTSCRIPT
C----
          CALL PSPTRK(IPRINT,ISPSP ,ITYPE ,ICOLR ,IPGT  ,NAMGT ,
     >                NAMLEG,NUNKNO,RBASE(IFLUX) )
          IF(ITRK .EQ. 0) THEN 
            CALL LCMCL(IPTRK2,ICLST2)
          ENDIF
          CALL PSCUTP(ISPSP)
          IF(IGR .NE. NGT) THEN
            NPAGE=NPAGE+1
            CALL PSPAGE(ISPSP,NPAGE,XYPOS)
          ENDIF
 120    CONTINUE
        CALL RLSARA(RBASE(IFLUX))
      ENDIF
      RETURN
      END
