*DECK PSPNXT
      SUBROUTINE PSPNXT(IPRINT,ISPSP ,ICOLR ,IPTRK ,ITYPBC,MAXMSH,
     >                  NDIM  ,NFSUR ,NFREG ,NUCELL,NBUCEL,
     >                  MXGREG,MAXPIN,COLREG,
     >                  IUNFLD,MATALB,DGMESH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To generate the graphics for a 2-D NXT geometry.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Parameters: input
* IPRINT  print level.
* ISPSP   pointer to the POSTSCRIPT file.
* ICOLR   color set used where:
*         \begin{itemize}
*         \item ICOLR=-4 HSB filling with no contour;
*         \item ICOLR=-3 CYMK filling with no contour;
*         \item ICOLR=-2 RGB filling with no contour;
*         \item ICOLR=-1 BW filling with no contour;
*         \item ICOLR= 0 no filling with contour;
*         \item ICOLR= 1 BW filling with contour;
*         \item ICOLR= 2 RGB filling with contour;
*         \item ICOLR= 3 CMYK filling with contour;
*         \item ICOLR= 4 HSB filling with  contour;
*         \end{itemize}
* IPTRK   pointer to the TRACKING data structure in 
*         update or creation mode.
* ITYPBC  type of cell boundary.
* MAXMSH  maximum number of elements in MESH array.
* NDIM    dimension of the problem.
* NFSUR   number of surfaces.
* NFREG   number of regions.
* NUCELL  number of cell after unfolding in 
*         $X$, $Y$ and $Z$ directions.
* NBUCEL  number of cells in unfolded geometry.
* MXGREG  maximum number of region for any geometry.
* MAXPIN  maximum number of pins in a cell.
* COLREG  region color.
* IUNFLD  description of unfolded geometry.
* MATALB  global mixture/albedo identification vector.
* DGMESH  meshing vector for global geometry.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPRINT,ISPSP,ICOLR,IPTRK,ITYPBC,MAXMSH,
     >                 NDIM,NFSUR,NFREG,NUCELL(3),NBUCEL,
     >                 MXGREG,MAXPIN
      REAL             COLREG(4,NFREG)
      INTEGER          IUNFLD(2,NBUCEL),
     >                 MATALB(-NFSUR:NFREG)
      DOUBLE PRECISION DGMESH(0:MAXMSH,4)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='PSPNXT')
      DOUBLE PRECISION DZERO,DONE,DTWO
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
      DOUBLE PRECISION DIMX,DIMY
      PARAMETER       (DIMX=3.5D0,DIMY=3.5D0)
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          DCMESH,IDREG,ITPIN,DRAPIN
*----
*  Local variables
*----
      INTEGER          KPSP(7)
      CHARACTER        NAMREC*12
      INTEGER          IDIR,ILPD,IX,IY,ICELL,ICEL,ITRN,ILONG,ITYLCM,
     1 ICOOR,REGI,EVENT
      DOUBLE PRECISION RCIRC,ABSC(2),OFFC(2),FACT,CELLPO(2,2)
      REAL             XYPOS(2,4)
*----
*  Data
*----
      CHARACTER        CDIR(4)*1
      SAVE             CDIR
      DATA             CDIR /'X','Y','Z','R'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
*----
*  Initialize ICOL for color treatment
*  and ICONT for contour 
*  KPSP(1)=ICONT
*  KPSP(2)=ICOL
*  KPSP(3)=IWLFAC (0->1.0,1->2.5)
*  KPSP(4)=KFS
*  KPSP(5)=KFR
*  KPSP(6)=KSS
*  KPSP(7)=KSR
*-----
      KPSP(1)=1
      KPSP(2)=ABS(ICOLR)
      KPSP(3)=0
      KPSP(4)=0
      KPSP(5)=0
      KPSP(6)=0
      KPSP(7)=0
      IF(ICOLR .EQ. 0) THEN
        KPSP(3)=1
      ELSE IF(ICOLR .LT. 0) THEN
        KPSP(1)=0
      ELSE
        KPSP(4)=1
        KPSP(7)=1      
      ENDIF
*----
*  Read global mesh for geometry
*  and determine graphics size
*----
      DO IDIR=1,NDIM
        NAMREC='G00000001SM'//CDIR(IDIR)
        ILPD=NUCELL(IDIR)
        CALL LCMGET(IPTRK,NAMREC,DGMESH(0,IDIR))
        ABSC(IDIR)=0.5D0*(DGMESH(ILPD,IDIR)-DGMESH(0,IDIR))
        OFFC(IDIR)=0.5D0*(DGMESH(ILPD,IDIR)+DGMESH(0,IDIR))
      ENDDO
      IF(ITYPBC .EQ. 1) THEN
        IDIR=4
        NAMREC='G00000001SM'//CDIR(IDIR)
        CALL LCMGET(IPTRK,NAMREC,DGMESH(0,IDIR))
        RCIRC=DGMESH(1,IDIR) 
      ELSE
        RCIRC=DZERO
        DO IDIR=1,NDIM
          RCIRC=MAX(RCIRC,ABSC(IDIR))
        ENDDO
      ENDIF
*----
*  Locate pen at center of page
*----
      XYPOS(1,1)=DIMX
      XYPOS(2,1)=DIMY
      CALL PSMOVE(ISPSP,XYPOS,-3)
      FACT=DIMX/RCIRC
      CALL SETARA(IBASE,MXGREG,IDREG)
      CALL SETARA(RBASE,2*4*(MAXMSH+2),DCMESH)
      CALL SETARA(IBASE,3*(MAXPIN),ITPIN)
      CALL SETARA(IBASE,6*2*(MAXPIN),DRAPIN)
*----
*  Scan over $Y$ directions
*----
      ICELL=0
      DO IY=1,NUCELL(2)
*----
*  Scan over $X$ directions
*----
        CELLPO(2,1)=(DGMESH(IY-1,2)-OFFC(2))
        CELLPO(2,2)=(DGMESH(IY,2)-OFFC(2))
        DO IX=1,NUCELL(1)
          CELLPO(1,1)=(DGMESH(IX-1,1)-OFFC(1))
          CELLPO(1,2)=(DGMESH(IX,1)-OFFC(1))
          ICELL=ICELL+1
          ICEL=IUNFLD(1,ICELL)
          ITRN=IUNFLD(2,ICELL)
          IF(ITRN .EQ. 1) THEN
            CALL PSPTCR(IPTRK ,ISPSP ,IPRINT,ICEL  ,NDIM  ,NFREG ,
     >                  MAXMSH,MXGREG,MAXPIN,KPSP  ,COLREG,FACT  ,
     >                  CELLPO,IBASE(IDREG) ,IBASE(ITPIN) ,
     >                  RBASE(DCMESH),RBASE(DRAPIN))
          ENDIF        
        ENDDO
      ENDDO
      CALL RLSARA(RBASE(DRAPIN))
      CALL RLSARA(IBASE(ITPIN))
      CALL RLSARA(RBASE(DCMESH))
      CALL RLSARA(IBASE(IDREG))
      IF(IPRINT .GE. 100) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
*----
*  plot MC: neutron paths if present
*----
      CALL LCMLEN(IPTRK,'MCpoints',ILONG,ITYLCM)
      IF (ITYLCM.EQ.0) THEN
         CALL LCMSIX(IPTRK,'MCpoints',1)
         CALL LCMLEN(IPTRK,'REGI',ILONG,ITYLCM)
         CALL SETARA(IBASE,ILONG,REGI)
         CALL SETARA(IBASE,ILONG,EVENT)
         CALL SETARA(RBASE,6*ILONG,ICOOR)
         CALL LCMGET(IPTRK,'COORD',RBASE(ICOOR))
         CALL LCMGET(IPTRK,'REGI',IBASE(REGI))
         CALL LCMGET(IPTRK,'EVENT',IBASE(EVENT))
         CALL PSPMCP(ISPSP,OFFC,FACT,ILONG,RBASE(ICOOR),IBASE(REGI),
     1        IBASE(EVENT))
         CALL RLSARA(IBASE(EVENT)) 
         CALL RLSARA(IBASE(REGI)) 
         CALL RLSARA(RBASE(ICOOR))
         CALL LCMSIX(IPTRK,' ',2)
      ENDIF
*----
*  Save track normalisation vector
*----
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
      END 
