*DECK PSPTRK
      SUBROUTINE PSPTRK(IPRINT,ISPSP ,ITYPE ,ICOLR ,IPTRKT,NAMFIL,
     >                  NAMLEG,NUNKNO,FLUX  )
*
*----------
*
*Purpose:
* To generate a POSTSCRIPT file containing a graphical description
* of a 2-D geometry from an \moc{EXCELL} generated
* \dds{tracking} data structure.
*
*Copyright:
* Copyright (C) 1999 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau.
*
*Update(s): 
* 2006/03/15: Add the possibility to process \moc{NXT}
*             processed 2-D geometry.
*
*
*Parameters: input
* IPRINT  print level.
* ISPSP   pointer to the POSTSCRIPT file.
* ITYPE   identifier for the type of graphics where:
*         \begin{itemize}
*         \item ITYPE=0 when the geometry is colored by region;
*         \item ITYPE=1 when the geometry is colored by mixture;
*         \item ITYPE=2 when the geometry is colored by flux 
*                       (one group);
*         \item ITYPE=3 when the geometry is colored by flux 
*                       (multigroup);
*         \end{itemize}
* ICOLR   color set used where:
*         \begin{itemize}
*         \item ICOLR=-4 HSB filling with no contour;
*         \item ICOLR=-3 CYMK filling with no contour;
*         \item ICOLR=-2 RGB filling with no contour;
*         \item ICOLR=-1 BW filling with no contour;
*         \item ICOLR= 0 no filling with contour;
*         \item ICOLR= 1 BW filling with contour;
*         \item ICOLR= 2 RGB filling with contour;
*         \item ICOLR= 3 CMYK filling with contour;
*         \item ICOLR= 4 HSB filling with  contour;
*         \end{itemize}
* IPTRKT  pointer to the TRACKING data structure.
* NAMFIL  geometry file name.
* NAMLEG  legend name.
* NUNKNO  number of flux unknowns.
*
*Parameters: temporary storage
* FLUX    flux storage array.
*
*----------
*
      IMPLICIT     NONE
*----
*  Subroutine arguments
*----
      INTEGER      IPRINT,ISPSP,ITYPE,ICOLR,IPTRKT,NUNKNO
      CHARACTER    NAMFIL*12,NAMLEG*24
      REAL         FLUX(NUNKNO)
*----
*  Local parameters
*----
      INTEGER      IOUT
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NAMSBR='PSPTRK')
      INTEGER      NSTATE
      PARAMETER   (NSTATE=40)
      INTEGER      ILCMUP,ILCMDN
      PARAMETER   (ILCMUP=1,ILCMDN=2)
*----
*  Memory allocation variables
*----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      NXRI,NRINFO,JRODR,NRODR,JRODS,NRODS,
     >             JRAN,JREMSH,INDEX,MAXDIM,MINDIM,MATALB,
     >             KEYMRG,JCOLRG,KEYFLX
      INTEGER      IUNFLD,DGMESH
*----
*  Local variables
*----
      INTEGER      ISTATE(NSTATE),IPARAM(NSTATE),ITROP,
     >             NDIM,NVOL,NSUR,NSURX,NBAN,NUNK,NRT,MSROD,
     >             MAROD,NTOTCL,MAXR,NUNKT,NREGT,NNSUR
      REAL         COTE
      INTEGER      IEDIMG(NSTATE),ITYPBC,NBUCEL,NUCELL(3),
     >             MAXMSH,MAXMDH,MAXREG,NBTCLS,MAXPIN,MAXMSP,
     >             MAXRSP,NFSUR,NFREG,MXGREG
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
*----
*  Get state vector from \dds{tracking}
*  and check if a graphical description
*  of the geometry is possible.
*----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRKT,'STATE-VECTOR',ISTATE)
      IF(ITYPE .EQ. 2 .OR. ITYPE .EQ. 3) THEN
        NREGT=ISTATE(1)
        NUNKT=ISTATE(2)
        IF(NUNKNO .NE. NUNKT) CALL XABORT(NAMSBR//
     >  ': Tracking is not consistent with fluxes')
        CALL SETARA(IBASE,NREGT,KEYFLX)
        CALL LCMGET(IPTRKT,'KEYFLX      ',IBASE(KEYFLX))
      ELSE
        NREGT=1
        CALL SETARA(IBASE,NREGT,KEYFLX)
        IBASE(KEYFLX)=0
      ENDIF
      ITROP=ISTATE(7)
      IF(ITROP .EQ. 4) THEN
*----
*  NXT processed geometry
*----
        NSUR=ISTATE(5)
        NNSUR=-NSUR
        NVOL=ISTATE(1)
        NUNK=NSUR+NVOL+1
        CALL SETARA(RBASE,4*NVOL,JCOLRG)
        CALL SETARA(IBASE,NUNK,KEYMRG)
        CALL SETARA(IBASE,NUNK,MATALB)
        CALL XDISET(IEDIMG,NSTATE,0)
        CALL LCMSIX(IPTRKT,'NXTRecords  ',ILCMUP)
        CALL LCMGET(IPTRKT,'G00000001DIM',IEDIMG)
        NDIM=IEDIMG( 1)
        IF(NDIM .EQ. 2) THEN
          CALL LCMGET(IPTRKT,'MATALB      ',IBASE(MATALB))
          CALL LCMGET(IPTRKT,'KEYMRG      ',IBASE(KEYMRG))
*----
*  Produce legend
*----
          CALL PSPLEG(IPRINT,ISPSP ,ITYPE ,ICOLR ,NNSUR ,NVOL  ,
     >                NAMLEG,NUNKNO,FLUX  ,NREGT ,
     >                IBASE(MATALB),IBASE(KEYMRG),IBASE(KEYFLX),
     >                RBASE(JCOLRG))
*----
*  Produce graphical description of geometry
*----
          ITYPBC   =IEDIMG( 2)
          NBUCEL   =IEDIMG( 5)
          NUCELL(1)=IEDIMG(13)
          NUCELL(2)=IEDIMG(14)
          NUCELL(3)=IEDIMG(15)
          MAXMSH   =IEDIMG(16)
          MAXREG   =IEDIMG(17)
          NBTCLS   =IEDIMG(18)
          MAXPIN   =IEDIMG(19)
          MAXMSP   =IEDIMG(20)
          MAXRSP   =IEDIMG(21)
          NFSUR    =IEDIMG(22)
          NFREG    =IEDIMG(23)
          MXGREG   =IEDIMG(25)
          NUNK=NFSUR+NFREG+1
          MAXMDH=MAX(MAXMSH,MAXMSP,MAXREG)
          CALL SETARA(IBASE,2*NBUCEL,IUNFLD)
          CALL SETARA(RBASE,2*(MAXMDH+2)*4,DGMESH)
          CALL LCMGET(IPTRKT,'G00000001CUF',IBASE(IUNFLD))
          CALL LCMGET(IPTRKT,'MATALB      ',IBASE(MATALB))
          CALL PSPNXT(IPRINT,ISPSP ,ICOLR ,IPTRKT,ITYPBC,MAXMDH,
     >                NDIM  ,NFSUR ,NFREG ,NUCELL,NBUCEL,
     >                MXGREG,MAXPIN,RBASE(JCOLRG),
     >                IBASE(IUNFLD),IBASE(MATALB),RBASE(DGMESH))
          CALL RLSARA(RBASE(DGMESH))
          CALL RLSARA(IBASE(IUNFLD))
        ELSE
          WRITE(IOUT,9000)
        ENDIF
        CALL LCMSIX(IPTRKT,'NXTRecords  ',ILCMDN)
      ELSE
        CALL LCMSIX(IPTRKT,'EXCELL      ',1)
        CALL XDISET(IPARAM,NSTATE,0)
        CALL LCMGET(IPTRKT,'STATE-VECTOR',IPARAM)
        NDIM=IPARAM(1)
        NSUR=-IPARAM(2)
        NVOL=IPARAM(3)
        NSURX=IPARAM(4)
        NBAN=IPARAM(5)
        NUNK=IPARAM(6)
        CALL SETARA(RBASE,4*NVOL,JCOLRG)
        CALL SETARA(IBASE,NUNK,KEYMRG)
        CALL SETARA(IBASE,NUNK,MATALB)
        CALL LCMGET(IPTRKT,'MATALB      ',IBASE(MATALB))
        CALL LCMGET(IPTRKT,'KEYMRG      ',IBASE(KEYMRG))
        IF(ITROP .EQ. 3) THEN
*----
*  EXCELL based CLUSTER geometries
*----
          NRT=IPARAM(7)
          MSROD=IPARAM(8)
          MAROD=IPARAM(9)
          CALL SETARA(RBASE,NBAN,JRAN)
          CALL SETARA(IBASE,3*NRT,NRODS)
          CALL SETARA(RBASE,2*NRT,JRODS)
          CALL SETARA(IBASE,NRT,NRODR)
          CALL SETARA(RBASE,MSROD*NRT,JRODR)
          CALL SETARA(IBASE,2*NBAN,NRINFO)
          CALL SETARA(IBASE,NRT*NBAN,NXRI)
          CALL LCMGET(IPTRKT,'RAN         ',RBASE(JRAN))
          IF(NSURX .EQ. 4)
     >    CALL LCMGET(IPTRKT,'COTE        ',COTE         )
          CALL LCMGET(IPTRKT,'NRODS       ',IBASE(NRODS))
          CALL LCMGET(IPTRKT,'RODS        ',RBASE(JRODS))
          CALL LCMGET(IPTRKT,'NRODR       ',IBASE(NRODR))
          CALL LCMGET(IPTRKT,'RODR        ',RBASE(JRODR))
          CALL LCMGET(IPTRKT,'NRINFO      ',IBASE(NRINFO))
          CALL LCMGET(IPTRKT,'NXRI        ',IBASE(NXRI))
*----
*  Produce legend
*----
          CALL PSPLEG(IPRINT,ISPSP ,ITYPE ,ICOLR ,NSUR  ,NVOL  ,
     >                NAMLEG,NUNKNO,FLUX  ,NREGT ,
     >                IBASE(MATALB),IBASE(KEYMRG),IBASE(KEYFLX),
     >                RBASE(JCOLRG))
*----
*  Produce graphical description of geometry
*----
          CALL PSPXCG(IPRINT,ISPSP ,ICOLR ,NBAN  ,NRT   ,MSROD ,
     >                NSURX ,NSUR  ,NVOL  ,COTE  ,
     >                RBASE(JRAN)  ,IBASE(NRODS) ,RBASE(JRODS) ,
     >                RBASE(JRODR) ,IBASE(NRINFO),IBASE(NRODR) ,
     >                IBASE(NXRI)  ,IBASE(KEYMRG),RBASE(JCOLRG))
          CALL RLSARA(IBASE(NXRI))
          CALL RLSARA(IBASE(NRINFO))
          CALL RLSARA(RBASE(JRODR))
          CALL RLSARA(IBASE(NRODR))  
          CALL RLSARA(RBASE(JRODS))
          CALL RLSARA(IBASE(NRODS))
          CALL RLSARA(RBASE(JRAN))
        ELSE IF(ITROP .EQ. 2 ) THEN
*----
*  EXCELL based hexagonal geometries
*  Not available yet
*----
*          CALL PSPXHX(IPRINT,IPTRKT,TITREC)
          WRITE(IOUT,6001)
        ELSE IF(ITROP .EQ. 1 ) THEN
*----
*  EXCELL based Cartesian geometries
*----
          NTOTCL=NSURX
          MAXR=NBAN
          CALL SETARA(IBASE,NTOTCL,MINDIM)
          CALL SETARA(IBASE,NTOTCL,MAXDIM)
          CALL SETARA(IBASE,4*NUNK,INDEX)
          CALL SETARA(RBASE,MAXR,JREMSH)
          CALL LCMGET(IPTRKT,'MINDIM      ',IBASE(MINDIM))
          CALL LCMGET(IPTRKT,'MAXDIM      ',IBASE(MAXDIM))
          CALL LCMGET(IPTRKT,'INDEX       ',IBASE(INDEX))
          CALL LCMGET(IPTRKT,'REMESH      ',RBASE(JREMSH))
          IF(NDIM .EQ. 2) THEN
*----
*  Produce legend
*----
            CALL PSPLEG(IPRINT,ISPSP ,ITYPE ,ICOLR ,NSUR  ,NVOL  ,
     >                  NAMLEG,NUNKNO,FLUX  ,NREGT ,
     >                  IBASE(MATALB),IBASE(KEYMRG),IBASE(KEYFLX),
     >                  RBASE(JCOLRG))
*----
*  Produce graphical description of geometry
*----
            CALL PSPXEL(IPRINT,ISPSP ,ICOLR ,NDIM  ,NSUR  ,NVOL  ,
     >                  NTOTCL,MAXR  ,
     >                  IBASE(MINDIM),IBASE(MAXDIM),IBASE(KEYMRG),
     >                  IBASE(INDEX) ,RBASE(JREMSH),RBASE(JCOLRG))
          ELSE
            WRITE(IOUT,9000)
          ENDIF
          CALL RLSARA(RBASE(JREMSH))
          CALL RLSARA(IBASE(INDEX))
          CALL RLSARA(IBASE(MAXDIM))
          CALL RLSARA(IBASE(MINDIM))
        ENDIF
        CALL LCMSIX(IPTRKT,'EXCELL      ',2)
      ENDIF
      CALL RLSARA(IBASE(MATALB))
      CALL RLSARA(IBASE(KEYMRG))
      CALL RLSARA(RBASE(JCOLRG))
      CALL RLSARA(IBASE(KEYFLX))
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 9000 FORMAT(' PSP: does not work yet for 3-D',
     >       ' Cartesian geometries')
      END
