*DECK SPHBIV
      SUBROUTINE SPHBIV(IPTRK2,JPSYS,NPSYS,NREG,NUN,NMERGE,NGCOND,
     1 NBMIX,MAT,VOL,KEY,MERG,SUNMER,FLXMER,IEX,FUNKNO,SUNKNO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* diffusion theory calculation with BIVAC over the macro-geometry.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK2  pointer to the tracking of the macro-geometry
*         (L_TRACK signature).
* JPSYS   pointer to the 'GROUP' entry in the system LCM object.
* NPSYS   group masks.
* IPRINT  print flag (equal to 0 for no print).
* NREG    number of macro-regions (in the macro calculation).
* NUN     number of unknowns in the macro-calculation.
* NMERGE  number of merged regions.
* NGCOND  number of condensed groups.
* NBMIX   number of macro-mixtures.
* MAT     mixture index per macro-region.
* VOL     volume of macro-regions.
* KEY     position of the flux components associated with each volume.
* MERG    index of merged macro-regions.
* SUNMER  incoming source (scattering+fission) cross sections.
* FLXMER  flux estimate per mixture.
* IEX     iteration index.
* FUNKNO  neutron flux.
*
*Parameters: output
* SUNKNO  neutron sources.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK2,JPSYS,NPSYS(NGCOND),NREG,NUN,NMERGE,NGCOND,NBMIX,
     1 MAT(NREG),KEY(NREG),MERG(NREG),IEX
      REAL VOL(NREG),SUNMER(NMERGE,NGCOND,NGCOND),FLXMER(NMERGE,NGCOND),
     1 FUNKNO(NUN,NGCOND),SUNKNO(NUN,NGCOND)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (NSTATE=40)
      INTEGER     IPAR(NSTATE),IBASE(1)
      LOGICAL     CYLIND 
      CHARACTER   HSMG*131
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER BIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK2,'STATE-VECTOR',IPAR)
      ITYPE=IPAR(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=IPAR(8)
      ICOL=IPAR(9)
      ISPLH=IPAR(10)
      LL4=IPAR(11)
      LX=IPAR(12)
      LY=IPAR(13)
      NLF=IPAR(14)
      ISCAT=IPAR(16)
      IF(ISCAT.GT.1) THEN
         WRITE(HSMG,'(45HSPHBIV: MACRO-CALCULATION WITH ANISOTROPIC SC,
     1   58HATTERING CURRENTLY NOT IMPLEMENTED; USE SCAT 1 KEYWORD IN ,
     2   13HBIVACT: DATA.)')
         CALL XABORT(HSMG)
      ENDIF
      CALL LCMLEN(IPTRK2,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK2,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY,IXX)
      CALL SETARA(BASE,LX*LY,IYY)
      CALL SETARA(BASE,LX*LY,IDD)
      CALL SETARA(BASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      IF(ITYPE.EQ.8) THEN
         CALL LCMGET(IPTRK2,'SIDE',SIDE)
      ELSE
         CALL LCMGET(IPTRK2,'XX',BASE(IXX))
         CALL LCMGET(IPTRK2,'YY',BASE(IYY))
         CALL LCMGET(IPTRK2,'DD',BASE(IDD))
      ENDIF
      CALL LCMGET(IPTRK2,'KN',BASE(IKN))
      CALL LCMGET(IPTRK2,'QFR',BASE(IQFR))
*----
*  COMPUTE THE NEUTRON SOURCES IN GROUP IGR.
*----
      DO 30 IGR=1,NGCOND
      CALL XDRSET(SUNKNO(1,IGR),NUN,0.0)
      IF(NPSYS(IGR).EQ.0) GO TO 30
      KPSYS=LCMDIL(JPSYS,IGR)
      COUR=0.0
      DO 10 I=1,NREG
      IF(MAT(I).GT.NBMIX) CALL XABORT('SPHBIV: NBMIX OVERFLOW.')
   10 CONTINUE
      CALL SETARA(BASE,NBMIX,IGAR1)
      CALL SETARA(BASE,NBMIX,IGAR2)
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
*        NON-HEXAGONAL GEOMETRY WITH PRIMAL DIFFUSION DISCRETIZATION
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL LCMLEN(IPTRK2,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK2,'R',BASE(IR))
         CALL LCMGET(IPTRK2,'RS',BASE(IRS))
         CALL LCMGET(IPTRK2,'T',BASE(IT))
         CALL LCMGET(IPTRK2,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK2,' ',2)
         CALL BIVFSO(IEX,MAXKN,NGCOND,NMERGE,FLXMER,SUNMER(1,1,IGR),
     1   CYLIND,NREG,NUN,NBMIX,BASE(IXX),BASE(IDD),MAT,IBASE(IKN),
     2   BASE(IQFR),VOL,MERG,COUR,FUNKNO,LC,BASE(IT),BASE(ITS),
     3   BASE(IR),BASE(IRS),SUNKNO(1,IGR),BASE(IGAR1),BASE(IGAR2))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
*        NON-HEXAGONAL GEOMETRY WITH THOMAS-RAVIART DISCRETIZATION
         CALL BIVGSO(IEX,NGCOND,NMERGE,FLXMER,SUNMER(1,1,IGR),CYLIND,
     1   IELEM,NREG,NUN,NBMIX,MAT,BASE(IXX),BASE(IDD),IBASE(IKN),
     2   BASE(IQFR),VOL,MERG,COUR,FUNKNO,SUNKNO(1,IGR),BASE(IGAR1),
     3   BASE(IGAR2))
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
*        HEXAGONAL GEOMETRY WITH PRIMAL DIFFUSION DISCRETIZATION
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL SETARA(BASE,2,IT)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK2,'T',BASE(IT))
         CALL LCMGET(IPTRK2,'RH',BASE(IRH))
         CALL LCMGET(IPTRK2,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK2,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL BIVFSH(IEX,MAXKN,MAXQF,NGCOND,NMERGE,FLXMER,
     1   SUNMER(1,1,IGR),NREG,NUN,ISPLH,NELEM,NBMIX,SIDE,MAT,
     2   IBASE(IKN),BASE(IQFR),VOL,MERG,COUR,FUNKNO,BASE(IT),
     3   BASE(IRH),BASE(IRT),SUNKNO(1,IGR),BASE(IGAR1),BASE(IGAR2))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(ICOL.EQ.4)) THEN
*        HEXAGONAL GEOMETRY WITH MCFD DISCRETIZATION
         CALL SETARA(BASE,NBMIX+1,IDIFF)
         CALL LCMGET(KPSYS,'DRAGON-DIFF',BASE(IDIFF))
         CALL BIVGSH(IEX,MAXKN,MAXQF,NGCOND,NMERGE,FLXMER,
     1   SUNMER(1,1,IGR),NREG,NUN,LL4,ISPLH,NBMIX,SIDE,MAT,IBASE(IKN),
     2   BASE(IQFR),VOL,BASE(IDIFF+1),MERG,COUR,FUNKNO,SUNKNO(1,IGR),
     3   BASE(IGAR1),BASE(IGAR2))
         CALL RLSARA(BASE(IDIFF))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
*        HEXAGONAL GEOMETRY WITH THOMAS-RAVIART-SCHNEIDER DISCRETIZATION
         CALL LCMIOF(IPTRK2,'ZGLOB',IZGLOB)
         CALL BIVGSR(IEX,NGCOND,NMERGE,FLXMER,SUNMER(1,1,IGR),IELEM,
     1   ISPLH,LX,NUN,NBMIX,SIDE,MAT,IBASE(IZGLOB),IBASE(IKN),
     2   BASE(IQFR),MERG,COUR,FUNKNO,SUNKNO(1,IGR),BASE(IGAR1),
     3   BASE(IGAR2))
      ELSE
         CALL XABORT('SPHBIV: UNKNOWN TYPE OF GEOMETRY(1).')
      ENDIF
      CALL RLSARA(BASE(IGAR2))
      CALL RLSARA(BASE(IGAR1))
*----
*  DIVISION OF THE EVEN-PARITY SOURCES BY THE VOLUMES.
*----
      DO 20 I=1,NREG
      IF(MAT(I).EQ.0) GO TO 20
      JND1=KEY(I)
      SUNKNO(JND1,IGR)=SUNKNO(JND1,IGR)/VOL(I)
   20 CONTINUE
   30 CONTINUE
*----
*  RELEASE BIVAC SPECIFIC TRACKING INFORMATION.
*----
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(BASE(IKN))
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      RETURN
  300 FORMAT(/43H SPHBIV: FLUX NORMALIZATION FACTOR IN GROUP,I4,1H=,1P,
     1 E13.5)
  310 FORMAT(/26H SPHBIV: VALUES OF VECTOR ,A6,4H ARE/(1X,1P,10E13.5))
      END
