*DECK SPHDRV
      SUBROUTINE SPHDRV(IPEDIT,IPGEO2,IPFLUX,IPMACR,IPSYS,IPRINT,
     1 ITRANC,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,ILEAKS,NWGTH,NL,NIFISS,
     2 ILK,MATCOD,VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,IPSPH,IPTRK2,
     3 IFTRK2,CURNAM,CTITRE,SPH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the SPH factors. These factors are used to multiply the
* cross sections and to divide the fluxes. The SPH factors calculation
* is generally application dependent. New SPH algorithms should be
* implemented in this driver.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPEDIT  pointer to edit information (L_EDIT signature).
* IPGEO2  pointer to the macro-geometry (L_GEOM signature). Equal to
*         zero if the macro-geometry is not defined.
* IPFLUX  pointer to the reference solution (L_FLUX signature).
* IPMACR  pointer to the reference macrolib (L_MACROLIB signature).
* IPSYS   pointer to the reference Pij (L_PIJ signature).
* IPRINT  print flag (equal to 0 for no print).
* ITRANC  type of transport correction.
* NGROUP  number of groups in the reference calculation.
* NGCOND  number of condensed groups.
* NBMIX   number of mixtures.
* NREGIO  number of regions in the reference calculation.
* NMERGE  number of merged regions.
* ILEAKS  type of leakage calculation: =0: no leakage; =1: homogeneous
*         leakage (Diffon); =2: isotropic streaming (Ecco);
*         =3: anisotropic streaming (Tibere).
* NWGTH   type of weighting for P1 cross section information:
*         = 0 P0; = 1 P1. P1-weighted SPH factors are computed if NWGTH
*         is greater than zero.
* NL      number of Legendre orders of the scattering cross sections.
* NIFISS  number of fissile isotopes.
* ILK     leakage flag (ILK=.true. if neutron leakage through external
*         boundary is present).
* MATCOD  mixture index per region.
* VOLUME  volume of regions.
* KEYFLX  position of average fluxes in unknown array.
* IGCOND  limits of condensed groups.
* IMERGE  position of merged region.
* FLUXES  fluxes.
* IPSPH   pointer to the LCM directory containing SPH-related options.
* IPTRK2  pointer to the macro-tracking LCM object.
* IFTRK2  pointer to the macro-tracking binary sequential file.
* CURNAM  name of the LCM directory where the SPH factors are read if
*         NSPH=1.
* CTITRE  title.
*
*Parameters: output
* SPH     SPH homogenization factors.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   IPEDIT,IPGEO2,IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,NGROUP,
     1          NGCOND,NBMIX,NREGIO,NMERGE,ILEAKS,NWGTH,NL,NIFISS,
     2          MATCOD(NREGIO),KEYFLX(NREGIO),IGCOND(NGCOND),
     3          IMERGE(NREGIO),IPSPH,IPTRK2,IFTRK2
      REAL      VOLUME(NREGIO),FLUXES(NREGIO,NGROUP,NWGTH+1),
     1          SPH(NMERGE,NGCOND,NWGTH+1)
      LOGICAL   ILK
      CHARACTER CURNAM*(*),CTITRE*(*)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MAXT=3,NSTATE=40)
      LOGICAL   LNEWDR,EMPTY,LCM
      CHARACTER HENTRY(MAXT)*12,TEXT12*12,CDOOR*12,MACGEO*12,CNDOOR*12
      INTEGER   ISTATE(NSTATE),KCHAR(18),IENTRY(MAXT),JENTRY(MAXT),
     >          KENTRY(MAXT),NCODE(6),IBASE(1)
      DOUBLE PRECISION DBLINP
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*----
*  RECOVER SPH-RELATED INFORMATION
*----
      IF(IPSPH.EQ.0) CALL XABORT('SPHDRV: NO SPH DIRECTORY AVAILABLE.')
      CALL LCMGET(IPSPH,'STATE-VECTOR',ISTATE)
      NSPH=ISTATE(1)
      KSPH=ISTATE(2)
      MAXIT=ISTATE(3)
      MAXNBI=ISTATE(4)
      IBELL=ISTATE(5)
      IGAP=ISTATE(6)
      IROW=ISTATE(7)
      IF(NSPH.GE.2) THEN
         CALL LCMGET(IPSPH,'SPH$TRK',KCHAR)
         WRITE(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMGET(IPSPH,'SPH-EPSILON',EPSPH)
      ENDIF
*----
*  TRACKING OF THE MACRO-GEOMETRY
*----
      IF(IPTRK2.NE.0) THEN
         CALL LCMGET(IPTRK2,'SIGNATURE',KCHAR)
         WRITE(TEXT12,'(3A4)') (KCHAR(ITC),ITC=1,3)
         IF(TEXT12.NE.'L_TRACK') THEN
            CALL XABORT('SPHDRV: 2ND TRACKING DATA STRUCTURE EXPECTED.')
         ENDIF
         IFTRAK=IFTRK2
         CALL LCMGET(IPTRK2,'STATE-VECTOR',ISTATE)
         NREG2=ISTATE(1)
         NUN2=ISTATE(2)
         ILK=ISTATE(3).EQ.0
      ENDIF
      LNEWDR=.FALSE.
      IF((NSPH.EQ.1).OR.(NSPH.EQ.2)) THEN
*        DO-IT-YOURSELF EQUIVALENCE OR HOMOGENEOUS MACRO-CALCULATION.
         NREG2=NREGIO
         NUN2=NREGIO
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2,IKEY2)
         CALL SETARA(IBASE,NREG2,IMERG2)
         NBMIX2=0
         DO 10 IREG=1,NREG2
         RBASE(IVOL2+IREG-1)=VOLUME(IREG)
         IBASE(IMAT2+IREG-1)=MATCOD(IREG)
         IBASE(IKEY2+IREG-1)=IREG
         NBMIX2=MAX(NBMIX2,IMERGE(IREG))
   10    IBASE(IMERG2+IREG-1)=IMERGE(IREG)
         IF(NBMIX2.NE.NMERGE) CALL XABORT('SPHDRV: TRACKING FAILURE.')
         ILK=.FALSE.
      ELSE IF(IPTRK2.NE.0) THEN
*        RECOVER AND USE AN EXISTING MACRO-TRACKING.
         CALL LCMGET(IPTRK2,'TRACK-TYPE',KCHAR)
         WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
         IF(CDOOR.EQ.'MCCG') THEN
            CALL LCMLEN(IPTRK2,'KEYFLX',LKFL,ITYLCM)
            NFUNL=LKFL/NREG2
         ELSE
            NFUNL=1
         ENDIF
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2*NFUNL,IKEY2)
         CALL SETARA(IBASE,NREG2,IMERG2)
         CALL LCMGET(IPTRK2,'VOLUME',RBASE(IVOL2))
         CALL LCMGET(IPTRK2,'MATCOD',IBASE(IMAT2))
         CALL LCMGET(IPTRK2,'KEYFLX',IBASE(IKEY2))
         NBMIX2=0
         DO 20 IREG=1,NREG2
         NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
   20    IBASE(IMERG2+IREG-1)=IBASE(IMAT2+IREG-1)
         IF(NBMIX2.NE.NMERGE) CALL XABORT('SPHDRV: TRACKING FAILURE.')
*
*        RECOVER TABULATED FUNCTIONS.
         CALL XDRTA2(IPTRK2)
      ELSE IF((NSPH.EQ.0).OR.(NSPH.EQ.3).OR.(NSPH.EQ.4)) THEN
*        TRACK AN EXISTING MACRO-GEOMETRY NAMED MACGEO.
         IF(IPGEO2.EQ.0) CALL XABORT('SPHDRV: NO MACRO-GEOMETRY DEFINE'
     1   //'D. USE ''MERGE CELL'' IN EDI: DATA.')
         LNEWDR=.TRUE.
         CALL LCMINF(IPGEO2,MACGEO,TEXT12,EMPTY,ILONG,LCM)
         IMACG=2
         IF(LCM) IMACG=1
         CALL LCMGET(IPGEO2,'SIGNATURE',KCHAR)
         WRITE(TEXT12,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(TEXT12.NE.'L_GEOM') THEN
            CALL XABORT('SPHDRV: SIGNATURE OF '//MACGEO//' IS '//
     1      TEXT12//'. L_GEOM EXPECTED(2).')
         ENDIF
         CALL LCMGET(IPGEO2,'STATE-VECTOR',ISTATE)
         IF(ISTATE(7).NE.NMERGE) THEN
            WRITE(6,'(16H SPHDRV: NMERGE=,I5,3H VS,I5)')ISTATE(7),NMERGE
            CALL XABORT('SPHDRV: MACRO-GEOMETRY '//MACGEO//' IS NOT CO'
     1      //'MPATIBLE WITH USER-SUPPLIED MERGE INDICES.')
         ENDIF
         IF(IPRINT.GT.0) WRITE(6,110) MACGEO,CNDOOR
	 IF(IPSYS.NE.0) THEN
	    CALL LCMGET(IPGEO2,'NCODE',NCODE)
	    DO 21 I=1,6
	    IF(NCODE(I).EQ.1) THEN
	       NCODE(I)=2
	    ENDIF
   21       CONTINUE
	    CALL LCMPUT(IPGEO2,'NCODE',6,1,NCODE)
	 ENDIF
         CALL XDRSET(SPH,NMERGE*NGCOND*(NWGTH+1),1.0)
         IF(NSPH.EQ.0) RETURN
*
*        BUILD THE CALLING PARAMETERS FOR THE TRACKING MODULE.
         CALL LCMOP(IPTRK2,'SPH$TRK',0,1,0)
         HENTRY(1)='SPH$TRK'
         IENTRY(1)=1
         JENTRY(1)=0
         KENTRY(1)=IPTRK2
         NENTRY=2
         IFTRAK=0
         IF((CNDOOR.EQ.'EXCELL').OR.(CNDOOR.EQ.'NXT')) THEN
            IFTRAK=KDROPN('SPH$FIL',0,2,0,0)
            HENTRY(2)='SPH$FIL'
            IENTRY(2)=3
            JENTRY(2)=0
            KENTRY(2)=IFTRAK
            NENTRY=3
         ENDIF
         HENTRY(NENTRY)=MACGEO
         IENTRY(NENTRY)=IMACG
         JENTRY(NENTRY)=2
         KENTRY(NENTRY)=IPGEO2
         CALL LCMCL(IPGEO2,1)
         CALL LCMOP(IPGEO2,MACGEO,2,IMACG,0)
*
*        TRACKING OF THE MACRO-GEOMETRY.
   25    IF(CNDOOR.EQ.'NXT') THEN
            CALL NXT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
            CNDOOR='EXCELL'
         ELSE IF(CNDOOR.EQ.'EXCELL') THEN
            CALL EXCELT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(CNDOOR.EQ.'SYBIL') THEN
            CALL SYBILT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(CNDOOR.EQ.'BIVAC') THEN
            CALL BIVACT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(CNDOOR.EQ.'TRIVAC') THEN
            CALL TRIVAT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(CNDOOR.EQ.'SN') THEN
            CALL SNT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE IF(CNDOOR.EQ.'MCCG') THEN
            CALL MCCGT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
         ELSE
            CALL XABORT('SPHDRV: '//CNDOOR//' IS AN INVALID TRACKING MO'
     1      //'DULE NAME.')
         ENDIF
*
*        RECOVER TABULATED FUNCTIONS.
         CALL XDRTA2(IPTRK2)
*
         CALL LCMGET(IPTRK2,'STATE-VECTOR',ISTATE)
         NREG2=ISTATE(1)
         NUN2=ISTATE(2)
         IF((CNDOOR.EQ.'SYBIL').AND.(NSPH.EQ.4)) NUN2=NUN2+ISTATE(9)
         ILK=ISTATE(3).EQ.0
         CALL LCMGET(IPTRK2,'TRACK-TYPE',KCHAR)
         WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
         IF(CDOOR.EQ.'MCCG') THEN
            CALL LCMLEN(IPTRK2,'KEYFLX',LKFL,ITYLCM)
            NFUNL=LKFL/NREG2
         ELSE
            NFUNL=1
         ENDIF
         CALL SETARA(RBASE,NREG2,IVOL2)
         CALL SETARA(IBASE,NREG2,IMAT2)
         CALL SETARA(IBASE,NREG2*NFUNL,IKEY2)
         CALL SETARA(IBASE,NREG2,IMERG2)
         CALL LCMGET(IPTRK2,'VOLUME',RBASE(IVOL2))
         CALL LCMGET(IPTRK2,'MATCOD',IBASE(IMAT2))
         CALL LCMGET(IPTRK2,'KEYFLX',IBASE(IKEY2))
         CALL LCMLEN(IPGEO2,'TITLE',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPGEO2,'TITLE',KCHAR)
            WRITE(CTITRE,'(18A4)') (KCHAR(I1),I1=1,18)
         ENDIF
         NBMIX2=0
         DO 30 IREG=1,NREG2
         NBMIX2=MAX(NBMIX2,IBASE(IMAT2+IREG-1))
   30    IBASE(IMERG2+IREG-1)=IBASE(IMAT2+IREG-1)
         IF(NBMIX2.NE.NMERGE) CALL XABORT('SPHDRV: TRACKING FAILURE.')
         CALL REDGET(ITYPE,NITMA,FLOTT,TEXT12,DBLINP)
         IF(ITYPE.NE.3) CALL XABORT('SPHDRV: CHARACTER DATA EXPECTED.')
	 IF(TEXT12.EQ.':::') THEN
            CALL REDGET(ITYPE,NITMA,FLOTT,TEXT12,DBLINP)
            IF(ITYPE.NE.3) CALL XABORT('SPHDRV: DOOR TYPE EXPECTED.')
            IF(TEXT12.EQ.'MCCGT:') THEN
*              MCCG TRANSPORT-TRANSPORT EQUIVALENCE
               NSPH=4
               CNDOOR='MCCG'
	    ELSE
	       CALL XABORT('SPHDRV: MCCGT: KEY-WORD EXPECTED.')
	    ENDIF
	    JENTRY(1)=1
	    JENTRY(2)=2
	    GO TO 25
	 ENDIF
         IF(TEXT12.NE.';') CALL XABORT('SPHDRV: ; KEY-WORD EXPECTED.')
      ENDIF
      IF(ILK) CALL XABORT('SPHDRV: LEAKAGE FORBIDDEN THROUGH BOUNDARY.')
*
      IF(NSPH.EQ.1) THEN
*       READ SPH FACTORS FROM THE EDITION MACROLIB.
        CALL LCMSIX(IPEDIT,CURNAM,1)
        CALL LCMSIX(IPEDIT,'MACROLIB',1)
        JPEDIT=LCMGID(IPEDIT,'GROUP')
        DO 40 IGR=1,NGCOND
          KPEDIT=LCMGIL(JPEDIT,IGR)
          CALL LCMGET(KPEDIT,'SPH00',SPH(1,IGR,1))
          IF(NWGTH.GE.1) CALL LCMGET(KPEDIT,'SPH01',SPH(1,IGR,2))
   40   CONTINUE
        CALL LCMSIX(IPEDIT,' ',2)
        CALL LCMSIX(IPEDIT,' ',2)
      ELSE IF((NSPH.EQ.2).OR.(NSPH.EQ.3).OR.(NSPH.EQ.4)) THEN
*       GENERAL PROCEDURE FOR COMPUTING THE SPH FACTORS.
        CALL SETARA(RBASE,NBMIX2+1,ISIGMA)
        CALL SETARA(RBASE,NBMIX2+1,ISIGMS)
        CALL SETARA(RBASE,NBMIX2+1,ISIGMD)
        CALL SETARA(RBASE,NGCOND*(NWGTH+1),ICOURI)
        CALL SETARA(RBASE,NGCOND*(NWGTH+1),ICOURO)
        CALL SETARA(RBASE,NMERGE,IVOLME)
        CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),ISIGT)
        CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),ISIGW)
        CALL SETARA(RBASE,NMERGE*NGCOND*(NWGTH+1),IFLXME)
        CALL SETARA(RBASE,NMERGE*NGCOND,IDIFF)
        CALL SETARA(RBASE,NMERGE*(NGCOND**2)*(NWGTH+1),ISUNME)
        CALL SETARA(RBASE,NUN2*NGCOND,IFUNKN)
        CALL SETARA(RBASE,NUN2*NGCOND,ISUNKN)
        CALL SETARA(RBASE,NMERGE,IFACTO)
        CALL SETARA(RBASE,NGCOND,INPSYS)
        CALL SPHEQU(NBMIX2,IPTRK2,IFTRAK,IPFLUX,IPMACR,IPSYS,IPSPH,
     1  CNDOOR,NSPH,KSPH,MAXIT,MAXNBI,IBELL,IGAP,IROW,EPSPH,IPRINT,
     2  ITRANC,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,ILEAKS,NWGTH,NL,
     3  NIFISS,MATCOD,VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,NREG2,NUN2,
     4  IBASE(IMAT2),RBASE(IVOL2),IBASE(IKEY2),IBASE(IMERG2),ILK,
     5  CTITRE,SPH,RBASE(ISIGMA),RBASE(ISIGMS),RBASE(ISIGMD),
     6  RBASE(ICOURI),RBASE(ICOURO),RBASE(IVOLME),RBASE(ISIGT),
     7  RBASE(ISIGW),RBASE(IFLXME),RBASE(IDIFF),RBASE(ISUNME),
     8  RBASE(IFUNKN),RBASE(ISUNKN),RBASE(IFACTO),RBASE(INPSYS))
        CALL RLSARA(RBASE(INPSYS))
        CALL RLSARA(RBASE(IFACTO))
        CALL RLSARA(RBASE(ISUNKN))
        CALL RLSARA(RBASE(IFUNKN))
        CALL RLSARA(RBASE(ISUNME))
        CALL RLSARA(RBASE(IDIFF))
        CALL RLSARA(RBASE(IFLXME))
        CALL RLSARA(RBASE(ISIGW))
        CALL RLSARA(RBASE(ISIGT))
        CALL RLSARA(RBASE(IVOLME))
        CALL RLSARA(RBASE(ICOURO))
        CALL RLSARA(RBASE(ICOURI))
        CALL RLSARA(RBASE(ISIGMD))
        CALL RLSARA(RBASE(ISIGMS))
        CALL RLSARA(RBASE(ISIGMA))
      ELSE
        CALL XABORT('SPHDRV: UNKNOWN TYPE OF EQUIVALENCE.')
      ENDIF
      IF(LNEWDR) THEN
         IF((CNDOOR.EQ.'EXCELL').OR.(CNDOOR.EQ.'NXT')) THEN
            IER=KDRCLS(IFTRAK,2)
            IF(IER.NE.0) CALL XABORT('SPHDRV: KDRCLS FAILURE.')
         ENDIF
         CALL LCMCL(IPTRK2,2)
      ENDIF
      CALL RLSARA(IBASE(IMERG2))
      CALL RLSARA(IBASE(IKEY2))
      CALL RLSARA(IBASE(IMAT2))
      CALL RLSARA(RBASE(IVOL2))
      RETURN
  110 FORMAT (/25H SPHDRV: MACRO-GEOMETRY ',A12,17H' WILL BE TRACKED,
     > 11H BY MODULE ,A12,1H.)
      END
