*DECK SPHEQU
      SUBROUTINE SPHEQU(NBMIX2,IPTRK2,IFTRAK,IPFLUX,IPMACR,IPSYS,IPSPH,
     1 CDOOR,NSPH,KSPH,MAXIT,MAXNBI,IBELL,IGAP,IROW,EPSPH,IPRINT,
     2 ITRANC,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,ILEAKS,NWGTH,NL,NIFISS,
     3 MATCOD,VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,NREG2,NUN2,MAT2,VOL2,
     4 KEY2,MERG2,ILK,CTITRE,SPH,SIGMA,SIGMS,SIGMD,COURIN,COUROW,VOLMER,
     5 SIGT,SIGW,FLXMER,DIFF,SUNMER,FUNKNO,SUNKNO,FACTOR,NPSYS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the SPH factors for the homogenization of any geometry
* using a transport-transport or transport-diffusion equivalence
* technique. The macro-calculation can be performed using a standard
* solution door of Dragon.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NBMIX2  number of macro-mixtures. Equal to MAX(MAT2(IREG)) for
*         (IREG .le. NREG2).
* IPTRK2  pointer to the tracking of the macro-geometry (L_TRACK
*         signature).
* IFTRAK  unit number of the sequential binary tracking file.
* IPFLUX  pointer to the reference solution (L_FLUX signature).
* IPMACR  pointer to the reference macrolib (L_MACROLIB signature).
* IPSYS   pointer to the reference pij (L_PIJ signature).
* IPSPH   pointer to the LCM directory containing SPH-related options.
* CDOOR   tracking operator used to track the macro-geometry.
* NSPH    type of SPH algorithm:
*         =2 homogeneous macro-calculation (non-iterative procedure
*            or Hebert-Benoist SPH-5 procedure);
*         =3 any type of pij macro-calculation;
*         =4 any type of diffusion, SN, PN or SPN macro-calculation;
* KSPH    type of SPH factor normalization:
*         <0 asymptotic normatization with respect to mixture -KSPH;
*         =1 average flux normalization;
*         =2 Selengut normalization;
*         =3 generalized Selengut normalization (EDF type);
*         =4 Selengut normalization with surface leakage.
*         The Hebert-Benoist procedure is used if NSPH=2 and KSPH=4.
* MAXIT   maximum number of SPH iterations.
* MAXNBI  acceptable number of SPH iterations with an increase in
*         convergence error before aborting.
* EPSPH   convergence criterion for stopping the SPH iterations.
* IBELL   Bell factor acceleration flag (0:off/1:on).
* IGAP    number of reference regions in GAP.
* IROW    number of reference regions in row.
* IPRINT  print flag (equal to 0 for no print).
* ITRANC  type of transport correction.
* NGROUP  number of energy groups in the reference calculation.
* NGCOND  number of condensed groups.
* NBMIX   number of mixtures.
* NREGIO  number of regions in the reference calculation.
* NMERGE  number of merged regions (equal to 1 or to the number of
*         different flux components in the macro-calculation).
* ILEAKS  type of leakage calculation =0: no leakage; =1: homogeneous
*         leakage (Diffon); =2: isotropic streaming (Ecco);
*         =3: anisotropic streaming (Tibere).
* NWGTH   type of weighting for P1 cross section information:
*         = 0 P0; = 1 P1. P1-weighted SPH factors are computed if NWGTH
*         is greater than zero.
* NL      number of Legendre orders of the scattering cross sections.
* NIFISS  number of fissile isotopes.
* MATCOD  mixture index per region.
* VOLUME  volume of regions.
* KEYFLX  position of average fluxes in unknown vector.
* IGCOND  limit of condensed groups.
* IMERGE  index of merged regions.
* FLUXES  fluxes.
* NREG2   number of macro-regions (in the macro-calculation).
* NUN2    number of unknowns in a one-group macro-calculation.
* MAT2    mixture index per macro-region.
* VOL2    volume of macro-regions.
* KEY2    pointer to flux values in unknown vector.
* MERG2   index of merged macro-regions.
* ILK     leakage switch.
* CTITRE  title.
*
*Parameters: output
* SPH     SPH homogenization factors.
*
*Parameters: scratch
* SIGMA
* SIGMS
* SIGMD
* COURIN
* COUROW
* VOLMER
* SIGT
* SIGW
* FLXMER
* DIFF
* SUNMER
* FUNKNO
* SUNKNO
* FACTOR
* NPSYS
*
*References(s):
*  A. Hebert, 'A Consistent Technique for the Pin-by-Pin Homogenization
*  of a Pressurized Water Reactor Assembly', Nucl. Sci. Eng., 113, 227
*  (1993).
*
*  A. Hebert and P. Benoist, 'A Consistent Technique for the Global
*  Homogenization of a Pressurized Water Reactor Assembly', Nucl. Sci.
*  Eng., 109, 360 (1991).
*
*  A. Hebert and G. Mathonniere, 'Development of a Third-Generation
*  Superhomogeneisation Method for the Homogenization of a
*  Pressurized Water Reactor Assembly', Nucl. Sci. Eng., 115, 129
*  (1993).
*
*  T. Courau, M. Cometto, E. Girardi, D. Couyras and N. Schwartz,
*  'Elements of Validation of Pin-by-Pin Calculations with the Future
*  EDF Calculation Scheme Based on APOLLO2 and COCAGNE Codes',
*  Proceedings of ICAPP '08 Anaheim, CA USA, June 8-12, 2008.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NBMIX2,IPTRK2,IFTRAK,IPFLUX,IPMACR,IPSYS,IPSPH,NSPH,KSPH,
     1 MAXIT,MAXNBI,IBELL,IGAP,IROW,IPRINT,ITRANC,NGROUP,NGCOND,
     2 NBMIX,NREGIO,NMERGE,ILEAKS,NWGTH,NL,NIFISS,MATCOD(NREGIO),
     3 KEYFLX(NREGIO),IGCOND(NGCOND),IMERGE(NREGIO),NREG2,NUN2,
     4 MAT2(NREG2),KEY2(NREG2),MERG2(NREG2),NPSYS(NGCOND)
      REAL VOLUME(NREGIO),FLUXES(NREGIO,NGROUP,NWGTH+1),VOL2(NREG2),
     1 SPH(NMERGE,NGCOND,NWGTH+1),SIGMA(0:NBMIX2),SIGMS(0:NBMIX2),
     2 SIGMD(0:NBMIX2),COURIN(NGCOND,NWGTH+1),COUROW(NGCOND,NWGTH+1),
     3 VOLMER(NMERGE),SIGT(NMERGE,NGCOND,NWGTH+1),
     4 SIGW(NMERGE,NGCOND,NWGTH+1),FLXMER(NMERGE,NGCOND,NWGTH+1),
     5 DIFF(NMERGE,NGCOND),SUNMER(NMERGE,NGCOND,NGCOND,NWGTH+1),
     6 FUNKNO(NUN2,NGCOND),SUNKNO(NUN2,NGCOND),FACTOR(NMERGE)
      LOGICAL ILK
      CHARACTER CDOOR*12,CTITRE*72,TEXT5*5
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      REAL B2(4)
      INTEGER IPAR(NSTATE)
      LOGICAL LNORM,LOK,LEXAC,LDIFF
      CHARACTER SUFF(2)*2
      COMMON RBASE(1)
      DATA SUFF/'00','01'/
*----
*  CALCULATION OF THE REFERENCE MERGED/CONDENSED SET OF CROSS SECTIONS
*----
      IF(ILK) CALL XABORT('SPHEQU: LEAKAGE IS FORBIDDEN.')
      IF((KSPH.EQ.3).AND.(IROW.EQ.0)) THEN
         CALL XABORT('SPHEQU: ROW IS NOT SET WITH GENERALIZED SELENGUT'
     >   //' NORMALIZATION.')
      ELSE IF((KSPH.GE.2).AND.(IGAP.EQ.0)) THEN
         IF(IPSYS.EQ.0) THEN
	    CALL XABORT('SPHEQU: THE L_PIJ INFO IS NOT AVAILABLE AND '
     >      //'GAP IS NOT SET WITH SELENGUT NORMALIZATION.')
	 ENDIF
         JPSYS=LCMGID(IPSYS,'GROUP')
         KPSYS=LCMGIL(JPSYS,1)
         CALL LCMLEN(KPSYS,'DRAGON-WIS',IXSLEN,ITYLCM)
         IF(IXSLEN.NE.NREGIO) THEN
           CALL LCMLIB(KPSYS)
           WRITE(TEXT5,'(I5)') NREGIO
           CALL XABORT('SPHEQU: THE ALBS OPTION OF THE ASM: MODULE HAS'
     >     //' NOT BEEN ACTIVATED. NREGIO='//TEXT5)
         ENDIF
      ENDIF
*
      IF(KSPH.NE.4) THEN
        CALL LCMLEN(IPFLUX,'B2  B1HOM',ILCMLN,ITYXS)
        IF(ILEAKS.GE.1) CALL LCMGET(IPFLUX,'B2  B1HOM',B2(4))
        IF(ILEAKS.EQ.3) CALL LCMGET(IPFLUX,'B2  HETE',B2)
      ENDIF
      CALL SETARA(RBASE,NREGIO*NGCOND*NIFISS,IPRODU)
      CALL SETARA(RBASE,NBMIX*NIFISS+1,ISIGMA)
      CALL SETARA(RBASE,NBMIX,INJJ)
      CALL SETARA(RBASE,NBMIX,IIJJ)
      CALL SETARA(RBASE,NBMIX,IIPOS)
      CALL SETARA(RBASE,NBMIX*NGROUP,IXSCAT)
      CALL SETARA(RBASE,NREGIO*2,IWORK)
      CALL SETARA(RBASE,NGROUP,IDIFF)
      CALL SETARA(RBASE,NGROUP,IGAMMA)
      CALL SPHREF(IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,ILEAKS,NWGTH,
     1 B2,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,NIFISS,MATCOD,VOLUME,
     2 KEYFLX,IGCOND,IMERGE,FLUXES,VOLMER,FLXMER,SUNMER,SIGT,SIGW,
     3 COURIN,RBASE(IPRODU),RBASE(ISIGMA),RBASE(INJJ),RBASE(IIJJ),
     4 RBASE(IIPOS),RBASE(IXSCAT),RBASE(IWORK),RBASE(IDIFF),
     5 RBASE(IGAMMA))
      CALL RLSARA(RBASE(IGAMMA))
      CALL RLSARA(RBASE(IDIFF))
      CALL RLSARA(RBASE(IWORK))
      CALL RLSARA(RBASE(IXSCAT))
      CALL RLSARA(RBASE(IIPOS))
      CALL RLSARA(RBASE(IIJJ))
      CALL RLSARA(RBASE(INJJ))
      CALL RLSARA(RBASE(ISIGMA))
      CALL RLSARA(RBASE(IPRODU))
      IF(IPRINT.GT.4) THEN
        WRITE(6,930) 'VOLMER',(VOLMER(IKK),IKK=1,NMERGE)
        DO 10 INL=1,NWGTH+1
        WRITE(6,930) 'SIGT'//SUFF(INL),((SIGT(IKK,IGR,INL),IKK=1,NMERGE)
     >  ,IGR=1,NGCOND)
        WRITE(6,930) 'SIGW'//SUFF(INL),((SIGW(IKK,IGR,INL),IKK=1,NMERGE)
     >  ,IGR=1,NGCOND)
        WRITE(6,930) 'FLXMER'//SUFF(INL),((FLXMER(IKK,IGR,INL),
     >  IKK=1,NMERGE),IGR=1,NGCOND)
        WRITE(6,930) 'SUNMER'//SUFF(INL),(((SUNMER(IKK,IGR,JGR,INL),
     >  IKK=1,NMERGE),IGR=1,NGCOND),JGR=1,NGCOND)
   10   CONTINUE
      ENDIF
      DO 20 INL=1,NWGTH+1
      IF((IPRINT.GT.3).AND.(IPSYS.NE.0)) THEN
         WRITE(6,900) SUFF(INL),(COURIN(IGR,INL),IGR=1,NGCOND)
         WRITE(6,'(/)')
      ENDIF
      DO 20 IGR=1,NGCOND
      DO 20 IKK=1,NMERGE
   20 SUNMER(IKK,IGR,IGR,INL)=SUNMER(IKK,IGR,IGR,INL)-SIGW(IKK,IGR,INL)
      LDIFF=.FALSE.
      IF((NSPH.EQ.4).AND.((CDOOR.EQ.'BIVAC').OR.(CDOOR.EQ.'TRIVAC')))
     > THEN
*        TRANSPORT-DIFFUSION EQUIVALENCE.
         IF(IPTRK2.EQ.0) CALL XABORT('SPHEQU: MACRO-TRACKING NOT DEFIN'
     >   //'ED(1).')
         CALL LCMGET(IPTRK2,'STATE-VECTOR',IPAR)
	 IF(CDOOR.EQ.'BIVAC') LDIFF=(IPAR(14).EQ.0)
	 IF(CDOOR.EQ.'TRIVAC') LDIFF=(IPAR(30).EQ.0)
	 IF(.NOT.LDIFF) GO TO 100
         IF(ILEAKS.EQ.0) CALL XABORT('SPHEQU: UNABLE TO COMPUTE DIFFUS'
     >   //'ION COEFFICIENTS.')
         IF(NWGTH.NE.0) CALL XABORT('SPHEQU: P0W WEIGHTING EXPECTED.')
*
*        COMPUTE THE DIFFUSION COEFFICIENTS.
         CALL XDRSET(DIFF,NGCOND*NMERGE,0.0)
         IF(ILEAKS.EQ.1) THEN
           CALL SETARA(RBASE,NGROUP,IDIFFU)
           CALL LCMGET(IPFLUX,'DIFFB1HOM',RBASE(IDIFFU))
           IFIN=0
           DO 50 IGR=1,NGCOND
           IDEB=IFIN+1
           IFIN=IGCOND(IGR)
           DO 40 IGR2=IDEB,IFIN
           DO 40 IREG=1,NREGIO
           IKK=IMERGE(IREG)
           IF(IKK.EQ.0) GO TO 40
           DIFF(IKK,IGR)=DIFF(IKK,IGR)+RBASE(IDIFFU+IGR2-1)
     >     *FLUXES(IREG,IGR2,1)*VOLUME(IREG)
   40      CONTINUE
           DO 50 IKK=1,NMERGE
   50      DIFF(IKK,IGR)=DIFF(IKK,IGR)/(FLXMER(IKK,IGR,1)*VOLMER(IKK))
           CALL RLSARA(RBASE(IDIFFU))
         ELSE IF(ILEAKS.EQ.2) THEN
           CALL LCMLEN(IPFLUX,'FLUX',ILCMLN,ITYLCM)
           IF(ILCMLN.EQ.0) CALL XABORT('SPHEQU: MISSING FLUX INFO.')
           JPFLUX=LCMGID(IPFLUX,'FLUX')
           CALL LCMLEL(JPFLUX,1,ILCMLN,ITYLCM)
           CALL SETARA(RBASE,ILCMLN,IWORK)
           IFIN=0
           DO 70 IGR=1,NGCOND
           IDEB=IFIN+1
           IFIN=IGCOND(IGR)
           DO 60 IGR2=IDEB,IFIN
           CALL LCMGDL(JPFLUX,IGR2,RBASE(IWORK))
           DO 60 IREG=1,NREGIO
           IKK=IMERGE(IREG)
           IF(IKK.NE.0) THEN
              CURH=RBASE(IWORK+KEYFLX(IREG)+ILCMLN/2-1)
              DIFF(IKK,IGR)=DIFF(IKK,IGR)+CURH*VOLUME(IREG)
           ENDIF
   60      CONTINUE
           DO 70 IKK=1,NMERGE
   70      DIFF(IKK,IGR)=DIFF(IKK,IGR)/(FLXMER(IKK,IGR,1)*VOLMER(IKK))
           CALL RLSARA(RBASE(IWORK))
         ELSE IF(ILEAKS.EQ.3) THEN
           CALL LCMGET(IPFLUX,'B2  HETE',B2)
           B2ALL=B2(1)+B2(2)
           IF(B2ALL.EQ.0.0) THEN
             B2(1)=1.0/2.0
             B2(2)=B2(1)
           ELSE
             B2(1)=B2(1)/B2ALL
             B2(2)=B2(2)/B2ALL
           ENDIF
           CALL LCMLEN(IPFLUX,'FLUX',ILCMLN,ITYLCM)
           IF(ILCMLN.EQ.0) CALL XABORT('SPHEQU: MISSING FLUX INFO.')
           JPFLUX=LCMGID(IPFLUX,'FLUX')
           CALL LCMLEL(JPFLUX,1,ILCMLN,ITYLCM)
           NUN4=ILCMLN/4
           CALL SETARA(RBASE,ILCMLN,IWORK)
           IFIN=0
           DO 90 IGR=1,NGCOND
           IDEB=IFIN+1
           IFIN=IGCOND(IGR)
           DO 80 IGR2=IDEB,IFIN
           CALL LCMGDL(JPFLUX,IGR2,RBASE(IWORK))
           DO 80 IREG=1,NREGIO
           IKK=IMERGE(IREG)
           IF(IKK.NE.0) THEN
              CURH=B2(1)*RBASE(IWORK+KEYFLX(IREG)+NUN4-1)+
     >             B2(2)*RBASE(IWORK+KEYFLX(IREG)+2*NUN4-1)
              DIFF(IKK,IGR)=DIFF(IKK,IGR)+CURH*VOLUME(IREG)
           ENDIF
   80      CONTINUE
           DO 90 IKK=1,NMERGE
   90      DIFF(IKK,IGR)=DIFF(IKK,IGR)/(FLXMER(IKK,IGR,1)*VOLMER(IKK))
           CALL RLSARA(RBASE(IWORK))
         ENDIF
         IF(IPRINT.GT.4) WRITE(6,930) 'DIFF',((DIFF(IKK,IGR),
     >   IKK=1,NMERGE),IGR=1,NGCOND)
      ENDIF
*----
*  RECOVER THE AVERAGED GAP AND ROW FLUXES FOR EDF-TYPE NORMALIZATION.
*----
  100 IF(IGAP.GT.0) THEN
         CALL SPHROW(IPSPH,'GAP',NWGTH,NGROUP,NGCOND,NREGIO,VOLUME,
     1   IGCOND,FLUXES,IPRINT,COURIN)
      ENDIF
      IF(IROW.GT.0) THEN
         CALL SPHROW(IPSPH,'ROW',NWGTH,NGROUP,NGCOND,NREGIO,VOLUME,
     1   IGCOND,FLUXES,IPRINT,COUROW)
      ENDIF
*----
*  ITERATIVE STRATEGY USED TO COMPUTE THE SPH FACTORS
*----
      IPRIN2=MAX(0,IPRINT-5)
      CALL LCMOP(IPSYS2,'SPH$SYS',0,1,0)
      JPSYS2=LCMLID(IPSYS2,'GROUP',NGCOND)
      DO 330 INL=NWGTH+1,1,-1
      IF(KSPH.LT.0) THEN
*        ASYMTTOTIC NORMALIZATION WITH RESPECT TO MIXTURE -KSPH.
         DO 120 IGR=1,NGCOND
         VOLTOT=0.0
         FLXTOT=0.0
         DO 110 IKK=1,NMERGE
         IF(IKK.EQ.-KSPH) THEN
            VOLTOT=VOLTOT+VOLMER(IKK)
            FLXTOT=FLXTOT+FLXMER(IKK,IGR,INL)*VOLMER(IKK)
         ENDIF
  110    CONTINUE
         IF(VOLTOT.EQ.0.0) CALL XABORT('SPHEQU: ASYMPTOTIC NORMALIZATI'
     >   //'ON FAILURE.')
         COURIN(IGR,INL)=FLXTOT/VOLTOT
         CALL XDRSET(SPH(1,IGR,INL),NMERGE,1.0)
  120    CONTINUE
         IF(IPRINT.GT.3) THEN
            WRITE(6,910) SUFF(INL),(COURIN(IGR,INL),IGR=1,NGCOND)
            WRITE(6,'(/)')
         ENDIF
      ELSE IF(KSPH.EQ.1) THEN
*        AVERAGE FLUX NORMALIZATION.
         DO 135 IGR=1,NGCOND
         VOLTOT=0.0
         FLXTOT=0.0
         DO 130 IKK=1,NMERGE
         VOLTOT=VOLTOT+VOLMER(IKK)
  130    FLXTOT=FLXTOT+FLXMER(IKK,IGR,INL)*VOLMER(IKK)
         COURIN(IGR,INL)=FLXTOT/VOLTOT
         CALL XDRSET(SPH(1,IGR,INL),NMERGE,1.0)
  135    CONTINUE
         IF(IPRINT.GT.3) THEN
            WRITE(6,910) SUFF(INL),(COURIN(IGR,INL),IGR=1,NGCOND)
            WRITE(6,'(/)')
         ENDIF
      ELSE IF((KSPH.EQ.2).OR.(KSPH.EQ.4)) THEN
*        SELENGUT NORMALIZATION.
         DO 145 IGR=1,NGCOND
         VOLTOT=0.0
         FLXTOT=0.0
         DO 140 IKK=1,NMERGE
         VOLTOT=VOLTOT+VOLMER(IKK)
  140    FLXTOT=FLXTOT+FLXMER(IKK,IGR,INL)*VOLMER(IKK)
         FLXTOT=FLXTOT/VOLTOT
         DO 145 IKK=1,NMERGE
  145    SPH(IKK,IGR,INL)=FLXTOT/COURIN(IGR,INL)
      ELSE IF(KSPH.EQ.3) THEN
*        GENERALIZED SELENGUT NORMALIZATION (EDF-TYPE).
         DO 148 IGR=1,NGCOND
         VOLTOT=0.0
         FLXTOT=0.0
         DO 146 IKK=1,NMERGE
         VOLTOT=VOLTOT+VOLMER(IKK)
  146    FLXTOT=FLXTOT+FLXMER(IKK,IGR,INL)*VOLMER(IKK)
         FLXTOT=FLXTOT/VOLTOT
         DO 147 IKK=1,NMERGE
  147    SPH(IKK,IGR,INL)=COUROW(IGR,INL)/COURIN(IGR,INL)
  148    COURIN(IGR,INL)=COURIN(IGR,INL)*FLXTOT/COUROW(IGR,INL)
      ENDIF
      IF((NSPH.EQ.2).AND.(KSPH.NE.4)) GO TO 340
*----
*  SPH ITERATIONS.
*----
      DO 150 IGR=1,NGCOND
      CALL XDRSET(FUNKNO(1,IGR),NUN2,0.0)
      DO 150 IREG=1,NREG2
  150 FUNKNO(KEY2(IREG),IGR)=FLXMER(MERG2(IREG),IGR,INL)
      OLDERR=1.0
      NBIERR=0
      ITER=0
  160 ITER=ITER+1
      IF(ITER.GE.MAXIT) THEN
        WRITE(6,'(/46H SPHEQU: MAX. NUMBER OF ITERATIONS IS REACHED.)')
        GO TO 320
      ENDIF
      ERROR=0.0
      ERR2=0.0
      DO 170 IREG=1,NREG2
      IF(MAT2(IREG).GT.NBMIX2) THEN
         CALL XABORT('SPHEQU: INVALID MACRO-MIXTURE INDEX.')
      ENDIF
  170 CONTINUE
*----
*  SET MACROSCOPIC CROSS SECTIONS IN THE IPSYS2 LCM OBJECT.
*----
      DO 210 IGR=1,NGCOND
      CALL XDRSET(SIGMS(0),NBMIX2+1,0.0)
      CALL XDRSET(SIGMA(0),NBMIX2+1,0.0)
      CALL XDRSET(SIGMD(0),NBMIX2+1,0.0)
      DO 180 IREG=1,NREG2
      IMERG=MERG2(IREG)
      IMAT=MAT2(IREG)
      IF(LDIFF) SIGMD(IMAT)=DIFF(IMERG,IGR)*SPH(IMERG,IGR,INL)
      IF((INL.EQ.1).AND.(NWGTH.EQ.1)) THEN
*        PERFORM A LEAKAGE CORRECTION.
         DELTA=SIGT(IMERG,IGR,1)*SPH(IMERG,IGR,1)-SIGT(IMERG,IGR,2)*
     >   SPH(IMERG,IGR,2)
         SIGMA(IMAT)=SIGW(IMERG,IGR,1)*SPH(IMERG,IGR,1)-DELTA
         SIGMS(IMAT)=SIGT(IMERG,IGR,1)*SPH(IMERG,IGR,1)-DELTA
      ELSE
         SIGMA(IMAT)=SIGW(IMERG,IGR,INL)*SPH(IMERG,IGR,INL)
         SIGMS(IMAT)=SIGT(IMERG,IGR,INL)*SPH(IMERG,IGR,INL)
      ENDIF
  180 CONTINUE
      NPSYS(IGR)=IGR
      KPSYS2=LCMDIL(JPSYS2,IGR)
      CALL LCMPUT(KPSYS2,'DRAGON-TXSC',NBMIX2+1,2,SIGMS(0))
      CALL LCMPUT(KPSYS2,'DRAGON-S0XSC',NBMIX2+1,2,SIGMA(0))
      IF(LDIFF) THEN
         SIGMD(0)=1.0E10
         CALL LCMPUT(KPSYS2,'DRAGON-DIFF',NBMIX2+1,2,SIGMD(0))
      ENDIF
*----
*  TRANSPORT-PIJ EQUIVALENCE WITH BELL FACTOR ACCELERATION.
*----
      IF(IBELL.EQ.1) THEN
         LOK=(NSPH.EQ.3).OR.((NSPH.EQ.2).AND.(KSPH.EQ.4))
         LOK=LOK.AND.(NMERGE.EQ.NREG2)
	 IF(.NOT.LOK) CALL XABORT('SPHEQU: THE BELL FACTOR ACCELERATIO'
     >   //'N TECHNIQUE IS NOT AVAILABLE.')
         IF(NUN2.NE.NREG2) CALL XABORT('SPHEQU: INVALID NUN2.')
         DO 200 IREG=1,NREG2
         IF(IREG.NE.KEY2(IREG)) CALL XABORT('SPHEQU: INVALID KEY2.')
  200    CONTINUE
         CALL SPHBEL(ITER,NGCOND,IGR,NMERGE,FLXMER(1,1,INL),
     1   SUNMER(1,1,IGR,INL),NREG2,NBMIX2,MAT2,VOL2,MERG2,
     2   FUNKNO,SIGMA,SIGMS,SPH(1,IGR,INL),SUNKNO(1,IGR))
         CALL XDRSET(SIGMA(0),NBMIX2+1,0.0)
         CALL LCMPUT(KPSYS2,'DRAGON-S0XSC',NBMIX2+1,2,SIGMA(0))
      ENDIF
  210 CONTINUE
*----
*  ASSEMBLY OF PIJ OR SYSTEM MATRICES AT ITERATION ITER.
*----
      IF(IPTRK2.EQ.0) CALL XABORT('SPHEQU: MACRO-TRACKING NOT DEFINED.')
      NANI=1
      KNORM=1
      IPHASE=2
      IF(NSPH.EQ.4) IPHASE=1
      IF(IPHASE.EQ.2) THEN
         IPIJK=1
         LNORM=.FALSE.
         CALL DOORPV(CDOOR,JPSYS2,NPSYS,IPTRK2,IFTRAK,IPRIN2,NGCOND,
     >   NREG2,NBMIX2,NANI,MAT2,VOL2,KNORM,IPIJK,ILK,LNORM,CTITRE)
      ELSE
         CALL DOORAV(CDOOR,JPSYS2,NPSYS,IPTRK2,IFTRAK,IPRIN2,NGCOND,
     >   NREG2,NBMIX2,NANI,MAT2,VOL2,KNORM,ILK,CTITRE)
      ENDIF
*----
*  MACRO-FLUX CALCULATION AT ITERATION ITER.
*----
      IF(IBELL.EQ.1) THEN
*        TRANSPORT-PIJ EQUIVALENCE WITH BELL FACTOR ACCELERATION.
         CALL SETARA(RBASE,NREG2*NREG2,IPIJ)
         CALL SETARA(RBASE,NBMIX2+1,ISIGMA)
         CALL SETARA(RBASE,2*(NREG2+1)*(NREG2+1),IWORK2)
         CALL SETARA(RBASE,2*NREG2*(NREG2+2),IWORK3)
         CALL SPHTRA(JPSYS2,NPSYS,KSPH,NREG2,NGCOND,NBMIX2,MAT2,
     1   VOL2,COURIN(1,INL),SUNKNO,FUNKNO,RBASE(IPIJ),RBASE(ISIGMA),
     2   RBASE(IWORK2),RBASE(IWORK3))
         CALL RLSARA(RBASE(IWORK3))
         CALL RLSARA(RBASE(IWORK2))
         CALL RLSARA(RBASE(ISIGMA))
         CALL RLSARA(RBASE(IPIJ))
	 GO TO 280
      ELSE IF((NSPH.EQ.4).AND.(CDOOR.EQ.'BIVAC')) THEN
         CALL SPHBIV(IPTRK2,JPSYS2,NPSYS,NREG2,NUN2,NMERGE,NGCOND,
     1   NBMIX2,MAT2,VOL2,KEY2,MERG2,SUNMER(1,1,1,INL),FLXMER(1,1,INL),
     2   ITER,FUNKNO,SUNKNO)
      ELSE IF((NSPH.EQ.4).AND.(CDOOR.EQ.'TRIVAC')) THEN
         CALL SPHTRI(IPTRK2,JPSYS2,NPSYS,NREG2,NUN2,NMERGE,NGCOND,
     1   NBMIX2,MAT2,VOL2,KEY2,MERG2,SUNMER(1,1,1,INL),FLXMER(1,1,INL),
     2   ITER,FUNKNO,SUNKNO)
      ELSE IF((NSPH.EQ.4).AND.(CDOOR.EQ.'SN')) THEN
*        TRANSPORT-SN EQUIVALENCE.
         CALL SPHSN(IPTRK2,NPSYS,NREG2,NUN2,NMERGE,NGCOND,NBMIX2,MAT2,
     1   VOL2,KEY2,MERG2,SUNMER(1,1,1,INL),FLXMER(1,1,INL),ITER,FUNKNO,
     2   SUNKNO)
      ELSE
*        TRANSPORT-PIJ EQUIVALENCE.
         CALL SPHPIJ(IPTRK2,NPSYS,NREG2,NUN2,NMERGE,NGCOND,NBMIX2,
     1   MAT2,VOL2,KEY2,MERG2,SUNMER(1,1,1,INL),FLXMER(1,1,INL),ITER,
     2   FUNKNO,SUNKNO)
      ENDIF
*----
*  COMPUTE THE MACRO-FLUX USING THE VECTORIAL DOOR.
*----
      IDIR=0
      NLF=1
      LEXAC=.FALSE.
      CALL DOORFV(CDOOR,JPSYS2,NPSYS,IPTRK2,IFTRAK,IPRIN2,NGCOND,NBMIX2,
     1 NLF,IDIR,NREG2,NUN2,IPHASE,LEXAC,MAT2,VOL2,KEY2,CTITRE,SUNKNO,
     2 FUNKNO)
*----
*  MACRO-FLUX NORMALIZATION.
*----
      IF(KSPH.LT.0) THEN
*        ASYMTTOTIC NORMALIZATION WITH RESPECT TO MIXTURE -KSPH.
         IF(-KSPH.GT.NMERGE) CALL XABORT('SPHEQU: INVALID ASYMPTOTIC M'
     >   //'IXTURE SET.')
         DO 240 IGR=1,NGCOND
	 IF(NPSYS(IGR).EQ.0) GO TO 240
         VOLTOT=0.0
         FLXTOT=0.0
         DO 220 IREG=1,NREG2
         IF(MERG2(IREG).EQ.-KSPH) THEN
            VOLTOT=VOLTOT+VOL2(IREG)
            FLXTOT=FLXTOT+FUNKNO(KEY2(IREG),IGR)*VOL2(IREG)
         ENDIF
  220    CONTINUE
         IF(VOLTOT.GT.0) THEN
            FLXTOT=FLXTOT/VOLTOT
            FFF=COURIN(IGR,INL)/FLXTOT
         ELSE
            FFF=1.0
         ENDIF
         IF(IPRIN2.GT.0) WRITE(6,960) IGR,FFF
         DO 230 IUN=1,NUN2
  230    FUNKNO(IUN,IGR)=FUNKNO(IUN,IGR)*FFF
  240    CONTINUE
      ELSE
*        AVERAGE OR SELENGUT FLUX NORMALIZATION.
         DO 270 IGR=1,NGCOND
	 IF(NPSYS(IGR).EQ.0) GO TO 270
         VOLTOT=0.0
         FLXTOT=0.0
         DO 250 IREG=1,NREG2
         VOLTOT=VOLTOT+VOL2(IREG)
  250    FLXTOT=FLXTOT+FUNKNO(KEY2(IREG),IGR)*VOL2(IREG)
         FLXTOT=FLXTOT/VOLTOT
         IF(IPRIN2.GT.0) WRITE(6,960) IGR,COURIN(IGR,INL)/FLXTOT
         DO 260 IUN=1,NUN2
  260    FUNKNO(IUN,IGR)=FUNKNO(IUN,IGR)*COURIN(IGR,INL)/FLXTOT
  270    CONTINUE
      ENDIF
*----
*  COMPUTE THE IMPROVED SPH FACTORS.
*----
  280 DO 310 IGR=1,NGCOND
      IF(NPSYS(IGR).EQ.0) GO TO 310
      CALL XDRSET(VOLMER,NMERGE,0.0)
      CALL XDRSET(FACTOR,NMERGE,0.0)
      DO 290 IREG=1,NREG2
      IKK=MERG2(IREG)
      VOLMER(IKK)=VOLMER(IKK)+VOL2(IREG)
  290 FACTOR(IKK)=FACTOR(IKK)+FUNKNO(KEY2(IREG),IGR)*VOL2(IREG)
      DO 300 IKK=1,NMERGE
      IF(VOLMER(IKK).EQ.0.0) GO TO 300
      FACTOR(IKK)=FACTOR(IKK)/VOLMER(IKK)
      SPHNEW=FLXMER(IKK,IGR,INL)/FACTOR(IKK)
      IF(SPHNEW.LT.0.0) THEN
         WRITE(6,980) IGR,IKK
         SPHNEW=1.0
      ENDIF
      ERRT=ABS((SPHNEW-SPH(IKK,IGR,INL))/SPHNEW)
      ERR2=ERR2+ERRT*ERRT
      ERROR=MAX(ERROR,ERRT)
      SPH(IKK,IGR,INL)=SPHNEW
  300 CONTINUE
      IF(IPRINT.GT.4) THEN
         WRITE(6,930) 'SPH'//SUFF(INL),(SPH(IKK,IGR,INL),IKK=1,NMERGE)
         WRITE(6,930) 'FUNKNO'//SUFF(INL),(FUNKNO(IUNK,IGR),IUNK=1,NUN2)
      ENDIF
  310 CONTINUE
      ERR2=SQRT(ERR2/(NMERGE*NGCOND))
      IF(IPRINT.GT.2) THEN
         WRITE(6,935) SUFF(INL),ITER,ERROR,ERR2
         IF(ERROR.GE.EPSPH) WRITE(6,940) ((IKK,IGR,SPH(IKK,IGR,INL),
     >   IKK=1,NMERGE),IGR=1,NGCOND)
      ENDIF
      IF(ERR2.LT.EPSPH) GO TO 320
      IF((ITER.GT.1).AND.(ERR2.GT.OLDERR)) THEN
         WRITE(6,970) ITER
         NBIERR=NBIERR+1
         IF(NBIERR.GE.MAXNBI) THEN
            WRITE(6,980) ITER
            GO TO 320
         ENDIF
      ENDIF
      OLDERR=ERR2
      GO TO 160
  320 WRITE(6,950) SUFF(INL),ITER
  330 CONTINUE
      CALL LCMCL(IPSYS2,2)
  340 IF(IPRINT.GT.2) THEN
         DO 350 INL=1,NWGTH+1
         WRITE(6,'(/29H SPHEQU: SPH FACTORS (ORDER P,A,2H):)') SUFF(INL)
         WRITE(6,940) ((IKK,IGR,SPH(IKK,IGR,INL),IKK=1,NMERGE),IGR=1,
     >   NGCOND)
  350    CONTINUE
      ENDIF
      RETURN
*
  900 FORMAT(/10H SPHEQU: P,A2,36H IN-CURRENTS (4J-/S) PER MACRO-GROUP,
     > 5HS ARE/(1X,1P,10E13.5))
  910 FORMAT(/18H SPHEQU: AVERAGE P,A2,28H FLUXES PER MACRO-GROUPS ARE/
     > (1X,1P,10E13.5))
  930 FORMAT(/26H SPHEQU: VALUES OF VECTOR ,A,4H ARE/(1X,1P,10E13.5))
  935 FORMAT(/15H SPHEQU: ORDER=,A2,4X,5HITER=,I3,4X,6HERROR=,1P,E10.3,
     > 1X,6HERR 2=,E10.3)
  940 FORMAT(4X,4HSPH(,I3,1H,,I3,2H)=,F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,
     > F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,
     > F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,F9.5)
  950 FORMAT(/27H SPHEQU: CONVERGENCE OF SPH,A2,13H ALGORITHM IN,I5,
     > 12H ITERATIONS.)
  960 FORMAT(/43H SPHEQU: FLUX NORMALIZATION FACTOR IN GROUP,I4,1H=,1P,
     1 E13.5)
  970 FORMAT(1X,'Warning: oscillations in SPH at iteration ',i10)
  980 FORMAT(1X,'Warning: maximum of 3 error oscillations ',
     >'in SPH convergence reached at iteration ',i10)
  990 FORMAT(1X,'Warning: SPH factor in group ',i10,' region ',i10,
     >       ' negative'/1X,'Reset SPH factor to 1.0')
      END
