*DECK SPHGET
      SUBROUTINE SPHGET(IPEDIT,IPSPH,NREG,NBMIX,MIX,CDOOR,CNDOOR,
     1 MACGEO,JSPH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read SPH option parameters.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* IPEDIT  pointer to the LCM edition object.
* IPSPH   pointer to the sub-sirectory containing SPF data.
* NREG    number of regions in the reference geometry.
* NBMIX   number of material mixtures in the reference geometry.
* MIX     mixture index associated to each region.
* CDOOR   type of door for reference calculation.
* CNDOOR  type of door for SPH macro-calculation.
* MACGEO  name of the SPH macro-geometry.
* JSPH    type of SPH framework use:
*         =0 do not use the SPH framework;
*         =1 use the SPH framework / no user-defined macro-geometry nor
*            macro-tracking;
*         =2 use the SPH framework with a user-defined macro-tracking;
*         =3 use the SPH framework with a user-defined macro-geometry.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER       IPEDIT,IPSPH,NREG,NBMIX,MIX(NREG),JSPH
      CHARACTER*12  CDOOR*12,CNDOOR*12,MACGEO*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER    (NSTATE=40,IOUT=6)
      INTEGER       ISTATE(NSTATE),KCHAR(3)
      CHARACTER     TEXT12*12
      LOGICAL       LARM
      REAL          REALIR
      DOUBLE PRECISION DBLLIR
      COMMON        IBASE(1)
*----
*  INITIALIZE OR RECOVER EXISTING SPH DATA
*----
      CALL LCMLEN(IPEDIT,'SPH',ILEN,ITYLCM)
      IF(ILEN.NE.0) THEN
	 IPSPH=LCMGID(IPEDIT,'SPH')
         CALL LCMGET(IPSPH,'STATE-VECTOR',ISTATE)
	 NSPH=ISTATE(1)
	 KSPH=ISTATE(2)
	 MAXIT=ISTATE(3)
	 MAXNBI=ISTATE(4)
	 IBELL=ISTATE(5)
	 IGAP=ISTATE(6)
	 IROW=ISTATE(7)
	 IF(NSPH.GE.2) THEN
	    CALL LCMGET(IPSPH,'SPH$TRK',KCHAR)
            WRITE(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
	    CALL LCMGET(IPSPH,'SPH-EPSILON',EPSPH)
	 ENDIF
         LARM=(NSPH.EQ.4)
      ELSE
         NSPH=0
         KSPH=0
	 MAXIT=200
	 MAXNBI=10
	 IBELL=0
	 IGAP=0
	 IROW=0
	 EPSPH=1.0E-4
         CNDOOR=' '
         LARM=.FALSE.
      ENDIF
*----
*  SPH DIRECTIVE ANALYSIS
*----
      MACGEO=' '
      KSPH=1
      JSPH=1
   10 CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
      IF(ITYPLU.NE.3) CALL XABORT('SPHGET: READ ERROR - CHARACTER VA'
     > //'RIABLE EXPECTED')
   20 IF(TEXT12.EQ.';') THEN
         GO TO 100
      ELSE IF(TEXT12.EQ.'ASYM') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT('SPHGET: READ ERROR - INTEGER V'
     >   //'ARIABLE EXPECTED')
         IF(INTLIR.LE.0) CALL XABORT('SPHGET: INVALID ASYMPTOTIC MIX'
     >   //'TURE SET')
         KSPH=-INTLIR
      ELSE IF(TEXT12.EQ.'STD') THEN
         KSPH=1
      ELSE IF(TEXT12.EQ.'SELE') THEN
         KSPH=2
      ELSE IF(TEXT12.EQ.'EDF') THEN
         KSPH=3
      ELSE IF(TEXT12.EQ.'MGEO') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,MACGEO,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT('SPHGET: READ ERROR - CHARACTER'
     >   //' VARIABLE EXPECTED')
         JSPH=3
      ELSE IF(TEXT12.EQ.'MTRK') THEN
         MACGEO=' '
         JSPH=2
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'OFF') THEN
*        NO SPH CORRECTION PERFORMED
         JSPH=0
         NSPH=0
         KSPH=0
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'SPRD') THEN
*        THE SPH FACTORS ARE READ FROM LCM
         JSPH=1
         NSPH=1
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'HOMO') THEN
*        HOMOGENEOUS MACRO CALCULATION (NO ITERATIONS ARE PERFORMED)
         JSPH=1
         NSPH=2
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'ALBS') THEN
         JSPH=1
         NSPH=2
         KSPH=4
         CNDOOR=' '
      ELSE IF(TEXT12.EQ.'ITER') THEN
*        SPH ITERATION MAIN CONTROL PARAMETERS
   30    CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.EQ.1) THEN
	    MAXIT=INTLIR
         ELSE IF(ITYPLU.EQ.2) THEN
	    EPSPH=REALIR
         ELSE IF(ITYPLU.EQ.3) THEN
	    GO TO 20
	 ENDIF
	 GO TO 30
      ELSE IF(TEXT12.EQ.'MAXNB') THEN
*        SPH ITERATION AUXILIARY CONTROL PARAMETERS
         CALL REDGET(ITYPLU,MAXNBI,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.1) CALL XABORT('SPHGET: READ ERROR - INTEGER'
     >   //' VARIABLE EXPECTED')
      ELSE IF(TEXT12.EQ.'NOBELL') THEN
         IBELL=0
      ELSE IF(TEXT12.EQ.'BELL') THEN
         IBELL=1
      ELSE IF(TEXT12.EQ.'ARM') THEN
         LARM=.TRUE.
      ELSE IF(TEXT12.EQ.'GAP') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT('SPHGET: READ ERROR - CHARACTER'
     >   //' VARIABLE EXPECTED')
         CALL SETARA(IBASE,NREG,IOFGAP)
	 CALL XDISET(IBASE(IOFGAP),NREG,0)
	 IGAP=0
         IF(TEXT12.EQ.'REGI') THEN
   40       CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
            IF(ITYPLU.EQ.1) THEN
	       IGAP=IGAP+1
	       IF(IGAP.GT.NREG) THEN
	          CALL XABORT('SPHGET: GAP OVERFLOW(1)')
	       ELSE IF(INTLIR.GT.NREG) THEN
	          CALL XABORT('SPHGET: GAP OVERFLOW(2)')
	       ELSE IF(IBASE(IOFGAP+IGAP-1).NE.0) THEN
		  CALL XABORT('SPHGET: REGI ALREADY DEFINED')
	       ENDIF
	       IBASE(IOFGAP+IGAP-1)=INTLIR
            ELSE IF((ITYPLU.EQ.3).AND.(TEXT12.EQ.'ENDR')) THEN
	       GO TO 10
            ELSE
	       CALL XABORT('SPHGET: INTEGER OR ENDR KEYWORD EXPECTED')
	    ENDIF
	    GO TO 40
	 ELSE IF(TEXT12.EQ.'MIX') THEN
   50       CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
            IF(ITYPLU.EQ.1) THEN
	       DO 60 IREG=1,NREG
	       IF(MIX(IREG).EQ.INTLIR) THEN
	          IGAP=IGAP+1
	          IF(IGAP.GT.NREG) THEN
		     CALL XABORT('SPHGET: GAP OVERFLOW(3)')
	          ELSE IF(INTLIR.GT.NBMIX) THEN
	             CALL XABORT('SPHGET: GAP OVERFLOW(4)')
		  ELSE IF(IBASE(IOFGAP+IGAP-1).NE.0) THEN
		     CALL XABORT('SPHGET: MIX ALREADY DEFINED')
		  ENDIF
                  IBASE(IOFGAP+IGAP-1)=IREG
	       ENDIF
   60          CONTINUE
            ELSE IF((ITYPLU.EQ.3).AND.(TEXT12.EQ.'ENDM')) THEN
	       GO TO 10
            ELSE
	       CALL XABORT('SPHGET: INTEGER OR ENDM KEYWORD EXPECTED')
	    ENDIF
	    GO TO 50
	 ELSE
	    CALL XABORT('SPHGET: REGI OR MIX KEYWORD EXPECTED(1)')
	 ENDIF
      ELSE IF(TEXT12.EQ.'ROW') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT('SPHGET: READ ERROR - CHARACTER'
     >   //' VARIABLE EXPECTED')
         CALL SETARA(IBASE,NREG,IIROW)
	 CALL XDISET(IBASE(IIROW),NREG,0)
	 IROW=0
         IF(TEXT12.EQ.'REGI') THEN
   70       CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
            IF(ITYPLU.EQ.1) THEN
	       IROW=IROW+1
	       IF(IROW.GT.NREG) THEN
	          CALL XABORT('SPHGET: ROW OVERFLOW(1)')
	       ELSE IF(INTLIR.GT.NREG) THEN
	          CALL XABORT('SPHGET: ROW OVERFLOW(2)')
	       ELSE IF(IBASE(IIROW+IROW-1).NE.0) THEN
		  CALL XABORT('SPHGET: REGI ALREADY DEFINED')
	       ENDIF
	       IBASE(IIROW+IROW-1)=INTLIR
            ELSE IF((ITYPLU.EQ.3).AND.(TEXT12.EQ.'ENDR')) THEN
	       GO TO 10
            ELSE
	       CALL XABORT('SPHGET: INTEGER OR ENDR KEYWORD EXPECTED')
	    ENDIF
	    GO TO 70
	 ELSE IF(TEXT12.EQ.'MIX') THEN
   80       CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
            IF(ITYPLU.EQ.1) THEN
	       DO 90 IREG=1,NREG
	       IF(MIX(IREG).EQ.INTLIR) THEN
	          IROW=IROW+1
	          IF(IROW.GT.NREG) THEN
		     CALL XABORT('SPHGET: ROW OVERFLOW(3)')
	          ELSE IF(INTLIR.GT.NBMIX) THEN
	             CALL XABORT('SPHGET: ROW OVERFLOW(4)')
		  ELSE IF(IBASE(IIROW+IROW-1).NE.0) THEN
		     CALL XABORT('SPHGET: MIX ALREADY DEFINED')
		  ENDIF
                  IBASE(IIROW+IROW-1)=IREG
	       ENDIF
   90          CONTINUE
            ELSE IF((ITYPLU.EQ.3).AND.(TEXT12.EQ.'ENDM')) THEN
	       GO TO 10
            ELSE
	       CALL XABORT('SPHGET: INTEGER OR ENDM KEYWORD EXPECTED')
	    ENDIF
	    GO TO 80
	 ELSE
	    CALL XABORT('SPHGET: REGI OR MIX KEYWORD EXPECTED(2)')
	 ENDIF
      ELSE IF(TEXT12.EQ.':::') THEN
         CALL REDGET(ITYPLU,INTLIR,REALIR,TEXT12,DBLLIR)
         IF(ITYPLU.NE.3) CALL XABORT('SPHGET: READ ERROR - CHARACTER'
     >   //' VARIABLE EXPECTED')
         IF(TEXT12.EQ.'SYBILT:') THEN
*           SYBIL TRANSPORT-TRANSPORT EQUIVALENCE
            NSPH=3
            IF(LARM) NSPH=4
            CNDOOR='SYBIL'
         ELSE IF(TEXT12.EQ.'NXT:') THEN
*           NXT TRANSPORT-TRANSPORT EQUIVALENCE
            NSPH=3
            CNDOOR='NXT'
         ELSE IF(TEXT12.EQ.'EXCELT:') THEN
*           EXCELL TRANSPORT-TRANSPORT EQUIVALENCE
            NSPH=3
            CNDOOR='EXCELL'
         ELSE IF(TEXT12.EQ.'SNT:') THEN
*           SN TRANSPORT-TRANSPORT EQUIVALENCE
            NSPH=4
            CNDOOR='SN'
         ELSE IF(TEXT12.EQ.'BIVACT:') THEN
*           BIVAC TRANSPORT-DIFFUSION EQUIVALENCE
            NSPH=4
            CNDOOR='BIVAC'
         ELSE IF(TEXT12.EQ.'TRIVAT:') THEN
*           TRIVAC TRANSPORT-DIFFUSION EQUIVALENCE
            NSPH=4
            CNDOOR='TRIVAC'
         ELSE
            CALL XABORT('SPHGET: '//TEXT12//' IS AN INVALID TRACKING '
     >      //'MODULE')
         ENDIF
         GO TO 100
      ELSE
         CALL XABORT('SPHGET: INVALID TYPE OF MACRO-CALCULATION')
      ENDIF
      GO TO 10
*----
*  STORE SPH-RELATED INFORMATION
*----
  100 IF(JSPH.GT.0) THEN
         IF((NSPH.EQ.1).OR.(NSPH.EQ.2)) CNDOOR=CDOOR
         IF(NSPH.EQ.2) IPTRK2=IPTRK1
         IPSPH=LCMDID(IPEDIT,'SPH')
         CALL XDISET(ISTATE,NSTATE,0)
         ISTATE(1)=NSPH
         ISTATE(2)=KSPH
         ISTATE(3)=MAXIT
         ISTATE(4)=MAXNBI
         ISTATE(5)=IBELL
	 ISTATE(6)=IGAP
	 ISTATE(7)=IROW
	 IF(IGAP.GT.0) THEN
	    CALL LCMPUT(IPSPH,'GAP',IGAP,1,IBASE(IOFGAP))
	    CALL RLSARA(IBASE(IOFGAP))
	 ENDIF
	 IF(IROW.GT.0) THEN
	    CALL LCMPUT(IPSPH,'ROW',IROW,1,IBASE(IIROW))
	    CALL RLSARA(IBASE(IIROW))
	 ENDIF
         CALL LCMPUT(IPSPH,'STATE-VECTOR',NSTATE,1,ISTATE)
	 IF(NSPH.GE.2) THEN
            READ(CNDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
	    CALL LCMPUT(IPSPH,'SPH$TRK',3,3,KCHAR)
	    CALL LCMPUT(IPSPH,'SPH-EPSILON',1,2,EPSPH)
	 ENDIF
         IF(IPRINT.GT.0) WRITE(IOUT,200) (ISTATE(I),I=1,7),EPSPH,CNDOOR
      ENDIF
      RETURN
*
  200 FORMAT(/20H SPH-RELATED OPTIONS/1X,19(1H-)/
     1  7H NSPH  ,I8,47H   (=0: NO SPH CORRECTION; =1: READ SPH FACTORS,
     2  32H; >1: TYPE OF MACRO-CALCULATION)/
     3  7H KSPH  ,I8,47H   (<0: ASYMPTOTIC SPH NORMALIZATION; =1: AVERA,
     4  54HGE FLUX SPH NORMALIZATION; >1: SELENGUT NORMALIZATION)/
     5  7H MAXIT ,I8,37H   (MAXIMUM NUMBER OF SPH ITERATIONS)/
     6  7H MAXNBI,I8,47H   (MAXIMUM NUMBER OF BAD ITERATIONS BEFORE ABO,
     7  6HRTING)/
     8  7H IBELL ,I8,42H   (=0/1: BELL FACTOR ACCELERATION OFF/ON)/
     9  7H IGAP  ,I8,39H   (NUMBER OF REFERENCE REGIONS IN GAP)/
     1  7H IROW  ,I8,39H   (NUMBER OF REFERENCE REGIONS IN ROW)/
     2  8H EPSPH  ,1P,E7.1,26H   (CONVERGENCE CRITERION)/
     3  8H CNDOOR ,A8,37H  (MACRO-CALCULATION TRACKING MODULE))
      END
