*DECK SPHREF
      SUBROUTINE SPHREF(IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,ILEAKS,NWGTH,
     1 B2,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,NIFISS,MATCOD,VOLUME,KEYFLX,
     2 IGCOND,IMERGE,FLUXES,VOLMER,FLXMER,SUNMER,SIGT,SIGW,COURIN,
     3 PRODUC,SIGMA,NJJ,IJJ,IPOS,XSCAT,WORK,DIFF,GAMMA)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the reference merged/condensed set of cross sections
* to be used by an SPH homogenization algorithm.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPFLUX  pointer to the reference solution (L_FLUX signature).
* IPMACR  pointer to the reference macrolib (L_MACROLIB signature).
* IPSYS   pointer to the reference pij (L_PIJ signature).
* IPRINT  print flag (equal to 0 for no print).
* ITRANC  type of transport correction.
* ILEAKS  type of leakage calculation =0: no leakage; =1: homogeneous
*         leakage (Diffon); =2: isotropic streaming (Ecco);
*         =3: anisotropic streaming (Tibere).
* NWGTH   type of weighting for P1 cross section information:
*         = 0 P0; = 1 P1.
* B2      square buckling array.
*         For ILEAKS = 1 or 2, B2(4) is the homogeneous square buckling;
*         for ILEAKS = 3, B2(1),B2(2),B2(3) are the directional
*         heterogeneous and B2(4) is the homogeneous square buckling.
* NGROUP  number of energy groups in the reference calculation.
* NGCOND  number of condensed groups.
* NBMIX   number of mixtures.
* NREGIO  number of regions in the reference calculation.
* NMERGE  number of merged regions.
* NIFISS  number of fissile isotopes.
* MATCOD  mixture index per region.
* VOLUME  volume of regions.
* KEYFLX  position of average fluxes.
* IGCOND  limit of condensed groups.
* IMERGE  index of merged regions.
* FLUXES  fluxes.
*
*Parameters: output
* VOLMER  merged volumes.
* FLXMER  merged/condensed fluxes.
* SUNMER  merged/condensed production (fission + scattering) cross
*         sections. The last dimension is for secondary neutrons.
* SIGT    merged/condensed total cross sections.
* SIGW    merged/condensed within-group scattering cross sections.
* COURIN  ingoing currents (4*J-/S). They are computed if PIS
*         information is available on LCM, otherwise they are set
*         to 0.0.
*
*Parameters: scratch
* PRODUC
* SIGMA
* NJJ
* IJJ
* IPOS
* XSCAT
* WORK
* DIFF
* GAMMA
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,ILEAKS,NWGTH,NGROUP,
     1          NGCOND,NBMIX,NREGIO,NMERGE,NIFISS,MATCOD(NREGIO),
     2          KEYFLX(NREGIO),IGCOND(NGCOND),IMERGE(NREGIO),
     3          NJJ(NBMIX),IJJ(NBMIX),IPOS(NBMIX)
      REAL      B2(4),VOLUME(NREGIO),FLUXES(NREGIO,NGROUP,NWGTH+1),
     1          VOLMER(NMERGE),FLXMER(NMERGE,NGCOND,NWGTH+1),
     2          SUNMER(NMERGE,NGCOND,NGCOND,NWGTH+1),
     3          SIGT(NMERGE,NGCOND,NWGTH+1),SIGW(NMERGE,NGCOND,NWGTH+1),
     4          COURIN(NGCOND,NWGTH+1),PRODUC(NREGIO,NGCOND,NIFISS),
     5          SIGMA(0:NBMIX*NIFISS),XSCAT(NBMIX*NGROUP),
     6          WORK(NREGIO,2),DIFF(NGROUP),GAMMA(NGROUP)
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION SS,SUM,SUD
      CHARACTER SUFF(2)*2
      COMMON    RBASE(1)
      DATA      SUFF/'00','01'/
*----
*  COMPUTE THE FISSION RATE INFORMATION
*----
      SIGMA(0)=0.0
      DO 20 IFIS=1,NIFISS
      DO 20 IGRCD=1,NGCOND
      DO 20 IREG=1,NREGIO
   20 PRODUC(IREG,IGRCD,IFIS)=0.0
      IGRFIN=0
      JPMACR=LCMGID(IPMACR,'GROUP')
      DO 40 IGRCD=1,NGCOND
      IGRDEB=IGRFIN+1
      IGRFIN=IGCOND(IGRCD)
      DO 40 IGR=IGRDEB,IGRFIN
      KPMACR=LCMGIL(JPMACR,IGR)
      CALL LCMLEN(KPMACR,'NUSIGF',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         DO 30 IFIS=1,NIFISS
         CALL LCMGET(KPMACR,'NUSIGF',SIGMA(1))
         DO 30 IREG=1,NREGIO
         IBM=MATCOD(IREG)
         IF(IBM.GT.0) THEN
            SS=FLUXES(IREG,IGR,1)*SIGMA((IFIS-1)*NBMIX+IBM)
            PRODUC(IREG,IGRCD,IFIS)=PRODUC(IREG,IGRCD,IFIS)+SS
         ENDIF
  30     CONTINUE
      ENDIF
  40  CONTINUE
      CALL LCMLEN(IPFLUX,'K-EFFECTIVE',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.1) THEN
         CALL LCMGET(IPFLUX,'K-EFFECTIVE',EIGENK)
      ELSE
         EIGENK=1.0
      ENDIF
      IF(IPRINT.GT.5) WRITE(6,'(/16H SPHREF: EIGENK=,1P,E12.4)') EIGENK
*----
*  COMPUTE MERGED/CONDENSED CROSS SECTIONS
*----
      IF(ILEAKS.EQ.1) CALL LCMGET(IPFLUX,'DIFFB1HOM',DIFF)
      IF(NWGTH.EQ.1) CALL LCMGET(IPFLUX,'GAMMA',GAMMA)
      DO 170 INL=1,NWGTH+1
      IGRFIN=0
      DO 170 IGRCD=1,NGCOND
      DO 45 NIRG=1,NMERGE
      FLXMER(NIRG,IGRCD,INL)=0.0
      SIGT(NIRG,IGRCD,INL)=0.0
      SIGW(NIRG,IGRCD,INL)=0.0
      DO 45 JGRCD=1,NGCOND
   45 SUNMER(NIRG,JGRCD,IGRCD,INL)=0.0
      COURIN(IGRCD,INL)=0.0
      IGRDEB=IGRFIN+1
      IGRFIN=IGCOND(IGRCD)
      IF((ILEAKS.EQ.2).OR.(ILEAKS.EQ.3)) THEN
         CALL LCMLEN(IPFLUX,'FLUX',ILON,ITYLCM)
         IF(ILON.EQ.0) CALL XABORT('SPHREF: MISSING FLUX INFO.')
         JPFLUX=LCMGID(IPFLUX,'FLUX')
      ENDIF
      DO 170 IGR=IGRDEB,IGRFIN
      KPMACR=LCMGIL(JPMACR,IGR)
      CALL LCMLEN(KPMACR,'NTOT0',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'NTOT0',SIGMA(1))
      ELSE
         CALL XABORT('SPHREF: READ ERROR ON LCM RECORD= TOTAL.')
      ENDIF
      IF((ITRANC.NE.0).AND.(INL.EQ.1)) THEN
*        TRANSPORT CORRECTION.
         CALL SETARA(RBASE,NBMIX,JSIG1)
         CALL LCMGET(KPMACR,'TRANC',RBASE(JSIG1))
         DO 50 IMAT=1,NBMIX
   50    SIGMA(IMAT)=SIGMA(IMAT)-RBASE(JSIG1+IMAT-1)
         CALL RLSARA(RBASE(JSIG1))
      ENDIF
      IF((ILEAKS.EQ.2).OR.(ILEAKS.EQ.3)) THEN
         CALL LCMLEL(JPFLUX,IGR,ILCMLN,ITYLCM)
         IF(ILCMLN.EQ.0) CALL XABORT('SPHREF: MISSING FLUX INFO.')
         CALL SETARA(RBASE,ILCMLN,IWORK)
         CALL LCMGDL(JPFLUX,IGR,RBASE(IWORK))
      ENDIF
      IF((NWGTH.EQ.1).AND.(INL.EQ.2)) THEN
*        USE WITH THE FUNDAMENTAL CURRENT EQUATION OF THE ECCO MODEL.
         ZNUM=0.0
         ZDEN=0.0
         DO 55 IREG=1,NREGIO
         IBM=MATCOD(IREG)
         ZNUM=ZNUM+SIGMA(IBM)*FLUXES(IREG,IGR,1)*VOLUME(IREG)
  55     ZDEN=ZDEN+FLUXES(IREG,IGR,1)*VOLUME(IREG)
         ZNUM=ZNUM/ZDEN
      ENDIF
      DO 60 IREG=1,NREGIO
      IBM=MATCOD(IREG)
      NIRG=IMERGE(IREG)
      IF((NWGTH.EQ.1).AND.(INL.EQ.1)) THEN
        ZLEAK=B2(4)*FLUXES(IREG,IGR,2)
      ELSE IF((NWGTH.EQ.1).AND.(INL.EQ.2)) THEN
        ZLEAK=(-(1.0-GAMMA(IGR))*(ZNUM-SIGMA(IBM))*FLUXES(IREG,IGR,2)
     >        -FLUXES(IREG,IGR,1)/3.0)/GAMMA(IGR)
      ELSE IF(ILEAKS.EQ.1) THEN
        ZLEAK=DIFF(IGR)*B2(4)*FLUXES(IREG,IGR,1)
      ELSE IF(ILEAKS.EQ.2) THEN
        ZLEAK=B2(4)*RBASE(IWORK+KEYFLX(IREG)+ILCMLN/2-1)
      ELSE IF(ILEAKS.EQ.3) THEN
        ZLEAK=B2(1)*RBASE(IWORK+KEYFLX(IREG)+ILCMLN/4-1)+
     >        B2(2)*RBASE(IWORK+KEYFLX(IREG)+ILCMLN/2-1)+
     >        B2(3)*RBASE(IWORK+KEYFLX(IREG)+3*ILCMLN/4-1)
      ELSE
        ZLEAK=0.0
      ENDIF
      IF(NIRG.GT.0) THEN
        FLXMER(NIRG,IGRCD,INL)=FLXMER(NIRG,IGRCD,INL)
     >                        +FLUXES(IREG,IGR,INL)*VOLUME(IREG)
        SIGT(NIRG,IGRCD,INL)=SIGT(NIRG,IGRCD,INL)+SIGMA(IBM)
     >                      *FLUXES(IREG,IGR,INL)*VOLUME(IREG)
        SUNMER(NIRG,IGRCD,IGRCD,INL)=SUNMER(NIRG,IGRCD,IGRCD,INL)
     >                              -ZLEAK*VOLUME(IREG)
      ENDIF
      WORK(IREG,1)=-ZLEAK
   60 CONTINUE
      IF((ILEAKS.EQ.2).OR.(ILEAKS.EQ.3)) CALL RLSARA(RBASE(IWORK))
*
      CALL LCMLEN(KPMACR,'CHI',ILCMLN,ITYLCM)
      IF((ILCMLN.GT.0).AND.(INL.EQ.1)) THEN
         DO 80 IFIS=1,NIFISS
         CALL LCMGET(KPMACR,'CHI',SIGMA(1))
         DO 80 IREG=1,NREGIO
         IBM=MATCOD(IREG)
         NIRG=IMERGE(IREG)
         IF(IBM.GT.0) THEN
            DO 70 JGRCD=1,NGCOND
            SS=SIGMA((IFIS-1)*NBMIX+IBM)*PRODUC(IREG,JGRCD,IFIS)/EIGENK
            IF(NIRG.GT.0) SUNMER(NIRG,JGRCD,IGRCD,1)=
     >                    SUNMER(NIRG,JGRCD,IGRCD,1)+SS*VOLUME(IREG)
   70       WORK(IREG,1)=WORK(IREG,1)+SS
         ENDIF
   80    CONTINUE
      ENDIF
*
      CALL LCMLEN(KPMACR,'SIGW'//SUFF(INL),ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'SIGW'//SUFF(INL),SIGMA(1))
      ELSE
         DO 100 I=1,NBMIX
  100    SIGMA(I)=0.0
      ENDIF
      IF((ITRANC.NE.0).AND.(INL.EQ.1)) THEN
*        TRANSPORT CORRECTION.
         CALL SETARA(RBASE,NBMIX,JSIG1)
         CALL LCMGET(KPMACR,'TRANC',RBASE(JSIG1))
         DO 120 IMAT=1,NBMIX
  120    SIGMA(IMAT)=SIGMA(IMAT)-RBASE(JSIG1+IMAT-1)
         CALL RLSARA(RBASE(JSIG1))
      ENDIF
      CALL LCMLEN(KPMACR,'NJJS'//SUFF(INL),ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(KPMACR,'NJJS'//SUFF(INL),NJJ)
         CALL LCMGET(KPMACR,'IJJS'//SUFF(INL),IJJ)
         CALL LCMGET(KPMACR,'IPOS'//SUFF(INL),IPOS)
         CALL LCMGET(KPMACR,'SCAT'//SUFF(INL),XSCAT)
         DO 150 IREG=1,NREGIO
         IBM=MATCOD(IREG)
         NIRG=IMERGE(IREG)
         IF(IBM.GT.0) THEN
            JGRFIN=0
            DO 140 JGRCD=1,NGCOND
            SS=0.0D0
            JGRDEB=JGRFIN+1
            JGRFIN=IGCOND(JGRCD)
            J2=MIN(JGRFIN,IJJ(IBM))
            J1=MAX(JGRDEB,IJJ(IBM)-NJJ(IBM)+1)
            IPO=IPOS(IBM)+IJJ(IBM)-J2
            DO 130 JGR=J2,J1,-1
            IF(IGR.EQ.JGR) THEN
               SS=SS+SIGMA(IBM)*FLUXES(IREG,JGR,INL)
            ELSE
               SS=SS+XSCAT(IPO)*FLUXES(IREG,JGR,INL)
            ENDIF
  130       IPO=IPO+1
            IF(INL.EQ.2) SS=SS/GAMMA(IGR)
            IF(NIRG.GT.0) THEN
               IF(IGRCD.EQ.JGRCD)
     >         SIGW(NIRG,IGRCD,INL)=SIGW(NIRG,IGRCD,INL)+SS*VOLUME(IREG)
               SUNMER(NIRG,JGRCD,IGRCD,INL)=SUNMER(NIRG,JGRCD,IGRCD,INL)
     >                                     +SS*VOLUME(IREG)
            ENDIF
  140       WORK(IREG,1)=WORK(IREG,1)+SS
         ENDIF
  150    CONTINUE
      ENDIF
*
      IF(IPSYS.NE.0) THEN
         IF(INL.EQ.1) THEN
            JPSYS=LCMGID(IPSYS,'GROUP')
         ELSE IF(INL.EQ.2) THEN
            IPSYS2=LCMGID(IPSYS,'STREAMING')
            JPSYS=LCMGID(IPSYS2,'GROUP')
         ENDIF
         KPSYS=LCMGIL(JPSYS,IGR)
         CALL LCMLEN(KPSYS,'DRAGON-WIS',ILCMLN,ITYLCM)
         IF(ILCMLN.EQ.NREGIO) THEN
            CALL LCMGET(KPSYS,'DRAGON-WIS',WORK(1,2))
            CALL LCMGET(KPSYS,'DRAGON-TXSC',SIGMA(0))
            SUM=0.0D0
            SUD=0.0D0
            DO 160 IREG=1,NREGIO
            FACTOR=VOLUME(IREG)
            IBM=MATCOD(IREG)
            SUM=SUM+SIGMA(IBM)*FACTOR*FLUXES(IREG,IGR,1)
     >             -WORK(IREG,1)*FACTOR*(1.0-WORK(IREG,2))
  160       SUD=SUD+SIGMA(IBM)*FACTOR*WORK(IREG,2)
            COURIN(IGRCD,INL)=COURIN(IGRCD,INL)+SUM/SUD
         ENDIF
      ENDIF
  170 CONTINUE
*
      DO 180 NIRG=1,NMERGE
  180 VOLMER(NIRG)=0.0
      DO 190 IREG=1,NREGIO
      NIRG=IMERGE(IREG)
      IF(NIRG.GT.0) VOLMER(NIRG)=VOLMER(NIRG)+VOLUME(IREG)
  190 CONTINUE
      DO 205 INL=1,NWGTH+1
      DO 205 IGRCD=1,NGCOND
      DO 205 NIRG=1,NMERGE
      IF(VOLMER(NIRG).NE.0.0) THEN
        SIGT(NIRG,IGRCD,INL)=SIGT(NIRG,IGRCD,INL)/FLXMER(NIRG,IGRCD,INL)
        SIGW(NIRG,IGRCD,INL)=SIGW(NIRG,IGRCD,INL)/FLXMER(NIRG,IGRCD,INL)
        DO 200 JGRCD=1,NGCOND
        SUNMER(NIRG,JGRCD,IGRCD,INL)=SUNMER(NIRG,JGRCD,IGRCD,INL)
     >                              /FLXMER(NIRG,JGRCD,INL)
  200   CONTINUE
      ENDIF
  205 CONTINUE
      DO 210 INL=1,NWGTH+1
      DO 210 IGRCD=1,NGCOND
      DO 210 NIRG=1,NMERGE
      IF(VOLMER(NIRG).NE.0.0) THEN
         FLXMER(NIRG,IGRCD,INL)=FLXMER(NIRG,IGRCD,INL)/VOLMER(NIRG)
      ENDIF
  210 CONTINUE
      RETURN
      END
