*DECK SPHSNO
      SUBROUTINE SPHSNO (IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,NREG,NUN,
     1 NBMIX,NSCT,MAT,VOL,KEY,MERG,COUR,FUNKNO,SOURCE,ZGAR1,ZGAR2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* SPH source term calculation for a PIJ or SN discretization.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IEX     iteration number.
* NGCOND  number of groups condensed.
* IGR     energy group index.
* NMERGE  number of merged regions.
* FLXMER  flux estimate per mixture.
* SUNMER  incoming source (scattering+fission) cross sections.
* NREG    number of regions in macro-calculation.
* NUN     number of unknowns per group in macro-calculation.
* NBMIX   number of macro-mixtures.
* NSCT    maximum number of spherical harmonics moments of the flux.
* MAT     mixture index per region.
* VOL     volume of regions.
* KEY     position of the flux components associated with each volume.
* MERG    index of merged regions.
* COUR    four times the incoming current per unit surface.
* FUNKNO  previously calculated fluxes.
*
*Parameters: output
* SOURCE  fission and diffusion sources.
*
*Parameters: scratch
* ZGAR1
* ZGAR2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IEX,NGCOND,IGR,NMERGE,NREG,NUN,NBMIX,MAT(NREG),KEY(NREG),
     1 MERG(NREG)
      REAL FLXMER(NMERGE,NGCOND),SUNMER(NMERGE,NGCOND),VOL(NREG),COUR,
     1 FUNKNO(NUN,NGCOND),SOURCE(NUN),ZGAR1(NBMIX),ZGAR2(NBMIX)
*----
*  INCOMING CURRENT SOURCE.
*----
      IF(COUR.NE.0.0) THEN
         CALL XABORT('SPHSNO: INCOMING CURRENT NOT IMPLEMENTED.')
      ENDIF
*----
*  SCATTERING AND FISSION SOURCE.
*----
      CALL XDRSET(SOURCE,NUN,0.0)
      IGR2=NGCOND
      IF(IEX.EQ.1) IGR2=IGR-1
      DO 110 NL=1,IGR2
      CALL XDRSET(ZGAR1,NBMIX,0.0)
      CALL XDRSET(ZGAR2,NBMIX,0.0)
      DO 80 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 80
      IF(VOL(K).EQ.0.0) GO TO 80
      ZGAR1(L)=ZGAR1(L)+VOL(K)
      ZGAR2(L)=ZGAR2(L)+FUNKNO(KEY(K),NL)*VOL(K)
   80 CONTINUE
      DO 110 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 110
      IF(VOL(K).EQ.0.0) GO TO 110
      DO 90 IL=1,NSCT
      IND1=(K-1)*NSCT+IL
      PV=SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)*ZGAR1(L)/ZGAR2(L)
   90 SOURCE(IND1)=SOURCE(IND1)+FUNKNO(IND1,NL)*PV
  110 CONTINUE
      IF(IEX.GT.1) RETURN
      DO 140 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 140
      IF(VOL(K).EQ.0.0) GO TO 140
      IND1=KEY(K)
      PV=0.0
      DO 120 NL=IGR,NGCOND
  120 PV=PV+SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)
      SOURCE(IND1)=SOURCE(IND1)+PV
  140 CONTINUE
      RETURN
      END
