*DECK SYBCP1
      SUBROUTINE SYBCP1 (IPTRK,ITG,IMPX,NREG,SIGT,SIGW,PIJ)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the scattering-reduced collision probabilities for
* Sybil.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the Sybil tracking (L_TRACK signature).
* ITG     type of Sybil one-speed solution operator.
* IMPX    print flag (equal to zero for no print).
* NREG    total number of regions.
* SIGT    total macroscopic cross sections ordered by volume.
* SIGW    P0 within-group scattering macroscopic cross sections
*         ordered by volume.
*
*Parameters: output
* PIJ     scattering-reduced collision probabilities matrix.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,ITG,IMPX,NREG
      REAL SIGT(NREG),SIGW(NREG),PIJ(NREG,NREG)
*----
*  LOCAL VARIABLES
*----
      INTEGER IPAR(16),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      IF(ITG.EQ.1) THEN
         PIJ(1,1)=1.0/(SIGT(1)-SIGW(1))
      ELSE IF(ITG.EQ.2) THEN
         CALL LCMSIX(IPTRK,'PURE-GEOM',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         ITYPE=IPAR(1)
         IHEX=IPAR(2)
         IQUA2=IPAR(3)
         IF(ITYPE.GE.8) CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL LCMIOF(IPTRK,'XXX',IXX2)
         CALL LCMIOF(IPTRK,'YYY',IYY2)
         CALL LCMIOF(IPTRK,'ZZZ',IZZ2)
         CALL LCMIOF(IPTRK,'NCODE',INCODE)
         CALL LCMIOF(IPTRK,'ZCODE',IZCODE)
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL SETARA (BASE,6*NREG,IX)
         IF(ITYPE.EQ.2) THEN
            CALL SETARA(BASE,4*NREG*NREG,IF2)
            CALL SYBALP(NREG,NREG,BASE(IXX2),SIGT,BASE(INCODE),
     1      BASE(IZCODE),PIJ,BASE(IX),BASE(IF2))
            CALL RLSARA(BASE(IF2))
         ELSE IF(ITYPE.EQ.3) THEN
            CALL SETARA(BASE,1+IQUA2*((NREG*(5+NREG))/2),IZ)
            CALL SYBT1D(NREG,BASE(IXX2),.FALSE.,IQUA2,BASE(IZ))
            CALL SYBALC(NREG,NREG,BASE(IXX2),SIGT,IQUA2,BASE(IZCODE+1),
     1      BASE(IZ),PIJ,BASE(IX))
            CALL RLSARA(BASE(IZ))
         ELSE IF(ITYPE.EQ.4) THEN
            CALL SETARA(BASE,1+IQUA2*((NREG*(5+NREG))/2),IZ)
            CALL SYBT1D(NREG,BASE(IXX2),.TRUE.,IQUA2,BASE(IZ))
            CALL SYBALS(NREG,NREG,BASE(IXX2),SIGT,IQUA2,BASE(IZCODE+1),
     1      BASE(IZ),PIJ,BASE(IX))
            CALL RLSARA(BASE(IZ))
         ENDIF
         CALL RLSARA(BASE(IX))
         CALL SYBWIJ(NREG,NREG,SIGT,SIGW,PIJ)
      ELSE IF(ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         IQUA3=IPAR(2)
         ISTAT=IPAR(3)
         CALL LCMIOF(IPTRK,'NMC',INMC3)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR3)
         CALL LCMIOF(IPTRK,'PROCEL',IPROCE)
         CALL LCMSIX(IPTRK,' ',2)
         NPIJ=0
         DO 10 IKG=1,NSUPCE
         J2=IBASE(INMC3+IKG)-IBASE(INMC3+IKG-1)
   10    NPIJ=NPIJ+J2*J2
*
         CALL SETARA(BASE,NREG,IIAP)
         CALL SETARA(BASE,NPIJ,IPIJW)
         CALL SETARA(BASE,NREG,IPISW)
         CALL SETARA(BASE,NREG,IPSJW)
         CALL SETARA(BASE,2*NSUPCE**2,IPSSB)
         CALL SETARA(BASE,NSUPCE,IPSSW)
         CALL SYBRXE(NREG,NPIJ,NSUPCE,BASE(IRAYR3),SIGT,SIGW,PIJ,IQUA3,
     1   ISTAT,IBASE(INMC3),BASE(IPROCE),IMPX,BASE(IIAP),BASE(IPIJW),
     2   BASE(IPISW),BASE(IPSJW),BASE(IPSSB),BASE(IPSSW))
         CALL RLSARA(BASE(IPSSW))
         CALL RLSARA(BASE(IPSSB))
         CALL RLSARA(BASE(IPSJW))
         CALL RLSARA(BASE(IPISW))
         CALL RLSARA(BASE(IPIJW))
         CALL RLSARA(BASE(IIAP))
      ELSE IF(ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IHEX=IPAR(1)
         MULTC=IPAR(2)
         IWIGN=IPAR(3)
         NMCEL=IPAR(4)
         NMERGE=IPAR(5)
         NGEN=IPAR(6)
         IJAT=IPAR(7)
         LMAILI=IPAR(15)
         LMAILR=IPAR(16)
         CALL LCMIOF(IPTRK,'XX',IXX4)
         CALL LCMIOF(IPTRK,'YY',IYY4)
         CALL LCMIOF(IPTRK,'LSECT',ILSEC4)
         CALL LCMIOF(IPTRK,'NMC',INMC4)
         CALL LCMIOF(IPTRK,'NMCR',INMCR4)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR4)
         CALL LCMIOF(IPTRK,'MAIL',IMAIL)
         IF(LMAILI.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILI',IIZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IIZMAI=IMAIL
         ENDIF
         IF(LMAILR.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILR',IRZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IRZMAI=IMAIL
         ENDIF
         CALL LCMIOF(IPTRK,'IFR',IIFR)
         CALL LCMIOF(IPTRK,'ALB',IALB)
         CALL LCMIOF(IPTRK,'INUM',IINUM)
         CALL LCMIOF(IPTRK,'MIX',IMIX)
         CALL LCMIOF(IPTRK,'DVX',IDVX)
         CALL LCMIOF(IPTRK,'IGEN',IIGEN)
         CALL LCMSIX(IPTRK,' ',2)
*
         NCOUR=4
         IF(IHEX.NE.0) NCOUR=6
         NPIJ=0
         DO 20 IKG=1,NGEN
         J2=IBASE(INMC4+IKG)-IBASE(INMC4+IKG-1)
   20    NPIJ=NPIJ+J2*J2
         NPIS=IBASE(INMC4+NGEN)
         CALL SETARA(BASE,2*IJAT*IJAT,IPSSB)
         CALL SETARA(BASE,NREG,ISIGT2)
         CALL SETARA(BASE,NREG,ISIGW2)
         IF(MULTC.EQ.1) THEN
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NPIS,IPISW)
            CALL SETARA(BASE,NPIS,IPSJW)
            CALL SETARA(BASE,NGEN,IPSSW)
            CALL SYBRX2(NREG,NPIJ,NPIS,SIGT,SIGW,PIJ,IMPX,NCOUR,
     1      IWIGN,NMCEL,NMERGE,NGEN,IPAR(8),BASE(IXX4),BASE(IYY4),
     2      IBASE(INMC4),BASE(IRAYR4),IBASE(IMAIL),BASE(IRZMAI),
     3      IBASE(IIFR),BASE(IALB),IBASE(IINUM),IBASE(IIGEN),
     4      BASE(IPSSB),BASE(ISIGT2),BASE(ISIGW2),BASE(IPIJW),
     5      BASE(IPISW),BASE(IPSJW),BASE(IPSSW))
            CALL RLSARA(BASE(IPSSW))
            CALL RLSARA(BASE(IPSJW))
            CALL RLSARA(BASE(IPISW))
            CALL RLSARA(BASE(IPIJW))
         ELSE
            IF(MULTC.EQ.4) NCOUR=3*NCOUR
            NRAYRE=IBASE(INMCR4+NGEN)
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NCOUR*NPIS,IPISW)
            CALL SETARA(BASE,NCOUR*NPIS,IPSJW)
            CALL SETARA(BASE,NGEN*NCOUR*NCOUR,IPSSW)
            CALL SYBRX3(MULTC,NREG,NPIJ,NPIS,NRAYRE,SIGT,SIGW,PIJ,IMPX,
     1      NCOUR,IWIGN,NMCEL,NMERGE,NGEN,IJAT,IPAR(8),BASE(IXX4),
     2      BASE(IYY4),IBASE(ILSEC4),IBASE(INMC4),IBASE(INMCR4),
     3      BASE(IRAYR4),IBASE(IMAIL),BASE(IIZMAI),BASE(IRZMAI),
     4      IBASE(IIFR),BASE(IALB),IBASE(IINUM),IBASE(IMIX),BASE(IDVX),
     5      IBASE(IIGEN),BASE(IPSSB),BASE(ISIGT2),BASE(ISIGW2),
     6      BASE(IPIJW),BASE(IPISW),BASE(IPSJW),BASE(IPSSW))
            CALL RLSARA(BASE(IPSSW))
            CALL RLSARA(BASE(IPSJW))
            CALL RLSARA(BASE(IPISW))
            CALL RLSARA(BASE(IPIJW))
         ENDIF
         CALL RLSARA(BASE(ISIGW2))
         CALL RLSARA(BASE(ISIGT2))
         CALL RLSARA(BASE(IPSSB))
      ELSE
         CALL XABORT('SYBCP1: UNKNOWN CP MODULE.')
      ENDIF
*
      IF(IMPX.GE.7) THEN
         WRITE (6,1130) (J,J=1,NREG)
         DO 90 I=1,NREG
         WRITE (6,1140) I,(PIJ(I,J),J=1,NREG)
   90    CONTINUE
         WRITE (6,'(//)')
      ENDIF
      RETURN
*
 1130 FORMAT (//49H SYBCP1: SCATTERING-REDUCED COLLISION PROBABILITY,
     1 9H MATRIX ://(11X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,
     2 I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,
     3 2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4))
 1140 FORMAT (3H I=,I4,2H: ,1P,11E11.3/(9X,11E11.3))
      END
