*DECK SYBHT3
      SUBROUTINE SYBHT3 (IR1,IR2,IPAS,IPAS2,NCOUR,NG,NSMAX,NS,IBI,FRACT,
     1 VOLK,P,PIS,PSI,RRRR,SIGT0,QKDEL,PIL,PLS,PSL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the reduced collision probabilities for the double
* heterogeneity option of sybil (part 2). This version allows the
* calculation of PIS and PSJ matrices.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* IPAS    number of volumes in the composite geometry.
* IPAS2   number of volumes in the macro geometry.
* NCOUR   number of surfaces.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given
*         macro volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* RRRR    information computed by SYBHT1.
* SIGT0   equivalent macroscopic cross section in each macro volume.
* QKDEL   information computed by SYBHT1.
* PIL     reduced collision probabilities between the macro volumes.
* PLS     escape probabilities for the geometry with macro volumes.
* PSL     reduced collision probabilities for neutrons entering in
*         the geometry with macro volumes.
*
*Parameters: output
* P       reduced collision probabilities for the composite geometry.
* PIS     escape probabilities for the composite geometry.
* PSI     reduced collision probabilities for neutrons entering in
*         the composite geometry.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IR1,IR2,IPAS,IPAS2,NCOUR,NG,NSMAX,NS(NG),IBI(IPAS2)
      REAL FRACT(NG,IR2),VOLK(NG,NSMAX),P(IPAS,IPAS),PIS(NCOUR,IPAS),
     1 PSI(NCOUR,IPAS),RRRR(IPAS2),SIGT0(IPAS2),QKDEL(NG,NSMAX,IPAS2),
     2 PIL(IPAS2,IPAS2),PLS(NCOUR,IPAS2),PSL(NCOUR,IPAS2)
*
      IND1=IPAS2
      DO 200 I=1,IPAS2
      IMIL=IBI(I)
      I1PL=0
      IF(IMIL.GT.IR1) THEN
         RSLN=(1.0-RRRR(I))*RRRR(I)/SIGT0(I)
         DILF=1.0
         DO 10 J=1,NG
   10    DILF=DILF-FRACT(J,IMIL)
         DO 20 IC=1,NCOUR
         PIS(IC,I)=RRRR(I)*PLS(IC,I)
   20    PSI(IC,I)=DILF*RRRR(I)*PSL(IC,I)
         P(I,I)=P(I,I)+DILF*RSLN
         IND1P=IND1
         DO 60 J=1,NG
         IF(FRACT(J,IMIL).LE.0.00001) GO TO 60
         I1PL=I1PL+NS(J)
         DO 50 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=RRRR(I)*QKDEL(J,K,I)
         DO 30 IC=1,NCOUR
         PIS(IC,IND1P)=RIKS*PLS(IC,I)
   30    PSI(IC,IND1P)=FRACT(J,IMIL)*VOLK(J,K)*RIKS*PSL(IC,I)
         RIKS=QKDEL(J,K,I)*RSLN
         P(IND1P,I)=P(IND1P,I)+DILF*RIKS
         P(I,IND1P)=P(I,IND1P)+FRACT(J,IMIL)*VOLK(J,K)*RIKS
         IND2P=IND1
         DO 50 M=1,NG
         FRT=FRACT(M,IMIL)
         IF(FRT.LE.0.00001) GO TO 50
         DO 40 N=1,NS(M)
         IND2P=IND2P+1
   40    P(IND1P,IND2P)=P(IND1P,IND2P)+FRT*VOLK(M,N)*QKDEL(M,N,I)*RIKS
   50    CONTINUE
   60    CONTINUE
      ELSE
         DO 70 IC=1,NCOUR
         PIS(IC,I)=PLS(IC,I)
   70    PSI(IC,I)=PSL(IC,I)
      ENDIF
*
      IND2=IPAS2
      DO 190 L=1,IPAS2
      LMIL=IBI(L)
      I2PL=0
      IF(LMIL.GT.IR1) THEN
         DILF=1.0
         DO 80 M=1,NG
   80    DILF=DILF-FRACT(M,LMIL)
         RSL0=DILF*RRRR(L)
         P(I,L)=P(I,L)+RRRR(I)*PIL(I,L)*RSL0
         IND2P=IND2
         DO 100 M=1,NG
         FRT=FRACT(M,LMIL)
         IF(FRT.LE.0.00001) GO TO 100
         I2PL=I2PL+NS(M)
         DO 90 N=1,NS(M)
         IND2P=IND2P+1
         RSLN=QKDEL(M,N,L)*FRT*VOLK(M,N)*RRRR(L)
   90    P(I,IND2P)=P(I,IND2P)+RRRR(I)*PIL(I,L)*RSLN
  100    CONTINUE
         IND1P=IND1
         DO 160 J=1,NG
         IF(FRACT(J,IMIL).LE.0.00001) GO TO 160
         DO 150 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RRRR(I)
         P(IND1P,L)=P(IND1P,L)+RIKS*PIL(I,L)*RSL0
         IND2P=IND2
         DO 150 M=1,NG
         FRT=FRACT(M,LMIL)
         IF(FRT.LE.0.00001) GO TO 150
         DO 140 N=1,NS(M)
         IND2P=IND2P+1
         RSLN=QKDEL(M,N,L)*FRT*VOLK(M,N)*RRRR(L)
  140    P(IND1P,IND2P)=P(IND1P,IND2P)+RIKS*PIL(I,L)*RSLN
  150    CONTINUE
  160    CONTINUE
      ELSE
         P(I,L)=P(I,L)+RRRR(I)*PIL(I,L)
         IND1P=IND1
         DO 180 J=1,NG
         IF(FRACT(J,IMIL).LE.0.00001) GO TO 180
         DO 170 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RRRR(I)
  170    P(IND1P,L)=P(IND1P,L)+RIKS*PIL(I,L)
  180    CONTINUE
      ENDIF
      IND2=IND2+I2PL
  190 CONTINUE
      IND1=IND1+I1PL
  200 CONTINUE
      RETURN
      END
