*DECK SYBILA
      SUBROUTINE SYBILA (IPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT,
     1 VOL,KNORM,SIGT0,SIGW0,ILK,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of cellwise scattering-reduced collision, escape and
* transmission probabilities for the current iteration method in
* Eurydice (Sybil).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the system matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* NREG    total number of merged regions for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NANI    number of Legendre orders (only NANI=1 allowed).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   not used.
* SIGT0   total macroscopic cross sections ordered by mixture.
* SIGW0   within-group scattering macroscopic cross section ordered
*         by mixture.
* ILK     leakage flag (ILK=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER*72 TITR
      LOGICAL ILK
      INTEGER IPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT(NREG),KNORM
      REAL VOL(NREG),SIGT0(0:NBMIX),SIGW0(0:NBMIX)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER JPAR(NSTATE),IBASE(1),IPAR(16)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER SYBIL SPECIFIC PARAMETERS
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      ITG=JPAR(6)
*
      IF(ITG.EQ.1) THEN
         CALL XABORT('SYBILA: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (1).')
      ELSE IF(ITG.EQ.2) THEN
         CALL XABORT('SYBILA: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (2).')
      ELSE IF(ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         IQUA3=IPAR(2)
         CALL LCMIOF(IPTRK,'NMC',INMC3)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR3)
         CALL LCMSIX(IPTRK,' ',2)
         NPIJ=0
         DO 10 IKG=1,NSUPCE
         J2=IBASE(INMC3+IKG)-IBASE(INMC3+IKG-1)
   10    NPIJ=NPIJ+J2*J2
         IF(IBASE(INMC3+NSUPCE).NE.NREG) CALL XABORT('SYBILA: ABORT.')
         CALL SETARA(BASE,NREG,ISIGT)
         CALL SETARA(BASE,NREG,ISIGW)
         DO 15 I=1,NREG
         BASE(ISIGT+I-1)=SIGT0(MAT(I))
   15    BASE(ISIGW+I-1)=SIGW0(MAT(I))
*
         CALL SETARA(BASE,NPIJ,IPIJW)
         CALL SETARA(BASE,NREG,IPISW)
         CALL SETARA(BASE,NREG,IPSJW)
         CALL SETARA(BASE,NSUPCE,IPSSW)
         CALL SETARA(BASE,NREG,IPIS)
         CALL SETARA(BASE,NREG,IPSJ)
         CALL SYB001 (NREG,NSUPCE,NPIJ,BASE(ISIGT),BASE(ISIGW),IMPX,
     1   IQUA3,IBASE(INMC3),BASE(IRAYR3),BASE(IPIJW),BASE(IPISW),
     2   BASE(IPSJW),BASE(IPSSW),BASE(IPIS),BASE(IPSJ))
         CALL RLSARA(BASE(IPSJ))
         CALL RLSARA(BASE(IPIS))
         CALL LCMPOF(IPSYS,'PSSW$SYBIL',NSUPCE,2,IPSSW)
         CALL LCMPOF(IPSYS,'PSJW$SYBIL',NREG,2,IPSJW)
         CALL LCMPOF(IPSYS,'PISW$SYBIL',NREG,2,IPISW)
         CALL LCMPOF(IPSYS,'PIJW$SYBIL',NPIJ,2,IPIJW)
         CALL RLSARA(BASE(ISIGW))
         CALL RLSARA(BASE(ISIGT))
      ELSE IF(ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IHEX=IPAR(1)
         MULTC=IPAR(2)
         IWIGN=IPAR(3)
         NMCEL=IPAR(4)
         NMERGE=IPAR(5)
         NGEN=IPAR(6)
         IJAT=IPAR(7)
         LMAILI=IPAR(15)
         LMAILR=IPAR(16)
         CALL LCMIOF(IPTRK,'XX',IXX4)
         CALL LCMIOF(IPTRK,'YY',IYY4)
         CALL LCMIOF(IPTRK,'LSECT',ILSEC4)
         CALL LCMIOF(IPTRK,'NMC',INMC4)
         CALL LCMIOF(IPTRK,'NMCR',INMCR4)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR4)
         CALL LCMIOF(IPTRK,'MAIL',IMAIL)
         IF(LMAILI.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILI',IIZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IIZMAI=IMAIL
         ENDIF
         IF(LMAILR.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILR',IRZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IRZMAI=IMAIL
         ENDIF
         CALL LCMIOF(IPTRK,'IGEN',IIGEN)
         CALL LCMSIX(IPTRK,' ',2)
*
         NCOUR=4
         IF(IHEX.NE.0) NCOUR=6
         NPIJ=0
         DO 20 IKG=1,NGEN
         J2=IBASE(INMC4+IKG)-IBASE(INMC4+IKG-1)
   20    NPIJ=NPIJ+J2*J2
         NPIS=IBASE(INMC4+NGEN)
         CALL SETARA(BASE,NREG,ISIGT2)
         CALL SETARA(BASE,NREG,ISIGW2)
         I1=0
         DO 40 IKK=1,NMERGE
         IKG=IBASE(IIGEN+IKK-1)
         J1=IBASE(INMC4+IKG-1)
         I2=IBASE(INMC4+IKG)-J1
         DO 30 I=1,I2
         BASE(ISIGT2+J1+I-1)=SIGT0(MAT(I1+I))
   30    BASE(ISIGW2+J1+I-1)=SIGW0(MAT(I1+I))
   40    I1=I1+I2
         IF(MULTC.EQ.1) THEN
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NPIS,IPISW)
            CALL SETARA(BASE,NPIS,IPSJW)
            CALL SETARA(BASE,NGEN,IPSSW)
*
            CALL SETARA(BASE,NPIS,IPIS)
            CALL SETARA(BASE,NPIS,IPSJ)
            CALL SYB002 (NGEN,NPIJ,NPIS,BASE(ISIGT2),BASE(ISIGW2),IMPX,
     1      NCOUR,IWIGN,IPAR(8),BASE(IXX4),BASE(IYY4),IBASE(INMC4),
     2      BASE(IRAYR4),IBASE(IMAIL),BASE(IRZMAI),BASE(IPIJW),
     3      BASE(IPISW),BASE(IPSJW),BASE(IPSSW),BASE(IPIS),BASE(IPSJ))
            CALL RLSARA(BASE(IPSJ))
            CALL RLSARA(BASE(IPIS))
            CALL LCMPOF(IPSYS,'PSSW$SYBIL',NGEN,2,IPSSW)
            CALL LCMPOF(IPSYS,'PSJW$SYBIL',NPIS,2,IPSJW)
            CALL LCMPOF(IPSYS,'PISW$SYBIL',NPIS,2,IPISW)
            CALL LCMPOF(IPSYS,'PIJW$SYBIL',NPIJ,2,IPIJW)
         ELSE
            IF(MULTC.EQ.4) NCOUR=3*NCOUR
            NRAYRE=IBASE(INMCR4+NGEN)
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NCOUR*NPIS,IPISW)
            CALL SETARA(BASE,NCOUR*NPIS,IPSJW)
            CALL SETARA(BASE,NCOUR*NCOUR*NGEN,IPSSW)
*
            CALL SETARA(BASE,NCOUR*NPIS,IPIS)
            CALL SETARA(BASE,NCOUR*NPIS,IPSJ)
            IF(MULTC.EQ.2) THEN
*              ROTH X 4 OR ROTH X 6 APPROXIMATION.
               CALL SYB003 (NGEN,NPIJ,NPIS,BASE(ISIGT2),BASE(ISIGW2),
     1         IMPX,NCOUR,IWIGN,IPAR(8),BASE(IXX4),BASE(IYY4),
     2         IBASE(INMC4),BASE(IRAYR4),IBASE(IMAIL),BASE(IRZMAI),
     3         BASE(IPIJW),BASE(IPISW),BASE(IPSJW),BASE(IPSSW),
     4         BASE(IPIS),BASE(IPSJ))
            ELSE IF(MULTC.EQ.3) THEN
*              DP-0 APPROXIMATION.
               CALL SYB004 (NGEN,NPIJ,NPIS,NRAYRE,BASE(ISIGT2),
     1         BASE(ISIGW2),IMPX,NCOUR,IPAR(8),BASE(IXX4),BASE(IYY4),
     2         IBASE(ILSEC4),IBASE(INMC4),IBASE(INMCR4),BASE(IRAYR4),
     3         IBASE(IMAIL),IBASE(IIZMAI),BASE(IRZMAI),BASE(IPIJW),
     4         BASE(IPISW),BASE(IPSJW),BASE(IPSSW),BASE(IPIS),
     5         BASE(IPSJ))
            ELSE IF(MULTC.EQ.4) THEN
*              DP-1 APPROXIMATION.
               CALL SYB005 (NGEN,NPIJ,NPIS,NRAYRE,BASE(ISIGT2),
     1         BASE(ISIGW2),IMPX,NCOUR,IPAR(8),BASE(IXX4),BASE(IYY4),
     2         IBASE(ILSEC4),IBASE(INMC4),IBASE(INMCR4),BASE(IRAYR4),
     3         IBASE(IMAIL),IBASE(IIZMAI),BASE(IRZMAI),BASE(IPIJW),
     4         BASE(IPISW),BASE(IPSJW),BASE(IPSSW),BASE(IPIS),
     5         BASE(IPSJ))
            ELSE
               CALL XABORT('SYBILA: UNKNOWN CP MODULE(1).')
            ENDIF
            CALL RLSARA(BASE(IPSJ))
            CALL RLSARA(BASE(IPIS))
            CALL LCMPOF(IPSYS,'PSSW$SYBIL',NCOUR*NCOUR*NGEN,2,IPSSW)
            CALL LCMPOF(IPSYS,'PSJW$SYBIL',NCOUR*NPIS,2,IPSJW)
            CALL LCMPOF(IPSYS,'PISW$SYBIL',NCOUR*NPIS,2,IPISW)
            CALL LCMPOF(IPSYS,'PIJW$SYBIL',NPIJ,2,IPIJW)
         ENDIF
         CALL RLSARA(BASE(ISIGW2))
         CALL RLSARA(BASE(ISIGT2))
      ELSE
         CALL XABORT('SYBILA: UNKNOWN CP MODULE(2).')
      ENDIF
      IF(IMPX.GT.2) CALL LCMLIB(IPSYS)
      RETURN
      END
