*DECK SYBILF
      SUBROUTINE SYBILF (IPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUNKNO,
     1 MAT,VOL,KEYFLX,FUNKNO,SUNKNO,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one-group transport equation for fluxes using the SYBIL current
* iteration method.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the assembly matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* IDIR    not used (=0 only for SYBIL).
* NREG    total number of regions for which specific values of the
*         neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NUNKNO  total number of unknowns in vectors SUNKNO and FUNKNO.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  position of averaged flux elements in FUNKNO vector.
* SUNKNO  input source vector.
* TITR    title.
*
*Parameters: input/output
* FUNKNO  unknown vector.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   TITR*72
      INTEGER     IPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUNKNO,
     >            MAT(NREG),KEYFLX(NREG)
      REAL        VOL(NREG),FUNKNO(NUNKNO),SUNKNO(NUNKNO)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40)
      CHARACTER   NAMLCM*12,NAMMY*12
      INTEGER     ISTATE(NSTATE),IPAR(16),IBASE(1)
      LOGICAL     EMPTY,LCM,LBIHET
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      CALL LCMINF(IPSYS,NAMLCM,NAMMY,EMPTY,ILONG,LCM)
      IF(IDIR.NE.0) CALL XABORT('SYBILF: PIJK FORBIDDEN HERE')
*----
*  RECOVER SYBIL SPECIFIC PARAMETERS
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      ITG=ISTATE(6)
      LBIHET=ISTATE(40).NE.0
*
      IF(IMPX.GT.2) THEN
        WRITE(IUNOUT,'(//33H N E U T R O N    S O U R C E S :)')
        WRITE(IUNOUT,'(1P,6(5X,E15.7))') (SUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL SETARA(BASE,NUNKNO,ISUNKN)
         DO 10 I=1,NUNKNO
   10    BASE(ISUNKN+I-1)=SUNKNO(I)
         NREGAR=NREG
         NBMIXG=NBMIX
         CALL DOORFB2(IPSYS,IPTRK,IMPX,NBMIX,NREG,NUNKNO,KEYFLX,NBMIX2,
     1   NREG2,SUNKNO)
         NREG=NREG2
         NBMIX=NBMIX2
         CALL LCMSIX(IPSYS,'BIHET',1)
      ENDIF
*
      IF(ITG.EQ.1) THEN
         CALL XABORT('SYBILF: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (1).')
      ELSE IF(ITG.EQ.2) THEN
         CALL XABORT('SYBILF: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (2).')
      ELSE IF(ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         ISTAT=IPAR(3)
         CALL LCMIOF(IPTRK,'NMC',INMC3)
         CALL LCMIOF(IPTRK,'PROCEL',IPROCE)
         CALL LCMSIX(IPTRK,' ',2)
         NPIJ=0
         DO 30 IKG=1,NSUPCE
         J2=IBASE(INMC3+IKG)-IBASE(INMC3+IKG-1)
   30    NPIJ=NPIJ+J2*J2
         IF(IBASE(INMC3+NSUPCE).NE.NREG) CALL XABORT('SYBILF: ABORT.')
*
         IF(LCM) THEN
            CALL LCMIOF(IPSYS,'PIJW$SYBIL',IPIJW)
            CALL LCMIOF(IPSYS,'PISW$SYBIL',IPISW)
            CALL LCMIOF(IPSYS,'PSJW$SYBIL',IPSJW)
            CALL LCMIOF(IPSYS,'PSSW$SYBIL',IPSSW)
         ELSE
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NREG,IPISW)
            CALL SETARA(BASE,NREG,IPSJW)
            CALL SETARA(BASE,NSUPCE,IPSSW)
            CALL LCMGET(IPSYS,'PIJW$SYBIL',BASE(IPIJW))
            CALL LCMGET(IPSYS,'PISW$SYBIL',BASE(IPISW))
            CALL LCMGET(IPSYS,'PSJW$SYBIL',BASE(IPSJW))
            CALL LCMGET(IPSYS,'PSSW$SYBIL',BASE(IPSSW))
         ENDIF
*
         CALL SETARA(BASE,NSUPCE,IINDPI)
         CALL SETARA(BASE,3*NSUPCE,ICITR)
         CALL SETARA(BASE,NSUPCE,ICIT0)
         CALL SETARA(BASE,2*NSUPCE,IAITR)
         CALL SETARA(BASE,2*NSUPCE,IWCURR)
         CALL SYBJJ0 (NREG,NSUPCE,NPIJ,NUNKNO,FUNKNO,SUNKNO,IMPX,
     1   ISTAT,IBASE(INMC3),BASE(IPROCE),BASE(IPIJW),BASE(IPISW),
     2   BASE(IPSJW),BASE(IPSSW),BASE(IINDPI),BASE(ICITR),BASE(ICIT0),
     3   BASE(IAITR),BASE(IWCURR))
         CALL RLSARA(BASE(IWCURR))
         CALL RLSARA(BASE(IAITR))
         CALL RLSARA(BASE(ICIT0))
         CALL RLSARA(BASE(ICITR))
         CALL RLSARA(BASE(IINDPI))
         IF(.NOT.LCM) THEN
            CALL RLSARA(BASE(IPSSW))
            CALL RLSARA(BASE(IPSJW))
            CALL RLSARA(BASE(IPISW))
            CALL RLSARA(BASE(IPIJW))
         ENDIF
      ELSE IF(ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IHEX=IPAR(1)
         MULTC=IPAR(2)
         NMCEL=IPAR(4)
         NMERGE=IPAR(5)
         NGEN=IPAR(6)
         IJAT=IPAR(7)
         CALL LCMIOF(IPTRK,'XX',IXX4)
         CALL LCMIOF(IPTRK,'YY',IYY4)
         CALL LCMIOF(IPTRK,'NMC',INMC4)
         CALL LCMIOF(IPTRK,'IFR',IIFR)
         CALL LCMIOF(IPTRK,'ALB',IALB)
         CALL LCMIOF(IPTRK,'INUM',IINUM)
         CALL LCMIOF(IPTRK,'MIX',IMIX)
         CALL LCMIOF(IPTRK,'DVX',IDVX)
         CALL LCMIOF(IPTRK,'IGEN',IIGEN)
         CALL LCMSIX(IPTRK,' ',2)
*
         NCOUR=4
         IF(IHEX.NE.0) NCOUR=6
         NPIJ=0
         DO 40 IKG=1,NGEN
         J2=IBASE(INMC4+IKG)-IBASE(INMC4+IKG-1)
   40    NPIJ=NPIJ+J2*J2
         NPIS=IBASE(INMC4+NGEN)
*
         IF(MULTC.EQ.1) THEN
            IF(LCM) THEN
               CALL LCMIOF(IPSYS,'PIJW$SYBIL',IPIJW)
               CALL LCMIOF(IPSYS,'PISW$SYBIL',IPISW)
               CALL LCMIOF(IPSYS,'PSJW$SYBIL',IPSJW)
               CALL LCMIOF(IPSYS,'PSSW$SYBIL',IPSSW)
            ELSE
               CALL SETARA(BASE,NPIJ,IPIJW)
               CALL SETARA(BASE,NPIS,IPISW)
               CALL SETARA(BASE,NPIS,IPSJW)
               CALL SETARA(BASE,NGEN,IPSSW)
               CALL LCMGET(IPSYS,'PIJW$SYBIL',BASE(IPIJW))
               CALL LCMGET(IPSYS,'PISW$SYBIL',BASE(IPISW))
               CALL LCMGET(IPSYS,'PSJW$SYBIL',BASE(IPSJW))
               CALL LCMGET(IPSYS,'PSSW$SYBIL',BASE(IPSSW))
            ENDIF
*
            CALL SETARA(BASE,NGEN,IINDPI)
            CALL SETARA(BASE,NMERGE,IINDNM)
            CALL SETARA(BASE,3*NMERGE,ICITR)
            CALL SETARA(BASE,NMERGE,ICIT0)
            CALL SETARA(BASE,2*NMERGE,IAITR)
            CALL SETARA(BASE,2*NMERGE,IWCURR)
            CALL SYBJJ1 (NREG,NMCEL,NMERGE,NGEN,NPIJ,NPIS,NUNKNO,
     1      FUNKNO,SUNKNO,IMPX,NCOUR,BASE(IXX4),BASE(IYY4),
     2      IBASE(INMC4),IBASE(IIFR),BASE(IALB),IBASE(IINUM),
     3      IBASE(IIGEN),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     4      BASE(IPSSW),BASE(IINDPI),BASE(IINDNM),BASE(ICITR),
     5      BASE(ICIT0),BASE(IAITR),BASE(IWCURR))
            CALL RLSARA(BASE(IWCURR))
            CALL RLSARA(BASE(IAITR))
            CALL RLSARA(BASE(ICIT0))
            CALL RLSARA(BASE(ICITR))
            CALL RLSARA(BASE(IINDNM))
            CALL RLSARA(BASE(IINDPI))
         ELSE
            IF(MULTC.EQ.4) NCOUR=3*NCOUR
            IF(LCM) THEN
               CALL LCMIOF(IPSYS,'PIJW$SYBIL',IPIJW)
               CALL LCMIOF(IPSYS,'PISW$SYBIL',IPISW)
               CALL LCMIOF(IPSYS,'PSJW$SYBIL',IPSJW)
               CALL LCMIOF(IPSYS,'PSSW$SYBIL',IPSSW)
            ELSE
               CALL SETARA(BASE,NPIJ,IPIJW)
               CALL SETARA(BASE,NCOUR*NPIS,IPISW)
               CALL SETARA(BASE,NCOUR*NPIS,IPSJW)
               CALL SETARA(BASE,NCOUR*NCOUR*NGEN,IPSSW)
               CALL LCMGET(IPSYS,'PIJW$SYBIL',BASE(IPIJW))
               CALL LCMGET(IPSYS,'PISW$SYBIL',BASE(IPISW))
               CALL LCMGET(IPSYS,'PSJW$SYBIL',BASE(IPSJW))
               CALL LCMGET(IPSYS,'PSSW$SYBIL',BASE(IPSSW))
            ENDIF
*
            CALL SETARA(BASE,NGEN,IINDPI)
            CALL SETARA(BASE,NMERGE,IINDNM)
            CALL SETARA(BASE,3*IJAT,ICITR)
            CALL SETARA(BASE,IJAT,ICIT0)
            CALL SETARA(BASE,2*IJAT,IAITR)
            CALL SETARA(BASE,2*IJAT,IWCURR)
            CALL SYBJJ2 (NREG,NMCEL,NMERGE,NGEN,IJAT,NPIJ,NPIS,
     1      NUNKNO,FUNKNO,SUNKNO,IMPX,NCOUR,IBASE(INMC4),IBASE(IIFR),
     2      BASE(IALB),IBASE(IINUM),IBASE(IMIX),BASE(IDVX),
     3      IBASE(IIGEN),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     4      BASE(IPSSW),BASE(IINDPI),BASE(IINDNM),BASE(ICITR),
     5      BASE(ICIT0),BASE(IAITR),BASE(IWCURR))
            CALL RLSARA(BASE(IWCURR))
            CALL RLSARA(BASE(IAITR))
            CALL RLSARA(BASE(ICIT0))
            CALL RLSARA(BASE(ICITR))
            CALL RLSARA(BASE(IINDNM))
            CALL RLSARA(BASE(IINDPI))
         ENDIF
         IF(.NOT.LCM) THEN
            CALL RLSARA(BASE(IPSSW))
            CALL RLSARA(BASE(IPSJW))
            CALL RLSARA(BASE(IPISW))
            CALL RLSARA(BASE(IPIJW))
         ENDIF
      ELSE
         CALL XABORT('SYBILF: UNKNOWN CP MODULE(2).')
      ENDIF
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL LCMSIX(IPSYS,' ',2)
         NREG=NREGAR
         NBMIX=NBMIXG
         DO 60 I=1,NUNKNO
   60    SUNKNO(I)=BASE(ISUNKN+I-1)
         CALL RLSARA(BASE(ISUNKN))
         CALL DOORFB3(IPSYS,IPTRK,IMPX,NBMIX,NREG,NUNKNO,KEYFLX,SUNKNO,
     1   FUNKNO)
      ENDIF
*
      IF(IMPX.GT.2) THEN
        WRITE(IUNOUT,'(//33H N E U T R O N    F L U X E S   :)')
        WRITE(IUNOUT,'(1P,6(5X,E15.7))') (FUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
      RETURN
      END
