*DECK SYBILP
      SUBROUTINE SYBILP (IPTRK,IMPX,NREG,NBMIX,MAT,VOL,KNORM,SIGT0,
     1 SIGW0,NELPIJ,IPIJK,PIJ,ILK,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities for Sybil
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the tracking (L_TRACK signature).
* IMPX    print flag (equal to zero for no print).
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MAT(i))).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   not used.
* SIGT0   total macroscopic cross sections ordered by mixture.
* SIGW0   P0 within-group scattering macroscopic cross sections
*         ordered by mixture.
* NELPIJ  number of elements in pij matrix.
* IPIJK   pij option (=1 pij, =4 pijk).
* ILK     leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*Parameters: output
* PIJ     reduced and symmetrized collision probabilities.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER*72 TITR
      LOGICAL ILK
      INTEGER IPTRK,IMPX,NREG,NBMIX,MAT(NREG),KNORM,NELPIJ,IPIJK
      REAL VOL(NREG),SIGT0(0:NBMIX),SIGW0(0:NBMIX),PIJ(NELPIJ)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (EPS1=1.0E-4,NSTATE=40)
      INTEGER JPAR(NSTATE)
      COMMON BASE(1)
*----
*  RECOVER SYBIL SPECIFIC PARAMETERS
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      ITG=JPAR(6)
*
      CALL SETARA(BASE,NREG,ISIGT)
      CALL SETARA(BASE,NREG,ISIGW)
      CALL SETARA(BASE,NREG*NREG,IP)
      DO 10 I=1,NREG
      BASE(ISIGT+I-1)=SIGT0(MAT(I))
   10 BASE(ISIGW+I-1)=SIGW0(MAT(I))
      CALL SYBCP1(IPTRK,ITG,IMPX,NREG,BASE(ISIGT),BASE(ISIGW),BASE(IP))
*
      IF((IMPX.GE.10).OR.(IMPX.LT.0)) THEN
*        CHECK THE RECIPROCITY CONDITIONS.
         VOLTOT=0.0
         DO 95 I=1,NREG
   95    VOLTOT=VOLTOT+VOL(I)
         VOLTOT=VOLTOT/REAL(NREG)
         WRK=0.0
         DO 100 I=1,NREG
         DO 100 J=1,NREG
         AAA=BASE(IP+(J-1)*NREG+I-1)*VOL(I)
         BBB=BASE(IP+(I-1)*NREG+J-1)*VOL(J)
  100    WRK=MAX(WRK,ABS(AAA-BBB)/VOLTOT)
         IF(WRK.GE.EPS1) WRITE (6,150) WRK
*        CHECK THE CONSERVATION CONDITIONS.
         IF(.NOT.ILK) THEN
            WRK=0.0
            DO 120 I=1,NREG
            F1=1.0
            DO 110 J=1,NREG
            AAA=BASE(IP+(J-1)*NREG+I-1)
  110       F1=F1-AAA*(BASE(ISIGT+J-1)-BASE(ISIGW+J-1))
  120       WRK=AMAX1(WRK,ABS(F1))
            IF(WRK.GE.EPS1) WRITE (6,160) WRK
         ENDIF
      ENDIF
*
      IC=0
      DO 125 IKK=1,NREG
      IOF=(IKK-1)*NREG
      DO 125 JKK=1,IKK
      IC=IC+1
  125 PIJ(IC)=BASE(IP+IOF+JKK-1)*VOL(JKK)
      CALL RLSARA(BASE(IP))
      CALL RLSARA(BASE(ISIGT))
      CALL RLSARA(BASE(ISIGW))
      RETURN
*
  150 FORMAT (/50H THE SCATTERING-REDUCED PIJ DO NOT MEET THE RECIPR,
     1 25HOCITY CONDITIONS. RECIP =,1P,E10.3/)
  160 FORMAT (/50H THE SCATTERING-REDUCED PIJ DO NOT MEET THE CONSER,
     1 25HVATION CONDITIONS. LEAK =,1P,E10.3/)
      END
