*DECK SYBTRK
      SUBROUTINE SYBTRK (IPTRK,IPGEOM,IMPX,MAXPTS,MAXJ,MAXZ,TITLE,MULTC,
     1 IWIGN,IHALT,ILIGN,INORM,IRECT,IQW,JQUA1,JQUA2,IQUA10,IBIHET,MAT,
     2 VOL,IDL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover of the geometry and tracking for Sybil modules.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* IPTRK   pointer to the Sybil tracking (L_TRACK signature).
* IPGEOM  pointer to the geometry (L_GEOM signature).
* IMPX    print flag.
* MAXPTS  allocated storage for arrays of dimension NREG.
* MAXJ    allocated storage for interface current arrays.
* MAXZ    allocated storage for tracking arrays.
* TITLE   title.
* MULTC   type of multicell approximation in Eurydice.
* IWIGN   type of cylinderization.
* IHALT   stop flag at the end of tracking (set with IHALT=1).
* ILIGN   on/off switch for track printout.
* INORM   on/off switch for track normalization.
* IRECT   on/off switch for using symmetries in square cells.
* IQW     type of quadrature.
* JQUA1   1-D quadrature parameter.
* JQUA2   2-D quadrature parameters.
* IQUA10  quadrature parameter for micro-structures in Bihet.
* IBIHET  type of double-heterogeneity method (=1: Sanchez-Pomraning
*         model; =2: Hebert model).
*
*Parameters: scratch
* MAT
* VOL
* IDL
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IPGEOM,IMPX,MAXPTS,MAXJ,MAXZ,MULTC,IWIGN,IHALT,
     1 ILIGN,INORM,IRECT,IQW,JQUA1,JQUA2(2),IQUA10,IBIHET,MAT(MAXPTS),
     2 IDL(MAXPTS)
      REAL VOL(MAXPTS)
      CHARACTER TITLE*72
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PI=3.141592654,NSTATE=40)
      LOGICAL ILK,LBIHET
      INTEGER ISTATE(NSTATE),IQUAD(4),IGP(NSTATE),IPARAM(16),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      LBIHET=(ISTATE(12).EQ.1)
      IF(ISTATE(1).EQ.1) THEN
         ITG=1
         IF(ISTATE(6).NE.1) CALL XABORT('SYBTRK: INVALID NUMBER OF REGI'
     1   //'ONS.')
         CALL LCMLEN(IPGEOM,'MIX',NMILG,ITYLCM)
         IF(NMILG.NE.1) CALL XABORT('SYBTRK: INVALID MIX VECTOR.')
         CALL LCMGET(IPGEOM,'MIX',MAT(1))
         IR=MAT(1)
         VOL(1)=1.0
         ILK=.FALSE.
         NREG=1
      ELSE IF((ISTATE(1).GT.1).AND.(ISTATE(1).LT.5).AND.(ISTATE(9).EQ.
     1 0)) THEN
         ITG=2
         CALL SETARA(BASE,MAXPTS+1,IXX2)
         CALL SETARA(BASE,MAXPTS+1,IYY2)
         CALL SETARA(BASE,MAXPTS+1,IZZ2)
         CALL SETARA(IBASE,6,INCODE)
         CALL SETARA(BASE,6,IZCODE)
*
         CALL SETARA(BASE,MAXPTS,IISPLX)
         CALL SETARA(BASE,MAXPTS,IISPLY)
         CALL SETARA(BASE,MAXPTS,IISPLZ)
         CALL READ3D(MAXPTS,MAXPTS,MAXPTS,MAXPTS,IPGEOM,IHEX,IR,MAT,
     1   ILK,SIDE,BASE(IXX2),BASE(IYY2),BASE(IZZ2),IMPX,LX,LY,LZ,NREG,
     2   IBASE(INCODE),BASE(IZCODE),BASE(IISPLX),BASE(IISPLY),
     3   BASE(IISPLZ))
         CALL RLSARA(BASE(IISPLZ))
         CALL RLSARA(BASE(IISPLY))
         CALL RLSARA(BASE(IISPLX))
         DO 10 IC=1,6
         IF(IBASE(INCODE+IC-1).EQ.7) CALL XABORT('SYBTRK: ZERO FLUX BOU'
     1   //'NDARY CONDITION NOT PERMITTED.')
   10    CONTINUE
*
*        COMPUTATION OF THE VOLUMES.
         DO 20 IKK=1,NREG
         A=BASE(IXX2+IKK-1)
         B=BASE(IXX2+IKK)
         IF(ISTATE(1).EQ.2) THEN
            VOL(IKK)=(B-A)
         ELSE IF(ISTATE(1).EQ.3) THEN
            VOL(IKK)=PI*(B-A)*(B+A)
         ELSE IF(ISTATE(1).EQ.4) THEN
            VOL(IKK)=4.0*PI*(B-A)*(A*A+A*B+B*B)/3.0
         ENDIF
   20    CONTINUE
         IF(IMPX.GE.1) WRITE (6,'(/29H QUADRATURE PARAMETER JQUA1 =,
     1   I2/)') JQUA1
*
         IPARAM(1)=ISTATE(1)
         IPARAM(2)=IHEX
         IPARAM(3)=JQUA1
         IPARAM(4)=LX
         IPARAM(5)=LY
         IPARAM(6)=LZ
         CALL LCMSIX(IPTRK,'PURE-GEOM',1)
         CALL LCMPUT(IPTRK,'PARAM',6,1,IPARAM)
         CALL LCMPUT(IPTRK,'XXX',LX+1,2,BASE(IXX2))
         CALL LCMPUT(IPTRK,'YYY',LY+1,2,BASE(IYY2))
         CALL LCMPUT(IPTRK,'ZZZ',LZ+1,2,BASE(IZZ2))
         CALL LCMPOF(IPTRK,'NCODE',6,1,INCODE)
         CALL LCMPOF(IPTRK,'ZCODE',6,2,IZCODE)
         IF(ISTATE(1).GE.8) CALL LCMPUT(IPTRK,'SIDE',1,2,SIDE)
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(IZZ2))
         CALL RLSARA(BASE(IYY2))
         CALL RLSARA(BASE(IXX2))
      ELSE IF(ISTATE(1).EQ.30) THEN
         ITG=3
         CALL SETARA(IBASE,1+MAXPTS,INMC3)
         CALL SETARA(BASE,2*MAXPTS,IRAYR3)
         CALL SETARA(BASE,MAXPTS**2,IPROCE)
         CALL SETARA(BASE,MAXPTS,IPOURC)
         CALL SETARA(BASE,MAXPTS,ISURFA)
*
         CALL SETARA(BASE,MAXPTS,INBREG)
         CALL SETARA(BASE,MAXPTS,IMILIE)
         CALL SETARA(BASE,MAXPTS,IRAYZO)
         CALL SETARA(BASE,3*MAXPTS,ICELL)
         CALL SETARA(BASE,MAXPTS,IIGEN)
         CALL READMT (MAXPTS,IPGEOM,IR,MAT,VOL,ILK,ISTAT,NSUPCE,
     1   NREG,IBASE(INMC3),BASE(IRAYR3),BASE(IPROCE),BASE(IPOURC),
     2   BASE(ISURFA),IMPX,BASE(INBREG),BASE(IMILIE),BASE(IRAYZO),
     3   BASE(ICELL),BASE(IIGEN))
         CALL RLSARA(BASE(IIGEN))
         CALL RLSARA(BASE(ICELL))
         CALL RLSARA(BASE(IRAYZO))
         CALL RLSARA(BASE(IMILIE))
         CALL RLSARA(BASE(INBREG))
         IF(IMPX.GE.1) WRITE (6,'(/29H QUADRATURE PARAMETER JQUA1 =,
     1   I2/)') JQUA1
*
         IPARAM(1)=NSUPCE
         IPARAM(2)=JQUA1
         IPARAM(3)=ISTAT
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMPUT(IPTRK,'PARAM',3,1,IPARAM)
         CALL LCMPUT(IPTRK,'NMC',1+NSUPCE,1,IBASE(INMC3))
         CALL LCMPUT(IPTRK,'RAYRE',NREG+NSUPCE,2,BASE(IRAYR3))
         CALL LCMPUT(IPTRK,'PROCEL',NSUPCE**2,2,BASE(IPROCE))
         CALL LCMPUT(IPTRK,'POURCE',NSUPCE,2,BASE(IPOURC))
         CALL LCMPUT(IPTRK,'SURFA',NSUPCE,2,BASE(ISURFA))
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(BASE(ISURFA))
         CALL RLSARA(BASE(IPOURC))
         CALL RLSARA(BASE(IPROCE))
         CALL RLSARA(BASE(IRAYR3))
         CALL RLSARA(IBASE(INMC3))
      ELSE IF( (ISTATE(1).EQ.5).OR.(ISTATE(1).EQ.8) .OR.
     1         ((ISTATE(1).EQ.20).AND.(ISTATE(13).EQ.0)) .OR.
     2         ((ISTATE(1).EQ.24).AND.(ISTATE(13).EQ.0)) ) THEN
         ITG=4
         MAXCEL=MAXPTS
         CALL SETARA(BASE,MAXCEL,IXX4)
         CALL SETARA(BASE,MAXCEL,IYY4)
         CALL SETARA(IBASE,MAXCEL,ILSECT)
         CALL SETARA(IBASE,MAXCEL+1,INMC4)
         CALL SETARA(IBASE,MAXCEL+1,INMCR4)
         CALL SETARA(BASE,MAXPTS,IRAYR4)
         CALL SETARA(IBASE,2*MAXCEL,IMAIL)
         CALL SETARA(IBASE,MAXZ,IIZMAI)
         CALL SETARA(BASE,MAXZ,IRZMAI)
         CALL SETARA(IBASE,MAXJ,IIFR)
         CALL SETARA(BASE,MAXJ,IALB)
         CALL SETARA(IBASE,MAXCEL,IINUM)
         CALL SETARA(IBASE,MAXJ,IMIX)
         CALL SETARA(BASE,MAXJ,IDVX)
         CALL SETARA(IBASE,MAXCEL,IIGEN)
         IQUAD(1)=JQUA2(1)
         IQUAD(2)=JQUA2(2)
         IQUAD(3)=JQUA1
         IQUAD(4)=JQUA1
*
         CALL SETARA(BASE,MAXCEL,IPOURC)
         CALL SETARA(BASE,MAXCEL,IIORI)
         CALL SYBEUR(MAXPTS,MAXCEL,MAXJ,MAXZ,IPGEOM,NREG,IR,MAT,VOL,
     1   ILK,IMPX,IHEX,NCOUR,LMAILI,LMAILR,NMCEL,NMERGE,NGEN,IJAT,MULTC,
     2   IWIGN,IHALT,ILIGN,INORM,IRECT,IQW,IQUAD,BASE(IXX4),BASE(IYY4),
     3   IBASE(ILSECT),IBASE(INMC4),IBASE(INMCR4),BASE(IRAYR4),
     4   IBASE(IMAIL),IBASE(IIZMAI),BASE(IRZMAI),IBASE(IIFR),BASE(IALB),
     5   IBASE(IINUM),IBASE(IMIX),BASE(IDVX),IBASE(IIGEN),BASE(IPOURC),
     6   BASE(IIORI))
         CALL RLSARA(BASE(IIORI))
         CALL RLSARA(BASE(IPOURC))
*
         IPARAM(1)=IHEX
         IPARAM(2)=MULTC
         IPARAM(3)=IWIGN
         IPARAM(4)=NMCEL
         IPARAM(5)=NMERGE
         IPARAM(6)=NGEN
         IPARAM(7)=IJAT
         IPARAM(8)=IQUAD(1)
         IPARAM(9)=IQUAD(2)
         IPARAM(10)=IQUAD(3)
         IPARAM(11)=IQUAD(4)
         IPARAM(12)=INORM
         IPARAM(13)=IQW
         IPARAM(14)=NCOUR
         IPARAM(15)=LMAILI
         IPARAM(16)=LMAILR
         IRDIM=IBASE(INMCR4+NGEN)
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMPUT(IPTRK,'PARAM',16,1,IPARAM)
         CALL LCMPUT(IPTRK,'XX',NGEN,2,BASE(IXX4))
         CALL LCMPUT(IPTRK,'YY',NGEN,2,BASE(IYY4))
         CALL LCMPUT(IPTRK,'LSECT',NGEN,1,IBASE(ILSECT))
         CALL LCMPUT(IPTRK,'NMC',1+NGEN,1,IBASE(INMC4))
         CALL LCMPUT(IPTRK,'NMCR',1+NGEN,1,IBASE(INMCR4))
         CALL LCMPUT(IPTRK,'RAYRE',IRDIM,2,BASE(IRAYR4))
         CALL LCMPUT(IPTRK,'MAIL',2*NGEN,1,IBASE(IMAIL))
         IF(LMAILI.GT.0) THEN
            CALL LCMPUT(IPTRK,'ZMAILI',LMAILI,1,IBASE(IIZMAI))
         ENDIF
         IF(LMAILR.GT.0) THEN
            CALL LCMPUT(IPTRK,'ZMAILR',LMAILR,2,BASE(IRZMAI))
         ENDIF
         CALL LCMPUT(IPTRK,'IFR',NCOUR*NMCEL,1,IBASE(IIFR))
         CALL LCMPUT(IPTRK,'ALB',NCOUR*NMCEL,2,BASE(IALB))
         CALL LCMPUT(IPTRK,'INUM',NMCEL,1,IBASE(IINUM))
         CALL LCMPUT(IPTRK,'MIX',NCOUR*NMERGE,1,IBASE(IMIX))
         CALL LCMPUT(IPTRK,'DVX',NCOUR*NMERGE,2,BASE(IDVX))
         CALL LCMPUT(IPTRK,'IGEN',NMERGE,1,IBASE(IIGEN))
         CALL LCMSIX(IPTRK,' ',2)
         CALL RLSARA(IBASE(IIGEN))
         CALL RLSARA(BASE(IDVX))
         CALL RLSARA(IBASE(IMIX))
         CALL RLSARA(IBASE(IINUM))
         CALL RLSARA(BASE(IALB))
         CALL RLSARA(IBASE(IIFR))
         CALL RLSARA(BASE(IRZMAI))
         CALL RLSARA(IBASE(IIZMAI))
         CALL RLSARA(IBASE(IMAIL))
         CALL RLSARA(BASE(IRAYR4))
         CALL RLSARA(IBASE(INMCR4))
         CALL RLSARA(IBASE(INMC4))
         CALL RLSARA(IBASE(ILSECT))
         CALL RLSARA(BASE(IYY4))
         CALL RLSARA(BASE(IXX4))
      ELSE
         CALL XABORT('SYBTRK: INVALID GEOMETRY MODULE.')
      ENDIF
*----
*  SAVE GENERAL AND SYBIL-SPECIFIC TRACKING INFORMATION
*----
      DO 30 I=1,NREG
   30 IDL(I)=I
      IF(ITG.EQ.3) THEN
         NUNCUR=NSUPCE
      ELSE IF(ITG.EQ.4) THEN
         NUNCUR=IJAT
      ELSE
         NUNCUR=0
      ENDIF
      CALL XDISET(IGP,NSTATE,0)
      IGP(1)=NREG
      IGP(2)=NREG
      IF(ILK) THEN
         IGP(3)=0
      ELSE
         IGP(3)=1
      ENDIF
      IGP(4)=IR
      IGP(5)=0
      IGP(6)=ITG
      IGP(7)=MAXZ
      IGP(8)=MAXJ
      IGP(9)=NUNCUR
      IF(LBIHET) IGP(40)=1
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,IGP)
      CALL LCMPUT(IPTRK,'MATCOD',NREG,1,MAT)
      CALL LCMPUT(IPTRK,'VOLUME',NREG,2,VOL)
      CALL LCMPUT(IPTRK,'KEYFLX',NREG,1,IDL)
*----
*  DOUBLE HETEROGENEITY OPTION
*----
      IF(LBIHET) CALL XDRTBH(IPGEOM,IPTRK,IQUA10,IBIHET,IMPX)
*----
*  PRINT TRACKING ARRAYS
*----
      IF(IMPX.GT.5) THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
         NREG=IGP(1)
         CALL LCMGET(IPTRK,'MATCOD',MAT)
         CALL LCMGET(IPTRK,'VOLUME',VOL)
         CALL LCMGET(IPTRK,'KEYFLX',IDL)
         I1=1
         DO 60 I=1,(NREG-1)/8+1
         I2=I1+7
         IF(I2.GT.NREG) I2=NREG
         WRITE (6,620) (J,J=I1,I2)
         WRITE (6,630) (MAT(J),J=I1,I2)
         WRITE (6,640) (IDL(J),J=I1,I2)
         WRITE (6,650) (VOL(J),J=I1,I2)
   60    I1=I1+8
      ENDIF
*----
*  SAVE GENERAL AND SYBIL-SPECIFIC TRACKING INFORMATION
*----
      RETURN
*
  620 FORMAT (///11H REGION    ,8(I8,6X,1HI))
  630 FORMAT (   11H MIXTURE   ,8(I8,6X,1HI))
  640 FORMAT (   11H POINTER   ,8(I8,6X,1HI))
  650 FORMAT (   11H VOLUME    ,8(1P,E13.6,2H I))
      END
