*DECK SYBUP0
      SUBROUTINE SYBUP0(ZZR,ZZI,NSURF,NREG,SIGT,TRONC,A,B,IMPX,VOL,PIJ,
     1 PIS,PSS,LGFULL,G)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the one-group collision, DP-0 leakage and DP-0 transmission
* probabilities in a Cartesian or hexagonal non-sectorized cell.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* ZZR     real tracking elements.
* ZZI     integer tracking elements.
* NSURF   number of surfaces (4 or 6).
* NREG    number of regions.
* SIGT    total macroscopic cross section.
* TRONC   voided block criterion.
* A       Cartesian dimension of the cell along the X axis or side of
*         the hexagon.
* B       Cartesian dimension of the cell along the Y axis.
* IMPX    print flag.
*
*Parameters: output
* VOL     volumes.
* PIJ     cellwise reduced collision probability matrices.
* PIS     cellwise reduced escape probability matrices.
* PSS     cellwise reduced transmission probability matrices.
*         PSS(i,j) is the probability from surface i to surface j.
*
*Parameters: scratch
* LGFULL
* G
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   ZZI(*),NSURF,NREG,IMPX
      REAL      ZZR(*),SIGT(NREG),TRONC,A,B,VOL(NREG),PIJ(NREG,NREG),
     1          PIS(NSURF,NREG),PSS(NSURF,NSURF),G(NREG+6)
      LOGICAL   LGFULL(NREG)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (SIGVID=1.0E-10)
      COMMON    BASE(1)
      REAL      SURF(6)
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
*----
*  CHECK FOR VOIDED REGIONS
*----
      DO 10 IR=1,NREG
      VOL(IR)=ZZR(IR)
      IF(VOL(IR).GT.0.) THEN
         DR=SQRT(VOL(IR))
      ELSE
         DR=0.0
      ENDIF
      LGFULL(IR)=(SIGT(IR)*DR).GT.TRONC
      IF(SIGT(IR).LE.SIGVID) SIGT(IR)=SIGVID
   10 CONTINUE
*----
*  COMPUTE SYMMETRIZED CP MATRIX
*----
      IOFI=ZZI(1)
      IOFR=ZZI(2)
      ICARE=ZZI(3)
      MNA=ZZI(4)
      CALL SETARA(BASE,(NREG+NSURF)*(NREG+NSURF+1)/2,IPIJS)
      CALL SETARA(BASE,2*NREG+4,IPOPTX)
      CALL SYBUQV(ZZR(IOFR),ZZI(IOFI),NSURF,NREG,SIGT,MNA,LGFULL,
     1 BASE(IPIJS),BASE(IPOPTX))
      CALL RLSARA(BASE(IPOPTX))
*----
*  APPLY SYMMETRIES
*----
      IF(NSURF.EQ.4) THEN
         SURF(1)=0.25*B
         SURF(2)=0.25*B
         SURF(3)=0.25*A
         SURF(4)=0.25*A
      ELSE
         DO 20 JC=1,6
   20    SURF(JC)=0.25*A
      ENDIF
      DO 30 I=1,NSURF
   30 G(I)=SURF(I)
      IF(ICARE.EQ.1) THEN
*        RECTANGULAR CELL.
         BASE(IPIJS+2-1)=2.0*BASE(IPIJS+2-1)
         BASE(IPIJS+5-1)=0.5*BASE(IPIJS+5-1)
         BASE(IPIJS+9-1)=2.0*BASE(IPIJS+9-1)
         BASE(IPIJS+4-1)=BASE(IPIJS+5-1)
         BASE(IPIJS+7-1)=BASE(IPIJS+5-1)
         BASE(IPIJS+8-1)=BASE(IPIJS+5-1)
         IOF=11
         DO 50 I=1,NREG
         G(4+I)=SIGT(I)*VOL(I)
         SUM1=BASE(IPIJS+IOF-1)+BASE(IPIJS+IOF)
         SUM2=BASE(IPIJS+IOF+1)+BASE(IPIJS+IOF+2)
         BASE(IPIJS+IOF-1)=SUM1
         BASE(IPIJS+IOF)=SUM1
         BASE(IPIJS+IOF+1)=SUM2
         BASE(IPIJS+IOF+2)=SUM2
         DO 40 J=4,3+I
   40    BASE(IPIJS+IOF+J-1)=2.0*BASE(IPIJS+IOF+J-1)
   50    IOF=IOF+4+I
      ELSE IF(ICARE.EQ.2) THEN
*        SQUARE CELL.
         BASE(IPIJS+9-1)=2.0*BASE(IPIJS+9-1)
         BASE(IPIJS+2-1)=BASE(IPIJS+9-1)
         BASE(IPIJS+4-1)=BASE(IPIJS+5-1)
         BASE(IPIJS+7-1)=BASE(IPIJS+5-1)
         BASE(IPIJS+8-1)=BASE(IPIJS+5-1)
         IOF=11
         DO 80 I=1,NREG
         G(4+I)=SIGT(I)*VOL(I)
         SUM=BASE(IPIJS+IOF-1)+BASE(IPIJS+IOF)+BASE(IPIJS+IOF+1)+
     1   BASE(IPIJS+IOF+2)
         DO 60 J=0,3
   60    BASE(IPIJS+IOF+J-1)=SUM
         DO 70 J=4,3+I
   70    BASE(IPIJS+IOF+J-1)=4.0*BASE(IPIJS+IOF+J-1)
   80    IOF=IOF+4+I
      ELSE IF(ICARE.EQ.3) THEN
*        HEXAGONAL CELL.
         BASE(IPIJS+12-1)=2.0*BASE(IPIJS+12-1)
         BASE(IPIJS+7-1)=BASE(IPIJS+12-1)
         BASE(IPIJS+18-1)=BASE(IPIJS+12-1)
         BASE(IPIJS+2-1)=BASE(IPIJS+20-1)
         BASE(IPIJS+5-1)=BASE(IPIJS+20-1)
         BASE(IPIJS+9-1)=BASE(IPIJS+20-1)
         BASE(IPIJS+14-1)=BASE(IPIJS+20-1)
         BASE(IPIJS+16-1)=BASE(IPIJS+20-1)
         BASE(IPIJS+4-1)=BASE(IPIJS+11-1)
         BASE(IPIJS+8-1)=BASE(IPIJS+11-1)
         BASE(IPIJS+13-1)=BASE(IPIJS+11-1)
         BASE(IPIJS+17-1)=BASE(IPIJS+11-1)
         BASE(IPIJS+19-1)=BASE(IPIJS+11-1)
         IOF=22
         DO 120 I=1,NREG
         G(6+I)=SIGT(I)*VOL(I)
         SUM=0.0
         DO 90 J=0,5
   90    SUM=SUM+BASE(IPIJS+IOF+J-1)
         DO 100 J=0,5
  100    BASE(IPIJS+IOF+J-1)=SUM
         DO 110 J=6,5+I
  110    BASE(IPIJS+IOF+J-1)=6.0*BASE(IPIJS+IOF+J-1)
  120    IOF=IOF+6+I
      ENDIF
*----
*  FIRST APPLY THE ORTHONORMALIZATION FACTOR
*----
      DO 125 I=0,(NSURF+NREG)*(NSURF+NREG+1)/2-1
  125 BASE(IPIJS+I)=BASE(IPIJS+I)*ZZR(IOFR)*ZZR(IOFR)
*----
*  VILLARINO-STAMM'LER NORMALIZATION
*----
      CALL SETARA(BASE,3*(NSURF+NREG),IWEIH)
      CALL SYBRHL(IMPX,NSURF,NREG,G,BASE(IPIJS),BASE(IWEIH))
      CALL RLSARA(BASE(IWEIH))
*----
*  REFORMAT PIJ, PIS AND PSS MATRICES
*----
      DO 130 I=1,NSURF
      DO 130 J=1,NSURF
  130 PSS(I,J)=BASE(IPIJS+INDPOS(I,J)-1)/SURF(I)
      DO 140 I=1,NSURF
      DO 140 J=1,NREG
  140 PIS(I,J)=BASE(IPIJS+INDPOS(I,NSURF+J)-1)/VOL(J)/SIGT(J)
      DO 150 I=1,NREG
      DO 150 J=1,NREG
  150 PIJ(I,J)=BASE(IPIJS+INDPOS(NSURF+I,NSURF+J)-1)/(VOL(I)*SIGT(I)*
     1 SIGT(J))
      CALL RLSARA(BASE(IPIJS))
      RETURN
      END
