*DECK TLMPLA
      SUBROUTINE TLMPLA(IPMAT ,IFTRK ,IPRINT,NSKTRK,NBTR  ,NDIM  ,
     >                  NREG  ,NSUR  ,MXSEG ,NANGL ,NBDR  ,
     >                  IPLP  ,DPLP  ,XYZL  ,DANGLT,DVNOR ,
     >                  NUMERO,LENGTH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To generate the Matlab instruction for drawing the
* intersections between the lines and an arbitrary plane
* in 3 D or a line in 2D.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal.
*
*Author(s): 
* C. Plamondon, G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
* None.
*
*Parameters: input
* IPMAT   pointer to Matlab -m file. 
* IFTRK   pointer to the TRACKING file. 
* IPRINT  print level.
* NSKTRK  number of records to skip on track file before tracking
*         lines can be extracted.
* NBTR    numbre of tracks.
* NDIM    number of dimensions for problem.
* NREG    number of regions for problem.
* NSUR    number of outer surfaces for problem.
* MAXSEG  maximum number of segments in a line.
* NANGL   number of direction for tracking.
* NBDR    number of direction for volume normalization.
* IPLP    integer plot parameters.
* DPLP    real plot parameters.
* XYLZ    mesh limits.
* DANGLT  track directions.
* DVNOR   track normalization factor for regional volumes.
*
*Parameters: temporary storage
* NUMERO  region/surface identification number
*         for segment.
* LENGTH  segment length.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPMAT,IFTRK
      INTEGER          IPRINT,NSKTRK,NBTR,NDIM,NREG,NSUR,MXSEG,
     >                 NANGL,NBDR,IPLP(6)
      DOUBLE PRECISION DPLP(4),XYZL(2,3),DANGLT(NDIM,NANGL),
     >                 DVNOR(NREG,NBDR)
      INTEGER          NUMERO(MXSEG)
      REAL             LENGTH(MXSEG)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='TLMPLA')
      DOUBLE PRECISION DZERO,DONE
      PARAMETER       (DZERO=0.0D0,DONE=1.0D0)
*----
*  Local functions
*----
      INTEGER          TLMVPL
*----
*  Local variables for tracking file
*----
      INTEGER          ILINE,IDUM,IANGL,NBSEG,NTLINE,ISEG,
     >                 IPLANE,IPTA2,IPTA3
      REAL             WEIGHT
      DOUBLE PRECISION TORIG(3),DXYZ(3),TSTRT
*----
*  Other local variables 
*----

      INTEGER          IPM,IDIR,IVALID,IREG,ILREG,ISV ,ITRACE  
      CHARACTER        TITLE*66        
      DOUBLE PRECISION NUMER,DENOM,FLEN,DINT,DINP(2),DPLPT(4)
*----
*  Data
*----
      CHARACTER        ACOL(0:6)*2
      DATA             ACOL /'b.','g.','r.','c.','m.','y.','k.'/
*----
*  Processing starts:
*  print routine opening header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
*----
* Print IPMAT header
*----
      DO IDIR=1,4 
        DPLPT(IDIR)=DPLP(IDIR)  
      ENDDO
      IF(NDIM .EQ. 2) THEN
        WRITE(TITLE,'(A22,4(1X,A2,F8.2))') 
     >  'Lines crossing plane= ',
     >  'A=',DPLPT(1),'B=',DPLPT(2),'D=',DPLPT(4)    
        WRITE(IPMAT,7000) NAMSBR,TITLE
        WRITE(IPMAT,7020) ((XYZL(IPM,IDIR),IPM=1,2),IDIR=1,NDIM)
        WRITE(IPMAT,7021)
      ELSE IF(NDIM .EQ. 3) THEN
        WRITE(TITLE,'(A22,4(1X,A2,F8.2))') 
     >  'Lines crossing plane= ',
     >  'A=',DPLPT(1),'B=',DPLPT(2),'C=',DPLPT(3),'D=',DPLPT(4)    
        WRITE(IPMAT,7000) NAMSBR,TITLE
        WRITE(IPMAT,7030) ((XYZL(IPM,IDIR),IPM=1,2),IDIR=1,NDIM)
        WRITE(IPMAT,7031) 
      ENDIF
*----
*  Test if plane is valid
*----
      IVALID=TLMVPL(NDIM,DPLPT,XYZL)
      IF(IVALID . LT. 0) RETURN
*----
*  Print matlab instructions for line segment
*----
      DO IREG=1,NREG
        IF(IPRINT .GE. 10) THEN
          WRITE(IOUT,6002) IREG
        ENDIF
        DO ILINE=1,NSKTRK
          READ(IFTRK) IDUM
        ENDDO
        ILREG=0
*----
*  Scan over lines
*----
        DO ILINE=1,NBTR
          READ(IFTRK) IANGL,NBSEG,WEIGHT, 
     >                (NUMERO(ISEG),ISEG=1,NBSEG),
     >                (LENGTH(ISEG),ISEG=1,NBSEG),
     >                NTLINE,IPLANE,IPTA2,IPTA3,
     >                (TORIG(IDIR),IDIR=1,NDIM),TSTRT
*----
*  find location of first intersection between line and geometry
*---- 
          DO IDIR=1,NDIM
            DXYZ(IDIR)=TORIG(IDIR)+DANGLT(IDIR,IANGL)*TSTRT
          ENDDO
*----
*  For surface A*X+B*Y+C*Z=D 
*  and line
*  X=X0+R*DANGLT(1)
*  Y=Y0+R*DANGLT(2)
*  Y=Z0+R*DANGLT(3)
*  Intersection must satisfy:
*  A*R*DANGLT(1)+B*R*DANGLT(2)+C*R*DANGLT(3)=D
*  R=(D-A*X0-B*Y0-C*Z0)/(A*DANGLT(1)+B*DANGLT(2)+C*DANGLT(3))
*----  
          NUMER=DPLPT(4)
          DENOM=DZERO
          DO IDIR=1,NDIM
            DENOM=DENOM+DPLPT(IDIR)*DANGLT(IDIR,IANGL)
            NUMER=NUMER-DPLPT(IDIR)*DXYZ(IDIR)
          ENDDO
          IF(DENOM .NE. DZERO) THEN
*----
*  DINT=+/- R
*----
            DINT=NUMER/DENOM
            DINP(1)=DZERO
            DO ISEG=1,NBSEG
              ISV=NUMERO(ISEG)
              IF(ISV .GT. 0) THEN
                FLEN=LENGTH(ISEG)/DVNOR(ISV,1)
                IF(NBDR .GT. 1) FLEN=LENGTH(ISEG)/DVNOR(ISV,IANGL+1)
                DINP(2)=DINP(1)+FLEN
                ITRACE=0
                IF(DINP(1) .LE. DINT .AND. DINT .LE. DINP(2)) THEN
                  DO IDIR=1,NDIM
                    DXYZ(IDIR)=DXYZ(IDIR)+DANGLT(IDIR,IANGL)*DINT
                  ENDDO
                  ITRACE=1
                ENDIF
                IF(ITRACE .EQ. 1) THEN
                  IF(ISV .EQ. IREG) THEN
                    ILREG=ILREG+1
                    IF(ILREG .EQ. 1) THEN
                      WRITE(IPMAT,7002)
                    ENDIF
                      if(IREG.eq.6 .and. DXYZ(2).gt.2.9) 
     >                write(6,*) 'Line ',ILINE,IREG,ISEG,
     >                (DXYZ(IDIR),IDIR=1,NDIM)
                    WRITE(IPMAT,7004)
     >                (DXYZ(IDIR),IDIR=1,NDIM),ILINE
                  ENDIF
                  GO TO 100
                ENDIF
                DINP(1)=DINP(2)
              ENDIF
            ENDDO
 100        CONTINUE
          ENDIF  
        ENDDO
*----
*  Write Matlab commands to print points
*----
        IF(ILREG .GE. 1) THEN
          WRITE(IPMAT,7003)
          IF(NDIM .EQ. 2) THEN
            WRITE(IPMAT,7010) ACOL(MOD(IREG,7))
          ELSE
            WRITE(IPMAT,7011) ACOL(MOD(IREG,7))
          ENDIF
          WRITE(IPMAT,7090)
          IF(IPLP(1) .GT. 0) WRITE(IPMAT,7091)
        ENDIF
        REWIND IFTRK
      ENDDO
      WRITE(IPMAT,7093)
      IF(IPLP(1) .LT. 0) WRITE(IPMAT,7091)
*----
*  Processing finished, return
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6002 FORMAT(' Processing lines for region = ',I8)
*----
*  Matlab .m file format
*----
 7000 FORMAT('%'/'% Output from ',A6/'%'/
     >'figure;'/'hold on;'/7Htitle(',A66,3H');/
     >12Hxlabel('x');/12Hylabel('y');)
 7001 FORMAT(12Hzlabel('z');)
 7002 FORMAT('TLMSurfacePoints=[')
 7003 FORMAT(12X,'];')
 7004 FORMAT(3F18.10,2X,I10)
 7010 FORMAT('plot(TLMSurfacePoints(:,1),',
     >            'TLMSurfacePoints(:,2),',1H',A2,3H');)
 7011 FORMAT('plot3(TLMSurfacePoints(:,1),',
     >             'TLMSurfacePoints(:,2),',
     >             'TLMSurfacePoints(:,3),',1H',A2,3H');)
 7020 FORMAT('xmin=',F18.10,';'/'xmax=',F18.10,';'/
     >       'ymin=',F18.10,';'/'ymax=',F18.10,';'/)
 7021 FORMAT('line([xmin,xmin],[ymin,ymax]);'/
     >       'line([xmin,xmax],[ymin,ymin]);'/
     >       'line([xmax,xmax],[ymin,ymax]);'/
     >       'line([xmin,xmax],[ymax,ymax]);')
 7030 FORMAT('xmin=',F18.10,';'/'xmax=',F18.10,';'/
     >       'ymin=',F18.10,';'/'ymax=',F18.10,';'/
     >       'zmin=',F18.10,';'/'zmax=',F18.10,';'/)
 7031 FORMAT('line([xmin,xmin],[ymin,ymax],[zmin,zmin]);'/
     >       'line([xmin,xmax],[ymin,ymin],[zmin,zmin]);'/
     >       'line([xmax,xmax],[ymin,ymax],[zmin,zmin]);'/
     >       'line([xmin,xmax],[ymax,ymax],[zmin,zmin]);'/    
     >       'line([xmin,xmin],[ymin,ymin],[zmin,zmax]);'/
     >       'line([xmax,xmax],[ymin,ymin],[zmin,zmax]);'/
     >       'line([xmax,xmax],[ymax,ymax],[zmin,zmax]);'/
     >       'line([xmin,xmin],[ymax,ymax],[zmin,zmax]);'/    
     >       'line([xmin,xmin],[ymin,ymax],[zmax,zmax]);'/
     >       'line([xmin,xmax],[ymin,ymin],[zmax,zmax]);'/
     >       'line([xmax,xmax],[ymin,ymax],[zmax,zmax]);'/
     >       'line([xmin,xmax],[ymax,ymax],[zmax,zmax]);')
 7090 FORMAT('clear TLMSurfacePoints;')
 7091 FORMAT('pause ;')
 7093 FORMAT('hold off;')
      END
