*DECK TLMPNT
      SUBROUTINE TLMPNT(IPMAT ,IFTRK ,IPRINT,NSKTRK,NBTR  ,NDIM  ,
     >                  NREG  ,NSUR  ,MXSEG ,NANGL ,NBDR  ,
     >                  IPLP  ,DPLP  ,XYZL  ,DANGLT,DVNOR ,
     >                  NUMERO,LENGTH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* To generate the Matlab instruction for drawing the
* external surface intersection points.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal.
*
*Author(s): 
* C. Plamondon, G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
* None.
*
*Parameters: input
* IPMAT   pointer to Matlab -m file. 
* IFTRK   pointer to the TRACKING file. 
* IPRINT  print level.
* NSKTRK  number of records to skip on track file before tracking
*         lines can be extracted.
* NBTR    numbre of tracks.
* NDIM    number of dimensions for problem.
* NREG    number of regions for problem.
* NSUR    number of outer surfaces for problem.
* MAXSEG  maximum number of segments in a line.
* NANGL   number of direction for tracking.
* NBDR    number of direction for volume normalization.
* IPLP    integer plot parameters.
* DPLP    real plot parameters.
* XYLZ    mesh limits.
* DANGLT  track directions.
* DVNOR   track normalization factor for regional volumes.
*
*Parameters: temporary storage
* NUMERO  region/surface identification number
*         for segment.
* LENGTH  segment length.
*
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPMAT,IFTRK
      INTEGER          IPRINT,NSKTRK,NBTR,NDIM,NREG,NSUR,MXSEG,
     >                 NANGL,NBDR,IPLP(6)
      DOUBLE PRECISION DPLP(4),XYZL(2,3),DANGLT(NDIM,NANGL),
     >                 DVNOR(NREG,NBDR)
      INTEGER          NUMERO(MXSEG)
      REAL             LENGTH(MXSEG)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='TLMPNT')
*----
*  Local variables for tracking file
*----
      INTEGER          ILINE,IDUM,IANGL,NBSEG,NTLINE,ISEG,
     >                 IPLANE,IPTA2,IPTA3
      REAL             WEIGHT
      DOUBLE PRECISION TORIG(3),TSTRT
*----
*  Other local variables
*----
      INTEGER          IPM,ISUR,IFACE,IDIR,ISV
      DOUBLE PRECISION DXYZ(3),FLEN         
      CHARACTER        TITLE*36        
*----
*  Data
*----
      CHARACTER        ACOL(0:6)*2
      DATA             ACOL /'b.','g.','r.','c.','m.','y.','k.'/
*----
*  Processing starts:
*  print routine opening header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
*----
* Print IPMAT header
*----
      WRITE(TITLE,'(A18,18X)') 'Points on surfaces'      
      WRITE(IPMAT,7000) NAMSBR,TITLE
      IF(NDIM .EQ. 3) WRITE(IPMAT,7001)
      IF(NDIM .EQ. 2) THEN
        WRITE(IPMAT,7020) ((XYZL(IPM,IDIR),IPM=1,2),IDIR=1,NDIM)
        WRITE(IPMAT,7021)
      ELSE IF(NDIM .EQ. 3) THEN
        WRITE(IPMAT,7030) ((XYZL(IPM,IDIR),IPM=1,2),IDIR=1,NDIM)
        WRITE(IPMAT,7031) 
      ENDIF
*----
* Identify points associated with each surface
*----                   
      DO ISUR=1,NSUR
        IF(IPRINT .GE. 10) THEN
          WRITE(IOUT,6002) ISUR
        ENDIF
        DO ILINE=1,NSKTRK
          READ(IFTRK) IDUM
        ENDDO
        IFACE=0
*----
*  Scan over lines
*----
        DO ILINE=1,NBTR
          READ(IFTRK) IANGL,NBSEG,WEIGHT, 
     >                (NUMERO(ISEG),ISEG=1,NBSEG),
     >                (LENGTH(ISEG),ISEG=1,NBSEG),
     >                NTLINE,IPLANE,IPTA2,IPTA3,
     >                (TORIG(IDIR),IDIR=1,NDIM),TSTRT
*----
*  Find line segment location
*----
          DO IDIR=1,NDIM
            DXYZ(IDIR)=TORIG(IDIR)+DANGLT(IDIR,IANGL)*TSTRT
          ENDDO
          DO ISEG=1,NBSEG
            ISV=NUMERO(ISEG)
            IF(ISV .EQ. -ISUR) THEN
              IFACE=IFACE+1
              IF(IFACE .EQ. 1) THEN
                WRITE(IPMAT,7002) ISUR
              ENDIF
              WRITE(IPMAT,7004) (DXYZ(IDIR),IDIR=1,NDIM),ILINE
            ELSE IF(ISV .GT. 0) THEN
              FLEN=LENGTH(ISEG)/DVNOR(ISV,1)
              IF(NBDR .GT. 1) FLEN=LENGTH(ISEG)/DVNOR(ISV,IANGL+1)
              DO IDIR=1,NDIM
                DXYZ(IDIR)=DXYZ(IDIR)+
     >          DANGLT(IDIR,IANGL)*FLEN
              ENDDO
            ENDIF
          ENDDO
        ENDDO
*----
*  Write Matlab commands to print points
*----
        IF(IFACE .GE. 1) THEN
          WRITE(IPMAT,7003)
          IF(NDIM .EQ. 2) THEN
            WRITE(IPMAT,7010) ISUR,ACOL(MOD(ISUR,7))
          ELSE
            WRITE(IPMAT,7011) ISUR,ACOL(MOD(ISUR,7))
          ENDIF
          WRITE(IPMAT,7090)
          IF(IPLP(1) .GT. 0) WRITE(IPMAT,7091)
        ENDIF
        REWIND IFTRK
      ENDDO
      WRITE(IPMAT,7093)
      IF(IPLP(1) .LT. 0) WRITE(IPMAT,7091)
*----
*  Processing finished, return
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      RETURN 
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6002 FORMAT(' Processing points for surface = ',I8)
*----
*  Matlab .m file format
*----
 7000 FORMAT('%'/'% Output from ',A6/'%'/
     >'figure ;'/'hold on;'/7Htitle(',A36,3H');/
     >12Hxlabel('x');/12Hylabel('y');)
 7001 FORMAT(12Hzlabel('z');)
 7002 FORMAT('% Points for Surface ',I10/
     >       'TLMSurfacePoints=[')
 7003 FORMAT(12X,'];')
 7004 FORMAT(3F18.10,2X,I10)
 7010 FORMAT('% Plot surface ',I10/
     >       'plot(TLMSurfacePoints(:,1),',
     >            'TLMSurfacePoints(:,2),',1H',A2,3H');)
 7011 FORMAT('% Plot surface ',I10/
     >       'plot3(TLMSurfacePoints(:,1),',
     >             'TLMSurfacePoints(:,2),',
     >             'TLMSurfacePoints(:,3),',1H',A2,3H');)
 7020 FORMAT('xmin=',F18.10,';'/'xmax=',F18.10,';'/
     >       'ymin=',F18.10,';'/'ymax=',F18.10,';'/)
 7021 FORMAT('line([xmin,xmin],[ymin,ymax]);'/
     >       'line([xmin,xmax],[ymin,ymin]);'/
     >       'line([xmax,xmax],[ymin,ymax]);'/
     >       'line([xmin,xmax],[ymax,ymax]);')
 7030 FORMAT('xmin=',F18.10,';'/'xmax=',F18.10,';'/
     >       'ymin=',F18.10,';'/'ymax=',F18.10,';'/
     >       'zmin=',F18.10,';'/'zmax=',F18.10,';'/)
 7031 FORMAT('line([xmin,xmin],[ymin,ymax],[zmin,zmin]);'/
     >       'line([xmin,xmax],[ymin,ymin],[zmin,zmin]);'/
     >       'line([xmax,xmax],[ymin,ymax],[zmin,zmin]);'/
     >       'line([xmin,xmax],[ymax,ymax],[zmin,zmin]);'/    
     >       'line([xmin,xmin],[ymin,ymin],[zmin,zmax]);'/
     >       'line([xmax,xmax],[ymin,ymin],[zmin,zmax]);'/
     >       'line([xmax,xmax],[ymax,ymax],[zmin,zmax]);'/
     >       'line([xmin,xmin],[ymax,ymax],[zmin,zmax]);'/    
     >       'line([xmin,xmin],[ymin,ymax],[zmax,zmax]);'/
     >       'line([xmin,xmax],[ymin,ymin],[zmax,zmax]);'/
     >       'line([xmax,xmax],[ymin,ymax],[zmax,zmax]);'/
     >       'line([xmin,xmax],[ymax,ymax],[zmax,zmax]);')
 7090 FORMAT('clear TLMSurfacePoints;')
 7091 FORMAT('pause ;')
 7093 FORMAT('hold off ;')
      END
 


