*DECK TRFICF
      SUBROUTINE TRFICF(IPSYS,IPTRK,IFTRAK,IPRNTF,IDIR,NREGIO,NBMIX,
     >                  NUNKNO,MATCOD,VOLUME,KEYFLX,FUNKNO,SUNKNO,TITRE)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one group transport equation for fluxes using the scattering
* modified collision probability matrix.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): G. Marleau
*
*Parameters: input
* IPSYS   pointer to the pij LCM object.
* IPTRK   pointer to the tracking LCM object.
* IFTRAK  not used.
* IPRNTF  print selection for flux modules.
* IDIR    directional collision probability flag:
*         =0 for pij or wij;
*         =k for pijk or wijk k=1,2,3.
* NREGIO  number of regions considered.
* NBMIX   number of mixtures.
* NUNKNO  number of unknown in the system.
* MATCOD  mixture code in region.
* VOLUME  volume of region.
* KEYFLX  flux elements in unknown system.
* SUNKNO  source for system of unknown.
* TITRE   title.
*
*Parameters: input/output
* FUNKNO  unknown vector solved for.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   TITRE*72
      INTEGER     IPSYS,IPTRK,IFTRAK,IPRNTF,IDIR,NREGIO,NBMIX,NUNKNO,
     >            MATCOD(NREGIO),KEYFLX(NREGIO)
      REAL        VOLUME(NREGIO),FUNKNO(NUNKNO),SUNKNO(NUNKNO)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40)
      CHARACTER   CNS(0:3)*1,CNFDIR(0:3)*9,NAMLCM*12,NAMMY*12
      INTEGER     ISTATE(NSTATE),ILCMLN
      LOGICAL     EMPTY,LCM,LBIHET
      COMMON      RBASE(1)
      SAVE        CNS,CNFDIR
      DATA   CNS  /'-','1','2','3'/
      DATA CNFDIR /'F L U X  ','C U R - X','C U R - Y','C U R - Z'/
*----
*  PRINT SOURCES WHEN REQUIRED
*----
      IF(IPRNTF.GT.3) THEN
        WRITE(IUNOUT,'(12H  SOURCES : ,3X,A9)') CNFDIR(IDIR)
        WRITE(IUNOUT,'(1P,6(5X,E15.7))') (SUNKNO(KEYFLX(I)),I=1,NREGIO)
      ENDIF
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      LBIHET=ISTATE(40).NE.0
      IF(LBIHET) THEN
         CALL SETARA(RBASE,NUNKNO,ISUNKN)
         DO 10 I=1,NUNKNO
           RBASE(ISUNKN+I-1)=SUNKNO(I)
   10    CONTINUE
         NREGAR=NREGIO
         NBMIXG=NBMIX
         CALL DOORFB2(IPSYS,IPTRK,IPRNTF,NBMIX,NREGIO,NUNKNO,KEYFLX,
     1   NBMIX2,NREGI2,SUNKNO)
         NREGIO=NREGI2
         NBMIX=NBMIX2
         CALL LCMSIX(IPSYS,'BIHET',1)
      ENDIF
*----
*  ALLOCATE STORAGE AND READ SCATTERING MODIFIED COLLISION PROBABILITIES
*----
      CALL LCMINF(IPSYS,NAMLCM,NAMMY,EMPTY,ILONG,LCM)
      CALL LCMLEN(IPSYS,'DRAGON'//CNS(IDIR)//'PCSCT',ILCMLN,ITYLCM)
      IF((ILCMLN.GT.0).AND.LCM) THEN
        CALL LCMIOF(IPSYS,'DRAGON'//CNS(IDIR)//'PCSCT',ICPMAT)
      ELSE IF(ILCMLN.GT.0) THEN
        CALL SETARA(RBASE,NREGIO*NREGIO,ICPMAT)
        CALL LCMGET(IPSYS,'DRAGON'//CNS(IDIR)//'PCSCT',RBASE(ICPMAT))
      ELSE
        CALL XABORT('TRFICF: RECORD DRAGON'//CNS(IDIR)//
     >              'PCSCT ABSENT FROM LCM')
      ENDIF
*----
*  SOLVE TRANSPORT EQUATION
*----
      JCPMAT=ICPMAT-1
      DO 30 I=1,NREGIO
        FUNKNO(KEYFLX(I))=0.0
   30 CONTINUE
      DO 50 I=1,NREGIO
        IPOS=KEYFLX(I)
        DO 40 J=1,NREGIO
          JPOS=KEYFLX(J)
          FUNKNO(JPOS)=FUNKNO(JPOS)+SUNKNO(IPOS)*RBASE(JCPMAT+J)
   40   CONTINUE
        JCPMAT=JCPMAT+NREGIO
   50 CONTINUE
      IF(.NOT.LCM) CALL RLSARA(RBASE(ICPMAT))
*----
*  DOUBLE HETEROGENEITY TREATMENT
*----
      IF(LBIHET) THEN
         CALL LCMSIX(IPSYS,' ',2)
         NREGIO=NREGAR
         NBMIX=NBMIXG
         DO 60 I=1,NUNKNO
           SUNKNO(I)=RBASE(ISUNKN+I-1)
   60    CONTINUE
         CALL RLSARA(RBASE(ISUNKN))
         CALL DOORFB3(IPSYS,IPTRK,IPRNTF,NBMIX,NREGIO,NUNKNO,KEYFLX,
     1   SUNKNO,FUNKNO)
      ENDIF
*----
*  PRINT FLUXES WHEN REQUIRED
*----
      IF(IPRNTF.GT.3) THEN
        WRITE(IUNOUT,'(12H SOLUTION : ,5X,A9)') CNFDIR(IDIR)
        WRITE(IUNOUT,'(1P,6(5X,E15.7))') (FUNKNO(KEYFLX(I)),I=1,NREGIO)
      ENDIF
      RETURN
      END
