*DECK TRIGSO
      SUBROUTINE TRIGSO (IEX,NGCOND,NMERGE,FLXMER,SUNMER,CYLIND,IELEM,
     1 NREG,NUN,NBMIX,MAT,XX,DD,KN,QFR,VOL,MERG,COUR,FUNKNO,SOURCE,
     2 ZGAR1,ZGAR2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* source term calculation for a mixed-dual formulation of the finite
* element technique in a 3-D Cartesian geometry. Used in DRAGON.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IEX     iteration number.
* NGCOND  number of groups condensed.
* NMERGE  number of merged regions.
* FLXMER  flux estimate per mixture.
* SUNMER  incoming source (scattering+fission) cross sections.
* CYLIND  cylinderization flag (=.true. for cylindrical geometry)
* IELEM   degree of the Lagrangian finite elements: =1 (linear);
*         =2 (parabolic); =3 (cubic); =4 (quartic).
* NREG    number of elements in BIVAC.
* NUN     number of unknowns per group in BIVAC.
* NBMIX   number of macro-mixtures.
* MAT     mixture index per region.
* XX      X-directed mesh spacings.
* DD      values used with a cylindrical geometry.
* KN      element-ordered unknown list.
* QFR     element-ordered boundary conditions.
* VOL     volume of regions.
* MERG    index of merged regions.
* COUR    four times the incoming current per unit surface.
* FUNKNO  previously calculated fluxes.
*
*Parameters: output
* SOURCE  fission and diffusion sources.
*
*Parameters: scratch
* ZGAR1
* ZGAR2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IEX,NGCOND,NMERGE,IELEM,NREG,NUN,NBMIX,MAT(NREG),
     1 KN(4*NREG),MERG(NREG)
      REAL FLXMER(NMERGE,NGCOND),SUNMER(NMERGE,NGCOND),XX(NREG),
     1 DD(NREG),QFR(4*NREG),VOL(NREG),COUR,FUNKNO(NUN,NGCOND),
     2 SOURCE(NUN),ZGAR1(NBMIX),ZGAR2(NBMIX)
      LOGICAL CYLIND
*
      CALL XDRSET(SOURCE,NUN,0.0)
*----
*  INCOMING CURRENT SOURCE.
*----
      IF(COUR.NE.0.0) THEN
         NUM1=0
         NUM2=0
         DO 50 K=1,NREG
         IF(MAT(K).EQ.0) GO TO 50
         IF(VOL(K).EQ.0.0) GO TO 40
         IF(CYLIND) THEN
            DIN=1.0-0.5*XX(K)/DD(K)
            DOT=1.0+0.5*XX(K)/DD(K)
         ELSE
            DIN=1.0
            DOT=1.0
         ENDIF
*
         IF(KN(NUM1+2).NE.0) THEN
            IND1=ABS(KN(NUM1+2))
            SG=REAL(SIGN(1,KN(NUM1+2)))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*DIN*QFR(NUM2+1)
         ENDIF
         IF(KN(NUM1+3).NE.0) THEN
            IND1=ABS(KN(NUM1+3))
            SG=REAL(SIGN(1,KN(NUM1+3)))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*DOT*QFR(NUM2+2)
         ENDIF
         IF(KN(NUM1+4).NE.0) THEN
            IND1=ABS(KN(NUM1+4))
            SG=REAL(SIGN(1,KN(NUM1+4)))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*QFR(NUM2+3)
         ENDIF
         IF(KN(NUM1+5).NE.0) THEN
            IND1=ABS(KN(NUM1+5))
            SG=REAL(SIGN(1,KN(NUM1+5)))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*QFR(NUM2+4)
         ENDIF
         IF(KN(NUM1+6).NE.0) THEN
            IND1=ABS(KN(NUM1+6))
            SG=REAL(SIGN(1,KN(NUM1+6)))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*QFR(NUM2+5)
         ENDIF
         IF(KN(NUM1+7).NE.0) THEN
            IND1=ABS(KN(NUM1+7))
            SG=REAL(SIGN(1,KN(NUM1+7)))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*QFR(NUM2+6)
         ENDIF
   40    NUM1=NUM1+7
         NUM2=NUM2+6
   50    CONTINUE
      ENDIF
*----
*  DIFFUSION AND FISSION SOURCE.
*----
      IF(IEX.EQ.1) GO TO 115
      DO 110 NL=1,NGCOND
      DO 60 L=1,NBMIX
      ZGAR1(L)=0.0
   60 ZGAR2(L)=0.0
      NUM1=0
      DO 80 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 80
      IF(VOL(K).EQ.0.0) GO TO 70
      ZGAR1(L)=ZGAR1(L)+VOL(K)
      ZGAR2(L)=ZGAR2(L)+FUNKNO(KN(NUM1+1),NL)*VOL(K)
   70 NUM1=NUM1+7
   80 CONTINUE
      NUM1=0
      DO 110 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 110
      IF(VOL(K).EQ.0.0) GO TO 100
      DO 90 I0=1,IELEM*IELEM
      IND1=KN(NUM1+1)+I0-1
      PV=SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)*ZGAR1(L)/ZGAR2(L)
   90 SOURCE(IND1)=SOURCE(IND1)+FUNKNO(IND1,NL)*VOL(K)*PV
  100 NUM1=NUM1+7
  110 CONTINUE
      RETURN
  115 NUM1=0
      DO 140 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 140
      IF(VOL(K).EQ.0.0) GO TO 130
      IND1=KN(NUM1+1)
      PV=0.0
      DO 120 NL=1,NGCOND
  120 PV=PV+SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)
      SOURCE(IND1)=SOURCE(IND1)+VOL(K)*PV
  130 NUM1=NUM1+7
  140 CONTINUE
      RETURN
      END
