*DECK TRIVA
      SUBROUTINE TRIVA(IPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT,VOL,
     1 KNORM,SIGT0,SIGW0,DIFF,ILK,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of one-speed finite-difference or finite-element matrices
* for a discretization of the 3D diffusion or SPN equation.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the system matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* NREG    total number of merged regions for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NANI    number of Legendre orders for the scattering cross sections.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   not used.
* SIGT0   total macroscopic cross sections ordered by mixture.
* SIGW0   within-group scattering macroscopic cross section ordered
*         by mixture.
* DIFF    diffusion coefficients ordered by mixture.
* ILK     leakage flag (ILK=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER*72 TITR
      LOGICAL ILK
      INTEGER IPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT(NREG),KNORM
      REAL VOL(NREG),SIGT0(0:NBMIX),SIGW0(0:NBMIX,NANI),DIFF(0:NBMIX)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER ISTATE(NSTATE),IGB(8),IBASE(1)
      LOGICAL CYLIND,CHEX,DIAG,LOGY,LOGZ,LBIHET
      CHARACTER NAMP*12,TEXT10*10
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
      PARAMETER(TEXT10='A001001')
*----
*  RECOVER TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      LBIHET=ISTATE(40).NE.0
      IF(LBIHET) THEN
         CALL LCMSIX(IPTRK,'BIHET',1)
         CALL LCMGET(IPTRK,'PARAM',IGB)
         IF(NREG.NE.IGB(3)) CALL XABORT('TRIVA: INVALID VALUE OF NREG('
     1   //'1).')
         CALL LCMSIX(IPTRK,' ',2)
      ELSE
         IF(NREG.NE.ISTATE(1)) CALL XABORT('TRIVA: INVALID VALUE OF NR'
     1   //'EG(2).')
      ENDIF
      ITYPE=ISTATE(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IHEX=ISTATE(7)
      DIAG=(ISTATE(8).EQ.1)
      IELEM=ISTATE(9)
      ICOL=ISTATE(10)
      LL4=ISTATE(11)
      ICHX=ISTATE(12)
      ISPLH=ISTATE(13)
      LX=ISTATE(14)
      LY=ISTATE(15)
      LZ=ISTATE(16)
      ISEG=ISTATE(17)
      IMPV=ISTATE(18)
      NR0=ISTATE(24)
      LL4F=ISTATE(25)
      IF(ICHX.EQ.2) THEN
         ITY=3
         LL4W=ISTATE(26)
         LL4X=ISTATE(27)
         LL4Y=ISTATE(28)
         LL4Z=ISTATE(29)
         LOGY=LL4Y.GT.0
         LOGZ=LL4Z.GT.0
      ELSE
         ITY=2
         LL4W=LL4
         LL4X=LL4
         LL4Y=LL4
         LL4Z=LL4
         LOGY=IDIM.GT.1
         LOGZ=IDIM.GT.2
      ENDIF
      NLF=ISTATE(30)
      ISPN=ISTATE(31)
      ISCAT=ABS(ISTATE(32))
      NVD=ISTATE(34)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY*LZ,IZZ)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL SETARA(IBASE,LL4W,IMUW)
         CALL LCMGET(IPTRK,'MUW',IBASE(IMUW))
      ELSE
         CALL SETARA(BASE,LX*LY*LZ,IXX)
         CALL SETARA(BASE,LX*LY*LZ,IYY)
         CALL SETARA(BASE,LX*LY*LZ,IDD)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      IF(LOGY) THEN
         CALL SETARA(IBASE,LL4Y,IMUY)
         CALL LCMGET(IPTRK,'MUY',IBASE(IMUY))
      ENDIF
      IF(.NOT.DIAG) THEN
         CALL SETARA(IBASE,LL4X,IMUX)
         CALL LCMGET(IPTRK,'MUX',IBASE(IMUX))
      ELSE
         IMUX=IMUY
      ENDIF
      IF(LOGZ) THEN
         CALL SETARA(IBASE,LL4Z,IMUZ)
         CALL LCMGET(IPTRK,'MUZ',IBASE(IMUZ))
      ENDIF
*----
*  RECOVER UNIT MATRICES
*----
      IF((ICHX.EQ.1).OR.(ICHX.EQ.2)) THEN
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC*LC,IQ)
         CALL SETARA(BASE,LC*LC,IQS)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,36,IQH)
         CALL SETARA(BASE,9,IRT)
         CALL SETARA(BASE,9,IQT)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'RS',BASE(IRS))
         CALL LCMGET(IPTRK,'Q',BASE(IQ))
         CALL LCMGET(IPTRK,'QS',BASE(IQS))
         CALL LCMGET(IPTRK,'V',BASE(IV))
         IF((IELEM.EQ.1).AND.(ICOL.LE.2)) THEN
            CALL LCMGET(IPTRK,'RH',BASE(IRH))
            CALL LCMGET(IPTRK,'QH',BASE(IQH))
            CALL LCMGET(IPTRK,'RT',BASE(IRT))
            CALL LCMGET(IPTRK,'QT',BASE(IQT))
         ENDIF
         CALL LCMSIX(IPTRK,' ',2)
      ENDIF
*----
*  INTRODUCE THE CYLINDER BOUNDARY APPROXIMATION IN CARTESIAN GEOMETRY
*----
      IF(NR0.GT.0) THEN
         CALL XABORT('TRIVA: CYLINDRICAL CORRECTION NOT IMPLEMENTED.')
      ENDIF
*----
*  COMPUTE THE WITHIN-GROUP SYSTEM MATRICES (LEAKAGE AND REMOVAL).
*  ASSEMBLY OF THE ADI SPLITTED SYSTEM MATRICES
*----
      IF(NLF.EQ.0) THEN
*
*  ++++ DIFFUSION THEORY ++++
*
         IF(NANI.GT.1) THEN
	    CALL XABORT('TRIVA: SPN MACRO-CALCULATION EXPECTED(1).')
	 ENDIF
	 CALL SETARA(BASE,4*NBMIX,ISGD)
	 DO 10 IBM=1,NBMIX
         BASE(ISGD+0*NBMIX+IBM-1)=DIFF(IBM)
         BASE(ISGD+1*NBMIX+IBM-1)=DIFF(IBM)
         BASE(ISGD+2*NBMIX+IBM-1)=DIFF(IBM)
   10    BASE(ISGD+3*NBMIX+IBM-1)=SIGT0(IBM)-SIGW0(IBM,1)
         IF(ICHX.EQ.2) THEN
	    CALL SETARA(BASE,4*NBMIX,IDSGD)
            DO 40 J=1,4
            DO 40 I=1,NBMIX
            IF(BASE(ISGD+(J-1)*NBMIX+I-1).NE.0.) THEN
              BASE(IDSGD+(J-1)*NBMIX+I-1)=1.0/BASE(ISGD+(J-1)*NBMIX+I-1)
            ELSE
              BASE(IDSGD+(J-1)*NBMIX+I-1)=1.0E10
            ENDIF
   40       CONTINUE
         ENDIF
*
*        DIMENSION W
         IF(CHEX) THEN
            IF((ICHX.EQ.3).AND.(ISPLH.GT.1)) THEN
               CALL SETARA(IBASE,LL4,IMATN)
               NUM1=0
               DO 110 I=1,LX*LZ
               IF(MAT(I).EQ.0) GO TO 110
               DO 100 J=1,6*(ISPLH-1)**2
               KEL=IBASE(IKN+NUM1+J-1)
  100          IBASE(IMATN+KEL-1)=MAT(I)
               NUM1=NUM1+18*(ISPLH-1)**2+8
  110          CONTINUE
            ENDIF
            IIMAW=IBASE(IMUW+LL4W-1)
            CALL SETARA(BASE,IIMAW,IWA)
            CALL XDRSET(BASE(IWA),IIMAW,0.0)
            IF(ICHX.EQ.1) THEN
*              MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               CALL LCMIOF(IPTRK,'IPW',IIPW)
               CALL TRIRWW(NBMIX,NREG,LL4,VOL,MAT,BASE(ISGD),SIDE,
     1         BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUW),IBASE(IIPW),
     2         BASE(IWA),ISPLH,BASE(IR),BASE(IQ),BASE(IRH),BASE(IQH),
     3         BASE(IRT),BASE(IQT))
            ELSE IF(ICHX.EQ.2) THEN
*              THOMAS-RAVIART-SCHNEIDER FINITE ELEMENTS IN HEXAGONAL
*              GEOMETRY.
               CALL SETARA(BASE,LL4F,ITF)
               CALL SETARA(BASE,IIMAW,IAW)
               CALL XDRSET(BASE(ITF),LL4F,0.0)
               CALL XDRSET(BASE(IAW),IIMAW,0.0)
               CALL LCMGET(IPTRK,'ZCODE',ZCODE)
               CALL LCMIOF(IPTRK,'IPBBW',IIPBW)
               CALL LCMIOF(IPTRK,'WB',IBW)
               CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
               CALL LCMIOF(IPTRK,'FRZ',IFRZ)
               CALL SETARA(BASE,IIMAW,IWA)
               NBLOS=LX*LZ*ISPLH**2
               CALL SETARA(BASE,NBLOS,IDIF)
               DO 120 I=1,NBLOS
               BASE(IDIF+I-1)=0.0
               IND=IBASE(IZGLOB+I-1)
               IF(IND.GT.0) THEN
                  IBM=MAT(IND)
                  DZ=BASE(IZZ+IND-1)*BASE(IFRZ+IND-1)
                  IF(IBM.GT.0) BASE(IDIF+I-1)=DZ/BASE(ISGD+IBM-1)
               ENDIF
  120          CONTINUE
               CALL LCMPOF(IPSYS,'DIFF'//TEXT10,NBLOS,2,IDIF)
               CALL TRIHWW(NBMIX,NBLOS,IELEM,ICOL,ISPLH,LX,LZ,LL4,
     1         LL4F,LL4W,MAT,VOL,SIDE,BASE(IZZ),BASE(IFRZ),BASE(IQFR),
     2         IBASE(IZGLOB),IBASE(IKN),BASE(ISGD),BASE(IDSGD),
     3         IBASE(IMUW),IBASE(IIPBW),LC,BASE(IR),BASE(IV),BASE(IBW),
     4         BASE(ITF),BASE(IAW),BASE(IWA))
            ELSE IF(ICHX.EQ.3) THEN
*              MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               CALL LCMIOF(IPTRK,'IPW',IIPW)
               IF(ISPLH.EQ.1) THEN
                  CALL SETARA(BASE,NREG,IIGAR)
                  CALL TRIMWW(NBMIX,IDIM,NREG,LL4,VOL,MAT,BASE(ISGD),
     1            BASE(ISGD),SIDE,BASE(IZZ),IBASE(IKN),BASE(IQFR),
     2            IBASE(IMUW),IBASE(IIPW),0,BASE(IWA),BASE(IIGAR))
                  CALL RLSARA(BASE(IIGAR))
               ELSE
                  CALL TRIMTW(ISPLH,NBMIX,IDIM,NREG,LL4,VOL,MAT,
     1            IBASE(IMATN),BASE(ISGD),BASE(ISGD),SIDE,BASE(IZZ),
     2            IBASE(IKN),BASE(IQFR),IBASE(IMUW),IBASE(IIPW),0,
     3            BASE(IWA))
               ENDIF
            ENDIF
            CALL LCMPOF(IPSYS,'W '//TEXT10,IIMAW,2,IWA)
            IF(ICHX.EQ.2) THEN
               CALL LCMPOF(IPSYS,'WA'//TEXT10,IIMAW,2,IAW)
               CALL LCMPOF(IPSYS,'TF'//TEXT10,LL4F,2,ITF)
            ENDIF
         ENDIF
*
*        DIMENSION X
         IIMAX=IBASE(IMUX+LL4X-1)
         IF(CHEX.AND.(ICHX.EQ.2)) THEN
*           THOMAS-RAVIART-SCHNEIDER FINITE ELEMENTS IN HEXAGONAL
*           GEOMETRY.
            CALL SETARA(BASE,IIMAX,IAX)
            CALL XDRSET(BASE(IAX),IIMAX,0.0)
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMGET(IPTRK,'ZCODE',ZCODE)
            CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
            CALL LCMIOF(IPTRK,'XB',IBX)
            CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
            CALL LCMIOF(IPTRK,'FRZ',IFRZ)
            CALL SETARA(BASE,IIMAX,IXA)
            NBLOS=LX*LZ*ISPLH**2
            CALL TRIHWX(NBMIX,NBLOS,IELEM,ICOL,ISPLH,LX,LZ,LL4,
     1      LL4F,LL4W,LL4X,MAT,VOL,SIDE,BASE(IZZ),BASE(IFRZ),
     2      BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),BASE(ISGD),BASE(IDSGD),
     3      IBASE(IMUX),IBASE(IIPBX),LC,BASE(IR),BASE(IV),BASE(IBX),
     4      BASE(ITF),BASE(IAX),BASE(IXA))
         ELSE IF(ICHX.EQ.2) THEN
*           THOMAS-RAVIART ADI ITERATIVE METHOD.
            CALL SETARA(BASE,LL4F,ITF)
            CALL SETARA(BASE,IIMAX,IAX)
            CALL XDRSET(BASE(ITF),LL4F,0.0)
            CALL XDRSET(BASE(IAX),IIMAX,0.0)
            CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
            CALL LCMIOF(IPTRK,'XB',IBX)
            CALL SETARA(BASE,IIMAX,IXA)
            CALL TRIDXX(NBMIX,CYLIND,IELEM,ICOL,IDIM,NREG,LL4,LL4F,
     1      LL4X,MAT,VOL,BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),
     2      IBASE(IKN),BASE(IQFR),BASE(ISGD),BASE(IDSGD),IBASE(IMUX),
     3      IBASE(IIPBX),LC,BASE(IR),BASE(IV),BASE(IBX),BASE(ITF),
     4      BASE(IAX),BASE(IXA))
         ELSE
*           GENERIC ADI ITERATIVE METHOD.
            CALL LCMIOF(IPTRK,'IPX',IIPX)
            CALL SETARA(BASE,IIMAX,IXA)
            CALL XDRSET(BASE(IXA),IIMAX,0.0)
            IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
               CALL TRIPXX(NBMIX,MAXKN,NREG,LL4,VOL,MAT,BASE(ISGD),
     1         BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2         BASE(IQFR),IBASE(IMUX),IBASE(IIPX),CYLIND,LC,BASE(IT),
     3         BASE(ITS),BASE(IQ),BASE(IQS),BASE(IXA))
            ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
               CALL SETARA(BASE,NREG,IIGAR)
               CALL TRIMXX(NBMIX,CYLIND,IELEM,IDIM,NREG,LL4,VOL,MAT,
     1         BASE(ISGD),BASE(ISGD),BASE(IXX),BASE(IYY),BASE(IZZ),
     2         BASE(IDD),IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPX),
     3         0,BASE(IXA),BASE(IIGAR))
               CALL RLSARA(BASE(IIGAR))
            ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*              MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               CALL TRIRWX(NBMIX,NREG,LL4,VOL,MAT,BASE(ISGD),SIDE,
     1         BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPX),
     2         BASE(IXA))
            ELSE IF((ICHX.EQ.3).AND.CHEX) THEN
*              MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               IF(ISPLH.EQ.1) THEN
                  CALL SETARA(BASE,NREG,IIGAR)
                  CALL TRIMWX(NBMIX,IDIM,NREG,LL4,VOL,MAT,BASE(ISGD),
     1            BASE(ISGD),SIDE,BASE(IZZ),IBASE(IKN),BASE(IQFR),
     2            IBASE(IMUX),IBASE(IIPX),0,BASE(IXA),BASE(IIGAR))
                  CALL RLSARA(BASE(IIGAR))
               ELSE
                  CALL TRIMTX(ISPLH,NBMIX,IDIM,NREG,LL4,VOL,MAT,
     1            IBASE(IMATN),BASE(ISGD),BASE(ISGD),SIDE,BASE(IZZ),
     2            IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPX),0,
     3            BASE(IXA))
               ENDIF
            ENDIF
         ENDIF
         IF(.NOT.DIAG) THEN
            CALL LCMPOF(IPSYS,'X '//TEXT10,IIMAX,2,IXA)
            IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'XA'//TEXT10,IIMAX,2,IAX)
         ELSE
*           IN DIAGONAL SYMMETRY CASE, DO NOT SAVE THE X-DIRECTED ADI
*           MATRIX COMPONENT SINCE IT IS EQUAL TO THE Y-DIRECTED
*           COMPONENT
            CALL RLSARA(BASE(IXA))
            IF(ICHX.EQ.2) CALL RLSARA(BASE(IAX))
         ENDIF
         IF(.NOT.CHEX.AND.(ICHX.EQ.2)) CALL LCMPOF(IPSYS,'TF'//TEXT10,
     >   LL4F,2,ITF)
*
*        DIMENSION Y
         IF(LOGY) THEN
            IIMAY=IBASE(IMUY+LL4Y-1)
            IF(CHEX.AND.(ICHX.EQ.2)) THEN
*              THOMAS-RAVIART-SCHNEIDER FINITE ELEMENTS IN HEXAGONAL
*              GEOMETRY.
               CALL SETARA(BASE,IIMAY,IAY)
               CALL XDRSET(BASE(IAY),IIMAY,0.0)
               CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
               CALL LCMGET(IPTRK,'ZCODE',ZCODE)
               CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
               CALL LCMIOF(IPTRK,'YB',IBY)
               CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
               CALL LCMIOF(IPTRK,'FRZ',IFRZ)
               CALL SETARA(BASE,IIMAY,IYA)
               NBLOS=LX*LZ*ISPLH**2
               CALL TRIHWY(NBMIX,NBLOS,IELEM,ICOL,ISPLH,LX,LZ,LL4,
     1         LL4F,LL4W,LL4X,LL4Y,MAT,VOL,SIDE,BASE(IZZ),BASE(IFRZ),
     2         BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),BASE(ISGD),
     3         BASE(IDSGD),IBASE(IMUY),IBASE(IIPBY),LC,BASE(IR),
     4         BASE(IV),BASE(IBY),BASE(ITF),BASE(IAY),BASE(IYA))
            ELSE IF(ICHX.EQ.2) THEN
*              THOMAS-RAVIART ADI ITERATIVE METHOD.
               CALL SETARA(BASE,IIMAY,IAY)
               CALL XDRSET(BASE(IAY),IIMAY,0.0)
               CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
               CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
               CALL LCMIOF(IPTRK,'YB',IBY)
               CALL SETARA(BASE,IIMAY,IYA)
               CALL TRIDXY(NBMIX,IELEM,ICOL,IDIM,NREG,LL4,LL4F,LL4X,
     1         LL4Y,MAT,VOL,BASE(IYY),IBASE(IKN),BASE(IQFR),BASE(ISGD),
     2         BASE(IDSGD),IBASE(IMUY),IBASE(IIPBY),LC,BASE(IR),
     3         BASE(IV),BASE(IBY),BASE(ITF),BASE(IAY),BASE(IYA))
            ELSE
*              GENERIC ADI ITERATIVE METHOD.
               CALL LCMIOF(IPTRK,'IPY',IIPY)
               CALL SETARA(BASE,IIMAY,IYA)
               CALL XDRSET(BASE(IYA),IIMAY,0.0)
               IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
                  CALL TRIPXY(NBMIX,MAXKN,NREG,LL4,VOL,MAT,BASE(ISGD),
     1            BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2            BASE(IQFR),IBASE(IMUY),IBASE(IIPY),CYLIND,BASE(IYA))
               ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
                  CALL SETARA(BASE,NREG,IIGAR)
                  CALL TRIMXY(NBMIX,CYLIND,IELEM,IDIM,NREG,LL4,VOL,MAT,
     1            BASE(ISGD),BASE(ISGD),BASE(IXX),BASE(IYY),BASE(IZZ),
     2            BASE(IDD),IBASE(IKN),BASE(IQFR),IBASE(IMUY),
     3            IBASE(IIPY),0,BASE(IYA),BASE(IIGAR))
                  CALL RLSARA(BASE(IIGAR))
               ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*                 MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
                  CALL TRIRWY(NBMIX,NREG,LL4,VOL,MAT,BASE(ISGD),SIDE,
     1            BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUY),
     2            IBASE(IIPY),BASE(IYA))
               ELSE IF((ICHX.EQ.3).AND.CHEX) THEN
*                 MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
                  IF(ISPLH.EQ.1) THEN
                     CALL SETARA(BASE,NREG,IIGAR)
                     CALL TRIMWY(NBMIX,IDIM,NREG,LL4,VOL,MAT,BASE(ISGD),
     1               BASE(ISGD),SIDE,BASE(IZZ),IBASE(IKN),BASE(IQFR),
     2               IBASE(IMUY),IBASE(IIPY),0,BASE(IYA),BASE(IIGAR))
                     CALL RLSARA(BASE(IIGAR))
                  ELSE
                     CALL TRIMTY(ISPLH,NBMIX,IDIM,NREG,LL4,VOL,MAT,
     1               IBASE(IMATN),BASE(ISGD),BASE(ISGD),SIDE,BASE(IZZ),
     2               IBASE(IKN),BASE(IQFR),IBASE(IMUY),IBASE(IIPY),0,
     3               BASE(IYA))
                  ENDIF
               ENDIF
            ENDIF
            CALL LCMPOF(IPSYS,'Y '//TEXT10,IIMAY,2,IYA)
            IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'YA'//TEXT10,IIMAY,2,IAY)
         ENDIF
*
*        DIMENSION Z
         IF(LOGZ) THEN
            IIMAZ=IBASE(IMUZ+LL4Z-1)
            IF(CHEX.AND.(ICHX.EQ.2)) THEN
*              THOMAS-RAVIART-SCHNEIDER FINITE ELEMENTS IN HEXAGONAL
*              GEOMETRY.
               CALL SETARA(BASE,IIMAZ,IAZ)
               CALL XDRSET(BASE(IAZ),IIMAZ,0.0)
               CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
               CALL LCMGET(IPTRK,'ZCODE',ZCODE)
               CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
               CALL LCMIOF(IPTRK,'ZB',IBZ)
               CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
               CALL LCMIOF(IPTRK,'FRZ',IFRZ)
               CALL SETARA(BASE,IIMAZ,IZA)
               NBLOS=LX*LZ*ISPLH**2
               CALL TRIHWZ(NBMIX,NBLOS,IELEM,ICOL,ISPLH,LX,LZ,LL4,
     1         LL4F,LL4W,LL4X,LL4Y,LL4Z,MAT,VOL,SIDE,BASE(IZZ),
     2         BASE(IFRZ),BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),
     3         BASE(ISGD),BASE(IDSGD),IBASE(IMUZ),IBASE(IIPBZ),LC,
     4         BASE(IR),BASE(IV),BASE(IBZ),BASE(ITF),BASE(IAZ),
     5         BASE(IZA))
            ELSE IF(ICHX.EQ.2) THEN
*              THOMAS-RAVIART ADI ITERATIVE METHOD.
               CALL SETARA(BASE,IIMAZ,IAZ)
               CALL XDRSET(BASE(IAZ),IIMAZ,0.0)
               CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
               CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
               CALL LCMIOF(IPTRK,'ZB',IBZ)
               CALL SETARA(BASE,IIMAZ,IZA)
               CALL TRIDXZ(NBMIX,IELEM,ICOL,IDIM,NREG,LL4,LL4F,LL4X,
     1         LL4Y,LL4Z,MAT,VOL,BASE(IZZ),IBASE(IKN),BASE(IQFR),
     2         BASE(ISGD),BASE(IDSGD),IBASE(IMUZ),IBASE(IIPBZ),LC,
     3         BASE(IR),BASE(IV),BASE(IBZ),BASE(ITF),BASE(IAZ),
     4         BASE(IZA))
            ELSE
               CALL LCMIOF(IPTRK,'IPZ',IIPZ)
               CALL SETARA(BASE,IIMAZ,IZA)
               CALL XDRSET(BASE(IZA),IIMAZ,0.0)
               IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
                  CALL TRIPXZ(NBMIX,MAXKN,NREG,LL4,VOL,MAT,BASE(ISGD),
     1            BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2            BASE(IQFR),IBASE(IMUZ),IBASE(IIPZ),CYLIND,BASE(IZA))
               ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
                  CALL SETARA(BASE,NREG,IIGAR)
                  CALL TRIMXZ(NBMIX,CYLIND,IELEM,IDIM,NREG,LL4,VOL,MAT,
     1            BASE(ISGD),BASE(ISGD),BASE(IXX),BASE(IYY),BASE(IZZ),
     2            BASE(IDD),IBASE(IKN),BASE(IQFR),IBASE(IMUZ),
     3            IBASE(IIPZ),0,BASE(IZA),BASE(IIGAR))
                  CALL RLSARA(BASE(IIGAR))
               ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*                 MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
                  CALL TRIRWZ(NBMIX,NREG,LL4,VOL,MAT,BASE(ISGD),SIDE,
     1            BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUZ),
     2            IBASE(IIPZ),BASE(IZA))
               ELSE IF((ICHX.EQ.3).AND.CHEX) THEN
*                 MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
                  IF(ISPLH.EQ.1) THEN
                     CALL SETARA(BASE,NREG,IIGAR)
                     CALL TRIMWZ(NBMIX,IDIM,NREG,LL4,VOL,MAT,BASE(ISGD),
     1               BASE(ISGD),SIDE,BASE(IZZ),IBASE(IKN),BASE(IQFR),
     2               IBASE(IMUZ),IBASE(IIPZ),0,BASE(IZA),BASE(IIGAR))
                     CALL RLSARA(BASE(IIGAR))
                  ELSE
                     CALL TRIMTZ(ISPLH,NBMIX,IDIM,NREG,LL4,VOL,MAT,
     1               IBASE(IMATN),BASE(ISGD),BASE(ISGD),SIDE,BASE(IZZ),
     2               IBASE(IKN),BASE(IQFR),IBASE(IMUZ),IBASE(IIPZ),0,
     3               BASE(IZA))
                  ENDIF
               ENDIF
            ENDIF
            CALL LCMPOF(IPSYS,'Z '//TEXT10,IIMAZ,2,IZA)
            IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'ZA'//TEXT10,IIMAZ,2,IAZ)
         ENDIF
         IF((ICHX.EQ.3).AND.(ISPLH.GT.1).AND.CHEX) THEN
            CALL RLSARA(IBASE(IMATN))
         ENDIF
         IF(ICHX.EQ.2) THEN
            CALL RLSARA(IBASE(IDSGD))
         ENDIF
      ELSE
*
*  ++++ PN OR SPN THEORY ++++
*
         IF(NLF.LT.2) THEN
	    CALL XABORT('TRIVA: PN OR SPN KEYWORD EXPECTED.')
	 ELSE IF(CHEX) THEN
            CALL XABORT('TRIVA: UNKNOWN TYPE OF GEOMETRY(2).')
         ENDIF
         NAN=MIN(ISCAT,NANI)+1
         CALL SETARA(BASE,NBMIX*NAN,ISGD)
         CALL SETARA(BASE,NBMIX*NAN,ISGDI)
         DO 30 IL=0,NAN-1
         DO 20 IBM=1,NBMIX
         GARS=SIGT0(IBM)
         IF(IL.LE.NAN-2) GARS=GARS-SIGW0(IBM,IL+1)
         BASE(ISGD+IL*NBMIX+IBM-1)=GARS
         IF(GARS.NE.0.0) THEN
            BASE(ISGDI+IL*NBMIX+IBM-1)=1.0/GARS
         ELSE
            BASE(ISGDI+IL*NBMIX+IBM-1)=1.0E10
         ENDIF
   20    CONTINUE
         WRITE(NAMP,'(4HSCAR,I2.2,6H001001)') IL
         CALL LCMPUT(IPSYS,NAMP,NBMIX,2,BASE(ISGD+IL*NBMIX))
         WRITE(NAMP,'(4HSCAI,I2.2,6H001001)') IL
         CALL LCMPUT(IPSYS,NAMP,NBMIX,2,BASE(ISGDI+IL*NBMIX))
   30    CONTINUE
         CALL XDISET(ISTATE,NSTATE,0)
         ISTATE(7)=NBMIX
         ISTATE(8)=NAN
         CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,ISTATE)
*
*        DIMENSION X
         IIMAX=IBASE(IMUX+LL4X-1)*NLF/2
         CALL SETARA(BASE,LL4F*NLF/2,ITF)
         CALL SETARA(BASE,IIMAX,IAX)
         CALL XDRSET(BASE(ITF),LL4F*NLF/2,0.0)
         CALL XDRSET(BASE(IAX),IIMAX,0.0)
         CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
         CALL LCMIOF(IPTRK,'XB',IBX)
         CALL SETARA(BASE,IIMAX,IXA)
         CALL PN3DXX(NBMIX,IELEM,ICOL,IDIM,NREG,NLF,NVD,NAN,LL4,LL4F,
     1   LL4X,BASE(ISGD),BASE(ISGDI),MAT,VOL,BASE(IXX),BASE(IYY),
     2   BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPBX),LC,
     3   BASE(IR),BASE(IV),BASE(IBX),BASE(ITF),BASE(IAX),BASE(IXA))
         IF(.NOT.DIAG) THEN
            CALL LCMPOF(IPSYS,'X '//TEXT10,IIMAX,2,IXA)
            CALL LCMPOF(IPSYS,'XA'//TEXT10,IIMAX,2,IAX)
         ELSE
*           IN DIAGONAL SYMMETRY CASE, DO NOT SAVE THE X-DIRECTED ADI
*           MATRIX COMPONENT SINCE IT IS EQUAL TO THE Y-DIRECTED COMPONENT
            CALL RLSARA(BASE(IXA))
            CALL RLSARA(BASE(IAX))
         ENDIF
         CALL LCMPOF(IPSYS,'TF'//TEXT10,LL4F*NLF/2,2,ITF)
*
*        DIMENSION Y
         IF(LOGY) THEN
            IIMAY=IBASE(IMUY+LL4Y-1)*NLF/2
            CALL SETARA(BASE,IIMAY,IAY)
            CALL XDRSET(BASE(IAY),IIMAY,0.0)
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
            CALL LCMIOF(IPTRK,'YB',IBY)
            CALL SETARA(BASE,IIMAY,IYA)
            CALL PN3DXY(NBMIX,IELEM,ICOL,NREG,NLF,NVD,NAN,LL4,LL4F,LL4X,
     1      LL4Y,BASE(ISGD),BASE(ISGDI),MAT,VOL,BASE(IYY),IBASE(IKN),
     2      BASE(IQFR),IBASE(IMUY),IBASE(IIPBY),LC,BASE(IR),BASE(IV),
     3      BASE(IBY),BASE(ITF),BASE(IAY),BASE(IYA))
            CALL LCMPOF(IPSYS,'Y '//TEXT10,IIMAY,2,IYA)
            CALL LCMPOF(IPSYS,'YA'//TEXT10,IIMAY,2,IAY)
         ENDIF
*
*        DIMENSION Z
         IF(LOGZ) THEN
            IIMAZ=IBASE(IMUZ+LL4Z-1)*NLF/2
            CALL SETARA(BASE,IIMAZ,IAZ)
            CALL XDRSET(BASE(IAZ),IIMAZ,0.0)
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
            CALL LCMIOF(IPTRK,'ZB',IBZ)
            CALL SETARA(BASE,IIMAZ,IZA)
            CALL PN3DXZ(NBMIX,IELEM,ICOL,NREG,NLF,NVD,NAN,LL4,LL4F,
     1      LL4X,LL4Y,LL4Z,BASE(ISGD),BASE(ISGDI),MAT,VOL,BASE(IZZ),
     2      IBASE(IKN),BASE(IQFR),IBASE(IMUZ),IBASE(IIPBZ),LC,BASE(IR),
     3      BASE(IV),BASE(IBZ),BASE(ITF),BASE(IAZ),BASE(IZA))
            CALL LCMPOF(IPSYS,'Z '//TEXT10,IIMAZ,2,IZA)
            CALL LCMPOF(IPSYS,'ZA'//TEXT10,IIMAZ,2,IAZ)
         ENDIF
         CALL RLSARA(BASE(ISGDI))
         CALL RLSARA(BASE(ISGD))
      ENDIF
*----
*  MATRIX FACTORIZATIONS
*----
      NAMP=TEXT10
      CALL MTLDLF(NAMP,IPTRK,IPSYS,ITY,IMPX)
*----
*  RELEASE UNIT MATRICES
*----
      IF((ICHX.EQ.1).OR.(ICHX.EQ.2)) THEN
         CALL RLSARA(BASE(IQT))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IQH))
         CALL RLSARA(BASE(IRH))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IQS))
         CALL RLSARA(BASE(IQ))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(IR))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
      ENDIF
*----
*  RELEASE TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IZZ))
      IF(CHEX) THEN
         CALL RLSARA(IBASE(IMUW))
      ELSE
         CALL RLSARA(BASE(IDD))
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ENDIF
      IF(LOGY) CALL RLSARA(IBASE(IMUY))
      IF(.NOT.DIAG) CALL RLSARA(IBASE(IMUX))
      IF(LOGZ) CALL RLSARA(IBASE(IMUZ))
      RETURN
      END
