*DECK TRIVSO
      SUBROUTINE TRIVSO(IG,IPTRK,KPMACR,NANIS,NREG,NMAT,NUNKNO,NGRP,
     > MATCOD,KEYFLX,VOL,FLUX,SOURCE,IJJ,NJJ,IPOS,XSCAT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the source for the solution of diffusion or PN equations.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IG      secondary group.
* IPTRK   pointer to the tracking LCM object.
* KPMACR  pointer to the secondary-group related macrolib information.
* NANIS   maximum cross section Legendre order.
* NREG    number of regions.
* NMAT    number of mixtures.
* NUNKNO  number of unknowns per energy group including spherical
*         harmonic terms, interface currents and fundamental
*         currents.
* NGRP    number of energy groups.
* MATCOD  mixture indices.
* KEYFLX  index of flux components in unknown vector.
* VOL     volumes.
* FLUX    fluxes.
*
*Parameters: output
* SOURCE  sources.
*
*Parameters: scratch
* IJJ
* NJJ
* IPOS
* XSCAT
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IG,IPTRK,KPMACR,NANIS,NREG,NMAT,NUNKNO,NGRP,MATCOD(NREG),
     > KEYFLX(NREG),IJJ(0:NMAT),NJJ(0:NMAT),IPOS(0:NMAT)
      REAL VOL(NREG),FLUX(NUNKNO,NGRP),SOURCE(NUNKNO,NGRP),
     > XSCAT(0:NMAT*NGRP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,PI4=12.5663706144)
      INTEGER JPAR(NSTATE),IBASE(1)
      REAL R1DP(5,5)
      CHARACTER CAN(0:9)*2
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
      DATA CAN /'00','01','02','03','04','05','06','07','08','09'/
*
*----
*  RECOVER TRIVAC SPECIFIC PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      IF(JPAR(1).NE.NREG) CALL XABORT('TRIVSO: INCONSISTENT NREG.')
      IF(JPAR(2).NE.NUNKNO) CALL XABORT('TRIVSO: INCONSISTENT NUNKNO.')
      ITYPE=JPAR(6)
      IELEM=JPAR(9)
      ICOL=JPAR(10)
      L4=JPAR(11)
      ISPLH=JPAR(13)
      LX=JPAR(14)
      LY=JPAR(15)
      LZ=JPAR(16)
      NLF=JPAR(30)
      ISPN=JPAR(31)
      ISCAT=JPAR(32)
      IF(IELEM.NE.1) CALL XABORT('TRIVSO: DUAL AND LINEAR FLUX APPROXI'
     1 //'MATION IN SPACE EXPECTED.')
*----
*  SPHERICAL HARMONICS (PN) CASE.
*----
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL SETARA(BASE,LX*LY*LZ,IXX)
      CALL LCMGET(IPTRK,'XX',BASE(IXX))
      IF(ITYPE.EQ.4) THEN
         CALL SETARA(BASE,LX+1,IXXX)
         CALL LCMGET(IPTRK,'XXX',BASE(IXXX))
      ENDIF
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      IF(NLF.GT.0) THEN
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMSIX(IPTRK,' ',2)
         DO 10 IC=1,LC
         DO 10 JC=1,LC
   10    R1DP(IC,JC)=BASE(IR+(JC-1)*LC+IC-1)
         CALL RLSARA(BASE(IR))
      ENDIF
*----
*  COMPUTE THE SOURCE
*----
      IF(NLF.EQ.0) THEN
*
*  ++++ DIFFUSION THEORY ++++
*
         CALL LCMGET(KPMACR,'NJJS00',NJJ(1))
         CALL LCMGET(KPMACR,'IJJS00',IJJ(1))
         CALL LCMGET(KPMACR,'IPOS00',IPOS(1))
         CALL LCMGET(KPMACR,'SCAT00',XSCAT(1))
         IF((ITYPE.EQ.2).OR.(ITYPE.EQ.5).OR.(ITYPE.EQ.7)) THEN
*----
*  CARTESIAN 2D DUAL (THOMAS-RAVIART) CASE.
*----
            DO 30 IR=1,NREG
            IBM=MATCOD(IR)
            IF(IBM.GT.0) THEN
               IND=KEYFLX(IR)
               JG=IJJ(IBM)
               DO 20 JND=1,NJJ(IBM)
               IF(JG.NE.IG) THEN
                  SOURCE(IND,IG)=SOURCE(IND,IG)+FLUX(IND,JG)*
     >            XSCAT(IPOS(IBM)+JND-1)
               ENDIF
   20          JG=JG-1
            ENDIF
   30       CONTINUE
         ELSE
            CALL XABORT('TRIVSO: DISCRETIZATION NOT IMPLEMENTED(1).')
         ENDIF
      ELSE
*
*  ++++ PN OR SPN THEORY ++++
*
         DO 330 IL=0,MIN(ISCAT-1,NANIS)
         FACT=REAL(2*IL+1)
         CALL LCMGET(KPMACR,'NJJS'//CAN(IL),NJJ(1))
         CALL LCMGET(KPMACR,'IJJS'//CAN(IL),IJJ(1))
         CALL LCMGET(KPMACR,'IPOS'//CAN(IL),IPOS(1))
         CALL LCMGET(KPMACR,'SCAT'//CAN(IL),XSCAT(1))
         IF((ITYPE.EQ.2).OR.((ITYPE.EQ.5).AND.(ISPN.EQ.1))
     >                  .OR.((ITYPE.EQ.7).AND.(ISPN.EQ.1))) THEN
*----
*  CARTESIAN 3D SPN CASE.
*----
            NUM1=0
            DO 320 IR=1,NREG
            IBM=MATCOD(IR)
            IF(IBM.LE.0) GO TO 320
            IF(MOD(IL,2).EQ.0) THEN
               IND=(IL/2)*L4+IBASE(IKN+(NUM1+1)-1)
               JG=IJJ(IBM)
               DO 250 JND=1,NJJ(IBM)
               IF(JG.NE.IG) THEN
                  SOURCE(IND,IG)=SOURCE(IND,IG)+FACT*FLUX(IND,JG)*
     >            XSCAT(IPOS(IBM)+JND-1)
               ENDIF
  250          JG=JG-1
            ELSE
               DO 270 IC=1,2
               IND1=(IL/2)*L4+ABS(IBASE(IKN+(NUM1+1+IC)-1))
               S1=REAL(SIGN(1,IBASE(IKN+(NUM1+1+IC)-1)))
               DO 270 JC=1,2
               IND2=(IL/2)*L4+ABS(IBASE(IKN+(NUM1+1+JC)-1))
               IF((IBASE(IKN+(NUM1+1+IC)-1).NE.0).AND.
     1            (IBASE(IKN+(NUM1+1+JC)-1).NE.0)) THEN
                  S2=REAL(SIGN(1,IBASE(IKN+(NUM1+1+JC)-1)))
		  AUXX=S1*S2*FACT*R1DP(IC,JC)*VOL(IR)
                  JG=IJJ(IBM)
                  DO 260 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     SOURCE(IND1,IG)=SOURCE(IND1,IG)-AUXX*FLUX(IND2,JG)*
     1               XSCAT(IPOS(IBM)+JND-1)
                  ENDIF
  260             JG=JG-1
               ENDIF
  270          CONTINUE
               DO 290 IC=3,4
               IND1=(IL/2)*L4+ABS(IBASE(IKN+(NUM1+1+IC)-1))
               S1=REAL(SIGN(1,IBASE(IKN+(NUM1+1+IC)-1)))
               DO 290 JC=3,4
               IND2=(IL/2)*L4+ABS(IBASE(IKN+(NUM1+1+JC)-1))
               IF((IBASE(IKN+(NUM1+1+IC)-1).NE.0).AND.
     1            (IBASE(IKN+(NUM1+1+JC)-1).NE.0)) THEN
                  S2=REAL(SIGN(1,IBASE(IKN+(NUM1+1+JC)-1)))
	          AUXX=S1*S2*FACT*R1DP(IC-2,JC-2)*VOL(IR)
                  JG=IJJ(IBM)
                  DO 280 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     SOURCE(IND1,IG)=SOURCE(IND1,IG)-AUXX*FLUX(IND2,JG)*
     1               XSCAT(IPOS(IBM)+JND-1)
                  ENDIF
  280             JG=JG-1
               ENDIF
  290          CONTINUE
               DO 310 IC=5,6
               IND1=(IL/2)*L4+ABS(IBASE(IKN+(NUM1+1+IC)-1))
               S1=REAL(SIGN(1,IBASE(IKN+(NUM1+1+IC)-1)))
               DO 310 JC=5,6
               IND2=(IL/2)*L4+ABS(IBASE(IKN+(NUM1+1+JC)-1))
               IF((IBASE(IKN+(NUM1+1+IC)-1).NE.0).AND.
     1            (IBASE(IKN+(NUM1+1+JC)-1).NE.0)) THEN
                  S2=REAL(SIGN(1,IBASE(IKN+(NUM1+1+JC)-1)))
	          AUXX=S1*S2*FACT*R1DP(IC-4,JC-4)*VOL(IR)
                  JG=IJJ(IBM)
                  DO 300 JND=1,NJJ(IBM)
                  IF(JG.NE.IG) THEN
                     SOURCE(IND1,IG)=SOURCE(IND1,IG)-AUXX*FLUX(IND2,JG)*
     1               XSCAT(IPOS(IBM)+JND-1)
                  ENDIF
  300             JG=JG-1
               ENDIF
  310          CONTINUE
            ENDIF
            NUM1=NUM1+7
  320       CONTINUE
         ELSE
            CALL XABORT('TRIVSO: DISCRETIZATION NOT IMPLEMENTED(2).')
         ENDIF
  330    CONTINUE
      ENDIF
      IF(ITYPE.EQ.4) CALL RLSARA(BASE(IXXX))
      CALL RLSARA(BASE(IXX))
      CALL RLSARA(IBASE(IKN))
      RETURN
      END
