*DECK USSDRV
      SUBROUTINE USSDRV(IPLI0,IPTRK,IPLIB,IFTRAK,INDREC,CDOOR,IMPX,
     1 IGRMIN,IGRMAX,NGRP,NBMIX,NREG,NUN,NBISO,NL,NED,NDEL,LEAKSW,
     2 ITRANC,IPHASE,TITR,KSPH,NRES,NPASS,ICALC,ICORR,ISUBG,MAXST,MAT,
     3 VOL,KEYFLX,ISONAM,ISONRF,MIX,IEVOL,ITYPE,TN,DEN,LSHI,IAPT,IHSUF,
     4 IREX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for a resonance self-shielding calculation.
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s):A. Hebert
*
*Parameters: input
* IPLI0   pointer to the internal microscopic cross section library
*         builded by the self-shielding module (L_LIBRARY signature).
* IPTRK   pointer to the tracking (L_TRACK signature).
* IPLIB   pointer to the internal microscopic cross section library
*         with subgroups (L_LIBRARY signature).
* IFTRAK  unit number of the sequential binary tracking file.
* INDREC  access flag for the internal microscopic cross section library
*         builded by the self-shielding module (=1: IPLI0 access in
*         creation mode; =2: in modification mode).
* CDOOR   name of the geometry/solution operator.
* IMPX    print flag.
* IGRMIN  first group where the self-shielding is applied.
* IGRMAX  most thermal group where the self-shielding is applied.
* NGRP    number of energy groups.
* NBMIX   number of mixtures in the internal library.
* NREG    number of regions.
* NUN     number of unknowns per energy group.
* NBISO   number of isotopes specifications in the internal library.
* NL      number of Legendre orders required in the calculation
*         (NL=1 or higher).
* NED     number of extra vector edits.
* NDEL    number of delayed neutron precursor groups.
* LEAKSW  leakage flag (LEAKSW=.true. if neutron leakage through
*         external boundary is present).
* ITRANC  type of transport correction.
* IPHASE  type of flux solution (=1: use a native flux solution door;
*         =2: use collision probabilities).
* TITR    title.
* KSPH    SPH equivalence flag (=0 no SPH correction; =1 SPH correction
*         in the fuel).
* NRES    number of self-shielding zones, as given by LIB:.
* NPASS   number of outer iterations.
* ICALC   simplified self-shielding flag (=1: IPLI0 is containing &CALC
*         data. =0: no &CALC data).
* ICORR   mutual resonance shielding flag (=1: to suppress the model
*         in cases it is required in LIB operator).
* ISUBG   type of self-shielding model (=1: use physical probability
*         tables; =3: use original Ribon method; =4: use Ribon extended
*         method).
* MAXST   maximum number of fixed point iterations for the ST scattering
*         source.
*
*Parameters: scratch
* MAT
* VOL
* KEYFLX
* ISONAM
* ISONRF
* MIX
* IEVOL
* ITYPE
* TN
* DEN
* LSHI
* IAPT
* IHSUF
* IREX
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLI0,IPTRK,IPLIB,IFTRAK,INDREC,IMPX,IGRMIN,IGRMAX,NGRP,
     1 NBMIX,NREG,NUN,NBISO,NL,NED,NDEL,ITRANC,IPHASE,KSPH,NRES,NPASS,
     2 ICALC,ICORR,ISUBG,MAXST,MAT(NREG),KEYFLX(NREG),ISONAM(3,NBISO),
     3 ISONRF(3,NBISO),MIX(NBISO),IEVOL(NBISO),ITYPE(NBISO),LSHI(NBISO),
     4 IAPT(NBISO),IHSUF(NBISO),IREX(NBMIX)
      REAL VOL(NREG),TN(NBISO),DEN(NBISO)
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MAXRSS=300)
      CHARACTER HSMG*131,HCAL*12,HNAMIS*12,TEXT4*4,NAM1*4,FNAM1*4,
     1 NAM2*12,FNAM2*12
      INTEGER IPAR(NSTATE),IRSS(MAXRSS),IBASE(1)
      REAL TMPDAY(3)
      LOGICAL LBASE(1),LTEST
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1),LBASE(1))
*----
*  RECOVER USEFUL INFORMATION FROM INPUT OBJECTS.
*----
      CALL LCMGET(IPLIB,'ISOTOPESUSED',ISONAM)
      CALL LCMGET(IPLIB,'ISOTOPERNAME',ISONRF)
      CALL LCMGET(IPLIB,'ISOTOPESMIX',MIX)
      CALL LCMGET(IPLIB,'ISOTOPESTODO',IEVOL)
      CALL LCMGET(IPLIB,'ISOTOPESTYPE',ITYPE)
      CALL LCMGET(IPLIB,'ISOTOPESTEMP',TN)
*
      CALL LCMPUT(IPLI0,'ISOTOPESMIX',NBISO,1,MIX)
      CALL LCMPUT(IPLI0,'ISOTOPESTODO',NBISO,1,IEVOL)
      CALL LCMPUT(IPLI0,'ISOTOPESTYPE',NBISO,1,ITYPE)
      CALL LCMPUT(IPLI0,'ISOTOPESTEMP',NBISO,2,TN)
      IF(INDREC.EQ.1) THEN
         CALL LCMGET(IPLIB,'ISOTOPESDENS',DEN)
         CALL LCMPUT(IPLI0,'ISOTOPESDENS',NBISO,2,DEN)
      ELSE IF(INDREC.EQ.2) THEN
         CALL LCMGET(IPLI0,'ISOTOPESDENS',DEN)
      ENDIF
      CALL LCMGET(IPLIB,'ISOTOPESSHI',LSHI)
      CALL SETARA(BASE,NGRP+1,IENER)
      CALL LCMGET(IPLIB,'ENERGY',BASE(IENER))
      CALL LCMPUT(IPLI0,'ENERGY',NGRP+1,2,BASE(IENER))
      CALL LCMGET(IPLIB,'DELTAU',BASE(IENER))
      CALL LCMPUT(IPLI0,'DELTAU',NGRP,2,BASE(IENER))
      CALL RLSARA(BASE(IENER))
      DO 10 ISO=1,NBISO
      DO 5 I=1,NREG
      IF(MAT(I).EQ.MIX(ISO)) GO TO 10
    5 CONTINUE
      LSHI(ISO)=0
   10 CONTINUE
*
      IF(INDREC.EQ.1) THEN
*        COPY THE NON RESONANT ISOTOPES.
         CALL KDRCPU(TK1)
         DO 20 ISO=1,NBISO
         IF((LSHI(ISO).EQ.0).OR.(DEN(ISO).EQ.0.0)) THEN
            WRITE(HNAMIS,'(3A4)') (ISONAM(J,ISO),J=1,3)
            CALL LCMLEN(IPLI0,HNAMIS,ILEN,ITYLCM)
            IF(ILEN.NE.0) GO TO 20
            CALL LCMSIX(IPLIB,HNAMIS,1)
            CALL LCMSIX(IPLI0,HNAMIS,1)
            CALL LCMEQU(IPLIB,IPLI0)
            CALL LCMSIX(IPLI0,' ',2)
            CALL LCMSIX(IPLIB,' ',2)
         ENDIF
   20    CONTINUE
         CALL KDRCPU(TK2)
         IF(IMPX.GT.1) WRITE(6,'(/33H USSDRV: CPU TIME SPENT TO COPY T,
     1   26HHE NON-RESONANT ISOTOPES =,F8.1,8H SECOND.)') TK2-TK1
*
*        WRITE THE OUTPUT INTERNAL LIBRARY PARAMETERS.
         CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
         IPAR(8)=0
         IPAR(17)=0
         CALL LCMPUT(IPLI0,'STATE-VECTOR',NSTATE,1,IPAR)
         IF(NED.GT.0) THEN
            CALL SETARA(BASE,2*NED,IJCEDM)
            CALL LCMGET(IPLIB,'ADDXSNAME-P0',BASE(IJCEDM))
            CALL LCMPUT(IPLI0,'ADDXSNAME-P0',2*NED,3,BASE(IJCEDM))
            CALL RLSARA(BASE(IJCEDM))
         ENDIF
         CALL LCMLEN(IPLIB,'DEPL-CHAIN',ILENG,ITYLCM)
         IF(ILENG.NE.0) THEN
            CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
            CALL LCMSIX(IPLI0,'DEPL-CHAIN',1)
            CALL LCMEQU(IPLIB,IPLI0)
            CALL LCMSIX(IPLI0,' ',2)
            CALL LCMSIX(IPLIB,' ',2)
         ENDIF
      ENDIF
*----
*  RECOMPUTE THE VECTOR LSHI.
*----
      CALL SETARA(IBASE,NBISO,ILSHI2)
      NRES1=0
      NRES2=0
      NRES3=0
      DO 30 ISO=1,NBISO
      IF(LSHI(ISO).NE.0) NRES3=NRES3+1
   30 IBASE(ILSHI2+ISO-1)=0
      DO 80 INRS=1,NRES
   40 DENMAX=0.0
      KSOT=0
      DO 60 ISO=1,NBISO
      IF(IBASE(ILSHI2+ISO-1).EQ.0) THEN
         VOLISO=0.0
         DO 50 I=1,NREG
         IF(MAT(I).EQ.MIX(ISO)) VOLISO=VOLISO+VOL(I)
   50    CONTINUE
         IF((ABS(LSHI(ISO)).EQ.INRS).AND.(DEN(ISO)*VOLISO.GT.DENMAX))
     1   THEN
            KSOT=ISO
            DENMAX=DEN(ISO)*VOLISO
         ENDIF
      ENDIF
   60 CONTINUE
      IF(KSOT.GT.0) THEN
         NRES2=NRES2+1
         DO 70 ISO=1,NBISO
         LTEST=((ISONRF(1,ISO).EQ.ISONRF(1,KSOT)).AND.
     1      (ISONRF(2,ISO).EQ.ISONRF(2,KSOT)).AND.
     2      (ISONRF(3,ISO).EQ.ISONRF(3,KSOT)).AND.
     3      (ABS(LSHI(ISO)).EQ.INRS))
         LTEST=LTEST.OR.((ISONAM(1,ISO).EQ.ISONAM(1,KSOT)).AND.
     1      (ISONAM(2,ISO).EQ.ISONAM(2,KSOT)).AND.
     2      (ABS(LSHI(ISO)).EQ.INRS))
         IF(LTEST) IBASE(ILSHI2+ISO-1)=NRES2
         IF(LTEST.AND.(LSHI(ISO).EQ.-INRS)) THEN
            DO 65 JSO=1,NBISO
            IF(LSHI(JSO).EQ.LSHI(ISO)) IBASE(ILSHI2+JSO-1)=NRES2
   65       CONTINUE
         ENDIF
   70    CONTINUE
         GO TO 40
      ENDIF
      IF(NRES2.EQ.NRES1) THEN
         WRITE(HSMG,'(45HUSSDRV: NO RESONANT ISOTOPES IN RESONANT REGI,
     1   9HON NUMBER,I4,5H (1).)') INRS
         CALL XABORT(HSMG)
      ENDIF
   80 NRES1=NRES2
*----
* FIND THE ISOTOPE-NAME SUFFIX VALUES.
*----
      TEXT4=' '
      READ(TEXT4,'(A4)') IHBLK
      DO 90 ISO=1,NBISO
      IF((IBASE(ILSHI2+ISO-1).NE.0).AND.(DEN(ISO).NE.0.0)) THEN
         WRITE(TEXT4,'(I4.4)') MIX(ISO)
         READ(TEXT4,'(A4)') IHSUF(ISO)
      ELSE
         IHSUF(ISO)=IHBLK
      ENDIF
   90 CONTINUE
      IF(ICALC.EQ.1) THEN
         CALL LCMSIX(IPLI0,'SHIBA_SG',1)
         CALL LCMSIX(IPLI0,'-DATA-CALC-',1)
         NAM1=' '
         CALL LCMNXT(IPLI0,NAM1)
         FNAM1=NAM1
  100    CALL LCMSIX(IPLI0,NAM1,1)
            NAM2=' '
            CALL LCMNXT(IPLI0,NAM2)
            FNAM2=NAM2
  110       CALL LCMLEN(IPLI0,NAM2,NRSS,ITYLCM)
            CALL LCMGET(IPLI0,NAM2,IRSS)
            READ(NAM2,'(2A4)') IN1,IN2
            DO 130 ISO=1,NBISO
            IF((ISONAM(1,ISO).EQ.IN1).AND.(ISONAM(2,ISO).EQ.IN2).AND.
     1      (IBASE(ILSHI2+ISO-1).NE.0)) THEN
               IF((NRSS.EQ.1).AND.(IRSS(1).EQ.-999)) THEN
                  READ(NAM1,'(A4)') IHSUF(ISO)
               ELSE
                  DO 120 I=1,NRSS
                  IF(IRSS(I).EQ.MIX(ISO)) READ(NAM1,'(A4)') IHSUF(ISO)
  120             CONTINUE
               ENDIF
            ENDIF
  130       CONTINUE
            CALL LCMNXT(IPLI0,NAM2)
            IF(NAM2.EQ.FNAM2) GO TO 140
            GO TO 110
  140    CALL LCMSIX(IPLI0,' ',2)
         CALL LCMNXT(IPLI0,NAM1)
         IF(NAM1.EQ.FNAM1) THEN
            CALL LCMSIX(IPLI0,' ',2)
            CALL LCMSIX(IPLI0,' ',2)
            GO TO 150
         ENDIF
         GO TO 100
      ENDIF
*
  150 NPASS2=NPASS
      IF(NRES3.EQ.1) NPASS2=1
      DO 260 IPASS=1,NPASS2
      IF((IMPX.GT.0).AND.(NPASS2.GT.1)) WRITE (6,'(/15H USSDRV: SELF S,
     1 25HHIELDING ITERATION NUMBER,I4,1H.)') IPASS
      DO 260 INRS=1,NRES2
*----
*  COMPUTE THE NUMBER OF RESONANT ISOTOPES IN REGION INRS AND THE
*  RESONANT ISOTOPE INDEX ASSOCIATED TO EACH ISOTOPE SPECIFICATION.
*----
      NIRES=0
      DO 200 ISO=1,NBISO
      IAPT(ISO)=0
      IF((IBASE(ILSHI2+ISO-1).EQ.INRS).AND.(DEN(ISO).NE.0.0)) THEN
         DO 170 I=1,NREG
         IF(MAT(I).EQ.MIX(ISO)) GO TO 180
  170    CONTINUE
         GO TO 200
  180    DO 190 JSO=1,ISO-1
         IF((ISONAM(1,ISO).EQ.ISONAM(1,JSO)).AND.
     1      (ISONAM(2,ISO).EQ.ISONAM(2,JSO)).AND.
     2      (ISONAM(3,ISO).EQ.ISONAM(3,JSO)).AND.
     3      (IBASE(ILSHI2+JSO-1).EQ.INRS).AND.
     4      (DEN(JSO).NE.0.0).AND.(IAPT(JSO).NE.0)) THEN
              IAPT(ISO)=IAPT(JSO)
              GO TO 200
         ENDIF
  190    CONTINUE
         IIII=ISO
         NIRES=NIRES+1
         IAPT(ISO)=NIRES
      ENDIF
  200 CONTINUE
      IF(NIRES.EQ.0) THEN
         WRITE(HSMG,'(45HUSSDRV: NO RESONANT ISOTOPES IN RESONANT REGI,
     1   9HON NUMBER,I4,5H (2).)') INRS
         CALL XABORT(HSMG)
      ENDIF
      WRITE(HCAL,'(1HC,I5.5,1H/,I5.5)') IIII,NBISO
      IF(IMPX.GT.0) WRITE (6,'(/35H USSDRV: PERFORMING SELF-SHIELDING ,
     1 18HCALCULATION NAMED ,A12,1H.)') HCAL
*----
*  FIND THE NUMBER OF FUEL REGIONS AND THE FUEL REGION INDICES ASSIGNED
*  TO EACH RESONANT MIXTURE.
*----
      NBNRS=0
      DO 210 IBM=1,NBMIX
  210 IREX(IBM)=0
      DO 230 ISO=1,NBISO
      IBM=MIX(ISO)
      IF((IAPT(ISO).GT.0).AND.(IREX(IBM).EQ.0)) THEN
         DO 220 JSO=1,ISO-1
         IF((IHSUF(JSO).EQ.IHSUF(ISO)).AND.(IAPT(JSO).EQ.IAPT(ISO)))
     1   THEN
            IREX(IBM)=IREX(MIX(JSO))
            GO TO 230
         ENDIF
  220    CONTINUE
         NBNRS=NBNRS+1
         IREX(IBM)=NBNRS
      ENDIF
  230 CONTINUE
      IF(NBNRS.EQ.0) THEN
         WRITE (HSMG,'(33HUSSDRV: INVALID RESONANT REGION =,I10)') INRS
         CALL XABORT(HSMG)
      ENDIF
      IF(IMPX.GE.1) WRITE(6,410) NIRES,NBNRS,INRS
*----
*  DETERMINE WHICH MODERATOR ISOTOPES ARE MIXED WITH RESONANT ONES.
*----
      DO 250 ISO=1,NBISO
      IF((IAPT(ISO).EQ.0).AND.(IREX(MIX(ISO)).GT.0)) IAPT(ISO)=NIRES+1
  250 CONTINUE
*----
*  PERFORM A SELF-SHIELDING CALCULATION NAMED HCAL.
*----
      CALL SETARA(BASE,3*NBISO,IISOBI)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP,ISPH)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP,IPHGAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP,ISTGAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP,ISFGAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP*NL,ISSGAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP*NGRP*NL,IS0GAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP*NED,ISAGAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP*NDEL,ISDGAR)
      CALL SETARA(BASE,NBNRS*NIRES*NGRP,ISWGAR)
      CALL SETARA(BASE,NBISO,IMASKI)
      CALL SETARA(BASE,NGRP,IMASKG)
      CALL USSONE(IPLI0,IPTRK,IPLIB,IFTRAK,CDOOR,IMPX,IGRMIN,IGRMAX,
     1 NIRES,NBNRS,IREX,NGRP,NBMIX,NREG,NUN,NBISO,NL,NED,NDEL,ISONAM,
     2 IHSUF,HCAL,DEN,MIX,IAPT,MAT,VOL,KEYFLX,LEAKSW,ITRANC,IPHASE,
     3 TITR,KSPH,ICORR,ISUBG,MAXST,BASE(IISOBI),BASE(ISPH),BASE(IPHGAR),
     4 BASE(ISTGAR),BASE(ISFGAR),BASE(ISSGAR),BASE(IS0GAR),BASE(ISAGAR),
     5 BASE(ISDGAR),BASE(ISWGAR),BASE(IMASKI),BASE(IMASKG))
      CALL RLSARA(BASE(IMASKG))
      CALL RLSARA(BASE(IMASKI))
      CALL RLSARA(BASE(ISWGAR))
      CALL RLSARA(BASE(ISDGAR))
      CALL RLSARA(BASE(ISAGAR))
      CALL RLSARA(BASE(IS0GAR))
      CALL RLSARA(BASE(ISSGAR))
      CALL RLSARA(BASE(ISFGAR))
      CALL RLSARA(BASE(ISTGAR))
      CALL RLSARA(BASE(IPHGAR))
      CALL RLSARA(BASE(ISPH))
      CALL RLSARA(BASE(IISOBI))
  260 CONTINUE
      CALL RLSARA(IBASE(ILSHI2))
      IF(IMPX.GE.3) CALL LCMLIB(IPLI0)
*----
*  BUILD THE MACROLIB IN THE OUTPUT INTERNAL LIBRARY.
*----
      CALL SETARA(LBASE,NBMIX,IMASK)
      DO 280 IBM=1,NBMIX
      LBASE(IMASK+IBM-1)=.TRUE.
      DO 270 I=1,NREG
      IF(MAT(I).EQ.IBM) GO TO 280
  270 CONTINUE
      LBASE(IMASK+IBM-1)=.FALSE.
  280 CONTINUE
      CALL SETARA(LBASE,NGRP,IMASKL)
      DO 290 I=1,NGRP
  290 LBASE(IMASKL+I-1)=.TRUE.
*
      ITSTMP=0
      TMPDAY(1)=0.0
      TMPDAY(2)=0.0
      TMPDAY(3)=0.0
      CALL KDRCPU(TK1)
      CALL LCMGET(IPLI0,'ISOTOPESUSED',ISONAM)
      CALL LIBMIX(IPLI0,NBMIX,NGRP,NBISO,ISONAM,MIX,DEN,LBASE(IMASK),
     1 LBASE(IMASKL),ITSTMP,TMPDAY)
      CALL KDRCPU(TK2)
      IF(IMPX.GT.1) WRITE(6,'(/37H USSDRV: CPU TIME SPENT TO BUILD THE ,
     1 19HEMBEDDED MACROLIB =,F8.1,8H SECOND.)') TK2-TK1
      CALL RLSARA(LBASE(IMASKL))
      CALL RLSARA(LBASE(IMASK))
      RETURN
*
  410 FORMAT(/48H USSDRV: NUMBER OF CORRELATED RESONANT ISOTOPES=,I4/9X,
     1 35HNUMBER OF CORRELATED FUEL MIXTURES=,I4,19H IN RESONANT REGION,
     2 I3)
      END
