*DECK USSONE
      SUBROUTINE USSONE(IPLI0,IPTRK,IPLIB,IFTRAK,CDOOR,IMPX,IGRMIN,
     1 IGRMAX,NIRES,NBNRS,IREX,NGRP,NBMIX,NREG,NUN,NBISO,NL,NED,NDEL,
     2 ISONAM,IHSUF,HCAL,DEN,MIX,IAPT,MAT,VOL,KEYFLX,LEAKSW,ITRANC,
     3 IPHASE,TITR,KSPH,ICORR,ISUBG,MAXST,ISOBIS,SPH,PHGAR,STGAR,SFGAR,
     4 SSGAR,S0GAR,SAGAR,SDGAR,SWGAR,MASKI,MASKG)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform a resonance self-shielding calculation named HCAL and build
* a corresponding internal library.
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLI0   pointer to the internal microscopic cross section library
*         builded by the self-shielding module (L_LIBRARY signature).
* IPTRK   pointer to the tracking. (L_TRACK signature).
* IPLIB   pointer to the internal microscopic cross section library
*         with subgroups (L_LIBRARY signature).
* IFTRAK  unit number of the sequential binary tracking file.
* CDOOR   name of the geometry/solution operator.
* IMPX    print flag (equal to zero for no print).
* IGRMIN  first group where the self-shielding is applied.
* IGRMAX  most thermal group where the self-shielding is applied.
* NIRES   number of correlated resonant isotopes in fuel regions.
* NBNRS   number of correlated fuel regions. Note that NBNRS=max(IREX).
* IREX    fuel region index assigned to each mixture. Equal to zero
*         in non-resonant mixtures or in mixtures not used.
* NGRP    number of energy groups.
* NBMIX   number of mixtures in the internal library.
* NREG    number of regions.
* NUN     number of unknowns per energy group.
* NBISO   number of isotopes specifications in the internal library.
* NL      number of Legendre orders required in the calculation
*         (NL=1 or higher).
* NED     number of extra vector edits.
* NDEL    number of delayed neutron precursor groups.
* ISONAM  alias name of isotopes.
* IHSUF   suffix name of isotopes.
* HCAL    name of the self-shielding calculation.
* DEN     density of each isotope.
* MIX     mix number of each isotope (can be zero).
* IAPT    resonant isotope index associated with isotope I. Mixed
*         moderator if IAPT(I)=NIRES+1. Out-of-fuel isotope if
*         IAPT(I)=0.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  pointers of fluxes in unknown vector.
* LEAKSW  leakage flag (LEAKSW=.true. if neutron leakage through
*         external boundary is present).
* ITRANC  type of transport correction.
* IPHASE  type of flux solution (=1: use a native flux solution door;
*         =2: use collision probabilities).
* TITR    title.
* KSPH    SPH equivalence flag (=0 no SPH correction; =1 SPH correction
*         in the fuel).
* ICORR   mutual resonance shielding flag (=1: to suppress the model
*         in cases it is required in LIB operator).
* ISUBG   type of self-shielding model (=1: use physical probability
*         tables; =3: use original Ribon method; =4: use Ribon extended
*         method).
* MAXST   maximum number of fixed point iterations for the ST scattering
*         source.
*
*Parameters: scratch
* ISOBIS
* SPH
* PHGAR
* STGAR
* SFGAR
* SSGAR
* S0GAR
* SAGAR
* SDGAR
* SWGAR
* MASKI
* MASKG
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLI0,IPTRK,IPLIB,IFTRAK,IMPX,IGRMIN,IGRMAX,NIRES,
     1 NBNRS,IREX(NBMIX),NGRP,NBMIX,NREG,NUN,NBISO,NL,NED,NDEL,
     2 ISONAM(3,NBISO),IHSUF(NBISO),MIX(NBISO),IAPT(NBISO),MAT(NREG),
     3 KEYFLX(NREG),ITRANC,IPHASE,KSPH,ICORR,ISUBG,MAXST,ISOBIS(3,NBISO)
      REAL DEN(NBISO),VOL(NREG),SPH(NBNRS,NIRES,NGRP),
     1 PHGAR(NBNRS,NIRES,NGRP),STGAR(NBNRS,NIRES,NGRP),
     2 SFGAR(NBNRS,NIRES,NGRP),SSGAR(NBNRS,NIRES,NL,NGRP),
     3 S0GAR(NBNRS,NIRES,NL,NGRP,NGRP),SAGAR(NBNRS,NIRES,NED,NGRP),
     4 SDGAR(NBNRS,NIRES,NDEL,NGRP),SWGAR(NBNRS,NIRES,NGRP)
      LOGICAL LEAKSW,MASKI(NBISO),MASKG(NGRP)
      CHARACTER CDOOR*12,HCAL*12,TITR*72
*----
*  LOCAL VARIABLES
*----
      PARAMETER(MAXNOR=12)
      COMMON BASE(1)
*----
*  FIND THE NEW ISOTOPE NAMES IN IPLI0.
*----
      CALL LCMLEN(IPLI0,'ISOTOPESUSED',ILONG,ITYLCM)
      IF(ILONG.NE.0) THEN
         CALL LCMGET(IPLI0,'ISOTOPESUSED',ISOBIS)
      ELSE
         CALL LCMGET(IPLIB,'ISOTOPESUSED',ISOBIS)
      ENDIF
      DO 10 ISO=1,NBISO
      IF((IAPT(ISO).GT.0).AND.(IAPT(ISO).LE.NIRES)) THEN
         ISOBIS(3,ISO)=IHSUF(ISO)
      ENDIF
   10 CONTINUE
      CALL LCMPUT(IPLI0,'ISOTOPESUSED',3*NBISO,3,ISOBIS)
*
      CALL SETARA(BASE,4*NBMIX*(NIRES+1)*NGRP,ISIGAR)
      CALL SETARA(BASE,NGRP*NL,IISMIN)
      CALL SETARA(BASE,NGRP*NL,IISMAX)
      CALL SETARA(BASE,NREG*NIRES*NGRP,IUNGAR)
*----
*  COMPUTE THE NEUTRON FLUX.
*----
      CALL SETARA(BASE,NGRP*NIRES,IGOLD)
      CALL SETARA(BASE,NGRP,IGAS)
      CALL SETARA(BASE,NGRP,IGA1)
      CALL SETARA(BASE,NGRP*NGRP,IGA2)
      CALL SETARA(BASE,NGRP*NIRES,INOR)
      CALL SETARA(BASE,NIRES,IIPPT1)
      CALL SETARA(BASE,NIRES*4,IIPPT2)
      CALL SETARA(BASE,NGRP,IIWRK)
      CALL SETARA(BASE,2*NL,IISM)
      CALL SETARA(BASE,(NBNRS+1)*MAXNOR*NL*NIRES,IXFLUX)
      CALL SETARA(BASE,NBNRS*NIRES,ICONR)
      CALL SETARA(BASE,NBNRS+1,IVOLME)
      CALL SETARA(BASE,NGRP,IDELTA)
      CALL SETARA(BASE,NGRP,IMASKH)
      CALL USSFLU(MAXNOR,IPTRK,IPLIB,IPLI0,IFTRAK,NREG,NUN,NBMIX,NBISO,
     1 NIRES,NL,NED,NDEL,ISONAM,ISOBIS,HCAL,MAT,VOL,KEYFLX,CDOOR,LEAKSW,
     2 IMPX,DEN,MIX,IAPT,ITRANC,IPHASE,NGRP,IGRMIN,IGRMAX,NBNRS,IREX,
     3 TITR,ICORR,ISUBG,MAXST,BASE(IUNGAR),PHGAR,STGAR,SFGAR,SSGAR,
     4 S0GAR,SAGAR,SDGAR,SWGAR,BASE(IISMIN),BASE(IISMAX),MASKG,
     5 BASE(IGOLD),BASE(IGAS),BASE(IGA1),BASE(IGA2),BASE(INOR),
     6 BASE(IIPPT1),BASE(IIPPT2),BASE(IIWRK),BASE(ISIGAR),BASE(IISM),
     7 BASE(IXFLUX),BASE(ICONR),BASE(IVOLME),BASE(IDELTA),BASE(IMASKH))
      CALL RLSARA(BASE(IMASKH))
      CALL RLSARA(BASE(IDELTA))
      CALL RLSARA(BASE(IVOLME))
      CALL RLSARA(BASE(ICONR))
      CALL RLSARA(BASE(IXFLUX))
      CALL RLSARA(BASE(IISM))
      CALL RLSARA(BASE(IIWRK))
      CALL RLSARA(BASE(IIPPT2))
      CALL RLSARA(BASE(IIPPT1))
      CALL RLSARA(BASE(INOR))
      CALL RLSARA(BASE(IGA2))
      CALL RLSARA(BASE(IGA1))
      CALL RLSARA(BASE(IGAS))
      CALL RLSARA(BASE(IGOLD))
*----
*  COMPUTE THE SPH FACTORS.
*----
      DO 20 IGRP=1,NGRP
      DO 20 IRES=1,NIRES
      DO 20 IND=1,NBNRS
   20 SPH(IND,IRES,IGRP)=1.0
      IF(KSPH.EQ.1) THEN
         CALL SETARA(BASE,NBMIX*5*NGRP,ISIGTI)
         CALL SETARA(BASE,NBNRS,IVOLME)
         CALL SETARA(BASE,NBMIX+1,ISIGTX)
         CALL SETARA(BASE,NBMIX+1,ISIS0X)
         CALL SETARA(BASE,NBNRS,IFLNEW)
         CALL SETARA(BASE,NUN*NGRP,ISUNKN)
         CALL SETARA(BASE,NUN*NGRP,IFUNKN)
         CALL SETARA(BASE,NGRP,INPSYS)
         CALL SETARA(BASE,NGRP,IITT)
         CALL USSSPH(IPLI0,IPTRK,IFTRAK,NREG,NUN,NBMIX,NBISO,NIRES,NL,
     1   NED,NDEL,ISONAM,HCAL,MAT,VOL,KEYFLX,CDOOR,LEAKSW,IMPX,DEN,MIX,
     2   IAPT,ITRANC,IPHASE,NGRP,MASKG,NBNRS,IREX,TITR,ISUBG,
     3   BASE(ISIGAR),BASE(IISMIN),BASE(IISMAX),BASE(IUNGAR),PHGAR,
     4   STGAR,SFGAR,SSGAR,S0GAR,SAGAR,SDGAR,SWGAR,SPH,BASE(ISIGTI),
     5   BASE(IVOLME),BASE(ISIGTX),BASE(ISIS0X),BASE(IFLNEW),
     6   BASE(ISUNKN),BASE(IFUNKN),BASE(INPSYS),BASE(IITT))
         CALL RLSARA(BASE(IITT))
         CALL RLSARA(BASE(INPSYS))
         CALL RLSARA(BASE(IFUNKN))
         CALL RLSARA(BASE(ISUNKN))
         CALL RLSARA(BASE(IFLNEW))
         CALL RLSARA(BASE(ISIS0X))
         CALL RLSARA(BASE(ISIGTX))
         CALL RLSARA(BASE(IVOLME))
         CALL RLSARA(BASE(ISIGTI))
      ENDIF
*
      CALL RLSARA(BASE(IUNGAR))
      CALL RLSARA(BASE(IISMAX))
      CALL RLSARA(BASE(IISMIN))
      CALL RLSARA(BASE(ISIGAR))
*----
*  CREATE THE SELF-SHIELDED INTERNAL LIBRARY USING A SIMPLE
*  TRANSCRIPTION OF THE SELF-SHIELDED CROSS SECTIONS.
*----
      CALL KDRCPU(TK1)
*     SIMPLE TRANSCRIPTION OF THE SELF-SHIELDED CROSS SECTIONS.
      DO 100 ISO=1,NBISO
  100 MASKI(ISO)=(IAPT(ISO).GT.0).AND.(IAPT(ISO).LE.NIRES)
      DO 120 ISO=1,NBISO
      IF(MASKI(ISO)) THEN
         DO 110 JSO=ISO+1,NBISO
         IF((ISOBIS(1,ISO).EQ.ISOBIS(1,JSO)).AND.
     1      (ISOBIS(2,ISO).EQ.ISOBIS(2,JSO)).AND.
     2      (ISOBIS(3,ISO).EQ.ISOBIS(3,JSO))) MASKI(JSO)=.FALSE.
  110    CONTINUE
      ENDIF
  120 CONTINUE
      CALL SETARA(BASE,NGRP,IGAR1)
      CALL SETARA(BASE,NGRP*NGRP,IGAR2)
      CALL SETARA(BASE,NGRP+1,IENER)
      CALL USSIN1(IPLI0,IPLIB,NGRP,NBMIX,NBISO,NREG,NIRES,NBNRS,
     1 NL,NED,NDEL,IREX,IMPX,ISONAM,ISOBIS,DEN,MIX,IAPT,MASKI,SPH,
     2 MAT,VOL,PHGAR,STGAR,SFGAR,SSGAR,S0GAR,SAGAR,SDGAR,BASE(IGAR1),
     3 BASE(IGAR2),BASE(IENER))
      CALL RLSARA(BASE(IENER))
      CALL RLSARA(BASE(IGAR2))
      CALL RLSARA(BASE(IGAR1))
      CALL KDRCPU(TK2)
      IF(IMPX.GT.1) WRITE(6,'(/36H USSONE: CPU TIME SPENT TO BUILD THE,
     1 33H SELF-SHIELDED INTERNAL LIBRARY =,F8.1,8H SECOND.)') TK2-TK1
      RETURN
      END
