*DECK XDRTBH
      SUBROUTINE XDRTBH(IPGEOM,IPTRK,IQUA10,IBIHET,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover the double-heterogeneity (Bihet) data from the geometry
* object IPGEOM and update the tracking object IPTRK.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the excell tracking (L_TRACK).
* IPGEOM  pointer to the geometry (L_GEOM).
* IQUA10  quadrature parameter for the double heterogeneity option.
* IBIHET  type of double-heterogeneity method (=1: Sanchez-Pomraning
*         model; =2: Hebert model).
* IMPX    tracking print level.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPGEOM,IPTRK,IQUA10,IBIHET,IMPX
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      INTEGER ISTRAK(NSTATE),ISTATE(NSTATE),IPARAM(8),KCHAR(18)
      REAL BASE(1)
      CHARACTER CDOOR*12
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
      IF(IQUA10.EQ.0) CALL XABORT('XDRTBH: INVALID IQUA10.')
      IF(IBIHET.EQ.0) CALL XABORT('XDRTBH: INVALID IBIHET.')
*
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTRAK)
      NREG2=ISTRAK(1)
      NUN2=ISTRAK(2)
      IR2=ISTRAK(4)
      CALL LCMSIX(IPGEOM,'BIHET',1)
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      NG=ISTATE(1)
      NSMAX=ISTATE(2)-1
      CALL SETARA(IBASE,NG,INS)
      CALL LCMGET(IPGEOM,'NS',IBASE(INS))
      NSMAX=0
      DO 10 I=1,NG
   10 NSMAX=MAX(NSMAX,IBASE(INS+I-1))
*
      CALL SETARA(IBASE,NREG2,IIBI)
      CALL SETARA(BASE,NG*(1+NSMAX),IRS)
      CALL SETARA(BASE,NG*IR2,IFRACT)
      CALL SETARA(BASE,NG*NSMAX,IVOLK)
*
      MAXPTS=NREG2*(NSMAX+1)*NG
      CALL SETARA(IBASE,MAXPTS,IMAT)
      CALL SETARA(IBASE,MAXPTS,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',IBASE(IVOL))
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMPUT(IPTRK,'IBI',NREG2,1,IBASE(IMAT))
      CALL LCMPUT(IPTRK,'VOLUME',NREG2,2,IBASE(IVOL))
      CALL LCMSIX(IPTRK,' ',2)
*
      DO 20 I=1,NREG2
   20 IBASE(IIBI+I-1)=IBASE(IMAT+I-1)
*----
*  RECOVER DOUBLE-HETEROGENEITY INFORMATION FROM GEOMETRY OBJECT
*----
      CALL SETARA(BASE,IR2,IIDIL)
      CALL SETARA(BASE,NG*NSMAX*IR2,IMIXGR)
      CALL SETARA(BASE,IR2,IMILIE)
      CALL SETARA(BASE,IR2,IFTEMP)
      CALL READBH(MAXPTS,IPGEOM,IR1,IR2,NREG,NREG2,IBASE(IMAT),
     1 IBASE(IVOL),NG,NSMAX,MICRO,IBASE(INS),IBASE(IIBI),BASE(IRS),
     2 BASE(IFRACT),BASE(IVOLK),IMPX,BASE(IIDIL),BASE(IMIXGR),
     3 BASE(IMILIE),BASE(IFTEMP))
      CALL RLSARA(BASE(IFTEMP))
      CALL RLSARA(BASE(IMILIE))
      CALL RLSARA(IBASE(IIBI))
      IF(IMPX.GE.1) THEN
         WRITE (6,'(/" QUADRATURE PARAMETER FOR THE MICRO STRUC",
     1   "TURES =",I2/)') IQUA10
         WRITE (6,'(" TYPE OF DOUBLE HETEROGENEITY MODEL (1/2: ",
     1   "SANCHEZ-POMRANING/HEBERT)=",I2/)') IBIHET
      ENDIF
      CALL LCMSIX(IPGEOM,' ',2)
*----
*  RESET STATE-VECTOR INFORMATION
*----
      IPARAM(1)=IR1
      IPARAM(2)=IR2
      IPARAM(3)=NREG2
      IPARAM(4)=NG
      IPARAM(5)=NSMAX
      IPARAM(6)=IBIHET
      IPARAM(7)=MICRO
      IPARAM(8)=IQUA10
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMPUT(IPTRK,'PARAM',8,1,IPARAM)
      CALL LCMPOF(IPTRK,'NS',NG,1,INS)
      CALL LCMPOF(IPTRK,'RS',NG*(1+NSMAX),2,IRS)
      CALL LCMPOF(IPTRK,'FRACT',NG*IR2,2,IFRACT)
      CALL LCMPOF(IPTRK,'VOLK',NG*NSMAX,2,IVOLK)
      CALL LCMPOF(IPTRK,'IDIL',IR2-IR1,1,IIDIL)
      CALL LCMPOF(IPTRK,'MIXGR',NG*NSMAX*(IR2-IR1),1,IMIXGR)
      CALL LCMSIX(IPTRK,' ',2)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTRAK)
      ISTRAK(1)=NREG
      ISTRAK(2)=NUN2+(NREG-NREG2)
      ISTRAK(4)=IR1
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,ISTRAK)
      CALL LCMPUT(IPTRK,'MATCOD',NREG,1,IBASE(IMAT))
      CALL LCMPUT(IPTRK,'VOLUME',NREG,2,IBASE(IVOL))
      CALL RLSARA(IBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
*----
*  RESET KEYFLX
*----
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(CDOOR.EQ.'MCCG') THEN
         CALL LCMLEN(IPTRK,'KEYFLX',LKFL,ITYLCM)
         NFUNL=LKFL/NREG2
      ELSE
         NFUNL=1
      ENDIF
      CALL SETARA(IBASE,NREG*NFUNL,IKEYF1)
      CALL SETARA(IBASE,NREG2*NFUNL,IKEYF2)
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKEYF2))
      CALL XDISET(IBASE(IKEYF1),NREG*NFUNL,0)
      DO 30 INF=1,NFUNL
      DO 30 I=1,NREG2
      IOF1=(INF-1)*NREG+I
      IOF2=(INF-1)*NREG2+I
   30 IBASE(IKEYF1+IOF1-1)=IBASE(IKEYF2+IOF2-1)
      IUNK=NUN2
      DO 40 I=NREG2+1,NREG
      IUNK=IUNK+1
   40 IBASE(IKEYF1+I-1)=IUNK
      CALL LCMPUT(IPTRK,'KEYFLX',NREG*NFUNL,1,IBASE(IKEYF1))
      CALL RLSARA(IBASE(IKEYF2))
      CALL RLSARA(IBASE(IKEYF1))
      RETURN
      END
