*DECK XELPR3
      SUBROUTINE XELPR3(IPTRK,IZ,NZP)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Create 2D projection (EXCELT geometry analysis) of a 3D prismatic geometry
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): R. Le Tellier
*
*Parameters: input
* IPTRK   pointer to the excell tracking (L_TRACK).
* IZ      projection axis.
*Parameters: output
* NZP     number of IZ-plans.
*-----------------------------------------------------------------------
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IZ,NZP
*----
*  LOCAL VARIABLES
*----
      INTEGER NSTATE,IOUT
      PARAMETER(NSTATE=40,IOUT=6)
      INTEGER GSTATE(NSTATE),ESTATE(NSTATE),ICODE(6),NCODE(6),KSIGN(3),
     1 KTYPE(3),LCLSYM(3)
      INTEGER IX,IY,NDIM,N3MS,N3MR,LDIM,LMESH,N3RS,I,N2MS,N2MR,
     1 N2RS,N3R,N3S,NFI
      INTEGER MINDIM,MAXDIM,ICORD,REMESH,VOLSUR,MATALB,KEYMRG,INDEX,
     1 MIN2,MAX2,ICOR2,REM2,VOL2,MAT2,KEY2,IND2,IND2T3,MATMRG,VOLMRG,
     2 IZCOR,ITEMP,ITEMP2
      REAL ALBEDO(6)
*
      INTEGER IBASE(1)
      REAL BASE
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*---
      IF (IZ.EQ.3) THEN
         IX=1
         IY=2
      ELSEIF (IZ.EQ.2) THEN
         IX=3
         IY=1
      ELSEIF (IZ.EQ.1) THEN
         IX=2
         IY=3
      ELSE
         CALL XABORT('XELPR3: ILLEGAL PROJECTION AXIS')
      ENDIF
*---
* RECOVER INFORMATION FROM EXCELL 3D GEOMETRY ANALYSIS
*---
      CALL LCMGET(IPTRK,'SIGNATURE',KSIGN)
      CALL LCMGET(IPTRK,'TRACK-TYPE',KTYPE)
      CALL XDISET(GSTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',GSTATE)
      CALL LCMGET(IPTRK,'ICODE',ICODE)
      CALL LCMGET(IPTRK,'NCODE',NCODE)
      CALL LCMPUT(IPTRK,'NCODE',6,1,NCODE)     
      CALL LCMGET(IPTRK,'ALBEDO',ALBEDO)
      CALL LCMSIX(IPTRK,'EXCELL',1)
      CALL XDISET(ESTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ESTATE)
      NDIM=ESTATE(1)
      IF (NDIM.NE.3) 
     1   CALL XABORT('XELPR3: NON 3D GEOMETRY')
      N3MS=ESTATE(2)
      N3MR=ESTATE(3)
      LDIM=ESTATE(4)
      LMESH=ESTATE(5)
      N3RS=ESTATE(6)
      LCLSYM(1)=ESTATE(8)
      LCLSYM(2)=ESTATE(9)
      LCLSYM(3)=ESTATE(10)      
      CALL SETARA(IBASE,LDIM,MINDIM)
      CALL SETARA(IBASE,LDIM,MAXDIM)
      CALL SETARA(IBASE,LDIM,ICORD)
      CALL SETARA(BASE,LMESH,REMESH)
      CALL SETARA(BASE,N3RS,VOLSUR)
      CALL SETARA(IBASE,N3RS,MATALB)
      CALL SETARA(IBASE,N3RS,KEYMRG)
      CALL SETARA(IBASE,4*N3RS,INDEX)
      CALL LCMGET(IPTRK,'MINDIM',IBASE(MINDIM))
      CALL LCMGET(IPTRK,'MAXDIM',IBASE(MAXDIM))
      CALL LCMGET(IPTRK,'ICORD',IBASE(ICORD))
      CALL LCMGET(IPTRK,'REMESH',BASE(REMESH))
      CALL LCMGET(IPTRK,'VOLSUR',BASE(VOLSUR))
      CALL LCMGET(IPTRK,'MATALB',IBASE(MATALB))
      CALL LCMGET(IPTRK,'KEYMRG',IBASE(KEYMRG))
      CALL LCMGET(IPTRK,'INDEX',IBASE(INDEX))
      CALL LCMSIX(IPTRK,' ',2)
*--- 
* CHECK FOR CYLINDER ORIENTATION
*---
      IF (LDIM.GT.3) THEN
         DO I=3,LDIM-1
            IF (IBASE(ICORD+I).NE.IZ)
     1         CALL XABORT('XELPR3: NON Z-PRISMATIC GEOMETRY')
         ENDDO
      ENDIF
*---
* CONSTRUCT 2D GEOMETRY ANALYSIS AND (2D,Z)->3D INDEX 
*---
      CALL LCMSIX(IPTRK,'PROJECTION',1)
      CALL LCMPUT(IPTRK,'SIGNATURE',3,3,KSIGN)
      CALL LCMPUT(IPTRK,'TRACK-TYPE',3,3,KTYPE)
      NZP=IBASE(MAXDIM-1+IZ)-IBASE(MINDIM-1+IZ)
      N2MR=N3MR/NZP
      N2MS=(N3MS-2*N2MR)/NZP
      N2RS=N2MR+N2MS+1
      GSTATE(1)=N2MR
      GSTATE(2)=N2MR
      GSTATE(5)=N2MS
      GSTATE(7)=1
      GSTATE(8)=1
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,GSTATE)
      CALL LCMPUT(IPTRK,'ICODE',6,1,ICODE)
      CALL LCMPUT(IPTRK,'NCODE',6,1,NCODE)
      CALL LCMPUT(IPTRK,'ALBEDO',6,2,ALBEDO)
      CALL LCMSIX(IPTRK,'EXCELL',1)
      CALL SETARA(IBASE,LDIM,MIN2)
      CALL SETARA(IBASE,LDIM,MAX2)
      CALL SETARA(IBASE,LDIM,ICOR2)
      CALL SETARA(BASE,LMESH,REM2)
      CALL SETARA(BASE,N2RS,VOL2)
      CALL SETARA(IBASE,N2RS,MAT2)
      CALL SETARA(IBASE,N2RS,KEY2)
      CALL SETARA(IBASE,4*N2RS,IND2)
      CALL SETARA(IBASE,N2RS*(NZP+2),IND2T3)
      CALL SETARA(BASE,(NZP+1),IZCOR)
      CALL SETARA(IBASE,N3RS,MATMRG)
      CALL SETARA(BASE,N3RS,VOLMRG)
      CALL SETARA(IBASE,LMESH,ITEMP)
      CALL SETARA(IBASE,LMESH,ITEMP2)
      CALL XEL3T2(IX,IY,IZ,LDIM,N3MS,N3MR,N3RS,LMESH,NZP,N2MS,N2MR,N2RS,
     1     N3S,N3R,NFI,IBASE(MINDIM),IBASE(MAXDIM),IBASE(ICORD),
     2     BASE(REMESH),BASE(VOLSUR),IBASE(MATALB),IBASE(KEYMRG),
     3     IBASE(INDEX),IBASE(MAX2),IBASE(MIN2),IBASE(ICOR2),BASE(REM2),
     4     BASE(VOL2),IBASE(MAT2),IBASE(KEY2),IBASE(IND2),IBASE(IND2T3),
     5     IBASE(MATMRG),BASE(VOLMRG),BASE(IZCOR),IBASE(ITEMP),
     6     IBASE(ITEMP2))
      ESTATE(1)=2
      ESTATE(2)=N2MS
      ESTATE(3)=N2MR
      ESTATE(6)=N2RS
      ESTATE(8)=LCLSYM(IX)
      ESTATE(9)=LCLSYM(IY)
      ESTATE(10)=LCLSYM(IZ)
      CALL LCMPUT(IPTRK,'MINDIM',LDIM,1,IBASE(MIN2))
      CALL LCMPUT(IPTRK,'MAXDIM',LDIM,1,IBASE(MAX2))
      CALL LCMPUT(IPTRK,'ICORD',LDIM,1,IBASE(ICOR2))
      CALL LCMPUT(IPTRK,'INDEX',4*N2RS,1,IBASE(IND2))
      CALL LCMPUT(IPTRK,'REMESH',LMESH,2,BASE(REM2))
      CALL LCMPUT(IPTRK,'KEYMRG',N2RS,1,IBASE(KEY2))
      CALL LCMPUT(IPTRK,'MATALB',N2RS,1,IBASE(MAT2))
      CALL LCMPUT(IPTRK,'VOLSUR',N2RS,2,BASE(VOL2))
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,ESTATE)
      CALL LCMSIX(IPTRK,' ',2)
      CALL LCMPUT(IPTRK,'MATALB',NFI,1,BASE(MATMRG))
      CALL LCMPUT(IPTRK,'VOLSUR',NFI,2,BASE(VOLMRG))
      CALL LCMPUT(IPTRK,'ZCOORD',NZP+1,2,BASE(IZCOR))
      CALL LCMPUT(IPTRK,'IND2T3',N2RS*(NZP+2),1,IBASE(IND2T3))
      CALL LCMSIX(IPTRK,' ',2)
      CALL RLSARA(IBASE(ITEMP2))
      CALL RLSARA(IBASE(ITEMP))
      CALL RLSARA(BASE(VOLMRG))
      CALL RLSARA(IBASE(MATMRG))
      CALL RLSARA(BASE(IZCOR))
      CALL RLSARA(IBASE(IND2T3))
      CALL RLSARA(IBASE(IND2))
      CALL RLSARA(IBASE(KEY2))
      CALL RLSARA(IBASE(MAT2))
      CALL RLSARA(BASE(VOL2))
      CALL RLSARA(BASE(REM2))
      CALL RLSARA(IBASE(ICOR2))
      CALL RLSARA(IBASE(MAX2))
      CALL RLSARA(IBASE(MIN2))
*
      CALL RLSARA(IBASE(INDEX))
      CALL RLSARA(IBASE(KEYMRG))
      CALL RLSARA(IBASE(MATALB))
      CALL RLSARA(BASE(VOLSUR))
      CALL RLSARA(BASE(REMESH))
      CALL RLSARA(IBASE(ICORD))
      CALL RLSARA(IBASE(MAXDIM))
      CALL RLSARA(IBASE(MINDIM))
*
      RETURN
*
      END
