      FUNCTION CLECST(CPARM,ITYP,NITMA,FLOTT,TEXT,DFLOT)
C
C     CLE-2000 CONSTANTS: R.ROY (11/1999)
C
C             *CLECST* WILL ATTEMPT TO FIND VALUES FOR
C                      A CLE-2000 CONSTANT.
C
C      INPUT: *CPARM * IS THE TENTATIVE CONSTANT NAME
C
C     OUTPUT: *ITYP  * IS THE CONSTANT TYPE (1:INTEGER)
C                                           (2:REAL)
C                                           (3:CHARACTER STRING)
C                                           (4:DOUBLE)
C                                           (5:LOGICAL)
C             *NITMA * IS AN INTEGER VALUE
C                      (= LENGTH OF STRING IF *ITYP* .EQ.3)
C                      (= -1 FOR .F. +1 FOR .T. IF *ITYP* .EQ.5)
C             *FLOTT * IS AN REAL VALUE
C             *TEXT  * IS AN CHARACTER STRING
C             *DFLOT * IS AN DOUBLE PRECISION VALUE
C
C       NOTE: *CLECST* = 0, IF WE FOUND THE PARAMETER *CPARM*
C
C             THIS FUNCTION DEPEND ON THE APPLICATION
C             THE EXAMPLE GIVEN HERE SHOULD HELP THE DEVELOPER
C             TO WRITE ITS OWN APPLICATION-BASED CONSTANT LIST.
C
C             PHYSICAL CONSTANTS GIVEN HERE WERE TAKEN FROM:
C             http://physics.nist.gov/cuu/Constants/
C
C    EXAMPLE: HERE *ITYP* IS IMPOSED AT THE END OF *CPARM*
C             (1:INTEGER) => END: _I
C             (2:REAL   ) => END: _R
C             (3:STRING ) => END: _S
C             (4:DOUBLE ) => END: _D
C             (5:LOGICAL) => END: _L
C             ALL FLOATING (_R, _D ) ARE KEPT IN DOUBLE 
C             AND CONVERTED INTO THE APPROPRIATE MODE.
C
C     NEXT STATEMENT (IN COMMENT) IS *NOT* STANDARD IN FORTRAN-77:
C     IMPLICIT         NONE
C
      INTEGER          CLECST
      CHARACTER*12     CPARM
      INTEGER          ITYP, NITMA
      REAL             FLOTT
      CHARACTER*72     TEXT
      DOUBLE PRECISION DFLOT
C
C     STACK OF INTEGER CONSTANT NAMES  (+ 2 LOGICALS)
      INTEGER          NINTEG
      PARAMETER      ( NINTEG = 7 )
      CHARACTER*12     CINTEG(NINTEG)
      INTEGER          DINTEG(NINTEG)
C
C     STACK OF FLOATING CONSTANT NAMES
      INTEGER          NFLOAT
      PARAMETER      ( NFLOAT = 8 )
      CHARACTER*12     CFLOAT(NFLOAT)
      DOUBLE PRECISION DFLOAT(NFLOAT)
C
C     STACK OF STRING  CONSTANT NAMES
      INTEGER          NSTRIN
      PARAMETER      ( NSTRIN = 8 )
      CHARACTER*12     CSTRIN(NSTRIN)
      CHARACTER*72     DSTRIN(NSTRIN)
C
C     OTHER STUPID LOCAL VALUES
      INTEGER          ILOOP1, IDFTYP, INDLEC
      CHARACTER*12     CPARIN, CBLA12
      CHARACTER*2      CTYPES(5)
      DATA CBLA12 / '            ' /
C
C     DEFINE ENDING TYPES
      DATA CTYPES / '_I', '_R', '_S', '_D', '_L' /
C
C     DATA FOR INTEGER CONSTANTS
      DATA CINTEG / '$Version    ',        '$XLangLvl   ',
     1              '$c0         ',        '$Date       ',
     2              '$Time       ',        '$True       ',
     3              '$False      ' / 
      DATA DINTEG /  2,                     77,
     1               299792458,             20000101,
     2               0,                     +1,
     3               -1            /
C
C     DATA FOR FLOATING CONSTANTS
      DATA CFLOAT / '$Pi         ',        '$E          ', 
     1              '$Euler      ',        '$c0         ',
     2              '$Na         ',        '$u          ',
     3              '$eV         ',        '$h          ' /
      DATA DFLOAT / 3.141592653589793D+00, 2.718281828459045D+00,
     1              0.577215664901533D+00, 2.997924580000000D+08,
     2              6.022141990000000D+23, 1.660538730000000D-27,
     3              1.602176462000000D-19, 6.626068760000000D-34 /
C
C     DATA FOR STRING  CONSTANTS
      DATA CSTRIN / '$Code       ',        '$Release    ',
     1              '$XLang      ',        '$Date       ',
     2              '$Time       ',        '$Bang       ',
     3              '$GetIn      ',        '$GetOut     ' /
      DATA DSTRIN / 'CLE2000',             '2.1',
     1              'Fortran',             '20000101',
     2              '000000',              '!',
     3              '>>',                  '<<'           /
C
      CLECST= 1
C
C     IDENTITY WHICH TYPE: _I ,_R, _D, _S, _L
      INDLEC= 0
      DO 10 ILOOP1= 1, 5
         IDFTYP= INDEX( CPARM, CTYPES(ILOOP1) )
         IF( IDFTYP.NE.0 )THEN
            INDLEC= ILOOP1
            CPARIN= CPARM(1:IDFTYP-1)//CBLA12(IDFTYP:12)
         ENDIF
   10 CONTINUE
C
      IF( INDLEC.EQ.1.OR.INDLEC.EQ.5 )THEN
C
C        LOOK FOR INTEGER VARIABLES
         DO 20 ILOOP1= 1, NINTEG
            IF( CPARIN.EQ.CINTEG(ILOOP1) )THEN
C
C              FOUND: RETURN => TYPE=1, INTEGER
C                            => TYPE=5, LOGICAL
               ITYP=  INDLEC
               NITMA= DINTEG(ILOOP1)
               CLECST= 0
               GO TO 666
            ENDIF
   20    CONTINUE
      ELSEIF( INDLEC.EQ.3 )THEN
C
C        LOOK FOR STRING  VARIABLES
         DO 30 ILOOP1= 1, NSTRIN
            IF( CPARIN.EQ.CSTRIN(ILOOP1) )THEN
C
C              FOUND: RETURN => TYPE=3, STRING AND ITS LENGTH
               ITYP=  3
               TEXT=  DSTRIN(ILOOP1)
               NITMA= INDEX(DSTRIN(ILOOP1),' ')-1
               CLECST= 0
               GO TO 666
            ENDIF
   30    CONTINUE
      ELSEIF( INDLEC.NE.0 )THEN
C
C        LOOK FOR FLOATING VARIABLES
         DO 40 ILOOP1= 1, NFLOAT
            IF( CPARIN.EQ.CFLOAT(ILOOP1) )THEN
               IF( INDLEC.EQ.2 )THEN
C
C                 FOUND: RETURN => TYPE=2, REAL
                  FLOTT= SNGL(DFLOAT(ILOOP1))
               ELSE
C
C                 FOUND: RETURN => TYPE=4, DOUBLE
                  DFLOT= DFLOAT(ILOOP1)
               ENDIF
               ITYP=   INDLEC
               CLECST= 0
               GO TO 666
            ENDIF
   40    CONTINUE
      ENDIF
C
  666 RETURN
      END
