*DECK KDROPN
      FUNCTION KDROPN(CUNAME,IACTIO,IUTYPE,LRDA,IHANDL)
C
C------------------------  KDROPN/KDRCLS/KDRFST ------------------------
C
C 1- PROGRAMME STATISTICS:
C     NAME     : KDROPN
C     ENTRY    : KDROPN,KDRCLS,KDRFST
C     USE      : ALLOCATE AND RELEASE FILE UNITS ASSOCIATED TO
C                A GIVEN FILE NAME. DIRECT ACCESS (XSM), SEQUENTIAL 
C                (FORMATTED OR NOT) AND DIRECT ACCESS (DA) FILES ARE
C                PERMITTED
C     MODIFIED : 91-01-24
C     AUTHOR   : G.MARLEAU
C
C 2- ROUTINE PARAMETERS:
C
C - FUNCTION KDROPN(CUNAME,IACTIO,IUTYPE,LRDA,IHANDL)
C
C   OPEN FILE AND ALLOCATE UNIT NUMBER. ALLOCATE UNIT NUMBER TO FILE
C   NAME IF UNIT IS ALREADY OPENED, RETURNS UNIT NUMBER.
C
C   INPUT
C     CUNAME   : FILENAME                                     C*12
C                IF CUNAME=' ', USE A DEFAULT NAME
C     IACTIO   : ACTION ON FILE                               I
C                =0 TO ALLOCATE A NEW FILE
C                =1 TO ACCESS AND MODIFY AN EXISTING FILE
C                =2 TO ACCESS AN EXISTING FILE IN 
C                   READ-ONLY MODE
C                =3 UNKNOWN
C     IUTYPE   : FILE TYPE                                    I
C                =1  XSM FILE TYPE 
C                =2  SEQUENTIAL UNFORMATTED
C                =3  SEQUENTIAL FORMATTED
C                =4  DIRECT ACCESS (DA) UNFORMATTED FILE
C     LDRA     : NUMBER OF WORDS IN DA FILE                   I 
C                REQUIRED FOR  IUTYPE = 4 ONLY
C     IHANDL   : HANDLE OF XSM FILE                           I
C                REQUIRED FOR  IUTYPE = 1 ONLY
C   OUTPUT
C     KDROPN   : UNIT NUMBER/ERROR STATUS                     I
C                >  0  UNIT NUMBER ALLOCATED                   
C                = -1  NO MORE UNIT AVAILABLE
C                = -2  FILE TYPE REQUESTED INCONSISTENT WITH
C                      FILE TYPE OF THIS FILE
C                = -3  THIS FILE HAS BEEN ALREADY OPENED
C                = -4  FILE NAME IS RESERVED OR TOO LONG
C                      (FT06F00, FT07F00)
C                = -5  ILLEGAL FILE TYPE
C                      0 < IUTYPE < 4
C                = -6  ERROR ON OPEN OF XSM FILE
C                = -7  ERROR ON OPEN OF UNFORMATTED SEQUENTIAL FILE
C                = -8  ERROR ON OPEN OF FORMATTED SEQUENTIAL FILE
C                = -9  ERROR ON OPEN OF DIRECT ACCESS FILE 
C                =-10  INVALID NUMBER OF WORD IN DIRECT ACCESS FILE
C                      LRDA MUST BE > 0
C
C - ENTRY KDRCLS(ITAPNO,IACTIO)
C
C   CLOSE FILE AND RELEASE UNIT NUMBER.
C
C   INPUT
C     ITAPNO   : UNIT NUMBER                                  I
C                = 0    CLOSE ALL UNITS
C                > 0    UNIT TO CLOSE
C     IACTIO   : ACTION ON FILE                               I
C                = 1    TO KEEP THE FILE;
C                = 2    TO DELETE THE FILE.
C   OUTPUT
C     KDRCLS   : ERROR STATUS                                 I
C                =  0  UNIT CLOSED
C                = -2  FILE NOT OPPENED
C                = -3  THIS FILE HAS BEEN OPENED BY ROUTINES
C                      OTHER THAN KDROPN
C                = -4  FILE UNIT IS RESERVED (5,6)
C                = -5  ILLEGAL UNIT NUMBER
C                = -6  ERROR ON CLOSE OF XSM FILE 
C                = -7  ERROR ON CLOSE OF UNFORMATTED 
C                      SEQUENTIAL FILE
C                = -8  ERROR ON CLOSE OF FORMATTED
C                      SEQUENTIAL FILE
C                = -9  ERROR ON CLOSE OF DA FILE
C                =-10  INVALID CLOSE ACTION
C                      IACTIO=1,2 PERMITTED ONLY
C
C - ENTRY KDRFST(CUNAME,ITAPNO,LDRA,IHANDL)
C
C   RETURN THE UNIT ALLOCATED TO FILE NAME OR THE FILE NAME 
C   ALLOCATED TO UNIT. ALSO RETURN THE IUTYPE OF THAT FILE.
C
C   INPUT
C     CUNAME   : FILE NAME. =' ' TO OBTAIN THE FILE NAME.     C*12
C     IUNIT    : UNIT NUMBER (IF CUNAME.EQ.' ').              I
C   OUTPUT
C     CUNAME   : FILE NAME (IF CUNAME.EQ.' ').                C*12
C     ITAPNO   : UNIT NUMBER (IF CUNAME.NE.' ').              I
C     LDRA     : NUMBER OF WORDS IN DA FILE                   I 
C                RETURNED ONLY IF IUTYPE = 4 
C     IHANDL   : HANDLE OF XSM FILE                           I
C                RETURNED ONLY IF IUTYPE = 1
C     KDRFST   : ERROR FLAG OR IUTYPE OF THE FILE.            I
C                >  0  FILE IUTYPE
C                = -2  FILE NOT OPPENED
C                = -3  THIS FILE HAS BEEN OPENED BY ROUTINES
C                      OTHER THAN KDROPN
C                = -4  FILE UNIT IS RESERVED (5,6)
C                = -5  ILLEGAL UNIT NUMBER
C
C 3- INTERNAL PARAMETERS:
C     NBTAPE   : NUMBER OF UNITS AVAILABLE                     I
C     NRECLD   : NUMBER OF DIRECT ACCESS RECORDS               I
C     LRECLD   : LENGTH OF XSM RECORDS                         I
C     NRESER   : NUMBER OF RESERVED NAMES                      I
C     NDUMMY   : NUMBER OF DUMMY UNITS                         I
C     CRESER   : RESERVED NAMES                     C(NRESER)*12
C     CDUMMY   : NAMES OF DUMMY FILES               C(NDUMMY)*12
C     CTAPEN   : NAMES ALLOCATED TO UNITS           C(NBTAPE)*12
C     ITAPET   : TYPE OF FILES FOR ALL UNITS           I(NBTAPE)
C                = 0  NO TYPE ASSOCIATED TO FILE
C                = IUTYPE  FILE OF TYPE IUTYPE OPENED
C                =-IUTYPE  FILE OF TYPE IUTYPE CLOSED
C     ILENDA   : LENGTH OF DA FILE                     I(NBTAPE)
C                = 0       NOT A DA FILE
C                = LRDA    LENGTH OF DA FILE
C     ITHAND   : HANDLE FOR XSM FILE                   I(NBTAPE)
C                = 0       NOT A XSM FILE
C                = IHANDL  HANDLE FOR XSM FILE
C
C---------------------------  KDROPN  ----------------------------------
C
      SAVE       CTAPEN,ITAPET,ILENDA,ITHAND,CRESER,CDUMMY,CTYPE
#if defined(NOSBE)
*     CDC SETTINGS:
      PARAMETER (NRECLD=1,NBTAPE=99)
#elif defined(VMS)
*     VAX SETTINGS:
      PARAMETER (NRECLD=1,NBTAPE=99)
#elif defined(CRAY)
*     CRAY SETTINGS:
      PARAMETER (NRECLD=8,NBTAPE=99)
#elif defined(Ultrix)
*     DEC ULTRIX SETTINGS:
      PARAMETER (NRECLD=1,NBTAPE=99)
#elif defined(OSF1)
*     ALPHA OSF/1 SETTINGS:
*     SET NRECLD=1 ON 32 BITS VERSION; NRECLD=2 ON 64 BITS VERSION.
      PARAMETER (NRECLD=2,NBTAPE=99)
#elif defined(IRIX)
*     SILICON GRAPHICS SETTINGS:
      PARAMETER (NRECLD=1,NBTAPE=99)
#elif defined(MVS)
*     IBM (MVS) SETTINGS:
      PARAMETER (NRECLD=4,NBTAPE=20)
#elif defined(ifort)
*     INTEL ifort SETTINGS:
      PARAMETER (NRECLD=1,NBTAPE=99)
#elif defined(Linux) && defined(__x86_64__)
      PARAMETER (NRECLD=8,NBTAPE=99)
#else
      PARAMETER (NRECLD=4,NBTAPE=99)
#endif
      PARAMETER (LRECLD=1024,NRESER=2,NDUMMY=4,NB=NBTAPE-6)
      CHARACTER  CUNAME*(*),CTAPEN(NBTAPE)*72,CRDNAM*72,
     >           CRESER(NRESER)*12,CDUMMY(NDUMMY)*12,CTYPE(NDUMMY)*22,
     >           CFORM*11,CSTATU*12
      INTEGER    KDROPN,KDRCLS,KDRFST,IUTYPE,ITAPNO,
     >           ITAPET(NBTAPE),ILENDA(NBTAPE),ITHAND(NBTAPE)
      LOGICAL    LFILOP
      DATA      (CRESER(JJ),JJ=1,NRESER)/'FT05F00','FT06F00'/
      DATA      (CDUMMY(KK),KK=1,NDUMMY)
     >          /'DUMMYDA','DUMMYSQ','DUMMYCA','DUMMYIN'/
      DATA      (CTAPEN(JJ),JJ=1,NBTAPE) /4*'            ','FT05F00',
     >           'FT06F00',NB*'            '/
      DATA      (ITAPET(JJ),JJ=1,NBTAPE) /NBTAPE*0/
      DATA      (ILENDA(JJ),JJ=1,NBTAPE) /NBTAPE*0/
      DATA      (ITHAND(JJ),JJ=1,NBTAPE) /NBTAPE*0/
      DATA      (CTYPE(KK),KK=1,NDUMMY)  /'DIRECT ACCESS XSM     ',
     >           'SEQUENTIAL UNFORMATTED','SEQUENTIAL CHARACTER  ',
     >           'DIRECT ACCESS DA      '/
C----
C  CHECK IF IUTYPE IS VALID
C----
      IF((IUTYPE.GT.4).OR.(IUTYPE.LT.1)) THEN
        KDROPN=-5
        RETURN
      ENDIF
C----
C  CHECK IF LRDA IS VALID
C----
      IF((IUTYPE.EQ.4).AND.(LRDA.LT.1)) THEN
        KDROPN=-10
        RETURN
      ENDIF
C----
C  CHECK IF FILE NAME IS MORE THAN 72 CHARACTERS
C----
      LUNAME= LEN(CUNAME)
      IF(LUNAME.GT.72) THEN
        KDROPN=-4
        RETURN
      ENDIF
      LUNAME= MAX(LUNAME,12)
C----
C  CHECK IF FILE NAME NOT FORBIDDEN
C----
      DO 100 IRESER=1,NRESER
        IF(CUNAME(1:LUNAME).EQ.CRESER(IRESER)) THEN
          KDROPN=-4
          RETURN
        ENDIF
 100  CONTINUE
C----
C  CHECK FOR DUMMY FILE NAME/ALLOCATE DUMMY FILE NAME IF REQUESTED
C----
      DO 110 IDUMMY=1,NDUMMY
        IF(CUNAME(1:LUNAME).EQ.CDUMMY(IDUMMY)) THEN
          IF(IDUMMY.NE.IUTYPE) THEN
            KDROPN=-2
            RETURN
          ENDIF
        ENDIF
 110  CONTINUE
      IF(CUNAME.EQ.' ') THEN
        CRDNAM=CDUMMY(IUTYPE)
      ELSE
#if defined(MVS)
        IF(LEN(CUNAME).GT.7) THEN
          DO 111 I=8,LEN(CUNAME)
          IF(CUNAME(I:I).NE.' ') THEN
            WRITE(6,'(/40H KDROPN: *** WARNING *** THE FILE NAME '',A12,
     >      31H'' IS TRUNCATED TO 7 CHARACTERS.)') CUNAME
            GO TO 112
          ENDIF
 111      CONTINUE
 112      CRDNAM=CUNAME(:7)
        ELSE
          CRDNAM=CUNAME
        ENDIF
#else
        CRDNAM=CUNAME
#endif
      ENDIF
C----
C  CHECK IF FILE OPENED/PERMITTED
C----
      INQUIRE(FILE=CRDNAM,OPENED=LFILOP,NUMBER=IBOUCL)
      IF(LFILOP) THEN
        KDROPN = -3
        RETURN
      ENDIF
C----
C  LOOK FOR A NEW UNIT NUMBER FOR FILE CRDNAM
C  LOOK IF FILE NAME IS IN TABLE OF PREVIOUSLY ALLOCATED UNITS
C----
      DO 120 JBOUCL=NBTAPE,1,-1
        IF(CTAPEN(JBOUCL).EQ.CRDNAM) THEN
          IBOUCL=JBOUCL
          IF(IUTYPE.NE.-ITAPET(IBOUCL)) THEN
            KDROPN=-2
            RETURN
          ENDIF
          GO TO 121
        ENDIF
 120  CONTINUE
C----
C  LOOK FOR NEVER ALLOCATED UNIT LOCATION
C----
      DO 122 JBOUCL=NBTAPE,1,-1
        IF(CTAPEN(JBOUCL).EQ.' ') THEN
          IBOUCL=JBOUCL
          INQUIRE(UNIT=IBOUCL,OPENED=LFILOP)
          IF(.NOT.LFILOP) GO TO 121
        ENDIF
 122  CONTINUE
C----
C  LOOK FOR A UNIT PREVIOUSLY ALLOCATED BUT RELEASED.
C----
      DO 123 JBOUCL=NBTAPE,1,-1
        IF(ITAPET(JBOUCL).LT.0) THEN
          IBOUCL=JBOUCL
          GO TO 121
        ENDIF
 123  CONTINUE
C----
C  ERROR - NO UNIT NUMBER AVAILABLE
C----
      KDROPN = -1
      RETURN
C
 121  IF(IUTYPE.EQ.1) THEN
C----
C  OPEN XSM FILE
C----
        KDROPN=-6
        LRECL=LRECLD*NRECLD
        CFORM='UNFORMATTED'
      ELSE IF(IUTYPE.EQ.2) THEN
C----
C  OPEN SEQUENTIAL UNFORMATTED FILE
C----
        KDROPN=-7
        CFORM='UNFORMATTED'
      ELSE IF(IUTYPE.EQ.3) THEN
C----
C  OPEN SEQUENTIAL FORMATTED FILE
C----
        KDROPN=-8
        CFORM='FORMATTED'
      ELSE IF(IUTYPE.EQ.4) THEN
C----
C  OPEN DA FILE
C----
        KDROPN=-9
        LRECL=LRDA*NRECLD
        CFORM='UNFORMATTED'
      ENDIF
      IF(IACTIO.EQ.0) THEN
        CSTATU='NEW'
      ELSE IF(IACTIO.EQ.1) THEN
        CSTATU='OLD'
      ELSE IF(IACTIO.EQ.2) THEN
        CSTATU='OLD'
      ELSE
        CSTATU='UNKNOWN'
      ENDIF
      IF(((IUTYPE.EQ.1).OR.(IUTYPE.EQ.4)).AND.(IACTIO.EQ.2)) THEN
#if defined(F90)
        IDUMMY=0
        IF(IUTYPE.EQ.1) THEN
           INQUIRE(IOLENGTH=LRECL) (IDUMMY,I=1,LRECLD)
        ELSE IF (IUTYPE.EQ.4) THEN
           INQUIRE(IOLENGTH=LRECL) (IDUMMY,I=1,LRDA)
        ENDIF
        OPEN(UNIT=IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM, &
     &  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU,ACTION='READ')
#elif defined(MVS)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU,ACTION='READ')
#elif defined(AIX)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU)
#elif defined(Linux)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU)
#else
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU,READONLY)
#endif
      ELSE IF((IUTYPE.EQ.1).OR.(IUTYPE.EQ.4)) THEN
#if defined(F90)
        IDUMMY=0
        IF(IUTYPE.EQ.1) THEN
           INQUIRE(IOLENGTH=LRECL) (IDUMMY,I=1,LRECLD)
        ELSE IF (IUTYPE.EQ.4) THEN
           INQUIRE(IOLENGTH=LRECL) (IDUMMY,I=1,LRDA)
        ENDIF
        OPEN(UNIT=IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM, &
     &  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU)
#else
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='DIRECT',RECL=LRECL,STATUS=CSTATU)
#endif
      ELSE IF(((IUTYPE.EQ.2).OR.(IUTYPE.EQ.3)).AND.(IACTIO.EQ.0)) THEN
#if defined(F90)
        OPEN(UNIT=IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM, &
     &  ACCESS='SEQUENTIAL',STATUS=CSTATU)
#else
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='SEQUENTIAL',STATUS=CSTATU)
#endif
      ELSE IF(((IUTYPE.EQ.2).OR.(IUTYPE.EQ.3)).AND.(IACTIO.EQ.1)) THEN
#if defined(F90)
        OPEN(UNIT=IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM, &
     &  ACCESS='SEQUENTIAL',POSITION='APPEND',STATUS=CSTATU)
#elif defined(AIX)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='SEQUENTIAL',POSITION='APPEND',STATUS=CSTATU)
#else
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='APPEND',STATUS=CSTATU)
#endif
      ELSE IF(((IUTYPE.EQ.2).OR.(IUTYPE.EQ.3)).AND.(IACTIO.EQ.2)) THEN
#if defined(F90)
        OPEN(UNIT=IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM, &
     &  ACCESS='SEQUENTIAL',STATUS=CSTATU,ACTION='READ')
#elif defined(MVS)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='SEQUENTIAL',STATUS=CSTATU,ACTION='READ')
#elif defined(AIX)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='SEQUENTIAL',STATUS=CSTATU)
#elif defined(Linux)
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='SEQUENTIAL',STATUS=CSTATU)
#else
        OPEN(IBOUCL,FILE=CRDNAM,ERR=9000,IOSTAT=IERCOD,FORM=CFORM,
     >  ACCESS='SEQUENTIAL',STATUS=CSTATU,READONLY)
#endif
        REWIND(IBOUCL)
      ENDIF
      KDROPN = IBOUCL
      CTAPEN(IBOUCL) = CRDNAM
      ITAPET(IBOUCL) = IUTYPE
      IF(IUTYPE.EQ.1) THEN
        ITHAND(IBOUCL)=IHANDL
      ELSE IF(IUTYPE.EQ.4) THEN
        ILENDA(IBOUCL)=LRDA
      ENDIF
      RETURN
 9000 WRITE(6,8000) CRDNAM, CTYPE(IUTYPE),IERCOD
C----
C  ERROR FORMAT
C----
 8000 FORMAT('1',5X,'ERROR IN OPENING OF FILE IN KDROPN'/
     >       6X,'FILE NAME = ',A/6X,'FILE TYPE = ',A22/
     >       6X,'   IERCOD = ',I7)
      RETURN
C
C------------------------  KDROPN/KDRCLS/KDRFST ------------------------
C       ENTRY    :  KDRCLS
C       USE      :  RELEASE FILE UNITS/CLOSE FILES
C-------------------------- KDRCLS -------------------------------------
C
      ENTRY KDRCLS(ITAPNO,IACTIO)
      IF(ITAPNO.EQ.0) THEN
C----
C  UNIT = 0 CLOSE ALL OPENED UNITS
C----
        IDEBTP=1
        IFINTP=NBTAPE
        GO TO 200
      ELSE IF((ITAPNO.GT.NBTAPE).OR.(ITAPNO.LT.0)) THEN
C----
C  INVALID UNIT NUMBER
C----
        KDRCLS=-5
        RETURN
      ENDIF
      INQUIRE(UNIT=ITAPNO,OPENED=LFILOP)
      IF(LFILOP) THEN
        IF(ITAPET(ITAPNO).EQ.0) THEN
C----
C  RESERVED UNIT
C----
          KDRCLS = -4
          RETURN
        ENDIF
C----
C  UNIT PERMITTED
C----
        IDEBTP=ITAPNO
        IFINTP=ITAPNO
        GO TO 200
      ELSE
C----
C  UNIT NOT OPENED
C----
        KDRCLS=-2
        RETURN
      ENDIF
 200  CONTINUE
      DO 210 IBOUCL=IDEBTP,IFINTP
        IF(ITAPET(IBOUCL).EQ.1) THEN
C----
C  CLOSE XSM TYPE FILE
C----
          KDRCLS=-6
          IF(IACTIO.EQ.1) THEN
            CLOSE(IBOUCL,IOSTAT=IERCOD,STATUS='KEEP',ERR=9001)
            ITAPET(IBOUCL)=-ITAPET(IBOUCL)
            ITHAND(IBOUCL)=0
          ELSE IF(IACTIO.EQ.2) THEN
            CLOSE(IBOUCL,IOSTAT=IERCOD,STATUS='DELETE',ERR=9001)
            ITAPET(IBOUCL)=0
            CTAPEN(IBOUCL)=' '
            ITHAND(IBOUCL)=0
          ELSE
            KDRCLS=-10
          ENDIF
        ELSE IF(ITAPET(IBOUCL).GT.0) THEN
C----
C  CLOSE OTHER TYPES OF FILE
C----
          KDRCLS=-(5+ITAPET(IBOUCL))
          IF(IACTIO.EQ.1) THEN
            CLOSE(IBOUCL,IOSTAT=IERCOD,STATUS='KEEP',ERR=9001)
            ITAPET(IBOUCL)=-ITAPET(IBOUCL)
          ELSE IF(IACTIO.EQ.2) THEN
            CLOSE(IBOUCL,IOSTAT=IERCOD,STATUS='DELETE',ERR=9001)
            ITAPET(IBOUCL)=0
            CTAPEN(IBOUCL)=' '
            ILENDA(IBOUCL)=0
          ELSE
            KDRCLS=-10
            IERCOD=1
          ENDIF
          IF(IERCOD.NE.0) GO TO 9001
        ENDIF
 210  CONTINUE
      KDRCLS=0
      RETURN
 9001 WRITE(6,8001) IBOUCL,CTAPEN(IBOUCL),CTYPE(ITAPET(IBOUCL)),IERCOD
C----
C  ERROR FORMAT
C----
 8001 FORMAT('1',5X,'ERROR IN CLOSE OF FILE IN KDROPN'/
     >6X,'UNIT NB.  = ',I10/6X,'FILE NAME = ',A7/6X,'FILE TYPE = ',A22/
     >6X,'   IERCOD = ',I10 )
      RETURN
C
C------------------------  KDROPN/KDRCLS/KDRFST-------------------------
C       ENTRY    :  KDRFST
C       USE      :  GET FILE STATISTICS
C----------------------------- KDRFST ----------------------------------
C
      ENTRY KDRFST(CUNAME,ITAPNO,LRDA,IHANDL)
      IF(CUNAME.EQ.' ') THEN
        IF((ITAPNO.LT.1).OR.(ITAPNO.GT.NBTAPE)) THEN
C----
C  ILLEGAL UNIT OR NO FILE NAME
C----
          KDRFST=-5
          RETURN
        ENDIF
C----
C  NO FILE NAME, RETURN FILE NAME AND TYPE
C----
        IF((CTAPEN(ITAPNO).EQ.' ').OR.(ITAPET(ITAPNO).LT.0)) THEN
          KDRFST=-2
        ELSE
          CUNAME=CTAPEN(ITAPNO)
          KDRFST=ITAPET(ITAPNO)
          LRDA=ILENDA(ITAPNO)
          IHANDL=ITHAND(ITAPNO)
        ENDIF
      ELSE
C----
C  FILE NAME PRESENT, RETURN FILE NUMBER AND TYPE
C----
#if defined(MVS)
        CRDNAM=CUNAME(:7)
#else
        CRDNAM=CUNAME
#endif
        INQUIRE(FILE=CRDNAM,OPENED=LFILOP,NUMBER=ITAPNO)
        IF(LFILOP) THEN
          IF((ITAPNO.LE.0).OR.(ITAPNO.GT.NBTAPE)) THEN
            KDRFST=-5
          ELSE
            KDRFST=ITAPET(ITAPNO)
            LRDA=ILENDA(ITAPNO)
            IHANDL=ITHAND(ITAPNO)
          ENDIF
        ELSE
          KDRFST=-2
        ENDIF
      ENDIF
      RETURN
      END
