*DECK MSTCPB
      SUBROUTINE MSTCPB(IPSTR,IPSTR2,ACSTR2,TYSTR2,IPRINT,NELEM,IBEG,
     1                  IEND,IINC,NAME,NAME2,NBLOCK,BLNAM,BLTYP,BLLEN)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Copy some elements from a structure's block to another.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Le Tellier
*
*Parameters: input
* IPSTR   address of the structure from which the information is retrieved.
* IPSTR2  destination structure address.
* ACSTR2  destination structure access.
* TYSTR2  destination structure type.
* IPRINT  level of print index.
* NELEM   number of elements.
* IBEG    index of the first element.
* IEND    index of the last element.
* IINC    index increment between two consecutive elements.
* NAME    name of the block from which the information is retrieved.
* NAME2   destination block name.
* NBLOCK  number of existing block in the directory.
* BLNAM   names of these blocks.
* BLTYP   types of these blocks.
* BLLEN   lengths of these blocks.
*
*-----------------------------------------------------------------------
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPSTR,IPSTR2,ACSTR2,TYSTR2,IPRINT,NELEM,IBEG,IEND,IINC,
     1 NBLOCK,BLNAM(3,NBLOCK+1),BLTYP(NBLOCK+1),BLLEN(NBLOCK+1)
      CHARACTER*12 NAME,NAME2  
*----
*  LOCAL VARIABLES
*----
      INTEGER IOUT,NSTATE
      PARAMETER (IOUT=6,NSTATE=40)
      INTEGER ISTATE(NSTATE),KCHAR(3),NARA,ITYP,SIZE,IARA,ITYPO,NELEO,
     1 NARA2,IARA2,II,JJ,SIZE2
      LOGICAL EXIST
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))  
*
*     RETRIEVING BLOCK TO BE COPIED
      CALL LCMLEN(IPSTR,NAME,SIZE,ITYP)
      IF (ITYP.LE.2) THEN
         NARA=SIZE
      ELSEIF (ITYP.EQ.3) THEN
         NARA=SIZE/3
      ELSEIF (ITYP.EQ.4) THEN
         NARA=SIZE/2
      ELSE 
         CALL XABORT('MSTCPB: UNSUPPORTED TYPE')
      ENDIF
      IF (NARA.LE.0) THEN
         CALL LCMLIB(IPSTR)
         CALL XABORT('MSTCPB: INVALID BLOCK '//NAME//'.')
      ENDIF
      IF (IEND.GT.NARA) 
     1  CALL XABORT('MSTCPB: INCOMPATIBLE SIZE')
      CALL SETARA(BASE,SIZE,IARA)
      CALL LCMGET(IPSTR,NAME,BASE(IARA))
*     DOES THIS BLOCK ALREADY EXIST IN THE DESTINATION STRUCTURE ?
      EXIST=.FALSE.
      NELEO=0
*     SPECIAL CASE OF STATE-VECTOR MODIFICATION
      IF (NAME2.EQ.'STATE-VECTOR') THEN
         IF (IEND.GT.40)
     1      CALL XABORT('MSTCPM: STATE-VECTOR SIZE IS LIMITED TO 40.')
         IF (IEND.EQ.40)
     2      CALL XABORT('MSTCPM: 40th STATE-VECTOR ELEMENT SHOULD'//
     3      ' NOT BE MODIFIED.')
         ITYPO=1
         NELEO=NSTATE
         EXIST=.TRUE.
      ENDIF
      IF (NBLOCK.NE.0) THEN
         READ(NAME2,'(3A4)') (KCHAR(II),II=1,3)
         DO II=1,NBLOCK
            IF((BLNAM(1,II).EQ.KCHAR(1)).AND.
     1         (BLNAM(2,II).EQ.KCHAR(2)).AND.
     2         (BLNAM(3,II).EQ.KCHAR(3))) THEN
               ITYPO=BLTYP(II)
               IF (ITYPO.EQ.0)
     1           CALL XABORT('MSTCPM: '//NAME2//
     2                       ' IS AN EXISTING DIRECTORY.')
               IF (ITYPO.NE.ITYP)
     1           CALL XABORT('MSTCPM: INCOMPATIBLE TYPES')
               NELEO=BLLEN(II)
               EXIST=.TRUE.
               GOTO 20
            ENDIF
         ENDDO
 20      CONTINUE
      ENDIF
      IF (IPRINT.GT.2) THEN
      IF (EXIST) THEN
*     YES: IT WILL BE UPDATED
         WRITE(IOUT,*) 'MSTCPB: BLOCK '//NAME//' IN UPDATE MODE'
      ELSE
*     NO: IT WILL BE CREATED
         WRITE(IOUT,*) 'MSTCPB: BLOCK '//NAME//' IN CREATION MODE'
      ENDIF
      ENDIF      
      NARA2=MAX(NARA,NELEO)
*     ALLOCATE MEMORY
      IF (ITYP.LE.2) THEN
         SIZE2=NARA2
      ELSEIF (ITYP.EQ.3) THEN
         SIZE2=3*NARA2
      ELSEIF (ITYP.EQ.4) THEN
         SIZE2=2*NARA2
      ENDIF
      CALL SETARA(BASE,SIZE2,IARA2)
*     INITIALIZE BLOCK
      IF (EXIST) THEN
         CALL LCMGET(IPSTR2,NAME2,BASE(IARA2))
      ELSE
         IF (ITYP.EQ.1) THEN
            CALL XDISET(IBASE(IARA2),NARA,0)
         ELSEIF (ITYP.EQ.2) THEN
            CALL XDRSET(BASE(IARA2),NARA,0.0)
         ELSEIF (ITYP.EQ.3) THEN
            CALL XDISET(IBASE(IARA2),3*NARA,0)
         ELSEIF (ITYP.EQ.4) THEN
            CALL XDDSET(BASE(IARA2),NARA,0.D0)
         ENDIF
      ENDIF
*     COPY ACTION
      DO II=IBEG,IEND,IINC
         IF (ITYP.EQ.1) THEN
            IBASE(IARA2-1+II)=IBASE(IARA-1+II)
         ELSEIF (ITYP.EQ.2) THEN
            BASE(IARA2-1+II)=BASE(IARA-1+II)
         ELSEIF (ITYP.EQ.3) THEN
            DO JJ=0,2
               IBASE(IARA2+3*(II-1)+JJ)=IBASE(IARA+3*(II-1)+JJ)
            ENDDO
         ELSEIF (ITYP.EQ.4) THEN
            CALL XDDSET(BASE(IARA2+2*(II-1)),1,BASE(IARA+2*(II-1)))         
         ENDIF
      ENDDO
      CALL LCMPUT(IPSTR2,NAME2,SIZE2,ITYP,BASE(IARA2))
      IF (.NOT.EXIST) THEN
*     UPDATE NB. BLOCKS, REC-NAMES, REC-TYPES, REC-LENGTHS IN STATE-VECTOR IF REQUIRED
         CALL LCMGET(IPSTR2,'STATE-VECTOR',ISTATE)
         ISTATE(40)=ISTATE(40)+1
         READ(NAME,'(3A4)') (BLNAM(II,NBLOCK+1),II=1,3)
         BLTYP(NBLOCK+1)=ITYP
         BLLEN(NBLOCK+1)=NARA
         CALL LCMPUT(IPSTR2,'STATE-VECTOR',NSTATE,1,ISTATE)
         CALL LCMPUT(IPSTR2,'REC-NAMES',3*(NBLOCK+1),3,BLNAM)
         CALL LCMPUT(IPSTR2,'REC-TYPES',(NBLOCK+1),1,BLTYP)
         CALL LCMPUT(IPSTR2,'REC-LENGTHS',(NBLOCK+1),1,BLLEN)
      ENDIF
*     DEALLOCATE MEMORY
      CALL RLSARA(BASE(IARA2))
      CALL RLSARA(BASE(IARA))
*
      RETURN
      END
