*deck dragr
c
      subroutine dragr
c
c     ******************************************************************
c     *                                                                *
c     *     Produce a draglib interface file from njoy intermediate    *
c     *   cross-section library.                                       *
c     *                                                                *
c     *     The draglib format provide an efficient way to store multi-*
c     *   group isotopic nuclear data to be used in a lattice code.    *
c     *   For example the draglib can then be used by the Dragon code. *
c     *                                                                *
c     *     Working from thermr and groupr output tape,this module     *
c     *   produce the standard interface file on the xsm file. The xsm *
c     *   data base is used to organize the data in a hierarchical     *
c     *   format. Therefore, it will be easy to convert back and forth *
c     *   between the binary direct access format (efficient during a  *
c     *   calculation) and the ascii or binary format (useful for      *
c     *   backup and exchange purposes).                               *
c     *                                                                *
c     * Authors: Hasan Saygin (original version -- 1992)               *
c     *          Alain Hebert (reprogrammed in 2003)                   *
c     *                                                                *
c     * Copyright:                                                     *
c     *  Copyright (C) 2003 Ecole Polytechnique de Montreal            *
c     *  This library is free software; you can redistribute it and/or *
c     *  modify it under the terms of the GNU Lesser General Public    *
c     *  License as published by the Free Software Foundation; either  *
c     *  version 2.1 of the License, or (at your option) any later     *
c     *  version.                                                      *
c     *                                                                *
c     *---input specifications (free format)---------------------------*
c     *                                                                *
c     * card 1 file units                                              *
c     *   nendf     input endf unit                                    *
c     *   npendf    input pendf unit                                   *
c     *   ngendf    input gendf unit                                   *
c     *   nfp       input endf unit for fission yield data             *
c     *   ndcy      input endf unit for radioactive decay data         *
c     *   nimpo     input draglib unit                                 *
c     *   nexpo     output draglib unit                                *
c     *   pfflag    fission product flag: off/on = 0/1. pfflag=1 to    *
c     *             avoid storing scattering matrices. (default=0)     *
c     * card 2 hollerith identification for the library                *
c     * (nimpo=0 only)                                                 *
c     *   labell    72 character identification for the library        *
c     * card 3 material data (one card per material)                   *
c     *   matno     integer material identifier                        *
c     *             (endf/b mat number)                                *
c     *   hmat      hollerith material identifier (up to 8 characters  *
c     *             each). By default, an ascii identifier is          *
c     *             constructed.                                       *
c     * card 4 material readme comment (up to 72 characters)           *
c     * card 5 energy limits for dilution-dependent xs data. This data *
c     *        is important to avoid self-shielding very low and very  *
c     *        high energy group xs and to reduce the size of the      *
c     *        draglib. (one card per material)                        *
c     *   eres0     lower limit of resolved resonance xs data (ev)     *
c     *   eres1     upper limit of unresolved resonance xs data (ev)   *
c     * card 6 autolib energy limits. This data is used with advanced  *
c     *        self-shielding models such as the Sanchez-Coste and     *
c     *        Ribon extended models. It is highly recommended to use  *
c     *        the same data for all resonant isotopes. (one card per  *
c     *        material)                                               *
c     * present if and only if npendf.ne.0                             *
c     *   eaut0     lower limit of autolib resonance xs data (ev)      *
c     *   eaut1     upper limit of autolib resonance xs data (ev)      *
c     *   deli      elementary lethargy width (ev)                     *
c     *                                                                *
c     *            repeat cards 3 to 6 for all materials desired       *
c     *            matno=0/ terminates groupr run.                     *
c     *                                                                *
c     * card 7 burnup chain data (one or two cards per isotope)        *
c     * present if and only if nfp.ne.0 and ndcy.ne.0                  *
c     *  hich      hollerith isotope identifier (same as hmat below)   *
c     *            hich must be constructed in a way compatible with   *
c     *            subroutine dranam (ex: Am242m). If an isotope is    *
c     *            missing in this burnup chain, an error message of   *
c     *            the type 'isotope xxx should not be lumped' may be  *
c     *            issued by subroutine dralum.                        *
c     *  hrch(1)   hollerith neutron-induced reaction identifier for   *
c     *            first daughter                                      *
c     *  en(1)     Q value in MeV for first daughter                   *
c     *  br(1)     branching ratio for an isomeric daughter (=0.0 if   *
c     *            there is no isomeric daughter)                      *
c     *  hrch(2)   hollerith neutron-induced reaction identifier for   *
c     *            second daughter                                     *
c     *  en(2)     Q value in MeV for second daughter                  *
c     *  ...                                                           *
c     *                                                                *
c     *            repeat card 7 for all isotopes present in the       *
c     *            burnup chain. hich='end'/ terminates the chain.     *
c     *                                                                *
c     ******************************************************************
c
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (namax=1500000,maxa=100000,nidmax=20,ipr=1,maxgr=400)
      character text72*72,text12*12,text8*8
      integer labell(18)
      real ener(maxgr+1)
      common/mainio/nsysi,nsyso,nsyse,ntty
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      dimension a(maxa)
      common base(namax)
c
c     ***initialize scratch memory.
      call storag(namax,nidmax,ipr,base)
c
      call timer(time)
      write(nsyso,'(/
     &  '' dragr...produce a draglib format output file'',
     &  24x,f8.1,''s'')') time
      write(nsyse,'(/'' dragr...'',60x,f8.1,''s'')') time
c
c     ***read users input/output unit numbers.
      nendf=0
      npen=0
      ngen=0
      nfp=0
      ndcy=0
      nimpo=0
      nexpo=0
      ipflag=0
      read(nsysi,*) nendf,npen,ngen,nfp,ndcy,nimpo,nexpo,ipflag
c
c     ***open the input/output files
      write(nsyso,'(/
     &  '' input endf unit ............................. '',i10/
     &  '' input pendf unit ............................ '',i10/
     &  '' input gendf unit ............................ '',i10/
     &  '' input endf unit for fission yield data ...... '',i10/
     &  '' input endf unit for radioactive decay data .. '',i10/
     &  '' input draglib unit .......................... '',i10/
     &  '' output draglib unit ......................... '',i10/
     &  '' fission product flag ........................ '',i10)')
     &  nendf,npen,ngen,nfp,ndcy,nimpo,nexpo,ipflag
      if(nendf.ne.0) call openz(nendf,0)
      if(npen.ne.0) call openz(npen,0)
      if(ngen.ne.0) call openz(ngen,0)
      if((nfp.ne.0).and.(ndcy.ne.0)) then
        call openz(nfp,0)
        call openz(ndcy,0)
      endif
      call openz(nexpo,1)
      if(nimpo.eq.0) then
c       ***create a new xsm file and write the signature
        call xsmop('drag',0,1)
        text12='L_DRAGLIB'
        read(text12,'(3a4)') (labell(i),i=1,3)
        call xsmput('SIGNATURE',3,3,labell)
        text12='RELEASE_2003'
        read(text12,'(3a4)') (labell(i),i=1,3)
        call xsmput('VERSION',3,3,labell)
      else
c       ***import the existing xsm file
        call openz(nimpo,0)
        call xsmop('drag',0,1)
        if(nimpo.gt.0) then
           call xsmexp(nimpo,2,2,1)
        else
           call xsmexp(-nimpo,1,2,1)
        endif
        call closz(nimpo)
      endif
c
c     ***read identification for the library
      if(nimpo.eq.0) then
        text72=' '
        read(nsysi,*) text72
        if(text72.ne.' ') then
          read(text72,'(18a4)') (labell(i),i=1,18)
          write(nsyso,'('' identification for the library ....... ''/
     &    1x,a)') text72
          call xsmput('README',18,3,labell)
        endif
      endif
c
c     ***material loop
      if(ngen.eq.0) go to 20
      do
        matno=0
        text8=' '
        read(nsysi,*) matno,text8
        if(matno.eq.0) then
          go to 10
        else if(text8.eq.' ') then
c         automatic composition of the ascii material name.
          if(nendf.eq.0) call error('dragr','endf unit not provided',
     &    ' ')
          call findf(matno,1,451,nendf)
          call contio(nendf,0,0,a,nb,nw)
          iza=nint(c1h+0.1)
          call contio(nendf,0,0,a,nb,nw)
          call dranam(10*iza+l2h,text8)
        endif
        write(nsyso,'('' --> processing material '',a,
     &  ''  (endf identification ='',i8,'')'')') text8,matno
c
        if(nendf.ne.0) then
c         ***construct the "isotopic specification line" for the
c            burnup data.
          call dradat(nendf,matno,text8)
        endif
c
c       ***energy group information for the library
        call drahd(ngen,matno,ng,maxgr,ener)
        call xsmsix(text8,1)
        text72=' '
        read(nsysi,*) text72
        read(text72,'(18a4)') (labell(i),i=1,18)
        write(nsyso,'('' identification for the material ....... '',
     &   a)') text72
        call xsmput('README',18,3,labell)
c
c       ***recover energy limits for dilution-dependent xs data
        read(nsysi,*) eres0,eres1
        eres0=max(0.1d0,eres0)
        if(eres1.eq.0.0d0) eres1=ener(1)
        igres0=1
        igres1=ng
        do ig=1,ng
          if(eres1.le.1.001d0*ener(ig+1)) igres0=ig+1
        enddo
        do ig=ng+1,2,-1
          if(eres0.ge.0.999d0*ener(ig)) igres1=ig-1
        enddo
        write(nsyso,'(/
     &  '' first group of resolved resonance xs data.... '',i10/
     &  '' last group of unresolved resonance xs data... '',i10)')
     &  igres0,igres1
c
c       ***generate draglib file for this material
        call dramat(ngen,matno,ng,igres0,igres1,ipflag)
c
c       ***put autolib data in draglib file for this material
        if(npen.ne.0) then
          read(nsysi,*) eaut0,eaut1,deli
          igaut0=1
          igaut1=ng
          do ig=1,ng
            if(eaut1.le.1.001d0*ener(ig+1)) igaut0=ig+1
          enddo
          do ig=ng+1,2,-1
            if(eaut0.ge.0.999d0*ener(ig)) igaut1=ig-1
          enddo
          write(nsyso,'(/
     &    '' first group of autolib xs data.............. '',i10/
     &    '' last group of autolib xs data............... '',i10/
     &    '' elementary lethargy width................... '',1p,d10.4)')
     &    igaut0,igaut1,deli
          call drauto(npen,matno,ng,ener,eaut0,eaut1,igaut0,igaut1,deli)
        endif
        call xsmsix(' ',2)
      enddo
c
c     ***finished
   10 call closz(ngen)
      call closz(npen)
      call closz(nendf)
c
c     ***compute depletion-related data
   20 if((nfp.ne.0).and.(ndcy.ne.0)) then
        call xsmsix('DEPL-CHAIN',1)
        call dradep(nfp,ndcy)
        call xsmsix(' ',2)
        call closz(ndcy)
        call closz(nfp)
      endif
      call xsmlib
      call xsmcl(1)
c
c     ***export the xsm file
      call xsmop('drag',2,1)
      if(nexpo.gt.0) then
         call xsmexp(nexpo,2,1,1)
      else
         call xsmexp(-nexpo,1,1,1)
      endif
      call xsmcl(2)
c
      call closz(nexpo)
      call timer(time)
      write(nsyso,'(69x,f8.1,''s''/1x,7(''**********''),
     &  ''*******'')') time
      return
      end
c
      subroutine dradat(nendf,matno,text8)
c     ******************************************************************
c     construct the "isotopic specification line" for the burnup data.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter(maxa=2000,maxedi=11,lz=6)
      character namedi(maxedi)*8,hline*80
      dimension a(maxa)
      integer malist(maxedi)
      logical lfind
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      common/mainio/nsysi,nsyso,nsyse,ntty
      data malist/16,17,18,28,37,102,103,104,105,107,108/
      data namedi/'n2n','n3n','nftot','nnp','n4n','ng','np','nd','nt',
     & 'na','n2a'/
c
      write(nsyso,'(/'' isotopic specification line for material'',i8,
     & '':''/1x,80(''-''))') matno
      hline=' '
      iof=1
      call findf(matno,1,451,nendf)
      call contio(nendf,0,0,a,nb,nw)
      iza=nint(c1h+0.1)
      call contio(nendf,0,0,a,nb,nw)
      write(hline(iof:),'(a8,1x)') text8
      iof=iof+9
      do ied=1,maxedi
        call repoz(nendf)
        lfind=.false.
        do while (.not.lfind)
          if (nendf.lt.0) then
            read(-nendf,end=50) math,mfh,mth,nb,nw
          else if (nendf.gt.0) then
            read(nendf,'(66x,i4,i2,i3,i5)',end=50) math,mfh,mth,nsp
          endif
          lfind=(math.eq.matno).and.(mfh.eq.3).and.(mth.eq.malist(ied))
        enddo
        if (.not.lfind) go to 50
        if(iof+26.gt.80) then
          write(nsyso,'(1x,a80)') hline
          hline=' '
          iof=10
        endif
        iof2=index(namedi(ied),' ')
        write(hline(iof:),'(a,1x)') namedi(ied)(:iof2-1)
        iof=iof+iof2
        if(mth.eq.18) then
c         ***fission -- recover info in mf=1, mt=458
          call repoz(nendf)
          lfind=.false.
          do while (.not.lfind)
            if (nendf.lt.0) then
              read(-nendf,end=30) math,mfh,mth,nb,nw
            else if (nendf.gt.0) then
              read(nendf,'(66x,i4,i2,i3,i5)',end=30) math,mfh,mth,nsp
            endif
            lfind=(math.eq.matno).and.(mfh.eq.1).and.(mth.eq.458)
          enddo
          if (lfind) go to 40
   30     write(hline(iof:),'(''???????????'',f6.3,1x)') 0.0
          iof=iof+18
          go to 50
   40     call listio(nendf,0,0,a,nb,nw)
c         ***remove energy from delayed betas and gammas as they are
c            taken into account separately.
          c1h=a(lz+15)-a(lz+9)-a(lz+11)
        else
c         ***else -- recover the Q of the reaction (may be negative)
          call contio(nendf,0,0,a,nb,nw)
        endif
        write(hline(iof:),'(1p,e11.4,'' 0.000 '')') c1h*1.0e-6
        iof=iof+18
   50   continue
      enddo
      if(iof+1.gt.80) then
        write(nsyso,'(1x,a80)') hline
        hline=' '
        iof=10
      endif
      write(hline(iof:),'(''/'')')
      write(nsyso,'(1x,a80)') hline
      write(nsyso,'(1x,80(''-''))')
      return
      end
c
      subroutine drahd(ngen,matno,ng,maxgr,ener)
c     ******************************************************************
c     write energy group data to the xsm file
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter(maxa=2000)
      real ener(maxgr+1)
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      dimension a(maxa)
      logical lsame
c
      call findf(matno,1,451,ngen)
      call contio(ngen,0,0,a,nb,nw)
      nz=nint(a(4))
      call listio(ngen,0,0,a(1),nb,nw)
      loc=1+nw
      do while (nb.ne.0)
        if (loc+302.gt.maxa) call error('drahd',
     &  'endf input size exceeded',' ')
        call moreio(ngen,0,0,a(loc),nb,nw)
        loc=loc+nw
      enddo
      ng=nint(a(3))
      if(ng.gt.maxgr) call error('drahd','maxgr overflow',' ')
      call xsmlen('ENERGY',ilong,ityxsm)
      if(ilong.eq.0) then
        do ig=1,ng+1
          ener(ng-ig+2)=a(7+nz+ig)
        enddo
        call xsmput('ENERGY',ng+1,2,ener)
      else
        if(ilong.ne.ng+1) call error('drahd','invalid number of groups',
     &  ' ')
        call xsmget('ENERGY',ener)
        lsame=.true.
        do ig=1,ng
          lsame=lsame.and.(abs(ener(ng-ig+2)-a(7+nz+ig)).le.1.0e-3*
     &    ener(ng-ig+2))
        enddo
        if(.not.lsame) call error('drahd','inconsistent energy mesh',
     &  ' ')
      endif
      return
      end
c
      subroutine dramat(ngen,matno,ng,igres0,igres1,ipflag)
c     ******************************************************************
c     write draglib data to the xsm (direct access) file for endf
c     material matno
c     the dilution-dependent xs deltas are recovered between groups
c     igres0 and igres1.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (maxa=2000,maxgr=400,maxnl=8,maxnz=30,maxtmp=10,
     & maxedi=13)
      dimension rv(maxgr,maxnl,maxnz),flux(maxgr,maxnl,maxnz),a(maxa),
     & aa(6),deltau(maxgr),rv2(maxgr,maxnl,1),rm2(maxgr,maxgr,maxnl,1)
      logical lfind,lsame,lover,exist,exist2,exist3
      character cd*4,namedi(maxedi)*8,hsmg*131
      real awr,dilut(maxnz),olddil(maxnz),temps(maxtmp),oldtmp(maxtmp),
     & vector(maxgr)
      integer igfirs(maxnl),iglast(maxnl),malist(maxedi)
      data malist/1,2,16,17,18,28,37,102,103,104,105,107,108/
      data namedi/'NTOT0','NELAS','N2N','N3N','NFTOT','NNP','N4N','NG',
     & 'NP','ND','NT','NA','N2A'/
c
c recover the new dilution values.
      call findf(matno,1,451,ngen)
      call contio(ngen,0,0,a,nb,nw)
      awr=a(2)
      call xsmput('AWR',1,2,awr)
      nz=nint(a(4))
      if(nz.gt.maxnz) call error('dramat','maxnz overflow',' ')
      call listio(ngen,0,0,a(1),nb,nw)
      loc=1+nw
      do while(nb.ne.0)
        if (loc+302.gt.maxa) call error('dramat',
     &  'endf input size exceeded',' ')
        call moreio(ngen,0,0,a(loc),nb,nw)
        loc=loc+nw
      enddo
      do iz=1,nz
        dilut(nz-iz+1)=a(7+iz)
      enddo
      if(dilut(nz).lt.1.0e10) call error('dramat','missing infinite di'
     & //'lution value',' ')
      do ig=1,ng
        deltau(ig)=log(a(7+nz+(ng-ig+2))/a(7+nz+(ng-ig+1)))
      enddo
c
c recover the temperature values.
      ntmp=0
   10 lfind=.false.
      do while(.not.lfind)
        if (ngen.lt.0) then
          read(-ngen,end=20) math,mfh,mth,nb,nw
        else if (ngen.gt.0) then
          read(ngen,'(6e11.0,i4,i2,i3,i5)',end=20) aa,math,mfh,mth,nsp
        endif
        lfind=(math.eq.matno).and.(mfh.eq.3).and.(mth.eq.1)
      enddo
      if(.not.lfind) go to 20
      call listio(ngen,0,0,a(1),nb,nw)
      ntmp=ntmp+1
      if(ntmp.gt.maxtmp) call error('dramat','maxtmp overflow',' ')
      temps(ntmp)=a(1)
      call tomend(ngen,0,0,a,nb,nw)
      go to 10
   20 if(ntmp.eq.0) call error('dramat','no temperatures on gendf',' ')
c
c set the 'TEMPERATURE' record on xsm.
      call xsmlen('TEMPERATURE',ilong,ityxsm)
      if(ilong.gt.0) then
        if(ilong.gt.maxtmp) call error('dramat','maxtmp overflow',' ')
        call xsmget('TEMPERATURE',oldtmp)
        lover=.true.
        do itm=1,ntmp
          lover=lover.and.(temps(itm).gt.oldtmp(ilong))
        enddo
        if(lover) then
          if(ntmp+ilong.gt.maxtmp) call error('dramat','maxtmp overflow'
     &    ,' ')
          do itm=1,ntmp
             ilong=ilong+1
             oldtmp(ilong)=temps(itm)
          enddo
        endif
      else
        ilong=ntmp
        do itm=1,ntmp
           oldtmp(itm)=temps(itm)
        enddo
      endif
      call xsmput('TEMPERATURE',ilong,2,oldtmp)
c
c loop over the temperatures.
      do itm=1,ntmp
        do itm0=1,ilong
          if (abs(temps(itm)-oldtmp(itm0)).le.1.0e-5*temps(itm)) then
            isbmat=itm0
            go to 30
          endif
        enddo
        call error('dramat','missing temperature',' ')
   30   ytemp=temps(itm)
        write (cd,'(i4.4)') isbmat
        call xsmsix('SUBTMP'//cd,1)
c
c       ***store the new dilutions
        call xsmlen('DILUTION',nz0,ityxsm)
        if(nz0.gt.0) then
          if(nz0.gt.maxnz) call error('dramat','maxnz overflow',' ')
          call xsmget('DILUTION',olddil)
          lover=.true.
          do iz=1,nz-1
            lover=lover.and.(dilut(iz).gt.olddil(nz0))
          enddo
          if(lover) then
            if(nz+nz0-1.gt.maxnz) call error('dramat','maxnz overflow',
     &      ' ')
            do iz=1,nz-1
              olddil(nz0+iz)=dilut(iz)
            enddo
            call xsmput('DILUTION',nz+nz0-1,2,olddil)
          else
            do iz0=1,nz0
              if(abs(dilut(1)-olddil(iz0)).le.1.0e-5*dilut(1)) then
                nz0bis=iz0-1
                go to 34
              endif
            enddo
            call error('dramat','inconsistent dilutions(1).',' ')
   34       if(nz0bis+nz-1.gt.nz0) call error('dramat','inconsistent d'
     &      //'ilutions(2).',' ')
            do iz=1,nz-1
              if(abs(dilut(iz)-olddil(nz0bis+iz)).gt.1.0e-5*dilut(iz))
     &        call error('dramat','inconsistent dilutions(3).',' ')
            enddo
            nz0=nz0bis
          endif
        else if(nz.gt.1) then
          call xsmput('DILUTION',nz-1,2,dilut)
        endif
c
c       ***recover thermal correction x-sections and compute the
c          correction term.
c       NOTE: Apparently, there is a bug in THERMR so that the matrix
c             221 to 250 reactions does not sum to the corresponding
c             vector reactions. We will therefore avoid using the
c             vector reactions for thermal scattering.
c       call fvect(maxgr,maxnl,1,ngen,matno,imt,ytemp,ng,nl2,nzgar,rv2,
c    &  flux,exist2)
        do il=1,maxnl
          do ig1=1,maxgr
            rv2(ig1,il,1)=0.0
          enddo
        enddo
        exist3=.false.
        do imt=250,221,-1
          call fmatr(maxgr,maxnl,1,ngen,matno,imt,ytemp,ng0,nl2,nzgar,
     &    rm2,flux,exist2)
          if(exist2) then
            exist3=.true.
            nlgar=nl2
            if(ng0.ne.ng) call error('dramat','inconsistent ng(1).',' ')
            do il=1,nl2
              do ig1=1,ng
                do ig2=1,ng
                  rv2(ig1,il,1)=rv2(ig1,il,1)+rm2(ig1,ig2,il,1)
                enddo
              enddo
            enddo
          endif
        enddo
   35   if(exist3) then
          call fvect(maxgr,maxnl,maxnz,ngen,matno,2,ytemp,ng0,nl,nzgar,
     &    rv,flux,exist)
          if(nl.gt.nlgar) call error('dramat','inconsistent number o'//
     &    'f Legendre orders(1)',' ')
          do il=1,nl
            do ig=1,ng
              if(rv2(ig,il,1).ne.0.0) then
                rv2(ig,il,1)=rv2(ig,il,1)-rv(ig,il,1)
              endif
            enddo
          enddo
        endif
c
c       ***loop over the vector cross section sets.
        do ied=1,maxedi
c
c         ***process infinite dilution x-sections.
          call fvect(maxgr,maxnl,maxnz,ngen,matno,malist(ied),ytemp,ng0,
     &    nl,nzgar,rv,flux,exist)
          if(.not.exist) go to 40
          if(ng0.ne.ng) call error('dramat','inconsistent ng(2).',' ')
c
          if((malist(ied).le.2).and.exist3) then
c            ***perform thermal correction on 'NTOT0' and 'NELAS'
c            x-sections.
             do iz=1,nzgar
               do ig=1,ng
                 rv(ig,1,iz)=rv(ig,1,iz)+rv2(ig,1,1)
               enddo
             enddo
          endif
c
          do ig=ng,1,-1
            igmax=ig
            if(rv(ig,1,1).ne.0.0) go to 36
          enddo
          cycle
   36     do ig=1,igmax
            vector(ig)=rv(ig,1,1)
          enddo
          call xsmput(namedi(ied),igmax,2,vector)
          if(nzgar.eq.1) go to 40
c
c         ***find the last self-shielded group.
          do il=1,maxnl
            igfirs(il)=igres0
            iglast(il)=igres1+1
          enddo
          do iz=1,nz-1
            write (cd,'(i4.4)') nz0+iz
            call xsmsix('SUBMAT'//cd,1)
c
c           ***process finite dilution x-sections
            do ig=1,iglast(1)-1
              vector(ig)=0.0
            enddo
            do ig=igfirs(1),iglast(1)-1
              if(flux(ig,1,1).eq.0.0) cycle
              if(rv(ig,1,1).eq.0.0) cycle
              flux(ig,1,nz-iz+1)=flux(ig,1,nz-iz+1)/flux(ig,1,1)
              if(flux(ig,1,nz-iz+1).gt.3.0) then
                write(hsmg,'(19hinconsistent flux (,1p,e10.3,5h) in ,
     &          5hgroup,i4,13h at dilution=,e10.2,5h barn)')
     &          flux(ig,1,nz-iz+1),ig,dilut(iz)
                call error('dramat',hsmg,' ')
              endif
              vector(ig)=rv(ig,1,nz-iz+1)*flux(ig,1,nz-iz+1)-rv(ig,1,1)
            enddo
            call xsmput(namedi(ied),iglast(1)-1,2,vector)
c
            if(ied.eq.1) then
c             ***process 'NWT0' finite dilution fluxes
              do ig=igfirs(1),iglast(1)-1
                vector(ig)=flux(ig,1,nz-iz+1)-1.0d0
              enddo
              call xsmput('NWT0',iglast(1)-1,2,vector)
            endif
c
            call xsmsix(' ',2)
          enddo
   40     continue
        enddo
c
c       ***process fission x-sections.
        call drafis(ngen,matno,ytemp,nz0,deltau,igfirs(1),iglast(1))
c
c       ***process scattering x-sections.
        call drasc(ngen,matno,ytemp,nz0,deltau,igfirs,iglast,ipflag)
c
        call xsmsix(' ',2)
      enddo
      return
      end
c
      subroutine drauto(npen,matno,ng,ener,eaut0,eaut1,igaut0,igaut1,
     & deli)
c     ******************************************************************
c     put autolib data in draglib file for material matno
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
      parameter (ndoubl=2)
*else
      parameter (ndoubl=1)
*endif
      parameter(maxa=2000,maxgr=400,maxtmp=10)
      common/util/npage,iverf
      common/mainio/nsysi,nsyso,nsyse,ntty
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      real ener(ng+1),oldtmp(maxtmp),gar1t(maxgr),gar1s(maxgr),
     & gar1f(maxgr),gar2(maxgr**2),rbase(1)
      integer nfs(maxgr),ijj(maxgr),njj(maxgr)
      dimension a(maxa)
      logical lfind,lfiss
      character cd*4
      common base(1)
      equivalence(base(1),rbase(1))
c
c     ***find the autolib energy limits.
      nbfine=0
      errmax=0.0
      do ig=igaut0,igaut1
        zbin=log(ener(ig)/ener(ig+1))
        nbin=int(zbin/deli+0.01)
        err1=abs(real(nbin)*deli-zbin)/abs(zbin)
        err2=abs(real(nbin)*deli+deli/3.0d0-zbin)/abs(zbin)
        err3=abs(real(nbin)*deli+2.0d0*deli/3.0d0-zbin)/abs(zbin)
        err=min(err1,err2,err3)
        if(err.ne.err1) nbin=nbin+1
        nbfine=nbfine+nbin
        errmax=max(errmax,err)
      enddo
      write(nsyso,'(/
     &  '' number of bin energy groups in autolib....... '',i10/
     &  '' percent accuracy of the bin mesh............. '',1p,d9.2,
     &  ''%'')') nbfine,errmax*100.0
      call reserv('bener',nbfine+1,lbener,base)
      base(lbener)=ener(igaut0)
      do ig=1,ng
        nfs(ig)=0
      enddo
      uuu=log(ener(1)/ener(igaut0))
      ibin=1
      do ig=igaut0,igaut1
        zbin=log(ener(ig)/ener(ig+1))
        nbin=int(zbin/deli+0.01)
        do i=1,nbin
          ibin=ibin+1
          uuu=uuu+deli
          base(lbener+ibin-1)=ener(1)*exp(-uuu)
        enddo
        err1=abs(real(nbin)*deli-zbin)/abs(zbin)
        err2=abs(real(nbin)*deli+deli/3.0d0-zbin)/abs(zbin)
        err3=abs(real(nbin)*deli+2.0d0*deli/3.0d0-zbin)/abs(zbin)
        err=min(err1,err2,err3)
        if(err.eq.err2) then
          nbin=nbin+1
          ibin=ibin+1
          uuu=uuu+deli/3.0d0
          base(lbener+ibin-1)=ener(1)*exp(-uuu)
        else if(err.eq.err3) then
          nbin=nbin+1
          ibin=ibin+1
          uuu=uuu+2.0d0*deli/3.0d0
          base(lbener+ibin-1)=ener(1)*exp(-uuu)
        endif
        if(ibin.gt.nbfine+1) call error('drauto','bin overflow',' ')
        nfs(ig)=nbin
      enddo
      call reserv('bsi1',nbfine,lbsig1,base)
      call reserv('bsi2',nbfine,lbsig2,base)
      call reserv('bsi3',nbfine,lbsig3,base)
      call reserv('bgar',nbfine+1,lbgar,base)
      lbgar=ndoubl*(lbgar-1)+1
      do ibin=1,nbfine+1
        rbase(lbgar+ibin-1)=real(base(lbener+ibin-1))
      enddo
      call xsmput('BIN-NFS',ng,1,nfs)
      call xsmput('BIN-ENERGY',nbfine+1,2,rbase(lbgar))
c
c     ***recover the temperature values.
      call repoz(npen)
   10 lfind=.false.
      do while(.not.lfind)
        if (npen.lt.0) then
          read(-npen,end=30) math,mfh,mth,nb,nw,aa
        else if (npen.gt.0) then
          read(npen,'(6e11.0,i4,i2,i3,i5)',end=30) aa,math,mfh,mth,nsp
        endif
        lfind=(math.eq.matno).and.(mfh.eq.1).and.(mth.eq.451)
      enddo
      if(.not.lfind) go to 30
      call contio(npen,0,0,a,nb,nw)
      call contio(npen,0,0,a,nb,nw)
      call contio(npen,0,0,a,nb,nw)
      temps=c1h
      call xsmlen('TEMPERATURE',ntmp,ityxsm)
      if(ntmp.eq.0) call error('drauto','missing temperatures on xsm',
     & ' ')
      if(ntmp.gt.maxtmp) call error('drauto','oldtmp overflow',' ')
      call xsmget('TEMPERATURE',oldtmp)
      isbmat=0
      do itm=1,ntmp
        if (abs(temps-oldtmp(itm)).le.1.0e-3*temps) then
          isbmat=itm
          go to 20
        endif
      enddo
   20 if(isbmat.eq.0) call error('drauto','missing temperature',' ')
      write (cd,'(i4.4)') isbmat
      call xsmsix('SUBTMP'//cd,1)
      nw=npage+50
      call reserv('scr1',nw,lscr1,base)
c
c     ***recover the pointwise total cross sections.
      call findf(matno,3,1,npen)
      call contio(npen,0,0,base(lscr1),nb,nw)
      ep=0.0d0
      call gety1(ep,enext,idis,tp,npen,base(lscr1))
      em=eaut0
      call gety1(em,enext,idis,tm,npen,base(lscr1))
      ibin=nbfine
      gar=0.0d0
      emlog=log(eaut1/em)
      do while(ep*(1.0d0+1.0d-6).lt.eaut1)
        if(ibin.le.0) call error('drauto','invalid index',' ')
        ep=min(enext,base(lbener+ibin-1))
        eplog=log(eaut1/ep)
        call gety1(ep,enext,idis,tp,npen,base(lscr1))
        gar=gar+0.5d0*(tm+tp)*(emlog-eplog)
        if(ep.eq.base(lbener+ibin-1)) then
          deltau=log(base(lbener+ibin-1)/base(lbener+ibin))
          base(lbsig1+ibin-1)=gar/deltau
          ibin=ibin-1
          gar=0.0d0
        endif
        em=ep
        tm=tp
        emlog=eplog
      enddo
c
c     ***recover the pointwise scattering cross sections.
      call findf(matno,3,2,npen)
      call contio(npen,0,0,base(lscr1),nb,nw)
      ep=0.0d0
      call gety1(ep,enext,idis,tp,npen,base(lscr1))
      em=eaut0
      call gety1(em,enext,idis,tm,npen,base(lscr1))
      ibin=nbfine
      gar=0.0d0
      emlog=log(eaut1/em)
      do while(ep*(1.0d0+1.0d-6).lt.eaut1)
        if(ibin.le.0) call error('drauto','invalid index',' ')
        ep=min(enext,base(lbener+ibin-1))
        eplog=log(eaut1/ep)
        call gety1(ep,enext,idis,tp,npen,base(lscr1))
        gar=gar+0.5d0*(tm+tp)*(emlog-eplog)
        if(ep.eq.base(lbener+ibin-1)) then
          deltau=log(base(lbener+ibin-1)/base(lbener+ibin))
          base(lbsig2+ibin-1)=gar/deltau
          ibin=ibin-1
          gar=0.0d0
        endif
        em=ep
        tm=tp
        emlog=eplog
      enddo
c
c     ***recover the pointwise fission cross sections.
      call xsmlen('NUSIGF',ilfiss,ityxsm)
      if(ilfiss.gt.0) then
        call findf(matno,3,18,npen)
        call contio(npen,0,0,base(lscr1),nb,nw)
        ep=0.0d0
        call gety1(ep,enext,idis,tp,npen,base(lscr1))
        em=eaut0
        call gety1(em,enext,idis,tm,npen,base(lscr1))
        ibin=nbfine
        gar=0.0d0
        emlog=log(eaut1/em)
        do while(ep*(1.0d0+1.0d-6).lt.eaut1)
          if(ibin.le.0) call error('drauto','invalid index',' ')
          ep=min(enext,base(lbener+ibin-1))
          eplog=log(eaut1/ep)
          call gety1(ep,enext,idis,tp,npen,base(lscr1))
          gar=gar+0.5d0*(tm+tp)*(emlog-eplog)
          if(ep.eq.base(lbener+ibin-1)) then
            deltau=log(base(lbener+ibin-1)/base(lbener+ibin))
            base(lbsig3+ibin-1)=gar/deltau
            ibin=ibin-1
            gar=0.0d0
          endif
          em=ep
          tm=tp
          emlog=eplog
        enddo
      endif
c
c     ***normalization of BIN cross sections.
      do ig=1,ng
        gar1t(ig)=0.0
        gar1s(ig)=0.0
        gar1f(ig)=0.0
      enddo
      do ig=1,ng*ng
        gar2(ig)=0.0
      enddo
      call xsmget('NTOT0',gar1t)
      call xsmget('IJJS00',ijj)
      call xsmget('NJJS00',njj)
      call xsmget('SCAT00',gar2)
      if(ilfiss.gt.0) call xsmget('NUSIGF',gar1f)
      igar=0
      do ig2=1,ng
        do ig1=ijj(ig2),ijj(ig2)-njj(ig2)+1,-1
          igar=igar+1
          gar1s(ig1)=gar1s(ig1)+gar2(igar)
        enddo
      enddo
      ibin0=0
      do ig=igaut0,igaut1
        sum1=0.0
        sum2=0.0
        sum3=0.0
        sum4=0.0
        do ibin=ibin0+1,ibin0+nfs(ig)
          deltau=log(base(lbener+ibin-1)/base(lbener+ibin))
          sum1=sum1+deltau
          sum2=sum2+base(lbsig1+ibin-1)*deltau
          sum3=sum3+(base(lbsig1+ibin-1)-base(lbsig2+ibin-1))*deltau
          if(ilfiss.gt.0) sum4=sum4+base(lbsig3+ibin-1)*deltau
        enddo
        fact1=gar1t(ig)*(sum1/sum2)
        fact2=(gar1t(ig)-gar1s(ig))*(sum1/sum3)
        if((ilfiss.gt.0).and.(sum4.gt.0.0)) fact3=gar1f(ig)*(sum1/sum4)
        do ibin=ibin0+1,ibin0+nfs(ig)
          base(lbsig2+ibin-1)=fact2*base(lbsig2+ibin-1)+(fact1-fact2)*
     &    base(lbsig1+ibin-1)
          base(lbsig1+ibin-1)=fact1*base(lbsig1+ibin-1)
          if(ilfiss.gt.0) base(lbsig3+ibin-1)=fact3*base(lbsig3+ibin-1)
        enddo
        ibin0=ibin0+nfs(ig)
      enddo
c
c     ***store the BIN cross sections on xsm.
      do ibin=1,nbfine
        rbase(lbgar+ibin-1)=real(base(lbsig1+ibin-1))
      enddo
      call xsmput('BIN-NTOT0',nbfine,2,rbase(lbgar))
      if(isbmat.eq.1) then
        write(nsyso,'(''-----------------------------------'')')
        write(nsyso,100) (ener(i),i=igaut0,igaut1+1)
        write(nsyso,'(''--BIN-ENERGY-----------------------'')')
        write(nsyso,100) (base(lbener+i-1),i=1,nbfine+1)
        write(nsyso,'(''--BIN-NTOT0------------------------'')')
        write(nsyso,100) (base(lbsig1+i-1),i=1,nbfine),
     &  base(lbsig1+nbfine-1)
      endif
      do ibin=1,nbfine
        rbase(lbgar+ibin-1)=real(base(lbsig2+ibin-1))
      enddo
      call xsmput('BIN-SIGS00',nbfine,2,rbase(lbgar))
      if(ilfiss.gt.0) then
        lfiss=.false.
        do ibin=1,nbfine
          lfiss=lfiss.or.(base(lbsig3+ibin-1).ne.0.0)
          rbase(lbgar+ibin-1)=real(base(lbsig3+ibin-1))
        enddo
        if(lfiss) call xsmput('BIN-NUSIGF',nbfine,2,rbase(lbgar))
      endif
      call xsmsix(' ',2)
c
      call tomend(npen,0,0,a,nb,nw)
      call releas('scr1',0,base)
      go to 10
   30 call releas('bgar',0,base)
      call releas('bsi3',0,base)
      call releas('bsi2',0,base)
      call releas('bsi1',0,base)
      call releas('bener',0,base)
      return
  100 format(10(1p,e12.5,1h,))
      end
c
      subroutine drafis(nin,matd,ytemp,nz0,deltau,igfirs,iglast)
c     ******************************************************************
c     utility routine for recovering fission-related information from a
c     gendf file.
c     input parameters:
c     nin     file unit number of the gendf file.
c     matd    material number.
c     ytemp   absolute temperature (Kelvin).
c     nz0     previous dilution index.
c     deltau  dilution widths.
c     igfirs  fastest group index with self-shielding effect.
c     iglast  fastest thermal group index with no self-shielding effect.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (maxa=10000,maxgr=400,maxnl=8,maxnz=30,maxdel=10,lz=6)
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      common/mainio/nsysi,nsyso,nsyse,ntty
      integer nin,matd,nz0,iglast,mflist(4)
      logical lfind,exist,exist2
      character text2*2,cd*4
      dimension sigfm(maxgr,maxgr,maxnl,maxnz),rv(maxgr,maxnl,maxnz),
     & sigf(maxgr,maxnl,maxnz),flux(maxgr,maxnl,maxnz),a(maxa),aa(6),
     & ssum(maxdel),deltau(maxgr),chidel(maxgr,maxdel),vsum(maxgr),
     & sigfm2(maxgr,maxgr,maxnl,maxnz)
      real gar1(maxdel),gar2(maxgr)
      data mflist/19,20,21,38/
c
c initialize the sigf and sigfm arrays.
      do iz=1,maxnz
        do il=1,maxnl
          do ig=1,maxgr
            sigf(ig,il,iz)=0.0
            do jg=1,maxgr
              sigfm(jg,ig,il,iz)=0.0
            enddo
          enddo
        enddo
      enddo
c
c recover the 'NFTOT' vector fission cross sections.
      call fvect(maxgr,maxnl,maxnz,nin,matd,18,ytemp,ng,nl,nzv,sigf,
     & flux,exist)
      if(.not.exist) return
c
c recover the 'NFTOT' matrix fission cross sections.
      call fmatr(maxgr,maxnl,maxnz,nin,matd,18,ytemp,ng,nl,nzm,sigfm,
     & flux,exist)
      if(.not.exist) then
c       try to recover a fission matrix by summing partial fission
c       matrices (if they exist).
        nzm=0
        do nf=1,4
          call fmatr(maxgr,maxnl,maxnz,nin,matd,mflist(nf),ytemp,ng,nl,
     &    nz2,sigfm2,flux,exist2)
          exist=exist.or.exist2
          if(exist2) then
            nzm=max(nzm,nz2)
            do iz=1,nz2
              do il=1,nl
                do ig=1,ng
                  do jg=1,ng
                    sigfm(jg,ig,il,iz)=sigfm(jg,ig,il,iz)+
     &              sigfm2(jg,ig,il,iz)
                  enddo
                enddo
              enddo
            enddo
          endif
        enddo
      endif
      if(.not.exist) then
        write(nsyso,'('' missing fission matrix  '',
     &  ''(endf identification ='',i8,'')'')') matd
        return
      endif
c
c recover the delayed spectra.
      do idel=1,maxdel
        do ig=1,maxgr
          chidel(ig,idel)=0.0
        enddo
      enddo
      call repoz(nin)
   10 lfind=.false.
      do while (.not.lfind)
        if (nin.lt.0) then
          read(-nin,end=20) math,mfh,mth,nb,nw
        else if (nin.gt.0) then
          read(nin,'(6e11.0,i4,i2,i3,i5)',end=20) aa,math,mfh,mth,nsp
        endif
        lfind=(math.eq.matd).and.(mfh.eq.5).and.(mth.eq.455)
      enddo
      if (.not.lfind) go to 20
      if (nin.lt.0) then
        call skiprz(nin,-1)
        read(-nin) math,mfh,mth,nb,nw,aa
      endif
      ndel=nint(aa(3))
      nz=nint(aa(4))
      ng=nint(aa(6))
      call listio(nin,0,0,a(1),nb,nw)
      if (abs(a(1)-ytemp).gt.1.0e-3) then
        call tomend(nin,0,0,a,nb,nw)
        go to 10
      endif
      if(ng.gt.maxgr) call error('drafis','maxgr overflow',' ')
      if(nz.ne.1) call error('drafis','no dilution dependence expected',
     & ' ')
      if(ndel.gt.maxdel) call error('drafis','maxdel overflow',' ')
      ng2=l1h
      loc=1+nw
      do while (nb.ne.0)
        if (loc+302.gt.maxa) call error('drafis',
     &  'endf input size exceeded',' ')
        call moreio(nin,0,0,a(loc),nb,nw)
        loc=loc+nw
      enddo
      do idel=1,ndel
        gar1(idel)=a(lz+idel)
      enddo
      if(ndel.gt.0) call xsmput('LAMBDA-D',ndel,2,gar1)
      do idel=1,ndel
        ssum(idel)=0.0
        do ig=1,ng2-1
          ssum(idel)=ssum(idel)+a(lz+ndel+(ig-1)*ndel+idel)
        enddo
        do ig=1,ng2-1
          chidel(ng-ig+1,idel)=a(lz+ndel+(ig-1)*ndel+idel)/ssum(idel)
        enddo
        do ig=1,ng
          gar2(ig)=chidel(ig,idel)
        enddo
        write(text2,'(i2.2)') idel
        call xsmput('CHI'//text2,ng,2,gar2)
      enddo
      call fvect(maxgr,maxnl,maxnz,nin,matd,455,ytemp,ng,nl,nz,rv,
     & flux,exist)
      call fvect(maxgr,maxnl,maxnz,nin,matd,18,ytemp,ng,nl,nz,sigf,
     & flux,exist)
      do iz=1,nz
        do ig=igfirs,iglast-1
          if(flux(ig,1,1).eq.0.0) cycle
          flux(ig,1,nz-iz+1)=flux(ig,1,nz-iz+1)/flux(ig,1,1)
        enddo
      enddo
      do idel=1,ndel
        do ig=1,ng
          gar2(ig)=rv(ig,1,1)*sigf(ig,1,1)*ssum(idel)
        enddo
        do ig=ng,1,-1
          igmax=ig
          if(gar2(ig).ne.0.0) go to 15
        enddo
   15   write(text2,'(i2.2)') idel
        call xsmput('NUSIGF'//text2,igmax,2,gar2)
        do iz=1,nz-1
c
c         ***process finite dilution x-sections
          do ig=1,iglast-1
            gar2(ig)=0.0
          enddo
          do ig=igfirs,iglast-1
            if(sigf(ig,1,1).eq.0.0) cycle
            gar2(ig)=rv(ig,1,1)*ssum(idel)*(sigf(ig,1,nz-iz+1)*
     &      flux(ig,1,nz-iz+1)-sigf(ig,1,1))
          enddo
          do ig=iglast-1,1,-1
            igmax=ig
            if(gar2(ig).ne.0.0) go to 16
          enddo
          go to 17
   16     write (cd,'(i4.4)') nz0+iz
          call xsmsix('SUBMAT'//cd,1)
          call xsmput('NUSIGF'//text2,igmax,2,gar2)
          call xsmsix(' ',2)
   17     continue
        enddo
        do iz=1,nz
          do ig2=1,ng
            do ig1=1,ng
              sigfm(ig1,ig2,1,iz)=sigfm(ig1,ig2,1,iz)+rv(ig1,1,1)*
     &        sigf(ig1,1,iz)*ssum(idel)*chidel(ig2,idel)
            enddo
          enddo
        enddo
      enddo
c
c process the prompt 'NUSIGF' and 'CHI' vectors.
   20 call fvect(maxgr,maxnl,maxnz,nin,matd,18,ytemp,ng,nl,nz,sigf,
     & flux,exist)
      if((nzm.ne.nz).and.(nzm.ne.1)) then
        call error('drafis','invalid number of dilutions with mf=6, mt='
     &  //'18',' ')
      endif
      do iz=1,nzm
        if(iz.eq.1) then
          all=0.0
          do ig2=1,ng
            vsum(ig2)=0.0
            do ig1=1,ng
              vsum(ig2)=vsum(ig2)+sigfm(ig1,ig2,1,iz)*flux(ig1,1,iz)
            enddo
            all=all+vsum(ig2)
          enddo
          do ig1=1,ng
            gar2(ig1)=vsum(ig1)/all
          enddo
          call xsmput('CHI',ng,2,gar2)
        endif
        do ig1=1,ng
          vsum(ig1)=0.0
          do ig2=1,ng
            vsum(ig1)=vsum(ig1)+sigfm(ig1,ig2,1,iz)
          enddo
        enddo
        if(nzm.gt.1) then
          do ig=1,ng
            sigf(ig,1,iz)=vsum(ig)
          enddo
        endif
      enddo
      if(nzm.eq.1) then
        do iz=nz,1,-1
          do ig=1,ng
            if(vsum(ig).ne.0.0) then
              sigf(ig,1,iz)=sigf(ig,1,iz)*vsum(ig)/sigf(ig,1,1)
            endif
          enddo
        enddo
      endif
      do ig=ng,1,-1
        igmax=ig
        if(sigf(ig,1,1).ne.0.0) go to 30
      enddo
   30 do ig=1,igmax
        gar2(ig)=sigf(ig,1,1)
      enddo
      call xsmput('NUSIGF',igmax,2,gar2)
      do iz=1,nz-1
c
c       ***process 'NUSIGF' finite dilution x-sections
        do ig=1,iglast-1
          gar2(ig)=0.0d0
        enddo
        do ig=igfirs,iglast-1
          if(flux(ig,1,1).eq.0.0) cycle
          if(sigf(ig,1,1).eq.0.0) cycle
          flux(ig,1,nz-iz+1)=flux(ig,1,nz-iz+1)/flux(ig,1,1)
          gar2(ig)=sigf(ig,1,nz-iz+1)*flux(ig,1,nz-iz+1)-sigf(ig,1,1)
        enddo
        do ig=iglast-1,1,-1
          igmax=ig
          if(gar2(ig).ne.0.0) go to 40
        enddo
        go to 50
   40   write (cd,'(i4.4)') nz0+iz
        call xsmsix('SUBMAT'//cd,1)
        call xsmput('NUSIGF',igmax,2,gar2)
        call xsmsix(' ',2)
   50   continue
      enddo
      return
      end
c
      subroutine drasc(nin,matd,ytemp,nz0,deltau,igfirs,iglast,ipflag)
c     ******************************************************************
c     utility routine for recovering scattering-related information from
c     a gendf file.
c     input parameters:
c     nin     file unit number of the gendf file.
c     matd    material number.
c     ytemp   absolute temperature (Kelvin).
c     nz0     previous dilution index.
c     deltau  dilution widths.
c     igfirs  fastest group index with self-shielding effect.
c     iglast  fastest thermal group index with no self-shielding effect.
c     ipflag  fission product flag.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (maxa=2000)
      parameter(maxgr=400,maxnl=8,maxnz=30,maxmat=45,maxgar=maxgr*maxgr)
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      common/mainio/nsysi,nsyso,nsyse,ntty
      integer nin,matd,nz0,mtlist(maxmat),ijj(maxgr),njj(maxgr),
     & igfirs(maxnl),iglast(maxnl)
      logical exist,exist2,exist3,lfind
      character cd*4,cdl*2
      dimension rm(maxgr,maxgr,maxnl,maxnz),flux(maxgr,maxnl,maxnz),
     & sigsm(maxgr,maxgr,maxnl,maxnz),deltau(maxgr),a(maxa),aa(6)
      real gar(maxgar)
      data mtlist/2,16,17,37,51,52,53,54,55,56,57,58,59,60,61,62,63,64,
     & 65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,
     & 86,87,88,89,90,91/
c
c recover the number of thermal-corrected groups (ngther).
      ngther=0
      call repoz(nin)
   10 lfind=.false.
      do while (.not.lfind)
        if (nin.lt.0) then
          read(-nin,end=20) math,mfh,mth,nb,nw
        else if (nin.gt.0) then
          read(nin,'(6e11.0,i4,i2,i3,i5)',end=20) aa,math,mfh,mth,nsp
        endif
        lfind=(math.eq.matd).and.(mfh.eq.6).and.(mth.ge.221).and.
     &  (mth.le.250)
      enddo
      if (.not.lfind) go to 20
      if (nin.lt.0) then
        call skiprz(nin,-1)
        read(-nin) math,mfh,mth,nb,nw,aa
      endif
      ng=nint(aa(6))
      call listio(nin,0,0,a(1),nb,nw)
      if (abs(a(1)-ytemp).gt.1.0e-3) then
        call tomend(nin,0,0,a,nb,nw)
        go to 10
      endif
      lfind=.true.
      do while (mth.ne.0) ! loop over energy groups
        if(.not.lfind) call listio(nin,0,0,a(1),nb,nw)
        if(n2h.eq.ng) go to 20
        ngther=max(ngther,n2h)
        lfind=.false.
        loc=1+nw
        do while (nb.ne.0)
          if (loc+302.gt.maxa) call error('drasc',
     &    'endf input size exceeded',' ')
          call moreio(nin,0,0,a(loc),nb,nw)
          loc=loc+nw
        enddo
      enddo
   20 ngther=ng-ngther
      write(nsyso,'(/
     &  '' number of energy groups affected by thermalization for '',
     &  ''material'',i6,'' ='',i6)') matd,ng-ngther
c
c     ***recover thermal correction x-sections and compute the
c        correction term for groups indices higher than ngther.
      do iz=1,maxnz
        do il=1,maxnl
          do ig2=1,maxgr
            do ig1=1,maxgr
             sigsm(ig1,ig2,il,iz)=0.0
            enddo
          enddo
        enddo
      enddo
      exist3=.false.
      do imt=250,221,-1
        call fmatr(maxgr,maxnl,1,nin,matd,imt,ytemp,ng,nl2,nz,rm,flux,
     &  exist2)
        if(exist2) then
          exist3=.true.
          nlgar=nl2
          do iz=1,nz
            do il=1,nl2
              do ig2=1,ng
                do ig1=ngther+1,ng
                  sigsm(ig1,ig2,il,iz)=sigsm(ig1,ig2,il,iz)
     &            +rm(ig1,ig2,il,1)
                enddo
              enddo
            enddo
          enddo
        endif
      enddo
      if(exist3) then
        call fmatr(maxgr,maxnl,maxnz,nin,matd,2,ytemp,ng,nl,nz,rm,flux,
     &  exist)
        if(nl.ne.nlgar) call error('drasc','inconsistent number of L'//
     &  'egendre orders(2)',' ')
        do iz=1,nz
          do il=1,nl
            do ig2=1,ng
              do ig1=ngther+1,ng
                sigsm(ig1,ig2,il,iz)=sigsm(ig1,ig2,il,iz)
     &          -rm(ig1,ig2,il,1)
              enddo
            enddo
          enddo
        enddo
      endif
c
      do imat=1,maxmat
        call fmatr(maxgr,maxnl,maxnz,nin,matd,mtlist(imat),ytemp,ng,nl,
     &  nz,rm,flux,exist)
        if(mtlist(imat).eq.2) nze=nz
        if (exist) then
          if(nz.eq.1) then
            do iz=1,nze
              do il=1,nl
                do ig2=1,ng
                  do ig1=1,ng
                    sigsm(ig1,ig2,il,iz)=sigsm(ig1,ig2,il,iz)+
     &              rm(ig1,ig2,il,1)
                  enddo
                enddo
              enddo
            enddo
          else if(nz.eq.nze) then
            do iz=1,nz
              do il=1,nl
                do ig2=1,ng
                  do ig1=1,ng
                    sigsm(ig1,ig2,il,iz)=sigsm(ig1,ig2,il,iz)+
     &              rm(ig1,ig2,il,iz)
                  enddo
                enddo
              enddo
            enddo
          else
            call error('drasc','invalid number of dilutions.',' ')
          endif
        endif
      enddo
c
c store the scattering matrices on the xsm file
      call fmatr(maxgr,maxnl,maxnz,nin,matd,2,ytemp,ng,nl,nz,rm,flux,
     & exist)
      do il=1,nl
c
c       ***lump the scattering matrices of a fission product
        if(ipflag.eq.1) then
          do iz=1,nz
            do ig1=1,ng
              dgar=0.0
              do ig2=1,ng
                dgar=dgar+sigsm(ig1,ig2,il,iz)
              enddo
              do ig2=1,ng
                sigsm(ig1,ig2,il,iz)=0.0
              enddo
              sigsm(ig1,ig1,il,iz)=dgar
            enddo
          enddo
        endif
c
        write(cdl,'(i2.2)') il-1
        igar=0
        garmax=0.0
c       ***ig2 is the secondary group
        do ig2=1,ng
          igmin=ig2
          igmax=ig2
c
c         ***ig1 is the primary group
          do ig1=ng,1,-1
            if (sigsm(ig1,ig2,il,1).ne.0.0) then
              igmin=min(igmin,ig1)
              igmax=max(igmax,ig1)
            endif
          enddo
          ijj(ig2)=igmax
          njj(ig2)=igmax-igmin+1
          do ig1=igmax,igmin,-1
            igar=igar+1
            if(igar.gt.maxgar) call error('drasc','gar overflow.',' ')
            gar(igar)=sigsm(ig1,ig2,il,1)
            if(abs(gar(igar)).gt.garmax) garmax=abs(gar(igar))
          enddo
        enddo
c
c       ***write on the xsm file
        call xsmput('SCAT'//cdl,igar,2,gar)
        call xsmput('NJJS'//cdl,ng,1,njj)
        call xsmput('IJJS'//cdl,ng,1,ijj)
        do iz=1,nz-1
          write (cd,'(i4.4)') nz0+iz
          call xsmsix('SUBMAT'//cd,1)
c
c         ***process finite dilution scattering information
          do ig=1,ng
            if(flux(ig,il,1).eq.0.0) cycle
            flux(ig,il,nz-iz+1)=flux(ig,il,nz-iz+1)/flux(ig,il,1)
          enddo
          igar=0
          garmax=0.0
c         ***ig2 is the secondary group
          do ig2=1,igfirs(il)-1
            ijj(ig2)=ig2
            njj(ig2)=1
            igar=igar+1
            if(igar.gt.maxgar) call error('drasc','gar overflow.',' ')
            gar(igar)=0.0d0
          enddo
          do ig2=igfirs(il),ng
            igmin=ig2
            igmax=ig2
c
c           ***ig1 is the primary group
            do ig1=iglast(il)-1,igfirs(il),-1
              delta=sigsm(ig1,ig2,il,nz-iz+1)*flux(ig1,il,nz-iz+1)-
     &        sigsm(ig1,ig2,il,1)
              if(abs(delta).gt.1.0e-6*abs(sigsm(ig1,ig1,il,1))) then
                igmin=min(igmin,ig1)
                igmax=max(igmax,ig1)
              endif
            enddo
            ijj(ig2)=igmax
            njj(ig2)=igmax-igmin+1
            do ig1=igmax,igmin,-1
              igar=igar+1
              if(igar.gt.maxgar) call error('drasc','gar overflow.',' ')
              if((ig1.ge.igfirs(il)).and.(ig1.lt.iglast(il))) then
                gar(igar)=sigsm(ig1,ig2,il,nz-iz+1)*flux(ig1,il,nz-iz+1)
     &          -sigsm(ig1,ig2,il,1)
                if(abs(gar(igar)).gt.garmax) garmax=abs(gar(igar))
              else
                gar(igar)=0.0d0
              endif
            enddo
          enddo
c
c         ***write on the xsm file
          imax=igar
          do i=imax,1,-1
            if(abs(gar(i)).ge.1.0e-8*garmax) go to 30
            igar=igar-1
          enddo
   30     if(igar.gt.0) then
            call xsmput('SCAT'//cdl,igar,2,gar)
            call xsmput('NJJS'//cdl,ng,1,njj)
            call xsmput('IJJS'//cdl,ng,1,ijj)
          endif
          call xsmsix(' ',2)
        enddo
      enddo
c     ***end of Legendre order loop
      return
      end
c
      subroutine fmatr(maxgr,maxnl,maxnz,nin,matd,mt,ytemp,ng,nl,nz,rm,
     & flux,exist)
c     ******************************************************************
c     utility routine for recovering a matrix reaction from a gendf
c     file.
c     input parameters:
c     maxgr   first and second dimensions of matrix rm. Maximum number
c             of energy groups.
c     maxnl   third dimension of matrix rm. Maximum number of Legendre
c             orders.
c     maxnz   4-th dimensions of matrix rm. Maximum number of dilutions.
c     nin     file unit number of the gendf file.
c     matd    material number.
c     mt      reaction index.
c     ytemp   absolute temperature (Kelvin).
c     output parameters:
c     ng      number of energy groups.
c     nl      number of Legendre orders.
c     nz      number of dilutions.
c     rm      matrix reaction (primary->secondary,Legendre,dilution).
c     flux    flux (primary,Legendre,dilution).
c     exist   matrix reaction existence flag.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (maxa=200000,lz=6)
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      integer maxgr,maxnl,maxnz,nin,matd,ng,nl,nz
      logical exist,lfind
      dimension rm(maxgr,maxgr,maxnl,maxnz),flux(maxgr,maxnl,maxnz),
     & a(maxa),aa(6)
      common base(1)
c
      do iz=1,maxnz
        do il=1,maxnl
          do ig2=1,maxgr
          flux(ig2,il,iz)=0.0
            do ig1=1,maxgr
              rm(ig1,ig2,il,iz)=0.0
            enddo
          enddo
        enddo
      enddo
      call reserv('cspc',maxgr,icspc,base)
      do ig1=1,maxgr
        base(icspc+ig1-1)=0.0
      enddo
      call repoz(nin)
   10 lfind=.false.
      do while (.not.lfind)
        if (nin.lt.0) then
          read(-nin,end=900) math,mfh,mth,nb,nw
        else if (nin.gt.0) then
          read(nin,'(6e11.0,i4,i2,i3,i5)',end=900) aa,math,mfh,mth,nsp
        endif
        lfind=(math.eq.matd).and.(mfh.eq.6).and.(mth.eq.mt)
      enddo
      if (.not.lfind) go to 900
      if (nin.lt.0) then
        call skiprz(nin,-1)
        read(-nin) math,mfh,mth,nb,nw,aa
      endif
      nl=nint(aa(3))
      nz=nint(aa(4))
      ng=nint(aa(6))
      call listio(nin,0,0,a(1),nb,nw)
      if (abs(a(1)-ytemp).gt.1.0e-3) then
        call tomend(nin,0,0,a,nb,nw)
        go to 10
      endif
      if(ng.gt.maxgr) call error('fmatr','maxgr overflow',' ')
      if(nl.gt.maxnl) call error('fmatr','maxnl overflow',' ')
      if(nz.gt.maxnz) call error('fmatr','maxnz overflow',' ')
      lfind=.true.
      do while (mth.ne.0) ! loop over energy groups
        if(.not.lfind) call listio(nin,0,0,a(1),nb,nw)
        ng2=l1h
        ig2lo=l2h
        ig=n2h
        lfind=.false.
        loc=1+nw
        do while (nb.ne.0)
          if (loc+302.gt.maxa) call error('fmatr',
     &    'endf input size exceeded',' ')
          call moreio(nin,0,0,a(loc),nb,nw)
          loc=loc+nw
        enddo
        if (ig.ne.0) then
          jg=ng-ig+1
          do il=1,nl
            do iz=1,nz
              locf=1+lz+nl*(iz-1)+(il-1)
              flux(jg,il,iz)=a(locf)
              do k=2,ng2
                loca=locf+nl*nz*(k-1)
                if (ig2lo.ne.0) then
c                 matrix part
                  ig2=ig2lo+k-2
                  rm(jg,ng-ig2+1,il,iz)=rm(jg,ng-ig2+1,il,iz)+a(loca)
                else
c                 spectrum part
                  do ig2=1,ng
                    rm(jg,ng-ig2+1,il,iz)=rm(jg,ng-ig2+1,il,iz)+
     &              base(icspc+ig2-1)*a(loca)
                  enddo
                endif
              enddo
            enddo
          enddo
        else
c         save constant spectrum
          do k=1,ng2
            base(icspc+(ig2lo+k-1)-1)=a(1+lz+nl*nz*(k-1))
          enddo
        endif
      enddo
      exist=.true.
      call releas('cspc',0,base)
      return
c
  900 exist=.false.
      ng=0
      nl=0
      nz=0
      call releas('cspc',0,base)
      return
      end
c
      subroutine fvect(maxgr,maxnl,maxnz,nin,matd,mt,ytemp,ng,nl,nz,rv,
     & flux,exist)
c     ******************************************************************
c     utility routine for recovering a vector reaction from a gendf
c     file.
c     input parameters:
c     maxgr   first dimension of vector rv. Maximum number of energy
c             groups.
c     maxnl   second dimension of vector rv. Maximum number of Legendre
c             orders.
c     maxnz   third dimensions of vector rv. Maximum number of
c             dilutions.
c     nin     file unit number of the gendf file.
c     matd    material number.
c     mt      reaction index.
c     ytemp   absolute temperature (Kelvin).
c     output parameters:
c     ng      number of energy groups.
c     nl      number of Legendre orders.
c     nz      number of dilutions.
c     rv      vector reaction (group,Legendre,dilution).
c     flux    flux (group,Legendre,dilution).
c     exist   vector reaction existence flag.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (maxa=2000,lz=6)
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      integer maxgr,maxnl,maxnz,nin,matd,ng,nl,nz
      logical exist,lfind
      dimension rv(maxgr,maxnl,maxnz),flux(maxgr,maxnl,maxnz),a(maxa),
     & aa(6)
c
      do iz=1,maxnz
        do il=1,maxnl
          do ig=1,maxgr
          flux(ig,il,iz)=0.0d0
          rv(ig,il,iz)=0.0d0
          enddo
        enddo
      enddo
      call repoz(nin)
   10 lfind=.false.
      do while (.not.lfind)
        if (nin.lt.0) then
          read(-nin,end=900) math,mfh,mth,nb,nw
        else if (nin.gt.0) then
          read(nin,'(6e11.0,i4,i2,i3,i5)',end=900) aa,math,mfh,mth,nsp
        endif
        lfind=(math.eq.matd).and.(mfh.eq.3).and.(mth.eq.mt)
      enddo
      if (.not.lfind) go to 900
      if (nin.lt.0) then
        call skiprz(nin,-1)
        read(-nin) math,mfh,mth,nb,nw,aa
      endif
      nl=nint(aa(3))
      nz=nint(aa(4))
      ng=nint(aa(6))
      call listio(nin,0,0,a(1),nb,nw)
      if (abs(a(1)-ytemp).gt.1.0e-3) then
        call tomend(nin,0,0,a,nb,nw)
        go to 10
      endif
      if(ng.gt.maxgr) call error('fvect','maxgr overflow',' ')
      if(nl.gt.maxnl) call error('fvect','maxnl overflow',' ')
      if(nz.gt.maxnz) call error('fvect','maxnz overflow',' ')
      lfind=.true.
      do while (mth.ne.0) ! loop over energy groups
        if(.not.lfind) call listio(nin,0,0,a(1),nb,nw)
        ng2=l1h
        ig=n2h
        lfind=.false.
        loc=1+nw
        do while (nb.ne.0)
          if (loc+302.gt.maxa) call error('fvect',
     &    'endf input size exceeded',' ')
          call moreio(nin,0,0,a(loc),nb,nw)
          loc=loc+nw
        enddo
        jg=ng-ig+1
        do il=1,nl
          do iz=1,nz
            locf=1+lz+nl*(iz-1)+(il-1)
            flux(jg,il,iz)=a(locf)
            rv(jg,il,iz)=a(locf+nl*nz)
          enddo
        enddo
      enddo
      exist=.true.
      return
c
  900 exist=.false.
      ng=0
      nl=0
      nz=0
      return
      end
c
      subroutine dradep(nfp,ndcy)
c     ******************************************************************
c     compute depletion-related information.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter(maxa=10000,maxiso=4000,nreac=13,nfath=40,maxch=800)
      common/mainio/nsysi,nsyso,nsyse,ntty
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      dimension a(maxa),en(nfath,maxch),br(nfath,maxch)
      integer mylist(maxiso,3),hiso(3,maxiso)
      character hname*8,hsmg*72,hich(maxch)*8,hrch(nfath,maxch)*8,
     & text4*4
      common base(1)
c
c     ***read the specification lines for the isotopes present in the
c        burnup chain. The specification is:
c     [[
c     hich [[ hrch en br ]] /
c     ]]
c     end /
c     where hich : character*8 name of the isotope
c           hrch : character*8 name of a neutron induced reaction (not
c                  a scattering type reaction)
c           en   : energy released by the neutron induced reaction
c                  (including the kinetic energy of the secondary
c                  neutrons)
c           br   : branching ratio to an isomeric daughter.
      do iso=1,maxiso
        mylist(iso,1)=0
        mylist(iso,2)=0
        mylist(iso,3)=0
      enddo
      nbch=0
      nbiso=0
      do
        nbch=nbch+1
        if(nbch.gt.maxch) call error('dradep','maxch overflow',' ')
        do i=1,nfath
          hrch(i,nbch)=' '
          en(i,nbch)=0.0
          br(i,nbch)=0.0
        enddo
        read(nsysi,*) hich(nbch),(hrch(i,nbch),en(i,nbch),br(i,nbch),
     &  i=1,nfath)
        do i=1,nfath
          if(br(i,nbch).ne.0.0) then
            write(nsyso,'('' isomeric '',a,'' branching ratio='',f7.3,
     &      '' set for '',a)') hrch(i,nbch),br(i,nbch),hich(nbch)
          endif
        enddo
        izae=0
        if(hich(nbch).eq.'end') go to 90
        i1=index(hich(nbch),'_')
        if(i1.eq.0) then
          hname=hich(nbch)
        else
          hname=hich(nbch)(:i1-1)
        endif
        call dranam(izae,hname)
        nbiso=nbiso+1
        if(nbiso.gt.maxiso) call error('dradep','maxiso overflow',' ')
        mylist(nbiso,1)=izae
        mylist(nbiso,3)=nbch
      enddo
   90 nbch=nbch-1
      if(nbch.eq.0) return
c
c     ***set the list of depleting isotopes
      nbfpch=0
      call repoz(nfp)
  100 if (nfp.gt.0) then
        read(nfp,'(66x,i4,i2,i3)') math,mfh,mth
      else
        read(-nfp) math,mfh,mth
      endif
      if (math.eq.0) go to 100
      nbfiss=0
      if((mfh.ne.0).and.(mth.ne.0)) call skiprz(nfp,-1)
      do
  150   call contio(nfp,0,0,a,nb,nw)
        if(math.eq.-1) then
          go to 300
        else if(mfh.ne.1) then
          call tofend(nfp,0,0,a,nb,nw)
          go to 150
        else if(mth.ne.451) then
          call tosend(nfp,0,0,a,nb,nw)
          go to 150
        endif
        nbfiss=nbfiss+1
        za=c1h
        awr=c2h
        call contio(nfp,0,0,a,nb,nw)
        izae=10*nint(za+0.1)+l2h
        if(izae.eq.0) call error('dradep','izae error',' ')
        call draind(nbiso,izae,mylist(1,1),ind)
        if(ind.le.0) then
          nbiso=nbiso+1
          if(nbiso.gt.maxiso) call error('dradep','maxiso overflow',' ')
          mylist(nbiso,1)=izae
          call dranam(izae,hname)
          do i=1,nbch
            if(hich(i).eq.hname) go to 160
          enddo
        endif
  160   call findf(math,8,454,nfp)
        call contio(nfp,0,0,a,nb,nw)
        lep1=l1h
        do ile=1,lep1
          call listio(nfp,0,0,a(1),nb,nw)
          nbfp=n2h
          energy=c1h
          loc=1+nw
          do while (nb.ne.0)
            if (loc+302.gt.maxa) call error('dradep',
     &      'endf input size exceeded',' ')
            call moreio(nfp,0,0,a(loc),nb,nw)
            loc=loc+nw
          enddo
          if(energy.le.2.0e6) then
            do ifp=1,nbfp
              iof=6+(ifp-1)*4+1
              if(a(iof+2).gt.1.0e-10) then
                iz=nint(a(iof)/1000+0.1)
                ifps=nint(a(iof+1)+0.1)
                ia=mod(nint(a(iof)+0.1),1000)
                izae=10000*iz+10*ia+ifps
                call draind(nbiso,izae,mylist(1,1),ind)
                if(ind.le.0) then
                  nbiso=nbiso+1
                  if(nbiso.gt.maxiso) call error('dradep',
     &            'maxiso overflow',' ')
                  mylist(nbiso,1)=izae
                  call dranam(izae,hname)
                  do i=1,nbch
                    if(hich(i).eq.hname) then
                      nbfpch=nbfpch+1
                      go to 170
                    endif
                  enddo
                endif
  170           continue
              endif
            enddo
          endif
        enddo
        call tomend(nfp,0,0,a,nb,nw)
      enddo
  300 nbdpf=nbiso-nbfiss
      nbfpch=nbch
c
      call repoz(ndcy)
  400 if (ndcy.gt.0) then
        read(ndcy,'(66x,i4,i2,i3)') math,mfh,mth
      else
        read(-ndcy) math,mfh,mth
      endif
      if (math.eq.0) go to 400
      if((mfh.ne.0).and.(mth.ne.0)) call skiprz(ndcy,-1)
      do
  500   call contio(ndcy,0,0,a,nb,nw)
        if(math.eq.-1) then
          go to 600
        else if(mfh.ne.1) then
          call tofend(ndcy,0,0,a,nb,nw)
          go to 500
        else if(mth.ne.451) then
          call tosend(ndcy,0,0,a,nb,nw)
          go to 500
        endif
        za=c1h
        call contio(ndcy,0,0,a,nb,nw)
        izae=10*nint(za+0.1)+l2h
        call draind(nbiso,izae,mylist(1,1),ind)
        if(ind.le.0) then
          nbiso=nbiso+1
          if(nbiso.gt.maxiso) call error('dradep','maxiso overflow',' ')
          mylist(nbiso,1)=izae
          mylist(nbiso,2)=math
        else
          if((mylist(ind,2).ne.math).and.(mylist(ind,2).ne.0)) then
            write(hsmg,'(''invalid mat index (='',i6,''; expected='',
     &      i6,'' izae='',i8,'')'')') math,mylist(ind,2),izae
            call error('dradep',hsmg,' ')
          endif
          mylist(ind,2)=math
        endif
        call tomend(ndcy,0,0,a,nb,nw)
      enddo
c
c     **sort the izae entries.
  600 do k=2,nbiso
        i=k-1
        ipos=0
        if(mylist(i,3).gt.0) ipos=index(hich(mylist(i,3)),'_')+1
        do j=k,nbiso
          jpos=0
          if(mylist(j,3).gt.0) jpos=index(hich(mylist(j,3)),'_')+1
          if((mylist(i,1).gt.mylist(j,1)).or.
     &    ((mylist(i,1).eq.mylist(j,1)).and.(ipos.gt.jpos))) then
            do ii=1,3
              igar=mylist(i,ii)
              mylist(i,ii)=mylist(j,ii)
              mylist(j,ii)=igar
            enddo
          endif
        enddo
      enddo
c
      text4=' '
      read(text4,'(a4)') itext4
      do iso=1,nbiso
        if(mylist(iso,3).ne.0) then
          hname=hich(mylist(iso,3))
        else
          call dranam(mylist(iso,1),hname)
        endif
        read(hname,'(2a4)') hiso(1,iso),hiso(2,iso)
        hiso(3,iso)=itext4
      enddo
c
      maxfp=nbdpf+40 ! reserve 40 location for lumped fp daughters
      call reserv('idrea',nreac*nbiso,iof1,base)
      call reserv('dener',nreac*nbiso,iof2,base)
      call reserv('ddeca',nbiso,iof3,base)
      call reserv('iprea',nfath*nbiso,iof4,base)
      call reserv('prate',nfath*nbiso,iof5,base)
      call reserv('yield',nbfiss*maxfp,iof6,base)
      call reserv('indpf',nbdpf,iof7,base)
      call draevo(maxfp,nbiso,nbfiss,nbdpf,nreac,nfath,mylist(1,1),nfp,
     & ndcy,base(iof1),base(iof2),base(iof3),base(iof4),base(iof5),
     & base(iof6),base(iof7))
      call releas('indpf',0,base)
c
c     ***lump the burnup chain from nbiso to nbch isotopes
      call reserv('jprea',20*nbch,iof8,base)
      call reserv('rrate',20*nbch,iof9,base)
      call reserv('jdrea',(nreac-1)*nbch,iof10,base)
      call reserv('eener',(nreac-1)*nbch,iof11,base)
      call reserv('eyiel',nbfiss*nbfpch,iof12,base)
      call reserv('ipos',2*nbch,iof13,base)
      call reserv('hhhh',3*nbch,iof14,base)
      call reserv('dddd',nbch,iof15,base)
      call dralum(maxfp,nbiso,nbfiss,nbdpf,nreac,nfath,mylist(1,1),
     & hiso,nfp,ndcy,nbch,nbfpch,hich,hrch,en,br,base(iof1),base(iof2),
     & base(iof3),base(iof4),base(iof5),base(iof6),base(iof8),
     & base(iof9),base(iof10),base(iof11),base(iof12),base(iof13),
     & base(iof14),base(iof15))
      call releas('dddd',0,base)
      call releas('hhhh',0,base)
      call releas('ipos',0,base)
      call releas('eyiel',0,base)
      call releas('eener',0,base)
      call releas('jdrea',0,base)
      call releas('rrate',0,base)
      call releas('jprea',0,base)
c
      call releas('yield',0,base)
      call releas('prate',0,base)
      call releas('iprea',0,base)
      call releas('ddeca',0,base)
      call releas('dener',0,base)
      call releas('idrea',0,base)
      return
      end
c
      subroutine draevo(maxfp,nbiso,nbfiss,nbdpf,nreac,nfath,mylist,nfp,
     & ndcy,idreac,dener,ddeca,ipreac,prate,yield,indpf)
c     ******************************************************************
c     compute blocks 'DEPLETE-IDEN', 'DEPLETE-REAC', 'DEPLETE-ENER',
c     'DEPLETE-DECA', etc. for the non-lumped chain
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter(maxa=10000,maxen=3,lz=6)
      common/mainio/nsysi,nsyso,nsyse,ntty
      common/cont/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,nsh,nsp,nsc
      dimension a(maxa),terp(maxen)
      integer mylist(nbiso),idreac(nreac,nbiso),ipreac(nfath,nbiso),
     & indpf(nbdpf)
      dimension dener(nreac,nbiso),ddeca(nbiso),prate(nfath,nbiso),
     & yield(nbfiss,maxfp)
      character text4*4,hname*8
      data terp/1.0, 0.0, 0.0/
c
      do iso=1,nbiso
        ddeca(iso)=0.0
        do ireac=1,nreac
          idreac(ireac,iso)=0
          dener(ireac,iso)=0.0
        enddo
      enddo
      do iso=1,nbiso
        do ifath=1,nfath
          ipreac(ifath,iso)=0
          prate(ifath,iso)=0.0
        enddo
      enddo
      do ipf=1,nbdpf
        indpf(ipf)=0
        do ifiss=1,nbfiss
          yield(ifiss,ipf)=0.0
        enddo
      enddo
c
      call repoz(nfp)
      ifpss=0
  100 if (nfp.gt.0) then
        read(nfp,'(66x,i4,i2,i3)') math,mfh,mth
      else
        read(-nfp) math,mfh,mth
      endif
      if (math.eq.0) go to 100
      if((mfh.ne.0).and.(mth.ne.0)) call skiprz(nfp,-1)
      ifiss=0
      do
  150   call contio(nfp,0,0,a,nb,nw)
        if(math.eq.-1) then
          go to 300
        else if(mfh.ne.1) then
          call tofend(nfp,0,0,a,nb,nw)
          go to 150
        else if(mth.ne.451) then
          call tosend(nfp,0,0,a,nb,nw)
          go to 150
        endif
        ifiss=ifiss+1
        if(ifiss.gt.nbfiss) call error('draevo','nbfiss overflow',' ')
        za=c1h
        awr=c2h
        call contio(nfp,0,0,a,nb,nw)
        izae=10*nint(za+0.1)+l2h
        if(izae.eq.0) call error('draevo','izae error(1)',' ')
        call draind(nbiso,izae,mylist,ind)
        if(ind.le.0) then
          call dranam(izae,hname)
          call error('draevo','missing isotope(1):'//hname,' ')
        endif
        idreac(2,ind)=ifiss*100+4 ! ind is producing fp
        call findf(math,8,454,nfp)
        call contio(nfp,0,0,a,nb,nw)
        lep1=l1h
        if(lep1.gt.maxen) then
          call error('draevo','invalid number of incident neutron '
     &    //'energies',' ')
        endif
        summ=0.0
        do ile=1,lep1
          call listio(nfp,0,0,a(1),nb,nw)
          nbfp=n2h
          energy=c1h
          loc=1+nw
          do while (nb.ne.0)
            if (loc+302.gt.maxa) call error('draevo',
     &      'endf input size exceeded',' ')
            call moreio(nfp,0,0,a(loc),nb,nw)
            loc=loc+nw
          enddo
          if(energy.le.2.0e6) then
            do ifp=1,nbfp
              iof=6+(ifp-1)*4+1
              if(a(iof+2).gt.1.0e-10) then
                iz=nint(a(iof)/1000+0.1)
                ifps=nint(a(iof+1)+0.1)
                ia=mod(nint(a(iof)+0.1),1000)
                jzae=10000*iz+10*ia+ifps
                if(jzae.eq.0) call error('draevo','jzae error',' ')
                call draind(nbiso,jzae,mylist,jnd)
                if(jnd.le.0) then
                  call dranam(jzae,hname)
                  call error('draevo','missing isotope(2):'//hname,' ')
                endif
                do i=1,ifpss
                  if(indpf(i).eq.jzae) then
                    ifpp=i
                    go to 200
                  endif
                enddo
                ifpss=ifpss+1
                if(ifpss.gt.nbdpf) call error('draevo','nbdpf overflow',
     &          ' ')
                ifpp=ifpss
                indpf(ifpp)=jzae
  200           idreac(2,jnd)=ifpp*100+5 ! jnd is a fission fragment
                yield(ifiss,ifpp)=yield(ifiss,ifpp)+terp(ile)*a(iof+2)
                summ=summ+terp(ile)*a(iof+2)
              endif
            enddo
          endif
        enddo
        call tomend(nfp,0,0,a,nb,nw)
      enddo
c
  300 call repoz(ndcy)
  400 if (ndcy.gt.0) then
        read(ndcy,'(66x,i4,i2,i3)') math,mfh,mth
      else
        read(-ndcy) math,mfh,mth
      endif
      if (math.eq.0) go to 400
      if((mfh.ne.0).and.(mth.ne.0)) call skiprz(ndcy,-1)
      do
  500   call contio(ndcy,0,0,a,nb,nw)
        if(math.eq.-1) then
          go to 700
        else if(mfh.ne.1) then
          call tofend(ndcy,0,0,a,nb,nw)
          go to 500
        else if(mth.ne.451) then
          call tosend(ndcy,0,0,a,nb,nw)
          go to 500
        endif
        za=c1h
        awr=c2h
        call contio(ndcy,0,0,a,nb,nw)
        izae=10*nint(za+0.1)+l2h
        if(izae.eq.0) call error('draevo','izae error(2)',' ')
        call draind(nbiso,izae,mylist,ind)
        if(ind.le.0) then
          call dranam(izae,hname)
          call error('draevo','missing isotope(3):'//hname,' ')
        endif
        idreac(1,ind)=1 ! ind can decay
        call findf(math,8,457,ndcy)
        call contio(ndcy,0,0,a,nb,nw)
        iza=nint(c1h+0.1)
        iz=nint(c1h/1000+0.1)
        ia=mod(iza,1000)
        call listio(ndcy,0,0,a,nb,nw)
        if(c1h.eq.0.0) then
          ddeca(ind)=0.0
        else
          ddeca(ind)=1.0e8*log(2.0)/c1h
        endif
        dener(1,ind)=(a(lz+1)+a(lz+3)+a(lz+5))*1.0e-6
        call listio(ndcy,0,0,a(1),nb,nw)
        nbdy=n2h
        loc=1+nw
        do while (nb.ne.0)
          if (loc+302.gt.maxa) call error('draevo',
     &    'endf input size exceeded',' ')
          call moreio(ndcy,0,0,a(loc),nb,nw)
          loc=loc+nw
        enddo
        do idy=1,nbdy
          rtyp=a(6*idy+1)
          jz=iz
          ja=ia
          if(abs(rtyp-1.0).le.0.00001) then
c           **beta- decay
            jz=iz+1
          else if(abs(rtyp-1.4).le.0.00001) then
c           **beta- decay followed by alpha emission
            jz=iz-1
            ja=ia-4
          else if(abs(rtyp-1.5).le.0.00001) then
c           **beta- decay followed by neutron emission (delayed neutron)
            jz=iz+1
            ja=ia-1
          else if(abs(rtyp-2.0).le.0.00001) then
c           **beta+ decay or electron capture
            jz=iz-1
          else if(abs(rtyp-2.4).le.0.00001) then
c           **beta+ decay followed by alpha emission
            jz=iz-3
            ja=ia-4
          else if(abs(rtyp-2.7).le.0.00001) then
c           **beta+ decay followed by proton emission
            jz=iz-2
            ja=ia-1
          else if(abs(rtyp-3.0).le.0.00001) then
c           **isomeric transition
          else if(abs(rtyp-3.4).le.0.00001) then
c           **isomeric transition followed by alpha decay
            jz=iz-2
            ja=ia-4
          else if(abs(rtyp-3.5).le.0.00001) then
c           **isomeric transition followed by neutron emission (delayed
c             neutron)
            ja=ia-1
          else if(abs(rtyp-4.0).le.0.00001) then
c           **alpha decay
            jz=iz-2
            ja=ia-4
          else if(abs(rtyp-5.0).le.0.00001) then
c           **neutron emission (not delayed neutron)
            ja=ia-1
          else if(abs(rtyp-6.0).le.0.00001) then
c           **spontaneous fission
            jz=0
          else if(abs(rtyp-7.0).le.0.00001) then
c           **proton emission
            jz=iz-1
            ja=ia-1
          else if(abs(rtyp-10.0).le.0.00001) then
c           **unknown origin
            jz=0
          else
            call dranam(izae,hname)
            write(text4,'(f4.2)') rtyp
            write(nsyso,'(A)')'unknown type of decay: '//text4//' for '
     &      //hname
            jz=0
          endif
          if(jz.ne.0) then
            izae=10000*jz+10*ja+nint(a(6*idy+2)+0.1)
            call draind(nbiso,izae,mylist,jnd)
            if(jnd.gt.0) then
              ifath=0
              do i=1,nfath
                if(ipreac(i,jnd).eq.0) then
                  ifath=i
                  go to 600
                endif
              enddo
              call error('draevo','nfath overflow',' ')
  600         ipreac(ifath,jnd)=ind*100+1 ! decay father identification
              prate(ifath,jnd)=a(6*idy+5)
            endif
          endif
        enddo
        call tomend(ndcy,0,0,a,nb,nw)
      enddo
  700 return
      end
c
      subroutine draind(nb,izae,mylist,ind)
c     ******************************************************************
c     find the position ind of material izae in array mylist.
c     ******************************************************************
      integer mylist(nb)
      do i=1,nb
        ind=i
        if(mylist(i).eq.izae) return
      enddo
      ind=-1
      return
      end
c
      subroutine dralum(maxfp,nbiso,nbfiss,nbdpf,nreac,nfath,mylist,
     & hiso,nfp,ndcy,nbch,nbfpch,hich,hrch,en,br,idreac,dener,ddeca,
     & ipreac,prate,yield,jpreac,rrate,jdreac,eener,eyiel,ipos,hhhh,
     & dddd)
c     ******************************************************************
c     complete and lump the burnup chain from nbiso to nbch isotopes.
c     write the lumped chain on the xsm file.
c     ******************************************************************
*if sw
      implicit real*8 (a-h,o-z)
*endif
      parameter (maxrea=13,nstate=40,maxit=20,maxfat=25)
      common/mainio/nsysi,nsyso,nsyse,ntty
      dimension en(nfath,nbch),br(nfath,nbch)
      character hrch(nfath,nbch)*8,hich(nbch)*8,reac(maxrea)*8,hname*8,
     & text4*4
      integer mylist(nbiso),hiso(3,nbiso),idreac(nreac,nbiso),
     & ipreac(nfath,nbiso),ipos(nbch,2),in(2),hreac(2,maxrea),
     & hhhh(3,nbch),istate(nstate)
      integer jpreac(maxfat,nbch),jdreac(nreac-1,nbch)
      real rrate(maxfat,nbch),eener(nreac-1,nbch),eyiel(nbfiss,nbfpch),
     & dddd(nbch)
      dimension dener(nreac,nbiso),ddeca(nbiso),prate(nfath,nbiso),
     & yield(nbfiss,maxfp)
      data reac/'DECAY','NFTOT','NG','N2N','N3N','N4N','NA','NP',
     & 'N2A','NNP','ND','NT','NODECAY'/
c
      if(nreac.ne.maxrea) call error('dralum','maxrea overflow',' ')
c
c     ***find the position of the lumped isotopes in the complete chain
      do iso=1,nbch
        read(hich(iso),'(2a4)') in(1),in(2)
        do jso=1,nbiso
          j0=jso
          if((in(1).eq.hiso(1,jso)).and.(in(2).eq.hiso(2,jso))) go to 10
        enddo
        call error('dralum','unable to find '//hich(iso),' ')
   10   ipos(iso,1)=j0
      enddo
c
c     ***complete idreac, dener, ipreac and prate.
      do iso=1,nbch
        ind=ipos(iso,1)
        iz=mylist(ind)/10000
        ia=mod(mylist(ind),10000)/10
        ifps=mod(mylist(ind),10)
        do i=1,nfath
          if(hrch(i,iso).eq.' ') go to 30
          jz=iz
          if(hrch(i,iso).eq.'nftot') then
            jz=0
            if(idreac(2,ind).eq.0) idreac(2,ind)=3
            ireac=2
c
          else if(hrch(i,iso).eq.'NODECAY') then
            jz=0
            if(idreac(2,ind).eq.0) idreac(2,ind)=3
            ireac=13
c
          else if(hrch(i,iso).eq.'ng') then
            ja=ia+1
            idreac(3,ind)=1
            ireac=3
          else if(hrch(i,iso).eq.'n2n') then
            ja=ia-1
            idreac(4,ind)=1
            ireac=4
          else if(hrch(i,iso).eq.'n3n') then
            ja=ia-2
            idreac(5,ind)=1
            ireac=5
          else if(hrch(i,iso).eq.'n4n') then
            ja=ia-3
            idreac(6,ind)=1
            ireac=6
          else if(hrch(i,iso).eq.'na') then
            jz=iz-2
            ja=ia-3
            idreac(7,ind)=1
            ireac=7
          else if(hrch(i,iso).eq.'np') then
            jz=iz-1
            ja=ia
            idreac(8,ind)=1
            ireac=8
          else if(hrch(i,iso).eq.'n2a') then
            jz=iz-4
            ja=ia-7
            idreac(9,ind)=1
            ireac=9
          else if(hrch(i,iso).eq.'nnp') then
            jz=iz-1
            ja=ia-1
            idreac(10,ind)=1
            ireac=10
          else if(hrch(i,iso).eq.'nd') then
            jz=iz-1
            ja=ia-1
            idreac(11,ind)=1
            ireac=11
          else if(hrch(i,iso).eq.'nt') then
            jz=iz-1
            ja=ia-2
            idreac(12,ind)=1
            ireac=12
          endif
          dener(ireac,ind)=en(i,iso)
          if(jz.ne.0) then
            call draind(nbiso,10000*jz+10*ja,mylist,jnd)
            if(jnd.gt.0) then
              do ifath=1,nfath
                if(ipreac(ifath,jnd).eq.0) then
                  ipreac(ifath,jnd)=ind*100+ireac
                  prate(ifath,jnd)=1.0-br(i,iso)
                  go to 15
                endif
              enddo
              call error('dralum','nfath overflow-1',' ')
            endif
          endif
   15     if((jz.ne.0).and.(br(i,iso).ne.0.0)) then
c           *** neutron induced production of an isomer
            call draind(nbiso,10000*jz+10*ja+1,mylist,jnd)
            if(jnd.gt.0) then
              do ifath=1,nfath
                if(ipreac(ifath,jnd).eq.0) then
                  ipreac(ifath,jnd)=ind*100+ireac
                  prate(ifath,jnd)=br(i,iso)
                  go to 20
                endif
              enddo
              call error('dralum','nfath overflow-2',' ')
            endif
          endif
   20     continue
        enddo
   30   continue
      enddo
c
c     **lump idreac, dener, ipreac and prate.
      iter=0
   40 iter=iter+1
      if(iter.gt.maxit) call error('dralum','too many iterations',' ')
      nlump=0
      do iso=1,nbch
        ind=ipos(iso,1)
        do ifath=1,nfath
          if(ipreac(ifath,ind).eq.0) go to 60
          if(mod(ipreac(ifath,ind),100).ne.1) go to 50
          jnd=ipreac(ifath,ind)/100
          if(mylist(jnd).eq.0) go to 50
          do j=1,nbch
            if(ipos(j,1).eq.jnd) go to 50
          enddo
          nlump=nlump+1 ! isotope jnd is lumped
          do ida=1,nbiso
            do ifa=1,nfath
              ipgar=ipreac(ifa,ida)
              if((ipgar/100.eq.jnd).and.(mod(ipgar,100).eq.1)) then
                if(mylist(ida).eq.0) go to 50
              endif
            enddo
          enddo
          do ida=1,nbiso
            do ifa=1,nfath
              ipgar=ipreac(ifa,ida)
              if((ipgar/100.eq.jnd).and.(mod(ipgar,100).eq.1)) then
                if(ida.eq.jnd) call error('dralum','bug',' ')
                prgar=prate(ifa,ida)
                do im=ifa,nfath-1
                  ipreac(im,ida)=ipreac(im+1,ida)
                  prate(im,ida)=prate(im+1,ida)
                enddo
                ipreac(nfath,ida)=0
                prate(nfath,ida)=0.0
                do jfath=1,nfath
                  if(ipreac(jfath,jnd).eq.0) go to 45
                  im=nfath+1
                  do k=nfath,1,-1
                    if(ipreac(k,ida).eq.ipreac(jfath,jnd)) then
                      prate(k,ida)=prate(k,ida)+prgar*prate(jfath,jnd)
                      go to 44
                    endif
                    if(ipreac(k,ida).eq.0) im=k
                  enddo
                  if(im.gt.nfath) then
                    call error('dralum','nfath overflow',' ')
                  endif
                  ipreac(im,ida)=ipreac(jfath,jnd)
                  prate(im,ida)=prgar*prate(jfath,jnd)
   44             continue
                enddo
   45           if(mod(idreac(2,jnd),100).eq.5) then
                  jfp=idreac(2,jnd)/100
                  if(mod(idreac(2,ida),100).eq.5) then
                    ifp=idreac(2,ida)/100
                  else
                    nbdpf=nbdpf+1
                    if(nbdpf.gt.maxfp) then
                      call error('dralum','maxfp overflow',' ')
                    endif
                    ifp=nbdpf
                    do ifi=1,nbfiss
                      yield(ifi,ifp)=0.0
                    enddo
                  endif
                  do ifi=1,nbfiss
                    yield(ifi,ifp)=yield(ifi,ifp)+yield(ifi,jfp)*prgar
                  enddo
                  idreac(2,ida)=ifp*100+5
                endif
              endif
            enddo
          enddo
          do jfath=1,nfath
            if(ipreac(jfath,jnd).gt.0) then
              kt=mod(ipreac(jfath,jnd),100)
              knd=ipreac(jfath,jnd)/100
              dener(kt,knd)=dener(kt,knd)+prate(jfath,jnd)*dener(1,jnd)
            endif
            ipreac(jfath,jnd)=0
            prate(jfath,jnd)=0.0
          enddo
          ymax=0.0
          if(mod(idreac(2,jnd),100).eq.5) then
            jfp=idreac(2,jnd)/100
            do kso=1,nbiso
              if(mod(idreac(2,kso),100).eq.4) then
                ifi=idreac(2,kso)/100
                dener(2,kso)=dener(2,kso)+yield(ifi,jfp)*dener(1,jnd)
              endif
            enddo
            do ifi=1,nbfiss
              ymax=max(ymax,abs(yield(ifi,jfp)))
              yield(ifi,jfp)=0.0
            enddo
            dener(2,jnd)=0.0
            idreac(2,jnd)=0
          endif
          dener(1,jnd)=0.0
          idreac(1,jnd)=0
          half=1.0e8*log(2.0)/ddeca(jnd)/86400.0
          if(ddeca(jnd).eq.0.0) then
            call dranam(mylist(jnd),hname)
            write(nsyso,'('' warning: isotope '',a8,'' is lumped and'',
     &      '' is stable. Max fission yield='',1p,e8.1,''%'')') hname,
     &      ymax*100.0
            if(ymax.gt.1.0E-2) call error('dralum','isotope '//hname//
     &      ' should not be lumped',' ')
          else if((half.gt.30.0).and.(half.lt.999999.99)) then
            call dranam(mylist(jnd),hname)
            write(nsyso,'('' warning: isotope '',a8,'' is lumped and'',
     &      '' has a half-life of'',f10.2,'' days. Max fission yield='',
     &      1p,e8.1,''%'')') hname,half,ymax*100.0
            if(ymax.gt.1.0E-2) call error('dralum','isotope '//hname//
     &      ' should not be lumped',' ')
          else if(half.gt.30.0) then
            call dranam(mylist(jnd),hname)
            write(nsyso,'('' warning: isotope '',a8,'' is lumped and'',
     &      '' has a half-life of'',1p,e10.3,'' days. Max fission yi'',
     &      ''eld='',e8.1,''%'')') hname,half,ymax*100.0
            if(ymax.gt.1.0E-2) call error('dralum','isotope '//hname//
     &      ' should not be lumped',' ')
          endif
          ddeca(jnd)=0.0
          mylist(jnd)=0
   50     continue
        enddo
   60   continue
      enddo
      write(nsyse,'('' ......... nlump='',i5)') nlump
      write(nsyso,'('' ......... nlump='',i5)') nlump
      if(nlump.gt.0) go to 40
c
c     **write vectors 'PRODUCE-REAC' and 'PRODUCE-RATE' to the xsm file
      do iso=1,nbch
        do ifath=1,maxfat
          jpreac(ifath,iso)=0
          rrate(ifath,iso)=0.0
        enddo
        ind=ipos(iso,1)
        nn=0
        do ifath=1,nfath
          if(ipreac(ifath,ind).ne.0) then
            do j=1,ifath-1
              if(ipreac(ifath,ind).eq.ipreac(j,ind)) then
                jnd1=ipreac(ifath,ind)/100
                jnd2=ipreac(j,ind)/100
                write(nsyso,'(/27h dralum: duplicate fathers:,2a4,
     &          1x,2a4)') hiso(1,jnd1),hiso(2,jnd1),hiso(1,jnd2),
     &          hiso(2,jnd2)
                write(hname,'(2a4)') hiso(1,ind),hiso(2,ind)
                call error('dralum','duplicate fathers for '//hname,' ')
              endif
            enddo
            call draind(nbch,ipreac(ifath,ind)/100,ipos(1,1),jso)
            if(jso.le.0) then
              jnd=ipreac(ifath,ind)/100
              write(nsyso,'(/24h dralum: unknown father ,2a4,5h for ,
     &        2a4)') hiso(1,jnd),hiso(2,jnd),hiso(1,ind),hiso(2,ind)
            else
              nn=nn+1
              if(nn.gt.maxfat) then
                write(text4,'(i4)') nn
                call error('dralum','maxfat overflow nn='//text4,' ')
              endif
              jpreac(nn,iso)=100*jso+mod(ipreac(ifath,ind),100)
              rrate(nn,iso)=prate(ifath,ind)
            endif
          endif
        enddo
      enddo
      call xsmput('PRODUCE-REAC',maxfat*nbch,1,jpreac)
      call xsmput('PRODUCE-RATE',maxfat*nbch,2,rrate)
c
c     **write the isotope ascii names and header information on xsm file
      do i=1,nreac-1
        read(reac(i),'(2a4)') hreac(1,i),hreac(2,i)
      enddo
      call xsmput('DEPLETE-IDEN',2*(nreac-1),3,hreac(1,1))
c
c     **write the lumped fission yield matrix to the xsm file
      ibfp=0
      do iso=1,nbch
        ind=ipos(iso,1)
        ipos(iso,2)=0
        if(mod(idreac(2,ind),100).eq.5) then
          ibfp=ibfp+1
          if(ibfp.gt.nbfpch) call error('dralum','nbfpch overflow',' ')
          ipos(iso,2)=ibfp
          do ifi=1,nbfiss
            eyiel(ifi,ibfp)=yield(ifi,idreac(2,ind)/100)
          enddo
        endif
      enddo
      if(ibfp.gt.0) call xsmput('FISSIONYIELD',nbfiss*ibfp,2,eyiel)
c
c     **write vectors 'DEPLETE-REAC' and 'DEPLETE-ENER' to the xsm file
      do iso=1,nbch
        ind=ipos(iso,1)
        do i=1,nreac-1
          if(idreac(i,ind)/100.gt.0) then
            kreac=mod(idreac(i,ind),100)
            if((kreac.le.0).or.(kreac.gt.5)) then
              call error('dralum','invalid reaction',' ')
            endif
          endif
          if((i.eq.2).and.(mod(idreac(i,ind),100).eq.5)) then
            jdreac(i,iso)=ipos(iso,2)*100+5
          else
            jdreac(i,iso)=idreac(i,ind)
          endif
          eener(i,iso)=dener(i,ind)
        enddo
        if (dener(13,ind).ne.0.0) then
           do isoo=1,nbch
              eener(1,isoo)=0.0
              do iii=3,nreac-1
                 eener(iii,isoo)=0.0
              enddo
             enddo
           eener(2,iso)=dener(13,ind)
        endif
      enddo
      call xsmput('DEPLETE-REAC',(nreac-1)*nbch,1,jdreac)
      call xsmput('DEPLETE-ENER',(nreac-1)*nbch,2,eener)
c
c     **write vectors 'CHARGEWEIGHT', 'DEPLETE-DECA', 'ISOTOPESDEPL'
c       and 'STATE-VECTOR' to the xsm file
      nbheav=0
      do iso=1,nbch
        hhhh(1,iso)=hiso(1,ipos(iso,1))
        hhhh(2,iso)=hiso(2,ipos(iso,1))
        hhhh(3,iso)=hiso(3,ipos(iso,1))
        dddd(iso)=ddeca(ipos(iso,1))
        ipos(iso,1)=mylist(ipos(iso,1))
        if(ipos(iso,1).ge.900000) nbheav=nbheav+1
      enddo
      call xsmput('ISOTOPESDEPL',3*nbch,3,hhhh)
      call xsmput('CHARGEWEIGHT',nbch,1,ipos(1,1))
      call xsmput('DEPLETE-DECA',nbch,2,dddd)
      do i=1,nstate
        istate(i)=0
      enddo
      istate(1)=nbch
      istate(2)=nbfiss
      istate(3)=ibfp
      istate(4)=nbheav
      istate(5)=nbch-nbheav
      istate(8)=nreac-1
      istate(9)=maxfat
      call xsmput('STATE-VECTOR',nstate,1,istate)
      return
      end
c
      subroutine dranam(izae,hname)
c     ******************************************************************
c     utility to compose a material ascii (character*8) name or to
c     recover the izae for an existing name.
c     ******************************************************************
      character hname*8,hformat*12,cs(0:111)*2,cm(0:3)*2
      data cs/'n ','H ','He','Li','Be','B ','C ','N ','O ','F ','Ne',
     1 'Na','Mg','Al','Si','P ','S ','Cl','Ar','K ','Ca','Sc','Ti',
     2 'V ','Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge','As','Se',
     3 'Br','Kr','Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd',
     4 'Ag','Cd','In','Sn','Sb','Te','I ','Xe','Cs','Ba','La','Ce',
     5 'Pr','Nd','Pm','Sm','Eu','Gd','Tb','Dy','Ho','Er','Tm','Yb',
     6 'Pr','Hf','Ta','W ','Re','Os','Ir','Pt','Au','Hg','Tl','Pb',
     7 'Bi','Po','At','Rn','Fr','Ra','Ac','Th','Pa','U ','Np','Pu',
     8 'Am','Cm','Bk','Cf','Es','Fm','Md','No','Lr','Rf','Db','Sg',
     9 'Bh','Hs','Mt','Ds','Rg'/
      data cm/'  ','m ','m2','m3'/
c
      if(izae.eq.0) then
c       **compute the izae index from name
        iz=0
        ifps=0
        do i=1,111
          if(hname(:2).eq.cs(i)) then
            iz=i
            iof1=3
            go to 10
          endif
        enddo
        do i=1,111
          if((hname(:1).eq.cs(i)(:1)).and.(cs(i)(2:).eq.' ')) then
            iz=i
            iof1=2
            go to 10
          endif
        enddo
        call error('dranam','invalid name:'//hname,' ')
   10   iof2=index(hname(iof1:),'m')
        if(iof2.gt.0) then
          do i=1,3
            if(hname(iof1+iof2-1:).eq.cm(i)) then
              ifps=i
              go to 20
            endif
          enddo
          call error('dranam','invalid suffix:'//hname,' ')
        else
          iof2=index(hname(iof1:),' ')
        endif
   20   read(hname(iof1:iof1+iof2-2),'(i3)') ia
        izae=iz*10000+ia*10+ifps
      else
c       **compute the name from izae index
        iz=izae/10000
        ia=mod(izae,10000)/10
        ifps=mod(izae,10)
        if(iz.gt.111) call error('dranam','cs overflow',' ')
        if(ifps.gt.3) call error('dranam','cm overflow',' ')
        niz=2
        if(cs(iz)(2:).eq.' ') niz=1
        nia=3
        if(ia.lt.100) nia=2
        if(ia.lt.10) nia=1
        write(hformat,'(''(a'',i1,'',i'',i1,'',a2)'')') niz,nia
        write(hname,hformat) cs(iz)(:niz),ia,cm(ifps)
      endif
      return
      end
c
      subroutine xsmop(fname,imp,impx)
*
*-----------------------------------------------------------------------
*
* Management and utilization of a xsm hierarchical database.
*
* The xsm database results from the juxtaposition of a hierarchical
* logical structure into a direct access file with fixe length records.
* the direct access file is Fortran-77 compatible and is managed by
* kdiget/put/cl. xsmop/put/get/len/six/nxt/cl entries provide a set of
* methods to access a xsm file.
*
* The logical structure of a xsm file is made of a root directory fol-
* lowed by variable-length blocks containing the useful information.
* each time a directory is full, an extent is automatically created at
* the end of the file, so that the total number of blocks in a direc-
* tory is only limited by the maximum size of the direct access file.
* any block can contain a sub-directory in order to create a hierar-
* chical structure.
*
* input parameters:
*  imp    : type of access.   0: new file mode;
*                             1: modification mode;
*                             2: read only mode.
*  impx   : if impx=0, we suppress printing on xsmop.
*  namp   : character*12 name of the current block.
*  nammy  : charecter*12 name of the active directory.
*  ilong  : number of information elements stored in the current block.
*  itype  : type of information elements stored in the current block.
*           0: directory                1: integer
*           2: single precision         3: character*4
*           4: double precision         5: logical
*           6: complex                  7: undefined
*  data1 :  information elements. dimension data1(ilong)
*  iact  :  type of movement in the hierarchical structure.
*           0: return to the root directory;
*           1: move to a son directory;
*           2: move to the parent directory.
*  istatu : =1 to keep the xsm file at close ; =2 to destroy it.
*
* entry points:
*  entry xsmop  : open a xsm file.
*  entry xsmput : copy a block from memory into the xsm file.
*  entry xsmget : copy a block from the xsm file into memory.
*  entry xsmlen : return the length and type of a block. return 0 if
*                 the block doed not exists.
*  entry xsmsix : move in the hierarchical structure.
*  entry xsmnxt : find the name of the next block stored in the active
*                 directory. if namp=' ' at input, find any name for
*                 any block stored in this directory.
*  entry xsmcl  : close the xsm file.
*
* saved variables (active directory extent):
*  cmt    : character*12 names of the blocks stored in the active
*           directory extent.
*  idir   : offset of the active directory extent.
*  iofmax : dimension of vectors iofset,jlong jtype, and nmt
*  nmt    : number of blocks stored in the active directory extent.
*  link   : offset of the next directory extent. the last extent is
*           always linked to the first one.
*  iroot  : offset of an extent that belong to the parent directory.
*           equal to -1 for the root directory.
*  myname : name of the active directory. myname='/' for the root level.
*  iofset : offset table for the active directory extent (position of
*           the first element of each block in the xsm file).
*  jlong  : number of information elements contained in each block of
*           the active directory extent.
*  jtype  : type of information elements contained in each block of the
*           active directory extent.
*
*------------------------------------- author: A. Hebert ; 30/09/86 ----
*
*if cray
      parameter (nrecld=8)
*elseif mpp
      parameter (nrecld=8)
*elseif mips
      parameter (nrecld=1)
*elseif dec
      parameter (nrecld=1)
*else
*     ibm and unix settings:
      parameter (nrecld=4)
*endif
      parameter (iofmax=30,iprim=3,iwrd=3,lrecld=1024)
      parameter (klong=5+iwrd+(3+iwrd)*iofmax)
*     iprim=address of the root directory.
*     klong=length of a directory extent in memory.
*
      character tex131*131,fstat*3
*
      character*(*) fname,namp,nammy
      character cmt(iofmax)*12,namt*12,header*4,myname*12,nament*6,
     > circ*8
      logical lgopen,modif
      dimension data1(*),data2(*),iofset(iofmax),jlong(iofmax),
     > jtype(iofmax)
      common/mainio/nsysi,nsyso,nsyse,ntty
      save ifile,nmt,link,iroot,myname,iofset,jlong,jtype,cmt,idir,
     > modif,ioftop,impfil,lf
      data ifile/0/
*
* open the xsm file.
      nament='xsmop '
      if (ifile.ne.0) go to 80
      lf=len(fname)
      if(lf.eq.0) lf=80
      do 10 iunit=30,98
         inquire(unit=iunit,opened=lgopen)
         if(.not.lgopen)then
            ifile=iunit
            go to 20
         endif
   10 continue
      go to 70
*
   20 if(imp.eq.0)then
         fstat='new'
         if(impx.gt.0) then
            write(nsyso,'(/39h xsmop: creation of the xsm file named ,
     1      a,8h on unit,i3)') fname(:lf),ifile
         endif
      else
         fstat='old'
         if(impx.gt.0)then
            if(imp.eq.1)then
               tex131='in modification mode.'
            else
               tex131='in read-only mode.'
            endif
            write(nsyso,'(/37h xsmop: open existing xsm file named ,
     1      a,8h on unit,i3,1x,a)') fname(:lf),ifile,tex131
         endif
      endif
*
      open(ifile,file=fname(:lf),err=90,iostat=iercod,access='direct',
     > form='unformatted',recl=lrecld*nrecld,status=fstat)
*
      impfil=imp
      if (impfil.ge.1) then
*        recover the root directory if the xsm file already exists.
         call kdiget(ifile,ibuf,0,1,irc)
         if (irc.ne.0) go to 140
         write (header,'(a4)') ibuf
         if (header.ne.'$xsm') go to 100
         call kdiget(ifile,ioftop,1,1,irc)
         if (irc.ne.0) go to 140
         call kdiget(ifile,idir,2,1,irc)
         if (irc.ne.0) go to 140
         call xsmdir(1,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >   jtype,idir)
         modif=.false.
         if (impx.gt.0) write (nsyso,185) fname(:lf),ioftop,myname
      else
         ioftop=iprim+klong
         idir=iprim
         iroot=-1
         nmt=0
         link=idir
         myname='/'
         modif=.true.
         header='$xsm'
         read (header,'(a4)') ibuf
         call kdiput(ifile,ibuf,0,1,irc)
         if (irc.ne.0) go to 150
         call kdiput(ifile,ioftop,1,1,irc)
         if (irc.ne.0) go to 150
         call kdiput(ifile,idir,2,1,irc)
         if (irc.ne.0) go to 150
      endif
      return
*
* copy a block from memory into the xsm file.
      entry xsmput(namp,ilong,itype,data1)
      if (impfil.eq.2) go to 130
      nament='xsmput'
      namt=namp
      if ((itype.le.0).or.(itype.ge.8)) go to 170
      if (ilong.le.0) go to 180
      call xsmrep(namt,2,ifile,modif,cmt,nmt,link,iroot,myname,iofset,
     > jlong,jtype,idir,iii)
      if (iii.gt.0) then
*        the block already exists. replace it.
         modif=.true.
         if (ilong.gt.jlong(iii)) then
            iofset(iii)=ioftop
            ioftop=ioftop+ilong
         endif
         jlong(iii)=ilong
         jtype(iii)=itype
         call kdiput(ifile,data1,iofset(iii),ilong,irc)
         if (irc.ne.0) go to 150
         return
      else
*        the block does not already exists. create it.
         if (nmt.eq.iofmax) then
*           the active directory is full. create an extent.
            ipos=link
            link=ioftop
            if (modif) then
               call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,
     >         jlong,jtype,idir)
            else
               call kdiput(ifile,link,idir+3,1,irc)
               if (irc.ne.0) go to 150
            endif
            idir=link
            link=ipos
            ioftop=ioftop+klong
            nmt=0
         endif
         nmt=nmt+1
         cmt(nmt)=namt
         jlong(nmt)=ilong
         jtype(nmt)=itype
         iofset(nmt)=ioftop
         call kdiput(ifile,data1,iofset(nmt),ilong,irc)
         if (irc.ne.0) go to 150
         modif=.true.
         ioftop=ioftop+ilong
      endif
      return
*
* copy a block from xsm file into memory.
      entry xsmget(namp,data2)
      nament='xsmget'
      namt=namp
      call xsmrep(namt,1,ifile,modif,cmt,nmt,link,iroot,myname,iofset,
     > jlong,jtype,idir,iii)
      if (iii.gt.0) then
*        the block exists. recover it.
         call kdiget(ifile,data2,iofset(iii),jlong(iii),irc)
         if (irc.ne.0) go to 140
      else
*        the block does not exists.
         go to 110
      endif
      return
*
* find the name and type of block namp.
      entry xsmlen(namp,ilong,itype)
      nament='xsmlen'
      namt=namp
      ilong=0
      itype=7
      call xsmrep(namt,1,ifile,modif,cmt,nmt,link,iroot,myname,iofset,
     > jlong,jtype,idir,iii)
      if (iii.gt.0) then
*        the block exists.
         ilong=jlong(iii)
         itype=jtype(iii)
      endif
      return
*
* move into the hierarchical structure.
      entry xsmsix(namp,iact)
      nament='xsmsix'
      if (iact.eq.0) then
         if (modif) then
            call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >      jtype,idir)
            modif=.false.
         endif
         idir=iprim
      else if (iact.eq.2) then
         if (modif) then
            call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >      jtype,idir)
            modif=.false.
         endif
         idir=iroot
      else if (iact.eq.1) then
         namt=namp
         call xsmrep(namt,2,ifile,modif,cmt,nmt,link,iroot,myname,
     >   iofset,jlong,jtype,idir,iii)
         if (iii.gt.0) then
*           the son level already exists.
            if (jtype(iii).ne.0) go to 120
            if (modif) then
               call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,
     >         jlong,jtype,idir)
               modif=.false.
            endif
            idir=iofset(iii)
         else
*           the son level does not exists and is created.
            if (impfil.eq.2) then
               write(nsyso,'(/25h try to create directory ,a12,
     >         16h from directory ,a12)') namt,myname
               go to 130
            endif
            if (nmt.eq.iofmax) then
*              the active directory is full. create an extent.
               ipos=link
               link=ioftop
               if (modif) then
                  call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,
     >            jlong,jtype,idir)
               else
                  call kdiput(ifile,link,idir+3,1,irc)
                  if (irc.ne.0) go to 150
               endif
               idir=link
               link=ipos
               ioftop=ioftop+klong
               nmt=0
            endif
            nmt=nmt+1
            cmt(nmt)=namt
            jlong(nmt)=-1
            jtype(nmt)=0
            iofset(nmt)=ioftop
            ioftop=ioftop+klong
            call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >      jtype,idir)
*           create a new directory.
            iroot=idir
            myname=namt
            idir=iofset(nmt)
            nmt=0
            link=idir
            modif=.true.
            return
         endif
      endif
      call xsmdir(1,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,jtype,
     > idir)
      return
*
* find the name of the next block stored into the active directory.
      entry xsmnxt(namp,nammy)
      nament='xsmnxt'
      nammy=myname
      namt=namp
      if (namt.eq.' ') then
         if (nmt.eq.0) return
         namp=cmt(1)
         return
      else if (nmt.eq.0) then
         go to 110
      endif
      call xsmrep(namt,1,ifile,modif,cmt,nmt,link,iroot,myname,iofset,
     > jlong,jtype,idir,iii)
      if (iii.eq.0) then
         go to 110
      else if (iii+1.le.nmt) then
         namp=cmt(iii+1)
         return
      endif
*
*     switch to the next directory.
      if (idir.ne.link) then
         if (modif) then
            call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >      jtype,idir)
            modif=.false.
         endif
         idir=link
*        recover the next directory.
         call xsmdir(1,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >   jtype,idir)
      endif
      namp=cmt(1)
      return
*
* close the xsm file.
      entry xsmcl(istatu)
      nament='xsmcl '
      if (modif) then
         if (impfil.eq.2) go to 130
         call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >   jtype,idir)
      endif
      if (impfil.eq.2) then
         call xsmdir(1,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     >   jtype,iprim)
      endif
      call kdiget(ifile,iii,1,1,irc)
      if (irc.ne.0) go to 140
      if (ioftop.gt.iii) then
         if (impfil.eq.2) go to 130
         call kdiput(ifile,ioftop,1,1,irc)
         if (irc.ne.0) go to 150
      endif
      call kdiget(ifile,iii,2,1,irc)
      if (irc.ne.0) go to 140
      if ((idir.ne.iii).and.(impfil.le.1)) then
         call kdiput(ifile,idir,2,1,irc)
         if (irc.ne.0) go to 150
      endif
      call kdicl(ifile,istatu,irc)
      if (irc.ne.0) go to 160
      ifile=0
      return
*
* error messages.
   70 write(tex131,'(a,a)') ' Too many files already open. Unable to '
     1 //'open file ',fname(:lf)
      call error('xsmop',tex131,' ')
   80 inquire(unit=ifile,name=fname)
      lf=index(fname,' ')
      if(lf.eq.0) lf=80
      write(tex131,'(a,a,a)') ' File ',fname(:lf),' already open'
      call error('xsmop',tex131,' ')
   90 write(tex131,'(a,a,a,i4,a)') ' unable to open xsm file ''',
     1 fname(:lf),''' (ier =',iercod,' ).'
      call error('xsmop',tex131,' ')
  100 call error(nament,'unable to import xsm file',' ')
      return
  110 write(tex131,190) namt,myname
      call error(nament,tex131,' ')
  120 write(tex131,200) namt
      call error(nament,tex131,' ')
  130 call error(nament,' forbidden operation in read-only mode.',' ')
  140 write(circ,'(i8)') irc
      call error(nament,'(kdiget) iostat='//circ,' ')
  150 write(circ,'(i8)') irc
      call error(nament,'(kdiput) iostat='//circ,' ')
  160 write(circ,'(i8)') irc
      call error(nament,'(kdrcls) iostat='//circ,' ')
  170 write(circ,'(i8)') itype
      call error(nament,circ//' is an invalid type number.',' ')
  180 write(circ,'(i8)') ilong
      call error(nament,'invalid length:'//circ//' record:'//namt,' ')
  185 format (/35h xsmop: xsm file recovery. file = ',a,1h'/27x,
     1 28hhighest attainable address =,i10/27x,20hactive directory = ',
     2 a12,1h'/)
  190 format (7hblock ',a12,19h' is not on level ',a12,
     1 18h' of the xsm file.)
  200 format (7hblock ',a12,28h' is not a directory of the ,9hxsm file.)
      end
c
      subroutine xsmdir(ind,ifile,cmt,nmt,link,iroot,myname,iofset,
     1 jlong,jtype,idir)
*
*-----------------------------------------------------------------------
*
* import or export a directory using the kdi utility.
*
* input parameters:
*  ind   : =1 for import ; =2 for export.
*  ifile  : unit number of the xsm file.
*  cmt    : list of character*12 names of each block (record or
*           directory) that belong to the active directory extent.
*  nmt    : number of blocks stored on the active directory extent.
*  link   : offset of the next directory extent.
*  iroot  : offset of any parent directory extent.
*  myname : character*12 name of the active directory. myname='/' for
*           the root level.
*  iofset : offset list (position of the first element of each block
*           that belong to the active directory extent).
*  jlong  : length of each record (jlong=0 for a directory) that belong
*           to the active directory extent.
*  jtype  : type of each block that belong to the active directory
*           extent.
*  idir   : offset of the active directory extent.
*
*------------------------------------- author: A. Hebert ; 30/09/86 ----
*
      character*(*) cmt(*),myname
      character header*4,circ*8,hgar*12
      parameter (iofmax=30,iwrd=3)
      dimension  iofset(*),jlong(*),jtype(*),ibuf(iwrd)
*
      if (ind.eq.1) then
         call kdiget(ifile,ibuf,idir,1,irc)
         if (irc.ne.0) go to 40
         write (header,'(a4)') ibuf(1)
         if (header.ne.'$$$$') go to 30
         call kdiget(ifile,iofma2,idir+1,1,irc)
         call kdiget(ifile,nmt,idir+2,1,irc)
         if (nmt.gt.iofmax) go to 30
         call kdiget(ifile,link,idir+3,1,irc)
         call kdiget(ifile,iroot,idir+4,1,irc)
         call kdiget(ifile,ibuf,idir+5,iwrd,irc)
         write (hgar(:4),'(a4)') ibuf(1)
         write (hgar(5:8),'(a4)') ibuf(2)
         write (hgar(9:),'(a4)') ibuf(3)
         myname=hgar
         if (nmt.eq.0) return
         ibase=idir+5+iwrd
         call kdiget(ifile,iofset,ibase,nmt,irc)
         call kdiget(ifile,jlong,ibase+iofma2,nmt,irc)
         call kdiget(ifile,jtype,ibase+2*iofma2,nmt,irc)
         do 10 i=1,nmt
         call kdiget(ifile,ibuf,ibase+3*iofma2+(i-1)*iwrd,iwrd,irc)
         if (irc.ne.0) go to 40
         write (hgar(:4),'(a4)') ibuf(1)
         write (hgar(5:8),'(a4)') ibuf(2)
         write (hgar(9:),'(a4)') ibuf(3)
   10    cmt(i)=hgar
      else if (ind.eq.2) then
         header='$$$$'
         read (header,'(a4)') ibuf(1)
         call kdiput(ifile,ibuf,idir,1,irc)
         if (irc.ne.0) go to 50
         call kdiput(ifile,iofmax,idir+1,1,irc)
         call kdiput(ifile,nmt,idir+2,1,irc)
         call kdiput(ifile,link,idir+3,1,irc)
         call kdiput(ifile,iroot,idir+4,1,irc)
         hgar=myname
         read (hgar(:4),'(a4)') ibuf(1)
         read (hgar(5:8),'(a4)') ibuf(2)
         read (hgar(9:),'(a4)') ibuf(3)
         call kdiput(ifile,ibuf,idir+5,iwrd,irc)
         if (nmt.eq.0) return
         ibase=idir+5+iwrd
         call kdiput(ifile,iofset,ibase,nmt,irc)
         call kdiput(ifile,jlong,ibase+iofmax,nmt,irc)
         call kdiput(ifile,jtype,ibase+2*iofmax,nmt,irc)
         do 20 i=1,nmt
         hgar=cmt(i)
         read (hgar(:4),'(a4)') ibuf(1)
         read (hgar(5:8),'(a4)') ibuf(2)
         read (hgar(9:),'(a4)') ibuf(3)
         call kdiput(ifile,ibuf,ibase+3*iofmax+(i-1)*iwrd,iwrd,irc)
         if (irc.ne.0) go to 50
   20    continue
      endif
      return
*
*     abort on fatal errors
*
   30 call error('xsmdir','unable to recover directory.',' ')
   40 write(circ,'(i8)') irc
      call error('xsmdir','(kdiget) error no. '//circ,' ')
   50 write(circ,'(i8)') irc
      call error('xsmdir','(kdiput) error no. '//circ,' ')
      end
c
      subroutine xsmrep(namt,ind,ifile,modif,cmt,nmt,link,iroot,myname,
     1 iofset,jlong,jtype,idir,iii)
*
*-----------------------------------------------------------------------
*
* find a block (record or directory) position in the active directory
* and related extents.
*
* input parameters:
*  namt   : character*12 name of the required block.
*  ind    : =1 search namt ; =2 search and positionning in an empty
*           slot of the active directory if namt does not exists.
*  ifile  : unit number of the xsm file.
*  modif  : =.true. if the active directory extent have been modified.
*  cmt    : list of character*12 names of each block (record or
*           directory) that belong to the active directory extent.
*  nmt    : number of blocks stored on the active directory extent.
*  link   : offset of the next directory extent.
*  iroot  : offset of any parent directory extent.
*  myname : character*12 name of the active directory. myname='/' for
*           the root level.
*  iofset : offset list (position of the first element of each block
*           that belong to the active directory extent).
*  jlong  : length of each record (jlong=0 for a directory) that belong
*           to the active directory extent.
*  jtype  : type of each block that belong to the active directory
*           extent.
*  idir   : offset of the active directory extent.
*
* output parameter:
*  iii    : return code. =0 if the block named namt does not exists;
*           =position in the active directory extent if namt extsts.
*
*------------------------------------- author: A. Hebert ; 30/09/86 ----
*
      parameter (iofmax=30)
      character*(*) namt,cmt(*),myname
      logical modif
      dimension iofset(*),jlong(*),jtype(*)
*
      ipos=-1
      if (nmt.lt.iofmax) ipos=idir
      if (nmt.eq.0) go to 50
      do 10 i=1,nmt
      if (namt.eq.cmt(i)) then
*        the block already exists.
         iii=i
         return
      endif
   10 continue
*
*     the block namt does not exists in the active directory extent. we
*     search in other extents that belong to the active directory.
      if (idir.ne.link) then
*        recover a new directory extent.
         istart=link
         if (modif) then
            call xsmdir(2,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     1      jtype,idir)
            modif=.false.
         endif
         idir=istart
   30    call xsmdir(1,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     1   jtype,idir)
         if (nmt.lt.iofmax) ipos=idir
         do 40 i=1,nmt
         if (namt.eq.cmt(i)) then
*           the block namt was found in the active directory extent.
            iii=i
            return
         endif
   40    continue
         if (link.eq.istart) go to 50
         idir=link
         go to 30
      endif
   50 iii=0
      if (ind.eq.1) return
*
* positionning in an extent with empty slots.
      if ((ipos.ge.0).and.(ipos.ne.idir)) then
*        an extent with an empty slot was found.
         idir=ipos
         call xsmdir(1,ifile,cmt,nmt,link,iroot,myname,iofset,jlong,
     1   jtype,idir)
      endif
      return
      end
c
      subroutine xsmlib
*
*-----------------------------------------------------------------------
*
* list the content of every extents that belong to the active directory.
*
*------------------------------------- author: A. Hebert ; 30/08/89 ----
*
      parameter (ntype=8)
      character namt*12,myname*12,first*12,ctype(ntype)*16
      common/mainio/nsysi,nsyso,nsyse,ntty
      save ctype
      data (ctype(ity),ity=1,ntype)/'directory','integer','real',
     > 'character','double precision','logical','complex','undefined'/
*
      namt=' '
      call xsmnxt(namt,myname)
      if (namt.eq.' ') then
         write (nsyso,90) myname
         return
      endif
      first=namt
      write(nsyso,100) myname
      itot=0
      inmt=0
*
   10 inmt=inmt+1
      call xsmlen(namt,ilong,ityxsm)
      if (ityxsm.eq.0) then
         write (nsyso,110) inmt,namt,ctype(1)
      else
         write (nsyso,120) inmt,namt,ilong,ctype(ityxsm+1)
         itot=itot+ilong
      endif
      call xsmnxt(namt,myname)
      if (namt.eq.first) go to 20
      go to 10
*
   20 write(nsyso,130) myname,itot
      return
*
   90 format (/10h xsmlib: ',a12,24h' is an empty directory.)
  100 format (//38h xsmlib: content of active directory ',a12,2h'://
     > 11h block name,10(1h-),4x,6hlength,4x,4htype/)
  110 format (1x,i4,3h  ',a12,1h',14x,a16)
  120 format (1x,i4,3h  ',a12,1h',i10,4x,a16)
  130 format (//37h total number of words on directory ',a12,3h' =,
     > i10/)
      end
c
      subroutine xsmexp(nunit,imode,idir,impx)
*
*-----------------------------------------------------------------------
*
* export/import  the content of the xsm file using the contour method.
* export start from the active directory. esope version.
*
*  nunit  : unit number of the file where the export is performed.
*  imode  : type of export/import file:
*           =1 sequential unformatted; =2 sequential formatted (ascii).
*  idir   : =1 to export ; =2 to import.
*  impx   : equal to zero for no print.
*
*------------------------------------- author: A. Hebert ; 05/10/89 ----
*
*if sw
      real*8 dbase(1)
      parameter (ndoubl=2)
*else
      real dbase(1)
      parameter (ndoubl=1)
*endif
      parameter (nlevel=50,nblk=24,lendat=4)
      character namt*12,myname*12,path(nlevel)*12,first(nlevel)*12,
     1 hsmg*131
      real rrr(2*nblk)
      integer iii(2*nblk),ibase(1)
      logical lll(2*nblk)
      double precision ddd(nblk)
      equivalence (rrr,iii,lll,ddd),(dbase(1),base(1),ibase(1))
      common base(1)
      common/mainio/nsysi,nsyso,nsyse,ntty
*
      if((imode.lt.1).or.(imode.gt.2)) then
         write(hsmg,'(44hxsmexp: invalid file type on the xsm file lo,
     1   13hcated on unit,i3,1h.)') nunit
         call error('xsmexp',hsmg,' ')
      else if((idir.lt.1).or.(idir.gt.2)) then
         write(hsmg,'(44hxsmexp: invalid action on the xsm file locat,
     1   10hed on unit,i3,1h.)') idir
         call error('xsmexp',hsmg,' ')
      endif
      itot=0
      ilevel=1
      namt=' '
      call xsmnxt(namt,myname)
      if (idir.eq.2) go to 170
*
* xsm file export.
      if (impx.gt.0) write(nsyso,300) 'export on',nunit,myname
   10 namt=' '
      lennam=12
      call xsmnxt(namt,myname)
      if(namt.eq.' ') then
         if(ilevel.eq.1) return
         namt=path(ilevel)
         ilevel=ilevel-1
         call xsmsix(' ',2)
         if((nunit.ne.0).and.(imode.eq.1)) then
            write(nunit) 0,0,0,0
         else if((nunit.ne.0).and.(imode.eq.2)) then
            write(nunit,310) 0,0,0,0
         endif
         if(impx.gt.0) write(nsyso,350) ilevel
         go to 130
      endif
      first(ilevel)=namt
*
   20 call xsmlen(namt,ilong,ityxsm)
      jlong=ilong
      if((ityxsm.eq.4).or.(ityxsm.eq.6)) jlong=ilong/2
      if(impx.gt.0) write(nsyso,320) ilevel,namt,ityxsm,abs(jlong)
      if((nunit.ne.0).and.(imode.eq.1)) then
         write(nunit) ilevel,lennam,ityxsm,abs(jlong)
         if(lennam.gt.0) write(nunit) namt
      else if((nunit.ne.0).and.(imode.eq.2)) then
         write(nunit,310) ilevel,lennam,ityxsm,abs(jlong)
         if(lennam.gt.0) write(nunit,'(a12,68(1h ))') namt
      endif
      if(ityxsm.eq.0) then
*        directory data.
         ilevel=ilevel+1
         if(ilevel.gt.nlevel) call error('xsmexp','too many directory'
     1   //' levels.',' ')
         call xsmsix(namt,1)
         path(ilevel)=namt
         go to 10
      else if((ilong.ne.0).and.(ityxsm.le.6)) then
         itot=itot+ilong
         call reserv('assmb',ilong,iass,dbase)
         iass=ndoubl*(iass-1)+1
         call xsmget(namt,base(iass))
*        export a node.
         if(ityxsm.eq.1) then
*           integer data.
            do 40 i=1,1+(jlong-1)/nblk
            jmin=min(nblk,jlong-(i-1)*nblk)
            iofset=iass+(i-1)*nblk-1
            do 30 j=1,jmin
   30       rrr(j)=base(iofset+j)
            if((nunit.ne.0).and.(imode.eq.1)) then
               write(nunit) (iii(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               write(nunit,'(8i10)') (iii(j),j=1,jmin)
            endif
   40       continue
         else if(ityxsm.eq.2) then
*           single precision data.
            if((nunit.ne.0).and.(imode.eq.1)) then
               do 50 i=1,1+(jlong-1)/nblk
               jmin=min(nblk,jlong-(i-1)*nblk)
               iofset=iass+(i-1)*nblk-1
               write(nunit) (base(iofset+j),j=1,jmin)
   50          continue
            else if((nunit.ne.0).and.(imode.eq.2)) then
               write(nunit,'(1p,5e16.8)') (base(iass+i-1),i=1,jlong)
            endif
         else if(ityxsm.eq.3) then
*           character*4 data.
            if((nunit.ne.0).and.(imode.eq.1)) then
               do 60 i=1,1+(jlong-1)/nblk
               jmin=min(nblk,jlong-(i-1)*nblk)
               write(nunit) (lendat,k=1,jmin)
   60          continue
               do 70 i=1,1+(jlong-1)/nblk
               jmin=min(nblk,jlong-(i-1)*nblk)
               iofset=iass+(i-1)*nblk-1
               write(nunit) (base(iofset+j),j=1,jmin)
   70          continue
            else if((nunit.ne.0).and.(imode.eq.2)) then
               write(nunit,'(8i10)') (lendat,i=1,jlong)
               write(nunit,'(20a4)') (base(iass+i-1),i=1,jlong)
            endif
         else if(ityxsm.eq.4) then
*           double precision data.
            do 90 i=1,1+(jlong-1)/nblk
            jmin=min(nblk,jlong-(i-1)*nblk)
            iofset=iass+2*(i-1)*nblk-1
            do 80 j=1,2*jmin
   80       rrr(j)=base(iofset+j)
            if((nunit.ne.0).and.(imode.eq.1)) then
               write(nunit) (ddd(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               write(nunit,'(1p,4d20.12)') (ddd(j),j=1,jmin)
            endif
   90       continue
         else if(ityxsm.eq.5) then
*           logical data.
            do 110 i=1,1+(jlong-1)/nblk
            jmin=min(nblk,jlong-(i-1)*nblk)
            iofset=iass+(i-1)*nblk-1
            do 100 j=1,jmin
  100       rrr(j)=base(iofset+j)
            if((nunit.ne.0).and.(imode.eq.1)) then
               write(nunit) (lll(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               write(nunit,'(8l10)') (lll(j),j=1,jmin)
            endif
  110       continue
         else if(ityxsm.eq.6) then
*           complex data.
            if((nunit.ne.0).and.(imode.eq.1)) then
               do 120 i=1,1+(jlong-1)/nblk
               jmin=min(nblk,jlong-(i-1)*nblk)
               iofset=iass+2*(i-1)*nblk-1
               write(nunit) (base(iofset+j),j=1,2*jmin)
  120          continue
            else if((nunit.ne.0).and.(imode.eq.2)) then
               write(nunit,'(1p,5e16.8)') (base(iass+i-1),i=1,2*jlong)
            endif
         endif
         call releas('assmb',0,dbase)
      endif
  130 call xsmnxt(namt,myname)
      if(namt.eq.first(ilevel)) then
         if((nunit.ne.0).and.(imode.eq.1)) then
            write(nunit) 0,0,0,0
         else if((nunit.ne.0).and.(imode.eq.2)) then
            write(nunit,310) -ilevel,0,0,0
         endif
         if(impx.gt.0) write(nsyso,350) ilevel
         if(ilevel.eq.1) go to 140
         namt=path(ilevel)
         ilevel=ilevel-1
         call xsmsix(' ',2)
         go to 130
      endif
      go to 20
  140 if(impx.gt.0) write(nsyso,330) 'exported',itot
      return
*
* file import.
  170 if (impx.gt.0) write(nsyso,300) 'import from',nunit,myname
  180 if((nunit.ne.0).and.(imode.eq.1)) then
         read(nunit,end=270) jlevel,lennam,ityxsm,ilong
      else if((nunit.ne.0).and.(imode.eq.2)) then
         read(nunit,340,end=270) jlevel,lennam,ityxsm,ilong
      endif
      if(jlevel.eq.-1) go to 270
      if(jlevel.eq.-ilevel) go to 260
      if(lennam.gt.12) call error('xsmexp','a record name is greater '
     1 //'than 12 characters.',' ')
      if(lennam.eq.0) then
         namt=' '
      else if((nunit.ne.0).and.(imode.eq.1)) then
         read(nunit) namt
      else if((nunit.ne.0).and.(imode.eq.2)) then
         read(nunit,'(a12)') namt
      endif
*
      if(impx.gt.0) write(nsyso,320) jlevel,namt,ityxsm,ilong
      if(jlevel.ne.ilevel) then
         write(hsmg,'(44hxsmexp: unable to import the xsm file locate,
     1   9hd on unit,i3,1h.)') nunit
         call error('xsmexp',hsmg,' ')
      endif
      if(ityxsm.eq.0) then
*        import associative table data.
         ilevel=ilevel+1
         call xsmsix(namt,1)
         go to 180
      else if(ityxsm.le.6) then
*        import a node
         if(ityxsm.eq.1) then
*           integer data.
            call reserv('assmb',ilong,iass,dbase)
            iass=ndoubl*(iass-1)+1
            do 190 i=1,1+(ilong-1)/nblk
            jmin=min(nblk,ilong-(i-1)*nblk)
            if((nunit.ne.0).and.(imode.eq.1)) then
               read(nunit) (iii(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(8i10)') (iii(j),j=1,jmin)
            endif
            iofset=iass+(i-1)*nblk-1
            do 190 j=1,jmin
  190       base(iofset+j)=rrr(j)
         else if(ityxsm.eq.2) then
*           single precision data.
            call reserv('assmb',ilong,iass,dbase)
            iass=ndoubl*(iass-1)+1
            if((nunit.ne.0).and.(imode.eq.1)) then
               do 200 i=1,1+(ilong-1)/nblk
               jmin=min(nblk,ilong-(i-1)*nblk)
               iofset=iass+(i-1)*nblk-1
               read(nunit) (base(iofset+j),j=1,jmin)
  200          continue
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(5e16.0)') (base(iass+i-1),i=1,ilong)
            endif
         else if(ityxsm.eq.3) then
*           character*4 data.
            klong=0
            do 210 i=1,1+(ilong-1)/nblk
            jmin=min(nblk,ilong-(i-1)*nblk)
            if((nunit.ne.0).and.(imode.eq.1)) then
               read(nunit) (iii(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(8i10)') (iii(j),j=1,jmin)
            endif
            do 210 j=1,jmin
  210       klong=klong+iii(j)
            ilong=(klong+3)/4
            call reserv('assmb',ilong,iass,dbase)
            iass=ndoubl*(iass-1)+1
            if((nunit.ne.0).and.(imode.eq.1)) then
               do 220 i=1,1+(ilong-1)/nblk
               jmin=min(nblk,ilong-(i-1)*nblk)
               iofset=iass+(i-1)*nblk-1
               read(nunit) (base(iofset+j),j=1,jmin)
  220          continue
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(20a4)') (base(iass+i-1),i=1,ilong)
            endif
         else if(ityxsm.eq.4) then
*           double precision data.
            call reserv('assmb',2*ilong,iass,dbase)
            iass=ndoubl*(iass-1)+1
            do 230 i=1,1+(ilong-1)/nblk
            jmin=min(nblk,ilong-(i-1)*nblk)
            if((nunit.ne.0).and.(imode.eq.1)) then
               read(nunit) (ddd(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(4d20.0)') (ddd(j),j=1,jmin)
            endif
            iofset=iass+2*(i-1)*nblk-1
            do 230 j=1,2*jmin
  230       base(iofset+j)=rrr(j)
         else if(ityxsm.eq.5) then
*           logical data.
            call reserv('assmb',ilong,iass,dbase)
            iass=ndoubl*(iass-1)+1
            do 240 i=1,1+(ilong-1)/nblk
            jmin=min(nblk,ilong-(i-1)*nblk)
            if((nunit.ne.0).and.(imode.eq.1)) then
               read(nunit) (lll(j),j=1,jmin)
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(8l10)') (lll(j),j=1,jmin)
            endif
            iofset=iass+(i-1)*nblk-1
            do 240 j=1,jmin
  240       base(iofset+j)=rrr(j)
         else if(ityxsm.eq.6) then
*           complex data.
            call reserv('assmb',2*ilong,iass,dbase)
            iass=ndoubl*(iass-1)+1
            if((nunit.ne.0).and.(imode.eq.1)) then
               do 250 i=1,1+(ilong-1)/nblk
               jmin=min(nblk,ilong-(i-1)*nblk)
               iofset=iass+2*(i-1)*nblk-1
               read(nunit) (base(iofset+j),j=1,2*jmin)
  250          continue
            else if((nunit.ne.0).and.(imode.eq.2)) then
               read(nunit,'(5e16.0)') (base(iass+i-1),i=1,2*ilong)
            endif
         endif
         jlong=ilong
         if((ityxsm.eq.4).or.(ityxsm.eq.6)) jlong=2*ilong
         call xsmput(namt,jlong,ityxsm,base(iass))
         call releas('assmb',0,dbase)
         itot=itot+jlong
      else
         write(hsmg,'(2a,i3,a)') 'try to import unknown type record f',
     1   'rom the xsm file located on unit ''',nunit,'''(1).'
         call error('xsmexp',hsmg,' ')
      endif
      go to 180
*
  260 if(impx.gt.0) write(nsyso,350) ilevel
      ilevel=ilevel-1
      call xsmsix(' ',2)
      go to 180
*
  270 if(ilevel.ne.1) call error('xsmexp','truncated import file.',' ')
      if(impx.gt.0) write(nsyso,330) 'imported',itot
      return
*
  300 format (//18h xsmexp: xsm file ,a,5h unit,i3,
     1 17h from directory ',a12,2h'.//18h level  block name,
     2 4(1h-),4x,12htype  length/)
  310 format ('->',4i8,46(1h ))
  320 format (1x,i5,3h  ',a12,1h',2i8)
  330 format (//31h xsmexp: total number of words ,a,2h =,i10/)
  340 format (2x,4i8)
  350 format (1x,i5,2x,14('-'))
      end
c
      subroutine kdiget(ifile,data,iofset,length,irc)
*
*-----------------------------------------------------------------------
*
*                   kdiget/put/cl (multi buffer version)
*                   ------------------------------------
*
*  function: This routine will provide the possibility to use a direct
*            access (da) file to simulate memory like support. maxbuf
*            static buffers are used.
*
*  entry points:
*  . kdiget    extract data from the da file.
*  . kdiput    save data into the da file.
*  . kdicl     terminate operations on the da file.
*
*  remark : file opening must be done using the open statement
*           provided in the fortran-77 standard:
*           open(ifile,access='direct',recl=lrecl*iunit) where iunit
*           is the number of memory units in a single precision word.
*
*  dummy arguments:
*  ifile  : unit number of the da file.
*  data   : addr. in memory where data is to be moved or extracted.
*  iofset : offset, from start of file where data is extracted from
*           or where data is to be stored.
*  length : number of words to read from or write into file.
*  istatu : =1 to keep the file at close ; =2 to destroy it.
*  irc    : return code (zero if no error is detected).
*
*  saved variables:
*  nbuf           : number of buffers used. nbuf.le.maxbuf
*  ibufs          : index of the last buffer used.
*  nrecs(ibuf)    : relative record number corresponding to the buffer.
*  ifiles(ibuf)   : unit number corresponding to the buffer.
*  iage(ibuf)     : age of the buffer.
*  modif(ibuf)    : modification flag for the buffer.
*  record(1,ibuf) : static buffer. dimension record(lrecl,maxbuf)
*
*------------------------------------- author: A. Hebert ; 21/07/88 ----
*
      implicit integer(a-z)
      parameter (maxbuf=10,lrecl=1024)
      character tex255*255
      logical modif(maxbuf)
      dimension data(*),nrecs(maxbuf),ifiles(maxbuf),iage(maxbuf),
     1 record(lrecl,maxbuf)
      save nbuf,ibufs,nrecs,ifiles,iage,modif,record
      data nbuf/0/
*
      irc=0
      nrec=1+iofset/lrecl
      n=mod(iofset,lrecl)
      lmin=1
   10 ngro=min(lrecl+lmin-n-1,length)
      kdm=-1
      do 40 iiii=1,nbuf
      ibuf=1+mod(iiii+ibufs-2,nbuf)
      if ((ifile.eq.ifiles(ibuf)).and.(nrec.eq.nrecs(ibuf))) then
         do 20 l=lmin,ngro
         n=n+1
   20    data(l)=record(n,ibuf)
         do 30 i=1,nbuf
         if (i.ne.ibuf) iage(i)=iage(i)+1
   30    continue
         ibufs=ibuf
         if (ngro.eq.length) return
         nrec=nrec+1
         n=0
         lmin=ngro+1
         ibufs=ibuf+1
         go to 10
      endif
      if (iage(ibuf).gt.kdm) then
         kdm=iage(ibuf)
         kbuf=ibuf
      endif
   40 continue
      if (nbuf.lt.maxbuf) then
         nbuf=nbuf+1
         ibufs=nbuf
      else
         ibufs=kbuf
         if (modif(ibufs)) then
*           -------------------------------------------------------
            write(ifiles(ibufs),rec=nrecs(ibufs),err=50,iostat=irc)
     1      (record(i,ibufs),i=1,lrecl)
*           -------------------------------------------------------
         endif
      endif
      nrecs(ibufs)=nrec
      ifiles(ibufs)=ifile
      iage(ibufs)=0
      modif(ibufs)=.false.
*     ------------------------------------------------------------------
      read(ifile,rec=nrec,err=50,iostat=irc) (record(i,ibufs),i=1,lrecl)
*     ------------------------------------------------------------------
      go to 10
   50 return
*
      entry kdiput(ifile,data,iofset,length,irc)
      irc=0
      nrec=1+iofset/lrecl
      n=mod(iofset,lrecl)
      lmin=1
   60 ngro=min(lrecl+lmin-n-1,length)
      kdm=-1
      do 90 iiii=1,nbuf
      ibuf=1+mod(iiii+ibufs-2,nbuf)
      if ((ifile.eq.ifiles(ibuf)).and.(nrec.eq.nrecs(ibuf))) then
         do 70 l=lmin,ngro
         n=n+1
   70    record(n,ibuf)=data(l)
         do 80 i=1,maxbuf
         if (i.ne.ibuf) iage(i)=iage(i)+1
   80    continue
         modif(ibuf)=.true.
         ibufs=ibuf
         if (ngro.eq.length) return
         nrec=nrec+1
         n=0
         lmin=ngro+1
         ibufs=ibuf+1
         go to 60
      endif
      if (iage(ibuf).gt.kdm) then
         kdm=iage(ibuf)
         kbuf=ibuf
      endif
   90 continue
      if (nbuf.lt.maxbuf) then
         nbuf=nbuf+1
         ibufs=nbuf
      else
         ibufs=kbuf
         if (modif(ibufs)) then
*           --------------------------------------------------------
            write(ifiles(ibufs),rec=nrecs(ibufs),err=100,iostat=irc)
     1      (record(i,ibufs),i=1,lrecl)
*           --------------------------------------------------------
         endif
      endif
      nrecs(ibufs)=nrec
      ifiles(ibufs)=ifile
      iage(ibufs)=0
      modif(ibufs)=.false.
      if ((n.ne.0).or.(lrecl+lmin-n-1.gt.length)) then
*        --------------------------------------------------------
         read(ifile,rec=nrec,err=95,iostat=ios) (record(i,ibufs),
     1   i=1,lrecl)
*        --------------------------------------------------------
         if(ios.ne.0)go to 95
      endif
      go to 60
   95 inquire(ifile,name=tex255,recl=nn)
      close(ifile,status='keep')
      open(ifile,file=tex255,access='direct',recl=nn,status='old')
      go to 60
  100 return
*
      entry kdicl(ifile,istatu,jrc)
      if(istatu.eq.1) then
         do 110 ibuf=1,nbuf
         if (modif(ibuf).and.(ifile.eq.ifiles(ibuf))) then
*           -----------------------------------------------
            write(ifile,rec=nrecs(ibuf),err=130,iostat=jrc)
     1      (record(i,ibuf),i=1,lrecl)
*           -----------------------------------------------
            modif(ibuf)=.false.
         endif
         if (ifile.eq.ifiles(ibuf)) ifiles(ibuf)=0
  110    continue
         close(ifile,err=130,status='keep',iostat=jrc)
      else if(istatu.eq.2) then
         do 120 ibuf=1,nbuf
         if (ifile.eq.ifiles(ibuf)) then
            iage(ibuf)=999999
            modif(ibuf)=.false.
            ifiles(ibuf)=0
         endif
  120    continue
         close(ifile,err=130,status='delete',iostat=jrc)
      else
         jrc=-999
      endif
  130 return
      end
