*DECK ADDOBJ
      SUBROUTINE ADDOBJ(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this module computes addition of objects.
*   1st option : Macrolib
*     (MAC1 - MAC2) OR (MAC1 + MAC2) OR (f1*MAC1+f2*MAC2) OR
*     (MAC1 + MAC2) from distributed calculations
*   2d option : Flux
*     f1*FLU1 + f2*FLU2 
* Note: the input order is important 
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): E. Varin, R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of lcm objects used by the module
* HENTRY  character*12 name of each lcm object
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the lcm object is created
*         =1 the lcm object is open for modifications;
*         =2 the lcm object is open in read-only mode
* KENTRY  =file unit number; =lcm object address otherwise
*         kentry(nentry)
* LCM OBJECTS:
* HENTRY(1)  create type (l_macrolib)
* HENTRY(2)  read only type (l_macrolib)
* HENTRY(3)  read only type (l_macrolib)
* or
* HENTRY(1)  create type (l_flux)
* HENTRY(2)  read only type (l_flux)
* HENTRY(3)  read only type (l_flux)
*                          
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
* STATE-VECTOR VARIABLES
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   DATXS1(NSTATE),DATXS2(NSTATE),FLUPRM(NSTATE)
* REDGET TYPES
      INTEGER ITYP,NITMA
      REAL FLOTT,FACT1,FACT2,KEFF1,KEFF2,KEFF3,FACT(2)
      DOUBLE PRECISION DFLOTT
*
* SIGNATURE INFORMATION
*
      CHARACTER*12 SGNTUR,TEXT,LEVEL,TEXT3(3)*12
      INTEGER      ISGNTR(3),IGA(3)
*
      INTEGER  IPOBJ3,IPOBJ2,IPOBJ1
*
      INTEGER I,IENERG,IPRT,IGR,J
* LOCAL VARIABLES
      INTEGER JPOBJ1,JPOBJ2,JPOBJ3,KPOBJ3,LCMLID,LCMGID,LCMLIL
      INTEGER NGRP,NMIXT,NL,NGRP2,NMIXT2,NL2,NDG,NDG2,ILONG,ITYLM,NUN
*
      INTEGER ITOTAL,INUSIG,ICHI,IOVERV,IDIFFX,IIJJP,INJJP,ICHIDL
      INTEGER IDIFFY,IDIFFZ,IH,IS,IIJJ,INJJ,ISCAT,IWORK,IIPOS,ISIGF,
     1        DATA1,DATA2
      LOGICAL DDIF,ADD,SUB,FMP,LADJ,LFLU,LF2P
      CHARACTER*12 TEXT12

* SETARA VARIABLES
      REAL BASE
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
*
*     INPUT OBJECT CONTROL
*
      IPOBJ2 = 0
      IPOBJ1 = 0
      IF(NENTRY.LT.3) CALL XABORT('ADDOBJ: PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('ADDOBJ:'
     + //' LCM OBJECT EXPECTED AT LHS.')
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('ADDOBJ:'
     + //' LCM OBJECT EXPECTED AT LHS.')
      IF((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)) CALL XABORT('ADDOBJ:'
     + //' LCM OBJECT EXPECTED AT LHS.')
      IF(JENTRY(1).GT.1) CALL XABORT('ADDOBJ:'
     + //' CREATE OR MODIFICATION MODE EXPECTED.')
      IF(JENTRY(2).NE.2) CALL XABORT('ADDOBJ:'
     + //' READ-ONLY MODE EXPECTED.')
*
      IPOBJ3=KENTRY(1)
*
      IPOBJ1=KENTRY(2)
      IPOBJ2=KENTRY(3)
*
      LEVEL= ' '
      FACT1 = 1.0
      FACT2 = 1.0
      IPRT = 0
      SUB= .FALSE.
      DDIF=.TRUE.
      FMP= .FALSE.
      ADD =.FALSE.
      LADJ=.FALSE.
      LF2P=.FALSE.
      LFLU=.TRUE.
 10   CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
 11   IF(ITYP.EQ.10) GOTO 20
      IF(ITYP.NE.3) THEN
        CALL XABORT('ADDOBJ: CHARACTER DATA EXPECTED')
      ELSE IF(TEXT.EQ.'STEP') THEN
        CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.1) THEN
         CALL XABORT('ADDOBJ: INTEGER DATA EXPECTED')
        ENDIF
        WRITE(LEVEL,'(4HSTEP,I8)') NITMA
      ELSE IF(TEXT.EQ.'EDIT') THEN
        CALL REDGET(ITYP,IPRT,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.1) THEN
         CALL XABORT('ADDOBJ: INTEGER DATA EXPECTED')
        ENDIF
      ELSE IF(TEXT.EQ.'SUB') THEN
        SUB = .TRUE.
        FACT1 = 1.0
        FACT2 = -1.0
      ELSE IF(TEXT.EQ.'ADD') THEN
        ADD = .TRUE.
        CALL REDGET(ITYP,NITMA,FACT1,TEXT,DFLOTT)
        IF(ITYP.NE.2) THEN
          GO TO 11
        ENDIF
        CALL REDGET(ITYP,NITMA,FACT2,TEXT,DFLOTT)
        IF(ITYP.NE.2) THEN
          GO TO 11
        ENDIF
        FMP=.TRUE.
      ELSE IF(TEXT.EQ.'FROM-MP') THEN
        FMP = .TRUE.
        FACT1=1.0
        FACT2=1.0
      ELSE IF(TEXT.EQ.'NODIF') THEN
        DDIF = .FALSE.
      ELSE IF(TEXT.EQ.'DDIF') THEN
        DDIF=.TRUE.
        IF(.NOT.ADD) WRITE(6,*) '*** ATTENTION *** IN USING THIS '
     +   //'OPTION YOU WILL OBTAIN DELTA D = 1/( 1/D1 - 1/D2) ***** '
        DDIF = .TRUE.
      ELSE IF(TEXT.EQ.'F-ADJOINT') THEN
        LADJ=.TRUE.
      ELSE IF(TEXT.EQ.'ADJ-ONLY') THEN
        LFLU=.FALSE.
      ELSE IF(TEXT.EQ.'FROM-2-PARTS') THEN
        LF2P=.TRUE.
        I=0
   12   I=I+1  
        CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
        IF(ITYP.NE.3) CALL XABORT('ADDOBJ: CHARACTER DATA EXPECTED(3)')
        IF((TEXT.EQ.'FLUX').OR.(TEXT.EQ.'AFLUX')) THEN 
          TEXT3(I)=TEXT
        ELSE IF((TEXT.EQ.'DFLUX').OR.(TEXT.EQ.'ADFLUX')) THEN
          TEXT3(I)=TEXT
          CALL REDGET(ITYP,IGA(I),FLOTT,TEXT,DFLOTT)
          IF(ITYP.NE.1) CALL XABORT('ADDOBJ: INTEGER DATA EXPECTED')
        ELSE
          CALL XABORT('ADDOBJ: WRONG FLUX KEYWORD')
        ENDIF
        IF(I.LE.2) THEN
          CALL REDGET(ITYP,NITMA,FACT(I),TEXT,DFLOTT)
          IF(ITYP.NE.2) CALL XABORT('ADDOBJ: REAL DATA EXPECTED')
        ENDIF
        IF(I.EQ.2) THEN
          CALL REDGET(ITYP,NITMA,FLOTT,TEXT,DFLOTT)
          IF(TEXT.NE.'IN') CALL XABORT('ADDOBJ: "IN" EXPECTED')
        ENDIF
        IF(I.LE.2) GO TO 12         
      ELSE IF(TEXT.EQ.';') THEN
        GOTO 20
      ELSE
        CALL XABORT('ADDOBJ: INVALID INPUT OPTION')
      ENDIF
      GOTO 10
*
 20   CALL LCMSIX(IPOBJ2,' ',0)
      CALL LCMSIX(IPOBJ1,' ',0)

      CALL LCMGET(IPOBJ1,'SIGNATURE',ISGNTR)
      WRITE(TEXT12,'(3A4)') (ISGNTR(J),J=1,3)

      IF(TEXT12.EQ.'L_MACROLIB') GO TO 100
      IF(TEXT12.EQ.'L_FLUX')     GO TO 200
      WRITE (6,*) 'NO OPERATION CAN BE PERFORMED ON ',TEXT12,' OBJECT'
      CALL XABORT('WRONG TYPE OF OBJECT')
C---------------------------
C     1      0000     0000    
C    11     0    0   0    0 
C   1 1    0      0 0      0 
C  1  1    0      0 0      0  
C 1   1    0      0 0      0  
C     1    0      0 0      0  
C     1     0    0   0    0  
C 11111111   0000     0000  
C---------------------------
C MACROLIB
C---------------------------
*
* NOW COPY MAC1 TO MAC3
* NB : ONLY THE CROSS-SECTIONS NEED TO BE MODIFIED
*
 100  CALL LCMGET(IPOBJ2,'STATE-VECTOR',DATXS2)
      NGRP2=DATXS2(1)
      NMIXT2=DATXS2(2)
      NL2=DATXS2(3)
      NDG2=DATXS2(7)
*
      CALL LCMGET(IPOBJ1,'STATE-VECTOR',DATXS1)
      NGRP=DATXS1(1)
      NMIXT=DATXS1(2)
      NL=DATXS1(3)
      NDG=DATXS1(7)
*
      IF(NL.NE.NL2) CALL XABORT('ADDOBJ: ILLEGAL LEGENDRE ORDER')
      IF(NDG.NE.NDG2) CALL XABORT('ADDOBJ: ILLEGAL PRECURSOR GROUP')
      IF(NGRP.NE.NGRP2) CALL XABORT('ADDOBJ: ILLEGAL GROUP NUMBER')
      IF(NMIXT.NE.NMIXT2) CALL XABORT('ADDOBJ: ILLEGAL MIXTURE NUMBER')
*
      CALL SETARA(BASE,NMIXT*NGRP,ITOTAL)
      CALL SETARA(BASE,NMIXT*NGRP,INUSIG)
      CALL SETARA(BASE,NMIXT*NGRP,ISIGF)
      CALL SETARA(BASE,NMIXT*NGRP,ICHI)
      IF(NDG.NE.0) CALL SETARA(BASE,NMIXT*NGRP*NDG,ICHIDL)
      CALL SETARA(BASE,NMIXT*NGRP,IOVERV)
      CALL SETARA(BASE,NMIXT*NGRP,IDIFFX)
      CALL SETARA(BASE,NMIXT*NGRP,IDIFFY)
      CALL SETARA(BASE,NMIXT*NGRP,IDIFFZ)
      CALL SETARA(BASE,NMIXT*NGRP,IH)
      CALL SETARA(BASE,NMIXT*NGRP,IS)
      CALL SETARA(IBASE,NMIXT*NGRP*NL,IIJJ)
      CALL SETARA(IBASE,NMIXT*NGRP*NL,INJJ)
      CALL SETARA(IBASE,NMIXT*NGRP*NL,IIJJP)
      CALL SETARA(IBASE,NMIXT*NGRP*NL,INJJP)
      CALL SETARA(BASE,NMIXT*NL*NGRP*NGRP,ISCAT)
      CALL SETARA(BASE,NMIXT*NGRP,IWORK)
      CALL SETARA(IBASE,NMIXT,IIPOS)
*
      CALL LCMSIX(IPOBJ2,' ',0)
      CALL LCMLEN(IPOBJ2,LEVEL,ILONG,ITYLM)
      IF(ITYLM.EQ.0) CALL LCMSIX(IPOBJ2,LEVEL,1)
      CALL LCMSIX(IPOBJ1,' ',0)
      CALL LCMLEN(IPOBJ1,LEVEL,ILONG,ITYLM)
      IF(ITYLM.EQ.0) CALL LCMSIX(IPOBJ1,LEVEL,1)
      IF( SUB )THEN
         CALL ADSUBM(IPOBJ1,IPOBJ2,IPOBJ3,LEVEL,NMIXT,NGRP,NDG,NL,
     X   BASE(ITOTAL),BASE(INUSIG),BASE(ISIGF),BASE(ICHI),BASE(ICHIDL),
     X   BASE(IOVERV),BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),
     X   BASE(IS),BASE(IIJJ),BASE(INJJ),BASE(ISCAT),BASE(IWORK),
     X   BASE(IIPOS),BASE(IIJJP),BASE(INJJP),IPRT,DDIF)
      ELSE IF(FMP) THEN
         CALL LCMSIX(IPOBJ3,' ',0)
         CALL ADMCMP(IPOBJ1,IPOBJ2,IPOBJ3,NMIXT,NGRP,NDG,NL,
     1   BASE(ITOTAL),BASE(INUSIG),BASE(ISIGF),BASE(ICHI),BASE(ICHIDL),
     2   BASE(IOVERV),BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),
     3   BASE(IS),BASE(IIJJ),BASE(INJJ),BASE(ISCAT),BASE(IWORK),
     4   BASE(IIPOS),BASE(IIJJP),BASE(INJJP),DDIF,FACT1,FACT2)
      ELSE IF(ADD) THEN
         CALL LCMSIX(IPOBJ3,' ',0)
         CALL ADIQSM(IPOBJ1,IPOBJ2,IPOBJ3,NMIXT,NGRP,NDG,NL,
     1   BASE(ITOTAL),BASE(INUSIG),BASE(ISIGF),BASE(ICHI),BASE(ICHIDL),
     x   BASE(IOVERV),BASE(IDIFFX),BASE(IDIFFY),BASE(IDIFFZ),BASE(IH),
     x   BASE(IS),BASE(IIJJ),BASE(INJJ),BASE(ISCAT),BASE(IWORK),
     x   BASE(IIPOS),BASE(IIJJP),BASE(INJJP),DDIF,FACT1)
      ELSE
         WRITE (6,*) 'NO KEYWORD SPECIFED, NOTHING PERFORMED'
      ENDIF
*
      CALL RLSARA(BASE(ITOTAL))
      CALL RLSARA(BASE(INUSIG))
      CALL RLSARA(BASE(ISIGF))
      CALL RLSARA(BASE(ICHI))
      IF(NDG.NE.0) CALL RLSARA(BASE(ICHIDL))
      CALL RLSARA(BASE(IOVERV))
      CALL RLSARA(BASE(IDIFFX))
      CALL RLSARA(BASE(IDIFFY))
      CALL RLSARA(BASE(IDIFFZ))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(BASE(IS))
      CALL RLSARA(IBASE(IIJJ))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(IBASE(IIJJP))
      CALL RLSARA(IBASE(INJJP))
      CALL RLSARA(BASE(ISCAT))
      CALL RLSARA(BASE(IWORK))
      CALL RLSARA(IBASE(IIPOS))
*
      CALL LCMSIX(IPOBJ3,' ',0)
      SGNTUR='L_MACROLIB'
      READ(SGNTUR,'(3A4)') (ISGNTR(I),I=1,3)
      CALL LCMPUT(IPOBJ3,'SIGNATURE',3,3,ISGNTR)
      CALL LCMPUT(IPOBJ3,'STATE-VECTOR',NSTATE,1,DATXS1)
      CALL LCMLEN(IPOBJ1,'ENERGY',ILONG,ITYLM)
      IF(ILONG.EQ.(NGRP+1)) THEN
        CALL SETARA(BASE,ILONG,IENERG)
        CALL LCMGET(IPOBJ1,'ENERGY',BASE(IENERG))
        CALL LCMPUT(IPOBJ3,'ENERGY',ILONG,2,BASE(IENERG))
        CALL RLSARA(BASE(IENERG))
      ENDIF
      CALL LCMLEN(IPOBJ1,'VOLUME',ILONG,ITYLM)
      IF(ILONG.EQ.NMIXT) THEN
        CALL SETARA(BASE,ILONG,IENERG)
        CALL LCMGET(IPOBJ1,'VOLUME',BASE(IENERG))
        CALL LCMPUT(IPOBJ3,'VOLUME',ILONG,2,BASE(IENERG))
        CALL RLSARA(BASE(IENERG))
      ENDIF
      CALL LCMSIX(IPOBJ3,' ',0)
      CALL LCMSIX(IPOBJ2,' ',0)
      CALL LCMSIX(IPOBJ1,' ',0)
*
      GO TO 1000

C---------------------------
C   2222     0000     0000    
C  2    2   0    0   0    0 
C 2      2 0      0 0      0 
C       2  0      0 0      0  
C      2   0      0 0      0  
C   22     0      0 0      0  
C  2        0    0   0    0  
C 22222222   0000     0000  
C---------------------------
C FLUX
C---------------------------
 200  CALL LCMSIX(IPOBJ1,' ',0)
      CALL LCMSIX(IPOBJ2,' ',0)
      CALL LCMSIX(IPOBJ3,' ',0)
      CALL LCMGET(IPOBJ1,'STATE-VECTOR',FLUPRM)
      CALL LCMPUT(IPOBJ3,'STATE-VECTOR',NSTATE,1,FLUPRM)
      NGRP=FLUPRM(1)
      NUN=FLUPRM(2)
      CALL SETARA(BASE,NUN,DATA1)
      CALL SETARA(BASE,NUN,DATA2)
C CAS GENERAL
      IF(.NOT.LF2P) THEN
* k_eff
        CALL LCMGET(IPOBJ1,'K-EFFECTIVE',KEFF1)
        CALL LCMGET(IPOBJ2,'K-EFFECTIVE',KEFF2)
        KEFF3=KEFF1*FACT1+KEFF2*FACT2
        CALL LCMPUT(IPOBJ3,'K-EFFECTIVE',1,2,KEFF3)
* flux et / ou adjoint
        IF(LFLU) THEN
           JPOBJ1=LCMGID(IPOBJ1,'FLUX')
           JPOBJ2=LCMGID(IPOBJ2,'FLUX')
           JPOBJ3=LCMLID(IPOBJ2,'FLUX',NGRP)
           DO 220 IGR=1,NGRP
             CALL LCMGDL(IPOBJ1,IGR,BASE(DATA1))
             CALL LCMGDL(IPOBJ2,IGR,BASE(DATA2))
             DO 210 I=0,NUN-1
 210         BASE(DATA2+I)= BASE(DATA1+I)*FACT1+BASE(DATA2+I)*FACT2
             CALL LCMPDL(IPOBJ3,IGR,NUN,2,BASE(DATA2))
 220      CONTINUE
        ENDIF
        IF(LADJ) THEN
           JPOBJ1=LCMGID(IPOBJ1,'AFLUX')
           JPOBJ2=LCMGID(IPOBJ2,'AFLUX')
           JPOBJ3=LCMLID(IPOBJ2,'AFLUX',NGRP)
           DO 240 IGR=1,NGRP
             CALL LCMGDL(IPOBJ1,IGR,BASE(DATA1))
             CALL LCMGDL(IPOBJ2,IGR,BASE(DATA2))
             DO 230 I=0,NUN-1
 230         BASE(DATA2+I)= BASE(DATA1+I)*FACT1+BASE(DATA2+I)*FACT2
             CALL LCMPDL(IPOBJ3,IGR,NUN,2,BASE(DATA2))
 240      CONTINUE
        ENDIF
C CAS PARTICULIER DE 2 PARTIES SPECIFIQUES
      ELSE
        JPOBJ1=LCMGID(IPOBJ1,TEXT3(1))
        JPOBJ2=LCMGID(IPOBJ2,TEXT3(2))
        JPOBJ3=LCMLID(IPOBJ2,TEXT3(3),MAX(IGA(I),20))
        KPOBJ3=LCMLIL(JPOBJ2,IGA(I),NGRP)
        DO 280 IGR=1,NGRP
           CALL LCMGDL(JPOBJ1,IGR,BASE(DATA1))
           CALL LCMGDL(JPOBJ2,IGR,BASE(DATA2))
           DO 270 I=0,NUN-1
  270      BASE(DATA2+I)= BASE(DATA1+I)*FACT(1)+BASE(DATA2+I)*FACT(2)
           CALL LCMPDL(KPOBJ3,IGR,NUN,2,BASE(DATA2))
  280   CONTINUE
      ENDIF
      CALL RLSARA(BASE(DATA1))
      CALL RLSARA(BASE(DATA2))
      SGNTUR='L_FLUX'
      READ(SGNTUR,'(3A4)') (ISGNTR(I),I=1,3)
      CALL LCMPUT(IPOBJ3,'SIGNATURE',3,3,ISGNTR)
      GO TO 1000
C---------------------------
C END
C---------------------------
 1000 CONTINUE 
      IF( IPRT.GT.1 )CALL LCMLIB(IPOBJ3)
      RETURN
      END
