*DECK ADIQSM
      SUBROUTINE ADIQSM(IPMAC1,IPMAC2,IPMAC3,NMIXT,NGRP,NDG,NL,TOTAL,
     1 ZNUG,ZSIGF,CHI,CHDL,OVERV,DIFFX,DIFFY,DIFFZ,H,S,IJJ,NJJ,SCAT,
     2 WORK,IPOS,IJJP,NJJP,DDIF,FACT1)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read and add cross sections and store them in a lcm object.
* Note: any XS not read is set to zero.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): M.T. Sissaoui
*  modified R. Chambon
*
*Parameters: input
* IPMAC1  ADDRESS OF THE LCM OBJECT.
* IPMAC2  ADDRESS OF THE LCM OBJECT.
* NMIXT   MAXIMUM NUMBER OF MATERIAL MIXTURES.
* NGRP    NUMBER OF ENERGY GROUPS.
* NL      NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
* NDG     NUMBER OF DELAYED NEUTRON SPECTRUM.
* DDIF    LOGICAL FOR DIFFUSION CALCULATION
* FACT1   COEFFICIENT TO MULTIPLY SECOND CROSS SECTION SET WITH
*
*
*Parameters: scratch
* TOTAL   DIMENSION TOTAL(NMIXT,NGRP)
* ZNUG    DIMENSION ZNUG(NMIXT,NGRP)
* CHI     DIMENSION CHI(NMIXT,NGRP)
* CHDL    DIMENSION CHDL(NMIXT,NDG,NGRP)
* OVERV   DIMENSION OVERV(NMIXT,NGRP)
* DIFFX   DIMENSION DIFFX(NMIXT,NGRP)
* DIFFY   DIMENSION DIFFY(NMIXT,NGRP)
* DIFFZ   DIMENSION DIFFZ(NMIXT,NGRP)
* H       DIMENSION H(NMIXT,NGRP)
* S       DIMENSION S(NMIXT,NGRP)
* IJJ     DIMENSION IJJ(NMIXT,NL,NGRP)
* NJJ     DIMENSION NJJ(NMIXT,NL,NGRP)
* SCAT    SCAT(NMIXT,NL,NGRP,NGRP)
* WORK    WORK(NMIXT*NGRP)
* IPOS    IPOS(NMIXT)
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPMAC1,IPMAC2,IPMAC3,NMIXT,NGRP,NDG,NL,IJJ(NMIXT,NL,NGRP),
     1 NJJ(NMIXT,NL,NGRP),IJJP(NMIXT,NL,NGRP),NJJP(NMIXT,NL,NGRP),
     2 IPOS(NMIXT)
      REAL TOTAL(NMIXT,NGRP),ZNUG(NMIXT,NGRP),ZSIGF(NMIXT,NGRP),
     1 CHI(NMIXT,NGRP),OVERV(NMIXT,NGRP),DIFFX(NMIXT,NGRP),
     2 DIFFY(NMIXT,NGRP),DIFFZ(NMIXT,NGRP),SCAT(NMIXT,NL,NGRP,NGRP),
     3 H(NMIXT,NGRP),S(NMIXT,NGRP),WORK(NMIXT*NGRP),
     4 CHDL(NMIXT,NDG,NGRP),FACT1
      LOGICAL DDIF
*----
*  LOCAL VARIABLES
*----
      LOGICAL LTO,LFI,LNFI,LCH,LOV,LHF,LSC,LSO,LBI
      CHARACTER CM*2,TEXT*8
      COMMON BASE(1)
*
      IF(NMIXT.EQ.0) CALL XABORT('IQSADD: ZERO NUMBER OF MIXTURES.')
      IF(NGRP.EQ.0) CALL XABORT('IQSADD: ZERO NUMBER OF GROUPS.')
*
      LTO=.FALSE.
      LFI=.FALSE.
      LNFI=.FALSE.
      LCH=.FALSE.
      LOV=.FALSE.
      LHF=.FALSE.
      LSC=.FALSE.
      LSO=.FALSE.
      LBI=.FALSE.
*
      DO 10 IGR=1,NGRP
      DO 10 IBM=1,NMIXT
      TOTAL(IBM,IGR)=0.0
      ZNUG(IBM,IGR)=0.0
      ZSIGF(IBM,IGR)=0.0
      OVERV(IBM,IGR)=0.0
      CHI(IBM,IGR)=0.0
      DIFFX(IBM,IGR)=0.0
      DIFFY(IBM,IGR)=0.0
      DIFFZ(IBM,IGR)=0.0
      H(IBM,IGR)=0.0
      S(IBM,IGR)=0.0
      DO 10 IL=1,NL
      IJJ(IBM,IL,IGR)=IGR
      NJJ(IBM,IL,IGR)=1
      DO 10 JGR=1,NGRP
   10 SCAT(IBM,IL,JGR,IGR)=0.0
*
* RECOVER THE UNPERTURBED DATA
      ILEAK=0
      JPMAC1=LCMGID(IPMAC1,'GROUP')
      DO 40 JGR=1,NGRP
         KPMAC1=LCMGIL(JPMAC1,JGR)
         CALL LCMLEN(KPMAC1,'NTOT0',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            LTO=.TRUE.
            CALL LCMGET(KPMAC1,'NTOT0',TOTAL(1,JGR))
         ELSE IF(ILENGT.NE.0) THEN
            CALL XABORT('MACXSI: INVALID INPUT MACROLIB(1).')
         ENDIF
         CALL LCMLEN(KPMAC1,'NUSIGF',ILENGT,ITYLCM)
          IF(ILENGT.EQ.NMIXT) THEN
          LNFI=.TRUE.
          CALL LCMGET(KPMAC1,'NUSIGF',ZNUG(1,JGR))
          ENDIF
         CALL LCMLEN(KPMAC1,'NFTOT',ILENGT,ITYLCM)
          IF(ILENGT.EQ.NMIXT) THEN
          LFI=.TRUE.
          CALL LCMGET(KPMAC1,'NFTOT',ZSIGF(1,JGR))
          ENDIF
         CALL LCMLEN(KPMAC1,'CHI',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
          LCH=.TRUE.
          CALL LCMGET(KPMAC1,'CHI',CHI(1,JGR))
         ENDIF
         DO 900 I=1,NDG
         WRITE (TEXT,'(4HCHDL,I3.3)') I
         CALL LCMLEN(KPMAC1,TEXT,ILENGT,ITYLCM)
          IF(ILENGT.EQ.NMIXT) THEN
          LBI=.TRUE.
          CALL LCMGET(KPMAC1,TEXT,CHDL(1,I,JGR))
          ENDIF
 900     CONTINUE
         CALL LCMLEN(KPMAC1,'OVERV',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
          LOV=.TRUE.
          CALL LCMGET(KPMAC1,'OVERV',OVERV(1,JGR))
         ENDIF
         IF(DDIF) THEN
            CALL LCMLEN(KPMAC1,'DIFF',ILENGT,ITYLCM)
            IF(ILENGT.EQ.NMIXT) THEN
               ILEAK=1
               CALL LCMGET(KPMAC1,'DIFF',DIFFX(1,JGR))
            ENDIF
            CALL LCMLEN(KPMAC1,'DIFFX',ILENGT,ITYLCM)
            IF(ILENGT.EQ.NMIXT) THEN
               ILEAK=2
               CALL LCMGET(KPMAC1,'DIFFX',DIFFX(1,JGR))
               CALL LCMGET(KPMAC1,'DIFFY',DIFFY(1,JGR))
               CALL LCMGET(KPMAC1,'DIFFZ',DIFFZ(1,JGR))
            ENDIF
         ENDIF
         CALL LCMLEN(KPMAC1,'H-FACTOR',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            LHF=.TRUE.
            CALL LCMGET(KPMAC1,'H-FACTOR',H(1,JGR))
         ENDIF
         CALL LCMLEN(KPMAC1,'FIXE',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
           LSO=.TRUE.
           CALL LCMGET(KPMAC1,'FIXE',S(1,JGR))
         ENDIF
         DO 30 IL=1,NL
           WRITE (CM,'(I2.2)') IL-1
           CALL LCMLEN(KPMAC1,'SCAT'//CM,ILENGT,ITYLCM)
           IF(ILENGT.GT.NMIXT*NL*NGRP*NGRP) THEN
              CALL XABORT('MACXSI: INVALID INPUT MACROLIB(2).')
           ELSE IF(ILENGT.GT.0) THEN
            LSC=.TRUE.
            CALL LCMGET(KPMAC1,'SCAT'//CM,WORK)
            CALL LCMGET(KPMAC1,'NJJS'//CM,NJJ(1,IL,JGR))
            CALL LCMGET(KPMAC1,'IJJS'//CM,IJJ(1,IL,JGR))
            IPOSDE=0
            DO 20 IBM=1,NMIXT
              IJJ0=IJJ(IBM,IL,JGR)
              DO 20 IGR=IJJ0,IJJ0-NJJ(IBM,IL,JGR)+1,-1
                IPOSDE=IPOSDE+1
   20           SCAT(IBM,IL,IGR,JGR)=WORK(IPOSDE)
           ENDIF
   30    CONTINUE
   40 CONTINUE
*
* RECOVER THE PERTURBED DATA
      ILEAKP=0
      JPMAC2=LCMGID(IPMAC2,'GROUP')
      DO 940 JGR=1,NGRP
      KPMAC2=LCMGIL(JPMAC2,JGR)
      CALL LCMLEN(KPMAC2,'NTOT0',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,ITOT)
         CALL LCMGET(KPMAC2,'NTOT0',BASE(ITOT))
         DO 100 IMX=1,ILENGT
 100     TOTAL(IMX,JGR)=TOTAL(IMX,JGR)+BASE(ITOT-1+IMX)*FACT1
         CALL RLSARA(BASE(ITOT))
      ELSE IF(ILENGT.NE.0) THEN
         CALL XABORT('IQSADD: INVALID INPUT MACROLIB(1).')
      ENDIF
*
      CALL LCMLEN(KPMAC2,'NUSIGF',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,INUS)
         CALL LCMGET(KPMAC2,'NUSIGF',BASE(INUS))
         DO 110 IMX=1,ILENGT
 110     ZNUG(IMX,JGR)=ZNUG(IMX,JGR)+BASE(INUS-1+IMX)*FACT1
         CALL RLSARA(BASE(INUS))
      ENDIF
*
      CALL LCMLEN(KPMAC2,'NFTOT',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,INUS)
         CALL LCMGET(KPMAC2,'NFTOT',BASE(INUS))
         DO 111 IMX=1,ILENGT
 111     ZSIGF(IMX,JGR)=ZSIGF(IMX,JGR)+BASE(INUS-1+IMX)*FACT1
         CALL RLSARA(BASE(INUS))
      ENDIF
*
      CALL LCMLEN(KPMAC2,'CHI',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,ICHI)
         CALL LCMGET(KPMAC2,'CHI',BASE(ICHI))
         DO 120 IMX=1,ILENGT
 120     CHI(IMX,JGR)=CHI(IMX,JGR)+BASE(ICHI-1+IMX)*FACT1
         CALL RLSARA(BASE(ICHI))
      ENDIF
*
      DO 990 I=1,NDG
      WRITE (TEXT,'(4HCHDL,I3.3)') I
      CALL LCMLEN(KPMAC2,TEXT,ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,ICHD)
         CALL LCMGET(KPMAC2,TEXT,BASE(ICHD))
         DO 220 IMX=1,ILENGT
 220     CHDL(IMX,I,JGR)=CHDL(IMX,I,JGR)+BASE(ICHD-1+IMX)*FACT1
         CALL RLSARA(BASE(ICHD))
      ENDIF
 990  CONTINUE
*
      CALL LCMLEN(KPMAC2,'OVERV',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,IOVR)
         CALL LCMGET(KPMAC2,'OVERV',BASE(IOVR))
         DO 130 IMX=1,ILENGT
 130     OVERV(IMX,JGR)=OVERV(IMX,JGR)+BASE(IOVR-1+IMX)*FACT1
         CALL RLSARA(BASE(IOVR))
      ENDIF
*
      IF(DDIF) THEN
         CALL LCMLEN(KPLISP,'DIFF',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            ILEAKP=1
            CALL SETARA(BASE,ILENGT,IDFX)
            CALL LCMGET(KPLISP,'DIFF',BASE(IDFX))
            DO 140 IMX=1,ILENGT
            IF(BASE(IDFX-1+IMX).NE.0.0) THEN
              DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)+1./BASE(IDFX-1+IMX)*FACT1
              DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)
            ENDIF
 140        CONTINUE
            CALL RLSARA(BASE(IDFX))
         ENDIF
*
         CALL LCMLEN(KPLISP,'DIFFX',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            ILEAKP=2
            CALL SETARA(BASE,ILENGT,IDFX)
            CALL SETARA(BASE,ILENGT,IDFY)
            CALL SETARA(BASE,ILENGT,IDFZ)
            CALL LCMGET(KPLISP,'DIFFX',BASE(IDFX))
            CALL LCMGET(KPLISP,'DIFFY',BASE(IDFY))
            CALL LCMGET(KPLISP,'DIFFZ',BASE(IDFZ))
            DO 150 IMX=1,ILENGT
            IF(BASE(IDFX-1+IMX).NE.0.0) THEN
              DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)+1./BASE(IDFX-1+IMX)*FACT1
              DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)
            ENDIF
            IF(BASE(IDFY-1+IMX).NE.0.0) THEN
              DIFFY(IMX,JGR)=1./DIFFY(IMX,JGR)+1./BASE(IDFY-1+IMX)*FACT1
              DIFFY(IMX,JGR)=1./DIFFY(IMX,JGR)
            ENDIF
            IF(BASE(IDFZ-1+IMX).NE.0.0) THEN
              DIFFZ(IMX,JGR)=1./DIFFZ(IMX,JGR)+1./BASE(IDFZ-1+IMX)*FACT1
              DIFFZ(IMX,JGR)=1./DIFFZ(IMX,JGR)
            ENDIF
 150        CONTINUE
            CALL RLSARA(BASE(IDFZ))
            CALL RLSARA(BASE(IDFY))
            CALL RLSARA(BASE(IDFX))
         ENDIF
      ENDIF
*
      CALL LCMLEN(KPMAC2,'H-FACTOR',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,IH)
         CALL LCMGET(KPMAC2,'H-FACTOR',BASE(IH))
         DO 270 IMX=1,ILENGT
 270     H(IMX,JGR)=H(IMX,JGR)+BASE(IH-1+IMX)*FACT1
         CALL RLSARA(BASE(IH))
      ENDIF
*
      CALL LCMLEN(KPMAC2,'FIXE',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,IS)
         CALL LCMGET(KPMAC2,'FIXE',BASE(IS))
         DO 280 IMX=1,ILENGT
 280     S(IMX,JGR)=S(IMX,JGR)+BASE(IS-1+IMX)*FACT1
         CALL RLSARA(BASE(IS))
      ENDIF
*
      DO 930 IL=1,NL
      WRITE (CM,'(I2.2)') IL-1
      CALL LCMLEN(KPMAC2,'SCAT'//CM,ILENGT,ITYLCM)
      IF(ILENGT.GT.NMIXT*NL*NGRP*NGRP) THEN
         CALL XABORT('MACXSI: INVALID INPUT MACROLIB(2).')
      ELSE IF(ILENGT.GT.0) THEN
         CALL LCMGET(KPMAC2,'SCAT'//CM,WORK)
         CALL LCMGET(KPMAC2,'NJJS'//CM,NJJP(1,IL,JGR))
         CALL LCMGET(KPMAC2,'IJJS'//CM,IJJP(1,IL,JGR))
         IPOSDE=0
         DO 920 IBM=1,NMIXT
         IJJ0=IJJP(IBM,IL,JGR)
         IJ=IJJ0-IJJ(IBM,IL,JGR)
         IN=NJJP(IBM,IL,JGR)-NJJ(1,IL,JGR)
         IF(IJ.EQ.0.AND.IN.EQ.0) THEN
            DO 820 IGR=IJJ0,IJJ0-NJJP(IBM,IL,JGR)+1,-1
            IPOSDE=IPOSDE+1
  820       SCAT(IBM,IL,IGR,JGR)=SCAT(IBM,IL,IGR,JGR)+WORK(IPOSDE)*FACT1
         ENDIF
  920    CONTINUE
      ENDIF
  930 CONTINUE
  940 CONTINUE
C
      JPMAC3=LCMLID(IPMAC3,'GROUP',NGRP)
      DO 190 JGR=1,NGRP
        KPMAC3=LCMDIL(JPMAC3,JGR)
        IF(LTO) CALL LCMPUT(KPMAC3,'NTOT0',NMIXT,2,TOTAL(1,JGR))
        IF(LNFI) CALL LCMPUT(KPMAC3,'NUSIGF',NMIXT,2,ZNUG(1,JGR))
        IF(LFI) CALL LCMPUT(KPMAC3,'NFTOT',NMIXT,2,ZSIGF(1,JGR))
        IF(LCH) CALL LCMPUT(KPMAC3,'CHI',NMIXT,2,CHI(1,JGR))
        IF(LOV) CALL LCMPUT(KPMAC3,'OVERV',NMIXT,2,OVERV(1,JGR))
        IF((ILEAK.EQ.1).AND.(ILEAKP.EQ.1)) THEN
           CALL LCMPUT(KPMAC3,'DIFF',NMIXT,2,DIFFX(1,JGR))
        ELSE IF((ILEAK.EQ.2).AND.(ILEAKP.EQ.2)) THEN
           CALL LCMPUT(KPMAC3,'DIFFX',NMIXT,2,DIFFX(1,JGR))
           CALL LCMPUT(KPMAC3,'DIFFY',NMIXT,2,DIFFY(1,JGR))
           CALL LCMPUT(KPMAC3,'DIFFZ',NMIXT,2,DIFFZ(1,JGR))
        ENDIF
        IF(LHF) CALL LCMPUT(KPMAC3,'H-FACTOR',NMIXT,2,H(1,JGR))
        IF(LSO) CALL LCMPUT(KPMAC3,'FIXE',NMIXT,2,S(1,JGR))
        IF(LBI) THEN
          DO 901 I=1,NDG
            WRITE (TEXT,'(4HCHDL,I3.3)') I
            CALL LCMPUT(KPMAC3,TEXT,NMIXT,2,CHDL(1,I,JGR))
 901      CONTINUE
        ENDIF
        IF(LSC) THEN
           DO 180 IL=1,NL
             WRITE (CM,'(I2.2)') IL-1
             IPOSDE=0
             DO 170 IBM=1,NMIXT
              IPOS(IBM)=IPOSDE+1
              DO 171 IGR=IJJ(IBM,IL,JGR),
     +               IJJ(IBM,IL,JGR)-NJJ(IBM,IL,JGR)+1,-1
                IPOSDE=IPOSDE+1
                WORK(IPOSDE)=SCAT(IBM,IL,IGR,JGR)
 171          CONTINUE
 170         CONTINUE
             CALL LCMPUT(KPMAC3,'SCAT'//CM,IPOSDE,2,WORK)
             CALL LCMPUT(KPMAC3,'IPOS'//CM,NMIXT,1,IPOS)
             CALL LCMPUT(KPMAC3,'NJJS'//CM,NMIXT,1,NJJ(1,IL,JGR))
             CALL LCMPUT(KPMAC3,'IJJS'//CM,NMIXT,1,IJJ(1,IL,JGR))
             CALL LCMPUT(KPMAC3,'SIGW'//CM,NMIXT,2,SCAT(1,IL,JGR,JGR))
 180       CONTINUE
        ENDIF
 190  CONTINUE
      RETURN
      END
