*DECK ADSUBM
      SUBROUTINE ADSUBM(IPLIST,IPLISP,IPLISD,LEVEL,NMIXT,NGRP,NDG,NL,
     1 TOTAL,ZNUG,ZSIGF,CHI,CHDL,OVERV,DIFFX,DIFFY,DIFFZ,H,S,IJJ,NJJ,
     2 SCAT,WORK,IPOS,IJJP,NJJP,IPRT,DDIF)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read and substract cross sections and store them in a lcm object.
* Note: any XS not read is set to zero.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): M.T. Sissaoui
*  modified E. Varin
*
*Parameters: input
*  IPLIST : ADDRESS OF THE LCM OBJECT.
*  IPLISP : ADDRESS OF THE LCM OBJECT.
*  IPLISD : ADDRESS OF THE LCM OBJECT.
*  NMIXT  : MAXIMUM NUMBER OF MATERIAL MIXTURES.
*  NGRP   : NUMBER OF ENERGY GROUPS.
*  NL     : NUMBER OF LEGENDRE ORDERS (=1 FOR ISOTROPIC SCATTERING).
*  NDG    : NUMBER OF DELAYED NEUTRON SPECTRUM.
*  DDIF   : LOGICAL FOR DIFFUSION CALCULATION
*  IPRT   : IMPRESSION LEVEL
*
*Parameters: scratch
*  TOTAL  : DIMENSION TOTAL(NMIXT,NGRP)
*  ZNUG   : DIMENSION ZNUG(NMIXT,NGRP)
*  ZSIGF  : DIMENSION ZSIGF(NMIXT,NGRP)
*  CHI    : DIMENSION CHI(NMIXT,NGRP)
*  CHDL   : DIMENSION CHDL(NMIXT,NDG,NGRP)
*  OVERV  : DIMENSION OVERV(NMIXT,NGRP)
*  DIFFX  : DIMENSION DIFFX(NMIXT,NGRP)
*  DIFFY  : DIMENSION DIFFY(NMIXT,NGRP)
*  DIFFZ  : DIMENSION DIFFZ(NMIXT,NGRP)
*  H      : DIMENSION H(NMIXT,NGRP)
*  S      : DIMENSION S(NMIXT,NGRP)
*  IJJ    : DIMENSION IJJ(NMIXT,NL,NGRP)
*  NJJ    : DIMENSION NJJ(NMIXT,NL,NGRP)
*  SCAT   : SCAT(NMIXT,NL,NGRP,NGRP)
*  WORK   : WORK(NMIXT*NGRP)
*  IPOS   : IPOS(NMIXT)
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      LOGICAL DDIF
      INTEGER IPLIST,IPLISP,IPLISD,NMIXT,NGRP,NDG,NL,IJJ(NMIXT,NL,NGRP),
     1 NJJ(NMIXT,NL,NGRP),IJJP(NMIXT,NL,NGRP),NJJP(NMIXT,NL,NGRP),
     2 IPOS(NMIXT),IPRT
      CHARACTER LEVEL*12
      REAL TOTAL(NMIXT,NGRP),ZNUG(NMIXT,NGRP),ZSIGF(NMIXT,NGRP),
     1 CHI(NMIXT,NGRP),OVERV(NMIXT,NGRP),DIFFX(NMIXT,NGRP),
     2 DIFFY(NMIXT,NGRP),DIFFZ(NMIXT,NGRP),SCAT(NMIXT,NL,NGRP,NGRP),
     3 H(NMIXT,NGRP),S(NMIXT,NGRP),WORK(NMIXT*NGRP),
     4 CHDL(NMIXT,NDG,NGRP)
*----
*  LOCAL VARIABLES
*----
      LOGICAL LTO,LFI,LNFI,LCH,LOV,LHF,LSC,LSO,LBI
      CHARACTER CM*2,TEXT*8
      COMMON BASE(1)
*
      IF(NMIXT.EQ.0) CALL XABORT('MACXSI: ZERO NUMBER OF MIXTURES.')
      IF(NGRP.EQ.0) CALL XABORT('MACXSI: ZERO NUMBER OF GROUPS.')
*
      LTO=.FALSE.
      LFI=.FALSE.
      LNFI=.FALSE.
      LCH=.FALSE.
      LOV=.FALSE.
      LHF=.FALSE.
      LSC=.FALSE.
      LSO=.FALSE.
      LBI=.FALSE.
*
      WRITE(6,*) DDIF,'DDIF'
      DO 10 IGR=1,NGRP
      DO 10 IBM=1,NMIXT
      TOTAL(IBM,IGR)=0.0
      ZNUG(IBM,IGR)=0.0
      ZSIGF(IBM,IGR)=0.0
      DIFFX(IBM,IGR)=0.0
      DIFFY(IBM,IGR)=0.0
      DIFFZ(IBM,IGR)=0.0
      H(IBM,IGR)=0.0
      S(IBM,IGR)=0.0
      DO 10 IL=1,NL
      IJJ(IBM,IL,IGR)=IGR
      NJJ(IBM,IL,IGR)=1
      DO 10 JGR=1,NGRP
   10 SCAT(IBM,IL,JGR,IGR)=0.0
*
*     RECOVER THE PERTURBED DATA
      ILEAK=0
      JPLIST=LCMGID(IPLIST,'GROUP')
      DO 40 JGR=1,NGRP
         KPLIST=LCMGIL(JPLIST,JGR)
         CALL LCMLEN(KPLIST,'NTOT0',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            CALL LCMGET(KPLIST,'NTOT0',TOTAL(1,JGR))
         ELSE IF(ILENGT.NE.0) THEN
            CALL XABORT('MACXSI: INVALID INPUT MACROLIB(1).')
         ENDIF
         CALL LCMLEN(KPLIST,'NUSIGF',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,'NUSIGF',ZNUG(1,JGR))
         CALL LCMLEN(KPLIST,'NFTOT',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,'NFTOT',ZSIGF(1,JGR))
         CALL LCMLEN(KPLIST,'CHI',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,'CHI',CHI(1,JGR))
         DO 900 I=1,NDG
         WRITE (TEXT,'(4HCHDL,I3.3)') I
         CALL LCMLEN(KPLIST,TEXT,ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,TEXT,CHDL(1,I,JGR))
 900     CONTINUE
         CALL LCMLEN(KPLIST,'OVERV',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,'OVERV',OVERV(1,JGR))
         IF(DDIF) THEN
            CALL LCMLEN(KPLIST,'DIFF',ILENGT,ITYLCM)
            IF(ILENGT.EQ.NMIXT) THEN
               ILEAK=1
               CALL LCMGET(KPLIST,'DIFF',DIFFX(1,JGR))
            ENDIF
            CALL LCMLEN(KPLIST,'DIFFX',ILENGT,ITYLCM)
            IF(ILENGT.EQ.NMIXT) THEN
               ILEAK=2
               CALL LCMGET(KPLIST,'DIFFX',DIFFX(1,JGR))
               CALL LCMGET(KPLIST,'DIFFY',DIFFY(1,JGR))
               CALL LCMGET(KPLIST,'DIFFZ',DIFFZ(1,JGR))
            ENDIF
         ENDIF
         CALL LCMLEN(KPLIST,'H-FACTOR',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,'H-FACTOR',H(1,JGR))
         CALL LCMLEN(KPLIST,'FIXE',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) CALL LCMGET(KPLIST,'FIXE',S(1,JGR))
         DO 30 IL=1,NL
         WRITE (CM,'(I2.2)') IL-1
         CALL LCMLEN(KPLIST,'SCAT'//CM,ILENGT,ITYLCM)
         IF(ILENGT.GT.NMIXT*NL*NGRP*NGRP) THEN
            CALL XABORT('MACXSI: INVALID INPUT MACROLIB(2).')
         ELSE IF(ILENGT.GT.0) THEN
            CALL LCMGET(KPLIST,'SCAT'//CM,WORK)
            CALL LCMGET(KPLIST,'NJJS'//CM,NJJ(1,IL,JGR))
            CALL LCMGET(KPLIST,'IJJS'//CM,IJJ(1,IL,JGR))
            IPOSDE=0
            DO 20 IBM=1,NMIXT
            IJJ0=IJJ(IBM,IL,JGR)
            DO 20 IGR=IJJ0,IJJ0-NJJ(IBM,IL,JGR)+1,-1
            IPOSDE=IPOSDE+1
   20       SCAT(IBM,IL,IGR,JGR)=WORK(IPOSDE)
         ENDIF
   30    CONTINUE
   40 CONTINUE
*
* RECOVER THE UNPERTURBED DATA AND COMPUTE DIFFERENCES
      ILEAKP=0
      JPLISP=LCMGID(IPLISP,'GROUP')
      DO 940 JGR=1,NGRP
      KPLISP=LCMGIL(JPLISP,JGR)
      CALL LCMLEN(KPLISP,'NTOT0',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,ITOT)
         CALL LCMGET(KPLISP,'NTOT0',BASE(ITOT))
         DO 100 IMX=1,ILENGT
         TOTAL(IMX,JGR)=TOTAL(IMX,JGR)-BASE(ITOT-1+IMX)
         IF(TOTAL(IMX,JGR).NE.0.0) LTO = .TRUE.
 100     CONTINUE
         CALL RLSARA(BASE(ITOT))
      ELSE IF(ILENGT.NE.0) THEN
         CALL XABORT('SUBMAC: INVALID INPUT MACROLIB(1).')
      ENDIF
*
      CALL LCMLEN(KPLISP,'NUSIGF',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,INUS)
         CALL LCMGET(KPLISP,'NUSIGF',BASE(INUS))
         DO 110 IMX=1,ILENGT
         ZNUG(IMX,JGR)=ZNUG(IMX,JGR)-BASE(INUS-1+IMX)
         IF(ZNUG(IMX,JGR).NE.0.0) LNFI = .TRUE.
 110     CONTINUE
         CALL RLSARA(BASE(INUS))
      ENDIF
*
      CALL LCMLEN(KPLISP,'NFTOT',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,INUS)
         CALL LCMGET(KPLISP,'NFTOT',BASE(INUS))
         DO 111 IMX=1,ILENGT
         ZSIGF(IMX,JGR)=ZSIGF(IMX,JGR)-BASE(INUS-1+IMX)
         IF(ZSIGF(IMX,JGR).NE.0.0) LFI = .TRUE.
 111     CONTINUE
         CALL RLSARA(BASE(INUS))
      ENDIF
*
      CALL LCMLEN(KPLISP,'CHI',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,ICHI)
         CALL LCMGET(KPLISP,'CHI',BASE(ICHI))
         DO 120 IMX=1,ILENGT
         CHI(IMX,JGR)=CHI(IMX,JGR)-BASE(ICHI-1+IMX)
         IF(CHI(IMX,JGR).NE.0.0) LCH = .TRUE.
 120     CONTINUE
         CALL RLSARA(BASE(ICHI))
      ENDIF
*
      DO 990 I=1,NDG
      WRITE (TEXT,'(4HCHDL,I3.3)') I
      CALL LCMLEN(KPLISP,TEXT,ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,ICHD)
         CALL LCMGET(KPLISP,TEXT,BASE(ICHD))
         DO 220 IMX=1,ILENGT
         CHDL(IMX,I,JGR)=CHDL(IMX,I,JGR)-BASE(ICHD-1+IMX)
         IF(CHDL(IMX,I,JGR).NE.0.0) LBI = .TRUE.
 220     CONTINUE
         CALL RLSARA(BASE(ICHD))
      ENDIF
 990  CONTINUE
*
      CALL LCMLEN(KPLISP,'OVERV',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,IOVR)
         CALL LCMGET(KPLISP,'OVERV',BASE(IOVR))
         DO 130 IMX=1,ILENGT
         OVERV(IMX,JGR)=OVERV(IMX,JGR)-BASE(IOVR-1+IMX)
         IF(OVERV(IMX,JGR).NE.0.0) LOV = .TRUE.
 130     CONTINUE
      CALL RLSARA(BASE(IOVR))
      ENDIF
*
      IF(DDIF) THEN
         CALL LCMLEN(KPLISP,'DIFF',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            ILEAKP=1
            CALL SETARA(BASE,ILENGT,IDFX)
            CALL LCMGET(KPLISP,'DIFF',BASE(IDFX))
            DO 140 IMX=1,ILENGT
            DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)-1./BASE(IDFX-1+IMX)
            IF(DIFFX(IMX,JGR).NE.0.0) DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)
 140        CONTINUE
            CALL RLSARA(BASE(IDFX))
         ENDIF
*
         CALL LCMLEN(KPLISP,'DIFFX',ILENGT,ITYLCM)
         IF(ILENGT.EQ.NMIXT) THEN
            ILEAKP=2
            CALL SETARA(BASE,ILENGT,IDFX)
            CALL SETARA(BASE,ILENGT,IDFY)
            CALL SETARA(BASE,ILENGT,IDFZ)
            CALL LCMGET(KPLISP,'DIFFX',BASE(IDFX))
            CALL LCMGET(KPLISP,'DIFFY',BASE(IDFY))
            CALL LCMGET(KPLISP,'DIFFZ',BASE(IDFZ))
            DO 150 IMX=1,ILENGT
            DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)-1./BASE(IDFX-1+IMX)
            DIFFY(IMX,JGR)=1./DIFFY(IMX,JGR)-1./BASE(IDFY-1+IMX)
            DIFFZ(IMX,JGR)=1./DIFFZ(IMX,JGR)-1./BASE(IDFZ-1+IMX)
            IF(DIFFX(IMX,JGR).NE.0.0) DIFFX(IMX,JGR)=1./DIFFX(IMX,JGR)
            IF(DIFFY(IMX,JGR).NE.0.0) DIFFY(IMX,JGR)=1./DIFFY(IMX,JGR)
            IF(DIFFZ(IMX,JGR).NE.0.0) DIFFZ(IMX,JGR)=1./DIFFZ(IMX,JGR)
 150        CONTINUE
            CALL RLSARA(BASE(IDFZ))
            CALL RLSARA(BASE(IDFY))
            CALL RLSARA(BASE(IDFX))
         ENDIF
      ENDIF
*
      CALL LCMLEN(KPLISP,'H-FACTOR',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,IH)
         CALL LCMGET(KPLISP,'H-FACTOR',BASE(IH))
         DO 270 IMX=1,ILENGT
         H(IMX,JGR)=H(IMX,JGR)-BASE(IH-1+IMX)
         IF(H(IMX,JGR).NE.0.0) LHF = .TRUE.
 270     CONTINUE
      CALL RLSARA(BASE(IH))
      ENDIF
*
      CALL LCMLEN(KPLISP,'FIXE',ILENGT,ITYLCM)
      IF(ILENGT.EQ.NMIXT) THEN
         CALL SETARA(BASE,ILENGT,IS)
         CALL LCMGET(KPLISP,'FIXE',BASE(IS))
         DO 280 IMX=1,ILENGT
         S(IMX,JGR)=S(IMX,JGR)-BASE(IS-1+IMX)
         IF(S(IMX,JGR).NE.0.0) LSO = .TRUE.
 280     CONTINUE
      CALL RLSARA(BASE(IS))
      ENDIF
*
      DO 930 IL=1,NL
      WRITE (CM,'(I2.2)') IL-1
      CALL LCMLEN(KPLISP,'SCAT'//CM,ILENGT,ITYLCM)
      IF(ILENGT.GT.NMIXT*NL*NGRP*NGRP) THEN
         CALL XABORT('MACXSI: INVALID INPUT MACROLIB(2).')
      ELSE IF(ILENGT.GT.0) THEN
         CALL LCMGET(KPLISP,'SCAT'//CM,WORK)
         CALL LCMGET(KPLISP,'NJJS'//CM,NJJP(1,IL,JGR))
         CALL LCMGET(KPLISP,'IJJS'//CM,IJJP(1,IL,JGR))
         IPOSDE=0
         DO 920 IBM=1,NMIXT
         IJJ0=IJJP(IBM,IL,JGR)
         IJ=IJJ0-IJJ(IBM,IL,JGR)
         IN=NJJP(IBM,IL,JGR)-NJJ(1,IL,JGR)
         IF(IJ.EQ.0.AND.IN.EQ.0) THEN
            DO 820 IGR=IJJ0,IJJ0-NJJP(IBM,IL,JGR)+1,-1
            IPOSDE=IPOSDE+1
            SCAT(IBM,IL,IGR,JGR)=SCAT(IBM,IL,IGR,JGR) - WORK(IPOSDE)
            IF(SCAT(IBM,IL,IGR,JGR).NE.0.0) LSC = .TRUE.
  820       CONTINUE
         ENDIF
  920    CONTINUE
      ENDIF
  930 CONTINUE
  940 CONTINUE
*
* PUT DIFFERENCES CROSS SECTIONS IN IPLISD
*
      IF(LEVEL.NE.' ') CALL LCMSIX(IPLISD,LEVEL,1)
      JPLISD=LCMLID(IPLISD,'GROUP',NGRP)
      DO 190 JGR=1,NGRP
        KPLISD=LCMDIL(JPLISD,JGR)
        IF(LTO) CALL LCMPUT(KPLISD,'NTOT0',NMIXT,2,TOTAL(1,JGR))
        IF(LNFI) CALL LCMPUT(KPLISD,'NUSIGF',NMIXT,2,ZNUG(1,JGR))
        IF(LFI) CALL LCMPUT(KPLISD,'NFTOT',NMIXT,2,ZSIGF(1,JGR))
        IF(LCH) CALL LCMPUT(KPLISD,'CHI',NMIXT,2,CHI(1,JGR))
        IF(LOV) CALL LCMPUT(KPLISD,'OVERV',NMIXT,2,OVERV(1,JGR))
        IF((ILEAK.EQ.1).AND.(ILEAKP.EQ.1)) THEN
           CALL LCMPUT(KPLISD,'DIFF',NMIXT,2,DIFFX(1,JGR))
        ELSE IF((ILEAK.EQ.2).AND.(ILEAKP.EQ.2)) THEN
           CALL LCMPUT(KPLISD,'DIFFX',NMIXT,2,DIFFX(1,JGR))
           CALL LCMPUT(KPLISD,'DIFFY',NMIXT,2,DIFFY(1,JGR))
           CALL LCMPUT(KPLISD,'DIFFZ',NMIXT,2,DIFFZ(1,JGR))
        ENDIF
        IF(LHF) CALL LCMPUT(KPLISD,'H-FACTOR',NMIXT,2,H(1,JGR))
        IF(LSO) CALL LCMPUT(KPLISD,'FIXE',NMIXT,2,S(1,JGR))
        IF(LBI) THEN
          DO 901 I=1,NDG
            WRITE (TEXT,'(4HCHDL,I3.3)') I
            CALL LCMPUT(KPLISD,TEXT,NMIXT,ITYLCM,CHDL(1,I,JGR))
 901      CONTINUE
        ENDIF
        IF(LSC) THEN
         DO 180 IL=1,NL
           WRITE (CM,'(I2.2)') IL-1
           IPOSDE=0
           DO 170 IBM=1,NMIXT
            IPOS(IBM)=IPOSDE+1
            DO 171 IGR=IJJ(IBM,IL,JGR),
     +        IJJ(IBM,IL,JGR)-NJJ(IBM,IL,JGR)+1,-1
             IPOSDE=IPOSDE+1
             WORK(IPOSDE)=SCAT(IBM,IL,IGR,JGR)
 171        CONTINUE
 170       CONTINUE
           CALL LCMPUT(KPLISD,'SCAT'//CM,IPOSDE,2,WORK)
           CALL LCMPUT(KPLISD,'IPOS'//CM,NMIXT,1,IPOS)
           CALL LCMPUT(KPLISD,'NJJS'//CM,NMIXT,1,NJJ(1,IL,JGR))
           CALL LCMPUT(KPLISD,'IJJS'//CM,NMIXT,1,IJJ(1,IL,JGR))
           CALL LCMPUT(KPLISD,'SIGW'//CM,NMIXT,2,SCAT(1,IL,JGR,JGR))
 180     CONTINUE
        ENDIF
        IF( IPRT.GT.3 )CALL LCMLIB(KPLISD)
 190  CONTINUE
      RETURN
      END
