*DECK GPTGRD
      SUBROUTINE GPTGRD(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this module computes the gradients using GPT theory.
* GPTGRD = Generalized Perturbation Theory GRaDient calculation
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of lcm objects used by the module
* HENTRY  character*12 name of each lcm object
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the lcm object is created
*         =1 the lcm object is open for modifications;
*         =2 the lcm object is open in read-only mode
* KENTRY  =file unit number; =lcm object address otherwise
*         kentry(nentry)
* LCM OBJECTS:
* HENTRY(1)  create type (l_macrolib)
* HENTRY(2)  read only type (l_macrolib)
* HENTRY(3)  read only type (l_macrolib)
* or
* HENTRY(1)  create type (l_flux)
* HENTRY(2)  read only type (l_flux)
* HENTRY(3)  read only type (l_flux)
*
*-----------------------------------------------------------------------
*
      IMPLICIT  NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
* General variable
      INTEGER   IPRINT,NITMA,I,J,IPMAP,IPOPT,IPFLU,IPGPT,JPGPT,KPGPT,
     1          ITYP,NGRP,MEMADR(20),IPMCR,IPTRK,IPFLUP,JPFLUP,KPFLUP,
     2          LENGT,LCMGID,LCMGIL
      DOUBLE PRECISION DFLOTT,GRAD
      REAL      FLOTT
      CHARACTER TEXT12*12
      INTEGER   NVARMX,VRLCMX,CSTMX,NLVL1,NLVL2,SIGNA(3),NBTDXS
      PARAMETER (NVARMX=100,VRLCMX=50,CSTMX=500)
      PARAMETER (NBTDXS=17,NLVL1=21,NLVL2=5)
* STATE-VECTOR VARIABLES
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   FLUPRM(NSTATE),TRKPRM(NSTATE),OPTPRI(NSTATE)
      REAL      OPTPRR(NSTATE),EPS,DX

      INTEGER   NEL,NUN,LL4,IFLAG,ISOUR,IEASS,IGRAD,IVARVL

      CHARACTER TXTPDF*12

      INTEGER   IGR,NVDXS(NBTDXS)
      LOGICAL   LIMP,LTST,LFLU

*'Optimization variables' variable
      INTEGER   NVAR,NCST
*'Local arrays' variable
      INTEGER   IVRLC(VRLCMX)
      CHARACTER TVRLC(VRLCMX)*12
      LOGICAL   LVRLC(VRLCMX)
*Objective and constraint evaluation variable
      REAL      RTFCN(NLVL1,1+2*NLVL2),FVAL,FVAL2(1)
      INTEGER   VTFCN(NLVL1,1+2*NLVL2),STFCN(NLVL1,1+2*NLVL2)
      CHARACTER OTFCN(NLVL1,1+2*NLVL2)*12
      INTEGER   FCTFRM,FCTTO,VARFRM,VARTO,IVAR,IFCN

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
C
C---------------------------
C     VALIDITY OF OBJECTS
C---------------------------
      LTST=.TRUE.
      IF (NENTRY.EQ.1) LTST=.FALSE.
*CHECK IF OBJECTS ARE LCM OBJECTS AND WHICH ONE CORRESPOND
*TO WHICH TYPE
      IPMAP=0
      IPFLU=0
      IPOPT=0
      IPGPT=0
      IPMCR=0
      IPTRK=0
*INITIALIZE THE FILE NUMBER UNIT FOR SUBROUTINE OPTEVL.f
      CALL XDISET(MEMADR,20,0)
      LFLU=.FALSE.
      DO 2 I=1,NENTRY
         IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2))CALL XABORT('GPTGRD:'
     1      //' LCM OBJECT EXPECTED')
         CALL LCMGET(KENTRY(I),'SIGNATURE',SIGNA)
         WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
         IF (TEXT12.EQ.'L_MAP') THEN
             IPMAP=KENTRY(I)
             MEMADR(2)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_FLUX') THEN
             IF (MEMADR(3).EQ.0) THEN
                IPFLU=KENTRY(I)
                MEMADR(3)=KENTRY(I)
             ELSE
                MEMADR(13)=KENTRY(I)
                IPFLUP=KENTRY(I)
                LFLU=.TRUE.
             ENDIF
         ELSEIF (TEXT12.EQ.'L_OPTIMIZE') THEN
             IPOPT=KENTRY(I)
             MEMADR(6)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_GPT') THEN
             IPGPT=KENTRY(I)
             MEMADR(5)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_MACROLIB') THEN
            IPMCR=KENTRY(I)
            IF (MEMADR(7).EQ.0) THEN
               MEMADR(7)=KENTRY(I)
            ELSE
               MEMADR(17)=KENTRY(I)
            ENDIF
         ELSEIF (TEXT12.EQ.'L_MATEX') THEN
             MEMADR(8)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_SYSTEM') THEN
            IF (MEMADR(4).EQ.0) THEN
               MEMADR(4)=KENTRY(I)
            ELSEIF (MEMADR(14).EQ.0) THEN
               MEMADR(14)=KENTRY(I)
            ELSEIF (MEMADR(11).EQ.0) THEN
               MEMADR(11)=KENTRY(I)
            ELSE
               MEMADR(19)=KENTRY(I)
            ENDIF
         ELSEIF (TEXT12.EQ.'L_TRACK') THEN
             IPTRK=KENTRY(I)
             MEMADR(9)=KENTRY(I)
         ELSE
             CALL XABORT('GPTGRD: NOT GOOD TYPE OF OBJECT')
         ENDIF
    2    CONTINUE
C
C---------------------------
C VARIABLE INITIALISATION	
C---------------------------
      IPRINT=  0
*initialisation des vecteurs locaux
      DO 3 I=1,VRLCMX
         IVRLC(I)=1
         TVRLC(I)=' '
    3    LVRLC(I)=.FALSE.
*aller chercher dans FLU les info necessaires, si applicable
      IF (IPFLU.GT.0) THEN
         CALL LCMGET(IPFLU,'STATE-VECTOR',FLUPRM)
*         NGRP   = FLUPRM(1)
         NUN    = FLUPRM(2)
      ENDIF
*aller chercher dans OPTIMIZE les info necessaires
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      NVAR   = OPTPRI(1)
      NCST   = OPTPRI(2)
      NGRP   = OPTPRI(20)
      CALL SETARA(RBASE,NVAR,IVARVL)
      CALL LCMGET(IPOPT,'VAR-VALUE',RBASE(IVARVL))
*aller chercher dans TRACK les info necessaires, si applicable
      IF (IPTRK.GT.0) THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',TRKPRM)
         NEL    = TRKPRM(1)
         NUN    = TRKPRM(2)
         LL4    = TRKPRM(11)
      ENDIF
* charger les gradiants deja calcules ou les initialiser
      CALL SETARA(RBASE,NVAR*(NCST+1),IGRAD)
      CALL LCMLEN(IPOPT,'GRADIENT',NITMA,ITYP)
      IF (NITMA.EQ.0) THEN
         CALL XDRSET(RBASE(IGRAD),NVAR*(NCST+1),0.0)
      ELSEIF (NVAR*(NCST+1).NE.NITMA) THEN
         CALL XABORT('GPTGRD: NUMBERS OF GRADIENTS DO NOT MATCH')
      ELSE
         CALL LCMGET(IPOPT,'GRADIENT',RBASE(IGRAD))
      ENDIF
      IF (NVAR.GT.NCST+1) THEN
         LIMP=.TRUE.
      ELSE
         LIMP=.FALSE.
      ENDIF
*The calculated gradient part is added to the stored value.
*  If it is a new iteration, gradient should have been initialized to
*  0.0 in the QLP: module. If not, and the stored value is different of
*  0.0, please use the NEW-VALUE keyword.
      CALL SETARA(RBASE,NVAR*(NCST+1),IFLAG)
      CALL XDRSET(RBASE(IFLAG),NVAR*(NCST+1),1.0)
      EPS=0.0
C
C---------------------------
C     READ USER INPUT:
C---------------------------
   20 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
   30 IF(ITYP.NE.3) CALL XABORT('GPTGRD: CHARACTER DATA EXPECTED.')
* Edition level
      IF (TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTGRD: *IPRINT* MUST BE INTEGER')
* Value of gradient independent of the chosen perturbation
      ELSEIF (TEXT12.EQ.'REL')THEN
         CALL LCMGET(IPOPT,'OPT-PARAM-R',OPTPRR)
         EPS    = OPTPRR(7)
* Value of gradients are new. Direct or Indirect part not yet calculated
      ELSEIF (TEXT12.EQ.'NEW-VALUE')THEN
         CALL XDRSET(RBASE(IFLAG),NVAR*(NCST+1),0.0)
* Value of gradient given manually
      ELSEIF (TEXT12.EQ.'DIRECT-VALUE')THEN
         GO TO 100
* Value of gradient given by GPT theory
      ELSEIF (TEXT12.EQ.'GPT')THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.3) CALL XABORT('GPTGRD: CHARACTER DATA EXPECTED.')
         IF (TEXT12.EQ.'DIRECT') THEN
            GO TO 200
         ELSEIF (TEXT12.EQ.'INDIRECT') THEN
            GO TO 300
         ELSE
            CALL XABORT('GPTGRD: "DIRECT" or "INDIRECT" EXPECTED')
         ENDIF
* End of this subroutine
      ELSEIF( TEXT12.EQ.';' )THEN
         GO TO 1000
      ELSE
         CALL XABORT('GPTGRD: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
C
C---------------------------
C     1      0000     0000
C    11     0    0   0    0
C   1 1    0      0 0      0
C  1  1    0      0 0      0
C 1   1    0      0 0      0
C     1    0      0 0      0
C     1     0    0   0    0
C 11111111   0000     0000
C---------------------------
C GRADIENT CALCULATION
C---------------------------
  100 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.EQ.3) GO TO 30
      IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER DATA EXPECTED.')
      VARFRM=NITMA
      IF ((VARFRM.LE.0).OR.(VARFRM.GT.NVAR)) CALL XABORT('GPTGRD: '
     1  //'INVALID VARIABLE NUMBER')
  107 CALL REDGET(ITYP,VARTO,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.EQ.1) THEN
         IF ((VARFRM.LE.0).OR.(VARFRM.GT.NVAR)) CALL XABORT('GPTGRD: '
     1  //'INVALID VARIABLE NUMBER')
         IF (VARFRM.GT.VARTO) CALL XABORT('GPTGRD: VARFRM > VARTO')
          CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      ELSE
          VARTO=VARFRM
      ENDIF
      IF (ITYP.NE.3) CALL XABORT('GPTGRD: CHARACTER DATA EXPECTED.')
      IF (TEXT12.EQ.'FOBJ') THEN
         FCTFRM=0
         FCTTO=0
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      ELSEIF (TEXT12.EQ.'CONSTRAINT') THEN
         CALL REDGET(ITYP,FCTFRM,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER DATA EXPECTED.')
         IF ((FCTFRM.LT.0).OR.(FCTFRM.GT.NCST)) CALL XABORT('GPTGRD:'
     1  //' INVALID FUNCTION NUMBER')
         CALL REDGET(ITYP,FCTTO,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.EQ.1) THEN
            CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         ELSEIF (ITYP.EQ.2) THEN
            FCTTO=FCTFRM
         ELSE
            CALL XABORT('GPTGRD: CHARACTER OR REAL EXPECTED.')
         ENDIF
      ELSE
         CALL XABORT('GPTGRD: FOBJ or CONSTRAINT EXPECTED.')
      ENDIF
      DO 140 IVAR=VARFRM,VARTO
* DG/DX
         IF (EPS.NE.0.0) THEN
            DX=RBASE(IVARVL+IVAR-1)*EPS
* DG
         ELSE
            DX=1.0
         ENDIF
         DO 120 IFCN=FCTFRM,FCTTO
            IF(ITYP.NE.2) CALL XABORT('GPTGRD: REAL DATA EXPECTED.')
            RBASE(IGRAD+IVAR-1+IFCN*NVAR)=FLOTT/DX
  120       CALL REDGET(ITYP,FCTTO,FLOTT,TEXT12,DFLOTT)
  140    CONTINUE
      GO TO 30
C---------------------------
C   2222     0000     0000
C  2    2   0    0   0    0
C 2      2 0      0 0      0
C       2  0      0 0      0
C      2   0      0 0      0
C   22     0      0 0      0
C  2        0    0   0    0
C 22222222   0000     0000
C---------------------------
C GPT CALCULATION DIRECT PART
C---------------------------
  200 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.EQ.1) THEN
         VARFRM=NITMA
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER EXPECTED '
     1  //'FOR VARTO')
         VARTO=NITMA
         IF((VARFRM.GT.VARTO).OR.(VARFRM.LT.1).OR.(VARTO.GT.NVAR))
     1  CALL XABORT('GPTGRD: VARFRM and VARTO DO NOT FIT')
      ELSEIF (TEXT12.EQ.'ALL') THEN
         VARFRM=1
         VARTO =NVAR
      ELSE
         CALL XABORT('GPTGRD: VARFRM and VARTO DEFINITION IS WRONG')
      ENDIF
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('GPTGRD: CHARACTER EXPECTED')
      IF (TEXT12.EQ.'FOBJ') THEN
         FCTFRM=0
         FCTTO=0
      ELSEIF (TEXT12.EQ.'CONSTRAINT') THEN
         CALL REDGET(ITYP,FCTFRM,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER EXPECTED')
         CALL REDGET(ITYP,FCTTO,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER EXPECTED')
         IF (FCTTO.LT.FCTFRM) CALL XABORT('GPTGRD: FROM GREATER '
     1  //'THAN TO')
         IF ((FCTFRM.GT.NCST).OR.(FCTTO.GT.NCST)) CALL XABORT
     1  ('GPTGRD: TOTAL NUMBER OF CONSTRAINT IS LESS THAN FCTTO')
      ELSE
         CALL XABORT('GPTGRD: WRONG TYPE OF FUNCTION SPECIFIED')
      ENDIF
*definition d'autres variables si applicable - DEBUT
      TXTPDF=' '
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (TEXT12.EQ.'VARDEF') THEN
         TXTPDF='FUNCTION'
      ELSEIF (TEXT12.EQ.'FUNCT-PREDEF') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TXTPDF,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('PERTUR: CHARACTER EXPECTED')
      ENDIF
      CALL OPTVDF(MEMADR,IVRLC,TVRLC,LVRLC,VRLCMX,NGRP,TXTPDF,TEXT12,
     1            NENTRY,KENTRY,HENTRY)
      IF (TXTPDF.EQ.' ') TXTPDF='FUNCTION'
*definition de la sequence de calcul
      CALL OPTSEQ(MEMADR,STFCN,VTFCN,RTFCN,OTFCN,NLVL1,NLVL2,IVRLC,
     1            TVRLC,VRLCMX,NGRP,TXTPDF)
*note: dans le cas d'une evaluation de fontionnelle LGDATA=1 obligatoire
C      CALL SETARA(RBASE,LGDATA,IDATA) -> FVAL
*mettre a jour les vecteurs predefinis qui dependent des variables
*    PART 1:  quels sont les numeros de variables a mettre a jour
      CALL OPTVUP(IVRLC,TVRLC,LVRLC,MEMADR,0,VTFCN,VRLCMX,NLVL1,NLVL2,
     1            NVDXS,NGRP,1)
      DO 230 IVAR=VARFRM,VARTO
* DG/DX
         IF (EPS.NE.0.0) THEN
            DX=RBASE(IVARVL+IVAR-1)*EPS
* DG
         ELSE
            DX=1.0
         ENDIF
         STFCN(1,1)=IVAR
*    PART 2:  mettre a jour les variables avec numeros valides
         CALL OPTVUP(IVRLC,TVRLC,LVRLC,MEMADR,IVAR,VTFCN,VRLCMX,NLVL1,
     1               NLVL2,NVDXS,NGRP,2)
         DO 230 IFCN=FCTFRM,FCTTO
            VTFCN(1,1)=IFCN
            CALL OPTSUP(MEMADR,STFCN,VTFCN,RTFCN,NLVL1,NLVL2,
     1                  IVRLC,VRLCMX,IFCN)
            CALL OPTEVL(MEMADR,VRLCMX,IVRLC,TVRLC,LVRLC,NLVL1,
     1               NLVL2,STFCN,VTFCN,RTFCN,OTFCN,FVAL2,1,TXTPDF)
            FVAL=FVAL2(1)
            RBASE(IGRAD+IVAR-1+IFCN*NVAR)=FVAL/DX+
     1       RBASE(IGRAD+IVAR-1+IFCN*NVAR)*RBASE(IFLAG+IVAR-1+IFCN*NVAR)
            RBASE(IFLAG+IVAR-1+IFCN*NVAR)=1.0
  230    CONTINUE
      GO TO 20
C---------------------------
C   3333     0000     0000
C  3    3   0    0   0    0
C 3      3 0      0 0      0
C      33  0      0 0      0
C       3  0      0 0      0
C 3      3 0      0 0      0
C  3    3   0    0   0    0
C   3333     0000     0000
C---------------------------
C    GPT INDIRECT PART
C---------------------------
  300 VARFRM=0
      CALL SETARA(RBASE,NUN*NGRP,ISOUR)
      CALL SETARA(RBASE,NUN*NGRP,IEASS)
  310 CALL REDGET(ITYP,VARFRM,FLOTT,TEXT12,DFLOTT)
      IF((ITYP.NE.3).AND.(ITYP.NE.1)) CALL XABORT('GPTGRD: CHARACTER '
     1  //'OR DATA EXPECTED')
      IF (TEXT12.EQ.'EXPLICIT') THEN
          LIMP=.FALSE.
          GO TO 310
      ELSEIF (TEXT12.EQ.'IMPLICIT') THEN
          LIMP=.TRUE.
          GO TO 310
      ELSEIF (ITYP.EQ.1) THEN
         CALL REDGET(ITYP,VARTO,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER EXPECTED '
     1  //'FOR VARTO')
         IF ((VARFRM.GT.VARTO).OR.(VARFRM.LT.1).OR.(VARTO.GT.NVAR))
     1  CALL XABORT('GPTGRD: WRONG VARFRM and VARTO')
      ELSEIF (TEXT12.EQ.'ALL') THEN
         VARFRM=1
         VARTO =NVAR
      ELSE
         CALL XABORT('GPTGRD: VARFRM and VARTO DEFINITION IS WRONG')
      ENDIF
      IF (VARFRM.EQ.0) CALL XABORT('GPTGRD: WRONG DEFINITION OF VARFRM')
      IF (IPRINT.GE.1) WRITE(6,*) 'Var: from ',VARFRM,' to ',VARTO
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('GPTGRD: CHARACTER EXPECTED')
      IF (TEXT12.EQ.'FOBJ') THEN
         FCTFRM=0
         FCTTO=0
      ELSEIF (TEXT12.EQ.'CONSTRAINT') THEN
         CALL REDGET(ITYP,FCTFRM,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER EXPECTED')
         CALL REDGET(ITYP,FCTTO,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTGRD: INTEGER EXPECTED')
         IF (FCTTO.LT.FCTFRM) CALL XABORT('GPTGRD: FROM > TO')
         IF ((FCTFRM.GT.NCST).OR.(FCTTO.GT.NCST)) CALL XABORT
     1  ('GPTGRD: TOTAL NUMBER OF CONSTRAINT IS LESS THAN FCTTO')
      ELSE
         CALL XABORT('GPTGRD: WRONG TYPE OF FUNCTION')
      ENDIF
      IF (IPRINT.GE.1) WRITE(6,*) 'Function: from ',FCTFRM,' to ',FCTTO
C IMPLICIT METHOD
      IF (LIMP) THEN
      JPGPT=LCMGID(IPGPT,'DSOUR')
      JPFLUP=LCMGID(IPFLUP,'ADFLUX')
      DO 389 IVAR=VARFRM,VARTO
* DG/DX
         IF (EPS.NE.0.0) THEN
            DX=RBASE(IVARVL+IVAR-1)*EPS
* DG
         ELSE
            DX=1.0
         ENDIF
* charger la source explicite
         KPGPT=LCMGIL(JPGPT,IVAR)
         DO 381 IGR=1,NGRP
            CALL LCMLEL(KPGPT,IGR,LENGT,ITYP)
            IF (LENGT.EQ.0) THEN
              WRITE(TEXT12,'(I5,3H-TH,4X)') IGR
              CALL XABORT('GPTGRD: THIS '//TEXT12//' GROUP DIRECTORY D'
     1        //'OES NOT EXIST(1)')
            ENDIF
            CALL LCMGDL(KPGPT,IGR,RBASE(ISOUR+(IGR-1)*NUN))
  381    CONTINUE
         IF(FCTTO.GT.0) THEN
         DO 388 IFCN=FCTFRM,FCTTO
            GRAD=0.0D0
* charger l'adjoint generalise implicite
            KPFLUP=LCMGIL(JPFLUP,IFCN)
            DO 382 IGR=1,NGRP
               CALL LCMLEL(KPFLUP,IGR,LENGT,ITYP)
               IF (LENGT.EQ.0) THEN
                 WRITE(TEXT12,'(I5,3H-TH,4X)') IGR
                 CALL XABORT('GPTGRD: THIS '//TEXT12//' GROUP DIRECTOR'
     1           //'Y DOES NOT EXIST(2)')
               ENDIF
               CALL LCMGDL(KPFLUP,TEXT12,RBASE(IEASS+(IGR-1)*NUN))
  382       CONTINUE
* multiplier adjoint generalise par source
            DO 383 I=0,NUN*NGRP-1
  383       GRAD=GRAD+RBASE(IEASS+I)*RBASE(ISOUR+I)
* sauvegarder le resultat ou l'ajouter si partie directe deja calculee
            RBASE(IGRAD+IVAR-1+IFCN*NVAR)=GRAD/DX+
     1       RBASE(IGRAD+IVAR-1+IFCN*NVAR)*RBASE(IFLAG+IVAR-1+IFCN*NVAR)
            RBASE(IFLAG+IVAR-1+IFCN*NVAR)=1.0
  388    CONTINUE
         ENDIF
  389    CONTINUE
C EXPLICIT METHOD
      ELSE
      JPGPT=LCMGID(IPGPT,'ASOUR')
      JPFLUP=LCMGID(IPFLUP,'DFLUX')
      IF(FCTTO.GT.0) THEN
      DO 399 IFCN=FCTFRM,FCTTO
* charger la source adjointe
         KPGPT=LCMGIL(JPGPT,IFCN)
         DO 391 IGR=1,NGRP
            CALL LCMLEL(KPGPT,IGR,LENGT,ITYP)
            IF (LENGT.EQ.0) THEN
              WRITE(TEXT12,'(I5,3H-TH,4X)') IGR
              CALL XABORT('GPTGRD: THIS '//TEXT12//' GROUP DIRECTORY D'
     1        //'OES NOT EXIST(3)')
            ENDIF
            CALL LCMGDL(KPGPT,IGR,RBASE(ISOUR+(IGR-1)*NUN))
  391    CONTINUE
         IF(VARTO.GT.0) THEN
         DO 398 IVAR=VARFRM,VARTO
            GRAD=0.0D0
* DG/DX
            IF (EPS.NE.0.0) THEN
               DX=RBASE(IVARVL+IVAR-1)*EPS
* DG
            ELSE
               DX=1.0
            ENDIF
* charger l'adjoint generalise explicite
            KPFLUP=LCMGIL(JPFLUP,IVAR)
            DO 392 IGR=1,NGRP
               CALL LCMLEL(KPFLUP,IGR,LENGT,ITYP)
               IF (LENGT.EQ.0) THEN
                 WRITE(TEXT12,'(I5,3H-TH,4X)') IGR
                 CALL XABORT('GPTGRD: THIS '//TEXT12//' GROUP DIRECTOR'
     1           //'Y DOES NOT EXIST(4)')
               ENDIF
               CALL LCMGDL(KPFLUP,IGR,RBASE(IEASS+(IGR-1)*NUN))
  392       CONTINUE
* multiplier adjoint generalise par source
            DO 393 I=0,NUN*NGRP-1
  393       GRAD=GRAD+RBASE(IEASS+I)*RBASE(ISOUR+I)
* sauvegarder le resultat ou l'ajouter si partie directe deja calculee
            RBASE(IGRAD+IVAR-1+IFCN*NVAR)=GRAD/DX+
     1       RBASE(IGRAD+IVAR-1+IFCN*NVAR)*RBASE(IFLAG+IVAR-1+IFCN*NVAR)
            RBASE(IFLAG+IVAR-1+IFCN*NVAR)=1.0
  398    CONTINUE
         ENDIF
  399    CONTINUE
         ENDIF
      ENDIF
      CALL RLSARA(RBASE(ISOUR))
      CALL RLSARA(RBASE(IEASS))
      GO TO 20
C---------------------------
C END
C---------------------------
*liberer l'espace utilise par les vecteurs locaux
 1000 CONTINUE
      DO 1010 I=1,VRLCMX
         IF (LVRLC(I)) THEN
            CALL RLSARA(RBASE(IVRLC(I)))
         ENDIF
 1010 CONTINUE
      CALL LCMPUT(IPOPT,'GRADIENT',NVAR*(NCST+1),2,RBASE(IGRAD))
      CALL RLSARA(RBASE(IFLAG))
      CALL RLSARA(RBASE(IGRAD))
      CALL RLSARA(RBASE(IVARVL))
      RETURN
      END
