*DECK GPTSRC
      SUBROUTINE GPTSRC(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this module computes the source terms for GPT theory.
* GPTSRC = Generalized Perturbation Theory SouRCes calculation
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of lcm objects used by the module
* HENTRY  character*12 name of each lcm object
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the lcm object is created
*         =1 the lcm object is open for modifications;
*         =2 the lcm object is open in read-only mode
* KENTRY  =file unit number; =lcm object address otherwise
*         kentry(nentry)
* LCM OBJECTS:
* HENTRY(1)  create type (l_macrolib)
* HENTRY(2)  read only type (l_macrolib)
* HENTRY(3)  read only type (l_macrolib)
* or
* HENTRY(1)  create type (l_flux)
* HENTRY(2)  read only type (l_flux)
* HENTRY(3)  read only type (l_flux)
*                          
*-----------------------------------------------------------------------
*
      IMPLICIT  NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
* General variable
      CHARACTER HENTRY2(NENTRY)*12
      INTEGER   KENTRY2(NENTRY)
      
      INTEGER   IPRINT,NITMA,IDATA,I,J,IPMAP,IPOPT,IPFLU,IPGPT,JPGPT,
     1          KPGPT,IPMCR,MEMADR(20),ITYP,NGRP,NUN,NVAR,NCST,SIGNA(3),
     2          IPIND,LCMLID,LCMLIL
      DOUBLE PRECISION DFLOTT
      REAL      FLOTT
      CHARACTER TEXT12*12,TXT12B*12
      INTEGER   NVARMX,VRLCMX,CSTMX,NLVL1,NLVL2
      PARAMETER (NVARMX=100,VRLCMX=50,CSTMX=500)
      PARAMETER (NLVL1=21,NLVL2=5)
* STATE-VECTOR VARIABLES
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   OPTPRI(NSTATE),FLUPRM(NSTATE),GPTPRM(NSTATE)
      REAL      OPTPRR(NSTATE)

      INTEGER   IGR,NVDXS(18)
*DIRECT SOURCE
      INTEGER   VARFRM,VARTO,IVAR,ISOUR,IAPPHI,IBPPHI,IAPHI,IBPHI,ILAMBD
      REAL      LAMBDA

*ADJOINT SOURCE
*'Local arrays' variable
      INTEGER   NVRLC,IVRLC(VRLCMX)
      CHARACTER TVRLC(VRLCMX)*12,TXTPDF*12
      LOGICAL   LVRLC(VRLCMX)
*Objective and constraint evaluation variable
      REAL      RTFCN(NLVL1,1+2*NLVL2)
      INTEGER   VTFCN(NLVL1,1+2*NLVL2),STFCN(NLVL1,1+2*NLVL2)
      CHARACTER OTFCN(NLVL1,1+2*NLVL2)*12
      INTEGER   LGDATA,FCTFRM,FCTTO

*OTHER SOURCE
      INTEGER   ISRC

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
C
C
C---------------------------
C     VALIDITY OF OBJECTS
C---------------------------
      IF (NENTRY.LT.2) CALL XABORT('GPTSRC: 2 OBJECTS MIN EXPECTED.')
*CHECK IF OBJECTS ARE LCM OBJECTS AND WHICH ONE CORRESPOND 
*TO WHICH TYPE
      IPMAP=0
      IPFLU=0
      IPOPT=0
      IPGPT=0
      IPMCR=0
      IPIND=0
*INITIALIZE THE FILE NUMBER UNIT FOR SUBROUTINE OPTEVL.f
      CALL XDISET(MEMADR,20,0)
      DO 2 I=1,NENTRY
         IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2))CALL XABORT('GPTSRC:'
     1      //' LCM OBJECT EXPECTED')
         IF (JENTRY(I).EQ.0) THEN
            TEXT12='L_GPT'
            READ(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
            CALL LCMPUT(KENTRY(I),'SIGNATURE',3,3,SIGNA)
            CALL XDISET(GPTPRM,40,0)
         ELSE
            CALL LCMGET(KENTRY(I),'SIGNATURE',SIGNA)
            WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
         ENDIF
         IF (TEXT12.EQ.'L_FLUX') THEN 
             IPFLU=KENTRY(I)
             MEMADR(3)=KENTRY(I)
             MEMADR(3)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_OPTIMIZE') THEN 
             IPOPT=KENTRY(I)
             MEMADR(6)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_GPT') THEN 
             IPGPT=KENTRY(I)
             MEMADR(5)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_MATEX') THEN 
             IPIND=KENTRY(I)
             MEMADR(8)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_MACROLIB') THEN 
             IPMCR=KENTRY(I)
             MEMADR(7)=KENTRY(I)
         ELSEIF (TEXT12.EQ.'L_SYSTEM') THEN 
            IF (MEMADR(4).EQ.0) THEN
               MEMADR(4)=KENTRY(I)
            ELSE
               MEMADR(14)=KENTRY(I)
            ENDIF
         ELSEIF (TEXT12.EQ.'L_TRACK') THEN 
             MEMADR(9)=KENTRY(I)
         ELSE 
             CALL XABORT('GPTSRC: NOT GOOD TYPE OF OBJECT')
         ENDIF
    2    CONTINUE
C
C---------------------------
C VARIABLE INITIALISATION	
C---------------------------
      IPRINT=  0
      NVRLC=0
*initialisation des vecteurs locaux
      DO 3 I=1,VRLCMX
         IVRLC(I)=1
         TVRLC(I)=' '
    3    LVRLC(I)=.FALSE.
*aller chercher dans OPT les info necessaires
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      NVAR   = OPTPRI(1)
      NCST   = OPTPRI(2)
      CALL LCMGET(IPOPT,'OPT-PARAM-R',OPTPRR)
*aller chercher dans FLU les info necessaires, si applicable
      CALL LCMGET(IPFLU,'STATE-VECTOR',FLUPRM)
      NGRP   = FLUPRM(1)
      NUN    = FLUPRM(2)
*initialisation des paramaetres si GPT est cree.
      IF (JENTRY(1).EQ.0) THEN
         GPTPRM(1)=NGRP
         GPTPRM(2)=NUN
         GPTPRM(3)=NVAR
         GPTPRM(4)=NCST
         CALL LCMPUT(IPGPT,'STATE-VECTOR',NSTATE,1,GPTPRM)
      ENDIF
C
C---------------------------
C     READ USER INPUT:
C---------------------------
   20 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('GPTSRC: CHARACTER EXPECTED')
* Edition level      
   30 IF (TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTSRC: *IPRINT* MUST BE INTEGER')
* Direct source terms calculation
      ELSEIF (TEXT12.EQ.'DIRECT')THEN
         GO TO 100
* ADjoint source terms calculation
      ELSEIF (TEXT12.EQ.'ADJOINT')THEN
         GO TO 200
* Other source terms calculation
      ELSEIF (TEXT12.EQ.'OTHER')THEN
         GO TO 300
* End of this subroutine
      ELSEIF( TEXT12.EQ.';' )THEN
         GO TO 1000
      ELSE
         CALL XABORT('SNDMPI: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20

C---------------------------
C     1      0000     0000    
C    11     0    0   0    0 
C   1 1    0      0 0      0 
C  1  1    0      0 0      0  
C 1   1    0      0 0      0  
C     1    0      0 0      0  
C     1     0    0   0    0  
C 11111111   0000     0000  
C---------------------------
C DIRECT SOURCE
C---------------------------
  100 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.EQ.1) THEN
         VARFRM=NITMA
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTSRC: INTEGER EXPECTED '
     1  //'FOR VARTO')
         VARTO=NITMA
         IF ((VARFRM.GT.VARTO).OR.(VARFRM.LT.1).OR.(VARTO.GT.NVAR))
     1  CALL XABORT('GPTSRC: VARFRM and VARTO DO NOT FIT')
      ELSEIF (TEXT12.EQ.'ALL') THEN
         VARFRM=1
         VARTO =NVAR
      ELSE
         CALL XABORT('GPTSRC: VARFRM and VARTO DEFINITION IS WRONG')
      ENDIF
*fonction predefinie
      TXTPDF='GPT-D-SOUR'
*definition des variables
      CALL OPTVDF(MEMADR,IVRLC,TVRLC,LVRLC,VRLCMX,NGRP,TXTPDF,TEXT12,
     1            NENTRY,KENTRY,HENTRY)
*definition de la sequence de calcul
      CALL OPTSEQ(MEMADR,STFCN,VTFCN,RTFCN,OTFCN,NLVL1,NLVL2,IVRLC,
     1            TVRLC,VRLCMX,NGRP,TXTPDF)
*definition des vecteurs MAT*PHI
      IAPHI =IVRLC(VTFCN(2,1))
      IAPPHI=IVRLC(VTFCN(3,1))
      IBPHI =IVRLC(VTFCN(4,1))
      IBPPHI=IVRLC(VTFCN(5,1))
* charger le vecteur D-LAMBDA
      CALL SETARA(RBASE,NVAR,ILAMBD)
      CALL LCMLEN(IPOPT,'D-LAMBDA',NITMA,ITYP)
      IF (NITMA.EQ.0) CALL XABORT('GPTSRC: D-LAMBDA NOT '
     1  //'YET DEFINED')
      CALL LCMGET(IPOPT,'D-LAMBDA',RBASE(ILAMBD))
      CALL LCMGET(IPFLU,'K-EFFECTIVE',LAMBDA)
      LAMBDA=1/LAMBDA
*mettre a jour les vecteurs predefinis qui dependent des variables
*    PART 1:  quels sont les numeros de variables a mettre a jour
      CALL OPTVUP(IVRLC,TVRLC,LVRLC,MEMADR,0,VTFCN,VRLCMX,NLVL1,NLVL2,
     1            NVDXS,NGRP,1)
      IF(VARTO.GT.0) THEN
         JPGPT=LCMLID(IPGPT,'DSOUR',VARTO)
         DO 190 IVAR=VARFRM,VARTO
            CALL SETARA(RBASE,NGRP*NUN,ISOUR)
            CALL OPTVUP(IVRLC,TVRLC,LVRLC,MEMADR,IVAR,VTFCN,VRLCMX,
     1                  NLVL1,NLVL2,NVDXS,NGRP,2)
            DO 110 I=0,NUN*NGRP-1
  110          RBASE(ISOUR+I)=(RBASE(IAPPHI+I)-RBASE(IAPHI+I))
     1         -(LAMBDA*(RBASE(IBPPHI+I)-RBASE(IBPHI+I))
     2         +RBASE(ILAMBD+IVAR-1)*RBASE(IBPHI+I))
            KPGPT=LCMLIL(JPGPT,IVAR,NGRP)
            DO 145 IGR=1,NGRP
               CALL LCMPDL(KPGPT,IGR,NUN,2,RBASE(ISOUR+(IGR-1)*NUN))
  145       CONTINUE
            CALL RLSARA(RBASE(ISOUR))
  190    CONTINUE
      ENDIF
      CALL RLSARA(RBASE(ILAMBD))
      GO TO 20
C---------------------------
C   2222     0000     0000    
C  2    2   0    0   0    0 
C 2      2 0      0 0      0
C       2  0      0 0      0  
C      2   0      0 0      0  
C   22     0      0 0      0  
C  2        0    0   0    0  
C 22222222   0000     0000  
C---------------------------
C ADJOINT SOURCE
C---------------------------
  200 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('GPTSRC: CHARACTER EXPECTED') 
      TXTPDF=' '
      IF (TEXT12.EQ.'FOBJ') THEN
         FCTFRM=0
         FCTTO=0
      ELSEIF (TEXT12.EQ.'CONSTRAINT') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTSRC: INTEGER EXPECTED') 
         FCTFRM=NITMA
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('GPTSRC: INTEGER EXPECTED') 
         FCTTO=NITMA
         IF (FCTTO.LT.FCTFRM) CALL XABORT('GPTSRC: FROM GREATER '
     1  //'THAN TO')
         IF ((FCTFRM.GT.NCST).OR.(FCTTO.GT.NCST)) CALL XABORT
     1  ('GPTSRC: TOTAL NUMBER OF CONSTRAINT IS LESS THAN FCTTO')
      ELSE
         GO TO 30
      ENDIF
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
*definition d'autres variables si applicable
      IF (TEXT12.EQ.'VARDEF') THEN
         TXTPDF='FUNCTION'
         CALL OPTVDF(MEMADR,IVRLC,TVRLC,LVRLC,VRLCMX,NGRP,TXTPDF,TEXT12,
     1            NENTRY,KENTRY,HENTRY)
         IF (TEXT12.NE.'INIT') CALL XABORT('FOBJCT: "INIT" KEYWORD '
     1  //'NEEDED BETWEEN VARIABLES AND SEQUENCE DEFINITIONS')
      ELSEIF (TEXT12.EQ.'FUNCT-PREDEF') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TXTPDF,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('FOBJCT: CHARACTER EXPECTED')
         CALL OPTVDF(MEMADR,IVRLC,TVRLC,LVRLC,VRLCMX,NGRP,TXTPDF,TEXT12,
     1            NENTRY,KENTRY2,HENTRY2)
      ENDIF
      IF (TXTPDF.EQ.' ') THEN
         IF (TEXT12.NE.'INIT') CALL XABORT('FOBJCT: "INIT" KEYWORD '
     1  //'NEEDED BEFORE SEQUENCE DEFINITIONS')
         TXTPDF='FUNCTION'
      ENDIF
*debut de la sequence de calcul
      CALL OPTSEQ(MEMADR,STFCN,VTFCN,RTFCN,OTFCN,NLVL1,NLVL2,IVRLC,
     1            TVRLC,VRLCMX,NGRP,TXTPDF)
*    longueur du vecteur a calculer et allocation de sa memoire
      LGDATA=NUN*NGRP
*note: dans le cas d'une evaluation de derivee par le flux LGDATA=NUN*NGRP
      IF(FCTTO.GT.0) THEN
         CALL SETARA(RBASE,LGDATA,IDATA)
         JPGPT=LCMLID(IPGPT,'ASOUR',FCTTO)
         DO 290 I=FCTFRM,FCTTO
            VTFCN(1,1)=I
            CALL OPTSUP(MEMADR,STFCN,VTFCN,RTFCN,NLVL1,NLVL2,IVRLC,
     1                  VRLCMX,I)
            CALL OPTEVL(MEMADR,VRLCMX,IVRLC,TVRLC,LVRLC,NLVL1,NLVL2,
     1                  STFCN,VTFCN,RTFCN,OTFCN,RBASE(IDATA),LGDATA,
     2                  TXTPDF)
            KPGPT=LCMLIL(JPGPT,I,NGRP)
            DO 285 IGR=1,NGRP
               CALL LCMPDL(KPGPT,IGR,NUN,2,RBASE(IDATA+(IGR-1)*NUN))
  285       CONTINUE
  290    CONTINUE
         CALL RLSARA(RBASE(IDATA))
      ENDIF
      GO TO 200
C---------------------------
C   3333     0000     0000
C  3    3   0    0   0    0
C 3      3 0      0 0      0
C      33  0      0 0      0
C       3  0      0 0      0
C 3      3 0      0 0      0
C  3    3   0    0   0    0
C   3333     0000     0000
C---------------------------
C    OTHER SOURCE
C---------------------------
  300 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('GPTSRC: CHARACTER EXPECTED') 
      IF (TEXT12.EQ.'DIRECT') THEN
         TXT12B='DSOUR'
      ELSEIF (TEXT12.EQ.'ADJOINT') THEN
         TXT12B='ASOUR'
      ELSE
         CALL XABORT('GPTSRC: "DIRECT" or "ADJOINT" KEYWORD EXPECTED')
      ENDIF
      CALL REDGET(ITYP,ISRC,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.1) CALL XABORT('GPTSRC: INTEGER EXPECTED (3)')
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
*definition d'autres variables si applicable
      IF (TEXT12.EQ.'VARDEF') THEN
         TXTPDF='FUNCTION'
         CALL OPTVDF(MEMADR,IVRLC,TVRLC,LVRLC,VRLCMX,NGRP,TXTPDF,TEXT12,
     1            NENTRY,KENTRY,HENTRY)
         IF (TEXT12.NE.'INIT') CALL XABORT('FOBJCT: "INIT" KEYWORD '
     1  //'NEEDED BETWEEN VARIABLES AND SEQUENCE DEFINITIONS')
      ELSEIF (TEXT12.EQ.'FUNCT-PREDEF') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TXTPDF,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('FOBJCT: CHARACTER EXPECTED')
         CALL OPTVDF(MEMADR,IVRLC,TVRLC,LVRLC,VRLCMX,NGRP,TXTPDF,TEXT12,
     1            NENTRY,KENTRY2,HENTRY2)
      ENDIF
      IF (TXTPDF.EQ.' ') THEN
         IF (TEXT12.NE.'INIT') CALL XABORT('FOBJCT: "INIT" KEYWORD '
     1  //'NEEDED BEFORE SEQUENCE DEFINITIONS')
         TXTPDF='FUNCTION'
      ENDIF
*debut de la sequence de calcul
      CALL OPTSEQ(MEMADR,STFCN,VTFCN,RTFCN,OTFCN,NLVL1,NLVL2,IVRLC,
     1            TVRLC,VRLCMX,NGRP,TXTPDF)
      LGDATA=NUN*NGRP
      CALL SETARA(RBASE,LGDATA,IDATA)
      CALL OPTEVL(MEMADR,VRLCMX,IVRLC,TVRLC,LVRLC,NLVL1,NLVL2,
     1            STFCN,VTFCN,RTFCN,OTFCN,RBASE(IDATA),LGDATA,TXTPDF)
      JPGPT=LCMLID(IPGPT,TXT12B,MAX(ISRC,20))
      KPGPT=LCMLIL(JPGPT,ISRC,NGRP)
      DO 385 IGR=1,NGRP
         CALL LCMPDL(KPGPT,IGR,NUN,2,RBASE(IDATA+(IGR-1)*NUN))
  385 CONTINUE
      CALL RLSARA(RBASE(IDATA))
      GO TO 20
C---------------------------
C END
C---------------------------
*liberer l'espace utilise par les vecteurs locaux
 1000 DO 1010 I=1,VRLCMX
         IF (LVRLC(I)) THEN
            CALL RLSARA(RBASE(IVRLC(I)))
         ENDIF
 1010 CONTINUE
      RETURN
      END
