*DECK MATLAB02
      SUBROUTINE  MATL02(IPASC,IPTRK,IPIND,IPFLU,IPGEO,NGRP,NUN,NEL,
     1       NMIX,EXNMIX,LX,LY,LZ,FLUOPT,ISRC,IGRID,FLUX,INDEX,XX,YY,
     2       ZZ,GEOM,FLXG,KEYFLX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this subroutine prepare input file for MATLAB for flux distribution 
*  mapping
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input
* IPASC   file unit number of the ASCII (future MATLAB file)
* IPTRK   adress of the lcm object of L_TRACK
* IPIND   adress of the lcm object of L_MATEX
* IPFLU   adress of the lcm object of L_FLUX
* IPGEO   adress of the lcm object of L_GEOM
* NGRP    number of energy group
* NUN     number of unknowns
* NEL     number of finite elements
* NMIX    number of materials mixtures
* LX      number elements for discretisation along X
* LY      number elements for discretisation along Y
* LZ      number elements for discretisation along Z
* FLUOPT  type of flux to be drawn
*          =1 FLUX
*          =2 ADJOINT
*          =3 GPT-FLUX
*          =4 GPT-ADJOINT
*          =5 HARMONIC
* ISRC    number of the source/harmonic if FLUOPT>2
* IGRID   type of mapping to be drawn
*          =1 NO-GRID
*          =2 CENTER-GRID
*          =3 CHANEL-GRID
*
*Parameters: scratch
* FLUX
* INDEX
* XX
* YY
* ZZ
* GEOM
* FLXG
* KEYFLX
*
* Note: the frontiers conditions and fine mesh can not taken into account
*         for the CHANEL-GRID option.
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER  IPASC,IPTRK,IPIND,IPFLU,IPGEO,NGRP,NUN,NEL,NMIX,EXNMIX,
     1         LX,LY,LZ,FLUOPT,ISRC,IGRID
      REAL     FLUX(NUN),XX(LX+1),YY(LY+1),ZZ(LZ+1),FLXG(LX,LY,LZ)
      INTEGER  INDEX(EXNMIX),GEOM(LX,LY,LZ),KEYFLX(NEL)
*----
*  LOCAL VARIABLES
*----
      INTEGER  I,J,K,IGR,JPFLU,KPFLU,LCMGID,LCMGIL

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))

* Table rase dans MATLAB      
      WRITE(IPASC,*) 'clear'
      WRITE(IPASC,*) 'isrc=',ISRC,';'
      WRITE(IPASC,*) 'fluopt=',FLUOPT,';'
      WRITE(IPASC,*) 'igrid=',IGRID,';'
* Calcul de la geometrie
      CALL LCMGET(IPIND,'MAT',INDEX)
      DO 10 I=1,LX
      DO 10 J=1,LY
      DO 10 K=1,LZ
         IF (INDEX(I+(J-1)*LX+(K-1)*LX*LY).EQ.0) THEN
            GEOM(I,J,K)=0
         ELSEIF (INDEX(I+(J-1)*LX+(K-1)*LX*LY).LT.0) THEN
            GEOM(I,J,K)=1
         ELSE
            GEOM(I,J,K)=-1
         ENDIF
   10    CONTINUE
      CALL LCMGET(IPGEO,'MESHX',XX)
      CALL LCMGET(IPGEO,'MESHY',YY)
      CALL LCMGET(IPGEO,'MESHZ',ZZ)
* Impression de la geometrie
      WRITE(IPASC,*) '% GEOMETRY'
      DO 25 K=1,LZ
         WRITE(IPASC,1010) K
         DO 20 J=1,LY
            CALL MATL0I(IPASC,LX,LX,GEOM(1,J,K))
            WRITE(IPASC,*) ';'
   20    CONTINUE
         WRITE(IPASC,1000)
   25 CONTINUE
      WRITE(IPASC,*) 'lx=',LX,';'
      WRITE(IPASC,*) 'ly=',LY,';'
      WRITE(IPASC,*) 'lz=',LZ,';'
      WRITE(IPASC,*) 'XX=['
      CALL MATL0R(IPASC,LX+1,LX+1,XX)
      WRITE(IPASC,1000)
      WRITE(IPASC,*) 'YY=['
      CALL MATL0R(IPASC,LY+1,LY+1,YY)
      WRITE(IPASC,1000)
      WRITE(IPASC,*) 'ZZ=['
      CALL MATL0R(IPASC,LZ+1,LZ+1,ZZ)
      WRITE(IPASC,1000)
*  Calcul du flux(LX,LY,LZ)
      CALL LCMGET(IPTRK,'KEYFLX',KEYFLX)
      DO 100 IGR=1,NGRP
         IF (FLUOPT.EQ.1) THEN
            JPFLU=LCMGID(IPFLU,'FLUX')
            CALL LCMGDL(JPFLU,IGR,FLUX)
            WRITE(IPASC,*) '% FLUX GR#'
         ELSEIF (FLUOPT.EQ.2) THEN
            JPFLU=LCMGID(IPFLU,'AFLUX')
            CALL LCMGDL(JPFLU,IGR,FLUX)
            WRITE(IPASC,*) '% ADJOINT GR#'
         ELSEIF (FLUOPT.EQ.3) THEN
            JPFLU=LCMGID(IPFLU,'DFLUX')
            KPFLU=LCMGIL(JPFLU,ISRC)
            CALL LCMGDL(KPFLU,IGR,FLUX)
            WRITE(IPASC,*) '% ADJOINT GENERALISE DIRECT GR#',IGR,
     1      ' POUR LA SOURCE#',ISRC
         ELSEIF (FLUOPT.EQ.4) THEN
            JPFLU=LCMGID(IPFLU,'ADFLUX')
            KPFLU=LCMGIL(JPFLU,ISRC)
            CALL LCMGDL(KPFLU,IGR,FLUX)
            WRITE(IPASC,*) '% ADJOINT GENERALISE INDIRECT GR#',IGR,
     1      ' POUR LA SOURCE#',ISRC
         ELSEIF (FLUOPT.EQ.5) THEN
            JPFLU=LCMGID(IPFLU,'AFLUX')
            KPFLU=LCMGIL(JPFLU,ISRC)
            CALL LCMGDL(KPFLU,IGR,FLUX)
            WRITE(IPASC,*) '% FLUX GR#',IGR,' POUR L''HARMONIQUE#',ISRC
         ENDIF
      DO 30 I=1,LX
      DO 30 J=1,LY
      DO 30 K=1,LZ
         IF (GEOM(I,J,K).EQ.0) THEN
            FLXG(I,J,K)=-1.1E25
         ELSE
            FLXG(I,J,K)=FLUX(KEYFLX(I+(J-1)*LX+(K-1)*LX*LY))
         ENDIF
   30    CONTINUE
      DO 45 K=1,LZ
         WRITE(IPASC,1020) K,IGR
         DO 40 J=1,LY
            CALL MATL0R(IPASC,LX,LX,FLXG(1,J,K))
            WRITE(IPASC,*) ';'
   40    CONTINUE
         WRITE(IPASC,1000)
   45 CONTINUE
         
  100 CONTINUE

* Boucle pour creer les figures
      WRITE(IPASC,*) '%%%%%%%%%%%%%%%%%%%%%'
      WRITE(IPASC,*) '% precalculs'
      WRITE(IPASC,*) 'ngrp=',NGRP,';'
      WRITE(IPASC,*) 'isrc=',ISRC,';'
      WRITE(IPASC,*) 'for i=1:lx,'
      WRITE(IPASC,*) '    FX(i)=(XX(i)+XX(i+1))/2;'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'for i=1:ly,'
      WRITE(IPASC,*) '    FY(i)=(YY(i)+YY(i+1))/2;'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'for i=1:lz,'
      WRITE(IPASC,*) '    FZ(i)=(ZZ(i)+ZZ(i+1))/2;'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'for i=1:lx,'
      WRITE(IPASC,*) 'for j=1:ly,'
      WRITE(IPASC,*) 'for k=1:lz,'
      WRITE(IPASC,*) 'for igr=1:ngrp,'
      WRITE(IPASC,*) '    if FLUX(i,j,k,igr) == -1.1E25'
      WRITE(IPASC,*) '        FLUX(i,j,k,igr) = NaN;'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'for i=1:lx+1,'
      WRITE(IPASC,*) 'for j=1:ly+1,'
      WRITE(IPASC,*) 'for k=1:lz,'
      WRITE(IPASC,*) 'for igr=1:ngrp,'
      WRITE(IPASC,*) '    FLUXC(i,j,k,igr) = NaN;'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'for i=1:lx+1,'
      WRITE(IPASC,*) 'for j=1:ly+1,'
      WRITE(IPASC,*) 'for k=1:lz,'
      WRITE(IPASC,*) 'for igr=1:ngrp,'
      WRITE(IPASC,*) '    if i > 1 '
      WRITE(IPASC,*) '    if i < lx '
      WRITE(IPASC,*) '    if j > 1 '
      WRITE(IPASC,*) '    if j < ly '
      WRITE(IPASC,*) '    if GEOM(i,j,k) > 0'
      WRITE(IPASC,*) '         a=(XX(i)-FX(i-1))/(FX(i)-FX(i-1));'
      WRITE(IPASC,*) '         b=(YY(j)-FY(j-1))/(FY(j)-FY(j-1));'
      WRITE(IPASC,*) '         FLUXC(i,j,k,igr) = (1-b)*(1-a)*FLUX(i-1'
     1  //',j-1,k,igr) ...'
      WRITE(IPASC,*) '                           +    (1-b)*a*FLUX(i,j'
     1  //'-1,k,igr)   ...'
      WRITE(IPASC,*) '                           +    b*(1-a)*FLUX(i-1,'
     1  //'j,k,igr)   ...'
      WRITE(IPASC,*) '                           +        b*a*FLUX(i,j,'
     1  //'k,igr)   ;'
      WRITE(IPASC,*) '         b=(YY(j+1)-FY(j))/(FY(j+1)-FY(j));'
      WRITE(IPASC,*) '         FLUXC(i,j+1,k,igr) = (1-b)*(1-a)*FLUX(i-'
     1  //'1,j,k,igr) ...'
      WRITE(IPASC,*) '                           +    (1-b)*a*FLUX(i,j,'
     1  //'k,igr)   ...'
      WRITE(IPASC,*) '                           +    b*(1-a)*FLUX(i-1,'
     1  //'j+1,k,igr)   ...'
      WRITE(IPASC,*) '                           +        b*a*FLUX(i,j+'
     1  //'1,k,igr)   ;'
      WRITE(IPASC,*) '         a=(XX(i+1)-FX(i))/(FX(i+1)-FX(i));'
      WRITE(IPASC,*) '         FLUXC(i+1,j+1,k,igr) = (1-b)*(1-a)*FLUX('
     1  //'i,j,k,igr) ...'
      WRITE(IPASC,*) '                           +    (1-b)*a*FLUX(i+1,'
     1  //'j,k,igr)   ...'
      WRITE(IPASC,*) '                           +     b*(1-a)*FLUX(i,j'
     1  //'+1,k,igr)   ...'
      WRITE(IPASC,*) '                           +        b*a*FLUX(i+1,'
     1  //'j+1,k,igr)   ;'
      WRITE(IPASC,*) '         b=(YY(j)-FY(j-1))/(FY(j)-FY(j-1));'
      WRITE(IPASC,*) '         FLUXC(i+1,j,k,igr) = (1-b)*(1-a)*FLUX(i,'
     1  //'j-1,k,igr) ...'
      WRITE(IPASC,*) '                           +    (1-b)*a*FLUX(i+1,'
     1  //'j-1,k,igr)   ...'
      WRITE(IPASC,*) '                           +    b*(1-a)*FLUX(i,j,'
     1  //'k,igr)   ...'
      WRITE(IPASC,*) '                           +        b*a*FLUX(i+1,'
     1  //'j,k,igr)   ;'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'end'
      WRITE(IPASC,*) 'FLUXC=FLUXC*1.01;'

      WRITE(IPASC,*) '%%%%%%%%%%%%%%%%%%%%%'
      WRITE(IPASC,*) 'for igr=1:ngrp,'
      WRITE(IPASC,*) '  for k=1:lz,'
      WRITE(IPASC,*) '    figure'
      WRITE(IPASC,*) '    g=surfc(FX(1:lx),FY(1:ly),FLUX(1:lx,1:ly,k,'
     1  //'igr));'
      WRITE(IPASC,*) '    shading interp;'
      WRITE(IPASC,*) '    if igrid == 2'
      WRITE(IPASC,*) '      set(g,''EdgeColor'',[0 0 0])'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    if fluopt == 1'
      WRITE(IPASC,*) '      text1=''FLUX'';'
      WRITE(IPASC,*) '    elseif fluopt == 2'
      WRITE(IPASC,*) '      text1=''FLUX ADJOINT'';'
      WRITE(IPASC,*) '    elseif fluopt == 3'
      WRITE(IPASC,*) '      text1=''ADJOINT GENERALISE DIRECT'';'
      WRITE(IPASC,*) '    elseif fluopt == 4'
      WRITE(IPASC,*) '      text1=''ADJOINT GENERALISE INDIRECT'';'
      WRITE(IPASC,*) '    elseif fluopt == 5'
      WRITE(IPASC,*) '      text1=''FLUX HARMONIQUE'';'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    text=[text1,'' GR#'', num2str(igr)'
     1  //','', Plan #'',num2str(k)];'
      WRITE(IPASC,*) '    if fluopt >= 3'
      WRITE(IPASC,*) '      text=[text,'' SOURCE#'', num2str(igr)''];'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '    title(text)'
      WRITE(IPASC,*) '    if igrid == 3'
      WRITE(IPASC,*) '      hold on'
      WRITE(IPASC,*) '      h=surf(XX(1:lx+1),YY(1:ly+1),FLUXC(1:lx+1,1'
     1  //':ly+1,k,igr));'
      WRITE(IPASC,*) '      set(h,''FaceAlpha'',0)'
      WRITE(IPASC,*) '      set(h,''EdgeColor'',[0 0 0])'
      WRITE(IPASC,*) '    end'
      WRITE(IPASC,*) '  end'
      WRITE(IPASC,*) 'end'
      
C FORMAT

 1000 FORMAT(1X,2H];)
 1010 FORMAT(1X,9HGEOM(:,:,,I4,3H)=[)
 1020 FORMAT(1X,9HFLUX(:,:,,I4,1H,,I4,3H)=[)
      END
