*DECK MATLAB
      SUBROUTINE MATLAB(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this module prepare input file for MATLAB
*    1st option : for gradiants verification
*    2nd option : for flux distribution mapping
*    3rd option : for XS graphical representation
*    4th option : for graphical representation of optimization results 
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of lcm objects used by the module
* HENTRY  character*12 name of each lcm object
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the lcm object is created
*         =1 the lcm object is open for modifications;
*         =2 the lcm object is open in read-only mode
* KENTRY  =file unit number; =lcm object address otherwise
*         kentry(nentry)
* LCM OBJECTS:
* HENTRY(1)  create type (l_macrolib)
* HENTRY(2)  read only type (l_macrolib)
* HENTRY(3)  read only type (l_macrolib)
* or
* HENTRY(1)  create type (l_flux)
* HENTRY(2)  read only type (l_flux)
* HENTRY(3)  read only type (l_flux)
*                          
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT  NONE
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)

      INTEGER   IPASC,IPOPT,IPFLU,IPGEO,IPIND,IPTRK,IPCPO
* STATE-VECTOR VARIABLES
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   OPTPRI(NSTATE),FLUPRM(NSTATE),TRKPRM(NSTATE),
     1          GEOPRM(NSTATE),INDPRM(NSTATE)
* General variable
      INTEGER   I,J,NITMA,IPRINT,ITYP,SIGNA(3)
      REAL      FLOTT
      CHARACTER TEXT12*12
      DOUBLE PRECISION DFLOTT
* 100 or 400 - L_OPTIMIZE
      INTEGER   NVAR,NFCT,NCHA
      INTEGER   IFCT,IGRAD,IVARVL,IINEPS,IPCHA
* 200 - L_FLUX
      INTEGER   NGRP,NUN,NEL,NMIX,EXNMIX,LX,LY,LZ,FLUOPT,ISRC,IGRID
      INTEGER   IFLUX,IIND,IGEOM,IXX,IYY,IZZ,IFLXG,IKEYF
* 300 - L_COMPO
      INTEGER   NPARAM,NPMAX,IPARMX,NBSTEP,IPARA,IPARN,PARVAL
      PARAMETER (NPMAX=6,IPARMX=100)
      INTEGER   IPAR(NPMAX),IXS,IXSGP
      CHARACTER NAMEPR(NPMAX)*12,REPERT(2)*12
   

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))

C---------------------------
C     VALIDITY OF OBJECTS
C---------------------------
      IF((IENTRY(1).NE.4).AND.(JENTRY(1).NE.0)) 
     1  CALL XABORT('MATLAB: SEQ_ASCII ON RHS IN CREATE MODE EXPECTED')
      IF(NENTRY.LT.2) CALL XABORT('MATLAB: 2 OBJECTS AT LEAST EXPECTED')
      DO 2 I=2,NENTRY
         IF((IENTRY(I).NE.1).OR.(JENTRY(I).NE.2))
     1  CALL XABORT('MATLAB: LCM OBJECT ON LHS IN READ-ONLY MODE'
     2  //' EXPECTED')
    2    CONTINUE
C---------------------------
C VARIABLE INITIALISATION	
C---------------------------
      IPRINT= 0
      IPASC = KENTRY(1)
      IPOPT = 0
      IPFLU = 0
      IPGEO = 0
      IPIND = 0
      IPTRK = 0
      IPCPO = 0
C---------------------------
C     READ USER INPUT:
C---------------------------
   20 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER EXPECTED (1)')
   30 IF (TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('MATLAB: *IPRINT* MUST BE INTEGER')
      ELSEIF (TEXT12.EQ.'OPT-GRAD-VRF') THEN
         GO TO 100
      ELSEIF (TEXT12.EQ.'MAP-FLUX') THEN
         GO TO 200
      ELSEIF (TEXT12.EQ.'PLOT-XS') THEN
         GO TO 300
      ELSEIF (TEXT12.EQ.'OPT-HIST') THEN
         GO TO 400
      ELSEIF( TEXT12.EQ.';' )THEN
         GO TO 1000
      ELSE
         CALL XABORT('MATLAB: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20

C---------------------------
C     1      0000     0000    
C    11     0    0   0    0 
C   1 1    0      0 0      0 
C  1  1    0      0 0      0  
C 1   1    0      0 0      0  
C     1    0      0 0      0  
C     1     0    0   0    0  
C 11111111   0000     0000  
C---------------------------
C Validity of the calculated gradient
C---------------------------
  100 IPOPT = KENTRY(2)
      CALL LCMGET(IPOPT,'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_OPTIMIZE') 
     1  CALL XABORT('MATLAB: "L_OPTIMIZE" OBJECT EXPECTED')
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      NVAR = OPTPRI(1)
      NFCT = OPTPRI(2)+1
      CALL SETARA(RBASE,NVAR*NFCT,IGRAD)
      CALL SETARA(RBASE,NFCT,IFCT)
      CALL SETARA(RBASE,NVAR,IVARVL)
      CALL SETARA(IBASE,NVAR,IINEPS)
      CALL MATL01(IPASC,IPOPT,NVAR,NFCT,RBASE(IGRAD),RBASE(IFCT),
     1            RBASE(IVARVL),IBASE(IINEPS))
      CALL RLSARA(RBASE(IGRAD))
      CALL RLSARA(RBASE(IFCT))
      CALL RLSARA(RBASE(IVARVL))
      CALL RLSARA(IBASE(IINEPS))
      GO TO 20
C---------------------------
C   2222     0000     0000    
C  2    2   0    0   0    0 
C 2      2 0      0 0      0 
C       2  0      0 0      0  
C      2   0      0 0      0  
C   22     0      0 0      0  
C  2        0    0   0    0  
C 22222222   0000     0000  
C---------------------------
C Map of the flux (CARTESIAN GEOMETRY)
C---------------------------
  200 IF (NENTRY.NE.5) CALL XABORT('MATLAB: 5 OBJECTS EXPECTED (200)')
* 2=FLUX
      IPFLU = KENTRY(2)
      CALL LCMGET(IPFLU,'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_FLUX') 
     1  CALL XABORT('MATLAB: RHS FIRST OBJECT "L_FLUX" EXPECTED ')
* 3=TRACKING = TRIVAC
      IPTRK = KENTRY(3)
      CALL LCMGET(IPTRK,'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_TRACK') 
     1  CALL XABORT('MATLAB: RHS SECOND OBJECT "L_TRACK" EXPECTED')
      CALL LCMGET(IPTRK,'TRACK-TYPE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'TRIVAC') 
     1  CALL XABORT('MATLAB: "TRIVAC" ONLY EXPECTED')
* 4=INDEX
      IPIND = KENTRY(4)
      CALL LCMGET(IPIND,'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_MATEX') 
     1  CALL XABORT('MATLAB: RHS THIRD OBJECT "L_MATEX" EXPECTED')
* 5=GEOMETRY
      IPGEO = KENTRY(5)
      CALL LCMGET(IPGEO,'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_GEOM') 
     1  CALL XABORT('MATLAB: RHS FORTH OBJECT "L_GEOM" EXPECTED')

* Lecture des options
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER EXPECTED (200)')
      IF (TEXT12.EQ.'FLUX') THEN
         FLUOPT=1
      ELSEIF (TEXT12.EQ.'ADJOINT') THEN
         FLUOPT=2
      ELSEIF (TEXT12.EQ.'GPT-FLUX') THEN
         FLUOPT=3
         CALL REDGET(ITYP,ISRC,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('MATLAB: INTEGER EXPECTED (201)')
      ELSEIF (TEXT12.EQ.'GPT-ADJOINT') THEN
         FLUOPT=4
         CALL REDGET(ITYP,ISRC,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('MATLAB: INTEGER EXPECTED (202)')
      ELSEIF (TEXT12.EQ.'HARMONIC') THEN
         FLUOPT=5
         CALL REDGET(ITYP,ISRC,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('MATLAB: INTEGER EXPECTED (203)')
      ELSE
         CALL XABORT('MATLAB:"FLUX TYPE" KEYWORD EXPECTED (200)')
      ENDIF
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER EXPECTED (200)')
      IF (TEXT12.EQ.'NO-GRID') THEN
         IGRID=1
      ELSEIF (TEXT12.EQ.'CENTER-GRID') THEN
         IGRID=2
      ELSEIF (TEXT12.EQ.'CHANEL-GRID') THEN
         IGRID=3
      ELSE
         CALL XABORT('MATLAB:"GRID TYPE" KEYWORD EXPECTED (200)')
      ENDIF
      CALL LCMGET(IPFLU,'STATE-VECTOR',FLUPRM)
      CALL LCMGET(IPTRK,'STATE-VECTOR',TRKPRM)
      CALL LCMGET(IPIND,'STATE-VECTOR',INDPRM)
      CALL LCMGET(IPGEO,'STATE-VECTOR',GEOPRM)
      NGRP  = FLUPRM(1)
      NUN   = FLUPRM(2)
      NEL   = TRKPRM(1)
      NMIX  = INDPRM(2)
      LX    = GEOPRM(3)
      LY    = GEOPRM(4)
      LZ    = GEOPRM(5)
      IF (IPRINT.GE.1) THEN
         WRITE(6,*) 'SIZE OF THE PROBLEM'
         WRITE(6,*) 'LX=',LX,'LY=',LY,'LZ=',LZ
      ENDIF
      IF (NEL.NE.LX*LY*LZ) CALL XABORT('MATLAB: NEL ne LX*LY*LZ')
      CALL LCMLEN(IPIND,'MAT',EXNMIX,ITYP)
      CALL SETARA(RBASE,NUN,IFLUX)
      CALL SETARA(IBASE,EXNMIX,IIND)
      CALL SETARA(RBASE,LX+1,IXX)
      CALL SETARA(RBASE,LY+1,IYY)
      CALL SETARA(RBASE,LZ+1,IZZ)
      CALL SETARA(IBASE,NEL,IGEOM)
      CALL SETARA(RBASE,NEL,IFLXG)
      CALL SETARA(IBASE,NEL,IKEYF)
      CALL MATL02(IPASC,IPTRK,IPIND,IPFLU,IPGEO,NGRP,NUN,NEL,NMIX,
     1            EXNMIX,LX,LY,LZ,FLUOPT,ISRC,IGRID,RBASE(IFLUX),
     2            IBASE(IIND),RBASE(IXX),RBASE(IYY),RBASE(IZZ),
     3            IBASE(IGEOM),RBASE(IFLXG),IBASE(IKEYF))
      CALL RLSARA(RBASE(IFLUX))
      CALL RLSARA(IBASE(IIND))
      CALL RLSARA(RBASE(IXX))
      CALL RLSARA(RBASE(IYY))
      CALL RLSARA(RBASE(IZZ))
      CALL RLSARA(IBASE(IGEOM))
      CALL RLSARA(RBASE(IFLXG))
      CALL RLSARA(IBASE(IKEYF))
       
      GO TO 20
C---------------------------
C   3333     0000     0000
C  3    3   0    0   0    0
C 3      3 0      0 0      0
C      33  0      0 0      0
C       3  0      0 0      0
C 3      3 0      0 0      0
C  3    3   0    0   0    0  
C   3333     0000     0000
C---------------------------
C Plot cross-section
C---------------------------
  300 DO 302 I=2,NENTRY
         CALL LCMGET(KENTRY(I),'SIGNATURE',SIGNA)
         WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
         IF (TEXT12.NE.'L_COMPO') 
     1  CALL XABORT('MATLAB: RHS FIRST OBJECT "L_COMPO" EXPECTED ')
  302 CONTINUE
* initialisation
      NPARAM=0
      NBSTEP=0
      DO 303 I=1,NPMAX
         WRITE(TEXT12,'(A7,I3.3)') 'PARAM #',I
         NAMEPR(I)=TEXT12
  303    CONTINUE
      CALL XDISET(IPAR,NPMAX,1)
      REPERT(1)=' '
      REPERT(2)='MACR'
* Lecture des options
  304 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER EXPECTED (300)')
      IF (TEXT12.EQ.'NB-PARAM') THEN
         CALL REDGET(ITYP,NPARAM,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('MATLAB: INTEGER  EXPECTED (300)')
         IF ((NPARAM.LE.1).OR.(NPARAM.GT.NPMAX)) CALL XABORT('MATLAB:'
     1  //'NUMBER OF PARAMETER INVALID')
         GO TO 304
      ELSEIF (TEXT12.EQ.'NB-BURN-STEP') THEN
         CALL REDGET(ITYP,NBSTEP,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('MATLAB: INTEGER  EXPECTED (301)')
         GO TO 304
      ELSEIF (TEXT12.EQ.'NGRP') THEN
         CALL REDGET(ITYP,NGRP,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('MATLAB: INTEGER  EXPECTED (301)')
         GO TO 304
      ELSEIF (TEXT12.EQ.'NAME-PARAM') THEN
         IF (NPARAM.EQ.0) CALL XABORT('MATLAB: NUMBER OF PARAMETER NOT'
     1  //'DEFINED YET')
         NAMEPR(1)='BURNUP'
         DO 305 I=2,NPARAM
            CALL REDGET(ITYP,NITMA,FLOTT,NAMEPR(I),DFLOTT)
            IF (ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER EXPECTED '
     1  //'(304)')
  305    CONTINUE
         WRITE(IPASC,*) 'namepara=['
         DO 306 I=1,NPARAM
            WRITE(IPASC,*) '''',NAMEPR(I),''''
  306       CONTINUE
         WRITE(IPASC,*) '];'
         GO TO 304
      ELSEIF (TEXT12.EQ.'DATA') THEN
         IF (NPARAM.EQ.0) CALL XABORT('MATLAB: NUMBER OF PARAMETER NOT'
     1  //'DEFINED YET')
         IF (NBSTEP.EQ.0) CALL XABORT('MATLAB: NUMBER OF BURNUP STEPS '
     1  //'NOT DEFINED YET')
         IF (NGRP.EQ.0) CALL XABORT('MATLAB: NUMBER OF GROUPS '
     1  //'NOT DEFINED YET')
* number for datas for parameter
         DO 307 I=2,NPARAM
            CALL REDGET(ITYP,IPAR(I),FLOTT,TEXT12,DFLOTT)
            IF(ITYP.NE.1) CALL XABORT('MATLAB: INTEGER  EXPECTED (306)')
            IF ((IPAR(I).LE.0).OR.(IPAR(I).GT.IPARMX)) THEN
               WRITE(6,*) 'PARAMETER #',I,',',IPAR(I),'>',IPARMX
               CALL XABORT('MATLAB TOO MUCH DATAS PARAMETER')
            ENDIF
  307    CONTINUE
* CPO name
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER EXPECTED (310)')
         DO 308 I=2,NENTRY
            IF (TEXT12.EQ.HENTRY(I)) IPCPO=KENTRY(I)
  308    CONTINUE
         IF (IPCPO.EQ.0) CALL XABORT('MATLAB: '//TEXT12//' WRONG L_CPO'
     1  //' NAME')
* repertory name in CPO
         CALL REDGET(ITYP,NITMA,FLOTT,REPERT(1),DFLOTT)
         IF(ITYP.NE.3) CALL XABORT('MATLAB: CHARACTER  EXPECTED (311)')
* MACR ou micro

* store data in ASCII file
         CALL SETARA(RBASE,NGRP*NBSTEP*20,IXS)
         CALL SETARA(RBASE,NGRP*20,IXSGP)
         CALL MATL03(IPASC,IPCPO,NPARAM,NPMAX,NBSTEP,NGRP,IPAR,
     1  REPERT,RBASE(IXS),RBASE(IXSGP))
         CALL RLSARA(RBASE(IXS))
         CALL RLSARA(RBASE(IXSGP))
         IPCPO=0
         GO TO 304
      ELSEIF (TEXT12.EQ.'PARAM-VALUE') THEN
         CALL REDGET(ITYP,IPARA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('MATLAB: INTEGER EXPECTED (312)')
         CALL REDGET(ITYP,IPARN,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('MATLAB: INTEGER EXPECTED (314)')
         CALL SETARA(RBASE,IPARN,PARVAL)
         DO 316 I=1,IPARN
            CALL REDGET(ITYP,NITMA,RBASE(PARVAL+I-1),TEXT12,DFLOTT)
            IF (ITYP.NE.2) CALL XABORT('MATLAB: REAL EXPECTED (316)')
  316       CONTINUE
         WRITE(IPASC,'(A6,I1,A2)') 'xparam',IPARA,'=['
         CALL MATL0R(IPASC,IPARN,IPARN,RBASE(PARVAL))
         WRITE(IPASC,*) '];'
         CALL RLSARA(RBASE(PARVAL))
         GO TO 304
      ELSEIF (TEXT12.EQ.'GUI-CODE') THEN
         WRITE(IPASC,*) 'npmax=',NPMAX,';'
         WRITE(IPASC,*) 'dim=size(XS);'
         WRITE(IPASC,*) 'dim2=size(namepara);'
         GO TO 304
      ELSE
         GO TO 30
      ENDIF
C---------------------------
C     4      0000     0000    
C    44     0    0   0    0 
C   4 4    0      0 0      0 
C  4  4    0      0 0      0  
C 4   4    0      0 0      0  
C  4444444 0      0 0      0  
C     4     0    0   0    0  
C     4      0000     0000  
C---------------------------
C OPT-HIST
C---------------------------
  400 IPOPT = KENTRY(2)
      CALL LCMGET(IPOPT,'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_OPTIMIZE') 
     1  CALL XABORT('MATLAB: "L_OPTIMIZE" OBJECT EXPECTED')
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      NVAR = OPTPRI(1)
      NCHA = OPTPRI(17)
      CALL SETARA(RBASE,NVAR,IVARVL)
      CALL SETARA(RBASE,NCHA,IPCHA)
      CALL MATL04(IPASC,IPOPT,NVAR,NCHA,RBASE(IVARVL),RBASE(IPCHA))
      CALL RLSARA(RBASE(IVARVL))
      CALL RLSARA(RBASE(IPCHA))
      GO TO 20
C---------------------------
C FORMAT
C---------------------------
10000 FORMAT(1X,2H];)

C---------------------------
C END
C---------------------------
 1000 CONTINUE
      RETURN

      END
