*DECK OPTAFI
      SUBROUTINE OPTAFI(IPFLU,IPTRK,IPSYS,NUN,NGRP,LL4,ITY,OPTION,
     1                  MATPHI,FLUX,WORK)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this subroutine compute the product of matrix system times flux
* OPTAFI = OPTimization - system matrix (A) times flux (FI)
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
*  IPFLU,IPTRK,IPSYS
*  NUN    : number of unknowns per energy group
*  NGRP   : number of energy groups
*  LL4    : order of the matrix.
*  ITY    : type of coefficient matrix (1: Bivac; 2: classical Trivac;
*           3: Thomas-Raviart).
*  OPTION : 1 for A*PHI
*           2 for B*PHI
*         : 3 for At*PHI_ADJ
*           2 for Bt*PHI_ADJ
*  MATPHI : result of the system matrix * flux
*
* SCRATCH STORAGE:
*  FLUX   : (NUN)
*  WORK   : (NUN)
*                          
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT  NONE
      INTEGER   NUN,NGRP,LL4,ITY,OPTION,IPFLU,IPTRK,IPSYS
      REAL      MATPHI(NUN*NGRP),FLUX(NUN),WORK(NUN)
*----
*  LOCAL VARIABLES
*----
* General variable
      INTEGER   IGR,JGR,LENGT,ITYP,I,JPFLU,LCMGID
      CHARACTER TEXT4*4,TXT12B*12

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))

      IF (IPFLU.EQ.0) CALL XABORT('OPTAFI: L_FLUX NEEDED')
      IF (IPTRK.EQ.0) CALL XABORT('OPTAFI: L_TRACK NEEDED')
      IF (IPSYS.EQ.0) CALL XABORT('OPTAFI: L_SYSTEM NEEDED')

      CALL XDRSET(MATPHI,NUN*NGRP,0.0)
      
      IF ((OPTION.EQ.1).OR.(OPTION.EQ.3)) THEN
         TEXT4='A'
      ELSE
         TEXT4='B'
      ENDIF
      IF (OPTION.LE.2) THEN         
* multiplier matrice (A) par le flux(F) 
*     F'=A*F ou F'=B*F
      JPFLU=LCMGID(IPFLU,'FLUX')
      DO 888 IGR=1,NGRP
         CALL LCMGDL(JPFLU,IGR,FLUX)
         DO 887 JGR=1,NGRP
            WRITE(TXT12B,'(A1,2I3.3)') TEXT4,JGR,IGR
            IF ((IGR.EQ.JGR).AND.(TEXT4.EQ.'A')) THEN
C              F'i=...+Aii*Fi...
               CALL MTLDLM(TXT12B,IPTRK,IPSYS,LL4,ITY,FLUX,WORK)
               DO 883 I=1,NUN
                  MATPHI((JGR-1)*NUN+I)=
     1                     MATPHI((JGR-1)*NUN+I)+WORK(I)
  883                CONTINUE
            ELSE
               CALL LCMLEN(IPSYS,TXT12B,LENGT,ITYP)
               IF (LENGT.EQ.0) GO TO 887
               CALL LCMGET(IPSYS,TXT12B,WORK)
               IF (TEXT4.EQ.'A') THEN
C              F'j=...-Aji*Fi...
                  DO 884 I=1,NUN
                     MATPHI((JGR-1)*NUN+I)=
     1                  MATPHI((JGR-1)*NUN+I)-FLUX(I)*WORK(I)
  884                   CONTINUE
               ELSE
C              F'j=...+Bji*Fi...
                  DO 885 I=1,NUN
                     MATPHI((JGR-1)*NUN+I)=
     1                  MATPHI((JGR-1)*NUN+I)+FLUX(I)*WORK(I)
  885                   CONTINUE
               ENDIF
            ENDIF
  887       CONTINUE
  888    CONTINUE
      ELSE
* multiplier matrice transposee At par le flux adjoint(F_ad) 
*     F'=At*F_ad ou F'=Bt*F_ad
      DO 988 IGR=1,NGRP
         WRITE(TXT12B,'(A5,I3.3)') 'AFLUX',IGR
         CALL LCMGET(IPFLU,TXT12B,FLUX)
  981    DO 987 JGR=1,NGRP
            WRITE(TXT12B,'(A1,2I3.3)') TEXT4,IGR,JGR
            IF ((IGR.EQ.JGR).AND.(TEXT4.EQ.'A')) THEN
C              F'i=...+Aii*Fi...
               CALL MTLDLM(TXT12B,IPTRK,IPSYS,LL4,ITY,FLUX,WORK)
               DO 983 I=1,NUN
                  MATPHI((JGR-1)*NUN+I)=
     1                     MATPHI((JGR-1)*NUN+I)+WORK(I)
  983                CONTINUE
            ELSE
               CALL LCMLEN(IPSYS,TXT12B,LENGT,ITYP)
               IF (LENGT.EQ.0) GO TO 987
               CALL LCMGET(IPSYS,TXT12B,WORK)
               IF (TEXT4.EQ.'A') THEN
C              F'j=...-Aij*Fi...
                  DO 984 I=1,NUN
                     MATPHI((JGR-1)*NUN+I)=
     1                  MATPHI((JGR-1)*NUN+I)-FLUX(I)*WORK(I)
  984                   CONTINUE
               ELSE
C              F'j=...+Bij*Fi...
                  DO 985 I=1,NUN
                     MATPHI((JGR-1)*NUN+I)=
     1                  MATPHI((JGR-1)*NUN+I)+FLUX(I)*WORK(I)
  985                   CONTINUE
               ENDIF
            ENDIF
  987       CONTINUE
  988    CONTINUE
      ENDIF
      RETURN
      END
