*DECK PLDRV
      SUBROUTINE PLDRV(IMTHD,XOBJ,FCOST,GF,GRADC,INEGAL,CONTR,DINF,DSUP,
     >          APLUS, BPLUS,INPLUS,PDG,XDROIT,LCONTQ,IERR,N0,NCST,
     >          MINMAX,M0,NPM,IMPR,EPSIM,NOPTX,WORK,MEMADR)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine prepares the different matrices for the resolution of 
*  the non-linear optimisation problem. The different available methods
*  are the map technique, the lemke, the augmented-lagrangian and the 
*  penalty function.
* PLDRV = Linear Programmation DRiVer for options
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): OPTEX
*  Modified : R. Chambon
*
*Parameters: input/ouput
*     IMTHD   : TYPE DE RESOLUTION. =1 : SIMPLEX / LEMKE 
*                                   =2 : LEMKE   / LEMKE  
*                                   =3 : MAP  
*                                   =4 : Augmented Lagragian
*                                   =5 : External penalty funnction
*     N0   : NOMBRE DE VARIABLES DE DECISION. 
*     NCST  : NOMBRE DE CONTRAINTES LINEAIRES.  
*     MINMAX : =-1 POUR MAXIMISER L'OBJECTIF ; =1 POUR LE MINIMISER.   
*     XOBJ     : VARIABLES DE CONTROLE. DIMENSION X(N0)                
*     FCOST  : FONCTION OBJECTIF.                                      
*     GF     : GFS DES VARIABLES DE CONTROLE. DIMENSION GF(N0)      
*     GRADC  : COEFFICIENTS DES CONTRAINTES LINEAIRES.                
*              DIMENSION GRADC(N0,NCST)                            
*     INEGAL : RELATIONS DES CONTRAINTES : =-1 POUR 'PLUS GRAND OU EGAL
*                                          =0  POUR 'EGAL'             
*                                          =1  POUR 'PLUS PETIT OU EGAL
*     CONTR     : TERMES DE DROITE DES CONTRAINTES LINEAIRES.          
*              DIMENSION CONTR(NCST)                                    
*     DINF   : BORNES INFERIEURES SUR LES VARIABLES DE DECISION.       
*     DSUP   : BORNES SUPERIEURES SUR LES VARIABLES DE DECISION.       
*              DIMENSION DINF(N0),DSUP(N0)                         
*     IERR   : CODE DE RETOUR (EGAL A ZERO NORMALEMENT).               
*     IMPR   : CONTROLE DE L'IMPRESSION.                               
*     M0     : NOMBRE DE CONTRAINTES LINEAIRES PLUS LE NOMBRE DE BORNES
*              (INFERIEURES OU SUPERIEURES) INTERCEPTANT LA CONTRAINTE 
*              QUADRATIQUE.                                            
*     NPM    : N0+(M0+1)  TAILLE DU SYSTEME LINEAIRE                 
*     XDROIT : pas externe
*     LCONTQ : logique contrainte quadratique
*     EPSIM  : 
*     NOPTX  : 
*     MEMADR : Adress memory of lcm object (description OPTVDF.f)
*
*Parameters: scratch
*     APLUS  : ESPACE DE TRAVAIL. DOUBLE PRECISION APLUS(M0+1,NPM)    
*     BPLUS  : ESPACE DE TRAVAIL. DIMENSION BPLUS(M0+2)                
*     INPLUS : ESPACE DE TRAVAIL. DIMENSION INPLUS(M0+1)               
*     PDG    : ESPACE DE TRAVAIL. DIMENSION PDG(N0)                 
*     WORK   : ESPACE DE TRAVAIL  DIMENSION N0*NCST
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT NONE
      INTEGER      N0,NCST,MINMAX,M0,NPM,IMPR,IERR,IMTHD,NOPTX,
     >             MEMADR(20)
      REAL         XOBJ(N0),CONTR(NCST),DINF(N0),DSUP(N0),
     >             BPLUS(M0+2) ,PDG(N0),EPSIM,XDROIT,
     >             WORK(N0,NCST+1)
      DOUBLE PRECISION  FCOST,GF(N0),GRADC(N0,NCST),APLUS(M0+1,NPM)
      INTEGER      INEGAL(NCST),INPLUS(M0+1)
      LOGICAL      LCONTQ
*----
*  LOCAL VARIABLES
*----
      INTEGER      ME,MI,I,J,IBINF,IBSUP,ISCALE,IPX,IRX,IDELT,IBGAR,
     >             LENGT,ITYP,M0B,NPMB,IB2,ILAGF,IINPL2,ICONT2
      DOUBLE PRECISION  XX
      CHARACTER    CLNAME*6
      DATA         CLNAME /'PLDRV'/

      INTEGER      ILAFAC,ICTWGT,IAPLS2

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
* 
*---- ORGANISATION DU TABLEAU GF et GRADC
      CALL LCMLEN(MEMADR(6),'GRADIENT',LENGT,ITYP)
      IF (LENGT.NE.N0*(NCST+1)) CALL XABORT('PQL: NUMBER OF '
     1  //'GRADIENT ERROR')
      CALL LCMGET(MEMADR(6),'GRADIENT',WORK)
      DO 3 I=1,N0
         GF(I)=DBLE(WORK(I,1))*DBLE(MINMAX)
         DO 2 J=1,NCST
    2       GRADC(I,J)=DBLE(WORK(I,1+J))
    3    CONTINUE
*
*---- ORGANISATION DU TABLEAU APLUS - CONTRAINTES D'EGALITE
*
      ME=0
*
      DO 10 I=1,NCST
         IF (INEGAL(I).EQ.0) THEN
            ME = ME + 1
*
            DO 20 J=1,N0
               APLUS(ME,J) = GRADC(J,I)
   20       CONTINUE
*
            BPLUS(ME)  = CONTR(I)
            INPLUS(ME) = 0
         ENDIF
   10 CONTINUE
*
*---- ORGANISATION DU TABLEAU APLUS - CONTRAINTES D'INEGALITE
*
      MI = 0
*
      DO 30 I=1,NCST
         IF (INEGAL(I).NE.0) THEN
            MI = MI + 1
*
            DO 40 J=1,N0
               APLUS(ME+MI,J) = GRADC(J,I)
   40       CONTINUE
*
            BPLUS(ME+MI)  = CONTR(I)
            INPLUS(ME+MI) = INEGAL(I)
         ENDIF
   30 CONTINUE
*
*---- CONTRAINTES ACTIVES DES BORNES DES VARIABLES
*
      DO 50 I=1,N0
         XX = SQRT(XDROIT/PDG(I))
*
         IF (DINF(I).GT.-XX) THEN
            MI = MI + 1
*
            DO 60 J=1,N0
               APLUS(ME+MI,J) = 0.0D0
   60       CONTINUE
*
            APLUS(ME+MI,I) = 1.0D0
            BPLUS(ME+MI)   = DINF(I)
            INPLUS(ME+MI)  = -1
         ENDIF
*
         IF (DSUP(I).LT.XX) THEN
            MI = MI + 1
*
            DO 70 J=1,N0
               APLUS(ME+MI,J) = 0.0D0
   70       CONTINUE
*
            APLUS(ME+MI,I) = 1.0D0
            BPLUS(ME+MI)   = DSUP(I)
            INPLUS(ME+MI)  = 1
         ENDIF
*
   50 CONTINUE
*
      DO 71 J=1,N0
         APLUS(M0+1,J) = 0.0D0
   71    CONTINUE
      BPLUS(M0+1)   = 0.0
      INPLUS(M0+1)  = 0
*
      IF (M0.NE.ME+MI) THEN
         WRITE (6,1000) M0,ME,MI
         CALL XABORT('PLDRV: M0 AND ME+MI ARE NOT THE SAME')
      ENDIF
*
*---- IMPRESSION DU PROBLEME QUASI-LINEAIRE
*
      IF (IMPR.GE.5) THEN
         CALL PRNTAB(GF,APLUS,INPLUS,BPLUS,PDG,DINF,DSUP,N0,M0,
     >             CLNAME)
      ENDIF
*
*---- METHODE DE LEMKE
*
      IF (IMTHD.LE.2) THEN
*
         CALL SETARA(RBASE,N0,IBINF)
         CALL SETARA(RBASE,N0,IBSUP)
         CALL SETARA(RBASE,N0,ISCALE)
*
         CALL PLMAP2(N0,M0,NPM,APLUS,PDG,BPLUS,INPLUS,XDROIT,GF,
     >      FCOST,XOBJ,RBASE(IBINF),RBASE(IBSUP),RBASE(ISCALE),IMTHD,
     >      LCONTQ,IERR,IMPR,EPSIM)
*
         CALL RLSARA(RBASE(IBINF))
         CALL RLSARA(RBASE(IBSUP))
         CALL RLSARA(RBASE(ISCALE))
*
*---- METHODE MAP
*
      ELSE IF (IMTHD.EQ.3) THEN
*
         CALL SETARA(RBASE,N0,IBINF)
         CALL SETARA(RBASE,N0,IBSUP)
         CALL SETARA(RBASE,N0,ISCALE)
         CALL SETARA(RBASE,N0,IPX)
         CALL SETARA(RBASE,N0,IRX)
         CALL SETARA(RBASE,N0,IDELT)
         CALL SETARA(RBASE,M0+1,IBGAR)
*
         CALL PLMAP1(N0,M0,APLUS,PDG,BPLUS,INPLUS,XDROIT,GF,FCOST,XOBJ,
     >      RBASE(IBINF),RBASE(IBSUP),RBASE(ISCALE),RBASE(IPX),
     >      RBASE(IRX),RBASE(IDELT),RBASE(IBGAR),IMTHD,LCONTQ,IERR,IMPR)
*
         CALL RLSARA(RBASE(IBGAR))
         CALL RLSARA(RBASE(IDELT))
         CALL RLSARA(RBASE(IRX))
         CALL RLSARA(RBASE(IPX))
         CALL RLSARA(RBASE(IBINF))
         CALL RLSARA(RBASE(IBSUP))
         CALL RLSARA(RBASE(ISCALE))
*
*---- METHODE DU LAGRANGIEN AUGMENTE
*
      ELSE IF (IMTHD.EQ.4) THEN
         M0B=M0-NCST
         NPMB=N0+M0B
         
         CALL SETARA(RBASE,NCST,ILAFAC)
         CALL SETARA(RBASE,NCST,ICTWGT)
         CALL SETARA(RBASE,2*(M0B+1)*NPMB,IAPLS2)
         CALL SETARA(RBASE,M0B+2,IB2)
         CALL SETARA(RBASE,2*N0,ILAGF)
         CALL SETARA(IBASE,M0B+1,IINPL2)
         CALL SETARA(RBASE,NCST,ICONT2)
         
         CALL PLLA(N0,M0,NPM,APLUS,PDG,BPLUS,INPLUS,XDROIT,FCOST,GF,
     >      FCOST,XOBJ,IMPR,EPSIM,
     >      M0B,NPMB,NCST,MEMADR,GRADC,CONTR,INEGAL,IERR,
     1      RBASE(ILAGF),RBASE(ILAFAC),RBASE(ICTWGT),RBASE(IAPLS2),
     2      RBASE(IB2),IBASE(IINPL2),RBASE(ICONT2))

         CALL RLSARA(IBASE(IINPL2))
         CALL RLSARA(RBASE(ILAGF))
         CALL RLSARA(RBASE(IB2))
         CALL RLSARA(RBASE(IAPLS2))
         CALL RLSARA(RBASE(ICTWGT))
         CALL RLSARA(RBASE(ILAFAC))
         CALL RLSARA(RBASE(ICONT2))
*
*---- METHODE DE PENALISATION EXTERNE
*
      ELSE IF (IMTHD.EQ.5) THEN
         M0B=M0-NCST
         NPMB=N0+M0B
         
         CALL SETARA(RBASE,NCST,ICTWGT)
         CALL SETARA(RBASE,2*(M0B+1)*NPMB,IAPLS2)
         CALL SETARA(RBASE,M0B+2,IB2)
         CALL SETARA(RBASE,2*N0,ILAGF)
         CALL SETARA(IBASE,M0B+1,IINPL2)
         CALL SETARA(RBASE,NCST,ICONT2)
         
         CALL PLPNLT(N0,M0,NPM,APLUS,PDG,BPLUS,INPLUS,XDROIT,FCOST,GF,
     >      FCOST,XOBJ,IMPR,EPSIM,
     >      M0B,NPMB,NCST,MEMADR,GRADC,CONTR,INEGAL,IERR,
     1      RBASE(ILAGF),RBASE(ICTWGT),RBASE(IAPLS2),
     2      RBASE(IB2),IBASE(IINPL2),RBASE(ICONT2))

         CALL RLSARA(IBASE(IINPL2))
         CALL RLSARA(RBASE(ILAGF))
         CALL RLSARA(RBASE(IB2))
         CALL RLSARA(RBASE(IAPLS2))
         CALL RLSARA(RBASE(ICTWGT))
         CALL RLSARA(RBASE(ICONT2))
*
      ENDIF
*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! A VERIFIER CAR SINON INUTILE
      IF (IERR.NE.0) THEN
         RETURN
      ENDIF
*
      RETURN
*
1000  FORMAT(//,5X,'INCONSISTANCE ENTRE M0 ET ME+MI ',3I5,//)
2000  FORMAT(//,5X,'TEMPS DE CALCUL DE OPTMAP : ',F10.5,' SECONDES',//)
      END

