*DECK PLLEMK
      SUBROUTINE PLLEMK(N,M,P,IROW,ICOL,IERR,IMPR,EPS,PLJ)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine solve the parametric linear complementary problem.
* PLLEMK = Linear Programmation LEMKe
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): OPTEX
*  Modified : R. Chambon
*
*Parameters: input/ouput
*     N         : NOMBRE DE VARIABLES DE CONTROLE.
*                 =2N0+M1 from PLLINR
*                 =M1+1   from PLQUAD                                  *
*     M         : =N+2    from PLLINR
*                 =N+3    from PLQUAD                                  *
*     P         : COEFFICIENT MATRIX. DIMENSION P(2*N*M)               *
*     IROW,ICOL : PERMUTATION VECTORS. DIMENSION IROW(N),ICOL(N+1)     *
*     IERR      : RETURN CODE. IERR.GE.1 MEANS FAILURE OF LEMKE        *
*     IMPR      : PRINT FLAG.                                          *
*     EPS       : ZERO OF THE LEMKE'S ALGORITHM.                       *
*     PLJ       : WORKING SPACE. DIMENSION PLJ(2*M)                    *
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT NONE
      INTEGER     M,N,IERR,IMPR,IROW(N),ICOL(N+1)
      DOUBLE PRECISION  P(N,M),PLJ(M)
      REAL        EPS
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION  WWW,S
      INTEGER     MNOP,NP1,IC,L,I,J,K,JJ,J1,LGAR

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*
      MNOP = N + M
      NP1  = N + 1
      IC = 0
*
      S = 0.0D0
      L = 0
*
      DO 50 I=1,N
         P(I,NP1) = 1.0D0
         IF (P(I,M).GE.0.0D0) GO TO 50
         WWW = P(I,M)
         IF (WWW.LE.S) THEN
            L = I
            S = WWW
         ENDIF
   50 CONTINUE
*
      IF (L.EQ.0) GO TO 290
      LGAR = L
*
      K = NP1
*
   90 IC = IC + 1
      IF (IC.GT.MNOP) THEN
         IERR = 1
         GOTO 300
      ENDIF
*
      IF (IMPR.GE.6) THEN
         WRITE (6,1000) L,K
         DO 110 J=1,M,12
            JJ = MIN0(J+11,M)
            WRITE (6,3000) (J1,J1=J,JJ)
            DO 100 I=1,N
               WRITE (6,4000) I,(P(I,J1),J1=J,JJ)
  100       CONTINUE
            IF (JJ.NE.M) WRITE (6,5000)
  110    CONTINUE
      ENDIF
*
      CALL PLPIVT(N,M,L,K,P,IROW,ICOL,PLJ)
*
      IF (ICOL(K).EQ.-NP1) GOTO 290
      DO 120 J=1,NP1
         IF (J.EQ.K) GOTO 120
         IF (IABS(ICOL(K)).EQ.IABS(ICOL(J))) THEN
            K = J
            GOTO 130
         ENDIF
  120 CONTINUE
*
      IERR = 2
      GOTO 300
*
  130 DO 140 I=1,N
         IF (P(I,K).LT.-EPS) THEN
            PLJ(I) = -P(I,M)/P(I,K)
         ELSE
            PLJ(I) = 1.0D50
         ENDIF
  140 CONTINUE
*
      S = PLJ(LGAR)
      L = LGAR
*
      IF (IMPR.GE.7) THEN
         WRITE (6,*) 'K=',K
         WRITE (6,6000) L,(PLJ(I),I=1,N)
      ENDIF
*
      DO 280 I=1,N
      WWW = PLJ(I)
      IF ((ABS(WWW-S).GT.ABS(S)*EPS).AND.(WWW.LT.S)) THEN
         S = WWW
         L = I
      ENDIF
  280 CONTINUE
*
      IF (S.EQ.1.0D50) THEN
         IERR = 3
         GOTO 300
      ENDIF
*
      GOTO 90
  290 IERR = 0
*
  300 IF (IMPR.GE.5) THEN
         WRITE (6,2000) IC
         DO 315 J=1,M,12
            JJ = MIN0(J+11,M)
            WRITE (6,3000) (J1,J1=J,JJ)
            DO 310 I=1,N
               WRITE (6,4000) I,(P(I,J1),J1=J,JJ)
  310       CONTINUE
            IF (JJ.NE.M) WRITE (6,5000)
  315    CONTINUE
      ENDIF
*
      RETURN
*
1000  FORMAT (//,10X,'+ + + + +   MATRICE P(N,N+3) :',5X,'PIVOT = (',
     >           I3,' ,',I3,' )',/)
2000  FORMAT (//,10X,'NOMBRE DE PIVOTS =',I5,
     >        //,10X,'+ + + + +   MATRICE FINALE P(N,N+3) :'/)
3000  FORMAT (1X,12I10)
4000  FORMAT (1X,I3,1P,10E12.5)
5000  FORMAT (//)
6000  FORMAT (//,10X,'LGAR =',I4,/,(1X,1P,8E12.5))
      END

