*DECK LINR
      SUBROUTINE PLLINR(N0,M1,N,NPM,COUT,APLUS,BPLUS,BINF,BSUP,XOBJ,
     >                EPS,IERR,IMPR,P,IROW,ICOL)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine minimizes a linear problem using LEMKE pivot.
* PLLINR = Linear Programmation LINeaR problem resolution
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): OPTEX
*  Modified : R. Chambon
*
*Parameters: input/ouput
*     N0      NOMBRE DE VARIABLES DE CONTROLE.
*     M1      NOMBRE DE CONTRAINTES (PLUS PETIT OU EGAL,=M0+1)
*     COUT    COUT DES VARIABLES DE CONTROLE. DIMENSION COUT(N0)
*     APLUS   COEFFICIENTS DES CONTRAINTES. DIMENSION A(M1,N0)
*     BPLUS   MEMBRES DE DROITE DES CONTRAINTES. DIMENSION BPLUS(M1+1)
*     BINF    BORNES INFERIEURES DES VARIABLES DE CONTROLE.
*     BSUP    BORNES SUPERIEURES DES VARIABLES DE CONTROLE.
*             DIMENSION BINF(N0),BSUP(N0)
*     XOBJ    VARIABLES DE CONTROLE. DIMENSION XOBJ(N0)
*     EPS     ZERO RELATIF DE L'ALGORITHME DE PIVOTAGE.
*     IERR    CODE DE RETOUR : .EQ.0 SOLUTION VALIDE.
*                              .GE.1 ECHEC DE L'ALGORITHME.
*     IMPR    .EQ.0 : PAS D'IMPRESSIONS ;
*             .GE.1 : DE PLUS EN PLUS D'IMPRESSIONS.
*     N       =2*N0+M1
*    NPM   : N0+M1  TAILLE DU SYSTEME LINEAIRE                        
*     P       ESPACE DE TRAVAIL. DIMENSION P(2*N*(N+2))
*     IROW    ESPACE DE TRAVAIL. DIMENSION IROW(N)
*     ICOL    ESPACE DE TRAVAIL. DIMENSION ICOL(N+1)
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT NONE
      INTEGER       N0,M1,N,IERR,IMPR,NPM
      INTEGER       IROW(N),ICOL(N+1)
      REAL          BPLUS(M1+1),BINF(N0),BSUP(N0),XOBJ(N0),EPS
      DOUBLE PRECISION  APLUS(M1,NPM),COUT(N0),P(N,N+2)
*----
*  LOCAL VARIABLES
*----
      INTEGER       NP1,NP2,I,J,IPLJ,II,IR
      DOUBLE PRECISION  XVAL
      CHARACTER*4   ROW(7)
      CHARACTER*6   CLNAME
      DATA CLNAME /'PLLINR'/

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*
*---- SET-UP AND SOLVE THE PARAMETRIC COMPLEMENTARITY PROBLEM.
*
!      N   = 2*N0 + M1
      NP1 = N + 1
      NP2 = N + 2
*
      DO 10 I=1,N
         DO 15 J=1,NP2
            P(I,J)=0.0D0
 15      CONTINUE
10    CONTINUE
*
      DO 20 I=1,N0
         P(I,NP2) = COUT(I)
         P(N0+M1+I,NP2) = BSUP(I) - BINF(I)
         P(I,N0+M1+I) =  1.0D0
         P(N0+M1+I,I) = -1.0D0
*
         DO 25 J=1,M1
            P(I,N0+J)=APLUS(J,I)
 25      CONTINUE
20    CONTINUE
*
      DO 40 I=1,M1
         XVAL = 0.0D0
         DO 30 J=1,N0
            XVAL = XVAL + APLUS(I,J)*BINF(J)
30       CONTINUE
         P(N0+I,NP2) = BPLUS(I) - XVAL
*
         DO 45 J=1,N0
            P(N0+I,J)=-APLUS(I,J)
  45     CONTINUE
 40   CONTINUE
*
      DO 50 I=1,N
         IROW(I)  =  I
         ICOL(I)  = -I
         P(I,NP1) = 1.0D0
 50   CONTINUE
      ICOL(NP1) = -NP1
*
      CALL SETARA(RBASE,2*NP2,IPLJ)
*
      CALL PLLEMK(N,NP2,P,IROW,ICOL,IERR,IMPR,EPS,RBASE(IPLJ))
*
      CALL RLSARA(RBASE(IPLJ))
*
      IF (IERR.GE.1) THEN
         WRITE(6,1000) IERR
         RETURN
      ENDIF
*
      IF (IMPR.GE.3) THEN
         WRITE(6,2000)
*
         DO 70 I=1,N,7
            II = MIN0(I+6,N)
            DO 60 J=I,II
               IF (IROW(J).LT.0) THEN
                  WRITE (ROW(J-I+1),'(1HX,I3.3)') (-IROW(J))
               ELSE
                  WRITE (ROW(J-I+1),'(1HY,I3.3)') IROW(J)
               ENDIF
   60       CONTINUE
            WRITE(6,3000) (ROW(J-I+1),P(J,NP2),J=I,II)
   70    CONTINUE
      ENDIF
*
      DO 80 I=1,N0
         XOBJ(I) = BINF(I)
 80   CONTINUE
*
      DO 90 I=1,N
         IR = -IROW(I)
         IF ((IR.GT.0).AND.(IR.LE.N0)) THEN
            XOBJ(IR)=BINF(IR)+P(I,NP2)
         ENDIF
   90 CONTINUE
*
      RETURN
*
1000  FORMAT(//,5X,'ECHEC DANS LA RESOLUTION DU P.L.C (LINR) IERR:',I5)
2000  FORMAT(//,5X,'SOLUTION DU P.L.C. (LINR) : ',
     >             '*** X: MULTIPLIEURS DE K-T ;',
     >          5X,'*** Y: VARIABLES D''ECART ',/)
3000  FORMAT(7(1X,A4,'=',E12.5),/)
      END

