*DECK PLMAP1
      SUBROUTINE PLMAP1(N0,M0,APLUS,PDG,BPLUS,INPLUS,
     >                XDROIT,COUT,OBJ,XOBJ,BINF,BSUP,
     >                SCALE,PX,RX,DELTA,BGAR,IMTHD,LCONTQ,IERR,IMPR)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine solves a linear optimization problem with quadratic 
*   constraints using the method of approximation programming (MAP). 
* PLMAP1 = Linear Programmation MAP1
*  See theory in:*                                                                      *
*    R.E. GRIFFITH AND R.A. STEWART, 'A NON-LINEAR PROGRAMMING TECHNIQU*
*    FOR THE OPTIMIZATION OF CONTINUOUS PROCESSING SYSTEMS', MANAGEMENT*
*    SCIENCE, VOL. 7, NO. 4, 379 (1961).                               *
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): OPTEX
*  Modified : R. Chambon
*
*Parameters: input/ouput
*    N0    : NOMBRE DE VARIABLES DE CONTROLE.                          *
*    M0    : NOMBRE DE CONTRAINTES.                                    *
*    APLUS : MATRICE DES COEFFICIENTS DES CONTRAINTES LINEAIRES.       *
*            DIMENSION APLUS(M0+1,N0)                                  *
*    PDG   : COEFFICIENTS DE LA CONTRAINTE QUADRATIQUE.                *
*            DIMENSION PDG(N0)                                         *
*    BPLUS : TERMES DE DROITE DES CONTRAINTES LINEAIRES.               *
*            DIMENSION B(M0+1)                                         *
*    INPLUS: RELATIONS DES CONTRAINTES : =-1 SI PLUS GRAND OU EGAL;    *
*                                        = 0 SI EGAL;                  *
*                                        = 1 SI PLUS PETIT OU EGAL.    *
*            DIMENSION INPLUS(M0+1)                                    *
*    XDROIT: TERME DE DROITE DE LA CONTRAINTE QUADRATIQUE.             *
*    COUT  : COUT DES VARIABLES DE CONTROLE. DIMENSION COUT(N0)        *
*    OBJ   : FONCTION OBJECTIF.                                        *
*    XOBJ  : VARIABLES DE CONTROLE. DIMENSION XOBJ(N0)                 *
*    IERR  : CODE DE RETOUR: IERR=0       : SUCCES DE L'ALGORITHME     *
*    IMPR  : INDEX D'IMPRESSION. IMPR.EQ.0 SIGNIFIE PAS D'IMPRESSIONS. *
*    IMTHD  : Numero de la methode employee 
*                 (IMTHD = 3 pour la methode MAP)
*    LCONTQ:
*    EPSIM : 
*    NOPTX :
*Parameters: scratch
*    BINF  : ESPACE DE TRAVAIL. DIMENSION BINF(N0)                     *
*    BSUP  : ESPACE DE TRAVAIL. DIMENSION BSUP(N0)                     *
*    SCALE : ESPACE DE TRAVAIL. DIMENSION SCALE(N0)                    *
*    PX    : ESPACE DE TRAVAIL. DIMENSION PX(N0)                       *
*    RX    : ESPACE DE TRAVAIL. DIMENSION RX(N0)                       *
*    DELTA : ESPACE DE TRAVAIL. DIMENSION DELTA(N0)                    *
*    BGAR  : ESPACE DE TRAVAIL. DIMENSION BGAR(M0+1)                   *
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   N0,M0,IERR,IMPR,IMTHD
      REAL      PDG(*),BPLUS(*) ,XOBJ(*) ,BINF(*),BSUP(*),SCALE(*),
     >          PX(*) ,RX(*),DELTA(*),BGAR(*),XDROIT
      DOUBLE PRECISION  APLUS(M0+1,*),COUT(*),OBJ
      INTEGER   INPLUS(*)
      LOGICAL   LCONTQ
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION  DELF,X,DUMY,ZMAX
      REAL        SCAL,DELX,TEMP,ERR,CONT,EPSIR,EPS,EPSS,XVAL
      INTEGER     ITER,MP2,NPM,IVAR,ICRE,IBASE0,IBORNE,IXIK,IBGAR,
     >            IAGAR,IGSUP,ITMAX,I,J,M,BGAR0
      CHARACTER*6   CLNAME
      DATA        CLNAME /'PLMAP'/

      INTEGER  IZMAXC

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*
      EPS   = 0.0001
      EPSIR =EPS
      ITMAX = 100
*
*---- SCALING DES VARIABLES
*
      DO 10 J=1,N0
         SCAL = SQRT(XDROIT/PDG(J))
         SCALE(J) = SCAL
         COUT(J) = COUT(J)*SCAL
*
         PDG(J)  = 1.0
         BINF(J) = 0.0
         BSUP(J) = 0.0
*
         DO 20 I=1,M0
            APLUS(I,J) = APLUS(I,J)*SCAL
   20    CONTINUE
*
  10  CONTINUE
*
*---- IMPRESSION DES TABLEAUX
*
      IF (IMPR.GE.5) THEN
         CALL PRNTAB(COUT,APLUS,INPLUS,BPLUS,PDG,BINF,BSUP,
     >               N0,M0,CLNAME//' APRES SCALING DES VARIABLES')
      ENDIF
*
      XDROIT=1.0
*
*---- SCALING DES CONTRAINTES
*
      CALL SETARA(RBASE,M0,IZMAXC)
      DO 30 I=1,M0
         ZMAX = ABS(BPLUS(I))
*
         DO 40 J=1,N0
            ZMAX = MAX(ZMAX,ABS(APLUS(I,J)))
   40    CONTINUE
         BGAR(I) = BPLUS(I)/ZMAX
*
         DO 42 J=1,N0
            APLUS(I,J) = APLUS(I,J)/ZMAX
   42    CONTINUE
         RBASE(IZMAXC+I-1) = ZMAX
  30  CONTINUE
*
*---- SCALING DES COUTS
*
      ZMAX = 0.0D0
      DO 45 J=1,N0
         ZMAX = MAX(ZMAX,ABS(COUT(J)))
   45 CONTINUE
*
      DO 50 J=1,N0
         COUT(J) = COUT(J)/ZMAX
   50 CONTINUE
*
*---- IMPRESSION DES TABLEAUX
*
      IF (IMPR.GE.5) THEN
         CALL PRNTAB(COUT,APLUS,INPLUS,BGAR,PDG,BINF,BSUP,N0,M0,
     >               CLNAME//' APRES SCALING DES CONTRAINTES/COUTS')
      ENDIF
      CALL SETARA(RBASE,M0,BGAR0)
      DO 52 I=1,M0
         RBASE(BGAR0+I-1) = BGAR(I)
   52 CONTINUE
*
*---- CALCUL DES ESTIMES INITIAUX
*
      DELX = SQRT(XDROIT)
      EPSS = EPS*DELX
*
      DO 55 I=1,N0
         DELTA(I) = DELX/10.0
         RX(I)    = 0.0
   55 CONTINUE
      TEMP = DELX/SQRT(REAL(N0))/10
*
*---- DEBUT DES ITERATIONS
*
      ITER = 0
   60 ITER = ITER + 1
      CONT = 0.0
*
      DO 70 I=1,M0+1
         BGAR(I) = RBASE(BGAR0+I-1)
   70 CONTINUE
*
*---- DETERMINATION DES BORNES ASSOCIEES AUX VARIABLES
*
      DO 90 I=1,N0
         IF (ITER.EQ.1) THEN
!            XOBJ(I) = EPSIR*10.0
            XOBJ(I) = 0.0
            BINF(I) = -TEMP
            BSUP(I) = TEMP
         ELSE
            BINF(I) = -DELTA(I)
            BSUP(I) = DELTA(I)
         ENDIF
*
*------- CONTRAINTE QUADRATIQUE SUR LE PAS D'AVANCE
*
         CONT = CONT + XOBJ(I)**2
         APLUS(M0+1,I) = 2.0*XOBJ(I)
*
         DO 95 J=1,M0
            BGAR(J) = BGAR(J) - APLUS(J,I)*XOBJ(I)
   95    CONTINUE
  90  CONTINUE
*
      INPLUS(M0+1) = 1
      BGAR(M0+1)   = XDROIT - CONT
*
      M = M0 + 1
*
*---- REORGANISATION DES TABLEAUX DU SIMPLEX
*
      DO 120 I=1,M
         DUMY = 0.0D0
*
         DO 100 J=1,N0
            DUMY = DUMY + APLUS(I,J)*BINF(J)
  100    CONTINUE
*
         BGAR(I) = BGAR(I) - DUMY
         IF (BGAR(I).GE.0.0) GOTO 120
*
         DO 110 J=1,N0
            APLUS(I,J) = -APLUS(I,J)
  110    CONTINUE
*
         BGAR(I)   = -BGAR(I)
         RBASE(BGAR0+I-1)= -RBASE(BGAR0+I-1)
         BPLUS(I)  = -BPLUS(I)
         INPLUS(I) = -INPLUS(I)
*
  120 CONTINUE
*
      DO 130 J=1,N0
         BSUP(J) = BSUP(J) - BINF(J)
         BINF(J) = 0.0
  130 CONTINUE
*
*---- IMPRESSION DES TABLEAUX
*
      IF (IMPR.GE.5) THEN
         CALL PRNTAB(COUT,APLUS,INPLUS,BGAR ,XOBJ  ,BINF  ,BSUP,N0,M0,
     >               CLNAME//' APRES REORGANISATION POUR LE SIMPLEX')
      ENDIF
*
      MP2 = M + 2
      NPM = N0 + M
*
      CALL SETARA(RBASE,MP2,IVAR)
      CALL SETARA(RBASE,NPM,ICRE)
      CALL SETARA(RBASE,2*MP2*MP2,IBASE0)
      CALL SETARA(RBASE,NPM,IBORNE)
      CALL SETARA(RBASE,MP2,IXIK)
      CALL SETARA(RBASE,MP2,IBGAR)
      CALL SETARA(RBASE,2*2*NPM,IAGAR)
      CALL SETARA(RBASE,NPM,IGSUP)
*
      CALL PLSPLX(N0,M,M,NPM,1,COUT,APLUS,BGAR,INPLUS,BINF,BSUP,PX,DELF,
     >          EPSS,IERR,IMPR,RBASE(IVAR),RBASE(ICRE),RBASE(IBASE0),
     >          RBASE(IBORNE),RBASE(IXIK),RBASE(IBGAR),RBASE(IAGAR),
     >          RBASE(IGSUP),IMTHD)
*
      CALL RLSARA(RBASE(IVAR))
      CALL RLSARA(RBASE(ICRE))
      CALL RLSARA(RBASE(IBASE0))
      CALL RLSARA(RBASE(IBORNE))
      CALL RLSARA(RBASE(IXIK))
      CALL RLSARA(RBASE(IBGAR))
      CALL RLSARA(RBASE(IAGAR))
      CALL RLSARA(RBASE(IGSUP))
*
      DO 140 I=1,N0
         IF (ITER.EQ.1) THEN
            PX(I) = PX(I) - TEMP
         ELSE
            PX(I) = PX(I) - DELTA(I)
         ENDIF
  140 CONTINUE
*
*---- RESULTAT DE L'ITERATION COURANTE
*
      IF (IMPR.GE.2) THEN
         IF (((ITER.GE.1).AND.(IMPR.LE.2)).OR.(IMPR.GE.3)) THEN
            WRITE (6,1000)
         ENDIF
         WRITE (6,2000) ITER,DELF,(PX(I),I=1,N0)
      ENDIF
*
*---- DEGENERESCENCE OU EPS TROP PETIT
*
      IF (IERR.EQ.1) THEN
         WRITE(6,3000) ITER
         IERR = 3
         RETURN
*
*---- PAS DE SOLUTION AU PROBLEME SI ITER=1
*
      ELSE IF (IERR.EQ.2) THEN
         IF (IMPR.GE.1) THEN
            WRITE(6,4000) ITER
         ENDIF
*
!         IF ((ITER.EQ.1).OR.(ITER.GE.ITMAX)) THEN
         IF (ITER.GE.ITMAX) THEN
            RETURN
         ENDIF
*
!         IF (ITER.NE.1) THEN
!            IERR = 0
!         ENDIF
*
!         DO 150 I=1,N0
!            XOBJ(I)  = XOBJ(I) - RX(I)
!            RX(I)    = 0.0
!            DELTA(I) = DELTA(I)*0.5
!  150    CONTINUE
*
!         GO TO 60
      ENDIF
*
      ERR = 0.0
      DO 160 I=1,N0
*
         IF ((RX(I)*PX(I).LT.0.0).AND.(IERR.EQ.0)) THEN
            DELTA(I) = DELTA(I)*0.5
         ENDIF
*
         RX(I)   = PX(I)
         XOBJ(I) = XOBJ(I) + PX(I)
         ERR = ERR + PX(I)**2
  160 CONTINUE
*
      ERR  = SQRT(ERR)
      EPSS = EPS*DELX/10.0
*
      IF (IMPR.GE.1) THEN
         WRITE(6,2000) ITER,DELF,(XOBJ(I),I=1,N0)
         WRITE(6,2000) ITER,0.0,(DELTA(I),I=1,N0)
      ENDIF
*
!      IF (ERR.LE.EPSIR) THEN
      IF (ERR.LE.EPSS) THEN
         IERR = 0
         GOTO 170
      ENDIF
*
      IF (ITER.GE.ITMAX) THEN
         IERR = 5
         WRITE (6,5000) ITER
         RETURN
      ENDIF
      GO TO 60
*
*---- RESCALE ET IMPRESSION DE LA SOLUTION
*
  170 DO 175 J=1,N0
         SCAL    = SCALE(J)
         COUT(J) = COUT(J)*ZMAX/SCAL
         XOBJ(J) = XOBJ(J)*SCAL
         PDG(J)  = 1.0/SCAL**2
*
         DO 177 I=1,M0
            APLUS(I,J) = APLUS(I,J)/SCAL
  177    CONTINUE
 175  CONTINUE
*
      X   = 0.0D0
      OBJ = 0.0D0
*
      DO 180 J=1,N0
         X   = X   + PDG(J)*XOBJ(J)*XOBJ(J)
         OBJ = OBJ + XOBJ(J)*COUT(J)
 180  CONTINUE
*
      IF (IMPR.GE.1) THEN
         WRITE (6,6000) OBJ,X,(XOBJ(J),J=1,N0)
         WRITE (6,7000)
*
         DO 190 I=1,M0
            XVAL = BPLUS(I)
            DO 185 J=1,N0
  185          XVAL = XVAL - APLUS(I,J)*XOBJ(J)
     1                          *RBASE(IZMAXC+I-1)/SCALE(J)
            WRITE (6,8000) I,XVAL
 190     CONTINUE
*
      ENDIF
      CALL RLSARA(RBASE(IZMAXC))
      CALL RLSARA(RBASE(BGAR0))
*
      RETURN
*
1000  FORMAT(/,5X,'ITERATION',8X,'DELF',5X,'VARIABLES DE DECISION ')
2000  FORMAT(5X,I6,5X,8E12.5,/,(28X,5E12.5))
3000  FORMAT(5X,I6,5X,'DEGENERESCENCE OU EPS TROP PETIT')
4000  FORMAT(5X,I6,5X,'PAS DE SOLUTION')
5000  FORMAT(5X,I6,5X,'NOMBRE MAXIMIM D''ITERATIONS ATTEINT')
6000  FORMAT(//,5X,'SOLUTION FINALE (MAP1-SIMPLEX)',
     >        /,5X,'------------------------------',
     >        /,5X,'FONCTION OBJECTIF          : ',1P,E12.5,
     >        /,5X,'CONTRAINTE QUADRATIQUE     : ',1P,E12.5,
     >        /,5X,'VARIABLES DE CONTROLE      : ',/,(1X,10E12.5))
7000  FORMAT(//,5X,'ECARTS SUR LES CONTRAINTES : ',/)
8000  FORMAT(2X,I3,'...',2X,1P,E12.5)
      END

