*DECK PLMAP2
      SUBROUTINE PLMAP2(N0,M0,NPM,APLUS,PDG,BPLUS,INPLUS,XDROIT,COUT,
     >                OBJ,XOBJ,BINF,BSUP,SCALE,IMTHD,LCONTQ,IERR,IMPR,
     >                EPSIM)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine solves a linear optimization problem with quadratic 
*   constraints using the method of LEMKE. 
* PLMAP2 = Linear Programmation MAP2
*  See theory in:*                                                                      *
*    J. A. FERLAND, 'A LINEAR PROGRAMMING PROBLEM WITH AN ADDITIONAL   *
*    QUADRATIC CONSTRAINT SOLVED BY PARAMETRIC LINEAR COMPLEMENTARITY',*
*    PUBLICATION NUMBER 497, DEPARTEMENT D'INFORMATIQUE ET DE RECHERCHE*
*    OPERATIONNELLE, UNIVERSITE DE MONTREAL, JANUARY 1984.             *
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): OPTEX
*  Modified : R. Chambon
*
*Parameters: input/ouput
*    N0    : NOMBRE DE VARIABLES DE CONTROLE.                          *
*    M0    : NOMBRE DE CONTRAINTES LINEAIRES.                          *
*    NPM   : N0+(M0+1)  TAILLE DU SYSTEME LINEAIRE                     *
*    APLUS : MATRICE DES COEFFICIENTS DES CONTRAINTES LINEAIRES.       *
*            DIMENSION A(M0+1,NPM)                                     *
*    PDG   : COEFFICIENTS DE LA CONTRAINTE QUADRATIQUE.                *
*            DIMENSION PDG(N0)                                         *
*    B     : TERMES DE DROITE DES CONTRAINTES LINEAIRES.               *
*            DIMENSION B(M0+2)                                         *
*    INPLUS: RELATIONS DES CONTRAINTES : =-1 SI PLUS GRAND OU EGAL;    *
*                                        = 0 SI EGAL;                  *
*                                        = 1 SI PLUS PETIT OU EGAL.    *
*            DIMENSION INPLUS(M0)                                      *
*    XDROIT: TERME DE DROITE DE LA CONTRAINTE QUADRATIQUE.             *
*    COUT  : COUT DES VARIABLES DE CONTROLE. DIMENSION COUT(N0)        *
*    OBJ   : FONCTION OBJECTIF.                                        *
*    XOBJ  : VARIABLES DE CONTROLE. DIMENSION XOBJ(N0)                 *
*    IERR  : CODE DE RETOUR: IERR=0       : SUCCES DE L'ALGORITHME     *
*                            IERR.GT.0    : ECHEC DE LINR              *
*                            IERR.GT.10   : ECHEC DE QUADR             *
*    IMPR  : INDEX D'IMPRESSION. IMPR.EQ.0 SIGNIFIE PAS D'IMPRESSIONS. *
*    IMTHD : CHOIX D'OPTION: .EQ.1 SIMPLEX / LEMKE                     *
*                            .EQ.2 LEMKE / LEMKE                       *
*    LCONTQ:
*    EPSIM :
* SCRATCH STORAGE:                                                     *
*    BINF  : ESPACE DE TRAVAIL. DIMENSION BINF(N0)                     *
*    BSUP  : ESPACE DE TRAVAIL. DIMENSION BSUP(N0)                     *
*    SCALE : ESPACE DE TRAVAIL. DIMENSION SCALE(N0)                    *
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT NONE
C INPUT VARIABLES
      INTEGER     N0,M0,NPM,IMPR,IERR,IMTHD
      INTEGER     INPLUS(M0+1)
      REAL        PDG(N0)  ,BPLUS(M0+2),XOBJ(N0),BINF(N0),BSUP(N0),
     >            SCALE(N0),EPSIM,XDROIT
      DOUBLE PRECISION  APLUS(M0+1,NPM),COUT(N0),OBJ
      LOGICAL       LCONTQ
*----
*  LOCAL VARIABLES
*----
      CHARACTER*6   CLNAME
*
      DATA CLNAME /'PLMAP2'/
*
C MODULE VARIABLES
      DOUBLE PRECISION  X,XNORM,ZMAX,XVAL
      REAL        SCAL,EPS
      INTEGER     I,J,M0NEW,MP2,IVAR,ICRE,ITAB,IBORNE,IXIK,IBGAR,
     >            IAGAR,IGSUP,N,NP1,NP2,IP,IIROW,IICOL,NP3,
     >            IU,IV,IPLJ,IWRK

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*
      EPS = EPSIM
*
*---- SCALING DES VARIABLES
*
      DO 10 J=1,N0
         SCAL = SQRT(XDROIT/PDG(J))
         SCALE(J) = SCAL
         COUT(J) = COUT(J)*SCAL
*
         PDG(J)  = 1.0
         BINF(J) = 0.0
         BSUP(J) = 0.0
*
         DO 20 I=1,M0
            APLUS(I,J) = APLUS(I,J)*SCAL
   20    CONTINUE
*
  10  CONTINUE
*
*---- IMPRESSION DES TABLEAUX
*
      IF (IMPR.GE.5) THEN
         CALL PRNTAB(COUT,APLUS,INPLUS,BPLUS,PDG,BINF,BSUP,
     >               N0,M0,CLNAME//' APRES SCALING DES VARIABLES')
      ENDIF
*
      XDROIT = 1.0
*
*---- SCALING DES CONTRAINTES
*
      DO 30 I=1,M0
         ZMAX = ABS(BPLUS(I))
*
         DO 40 J=1,N0
            ZMAX = MAX(ZMAX,ABS(APLUS(I,J)))
   40    CONTINUE
         BPLUS(I) = BPLUS(I)/ZMAX
*
         DO 42 J=1,N0
            APLUS(I,J) = APLUS(I,J)/ZMAX
   42    CONTINUE
  30  CONTINUE
*
*---- SCALING DES COUTS
*
      ZMAX = 0.0D0
      DO 45 J=1,N0
         ZMAX = MAX(ZMAX,ABS(COUT(J)))
   45 CONTINUE
*
      DO 50 J=1,N0
         COUT(J) = COUT(J)/ZMAX
   50 CONTINUE
*
*---- IMPRESSION DES TABLEAUX
*
      IF (IMPR.GE.5) THEN
         CALL PRNTAB(COUT,APLUS,INPLUS,BPLUS,PDG,BINF,BSUP,N0,M0,
     >               CLNAME//' APRES SCALING DES CONTRAINTES/COUTS')
      ENDIF
*
!      GOTO (200,300),ICODE
*
*---- STEP 1
*
  200 M0NEW = M0 + 1
      DO 55 I=1,N0
         BINF(I) = -1.0
         BSUP(I) =  1.0
         APLUS(M0NEW,I) = 0.0D0
   55 CONTINUE
*
      BPLUS(M0NEW) = 0.0
*
*---- RESOLUTION DU PROBLEME LINEAIRE PAR LE SIMPLEX
*
      IF (IMTHD.EQ.1) THEN
*
         MP2 = M0NEW + 2
*
         CALL SETARA(RBASE,MP2,IVAR)
         CALL SETARA(RBASE,NPM,ICRE)
         CALL SETARA(RBASE,2*MP2*MP2,ITAB)
         CALL SETARA(RBASE,NPM,IBORNE)
         CALL SETARA(RBASE,MP2,IXIK)
         CALL SETARA(RBASE,MP2,IBGAR)
         CALL SETARA(RBASE,2*2*NPM,IAGAR)
         CALL SETARA(RBASE,NPM,IGSUP)
*
         CALL PLSPLX (N0    ,M0     ,M0NEW  ,NPM    ,1       ,
     >                COUT  ,APLUS  ,BPLUS  ,INPLUS ,
     >                BINF  ,BSUP   ,XOBJ   ,OBJ    ,
     >                EPS   ,IERR   ,IMPR   ,
     >                RBASE(IVAR)   ,RBASE(ICRE)    ,RBASE(ITAB),
     >                RBASE(IBORNE) ,RBASE(IXIK)    ,
     >                RBASE(IBGAR)  ,RBASE(IAGAR)   ,RBASE(IGSUP),IMTHD)
*
         CALL RLSARA(RBASE(IVAR))
         CALL RLSARA(RBASE(ICRE))
         CALL RLSARA(RBASE(ITAB))
         CALL RLSARA(RBASE(IBORNE))
         CALL RLSARA(RBASE(IXIK))
         CALL RLSARA(RBASE(IBGAR))
         CALL RLSARA(RBASE(IAGAR))
         CALL RLSARA(RBASE(IGSUP))
*
*------- REORGANISATION DES TABLEAUX
*
         DO 70 I=1,M0
            IF (INPLUS(I).EQ.-1) THEN
               DO 60 J=1,N0
   60             APLUS(I,J) = -APLUS(I,J)
                  BPLUS(I)   = -BPLUS(I)
                  INPLUS(I)  = 1
            ELSE IF (INPLUS(I).EQ.0) THEN
               DO 65 J=1,N0
                  APLUS(M0NEW,J) = APLUS(M0NEW,J) - APLUS(I,J)
   65          CONTINUE
               BPLUS(M0NEW)   = BPLUS(M0NEW)   - BPLUS(I)
            ENDIF
   70    CONTINUE
*
*---- RESOLUTION PAR LA METHODE LEMKE
*
      ELSE
*
         DO 90 I=1,M0
            IF (INPLUS(I).EQ.-1) THEN
               DO 75 J=1,N0
                  APLUS(I,J) = -APLUS(I,J)
   75          CONTINUE
               BPLUS(I)  = -BPLUS(I)
               INPLUS(I) = 1
            ELSE IF (INPLUS(I).EQ.0) THEN
               DO 80 J=1,N0
                  APLUS(M0NEW,J) = APLUS(M0NEW,J) - APLUS(I,J)
   80          CONTINUE
               BPLUS(M0NEW) = BPLUS(M0NEW) - BPLUS(I)
            ENDIF
   90    CONTINUE
*
*------- RESOLUTION DU PROBLEME LINEAIRE SANS CONTRAINTE
*
         N   = 2*N0 + M0NEW
         NP1 = N + 1
         NP2 = N + 2
*
         CALL SETARA(RBASE,2*N*NP2,IP)
         CALL SETARA(IBASE,N,IIROW)
         CALL SETARA(IBASE,NP1,IICOL)
*
         CALL PLLINR(N0,M0NEW,N,NPM,COUT,APLUS,BPLUS,BINF,BSUP,XOBJ,
     >           EPS,IERR,IMPR,RBASE(IP),IBASE(IIROW),IBASE(IICOL))
*
         CALL RLSARA(RBASE(IP))
         CALL RLSARA(IBASE(IIROW))
         CALL RLSARA(IBASE(IICOL))
      ENDIF
*
      IF (IERR.GE.1) THEN
         WRITE (6,6000) IERR
         RETURN
      ENDIF
*
      OBJ = 0.0D0
      DO 100 I=1,N0
         OBJ = OBJ + COUT(I)*XOBJ(I)
  100 CONTINUE
*
      X = 0.0D0
      DO 110 J=1,N0
         X = X + PDG(J)*XOBJ(J)*XOBJ(J)
  110 CONTINUE
*
      IF (IMPR.GE.2) THEN
         IF (IMTHD.EQ.1) THEN
            WRITE (6,1000) OBJ,X,(XOBJ(I),I=1,N0)
         ELSE IF (IMTHD.EQ.2) THEN
            WRITE (6,1500) OBJ,X,(XOBJ(I),I=1,N0)
         ENDIF
      ENDIF
*
      IF (IMPR.GE.5) THEN
         WRITE(6,*) 'XOBJ',(XOBJ(J),J=1,N0)
         WRITE(6,*) 'PDG ',(PDG(J),J=1,N0)
         WRITE(6,*) 'X',X,'XDROIT',XDROIT
      ENDIF
      IF(X.GT.XDROIT) GOTO 150
      GOTO 160
*
*---- PRISE EN COMPTE DE LA CONTRAINTE QUADRATIQUE
*
  300 XNORM = 0.0D0
      DO 130 I=1,N0
         XOBJ(I) = -COUT(I)/PDG(I)
         XNORM   = XNORM + XOBJ(I)**2
  130 CONTINUE
*
      XNORM = SQRT(XDROIT/XNORM)
      OBJ   = 0.0D0
      DO 140 I=1,N0
         XOBJ(I) = XOBJ(I)*XNORM
         OBJ     = OBJ + COUT(I)*XOBJ(I)
  140 CONTINUE
*
*---- METHODE DE LEMKE POUR UN PROBLEME QUADRATIQUE
*
 150  N   = M0NEW + 1
      NP1 = N + 1
      NP2 = N + 2
      NP3 = N + 3
*
      BPLUS(N) = OBJ
*
      CALL SETARA(RBASE,2*N*NP3,IP)
      CALL SETARA(IBASE,N,IIROW)
      CALL SETARA(IBASE,NP1,IICOL)
      CALL SETARA(RBASE,N,IU)
      CALL SETARA(RBASE,N,IV)
      CALL SETARA(RBASE,2*NP3,IPLJ)
      CALL SETARA(RBASE,2*N0,IWRK)
*
      CALL PLQUAD(N0,M0NEW,NPM,APLUS,BPLUS,PDG,XDROIT,COUT,XOBJ,EPS,
     >         IERR,IMPR,RBASE(IP),IBASE(IIROW),IBASE(IICOL),RBASE(IU),
     >         RBASE(IV),RBASE(IPLJ),RBASE(IWRK))
*
      CALL RLSARA(RBASE(IP))
      CALL RLSARA(IBASE(IIROW))
      CALL RLSARA(IBASE(IICOL))
      CALL RLSARA(RBASE(IU))
      CALL RLSARA(RBASE(IV))
      CALL RLSARA(RBASE(IPLJ))
      CALL RLSARA(RBASE(IWRK))
*
      IF (IERR.GE.1) THEN
         WRITE(6,2000) IERR
         IERR = IERR + 10
         RETURN
      ENDIF
*
*---- RESCALE ET IMPRESSION DE LA SOLUTION
*
  160 DO 170 J=1,N0
         SCAL = SCALE(J)
         COUT(J) = COUT(J)*ZMAX/SCAL
         XOBJ(J) = XOBJ(J)*SCAL
         PDG(J)  = 1.0/SCAL**2
*
         DO 175 I=1,M0
            APLUS(I,J) = APLUS(I,J)/SCAL
  175    CONTINUE
 170  CONTINUE
*
*---- CALCUL DU NOUVEAU POINT OPTIMAL
*
      X   = 0.0D0
      OBJ = 0.0D0
*
      DO 180 J=1,N0
         X   = X   + PDG(J)*XOBJ(J)*XOBJ(J)
         OBJ = OBJ + XOBJ(J)*COUT(J)
  180 CONTINUE
*
      IF (IMPR.GE.1) THEN
         WRITE (6,3000) OBJ,X,(XOBJ(J),J=1,N0)
         WRITE (6,4000)
*
         DO 190 I=1,M0
            XVAL = BPLUS(I)
            DO 185 J=1,N0
               XVAL = XVAL - APLUS(I,J)*XOBJ(J)
  185       CONTINUE
            WRITE (6,5000) I,XVAL
 190     CONTINUE
*
      ENDIF
      RETURN
*
1000  FORMAT(//,5X,'SOLUTION SANS CONTRAINTE QUADRATIQUE (SIMPLEX) :',
     >        /,5X,'------------------------------------------------',
     >        /,5X,'FONCTION OBJECTIF     : ',1P,E12.5,
     >        /,5X,'CONTRAINTE QUADRATIQUE: ',1P,E12.5,
     >        /,5X,'VARIABLES DE CONTROLE : ',/,(10X,10E12.5))
1500  FORMAT(//,5X,'SOLUTION SANS CONTRAINTE QUADRATIQUE (LINR) :',
     >        /,5X,'---------------------------------------------',
     >        /,5X,'FONCTION OBJECTIF     : ',1P,E12.5,
     >        /,5X,'CONTRAINTE QUADRATIQUE: ',1P,E12.5,
     >        /,5X,'VARIABLES DE CONTROLE : ',/,(10X,10E12.5))
2000  FORMAT(//,5X,'ECHEC DU MODULE QUADR  IERR = ',I2)
3000  FORMAT(//,5X,'SOLUTION FINALE     :',
     >        /,5X,'---------------------',
     >        /,5X,'FONCTION OBJECTIF      : ',1P,E12.5,
     >        /,5X,'CONTRAINTE QUADRATIQUE : ',1P,E12.5,
     >        /,5X,'VARIABLES DE CONTROLE  : ',/,(10X,10E12.5))
4000  FORMAT(//,5X,'ECARTS DES CONTRAINTES :',/)
5000  FORMAT(2X,I3,'...',2X,1P,E12.5)
6000  FORMAT(//,5X,'ECHEC DE L''ALGORITHME LINEAIRE IERR =',I5)
      END

