*DECK PLPIVT
      SUBROUTINE PLPIVT(N,M,L,K,P,IROW,ICOL,PLJ)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine performs a pivot (L,K) on matrix P.                  *
* PLPIVT = Linear Programmation PIVoT
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): OPTEX
*  Modified : R. Chambon
*
*Parameters: input/ouput
*     N         : NOMBRE DE VARIABLES DE CONTROLE.
*                 =2N0+M1 from PLLINR
*                 =M1+1   from PLQUAD                                  *
*     M         : =N+2    from PLLINR
*                 =N+3    from PLQUAD                                  *
*     L,K       : 
*     P         : COEFFICIENT MATRIX. DIMENSION P(2*N*M)               *
*     IROW,ICOL : PERMUTATION VECTORS. DIMENSION IROW(N),ICOL(N+1)     *
*     PLJ       : WORKING SPACE. DIMENSION PLJ(2*M)                    *
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT NONE
      INTEGER     M,N,L,K,IROW(N),ICOL(N+1)
      DOUBLE PRECISION  P(N,M) ,PLJ(M)
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION  PVT
      INTEGER     I,J
*
      PVT = 1.0D0/P(L,K)
      DO 10 J=1,M
         PLJ(J) = -PVT*P(L,J)
         P(L,J) =  PLJ(J)
   10 CONTINUE
*
      DO 30 I=1,N
         IF (I.EQ.L) GO TO 30
         DO 20 J=1,M
            IF (J.EQ.K) GO TO 20
            P(I,J) = P(I,J) + PLJ(J)*P(I,K)
   20    CONTINUE
         P(I,K) = PVT*P(I,K)
   30 CONTINUE
*
      P(L,K) = PVT
      I = IROW(L)
      IROW(L) = ICOL(K)
      ICOL(K) = I
*
      RETURN
      END

