*DECK PLPNLT
      SUBROUTINE PLPNLT(N0,M0,NPM,APLUS,PDG,BPLUS,INPLUS,XDROIT,FCOST,
     >            GF,OBJ,XOBJ,IMPR,EPSIM,M0B,NPMB,NCST,MEMADR,GRADC,
     >            CONTR,INEGAL,IERR,LAGF,CSTWGT,APLUS2,B2,INPL2)
*----------------------------------------------------------------------*
*                                                                      *
*Purpose:
* this subroutine solves the quasi-linear problem using the external 
* penalty function.
* PLPNLT = Linear Programmation external PeNaLTy function
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
*    N0    : NOMBRE DE VARIABLES DE CONTROLE.                          *
*    M0    : NOMBRE DE CONTRAINTES LINEAIRES.                          *
*    NPM   : N0+(M0+1)  TAILLE DU SYSTEME LINEAIRE                     *
*    APLUS : MATRICE DES COEFFICIENTS DES CONTRAINTES LINEAIRES.       *
*            DIMENSION A(M0+1,NPM)                                     *
*    PDG   : COEFFICIENTS DE LA CONTRAINTE QUADRATIQUE.                *
*            DIMENSION PDG(N0)                                         *
*    BPLUS : TERMES DE DROITE DES CONTRAINTES LINEAIRES.               *
*            DIMENSION B(M0+2)                                         *
*    INPLUS: RELATIONS DES CONTRAINTES : =-1 SI PLUS GRAND OU EGAL;    *
*                                        = 0 SI EGAL;                  *
*                                        = 1 SI PLUS PETIT OU EGAL.    *
*            DIMENSION INPLUS(M0)                                      *
*    XDROIT: TERME DE DROITE DE LA CONTRAINTE QUADRATIQUE.             *
*    GF  : COUT DES VARIABLES DE CONTROLE. DIMENSION GF(N0)            *
*    OBJ   : FONCTION OBJECTIF.                                        *
*    XOBJ  : VARIABLES DE CONTROLE. DIMENSION XOBJ(N0)                 *
*    IMPR  : INDEX D'IMPRESSION. IMPR.EQ.0 SIGNIFIE PAS D'IMPRESSIONS. *
*    EPSIM  : 
*    MINMAX: =-1 POUR MAXIMISER L'OBJECTIF ; =1 POUR LE MINIMISER.     *
*    M0B   : NOMBRE DE CONTRAINTES LINEAIRES pour le LA, correspond au *
*            nombre de borne possiblement actives.                     *
*    NPMB  : N0+(M0B+1)  TAILLE DU SYSTEME LINEAIRE pour le LA.        *
*    NCST  : NOMBRE DE CONTRAINTES LINEAIRES du probleme d'origine     *
*    LAGF  : COUT DES VARIABLES DE CONTROLE pour le LA. dim LAGF(N0)   *
*    CSTWGT: poids des contraintes. dim  CSTWGT(NCST)                  *
*    APLUS2: MATRICE DES COEFFICIENTS DES CONTRAINTES LINEAIRES pour   *
*            le LA.     DIMENSION A(M0B+1,NPMB)                        *
*    B2    : TERMES DE DROITE DES CONTRAINTES LINEAIRES pour le LA.    *
*            DIMENSION B(M0B+2)                                        *
*    MEMADR: Adress memory of lcm object (description OPTVDF.f)
*
* ATTENTION CONTR(J)= objectif - fonction
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   N0,NCST,M0,NPM,M0B,NPMB,IMPR,MEMADR(20),IERR
      INTEGER   INPLUS(M0+1),INPL2(M0B+1),INEGAL(NCST)
      REAL      PDG(N0)  ,BPLUS(M0+2),XOBJ(N0),
     >          XDROIT,CSTWGT(NCST),B2(M0B+2),
     >          CONTR(NCST),EPSIM,CONTR2(NCST)
      DOUBLE PRECISION  APLUS(M0+1,NPM),GF(N0),OBJ,APLUS2(M0B+1,NPMB),
     >            GRADC(N0,NCST),LAGF(N0),FCOST,LACOST
*----
*  LOCAL VARIABLES
*----
      INTEGER   ITERMX
      PARAMETER (ITERMX=10)
      INTEGER   LENGT,ITYP,IBINF,IBSUP,ISCALE,I,J,K,IPOPT,ITER,LAGFR
      LOGICAL   LCST(NCST),LCST2(NCST),LTST
      REAL      NORM,CRIT,LA0E

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))

C    STEP 0: Initialisation
      IPOPT=MEMADR(6)

      CALL LCMLEN(IPOPT,'CST-WEIGHT',LENGT,ITYP)
      IF (LENGT.EQ.0) THEN
         CALL XABORT('PLPNLT: CONSTRAINTS PENALTIES NON INITIALIZED')
      ELSEIF (LENGT.EQ.NCST) THEN
         CALL LCMGET(IPOPT,'CST-WEIGHT',CSTWGT)
      ELSE
         CALL XABORT('PLPNLT: WRONG NUMBER OF CONSTRAINT WEIGHTS')
      ENDIF
      DO 1 J=1,NCST
    1    CONTR2(J)=-CONTR(J)
      
      CALL XDRSET(XOBJ,N0,0.0)

      DO 10 J=1,NCST
         LCST2(J)=.TRUE.
   10    LCST(J) =.TRUE.
      
      ITER=0

C     ITERATION INTERNE AU PROBLEME LINEARISE
   99 CONTINUE
      LTST=.TRUE.
C    STEP 1: Penalty function evaluation
      DO 110 J=1,NCST
         IF (INEGAL(J).NE.0) THEN
            CRIT=CONTR2(J)
            DO 100 I=1,N0
               CRIT=CRIT+GRADC(I,J)*XOBJ(I)
  100          CONTINUE
            CRIT=INEGAL(J)*CRIT
            LCST(J)=(CRIT.LE.0.0)
         ENDIF
  110    CONTINUE

      DO 150 I=1,N0
         LAGF(I)=GF(I)
         DO 140 J=1,NCST
            IF (INEGAL(J).EQ.0) THEN
               LAGF(I)=LAGF(I)+GRADC(I,J)*CSTWGT(J)*CONTR2(J)
            ELSEIF (.NOT.LCST(J)) THEN
               LAGF(I)=LAGF(I)+GRADC(I,J)*CSTWGT(J)*CONTR2(J)
            ENDIF
  140       CONTINUE
  150    CONTINUE

      LACOST=FCOST
      DO 160 J=1,NCST
         IF (INEGAL(J).EQ.0) THEN
            LACOST=LACOST+CSTWGT(J)/2.0*CONTR2(J)**2
         ELSEIF (.NOT.LCST(J)) THEN
            LACOST=LACOST+CSTWGT(J)/2.0*CONTR2(J)**2
         ENDIF
  160    CONTINUE
      IF (ITER.EQ.0) LA0E=LACOST
      IF (IMPR.GE.3) THEN
         WRITE(6,*) 'GF',(GF(I),I=1,N0)
         WRITE(6,*) 'LAGF',(LAGF(I),I=1,N0)
         WRITE(6,*) 'PDG',(PDG(I),I=1,N0)
         WRITE(6,*) 'LACOST',LACOST,'M0B',M0B,'XDROIT',XDROIT
      ENDIF

C    STEP 2: Resolution
C case 1
C If there is no constraints for the LA problem (M0B=0), 
C    then the solution is obvious: on the hypersphere(radius XDROIT) 
C    in the direction LAGF
      IF (M0B.EQ.0) THEN
*
      NORM=0.0
      DO 200 I=1,N0
  200    NORM=NORM+LAGF(I)**2/PDG(I)
      NORM=NORM**0.5
*
      IF (NORM.EQ.0.0) THEN
         CALL XDRSET(XOBJ,N0,0.0)
      ELSE
         DO 210 I=1,N0
  210       XOBJ(I)=-XDROIT**0.5*LAGF(I)/PDG(I)/NORM
      ENDIF
C case 2
C Resolution with the LEMKE method: appel a PLMAP1.f
      ELSE
*
      DO 260 K=1,M0B
         DO 250 I=1,N0
  250       APLUS2(K,I)=APLUS(NCST+K,I)
         B2(K)=BPLUS(NCST+K)
         INPL2(K)=INPLUS(NCST+K)
  260    CONTINUE
      DO 270 I=1,N0
         APLUS2(M0B+1,I) = 0.0D0
  270    CONTINUE
      BPLUS(M0B+1)   = 0.0
      INPL2(M0B+1)  = 0

      CALL SETARA(RBASE,N0,IBINF)
      CALL SETARA(RBASE,N0,IBSUP)
      CALL SETARA(RBASE,N0,ISCALE)

 
      CALL PLMAP2(N0,M0B,NPMB,APLUS2,PDG,B2,INPL2,XDROIT,LAGF,
     >      LACOST,XOBJ,RBASE(IBINF),RBASE(IBSUP),RBASE(ISCALE),2,
     >      .FALSE.,IERR,IMPR,EPSIM)
 
      CALL RLSARA(RBASE(IBINF))
      CALL RLSARA(RBASE(IBSUP))
      CALL RLSARA(RBASE(ISCALE))
*
      ENDIF

      DO 410 J=1,NCST
         IF (INEGAL(J).NE.0) THEN
            CRIT=CONTR2(J)
            DO 400 I=1,N0
               CRIT=CRIT+GRADC(I,J)*XOBJ(I)
  400          CONTINUE
            CRIT=INEGAL(J)*CRIT
            LCST2(J)=(CRIT.LE.0.0)
         ENDIF
  410    CONTINUE
      
      IF (IMPR.GE.2) THEN
         WRITE(6,*) (LCST(J),J=1,NCST)
         WRITE(6,*) (LCST2(J),J=1,NCST)
      ENDIF
      DO 420 J=1,NCST
         LTST=LTST.AND.(LCST(J).EQV.LCST2(J))
  420    CONTINUE

      ITER=ITER+1
      IF ((.NOT.LTST) .AND.(ITER.LE.ITERMX)) GO TO 99
C                           k,l
C    STEP 3: Sauvegarde de P
C                           
      CALL LCMSIX(IPOPT,'OLD-VALUE',1)
      CALL LCMPUT(IPOPT,'F-PENAL-EVAL',1,2,LA0E)
      IF (IMPR.GE.1)   WRITE(6,*) 'LAGF',(LAGF(I),I=1,N0)  
      CALL SETARA(RBASE,N0,LAGFR)
      DO 510 I=1,N0
  510    RBASE(LAGFR+I-1)=REAL(LAGF(I))
      CALL LCMPUT(IPOPT,'DF-LA-PENAL',N0,2,RBASE(LAGFR))
      CALL RLSARA(RBASE(LAGFR))
      CALL LCMSIX(IPOPT,' ',0)
*     
      IF (IMPR.GE.1) THEN
         WRITE(6,*) 'Dvar',(XOBJ(I),I=1,N0)
      ENDIF

      RETURN
      END
