*DECK QLP
      SUBROUTINE QLP(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this module solves the quasi linear optimization problem.
* QLP = Quasi Linear Programmation 
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of lcm objects used by the module
* HENTRY  character*12 name of each lcm object
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the lcm object is created
*         =1 the lcm object is open for modifications;
*         =2 the lcm object is open in read-only mode
* KENTRY  =file unit number; =lcm object address otherwise
*         kentry(nentry)
*                          
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT  NONE
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
* General variable
      INTEGER   NITMA,ITYP,ITYP1
      DOUBLE PRECISION DFLOTT
      REAL      FLOTT
      CHARACTER TEXT12*12
* STATE-VECTOR VARIABLES
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   OPTPRI(NSTATE)
      REAL      OPTPRR(NSTATE)

      INTEGER   IPOPT,MEMADR(20),I,J,NVAR,NCST,LENGT,IPRINT,NSTPEX,
     1          IMTHD,M0,NPM,MINMAX,NOPTX,CNVTST,IERR,CONV,SIGNA(3)
      INTEGER   GRAD,IDX,IDF,DX,GF,GRADC,INEGAL,CONTR,GRAD0,
     1          VALMAX,VALMIN,DINF,DSUP,APLUS,BPLUS,INPLUS,WORK
      REAL      XDROIT,XS,XXS,EPS4,EPSIM,ECOST,DELTA,S,NORM,EPSEXT
      DOUBLE PRECISION  FCOST
      LOGICAL   LCONTQ,LSAVE,LNORM2,LWAR
*
      INTEGER   VARVAL,VARWGT,FCSTV,VARVL2
      REAL      COST,CQUAD

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
C
C---------------------------
C     VALIDITY OF OBJECTS
C---------------------------
      IF (NENTRY.NE.1) CALL XABORT('QLP: 1 OBJECT EXPECTED.')
*CHECK IF OBJECTS ARE LCM OBJECTS AND WHICH ONE CORRESPOND
*TO WHICH TYPE
      IF (JENTRY(1).NE.1) CALL XABORT('QLP: OBJECT IN MODIFICATION '
     1  //'MODE ONLY')
      IPOPT=KENTRY(1)
      CALL XDISET(MEMADR,20,0)
      MEMADR(6)=KENTRY(1)
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))CALL XABORT('PQL:'
     1      //' LCM OBJECT EXPECTED')
      CALL LCMGET(KENTRY(1),'SIGNATURE',SIGNA)
      WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
      IF (TEXT12.NE.'L_OPTIMIZE') CALL XABORT('QLP: L_OPTIMIZE NEEDED')
C
C---------------------------
C VARIABLE INITIALISATION	
C---------------------------
* variables generales
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      NVAR  =OPTPRI(1)
      NCST  =OPTPRI(2)
      MINMAX=OPTPRI(3)
      IF ((MINMAX.NE.1).AND.(MINMAX.NE.-1)) CALL XABORT('QLP: '
     1  //'MINMAX not equal to 1 or -1')
      CONV  =OPTPRI(4)
      NSTPEX=OPTPRI(5)
      IMTHD =OPTPRI(9)
      CALL LCMGET(IPOPT,'OPT-PARAM-R',OPTPRR)
      S     =OPTPRR(1)
      ECOST =OPTPRR(6)
      EPSEXT=OPTPRR(3)
      EPSIM =OPTPRR(4)
      EPS4  =OPTPRR(5)
* ??????????
      NOPTX  = 100
* variables de decision
      CALL SETARA(RBASE,NVAR,VARVAL)
      CALL LCMLEN(IPOPT,'VAR-VALUE',LENGT,ITYP)
      IF (LENGT.NE.NVAR) CALL XABORT('QLP: WRONG NUMBER OF VARIABLE')
      CALL LCMGET(IPOPT,'VAR-VALUE',RBASE(VARVAL))
* poids des variables de decision
      CALL SETARA(RBASE,NVAR,VARWGT)
      CALL LCMLEN(IPOPT,'VAR-WEIGHT',LENGT,ITYP)
      IF (LENGT.EQ.0) THEN
         DO 3 I=1,NVAR
    3       RBASE(VARWGT+I-1)=1.0
      ELSEIF (LENGT.EQ.NVAR) THEN
         CALL LCMGET(IPOPT,'VAR-WEIGHT',RBASE(VARWGT))
      ELSE
         CALL XABORT('PQL: NVAR - LENGT ARE NOT THE SAME')
      ENDIF
* valeurs des fonctionnelles (fonction objectif = premiere valeur)
      CALL SETARA(RBASE,NCST+1,FCSTV)
      CALL LCMLEN(IPOPT,'FOBJ-CST-VAL',LENGT,ITYP)
      IF (LENGT.EQ.0) CALL XABORT('QLP: OBJECTIVE FUNCTION AND CONSTRA'
     1  //'INTS NOT YET EVALUATED')
      CALL LCMGET(IPOPT,'FOBJ-CST-VAL',RBASE(FCSTV))
      COST=RBASE(FCSTV)
* valeurs des gradiants des fonctionnelles
      CALL SETARA(RBASE,NVAR*(NCST+1),GRAD)
      CALL LCMGET(IPOPT,'GRADIENT',RBASE(GRAD))
* allocation memoire vecteurs commun aux differentes options
      CALL SETARA(RBASE,NVAR,IDX)
      CALL SETARA(RBASE,NVAR,IDF)
      IPRINT=0
* Warning-only
      LWAR=.FALSE.
C
C---------------------------
C     READ USER INPUT:
C---------------------------
   20 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
* Edition level
   30 IF (ITYP.NE.3) CALL XABORT('QLP: CHARACTER EXPECTED')
      IF (TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('QLP: *IPRINT* MUST BE INTEGER')
* Warning Only for failure of recovery of a valid point
      ELSEIF (TEXT12.EQ.'WARNING-ONLY')THEN
         LWAR=.TRUE.
* Calculation of next point
      ELSEIF (TEXT12.EQ.'CALCUL-DX')THEN
         GO TO 100
* Cost extrapolation
      ELSEIF (TEXT12.EQ.'COST-EXTRAP')THEN
         GO TO 200
* Convergence test
      ELSEIF (TEXT12.EQ.'CONV-TEST') THEN
         GO TO 300
* End of this subroutine
      ELSEIF( TEXT12.EQ.';' )THEN
         GO TO 1000
      ELSE
         CALL XABORT('QLP: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20
C---------------------------
C     1      0000     0000
C    11     0    0   0    0
C   1 1    0      0 0      0
C  1  1    0      0 0      0
C 1   1    0      0 0      0
C     1    0      0 0      0
C     1     0    0   0    0
C 11111111   0000     0000
C---------------------------
C Next step calculation
C---------------------------
  100 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (TEXT12.EQ.'NO-STORE-OLD') THEN
         LSAVE=.TRUE.
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      ELSE
         LSAVE=.FALSE.
      ENDIF
      ITYP1=ITYP
C appel a la fonction PLDRV.f calcul de DX
*     initialisation de tous les parametres
      CALL SETARA(RBASE,NVAR,DX)
      FCOST=DBLE(COST)
      CALL SETARA(RBASE,2*NVAR,GF)
      CALL SETARA(RBASE,2*NVAR*NCST,GRADC)
*   INEGAL
      CALL SETARA(IBASE,NCST,INEGAL)
      CALL LCMLEN(IPOPT,'CST-TYPE',LENGT,ITYP)
      IF (LENGT.NE.NCST) CALL XABORT('QLP: NCST - LENGT NOT EQUAL')
      CALL LCMGET(IPOPT,'CST-TYPE',IBASE(INEGAL))
*
*   CONTR
      CALL SETARA(RBASE,NCST,CONTR)
      CALL LCMLEN(IPOPT,'CST-OBJ',LENGT,ITYP)
      IF (LENGT.NE.NCST) CALL XABORT('QLP: NCST - LENGT NOT EQUAL')
      CALL LCMGET(IPOPT,'CST-OBJ',RBASE(CONTR))
      DO 110 I=0,NCST-1
  110    RBASE(CONTR+I)  = RBASE(CONTR+I)-RBASE(FCSTV+I+1)
*
*   DINF et DSUP
      CALL LCMLEN(IPOPT,'VAR-VAL-MAX',LENGT,ITYP)
      IF (LENGT.EQ.0) CALL XABORT('QLP: NO MAXIMUM VALUE DEFINED')
      CALL SETARA(RBASE,NVAR,VALMAX)
      CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(VALMAX))
      CALL LCMLEN(IPOPT,'VAR-VAL-MIN',LENGT,ITYP)
      IF (LENGT.EQ.0) CALL XABORT('QLP: NO MAXIMUM VALUE DEFINED')
      CALL SETARA(RBASE,NVAR,VALMIN)
      CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(VALMIN))
      CALL SETARA(RBASE,NVAR,DINF)
      CALL SETARA(RBASE,NVAR,DSUP)
      DO 115 I=0,NVAR-1
         RBASE(DINF+I) = RBASE(VALMIN+I) - RBASE(VARVAL+I)
         RBASE(DSUP+I) = RBASE(VALMAX+I) - RBASE(VARVAL+I)
  115    CONTINUE
      CALL RLSARA(RBASE(VALMAX))
      CALL RLSARA(RBASE(VALMIN))
*
      M0     = NCST
      XDROIT = S**2
      IF(IPRINT.GE.1) WRITE(6,40001) XDROIT,(RBASE(VARWGT+I-1),I=1,NVAR)
*   DETERMINATION DES CONTRAINTES ACTIVES POUR LES BORNES DES XK(I)
      DO 140 I=1,NVAR
         XS = SQRT(XDROIT/RBASE(VARWGT+I-1))
         XXS=-XS
         IF (RBASE(DINF+I-1).GT.XXS) THEN
            M0 = M0 + 1
         ENDIF
         IF (RBASE(DSUP+I-1).LT.XS) THEN
            M0 = M0 + 1
         ENDIF
  140    CONTINUE
      NPM = (M0 + 1) + NVAR
      CALL SETARA(RBASE,2*(M0+1)*NPM,APLUS)
      CALL SETARA(RBASE,M0+2,BPLUS)
      CALL SETARA(IBASE,M0+1,INPLUS)
      CALL SETARA(RBASE,NVAR*(NCST+1),WORK)
*   LCONTQ
      LCONTQ = .FALSE.
      DO 155 J=0,NCST-1
         IF (IBASE(INEGAL+J).EQ.-1) THEN
            IF (RBASE(CONTR+J).GT.EPS4) THEN
               LCONTQ = .TRUE.
               WRITE(6,40002) J,IBASE(INEGAL+J),RBASE(CONTR+J)
            ENDIF
         ELSEIF (IBASE(INEGAL+J).EQ.0) THEN
            IF (ABS(RBASE(CONTR+J)).GT.EPS4) THEN
               LCONTQ = .TRUE.
               WRITE(6,40002) J,IBASE(INEGAL+J),RBASE(CONTR+J)
            ENDIF
         ELSEIF (IBASE(INEGAL+J).EQ.1) THEN
            IF (RBASE(CONTR+J).LT.-EPS4) THEN
               LCONTQ = .TRUE.
               WRITE(6,40002) J,IBASE(INEGAL+J),RBASE(CONTR+J)
            ENDIF
         ENDIF
  155    CONTINUE
*   SOLUTION DU PROBLEME QUASILINEAIRE
      IERR=0
      CALL PLDRV(IMTHD,RBASE(DX),FCOST,RBASE(GF),RBASE(GRADC),
     > IBASE(INEGAL),RBASE(CONTR),RBASE(DINF),RBASE(DSUP),RBASE(APLUS),
     > RBASE(BPLUS),IBASE(INPLUS),RBASE(VARWGT),XDROIT,LCONTQ,IERR,NVAR,
     > NCST,MINMAX,M0,NPM,IPRINT,EPSIM,NOPTX,RBASE(WORK),MEMADR)
* marche arriere si echec du PROBLEME QUASILINEAIRE
      IF (IERR.GE.1) THEN
         OPTPRI(13)=OPTPRI(13)+1
         CALL LCMSIX(IPOPT,'OLD-VALUE',1)
         CALL LCMLEN(IPOPT,'VAR-VALUE2',LENGT,ITYP)
         IF (LENGT.EQ.0) THEN
            IF (LWAR) THEN
               WRITE(6,*) 'WARNING: UNABLE TO RECOVER A VALID POINT'
     1 //' WITH SUCCESSFUL "QLP" RESOLUTION'
            ELSE
               CALL XABORT('UNABLE TO RECOVER A VALID POINT'
     1 //' WITH SUCCESSFUL "QLP" RESOLUTION')
            ENDIF
         ELSE
            CALL SETARA(RBASE,NVAR,VARVL2)
            CALL LCMGET(IPOPT,'VAR-VALUE2',RBASE(VARVL2))
            DO 156 I=0,NVAR-1
  156          RBASE(DX+I)=(RBASE(VARVL2+I)-RBASE(VARVAL+I))/2.0
            CALL RLSARA(RBASE(VARVL2))
         ENDIF
         IF (IPRINT.GE.1) WRITE(6,*) 'IERR>0'
         IF (IPRINT.GE.1) WRITE(6,*) 'DX=',(RBASE(DX+I),I=0,NVAR-1)
      ELSE
         OPTPRI(13)=0
      ENDIF
*
      DO 157 I=0,NVAR-1
         RBASE(IDX+I)=RBASE(DX+I)
  157    RBASE(IDF+I)=RBASE(WORK+I)
      CALL RLSARA(RBASE(DX))
      CALL RLSARA(RBASE(WORK))
      CALL RLSARA(RBASE(GF))
      CALL RLSARA(RBASE(GRADC))
      CALL RLSARA(IBASE(INEGAL))
      CALL RLSARA(RBASE(DINF))
      CALL RLSARA(RBASE(DSUP))
      CALL RLSARA(RBASE(CONTR))
      CALL RLSARA(RBASE(APLUS))
      CALL RLSARA(RBASE(BPLUS))
      CALL RLSARA(IBASE(INPLUS))
C
C Backup des vecteurs de l'iteration precedante
      IF (.NOT.LSAVE) THEN
         CALL LCMSIX(IPOPT,' ',0)
         CALL LCMSIX(IPOPT,'OLD-VALUE',1)
         CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(VARVAL))
         CALL LCMPUT(IPOPT,'FOBJ-CST-VAL',NCST+1,2,RBASE(FCSTV))
         CALL LCMPUT(IPOPT,'GRADIENT',NVAR*(NCST+1),2,RBASE(GRAD))
         CALL LCMSIX(IPOPT,' ',0)
      ENDIF
C
C Calculer et sauvegarder Xk+1
      DO 180 I=0,NVAR-1
  180    RBASE(VARVAL+I)=RBASE(VARVAL+I)+RBASE(IDX+I)
      CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(VARVAL))
      ITYP=ITYP1
C Extrapoler la fonction cout
      IF (IMTHD.LE.3) THEN
         ECOST=COST
      ELSE 
         CALL LCMSIX(IPOPT,'OLD-VALUE',1)
         IF (IMTHD.EQ.4) THEN
            CALL LCMGET(IPOPT,'F-LA-EVAL',COST)
         ELSE
            CALL LCMGET(IPOPT,'F-PENAL-EVAL',COST)
         ENDIF
         CALL LCMGET(IPOPT,'DF-LA-PENAL',RBASE(IDF))
         CALL LCMSIX(IPOPT,' ',0)
      ENDIF
      DO 190 I=0,NVAR-1
  190    ECOST=ECOST+RBASE(IDX+I)*RBASE(IDF+I)
C Reinitialiser les gradients pour la prochaine iteration      
      CALL SETARA(RBASE,NVAR*(NCST+1),GRAD0)
      CALL XDRSET(RBASE(GRAD0),NVAR*(NCST+1),0.0)
      CALL LCMPUT(IPOPT,'GRADIENT',NVAR*(NCST+1),2,RBASE(GRAD0))
      CALL RLSARA(RBASE(GRAD0))
      GO TO 30
C---------------------------
C   2222     0000     0000
C  2    2   0    0   0    0
C 2      2 0      0 0      0
C       2  0      0 0      0
C      2   0      0 0      0
C   22     0      0 0      0
C  2        0    0   0    0
C 22222222   0000     0000
C---------------------------
C Sortir la valeur de la fonction cout extrapolee
C---------------------------
  200 CALL REDGET(ITYP,NITMA,ECOST,TEXT12,DFLOTT)
      IF (ITYP.NE.-2) CALL XABORT('QLP: OUTPUT REAL EXPECTED')
      ITYP=2
      CALL REDPUT(ITYP,NITMA,ECOST,TEXT12,DFLOTT)
      GO TO 20
C---------------------------
C   3333     0000     0000
C  3    3   0    0   0    0
C 3      3 0      0 0      0
C      33  0      0 0      0
C       3  0      0 0      0
C 3      3 0      0 0      0
C  3    3   0    0   0    0
C   3333     0000     0000
C---------------------------
C Test convergence
C---------------------------
  300 LNORM2=.TRUE.
      CALL REDGET(ITYP,CNVTST,FLOTT,TEXT12,DFLOTT)
      IF ((ITYP.EQ.3).AND.(TEXT12.EQ.'NORM-INF')) THEN
         LNORM2=.FALSE.
         CALL REDGET(ITYP,CNVTST,FLOTT,TEXT12,DFLOTT)
      ENDIF
      IF (ITYP.NE.-5) CALL XABORT('QLP: OUTPUT LOGICAL EXPECTED')
      DELTA=ABS((ECOST-COST)/COST)
      NORM=0.0
      CQUAD=0.0
      IF (LNORM2) THEN
         DO 350 I=0,NVAR-1
            NORM=NORM+RBASE(VARWGT+I)*RBASE(VARVAL+I)*RBASE(VARVAL+I)
  350          CQUAD=CQUAD+RBASE(VARWGT+I)*RBASE(IDX+I)*RBASE(IDX+I)
         IF (NORM.NE.0.0) THEN
            CQUAD=SQRT(CQUAD/NORM)
         ELSE
            CQUAD=0.0
         ENDIF
      ELSE
         DO 360 I=0,NVAR-1
            NORM=MAX(NORM,ABS(RBASE(VARWGT+I)**0.5*RBASE(VARVAL+I)))
  360       CQUAD=MAX(CQUAD,ABS(RBASE(VARWGT+I)**0.5*RBASE(IDX+I)))
         IF (NORM.NE.0.0) THEN
            CQUAD=CQUAD/NORM
         ELSE
            CQUAD=0.0
         ENDIF
      ENDIF
      IF (EPSEXT.EQ.0.0) EPSEXT = 0.001
      IF (((DELTA.LT.EPSEXT).AND.(CQUAD.LE.EPSEXT)) .OR.
     1    (CQUAD.LE.(EPSEXT/10.0)))    THEN
         CNVTST=1
         CONV  =1
      ELSE
         CNVTST=-1
         CONV  =0
      ENDIF
      IF (IPRINT.GT.1) THEN
         WRITE(6,*) 'It= convergence?', DELTA,CQUAD,EPSEXT
         IF (IPRINT.GT.2) THEN
            WRITE(6,*) 'DX',(RBASE(IDX+I),I=0,NVAR-1)
            WRITE(6,*) 'X',(RBASE(VARVAL+I),I=0,NVAR-1)
         ENDIF
      ENDIF
      ITYP=5
      CALL REDPUT(ITYP,CNVTST,FLOTT,TEXT12,DFLOTT)
      GO TO 20
C---------------------------
C END
C---------------------------
 1000 CALL RLSARA(RBASE(VARWGT))
      CALL RLSARA(RBASE(FCSTV))
      CALL RLSARA(RBASE(GRAD))
      CALL RLSARA(RBASE(IDX))
      CALL RLSARA(RBASE(IDF))
      CALL RLSARA(RBASE(VARVAL))
      OPTPRI(4)=CONV
      OPTPRI(5)=NSTPEX
      OPTPRR(6)=ECOST
      CALL LCMPUT(IPOPT,'OPT-PARAM-R',NSTATE,2,OPTPRR)
      CALL LCMPUT(IPOPT,'STATE-VECTOR',NSTATE,1,OPTPRI)


20001 FORMAT(/'NO STEP REDUCTION SPECIFIED YET, PARABOLIC AS DEFAULT')
20002 FORMAT(//,5X,'LE PAS D''AVANCE NE PEUT PLUS ETRE REDUIT',//)
40001 FORMAT(//,5X,'S**2 (XDROIT) = ',1P,E13.5,
     >        /,5X,'FPOIDS  = ',/,(11X,1P,8E13.5))
40002 FORMAT(   5X,'*** LA CONTRAINTE #NO: ',I3,' DU TYPE INEGAL = ',I2,
     >             ' EST VIOLEES DE ',1PE12.5)

      END

