      SUBROUTINE TABINL(IPTABU,NGBTST,MINVAL,MAXVAL,TSTVAL,TSTEPS,WORK,
     1                  LIST,TBRAD,PRORAD,NGBTYP,TBLST,TBLSC,PROLST,
     2                  PROLSC,LISTMX,NVAR,NCST,VALID)
*
*-----------------------------------------------------------------------
*Purpose:
* this subroutine validates a neighbor for tabu search (in list or not)
* TABINL= TABu? (=IN one List?)
* 
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon
*
*Parameters: input
* IPTABU   adress of the lcm object of L_TABU
* NGBTST   relative value of the solution to be validated (NVAR)
* MINVAL   absolute minimum value of the decision vector
* MAXVAL   absolute maximum value of the decision vector
* LIST     lists to be used for validity check
*                     TABU-LIST     PROMISE-LIST
*            =1         yes             no
*            =2         yes             yes
*            =3         no              yes
* TBRAD    relative radius of the tabu list
* PRORAD   relative radius of the promising list
* NGBTYP   type of neighborhood (1= rectangle, 2= ball)
* TBLST    maximum size of the tabu list
* TBLSC    actual size/indent for the tabu list
* PROLST   maximum size of the promising list
* PROLSC   actual size/indent for the promising list
* LISTMX   maximum size for infinity list 
*
*Parameters: output
* VALID    validity of the neighbor value
*                          
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT  NONE
      INTEGER   IPTABU,LIST,NGBTYP,TBLST,TBLSC,PROLST,PROLSC,LISTMX,
     1          NVAR,NCST,VALID
      REAL      NGBTST(NVAR),MINVAL(NVAR),MAXVAL(NVAR),TSTVAL(NVAR),
     1          TSTEPS(NVAR),WORK(NVAR+NCST+2),TBRAD,PRORAD
*----
*  LOCAL VARIABLES
*----
      INTEGER   I,J,LENGT,ITYP,JPTABU,ILAST,LCMGID
      REAL      EPSRAD,NORM
      LOGICAL   LPRO

      VALID =1
      LPRO  =.TRUE.
      DO 5 I=1,NVAR
         TSTEPS(I)=MAXVAL(I)-MINVAL(I)
 5       TSTVAL(I)=MINVAL(I)+NGBTST(I)*TSTEPS(I)
 
 10   IF (LIST.LE.2) THEN
         CALL LCMLEN(IPTABU,'TABU-LIST',LENGT,ITYP)
         IF (LENGT.NE.0) THEN
            CALL LCMSIX(IPTABU,'TABU-LIST',1)
         ELSE
            GO TO 500
         ENDIF
         ILAST=TBLST
         IF (TBLST.EQ.LISTMX) ILAST=TBLSC
         EPSRAD=TBRAD
         GO TO 100
      ENDIF

 20   LPRO  =.FALSE.
      IF (LIST.GE.2) THEN
         CALL LCMLEN(IPTABU,'PROMISE-LIST',LENGT,ITYP)
         IF (LENGT.NE.0) THEN
            CALL LCMSIX(IPTABU,'PROMISE-LIST',1)
         ELSE
            GO TO 500
         ENDIF
         ILAST=PROLST
         IF (PROLST.EQ.LISTMX) ILAST=PROLSC
         EPSRAD=PRORAD
      ELSE 
         GO TO 500
      ENDIF
      
 100        write(6,*) 'inl len avant'
      CALL LCMLEN(IPTABU,'ITEM',LENGT,ITYP)
      write(6,*) 'inl len apres', LENGT
      IF (LENGT.EQ.0) GO TO 500
      JPTABU=LCMGID(IPTABU,'ITEM')
 	  DO 400 I=1,ILAST
         CALL LCMLEL(JPTABU,I,LENGT,ITYP)
         IF (LENGT.EQ.0) GO TO 400
         CALL LCMGDL(JPTABU,I,WORK)
         NORM=0.0
*  case 1: hyperrectangular
         IF (NGBTYP.EQ.1) THEN
            DO 150 J=1,NVAR
 150           NORM=MAX(NORM,ABS((WORK(J)-TSTVAL(J))/TSTEPS(J)))
*  case 2: ball
         ELSEIF (NGBTYP.EQ.2) THEN
            DO 250 J=1,NVAR
 250           NORM=NORM+((WORK(J)-TSTVAL(J))/TSTEPS(J))**2
            NORM=NORM**0.5
         ENDIF
*
         
         IF (NORM.LE.EPSRAD) VALID = 0
         IF (VALID.EQ.0) GO TO 490
 400     CONTINUE

 490  CALL LCMSIX(IPTABU,' ',2)
 500  IF ((LPRO).AND.(VALID.NE.0)) GO TO 20 
 


      RETURN
      END
