*DECK TABU
      SUBROUTINE TABU(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* this module defines the TABU method tools and solves optimization 
*  problem.
* TABU = TABU search optimization tools
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Chambon 20/01/2004
*
*Parameters: input/output
* NENTRY  number of lcm objects or files used by the module
* HENTRY  character*12 name of each lcm object or file
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the lcm object is created
*         =1 the lcm object is open for modifications;
*         =2 the lcm object is open in read-only mode
* KENTRY  =file unit number; =lcm object address otherwise
*         kentry(nentry)
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      IMPLICIT  NONE
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
* General variables
      INTEGER   NITMA,ITYP,IPRINT,IPOPT,IPTABU,JPTABU,I,J,SIGNA(3),
     1          IVALID,IFACE,VLDTST,IMIN,IMIN2,IMAX,LENGT,CHOICE,VALID,
     2          ITYP2,K,T1,T2,LCMGID,LCMLID
      DOUBLE PRECISION DFLOTT
      REAL      FLOTT,TABUVL,TEMP,S,TRSHLD,MEAN
      CHARACTER TEXT12*12,TXT12B*12,TXTREP*12
      LOGICAL   LCVT,LPRO,LTST,LTSTN,LTSTP,LTSTS
* STATE-VECTOR VARIABLES
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   OPTPRI(NSTATE),TBPRI(NSTATE)
      REAL      OPTPRR(NSTATE),TBPRR(NSTATE)
* Option variables
      INTEGER   NVAR,NCST,MINMAX,NGB,NGBTYP,NGBDIS,NITCUR,NITMAX,TBLSM,
     1          PROLSM,TBLSC,PROLSC,NTRYNB,LISTMX
      PARAMETER (NTRYNB=100)
      REAL      NBGRAD,TBRAD,PRORAD,GEOFAC,BEST,BESTN,EPSNM
* Random number
      INTEGER   ISEED,IFIRST
      REAL      RAND,XNORM
* Offset
      INTEGER   INGBVL,IMINVL,IMAXVL,INGBRD,INGBCU,INGBTS,ITSTVL,ITSTEP,
     1          IWORK,IFCTVL,ICSTWT,IVARWT,INGBFV,INLDR,INLDRV,INLDRM,
     2          INLDRR,INLDRO
* Nelder-Meat
      REAL      ALPHA,BETA,GAMMA,FVAL,FVALW,FVALW2,FVALB,FVALE,
     1          WORST,WORST2,BESTND,DELTA,NLDEPS,CONV
      INTEGER   VTST
      PARAMETER (ALPHA=1.0,BETA=0.5,GAMMA=2.)
      LOGICAL   LEXP,LCTT,LCVG

      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
C
C---------------------------
C     VALIDITY OF OBJECTS
C---------------------------
      IPOPT =0
      IPTABU=0
      DO 2 I=1,NENTRY
         IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2))CALL XABORT('FOBJCT:'
     1      //' LCM OBJECT EXPECTED')
         IF (JENTRY(I).EQ.0) THEN
            TEXT12='L_TABU'
            READ(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
            CALL LCMPUT(KENTRY(I),'SIGNATURE',3,3,SIGNA)
         ELSE
            CALL LCMGET(KENTRY(I),'SIGNATURE',SIGNA)
            WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
         ENDIF
         IF (TEXT12.EQ.'L_TABU') THEN 
            IPTABU=KENTRY(I)
            IF (I.NE.1) CALL XABORT('TABU: L_TABU MUST BE THE FIRST')
         ELSEIF (TEXT12.EQ.'L_OPTIMIZE') THEN 
            IPOPT =KENTRY(I)
            IF (I.NE.2) CALL XABORT('TABU: L_TABU MUST BE THE SECOND')
         ELSE 
            CALL XABORT('TABU:'//TEXT12 //' NOT GOOD TYPE OF OBJECT')
         ENDIF
    2    CONTINUE
      IF (IPTABU*IPOPT.EQ.0) CALL XABORT('TABU: L_TABU OR L_OPTIMIZE '
     1  //'MISSING')
C
C---------------------------
C VARIABLE INITIALISATION	
C---------------------------
      IPRINT = 0
      LISTMX = 2000000001
* aller chercher dans L_OPTI les info necessaires
      CALL LCMGET(IPOPT,'STATE-VECTOR',OPTPRI)
      CALL LCMGET(IPOPT,'OPT-PARAM-R',OPTPRR)
*    minimisation/maximisation
      MINMAX=OPTPRI(3)
*    size of the optimization problem
      NVAR  =OPTPRI(1)
      NCST  =OPTPRI(2)
* initialiser les parametres de L_TABU si l'objet est cree.
      IF (JENTRY(1).EQ.0) THEN
         CALL XDISET(TBPRI,NSTATE,0)
         CALL XDRSET(TBPRR,NSTATE,0.0)
*    neighborhood size 
         NGB   =5
         TBPRI(1)=NGB
*    neighborhood type (1=rectangle 2=ball)
         NGBTYP=1
         TBPRI(2)=NGBTYP
*    neighborhood discretization type (1=geometric 2=linear 3=isovolume)
         NGBDIS=1
         TBPRI(3)=NGBDIS
*    required iteration number without improvement for convergence test
*     (current = TBPRI(5))
         NITMAX=10
         TBPRI(4)=NITMAX
*    maximum tabu list length (current: TBPRI(8))
         TBLSM =50
         TBPRI(6)=TBLSM
         TBLSC =0
*    maximum promising list length (current: TBPRI(9))
         PROLSM=10
         TBPRI(7)=PROLSM
         PROLSC=0
*    radius of the neighborhood (fraction of the decision space)
         NBGRAD=0.01
         TBPRR(1)=NBGRAD
*    radius of the tabu points (fraction of the decision space)
         TBRAD =0.001
         TBPRR(2)=TBRAD
*    radius of the promising area (fraction of the decision space)
         TBRAD =0.002
         TBPRR(3)=PRORAD
*    factor for a geometric discretisation
         GEOFAC=2.0
         TBPRR(4)=GEOFAC
*    ramdom number (commun number)
         CALL CLETIM(T1,T2)
         ISEED = T1+T2
         IFIRST=0
         DO 4 I=0,MOD(ISEED,10)
            CALL RANDF(ISEED,IFIRST,RAND)   
    4       CONTINUE
         TBPRI(39)=ISEED
         TBPRI(40)=IFIRST
*    best value for tabu function ever achieved
         BEST  =MINMAX*1.0E10
         TBPRR(5)=BEST
*    treeshold for the promising list acceptance
         TRSHLD=MINMAX*1.0E10
         TBPRR(6)=TRSHLD
*    precision for the NELDER-MEAD algorithm
         NLDEPS=0.01
         TBPRR(7)=NLDEPS
         CALL LCMPUT(IPTABU,'STATE-VECTOR',NSTATE,1,TBPRI)
         CALL LCMPUT(IPTABU,'TABU-PARAM-R',NSTATE,2,TBPRR)
      ELSE
         CALL LCMGET(IPTABU,'STATE-VECTOR',TBPRI)
         CALL LCMGET(IPTABU,'TABU-PARAM-R',TBPRR)
         NGB   =TBPRI(1)
         NGBTYP=TBPRI(2)         
         NGBDIS=TBPRI(3)
         NITMAX=TBPRI(4)
         NITCUR=TBPRI(5)
         TBLSM =TBPRI(6)
         PROLSM=TBPRI(7)
         TBLSC =TBPRI(8)
         PROLSC=TBPRI(9)
         ISEED =TBPRI(39)
         IFIRST=TBPRI(40)
         NBGRAD=TBPRR(1)
         TBRAD =TBPRR(2)
         PRORAD=TBPRR(3)
         GEOFAC=TBPRR(4)
         BEST  =TBPRR(5)
         TRSHLD=TBPRR(6)
         NLDEPS=TBPRR(7)
      ENDIF
* NELDER-MEAD Simplex 
      EPSNM=0.1
C
C---------------------------
C     READ USER INPUT:
C---------------------------
   20 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED (20)')
* Edition level      
   30 IF (TEXT12.EQ.'EDIT')             THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('TABU: *IPRINT* MUST BE INTEGER')
* Option definition
      ELSEIF (TEXT12.EQ.'DEFINITION')   THEN
         GO TO 100
* Neighbors definition
      ELSEIF (TEXT12.EQ.'NEIGHB-CREAT') THEN
         GO TO 200
* Neighbor choice
      ELSEIF (TEXT12.EQ.'NEIGHB-CHOIC') THEN
         GO TO 300
* Neighbor evaluation
      ELSEIF (TEXT12.EQ.'NEIGHB-EVAL')  THEN
         GO TO 400
* Neighbor best one
      ELSEIF (TEXT12.EQ.'NEIGHB-BEST')  THEN
         GO TO 500
* Promizing area definition
      ELSEIF (TEXT12.EQ.'PROMISE-AREA') THEN
         GO TO 600
* Nelder-Mead Simplex Algorithm
      ELSEIF (TEXT12.EQ.'NELDER-MEAD') THEN
         GO TO 700
* End of this subroutine
      ELSEIF (TEXT12.EQ.';' )           THEN
         GO TO 1000
      ELSE
         CALL XABORT('TABU: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20

C---------------------------
C     1      0000     0000    
C    11     0    0   0    0 
C   1 1    0      0 0      0 
C  1  1    0      0 0      0  
C 1   1    0      0 0      0  
C     1    0      0 0      0  
C     1     0    0   0    0  
C 11111111   0000     0000  
C---------------------------
C OPTION DEFINITION
C---------------------------
  100 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
  101 IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED (101)') 
      LTST=.FALSE.
      IF (TEXT12.EQ.'NEIGHBOR-NB')      THEN
         CALL REDGET(ITYP,NGB,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('TABU: INTEGER EXPECTED(101a)') 
         TBPRI(1)=NGB
      ELSEIF (TEXT12.EQ.'NEIGHBOR-TYP') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED(101b)') 
         IF (TEXT12.EQ.'RECTANGLE') THEN
            TBPRI(2)=1
         ELSEIF (TEXT12.EQ.'BALL') THEN
            TBPRI(2)=2
         ELSE
            CALL XABORT('TABU: WRONG KEYWORD FOR NEIGHBORHOOD TYPE')
         ENDIF
         NGBTYP=TBPRI(2)
      ELSEIF (TEXT12.EQ.'NEIGHBOR-DIS') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED(101c)')
         IF (TEXT12.EQ.'GEOMETRIC') THEN
            NGBDIS=1
            CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
            IF (ITYP.EQ.3) THEN
               TBPRI(3)=NGBDIS
            ELSEIF (ITYP.EQ.2) THEN
               GEOFAC=FLOTT
            ELSE
               CALL XABORT('TABU: REAL EXPECTED FOR GEOFAC (101c1)')
            ENDIF
         ELSEIF (TEXT12.EQ.'LINEAR') THEN
            NGBDIS=2
         ELSEIF (TEXT12.EQ.'ISOVOLUME') THEN
            NGBDIS=3
         ELSE
            CALL XABORT('TABU: WRONG KEYWORD AFTER NEIGHBOR-DIS(101c2)')
         ENDIF
         TBPRR(4)=GEOFAC
         TBPRI(3)=NGBDIS
      ELSEIF (TEXT12.EQ.'NIT-MAX-CONV') THEN
         CALL REDGET(ITYP,NITMAX,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('TABU: INTEGER EXPECTED(101d)') 
         TBPRI(4)=NITMAX
      ELSEIF (TEXT12.EQ.'NEIGHBOR-RAD') THEN
         CALL REDGET(ITYP,NITMA,NBGRAD,TEXT12,DFLOTT)
         IF (ITYP.NE.2) CALL XABORT('TABU: REAL EXPECTED(101e)') 
         TBPRR(1)=NBGRAD
      ELSEIF (TEXT12.EQ.'TABU-RAD')     THEN
         CALL REDGET(ITYP,NITMA,TBRAD,TEXT12,DFLOTT)
         IF (ITYP.NE.2) CALL XABORT('TABU: REAL EXPECTED(101f)') 
         TBPRR(2)=TBRAD
      ELSEIF (TEXT12.EQ.'PROMIS-RAD')   THEN
         CALL REDGET(ITYP,NITMA,PRORAD,TEXT12,DFLOTT)
         IF (ITYP.NE.2) CALL XABORT('TABU: REAL EXPECTED(101g)') 
         TBPRR(3)=PRORAD
      ELSEIF (TEXT12.EQ.'TABU-LIST-LG') THEN
         CALL REDGET(ITYP,TBLSM,FLOTT,TEXT12,DFLOTT)
         IF (TEXT12.EQ.'ALL') THEN
            TBLSM =LISTMX
            TBPRI(6)=TBLSM
         ELSEIF (ITYP.EQ.1) THEN
            TBPRI(6)=TBLSM
         ELSE
            CALL XABORT('TABU: INTEGER EXPECTED(101h)') 
         ENDIF
      ELSEIF (TEXT12.EQ.'PROM-LIST-LG') THEN
         CALL REDGET(ITYP,PROLSM,FLOTT,TEXT12,DFLOTT)
         IF (TEXT12.EQ.'ALL') THEN
            PROLSM =LISTMX
            TBPRI(7)=PROLSM
         ELSEIF (ITYP.EQ.1) THEN
            TBPRI(7)=PROLSM
         ELSE
            CALL XABORT('TABU: INTEGER EXPECTED(101hi)') 
         ENDIF
      ELSEIF (TEXT12.EQ.'RESET-BEST') THEN
         WRITE(6,*) 'RESET-BEST: option a venir'
         CONTINUE
      ELSEIF (TEXT12.EQ.'RESET-LIST') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('TABU:CHARACTER EXPECTED(101d)') 
         IF (TEXT12.EQ.'PROMISE-LIST') THEN
            PROLSC=0
            TBPRI(9)=PROLSC
            TRSHLD=MINMAX*1.0E10
            TBPRR(6)=TRSHLD
         ELSE IF(TEXT12.EQ.'TABU-LIST') THEN
            TBLSC =0
            TBPRI(8)=TBLSC
         ELSE
            CALL XABORT('TABU: Wrong Name of List')
         ENDIF
         CALL LCMSIX(IPTABU,TEXT12,1)
         CALL LCMLEN(IPTABU,'TABU-F-VAL',LENGT,ITYP)
         IF (LENGT.EQ.0) THEN
            WRITE(6,*) 'TABU: The List is empty already'
            GO TO 151
         ENDIF
         CALL LCMDEL(IPTABU,'ITEM')
!         DO 150 I=1,LENGT
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',I
!  150       CALL LCMDEL(IPTABU,TEXT12)
         CALL LCMDEL(IPTABU,'TABU-F-VAL')
  151    CALL LCMSIX(IPTABU,' ',0)            
         CONTINUE
      ELSEIF (TEXT12.EQ.'INITIALIZE') THEN
         CALL SETARA(RBASE,NVAR,IMINVL)
         CALL SETARA(RBASE,NVAR,IMAXVL)
         CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
         CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(IMINVL))
         CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(IMAXVL))
         CALL XDRSET(RBASE(IWORK),NVAR+NCST+2,0.0)
         DO 160 I=0,NVAR-1
            CALL RANDF(ISEED,IFIRST,RAND)
  160       RBASE(IWORK+I)=RBASE(IMINVL+I)
     1                 +RAND*(RBASE(IMAXVL+I)-RBASE(IMINVL+I))
         CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(IWORK))
         CALL RLSARA(RBASE(IMINVL))
         CALL RLSARA(RBASE(IMAXVL))
         CALL RLSARA(RBASE(IWORK))
      ELSEIF (TEXT12.EQ.'BEST-AS-CURR') THEN
         CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
         CALL LCMGET(IPTABU,'BEST-VAL',RBASE(IWORK))
         CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(IWORK))
         CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(IWORK))
         CALL LCMPUT(IPOPT,'FOBJ-CST-VAL',NCST+1,2,RBASE(IWORK+NVAR))
         CALL LCMPUT(IPOPT,'F-LA-PNLT',1,2,RBASE(IWORK+NVAR+NCST+1))
         CALL RLSARA(RBASE(IWORK))
      ELSEIF (TEXT12.EQ.'PUT-CURRENT') THEN
         CALL SETARA(RBASE,NVAR,ITSTVL)
         CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
         CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(IWORK))
         DO 170 I=0,NVAR-1
  170       RBASE(ITSTVL+I)=RBASE(IWORK+I)
         CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(ITSTVL))
         CALL RLSARA(RBASE(ITSTVL))
         CALL RLSARA(RBASE(IWORK))
      ELSEIF (TEXT12.EQ.'GET-CURRENT') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (TEXT12.NE.'COMPLETE') LTST=.TRUE.
         CALL SETARA(RBASE,NVAR,ITSTVL)
         CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
         CALL LCMGET(IPOPT,'VAR-VALUE',RBASE(ITSTVL))
         CALL XDRSET(RBASE(IWORK),NVAR+NCST+2,0.0)
         DO 180 I=0,NVAR-1
  180       RBASE(IWORK+I)=RBASE(ITSTVL+I)
         IF (.NOT.LTST) THEN
            CALL SETARA(RBASE,NCST+1,IFCTVL)
            CALL LCMLEN(IPOPT,'FOBJ-CST-VAL',LENGT,ITYP2)
            IF (LENGT.EQ.0) CALL XABORT('TABU: FUNCTION EVALUATION '
     1  //'NOT DONE (180)')
            CALL LCMGET(IPOPT,'FOBJ-CST-VAL',RBASE(IFCTVL))
            DO 181 I=0,NCST
  181          RBASE(IWORK+I+NVAR)=RBASE(IFCTVL+I)
            CALL RLSARA(RBASE(IFCTVL))
            CALL LCMLEN(IPOPT,'F-LA-PNLT',LENGT,ITYP2)
            IF (LENGT.EQ.0) CALL XABORT('TABU: PENALTY FUNCTION '
     1  //'EVALUATION NOT DONE (181)')
            CALL LCMGET(IPOPT,'F-LA-PNLT',RBASE(IWORK+NVAR+NCST+1))
         ENDIF
         CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(IWORK))
         CALL RLSARA(RBASE(ITSTVL))
         CALL RLSARA(RBASE(IWORK))
      ELSEIF (TEXT12.EQ.'INIT-PRO-LST') THEN
         IF ((PROLSM.EQ.0).OR.(PROLSM.GT.LISTMX)) CALL XABORT('TABU:'
     1  //'Promising List Length is 0 or infinite')
         CALL SETARA(RBASE,NVAR,INGBTS)
         CALL SETARA(RBASE,NVAR,IMINVL)
         CALL SETARA(RBASE,NVAR,IMAXVL)
         CALL SETARA(RBASE,NVAR,ITSTVL)
         CALL SETARA(RBASE,NVAR,ITSTEP)
         CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
         CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(IMINVL))
         CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(IMAXVL))
         DO 190 J=1,PROLSM
            IVALID=0
  182       CALL XDRSET(RBASE(IWORK),NVAR+NCST+2,0.0)
            DO 185 I=0,NVAR-1
               CALL RANDF(ISEED,IFIRST,RAND)
  185          RBASE(INGBTS+I)=RAND
       write(6,*) 'inl avant', J
            CALL TABINL(IPTABU,RBASE(INGBTS),RBASE(IMINVL),
     1      RBASE(IMAXVL),RBASE(ITSTVL),RBASE(ITSTEP),RBASE(IWORK),3,
     2      TBRAD,PRORAD,NGBTYP,TBLSM,TBLSC,PROLSM,PROLSC,LISTMX,NVAR,
     3      NCST,VALID)
       write(6,*) 'inl apres'
            IF ((VALID.EQ.0).AND.(IVALID.LE.NTRYNB)) THEN
               IVALID=IVALID+1
               IF (IVALID.EQ.NTRYNB) WRITE(6,*) 'TABU: Too many'
     1  //' attempts to create initial promising list'
               GO TO 182
            ENDIF
            CALL XDRSET(RBASE(IWORK),NVAR+NCST+2,0.0)
            DO 186 I=0,NVAR-1
  186          RBASE(IWORK+I)=RBASE(IMINVL+I)
     1                +RBASE(INGBTS+I)*(RBASE(IMAXVL+I)-RBASE(IMINVL+I))
       write(6,*) 'up avant'
            CALL TABUPD(IPTABU,PROLSM,PROLSC,2,RBASE(IWORK),NVAR,NCST,1,
     1                  MINMAX,TRSHLD)
       write(6,*) 'up apres'
  190    CONTINUE
         CALL RLSARA(RBASE(INGBTS))
         CALL RLSARA(RBASE(IMINVL))
         CALL RLSARA(RBASE(IMAXVL))
         CALL RLSARA(RBASE(ITSTVL))
         CALL RLSARA(RBASE(ITSTEP))
         CALL RLSARA(RBASE(IWORK))
      ELSEIF (TEXT12.EQ.'NELDER-EPS') THEN
         CALL REDGET(ITYP,NITMA,NLDEPS,TEXT12,DFLOTT)
         IF (ITYP.NE.2) CALL XABORT('TABU: REAL EXPECTED(101d)') 
         TBPRR(7)=NLDEPS
      ELSEIF (TEXT12.EQ.'ISEED') THEN
         CALL REDGET(ITYP,ISEED,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.1) CALL XABORT('TABU: INTEGER EXPECTED(101i)') 
         TBPRI(39)=ISEED
         IFIRST=1
         TBPRI(40)=IFIRST
      ELSE
         IF (IPRINT.GE.2) WRITE(6,5000)(TBPRI(I),I=1,NSTATE)
         CALL LCMPUT(IPTABU,'STATE-VECTOR',NSTATE,1,TBPRI)
         CALL LCMPUT(IPTABU,'TABU-PARAM-R',NSTATE,2,TBPRR)
         GO TO 30
      ENDIF
      IF (LTST) GO TO 101
  199 GO TO 100

C---------------------------
C   2222     0000     0000    
C  2    2   0    0   0    0 
C 2      2 0      0 0      0 
C       2  0      0 0      0  
C      2   0      0 0      0  
C   22     0      0 0      0  
C  2        0    0   0    0  
C 22222222   0000     0000  
C---------------------------
C NEIGHBORHOOD DEFINITION
C---------------------------
  200 CALL SETARA(RBASE,NVAR*NGB,INGBVL)
      CALL SETARA(RBASE,NVAR,INGBTS)
      CALL SETARA(RBASE,NVAR,INGBCU)
      CALL SETARA(RBASE,NVAR,IMINVL)
      CALL SETARA(RBASE,NVAR,IMAXVL)
      CALL SETARA(RBASE,NVAR,ITSTVL)
      CALL SETARA(RBASE,NVAR,ITSTEP)
      CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
      CALL SETARA(RBASE,NGB+1,INGBRD)
      CALL SETARA(RBASE,NGB,INGBFV)
      CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(IMINVL))
      CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(IMAXVL))
* Radius of the neighborhood crowns
      RBASE(INGBRD)=0.0
      IF (NGBDIS.EQ.1) THEN
         DO 202 I=1,NGB
  202       RBASE(INGBRD+I)=NBGRAD*GEOFAC**(REAL(I-NGB))
      ELSEIF (NGBDIS.EQ.2) THEN
         DO 203 I=1,NGB
  203       RBASE(INGBRD+I)=NBGRAD*(REAL(I))/(REAL(NGB))
      ELSEIF (NGBDIS.EQ.3) THEN
         DO 204 I=1,NGB
  204       RBASE(INGBRD+I)=NBGRAD
     1         *((REAL(I))/(REAL(NGB)))**(1.0/(REAL(NVAR)))
      ENDIF
!      IF (NGBDIS.EQ.1) THEN
!         DO 202 I=0,NGB
!  202       RBASE(INGBRD+I)=NBGRAD*GEOFAC**(REAL(I-NGB))
!      ELSEIF (NGBDIS.EQ.2) THEN
!         DO 203 I=0,NGB
!  203       RBASE(INGBRD+I)=NBGRAD*(REAL(I+1))/(REAL(NGB+1))
!      ELSEIF (NGBDIS.EQ.3) THEN
!         DO 204 I=0,NGB
!  204       RBASE(INGBRD+I)=NBGRAD
!     1         *((REAL(I+1))/(REAL(NGB+1)))**(1.0/(REAL(NVAR)))
!      ENDIF
* Current value (normalized between 0 and 1)
      CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(IWORK))
      DO 206 J=0,NVAR-1
          RBASE(INGBCU+J)=(RBASE(IWORK+J)-RBASE(IMINVL+J))
     1                   /(RBASE(IMAXVL+J)-RBASE(IMINVL+J))
  206 CONTINUE 		
      IF (IPRINT.GE.4) WRITE(6,*) 'CURRENT      :',
     1                      (RBASE(IWORK+J),J=0,NVAR-1)
      DO 280 I=1,NGB
         IVALID=0
  210    IVALID=IVALID+1
* calcul des deplacements relatifs par rapport au point courent
*    Case 1: hyperrectangle
         IF (NGBTYP.EQ.1) THEN
*           choix de la facette de l'hypercube
            CALL RANDF(ISEED,IFIRST,RAND)
            IFACE=INT(RAND*2.0*REAL(NVAR))-NVAR+1
            IF (IFACE.LE.0) IFACE=IFACE-1
            IF (IFACE.GT.NVAR) IFACE=NVAR
*           valeur de Dxj entre +/-(rj et rj-1) 
            CALL RANDF(ISEED,IFIRST,RAND)
            RBASE(INGBTS+ABS(IFACE)-1)=SIGN(RBASE(INGBRD+I-1)
     1           +RAND*(RBASE(INGBRD+I)-RBASE(INGBRD+I-1)),REAL(IFACE))
            IF (IPRINT.GE.5) WRITE(6,*) 'NBG rj rj-1 #',I,
     1       RBASE(INGBRD+I), RBASE(INGBRD+I-1),IFACE
*           choix des autres Dxj
            DO 230 J=1,NVAR
               IF (J.EQ.ABS(IFACE)) GO TO 230
               CALL RANDF(ISEED,IFIRST,RAND)
               RBASE(INGBTS+J-1)=RBASE(INGBRD+I)*(2.0*RAND-1.0)
  230          CONTINUE
*    Case 2: ball
         ELSEIF (NGBTYP.EQ.2) THEN
            XNORM=0.0
            DO 240 J=0,NVAR-1
               CALL RANDF(ISEED,IFIRST,RBASE(INGBTS+J))
  240          XNORM=RBASE(INGBTS+J)**2.0+XNORM
            CALL RANDF(ISEED,IFIRST,RAND)
            TEMP=RBASE(INGBRD+I-1)
     1           +RAND*(RBASE(INGBRD+I)-RBASE(INGBRD+I-1))
            XNORM=TEMP/XNORM**0.5
            DO 241 J=0,NVAR-1
  241          RBASE(INGBTS+J)=XNORM*RBASE(INGBTS+J)
         ENDIF
* calcul de la valeur test pour le voisin I 
*  (et verification pour les bornes inf et sup)
         DO 260 J=0,NVAR-1
            TEMP=RBASE(INGBTS+J)+RBASE(INGBCU+J)
            IF ((TEMP.LT.0.0).OR.(TEMP.GT.1.0)) THEN
               RBASE(INGBTS+J)=-RBASE(INGBTS+J)+RBASE(INGBCU+J)
            ELSE
               RBASE(INGBTS+J)=TEMP
            ENDIF
  260      CONTINUE
* validation du point aleatoire (tabu?)
      write(6,*) 'ngb creat avant inl'
         CALL TABINL(IPTABU,RBASE(INGBTS),RBASE(IMINVL),RBASE(IMAXVL),
     1        RBASE(ITSTVL),RBASE(ITSTEP),RBASE(IWORK),2,TBRAD,PRORAD,
     2        NGBTYP,TBLSM,TBLSC,PROLSM,PROLSC,LISTMX,NVAR,NCST,VALID)
      write(6,*) 'ngb creat ap inl'
         IF ((VALID.EQ.0).AND.(IVALID.LE.NTRYNB)) GO TO 210
         DO 270 J=0,NVAR-1
  270       RBASE(INGBVL+(I-1)*NVAR+J)=RBASE(INGBTS+J)
  280    CONTINUE
      CALL LCMSIX(IPTABU,'NEIGHBORHOOD',1)
      JPTABU=LCMLID(IPTABU,'ITEM',NGB)
      DO 290 I=1,NGB
!         WRITE(TEXT12,'(A4,I8)') 'ITEM',I
!         CALL XDRSET(RBASE(IWORK),NVAR+NCST+2,0.0)
         DO 285 J=0,NVAR-1
  285       RBASE(IWORK+J)=RBASE(IMINVL+J)
     1     +RBASE(INGBVL+(I-1)*NVAR+J)*(RBASE(IMAXVL+J)-RBASE(IMINVL+J))
         IF (IPRINT.GE.2) WRITE(6,*) 'NEIGHBOR #',I,':',
     1                        (RBASE(IWORK+J),J=0,NVAR+NCST+1)
      write(6,*) 'ngb creat av put',I
  290    CALL LCMPDL(JPTABU,I,NVAR+NCST+2,2,RBASE(IWORK))
!  290    CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(IWORK))
      CALL XDRSET(RBASE(INGBFV),NGB,MINMAX*1.0E10)
      CALL LCMPUT(IPTABU,'TABU-F-VAL',NGB,2,RBASE(INGBFV))
      CALL LCMSIX(IPTABU,' ',2)
      CALL RLSARA(RBASE(INGBVL))
      CALL RLSARA(RBASE(INGBTS))
      CALL RLSARA(RBASE(INGBCU))
      CALL RLSARA(RBASE(IMINVL))
      CALL RLSARA(RBASE(IMAXVL))
      CALL RLSARA(RBASE(ITSTVL))
      CALL RLSARA(RBASE(ITSTEP))
      CALL RLSARA(RBASE(IWORK))
      CALL RLSARA(RBASE(INGBRD))
      CALL RLSARA(RBASE(INGBFV))
      GO TO 20
C---------------------------
C   3333     0000     0000    
C  3    3   0    0   0    0 
C 3      3 0      0 0      0 
C      33  0      0 0      0  
C       3  0      0 0      0  
C 3      3 0      0 0      0  
C  3    3   0    0   0    0  
C   3333     0000     0000  
C---------------------------
C NEIGHBOR CHOICE
C---------------------------
  300 CALL REDGET(ITYP,CHOICE,FLOTT,TEXT12,DFLOTT)
      LTSTN=.FALSE.
      LTSTP=.FALSE.
      LTSTS=.FALSE.
      IF (TEXT12.EQ.'INIT-PRO-LST') THEN
         CALL REDGET(ITYP,CHOICE,FLOTT,TEXT12,DFLOTT)
         TXTREP='PROMISE-LIST'
         LTSTP=.TRUE.
      ELSEIF (TEXT12.EQ.'NELDER-MEAD') THEN
         CALL REDGET(ITYP,CHOICE,FLOTT,TEXT12,DFLOTT)
         TXTREP='NELDER-MEAD'
         LTSTS=.TRUE.
      ELSE
         TXTREP='NEIGHBORHOOD'
         LTSTN=.TRUE.
      ENDIF
      IF (ITYP.NE.1) CALL XABORT('TABU: INTEGER EXPECTED(300)')
      IF (LTSTN) THEN
         IF ((CHOICE.LE.0).OR.(CHOICE.GT.NGB)) CALL XABORT('TABU: '
     1  //'WRONG  NUMBER FOR NEIGHBOR(300)')
      ELSEIF (LTSTP) THEN
         IF ((CHOICE.LE.0).OR.(CHOICE.GT.PROLSM)) CALL XABORT('TABU: '
     1  //'WRONG  NUMBER FOR Promising list Item(300)')
      ELSEIF (LTSTS) THEN
         IF ((CHOICE.LE.0).OR.(CHOICE.GT.(NVAR+1))) CALL XABORT('TABU: '
     1  //'WRONG  NUMBER FOR Nelder-Mead list Item(300)')
      ENDIF
      CALL SETARA(RBASE,NVAR,ITSTVL)
      CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
      CALL XDRSET(RBASE(IWORK),NVAR+NCST+2,0.0)
      CALL LCMLEN(IPTABU,TXTREP,LENGT,ITYP)
      IF (LENGT.EQ.0) CALL XABORT('TABU: NO NEIGHBOR DEFINED')
      CALL LCMSIX(IPTABU,TXTREP,1)
      call LCMLEN(IPTABU,'ITEM',LENGT,ITYP)
      write(6,*) 'ici item', LENGT
      JPTABU=LCMGID(IPTABU,'ITEM')
!      WRITE(TEXT12,'(A4,I8)') 'ITEM',CHOICE
!      CALL LCMGET(IPTABU,TEXT12,RBASE(IWORK))
      write(6,*) 'ici choice', CHOICE
      CALL LCMGDL(JPTABU,CHOICE,RBASE(IWORK))
      CALL LCMSIX(IPTABU,' ',2)
      DO 310 I=0,NVAR-1
  310    RBASE(ITSTVL+I)=RBASE(IWORK+I)
      CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(ITSTVL))
      CALL RLSARA(RBASE(ITSTVL))
      CALL RLSARA(RBASE(IWORK))
      GO TO 20
C---------------------------
C     4      0000     0000    
C    44     0    0   0    0 
C   4 4    0      0 0      0 
C  4  4    0      0 0      0  
C 4   4    0      0 0      0  
C  4444444 0      0 0      0  
C     4     0    0   0    0  
C     4      0000     0000  
C---------------------------
C NEIGHBOR EVALUATION
C---------------------------
  400 CALL REDGET(ITYP,CHOICE,FLOTT,TEXT12,DFLOTT)
      LTSTN=.FALSE.
      LTSTP=.FALSE.
      LTSTS=.FALSE.
      IF (TEXT12.EQ.'INIT-PRO-LST') THEN
         CALL REDGET(ITYP,CHOICE,FLOTT,TEXT12,DFLOTT)
         TXTREP='PROMISE-LIST'
         LTSTP=.TRUE.
      ELSEIF (TEXT12.EQ.'NELDER-MEAD') THEN
         CALL REDGET(ITYP,CHOICE,FLOTT,TEXT12,DFLOTT)
         TXTREP='NELDER-MEAD'
         LTSTS=.TRUE.
      ELSE
         TXTREP='NEIGHBORHOOD'
         LTSTN=.TRUE.
      ENDIF
      IF (ITYP.NE.1) CALL XABORT('TABU: INTEGER EXPECTED(300)')
      IF (LTSTN) THEN
         IF ((CHOICE.LE.0).OR.(CHOICE.GT.NGB)) CALL XABORT('TABU: '
     1  //'WRONG  NUMBER FOR NEIGHBOR(300)')
      ELSEIF (LTSTP) THEN
         IF ((CHOICE.LE.0).OR.(CHOICE.GT.PROLSM)) CALL XABORT('TABU: '
     1  //'WRONG  NUMBER FOR Promising list Item(300)')
      ELSEIF (LTSTS) THEN
         IF ((CHOICE.LE.0).OR.(CHOICE.GT.(NVAR+1))) CALL XABORT('TABU: '
     1  //'WRONG  NUMBER FOR Nelder-Mead list Item(300)')
      ENDIF
      CALL SETARA(RBASE,NVAR,ITSTVL)
      CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
      CALL SETARA(RBASE,NCST+1,IFCTVL)
      CALL SETARA(RBASE,NCST,ICSTWT)
      CALL LCMGET(IPOPT,'VAR-VALUE',RBASE(ITSTVL))
      CALL LCMGET(IPOPT,'FOBJ-CST-VAL',RBASE(IFCTVL))
      CALL LCMGET(IPOPT,'CST-WEIGHT',RBASE(ICSTWT))
      CALL LCMGET(IPOPT,'F-LA-PNLT',TABUVL)
      DO 410 I=0,NVAR-1
  410    RBASE(IWORK+I)=RBASE(ITSTVL+I)
      DO 420 I=0,NCST
  420    RBASE(IWORK+I+NVAR)=RBASE(IFCTVL+I)
      RBASE(IWORK+NVAR+NCST+1)=TABUVL
      CALL LCMSIX(IPTABU,TXTREP,1)
      CALL LCMLEN(IPTABU,'TABU-F-VAL',LENGT,ITYP)
      CALL SETARA(RBASE,LENGT,INGBVL)
      CALL LCMGET(IPTABU,'TABU-F-VAL',RBASE(INGBVL))
      RBASE(INGBVL+CHOICE-1)=TABUVL
      CALL LCMPUT(IPTABU,'TABU-F-VAL',LENGT,2,RBASE(INGBVL))
      IF (LTSTP) THEN
         MEAN=0.0
         DO 430 I=0,LENGT-1
  430       MEAN=MEAN+RBASE(INGBVL+I)
         MEAN=MEAN/REAL(LENGT)
         TRSHLD=MEAN
      ENDIF
!      WRITE(TEXT12,'(A4,I8)') 'ITEM',CHOICE
!      IF (IPRINT.GE.2) WRITE(6,*) TEXT12
!      CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(IWORK))
      JPTABU=LCMGID(IPTABU,'ITEM')
      CALL LCMPDL(JPTABU,CHOICE,NVAR+NCST+2,2,RBASE(IWORK))
      CALL LCMSIX(IPTABU,' ',2)
      CALL RLSARA(RBASE(ITSTVL))
      CALL RLSARA(RBASE(IWORK))
      CALL RLSARA(RBASE(IFCTVL))
      CALL RLSARA(RBASE(ICSTWT))
      CALL RLSARA(RBASE(INGBVL))
      GO TO 20
C---------------------------
C 55555555   0000     0000    
C 5         0    0   0    0 
C 5  55    0      0 0      0 
C 555  55  0      0 0      0  
C        5 0      0 0      0  
C        5 0      0 0      0  
C 55    55  0    0   0    0  
C  555555    0000     0000  
C---------------------------
C BEST NEIGHBOR
C---------------------------
  500 LCVT=.FALSE.
      LPRO=.FALSE.
      CALL SETARA(RBASE,NGB,INGBFV)
      CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
* meilleur voisin? update du meileur?
      CALL LCMSIX(IPTABU,'NEIGHBORHOOD',1)
      CALL LCMGET(IPTABU,'TABU-F-VAL',RBASE(INGBFV))
      IMIN=0
      BESTN=MINMAX*1.0E10
      DO 510 I=0,NGB-1
!         WRITE(TEXT12,'(A4,I8)') 'ITEM',I+1
!         CALL LCMGET(IPTABU,TEXT12,RBASE(IWORK))
         IF ((MINMAX*RBASE(INGBFV+I)).LT.(MINMAX*BESTN)) THEN
            IMIN=I+1
            BESTN=RBASE(INGBFV+I)
         ENDIF
  510    CONTINUE
      BESTN=RBASE(INGBFV+IMIN-1)
      IF (IPRINT.GT.2) WRITE(6,*) 'Neigb.vs.Ever',BESTN,BEST
      IF ((MINMAX*BESTN).LT.(MINMAX*BEST)) THEN
         NITCUR=0
!         WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!         CALL LCMGET(IPTABU,TEXT12,RBASE(IWORK))
      	 JPTABU=LCMGID(IPTABU,'ITEM')
         CALL LCMGDL(JPTABU,IMIN,RBASE(IWORK))
         CALL LCMSIX(IPTABU,' ',2)
         CALL LCMPUT(IPTABU,'BEST-VAL',NVAR+NCST+2,2,RBASE(IWORK))
         TBPRR(5)=BESTN
      ELSE
         NITCUR=NITCUR+1
         CALL LCMSIX(IPTABU,' ',2)
      ENDIF
* convergence?         
  550 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED(550)')
      IF (TEXT12.EQ.'CONV-TEST') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('QLPUTL: OUTPUT LOGICAL EXPECTED')
         ITYP=5
         VLDTST=-1
         IF (NITCUR.GT.NITMAX) VLDTST=1
         CALL REDPUT(ITYP,VLDTST,FLOTT,TEXT12,DFLOTT)
         GO TO 550
* promising area ?
      ELSEIF (TEXT12.EQ.'PROMISE-TEST') THEN
         LTST=.TRUE.
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (TEXT12.EQ.'NO-THRESHOLD') THEN
            LTST=.FALSE.
            CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         ENDIF
         IF (ITYP.NE.-5) CALL XABORT('QLPUTL: OUTPUT LOGICAL EXPECTED')
         CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(IWORK))
         TABUVL=RBASE(IWORK+NVAR+NCST+1)
         IF ((MINMAX*TABUVL).LT.(MINMAX*BESTN)) LPRO=.TRUE.
         IF (LTST) THEN
            IF ((MINMAX*TABUVL).GT.(MINMAX*TRSHLD)) LPRO=.FALSE.
         ENDIF
         IF (LPRO) THEN
            CALL SETARA(RBASE,NVAR,INGBTS)
            CALL SETARA(RBASE,NVAR,IMINVL)
            CALL SETARA(RBASE,NVAR,IMAXVL)
            CALL SETARA(RBASE,NVAR,ITSTVL)
            CALL SETARA(RBASE,NVAR,ITSTEP)
            CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(IMINVL))
            CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(IMAXVL))
            DO 555 I=0,NVAR-1
               RBASE(INGBTS+I)=(RBASE(IWORK+I)-RBASE(IMINVL+I))
     1                         /(RBASE(IMAXVL+I)-RBASE(IMINVL+I))
  555          CONTINUE
            CALL TABINL(IPTABU,RBASE(INGBTS),RBASE(IMINVL),
     1        RBASE(IMAXVL),RBASE(ITSTVL),RBASE(ITSTEP),RBASE(IWORK),3,
     2        TBRAD,PRORAD,NGBTYP,TBLSM,TBLSC,PROLSM,PROLSC,LISTMX,NVAR,
     3        NCST,VALID) 
            IF (VALID.EQ.0) LPRO=.FALSE.
            CALL RLSARA(RBASE(INGBTS))
            CALL RLSARA(RBASE(IMINVL))
            CALL RLSARA(RBASE(IMAXVL))
            CALL RLSARA(RBASE(ITSTVL))
            CALL RLSARA(RBASE(ITSTEP))
         ENDIF
         ITYP=5
         VLDTST=-1
         IF (LPRO) VLDTST=1
         CALL REDPUT(ITYP,VLDTST,FLOTT,TEXT12,DFLOTT)
         GO TO 550
      ELSE 
         TXT12B=TEXT12
      ENDIF
* copier le meilleur voisin en temps que CURRENT
      IF (.NOT.LPRO) THEN
         CALL LCMSIX(IPTABU,'NEIGHBORHOOD',1)
!         WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!         CALL LCMGET(IPTABU,TEXT12,RBASE(IWORK))
      	 JPTABU=LCMGID(IPTABU,'ITEM')
         CALL LCMGDL(JPTABU,IMIN,RBASE(IWORK))
         CALL LCMSIX(IPTABU,' ',2)
         CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(IWORK))
* mettre tabu le meilleur voisin
         CALL TABUPD(IPTABU,TBLSM,TBLSC,1,RBASE(IWORK),NVAR,NCST,1,
     1               MINMAX,TRSHLD)
      ENDIF
      CALL RLSARA(RBASE(INGBFV))
      CALL RLSARA(RBASE(IWORK))
      TEXT12=TXT12B
      GO TO 30
C---------------------------
C  666666    0000     0000    
C 6      6  0    0   0    0 
C 6  66    0      0 0      0 
C 666  66  0      0 0      0  
C 6      6 0      0 0      0  
C 6      6 0      0 0      0  
C  6    6   0    0   0    0  
C   6666     0000     0000  
C---------------------------
C PROMISING AREA
C---------------------------
  600 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED(600)')
      IF (TEXT12.EQ.'NELDER-MEAD') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED(600)')
         LTSTS=.TRUE.
      ELSE 
         LTSTS=.FALSE.
      ENDIF
* Preparation pour un calcul de gradient local
      IF (TEXT12.EQ.'CREATION') THEN
         CALL SETARA(RBASE,NVAR+NCST+2,INGBCU)
         CALL SETARA(RBASE,NVAR,IMINVL)
         CALL SETARA(RBASE,NVAR,IMAXVL)
         CALL SETARA(RBASE,NVAR,ITSTVL)
         CALL SETARA(RBASE,NVAR,IVARWT)
         CALL LCMGET(IPOPT,'VAR-WEIGHT',RBASE(IVARWT))
*   backup des bornes
         CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(IMINVL))
         CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(IMAXVL))
         CALL LCMSIX(IPTABU,'BACKUP',1)
         CALL LCMPUT(IPTABU,'VAR-VAL-MIN',NVAR,2,RBASE(IMINVL))
         CALL LCMPUT(IPTABU,'VAR-VAL-MAX',NVAR,2,RBASE(IMAXVL))
         CALL LCMSIX(IPTABU,' ',2)
*   calcul des nouvelles bornes
         CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(INGBCU))
         S=0.0
         DO 610 I=0,NVAR-1
            RBASE(ITSTVL+I)=RBASE(INGBCU+I)
            S=S+RBASE(IVARWT+I)
     1           *(PRORAD*(RBASE(IMAXVL+I)-RBASE(IMINVL+I)))**2
            TEMP=PRORAD*(RBASE(IMAXVL+I)-RBASE(IMINVL+I))
            RBASE(IMINVL+I)=MAX(RBASE(IMINVL+I),RBASE(ITSTVL+I)-TEMP)
            RBASE(IMAXVL+I)=MIN(RBASE(IMAXVL+I),RBASE(ITSTVL+I)+TEMP)
  610       CONTINUE
!         S=((S/REAL(NVAR))**0.5)/2.0
!         OPTPRR(1)=S
         CALL LCMPUT(IPOPT,'VAR-VALUE',NVAR,2,RBASE(ITSTVL))
         CALL LCMPUT(IPOPT,'VAR-VAL-MIN',NVAR,2,RBASE(IMINVL))
         CALL LCMPUT(IPOPT,'VAR-VAL-MAX',NVAR,2,RBASE(IMAXVL))
         CALL LCMPUT(IPOPT,'OPT-PARAM-R',NSTATE,2,OPTPRR)
         IF (LTSTS) THEN
            CALL SETARA(RBASE,NVAR+NCST+2,INLDR)
            CALL SETARA(RBASE,NVAR+1,INLDRV)
            CALL LCMSIX(IPTABU,'NELDER-MEAD',1)
            CALL XDRSET(RBASE(INLDRV),NVAR+1,0.0)
            RBASE(INLDRV)=RBASE(INGBCU+NVAR+NCST+1)
            CALL LCMPUT(IPTABU,'TABU-F-VAL',NVAR+1,2,RBASE(INLDRV))
!            CALL LCMPUT(IPTABU,'ITEM       1',NVAR+NCST+2,2,
!     1                  RBASE(INGBCU))
            CALL LCMLEN(IPTABU,'ITEM',LENGT,ITYP)
            IF (LENGT.EQ.0) THEN
               JPTABU=LCMLID(IPTABU,'ITEM',NVAR+1)
            ELSE   
               JPTABU=LCMGID(IPTABU,'ITEM')
            ENDIF
            CALL LCMPDL(JPTABU,1,NVAR+NCST+2,2,RBASE(INGBCU))
            CALL XDRSET(RBASE(INLDR),NVAR+NCST+2,0.0)
            DO 620 I=0,NVAR-1
               RBASE(INLDR+I)=RBASE(INGBCU+I)
  620          CONTINUE
            DO 630 I=1,NVAR
               IF (I.GT.1) RBASE(INLDR+I-2)=RBASE(INGBCU+I-2)
               DELTA=EPSNM*(RBASE(IMAXVL+I-1)-RBASE(IMINVL+I-1))
               RBASE(INLDR+I-1)=RBASE(INGBCU+I-1)+DELTA
!               WRITE(TEXT12,'(A4,I8)') 'ITEM',I+1
!               CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(INLDR))
               CALL LCMPDL(JPTABU,I+1,NVAR+NCST+2,2,RBASE(INLDR))
  630          CONTINUE
            CALL LCMSIX(IPTABU,' ',2)
            CALL RLSARA(RBASE(INLDR))
            CALL RLSARA(RBASE(INLDRV))
         ENDIF
         CALL RLSARA(RBASE(INGBCU))
         CALL RLSARA(RBASE(IMINVL))
         CALL RLSARA(RBASE(IMAXVL))
         CALL RLSARA(RBASE(ITSTVL))
         CALL RLSARA(RBASE(IVARWT))
* retour au calcul de tabu search sur tout le domaine
      ELSEIF (TEXT12.EQ.'UPDATE') THEN
         CALL SETARA(RBASE,NVAR,IMINVL)
         CALL SETARA(RBASE,NVAR,IMAXVL)
         CALL SETARA(RBASE,NVAR,ITSTVL)
         CALL SETARA(RBASE,NVAR+NCST+2,IWORK)
         CALL SETARA(RBASE,NCST+1,IFCTVL)
*   recover des bornes
         CALL LCMSIX(IPTABU,'BACKUP',1)
         CALL LCMGET(IPTABU,'VAR-VAL-MIN',RBASE(IMINVL))
         CALL LCMGET(IPTABU,'VAR-VAL-MAX',RBASE(IMAXVL))
         CALL LCMSIX(IPTABU,' ',2)
         CALL LCMPUT(IPOPT,'VAR-VAL-MIN',NVAR,2,RBASE(IMINVL))
         CALL LCMPUT(IPOPT,'VAR-VAL-MAX',NVAR,2,RBASE(IMAXVL))
*   amelioration de la fonction tabu?
         BEST=TBPRR(5)
         IF (LTSTS) THEN
            CALL LCMSIX(IPTABU,'NELDER-MEAD',1)
            CALL LCMGET(IPTABU,'IBEST',IMAX)
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',IMAX
!            CALL LCMGET(IPTABU,TEXT12,RBASE(IWORK))
            JPTABU=LCMGID(IPTABU,'ITEM')
            CALL LCMGDL(JPTABU,IMAX,RBASE(IWORK))
            CALL LCMSIX(IPTABU,' ',2)
            TEMP=RBASE(IWORK+NVAR+NCST+1)
         ELSE
            CALL LCMGET(IPOPT,'F-LA-PNLT',TEMP)
            CALL LCMGET(IPOPT,'VAR-VALUE',RBASE(ITSTVL))
            CALL LCMGET(IPOPT,'FOBJ-CST-VAL',RBASE(IFCTVL))
            DO 660 I=0,NVAR-1
  660          RBASE(IWORK+I)=RBASE(ITSTVL+I)
            DO 670 I=0,NCST
  670          RBASE(IWORK+I+NVAR)=RBASE(IFCTVL+I)
            RBASE(IWORK+NVAR+NCST+1)=TEMP
         ENDIF
         CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(IWORK))
         CALL TABUPD(IPTABU,PROLSM,PROLSC,2,RBASE(IWORK),NVAR,NCST,1,
     1               MINMAX,TRSHLD)
         IF (IPRINT.GE.2) WRITE(6,*) 'Current=',TEMP,' <> Best=',BEST
         IF (MINMAX*(TEMP-BEST).LT.0.0) THEN
            CALL LCMPUT(IPTABU,'BEST-VAL',NVAR+NCST+2,2,RBASE(IWORK))
            BEST=TEMP
            TBPRR(5)=BEST
            CALL LCMPUT(IPTABU,'TABU-PARAM-R',NSTATE,2,TBPRR)
            TBLSC =0
         ENDIF
         CALL RLSARA(RBASE(IMINVL))
         CALL RLSARA(RBASE(IMAXVL))
         CALL RLSARA(RBASE(ITSTVL))
         CALL RLSARA(RBASE(IWORK))
         CALL RLSARA(RBASE(IFCTVL))
      ELSE
         WRITE(6,*) 'Nothing performed, keyword missing after '
     1  //'PROMISE-AREA'
         GO TO 30
      ENDIF
      GO TO 20
C---------------------------
C 77777777   0000     0000    
C       7   0    0   0    0 
C      7   0      0 0      0 
C     7    0      0 0      0  
C  77777   0      0 0      0  
C   7      0      0 0      0  
C  7        0    0   0    0  
C 7          0000     0000  
C---------------------------
C Nelder-Mead
C---------------------------
  700 CALL SETARA(RBASE,NVAR+NCST+2,INLDR)
      CALL SETARA(RBASE,NVAR+1,INLDRV)
      CALL SETARA(RBASE,NVAR,INLDRM)
      CALL SETARA(RBASE,NVAR,IMINVL)
      CALL SETARA(RBASE,NVAR,IMAXVL)
      CALL LCMGET(IPOPT,'VAR-VAL-MIN',RBASE(IMINVL))
      CALL LCMGET(IPOPT,'VAR-VAL-MAX',RBASE(IMAXVL))
      CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
      IF (ITYP.NE.3) CALL XABORT('TABU: CHARACTER EXPECTED(600)')
*     initialisation pour test de convergence
      CALL SETARA(RBASE,NVAR+NCST+2,INLDRO)
      CALL SETARA(RBASE,NVAR,IVARWT)
      CALL LCMGET(IPOPT,'VAR-WEIGHT',RBASE(IVARWT))
      CONV=0.0
* Do the reflection
      IF (TEXT12.EQ.'FIND-NEW') THEN
         CALL LCMSIX(IPTABU,'NELDER-MEAD',1)
         CALL LCMGET(IPTABU,'TABU-F-VAL',RBASE(INLDRV))
C   find the worst, the second worst and best point
         WORST =-MINMAX*1.0E20
         WORST2=-MINMAX*1.0E20
         BESTND=MINMAX*1.0E20
         IMIN=-1
         IMIN2=-1
         IMAX=-1
         DO 710 I=0,NVAR
            WORST=MINMAX*MAX(MINMAX*WORST,MINMAX*RBASE(INLDRV+I))
            BESTND=MINMAX*MIN(MINMAX*BESTND,MINMAX*RBASE(INLDRV+I))
            IF (WORST.EQ.RBASE(INLDRV+I)) THEN
               IMIN2=IMIN
               IMIN=I+1
            ELSE
               WORST2=MINMAX*MAX(MINMAX*WORST2,MINMAX*RBASE(INLDRV+I))
               IF (WORST2.EQ.RBASE(INLDRV+I)) IMIN2=I+1
            ENDIF
            IF (BESTND.EQ.RBASE(INLDRV+I)) IMAX=I+1
  710       CONTINUE
         CALL LCMPUT(IPTABU,'IWORST',1,1,IMIN)
         CALL LCMPUT(IPTABU,'IWORST2',1,1,IMIN2)
         CALL LCMPUT(IPTABU,'IBEST',1,1,IMAX)
*   calculate the average point
         CALL XDRSET(RBASE(INLDRM),NVAR,0.0)
         DO 720 J=1,NVAR+1
            IF (J.EQ.IMIN) GO TO 720
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',J
!            CALL LCMGET(IPTABU,TEXT12,RBASE(INLDR))
            JPTABU=LCMGID(IPTABU,'ITEM')
            CALL LCMGDL(JPTABU,J,RBASE(INLDR))
            DO 715 I=0,NVAR-1
  715          RBASE(INLDRM+I)=RBASE(INLDRM+I)+RBASE(INLDR+I)
  720       CONTINUE
         DO 725 I=0,NVAR-1
  725       RBASE(INLDRM+I)=RBASE(INLDRM+I)/REAL(NVAR)
         CALL LCMPUT(IPTABU,'XAVERG',NVAR,2,RBASE(INLDRM))
*   calculate the refection point Xr=-a.Xw+(1+a)Xm
!         WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!         CALL LCMGET(IPTABU,TEXT12,RBASE(INLDR))
         JPTABU=LCMGID(IPTABU,'ITEM')
         CALL LCMGDL(JPTABU,IMIN,RBASE(INLDR))
         CALL XDRSET(RBASE(INLDR+NVAR),NCST+2,0.0)
         DO 730 I=0,NVAR-1
  730       RBASE(INLDR+I)=-ALPHA*RBASE(INLDR+I)
     1                      +(1.0+ALPHA)*RBASE(INLDRM+I)
         CALL TABMML(RBASE(INLDR),RBASE(IMINVL),RBASE(IMAXVL),NVAR)
         CALL LCMSIX(IPTABU,' ',2)
         CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(INLDR))
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-1) CALL XABORT('TABU: INTEGER EXPECTED(730)')
         CALL REDPUT(-ITYP,IMIN,FLOTT,TEXT12,DFLOTT)
C Expansion/Next point/Contraction test
      ELSEIF (TEXT12.EQ.'COMPARE-NEW') THEN
         LEXP=.FALSE.
         LCTT=.FALSE.
         LCVG=.FALSE.
         CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(INLDR))
         CALL LCMSIX(IPTABU,'NELDER-MEAD',1)
         CALL LCMGET(IPTABU,'TABU-F-VAL',RBASE(INLDRV))
         CALL LCMPUT(IPTABU,'REFLECT-VAL',NVAR+NCST+2,2,RBASE(INLDR))
         CALL LCMGET(IPTABU,'IWORST',IMIN)
         CALL LCMGET(IPTABU,'IWORST2',IMIN2)
         CALL LCMGET(IPTABU,'IBEST',IMAX)
         CALL LCMGET(IPTABU,'XAVERG',RBASE(INLDRM))
         FVAL=RBASE(INLDR+NVAR+NCST+1)
         FVALW=RBASE(INLDRV+IMIN-1)
         FVALW2=RBASE(INLDRV+IMIN2-1)
         FVALB=RBASE(INLDRV+IMAX-1)
*   expansion possible Xe=g.Xr+(1-g)Xm
         IF (MINMAX*FVAL.LT.MINMAX*FVALB) THEN
            LEXP=.TRUE.
            DO 750 I=0,NVAR-1
  750       RBASE(INLDR+I)=GAMMA*RBASE(INLDR+I)
     1                      +(1.0-GAMMA)*RBASE(INLDRM+I)
            CALL TABMML(RBASE(INLDR),RBASE(IMINVL),RBASE(IMAXVL),NVAR)
            CALL XDRSET(RBASE(INLDR+NVAR),NCST+2,0.0)
            CALL LCMSIX(IPTABU,' ',2)
            CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(INLDR))
*    juste reflection
         ELSEIF (MINMAX*FVAL.LT.MINMAX*FVALW2) THEN
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!            CALL LCMGET(IPTABU,TEXT12,RBASE(INLDRO))
!            CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(INLDR))
            JPTABU=LCMGID(IPTABU,'ITEM')
            CALL LCMGDL(JPTABU,IMIN,RBASE(INLDRO))
            CALL LCMPDL(JPTABU,IMIN,NVAR+NCST+2,2,RBASE(INLDR))
            RBASE(INLDRV+IMIN-1)=RBASE(INLDR+NVAR+NCST+1)
            CALL LCMPUT(IPTABU,'TABU-F-VAL',NVAR+1,2,RBASE(INLDRV))
            CALL LCMSIX(IPTABU,' ',2)
            CALL TABNMC(RBASE(INLDR),RBASE(INLDRO),RBASE(IVARWT),NLDEPS,
     1                  NVAR,LCVG)
*    contraction externe Xc'=b.Xr+(1-b)Xm
         ELSEIF (MINMAX*FVAL.LT.MINMAX*FVALW) THEN
            LCTT=.TRUE.
            DO 760 I=0,NVAR-1
  760       RBASE(INLDR+I)=BETA*RBASE(INLDR+I)
     1                      +(1.0-BETA)*RBASE(INLDRM+I)
            CALL TABMML(RBASE(INLDR),RBASE(IMINVL),RBASE(IMAXVL),NVAR)
            CALL XDRSET(RBASE(INLDR+NVAR),NCST+2,0.0)
            CALL LCMSIX(IPTABU,' ',2)
            CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(INLDR))
*    contraction interne Xc=b.Xw+(1-b)Xm
         ELSE
            LCTT=.TRUE.
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!            CALL LCMGET(IPTABU,TEXT12,RBASE(INLDR))
            JPTABU=LCMGID(IPTABU,'ITEM')
            CALL LCMGDL(JPTABU,IMIN,RBASE(INLDR))
            DO 770 I=0,NVAR-1
  770       RBASE(INLDR+I)=BETA*RBASE(INLDR+I)
     1                      +(1.0-BETA)*RBASE(INLDRM+I)
            CALL TABMML(RBASE(INLDR),RBASE(IMINVL),RBASE(IMAXVL),NVAR)
            CALL XDRSET(RBASE(INLDR+NVAR),NCST+2,0.0)
            CALL LCMSIX(IPTABU,' ',2)
            CALL LCMPUT(IPTABU,'CURRENT-VAL',NVAR+NCST+2,2,RBASE(INLDR))
         ENDIF
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('TABU: LOGICAL EXPECTED(760)')
         IF (LEXP) THEN
            VTST=1
         ELSE
            VTST=-1
         ENDIF
         CALL REDPUT(-ITYP,VTST,FLOTT,TEXT12,DFLOTT)
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('TABU: LOGICAL EXPECTED(760b)')
         IF (LCTT) THEN
            VTST=1
         ELSE
            VTST=-1
         ENDIF
         CALL REDPUT(-ITYP,VTST,FLOTT,TEXT12,DFLOTT)
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('TABU: LOGICAL EXPECTED(760c)')
         IF (LCVG) THEN
            VTST=1
         ELSE
            VTST=-1
         ENDIF
         CALL REDPUT(-ITYP,VTST,FLOTT,TEXT12,DFLOTT)
C Expansion validity test
      ELSEIF (TEXT12.EQ.'EXPAN-VLD') THEN
         CALL SETARA(RBASE,NVAR+NCST+2,INLDRR)
         CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(INLDR))
         CALL LCMSIX(IPTABU,'NELDER-MEAD',1)
         CALL LCMGET(IPTABU,'TABU-F-VAL',RBASE(INLDRV))
         CALL LCMGET(IPTABU,'REFLECT-VAL',RBASE(INLDRR))
         CALL LCMGET(IPTABU,'IWORST',IMIN)
         FVALE=RBASE(INLDR+NVAR+NCST+1)
         FVAL=RBASE(INLDRR+NVAR+NCST+1)
!         WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!         CALL LCMGET(IPTABU,TEXT12,RBASE(INLDRO))
         JPTABU=LCMGID(IPTABU,'ITEM')
         CALL LCMGDL(JPTABU,IMIN,RBASE(INLDRO))
C    expansion non valide
         IF (MINMAX*FVAL.LT.MINMAX*FVALE) THEN
            IF (IPRINT.GT.3) WRITE(6,*) 'expansion non valide'
            CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(INLDRR))
            RBASE(INLDRV+IMIN-1)=RBASE(INLDRR+NVAR+NCST+1)
            CALL TABNMC(RBASE(INLDRR),RBASE(INLDRO),RBASE(IVARWT),
     1               NLDEPS,NVAR,LCVG)
*    expansion valide
         ELSE
            IF (IPRINT.GT.3) WRITE(6,*) 'expansion valide'
            CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(INLDR))
            RBASE(INLDRV+IMIN-1)=RBASE(INLDR+NVAR+NCST+1)
            CALL TABNMC(RBASE(INLDR),RBASE(INLDRO),RBASE(IVARWT),NLDEPS,
     1               NVAR,LCVG)
         ENDIF
         CALL LCMPUT(IPTABU,'TABU-F-VAL',NVAR+1,2,RBASE(INLDRV))
         CALL LCMSIX(IPTABU,' ',2)
         CALL RLSARA(RBASE(INLDRR))          
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('TABU: LOGICAL EXPECTED(760b)')
         IF (LCVG) THEN
            VTST=1
         ELSE
            VTST=-1
         ENDIF
         CALL REDPUT(-ITYP,VTST,FLOTT,TEXT12,DFLOTT)
* Contraction/ Multicontraction / Convergence test
      ELSEIF (TEXT12.EQ.'CONTRACTION') THEN
         LCTT=.FALSE.
         LCVG=.FALSE.
         CALL LCMGET(IPTABU,'CURRENT-VAL',RBASE(INLDR))
         CALL LCMSIX(IPTABU,'NELDER-MEAD',1)
         CALL LCMGET(IPTABU,'TABU-F-VAL',RBASE(INLDRV))
         CALL LCMGET(IPTABU,'IWORST',IMIN)
         CALL LCMGET(IPTABU,'IBEST',IMAX)
         FVAL=RBASE(INLDR+NVAR+NCST+1)
         FVALW=RBASE(INLDRV+IMIN-1)
*  multicontraction
         IF (MINMAX*FVAL.GT.MINMAX*FVALW) THEN
            LCTT=.TRUE.
*     Xval of the best=Xval_best
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',IMAX
!            CALL LCMGET(IPTABU,TEXT12,RBASE(INLDR))
            JPTABU=LCMGID(IPTABU,'ITEM')
            CALL LCMGDL(JPTABU,IMAX,RBASE(INLDR))
            DO 765 I=0,NVAR-1
  765          RBASE(INLDRM+I)=RBASE(INLDR+I)
               IF (IPRINT.GE.1) WRITE(6,*) 'XBEST',
     1                      (RBASE(INLDRM+K),K=0,NVAR-1)
*     (Xval+Xval_best)/2
            DO 780 J=1,NVAR+1
               IF (J.EQ.IMAX) GOTO 780
!               WRITE(TEXT12,'(A4,I8)') 'ITEM',J
!               CALL LCMGET(IPTABU,TEXT12,RBASE(INLDR))
               JPTABU=LCMGID(IPTABU,'ITEM')
               CALL LCMGDL(JPTABU,J,RBASE(INLDR))
               DO 775 I=0,NVAR-1
                  CONV=CONV+RBASE(IVARWT+I)
     1                      *(RBASE(INLDR+I)-RBASE(INLDRM+I))**2.0
  775             RBASE(INLDR+I)=(RBASE(INLDR+I)+RBASE(INLDRM+I))/2.0
               CALL XDRSET(RBASE(INLDR+NVAR),NCST+2,0.0)
!               CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(INLDR))
               CALL LCMPDL(JPTABU,J,NVAR+NCST+2,2,RBASE(INLDR))
  780          CONTINUE
*     convergence
            CONV=CONV/REAL(NVAR**2)
            IF (CONV.LT.NLDEPS) LCVG=.TRUE.
            CALL LCMSIX(IPTABU,' ',2)
* Contraction simple
         ELSE 
!            WRITE(TEXT12,'(A4,I8)') 'ITEM',IMIN
!            CALL LCMGET(IPTABU,TEXT12,RBASE(INLDRO))
!            CALL LCMPUT(IPTABU,TEXT12,NVAR+NCST+2,2,RBASE(INLDR))
            JPTABU=LCMGID(IPTABU,'ITEM')
            CALL LCMGDL(JPTABU,IMIN,RBASE(INLDRO))
            CALL LCMPDL(JPTABU,IMIN,NVAR+NCST+2,2,RBASE(INLDR))
            RBASE(INLDRV+IMIN-1)=RBASE(INLDR+NVAR+NCST+1)
            CALL LCMPUT(IPTABU,'TABU-F-VAL',NVAR+1,2,RBASE(INLDRV))
            CALL LCMSIX(IPTABU,' ',2)
            CALL TABNMC(RBASE(INLDR),RBASE(INLDRO),RBASE(IVARWT),NLDEPS,
     1               NVAR,LCVG)
         ENDIF         
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('TABU: LOGICAL EXPECTED(760b)')
         IF (LCTT) THEN
            VTST=1
         ELSE
            VTST=-1
         ENDIF
         CALL REDPUT(-ITYP,VTST,FLOTT,TEXT12,DFLOTT)
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-1) CALL XABORT('TABU: INTEGER EXPECTED(730b)')
         CALL REDPUT(-ITYP,IMAX,FLOTT,TEXT12,DFLOTT)
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         IF (ITYP.NE.-5) CALL XABORT('TABU: LOGICAL EXPECTED(760b)')
         IF (LCVG) THEN
            VTST=1
         ELSE
            VTST=-1
         ENDIF
         CALL REDPUT(-ITYP,VTST,FLOTT,TEXT12,DFLOTT)
      ELSE
         WRITE(6,*) 'Nothing performed, keyword missing after '
     1  //'NELDER-MEAD'
         GO TO 30
      ENDIF
C     CALL LCMSIX(IPTABU,' ',2)
      CALL RLSARA(RBASE(INLDR))
      CALL RLSARA(RBASE(INLDRV))
      CALL RLSARA(RBASE(INLDRM))
      CALL RLSARA(RBASE(IMINVL))
      CALL RLSARA(RBASE(IMAXVL))
      CALL RLSARA(RBASE(INLDRO))
      CALL RLSARA(RBASE(IVARWT))
      GO TO 20
C---------------------------
C END
C---------------------------
 1000 CONTINUE
      TBPRI(5)=NITCUR
      TBPRI(8)=TBLSC
      TBPRI(9)=PROLSC
      TBPRI(39)=ISEED
      TBPRI(40)=IFIRST
      TBPRR(6)=TRSHLD
      CALL LCMPUT(IPTABU,'STATE-VECTOR',NSTATE,1,TBPRI)
      CALL LCMPUT(IPTABU,'TABU-PARAM-R',NSTATE,2,TBPRR)
      RETURN

C---------------------------
C FORMAT
C---------------------------
 5000 FORMAT (
     > ' STATE VECTOR:'/
     > (10P,8I12))
      END





















