*DECK BIVACA
      SUBROUTINE BIVACA(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* BIVAC type (2-D) system matrix assembly operator.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_SYSTEM);
*         HENTRY(2): read-only type(L_MACROLIB);
*         HENTRY(3): read-only type(L_TRACK).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  	
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      CHARACTER TEXT4*4,HSIGN*12,TEXT12*12,HSMG*131,CNAM*12
      DOUBLE PRECISION DFLOTT
      INTEGER IGP(NSTATE),IPAR(NSTATE),IBR(NSTATE),KCHAR(3),IBASE(1)
      LOGICAL LDIFF
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION.
*----
      IF(NENTRY.LE.2) CALL XABORT('BIVACA: THREE PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('BIVACA: L'
     1 //'CM OBJECT EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('BIVACA: E'
     1 //'NTRY IN CREATE OR MODIFICATION MODE EXPECTED.')
      IF((JENTRY(2).NE.2).OR.((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)))
     1 CALL XABORT('BIVACA: LCM OBJECT IN READ-ONLY MODE EXPECTED AT S'
     2 //'ECOND RHS.')
      IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)))
     1 CALL XABORT('BIVACA: LCM OBJECT IN READ-ONLY MODE EXPECTED AT F'
     2 //'IRST RHS.')
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_TRACK') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('BIVACA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_TRACK EXPECTED.')
      ENDIF
      CALL LCMGET(KENTRY(3),'TRACK-TYPE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'BIVAC') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('BIVACA: TRACK-TYPE OF '//TEXT12//' IS '//HSIGN//
     1   '. BIVAC EXPECTED.')
      ENDIF
      HSIGN='L_SYSTEM'
      IPSYS=KENTRY(1)
      READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'SIGNATURE',3,3,KCHAR)
      IPMACR=KENTRY(2)
      IPTRK=KENTRY(3)
      TEXT12=HENTRY(2)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'LINK.MACRO',3,3,KCHAR)
      TEXT12=HENTRY(3)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'LINK.TRACK',3,3,KCHAR)
*----
*  RECOVER GENERAL TRACKING INFORMATION.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      NEL=IGP(1)
      NLF=IGP(14)
      ISCAT=IGP(16)
      LDIFF=(ISCAT.LT.0)
      ISCAT=ABS(ISCAT)
      IF((NLF.NE.0).AND.(IGP(15).NE.1)) CALL XABORT('BIVACA: ONLY SPN '
     1 //'DISCRETIZATIONS ARE ALLOWED.')
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(BASE,NEL,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
*----
*  RECOVER MACROLIB PARAMETERS.
*----
      CALL LCMGET(IPMACR,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_MACROLIB') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('BIVACA: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_MACROLIB EXPECTED.')
      ENDIF
      CALL LCMGET(IPMACR,'STATE-VECTOR',IPAR)
      NGRP=IPAR(1)
      NBMIX=IPAR(2)
      NANI=IPAR(3)
      NBFIS=IPAR(4)
      IF(IGP(4).GT.NBMIX) THEN
         WRITE(HSMG,'(46HBIVACA: THE NUMBER OF MIXTURES IN THE TRACKING,
     1   2H (,I5,51H) IS GREATER THAN THE NUMBER OF MIXTURES IN THE MAC,
     2   7HROLIB (,I5,2H).)') IGP(4),NBMIX
         CALL XABORT(HSMG)
      ENDIF
*
      IMPX=1
      IUNIT=0
      IOVEL=0
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 40
      IF(INDIC.NE.3) CALL XABORT('BIVACA: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('BIVACA: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'UNIT') THEN
*       COMPUTE THE UNITARY WEIGHTING MATRIX.
        IUNIT=1
        CALL SETARA(BASE,NBMIX,IUN)
        CALL XDRSET(BASE(IUN),NBMIX,1.0)
        CALL BIVASM('RM',1,IPTRK,IPSYS,IMPX,NBMIX,NEL,0,1,IBASE(IMAT),
     1  BASE(IVOL),BASE(IUN))
        CALL RLSARA(BASE(IUN))
      ELSE IF(TEXT4.EQ.'OVEL') THEN
*        COMPUTE THE RECIPROCAL NEUTRON VELOCITIES MATRIX.
         IOVEL=1
         JPMACR=LCMGID(IPMACR,'GROUP')
         DO 30 IGR=1,NGRP
         CALL SETARA(BASE,NBMIX,IVII)
         KPMACR=LCMGIL(JPMACR,IGR)
         CALL LCMLEN(KPMACR,'OVERV',LENGT,ITYLCM)
         IF(LENGT.EQ.0) THEN
            CALL XABORT('BIVACA: NO ''VELOCITY'' INFORMATION.')
         ELSE IF(LENGT.GT.NBMIX) THEN
            CALL XABORT('BIVACA: INVALID LENGTH FOR ''VELOCITY'' IN'
     1      //'FORMATION.')
         ENDIF
         CALL LCMGET(KPMACR,'OVERV',BASE(IVII))
         WRITE(CNAM,'(1HV,2I3.3)') IGR,IGR
         CALL BIVASM(CNAM,1,IPTRK,IPSYS,IMPX,NBMIX,NEL,0,1,IBASE(IMAT),
     1   BASE(IVOL),BASE(IVII))
         CALL RLSARA(BASE(IVII))
   30    CONTINUE
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 40
      ELSE
         CALL XABORT('BIVACA: '//TEXT4//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 10
*----
*  SET THE STATE VECTOR FOR THE L_SYSTEM OBJECT
*----
   40 CALL XDISET(IBR,NSTATE,0)
      IBR(1)=NGRP
      IBR(2)=IGP(11)
      IBR(4)=1
      IF(NLF.GT.0) IBR(4)=11
      IF(IUNIT.EQ.1) IBR(5)=1
      IBR(7)=NBMIX
      NAN=MIN(ISCAT,NANI)
      IBR(8)=NAN+1
      CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,IBR)
*----
*  BIVAC SYSTEM MATRIX ASSEMBLY.
*----
      CALL SETARA(BASE,NBMIX*NGRP,IWORK)
      CALL SETARA(BASE,NBMIX,IIJJ)
      CALL SETARA(BASE,NBMIX,INJJ)
      CALL SETARA(BASE,NBMIX,IIPOS)
      CALL SETARA(BASE,NBMIX*NBFIS*NGRP,ICHI)
      CALL SETARA(BASE,NBMIX*NBFIS,IZUFIS)
      IF(NLF.EQ.0) THEN
*        DIFFUSION THEORY.
         CALL SETARA(BASE,NBMIX*3,ISGD)
         CALL BIVSYS(IPTRK,IPMACR,IPSYS,IMPX,NGRP,NEL,NBFIS,IBASE(IMAT),
     1   BASE(IVOL),NBMIX,BASE(ISGD),BASE(IWORK),BASE(IIJJ),BASE(INJJ),
     2   BASE(IIPOS),BASE(ICHI),BASE(IZUFIS))
         CALL RLSARA(BASE(ISGD))
      ELSE
*        SIMPLIFIED PN THEORY.
         CALL SETARA(BASE,NBMIX*2*(NAN+1),ISGD)
         CALL SETARA(BASE,NGRP,IIND)
         CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCAT)
         CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCATI)
         CALL BIVSPS(IPTRK,IPMACR,IPSYS,IMPX,NGRP,NEL,NLF,NAN,NBFIS,
     1   LDIFF,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),BASE(IWORK),
     2   BASE(IIJJ),BASE(INJJ),BASE(IIPOS),BASE(ICHI),BASE(IZUFIS),
     3   BASE(IIND),BASE(IRCAT),BASE(IRCATI))
         CALL RLSARA(BASE(IRCATI))
         CALL RLSARA(BASE(IRCAT))
         CALL RLSARA(BASE(IIND))
         CALL RLSARA(BASE(ISGD))
      ENDIF
      CALL RLSARA(BASE(IZUFIS))
      CALL RLSARA(BASE(ICHI))
      CALL RLSARA(BASE(IIPOS))
      CALL RLSARA(BASE(INJJ))
      CALL RLSARA(BASE(IIJJ))
      CALL RLSARA(BASE(IWORK))
*
      IF(IMPX.GE.3) CALL LCMLIB(IPSYS)
*----
*  RELEASE GENERAL TRACKING INFORMATION.
*----
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
