*DECK BIVASM
      SUBROUTINE BIVASM(HNAMT,ITY,IPTRK,IPSYS,IMPX,NBMIX,NEL,NLF,NDIM,
     1 MAT,VOL,SGD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Assembling of a single-group system matrix for BIVAC.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): A. Hebert
*
*Parameters: input/output
* HNAMT   name of the matrix.
* ITY     type of assembly: =0: leakage-removal matrix assembly;
*         =1: cross section matrix assembly.
* IPTRK   L_TRACK pointer to the BIVAC tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* IMPX    print parameter (equal to zero for no print).
* NBMIX   total number of material mixtures.
* NEL     total number of finite elements.
* NLF     number of Legendre orders for the flux (even number). Equal
*         to zero for diffusion theory.
* NDIM    second dimension of matrix SGD.
* MAT     mixture index assigned to each volume.
* VOL     volume of each element.
* SGD     nuclear properties per material mixture.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER HNAMT*(*)
      INTEGER ITY,IPTRK,IPSYS,IMPX,NBMIX,NEL,NLF,NDIM,MAT(NEL)
      REAL VOL(NEL),SGD(NBMIX,NDIM)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      LOGICAL CYLIND
      CHARACTER TEXT11*11
      INTEGER IBASE(1),ITP(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER BIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      ITYPE=ITP(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=ITP(8)
      ICOL=ITP(9)
      ISPLH=ITP(10)
      LL4=ITP(11)
      LX=ITP(12)
      LY=ITP(13)
      NVD=ITP(17)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY,IXX)
      CALL SETARA(BASE,LX*LY,IYY)
      CALL SETARA(BASE,LX*LY,IDD)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL SETARA(IBASE,LL4,IMU)
      IF(ITYPE.EQ.8) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      CALL LCMGET(IPTRK,'MU',IBASE(IMU))
*
      TEXT11=HNAMT
      IF(IMPX.GT.0) WRITE(6,'(/36H BIVASM: ASSEMBLY OF SYMMETRIC MATRI,
     1 3HX '',A11,38H'' IN COMPRESSED DIAGONAL STORAGE MODE.)') TEXT11
*----
*  ASSEMBLY OF THE SYSTEM MATRICES
*----
      CALL KDRCPU(TK1)
      IIMAX=IBASE(IMU+LL4-1)
      IF(NLF.NE.0) IIMAX=IIMAX*NLF/2
      CALL SETARA(BASE,IIMAX,ISYS)
      CALL XDRSET(BASE(ISYS),IIMAX,0.0)
*
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
*        MESH CORNER FINITE DIFFERENCES OR LAGRANGIAN FINITE ELEMENTS
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC*LC,IQ)
         CALL SETARA(BASE,LC*LC,IQS)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'RS',BASE(IRS))
         CALL LCMGET(IPTRK,'Q',BASE(IQ))
         CALL LCMGET(IPTRK,'QS',BASE(IQS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL BIVA01(ITY,MAXKN,SGD,CYLIND,NEL,LL4,NBMIX,IIMAX,
     1   BASE(IXX),BASE(IYY),BASE(IDD),MAT,IBASE(IKN),BASE(IQFR),
     2   VOL,IBASE(IMU),LC,BASE(IR),BASE(IRS),BASE(IQ),BASE(IQS),
     3   BASE(ISYS))
         CALL RLSARA(BASE(IQS))
         CALL RLSARA(BASE(IQ))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8).AND.(NLF.GT.0)) THEN
*        MIXED-DUAL FINITE ELEMENTS (SIMPLIFIED PN THEORY)
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'V',BASE(IV))
         CALL LCMSIX(IPTRK,' ',2)
         CALL PNDM2E(ITY,NEL,LL4,IELEM,ICOL,MAT,VOL,NBMIX,NLF,NVD,
     1   NDIM/2,SGD(1,1),SGD(1,1+NDIM/2),BASE(IXX),BASE(IYY),IBASE(IKN),
     2   BASE(IQFR),IBASE(IMU),IIMAX,LC,BASE(IR),BASE(IV),BASE(ISYS))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
*        MIXED-DUAL FINITE ELEMENTS (DIFFUSION THEORY).
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'V',BASE(IV))
         CALL LCMSIX(IPTRK,' ',2)
         CALL BIVA02(ITY,SGD,CYLIND,IELEM,ICOL,NEL,LL4,NBMIX,IIMAX,
     1   BASE(IXX),BASE(IYY),BASE(IDD),MAT,IBASE(IKN),BASE(IQFR),VOL,
     2   IBASE(IMU),LC,BASE(IR),BASE(IV),BASE(ISYS))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
*        MESH CORNER FINITE DIFFERENCES FOR HEXAGONS
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL SETARA(BASE,4,IR)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,36,IQH)
         CALL SETARA(BASE,9,IRT)
         CALL SETARA(BASE,9,IQT)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'RH',BASE(IRH))
         CALL LCMGET(IPTRK,'QH',BASE(IQH))
         CALL LCMGET(IPTRK,'RT',BASE(IRT))
         CALL LCMGET(IPTRK,'QT',BASE(IQT))
         CALL LCMSIX(IPTRK,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL BIVA03(ITY,MAXKN,MAXQF,SGD,NEL,LL4,ISPLH,NELEM,NBMIX,
     1   IIMAX,SIDE,MAT,IBASE(IKN),BASE(IQFR),VOL,IBASE(IMU),BASE(IR),
     2   BASE(IRH),BASE(IQH),BASE(IRT),BASE(IQT),BASE(ISYS))
         CALL RLSARA(BASE(IQT))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IQH))
         CALL RLSARA(BASE(IRH))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(ICOL.EQ.4)) THEN
*        MESH CENTERED FINITE DIFFERENCES FOR HEXAGONS
         CALL BIVA04(ITY,MAXKN,MAXQF,SGD,NEL,LL4,ISPLH,NBMIX,IIMAX,
     1   SIDE,MAT,IBASE(IKN),BASE(IQFR),VOL,IBASE(IMU),BASE(ISYS))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(NLF.GT.0)) THEN
*        THOMAS-RAVIART-SCHNEIDER METHOD FOR HEXAGONS (SIMPLIFIED PN
*        THEORY)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC**2,IR)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL SETARA(BASE,LC*(LC-1),IH)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'V',BASE(IV))
         CALL LCMGET(IPTRK,'H',BASE(IH))
         CALL LCMSIX(IPTRK,' ',2)
         CALL PNDH2E(ITY,IELEM,ICOL,ISPLH,LX,LL4,NBMIX,IIMAX,SIDE,MAT,
     1   NLF,NVD,NDIM/2,SGD(1,1),IBASE(IZGLOB),IBASE(IKN),BASE(IQFR),
     2   IBASE(IMU),LC,BASE(IR),BASE(IV),BASE(IH),BASE(ISYS))
         CALL RLSARA(BASE(IH))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
*        THOMAS-RAVIART-SCHNEIDER METHOD FOR HEXAGONS (DIFFUSION THEORY)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC**2,IR)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL SETARA(BASE,LC*(LC-1),IH)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'V',BASE(IV))
         CALL LCMGET(IPTRK,'H',BASE(IH))
         CALL LCMSIX(IPTRK,' ',2)
         CALL BIVA05(ITY,SGD,IELEM,ICOL,ISPLH,LX,LL4,NBMIX,IIMAX,
     1   SIDE,MAT,IBASE(IZGLOB),IBASE(IKN),BASE(IQFR),IBASE(IMU),
     2   LC,BASE(IR),BASE(IV),BASE(IH),BASE(ISYS))
         CALL RLSARA(BASE(IH))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IR))
      ENDIF
   40 CALL LCMPOF(IPSYS,TEXT11,IIMAX,2,ISYS)
      CALL KDRCPU(TK2)
      IF(IMPX.GT.0) WRITE(6,'(/35H BIVASM: CPU TIME FOR SYSTEM MATRIX,
     1 11H ASSEMBLY =,F9.2,3H S.)') TK2-TK1
*----
*  MATRIX FACTORIZATIONS
*----
      IF((ITY.EQ.0).OR.(TEXT11.EQ.'RM')) THEN
         CALL KDRCPU(TK1)
         CALL SETARA(BASE,IIMAX,IASS)
         CALL LCMGET(IPSYS,TEXT11,BASE(IASS))
         IF(NLF.EQ.0) THEN
            CALL ALLDLF(LL4,BASE(IASS),IBASE(IMU))
         ELSE
            IOF=1
            DO 50 IL=0,NLF-2,2
            CALL ALLDLF(LL4,BASE(IASS+IOF-1),IBASE(IMU))
   50       IOF=IOF+IBASE(IMU+LL4-1)
         ENDIF
         CALL LCMPOF(IPSYS,'I'//TEXT11,IIMAX,2,IASS)
         CALL KDRCPU(TK2)
         IF(IMPX.GT.1) WRITE(6,'(/34H BIVASM: CPU TIME FOR LDLT FACTORI,
     1   18HZATION OF MATRIX '',A11,2H''=,F9.2,3H S.)') TEXT11,TK2-TK1
      ENDIF
*----
*  RELEASE BIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IMU))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      RETURN
      END
