*DECK BIVTRK
      SUBROUTINE BIVTRK (MAXPTS,IPTRK,IPGEOM,IMPX,IELEM,ICOL,ISPLH,NLF,
     1 NVD,ISPN,ISCAT,MAT,VOL,IDL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover of the geometry and tracking for BIVAC.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXPTS  allocated storage for arrays of dimension NEL.
* IPTRK   L_TRACK pointer to the tracking information.
* IPGEOM  L_GEOM pointer to the geometry.
* IMPX    print flag.
* IELEM   degree of the Lagrangian finite elements:
*         <0: order -IELEM primal finite elements;
*         >0: order IELEM dual finite elements.
* ICOL    type of quadrature used to integrate the mass matrix:
*         =1: analytical integration;
*         =2: Gauss-Lobatto quadrature (collocation method);
*         =3: Gauss Legendre quadrature (superconvergent).
*         =4: mesh centered finite differences in hexagonal geometry.
*         IELEM=-1 and ICOL=2 : mesh corner finite differences;
*         IELEM=1 and ICOL=2 : mesh centered finite differences.
* ISPLH   type of hexagonal mesh-splitting:
*         =1: no mesh splitting (complete hexagons);
*         =K: 6*(K-1)*(K-1) triangles per hexagon.
* NLF     number of Legendre orders for the flux. Equal to zero for
*         diffusion theory.
* NVD     type of void boundary condition if NLF>0 and ICOL=3.
* ISPN    type of transport solution:
*         =0: complete PN method;
*         =1: simplified PN method.
* ISCAT   source anisotropy:
*         =1: isotropic sources in laboratory system;
*         =2: linearly anisotropic sources in laboratory system.
*
*Parameters: scratch
* MAT
* VOL
* IDL
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MAXPTS,IPTRK,IPGEOM,IMPX,IELEM,ICOL,ISPLH,NLF,NVD,ISPN,
     1 ISCAT,MAT(MAXPTS),IDL(MAXPTS)
      REAL VOL(MAXPTS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      LOGICAL ILK,CYLIND
      CHARACTER HSMG*131
      INTEGER ISTATE(NSTATE),IGP(NSTATE),NCODE(6),IBASE(1)
      REAL ZCODE(6)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
******************* BIVAC GEOMETRICAL STRUCTURE. ***********************
*                                                                      *
*   ITYPE       : =2  : CARTESIAN 1-D GEOMETRY;                        *
*                 =3  : TUBE 1-D GEOMETRY;                             *
*                 =4  : SPHERICAL 1-D GEOMETRY;                        *
*                 =5  : CARTESIAN 2-D GEOMETRY;                        *
*                 =6  : TUBE 2-D GEOMETRY;                             *
*                 =8  : HEXAGONAL 2-D GEOMETRY.                        *
*   IHEX        : TYPE OF HEXAGONAL SYMMETRY.                          *
*   IELEM       : .LT.0 : ORDER -IELEM PRIMAL FINITE ELEMENTS;         *
*                 .GT.0 : ORDER IELEM DUAL FINITE ELEMENTS.            *
*   ICOL        : TYPE OF QUADRATURE USED TO INTEGRATE THE MASS MATRIX.*
*                 =1  : ANALYTICAL INTEGRATION;                        *
*                 =2  : GAUSS-LOBATTO QUADRATURE (COLLOCATION METHOD); *
*                 =3  : GAUSS LEGENDRE QUADRATURE (SUPERCONVERGENT).   *
*                IELEM=-1 AND ICOL=2 : MESH CORNER FINITE DIFFERENCES. *
*                IELEM=1 AND ICOL=2 : MESH CENTERED FINITE DIFFERENCES.*
*   ISPLH       : TYPE OF HEXAGONAL MESH-SPLITTING.                    *
*                 =1 : NO MESH SPLITTING (COMPLETE HEXAGONS);          *
*                 =K : 6*(K-1)*(K-1) TRIANGLES PER HEXAGON.            *
*   SIDE        : SIDE OF THE HEXAGONS.                                *
*   LL4         : ORDER OF THE MATRICES PER GROUP IN BIVAC.            *
*   NCODE       : TYPES OF BOUNDARY CONDITIONS. DIMENSION=6            *
*   ZCODE       : ALBEDOS. DIMENSION=6                                 *
*   LX          : NUMBER OF ELEMENTS ALONG THE X AXIS.                 *
*   LY          : NUMBER OF ELEMENTS ALONG THE Y AXIS.                 *
*   BASE(IXX)   : X-DIRECTED MESH SPACINGS. DIMENSION=LX*LY            *
*   BASE(IYY)   : Y-DIRECTED MESH SPACINGS. DIMENSION=LX*LY            *
*   BASE(IDD)   : USED WITH CYLINDRICAL GEOMETRIES. DIMENSION=LX*LY    *
*  IBASE(IKN)   : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION LX*LY*ICOEF  *
*                 WHERE ICOEF IS THE NUMBER OF UNKNOWN PER ELEMENT.    *
*   BASE(IQFR)  : ELEMENT-ORDERED BOUNDARY CONDITIONS.                 *
*                 DIMENSION 4*LX*LY                                    *
*   BASE(IBFR)  : ELEMENT-ORDERED SURFACE FRACTIONS.                   *
*                 DIMENSION 4*LX*LY                                    *
*  IBASE(IMU)   : INDICES USED WITH COMPRESSED DIAGONAL STORAGE MODE   *
*                 MATRICES. DIMENSION MAXEV                            *
*                                                                      *
************************************************************************
*
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      ITYPE=ISTATE(1)
*
      IF((IMPX.GE.1).AND.(ITYPE.NE.8)) THEN
         WRITE (6,'(/39H BIVTRK: TYPE OF FINITE ELEMENT IELEM =,I3,
     1   8H  ICOL =,I3/)') IELEM,ICOL
      ELSE IF(IMPX.GE.1) THEN
         WRITE (6,'(/39H BIVTRK: TYPE OF FINITE ELEMENT IELEM =,I3,
     1   8H  ICOL =,I3,9H  ISPLH =,I3/)') IELEM,ICOL,ISPLH
      ENDIF
      IF(ISTATE(9).EQ.0) THEN
         IF((ITYPE.NE.1).AND.(ITYPE.NE.2).AND.(ITYPE.NE.3).AND.
     1      (ITYPE.NE.4).AND.(ITYPE.NE.5).AND.(ITYPE.NE.6).AND.
     2      (ITYPE.NE.8)) THEN
            CALL XABORT('BIVTRK: DISCRETIZATION NOT AVAILABLE.')
         ENDIF
         CALL SETARA(BASE,MAXPTS+1,IXXX)
         CALL SETARA(BASE,MAXPTS+1,IYYY)
         CALL SETARA(BASE,MAXPTS+1,IZZZ)
*
         CALL SETARA(BASE,MAXPTS,IISPLX)
         CALL SETARA(BASE,MAXPTS,IISPLY)
         CALL SETARA(BASE,MAXPTS,IISPLZ)
         CALL READ3D(MAXPTS,MAXPTS,MAXPTS,MAXPTS,IPGEOM,IHEX,IR,MAT,
     1   ILK,SIDE,BASE(IXXX),BASE(IYYY),BASE(IZZZ),IMPX,LX,LY,LZ,NEL,
     2   NCODE,ZCODE,BASE(IISPLX),BASE(IISPLY),BASE(IISPLZ))
         CALL RLSARA(BASE(IISPLZ))
         CALL RLSARA(BASE(IISPLY))
         CALL RLSARA(BASE(IISPLX))
      ELSE
         CALL XABORT('BIVTRK: DISCRETIZATION NOT AVAILABLE.')
      ENDIF
*
      IF(LX*LY*LZ.GT.MAXPTS) THEN
         WRITE (HSMG,'(39HBIVTRK: MAXPTS SHOULD BE INCREASED FROM,I7,
     1   3H TO,I7)') MAXPTS,LX*LY*LZ
         CALL XABORT(HSMG)
      ENDIF
*----
*  1-D AND 2-D CYLINDRICAL CASES.
*----
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.4).OR.(ITYPE.EQ.6)
      IF((ITYPE.EQ.2).OR.(ITYPE.EQ.3)) THEN
         NCODE(3)=2
         NCODE(4)=5
         ZCODE(3)=1.0
         ZCODE(4)=1.0
         BASE(IYYY)=0.0
         BASE(IYYY+1)=2.0
      ELSE IF(ITYPE.EQ.6) THEN
         LY=LZ
         DO 10 I=1,LZ+1
   10    BASE(IYYY+I-1)=BASE(IZZZ+I-1)
         NCODE(3)=NCODE(5)
         NCODE(4)=NCODE(6)
         ZCODE(3)=ZCODE(5)
         ZCODE(4)=ZCODE(6)
      ENDIF
*----
*  UNFOLD THE DOMAIN IN DIAGONAL SYMMETRY CASES.
*----
      IF((NCODE(2).EQ.3).AND.(NCODE(3).EQ.3)) THEN
         NCODE(3)=NCODE(1)
         NCODE(2)=NCODE(4)
         ZCODE(3)=ZCODE(1)
         ZCODE(2)=ZCODE(4)
         K=LX*(LX+1)/2
         DO 30 IY=LY,1,-1
         DO 20 IX=LX,IY+1,-1
   20    MAT((IY-1)*LX+IX)=MAT((IX-1)*LY+IY)
         DO 30 IX=IY,1,-1
         MAT((IY-1)*LX+IX)=MAT(K)
   30    K=K-1
         NEL=LX*LY
         IF(K.NE.0) THEN
            CALL XABORT('BIVTRK: UNABLE TO UNFOLD THE DOMAIN.')
         ENDIF
      ELSE IF((NCODE(1).EQ.3).AND.(NCODE(4).EQ.3)) THEN
         NCODE(1)=NCODE(3)
         NCODE(4)=NCODE(2)
         ZCODE(1)=ZCODE(3)
         ZCODE(4)=ZCODE(2)
         K=LX*(LX+1)/2
         DO 40 IY=LY,1,-1
         DO 40 IX=LX,IY,-1
         MAT((IY-1)*LX+IX)=MAT(K)
   40    K=K-1
         DO 50 IY=1,LY
         DO 50 IX=1,IY-1
   50    MAT((IY-1)*LX+IX)=MAT((IX-1)*LY+IY)
         NEL=LX*LY
         IF(K.NE.0) THEN
            CALL XABORT('BIVTRK: UNABLE TO UNFOLD THE DOMAIN.')
         ENDIF
      ENDIF
      IF(IMPX.GT.5) THEN
         WRITE(6,600) 'NCODE',(NCODE(I),I=1,4)
         WRITE(6,600) 'MAT',(MAT(I),I=1,LX*LY)
      ENDIF
*
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
         IEL=-IELEM
         MAXEV=(IEL*LX+1)*(IEL*LY+1)
         MAXKN=(IEL+1)*(IEL+1)*NEL
         MAXQF=4*NEL
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.3).AND.(NLF.NE.0)) THEN
*        PN METHOD / 1D CYLINDRICAL GEOMETRY.
         MAXEV=(2*LX+1)*(NLF/2)*(NLF/2+1)/2
         MAXKN=3*NEL*(NLF/2)*(NLF/2)
         MAXQF=2*NEL
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.4).AND.(NLF.NE.0)) THEN
*        PN METHOD / 1D SPHERICAL GEOMETRY.
         MAXEV=(2*LX+1)*(NLF/2)
         MAXKN=3*NEL*(NLF/2)
         MAXQF=2*NEL
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.5).AND.(NLF.NE.0).AND.
     1 (ISPN.EQ.0)) THEN
*        PN METHOD / 2D CARTESIAN GEOMETRY.
         MAXEV=0
         DO 60 IL=1,NLF-1,2
   60    MAXEV=MAXEV+(IL*LX+(IL+1)*(LX+1))*LY+(IL+1)*(LX+1)
         MAXKN=5*NEL*NLF*(NLF/2)
         MAXQF=4*NEL
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.5).AND.(NLF.NE.0).AND.
     1 (ISPN.EQ.1)) THEN
*        SPN METHOD / 2D CARTESIAN GEOMETRY.
         MAXEV=(LX+1)*LY*IELEM+LX*(LY+1)*IELEM+LX*LY*IELEM*IELEM
         MAXEV=MAXEV*NLF/2
         MAXKN=5*NEL
         MAXQF=4*NEL
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
         MAXEV=(LX+1)*LY*IELEM+LX*(LY+1)*IELEM+LX*LY*IELEM*IELEM
         MAXKN=5*NEL
         MAXQF=4*NEL
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
         IEL=-IELEM
         NEL=LX
         IF(ISPLH.EQ.1) THEN
            MAXEV=6*NEL
            MAXKN=7*NEL
         ELSE
            MAXEV=(1+ISPLH*(ISPLH-1)*3)*NEL
            MAXKN=(6*(ISPLH-1)**2)*NEL*4
         ENDIF
         MAXQF=MAXKN
      ELSE IF((ICOL.EQ.4).AND.(ITYPE.EQ.8)) THEN
         NEL=LX
         IF(ISPLH.EQ.1) THEN
            MAXEV=NEL
            MAXKN=7*NEL
         ELSE
            MAXEV=(6*(ISPLH-1)**2)*NEL
            MAXKN=(6*(ISPLH-1)**2)*NEL*4
         ENDIF
         MAXQF=MAXKN
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
         IF(IHEX.NE.9) THEN
            CALL SETARA(IBASE,MAXPTS,IDP)
            CALL SETARA(IBASE,LX,IMX)
            DO 70 I=1,LX
   70       IBASE(IMX+I-1)=MAT(I)
            LXOLD=LX
            CALL BIVALL(MAXPTS,IHEX,LXOLD,LX,IBASE(IDP))
            DO 80 KX=1,LX
            KEL=IBASE(IDP+KX-1)
   80       MAT(KX)=IBASE(IMX+KEL-1)
            CALL RLSARA(IBASE(IMX))
            CALL RLSARA(IBASE(IDP))
         ENDIF
         NBLOS=LX*ISPLH**2
         NBC=INT((SQRT(REAL((4*LX-1)/3))+1.)/2.)
         MAXEV=3*(2*NBLOS*IELEM+(2*NBC-1)*ISPLH)*IELEM+3*NBLOS*IELEM**2
         MAXKN=(LX*ISPLH**2)*(4+6*IELEM*(IELEM+1))
         MAXQF=(LX*ISPLH**2)*6
         NEL=LX
      ELSE
         CALL XABORT('BIVTRK: INVALID TYPE OF DISCRETIZATION.')
      ENDIF
      IF(CYLIND) THEN
         MAXDD=NEL
      ELSE
         MAXDD=1
      ENDIF
      IF((ICOL.EQ.4).AND.(ITYPE.EQ.8).AND.(IELEM.NE.1)) THEN
         CALL XABORT('BIVTRK: THIS HEXAGONAL MCFD DISCRETIZATIONS IS L'
     1   //'IMITED TO LINEAR ORDER.')
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8).AND.(IELEM.NE.-1)) THEN
         CALL XABORT('BIVTRK: THIS HEXAGONAL PRIM DISCRETIZATIONS IS L'
     1   //'IMITED TO LINEAR ORDER.')
      ENDIF
      IF(ICOL.LE.3) CALL BIVCOL(IPTRK,IMPX,ABS(IELEM),ICOL)
      CALL SETARA(BASE,NEL,IXX)
      CALL SETARA(BASE,NEL,IYY)
      CALL SETARA(BASE,MAXDD,IDD)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL SETARA(BASE,MAXQF,IBFR)
      CALL SETARA(IBASE,MAXEV,IMU)
      CALL XDISET(IBASE(IKN),MAXKN,0)
      CALL XDRSET(BASE(IQFR),MAXQF,0.0)
      CALL XDRSET(BASE(IBFR),MAXQF,0.0)
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
         IEL=-IELEM
         CALL SETARA(BASE,(IEL*LX+1)*(IEL*LY+1),IIP)
         CALL SETARA(BASE,(IEL*LX+1)*(IEL*LY+1),IWRK)
         CALL BIVPKN(MAXEV,IMPX,LX,LY,CYLIND,IEL,LL4,NCODE,ZCODE,
     1   MAT,VOL,BASE(IXXX),BASE(IYYY),BASE(IXX),BASE(IYY),BASE(IDD),
     2   IBASE(IKN),BASE(IQFR),BASE(IBFR),IBASE(IMU),BASE(IIP),
     3   BASE(IWRK))
         CALL RLSARA(BASE(IWRK))
         CALL RLSARA(BASE(IIP))
      ELSE IF(((ITYPE.EQ.2).OR.((ITYPE.EQ.5).AND.(ISPN.EQ.1))).AND.
     1 (IELEM.GT.0).AND.(NLF.NE.0)) THEN
*        MIXED-DUAL SPN APPROXIMATION IN 1D OR 2D CARTESIAN GEOMETRY.
         CALL SETARA(BASE,MAXEV,IIP)
         CALL BIVDKN(MAXEV,IMPX,LX,LY,CYLIND,IELEM,ICOL,LL4,NCODE,
     1   ZCODE,MAT,VOL,BASE(IXXX),BASE(IYYY),BASE(IXX),BASE(IYY),
     2   BASE(IDD),IBASE(IKN),BASE(IQFR),BASE(IBFR),IDL,IBASE(IMU),
     3   BASE(IIP))
         CALL RLSARA(BASE(IIP))
         NUN=LL4*NLF/2
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
         CALL SETARA(BASE,MAXEV,IIP)
         CALL BIVDKN(MAXEV,IMPX,LX,LY,CYLIND,IELEM,ICOL,LL4,NCODE,
     1   ZCODE,MAT,VOL,BASE(IXXX),BASE(IYYY),BASE(IXX),BASE(IYY),
     2   BASE(IDD),IBASE(IKN),BASE(IQFR),BASE(IBFR),IDL,IBASE(IMU),
     3   BASE(IIP))
         CALL RLSARA(BASE(IIP))
         NUN=LL4
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
*        HEXAGONAL GEOMETRY MESH CORNER FINITE DIFFERENCES.
         CALL SETARA(BASE,MAXEV,IGAR)
         CALL BIVPRH(MAXEV,MAXKN,IMPX,ISPLH,LX,IHEX,NCODE,ZCODE,MAT,
     1   SIDE,LL4,NELEM,VOL,IBASE(IKN),BASE(IQFR),BASE(IBFR),IBASE(IMU),
     2   BASE(IGAR))
         CALL RLSARA(BASE(IGAR))
         IF(ISPLH.EQ.1) THEN
            MAXKN=7*NELEM
            MAXQF=7*NELEM
         ELSE
            MAXKN=4*NELEM
            MAXQF=4*NELEM
         ENDIF
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(ICOL.EQ.4)) THEN
*        HEXAGONAL GEOMETRY MESH CENTERED FINITE DIFFERENCES.
         CALL BIVDFH(MAXEV,MAXKN,IMPX,ISPLH,LX,SIDE,LL4,NUN,IHEX,
     1   NCODE,ZCODE,MAT,VOL,IDL,IBASE(IKN),BASE(IQFR),BASE(IBFR),
     2   IBASE(IMU))
         IF(ISPLH.EQ.1) THEN
            MAXKN=7*LL4
            MAXQF=7*LL4
         ELSE
            MAXKN=4*LL4
            MAXQF=4*LL4
         ENDIF
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
*        HEXAGONAL GEOMETRY THOMAS-RAVIART-SCHNEIDER FINITE ELEMENTS.
         NBLOS=LX*ISPLH**2
         CALL SETARA(IBASE,3*NBLOS,IZGLOB)
         CALL SETARA(BASE,LX*ISPLH**2,IIJP)
         CALL SETARA(BASE,MAXEV,IIP)
         CALL BIVSFH(MAXEV,NBLOS,IMPX,ISPLH,IELEM,ICOL,LX,MAT,SIDE,
     1   NCODE,ZCODE,LL4,NUN,VOL,IDL,IBASE(IZGLOB),IBASE(IKN),
     2   BASE(IQFR),BASE(IBFR),IBASE(IMU),BASE(IIJP),BASE(IIP))
         CALL RLSARA(BASE(IIP))
         CALL RLSARA(BASE(IIJP))
         CALL LCMPUT(IPTRK,'ZGLOB',NBLOS,1,IBASE(IZGLOB))
         CALL RLSARA(IBASE(IZGLOB))
      ENDIF
      CALL RLSARA(BASE(IZZZ))
      CALL RLSARA(BASE(IYYY))
*----
*  APPEND THE PN FLUXES AT THE END OF UNKNOWN VECTOR.
*----
      IF(NLF.GE.2) THEN
         IF((ITYPE.EQ.2).OR.((ITYPE.EQ.5).AND.(ISPN.EQ.1))) THEN
            NUN=LL4+LL4*(NLF-2)/2
         ELSE IF((ITYPE.EQ.8).AND.(ISPN.EQ.1)) THEN
            NUN=NUN+NUN*(NLF-2)/2
         ELSE IF((ITYPE.NE.2).AND.(ITYPE.NE.5).AND.(ITYPE.NE.8)) THEN
            CALL XABORT('BIVTRK: GEOMETRY NOT SUPPORTED WITH PN.')
         ENDIF
      ENDIF
*----
*  APPEND THE AVERAGED FLUXES AT THE END OF UNKNOWN VECTOR.
*----
      IF(IELEM.LT.0) THEN
         NUN=LL4
         DO 190 I=1,NEL
         IF(MAT(I).EQ.0) THEN
            IDL(I)=0
         ELSE
            NUN=NUN+1
            IDL(I)=NUN
         ENDIF
  190    CONTINUE
      ENDIF
*----
*  RESERVE A COMPONENT TO STORE THE SURFACE-AVERAGED FLUX.
*----
      IF(NLF.EQ.0) NUN=NUN+1
      IF(IMPX.GT.0) WRITE (6,'(/34H BIVTRK: ORDER OF LINEAR SYSTEMS =,
     1 I7/9X,37HNUMBER OF UNKNOWNS PER ENERGY GROUP =,I7)') LL4,NUN
*
      IF(IMPX.GT.5) THEN
         I1=1
         DO 200 I=1,(NEL-1)/8+1
         I2=I1+7
         IF(I2.GT.NEL) I2=NEL
         WRITE (6,620) (J,J=I1,I2)
         WRITE (6,630) (MAT(J),J=I1,I2)
         WRITE (6,640) (IDL(J),J=I1,I2)
         WRITE (6,650) (VOL(J),J=I1,I2)
  200    I1=I1+8
      ENDIF
*----
*  SAVE GENERAL AND BIVAC-SPECIFIC TRACKING INFORMATION.
*----
      CALL XDISET(IGP,NSTATE,0)
      IGP(1)=NEL
      IGP(2)=NUN
      IF(ILK) THEN
         IGP(3)=0
      ELSE
         IGP(3)=1
      ENDIF
      IGP(4)=ISTATE(7)
      IGP(5)=1
      IGP(6)=ITYPE
      IGP(7)=IHEX
      IGP(8)=IELEM
      IGP(9)=ICOL
      IGP(10)=ISPLH
      IGP(11)=LL4
      IGP(12)=LX
      IGP(13)=LY
      IGP(14)=NLF
      IGP(15)=ISPN
      IGP(16)=ISCAT
      IGP(17)=NVD
      CALL LCMPUT(IPTRK,'STATE-VECTOR',NSTATE,1,IGP)
      CALL LCMPUT(IPTRK,'MATCOD',NEL,1,MAT)
      CALL LCMPUT(IPTRK,'VOLUME',NEL,2,VOL)
      CALL LCMPUT(IPTRK,'KEYFLX',NEL,1,IDL)
      CALL LCMPUT(IPTRK,'NCODE',6,1,NCODE)
      CALL LCMPUT(IPTRK,'ZCODE',6,2,ZCODE)
      CALL LCMPUT(IPTRK,'BC-REFL+TRAN',1,1,NUN)
      IF(ITYPE.EQ.4) CALL LCMPUT(IPTRK,'XXX',LX+1,2,BASE(IXXX))
      CALL RLSARA(BASE(IXXX))
      IF(ITYPE.EQ.8) THEN
         CALL LCMPUT(IPTRK,'SIDE',1,2,SIDE)
         CALL RLSARA(BASE(IXX))
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IDD))
      ELSE
         CALL LCMPOF(IPTRK,'XX',LX*LY,2,IXX)
         CALL LCMPOF(IPTRK,'YY',LX*LY,2,IYY)
         IF(.NOT.CYLIND) BASE(IDD)=0.0
         CALL LCMPOF(IPTRK,'DD',MAXDD,2,IDD)
      ENDIF
      CALL LCMPUT(IPTRK,'KN',MAXKN,1,IBASE(IKN))
      CALL RLSARA(IBASE(IKN))
      CALL LCMPUT(IPTRK,'QFR',MAXQF,2,BASE(IQFR))
      CALL RLSARA(BASE(IQFR))
      CALL LCMPUT(IPTRK,'BFR',MAXQF,2,BASE(IBFR))
      CALL RLSARA(BASE(IBFR))
      CALL LCMPOF(IPTRK,'MU',LL4,1,IMU)
      RETURN
*
  600 FORMAT(/26H BIVTRK: VALUES OF VECTOR ,A6,4H ARE/(1X,1P,20I6))
  620 FORMAT (///11H REGION    ,8(I8,6X,1HI))
  630 FORMAT (   11H MIXTURE   ,8(I8,6X,1HI))
  640 FORMAT (   11H POINTER   ,8(I8,6X,1HI))
  650 FORMAT (   11H VOLUME    ,8(1P,E13.6,2H I))
      END
