*DECK DELDRV
      SUBROUTINE DELDRV (IPTRK,IPSYS0,IPSYSP,IPFLU0,IPGPT,REC,NEL,NUN,
     1 NGRP,MAT,VOL,IDL,TITR,EVECT,ADECT,SUNKNO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for the calculation of direct or adjoint sources for a fixed
* source eigenvalue problem.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS0  L_SYSTEM pointer to unperturbed system matrices.
* IPSYSP  L_SYSTEM pointer to delta system matrices.
* IPFLU0  L_FLUX pointer to the unperturbed solution.
* IPGPT   L_GPT pointer to the GPT fixed source.
* REC     recover flag (=.TRUE.: recover the existing solution as
*         initial GPT estimate; =.FALSE.: use a uniform GPT initial
*         estimate).
* NEL     total number of finite elements.
* NUN     total number of unknowns per energy group.
* NGRP    number of energy groups.
* MAT     index-number of the mixture type associated to each volume.
* VOL     volumes.
* IDL     position of the average flux component associated with each
*         volume.
* TITR    title.
*
*Parameters: scratch
* EVECT
* ADECT
* SUNKNO
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IPSYS0,IPSYSP,IPFLU0,IPGPT,NEL,NUN,NGRP,MAT(NEL),
     1 IDL(NEL)
      LOGICAL REC
      REAL VOL(NEL),EVECT(NUN,NGRP),ADECT(NUN,NGRP),SUNKNO(NUN,NGRP)
      CHARACTER TITR*72
*----
*  LOCAL VARIABLES
*----
      LOGICAL ADJ
      DOUBLE PRECISION DFLOTT
      CHARACTER TEXT4*4
      COMMON BASE(1)
*----
*  READ THE INPUT DATA.
*----
*     DEFAULT OPTIONS.
      IMPX=1
      ADJ=.FALSE.
*
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 20
      IF(INDIC.NE.3) CALL XABORT('DELDRV: CHARACTER DATA EXPECTED.')
*
      IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('DELDRV: INTEGER DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'ADJ') THEN
         ADJ=.TRUE.
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 20
      ELSE
         CALL XABORT('DELDRV: ; EXPECTED.')
      ENDIF
      GO TO 10
*----
*  RECOVER UNPERTURBED K-EFFECTIVE AND FLUXES.
*----
   20 CALL LCMGET(IPFLU0,'K-EFFECTIVE',FKEFF)
      JPFLU1=LCMGID(IPFLU0,'FLUX')
      JPFLU2=LCMGID(IPFLU0,'AFLUX')
      DO 30 IGR=1,NGRP
      CALL LCMGDL(JPFLU1,IGR,EVECT(1,IGR))
      CALL LCMGDL(JPFLU2,IGR,ADECT(1,IGR))
   30 CONTINUE
*----
*  COMPUTE THE DIRECT OR ADJOINT FIXED SOURCES.
*----
      CALL SETARA(BASE,NUN,IAGAR)
      CALL SETARA(BASE,NUN,IWORK)
      CALL DELPER(IPTRK,IPSYS0,IPSYSP,ADJ,NUN,NGRP,FKEFF,IMPX,EVECT,
     1 ADECT,DELKEF,SUNKNO,BASE(IAGAR),BASE(IWORK))
      CALL RLSARA(BASE(IWORK))
      CALL RLSARA(BASE(IAGAR))
*----
*  SAVE THE FIXED SOURCES.
*----
      IF(ADJ) THEN
         JPGPT=LCMLID(IPGPT,'ASOUR',1)
      ELSE
         JPGPT=LCMLID(IPGPT,'DSOUR',1)
      ENDIF
      KPGPT=LCMLIL(JPGPT,1,NGRP)
      DO 40 IGR=1,NGRP
      CALL LCMPDL(KPGPT,IGR,NUN,2,SUNKNO(1,IGR))
   40 CONTINUE
*
      RETURN
      END
