*DECK FLD
      SUBROUTINE FLD(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* multigroup flux solution operator for BIVAC and TRIVAC.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_FLUX);
*         HENTRY(2): read-only type(L_SYSTEM);
*         HENTRY(3): read-only type(L_TRACK);
*         HENTRY(4): optional read-only type(L_MACROLIB).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  	
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER TEXT12*12,TITLE*72,CMODUL*12,HSIGN*12
      LOGICAL REC
      INTEGER IGP(NSTATE),ITR(NSTATE),KCHAR(18),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.1) CALL XABORT('FLD: TWO PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('FLD: LCM '
     1 //'OBJECT EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('FLD: ENTR'
     1 //'Y IN CREATE OR MODIFICATION MODE EXPECTED.')
      IF((JENTRY(2).NE.2).OR.((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)))
     1 CALL XABORT('FLD: LCM OBJECT IN READ-ONLY MODE EXPECTED AT RHS.')
      IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_FLUX') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('FLD: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_FLUX EXPECTED.')
         ENDIF
      ENDIF
      CALL LCMGET(KENTRY(2),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_SYSTEM') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('FLD: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_SYSTEM EXPECTED.')
      ENDIF
      HSIGN='L_FLUX'
      READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(KENTRY(1),'SIGNATURE',3,3,KCHAR)
      TEXT12=HENTRY(2)
      READ(TEXT12,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(KENTRY(1),'LINK.SYSTEM',3,3,KCHAR)
      IPFLUX=KENTRY(1)
      IPSYS=KENTRY(2)
      REC=(JENTRY(1).EQ.1)
*----
*  RECOVER IPTRK POINTER AND VALIDATE IT
*----
      IF(NENTRY.EQ.4) THEN
         CALL LCMGET(KENTRY(4),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_MACROLIB') THEN
            TEXT12=HENTRY(4)
            CALL XABORT('FLD: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_MACROLIB EXPECTED.')
         ENDIF
         TEXT12=HENTRY(4)
         READ(TEXT12,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(KENTRY(1),'LINK.MACRO',3,3,KCHAR)
      ELSE
         CALL LCMGET(IPSYS,'LINK.MACRO',KCHAR)
         CALL LCMPUT(KENTRY(1),'LINK.MACRO',3,3,KCHAR)
      ENDIF
      CALL LCMGET(IPSYS,'LINK.TRACK',KCHAR)
      CALL LCMPUT(KENTRY(1),'LINK.TRACK',3,3,KCHAR)
      WRITE(TEXT12,'(3A4)') (KCHAR(I1),I1=1,3)
      DO 10 I=1,NENTRY
      IF(HENTRY(I).EQ.TEXT12) THEN
         IPTRK=KENTRY(I)
         CALL LCMGET(IPTRK,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_TRACK') THEN
            TEXT12=HENTRY(I)
            CALL XABORT('FLD: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_TRACK EXPECTED.')
         ENDIF
         CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
         WRITE(CMODUL,'(3A4)') (KCHAR(I1),I1=1,3)
         IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2)) CALL XABORT('FLD: L'
     1   //'CM OBJECT EXPECTED TO CONTAIN THE TRACKING.')
         GO TO 20
      ENDIF
   10 CONTINUE
      CALL XABORT('FLD: UNABLE TO FIND A POINTER TO TRACKING.')
*----
*  RECOVER GENERAL TRACKING INFORMATION
*----
   20 CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      NEL=IGP(1)
      NUN=IGP(2)
      IF(CMODUL.EQ.'BIVAC') THEN
         NLF=IGP(14)
      ELSE IF(CMODUL.EQ.'TRIVAC') THEN
         NLF=IGP(30)
      ENDIF
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(BASE,NEL,IVOL)
      CALL SETARA(IBASE,NEL,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
*----
*  RECOVER GENERAL L_SYSTEM INFORMATION
*----
      CALL LCMGET(IPSYS,'STATE-VECTOR',ITR)
      NGRP=ITR(1)
      LL4=ITR(2)
      ITY=ITR(4)
      NBMIX=ITR(7)
      IF((ITY.EQ.11).OR.(ITY.EQ.13)) LL4=LL4*NLF/2
*----
*  COMPUTE THE FLUX
*----
      CALL SETARA(BASE,NUN*NGRP,IEVECT)
      CALL FLDDRV(CMODUL,IPTRK,IPSYS,REC,NEL,LL4,ITY,NUN,NBMIX,
     1 IBASE(IMAT),BASE(IVOL),IBASE(IIDL),NGRP,TITLE,IPFLUX,
     2 BASE(IEVECT))
      CALL RLSARA(BASE(IEVECT))
*----
*  RELEASE GENERAL TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
