*DECK FLDBSS
      SUBROUTINE FLDBSS(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F1,SGD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform a one-group SPN flux iteration in Cartesian or hexagonal 2D
* geometry in BIVAC.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* NBMIX   total number of material mixtures in the macrolib.
* NAN     number of Legendre orders in the cross sections.
* F1      source term of the linear system.
*
*Parameters: output
* F1      approached solution of the linear system.
*
*Parameters: scratch
* SGD
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,NBMIX,NAN
      REAL F1(LL4),SGD(NBMIX,NAN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,TEXT12*12
      INTEGER IBASE(1),IPAR(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER PN SPECIFIC PARAMETERS.
*----
      NAMT=NAMP
      READ(NAMT,'(1X,2I3)') IGR,JGR
      IF(IGR.NE.JGR) CALL XABORT('FLDBSS: INVALIB GROUP INDICES.')
      IF(NAMT(1:1).NE.'A') CALL XABORT('FLDBSS: ''A'' MATRIX EXPECTED.')
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      NREG=IPAR(1)
      NUN=IPAR(2)
      ITYPE=IPAR(6)
      IELEM=IPAR(8)
      ICOL=IPAR(9)
      ISPLH=IPAR(10)
      L4=IPAR(11)
      LX=IPAR(12)
      NLF=IPAR(14)
      ISPN=IPAR(15)
      NVD=IPAR(17)
      IF(ITYPE.EQ.8) THEN
         IF(NUN.GT.(LX+L4)*NLF/2) CALL XABORT('FLDBSS: INVALID NUN OR '
     1   //'L4.')
      ELSE
        IF(NUN.NE.L4*NLF/2) CALL XABORT('FLDBSS: INVALID NUN OR L4.')
      ENDIF
      IF(L4*NLF/2.NE.LL4) CALL XABORT('FLDBSS: INVALID L4 OR LL4.')
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(IBASE,NREG,IKEY)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKEY))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL SETARA(IBASE,L4,IMU)
      CALL LCMGET(IPTRK,'MU',IBASE(IMU))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL LCMLEN(IPTRK,'BFR',MAXBF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
*----
*  RECOVER THE CROSS SECTIONS.
*----
      DO 10 IL=1,NAN
      WRITE(TEXT12,'(4HSCAI,I2.2,A6)') IL-1,NAMT(2:7)
      CALL LCMGET(IPSYS,TEXT12,SGD(1,IL))
   10 CONTINUE
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  SOLVE THE LINEAR SYSTEM.
*----
      IIMAX=IBASE(IMU+L4-1)*NLF/2
      CALL SETARA(BASE,IIMAX,ISYS)
      CALL SETARA(BASE,NUN,ISOUR)
      CALL LCMGET(IPSYS,'I'//NAMT,BASE(ISYS))
      DO 30 IUN=1,NUN
      BASE(ISOUR+IUN-1)=F1(IUN)
   30 F1(IUN)=0.0
      IF((ITYPE.EQ.2).OR.((ITYPE.EQ.5).AND.(ISPN.EQ.1))) THEN
         CALL SETARA(BASE,NREG,IXX)
         CALL SETARA(BASE,NREG,IYY)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL PNFL2E(NREG,IELEM,ICOL,BASE(IXX),BASE(IYY),IBASE(IMAT),
     1   BASE(IVOL),NBMIX,NLF,NVD,NAN,SGD,L4,IBASE(IKN),BASE(IQFR),
     2   IBASE(IMU),IIMAX,LC,BASE(IR),BASE(IV),BASE(ISYS),BASE(ISOUR),
     3   F1)
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ELSE IF(ITYPE.EQ.8) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL PNFH2E(IELEM,ICOL,ISPLH,LX,SIDE,IBASE(IMAT),NBMIX,
     1   NLF,NVD,L4,IBASE(IZGLOB),IBASE(IKN),BASE(IQFR),IBASE(IMU),
     2   IIMAX,LC,BASE(IV),BASE(ISYS),BASE(ISOUR),F1)
      ENDIF
      CALL RLSARA(BASE(ISOUR))
      CALL RLSARA(BASE(ISYS))
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(IBASE(IMU))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IKEY))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
