*DECK FLDSPN
      SUBROUTINE FLDSPN(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F1,NADI,SIGT,
     1 SIGTI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform NADI inner iterations with the ADI preconditionning.
* Special version for Thomas-Raviart basis (simplified PN).
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the ADI-splitted matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* NBMIX   total number of material mixtures in the macrolib.
* NAN     number of Legendre orders in the cross sections.
* F1      source term of the linear system.
* NADI    number of inner ADI iterations.
*
*Parameters: output
* F1      approached solution of the linear system.
*
*Parameters: scratch
* SIGT
* SIGTI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,NBMIX,NAN,NADI
      REAL F1(LL4),SIGT(NBMIX,NAN),SIGTI(NBMIX,NAN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,TEXT12*12
      INTEGER IBASE(1),ITP(NSTATE)
      LOGICAL LMUX,DIAG,CHEX
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER PN SPECIFIC PARAMETERS.
*----
      NAMT=NAMP
      IF(NAMT(1:1).NE.'A') CALL XABORT('FLDSPN: ''A'' MATRIX EXPECTED.')
      READ(NAMT,'(1X,2I3)') IGR,JGR
      IF(IGR.NE.JGR) CALL XABORT('FLDSPN: INVALIB GROUP INDICES.')
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      NREG=ITP(1)
      NUN=ITP(2)
      ITYPE=ITP(6)
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IELEM=ITP(9)
      ICOL=ITP(10)
      L4=ITP(11)
      LX=ITP(14)
      LZ=ITP(16)
      ISEG=ITP(17)
      LTSW=ITP(19)
      LL4F=ITP(25)
      LL4W=ITP(26)
      LL4X=ITP(27)
      LL4Y=ITP(28)
      LL4Z=ITP(29)
      NLF=ITP(30)
      NVD=ITP(34)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IF(CHEX) THEN
         IOFW=LL4F
         IOFX=LL4F+LL4W
         IOFY=LL4F+LL4W+LL4X
         IOFZ=LL4F+LL4W+LL4X+LL4Y
         IF(NUN.GT.(LX*LZ+L4)*NLF/2) CALL XABORT('FLDSPN: INVALID NUN '
     1   //'OR L4.')
      ELSE
         IOFX=LL4F
         IOFY=LL4F+LL4X
         IOFZ=LL4F+LL4X+LL4Y
         IF(NUN.NE.L4*NLF/2) CALL XABORT('FLDSPN: INVALID NUN OR L4.')
      ENDIF
      IF(L4*NLF/2.NE.LL4) CALL XABORT('FLDSPN: INVALID L4 OR LL4.')
*----
*  RECOVER TRACKING-RELATED INFORMATIONS
*----
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
         CALL SETARA(BASE,NREG,IXX)
         CALL SETARA(BASE,NREG,IYY)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
      ENDIF
      CALL SETARA(BASE,NREG,IZZ)
      CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
*----
*  RECOVER THE CROSS SECTIONS.
*----
      DO 10 IL=1,NAN
      WRITE(TEXT12,'(4HSCAR,I2.2,A6)') IL-1,NAMT(2:7)
      CALL LCMGET(IPSYS,TEXT12,SIGT(1,IL))
      WRITE(TEXT12,'(4HSCAI,I2.2,A6)') IL-1,NAMT(2:7)
      CALL LCMGET(IPSYS,TEXT12,SIGTI(1,IL))
   10 CONTINUE
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  RECOVER INFORMATIONS RELATED TO SYSTEM MATRICES
*----
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=(LL4Y.GT.0).AND.(.NOT.LMUX)
      CALL LCMIOF(IPSYS,'TF'//NAMT,ITF)
*
      IF(LL4W.GT.0) THEN
         ISPLH=ITP(13)
         LX=ITP(14)
         LZ=ITP(16)
         NBLOS=LX*LZ*ISPLH**2
         CALL LCMIOF(IPTRK,'CTRAN',ICTRAN)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPTRK,'FRZ',IFRZ)
*
         CALL LCMIOF(IPTRK,'IPBBW',IIPBW)
         CALL LCMLEN(IPSYS,'WB',LENWB,ITYL)
         IF(LENWB.EQ.0) THEN
           CALL LCMIOF(IPTRK,'WB',IBW)
         ELSE
           CALL LCMIOF(IPSYS,'WB',IBW)
         ENDIF
         CALL LCMIOF(IPSYS,'WI'//NAMT,IC11W)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUW',IMUW)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VW',LL4VW)
            CALL LCMIOF(IPTRK,'MUVW',IMUW)
            CALL LCMIOF(IPTRK,'IPVW',IIPVW)
            CALL LCMLEN(IPTRK,'NBLW',LONW,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLW',INBLW)
            CALL LCMIOF(IPTRK,'LBLW',ILBLW)
         ENDIF
      ENDIF
      CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
      CALL LCMLEN(IPSYS,'XB',LENXB,ITYL)
      IF(LENXB.EQ.0) THEN
        CALL LCMIOF(IPTRK,'XB',IBX)
      ELSE
        CALL LCMIOF(IPSYS,'XB',IBX)
      ENDIF
      IF(DIAG) THEN
         CALL LCMIOF(IPSYS,'YI'//NAMT,IC11X)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMUX)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4VX)
            CALL LCMIOF(IPTRK,'MUVY',IMUX)
            CALL LCMIOF(IPTRK,'IPVY',IIPVX)
            CALL LCMLEN(IPTRK,'NBLY',LONX,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBLX)
            CALL LCMIOF(IPTRK,'LBLY',ILBLX)
         ENDIF
      ELSE
         CALL LCMIOF(IPSYS,'XI'//NAMT,IC11X)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUX',IMUX)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VX',LL4VX)
            CALL LCMIOF(IPTRK,'MUVX',IMUX)
            CALL LCMIOF(IPTRK,'IPVX',IIPVX)
            CALL LCMLEN(IPTRK,'NBLX',LONX,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBLX)
            CALL LCMIOF(IPTRK,'LBLX',ILBLX)
         ENDIF
      ENDIF
      IF(LL4Y.GT.0) THEN
         CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
         CALL LCMLEN(IPSYS,'YB',LENYB,ITYL)
         IF(LENYB.EQ.0) THEN
            CALL LCMIOF(IPTRK,'YB',IBY)
         ELSE
            CALL LCMIOF(IPSYS,'YB',IBY)
         ENDIF
         CALL LCMIOF(IPSYS,'YI'//NAMT,IC11Y)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMUY)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4VY)
            CALL LCMIOF(IPTRK,'MUVY',IMUY)
            CALL LCMIOF(IPTRK,'IPVY',IIPVY)
            CALL LCMLEN(IPTRK,'NBLY',LONY,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBLY)
            CALL LCMIOF(IPTRK,'LBLY',ILBLY)
         ENDIF
      ENDIF
      IF(LL4Z.GT.0) THEN
         CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
         CALL LCMLEN(IPSYS,'ZB',LENZB,ITYL)
         IF(LENZB.EQ.0) THEN
            CALL LCMIOF(IPTRK,'ZB',IBZ)
         ELSE
            CALL LCMIOF(IPSYS,'ZB',IBZ)
         ENDIF
         CALL LCMIOF(IPSYS,'ZI'//NAMT,IC11Z)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUZ',IMUZ)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VZ',LL4VZ)
            CALL LCMIOF(IPTRK,'MUVZ',IMUZ)
            CALL LCMIOF(IPTRK,'IPVZ',IIPVZ)
            CALL LCMLEN(IPTRK,'NBLZ',LONZ,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLZ',INBLZ)
            CALL LCMIOF(IPTRK,'LBLZ',ILBLZ)
         ENDIF
      ENDIF
      IF(CHEX) CALL SETARA(BASE,NBLOS,IDIFF)
*----
*  PERFORM ADI ITERATIONS AND LEGENDRE ORDER SWAPPING
*----
      CALL SETARA(BASE,LL4F,IFL)
      IF(LL4W.GT.0) THEN
         CALL SETARA(BASE,LL4W,IFW)
      ENDIF
      CALL SETARA(BASE,LL4X,IFX)
      IF(LL4Y.GT.0) THEN
         CALL SETARA(BASE,LL4Y,IFY)
      ENDIF
      IF(LL4Z.GT.0) THEN
         CALL SETARA(BASE,LL4Z,IFZ)
      ENDIF
      CALL SETARA(BASE,NUN,ISOUR)
      IF(ISEG.GT.0) CALL SETARA(BASE,ISEG,IT)
      DO 20 IUN=1,NUN
      BASE(ISOUR+IUN-1)=F1(IUN)
   20 F1(IUN)=0.0
      DO 610 IADI=1,NADI
      DO 610 IL=0,NLF-1
      JOFF=(IL/2)*L4
      IF(MOD(IL,2).EQ.0) THEN
         DO 21 I0=1,LL4X
   21    BASE(IFX+I0-1)=F1(JOFF+IOFX+I0)
         DO 22 I0=1,LL4Y
   22    BASE(IFY+I0-1)=F1(JOFF+IOFY+I0)
         DO 23 I0=1,LL4Z
   23    BASE(IFZ+I0-1)=F1(JOFF+IOFZ+I0)
      ENDIF
      IF(CHEX) THEN
         CALL PNFH3E(IL,NBMIX,NBLOS,IELEM,ICOL,ISPLH,IDIM,NLF,NVD,
     1   NAN,LX,LZ,L4,LL4F,LL4W,LL4X,LL4Y,IBASE(IMAT),SIGTI,SIDE,
     2   BASE(IZZ),BASE(IFRZ),BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),
     3   LC,BASE(IR),BASE(IV),BASE(ISOUR),F1)
      ELSE
         CALL PNFL3E(IL,NREG,IELEM,ICOL,IDIM,BASE(IXX),BASE(IYY),
     1   BASE(IZZ),IBASE(IMAT),BASE(IVOL),NBMIX,NLF,NVD,NAN,SIGTI,L4,
     2   IBASE(IKN),BASE(IQFR),LC,BASE(IR),BASE(IV),BASE(ISOUR),F1)
      ENDIF
      IF(MOD(IL,2).EQ.1) THEN
*----
*  RECOVER CROSS SECTIONS FOR THE PIOLAT TERMS.
*----
         IF(CHEX) THEN
            FACT=REAL(2*IL+1)
            DO 25 I=1,NBLOS
            BASE(IDIFF+I-1)=0.0
            IND=IBASE(IZGLOB+I-1)
            IF(IND.GT.0) THEN
               IBM=IBASE(IMAT+IND-1)
               GARS=SIGT(IBM,MIN(IL+1,NAN))
               DZ=BASE(IZZ+IND-1)*BASE(IFRZ+IND-1)
               BASE(IDIFF+I-1)=FACT*DZ*GARS
            ENDIF
   25       CONTINUE
         ENDIF
*----
*  W DIRECTION
*----
         IF(LL4W.GT.0) THEN
            DO 30 I0=1,LL4F
   30       BASE(IFL+I0-1)=F1(JOFF+I0)
            DO 50 I0=1,LL4X
            DO 40 J0=1,2*IELEM
            JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 50
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFX+I0-1)
   40       CONTINUE
   50       CONTINUE
            DO 70 I0=1,LL4Y
            DO 60 J0=1,2*IELEM
            JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 70
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFY+I0-1)
   60       CONTINUE
   70       CONTINUE
            DO 90 I0=1,LL4Z
            DO 80 J0=1,2*IELEM
            JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 90
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFZ+I0-1)
   80       CONTINUE
   90       CONTINUE
            DO 110 I0=1,LL4W
            GGW=-F1(JOFF+IOFW+I0)
            DO 100 J0=1,2*IELEM
            JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 110
            GGW=GGW+BASE(IBW+(I0-1)*2*IELEM+J0-1)*REAL(IL)*
     1      BASE(IFL+JJ-1)/BASE((IL/2)*LL4F+ITF+JJ-1)
  100       CONTINUE
  110       BASE(IFW+I0-1)=GGW
*
*           PIOLAT TRANSFORM TERM.
            CALL FLDPWY(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      IBASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFY),BASE(IFW))
            CALL FLDPWX(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      IBASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFX),BASE(IFW))
            MUMAX=IBASE(IMUW+LL4W-1)
            IF(ISEG.EQ.0) THEN
*              SCALAR SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
               CALL ALLDLS(LL4W,IBASE(IMUW),BASE((IL/2)*MUMAX+IC11W),
     1         BASE(IFW))
            ELSE IF(ISEG.GT.0) THEN
*              SUPERVECTORIAL SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
               CALL SETARA(BASE,LL4VW,IGAR)
               CALL XDRSET(BASE(IGAR),LL4VW,0.0)
               DO 120 I=1,LL4W
  120          BASE(IGAR+IBASE(IIPVW+I-1)-1)=BASE(IFW+I-1)
               CALL ALVDLS(LTSW,IBASE(IMUW),BASE((IL/2)*MUMAX+IC11W),
     1         BASE(IGAR),ISEG,LONW,IBASE(INBLW),IBASE(ILBLW),BASE(IT))
               DO 130 I=1,LL4W
  130          BASE(IFW+I-1)=BASE(IGAR+IBASE(IIPVW+I-1)-1)
               CALL RLSARA(BASE(IGAR))
            ENDIF
         ENDIF
*----
*  X DIRECTION
*----
         DO 140 I0=1,LL4F
  140    BASE(IFL+I0-1)=F1(JOFF+I0)
         DO 160 I0=1,LL4W
         DO 150 J0=1,2*IELEM
         JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 160
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBW+(I0-1)*2*IELEM+J0-1)*
     1   REAL(IL)*BASE(IFW+I0-1)
  150    CONTINUE
  160    CONTINUE
         DO 180 I0=1,LL4Y
         DO 170 J0=1,2*IELEM
         JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 180
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)*
     1   REAL(IL)*BASE(IFY+I0-1)
  170    CONTINUE
  180    CONTINUE
         DO 200 I0=1,LL4Z
         DO 190 J0=1,2*IELEM
         JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 200
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)*
     1   REAL(IL)*BASE(IFZ+I0-1)
  190    CONTINUE
  200    CONTINUE
         DO 220 I0=1,LL4X
         GGX=-F1(JOFF+IOFX+I0)
         DO 210 J0=1,2*IELEM
         JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 220
         GGX=GGX+BASE(IBX+(I0-1)*2*IELEM+J0-1)*REAL(IL)*BASE(IFL+JJ-1)/
     1   BASE((IL/2)*LL4F+ITF+JJ-1)
  210    CONTINUE
  220    BASE(IFX+I0-1)=GGX
         IF(LL4W.GT.0) THEN
*           PIOLAT TRANSFORM TERM.
            CALL FLDPXW(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      IBASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFW),BASE(IFX))
            CALL FLDPXY(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      IBASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFY),BASE(IFX))
         ENDIF
         MUMAX=IBASE(IMUX+LL4X-1)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL ALLDLS(LL4X,IBASE(IMUX),BASE((IL/2)*MUMAX+IC11X),
     1      BASE(IFX))
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL SETARA(BASE,LL4VX,IGAR)
            CALL XDRSET(BASE(IGAR),LL4VX,0.0)
            DO 230 I=1,LL4X
  230       BASE(IGAR+IBASE(IIPVX+I-1)-1)=BASE(IFX+I-1)
            CALL ALVDLS(LTSW,IBASE(IMUX),BASE((IL/2)*MUMAX+IC11X),
     1      BASE(IGAR),ISEG,LONX,IBASE(INBLX),IBASE(ILBLX),BASE(IT))
            DO 240 I=1,LL4X
  240       BASE(IFX+I-1)=BASE(IGAR+IBASE(IIPVX+I-1)-1)
            CALL RLSARA(BASE(IGAR))
         ENDIF
*----
*  Y DIRECTION
*----
         IF(LL4Y.GT.0) THEN
            DO 250 I0=1,LL4F
  250       BASE(IFL+I0-1)=F1(JOFF+I0)
            DO 270 I0=1,LL4W
            DO 260 J0=1,2*IELEM
            JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 270
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBW+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFW+I0-1)
  260       CONTINUE
  270       CONTINUE
            DO 290 I0=1,LL4X
            DO 280 J0=1,2*IELEM
            JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 290
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFX+I0-1)
  280       CONTINUE
  290       CONTINUE
            DO 310 I0=1,LL4Z
            DO 300 J0=1,2*IELEM
            JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 310
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFZ+I0-1)
  300       CONTINUE
  310       CONTINUE
            DO 330 I0=1,LL4Y
            GGY=-F1(JOFF+IOFY+I0)
            DO 320 J0=1,2*IELEM
            JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 330
            GGY=GGY+BASE(IBY+(I0-1)*2*IELEM+J0-1)*REAL(IL)*
     1      BASE(IFL+JJ-1)/BASE((IL/2)*LL4F+ITF+JJ-1)
  320       CONTINUE
  330       BASE(IFY+I0-1)=GGY
            IF(LL4W.GT.0) THEN
*              PIOLAT TRANSFORM TERM.
               CALL FLDPYX(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1         IBASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFX),BASE(IFY))
               CALL FLDPYW(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1         IBASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFW),BASE(IFY))
            ENDIF
            MUMAX=IBASE(IMUY+LL4Y-1)
            IF(ISEG.EQ.0) THEN
*              SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
               CALL ALLDLS(LL4Y,IBASE(IMUY),BASE((IL/2)*MUMAX+IC11Y),
     1         BASE(IFY))
            ELSE IF(ISEG.GT.0) THEN
*              SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
               CALL SETARA(BASE,LL4VY,IGAR)
               CALL XDRSET(BASE(IGAR),LL4VY,0.0)
               DO 340 I=1,LL4Y
  340          BASE(IGAR+IBASE(IIPVY+I-1)-1)=BASE(IFY+I-1)
               CALL ALVDLS(LTSW,IBASE(IMUY),BASE((IL/2)*MUMAX+IC11Y),
     1         BASE(IGAR),ISEG,LONY,IBASE(INBLY),IBASE(ILBLY),BASE(IT))
               DO 350 I=1,LL4Y
  350          BASE(IFY+I-1)=BASE(IGAR+IBASE(IIPVY+I-1)-1)
               CALL RLSARA(BASE(IGAR))
            ENDIF
         ENDIF
*----
*  Z DIRECTION
*----
         IF(LL4Z.GT.0) THEN
            DO 360 I0=1,LL4F
  360       BASE(IFL+I0-1)=F1(JOFF+I0)
            DO 380 I0=1,LL4W
            DO 370 J0=1,2*IELEM
            JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 380
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBW+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFW+I0-1)
  370       CONTINUE
  380       CONTINUE
            DO 400 I0=1,LL4X
            DO 390 J0=1,2*IELEM
            JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 400
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFX+I0-1)
  390       CONTINUE
  400       CONTINUE
            DO 420 I0=1,LL4Y
            DO 410 J0=1,2*IELEM
            JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 420
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFY+I0-1)
  410       CONTINUE
  420       CONTINUE
            DO 440 I0=1,LL4Z
            GGZ=-F1(JOFF+IOFZ+I0)
            DO 430 J0=1,2*IELEM
            JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 440
            GGZ=GGZ+BASE(IBZ+(I0-1)*2*IELEM+J0-1)*REAL(IL)*
     1      BASE(IFL+JJ-1)/BASE((IL/2)*LL4F+ITF+JJ-1)
  430       CONTINUE
  440       BASE(IFZ+I0-1)=GGZ
            MUMAX=IBASE(IMUZ+LL4Z-1)
            IF(ISEG.EQ.0) THEN
*              SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
               CALL ALLDLS(LL4Z,IBASE(IMUZ),BASE((IL/2)*MUMAX+IC11Z),
     1         BASE(IFZ))
            ELSE IF(ISEG.GT.0) THEN
*              SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
               CALL SETARA(BASE,LL4VZ,IGAR)
               CALL XDRSET(BASE(IGAR),LL4VZ,0.0)
               DO 450 I=1,LL4Z
  450          BASE(IGAR+IBASE(IIPVZ+I-1)-1)=BASE(IFZ+I-1)
               CALL ALVDLS(LTSW,IBASE(IMUZ),BASE((IL/2)*MUMAX+IC11Z),
     1         BASE(IGAR),ISEG,LONZ,IBASE(INBLZ),IBASE(ILBLZ),BASE(IT))
               DO 460 I=1,LL4Z
  460          BASE(IFZ+I-1)=BASE(IGAR+IBASE(IIPVZ+I-1)-1)
               CALL RLSARA(BASE(IGAR))
            ENDIF
         ENDIF
*----
*  COMPUTE FLUX AND RECOVER CURRENTS
*----
         DO 470 I0=1,LL4F
  470    BASE(IFL+I0-1)=F1(JOFF+I0)
         DO 490 J0=1,LL4W
         DO 480 I0=1,2*IELEM
         II=IBASE(IIPBW+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 490
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBW+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFW+J0-1)
  480    CONTINUE
  490    CONTINUE
         DO 510 J0=1,LL4X
         DO 500 I0=1,2*IELEM
         II=IBASE(IIPBX+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 510
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBX+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFX+J0-1)
  500    CONTINUE
  510    CONTINUE
         DO 530 J0=1,LL4Y
         DO 520 I0=1,2*IELEM
         II=IBASE(IIPBY+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 530
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBY+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFY+J0-1)
  520    CONTINUE
  530    CONTINUE
         DO 550 J0=1,LL4Z
         DO 540 I0=1,2*IELEM
         II=IBASE(IIPBZ+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 550
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBZ+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFZ+J0-1)
  540    CONTINUE
  550    CONTINUE
         DO 560 I0=1,LL4F
  560    F1(JOFF+I0)=BASE(IFL+I0-1)/BASE((IL/2)*LL4F+ITF+I0-1)
         IF(LL4W.GT.0) THEN
            DO 570 I0=1,LL4W
  570       F1(JOFF+IOFW+I0)=BASE(IFW+I0-1)
         ENDIF
         DO 580 I0=1,LL4X
  580    F1(JOFF+IOFX+I0)=BASE(IFX+I0-1)
         IF(LL4Y.GT.0) THEN
            DO 590 I0=1,LL4Y
  590       F1(JOFF+IOFY+I0)=BASE(IFY+I0-1)
         ENDIF
         IF(LL4Z.GT.0) THEN
            DO 600 I0=1,LL4Z
  600       F1(JOFF+IOFZ+I0)=BASE(IFZ+I0-1)
         ENDIF
      ENDIF
  610 CONTINUE
      IF(ISEG.GT.0) CALL RLSARA(BASE(IT))
      CALL RLSARA(BASE(ISOUR))
      CALL RLSARA(BASE(IFL))
      IF(LL4W.GT.0) CALL RLSARA(BASE(IFW))
      CALL RLSARA(BASE(IFX))
      IF(LL4Y.GT.0) CALL RLSARA(BASE(IFY))
      IF(LL4Z.GT.0) CALL RLSARA(BASE(IFZ))
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(IZZ))
      IF(.NOT.CHEX) THEN
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ENDIF
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      IF(CHEX) CALL RLSARA(BASE(IDIFF))
      RETURN
      END
