*DECK FLDTRI
      SUBROUTINE FLDTRI(IPTRK,NEL,NUN,EVECT,MAT,VOL,IDL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the averaged flux in TRIVAC.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the trivac tracking information.
* NEL     total number of finite elements.
* NUN     total number of unknown per energy group.
* EVECT   variational coefficients of the flux (contained in position
*         EVECT(1) to EVECT(LL4)).
* MAT     mixture index assigned to each element.
* VOL     volume of each element.
* IDL     position of the average flux component associated with each
*         volume.
*
*Parameters: output
* EVECT   averaged fluxes (contained in positions EVECT(IDL(I))).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,NEL,NUN,MAT(NEL),IDL(NEL)
      REAL EVECT(NUN),VOL(NEL)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      LOGICAL CYLIND,CHEX
      INTEGER IBASE(1),ITP(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      ITYPE=ITP(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IELEM=ABS(ITP(9))
      LL4=ITP(11)
      ICHX=ITP(12)
      ISPLH=ITP(13)
      LX=ITP(14)
      LY=ITP(15)
      LZ=ITP(16)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
*
      IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
*        LAGRANGIAN FINITE ELEMENTS.
         CALL SETARA(BASE,LX*LY*LZ,IXX)
         CALL SETARA(BASE,LX*LY*LZ,IDD)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL FLDTN2(NEL,LL4,IELEM,CYLIND,EVECT,BASE(IXX),BASE(IDD),
     1   MAT,VOL,IDL,IBASE(IKN),LC,BASE(IT),BASE(ITS))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IDD))
         CALL RLSARA(BASE(IXX))
      ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*        MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL FLDTH2(ISPLH,NEL,NUN,EVECT,MAT,VOL,IDL,IBASE(IKN))
      ELSE IF((ICHX.EQ.2).AND.CHEX) THEN
*        THOMAS-RAVIART-SCHNEIDER FINITE ELEMENTS IN HEXAGONAL GEOMETRY.
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         DO 10 IHEX=1,NEL
         IF(IDL(IHEX).NE.0) EVECT(IDL(IHEX))=0.0
   10    CONTINUE
         NBLOS=LX*LZ*ISPLH**2
         NUM=0
         DO 20 KEL=1,NBLOS
         IHEX=IBASE(IZGLOB+KEL-1)
         IF(IHEX.EQ.0) GO TO 20
         NUM=NUM+1
         JND1=(NUM-1)*IELEM**3+1
         JND2=(IBASE(IKN+NUM-1)-1)*IELEM**3+1
         JND3=(IBASE(IKN+NBLOS+NUM-1)-1)*IELEM**3+1
         TEMP=EVECT(JND1)+EVECT(JND2)+EVECT(JND3)
         EVECT(IDL(IHEX))=EVECT(IDL(IHEX))+TEMP/REAL(3*ISPLH**2)
   20    CONTINUE
      ELSE IF((ICHX.EQ.3).AND.(ISPLH.GT.1).AND.CHEX) THEN
*        MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL FLDTH1(ISPLH,NEL,LL4,EVECT,MAT,VOL,IDL,IBASE(IKN))
      ENDIF
*----
*  RELEASE TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IKN))
      RETURN
      END
