*DECK FLDTRM
      SUBROUTINE FLDTRM(NAMP,IPTRK,IPSYS,LL4,F2,F3)
*
*-----------------------------------------------------------------------
*
*Purpose:
* LCM driver for the multiplication of a matrix by a vector. Special
* version for Thomas-Raviart or Thomas-Raviart-Schneider basis.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*12
      INTEGER IPTRK,IPSYS,LL4
      REAL F2(LL4),F3(LL4)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12
      INTEGER IBASE(1),ITP(NSTATE)
      LOGICAL LMUX,DIAG
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  INITIALIZATION
*----
      NAMT=NAMP
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      IELEM=ITP(9)
      ISEG=ITP(17)
      LTSW=ITP(19)
      LL4F=ITP(25)
      LL4W=ITP(26)
      LL4X=ITP(27)
      LL4Y=ITP(28)
      LL4Z=ITP(29)
      IOFW=LL4F
      IOFX=LL4F+LL4W
      IOFY=LL4F+LL4W+LL4X
      IOFZ=LL4F+LL4W+LL4X+LL4Y
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=(LL4Y.GT.0).AND.(.NOT.LMUX)
      CALL LCMIOF(IPSYS,'TF'//NAMT,ITF)
*
      IF(LL4W.GT.0) THEN
         ISPLH=ITP(13)
         LX=ITP(14)
         LZ=ITP(16)
         NBLOS=LX*LZ*ISPLH**2
         CALL LCMIOF(IPTRK,'CTRAN',ICTRAN)
         CALL LCMIOF(IPTRK,'KN',IKN)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPSYS,'DIFF'//NAMT,IDIFF)
*
         CALL LCMIOF(IPSYS,'WA'//NAMT,IAW)
         CALL LCMIOF(IPTRK,'IPBBW',IIPBW)
         CALL LCMIOF(IPTRK,'WB',IBW)
         IF(ISEG.EQ.0) THEN
*           SCALAR MULTIPLICATION FOR A W-ORIENTED MATRIX.
            CALL LCMIOF(IPTRK,'MUW',IMU)
            CALL ALLDLM(LL4W,BASE(IAW),F2(IOFW+1),F3(IOFW+1),IBASE(IMU),
     1      1)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A W-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VW',LL4V)
            CALL LCMIOF(IPTRK,'MUVW',IMU)
            CALL LCMIOF(IPTRK,'IPVW',IIPV)
            CALL LCMLEN(IPTRK,'NBLW',LON,ITWLCM)
            CALL LCMIOF(IPTRK,'NBLW',INBL)
            CALL LCMIOF(IPTRK,'LBLW',ILBL)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            CALL XDRSET(BASE(IGAR),LL4V,0.0)
            DO 20 I=1,LL4W
   20       BASE(IGAR+IBASE(IIPV+I-1)-1)=F2(IOFW+I)
            CALL ALVDLM(LTSW,BASE(IAW),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            DO 30 I=1,LL4W
   30       F3(IOFW+I)=BASE(IGAF+IBASE(IIPV+I-1)-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
         ENDIF
         DO 50 I=1,LL4W
         GG=F3(IOFW+I)
         DO 40 J=1,2*IELEM
         II=IBASE(IIPBW+(I-1)*2*IELEM+J-1)
         IF(II.EQ.0) GO TO 50
   40    GG=GG+BASE(IBW+(I-1)*2*IELEM+J-1)*F2(II)
   50    F3(IOFW+I)=GG
*
*        PIOLAT TRANSFORM TERM.
         CALL FLDPWY(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),F2(IOFY+1),F3(IOFW+1))
         CALL FLDPWX(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),F2(IOFX+1),F3(IOFW+1))
      ENDIF
*
      IF(DIAG) THEN
         CALL LCMIOF(IPSYS,'YA'//NAMT,IAX)
      ELSE
         CALL LCMIOF(IPSYS,'XA'//NAMT,IAX)
      ENDIF
      CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
      CALL LCMIOF(IPTRK,'XB',IBX)
      IF(ISEG.EQ.0) THEN
*        SCALAR MULTIPLICATION FOR A X-ORIENTED MATRIX.
         IF(DIAG) THEN
            CALL LCMIOF(IPTRK,'MUY',IMU)
         ELSE
            CALL LCMIOF(IPTRK,'MUX',IMU)
         ENDIF
         CALL ALLDLM(LL4X,BASE(IAX),F2(IOFX+1),F3(IOFX+1),IBASE(IMU),1)
      ELSE IF(ISEG.GT.0) THEN
*        SUPERVECTORIAL MULTIPLICATION FOR A X-ORIENTED MATRIX.
         IF(DIAG) THEN
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            CALL LCMIOF(IPTRK,'MUVY',IMU)
            CALL LCMIOF(IPTRK,'IPVY',IIPV)
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBL)
            CALL LCMIOF(IPTRK,'LBLY',ILBL)
         ELSE
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            CALL LCMIOF(IPTRK,'MUVX',IMU)
            CALL LCMIOF(IPTRK,'IPVX',IIPV)
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBL)
            CALL LCMIOF(IPTRK,'LBLX',ILBL)
         ENDIF
         CALL SETARA(BASE,LL4V,IGAR)
         CALL SETARA(BASE,LL4V,IGAF)
         CALL XDRSET(BASE(IGAR),LL4V,0.0)
         DO 70 I=1,LL4X
   70    BASE(IGAR+IBASE(IIPV+I-1)-1)=F2(IOFX+I)
         CALL ALVDLM(LTSW,BASE(IAX),BASE(IGAR),BASE(IGAF),IBASE(IMU),
     1   1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
         DO 80 I=1,LL4X
   80    F3(IOFX+I)=BASE(IGAF+IBASE(IIPV+I-1)-1)
         CALL RLSARA(BASE(IGAF))
         CALL RLSARA(BASE(IGAR))
      ENDIF
      DO 100 I=1,LL4X
      GG=F3(IOFX+I)
      DO 90 J=1,2*IELEM
      II=IBASE(IIPBX+(I-1)*2*IELEM+J-1)
      IF(II.EQ.0) GO TO 100
   90 GG=GG+BASE(IBX+(I-1)*2*IELEM+J-1)*F2(II)
  100 F3(IOFX+I)=GG
*
      IF(LL4W.GT.0) THEN
*        PIOLAT TRANSFORM TERM.
         CALL FLDPXW(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),F2(IOFW+1),F3(IOFX+1))
         CALL FLDPXY(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),F2(IOFY+1),F3(IOFX+1))
      ENDIF
*
      IF(LL4Y.GT.0) THEN
         CALL LCMIOF(IPSYS,'YA'//NAMT,IAY)
         CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
         CALL LCMIOF(IPTRK,'YB',IBY)
         IF(ISEG.EQ.0) THEN
*           SCALAR MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            CALL LCMIOF(IPTRK,'MUY',IMU)
            CALL ALLDLM(LL4Y,BASE(IAY),F2(IOFY+1),F3(IOFY+1),IBASE(IMU),
     1      1)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            CALL LCMIOF(IPTRK,'MUVY',IMU)
            CALL LCMIOF(IPTRK,'IPVY',IIPV)
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBL)
            CALL LCMIOF(IPTRK,'LBLY',ILBL)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            CALL XDRSET(BASE(IGAR),LL4V,0.0)
            DO 120 I=1,LL4Y
  120       BASE(IGAR+IBASE(IIPV+I-1)-1)=F2(IOFY+I)
            CALL ALVDLM(LTSW,BASE(IAY),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            DO 130 I=1,LL4Y
  130       F3(IOFY+I)=BASE(IGAF+IBASE(IIPV+I-1)-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
         ENDIF
         DO 150 I=1,LL4Y
         GG=F3(IOFY+I)
         DO 140 J=1,2*IELEM
         II=IBASE(IIPBY+(I-1)*2*IELEM+J-1)
         IF(II.EQ.0) GO TO 150
  140    GG=GG+BASE(IBY+(I-1)*2*IELEM+J-1)*F2(II)
  150    F3(IOFY+I)=GG
*
         IF(LL4W.GT.0) THEN
*           PIOLAT TRANSFORM TERM.
            CALL FLDPYX(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      BASE(IZGLOB),BASE(IKN),BASE(IDIFF),F2(IOFX+1),F3(IOFY+1))
            CALL FLDPYW(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      BASE(IZGLOB),BASE(IKN),BASE(IDIFF),F2(IOFW+1),F3(IOFY+1))
         ENDIF
      ENDIF
*
      IF(LL4Z.GT.0) THEN
         CALL LCMIOF(IPSYS,'ZA'//NAMT,IAZ)
         CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
         CALL LCMIOF(IPTRK,'ZB',IBZ)
         IF(ISEG.EQ.0) THEN
*           SCALAR MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            CALL LCMIOF(IPTRK,'MUZ',IMU)
            CALL ALLDLM(LL4Z,BASE(IAZ),F2(IOFZ+1),F3(IOFZ+1),IBASE(IMU),
     1      1)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A Z-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VZ',LL4V)
            CALL LCMIOF(IPTRK,'MUVZ',IMU)
            CALL LCMIOF(IPTRK,'IPVZ',IIPV)
            CALL LCMLEN(IPTRK,'NBLZ',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLZ',INBL)
            CALL LCMIOF(IPTRK,'LBLZ',ILBL)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            CALL XDRSET(BASE(IGAR),LL4V,0.0)
            DO 170 I=1,LL4Z
  170       BASE(IGAR+IBASE(IIPV+I-1)-1)=F2(IOFZ+1)
            CALL ALVDLM(LTSW,BASE(IAZ),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            DO 180 I=1,LL4Z
  180       F3(IOFZ+I)=BASE(IGAF+IBASE(IIPV+I-1)-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
         ENDIF
         DO 200 I=1,LL4Z
         GG=F3(IOFZ+I)
         DO 190 J=1,2*IELEM
         II=IBASE(IIPBZ+(I-1)*2*IELEM+J-1)
         IF(II.EQ.0) GO TO 200
  190    GG=GG+BASE(IBZ+(I-1)*2*IELEM+J-1)*F2(II)
  200    F3(IOFZ+I)=GG
      ENDIF
*
      DO 210 I=1,LL4F
  210 F3(I)=BASE(ITF+I-1)*F2(I)
      DO 230 I=1,LL4W
      DO 220 J=1,2*IELEM
      II=IBASE(IIPBW+(I-1)*2*IELEM+J-1)
      IF(II.EQ.0) GO TO 230
      F3(II)=F3(II)+BASE(IBW+(I-1)*2*IELEM+J-1)*F2(IOFW+I)
  220 CONTINUE
  230 CONTINUE
      DO 250 I=1,LL4X
      DO 240 J=1,2*IELEM
      II=IBASE(IIPBX+(I-1)*2*IELEM+J-1)
      IF(II.EQ.0) GO TO 250
      F3(II)=F3(II)+BASE(IBX+(I-1)*2*IELEM+J-1)*F2(IOFX+I)
  240 CONTINUE
  250 CONTINUE
      DO 270 I=1,LL4Y
      DO 260 J=1,2*IELEM
      II=IBASE(IIPBY+(I-1)*2*IELEM+J-1)
      IF(II.EQ.0) GO TO 270
      F3(II)=F3(II)+BASE(IBY+(I-1)*2*IELEM+J-1)*F2(IOFY+I)
  260 CONTINUE
  270 CONTINUE
      DO 290 I=1,LL4Z
      DO 280 J=1,2*IELEM
      II=IBASE(IIPBZ+(I-1)*2*IELEM+J-1)
      IF(II.EQ.0) GO TO 290
      F3(II)=F3(II)+BASE(IBZ+(I-1)*2*IELEM+J-1)*F2(IOFZ+I)
  280 CONTINUE
  290 CONTINUE
      RETURN
      END
