*DECK FLDTRS
      SUBROUTINE FLDTRS(NAMP,IPTRK,IPSYS,LL4,F1,NADI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform NADI inner iterations with the ADI preconditionning.  Special
* version for Thomas-Raviart or Thomas-Raviart-Schneider basis.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
*  NAMP   name of the ADI-splitted matrix.
*  IPTRK  L_TRACK pointer to the tracking information.
*  IPSYS  L_SYSTEM pointer to system matrices.
*  LL4    order of the matrix.
*  F1     source term of the linear system.
*  NADI   number of inner ADI iterations.
*
*Parameters: output
*  F1     approached solution of the linear system.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*12
      INTEGER IPTRK,IPSYS,LL4,NADI
      REAL F1(LL4)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12
      INTEGER IBASE(1),ITP(NSTATE)
      LOGICAL LMUX,DIAG
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  INITIALIZATION
*----
      NAMT=NAMP
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      IELEM=ITP(9)
      ISEG=ITP(17)
      LTSW=ITP(19)
      LL4F=ITP(25)
      LL4W=ITP(26)
      LL4X=ITP(27)
      LL4Y=ITP(28)
      LL4Z=ITP(29)
      IOFW=LL4F
      IOFX=LL4F+LL4W
      IOFY=LL4F+LL4W+LL4X
      IOFZ=LL4F+LL4W+LL4X+LL4Y
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=(LL4Y.GT.0).AND.(.NOT.LMUX)
      CALL LCMIOF(IPSYS,'TF'//NAMT,ITF)
*
      IF(LL4W.GT.0) THEN
         ISPLH=ITP(13)
         LX=ITP(14)
         LZ=ITP(16)
         NBLOS=LX*LZ*ISPLH**2
         CALL LCMIOF(IPTRK,'CTRAN',ICTRAN)
         CALL LCMIOF(IPTRK,'KN',IKN)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPSYS,'DIFF'//NAMT,IDIFF)
*
         CALL SETARA(BASE,LL4W,IFW)
         CALL LCMIOF(IPTRK,'IPBBW',IIPBW)
         CALL LCMLEN(IPSYS,'WB',LENWB,ITYL)
         IF(LENWB.EQ.0)THEN
           CALL LCMIOF(IPTRK,'WB',IBW)
         ELSE
           CALL LCMIOF(IPSYS,'WB',IBW)
         ENDIF
         CALL LCMIOF(IPSYS,'WI'//NAMT,IC11W)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUW',IMUW)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VW',LL4VW)
            CALL LCMIOF(IPTRK,'MUVW',IMUW)
            CALL LCMIOF(IPTRK,'IPVW',IIPVW)
            CALL LCMLEN(IPTRK,'NBLW',LONW,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLW',INBLW)
            CALL LCMIOF(IPTRK,'LBLW',ILBLW)
         ENDIF
      ENDIF
      CALL SETARA(BASE,LL4X,IFX)
      CALL XDRSET(BASE(IFX),LL4X,0.0)
      CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
      CALL LCMLEN(IPSYS,'XB',LENXB,ITYL)
      IF(LENXB.EQ.0)THEN
        CALL LCMIOF(IPTRK,'XB',IBX)
      ELSE
        CALL LCMIOF(IPSYS,'XB',IBX)
      ENDIF
      IF(DIAG) THEN
         CALL LCMIOF(IPSYS,'YI'//NAMT,IC11X)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMUX)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4VX)
            CALL LCMIOF(IPTRK,'MUVY',IMUX)
            CALL LCMIOF(IPTRK,'IPVY',IIPVX)
            CALL LCMLEN(IPTRK,'NBLY',LONX,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBLX)
            CALL LCMIOF(IPTRK,'LBLY',ILBLX)
         ENDIF
      ELSE
         CALL LCMIOF(IPSYS,'XI'//NAMT,IC11X)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUX',IMUX)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VX',LL4VX)
            CALL LCMIOF(IPTRK,'MUVX',IMUX)
            CALL LCMIOF(IPTRK,'IPVX',IIPVX)
            CALL LCMLEN(IPTRK,'NBLX',LONX,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBLX)
            CALL LCMIOF(IPTRK,'LBLX',ILBLX)
         ENDIF
      ENDIF
      IF(LL4Y.GT.0) THEN
         CALL SETARA(BASE,LL4Y,IFY)
         CALL XDRSET(BASE(IFY),LL4Y,0.0)
         CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
         CALL LCMLEN(IPSYS,'YB',LENYB,ITYL)
         IF(LENYB.EQ.0)THEN
           CALL LCMIOF(IPTRK,'YB',IBY)
         ELSE
           CALL LCMIOF(IPSYS,'YB',IBY)
         ENDIF
         CALL LCMIOF(IPSYS,'YI'//NAMT,IC11Y)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMUY)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4VY)
            CALL LCMIOF(IPTRK,'MUVY',IMUY)
            CALL LCMIOF(IPTRK,'IPVY',IIPVY)
            CALL LCMLEN(IPTRK,'NBLY',LONY,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBLY)
            CALL LCMIOF(IPTRK,'LBLY',ILBLY)
         ENDIF
      ENDIF
      IF(LL4Z.GT.0) THEN
         CALL SETARA(BASE,LL4Z,IFZ)
         CALL XDRSET(BASE(IFZ),LL4Z,0.0)
         CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
         CALL LCMLEN(IPSYS,'ZB',LENZB,ITYL)
         IF(LENZB.EQ.0)THEN
           CALL LCMIOF(IPTRK,'ZB',IBZ)
         ELSE
           CALL LCMIOF(IPSYS,'ZB',IBZ)
         ENDIF
         CALL LCMIOF(IPSYS,'ZI'//NAMT,IC11Z)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUZ',IMUZ)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VZ',LL4VZ)
            CALL LCMIOF(IPTRK,'MUVZ',IMUZ)
            CALL LCMIOF(IPTRK,'IPVZ',IIPVZ)
            CALL LCMLEN(IPTRK,'NBLZ',LONZ,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLZ',INBLZ)
            CALL LCMIOF(IPTRK,'LBLZ',ILBLZ)
         ENDIF
      ENDIF
      CALL SETARA(BASE,LL4F,IFL)
*----
*  W DIRECTION
*----
      IF(ISEG.GT.0) CALL SETARA(BASE,ISEG,IT)
      DO 430 IADI=1,NADI
      IF(LL4W.GT.0) THEN
         DO 10 I0=1,LL4F
   10    BASE(IFL+I0-1)=F1(I0)
         DO 30 I0=1,LL4X
         DO 20 J0=1,2*IELEM
         JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 30
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFX+I0-1)
   20    CONTINUE
   30    CONTINUE
         DO 50 I0=1,LL4Y
         DO 40 J0=1,2*IELEM
         JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 50
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFY+I0-1)
   40    CONTINUE
   50    CONTINUE
         DO 70 I0=1,LL4Z
         DO 60 J0=1,2*IELEM
         JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 70
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFZ+I0-1)
   60    CONTINUE
   70    CONTINUE
         DO 90 I0=1,LL4W
         GGW=-F1(IOFW+I0)
         DO 80 J0=1,2*IELEM
         JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 90
         GGW=GGW+BASE(IBW+(I0-1)*2*IELEM+J0-1)*BASE(IFL+JJ-1)/
     1   BASE(ITF+JJ-1)
   80    CONTINUE
   90    BASE(IFW+I0-1)=GGW
*
*        PIOLAT TRANSFORM TERM.
         CALL FLDPWY(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFY),BASE(IFW))
         CALL FLDPWX(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFX),BASE(IFW))
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
            CALL ALLDLS(LL4W,IBASE(IMUW),BASE(IC11W),BASE(IFW))
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A W-ORIENTED LINEAR SYSTEM.
            CALL SETARA(BASE,LL4VW,IGAR)
            CALL XDRSET(BASE(IGAR),LL4VW,0.0)
            DO 100 I=1,LL4W
  100       BASE(IGAR+IBASE(IIPVW+I-1)-1)=BASE(IFW+I-1)
            CALL ALVDLS(LTSW,IBASE(IMUW),BASE(IC11W),BASE(IGAR),ISEG,
     1      LONW,IBASE(INBLW),IBASE(ILBLW),BASE(IT))
            DO 110 I=1,LL4W
  110       BASE(IFW+I-1)=BASE(IGAR+IBASE(IIPVW+I-1)-1)
            CALL RLSARA(BASE(IGAR))
         ENDIF
      ENDIF
*----
*  X DIRECTION
*----
      DO 120 I0=1,LL4F
  120 BASE(IFL+I0-1)=F1(I0)
      DO 122 I0=1,LL4W
      DO 121 J0=1,2*IELEM
      JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
      IF(JJ.EQ.0) GO TO 122
      BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBW+(I0-1)*2*IELEM+J0-1)*
     1 BASE(IFW+I0-1)
  121 CONTINUE
  122 CONTINUE
      DO 140 I0=1,LL4Y
      DO 130 J0=1,2*IELEM
      JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
      IF(JJ.EQ.0) GO TO 140
      BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)*
     1 BASE(IFY+I0-1)
  130 CONTINUE
  140 CONTINUE
      DO 160 I0=1,LL4Z
      DO 150 J0=1,2*IELEM
      JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
      IF(JJ.EQ.0) GO TO 160
      BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)*
     1 BASE(IFZ+I0-1)
  150 CONTINUE
  160 CONTINUE
      DO 180 I0=1,LL4X
      GGX=-F1(IOFX+I0)
      DO 170 J0=1,2*IELEM
      JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
      IF(JJ.EQ.0) GO TO 180
      GGX=GGX+BASE(IBX+(I0-1)*2*IELEM+J0-1)*BASE(IFL+JJ-1)/
     1 BASE(ITF+JJ-1)
  170 CONTINUE
  180 BASE(IFX+I0-1)=GGX
      IF(LL4W.GT.0) THEN
*        PIOLAT TRANSFORM TERM.
         CALL FLDPXW(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFW),BASE(IFX))
         CALL FLDPXY(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1   BASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFY),BASE(IFX))
      ENDIF
      IF(ISEG.EQ.0) THEN
*        SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
         CALL ALLDLS(LL4X,IBASE(IMUX),BASE(IC11X),BASE(IFX))
      ELSE IF(ISEG.GT.0) THEN
*        SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
         CALL SETARA(BASE,LL4VX,IGAR)
         CALL XDRSET(BASE(IGAR),LL4VX,0.0)
         DO 190 I=1,LL4X
  190    BASE(IGAR+IBASE(IIPVX+I-1)-1)=BASE(IFX+I-1)
         CALL ALVDLS(LTSW,IBASE(IMUX),BASE(IC11X),BASE(IGAR),ISEG,LONX,
     1   IBASE(INBLX),IBASE(ILBLX),BASE(IT))
         DO 200 I=1,LL4X
  200    BASE(IFX+I-1)=BASE(IGAR+IBASE(IIPVX+I-1)-1)
         CALL RLSARA(BASE(IGAR))
      ENDIF
*----
*  Y DIRECTION
*----
      IF(LL4Y.GT.0) THEN
         DO 210 I0=1,LL4F
  210    BASE(IFL+I0-1)=F1(I0)
         DO 230 I0=1,LL4W
         DO 220 J0=1,2*IELEM
         JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 230
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBW+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFW+I0-1)
  220    CONTINUE
  230    CONTINUE
         DO 250 I0=1,LL4X
         DO 240 J0=1,2*IELEM
         JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 250
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFX+I0-1)
  240    CONTINUE
  250    CONTINUE
         DO 270 I0=1,LL4Z
         DO 260 J0=1,2*IELEM
         JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 270
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFZ+I0-1)
  260    CONTINUE
  270    CONTINUE
         DO 290 I0=1,LL4Y
         GGY=-F1(IOFY+I0)
         DO 280 J0=1,2*IELEM
         JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 290
         GGY=GGY+BASE(IBY+(I0-1)*2*IELEM+J0-1)*BASE(IFL+JJ-1)/
     1   BASE(ITF+JJ-1)
  280    CONTINUE
  290    BASE(IFY+I0-1)=GGY
         IF(LL4W.GT.0) THEN
*           PIOLAT TRANSFORM TERM.
            CALL FLDPYX(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      BASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFX),BASE(IFY))
            CALL FLDPYW(LL4W,LL4X,LL4Y,NBLOS,IELEM,BASE(ICTRAN),
     1      BASE(IZGLOB),BASE(IKN),BASE(IDIFF),BASE(IFW),BASE(IFY))
         ENDIF
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL ALLDLS(LL4Y,IBASE(IMUY),BASE(IC11Y),BASE(IFY))
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL SETARA(BASE,LL4VY,IGAR)
            CALL XDRSET(BASE(IGAR),LL4VY,0.0)
            DO 300 I=1,LL4Y
  300       BASE(IGAR+IBASE(IIPVY+I-1)-1)=BASE(IFY+I-1)
            CALL ALVDLS(LTSW,IBASE(IMUY),BASE(IC11Y),BASE(IGAR),ISEG,
     1      LONY,IBASE(INBLY),IBASE(ILBLY),BASE(IT))
            DO 310 I=1,LL4Y
  310       BASE(IFY+I-1)=BASE(IGAR+IBASE(IIPVY+I-1)-1)
            CALL RLSARA(BASE(IGAR))
         ENDIF
      ENDIF
*----
*  Z DIRECTION
*----
      IF(LL4Z.GT.0) THEN
         DO 320 I0=1,LL4F
  320    BASE(IFL+I0-1)=F1(I0)
         DO 340 I0=1,LL4W
         DO 330 J0=1,2*IELEM
         JJ=IBASE(IIPBW+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 340
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBW+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFW+I0-1)
  330    CONTINUE
  340    CONTINUE
         DO 360 I0=1,LL4X
         DO 350 J0=1,2*IELEM
         JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 360
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFX+I0-1)
  350    CONTINUE
  360    CONTINUE
         DO 380 I0=1,LL4Y
         DO 370 J0=1,2*IELEM
         JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 380
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)
     1   *BASE(IFY+I0-1)
  370    CONTINUE
  380    CONTINUE
         DO 400 I0=1,LL4Z
         GGZ=-F1(IOFZ+I0)
         DO 390 J0=1,2*IELEM
         JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 400
         GGZ=GGZ+BASE(IBZ+(I0-1)*2*IELEM+J0-1)*BASE(IFL+JJ-1)/
     1   BASE(ITF+JJ-1)
  390    CONTINUE
  400    BASE(IFZ+I0-1)=GGZ
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL ALLDLS(LL4Z,IBASE(IMUZ),BASE(IC11Z),BASE(IFZ))
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL SETARA(BASE,LL4VZ,IGAR)
            CALL XDRSET(BASE(IGAR),LL4VZ,0.0)
            DO 410 I=1,LL4Z
  410       BASE(IGAR+IBASE(IIPVZ+I-1)-1)=BASE(IFZ+I-1)
            CALL ALVDLS(LTSW,IBASE(IMUZ),BASE(IC11Z),BASE(IGAR),ISEG,
     1      LONZ,IBASE(INBLZ),IBASE(ILBLZ),BASE(IT))
            DO 420 I=1,LL4Z
  420       BASE(IFZ+I-1)=BASE(IGAR+IBASE(IIPVZ+I-1)-1)
            CALL RLSARA(BASE(IGAR))
         ENDIF
      ENDIF
  430 CONTINUE
      IF(ISEG.GT.0) CALL RLSARA(BASE(IT))
      CALL RLSARA(BASE(IFL))
*----
*  COMPUTE FLUX AND RECOVER CURRENTS
*----
      DO 460 J0=1,LL4W
      DO 450 I0=1,2*IELEM
      II=IBASE(IIPBW+(J0-1)*2*IELEM+I0-1)
      IF(II.EQ.0) GO TO 460
      F1(II)=F1(II)-BASE(IBW+(J0-1)*2*IELEM+I0-1)*BASE(IFW+J0-1)
  450 CONTINUE
  460 CONTINUE
      DO 480 J0=1,LL4X
      DO 470 I0=1,2*IELEM
      II=IBASE(IIPBX+(J0-1)*2*IELEM+I0-1)
      IF(II.EQ.0) GO TO 480
      F1(II)=F1(II)-BASE(IBX+(J0-1)*2*IELEM+I0-1)*BASE(IFX+J0-1)
  470 CONTINUE
  480 CONTINUE
      DO 500 J0=1,LL4Y
      DO 490 I0=1,2*IELEM
      II=IBASE(IIPBY+(J0-1)*2*IELEM+I0-1)
      IF(II.EQ.0) GO TO 500
      F1(II)=F1(II)-BASE(IBY+(J0-1)*2*IELEM+I0-1)*BASE(IFY+J0-1)
  490 CONTINUE
  500 CONTINUE
      DO 520 J0=1,LL4Z
      DO 510 I0=1,2*IELEM
      II=IBASE(IIPBZ+(J0-1)*2*IELEM+I0-1)
      IF(II.EQ.0) GO TO 520
      F1(II)=F1(II)-BASE(IBZ+(J0-1)*2*IELEM+I0-1)*BASE(IFZ+J0-1)
  510 CONTINUE
  520 CONTINUE
      DO 530 I0=1,LL4F
  530 F1(I0)=F1(I0)/BASE(ITF+I0-1)
      IF(LL4W.GT.0) THEN
         DO 540 I0=1,LL4W
  540    F1(IOFW+I0)=BASE(IFW+I0-1)
         CALL RLSARA(BASE(IFW))
      ENDIF
      DO 550 I0=1,LL4X
  550 F1(IOFX+I0)=BASE(IFX+I0-1)
      CALL RLSARA(BASE(IFX))
      IF(LL4Y.GT.0) THEN
         DO 560 I0=1,LL4Y
  560    F1(IOFY+I0)=BASE(IFY+I0-1)
         CALL RLSARA(BASE(IFY))
      ENDIF
      IF(LL4Z.GT.0) THEN
         DO 570 I0=1,LL4Z
  570    F1(IOFZ+I0)=BASE(IFZ+I0-1)
         CALL RLSARA(BASE(IFZ))
      ENDIF
      RETURN
      END
