*DECK FLDTSM
      SUBROUTINE FLDTSM(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F2,F3,SGD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* LCM driver for the multiplication of a matrix by a vector.
* Special version for the simplified PN method in TRIVAC.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* NBMIX   total number of material mixtures in the macrolib.
* NAN     number of Legendre orders in the cross sections.
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*Parameters: scratch
* SGD
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,NBMIX,NAN
      REAL F2(LL4),F3(LL4),SGD(NBMIX,2*NAN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,TEXT12*12
      INTEGER IBASE(1),IPAR(NSTATE)
      LOGICAL CHEX
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER PN SPECIFIC PARAMETERS.
*----
      NAMT=NAMP
      READ(NAMT,'(1X,2I3)') IGR,JGR
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      NREG=IPAR(1)
      NUN=IPAR(2)
      ITYPE=IPAR(6)
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IELEM=IPAR(9)
      ICOL=IPAR(10)
      L4=IPAR(11)
      ISPLH=IPAR(13)
      LX=IPAR(14)
      LZ=IPAR(16)
      LL4F=IPAR(25)
      LL4W=IPAR(26)
      LL4X=IPAR(27)
      LL4Y=IPAR(28)
      NLF=IPAR(30)
      NVD=IPAR(34)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IF(CHEX) THEN
         IF(NUN.GT.(LX*LZ+L4)*NLF/2) CALL XABORT('FLDTSM: INVALID NUN '
     1   //'OR L4.')
      ELSE
        IF(NUN.NE.L4*NLF/2) CALL XABORT('FLDTSM: INVALID NUN OR L4.')
      ENDIF
      IF(L4*NLF/2.NE.LL4) CALL XABORT('FLDTSM: INVALID L4 OR LL4.')
*----
*  RECOVER TRACKING INFORMATION.
*----
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
         CALL SETARA(BASE,NREG,IXX)
         CALL SETARA(BASE,NREG,IYY)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
      ENDIF
      CALL SETARA(BASE,NREG,IZZ)
      CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
*----
*  RECOVER THE CROSS SECTIONS.
*----
      DO 20 IL=1,NAN
      WRITE(TEXT12,'(4HSCAR,I2.2,A6)') IL-1,NAMT(2:7)
      CALL LCMLEN(IPSYS,TEXT12,LENGT,ITYLCM)
      IF(LENGT.EQ.0) THEN
         CALL XDRSET(SGD(1,IL),NBMIX,0.0)
         CALL XDRSET(SGD(1,NAN+IL),NBMIX,0.0)
      ELSE
         CALL LCMGET(IPSYS,TEXT12,SGD(1,IL))
         WRITE(TEXT12,'(4HSCAI,I2.2,A6)') IL-1,NAMT(2:7)
         CALL LCMGET(IPSYS,TEXT12,SGD(1,NAN+IL))
      ENDIF
   20 CONTINUE
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  COMPUTE THE SOURCE
*----
      ITY=0
      IF(IGR.NE.JGR) ITY=1
      IF(CHEX) THEN
         NBLOS=LX*LZ*ISPLH**2
         CALL LCMIOF(IPTRK,'CTRAN',ICTRAN)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPTRK,'FRZ',IFRZ)
         CALL SETARA(BASE,NBLOS,IDIFF)
         CALL PNSH3D(ITY,NBMIX,NBLOS,IELEM,ICOL,ISPLH,IDIM,NLF,NVD,
     1   NAN,LX,LZ,L4,LL4F,LL4W,LL4X,LL4Y,IBASE(IMAT),SGD(1,1),
     2   SGD(1,NAN+1),SIDE,BASE(IZZ),BASE(IFRZ),BASE(IQFR),
     3   IBASE(IZGLOB),IBASE(IKN),LC,BASE(IR),BASE(IV),BASE(ICTRAN),
     4   F2,F3,BASE(IDIFF))
         CALL RLSARA(BASE(IDIFF))
      ELSE
         CALL PNSZ3D(ITY,NREG,IELEM,ICOL,IDIM,BASE(IXX),BASE(IYY),
     1   BASE(IZZ),IBASE(IMAT),BASE(IVOL),NBMIX,NLF,NVD,NAN,SGD(1,1),
     2   SGD(1,NAN+1),L4,IBASE(IKN),BASE(IQFR),LC,BASE(IR),BASE(IV),
     3   F2,F3)
      ENDIF
      IF(ITY.EQ.1) THEN
         DO 30 I=1,LL4
   30    F3(I)=-F3(I)
      ENDIF
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(IZZ))
      IF(.NOT.CHEX) THEN
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ENDIF
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
