*DECK GPTDFL
      SUBROUTINE GPTDFL (IPTRK,IPSYS0,IPFLUP,LL4,ITY,NUN,NGRP,ICL1,ICL2,
     1 IMPX,IMPH,TITR,EPS2,MAXINR,EPSINR,NADI,MAXX0,FKEFF,EVECT,ADECT,
     2 FKEFF2,EASS,SOUR,GRAD1,GRAD2,GAR1,GAR2,GAR3,WORK1,WORK2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solution of a multigroup fixed source eigenvalue problem for the
* calculation of a direct GPT solution in Trivac. Use the precondi-
* tioned power method with a two group SVAT acceleration technique.
*
*Copyright:
* Copyright (C) 1987 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the tracking information
* IPSYS0  L_SYSTEM pointer to unperturbed system matrices
* IPFLUP  L_FLUX pointer to the gpt solution
* LL4     order of the system matrices.
* ITY     type of solution (2: classical Trivac; 3: Thomas-Raviart).
* NUN     number of unknowns in each energy group.
* NGRP    number of energy groups.
* ICL1    number of free iretations in one cycle of the inverse power
*         method
* ICL2    number of accelerated iterations in one cycle
* IMPX    print parameter. =0: no print ; =1: minimum printing ;
*         =2: iteration history is printed. =3: solution is printed
* IMPH    =0: no action is taken
*         =1: the flux is compared to a reference flux stored on lcm
*         =2: the convergence histogram is printed
*         =3: the convergence histogram is printed with axis and
*            titles. the plotting file is completed
*         =4: the convergence histogram is printed with axis, acce-
*            leration factors and titles. the plotting file is
*            completed
* TITR    character*72 title
* EPS2    convergence criteria for the flux
* MAXINR  maximum number of thermal iterations.
* EPSINR  thermal iteration epsilon.
* NADI    number of inner adi iterations per outer iteration
* MAXX0   maximum number of outer iterations
* FKEFF   effective multiplication factor
* EVECT   unknown vector for the non perturbed direct flux
* ADECT   unknown vector for the non perturbed adjoint flux
* SOUR    fixed source
*
*Parameters: output
* FKEFF2  perturbed effective multiplication factor
* EASS    converged solution
*
*Parameters: scratch
* GRAD1
* GRAD2
* GAR1
* GAR2
* GAR3
* WORK1
* WORK2
*
*References:
* A. H\'ebert, 'Preconditioning the power method for reactor
* calculations', Nucl. Sci. Eng., 94, 1 (1986).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER TITR*72
      INTEGER IPTRK,IPSYS0,IPFLUP,LL4,ITY,NUN,NGRP,ICL1,ICL2,IMPX,IMPH,
     1 MAXINR,NADI,MAXX0
      REAL EPS2,EPSINR,FKEFF,EVECT(NUN,NGRP),ADECT(NUN,NGRP),FKEFF2,
     1 EASS(NUN,NGRP),SOUR(NUN,NGRP),GRAD1(NUN,NGRP),GRAD2(NUN,NGRP),
     2 GAR1(NUN,NGRP),GAR2(NUN,NGRP),GAR3(NUN,NGRP),WORK1(NUN),
     3 WORK2(NUN)
*----
*  LOCAL VARIABLES
*----
      CHARACTER*12 TEXT12
      LOGICAL LOGTES,LMPH
      DOUBLE PRECISION AIL,BIL,EVAL,D2F(2,3),ALP,BET,DDELN1,DDELD1,
     1 ZNORM,GAZ
      REAL ERR(250),ALPH(250),BETA(250),TKT,TKB
      INTEGER  ITITR(18)
      COMMON BASE(1)
      DATA EPS1,MMAXX/1.0E-4,250/
*
      CALL MTOPEN(IMPX,IPTRK,LL4)
      IF(LL4.GT.NUN) CALL XABORT('GPTDFL: INVALID NUMBER OF UNKNOWNS.')
*----
*  UNPERTURBED EIGENVALUE CALCULATION.
*----
      AIL=0.0D0
      BIL=0.0D0
      DO 80 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS0,LL4,ITY,EVECT(1,IGR),GRAD1(1,IGR))
      CALL XDRSET(WORK1,LL4,0.0)
      DO 70 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 40
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 40
      CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
      DO 20 I=1,LL4
   20 GRAD1(I,IGR)=GRAD1(I,IGR)-BASE(IAGAR+I-1)*EVECT(I,JGR)
   40 WRITE(TEXT12,'(1HB,2I3.3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 70
      CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
      DO 50 I=1,LL4
   50 WORK1(I)=WORK1(I)+BASE(IAGAR+I-1)*EVECT(I,JGR)
   70 CONTINUE
      DO 80 I=1,LL4
      AIL=AIL+ADECT(I,IGR)*GRAD1(I,IGR)
   80 BIL=BIL+ADECT(I,IGR)*WORK1(I)
      EVAL=AIL/BIL
      FKEFF2=1.0/EVAL
      IF(ABS(FKEFF-1.0/EVAL).GT.EPS1) CALL XABORT('GPTDFL: THE COMPUTE'
     1 //'D AND PROVIDED K-EFFECTIVES ARE INCONSISTENTS.')
*----
*  VALIDATION OF THE FIXED SOURCE TERM.
*----
      AIL=0.0D0
      BIL=0.0D0
      DO 90 IGR=1,NGRP
      DO 90 I=1,LL4
      GAZ=ADECT(I,IGR)*SOUR(I,IGR)
      AIL=AIL+GAZ
   90 BIL=BIL+GAZ**2
      GAZ=ABS(AIL)/SQRT(REAL(LL4)*BIL)
      IF(GAZ.GT.EPS2) CALL XABORT('GPTDFL: THE SOURCE TERM IS NOT ORT'
     1 //'HOGONAL TO THE ADJOINT REFERENCE FLUX.')
*----
*  PRECONDITIONED POWER METHOD.
*----
      ISTART=1
      NNADI=NADI
      IF(IMPX.GE.1) WRITE (6,600) NADI
      IF(IMPX.GE.2) WRITE (6,610)
      M=0
  100 M=M+1
*
      IF(MOD(M-ISTART+1,ICL1+ICL2).EQ.1) THEN
         DO 135 IGR=1,NGRP
         WRITE(TEXT12,'(1HA,2I3.3)') IGR,IGR
         CALL MTLDLM(TEXT12,IPTRK,IPSYS0,LL4,ITY,EASS(1,IGR),
     +               GAR1(1,IGR))
         DO 135 JGR=1,NGRP
         IF(JGR.EQ.IGR) GO TO 120
         WRITE(TEXT12,'(1HA,2I3.3)') IGR,JGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 120
         CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
         DO 110 I=1,LL4
  110    GAR1(I,IGR)=GAR1(I,IGR)-BASE(IAGAR+I-1)*EASS(I,JGR)
  120    WRITE(TEXT12,'(1HB,2I3.3)') IGR,JGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 135
         CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
         DO 125 I=1,LL4
  125    GAR1(I,IGR)=GAR1(I,IGR)-EVAL*BASE(IAGAR+I-1)*EASS(I,JGR)
  135    CONTINUE
      ENDIF
*----
*  DIRECTION EVALUATION.
*----
      DO 180 IGR=1,NGRP
      DO 140 I=1,LL4
  140 GRAD1(I,IGR)=-SOUR(I,IGR)-GAR1(I,IGR)
      DO 170 JGR=1,IGR-1
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 170
      CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
      DO 150 I=1,LL4
  150 GRAD1(I,IGR)=GRAD1(I,IGR)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
  170 CONTINUE
*
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,IGR
      CALL FLDADI(TEXT12,IPTRK,IPSYS0,LL4,ITY,GRAD1(1,IGR),NNADI)
  180 CONTINUE
*----
*  PERFORM THERMAL (UP-SCATTERING) ITERATIONS
*----
      IF(MAXINR.GT.1) THEN
         CALL SETARA(BASE,LL4*NGRP*3,IWORK1)
         CALL FLDTHR(IPTRK,IPSYS0,.FALSE.,LL4,ITY,NUN,NGRP,ICL1,ICL2,
     1   IMPX,NNADI,MAXINR,EPSINR,TKT,TKB,AGAR,GRAD1,GAR2,BASE(IWORK1))
         CALL RLSARA(BASE(IWORK1))
      ENDIF
*----
*  HOTELLING DEFLATION.
*----
      DDELN1=0.0D0
      DDELD1=0.0D0
      DO 220 IGR=1,NGRP
      CALL XDRSET(WORK1,LL4,0.0)
      DO 210 JGR=1,NGRP
      WRITE(TEXT12,'(1HB,2I3.3)') JGR,IGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 210
      CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
      DO 200 I=1,LL4
  200 WORK1(I)=WORK1(I)+BASE(IAGAR+I-1)*ADECT(I,JGR)
  210 CONTINUE
      DO 220 I=1,LL4
      DDELN1=DDELN1+WORK1(I)*EASS(I,IGR)
  220 DDELD1=DDELD1+WORK1(I)*EVECT(I,IGR)
      ZNORM=DDELN1/DDELD1
      DO 230 IGR=1,NGRP
      DO 230 I=1,LL4
  230 GRAD1(I,IGR)=GRAD1(I,IGR)-ZNORM*EVECT(I,IGR)
*----
*  EVALUATION OF THE DISPLACEMENT AND OF THE TWO ACCELERATION PARAMETERS
*  ALP AND BET.
*----
      ALP=1.0D0
      BET=0.0D0
      DO 240 I=1,2
      DO 240 J=1,3
  240 D2F(I,J)=0.0D0
      DO 280 IGR=1,NGRP
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,IGR
      CALL MTLDLM(TEXT12,IPTRK,IPSYS0,LL4,ITY,GRAD1(1,IGR),GAR2(1,IGR))
      DO 280 JGR=1,NGRP
      IF(JGR.EQ.IGR) GO TO 260
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 260
      CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
      DO 250 I=1,LL4
  250 GAR2(I,IGR)=GAR2(I,IGR)-BASE(IAGAR+I-1)*GRAD1(I,JGR)
  260 WRITE(TEXT12,'(1HB,2I3.3)') IGR,JGR
      CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
      IF(ILONG.EQ.0) GO TO 280
      CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
      DO 270 I=1,LL4
  270 GAR2(I,IGR)=GAR2(I,IGR)-EVAL*BASE(IAGAR+I-1)*GRAD1(I,JGR)
  280 CONTINUE
      IF(1+MOD(M-ISTART,ICL1+ICL2).GT.ICL1) THEN
         DO 290 IGR=1,NGRP
         DO 290 I=1,LL4
         D2F(1,1)=D2F(1,1)+GAR2(I,IGR)**2
         D2F(1,2)=D2F(1,2)+GAR2(I,IGR)*GAR3(I,IGR)
         D2F(2,2)=D2F(2,2)+GAR3(I,IGR)**2
         D2F(1,3)=D2F(1,3)-(GAR1(I,IGR)+SOUR(I,IGR))*GAR2(I,IGR)
  290    D2F(2,3)=D2F(2,3)-(GAR1(I,IGR)+SOUR(I,IGR))*GAR3(I,IGR)
         D2F(2,1)=D2F(1,2)
*        SOLUTION OF A LINEAR SYSTEM.
         CALL ALSBD(2,1,D2F,IER,2)
         IF(IER.NE.0) CALL XABORT('GPTDFL: SINGULAR MATRIX.')
         ALP=D2F(1,3)
         BET=D2F(2,3)/ALP
         IF((ALP.LT.1.0D0).AND.(ALP.GT.0.0D0)) THEN
            ALP=1.0D0
            BET=0.0D0
         ELSE IF(ALP.LE.0.0D0) THEN
            ISTART=M+1
            ALP=1.0D0
            BET=0.0D0
         ENDIF
         DO 300 IGR=1,NGRP
         DO 300 I=1,LL4
         GRAD1(I,IGR)=ALP*(GRAD1(I,IGR)+BET*GRAD2(I,IGR))
  300    GAR2(I,IGR)=ALP*(GAR2(I,IGR)+BET*GAR3(I,IGR))
      ENDIF
*
      LOGTES=(M.LT.ICL1).OR.(MOD(M,ICL1+ICL2).EQ.ICL1)
      IF(LOGTES) THEN
         DELT=0.0
         DO 350 IGR=1,NGRP
         CALL XDRSET(WORK1,LL4,0.0)
         CALL XDRSET(WORK2,LL4,0.0)
         DO 320 JGR=1,NGRP
         WRITE(TEXT12,'(1HB,2I3.3)') IGR,JGR
         CALL LCMLEN(IPSYS0,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.0) GO TO 320
         CALL LCMIOF(IPSYS0,TEXT12,IAGAR)
         DO 310 I=1,ILONG
         WORK1(I)=WORK1(I)+BASE(IAGAR+I-1)*EASS(I,JGR)
  310    WORK2(I)=WORK2(I)+BASE(IAGAR+I-1)*GRAD1(I,JGR)
  320    CONTINUE
         DELN=0.0
         DELD=0.0
         DO 340 I=1,LL4
         EASS(I,IGR)=EASS(I,IGR)+GRAD1(I,IGR)
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
         GAR3(I,IGR)=GAR2(I,IGR)
         DELN=MAX(DELN,ABS(WORK2(I)))
  340    DELD=MAX(DELD,ABS(WORK1(I)))
         IF(DELD.NE.0.0) DELT=MAX(DELT,DELN/DELD)
  350    CONTINUE
         IF(IMPX.GE.2) WRITE (6,620) M,ALP,BET,ZNORM,DELT
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL FLDXCO(IPFLUP,LL4,NUN,EASS(1,NGRP),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
         IF(DELT.LT.EPS2) GO TO 370
      ELSE
         DO 360 IGR=1,NGRP
         DO 360 I=1,LL4
         EASS(I,IGR)=EASS(I,IGR)+GRAD1(I,IGR)
         GAR1(I,IGR)=GAR1(I,IGR)+GAR2(I,IGR)
         GRAD2(I,IGR)=GRAD1(I,IGR)
  360    GAR3(I,IGR)=GAR2(I,IGR)
         IF(IMPX.GE.2) WRITE (6,620) M,ALP,BET,ZNORM
*        COMPUTE THE CONVERGENCE HISTOGRAM.
         IF(IMPH.GE.1) THEN
            LMPH=IMPH.GE.1
            CALL FLDXCO(IPFLUP,LL4,NUN,EASS(1,NGRP),LMPH,ERR(M))
            ALPH(M)=ALP
            BETA(M)=BET
         ENDIF
      ENDIF
      IF(M.EQ.1) TEST=DELT
      IF((M.GT.20).AND.(DELT.GT.TEST)) CALL XABORT('GPTDFL: CONVERGENC'
     1 //'E FAILURE.')
      IF(M.GE.MIN(MAXX0,MMAXX)) THEN
         WRITE (6,710)
         GO TO 370
      ENDIF
      IF(MOD(M,36).EQ.0) THEN
         ISTART=M+1
         NNADI=NNADI+1
         IF (IMPX.NE.0) WRITE (6,720) NNADI
      ENDIF
      GO TO 100
*----
*  SOLUTION EDITION.
*----
  370 IF(IMPX.EQ.1) WRITE (6,640) M
      IF(IMPX.GE.3) THEN
         DO 380 IGR=1,NGRP
         WRITE (6,690) IGR,(EASS(I,IGR),I=1,LL4)
  380    CONTINUE
      ENDIF
      IF(IMPH.GE.2) THEN
         IGRAPH=0
  390    IGRAPH=IGRAPH+1
         WRITE (TEXT12,'(5HHISTO,I3)') IGRAPH
         CALL LCMLEN (IPFLUP,TEXT12,ILENG,ITYLCM)
         IF(ILENG.EQ.0) THEN
            READ (TITR,'(18A4)') ITITR
            CALL LCMSIX (IPFLUP,TEXT12,1)
            CALL LCMPUT (IPFLUP,'HTITLE',18,3,ITITR)
            CALL LCMPUT (IPFLUP,'ALPHA',M,2,ALPH)
            CALL LCMPUT (IPFLUP,'BETA',M,2,BETA)
            CALL LCMPUT (IPFLUP,'ERROR',M,2,ERR)
            CALL LCMPUT (IPFLUP,'IMPH',1,1,IMPH)
            CALL LCMSIX (IPFLUP,' ',2)
         ELSE
            GO TO 390
         ENDIF
      ENDIF
      RETURN
*
  600 FORMAT(1H1/50H GPTDFL: ITERATIVE PROCEDURE BASED ON PRECONDITION,
     1 17HED POWER METHOD (,I2,37H ADI ITERATIONS PER OUTER ITERATION)./
     2 9X,39HDIRECT FIXED SOURCE EIGENVALUE PROBLEM.)
  610 FORMAT (/29X,15HORTHONORMALIZA-/11X,5HALPHA,3X,4HBETA,6X,
     1 11HTION FACTOR,6X,8HACCURACY,12(1H.))
  620 FORMAT (1X,I3,4X,2F8.3,1PE14.2,6X,2E10.2)
  640 FORMAT(/23H GPTDFL: CONVERGENCE IN,I4,12H ITERATIONS.)
  650 FORMAT(/53H GPTDFL: CPU TIME USED TO SOLVE THE TRIANGULAR LINEAR,
     1 10H SYSTEMS =,F9.3/23X,34HTO COMPUTE THE BILINEAR PRODUCTS =,
     2 F9.3,20X,16HTOTAL CPU TIME =,F9.3)
  690 FORMAT(//52H GPTDFL: DIRECT FIXED SOURCE PROBLEM SOLUTION CORRES,
     1 20HPONDING TO THE GROUP,I4//(5X,1P,8E14.5))
  710 FORMAT(/53H GPTDFL: ***WARNING*** THE MAXIMUM NUMBER OF OUTER IT,
     1 20HERATIONS IS REACHED.)
  720 FORMAT(/53H GPTDFL: INCREASING THE NUMBER OF INNER ITERATIONS TO,
     1 I3,36H ADI ITERATIONS PER OUTER ITERATION./)
      END
