*DECK GPTFLU
      SUBROUTINE GPTFLU(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* computes generalized adjoints.
* GPTFLU = Generalized Perturbation Theory FLUx calculation
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert,E. Varin and R. Chambon
*
*Parameters: input/ouput
* NENTRY  number of linked lists or files used by the module
* HENTRY  character*12 name of each linked list or file
*         HENTRY(1): create or modification type(L_FLUX) (GPT solution)
*         HENTRY(2): read-only type(L_GPT) => GPT fixed source
*         HENTRY(3): read-only type(L_FLUX) => unperturbed solution
*         HENTRY(4): read-only type(L_SYSTEM) => reference matrices
*         HENTRY(5): read-only type(L_TRACK) => TRIVAC tracking.
* IENTRY  =1 linked list; =2 xsm file; =3 sequential binary file;
*         =4 sequential ascii file
* JENTRY  =0 the linked list or file is created
*         =1 the linked list or file is open for modifications;
*         =2 the linked list or file is open in read-only mode
* KENTRY  =file unit number; =linked list address otherwise.
*
*-----------------------------------------------------------------------
*
      IMPLICIT  NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NENTRY
      CHARACTER HENTRY(NENTRY)*12
      INTEGER   IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      INTEGER   IPRINT,NITMA,I,J,IGR,IPFLUP,IPFLU,IPGPT,IPSYS,IPTRK,
     1          ITYP,LENGT,SIGNA(3),ITY,KCHAR(18)
      DOUBLE PRECISION DFLOTT
      REAL      FLOTT
      CHARACTER TEXT12*12,CMODUL*12
      LOGICAL   LFLU
      INTEGER   NEL,NUN,NGRP,LL4,ISRC
      CHARACTER TITLE*72
*----
*  STATE-VECTOR VARIABLES
*----
      INTEGER   NSTATE
      PARAMETER (NSTATE=40)
      INTEGER   FLUPRM(NSTATE),SYSPRM(NSTATE),TRKPRM(NSTATE),
     1          GPTPRM(NSTATE)
*----
*  Generalized Adjoint calculation
*----
      INTEGER   IGRAD1,IGRAD2,IGAR1,IGAR2,IGAR3,IEASS,IADECT,IEVECT,
     1          SRCFRM,SRCTO,MAXOUT,ICL1,ICL2,NADI,IMPH,NLF,ISOUR,
     2          IMAT,IVOL,IIDL,IWORK1,IWORK2,MAXINR,JPFLU1,JPFLU2,
     3          JPGPT,KPGPT,JPFLUP,KPFLUP,LCMGID,LCMLID,LCMGIL,LCMLIL
      REAL      FKEFF,FKEFF2,EPSOUT,EPSINR,EPSCON(3)
      LOGICAL   ADJ,REC,RECP
      REAL      RBASE
      INTEGER   IBASE(1)
      COMMON    RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*----
*  VALIDITY OF OBJECTS
*----
      IF(NENTRY.LT.4) CALL XABORT('GPTFLU: 5 OBJECTS EXPECTED.')
      IPFLUP=0
      IPFLU =0
      IPGPT =0
      IPTRK =0
      IPSYS =0
      RECP=(JENTRY(1).EQ.1)
      LFLU=.FALSE.
      DO 2 I=1,NENTRY
         IF((IENTRY(I).NE.1).AND.(IENTRY(I).NE.2))CALL XABORT('GPTFLU:'
     1      //' LINKED LIST OR XSM FILE EXPECTED')
         IF((JENTRY(I).EQ.0).AND.(I.EQ.1)) THEN
            TEXT12='L_FLUX'
            READ(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
            CALL LCMPUT(KENTRY(I),'SIGNATURE',3,3,SIGNA)
         ELSE
            CALL LCMGET(KENTRY(I),'SIGNATURE',SIGNA)
            WRITE(TEXT12,'(3A4)') (SIGNA(J),J=1,3)
            IF(JENTRY(I).EQ.0)  CALL XABORT('GPTFLU:'//TEXT12//' IS '
     1      //'NOT ON RHS')
         ENDIF
         IF(TEXT12.EQ.'L_FLUX') THEN 
             IF(LFLU) THEN
                IPFLU=KENTRY(I)
             ELSE
                IPFLUP=KENTRY(I)
                LFLU=.TRUE.
             ENDIF
         ELSEIF(TEXT12.EQ.'L_GPT') THEN 
             IPGPT=KENTRY(I)
         ELSEIF(TEXT12.EQ.'L_TRACK') THEN 
             IPTRK=KENTRY(I)
         ELSEIF(TEXT12.EQ.'L_SYSTEM') THEN 
             IPSYS=KENTRY(I)
         ELSE
             CALL XABORT('GPTFLU: NOT GOOD TYPE OF OBJECT')
         ENDIF
    2 CONTINUE
      IF(IPGPT.EQ.0) CALL XABORT('GPTFLU: MISSING GPT SOURCE OBJECT.')
      IF(IPFLU.EQ.0) CALL XABORT('GPTFLU: MISSING FLUX OBJECT.')
      IF(IPSYS.EQ.0) CALL XABORT('GPTFLU: MISSING SYSTEM OBJECT.')
      IF(IPTRK.EQ.0) CALL XABORT('GPTFLU: MISSING TRACK OBJECT.')
*----
*  VARIABLE INITIALISATION	
*----
      CALL LCMGET(IPFLU,'STATE-VECTOR',FLUPRM)
      NGRP   = FLUPRM(1)
      NUN    = FLUPRM(2)
      MAXINR = FLUPRM(11)
      MAXOUT = FLUPRM(12)
      CALL LCMGET(IPFLU,'EPS-CONVERGE',EPSCON)
      EPSINR=EPSCON(1)
      EPSOUT=EPSCON(2)
      CALL LCMGET(IPTRK,'STATE-VECTOR',TRKPRM)
      NEL    = TRKPRM(1)
      IF(NUN.NE.TRKPRM(2)) CALL XABORT('GPTFLU: TRACKING AND UNPERTURB'
     + //'ED FLUX HAVE DIFFERENT NUMBER OF UNKNOWS')
      CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYP)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(CMODUL,'(3A4)') (KCHAR(I),I=1,3)
      IF(CMODUL.NE.'TRIVAC') CALL XABORT('GPTFLU: TRIVAC TRACKING EXPE'
     + //'CTED.')
      LL4 = TRKPRM(11)
      NLF = TRKPRM(30)
      CALL LCMGET(IPSYS,'STATE-VECTOR',SYSPRM)
      IF( SYSPRM(1).NE.NGRP )CALL XABORT('GPTFLU: L_SYSTEM AND L_FLUX'
     + //'FOR UNPERTURBED STATE HAVE DIFFERENT NUMBER OF GROUPS')
      IF( SYSPRM(2).NE.LL4 )CALL XABORT('GPTFLU: UNPERTURBED SYSTEM A'
     + //'ND TRACKING OBJECTS HAVE DIFFERENT NUMBER OF LINEAR ORDER')
      ITY = SYSPRM(4)
      IF(ITY.EQ.13) LL4=LL4*NLF/2
      CALL LCMGET(IPGPT,'STATE-VECTOR',GPTPRM)
      IF( GPTPRM(1).NE.NGRP )CALL XABORT('GPTFLU: L_GPT AND L_FLUX HAV'
     + //'E DIFFERENT NUMBER OF GROUPS')
      IF( GPTPRM(2).NE.NUN )CALL XABORT('GPTFLU: L_GPT AND L_FLUX HAVE'
     + //' DIFFERENT NUMBER OF UNKNOWS')
*----
*  READ USER INPUT:
*----
  500 IPRINT=0
      IMPH=0
      IF(RECP) THEN
*        RECOVER EXISTING OPTIONS.
         CALL LCMGET(IPFLU,'STATE-VECTOR',FLUPRM)
         ICL1=FLUPRM(8)
         ICL2=FLUPRM(9)
         MAXINR=FLUPRM(11)
         MAXOUT=FLUPRM(12)
         NADI=FLUPRM(13)
         CALL LCMGET(IPFLU,'EPS-CONVERGE',EPSCON)
         EPSINR=EPSCON(1)
         EPSOUT=EPSCON(2)
      ELSE
*        DEFAULT OPTIONS.
         ICL1=3
         ICL2=3
         NADI=TRKPRM(33)
         MAXINR=0
         MAXOUT=200
         EPSINR=1.0E-2
         EPSOUT=1.0E-4
      ENDIF
      IF(GPTPRM(3).LE.GPTPRM(4)) THEN
         ADJ=.FALSE.
      ELSE
         ADJ=.TRUE.
      ENDIF
      REC=.FALSE.
  505 CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
  506 IF(ITYP.NE.3) CALL XABORT('GPTFLU: CHARACTER DATA EXPECTED.')
      IF(TEXT12.EQ.'EDIT') THEN
         CALL REDGET(ITYP,IPRINT,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTFLU: *IPRINT* MUST BE INTEGER')
         GO TO 505
      ELSEIF((TEXT12.EQ.'VAR1').OR.(TEXT12.EQ.'ACCE')) THEN
         CALL REDGET(ITYP,ICL1,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTFLU: INTEGER DATA EXPECTED.')
         CALL REDGET(ITYP,ICL2,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTFLU: INTEGER DATA EXPECTED.')
         GO TO 505
      ELSEIF(TEXT12.EQ.'IMPLICIT') THEN
         ADJ=.TRUE.
         GO TO 505
      ELSEIF(TEXT12.EQ.'EXPLICIT') THEN
         ADJ=.FALSE.
         GO TO 505
      ELSEIF(TEXT12.EQ.'RCVR-LAST') THEN
         REC=.TRUE.
         GO TO 505
      ELSEIF(TEXT12.EQ.'PREC') THEN
         CALL REDGET(ITYP,NITMA,EPSOUT,TEXT12,DFLOTT)
         IF(ITYP.NE.2) CALL XABORT('GPTFLU: REAL DATA EXPECTED.')
         GO TO 505
      ELSEIF((TEXT12.EQ.'EXTE').OR.(TEXT12.EQ.'MAX')) THEN
         CALL REDGET(ITYP,MAXOUT,FLOTT,TEXT12,DFLOTT)
         GO TO 505
      ELSEIF(TEXT12.EQ.'ADI') THEN
         CALL REDGET(ITYP,NADI,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTFLU: INTEGER DATA EXPECTED.')
         GO TO 505
      ELSEIF(TEXT12.EQ.'THER') THEN
         CALL REDGET(ITYP,NITMA,FLOTT,TEXT12,DFLOTT)
         MAXINR = NGRP*2
         IF(ITYP.EQ.1) THEN
            MAXINR=NITMA
         ELSEIF(ITYP.EQ.2) THEN
            EPSINR=FLOTT
         ELSE
            GO TO 506
         ENDIF 
      ELSEIF(TEXT12.EQ.'HIST') THEN
         CALL REDGET(ITYP,IMPH,FLOTT,TEXT12,DFLOTT)
         IF(ITYP.NE.1) CALL XABORT('GPTFLU: INTEGER DATA EXPECTED.')
         GO TO 505
      ENDIF
      IF(TEXT12.EQ.'FROM-TO') THEN
         CALL REDGET(ITYP,SRCFRM,FLOTT,TEXT12,DFLOTT)
         IF((ITYP.EQ.3).AND.(TEXT12.EQ.'ALL')) THEN
            SRCFRM=1
            IF(ADJ) THEN
               SRCTO=GPTPRM(4)
            ELSE
               SRCTO=GPTPRM(3)
            ENDIF
         ELSE
            IF(ITYP.NE.1) CALL XABORT('GPTFLU: INTEGER DATA EXPECTED.')
            CALL REDGET(ITYP,SRCTO,FLOTT,TEXT12,DFLOTT)
            IF(ITYP.NE.1) CALL XABORT('GPTFLU: INTEGER DATA EXPECTED.')
            IF(ADJ) THEN
               IF(SRCTO.GT.GPTPRM(4)) WRITE(6,*) 'THE NUMBER OF THE '//
     1         'SOURCE ',SRCTO,' IS GREATER THAN THE NUMBER OF CONST'//
     2         'RAINTS +1',GPTPRM(4)
            ELSE
               IF(SRCTO.GT.GPTPRM(3)) WRITE(6,*) 'THE NUMBER OF THE '//
     1         'SOURCE ',SRCTO,' IS GREATER THAN THE NUMBER OF VARIA'//
     2         'BLES',GPTPRM(3)
            ENDIF
            IF(SRCFRM.GT.SRCTO) CALL XABORT('GPTFLU:SCRFRM .GT. SCRTO')
         ENDIF
      ELSEIF(TEXT12.EQ.';') THEN
         GO TO 1000 
      ELSE
         WRITE(6,*) 'Your keyword is : ',TEXT12
         CALL XABORT('GPTFLU:"FROM-TO" or ";" EXPECTED')
      ENDIF
*----
*  RECOVER TRACKING INFORMATION
*----
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(RBASE,NEL,IVOL)
      CALL SETARA(IBASE,NEL,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
*----
*  RECOVER UNPERTURBED K-EFFECTIVE AND FLUXES.
*----
      CALL SETARA(RBASE,NUN*NGRP,IEVECT)
      CALL SETARA(RBASE,NUN*NGRP,IADECT)
      CALL SETARA(RBASE,NUN*NGRP,IEASS)
      CALL LCMGET(IPFLU,'K-EFFECTIVE',FKEFF)
      JPFLU1=LCMGID(IPFLU,'FLUX')
      JPFLU2=LCMGID(IPFLU,'AFLUX')
      DO 510 IGR=1,NGRP
      CALL LCMGDL(JPFLU1,IGR,RBASE(IEVECT+(IGR-1)*NUN))
      CALL LCMGDL(JPFLU2,IGR,RBASE(IADECT+(IGR-1)*NUN))
  510 CONTINUE
      CALL SETARA(RBASE,NUN*NGRP,IGRAD1)
      CALL SETARA(RBASE,NUN*NGRP,IGRAD2)
      CALL SETARA(RBASE,NUN*NGRP,IGAR1)
      CALL SETARA(RBASE,NUN*NGRP,IGAR2)
      CALL SETARA(RBASE,NUN*NGRP,IGAR3)
      CALL SETARA(RBASE,NUN,IWORK1)
      CALL SETARA(RBASE,NUN,IWORK2)
      CALL SETARA(RBASE,NUN*NGRP,ISOUR)
*----
*  RECOVER FIXED SOURCE AND SET INITIAL VALUE OF GPFLUX 
*----
      IF(ADJ) THEN
         JPFLUP=LCMLID(IPFLUP,'ADFLUX',SRCTO)
      ELSE
         JPFLUP=LCMLID(IPFLUP,'DFLUX',SRCTO)
      ENDIF
      DO 590 ISRC=SRCFRM,SRCTO
      IF(ADJ) THEN
         JPGPT=LCMGID(IPGPT,'ASOUR')
      ELSE
         JPGPT=LCMGID(IPGPT,'DSOUR')
      ENDIF
      CALL LCMLEL(JPGPT,ISRC,LENGT,ITYP)
      IF(LENGT.EQ.0) GO TO 590
      KPGPT=LCMGIL(JPGPT,ISRC)
      DO 520 IGR=1,NGRP
      CALL LCMGDL(KPGPT,IGR,RBASE(ISOUR+(IGR-1)*NUN))
  520 CONTINUE
      IF(REC.AND.(IMPH.EQ.0)) THEN
         CALL LCMLEL(JPFLUP,ISRC,LENGT,ITYP)
         IF(LENGT.EQ.0) THEN
            WRITE(TEXT12,'(I4,3H-TH)') ISRC
            CALL XABORT('GPTFLU: '//TEXT12//' GENERALIZED ADJOINT CANN'
     1      //'OT BE RECOVERED.')
         ENDIF
         KPFLUP=LCMGIL(JPFLUP,ISRC)
         DO 530 IGR=1,NGRP
         CALL LCMGDL(KPFLUP,IGR,RBASE(IEASS+(IGR-1)*NUN))
  530    CONTINUE
      ELSE
         DO 540 IGR=1,NGRP
         CALL XDRSET(RBASE(IEASS+(IGR-1)*NUN),NUN,1.0)
  540    CONTINUE
      ENDIF
*----
*  ADJOINT NEUTRON FLUX CALCULATION
*----
      IF(IPRINT.GE.2) WRITE(6,*) 'ISRC',ISRC
      IF(ADJ) THEN
         IF(IPRINT.GE.2) WRITE(6,*) 'implicit'
         CALL GPTAFL(IPTRK,IPSYS,IPFLUP,LL4,ITY,NUN,NGRP,ICL1,ICL2,
     1   IPRINT,IMPH,TITLE,EPSOUT,MAXINR,EPSINR,NADI,MAXOUT,FKEFF,
     2   RBASE(IEVECT),RBASE(IADECT),FKEFF2,RBASE(IEASS),RBASE(ISOUR),
     3   RBASE(IGRAD1),RBASE(IGRAD2),RBASE(IGAR1),RBASE(IGAR2),
     4   RBASE(IGAR3),RBASE(IWORK1),RBASE(IWORK2))
      ELSE
         IF(IPRINT.GE.2) WRITE(6,*) 'explicit'
         CALL GPTDFL(IPTRK,IPSYS,IPFLUP,LL4,ITY,NUN,NGRP,ICL1,ICL2,
     1   IPRINT,IMPH,TITLE,EPSOUT,MAXINR,EPSINR,NADI,MAXOUT,FKEFF,
     2   RBASE(IEVECT),RBASE(IADECT),FKEFF2,RBASE(IEASS),RBASE(ISOUR),
     3   RBASE(IGRAD1),RBASE(IGRAD2),RBASE(IGAR1),RBASE(IGAR2),
     4   RBASE(IGAR3),RBASE(IWORK1),RBASE(IWORK2))
      ENDIF
      CALL LCMPUT(IPFLUP,'K-EFFECTIVE',1,2,FKEFF2)
      KPFLUP=LCMLIL(JPFLUP,ISRC,NGRP)
      DO 550 IGR=1,NGRP
      CALL FLDTRI(IPTRK,NEL,NUN,RBASE(IEASS+(IGR-1)*NUN),IBASE(IMAT),
     1            RBASE(IVOL),IBASE(IIDL))
      CALL LCMPDL(KPFLUP,IGR,NUN,2,RBASE(IEASS+(IGR-1)*NUN))
  550 CONTINUE
  590 CONTINUE
      CALL RLSARA(RBASE(IWORK2))
      CALL RLSARA(RBASE(IWORK1))
      CALL RLSARA(RBASE(IGAR3))
      CALL RLSARA(RBASE(IGAR2))
      CALL RLSARA(RBASE(IGAR1))
      CALL RLSARA(RBASE(IGRAD2))
      CALL RLSARA(RBASE(IGRAD1))
      CALL RLSARA(RBASE(IEASS))
      CALL RLSARA(RBASE(IADECT))
      CALL RLSARA(RBASE(IEVECT))
      CALL RLSARA(RBASE(ISOUR))
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      GO TO 500
*----
*  END
*----
 1000 IF(ADJ) THEN
         FLUPRM(3)=1000
      ELSE
         FLUPRM(3)=100
      ENDIF
      FLUPRM(5)=SRCTO
      FLUPRM(6)=1
      CALL LCMPUT(IPFLUP,'STATE-VECTOR',NSTATE,1,FLUPRM)
      IF(IPRINT.GT.2) THEN
      DO 2010 I=1,NENTRY
         WRITE(6,*) '1 ',KENTRY(I),'  2 ',KENTRY(I)
         WRITE(6,*) '1 ',HENTRY(I),'  2 ',HENTRY(I)
 2010    CONTINUE
      ENDIF
      RETURN
      END
