*DECK KINB05
      SUBROUTINE KINB05(SGD,IELEM,ISPLH,LX,LL4,NBMIX,SIDE,MAT,IZGLOB,
     1 KN,F2,F3)
*
*-----------------------------------------------------------------------
*
*Purpose:
* multiplication of a matrix by a vector in Thomas-Raviart-Schneider
* (dual) finite element diffusion approximation (hexagonal geometry).
* Special version for Bivac.
*
*Copyright:
* Copyright (C) 2010 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
* SGD     mixture-ordered cross sections.
* IELEM   degree of the Lagrangian finite elements: =1 (linear);
*         =2 (parabolic); =3 (cubic); =4 (quartic).
* ISPLH   mesh-splitting index. Each hexagon is splitted into 3*ISPLH**2
*         lozenges.
* LX      number of hexagons in the W-X-Y plane.
* LL4     number of unknowns per group in Bivac.
* NBMIX   number of macro-mixtures.
* SIDE    side of the hexagons.
* MAT     mixture index per region.
* IZGLOB  hexagon index assigned to each lozenge.
* KN      element-ordered unknown list.
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IELEM,ISPLH,LX,LL4,NBMIX,MAT(LX),IZGLOB(LX*ISPLH**2),
     1 KN(LX*ISPLH**2,4+6*IELEM*(IELEM+1))
      REAL SGD(NBMIX),SIDE,F2(LL4),F3(LL4)
*----
*  ASSEMBLY OF A SYSTEM MATRIX.
*----
      NBLOS=LX*ISPLH**2
      TTTT=0.5D0*SQRT(3.D00)*SIDE*SIDE
      NUM=0
      DO 20 KEL=1,NBLOS
      IF(IZGLOB(KEL).EQ.0) GO TO 20
      NUM=NUM+1
      SIG=SGD(MAT(IZGLOB(KEL)))
      DO 10 K2=0,IELEM-1
      DO 10 K1=0,IELEM-1
      JND1=KN(NUM,1)+K2*IELEM+K1
      JND2=KN(NUM,2)+K2*IELEM+K1
      JND3=KN(NUM,3)+K2*IELEM+K1
      F3(JND1)=F3(JND1)+TTTT*SIG*F2(JND1)
      F3(JND2)=F3(JND2)+TTTT*SIG*F2(JND2)
   10 F3(JND3)=F3(JND3)+TTTT*SIG*F2(JND3)
   20 CONTINUE
      RETURN
      END
