*DECK KINBLM
      SUBROUTINE KINBLM(IPTRK,NBMIX,LDIM,SGD,F2,F3)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for the multiplication of a matrix by a vector. Special
* version for Bivac.
*
*Copyright:
* Copyright (C) 2010 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the tracking information.
* NBMIX   number of material mixtures.     
* LDIM    dimension of vectors F2 and F3.
* SGD     mixture-ordered cross sections.
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,NBMIX,LDIM
      REAL SGD(NBMIX),F2(LDIM),F3(LDIM)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      INTEGER IBASE(1),ISTATE(NSTATE)
      LOGICAL CYLIND
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER TRACKING INFORMATION.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NREG=ISTATE(1)
      NBMIX=ISTATE(4)
      ITYPE=ISTATE(6)
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
*----
*  ALGORITHM-DEPENDENT MULTIPLICATION
*----
      CALL XDRSET(F3,LDIM,0.0)
      ITYPE=ISTATE(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IHEX=ISTATE(7)
      IELEM=ISTATE(8)
      ICOL=ISTATE(9)
      ISPLH=ISTATE(10)
      LL4=ISTATE(11)
      LX=ISTATE(12)
      LY=ISTATE(13)
      NVD=ISTATE(17)
      IF(LL4.GT.LDIM) CALL XABORT('KINBLM: LDIM OVERFLOW.')
      CALL SETARA(BASE,LX*LY,IXX)
      CALL SETARA(BASE,LX*LY,IDD)
      IF(ITYPE.EQ.8) THEN
        CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
        CALL LCMGET(IPTRK,'XX',BASE(IXX))
        CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      IF((IHEX.EQ.0).AND.(IELEM.LT.0)) THEN
*       --- PRIMAL FINITE ELEMENTS (CARTESIAN)
        CALL LCMSIX(IPTRK,'BIVCOL',1)
        CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
        CALL SETARA(BASE,LC*LC,IR)
        CALL SETARA(BASE,LC*LC,IRS)
        CALL LCMGET(IPTRK,'R',BASE(IR))
        CALL LCMGET(IPTRK,'RS',BASE(IRS))
        CALL LCMSIX(IPTRK,' ',2)
        CALL KINB01(MAXKN,SGD,CYLIND,NREG,LL4,NBMIX,BASE(IXX),
     1  BASE(IDD),IBASE(IMAT),IBASE(IKN),BASE(IVOL),LC,BASE(IR),
     2  BASE(IRS),F2,F3)
        CALL RLSARA(BASE(IRS))
        CALL RLSARA(BASE(IR))
      ELSE IF((IHEX.EQ.0).AND.(IELEM.GT.0)) THEN
*       --- MIXED-DUAL FINITE ELEMENTS (CARTESIAN)
        CALL KINB02(SGD,IELEM,NREG,LL4,NBMIX,IBASE(IMAT),IBASE(IKN),
     1  BASE(IVOL),F2,F3)
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
*       --- MESH CORNER FINITE DIFFERENCES (HEXAGONAL)
        CALL LCMSIX(IPTRK,'BIVCOL',1)
        CALL LCMIOF(IPTRK,'RH',IRH)
        CALL LCMIOF(IPTRK,'RT',IRT)
        CALL LCMSIX(IPTRK,' ',2)
        IF(ISPLH.EQ.1) THEN
          NELEM=MAXKN/7
        ELSE
          NELEM=MAXKN/4
        ENDIF
        CALL KINB03(MAXKN,MAXQF,SGD,NREG,LL4,ISPLH,NELEM,NBMIX,
     1  IBASE(IMAT),IBASE(IKN),BASE(IQFR),BASE(IVOL),BASE(IRH),
     2  BASE(IRT),F2,F3)
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8).AND.(ICOL.EQ.4)) THEN
*       --- MESH CENTERED FINITE DIFFERENCES FOR HEXAGONS
        CALL KINB04(MAXKN,MAXQF,SGD,NREG,LL4,ISPLH,NBMIX,IBASE(IMAT),
     1  IBASE(IKN),BASE(IQFR),BASE(IVOL),F2,F3)
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
*       --- THOMAS-RAVIART-SCHNEIDER METHOD (HEXAGONAL)
        CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
        CALL KINB05(SGD,IELEM,ISPLH,LX,LL4,NBMIX,SIDE,IBASE(IMAT),
     1  IBASE(IZGLOB),IBASE(IKN),F2,F3)
      ELSE
        CALL XABORT('KINBLM: TRACKING NOT AVAILABLE.')
      ENDIF
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IXX))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
