*DECK KINDRV
      SUBROUTINE KINDRV(NEN,KEN,CMOD,NGR,NBM,NBFIS,NDG,NAN,NLF,ITY,NEL,
     1 LL4,NUN,NUP,TTF,TTP,DT,IMPH,ICL1,ICL2,NADI,MAXOUT,EPSOUT,MAXINR,
     2 EPSINR,IFL,IPR,LDIF,IMPX,EVECT,PC,SGD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver to perform the space-time kinetics calculations.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input
* NEN     number of LCM objects used in the module.
* KEN     addresses of LCM objects: (1) L_KINET; (2) L_MACROLIB;
*         (3) L_TRACK; (4) L_SYSTEM; (5) L_MACROLIB.
* CMOD    name of the assembly door (BIVAC or TRIVAC).
* NGR     number of energy groups.
* NBM     number of material mixtures.
* NBFIS   number of fissile isotopes.
* NDG     number of delayed-neutron groups.
* NAN     number of Legendre orders for scattering.
* NLF     number of Legendre orders for fluxes.
* ITY     type of finite elements and tracking.
* NEL     total number of finite elements.
* LL4     number of flux unknowns per energy group.
* NUN     total number of unknowns per energy group.
* NUP     number of precursor unknowns per delayed group.
* TTF     value of theta-parameter for fluxes.
* TTP     value of theta-parameter for precursors.
* DT      current time increment.
* IMPH    management of convergence histogram.
* ICL1    number of free iretations in one cycle of the inverse power
*         method
* ICL2    number of accelerated iterations in one cycle
* NADI    number of inner adi iterations per outer iteration
* MAXOUT  maximum number of outer iterations
* EPSOUT  convergence criteria for the flux
* MAXINR  maximum number of thermal iterations.
* EPSINR  thermal iteration epsilon.
* IFL     temporal integration scheme for fluxes.
* IPR     temporal integration scheme for precursors.
* LDIF    =.true. to use 1/3D as P1 scattering cross sections.
* IMPX    printing parameter (=0 for no print).
*
*Parameters: scratch
* EVECT   unknown array for fluxes.
* PC      initial precursor unknown array.
* SGD     delayed nu*fission macroscopic x-sections/keff.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NEN,KEN(NEN),NGR,NBM,NBFIS,NDG,NAN,NLF,ITY,NEL,LL4,NUN,
     1 NUP,IMPH,ICL1,ICL2,NADI,MAXOUT,MAXINR,IFL,IPR,IMPX
      REAL TTF,TTP,DT,EPSOUT,EPSINR,EVECT(NUN,NGR),PC(NUP,NDG,NBFIS),
     1 SGD(NBM,NBFIS,NGR,NDG)
      CHARACTER CMOD*12
      LOGICAL LDIF
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOS=6)
      INTEGER MAT(NEL),IDL(NEL),IDLPC(NEL)
      REAL VOL(NEL),PDC(NDG),PMAX(NDG,NBFIS)
      COMMON BASE(1)
      LOGICAL LNUD,LCHD
*----
*  RECOVER INFORMATION
*----
      IF(IMPX.GT.0) THEN
        CALL KDRCPU(TK1)
        TA1=TK1
        WRITE(IOS,1001)
      ENDIF
      CALL LCMGET(KEN(1),'E-KEFF',EVL)
      CALL LCMGET(KEN(1),'LAMBDA-D',PDC)
      CALL LCMGET(KEN(1),'E-IDLPC',IDLPC)
      CALL LCMGET(KEN(3),'VOLUME',VOL)
      CALL LCMGET(KEN(3),'MATCOD',MAT)
      CALL LCMGET(KEN(3),'KEYFLX',IDL)
*----
*  RECOVER CROSS SECTIONS (BEGINNING-OF-STEP)
*----
      CALL SETARA(BASE,NDG,IDNF)
      CALL SETARA(BASE,NGR*NDG,IDNS)
      CALL LCMLEN(KEN(1),'BETA-D',LEN,ITYL)
      LNUD=(LEN.EQ.NDG)
      IF(LNUD) CALL LCMGET(KEN(1),'BETA-D',BASE(IDNF))
      CALL LCMLEN(KEN(1),'CHI-D',LEN,ITYL)
      LCHD=(LEN.EQ.NGR*NDG)
      IF(LCHD) CALL LCMGET(KEN(1),'CHI-D',BASE(IDNS))
      CALL SETARA(BASE,NBM*NGR,IOVR)
      CALL SETARA(BASE,NBM*NBFIS*NGR,ICHI)
      CALL SETARA(BASE,NBM*NBFIS*NGR*NDG,ICHD)
      CALL SETARA(BASE,NBM*NBFIS*NGR,ISGF)
      CALL SETARA(BASE,NBM*NBFIS*NGR*NDG,ISGO)
      IF(NEN.EQ.4) THEN
        IPMAC=KEN(2)
        IPSYS=KEN(4)
      ELSE IF(NEN.EQ.6) THEN
        IPMAC=KEN(5)
        IPSYS=KEN(6)
      ENDIF
      CALL KINXSD(IPMAC,NGR,NBM,NBFIS,NDG,EVL,DT,BASE(IDNF),BASE(IDNS),
     1 LNUD,LCHD,BASE(IOVR),BASE(ICHI),BASE(ICHD),BASE(ISGF),BASE(ISGO))
*----
*  COMPUTE THE SOURCE TERM
*----
      LL4B=LL4
      IF((ITY.EQ.11).OR.(ITY.EQ.13)) LL4B=LL4*NLF/2
      CALL SETARA(BASE,NGR*NUN,ISRC)
      CALL SETARA(BASE,NGR*NUN,IEVEC)
      CALL LCMGET(KEN(1),'E-PREC',PC)
      CALL LCMGET(KEN(1),'E-VECTOR',BASE(IEVEC))
      CALL SETARA(BASE,LL4B,IWORK1)
      CALL SETARA(BASE,NBM,IWORK2)
      CALL KINSRC(KEN(3),IPSYS,CMOD,IMPX,IFL,IPR,NGR,NBM,NBFIS,
     1 NDG,ITY,NEL,LL4B,NUN,NUP,MAT,VOL,PDC,TTF,TTP,DT,BASE(IOVR),
     2 BASE(ICHI),BASE(ICHD),BASE(ISGF),BASE(ISGO),BASE(IEVEC),
     3 PC,BASE(ISRC),BASE(IWORK1),BASE(IWORK2))
      CALL RLSARA(BASE(IWORK2))
      CALL RLSARA(BASE(IWORK1))
      CALL RLSARA(BASE(IEVEC))
*----
*  RECOVER CROSS SECTIONS (END-OF-STEP)
*----
      CALL KINXSD(KEN(2),NGR,NBM,NBFIS,NDG,EVL,DT,BASE(IDNF),BASE(IDNS),
     1 LNUD,LCHD,BASE(IOVR),BASE(ICHI),BASE(ICHD),BASE(ISGF),SGD)
      CALL RLSARA(BASE(IDNS))
      CALL RLSARA(BASE(IDNF))
*----
*  RECOVER THE BEGINNING-OF-STEP FLUX
*----
      IF(IMPX.GT.0)THEN
        CALL KDRCPU(TA2)
        WRITE(IOS,1002) TA2-TA1
        WRITE(IOS,1003)
      ENDIF
      CALL SETARA(BASE,NGR*NUN,IPHO)
      CALL LCMGET(KEN(1),'E-VECTOR',EVECT)
      CALL LCMGET(KEN(1),'E-VECTOR',BASE(IPHO))
*----
*  COMPUTE THE FLUX SOLUTION
*----
      CALL SETARA(BASE,NUN*NGR,IGRAD1)
      CALL SETARA(BASE,NUN*NGR,IGRAD2)
      CALL SETARA(BASE,NUN*NGR,IGAR1)
      CALL SETARA(BASE,NUN*NGR,IGAR2)
      CALL SETARA(BASE,NUN*NGR,IGAR3)
      CALL SETARA(BASE,LL4B,IWORK1)
      CALL SETARA(BASE,LL4B,IWORK2)
      CALL SETARA(BASE,NEL,IWORK3)
      IF(CMOD.EQ.'BIVAC')THEN
        CALL KINSLB(KEN(3),KEN(4),KEN(1),LL4B,ITY,NUN,NGR,IFL,IPR,NBM,
     1  NBFIS,NDG,NEL,ICL1,ICL2,IMPX,IMPH,TITR,EPSOUT,MAXINR,EPSINR,
     2  MAXOUT,MAT,VOL,PDC,TTF,TTP,DT,BASE(IOVR),BASE(ICHI),BASE(ICHD),
     3  BASE(ISGF),SGD,EVECT,BASE(ISRC),BASE(IGRAD1),BASE(IGRAD2),
     4  BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IWORK1),BASE(IWORK2),
     5  BASE(IWORK3))
      ELSEIF(CMOD.EQ.'TRIVAC')THEN
        CALL KINSLT(KEN(3),KEN(4),KEN(1),LL4B,ITY,NUN,NGR,IFL,IPR,NBM,
     1  NBFIS,NDG,NEL,ICL1,ICL2,IMPX,IMPH,TITR,EPSOUT,MAXINR,EPSINR,
     2  NADI,MAXOUT,MAT,VOL,PDC,TTF,TTP,DT,BASE(IOVR),BASE(ICHI),
     3  BASE(ICHD),BASE(ISGF),SGD,EVECT,BASE(ISRC),BASE(IGRAD1),
     4  BASE(IGRAD2),BASE(IGAR1),BASE(IGAR2),BASE(IGAR3),BASE(IWORK1),
     5  BASE(IWORK2),BASE(IWORK3))
      ENDIF
      CALL RLSARA(BASE(IWORK3))
      CALL RLSARA(BASE(IWORK2))
      CALL RLSARA(BASE(IWORK1))
      CALL RLSARA(BASE(IGAR3))
      CALL RLSARA(BASE(IGAR2))
      CALL RLSARA(BASE(IGAR1))
      CALL RLSARA(BASE(IGRAD2))
      CALL RLSARA(BASE(IGRAD1))
      CALL RLSARA(BASE(ISRC))
      DO 10 IGR=1,NGR
      IF(CMOD.EQ.'BIVAC')THEN
        CALL FLDBIV(KEN(3),NEL,NUP,EVECT(1,IGR),MAT,VOL,IDL)
      ELSEIF(CMOD.EQ.'TRIVAC')THEN
        CALL FLDTRI(KEN(3),NEL,NUP,EVECT(1,IGR),MAT,VOL,IDL)
      ENDIF
   10 CONTINUE
      CALL LCMPUT(KEN(1),'E-VECTOR',NGR*NUN,2,EVECT)
*----
*  COMPUTE THE PRECURSOR SOLUTION
*----
      CALL KINPRC(KEN(3),KEN(4),CMOD,NGR,NBM,NBFIS,NDG,NEL,LL4,NUN,NUP,
     1 MAT,VOL,IDLPC,EVECT,BASE(IPHO),SGD,BASE(ISGO),PDC,DT,TTP,PC,IPR,
     2 IMPX)
      CALL LCMPUT(KEN(1),'E-PREC',NDG*NUP*NBFIS,2,PC)
*----
*  FIND THE MAXIMUM FLUX VALUE
*----
      FMAX=0.0
      IDMX=0
      DO 20 IGR=1,NGR
      DO 20 IEL=1,NEL
      IND=IDL(IEL)
      IF(IND.EQ.0) GO TO 20
      IF(ABS(EVECT(IND,IGR)).GT.FMAX) THEN
        FMAX=EVECT(IND,IGR)
        IDMX=IEL
        IGMX=IGR
      ENDIF
   20 CONTINUE
      IF(IDMX.EQ.0) CALL XABORT('KINDRV: UNABLE TO SET FMAX.')
      IND=IDLPC(IDMX)
      IF(IND.EQ.0) CALL XABORT('KINDRV: UNABLE TO SET PMAX.')
      DO 30 IFIS=1,NBFIS
      DO 30 IDG=1,NDG
   30 PMAX(IDG,IFIS)=PC(IND,IDG,IFIS)
      IF(IMPX.GT.0) WRITE(IOS,1004) FMAX,IDMX,IGMX
      IF(IMPX.GT.0) THEN
        CALL KDRCPU(TK2)
        WRITE(IOS,1005)TK2-TK1
      ENDIF
      CALL LCMPUT(KEN(1),'CTRL-FLUX',1,2,FMAX)
      CALL LCMPUT(KEN(1),'CTRL-PREC',NDG*NBFIS,2,PMAX)
      CALL LCMPUT(KEN(1),'CTRL-IDL',1,1,IDMX)
      CALL LCMPUT(KEN(1),'CTRL-IGR',1,1,IGMX)
*----
*  RELEASE ALLOCATED MEMORY
*----
      CALL RLSARA(BASE(IPHO))
      CALL RLSARA(BASE(ISGO))
      CALL RLSARA(BASE(ISGF))
      CALL RLSARA(BASE(ICHD))
      CALL RLSARA(BASE(ICHI))
      CALL RLSARA(BASE(IOVR))
      RETURN
*
 1001 FORMAT(/1X,'=> ASSEMBLY OF THE SYSTEM MATRICES'/)
 1002 FORMAT(/1X,'TOTAL CPU TIME USED FOR THE ASSEMBLING',
     1 1X,'OF ALL SYSTEM MATRICES =',F6.3/)
 1003 FORMAT(/1X,'=> COMPUTING THE KINETICS SOLUTION'/)
 1004 FORMAT(/1X,'CONTROLLING PARAMETERS:',2X,'MAX-VA',
     1 'L',1X,1PE12.5,3X,'IDL #',I5.5,3X,'IGR #',I2.2/)
 1005 FORMAT(/1X,'TOTAL CPU TIME USED FOR KINETICS CALC',
     1 'ULATIONS =',F10.3//1X,'=> SPACE-TIME',1X,
     2 'KINETICS CALCULATION IS DONE.')
      END
