*DECK KINRD1
      SUBROUTINE KINRD1(NEN,KEN,CMOD,NGR,NBM,NBFIS,NEL,NUN,NDG)
*
*-----------------------------------------------------------------------
*
*Purpose:
* read and validate the module options from the input file.
*
*Copyright:
* Copyright (C) 2008 Ecole Polytechnique de Montreal.
*
*Author(s): D. Sekki
*
*Parameters: input/output
* NEN    number of LCM objects used in the module.
* KEN    addresses of LCM objects: (1) L_KINET; (2) L_MACROLIB;
*        (3) L_TRACK; (4) L_SYSTEM; (6) L_FLUX.
* CMOD   name of the assembly door (BIVAC or TRIVAC).
* NGR    number of energy groups.
* NBM    number of material mixtures.
* NBFIS  number of fissile isotopes.
* NEL    total number of finite elements.
* NUN    total number of unknowns per energy group.
* NDG    number of delayed-neutron groups (=0 if not in macrolib).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NEN,KEN(NEN),NGR,NBM,NBFIS,NEL,NUN,NDG
      CHARACTER CMOD*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOS=6)
      INTEGER ISTATE(NSTATE),MAT(NEL),IDLPC(NEL)
      DOUBLE PRECISION DFLOT
      COMMON BASE(1)
      CHARACTER TEXT*12
      LOGICAL LNUD,LCHD,LLAD,LPRIMA
*----
*  READ THE INPUT DATA
*----
      IMPX=1
      LNUD=.FALSE.
      LCHD=.FALSE.
      LLAD=.FALSE.
      FNORM=1.0
   10 CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
      IF(ITYP.NE.3)CALL XABORT('@KINRD1: CHARACTER DATA EXPECTED(1).')
      IF(TEXT.EQ.';')THEN
        GOTO 60
      ELSEIF(TEXT.EQ.'EDIT') THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@KINRD1: INTEGER FOR EDIT EXPECTED.')
        IMPX=MAX(0,NITMA)
        IF(IMPX.GT.9)WRITE(IOS,1001)
      ELSEIF(TEXT.EQ.'NGRP') THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@KINRD1: INTEGER FOR NGRP EXPECTED.')
        IF(NGR.NE.NITMA)CALL XABORT('@KINRD1: INVALID INPUT FOR NGRP.')
      ELSEIF(TEXT.EQ.'NDEL') THEN
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.1)CALL XABORT('@KINRD1: INTEGER FOR NDEL EXPECTED.')
        IF(NDG.EQ.0)THEN
          NDG=NITMA
        ELSEIF(NDG.NE.NITMA)THEN
          CALL XABORT('@KINRD1: INVALID INPUT FOR NDEL NUMBER.')
        ENDIF
      ELSEIF(TEXT.EQ.'BETA')THEN
        LNUD=.TRUE.
        CALL SETARA(BASE,NDG,IDNF)
        DO 20 IDG=1,NDG
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@KINRD1: REAL DATA EXPECTED(1).')
        IF(FLOT.LE.0.)CALL XABORT('@KINRD1: INVALID BETA VALUE.')
   20   BASE(IDNF+IDG-1)=FLOT
      ELSEIF(TEXT.EQ.'LAMBDA')THEN
        LLAD=.TRUE.
        CALL SETARA(BASE,NDG,IPD)
        DO 30 IDG=1,NDG
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@KINRD1: REAL DATA EXPECTED(2).')
        IF(FLOT.LE.0.)CALL XABORT('@KINRD1: INVALID LAMBDA VALUE.')
   30   BASE(IPD+IDG-1)=FLOT
      ELSEIF(TEXT.EQ.'CHID')THEN
        LCHD=.TRUE.
        CALL SETARA(BASE,NDG*NGR,IDNS)
        IC=0
        DO 50 JGR=1,NGR
        DO 50 IDG=1,NDG
        CALL REDGET(ITYP,NITMA,FLOT,TEXT,DFLOT)
        IF(ITYP.NE.2)CALL XABORT('@KINRD1: REAL DATA EXPECTED(3).')
        IC=IC+1
   50   BASE(IDNS+IC-1)=FLOT
      ELSEIF(TEXT.EQ.'NORM')THEN
        CALL REDGET(ITYP,NITMA,FNORM,TEXT,DFLOT)
        IF((ITYP.EQ.3).AND.(TEXT.EQ.'MAX')) THEN
          FNORM=0.0
        ELSEIF(ITYP.NE.2) THEN
          CALL XABORT('@KINRD1: ''MAX'' OR REAL DATA EXPECTED(4)')
        ENDIF
      ELSE
        CALL XABORT('@KINRD1: INVALID KEYWORD '//TEXT//'.')
      ENDIF
      GO TO 10
   60 IF(NEN.NE.5)CALL XABORT('@KINRD1: INVALID NUMBER'
     1 //' OF MODULE PARAMETERS.')
      IF(IMPX.GT.9)WRITE(IOS,1002)
*----
*  RECOVER DELAYED NEUTRON DATA FROM MICROLIB
*----
      IF(.NOT.LNUD) THEN
        CALL SETARA(BASE,NDG,IDNF)
        CALL LCMLEN(KEN(2),'BETA-D',LEN,ITLCM)
        IF(LEN.GT.0) CALL LCMGET(KEN(2),'BETA-D',BASE(IDNF))
      ENDIF
      IF(.NOT.LLAD) THEN
        CALL SETARA(BASE,NDG,IPD)
        CALL LCMLEN(KEN(2),'LAMBDA-D',LEN,ITLCM)
        IF(LEN.EQ.0)CALL XABORT('@KINRD1: MISSING DATA FOR THE PRECURS'
     1  //'OR DECAY CONSTANTS.')
        CALL LCMGET(KEN(2),'LAMBDA-D',BASE(IPD))
      ENDIF
      IF(.NOT.LCHD) CALL SETARA(BASE,NDG*NGR,IDNS)
*----
*  RECOVER THE INITIAL STATE
*----
      IF(IMPX.GT.0)WRITE(IOS,1003)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(KEN(3),'STATE-VECTOR',ISTATE)
      LL4=ISTATE(11)
      NUP=LL4
      IF(CMOD.EQ.'BIVAC')THEN
        IELEM=ISTATE(8)
        NLF=ISTATE(14)
        LPRIMA=(IELEM.LT.0)
      ELSEIF(CMOD.EQ.'TRIVAC')THEN
        IELEM=ISTATE(9)
        ICHX=ISTATE(12)
        NLF=ISTATE(30)
        LPRIMA=(ICHX.EQ.1)
        IF(ICHX.EQ.2) NUP=ISTATE(25)
      ENDIF
      IF(LPRIMA) THEN
        CALL LCMGET(KEN(3),'MATCOD',MAT)
        DO 70 K=1,NEL
         IF(MAT(K).EQ.0) THEN
            IDLPC(K)=0
         ELSE
            NUP=NUP+1
            IDLPC(K)=NUP
         ENDIF
   70   CONTINUE
      ELSE
        CALL LCMGET(KEN(3),'KEYFLX',IDLPC)
      ENDIF
      IF(IMPX.GT.0) WRITE(IOS,1004) NEL,NUN,NUP,CMOD
      IF(LL4*NLF/2.GT.NUN)
     1 CALL XABORT('@KINRD1: INVALID NUMBER OF UNKNOWNS.')
      CALL SETARA(BASE,NUN*NGR,IEV)
      CALL SETARA(BASE,NUP*NDG,IPC)
      CALL SETARA(BASE,NBM*NBFIS*NGR*NDG,ISGD)
      CALL KINST1(NEN,KEN,CMOD,NGR,NBM,NBFIS,NDG,NEL,NUN,LL4,NUP,IDLPC,
     1 FNORM,BASE(IDNF),BASE(IDNS),BASE(IPD),BASE(IEV),BASE(IPC),LNUD,
     2 LCHD,IMPX,BASE(ISGD))
      CALL RLSARA(BASE(ISGD))
      CALL RLSARA(BASE(IPC))
      CALL RLSARA(BASE(IEV))
      CALL RLSARA(BASE(IDNS))
      CALL RLSARA(BASE(IPD))
      CALL RLSARA(BASE(IDNF))
      RETURN
*
 1001 FORMAT(/1X,'KINRD1: READING DATA FROM INPUT FILE')
 1002 FORMAT(1X,'KINRD1: THE INPUT DATA HAVE BEEN READ.')
 1003 FORMAT(/1X,'RECOVERING THE INITIAL STEADY-STATE'/)
 1004 FORMAT(1X,'TOTAL NUMBER OF ELEMENTS',1X,I6/1X,'NU',
     1 'MBER OF FLUX UNKNOWNS PER ENERGY GROUP',1X,I6/1X,
     2 'NUMBER OF PRECURSOR UNKNOWNS PER DELAYED GROUP',
     3 1X,I6/1X,'USING TRACKING TYPE:',1X,A6)
      END
