*DECK KINTLM
      SUBROUTINE KINTLM(IPTRK,NBMIX,LDIM,SGD,F2,F3)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for the multiplication of a matrix by a vector. Special
* version for Trivac.
*
*Copyright:
* Copyright (C) 2010 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the tracking information.
* NBMIX   number of material mixtures.     
* LDIM    dimension of vectors F2 and F3.
* SGD     mixture-ordered cross sections.
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,NBMIX,LDIM
      REAL SGD(NBMIX),F2(LDIM),F3(LDIM)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      INTEGER IBASE(1),ISTATE(NSTATE)
      LOGICAL CYLIND,CHEX
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER TRACKING INFORMATION.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NREG=ISTATE(1)
      NBMIX=ISTATE(4)
      ITYPE=ISTATE(6)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
*----
*  ALGORITHM-DEPENDENT MULTIPLICATION
*----
      CALL XDRSET(F3,LDIM,0.0)
      ITYPE=ISTATE(6)
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IHEX=ISTATE(7)
      IELEM=ISTATE(9)
      ICOL=ISTATE(10)
      LL4=ISTATE(11)
      ICHX=ISTATE(12)
      IF(ICHX.EQ.2) LL4=ISTATE(25)
      ISPLH=ISTATE(13)
      LX=ISTATE(14)
      LY=ISTATE(15)
      LZ=ISTATE(16)
      NVD=ISTATE(34)
      IF(LL4.GT.LDIM) CALL XABORT('KINTLM: LDIM OVERFLOW.')
      CALL SETARA(BASE,LX*LY*LZ,IXORZ)
      CALL SETARA(BASE,LX*LY*LZ,IDD)
      IF(CHEX) THEN
        CALL LCMGET(IPTRK,'ZZ',BASE(IXORZ))
        CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
        CALL LCMGET(IPTRK,'XX',BASE(IXORZ))
        CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      IF((.NOT.CHEX).AND.(ICHX.EQ.1)) THEN
*       --- MIXED-PRIMAL FINITE ELEMENTS (CARTESIAN)
        CALL LCMSIX(IPTRK,'BIVCOL',1)
        CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
        CALL LCMIOF(IPTRK,'T',IT)
        CALL LCMIOF(IPTRK,'TS',ITS)
        CALL LCMSIX(IPTRK,' ',2)
        CALL KINT01(MAXKN,SGD,CYLIND,NREG,LL4,NBMIX,BASE(IXORZ),
     1  BASE(IDD),IBASE(IMAT),IBASE(IKN),BASE(IVOL),LC,BASE(IT),
     2  BASE(ITS),F2,F3)
      ELSEIF((.NOT.CHEX).AND.(ICHX.GE.2)) THEN
*       --- DUAL FINITE ELEMENTS (CARTESIAN)
        CALL SETARA(BASE,NREG,IGAR)
        CALL KINT02(MAXKN,SGD,IELEM,ICHX,IDIM,NREG,LL4,NBMIX,
     1  IBASE(IMAT),IBASE(IKN),BASE(IVOL),F2,F3,BASE(IGAR))
        CALL RLSARA(BASE(IGAR))
      ELSEIF(CHEX.AND.(ICHX.EQ.1)) THEN
*       --- MESH CORNER FINITE DIFFERENCES (HEXAGONAL)
        CALL LCMSIX(IPTRK,'BIVCOL',1)
        CALL LCMIOF(IPTRK,'R',IR)
        CALL LCMIOF(IPTRK,'RH',IRH)
        CALL LCMIOF(IPTRK,'RT',IRT)
        CALL LCMSIX(IPTRK,' ',2)
        CALL KINT03(MAXKN,ISPLH,NBMIX,NREG,LL4,SGD,SIDE,BASE(IXORZ),
     1  BASE(IVOL),IBASE(IMAT),IBASE(IKN),BASE(IR),BASE(IRH),BASE(IRT),
     2  F2,F3)
      ELSEIF(CHEX.AND.(ICHX.EQ.2)) THEN
*       --- DUAL (THOMAS-RAVIART-SCHNEIDER) FINITE ELEMENT METHOD.
        CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
        CALL LCMIOF(IPTRK,'FRZ',IFRZ)
        NBLOS=LX*LZ*ISPLH**2
        CALL KINT04(IELEM,NBMIX,LX,LZ,LL4,NBLOS,IBASE(IMAT),SIDE,
     1  BASE(IXORZ),BASE(IFRZ),SGD,IBASE(IKN),BASE(IZGLOB),F2,F3)
      ELSE IF(CHEX.AND.(ICHX.EQ.3).AND.(ISPLH.EQ.1)) THEN
*       --- MESH CENTERED FINITE DIFFERENCES (HEXAGONAL)
        CALL KINT05(NBMIX,NREG,LL4,SGD,BASE(IVOL),IBASE(IMAT),F2,F3)
      ELSE IF(CHEX.AND.(ICHX.EQ.3).AND.(ISPLH.GT.1)) THEN
*       --- MESH CENTERED FINITE DIFFERENCES (HEXAGONAL)
        CALL LCMIOF(IPTRK,'IPW',IIPW)
        CALL KINT06(ISPLH,NBMIX,NREG,LL4,BASE(IVOL),IBASE(IMAT),SGD,
     1  IBASE(IKN),IBASE(IIPW),F2,F3)
      ELSE
        CALL XABORT('KINTLM: TRACKING NOT AVAILABLE.')
      ENDIF
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IXORZ))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
