*DECK MTLDLF
      SUBROUTINE MTLDLF(NAMP,IPTRK,IPSYS,ITY,IMPX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* LCM driver for the L-D-L(t) factorization of a symmetric matrix.
* The factorized matrix is stored on LCM under name 'I'//NAMP.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): Alain Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* ITY     type of coefficient matrix (1: Bivac; 2: classical Trivac;
*         3: Thomas-Raviart).
* IMPX    print flag (equal to zero for no print).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*12
      INTEGER IPTRK,IPSYS,ITY,IMPX
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,NPREF=5)
      CHARACTER HIN*12,HOUT*12,PREFIX(5)*2,NAMLCM*12,NAMMY*12
      LOGICAL EMPTY,LCM
      INTEGER IBASE(1),ITP(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
      DATA (PREFIX(I),I=1,NPREF)/'  ','W ','X ','Y ','Z '/
*
      IF(ITY.EQ.1) THEN
*        BIVAC TRACKING.
         CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
         ISEG=0
	 NLF=ITP(14)
      ELSE
*        CLASSICAL TRIVAC TRACKING.
         CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
         ISEG=ITP(17)
	 NLF=ITP(30)
      ENDIF
*
      DO 30 IS=1,NPREF
      IF(PREFIX(IS).EQ.'  ') THEN
         HIN=NAMP
         HOUT='I'//NAMP
      ELSE
         HIN=PREFIX(IS)//NAMP
         HOUT=PREFIX(IS)(:1)//'I'//NAMP
      ENDIF
*----
*  PERFORM FACTORIZATION OF MATRICES
*----
      CALL LCMLEN(IPSYS,HIN,ILENG,ITYLCM)
      IF(ILENG.GT.0) THEN
         IF(ISEG.EQ.0) THEN
            CALL LCMLEN(IPTRK,'MU'//PREFIX(IS),LMU,ITYLCM)
            CALL SETARA(IBASE,LMU,IMU)
            CALL LCMGET(IPTRK,'MU'//PREFIX(IS),IBASE(IMU))
         ELSE
            CALL LCMLEN(IPTRK,'MUV'//PREFIX(IS),LMU,ITYLCM)
            CALL SETARA(IBASE,LMU,IMU)
            CALL LCMGET(IPTRK,'MUV'//PREFIX(IS),IBASE(IMU))
         ENDIF
         ILEN=IBASE(IMU+LMU-1)
	 IF(NLF.GT.0) ILEN=ILEN*NLF/2
         IF(IMPX.GT.0) THEN
            CALL LCMINF(IPSYS,NAMLCM,NAMMY,EMPTY,ILONG,LCM)
            WRITE(6,'(/30H MTLDLF: FACTORIZATION OF LCM ,
     1      8HMATRIX '',A12,23H''. CREATION OF MATRIX '',A12,
     2      14H'' LOCATED IN '',A12,2H''.)') HIN,HOUT,NAMLCM
         ENDIF
         CALL SETARA(BASE,ILENG,IOF2)
         CALL LCMGET(IPSYS,HIN,BASE(IOF2))
         IF(ISEG.EQ.0) THEN
            IF(ILEN.NE.ILENG) CALL XABORT('MTLDLF: INCONSISTENT INF'
     1      //'ORMATION ON LCM (1).')
            IF(NLF.EQ.0) THEN
               CALL ALLDLF(LMU,BASE(IOF2),IBASE(IMU))
            ELSE
               IOF=1
               DO 10 IL=0,NLF-2,2
               CALL ALLDLF(LMU,BASE(IOF2+IOF-1),IBASE(IMU))
   10          IOF=IOF+IBASE(IMU+LMU-1)
            ENDIF
         ELSE
            IF(ISEG*ILEN.NE.ILENG) CALL XABORT('MTLDLF: INCONSISTEN'
     1      //'T INFORMATION ON LCM (2).')
            CALL LCMLEN(IPTRK,'NBL'//PREFIX(IS),LON,ITYLCM)
            CALL SETARA(IBASE,LON,INBL)
            CALL SETARA(IBASE,LON,ILBL)
            CALL LCMGET(IPTRK,'NBL'//PREFIX(IS),IBASE(INBL))
            CALL LCMGET(IPTRK,'LBL'//PREFIX(IS),IBASE(ILBL))
            CALL SETARA(BASE,ISEG,IT)
            IF(NLF.EQ.0) THEN
               CALL ALVDLF(BASE(IOF2),IBASE(IMU),ISEG,LON,IBASE(INBL),
     1         IBASE(ILBL),BASE(IT))
            ELSE
               IOF=1
               DO 20 IL=0,NLF-2,2
               CALL ALVDLF(BASE(IOF2+IOF-1),IBASE(IMU),ISEG,LON,
     1         IBASE(INBL),IBASE(ILBL),BASE(IT))
   20          IOF=IOF+IBASE(IMU+LMU-1)
            ENDIF
            CALL RLSARA(BASE(IT))
            CALL RLSARA(IBASE(ILBL))
            CALL RLSARA(IBASE(INBL))
         ENDIF
         CALL RLSARA(IBASE(IMU))
         CALL LCMPOF(IPSYS,HOUT,ILENG,2,IOF2)
      ENDIF
   30 CONTINUE
      RETURN
      END
