*DECK MTLDLM
      SUBROUTINE MTLDLM(NAMP,IPTRK,IPSYS,LL4,ITY,F2,F3)
*
*-----------------------------------------------------------------------
*
*Purpose:
* LCM driver for the multiplication of a matrix by a vector.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* ITY     type of coefficient matrix (1: Bivac; 2: classical Trivac;
*         3: Thomas-Raviart; 11: SPN/Bivac; 13: SPN/Thomas-Raviart).
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,ITY
      REAL F2(LL4),F3(LL4)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,TEXT12*12
      INTEGER IBASE(1),ITP(NSTATE)
      LOGICAL LMU,LMUW,LMUX,LMUY,LMUZ,DIAG
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
*-----------------------------------------------------------------------
*
* INFORMATION RECOVERED FROM XSM OR LCM (SPLITTED MATRIX):
*  'W '//NAMP 'X '//NAMP 'Y '//NAMP 'Z '//NAMP : W-, X-, Y- AND Z-
*                            ORIENTED MATRIX COMPONENTS.
*
* SCALAR INFORMATION RECOVERED FROM LCM.
*  'MUW' 'MUX' 'MUY' 'MUZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPW' 'IPX' 'IPY' 'IPZ' : PERMUTATION INFORMATION FOR W-, X-, Y- OR
*                            Z-ORIENTED MATRIX COMPONENTS.
*
* SUPERVECTORIZATION INFORMATION RECOVERED FROM LCM.
*  'LL4VW' 'LL4VX' 'LL4VY' 'LL4VZ' : ORDER OF THE REORDERED W-, X-, Y-
*                            AND Z-ORIENTED MATRIX COMPONENTS.
*  'MUVW' 'MUVX' 'MUVY' 'MUVZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-,
*                            Y-OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPVW' 'IPVX' 'IPVY' 'IPVZ' : PERMUTATION INFORMATION FOR W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'NBLW' 'NBLX' 'NBLY' 'NBLZ' : NUMBER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*  'LBLW' 'LBLX' 'LBLY' 'LBLZ' : ORDER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*
*-----------------------------------------------------------------------
*
      IF(ITY.EQ.1) THEN
*        DIFFUSION BIVAC TRACKING.
         ISEG=0
      ELSE IF(ITY.EQ.2) THEN
*        CLASSICAL TRIVAC TRACKING.
         CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
         ISEG=ITP(17)
         LTSW=ITP(19)
      ELSE IF(ITY.EQ.3) THEN
*        THOMAS-RAVIART TRIVAC TRACKING.
         CALL FLDTRM(NAMP,IPTRK,IPSYS,LL4,F2,F3)
         RETURN
      ELSE IF(ITY.EQ.11) THEN
*        SIMPLIFIED PN BIVAC TRACKING.
         CALL LCMGET(IPSYS,'STATE-VECTOR',ITP)
         NBMIX=ITP(7)
         NAN=ITP(8)
         IF(NAN.EQ.0) CALL XABORT('MTLDLM: SPN-ONLY ALGORITHM(1).')
         CALL XDRSET(F3,LL4,0.0)
         CALL SETARA(BASE,NBMIX*2*NAN,ISGD)
         CALL FLDBSM(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F2,F3,BASE(ISGD))
         CALL RLSARA(BASE(ISGD))
         RETURN
      ELSE IF(ITY.EQ.13) THEN
*        SIMPLIFIED PN TRIVAC TRACKING.
         CALL LCMGET(IPSYS,'STATE-VECTOR',ITP)
         NBMIX=ITP(7)
         NAN=ITP(8)
         IF(NAN.EQ.0) CALL XABORT('MTLDLM: SPN-ONLY ALGORITHM(2).')
         CALL XDRSET(F3,LL4,0.0)
         CALL SETARA(BASE,NBMIX*2*NAN,ISGD)
         CALL FLDTSM(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F2,F3,BASE(ISGD))
         CALL RLSARA(BASE(ISGD))
         RETURN
      ENDIF
*
      CALL LCMLEN(IPTRK,'MU',IDUM,ITYLCM)
      LMU=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUW',IDUM,ITYLCM)
      LMUW=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUY',IDUM,ITYLCM)
      LMUY=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUZ',IDUM,ITYLCM)
      LMUZ=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=LMUY.AND.(.NOT.LMUX)
*
      NAMT=NAMP
      IF(LMU) THEN
         CALL LCMLEN(IPTRK,'MU',LL4TS,ITYLCM)
         IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(1).')
         CALL LCMIOF(IPTRK,'MU',IMU)
         CALL LCMIOF(IPSYS,NAMT,IASS)
         CALL ALLDLM(LL4,BASE(IASS),F2,F3,IBASE(IMU),1)
      ELSE IF(ISEG.EQ.0) THEN
*        SCALAR MULTIPLICATION FOR A W- OR X-ORIENTED MATRIX.
         IF(LMUW) THEN
            CALL LCMIOF(IPTRK,'MUW',IMU)
            TEXT12='W '//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(2).')
            CALL LCMIOF(IPTRK,'IPW',IIP)
         ELSE IF(DIAG) THEN
            CALL LCMIOF(IPTRK,'MUY',IMU)
            TEXT12='Y '//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(3).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
         ELSE
            CALL LCMIOF(IPTRK,'MUX',IMU)
            TEXT12='X '//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(4).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
         ENDIF
         CALL SETARA(BASE,LL4,IGAR)
         DO 10 I=1,LL4
   10    BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
         DO 20 I=1,LL4
   20    F2(I)=BASE(IGAR+I-1)
         CALL LCMIOF(IPSYS,TEXT12,IASS)
         CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),1)
         DO 30 I=1,LL4
         II=IBASE(IIP+I-1)
         F3(I)=BASE(IGAR+II-1)
   30    BASE(IGAR+II-1)=F2(II)
         DO 40 I=1,LL4
   40    F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         IF(LMUW) THEN
*           SCALAR MULTIPLICATION FOR A X-ORIENTED MATRIX.
            CALL LCMIOF(IPTRK,'MUX',IMU)
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(5).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
            DO 50 I=1,LL4
   50       BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
            DO 60 I=1,LL4
            II=IBASE(IIP+I-1)
            F2(II)=BASE(IGAR+II-1)
   60       BASE(IGAR+II-1)=F3(I)
            CALL LCMIOF(IPSYS,'X '//NAMT,IASS)
            CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
            DO 70 I=1,LL4
            II=IBASE(IIP+I-1)
            F3(I)=BASE(IGAR+II-1)
   70       BASE(IGAR+II-1)=F2(II)
            DO 80 I=1,LL4
   80       F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUY) THEN
*           SCALAR MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            CALL LCMIOF(IPTRK,'MUY',IMU)
            CALL LCMLEN(IPTRK,'IPY',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(6).')
            CALL LCMIOF(IPTRK,'IPY',IIP)
            DO 90 I=1,LL4
   90       BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
            DO 100 I=1,LL4
            II=IBASE(IIP+I-1)
            F2(II)=BASE(IGAR+II-1)
  100       BASE(IGAR+II-1)=F3(I)
            CALL LCMIOF(IPSYS,'Y '//NAMT,IASS)
            CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
            DO 110 I=1,LL4
            II=IBASE(IIP+I-1)
            F3(I)=BASE(IGAR+II-1)
  110       BASE(IGAR+II-1)=F2(II)
            DO 120 I=1,LL4
  120       F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUZ) THEN
*           SCALAR MULTIPLICATION FOR A Z-ORIENTED MATRIX.
            CALL LCMIOF(IPTRK,'MUZ',IMU)
            CALL LCMLEN(IPTRK,'IPZ',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(7).')
            CALL LCMIOF(IPTRK,'IPZ',IIP)
            DO 130 I=1,LL4
  130       BASE(IGAR+IBASE(IIP+I-1)-1)=F2(I)
            DO 140 I=1,LL4
            II=IBASE(IIP+I-1)
            F2(II)=BASE(IGAR+II-1)
  140       BASE(IGAR+II-1)=F3(I)
            CALL LCMIOF(IPSYS,'Z '//NAMT,IASS)
            CALL ALLDLM(LL4,BASE(IASS),F2,BASE(IGAR),IBASE(IMU),2)
            DO 150 I=1,LL4
            II=IBASE(IIP+I-1)
            F3(I)=BASE(IGAR+II-1)
  150       BASE(IGAR+II-1)=F2(II)
            DO 160 I=1,LL4
  160       F2(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         CALL RLSARA(BASE(IGAR))
      ELSE IF(ISEG.GT.0) THEN
*        SUPERVECTORIAL MULTIPLICATION FOR A W- OR X-ORIENTED MATRIX.
         IF(LMUW) THEN
            CALL LCMGET(IPTRK,'LL4VW',LL4V)
            CALL LCMIOF(IPTRK,'MUVW',IMU)
            TEXT12='W '//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(8).')
            CALL LCMIOF(IPTRK,'IPW',IIP)
            CALL LCMIOF(IPTRK,'IPVW',IIPV)
            CALL LCMLEN(IPTRK,'NBLW',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLW',INBL)
            CALL LCMIOF(IPTRK,'LBLW',ILBL)
         ELSE IF(DIAG) THEN
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            CALL LCMIOF(IPTRK,'MUVY',IMU)
            TEXT12='Y '//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(9).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPTRK,'IPVY',IIPV)
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBL)
            CALL LCMIOF(IPTRK,'LBLY',ILBL)
         ELSE
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            CALL LCMIOF(IPTRK,'MUVX',IMU)
            TEXT12='X '//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(10).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPTRK,'IPVX',IIPV)
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBL)
            CALL LCMIOF(IPTRK,'LBLX',ILBL)
         ENDIF
         CALL SETARA(IBASE,LL4,IIPB)
         DO 165 I=1,LL4
  165    IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
         CALL SETARA(BASE,LL4V,IGAR)
         CALL SETARA(BASE,LL4V,IGAF)
         DO 170 I=1,LL4V
  170    BASE(IGAR+I-1)=0.0
         DO 180 I=1,LL4
  180    BASE(IGAR+IBASE(IIPB+I-1)-1)=F2(I)
         CALL LCMIOF(IPSYS,TEXT12,IASS)
         CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1   IBASE(IMU),1,ISEG,LON,IBASE(INBL),IBASE(ILBL))
         DO 190 I=1,LL4
         II=IBASE(IIPB+I-1)
         F2(I)=BASE(IGAR+II-1)
  190    F3(I)=BASE(IGAF+II-1)
         CALL RLSARA(BASE(IGAF))
         CALL RLSARA(BASE(IGAR))
         CALL RLSARA(BASE(IIPB))
         IF(LMUW) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A X-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            CALL LCMIOF(IPTRK,'MUVX',IMU)
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(11).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPTRK,'IPVX',IIPV)
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBL)
            CALL LCMIOF(IPTRK,'LBLX',ILBL)
            CALL SETARA(IBASE,LL4,IIPB)
            DO 195 I=1,LL4
  195       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            DO 200 I=1,LL4V
  200       BASE(IGAR+I-1)=0.0
            DO 210 I=1,LL4V
  210       BASE(IGAF+I-1)=0.0
            DO 220 I=1,LL4
            II=IBASE(IIPB+I-1)
            BASE(IGAR+II-1)=F2(I)
  220       BASE(IGAF+II-1)=F3(I)
            CALL LCMIOF(IPSYS,'X '//NAMT,IASS)
            CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            DO 230 I=1,LL4
            II=IBASE(IIPB+I-1)
            F2(I)=BASE(IGAR+II-1)
  230       F3(I)=BASE(IGAF+II-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUY) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A Y-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            CALL LCMIOF(IPTRK,'MUVY',IMU)
            CALL LCMLEN(IPTRK,'IPY',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(12).')
            CALL LCMIOF(IPTRK,'IPY',IIP)
            CALL LCMIOF(IPTRK,'IPVY',IIPV)
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBL)
            CALL LCMIOF(IPTRK,'LBLY',ILBL)
            CALL SETARA(IBASE,LL4,IIPB)
            DO 235 I=1,LL4
  235       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            DO 240 I=1,LL4V
  240       BASE(IGAR+I-1)=0.0
            DO 250 I=1,LL4V
  250       BASE(IGAF+I-1)=0.0
            DO 260 I=1,LL4
            II=IBASE(IIPB+I-1)
            BASE(IGAR+II-1)=F2(I)
  260       BASE(IGAF+II-1)=F3(I)
            CALL LCMIOF(IPSYS,'Y '//NAMT,IASS)
            CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            DO 270 I=1,LL4
            II=IBASE(IIPB+I-1)
            F2(I)=BASE(IGAR+II-1)
  270       F3(I)=BASE(IGAF+II-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUZ) THEN
*           SUPERVECTORIAL MULTIPLICATION FOR A Z-ORIENTED MATRIX.
            CALL LCMGET(IPTRK,'LL4VZ',LL4V)
            CALL LCMIOF(IPTRK,'MUVZ',IMU)
            CALL LCMLEN(IPTRK,'IPZ',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLM: INVALID LL4(13).')
            CALL LCMIOF(IPTRK,'IPZ',IIP)
            CALL LCMIOF(IPTRK,'IPVZ',IIPV)
            CALL LCMLEN(IPTRK,'NBLZ',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLZ',INBL)
            CALL LCMIOF(IPTRK,'LBLZ',ILBL)
            CALL SETARA(IBASE,LL4,IIPB)
            DO 275 I=1,LL4
  275       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            CALL SETARA(BASE,LL4V,IGAF)
            DO 280 I=1,LL4V
  280       BASE(IGAR+I-1)=0.0
            DO 290 I=1,LL4V
  290       BASE(IGAF+I-1)=0.0
            DO 300 I=1,LL4
            II=IBASE(IIPB+I-1)
            BASE(IGAR+II-1)=F2(I)
  300       BASE(IGAF+II-1)=F3(I)
            CALL LCMIOF(IPSYS,'Z '//NAMT,IASS)
            CALL ALVDLM(LTSW,BASE(IASS),BASE(IGAR),BASE(IGAF),
     1      IBASE(IMU),2,ISEG,LON,IBASE(INBL),IBASE(ILBL))
            DO 310 I=1,LL4
            II=IBASE(IIPB+I-1)
            F2(I)=BASE(IGAR+II-1)
  310       F3(I)=BASE(IGAF+II-1)
            CALL RLSARA(BASE(IGAF))
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
      ENDIF
      RETURN
      END
