*DECK MTLDLS
      SUBROUTINE MTLDLS(NAMP,IPTRK,IPSYS,LL4,ITY,F1)
*
*-----------------------------------------------------------------------
*
*Purpose:
* LCM driver for the solution of a linear system after LDL(t)
* factorization.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* ITY     type of coefficient matrix (1: Bivac; 2: classical Trivac;
*         3: Thomas-Raviart; 11: SPN/Bivac; 13: SPN/Thomas-Raviart).
* F1      right-hand side of the linear system.
*
*Parameters: output
* F1      solution of the linear system.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,ITY
      REAL F1(LL4)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,TEXT12*12
      INTEGER IBASE(1),ITP(NSTATE)
      LOGICAL LMU,LMUW,LMUX,LMUY,LMUZ,DIAG
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
*-----------------------------------------------------------------------
*
* INFORMATION RECOVERED FROM XSM OR LCM (NON-SPLITTED MATRIX):
*  NAMP       : COEFFICIENT MATRIX.
*  'I'//NAMP  : FACTORIZED COEFFICIENT MATRIX.
*  'MU'       : POSITION OF DIAGONAL ELEMENT IN COEFFICIENT MATRIX.
*
* INFORMATION RECOVERED FROM XSM OR LCM (SPLITTED MATRIX):
*  'W '//NAMP 'X '//NAMP 'Y '//NAMP 'Z '//NAMP : W-, X-, Y- AND Z-
*                            ORIENTED MATRIX COMPONENTS.
*  'WI'//NAMP 'XI'//NAMP 'YI'//NAMP 'ZI'//NAMP : W-, X-, Y- AND Z-
*                            ORIENTED FACTORIZED MATRIX COMPONENTS.
*
* SCALAR INFORMATION RECOVERED FROM LCM.
*  'MUW' 'MUX' 'MUY' 'MUZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPW' 'IPX' 'IPY' 'IPZ' : PERMUTATION INFORMATION FOR W-, X-, Y- OR
*                            Z-ORIENTED MATRIX COMPONENTS.
*
* SUPERVECTORIZATION INFORMATION RECOVERED FROM LCM.
*  'WD'//NAMP 'XD'//NAMP 'YD'//NAMP 'ZD'//NAMP : DIAGONAL ELEMENTS FOR
*                           W-, X-, Y- AND Z-ORIENTED MATRIX COMPONENTS.
*  'LL4VW' 'LL4VX' 'LL4VY' 'LL4VZ' : ORDER OF THE REORDERED W-, X-, Y-
*                            AND Z-ORIENTED MATRIX COMPONENTS.
*  'MUVW' 'MUVX' 'MUVY' 'MUVZ' : POSITION OF DIAGONAL ELEMENT IN W-, X-,
*                            Y-OR Z-ORIENTED MATRIX COMPONENTS.
*  'IPVW' 'IPVX' 'IPVY' 'IPVZ' : PERMUTATION INFORMATION FOR W-, X-, Y-
*                            OR Z-ORIENTED MATRIX COMPONENTS.
*  'NBLW' 'NBLX' 'NBLY' 'NBLZ' : NUMBER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*  'LBLW' 'LBLX' 'LBLY' 'LBLZ' : ORDER OF LINEAR SYSTEMS IN EACH SUPER-
*                            VECTORIAL UNKNOWN GROUP.
*
*-----------------------------------------------------------------------
*
      IF(ITY.EQ.1) THEN
*        BIVAC TRACKING.
         ISEG=0
      ELSE IF(ITY.EQ.2) THEN
*        CLASSICAL TRIVAC TRACKING.
         CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
         ISEG=ITP(17)
         LTSW=ITP(19)
      ELSE IF(ITY.EQ.3) THEN
*        THOMAS-RAVIART/DIFFUSION TRIVAC TRACKING.
         CALL FLDTRS(NAMP,IPTRK,IPSYS,LL4,F1,1)
         RETURN
      ELSE IF(ITY.EQ.11) THEN
*        SIMPLIFIED PN BIVAC TRACKING.
         CALL LCMGET(IPSYS,'STATE-VECTOR',ITP)
         NBMIX=ITP(7)
         NAN=ITP(8)
         IF(NAN.EQ.0) CALL XABORT('MTLDLS: SPN-ONLY ALGORITHM(1).')
         CALL SETARA(BASE,NBMIX*NAN,ISGD)
         CALL FLDBSS(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F1,BASE(ISGD))
         CALL RLSARA(BASE(ISGD))
         RETURN
      ELSE IF(ITY.EQ.13) THEN
*        THOMAS-RAVIART/SIMPLIFIED PN TRIVAC TRACKING.
         CALL LCMGET(IPSYS,'STATE-VECTOR',ITP)
         NBMIX=ITP(7)
         NAN=ITP(8)
         IF(NAN.EQ.0) CALL XABORT('MTLDLS: SPN-ONLY ALGORITHM(2).')
         CALL SETARA(BASE,NBMIX*NAN,ISGD)
         CALL SETARA(BASE,NBMIX*NAN,ISGDI)
         CALL FLDSPN(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,F1,1,BASE(ISGD),
     1   BASE(ISGDI))
         CALL RLSARA(BASE(ISGDI))
         CALL RLSARA(BASE(ISGD))
         RETURN
      ENDIF
*
      CALL LCMLEN(IPTRK,'MU',IDUM,ITYLCM)
      LMU=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUW',IDUM,ITYLCM)
      LMUW=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUY',IDUM,ITYLCM)
      LMUY=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      CALL LCMLEN(IPTRK,'MUZ',IDUM,ITYLCM)
      LMUZ=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=LMUY.AND.(.NOT.LMUX)
*
      NAMT=NAMP
      IF(LMU) THEN
         CALL LCMLEN(IPTRK,'MU',LL4TS,ITYLCM)
         IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(1).')
         CALL LCMIOF(IPTRK,'MU',IMU)
         CALL LCMIOF(IPSYS,'I'//NAMT,IASS)
         CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),F1)
      ELSE IF(ISEG.EQ.0) THEN
*        SCALAR SOLUTION FOR A W- OR X-ORIENTED LINEAR SYSTEM.
         IF(LMUW) THEN
            CALL LCMIOF(IPTRK,'MUW',IMU)
            TEXT12='WI'//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(2).')
            CALL LCMIOF(IPTRK,'IPW',IIP)
         ELSE IF(DIAG) THEN
            CALL LCMIOF(IPTRK,'MUY',IMU)
            TEXT12='YI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(3).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
         ELSE
            CALL LCMIOF(IPTRK,'MUX',IMU)
            TEXT12='XI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(4).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
         ENDIF
         CALL SETARA(BASE,LL4,IGAR)
         DO 10 I=1,LL4
   10    BASE(IGAR+IBASE(IIP+I-1)-1)=F1(I)
         CALL LCMIOF(IPSYS,TEXT12,IASS)
         CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
         DO 20 I=1,LL4
   20    F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         IF(LMUW) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUX',IMU)
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPSYS,'X '//NAMT,IASS)
            DO 30 I=1,LL4
            II=IBASE(IIP+I-1)
   30       BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
            CALL LCMIOF(IPSYS,'XI'//NAMT,IASS)
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            DO 50 I=1,LL4
   50       F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUY) THEN
*           SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMU)
            CALL LCMIOF(IPTRK,'IPY',IIP)
            CALL LCMIOF(IPSYS,'Y '//NAMT,IASS)
            DO 60 I=1,LL4
            II=IBASE(IIP+I-1)
   60       BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
            CALL LCMIOF(IPSYS,'YI'//NAMT,IASS)
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            DO 80 I=1,LL4
   80       F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         IF(LMUZ) THEN
*           SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUZ',IMU)
            CALL LCMIOF(IPTRK,'IPZ',IIP)
            CALL LCMIOF(IPSYS,'Z '//NAMT,IASS)
            DO 90 I=1,LL4
            II=IBASE(IIP+I-1)
   90       BASE(IGAR+II-1)=F1(I)*BASE(IASS+IBASE(IMU+II-1)-1)
            CALL LCMIOF(IPSYS,'ZI'//NAMT,IASS)
            CALL ALLDLS(LL4,IBASE(IMU),BASE(IASS),BASE(IGAR))
            DO 110 I=1,LL4
  110       F1(I)=BASE(IGAR+IBASE(IIP+I-1)-1)
         ENDIF
         CALL RLSARA(BASE(IGAR))
      ELSE IF(ISEG.GT.0) THEN
*        SUPERVECTORIAL SOLUTION FOR A W- OR X-ORIENTED LINEAR SYSTEM.
         CALL SETARA(BASE,ISEG,IT)
         IF(LMUW) THEN
            CALL LCMGET(IPTRK,'LL4VW',LL4V)
            CALL LCMIOF(IPTRK,'MUVW',IMU)
            TEXT12='WI'//NAMT
            CALL LCMLEN(IPTRK,'IPW',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(5).')
            CALL LCMIOF(IPTRK,'IPW',IIP)
            CALL LCMIOF(IPTRK,'IPVW',IIPV)
            CALL LCMLEN(IPTRK,'NBLW',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLW',INBL)
            CALL LCMIOF(IPTRK,'LBLW',ILBL)
         ELSE IF(DIAG) THEN
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            CALL LCMIOF(IPTRK,'MUVY',IMU)
            TEXT12='YI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(6).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPTRK,'IPVY',IIPV)
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBL)
            CALL LCMIOF(IPTRK,'LBLY',ILBL)
         ELSE
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            CALL LCMIOF(IPTRK,'MUVX',IMU)
            TEXT12='XI'//NAMT
            CALL LCMLEN(IPTRK,'IPX',LL4TS,ITYLCM)
            IF(LL4.NE.LL4TS) CALL XABORT('MTLDLS: INVALID LL4(7).')
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPTRK,'IPVX',IIPV)
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBL)
            CALL LCMIOF(IPTRK,'LBLX',ILBL)
         ENDIF
         CALL SETARA(IBASE,LL4,IIPB)
         DO 115 I=1,LL4
  115    IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
         CALL SETARA(BASE,LL4V,IGAR)
         DO 120 I=1,LL4V
  120    BASE(IGAR+I-1)=0.0
         DO 130 I=1,LL4
  130    BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
         CALL LCMIOF(IPSYS,TEXT12,IASS)
         CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,LON,
     1   IBASE(INBL),IBASE(ILBL),BASE(IT))
         DO 140 I=1,LL4
  140    F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
         CALL RLSARA(BASE(IGAR))
         CALL RLSARA(BASE(IIPB))
         IF(LMUW) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VX',LL4V)
            CALL LCMIOF(IPTRK,'MUVX',IMU)
            CALL LCMLEN(IPTRK,'IPX',LL4,ITYLCM)
            CALL LCMIOF(IPTRK,'IPX',IIP)
            CALL LCMIOF(IPTRK,'IPVX',IIPV)
            CALL LCMLEN(IPTRK,'NBLX',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBL)
            CALL LCMIOF(IPTRK,'LBLX',ILBL)
            CALL SETARA(IBASE,LL4,IIPB)
            DO 145 I=1,LL4
  145       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            DO 150 I=1,LL4V
  150       BASE(IGAR+I-1)=0.0
            DO 160 I=1,LL4
  160       BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
            CALL LCMIOF(IPSYS,'XI'//NAMT,IASS)
            CALL LCMIOF(IPSYS,'XD'//NAMT,IDGV)
CDIR$ IVDEP
            DO 170 I=1,LL4V
  170       BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IDGV+I-1)
            CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1      LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
            DO 190 I=1,LL4
  190       F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUY) THEN
*           SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4V)
            CALL LCMIOF(IPTRK,'MUVY',IMU)
            CALL LCMLEN(IPTRK,'IPY',LL4,ITYLCM)
            CALL LCMIOF(IPTRK,'IPY',IIP)
            CALL LCMIOF(IPTRK,'IPVY',IIPV)
            CALL LCMLEN(IPTRK,'NBLY',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBL)
            CALL LCMIOF(IPTRK,'LBLY',ILBL)
            CALL SETARA(IBASE,LL4,IIPB)
            DO 195 I=1,LL4
  195       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            DO 200 I=1,LL4V
  200       BASE(IGAR+I-1)=0.0
            DO 210 I=1,LL4
  210       BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
            CALL LCMIOF(IPSYS,'YI'//NAMT,IASS)
            CALL LCMIOF(IPSYS,'YD'//NAMT,IDGV)
CDIR$ IVDEP
            DO 220 I=1,LL4V
  220       BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IDGV+I-1)
            CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1      LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
            DO 240 I=1,LL4
  240       F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         IF(LMUZ) THEN
*           SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VZ',LL4V)
            CALL LCMIOF(IPTRK,'MUVZ',IMU)
            CALL LCMLEN(IPTRK,'IPZ',LL4,ITYLCM)
            CALL LCMIOF(IPTRK,'IPZ',IIP)
            CALL LCMIOF(IPTRK,'IPVZ',IIPV)
            CALL LCMLEN(IPTRK,'NBLZ',LON,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLZ',INBL)
            CALL LCMIOF(IPTRK,'LBLZ',ILBL)
            CALL SETARA(IBASE,LL4,IIPB)
            DO 245 I=1,LL4
  245       IBASE(IIPB+I-1)=IBASE(IIPV+IBASE(IIP+I-1)-1)
            CALL SETARA(BASE,LL4V,IGAR)
            DO 250 I=1,LL4V
  250       BASE(IGAR+I-1)=0.0
            DO 260 I=1,LL4
  260       BASE(IGAR+IBASE(IIPB+I-1)-1)=F1(I)
            CALL LCMIOF(IPSYS,'ZI'//NAMT,IASS)
            CALL LCMIOF(IPSYS,'ZD'//NAMT,IDGV)
CDIR$ IVDEP
            DO 270 I=1,LL4V
  270       BASE(IGAR+I-1)=BASE(IGAR+I-1)*BASE(IDGV+I-1)
            CALL ALVDLS(LTSW,IBASE(IMU),BASE(IASS),BASE(IGAR),ISEG,
     1      LON,IBASE(INBL),IBASE(ILBL),BASE(IT))
            DO 290 I=1,LL4
  290       F1(I)=BASE(IGAR+IBASE(IIPB+I-1)-1)
            CALL RLSARA(BASE(IGAR))
            CALL RLSARA(BASE(IIPB))
         ENDIF
         CALL RLSARA(BASE(IT))
      ENDIF
      RETURN
      END
