*DECK OUTAUX
      SUBROUTINE OUTAUX (IPMAC1,IPMAC2,NBMIX,NBFIS,NGRP,NEL,NUN,NZS,
     1 MAT,VOL,IDL,EVECT,IHOM,IMPX,MAXALC,WORK,IJJ,NJJ,IPOS,RATE,FLINT,
     2 GAR,SCAT,CHI,ZUFIS,OUTR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform homogenization into NZS regions based on averaged fluxes
* contained in EVECT. Create an output extended macrolib containing
* homogenized volumes, integrated fluxes and cross sections.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPMAC1  L_MACROLIB pointer to the input macrolib.
* IPMAC2  L_MACROLIB pointer to the output extended macrolib.
* NBMIX   number of material mixtures.
* NBFIS   number of fissionable isotopes.
* NGRP    total number of energy groups.
* NEL     number of finite elements.
* NUN     number of unknowns per energy group.
* NZS     number of homogenized regions so that NZS=max(IHOM(i)).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* IDL     position of the average flux component associated with
*         each volume.
* EVECT   unknowns.
* IHOM    homogenized index assigned to each element.
* IMPX    print parameter (equal to zero for no print).
* MAXALC  maximum of NBMIX and NZS.
*
*Parameters: scratch
* WORK
* IJJ
* NJJ
* IPOS
* RATE
* FLINT
* GAR
* SCAT
* CHI
* ZUFIS
* OUTR
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      PARAMETER(NREAC=9)
      INTEGER IPMAC1,IPMAC2,NBMIX,NBFIS,NGRP,NEL,NUN,NZS,MAT(NEL),
     1 IDL(NEL),IHOM(NEL),IMPX,MAXALC,IJJ(MAXALC),NJJ(MAXALC),
     2 IPOS(MAXALC)
      REAL VOL(NEL),EVECT(NUN,NGRP),WORK(MAXALC*NGRP),RATE(NZS),
     1 FLINT(NZS,NGRP),GAR(NGRP),SCAT(NZS,NGRP),CHI(NBMIX,NBFIS),
     2 ZUFIS(NBMIX,NBFIS),OUTR(NZS+1,NREAC+3)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      CHARACTER HREAC(NREAC)*12,TEXT12*12
      INTEGER IDATA(NSTATE)
      LOGICAL LNUSIG
      DOUBLE PRECISION ACCUM
      DATA HREAC/'NTOT0','SIGW00','NUSIGF','NFTOT','H-FACTOR',
     1 'OVERV','DIFFX','DIFFY','DIFFZ'/
*
      DO 10 IBM=1,NZS
      WORK(IBM)=0.0
      DO 10 IGR=1,NGRP
   10 FLINT(IBM,IGR)=0.0
      DO 30 K=1,NEL
      IBM=IHOM(K)
      IPFL=IDL(K)
      IF((IBM.NE.0).AND.(MAT(K).NE.0).AND.(IPFL.NE.0)) THEN
         WORK(IBM)=WORK(IBM)+VOL(K)
         DO 20 IGR=1,NGRP
   20    FLINT(IBM,IGR)=FLINT(IBM,IGR)+EVECT(IPFL,IGR)*VOL(K)
      ENDIF
   30 CONTINUE
      CALL LCMPUT(IPMAC2,'VOLUME',NZS,2,WORK)
      DO 40 IBM=1,NZS
   40 OUTR(IBM,NREAC+1)=WORK(IBM)
*
      IF(IMPX.GT.0) WRITE(6,'(/35H OUTAUX: REACTION RATE CALCULATION.)')
      JPMAC1=LCMGID(IPMAC1,'GROUP')
      JPMAC2=LCMLID(IPMAC2,'GROUP',NGRP)
      DO 280 IGR=1,NGRP
      KPMAC1=LCMGIL(JPMAC1,IGR)
      KPMAC2=LCMDIL(JPMAC2,IGR)
      CALL LCMPUT(KPMAC2,'FLUX-INTG',NZS,2,FLINT(1,IGR))
      DO 70 IBM=1,NZS
   70 OUTR(IBM,NREAC+2)=FLINT(IBM,IGR)
*----
*  CROSS SECTIONS HOMOGENIZATION.
*----
      LNUSIG=.FALSE.
      DO 130 IREAC=1,NREAC
      DO 80 IBM=1,NZS
   80 OUTR(IBM,IREAC)=0.0
      CALL LCMLEN(KPMAC1,HREAC(IREAC),LENGT,ITYLCM)
      IF((HREAC(IREAC).EQ.'H-FACTOR').AND.(LENGT.EQ.0)) THEN
         WRITE(6,'(/46H OUTAUX: *** WARNING *** NO H-FACTOR FOUND ON ,
     1   25HLCM. USE NU*SIGF INSTEAD.)')
         LNUSIG=.TRUE.
         GO TO 130
      ELSE IF(HREAC(IREAC).EQ.'NUSIGF') THEN
         GO TO 130
      ELSE
         TEXT12=HREAC(IREAC)
      ENDIF
      IF(LENGT.GT.0) THEN
         IF(LENGT.GT.NBMIX) CALL XABORT('OUTAUX: INVALID LENGTH FOR '//
     1   HREAC(IREAC)//' CROSS SECTIONS.')
         CALL LCMGET(KPMAC1,TEXT12,WORK)
         DO 90 IBM=1,NZS
   90    RATE(IBM)=0.0
         DO 100 K=1,NEL
         IBM=IHOM(K)
         L=MAT(K)
         IPFL=IDL(K)
         IF((IBM.NE.0).AND.(L.NE.0).AND.(IPFL.NE.0)) THEN
            RATE(IBM)=RATE(IBM)+EVECT(IPFL,IGR)*VOL(K)*WORK(L)
         ENDIF
  100    CONTINUE
         DO 120 IBM=1,NZS
         OUTR(IBM,IREAC)=RATE(IBM)
         IF(RATE(IBM).NE.0.0) RATE(IBM)=RATE(IBM)/FLINT(IBM,IGR)
  120    CONTINUE
         CALL LCMPUT(KPMAC2,HREAC(IREAC),NZS,2,RATE)
      ENDIF
  130 CONTINUE
*----
*  SCATTERING MATRIX HOMOGENIZATION.
*----
      CALL LCMLEN(KPMAC1,'NJJS00',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         IF(LENGT.GT.NBMIX) CALL XABORT('OUTAUX: INVALID LENGTH FOR '//
     1   'SCATTERING CROSS SECTIONS.')
         DO 140 JGR=1,NGRP
         DO 140 IBM=1,NZS
  140    SCAT(IBM,JGR)=0.0
         CALL LCMGET(KPMAC1,'NJJS00',NJJ)
         CALL LCMGET(KPMAC1,'IJJS00',IJJ)
         CALL LCMGET(KPMAC1,'IPOS00',IPOS)
         CALL LCMGET(KPMAC1,'SCAT00',WORK)
         IPOSDE=0
         DO 180 K=1,NEL
         IBM=IHOM(K)
         L=MAT(K)
         IPFL=IDL(K)
         IF((IBM.NE.0).AND.(L.NE.0).AND.(IPFL.NE.0)) THEN
            DO 150 JGR=1,NGRP
  150       GAR(JGR)=0.0
            IPOSDE=IPOS(L)-1
            DO 160 JGR=IJJ(L),IJJ(L)-NJJ(L)+1,-1
            IPOSDE=IPOSDE+1
  160       GAR(JGR)=WORK(IPOSDE)
            DO 170 JGR=1,NGRP
  170       SCAT(IBM,JGR)=SCAT(IBM,JGR)+EVECT(IPFL,JGR)*VOL(K)*GAR(JGR)
         ENDIF
  180    CONTINUE
         DO 200 IBM=1,NZS
         IGMIN=IGR
         IGMAX=IGR
         DO 190 JGR=NGRP,1,-1
         IF(SCAT(IBM,JGR).NE.0.0) THEN
            IGMIN=MIN(IGMIN,JGR)
            IGMAX=MAX(IGMAX,JGR)
            SCAT(IBM,JGR)=SCAT(IBM,JGR)/FLINT(IBM,JGR)
         ENDIF
  190    CONTINUE
         IJJ(IBM)=IGMAX
  200    NJJ(IBM)=IGMAX-IGMIN+1
         IPOSDE=0
         DO 210 IBM=1,NZS
         IPOS(IBM)=IPOSDE+1
         DO 210 JGR=IJJ(IBM),IJJ(IBM)-NJJ(IBM)+1,-1
         IPOSDE=IPOSDE+1
  210    WORK(IPOSDE)=SCAT(IBM,JGR)
         CALL LCMPUT(KPMAC2,'SCAT00',IPOSDE,2,WORK)
         CALL LCMPUT(KPMAC2,'IPOS00',NZS,1,IPOS)
         CALL LCMPUT(KPMAC2,'NJJS00',NZS,1,NJJ)
         CALL LCMPUT(KPMAC2,'IJJS00',NZS,1,IJJ)
         CALL LCMPUT(KPMAC2,'SIGW00',NZS,2,SCAT(1,IGR))
      ENDIF
*----
*  FISSION SPECTRUM AND NUSIGF HOMOGENIZATION.
*----
      CALL XDRSET(OUTR(1,NREAC+3),NZS,0.0)
      CALL LCMLEN(KPMAC1,'CHI',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         IF(LENGT.NE.NBMIX*NBFIS) CALL XABORT('OUTAUX: INVALID LENGTH '
     1   //'FOR FISSION SPECTRUM.')
         CALL LCMGET(KPMAC1,'CHI',CHI)
         CALL LCMGET(KPMAC1,'NUSIGF',ZUFIS)
         CALL XDRSET(RATE,NZS,0.0)
         CALL XDRSET(SCAT(1,1),NZS,0.0)
         DO 230 K=1,NEL
         IBM=IHOM(K)
         L=MAT(K)
         IPFL=IDL(K)
         IF((IBM.NE.0).AND.(L.NE.0).AND.(IPFL.NE.0)) THEN
            DO 220 IFISS=1,NBFIS
            ACCUM=EVECT(IPFL,IGR)*VOL(K)*ZUFIS(L,IFISS)
            RATE(IBM)=RATE(IBM)+CHI(L,IFISS)*ACCUM
  220       SCAT(IBM,1)=SCAT(IBM,1)+ACCUM
         ENDIF
  230    CONTINUE
         DO 240 IBM=1,NZS
         IF(SCAT(IBM,1).NE.0.0) THEN
            RATE(IBM)=RATE(IBM)/SCAT(IBM,1)
            OUTR(IBM,NREAC+3)=RATE(IBM)
         ENDIF
  240    CONTINUE
         CALL LCMPUT(KPMAC2,'CHI',NZS,2,RATE)
         CALL XDRSET(RATE,NZS,0.0)
         DO 245 IBM=1,NZS
         OUTR(IBM,3)=SCAT(IBM,1)
         IF(LNUSIG) OUTR(IBM,5)=SCAT(IBM,1)
         IF(SCAT(IBM,1).NE.0.0) RATE(IBM)=SCAT(IBM,1)/FLINT(IBM,IGR)
  245    CONTINUE
         CALL LCMPUT(KPMAC2,'NUSIGF',NZS,2,RATE)
         IF(LNUSIG) CALL LCMPUT(KPMAC2,'H-FACTOR',NZS,2,RATE)
      ENDIF
*----
*  PRINT THE REACTION RATES:
*----
      IF(IMPX.GT.0) THEN
         DO 250 I=1,NREAC+2
  250    OUTR(NZS+1,I)=0.0
         WRITE(6,500) IGR,'VOLUME     ','FLUX-INTG  ',(HREAC(I),I=1,6),
     1   'CHI        '
         DO 270 IBM=1,NZS
         DO 260 I=1,NREAC+2
  260    OUTR(NZS+1,I)=OUTR(NZS+1,I)+OUTR(IBM,I)
         WRITE(6,510) IBM,OUTR(IBM,NREAC+1),OUTR(IBM,NREAC+2),
     1   (OUTR(IBM,I),I=1,6),OUTR(IBM,NREAC+3)
  270    CONTINUE
         WRITE(6,520) OUTR(NZS+1,NREAC+1),OUTR(NZS+1,NREAC+2),
     1   (OUTR(NZS+1,I),I=1,6)
      ENDIF
  280 CONTINUE
      DO 290 I=1,NSTATE
  290 IDATA(I)=0
      IDATA(1)=NGRP
      IDATA(2)=NZS
      IDATA(3)=1
      IDATA(4)=1
      CALL LCMPUT(IPMAC2,'STATE-VECTOR',NSTATE,1,IDATA)
      RETURN
  500 FORMAT(/' G R O U P   : ',I3/1X,'IHOM',9A14)
  510 FORMAT(1X,I4,1P,9E14.5)
  520 FORMAT(/5H  SUM,1P,8E14.5)
      END
