*DECK TRIASN
      SUBROUTINE TRIASN(HNAMT,IPTRK,IPSYS,IMPX,NBMIX,NEL,NAN,IPR,MAT,
     1 VOL,SIGT,SIGTI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of a single-group system matrix with leakage and removal
* cross sections for the simplified PN method.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* HNAMT   name of the matrix.
* IPTRK   L_TRACK pointer to the TRIVAC tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* IMPX    print parameter (equal to zero for no print).
* NBMIX   number of mixtures.
* NEL     total number of finite elements.
* NAN     number of Legendre orders for the cross sections.
* IPR     type of assembly:
*         =0: calculation of the system matrices;
*         =1: calculation of the derivative of these matrices;
*         =2: calculation of the first variation of these matrices;
*         =3: identical to IPR=2, but these variation are added to
*         unperturbed system matrices.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* SIGT    total minus self-scattering macroscopic cross sections.
*         SIGT(:,NAN) generally contains the total cross section only.
*         If IPR.gt.0, SIGT contains perturbed or derivative values.
* SIGTI   inverse macroscopic cross sections ordered by mixture.
*         SIGTI(:,NAN) generally contains the inverse total cross
*         section only. If IPR.gt.0, SIGTI contains perturbed or
*         derivative values.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER HNAMT*10
      INTEGER IPTRK,IPSYS,IMPX,NBMIX,NEL,NAN,IPR,MAT(NEL)
      REAL VOL(NEL),SIGT(NBMIX,NAN),SIGTI(NBMIX,NAN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      LOGICAL CYLIND,CHEX,DIAG,LSGD
      CHARACTER TEXT10*10
      INTEGER IBASE(1),ISTATE(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      ITYPE=ISTATE(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IF(CYLIND) CALL XABORT('TRIASN: GEOMETRY NOT AVAILABLE.')
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IHEX=ISTATE(7)
      DIAG=(ISTATE(8).EQ.1)
      IELEM=ISTATE(9)
      ICOL=ISTATE(10)
      LL4=ISTATE(11)
      ICHX=ISTATE(12)
      IF(ICHX.NE.2) CALL XABORT('TRIASN: DISCRETIZATION NOT AVAILABLE.')
      ISPLH=ISTATE(13)
      LX=ISTATE(14)
      LY=ISTATE(15)
      LZ=ISTATE(16)
      ISEG=ISTATE(17)
      IMPV=ISTATE(18)
      NR0=ISTATE(24)
      LL4F=ISTATE(25)
      ITY=3
      LL4W=ISTATE(26)
      LL4X=ISTATE(27)
      LL4Y=ISTATE(28)
      LL4Z=ISTATE(29)
      NLF=ISTATE(30)
      NVD=ISTATE(34)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY*LZ,IZZ)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL SETARA(IBASE,LL4W,IMUW)
         CALL LCMGET(IPTRK,'MUW',IBASE(IMUW))
      ELSE
         CALL SETARA(BASE,LX*LY*LZ,IXX)
         CALL SETARA(BASE,LX*LY*LZ,IYY)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
      ENDIF
      IF(LL4Y.GT.0) THEN
         CALL SETARA(IBASE,LL4Y,IMUY)
         CALL LCMGET(IPTRK,'MUY',IBASE(IMUY))
      ENDIF
      IF(.NOT.DIAG) THEN
         CALL SETARA(IBASE,LL4X,IMUX)
         CALL LCMGET(IPTRK,'MUX',IBASE(IMUX))
      ELSE
         IMUX=IMUY
      ENDIF
      IF(LL4Z.GT.0) THEN
         CALL SETARA(IBASE,LL4Z,IMUZ)
         CALL LCMGET(IPTRK,'MUZ',IBASE(IMUZ))
      ENDIF
*----
*  RECOVER UNIT MATRICES
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*
      TEXT10=HNAMT
      IF(IMPX.GT.0) WRITE(6,'(/36H TRIASN: ASSEMBLY OF SYMMETRIC MATRI,
     1 3HX '',A10,38H'' IN COMPRESSED DIAGONAL STORAGE MODE.)') TEXT10
      CALL KDRCPU(TK1)
*----
*  DETERMINATION OF THE PERTURBED ELEMENTS. NON-PERTURBED ELEMENTS WILL
*  HAVE BASE(IVOL+K-1)=0.0
*----
      CALL SETARA(BASE,NEL,IVOL)
      IF(IPR.EQ.0) THEN
         DO 25 K=1,NEL
   25    BASE(IVOL+K-1)=VOL(K)
      ELSE
         DO 26 K=1,NEL
   26    BASE(IVOL+K-1)=0.0
         DO 50 L=1,NEL
	 IBM=MAT(L)
         IF(IBM.EQ.0) GO TO 50
         LSGD=.FALSE.
         DO 45 I=1,NAN
   45    LSGD=LSGD.OR.(SIGT(IBM,I).NE.0.0).OR.(SIGTI(IBM,I).NE.0.0)
         IF(LSGD) BASE(IVOL+L-1)=VOL(L)
   50    CONTINUE
      ENDIF
*----
*  INTRODUCE THE CYLINDER BOUNDARY APPROXIMATION IN CARTESIAN GEOMETRY
*----
      IF(NR0.GT.0) THEN
         CALL XABORT('TRIASN: CYLINDRICAL CORRECTION NOT IMPLEMENTED.')
      ENDIF
*----
*  ASSEMBLY OF THE ADI SPLITTED SYSTEM MATRICES
*----
*
* DIMENSION W
      IF(CHEX) THEN
         IIMAW=IBASE(IMUW+LL4W-1)*NLF/2
         IF(DIAG.OR.(IPR.NE.3)) THEN
            CALL SETARA(BASE,LL4F*NLF/2,ITF)
            CALL SETARA(BASE,IIMAW,IAW)
            CALL XDRSET(BASE(ITF),LL4F*NLF/2,0.0)
            CALL XDRSET(BASE(IAW),IIMAW,0.0)
         ELSE
            IF(ISEG.GT.0) CALL MTBLD('WA'//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPSYS,'WA'//TEXT10,IAW)
         ENDIF
         CALL LCMIOF(IPTRK,'IPBBW',IIPBW)
         CALL LCMLEN(IPSYS,'WB',LENWB,ITYL)
         IF(LENWB.EQ.0) THEN
           CALL LCMIOF(IPTRK,'WB',IBW)
         ELSE
           CALL LCMIOF(IPSYS,'WB',IBW)
         ENDIF
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPTRK,'FRZ',IFRZ)
         CALL SETARA(BASE,IIMAW,IWA)
         NBLOS=LX*LZ*ISPLH**2
         CALL SETARA(BASE,NBLOS,IDIF)
         DO 60 I=1,NBLOS
         BASE(IDIF+I-1)=0.0
         IND=IBASE(IZGLOB+I-1)
         IF(IND.GT.0) THEN
            IBM=MAT(IND)
            DZ=BASE(IZZ+IND-1)*BASE(IFRZ+IND-1)
            IF(IBM.GT.0) BASE(IDIF+I-1)=DZ*SIGT(IBM,1)
         ENDIF
   60    CONTINUE
         CALL LCMPOF(IPSYS,'SIGT'//TEXT10,NBLOS,2,IDIF)
         CALL PN3HWW(NBMIX,NBLOS,IELEM,ICOL,IDIM,NLF,NVD,NAN,LX,LZ,
     1   LL4,LL4F,LL4W,LL4X,LL4Y,LL4Z,MAT,SIGT,SIGTI,SIDE,BASE(IZZ),
     2   BASE(IFRZ),BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),IBASE(IMUW),
     3   IBASE(IIPBW),LC,BASE(IR),BASE(IV),BASE(IBW),BASE(ITF),
     4   BASE(IAW),BASE(IWA))
         CALL LCMPOF(IPSYS,'W '//TEXT10,IIMAW,2,IWA)
         CALL LCMPOF(IPSYS,'WA'//TEXT10,IIMAW,2,IAW)
         CALL LCMPOF(IPSYS,'TF'//TEXT10,LL4F*NLF/2,2,ITF)
      ENDIF
*
* DIMENSION X
      IIMAX=IBASE(IMUX+LL4X-1)*NLF/2
      CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
      CALL LCMLEN(IPSYS,'XB',LENXB,ITYL)
      IF(LENXB.EQ.0) THEN
        CALL LCMIOF(IPTRK,'XB',IBX)
      ELSE
        CALL LCMIOF(IPSYS,'XB',IBX)
      ENDIF
      IF(CHEX) THEN
         IF(IPR.EQ.3) THEN
            IF(ISEG.GT.0) CALL MTBLD('XA'//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'XA'//TEXT10,IAX)
         ELSE
            CALL SETARA(BASE,IIMAX,IAX)
            CALL XDRSET(BASE(IAX),IIMAX,0.0)
         ENDIF
         CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPTRK,'FRZ',IFRZ)
         CALL SETARA(BASE,IIMAX,IXA)
         CALL PN3HWX(NBMIX,NBLOS,IELEM,ICOL,NLF,NVD,NAN,LX,LZ,LL4,LL4F,
     1   LL4W,LL4X,LL4Y,LL4Z,MAT,SIGT,SIGTI,SIDE,BASE(IZZ),BASE(IFRZ),
     2   BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),IBASE(IMUX),IBASE(IIPBX),
     3   LC,BASE(IR),BASE(IV),BASE(IBX),BASE(ITF),BASE(IAX),BASE(IXA))
         CALL LCMPOF(IPSYS,'X '//TEXT10,IIMAX,2,IXA)
         CALL LCMPOF(IPSYS,'XA'//TEXT10,IIMAX,2,IAX)
      ELSE
         IF(DIAG.OR.(IPR.NE.3)) THEN
            CALL SETARA(BASE,LL4F*NLF/2,ITF)
            CALL SETARA(BASE,IIMAX,IAX)
            CALL XDRSET(BASE(ITF),LL4F*NLF/2,0.0)
            CALL XDRSET(BASE(IAX),IIMAX,0.0)
         ELSE
            IF(ISEG.GT.0) CALL MTBLD('XA'//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPSYS,'XA'//TEXT10,IAX)
         ENDIF
         CALL SETARA(BASE,IIMAX,IXA)
         CALL PN3DXX(NBMIX,IELEM,ICOL,IDIM,NEL,NLF,NVD,NAN,LL4,LL4F,
     1   LL4X,SIGT,SIGTI,MAT,BASE(IVOL),BASE(IXX),BASE(IYY),BASE(IZZ),
     2   IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPBX),LC,BASE(IR),
     3   BASE(IV),BASE(IBX),BASE(ITF),BASE(IAX),BASE(IXA))
         IF(.NOT.DIAG) THEN
            CALL LCMPOF(IPSYS,'X '//TEXT10,IIMAX,2,IXA)
            CALL LCMPOF(IPSYS,'XA'//TEXT10,IIMAX,2,IAX)
         ELSE
*           IN DIAGONAL SYMMETRY CASE, DO NOT SAVE THE X-DIRECTED
*           ADI MATRIX COMPONENT SINCE IT IS EQUAL TO THE Y-DIRECTED
*           COMPONENT
            CALL RLSARA(BASE(IXA))
            CALL RLSARA(BASE(IAX))
         ENDIF
         CALL LCMPOF(IPSYS,'TF'//TEXT10,LL4F*NLF/2,2,ITF)
      ENDIF
*
* DIMENSION Y
      IF(CHEX) THEN
         IIMAY=IBASE(IMUY+LL4Y-1)*NLF/2
         IF(IPR.EQ.3) THEN
            IF(ISEG.GT.0) CALL MTBLD('YA'//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'YA'//TEXT10,IAY)
         ELSE
            CALL SETARA(BASE,IIMAY,IAY)
            CALL XDRSET(BASE(IAY),IIMAY,0.0)
         ENDIF
         CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
         CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
         CALL LCMLEN(IPSYS,'YB',LENYB,ITYL)
         IF(LENYB.EQ.0) THEN
            CALL LCMIOF(IPTRK,'YB',IBY)
         ELSE
            CALL LCMIOF(IPSYS,'YB',IBY)
         ENDIF
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPTRK,'FRZ',IFRZ)
         CALL SETARA(BASE,IIMAY,IYA)
         CALL PN3HWY(NBMIX,NBLOS,IELEM,ICOL,NLF,NVD,NAN,LX,LZ,LL4,LL4F,
     1   LL4W,LL4X,LL4Y,LL4Z,MAT,SIGT,SIGTI,SIDE,BASE(IZZ),BASE(IFRZ),
     2   BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),IBASE(IMUY),IBASE(IIPBY),
     3   LC,BASE(IR),BASE(IV),BASE(IBY),BASE(ITF),BASE(IAY),BASE(IYA))
         CALL LCMPOF(IPSYS,'Y '//TEXT10,IIMAY,2,IYA)
         CALL LCMPOF(IPSYS,'YA'//TEXT10,IIMAY,2,IAY)
      ELSE IF(LL4Y.GT.0) THEN
         IIMAY=IBASE(IMUY+LL4Y-1)*NLF/2
         IF(IPR.EQ.3) THEN
            IF(ISEG.GT.0) CALL MTBLD('YA'//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'YA'//TEXT10,IAY)
         ELSE
            CALL SETARA(BASE,IIMAY,IAY)
            CALL XDRSET(BASE(IAY),IIMAY,0.0)
         ENDIF
         CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
         CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
         CALL LCMIOF(IPTRK,'YB',IBY)
         CALL SETARA(BASE,IIMAY,IYA)
         CALL PN3DXY(NBMIX,IELEM,ICOL,NEL,NLF,NVD,NAN,LL4,LL4F,LL4X,
     1   LL4Y,SIGT,SIGTI,MAT,BASE(IVOL),BASE(IYY),IBASE(IKN),BASE(IQFR),
     2   IBASE(IMUY),IBASE(IIPBY),LC,BASE(IR),BASE(IV),BASE(IBY),
     3   BASE(ITF),BASE(IAY),BASE(IYA))
         CALL LCMPOF(IPSYS,'Y '//TEXT10,IIMAY,2,IYA)
         CALL LCMPOF(IPSYS,'YA'//TEXT10,IIMAY,2,IAY)
      ENDIF
*
* DIMENSION Z
      IF(LL4Z.GT.0) THEN
         IIMAZ=IBASE(IMUZ+LL4Z-1)*NLF/2
         IF(CHEX) THEN
            IF(IPR.EQ.3) THEN
               IF(ISEG.GT.0) CALL MTBLD('ZA'//TEXT10,IPTRK,IPSYS,1)
               CALL LCMIOF(IPSYS,'ZA'//TEXT10,IAZ)
            ELSE
               CALL SETARA(BASE,IIMAZ,IAZ)
               CALL XDRSET(BASE(IAZ),IIMAZ,0.0)
            ENDIF
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
            CALL LCMLEN(IPSYS,'ZB',LENZB,ITYL)
            IF(LENZB.EQ.0) THEN
               CALL LCMIOF(IPTRK,'ZB',IBZ)
            ELSE
               CALL LCMIOF(IPSYS,'ZB',IBZ)
            ENDIF
            CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
            CALL LCMIOF(IPTRK,'FRZ',IFRZ)
            CALL SETARA(BASE,IIMAZ,IZA)
            CALL PN3HWZ(NBMIX,NBLOS,IELEM,ICOL,NLF,NVD,NAN,LX,LZ,LL4,
     1      LL4F,LL4W,LL4X,LL4Y,LL4Z,MAT,SIGT,SIGTI,SIDE,BASE(IZZ),
     2      BASE(IFRZ),BASE(IQFR),IBASE(IZGLOB),IBASE(IKN),IBASE(IMUZ),
     3      IBASE(IIPBZ),LC,BASE(IR),BASE(IV),BASE(IBZ),BASE(ITF),
     4      BASE(IAZ),BASE(IZA))
         ELSE
            IF(IPR.EQ.3) THEN
               IF(ISEG.GT.0) CALL MTBLD('ZA'//TEXT10,IPTRK,IPSYS,1)
               CALL LCMIOF(IPSYS,'ZA'//TEXT10,IAZ)
            ELSE
               CALL SETARA(BASE,IIMAZ,IAZ)
               CALL XDRSET(BASE(IAZ),IIMAZ,0.0)
            ENDIF
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
            CALL LCMIOF(IPTRK,'ZB',IBZ)
            CALL SETARA(BASE,IIMAZ,IZA)
            CALL PN3DXZ(NBMIX,IELEM,ICOL,NEL,NLF,NVD,NAN,LL4,LL4F,LL4X,
     1      LL4Y,LL4Z,SIGT,SIGTI,MAT,BASE(IVOL),BASE(IZZ),IBASE(IKN),
     2      BASE(IQFR),IBASE(IMUZ),IBASE(IIPBZ),LC,BASE(IR),BASE(IV),
     3      BASE(IBZ),BASE(ITF),BASE(IAZ),BASE(IZA))
         ENDIF
         CALL LCMPOF(IPSYS,'Z '//TEXT10,IIMAZ,2,IZA)
         CALL LCMPOF(IPSYS,'ZA'//TEXT10,IIMAZ,2,IAZ)
      ENDIF
      CALL RLSARA(BASE(IVOL))
      CALL KDRCPU(TK2)
      IF(IMPX.GT.1) WRITE(6,'(/35H TRIASN: CPU TIME FOR SYSTEM MATRIX,
     1 11H ASSEMBLY =,F9.2,3H S.)') TK2-TK1
*----
*  PERFORM SUPERVECTORIZATION REBUILD OF THE COEFFICIENT MATRICES
*----
      IF(ISEG.GT.0) THEN
         IF(CHEX) CALL MTBLD('W '//TEXT10,IPTRK,IPSYS,3)
         IF(.NOT.DIAG) CALL MTBLD('X '//TEXT10,IPTRK,IPSYS,3)
         IF(LL4Y.GT.0) CALL MTBLD('Y '//TEXT10,IPTRK,IPSYS,3)
         IF(LL4Z.GT.0) CALL MTBLD('Z '//TEXT10,IPTRK,IPSYS,3)
         IF(CHEX) CALL MTBLD('WA'//TEXT10,IPTRK,IPSYS,3)
         IF(.NOT.DIAG) CALL MTBLD('XA'//TEXT10,IPTRK,IPSYS,3)
         IF(LL4Y.GT.0) CALL MTBLD('YA'//TEXT10,IPTRK,IPSYS,3)
         IF(LL4Z.GT.0) CALL MTBLD('ZA'//TEXT10,IPTRK,IPSYS,3)
      ENDIF
*----
*  MATRIX FACTORIZATIONS
*----
      IF(IPR.EQ.0) THEN
         CALL KDRCPU(TK1)
         CALL MTLDLF(TEXT10,IPTRK,IPSYS,ITY,IMPX)
         CALL KDRCPU(TK2)
         IF(IMPX.GT.1) WRITE(6,'(/34H TRIASN: CPU TIME FOR LDLT FACTORI,
     1   18HZATION OF MATRIX '',A10,2H''=,F9.2,3H S.)') TEXT10,TK2-TK1
      ENDIF
*----
*  RELEASE UNIT MATRICES
*----
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
*----
*  RELEASE TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IZZ))
      IF(CHEX) THEN
         CALL RLSARA(IBASE(IMUW))
      ELSE
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ENDIF
      IF(LL4Z.GT.0) CALL RLSARA(IBASE(IMUZ))
      IF(LL4Y.GT.0) CALL RLSARA(IBASE(IMUY))
      IF(.NOT.DIAG) CALL RLSARA(IBASE(IMUX))
      RETURN
      END
