*DECK TRICHK
      SUBROUTINE TRICHK (HNAMT,IPTRK,IPSYS,IDIM,DIAG,CHEX,IPR,LL4)
*
*-----------------------------------------------------------------------
*
*Purpose:
* partial consistency check for an ADI-splitted system matrix.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* HNAMT   name of the matrix to check.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* IDIM    number of dimensions.
* DIAG    diagonal symmetry flag for cartesian geometries.
* CHEX    hexagonal geometry flag.
* IPR     perturbation flag (if IPR.ne.0, matrix may contain
*         perturbation values).
* LL4     order of system matrices.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER HNAMT*10
      INTEGER IPTRK,IPSYS,IDIM,IPR,LL4
      LOGICAL DIAG,CHEX
*----
*  LOCAL VARIABLES
*----
      PARAMETER (EPSMAX=5.0E-5)
      CHARACTER TEXT10*10,HSMG*60,TEXT8*8
      COMMON BASE(1)
      INTEGER IBASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      TEXT10=HNAMT
*----
*  DIMENSION X
*----
      CALL SETARA(BASE,LL4,IXTT1)
      CALL SETARA(IBASE,LL4,IMUX)
      CALL SETARA(IBASE,LL4,IIPX)
      CALL LCMGET(IPTRK,'IPX',IBASE(IIPX))
      IF(.NOT.DIAG) THEN
         CALL LCMGET(IPTRK,'MUX',IBASE(IMUX))
         CALL LCMIOF(IPSYS,'X '//TEXT10,IXA11)
      ELSE
*        DIAGONAL SYMMETRY
         CALL LCMGET(IPTRK,'MUY',IBASE(IMUX))
         CALL LCMIOF(IPSYS,'Y '//TEXT10,IXA11)
      ENDIF
      DO 10 I=1,LL4
      IGAR=IBASE(IMUX+IBASE(IIPX+I-1)-1)
      BASE(IXTT1+I-1)=BASE(IXA11+IGAR-1)
      IF((IPR.EQ.0).AND.(BASE(IXTT1+I-1).EQ.0.0)) THEN
         WRITE (TEXT8,'(I8)') I
         CALL XABORT('TRICHK: ZERO ELEMENT ON DIAGONAL ELEMENT'//
     1   TEXT8//' OF MATRIX '//TEXT10//'.')
      ENDIF
   10 CONTINUE
      CALL RLSARA(IBASE(IIPX))
      CALL RLSARA(IBASE(IMUX))
      IF(IDIM.EQ.1) GO TO 50
*----
*  DIMENSION W
*----
      IF(CHEX) THEN
         CALL SETARA(IBASE,LL4,IMUW)
         CALL SETARA(IBASE,LL4,IIPW)
         CALL LCMGET(IPTRK,'MUW',IBASE(IMUW))
         CALL LCMGET(IPTRK,'IPW',IBASE(IIPW))
         CALL LCMIOF(IPSYS,'W '//TEXT10,IWA11)
         DO 20 I=1,LL4
         RR=BASE(IXTT1+I-1)
         IGAR=IBASE(IMUW+IBASE(IIPW+I-1)-1)
         IF(ABS(RR-BASE(IWA11+IGAR-1)).GT.ABS(RR)*EPSMAX) THEN
            WRITE(HSMG,'(8H: DIAGX(,I6,3H )=,1P,E12.5,7H DIAGW(,I6,
     1      3H )=,E12.5)') I,RR,I,BASE(IWA11+IGAR-1)
            CALL XABORT('TRICHK: W-AXIS INCONSISTENT ASSEMBLY(1)'//HSMG)
         ENDIF
   20    CONTINUE
         CALL RLSARA(IBASE(IIPW))
         CALL RLSARA(IBASE(IMUW))
      ENDIF
*----
*  DIMENSION Y
*----
      CALL SETARA(IBASE,LL4,IMUY)
      CALL SETARA(IBASE,LL4,IIPY)
      CALL LCMGET(IPTRK,'MUY',IBASE(IMUY))
      CALL LCMGET(IPTRK,'IPY',IBASE(IIPY))
      CALL LCMIOF(IPSYS,'Y '//TEXT10,IYA11)
      DO 30 I=1,LL4
      RR=BASE(IXTT1+I-1)
      IGAR=IBASE(IMUY+IBASE(IIPY+I-1)-1)
      IF(ABS(RR-BASE(IYA11+IGAR-1)).GT.ABS(RR)*EPSMAX) THEN
         WRITE(HSMG,'(8H: DIAGX(,I6,3H )=,1P,E12.5,7H DIAGY(,I6,3H )=,
     1   E12.5)') I,RR,I,BASE(IYA11+IGAR-1)
         CALL XABORT('TRICHK: Y-AXIS INCONSISTENT ASSEMBLY(1)'//HSMG)
      ENDIF
   30 CONTINUE
      CALL RLSARA(IBASE(IIPY))
      CALL RLSARA(IBASE(IMUY))
*----
*  DIMENSION Z
*----
      IF(IDIM.GT.2) THEN
         CALL SETARA(IBASE,LL4,IMUZ)
         CALL SETARA(IBASE,LL4,IIPZ)
         CALL LCMGET(IPTRK,'MUZ',IBASE(IMUZ))
         CALL LCMGET(IPTRK,'IPZ',IBASE(IIPZ))
         CALL LCMIOF(IPSYS,'Z '//TEXT10,IZA11)
         DO 40 I=1,LL4
         RR=BASE(IXTT1+I-1)
         IGAR=IBASE(IMUZ+IBASE(IIPZ+I-1)-1)
         IF(ABS(RR-BASE(IZA11+IGAR-1)).GT.ABS(RR)*EPSMAX) THEN
            WRITE(HSMG,'(8H: DIAGX(,I6,3H )=,1P,E12.5,7H DIAGZ(,I6,
     1      3H )=,E12.5)') I,RR,I,BASE(IZA11+IGAR-1)
            CALL XABORT('TRICHK: Z-AXIS INCONSISTENT ASSEMBLY(1)'//HSMG)
         ENDIF
   40    CONTINUE
         CALL RLSARA(IBASE(IIPZ))
         CALL RLSARA(IBASE(IMUZ))
      ENDIF
   50 CALL RLSARA(BASE(IXTT1))
      RETURN
      END
