*DECK TRIDIG
      SUBROUTINE TRIDIG(HNAME,IPTRK,IPSYS,IMPX,MAXMIX,NEL,IPR,MAT,
     1 VOL,SGD)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for the assembly of a cross section diagonal matrix.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* HNAME   name of the diagonal matrix.
* IPTRK   L_TRACK pointer to the TRIVAC tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* IMPX    print parameter. Equal to zero for no print.
* MAXMIX   dimension of matrix SGD.
* NEL     total number of finite elements.
* IPR     type of assembly:
*         =3: the new contribution is added to existing matrix.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* SGD     cross section per material mixture.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER HNAME*(*)
      INTEGER IPTRK,IPSYS,IMPX,MAXMIX,NEL,IPR,MAT(NEL)
      REAL VOL(NEL),SGD(MAXMIX)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER TEXT12*12
      LOGICAL CYLIND,CHEX
      INTEGER IBASE(1),ISTATE(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      ITYPE=ISTATE(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IELEM=ABS(ISTATE(9))
      LL4=ISTATE(11)
      ICHX=ISTATE(12)
      ISPLH=ISTATE(13)
      LX=ISTATE(14)
      LY=ISTATE(15)
      LZ=ISTATE(16)
      LL4F=ISTATE(25)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL SETARA(BASE,LX*LY*LZ,IXORZ)
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'ZZ',BASE(IXORZ))
         CALL LCMGET(IPTRK,'SIDE',SIDE)
      ELSE
         CALL LCMGET(IPTRK,'XX',BASE(IXORZ))
         CALL SETARA(BASE,LX*LY*LZ,IDD)
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      TEXT12=HNAME
      IF(IMPX.GT.0) WRITE(6,'(/37H TRIDIG: ASSEMBLY OF DIAGONAL MATRIX ,
     1 1H'',A12,2H''.)') TEXT12
*----
*  INITIALIZATION OF A DIAGONAL SYSTEM MATRIX
*----
      IF(IPR.EQ.3) THEN
         CALL LCMIOF(IPSYS,TEXT12,IVEC)
      ELSE
         CALL SETARA(BASE,LL4,IVEC)
         CALL XDRSET(BASE(IVEC),LL4,0.0)
      ENDIF
*----
*  COMPUTE THE DIAGONAL SYSTEM MATRIX
*----
      IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
*        VARIATIONAL COLLOCATION METHOD.
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL TRIASP(IELEM,MAXMIX,NEL,LL4,CYLIND,SGD,BASE(IXORZ),
     1   BASE(IDD),VOL,MAT,IBASE(IKN),LC,BASE(IT),BASE(ITS),BASE(IVEC))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
      ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*        MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL SETARA(BASE,4,IR)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'RH',BASE(IRH))
         CALL LCMGET(IPTRK,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK,' ',2)
         CALL TRIAHP(MAXKN,ISPLH,MAXMIX,NEL,LL4,SGD,SIDE,BASE(IXORZ),
     1   VOL,MAT,IBASE(IKN),BASE(IR),BASE(IRH),BASE(IRT),BASE(IVEC))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
         CALL RLSARA(BASE(IR))
      ELSE IF((ICHX.EQ.2).AND.CHEX) THEN
*        DUAL (THOMAS-RAVIART-SCHNEIDER) FINITE ELEMENT METHOD.
         CALL LCMIOF(IPTRK,'ZGLOB',IZGLOB)
         CALL LCMIOF(IPTRK,'FRZ',IFRZ)
         NBLOS=LX*LZ*ISPLH**2
         CALL TRIASH(IELEM,MAXMIX,LX,LZ,LL4,NBLOS,MAT,SIDE,BASE(IXORZ),
     1   BASE(IFRZ),SGD,IBASE(IKN),BASE(IZGLOB),BASE(IVEC))
      ELSE IF(.NOT.CHEX) THEN
*        DUAL FINITE ELEMENT METHOD.
         IDIM=1
         IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
         IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
         CALL SETARA(BASE,NEL,IGAR)
         CALL TRIASD(MAXKN,IELEM,ICHX,IDIM,MAXMIX,NEL,LL4,CYLIND,SGD,
     1   VOL,MAT,IBASE(IKN),BASE(IVEC),BASE(IGAR))
         CALL RLSARA(BASE(IGAR))
      ELSE IF(CHEX.AND.(ISPLH.EQ.1)) THEN
*        MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL TRIAHD(MAXMIX,NEL,LL4,SGD,VOL,MAT,BASE(IVEC))
      ELSE IF(CHEX.AND.(ISPLH.GT.1)) THEN
*        MESH CENTERED FINITE DIFFERENCES IN TRIANGULAR GEOMETRY.
         CALL LCMIOF(IPTRK,'IPW',IIPW)
         CALL TRIMTD(ISPLH,MAXMIX,NEL,LL4,VOL,MAT,SGD,IBASE(IKN),
     1   IBASE(IIPW),BASE(IVEC))
      ENDIF
*----
*  STORAGE OF THE DIAGONAL SYSTEM MATRIX
*----
      IF(ICHX.EQ.2) THEN
         CALL LCMPUT(IPSYS,TEXT12,LL4F,2,BASE(IVEC))
	 CALL RLSARA(BASE(IVEC))
      ELSE
         CALL LCMPOF(IPSYS,TEXT12,LL4,2,IVEC)
      ENDIF
*----
*  RELEASE TRIVAC SPECIFIC TRACKING INFORMATION
*----
      IF(.NOT.CHEX) CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IXORZ))
      CALL RLSARA(IBASE(IKN))
      RETURN
      END
