*DECK TRIPXX
      SUBROUTINE TRIPXX (IR,MAXKN,NEL,LL4,VOL,MAT,XSGD,XX,YY,ZZ,DD,KN,
     1 QFR,MUX,IPX,CYLIND,LC,T,TS,Q,QS,A11X)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of system matrices for a primal finite element method in 3-D.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IR      first dimension for matrix SGD.
* MAXKN   first dimension for matrix KN.
* NEL     total number of finite elements.
* LL4     order of system matrices.
* MAT     mixture index assigned to each element.
* VOL     volume of each element.
* XX      X-directed mesh spacings.
* YY      Y-directed mesh spacings.
* ZZ      Z-directed mesh spacings.
* DD      values used with a cylindrical geometry.
* KN      element-ordered unknown list.
* QFR     element-ordered boundary conditions.
* XSGD    nuclear properties, derivatives or first variations of
*         nuclear properties per material mixture:
*         XSGD(L,1): X-oriented diffusion coefficients;
*         XSGD(L,2): Y-oriented diffusion coefficients;
*         XSGD(L,3): Z-oriented diffusion coefficients;
*         XSGD(L,4): removal macroscopic cross section.
* MUX     X-oriented compressed storage mode indices.
* MUY     Y-oriented compressed storage mode indices.
* MUZ     Z-oriented compressed storage mode indices.
* IPX     X-oriented permutation matrices.
* IPY     Y-oriented permutation matrices.
* IPZ     Z-oriented permutation matrices.
* CYLIND  cylinderization flag (=.true. for cylindrical geometry).
* LC      order of the unit matrices.
* T       cartesian linear product vector.
* TS      cylindrical linear product vector.
* Q       cartesian stiffness matrix.
* QS      cylindrical stiffness matrix.
*
*Parameters: output
* A11X    X-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
* A11Y    Y-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
* A11Z    Z-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IR,MAXKN,NEL,LL4,MAT(NEL),KN(MAXKN),MUX(LL4),MUY(LL4),
     1 MUZ(LL4),IPX(LL4),IPY(LL4),IPZ(LL4),LC
      REAL VOL(NEL),XSGD(IR,4),XX(NEL),YY(NEL),ZZ(NEL),DD(NEL),
     1 QFR(6*NEL),T(LC),TS(LC),Q(LC,LC),QS(LC,LC),A11X(*),A11Y(*),
     2 A11Z(*)
      LOGICAL CYLIND
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION RR,QQX,QQY,QQZ,VOL0,VOL1,VOL2,VOL3,DX,DY,DZ,CRZ
      COMMON /ELEM2/LL,LCC,IJ1(125),IJ2(125),IJ3(125),ISR(6,25),
     1 Q3DP1(125,125),Q3DP2(125,125),Q3DP3(125,125),R3DP(125),
     2 Q3DC1(125,125),Q3DC2(125,125),Q3DC3(125,125),R3DC(125),
     3 R2DP(25),R2DC(25)
      SAVE /ELEM2/
*----
*  CALCULATION OF COMMON /ELEM2/
*----
      LCC=LC*LC
      LL=LC*LC*LC
      DO 5 L=1,LL
      L1=1+MOD(L-1,LC)
      L2=1+(L-L1)/LC
      L3=1+MOD(L2-1,LC)
      IJ1(L)=L1
      IJ2(L)=L3
    5 IJ3(L)=1+(L2-L3)/LC
*----
*  CALCULATION OF MATRIX ISR.
*----
      K1=0
      K2=0
      J1=0
      J2=0
      I1=0
      I2=0
      L=0
      DO 6 I=1,LC
      DO 6 J=1,LC
      DO 6 K=1,LC
      L=L+1
      IF(K.EQ.1) THEN
         K1=K1+1
         ISR(1,K1)=L
      ELSE IF(K.EQ.LC) THEN
         K2=K2+1
         ISR(2,K2)=L
      ENDIF
      IF(J.EQ.1) THEN
         J1=J1+1
         ISR(3,J1)=L
      ELSE IF(J.EQ.LC) THEN
         J2=J2+1
         ISR(4,J2)=L
      ENDIF
      IF(I.EQ.1) THEN
         I1=I1+1
         ISR(5,I1)=L
      ELSE IF(I.EQ.LC) THEN
         I2=I2+1
         ISR(6,I2)=L
      ENDIF
    6 CONTINUE
*----
*  CALCULATION OF 3-D MASS AND STIFFNESS MATRICES FROM TENSORIAL PRODUCT
*  OF 1-D MATRICES.
*----
      DO 20 I=1,LL
      I1=IJ1(I)
      I2=IJ2(I)
      I3=IJ3(I)
      DO 10 J=1,LL
      J1=IJ1(J)
      J2=IJ2(J)
      J3=IJ3(J)
      IF((I2.EQ.J2).AND.(I3.EQ.J3)) THEN
         Q3DP1(I,J)=Q(I1,J1)*T(I2)*T(I3)
         Q3DC1(I,J)=QS(I1,J1)*T(I2)*T(I3)
      ELSE
         Q3DP1(I,J)=0.0
         Q3DC1(I,J)=0.0
      ENDIF
      IF((I1.EQ.J1).AND.(I3.EQ.J3)) THEN
         Q3DP2(I,J)=T(I1)*Q(I2,J2)*T(I3)
         Q3DC2(I,J)=TS(I1)*Q(I2,J2)*T(I3)
      ELSE
         Q3DP2(I,J)=0.0
         Q3DC2(I,J)=0.0
      ENDIF
      IF((I1.EQ.J1).AND.(I2.EQ.J2)) THEN
         Q3DP3(I,J)=T(I1)*T(I2)*Q(I3,J3)
         Q3DC3(I,J)=TS(I1)*T(I2)*Q(I3,J3)
      ELSE
         Q3DP3(I,J)=0.0
         Q3DC3(I,J)=0.0
      ENDIF
   10 CONTINUE
      R3DP(I)=T(I1)*T(I2)*T(I3)
   20 R3DC(I)=TS(I1)*T(I2)*T(I3)
*----
*  CALCULATION OF 2-D MASS MATRICES FROM TENSORIAL PRODUCT OF 1-D
*  MATRICES.
*----
      DO 30 I=1,LC*LC
      I1=IJ1(I)
      I2=IJ2(I)
      R2DP(I)=T(I1)*T(I2)
   30 R2DC(I)=TS(I1)*T(I2)
*----
*  X-DIRECTED COUPLINGS.
*
*  ASSEMBLY OF MATRIX A11X.
*----
      NUM1=0
      NUM2=0
      DO 90 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 90
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 80
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      VOL1=VOL0/(DX*DX)
      VOL2=VOL0/(DY*DY)
      VOL3=VOL0/(DZ*DZ)
      DO 50 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 50
      INX1=IPX(IND1)
      KEY0=MUX(INX1)-INX1
      DO 40 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 40
      INX2=IPX(IND2)
      IF(INX2.EQ.INX1) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQX=Q3DP1(I,J)*VOL1
            QQY=Q3DP2(I,J)*VOL2
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INX2
         A11X(KEY)=A11X(KEY)+QQX*XSGD(L,1)+QQY*XSGD(L,2)+QQZ*XSGD(L,3)
      ELSE IF((INX2.LT.INX1).AND.(IJ2(I).EQ.IJ2(J)).AND.
     1 (IJ3(I).EQ.IJ3(J))) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
         ELSE
            QQX=Q3DP1(I,J)*VOL1
         ENDIF
         KEY=KEY0+INX2
         A11X(KEY)=A11X(KEY)+QQX*XSGD(L,1)
      ENDIF
   40 CONTINUE
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      KEY=KEY0+INX1
      A11X(KEY)=A11X(KEY)+RR*XSGD(L,4)
   50 CONTINUE
      DO 70 IC=1,6
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 70
      DO 60 I1=1,LCC
      I=ISR(IC,I1)
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 60
      INX1=IPX(IND1)
      KEY=MUX(INX1)
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R2DP(I1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R2DP(I1)
         ELSE
            CRZ=R2DC(I1)
         ENDIF
         RR=(R2DP(I1)+CRZ*DX/DD(K))
      ELSE
         RR=R2DP(I1)
      ENDIF
      A11X(KEY)=A11X(KEY)+RR*QFR1
   60 CONTINUE
   70 CONTINUE
   80 NUM1=NUM1+LL
      NUM2=NUM2+6
   90 CONTINUE
      RETURN
*----
*  Y-DIRECTED COUPLINGS.
*----
      ENTRY TRIPXY (IR,MAXKN,NEL,LL4,VOL,MAT,XSGD,XX,YY,ZZ,DD,KN,QFR,
     1 MUY,IPY,CYLIND,LC,T,TS,Q,QS,A11Y)
*----
*  ASSEMBLY OF MATRIX A11Y.
*----
      NUM1=0
      NUM2=0
      DO 180 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 180
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 170
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      VOL1=VOL0/(DX*DX)
      VOL2=VOL0/(DY*DY)
      VOL3=VOL0/(DZ*DZ)
      DO 140 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 140
      INY1=IPY(IND1)
      KEY0=MUY(INY1)-INY1
      DO 130 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 130
      INY2=IPY(IND2)
      IF(INY2.EQ.INY1) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQX=Q3DP1(I,J)*VOL1
            QQY=Q3DP2(I,J)*VOL2
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INY2
         A11Y(KEY)=A11Y(KEY)+QQX*XSGD(L,1)+QQY*XSGD(L,2)+QQZ*XSGD(L,3)
      ELSE IF((INY2.LT.INY1).AND.(IJ1(I).EQ.IJ1(J)).AND.
     1 (IJ3(I).EQ.IJ3(J))) THEN
         IF(CYLIND) THEN
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
         ELSE
            QQY=Q3DP2(I,J)*VOL2
         ENDIF
         KEY=KEY0+INY2
         A11Y(KEY)=A11Y(KEY)+QQY*XSGD(L,2)
      ENDIF
  130 CONTINUE
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      KEY=KEY0+INY1
      A11Y(KEY)=A11Y(KEY)+RR*XSGD(L,4)
  140 CONTINUE
      DO 160 IC=1,6
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 160
      DO 150 I1=1,LCC
      I=ISR(IC,I1)
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 150
      INY1=IPY(IND1)
      KEY=MUY(INY1)
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R2DP(I1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R2DP(I1)
         ELSE
            CRZ=R2DC(I1)
         ENDIF
         RR=(R2DP(I1)+DX*CRZ/DD(K))
      ELSE
         RR=R2DP(I1)
      ENDIF
      A11Y(KEY)=A11Y(KEY)+RR*QFR1
  150 CONTINUE
  160 CONTINUE
  170 NUM1=NUM1+LL
      NUM2=NUM2+6
  180 CONTINUE
      RETURN
*----
*  Z-DIRECTED COUPLINGS.
*----
      ENTRY TRIPXZ (IR,MAXKN,NEL,LL4,VOL,MAT,XSGD,XX,YY,ZZ,DD,KN,QFR,
     1 MUZ,IPZ,CYLIND,LC,T,TS,Q,QS,A11Z)
*----
*  ASSEMBLY OF MATRIX A11Z.
*----
      NUM1=0
      NUM2=0
      DO 270 K=1,NEL
      L=MAT(K)
      IF(L.EQ.0) GO TO 270
      VOL0=VOL(K)
      IF(VOL0.EQ.0.0) GO TO 260
      DX=XX(K)
      DY=YY(K)
      DZ=ZZ(K)
      VOL1=VOL0/(DX*DX)
      VOL2=VOL0/(DY*DY)
      VOL3=VOL0/(DZ*DZ)
      DO 230 I=1,LL
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 230
      INZ1=IPZ(IND1)
      KEY0=MUZ(INZ1)-INZ1
      DO 220 J=1,LL
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 220
      INZ2=IPZ(IND2)
      IF(INZ2.EQ.INZ1) THEN
         IF(CYLIND) THEN
            QQX=(Q3DP1(I,J)+Q3DC1(I,J)*DX/DD(K))*VOL1
            QQY=(Q3DP2(I,J)+Q3DC2(I,J)*DX/DD(K))*VOL2
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQX=Q3DP1(I,J)*VOL1
            QQY=Q3DP2(I,J)*VOL2
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INZ2
         A11Z(KEY)=A11Z(KEY)+QQX*XSGD(L,1)+QQY*XSGD(L,2)+QQZ*XSGD(L,3)
      ELSE IF((INZ2.LT.INZ1).AND.(IJ1(I).EQ.IJ1(J)).AND.
     1 (IJ2(I).EQ.IJ2(J))) THEN
         IF(CYLIND) THEN
            QQZ=(Q3DP3(I,J)+Q3DC3(I,J)*DX/DD(K))*VOL3
         ELSE
            QQZ=Q3DP3(I,J)*VOL3
         ENDIF
         KEY=KEY0+INZ2
         A11Z(KEY)=A11Z(KEY)+QQZ*XSGD(L,3)
      ENDIF
  220 CONTINUE
      IF(CYLIND) THEN
         RR=(R3DP(I)+R3DC(I)*DX/DD(K))*VOL0
      ELSE
         RR=R3DP(I)*VOL0
      ENDIF
      KEY=KEY0+INZ1
      A11Z(KEY)=A11Z(KEY)+RR*XSGD(L,4)
  230 CONTINUE
      DO 250 IC=1,6
      QFR1=QFR(NUM2+IC)
      IF(QFR1.EQ.0.0) GO TO 250
      DO 240 I1=1,LCC
      I=ISR(IC,I1)
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 240
      INZ1=IPZ(IND1)
      KEY=MUZ(INZ1)
      IF(CYLIND) THEN
         IF(IC.EQ.1) THEN
            CRZ=-0.5*R2DP(I1)
         ELSE IF(IC.EQ.2) THEN
            CRZ=0.5*R2DP(I1)
         ELSE
            CRZ=R2DC(I1)
         ENDIF
         RR=(R2DP(I1)+DX*CRZ/DD(K))
      ELSE
         RR=R2DP(I1)
      ENDIF
      A11Z(KEY)=A11Z(KEY)+RR*QFR1
  240 CONTINUE
  250 CONTINUE
  260 NUM1=NUM1+LL
      NUM2=NUM2+6
  270 CONTINUE
      RETURN
      END
