*DECK TRIRWW
      SUBROUTINE TRIRWW (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUW,
     1 IPW,A11W,ISPLH,R,Q,RH,QH,RT,QT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of system matrices for a mesh corner finite difference
* discretization in hexagonal geometry.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Benaboud
*
*Parameters: input
* IR      first dimension of matrix SGD.
* NEL     total number of finite elements.
* ll4     order of system matrices.
* VOL     volume of each element.
* MAT     mixture index assigned to each element.
* XSGD    nuclear properties, derivatives or first variations of
*         nuclear properties per material mixture:
*         XSGD(L,1): W-, X-, and Y-oriented diffusion coefficients;
*         XSGD(L,3): Z-oriented diffusion coefficients;
*         XSGD(L,4): removal macroscopic cross section.
* SIDE    side of an hexagon.
* ZZ      Z-directed mesh spacings.
* KN      element-ordered unknown list (dimensionned to KN(ICOF*NEL)
*         where ICOF=12 or 14).
* QFR     element-ordered boundary conditions.
* MUW     W-oriented compressed storage mode indices.
* MUX     X-oriented compressed storage mode indices.
* MUY     Y-oriented compressed storage mode indices.
* MUZ     Z-oriented compressed storage mode indices.
* IPW     W-oriented permutation matrices.
* IPX     X-oriented permutation matrices.
* IPY     Y-oriented permutation matrices.
* IPZ     Z-oriented permutation matrices.
* ISPLH   hexagonal mesh-splitting flag:
*         =1 for complete hexagons; >1 for triangular elements.
* R       unit matrix.
* Q       unit matrix.
* RH      unit matrix.
* QH      unit matrix.
* RT      unit matrix.
* QT      unit matrix.
*
*Parameters: output
* A11W    W-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
* A11X    X-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
* A11Y    Y-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
* A11Z    Z-oriented matrix corresponding to the divergence (i.e
*         leakage) and removal terms (should be initialized by the
*         calling program).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IR,NEL,LL4,MAT(NEL),KN(*),MUW(LL4),MUX(LL4),MUY(LL4),
     > MUZ(LL4),IPW(LL4),IPX(LL4),IPY(LL4),IPZ(LL4),ISPLH
      REAL VOL(NEL),XSGD(IR,4),SIDE,ZZ(NEL),QFR(8*NEL),A11W(*),A11X(*),
     > A11Y(*),A11Z(*),R(2,2),Q(2,2),RH(6,6),QH(6,6),RT(3,3),QT(3,3)
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION RR,QQP,QQZ,VOL0,VOL1,DZ
      DOUBLE PRECISION QTHP(14,14),QTHZ(14,14),RTHG(14,14),
     >           HW(14,14),HX(14,14),HY(14,14),HZ(14,14)
      INTEGER IJ1(14),IJ2(14),ISR(8,25),ILIEN(6,3),IJ27(14),ISRH(8,6),
     > ISRT(8,7),IJ16(12),IJ26(12),IJ17(14)
      REAL HL(2,2),R2DP(4),RFAC(28,7),RH2(7,7),QH2(7,7),RF6(24,6),
     > RF7(28,7)
      DATA HL / 1.0,2*0.0,1.0/
      DATA R2DP / 4*0.25 /
      DATA ILIEN/6*4,2,1,5,6,7,3,1,5,6,7,3,2/
      DATA IJ16,IJ26 /1,2,3,4,5,6,1,2,3,4,5,6,6*1,6*2/
      DATA IJ17,IJ27 /1,2,3,4,5,6,7,1,2,3,4,5,6,7,7*1,7*2/
      DATA ISRT/2,1,5,6,7,3,1,8,1,5,6,7,3,2,2,9,9,8,12,13,14,10,3,10,
     >          8,12,13,14,10,9,4,11,6*0,5,12,6*0,6,13,6*0,7,14/
      DATA ISRH/2,1,4,5,6,3,1,7,1,4,5,6,3,2,2,8,8,7,10,11,12,9,3,9,
     >          7,10,11,12,9,8,4,10,6*0,5,11,6*0,6,12/
      DATA RF6/
     >1.0,0.0,0.0,0.0,0.0,0.0,1.0,1.0,0.0,0.0,1.0,0.5,
     >1.0,0.0,1.0,1.0,0.0,0.5,1.0,0.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,1.0,0.5,0.0,1.0,1.0,0.0,0.0,0.5,1.0,
     >0.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,0.5,1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,
     >1.0,0.0,1.0,0.5,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,
     >0.0,1.0,0.5,1.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >1.0,0.0,0.5,1.0,0.0,1.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >0.0,0.5,1.0,1.0,1.0,0.0,1.0,0.5,0.0,0.0,1.0,1.0,
     >0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,
     >0.0,0.0,0.0,0.0,0.0,1.0,0.5,1.0,0.0,0.0,1.0,1.0,
     >0.5,0.0,1.0,1.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0/
      DATA RF7/
     >1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,1.0,0.0,0.5,0.0,1.0,0.5,
     >1.0,0.0,1.0,0.5,1.0,0.0,0.5,1.0,0.0,0.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,0.5,1.0,0.5,0.0,1.0,1.0,0.0,0.5,0.0,0.5,1.0,
     >0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,
     >0.0,1.0,1.0,0.5,0.5,1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,
     >1.0,0.0,1.0,0.5,0.5,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,
     >0.0,0.5,0.5,1.0,0.5,0.5,0.0,0.5,0.5,0.0,1.0,0.0,0.5,0.5,
     >0.5,0.0,0.5,1.0,0.5,0.0,0.5,0.0,0.0,0.0,1.0,0.0,0.0,0.0,
     >0.0,1.0,0.5,0.5,1.0,1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >1.0,0.0,0.5,0.5,1.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,
     >0.0,0.5,1.0,0.5,1.0,1.0,0.0,1.0,0.5,0.0,0.5,0.0,1.0,1.0,
     >0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,
     >0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.5,1.0,0.0,0.5,0.0,1.0,1.0,
     >0.5,0.0,1.0,0.5,1.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0/
      SAVE LL,LC,ISR,HW,HX,HY,HZ,RTHG,QTHP,QTHZ
*----
*  COMPUTE THE HEXAGONAL MASS (RH2) AND STIFFNESS (QH2) MATRICES
*----
      IF(ISPLH.EQ.1) THEN
         LC=6
         DO 10 I=1,8
         DO 10 J=1,6
   10    ISR(I,J)=ISRH(I,J)
         DO 20 I=1,2*LC
         IJ1(I)=IJ16(I)
   20    IJ2(I)=IJ26(I)
         DO 30 I=1,4*LC
         DO 30 J=1,LC
   30    RFAC(I,J)=RF6(I,J)
         DO 40 I=1,LC
         DO 40 J=1,LC
         RH2(I,J)=RH(I,J)
   40    QH2(I,J)=QH(I,J)
      ELSE
         LC=7
         DO 50 I=1,8
         DO 50 J=1,7
   50    ISR(I,J)=ISRT(I,J)
         DO 60 I=1,2*LC
         IJ1(I)=IJ17(I)
   60    IJ2(I)=IJ27(I)
         DO 70 I=1,4*LC
         DO 70 J=1,LC
   70    RFAC(I,J)=RF7(I,J)
         DO 75 I=1,LC
         DO 75 J=1,LC
         RH2(I,J)=0.0
   75    QH2(I,J)=0.0
         DO 80 K=1,6
         DO 80 I=1,3
         NUMI=ILIEN(K,I)
         DO 80 J=1,3
         NUMJ=ILIEN(K,J)
         RH2(NUMI,NUMJ)=RH2(NUMI,NUMJ)+RT(I,J)
   80    QH2(NUMI,NUMJ)=QH2(NUMI,NUMJ)+QT(I,J)
      ENDIF
      LL=2*LC
      DO 90 I=1,LL
      I1=IJ1(I)
      I2=IJ2(I)
      DO 90 J=1,LL
      J1=IJ1(J)
      J2=IJ2(J)
      HW(I,J)  =RFAC(I1     ,J1) * HL(I2,J2)
      HX(I,J)  =RFAC(I1+LC  ,J1) * HL(I2,J2)
      HY(I,J)  =RFAC(I1+2*LC,J1) * HL(I2,J2)
      HZ(I,J)  =RFAC(I1+3*LC,J1)
      RTHG(I,J)=RH2(I1,J1) * R(I2,J2)
      QTHP(I,J)=QH2(I1,J1) * R(I2,J2)
   90 QTHZ(I,J)=RH2(I1,J1) * Q(I2,J2)
*----
*  ASSEMBLY OF MATRIX A11W
*----
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 160 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 160
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 150
         DZ=ZZ(K)
         DO 110 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 110
            KEY0=MUW(INW1)-INW1
            DO 100 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 100
               IF(INW2.EQ.INW1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INW2
                  A11W(KEY)=A11W(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INW2.LT.INW1).AND.(HW(I,J).NE.0.0)) THEN
                  QQP=QTHP(I,J)*HW(I,J)*DZ
                  KEY=KEY0+INW2
                  A11W(KEY)=A11W(KEY)+QQP*XSGD(L,1)
               ENDIF
  100       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INW1
            A11W(KEY)=A11W(KEY)+RR*XSGD(L,4)
  110    CONTINUE
         DO 140 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 140
            IF(IC.LT.7) THEN
               DO 120 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 120
                  KEY=MUW(INW1)
                  RR=R2DP(I1)
                  A11W(KEY)=A11W(KEY)+RR*QFR1
  120          CONTINUE
            ELSE
               DO 130 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 130
                  KEY=MUW(INW1)
                  RR=RTHG(I1,I1)
                  A11W(KEY)=A11W(KEY)+RR*QFR1
  130          CONTINUE
            ENDIF
  140    CONTINUE
  150    NUM1=NUM1+LL
         NUM2=NUM2+8
  160 CONTINUE
      RETURN
*
      ENTRY TRIRWX (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUX,IPX,A11X)
*----
*  ASSEMBLY OF MATRIX A11X
*----
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 230 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 230
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 220
         DZ=ZZ(K)
         DO 180 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 180
            INX1=IPX(INW1)
            KEY0=MUX(INX1)-INX1
            DO 170 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 170
               INX2=IPX(INW2)
               IF(INX2.EQ.INX1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INX2
                  A11X(KEY)=A11X(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INX2.LT.INX1).AND.(HX(I,J).NE.0.0)) THEN
                  QQP=QTHP(I,J)*HX(I,J)*DZ
                  KEY=KEY0+INX2
                  A11X(KEY)=A11X(KEY)+QQP*XSGD(L,1)
               ENDIF
  170       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INX1
            A11X(KEY)=A11X(KEY)+RR*XSGD(L,4)
  180    CONTINUE
         DO 210 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 210
            IF(IC.LT.7) THEN
               DO 190 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 190
                  INX1=IPX(INW1)
                  KEY=MUX(INX1)
                  RR=R2DP(I1)
                  A11X(KEY)=A11X(KEY)+RR*QFR1
  190          CONTINUE
            ELSE
               DO 200 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 200
                  INX1=IPX(INW1)
                  KEY=MUX(INX1)
                  RR=RTHG(I1,I1)
                  A11X(KEY)=A11X(KEY)+RR*QFR1
  200          CONTINUE
            ENDIF
  210    CONTINUE
  220    NUM1=NUM1+LL
         NUM2=NUM2+8
  230 CONTINUE
      RETURN
*
      ENTRY TRIRWY (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUY,IPY,A11Y)
*----
*  ASSEMBLY OF MATRIX A11Y
*----
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 300 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 300
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 290
         DZ=ZZ(K)
         DO 250 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 250
            INY1=IPY(INW1)
            KEY0=MUY(INY1)-INY1
            DO 240 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 240
               INY2=IPY(INW2)
               IF(INY2.EQ.INY1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INY2
                  A11Y(KEY)=A11Y(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INY2.LT.INY1).AND.(HY(I,J).NE.0.0)) THEN
                  QQP=QTHP(I,J)*HY(I,J)*DZ
                  KEY=KEY0+INY2
                  A11Y(KEY)=A11Y(KEY)+QQP*XSGD(L,1)
               ENDIF
  240       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INY1
            A11Y(KEY)=A11Y(KEY)+RR*XSGD(L,4)
  250    CONTINUE
         DO 280 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 280
            IF(IC.LT.7) THEN
               DO 260 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 260
                  INY1=IPY(INW1)
                  KEY=MUY(INY1)
                  RR=R2DP(I1)
                  A11Y(KEY)=A11Y(KEY)+RR*QFR1
  260          CONTINUE
            ELSE
               DO 270 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 270
                  INY1=IPY(INW1)
                  KEY=MUY(INY1)
                  RR=RTHG(I1,I1)
                  A11Y(KEY)=A11Y(KEY)+RR*QFR1
  270          CONTINUE
            ENDIF
  280    CONTINUE
  290    NUM1=NUM1+LL
         NUM2=NUM2+8
  300 CONTINUE
      RETURN
*
      ENTRY TRIRWZ (IR,NEL,LL4,VOL,MAT,XSGD,SIDE,ZZ,KN,QFR,MUZ,IPZ,A11Z)
*----
*  ASSEMBLY OF MATRIX A11Z
*----
      NUM1=0
      NUM2=0
      VOL1=SIDE*SIDE
      DO 360 K=1,NEL
         L=MAT(K)
         IF(L.EQ.0) GO TO 360
         VOL0=VOL(K)
         IF(VOL0.EQ.0.0) GO TO 350
         DZ=ZZ(K)
         DO 320 I=1,LL
            INW1=KN(NUM1+I)
            IF(INW1.EQ.0) GO TO 320
            INZ1=IPZ(INW1)
            KEY0=MUZ(INZ1)-INZ1
            DO 310 J=1,LL
               INW2=KN(NUM1+J)
               IF(INW2.EQ.0) GO TO 310
               INZ2=IPZ(INW2)
               IF(INZ2.EQ.INZ1) THEN
                  QQP=QTHP(I,J)*DZ
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INZ2
                  A11Z(KEY)=A11Z(KEY)+QQP*XSGD(L,1)+QQZ*XSGD(L,3)
               ELSE IF((INZ2.LT.INZ1).AND.(HZ(I,J).NE.0.0)) THEN
                  QQZ=QTHZ(I,J)*VOL1/DZ
                  KEY=KEY0+INZ2
                  A11Z(KEY)=A11Z(KEY)+QQZ*XSGD(L,1)
               ENDIF
  310       CONTINUE
            RR=RTHG(I,I)*VOL1*DZ
            KEY=KEY0+INZ1
            A11Z(KEY)=A11Z(KEY)+RR*XSGD(L,4)
  320    CONTINUE
         DO 340 IC=1,8
            QFR1=QFR(NUM2+IC)
            IF(QFR1.EQ.0.0) GO TO 340
            IF(IC.LT.7) THEN
               DO 330 I1=1,4
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 330
                  INZ1=IPZ(INW1)
                  KEY=MUZ(INZ1)
                  RR=R2DP(I1)
                  A11Z(KEY)=A11Z(KEY)+RR*QFR1
  330          CONTINUE
            ELSE
               DO 335 I1=1,LC
                  I=ISR(IC,I1)
                  INW1=KN(NUM1+I)
                  IF(INW1.EQ.0) GO TO 335
                  INZ1=IPZ(INW1)
                  KEY=MUZ(INZ1)
                  RR=RTHG(I1,I1)
                  A11Z(KEY)=A11Z(KEY)+RR*QFR1
  335          CONTINUE
            ENDIF
  340    CONTINUE
  350    NUM1=NUM1+LL
         NUM2=NUM2+8
  360 CONTINUE
      RETURN
      END
