*DECK TRIVAA
      SUBROUTINE TRIVAA(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* TRIVAC type (3-D and ADI) system matrix assembly operator.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): create or modification type(L_SYSTEM);
*         HENTRY(2): read-only type(L_MACROLIB) (unperturbed);
*         HENTRY(3): read-only type(L_TRACK);
*         HENTRY(4): optional read-only type(L_MACROLIB) (perturbed).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NENTRY                  	
      CHARACTER HENTRY(NENTRY)*12
      INTEGER IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      CHARACTER TEXT4*4,TEXT11*12,TEXT12*12,HSMG*131,TITLE*72,CNAM*12
      DOUBLE PRECISION DFLOTT
      INTEGER IGP(NSTATE),IPAR(NSTATE),ITR(NSTATE),KCHAR(18),IBASE(1)
      LOGICAL LDIFF
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION.
*----
      IF(NENTRY.LE.2) CALL XABORT('TRIVAA: THREE PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('TRIVAA: L'
     1 //'CM OBJECT EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('TRIVAA: E'
     1 //'NTRY IN CREATE OR MODIFICATION MODE EXPECTED.')
      IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2)))
     1 CALL XABORT('TRIVAA: LCM OBJECT IN READ-ONLY MODE EXPECTED AT F'
     2 //'IRST RHS.')
      IF((JENTRY(2).NE.2).OR.((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)))
     1 CALL XABORT('TRIVAA: LCM OBJECT IN READ-ONLY MODE EXPECTED AT S'
     2 //'ECOND RHS.')
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(TEXT11,'(3A4)') (KCHAR(I),I=1,3)
      IF(TEXT11.NE.'L_TRACK') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('TRIVAA: SIGNATURE OF '//TEXT12//' IS '//TEXT11//
     1   '. L_TRACK EXPECTED.')
      ENDIF
      CALL LCMGET(KENTRY(3),'TRACK-TYPE',KCHAR)
      WRITE(TEXT11,'(3A4)') (KCHAR(I),I=1,3)
      IF(TEXT11.NE.'TRIVAC') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('TRIVAA: TRACK-TYPE OF '//TEXT12//' IS '//TEXT11
     1   //'. TRIVAC EXPECTED.')
      ENDIF
      TEXT11='L_SYSTEM'
      IPSYS=KENTRY(1)
      READ(TEXT11,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'SIGNATURE',3,3,KCHAR)
      IPMACR=KENTRY(2)
      IPTRK=KENTRY(3)
      TEXT12=HENTRY(2)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'LINK.MACRO',3,3,KCHAR)
      TEXT12=HENTRY(3)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'LINK.TRACK',3,3,KCHAR)
*----
*  RECOVER GENERAL TRACKING INFORMATION.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      NEL=IGP(1)
      NLF=IGP(30)
      ISCAT=IGP(32)
      LDIFF=(ISCAT.LT.0)
      ISCAT=ABS(ISCAT)
      IF((NLF.NE.0).AND.(IGP(31).NE.1)) CALL XABORT('TRIVAA: ONLY SPN '
     1 //'DISCRETIZATIONS ARE ALLOWED.')
      ITY=2
      IF(IGP(12).EQ.2) ITY=3
      CALL SETARA(IBASE,NEL,IMAT)
      CALL SETARA(BASE,NEL,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPTRK,'TITLE',KCHAR)
         WRITE(TITLE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
         TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
*----
*  RECOVER MACROLIB PARAMETERS.
*----
      CALL LCMGET(IPMACR,'SIGNATURE',KCHAR)
      WRITE(TEXT11,'(3A4)') (KCHAR(I),I=1,3)
      IF(TEXT11.NE.'L_MACROLIB') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('TRIVAA: SIGNATURE OF '//TEXT12//' IS '//TEXT11//
     1   '. L_MACROLIB EXPECTED.')
      ENDIF
      CALL LCMGET(IPMACR,'STATE-VECTOR',IPAR)
      NGRP=IPAR(1)
      NBMIX=IPAR(2)
      NANI=IPAR(3)
      NBFIS=IPAR(4)
      IF(IGP(4).GT.NBMIX) THEN
         WRITE(HSMG,'(46HTRIVAA: THE NUMBER OF MIXTURES IN THE TRACKING,
     1   2H (,I5,51H) IS GREATER THAN THE NUMBER OF MIXTURES IN THE MAC,
     2   7HROLIB (,I5,2H).)') IGP(4),NBMIX
         CALL XABORT(HSMG)
      ENDIF
*
      IMPX=1
      IASM=0
      IPR=0
      IUNIT=0
      IOVEL=0
      NSTEP=0
   10 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 30
      IF(INDIC.NE.3) CALL XABORT('TRIVAA: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('TRIVAA: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'SKIP') THEN
*        OPTION TO SKIP THE SYSTEM MATRIX ASSEMBLY (DO NOT SKIP THE
*        LDLT FACTORIZATION).
         IASM=1
      ELSE IF(TEXT4.EQ.'DERI') THEN
         IPR=1
         WRITE(6,'(/43H TRIVAA: USE DERIVATIVE OF SYSTEM MATRICES.)')
      ELSE IF(TEXT4.EQ.'PERT') THEN
         IPR=2
         WRITE(6,'(/41H TRIVAA: PERTURBATION OF SYSTEM MATRICES.)')
      ELSE IF(TEXT4.EQ.'UNIT') THEN
*       COMPUTE THE UNITARY WEIGHTING MATRIX.
        IUNIT=1
        CALL SETARA(BASE,NBMIX,IUN)
        DO 20 I=1,NBMIX
   20   BASE(IUN+I-1)=1.0
        CALL TRIDIG('RM',IPTRK,IPSYS,IMPX,NBMIX,NEL,0,IBASE(IMAT),
     1  BASE(IVOL),BASE(IUN))
        CALL RLSARA(BASE(IUN))
      ELSE IF(TEXT4.EQ.'OVEL') THEN
*        COMPUTE THE RECIPROCAL NEUTRON VELOCITIES MATRIX.
         IOVEL=1
         JPMACR=LCMGID(IPMACR,'GROUP')
         DO 25 IGR=1,NGRP
         CALL SETARA(BASE,NBMIX,IVII)
         KPMACR=LCMGIL(JPMACR,IGR)
         CALL LCMLEN(KPMACR,'OVERV',LENGT,ITYLCM)
         IF(LENGT.EQ.0) THEN
            CALL XABORT('TRIVAA: NO ''VELOCITY'' INFORMATION.')
         ELSE IF(LENGT.GT.NBMIX) THEN
            CALL XABORT('TRIVAA: INVALID LENGTH FOR ''VELOCITY'' IN'
     1      //'FORMATION.')
         ENDIF
         CALL LCMGET(KPMACR,'OVERV',BASE(IVII))
         WRITE (CNAM,'(1HV,2I3.3)') IGR,IGR
         CALL TRIDIG(CNAM,IPTRK,IPSYS,IMPX,NBMIX,NEL,0,IBASE(IMAT),
     1   BASE(IVOL),BASE(IVII))
         CALL RLSARA(BASE(IVII))
   25    CONTINUE
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 30
      ELSE
         CALL XABORT('TRIVAA: '//TEXT4//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 10
*----
*  2-MACROLIBS PERTURBATION CALCULATION.
*----
   30 IF(IPR.GT.0) THEN
         IF(NENTRY.LE.3) CALL XABORT('TRIVAA: 4 PARAMETERS EXPECTED WIT'
     1   //'H DERI OR PERT OPTIONS.')
         IF((JENTRY(4).NE.2).OR.((IENTRY(4).NE.1).AND.(IENTRY(4).NE.2)))
     1   CALL XABORT('TRIVAA: LINKED LIST OR XSM FILE IN READ-ONLY MODE'
     2   //' EXPECTED AT THIRD RHS.')
         IPMACP=KENTRY(4)
         CALL LCMGET(IPMACP,'SIGNATURE',KCHAR)
         WRITE(TEXT11,'(3A4)') (KCHAR(I),I=1,3)
         IF(TEXT11.NE.'L_MACROLIB') THEN
            TEXT12=HENTRY(4)
            CALL XABORT('TRIVAA: SIGNATURE OF '//TEXT12//' IS '
     1      //TEXT11//'. L_MACROLIB EXPECTED.')
         ENDIF
         CALL LCMGET(IPMACP,'STATE-VECTOR',IPAR)
         NSTEP=IPAR(11)
         IF((IPAR(1).NE.NGRP).OR.(IPAR(2).GT.NBMIX)) THEN
            WRITE(HSMG,'(43HTRIVAA: INCONSISTENT PERTURBATION MACROLIB ,
     1      1H'',A12,8H''. NGRP=,2I5,7H NBMIX=,2I9)') HENTRY(4),IPAR(1),
     2      NGRP,IPAR(2),NBMIX
            CALL XABORT(HSMG)
         ENDIF
      ENDIF
*----
*  SET THE STATE VECTOR FOR THE L_SYSTEM OBJECT
*----
      CALL XDISET(ITR,NSTATE,0)
      ITR(1)=NGRP
      ITR(2)=IGP(11)
      ITR(3)=0
      ITR(4)=ITY
      IF((NLF.GT.0).AND.(ITY.GE.3)) ITR(4)=10+ITR(4)
      IF(IUNIT.EQ.1) ITR(5)=1
      ITR(6)=NSTEP
      ITR(7)=NBMIX
      NAN=MIN(ISCAT,NANI)
      ITR(8)=NAN+1
      CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,ITR)
*----
*  SYSTEM MATRIX ASSEMBLY.
*----
      IF((IASM.EQ.0).AND.(IPR.EQ.0)) THEN
         CALL SETARA(BASE,NBMIX*NGRP,IWORK)
         CALL SETARA(BASE,NBMIX,IIJJ)
         CALL SETARA(BASE,NBMIX,INJJ)
         CALL SETARA(BASE,NBMIX,IIPOS)
         CALL SETARA(BASE,NBMIX*NBFIS*NGRP,ICHI)
         CALL SETARA(BASE,NBMIX*NBFIS,IZUFIS)
         IF(NLF.EQ.0) THEN
*           DIFFUSION THEORY.
            CALL SETARA(BASE,NBMIX*4,ISGD)
            CALL TRISYS(IPTRK,IPMACR,IPMACR,IPSYS,IMPX,NGRP,NEL,NBFIS,
     1      IPR,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),BASE(ISGD),
     2      BASE(IWORK),BASE(IIJJ),BASE(INJJ),BASE(IIPOS),BASE(ICHI),
     3      BASE(IZUFIS))
            CALL RLSARA(BASE(ISGD))
         ELSE
*           SIMPLIFIED PN THEORY.
            CALL SETARA(BASE,NBMIX*2*(NAN+1),ISGD)
            CALL SETARA(BASE,NGRP,IIND)
            CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCAT)
            CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCATI)
            CALL TRISPS(IPTRK,IPMACR,IPMACR,IPSYS,IMPX,NGRP,NEL,NLF,
     1      NAN,NBFIS,LDIFF,IPR,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),
     2      BASE(ISGD),BASE(IWORK),BASE(IIJJ),BASE(INJJ),BASE(IIPOS),
     3      BASE(ICHI),BASE(IZUFIS),BASE(IIND),BASE(IRCAT),BASE(IRCATI))
            CALL RLSARA(BASE(IRCATI))
            CALL RLSARA(BASE(IRCAT))
            CALL RLSARA(BASE(IIND))
            CALL RLSARA(BASE(ISGD))
	 ENDIF
         CALL RLSARA(BASE(IZUFIS))
         CALL RLSARA(BASE(ICHI))
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IWORK))
      ELSE IF((IASM.EQ.1).AND.(IPR.EQ.0)) THEN
*        PERFORM FACTORIZATION WITHOUT ASSEMBLY.
         DO 40 I=1,NGRP
         WRITE(TEXT11,'(1HA,2I3.3)') I,I
         CALL MTLDLF(TEXT11,IPTRK,IPSYS,ITY,IMPX)
   40    CONTINUE
      ELSE IF((IPR.GT.0).AND.(NSTEP.EQ.0)) THEN
*        ASSEMBLY OF PERTURBED SYSTEM MATRICES (NO STEP DIRECTORIES).
         CALL SETARA(BASE,NBMIX*NGRP,IWORK)
         CALL SETARA(BASE,NBMIX,IIJJ)
         CALL SETARA(BASE,NBMIX,INJJ)
         CALL SETARA(BASE,NBMIX,IIPOS)
         CALL SETARA(BASE,NBMIX*NBFIS*NGRP,ICHI)
         CALL SETARA(BASE,NBMIX*NBFIS,IZUFIS)
         IF(NLF.EQ.0) THEN
*           DIFFUSION THEORY.
            CALL SETARA(BASE,NBMIX*4,ISGD)
            CALL SETARA(BASE,NBMIX*4,IDSGD)
            CALL TRISYS(IPTRK,IPMACR,IPMACP,IPSYS,IMPX,NGRP,NEL,NBFIS,
     1      IPR,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),BASE(IDSGD),
     2      BASE(IWORK),BASE(IIJJ),BASE(INJJ),BASE(IIPOS),BASE(ICHI),
     3      BASE(IZUFIS))
            CALL RLSARA(BASE(IDSGD))
            CALL RLSARA(BASE(ISGD))
         ELSE
*           SIMPLIFIED PN THEORY.
            CALL SETARA(BASE,NBMIX*2*(NAN+1),ISGD)
            CALL SETARA(BASE,NBMIX*2*(NAN+1),IDSGD)
            CALL SETARA(BASE,NGRP,IIND)
            CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCAT)
            CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCATI)
            CALL TRISPS(IPTRK,IPMACR,IPMACP,IPSYS,IMPX,NGRP,NEL,NLF,
     1      NAN,NBFIS,LDIFF,IPR,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),
     2      BASE(IDSGD),BASE(IWORK),BASE(IIJJ),BASE(INJJ),BASE(IIPOS),
     3      BASE(ICHI),BASE(IZUFIS),BASE(IIND),BASE(IRCAT),BASE(IRCATI))
            CALL RLSARA(BASE(IRCATI))
            CALL RLSARA(BASE(IRCAT))
            CALL RLSARA(BASE(IIND))
            CALL RLSARA(BASE(IDSGD))
            CALL RLSARA(BASE(ISGD))
	 ENDIF
         CALL RLSARA(BASE(IZUFIS))
         CALL RLSARA(BASE(ICHI))
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IWORK))
      ELSE IF(NSTEP.GT.0) THEN
*        ASSEMBLY OF PERTURBED SYSTEM MATRICES (WITH STEP DIRECTORIES).
         JPMACR=LCMGID(IPMACP,'STEP')
         JPSYS=LCMLID(IPSYS,'STEP',NSTEP)
         DO 50 ISTEP=1,NSTEP
         KPMACR=LCMGIL(JPMACR,ISTEP)
         KPSYS=LCMDIL(JPSYS,ISTEP)
         CALL SETARA(BASE,NBMIX*NGRP,IWORK)
         CALL SETARA(BASE,NBMIX,IIJJ)
         CALL SETARA(BASE,NBMIX,INJJ)
         CALL SETARA(BASE,NBMIX,IIPOS)
         CALL SETARA(BASE,NBMIX*NBFIS*NGRP,ICHI)
         CALL SETARA(BASE,NBMIX*NBFIS,IZUFIS)
         IF(NLF.EQ.0) THEN
*           DIFFUSION THEORY.
            CALL SETARA(BASE,NBMIX*4,ISGD)
            CALL SETARA(BASE,NBMIX*4,IDSGD)
            CALL TRISYS(IPTRK,IPMACR,KPMACR,KPSYS,IMPX,NGRP,NEL,NBFIS,
     1      IPR,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),BASE(IDSGD),
     2      BASE(IWORK),BASE(IIJJ),BASE(INJJ),BASE(IIPOS),
     3      BASE(ICHI),BASE(IZUFIS))
            CALL RLSARA(BASE(IDSGD))
            CALL RLSARA(BASE(ISGD))
         ELSE
*           SIMPLIFIED PN THEORY.
            CALL SETARA(BASE,NBMIX*2*(NAN+1),ISGD)
            CALL SETARA(BASE,NBMIX*2*(NAN+1),IDSGD)
            CALL SETARA(BASE,NGRP,IIND)
            CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCAT)
            CALL SETARA(BASE,NGRP*NGRP*NBMIX,IRCATI)
            CALL TRISPS(IPTRK,IPMACR,KPMACR,KPSYS,IMPX,NGRP,NEL,NLF,
     1      NAN,NBFIS,LDIFF,IPR,IBASE(IMAT),BASE(IVOL),NBMIX,BASE(ISGD),
     2      BASE(IDSGD),BASE(IWORK),BASE(IIJJ),BASE(INJJ),BASE(IIPOS),
     3      BASE(ICHI),BASE(IZUFIS),BASE(IIND),BASE(IRCAT),BASE(IRCATI))
            CALL RLSARA(BASE(IRCATI))
            CALL RLSARA(BASE(IRCAT))
            CALL RLSARA(BASE(IIND))
            CALL RLSARA(BASE(IDSGD))
            CALL RLSARA(BASE(ISGD))
	 ENDIF
         CALL RLSARA(BASE(IZUFIS))
         CALL RLSARA(BASE(ICHI))
         CALL RLSARA(BASE(IIPOS))
         CALL RLSARA(BASE(INJJ))
         CALL RLSARA(BASE(IIJJ))
         CALL RLSARA(BASE(IWORK))
   50    CONTINUE
      ELSE
         CALL XABORT('TRIVAA: INVALID REQUEST.')
      ENDIF
*
   60 IF(IMPX.GE.3) CALL LCMLIB(IPSYS)
*----
*  RELEASE GENERAL TRACKING INFORMATION.
*----
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
