#!/usr/bin/perl
#####################################################################################
#   																				#
#   	2001-2002, Jim Melanson
#
#   	uploadformmailer.cgi	V3.3 [July 16th, 2002]
#
#       History: V3.0 [November 19th, 2001]
#                V3.1 [December 12th, 2001]
#                V3.2 [April 1st, 2002]
#                V3.3 [July 16th, 2002]
#
#   	A utility for processing form input to e-mail with file attachments
#
#   	jim@perlservices.com, jim@perlservices.net
#   	http://www.perlservices.net/
#
#       Use of this script is subject to the terms and conditions that are fully
#       available in the users guide at:
#
#       http://www.perlservices.net/en/programs/attachment_formmailer/users_guide.shtml
#
#####################################################################################
#
#   	SYSTEM REQUIREMENTS
#   	This script requires a UNIX server running Perl5.0 or higher
#   	and it requires the CGI module.
#
#####################################################################################
#
#   	REQUIRED INPUT PARAMETERS
#
#   	In this script, you must set three variables, you will find
#   	them below these instructions. Note that there are several variables
#       you can assign to customize the way this script responds, however,
#       the following three are the only REQUIRED variables
#
#   	$MailProgram = '/usr/sbin/sendmail';
#   		This is the absolute path to sendmail. It will be provided
#   		in your virtual hosts support/FAQ pages.
#
#   	$TempPath = '/usr/home/somename/path/to/folder';
#   		If you are going to use this script for file upload, you
#   		must create a directory on your server where the program
#   		can temporarily store them. The program has to upload the
#   		files first, then attach them to the e-mail. The program
#   		will then remove those files as it exits. Therefore, set
#   		the above path to the absolute path to the directory you
#   		create. Make sure you CHMOD the directory 777. Do NOT store
#   		anything else in this folder, it gets wiped every time the
#   		program is called.
#
#   	$AdminEmail = 'yourname@foo.com';
#   		If you do not wish to hide your e-mail address in the form,
#   		set the address you want the form delivered to in this
#   		variable. Note that if you specify the emailto field (see
#   		below) in the form, this will over ride the above setting.
#
#       THE REMAINDER OF THE OPTIONAL VARIABLES ARE ALL EXPLAINED IN DETAIL
#       IN THE VARIABLE ASSIGNMENT AREA OF THIS SCRIPT, APPROX. 100 LINES
#       FURTHER DOWN THE PAGE
#
#####################################################################################
#
#   	<input type="text" name="emailfrom" value="whatever the user enters">
#   	This is the field you would enter on your form if you would like to reply to your
#   	site's visitors. The script will check to see if the address is in a valid format.
#		If not, the script will show an error. If you are using this, remember to take 
#		out "emailto" form the required fields.
#
#   	HIDDEN VARIABLES TO SET IN YOUR FORM:
#   	(None of these are mandatory, they are all optional)
#
#   	<input type="hidden" name="emailto" value="yourname@yourdomain.com">
#   	Set this hidden field to the address that you want the form results
#   	and any files delivered to. If you do not specify this hidden field,
#   	then the $AdminEmail variable specified above will be the default.
#   	If you DO specify this hidden field, it's value will OVERRIDE the above
#   	specified $AdminEmail value. If you are using the "emailto" field,
#   	then don't forget to list the recipient in the $ValidRecipients variable.
#
#   	<input type="hidden" name="emailsubject" value="Subject Line Of Email">
#   	Specify this hidden field to supply the value that is placed in the
#   	subject line of the e-mail that gets sent with the form contents and any
#   	attached files. If you do not specify this value, then the subject line
#   	will be 'Form Submission Contents'.
#
#   	<input type="hidden" name="emailcc" value="someone@somewhere.com">
#   	If you specify this hidden field, you can send a copy of this forms
#   	submission to an additional e-mail address.If you are using the "emailcc"
#   	field, then don't forget to list the recipient in the $ValidRecipients variable.
#
#   	<input type="hidden" name="emailbcc" value="someone@somewhere.com">
#   	If you specify this hidden field, you can send a blind carbon copy of
#   	this forms submission to an additional e-mail address. By using the 'bcc'
#   	field, this e-mail address will not be visible to the original recipient.
#   	If you are using the "emailto" field, then don't forget to list the recipient 
#   	in the $ValidRecipients variable.
#
#   	<input type="hidden" name="requiredfields" value="Name,E-Mail,Favourite Colour,Phone">
#   	Use this hidden field to specify the fields that you want to be required. You must enter
#   	the names exactly as you entered them in the "name" attribute of the form field. Separate
#   	each name with a comma only, do not add any spaces in front of or behind the field name.
#
#       There are additional hidden field options, please see the section below in the
#       variable assignment area that refers to Redirect URL's.
#
#   	ATTACHING FILES TO FORM SUBMISSION:
#   	There are two things you must do to attache a file to a form submission.
#
#   	First, the opening form tag MUST contain an ENCTYPE attribute, copy it exactly
#   	from the following example:
#
#	<form method="POST" action="http://www.foo.com/cgi-bin/uploadformmailer.cgi" ENCTYPE="multipart/form-data">
#
#   	Next, you must add FILE type form fields. You can attach up to 10 files at a time.
#   	These form fields MUST be named "FILE1" through "FILE10". Not that these must be
#   	capital as they are case sensitive:
#
#   	<input type="file" name="FILE1">
#   	<input type="file" name="FILE2">
#   	<input type="file" name="FILE3">
#   	...
#   	<input type="file" name="FILE9">
#   	<input type="file" name="FILE10">
#
#####################################################################################
#
#   	A FEW WORDS ON UPLOADS
#   	The ability to successfully process attachments, which are in effect, uploads,
#   	depends on several factors. These factors are whether they are on dial up or
#   	high-speed access, speed of their connection, how busy it is in their area, the
#   	speed of the server etc. Also, take into account the fact that some servers limit
#   	the size of uploads that it will permit via HTTP. Also, some versions of IE and
#   	NN have a bug regarding these type of uploads. If the file on the users hard drive
#   	has a space in the name of any of the directories the file is in, the file will
#   	upload as zero bytes. Spaces in the files name are corrected by this script.
#
#   	The decision to accept file attachments via the form is yours and can be a very
#   	beneficial move. Keep in mind, however, that there will be occasional problems
#   	and errors. It is best that you include a field in your form asking the person
#   	to comment on the attached file(s) or give instructions for the attached file(s).
#   	This way you will be able to determine if there is a missing file when you
#   	receive the form submission with attachments.
#
#   	This program uses a Perl Module that has been placed in the script strictly as
#   	a package. The Package/Module is called "MongerMail" and you can get more
#   	information on it at:  http://wwww.perlservices.net/en/modules/
#
#       The first thing you should do after configuring and uploading this script is
#       call the debug function by appending ?debug to the URL when you call it in
#       your browser, like this:
#               http://www.yourdomain.com/cgi-bin/uploadformmailer.cgi?debug
#
#       This will help you in your installation. If you get an error when you call
#       this then most likely your path to Perl is incorrect of you did not CHMOD
#       the script 755.
#
#####################################################################################


# Path to sendmail on your UNIX server 
my $MailProgram = '/usr/sbin/sendmail';

# Path to any folder on your UNIX server
my $TempPath = "/tmp/uploadformmailer_temp";

# Administrators email address ( must be between single quotes )
my $AdminEmail = 'alain.hebert@polymtl.ca';

# Encoding for this program is base64. If the support departments advises you to
# switch to Uuencode encoding, then set this value to 1. Otherwise, leave it set
# to zero.
my $uuencode = 0;

# Maximum kilobyte size of any file # 1024 kilobytes = 1 mb 
# set this variable to 0 ( zero ) to use unlimited size. Eg. my $maxsize = 0;
# Please note that this program has successfully been used to deliver e-mails
# up to 21 Mb in size. If you exceed that, please let us know!
my $maxsize = 100;

# These are the hidden fields. Adding a hidden field name will prevent it from
#showing up in the email with the form submission. All fields that are not listed
#here WILL show up in the e-mail message.
my @hidden = qw~
subject 
redirect 
emailto 
emailsubject 
emailcc 
emailbcc 
redirect
redirect_required
redirect_oversize
redirect_invalid
redirect_nodata
requiredfields
~;


#To prevent your server being used as a spam tool, only e-mail addresses listed
#here will be able to receive e-mails from the program. List one e-mail address
#per line.
my $ValidRecipients = q~
elisabeth.varin@polymtl.ca
guy.marleau@polymtl.ca
robert.roy@polymtl.ca
daniel.rozon@polymtl.ca
~;

#The next two sections allow you to specify permitted file types that can
#be uploaded or file types that are not permitted to be uploaded. This works
#on precedence. What this means is that:
#        1.) If you specify valid file types, only files with any those extensions
#            will be attached.
#        2.) If you specify invalid file types, only files that do NOT have any
#            of those extensions will be attached.
#        3.) IF you specify both valid AND invalid file types, then only the valid
#            file types will be checked and the invalid file types will be ignored.
#        4.) IF you do not specify either of these, then all file types will be
#            attached.
#        NOTE: If your users will routinely be attaching files that have NO extension
#        then you must NOT specify any valid file types. This is common with several
#        file types generated on the Mac O/S.

#Enter the extensions of valid file types
my @Valid_File_types = qw~
~;

# Enter only unaccepted file types here. 
my @inValid_File_types = qw~
.com
.dll
.vbs
.exe
.cgi
.com  
.pl
~;

#If you would like to set some required fields, you can set them globally here
#or you can over-ride the global required fields with a hidden tag in the form
my @Required_Fields = qw~
emailto
~;

# To over-ride this in an individual form, set the required field names separated
# by a comman and NO spaces in the following format:
#
# <input type="hidden" name="requiredfields" value="FieldName1,Field_Name2,FieldName3">

#############################################################################
#
#       Redirect Pages:
#
#       This program allows you to specify a default redirect page based on
#       one of four submission results. Note that you can set these as global
#       defaults here but you can also over-ride the global setting by using
#       a hidden tag of the same name in your individual forms. A sample hidden
#       tag is given with each of these global redirects.
#
#       NOTE: To users of versions prior to V3.0, older versions using the
#       hidden tag "redirectbad", this has been renamed to "redirect_nodata" in
#       this version, please change your forms accordingly.
#

my $redirect = qq~~;
    #This is the global default value for a successful submission. Set it like this:
    #$redirect = qq~http://www.foo.com/path/to/page.html~;
    #
    #Hidden Tag to over-ride this setting:
    # <input type="hidden" name="redirect" value="http://www.foo.com/path/to/page.html">
    #
    # Don't forget the: http://



my $redirect_required = qq~~;
    #This is the global default value for a successful submission. Set it like this:
    #$redirect_required = qq~http://www.foo.com/path/to/page.html~;
    #
    #Hidden Tag to over-ride this setting:
    # <input type="hidden" name="redirect_required" value="http://www.foo.com/path/to/missing_info.html">
    #
    # Don't forget the: http://



my $redirect_nodata = qq~~;
    #This is the global default value for a form submission that failed because none
    #of the form fields contained any data. Set it like this:
    #$redirect_nodata = qq~http://www.foo.com/path/to/missing_info.html~;
    #
    #Hidden Tag to over-ride this setting:
    # <input type="hidden" name="redirect_nodata" value="http://www.foo.com/path/to/missing_info.html">
    #
    # Don't forget the: http://


my $redirect_oversize = qq~~;
    #This is the global default value for a form submission that failed because the user
    #attempted to attache a file that was too large. Set it like this:
    #$redirect_oversize = qq~http://www.foo.com/path/to/too_big.html~;
    #
    #Hidden Tag to over-ride this setting:
    # <input type="hidden" name="redirect_oversize" value="http://www.foo.com/path/to/too_big.html">
    #
    # Don't forget the: http://


my $redirect_invalid = qq~~;
    #This is the global default value for a form submission that failed because the user
    #attempted to attach a file of a type that is not permitted. Set it like this:
    #$redirect_invalid = qq~http://www.foo.com/path/to/invalid_type.html~;
    #
    #Hidden Tag to over-ride this setting:
    # <input type="hidden" name="redirect_invalid" value="http://www.foo.com/path/to/invalid_type.html">
    #
    # Don't forget the: http://

#
#############################################################################
#

# Please see the following URL for more info on accepted file types
# http://www.perlservices.net/en/programs/attachment_formmailer/users_guide.shtml

#####################################################################################
#
#                               DO NOT EDIT ANYTHING
#                                 BELOW THIS LINE
#
#
#####################################################################################

END {
    opendir(DELDIR, "$TempPath/$$");
    my @temp_files = grep {/.+\..+/} readdir DELDIR;
    closedir(DELDIR);
    foreach(@temp_files) {
        unlink("$TempPath/$$/$_");
    }
    rmdir("$TempPath/$$");
}

require 5;
use CGI;
#use strict;

my $req = new CGI;

my $tried_to_attach;
my $too_large;
my $bad_type;

#Add the admin e-mail address to the list of valid recipients
$ValidRecipients .= "\n$AdminEmail" if $AdminEmail;

#Turn the list of valid recipients into a hash.
my @ValidRecipients = split(/\n/, $ValidRecipients);
my %ValidRecipients = ();
foreach(@ValidRecipients) {
    $ValidRecipients{$_} = 1;
}

#Now we build a hash of excluded (hidden) fields
my %hidden;
foreach(@hidden) {
    $hidden{$_} = 1;
}

#Get the form input and validate that something was submitted.
my $GOOD_ENTRY;
my @params = $req->param;
foreach(@params) {
    if($req->param($_) =~ /[a-zA-Z1-9]/) {
        unless($hidden{$_}) {
            $GOOD_ENTRY = 1;
        }
    }
}

if(-d $TempPath) {
} else {
    mkdir("$TempPath", 0777);
}
if($ENV{'QUERY_STRING'} =~ /^debug/) {
    &debug_script;
}
elsif($GOOD_ENTRY) {
    #Now we check for required fields.
    my $MISSING = 0;
    my @requiredfields;
    if($req->param("requiredfields")) {
        @requiredfields = split(/,/, $req->param("requiredfields"));
    }
    elsif(@Required_Fields) {
        @requiredfields = @Required_Fields;
    }
    if(@requiredfields) {
        foreach(@requiredfields) {
            if($_ =~ /[a-zA-Z1-9]/) {
                unless($req->param($_) =~ /[a-zA-Z1-9]/) {
                    $MISSING = 1;
                }
            }
        }
    }
    if($MISSING) {
        if($req->param("redirect_required")) {
            my $goto = $req->param("redirect_required");
            print "Pragma: no-cache\nLocation: $goto\n\n";
        }
        elsif($redirect_required) {
            print "Pragma: no-cache\nLocation: $redirect_required\n\n";
        } else {
            print "Pragma: no-cache\nContent-type: text/html\n\n";
            print qq~<HEAD>
<TITLE>Missing Required Field</TITLE>
</HEAD>
<BODY bgcolor="#ffffff">
<BR><BR><BR>
<table align="center" width="500">
  <tr>
    <td>
      <FONT face="Arial" size="2" COLOR="#FF0000"><B>
      You forgot to fill in one or more required fields.<BR><BR>Please hit the back button on your browser and re-try.
      </B></FONT>
    </td>
  </tr>
</table>
</BODY></HTML>
            ~;
        }
    } else {
        #Set Default Subject if not supplied
        my $emailsubject = $req->param("emailsubject") || 'Form Submission Contents';

        #Set the send email to address to the default Admin Email address if no
        #"emailto" field was supplied.
        my $emailto  	= $req->param("emailto") || $AdminEmail;

        #If no "emailfrom" field, then default the email from to the email to value.
	my $emailfrom = $req->param("emailfrom");
	$emailfrom ||= $emailto;

        #Check and ensure the "emailto" address is a valid address in the list.	
        if($ValidRecipients{$emailto}) {
            #We are sending to a valid address.
            my @attach_files; my $tried_to_attach;
            my $ThisPath = $TempPath . "/$$";
            mkdir("$ThisPath", 0777);
            my $a;
            for($a = 1; $a <= 10; $a++) {
                if($req->param("FILE$a")) {
                    my $upload = psupload($ThisPath, "FILE$a", "", $maxsize);
                    if($upload) {
                        push(@attach_files, $upload);
                    }
                    $tried_to_attach = 1;
                }
            }
			
            #Check and see if any of the files were too large or of an invalid type
            my ($f, $too_large, $bad_type);
            for($f = 0; $f <=$#attach_files; $f++ ){
                if($attach_files[$f] == 7){
                    $too_large = 1;
                }
                if($attach_files[$f] == 8){
                    $bad_type = 1;
                }
            }			
            if(($tried_to_attach) && ($too_large || $bad_type)){
                #There was a problem, so print out a warning message and then
                #exit. This will force all files to be deleted and no email
                #message sent. Check for hidden fields for better customization

                #User attempted to attach a file that was too large
                if($too_large) {
                    if($req->param("redirect_oversize")) {
                        my $goto = $req->param("redirect_oversize");
                        print "Pragma: no-cache\nLocation: $goto\n\n";
                    }
                    elsif($redirect_oversize) {
                        print "Pragma: no-cache\nLocation: $redirect_oversize\n\n";
                    } else {
                        print "Pragma: no-cache\nContent-type: text/html\n\n";
			print qq~<HEAD><TITLE>Error</TITLE></HEAD><BODY bgcolor="#ffffff"><BR><BR><BR><table align="center" width="500"><tr><td align="center"><B><FONT face="Arial" size="2" COLOR="#000080"><H2>Error!</h2><BR>One or more of the files you tried to attach exceeds the maximum file<BR>size accepted. The size limit for each file is $maxsize kilobytes.<BR><BR><BR>Please press back the back button on your browser to select a smaller<BR>file and re-submit the form.</FONT></B></td></tr></table><BR><BR><BR><BR></BODY></HTML>~;
			exit;
                    }
                }

                #User attempted to attach a file that was of an invalid type.
                if($bad_type) {
                    if($req->param("redirect_invalid")) {
                        my $goto = $req->param("redirect_invalid");
                        print "Pragma: no-cache\nLocation: $goto\n\n";
                    }
                    elsif($redirect_invalid) {
                        print "Pragma: no-cache\nLocation: $redirect_invalid\n\n";
                    } else {
                        print "Pragma: no-cache\nContent-type: text/html\n\n";
                        print qq~<HEAD><TITLE>Error</TITLE></HEAD><BODY bgcolor="#ffffff"><BR><BR><BR><table align="center" width="500"><tr><td><B><FONT face="verdana" size="2" COLOR="#000080"><center><H2>Error!</h2><BR>One or more of the files you tried to attach does not<BR>match the types of files accepted.<BR><BR>~;
                        if(@Valid_File_types) {
                            print qq~Valid file types are : <br><br><font color="#000000">~, @Valid_File_types, qq~<BR><BR>~;
                        }
                        if(@inValid_File_types){
                            print qq~Invalid file types are : <br><br><font color="#000000">~, @inValid_File_types, qq~<BR><BR>~;
                        }
                        print qq~Please press back the back button on your browser to select a<BR>valid file type file and re-submit the form.<BR><BR></BODY></HTML>~;
                        exit;
                    }
                }
            } else {
                #Either there were no attached files or the attached files were okay
		my $mail = new MongerMail($MailProgram);
                $mail->to($emailto);
                #Check to see if we have to switch encoding types.
                if($uuencode) {
                    $mail->encoding('uuencode');
                }
                $mail->from($emailfrom);

                #Check for and compile CC email addresses
                if($req->param("emailcc")) {
                    my @cc_temp = split(/,/, $req->param("emailcc")) if $req->param("emailcc");
                    my @cc;
                    foreach(@cc_temp) {
                        if($ValidRecipients{$_}) {
                            push(@cc, $_);
                        }
                    }
                    $mail->cc(@cc);
                }
                #Check for and compile BCC email addresses
                if($req->param("emailbcc")) {
                    my @bcc_temp = split(/,/, $req->param("emailbcc")) if $req->param("emailbcc");
                    my @bcc;
                    foreach(@bcc_temp) {
                        if($ValidRecipients{$_}) {
                            push(@bcc, $_);
                        }
                    }
                    $mail->bcc(@bcc);
                }
                #Add the subject line
                $mail->subject($emailsubject);

                my $message = "\nHere are the contents of your form submission:\n\n";
                if(@attach_files) {
                    $message .= "There are " . ($#attach_files + 1) . " attached files.\n\n";
                }

                #First we need to sort the list of input fields:
                my @input_fields = sort @params;

                #Now we concantenate the email data
                foreach(@params) {
                    unless($hidden{$_}) {
                        my $value = $req->param($_);
                        if($_ =~ /^FILE(\d+)$/) {
                            $value =~ s/^.*(\\|\/)//;
                        }
                        $message .= "\[$_\] = " . $value . "\n";
                    }
                }
                $mail->content($message);
                $mail->attachment(@attach_files);
                $mail->send;
                #Mail has been sent. Let's print out the appropriate response page.
                if($req->param("redirect")) {
                    my $goto = $req->param("redirect");
                    print "Pragma: no-cache\nLocation: $goto\n\n";
                }
                elsif($redirect) {
                    print "Pragma: no-cache\nLocation: $redirect\n\n";
                } else {
                    print "Pragma: no-cache\nContent-type: text/html\n\n";
                    print qq~<HEAD><TITLE>Submission Received</TITLE></HEAD><BODY bgcolor="#ffffff"><BR><BR><BR><table align="center" width="500"><tr><td align="center"><B><FONT face="Arial" size="2" COLOR="#000080"><H2>Thank You!</h2><BR>Your submission has been received!</FONT></B></td></tr></table><BR><BR><BR><BR></BODY></HTML>~;
                }
            }
        } else {
            print "Pragma: no-cache\nContent-type: text/html\n\n";
            print qq~<HEAD>
<TITLE>Problem</TITLE></HEAD>
<BODY bgcolor="#ffffff">
<BR><BR><BR>
<center><table align="center" width="500">
  <tr>
    <td>
      <B><FONT face="Arial" size="2" COLOR="#FF0000">
uploadformmailer.cgi exited due to an invalid emailto address being submitted.<BR>
There is a couple possible causes for this:<BR><BR>
<OL>
<LI>There was no address embedded in the "emailto" field AND no \$AdminEmail address specified.<BR><BR>
<LI>The address specified in the "emailto" field is NOT in a list of valid e-mail addresses.<BR><BR>
</OL>
      </FONT></B>
    </td>
  </tr>
</table></center>
</BODY></HTML>
            ~;
            exit;
        }
    }
} else {
    #There was no input so print a warning message.
    if($req->param("redirect_nodata")) {
        my $goto = $req->param("redirect_nodata");
        print "Pragma: no-cache\nLocation: $goto\n\n";
    }
    elsif($redirect_nodata) {
        print "Pragma: no-cache\nLocation: $redirect_nodata\n\n";
    } else {
        print "Pragma: no-cache\nContent-type: text/html\n\n";
        print qq~<HEAD>
<TITLE>Missing Required Field</TITLE>
</HEAD>
<BODY bgcolor="#ffffff">
<BR><BR><BR>
<table align="center" width="500">
  <tr>
    <td>
      <FONT face="Arial" size="2" COLOR="#FF0000"><B>
      You did not fill in any information.<BR><BR>Please hit the back button on your browser and re-try.
      </B></FONT>
    </td>
  </tr>
</table>
</BODY></HTML>
        ~;
    }
}
exit;
	
sub psupload {
    my $path		= shift;
    my $inputfieldname	= shift;
    my $newname	 = shift;
    my $maxsize		= shift;
    my ($bad,$all,$type_ok);
    if($path) {
        if($inputfieldname) {
            my $ref = new CGI; 
            if($ref->param($inputfieldname)) {
                my $file = $ref->param($inputfieldname); 
                my $filename = $file; 
                $filename =~ s/^.*(\\|\/)//;
                $filename = $newname if $newname;
                $filename =~ s/ /\_/g;
                if(open(OUTFILE, ">$path/$filename")) {
                    while (my $bytesread = read($file, my $buffer, 1024)) { 
                        print OUTFILE $buffer; 
                    } 
                    close (OUTFILE);
					
                    #Now check for good types or bad types. We have
                    #these in precedence:
                    # 1.) If there are good file types listed,
                    #     we will compare the file types.
                    # 2.) If there are no good file types listed and
                    #     there are bad file types listed, then we
                    #     will checkthe bad file types.
                    # 3.) If there are both good and bad file types
                    #     listed, then we will only check the good
                    #     types.
                    # 4.) If there are no good and no bad file types
                    #     listed, then there will be no file type
                    #     check.

                    #Check for good file types:
                    if(@Valid_File_types) {
                        my $b; my $good_file = 0;
			for($b = 0; $b <= $#Valid_File_types; $b++){
                            if($Valid_File_types[$b] =~ /[a-zA-Z1-9]/) {
			        if($filename =~ /$Valid_File_types[$b]$/i){
				    $type_ok++;
                                    $good_file = 1;
                                    last;
                                }
                            }
                        }
                        unless($good_file) {
                            unlink("$path/$filename");
                            return(8);
                        }
                    }
                    #If there were no good types, then we'll check
                    #for bad file types.
                    elsif(@inValid_File_types) {
                        my $b; my $bad_file = 0;
			for($b = 0; $b <= $#inValid_File_types; $b++){
                            if($inValid_File_types[$b] =~ /[a-zA-Z1-9]/) {
				if($filename =~ /$inValid_File_types[$b]$/i){
				    $bad++;
                                    unlink("$path/$filename");
                                    return(8);
                                }
                            }
                        }
                    }
                    #If there were no good or bad types listed,
                    #then we have nothing to check, so proceed.

                    #Now we will check the size
                    if($maxsize > 0) {
                        if((-s "$path/$filename") > ($maxsize * 1024)) {
                            unlink("$path/$filename");
                            return(7);
			} else {
			    return("$path/$filename");
			}
		    } else {
			return("$path/$filename");
		    }
		} else {
		    return();
		}
	    } else {
		return();
	    }
	} else {
	    return();
	}
    } else {
	return();
    }
}

sub debug_script {
    print "Pragma: no-cache\nContent-type: text/html\n\n";
    print qq~<TITLE>Form Mailer with Attachments - Debug Mode</TITLE></HEAD>
<BODY BGCOLOR="#ffffff" TEXT="#330066" LINK="#336666" ALINK="#336666" VLINK="#336666" BGCOLOR="#FFFFFF"><FONT SIZE="2" FACE="verdana, arial, helvetica">
<CENTER><B><H2>Perl Services Form Mailer with Attachments Program</H2></B><BR><BR><TABLE BORDER=0><TR><TD COLSPAN=2><FONT SIZE="2" FACE="verdana,helvetica,arial">
<DL><DT><B>Your web root directory appears to be located at:</B><DD>$ENV{'DOCUMENT_ROOT'}<BR><BR>
<DT><B>You specified directory for storing the uploads is:</B>
<DD>$TempPath<BR><BR>
<DT><B>Your specified directory...</B><DD>
    ~;
    if(-d $TempPath) {
	print "...appears to be a valid directory.<BR><BR>Make sure this \$TempPath directory is CHMOD 777.\n";
    } else {
	print "...does not appear to be a valid directory.<BR><BR>\n";
	unless($TempPath =~ /^$ENV{'DOCUMENT_ROOT'}/) {
	    print "The value you specified in the \$TempPath variable is incorrect. Please<BR>correct your \$TempPath variable and run debug again.<BR><BR>\n";
	}
    }
    if($TempPath =~ /\/$/) {
	print "<FONT COLOR=\"#FF0000\">NOTE: Your variable \$TempPath ends with a trailing slash. Please<BR>remove this trailing slash, upload the program again<BR>and run debug once more to see if you have a valid directory.</FONT><BR><BR>\n";
    }
    unless(($AdminEmail =~ /\@/) || ($ValidRecipients =~ /\@/)) {
        print "<DT>Delivery Error<DD>You have not specified any valid recipients no an admin e-mail address. The form submission will NOT be processed.<BR><BR>\n";
    }
    print "</DL><BR><BR></FONT></TD></TR><TR><TD WIDTH=\"50%\" VALIGN=\"TOP\"><FONT SIZE=\"2\" FACE=\"verdana, arial, helvetica\"><B>OS:</B><BR>$^O<BR><BR><B>Perl:</B><BR>$]</FONT></TD><TD VALIGN=\"TOP\"><FONT SIZE=\"2\" FACE=\"verdana, arial, helvetica\"><B>Installed:</B><BR>"; my @inst = split(/\//, $ENV{'SERVER_SOFTWARE'}); print join("<BR>", @inst); print"</FONT></TD></TR></TABLE><BR><BR><BR><BR><A HREF=\"http://www.charityware.ws/\">&copy; 2001, Jim Melanson</A></CENTER><BR><BR></FONT></BODY></HTML>\n";

}

#################################################################################

package MongerMail;

my $VERSION     = ".02";

require 5;
use Carp;
use strict;

sub new {
    my ($pkg, $sendmail) = @_;
    if($sendmail =~ /\-t$/) {
	chop($sendmail); chop($sendmail);
    }
    $sendmail =~ s/ +//g;
    bless {
	sendmail => "$sendmail -t",
	error => '',
	to    => '',
	from  => '',
	cc    => '',
	bcc   => '',
	content => '',
        content_type => 'text/plain',
	no_erase   => 0,
	attachment => '',
	subject    => 'No subject',
	encoding   => 'base64',
	boundary   => 'Perl_Services_MongerMail-' . time,
	crlf	   => "\015\012",
    }, $pkg;
}

sub DESTROY {
    my $self = shift;
    for(keys %$self) {
	delete $self->{$_};
    }
    undef(%$self);
    undef $self;
}

sub attachment {
    my $self = shift;
    my @attachment = @_;
    foreach(@attachment) {
	$_ =~ s/ /\_/g;
	if(-e $_) {
	    if($_ =~ /.+\.\w+/) {
		if($self->{attachment}) {
		    $self->{attachment} .= ',';
		}
		$self->{attachment} .= $_;
	    }
	}
    }
    if($self->{attachment}) {
	return(1);
    } else {
	return(0);
    }
}

sub bcc {
    my $self = shift;
    my @bcc = @_;
    my($temp_bcc, $bad_bcc);
    foreach(@bcc) {
	if($_ =~ /[a-zA-Z1-9]/) {
	    if($self->_check_email($_)) {
		if($temp_bcc) {
		    $temp_bcc .= ',';
		}
		$temp_bcc .= $_;
	    } else {
		if($bad_bcc) {
		    $bad_bcc .= ', ';
		}
		$bad_bcc .= $_;
	    }
	}
    }
    if($bad_bcc) {
	$self->{error} = "You attempted to BCC this mailing to the following address which were found to be in an invalid format: $bad_bcc";
    }
    if($temp_bcc) {
	$self->{bcc} = $temp_bcc;
	return(1);
    } else {
	if($self->{error}) {$self->{error} .= "\n\n"}
 	$self->{error} .= "You did not provide any validly formatted e-mail address to BCC this mailing to.";
	return(0);
    }
}

sub boundary {
    my $self = shift;
    my $boundary = shift;
    if($boundary =~ /[a-zA-Z1-9]/) {
	$self->{boundary} = $boundary;
	return(1);
    } else {
	$self->{boundary} = 'Perl_Services_MongerMail-' . time;
	return(0);
    }
}

sub cc {
    my $self = shift;
    my @cc = @_;
    my($temp_cc, $bad_cc);
    foreach(@cc) {
	if($_ =~ /[a-zA-Z1-9]/) {
	    if($self->_check_email($_)) {
		if($temp_cc) {
		    $temp_cc .= ',';
		}
		$temp_cc .= $_;
	    } else {
		if($bad_cc) {
		    $bad_cc .= ', ';
		}
		$bad_cc .= $_;
	    }
	}
    }
    if($bad_cc) {
	$self->{error} = "You attempted to CC this mailing to the following address which were found to be in an invalid format: $bad_cc";
    }
    if($temp_cc) {
	$self->{cc} = $temp_cc;
	return(1);
    } else {
	if($self->{error}) {$self->{error} .= "\n\n"}
 	$self->{error} .= "You did not provide any validly formatted e-mail address to CC this mailing to.";
	return(0);
    }
}

sub content {
    my $self = shift;
    my @content = @_;
    my $content = join("\n", @content);
    $self->{content} = $content;
    return(1);
}

sub encoding {
    my $self = shift;
    my $encoding = shift;
    if(($encoding eq 'base64') || ($encoding eq 'uuencode')) {
	$self->{encoding} = $encoding;
	return(1);
    } else {
	$self->{error} = "You attempted to assign an invalid encoding format. The only valid encoding formats are \'uuencode\' and \'base64\'. The \'base64\' format is the default encoding format.";
	return(0);
    }
}

sub from {
    my $self = shift;
    my $from = shift;
    if($from =~ /[a-zA-Z1-9]/) {
	if($self->_check_email($from)) {
	    $self->{from} = $from;
	    return(1);
	} else {
	    $self->{error} = "The email address you attempted to assign to the \"From\" field was not in a valid e-mail address format.";
	    return(0);
	}
    } else {
	$self->{error} = "You attempted to assign a null value to the \"From\" field.";
	return(0);
    }
}

sub html {
    $_[0]->{content_type} = 'text/html';
}

sub no_erase {
    my $self = shift;
    $self->{no_erase} = 1;
}

sub no_html {
    $_[0]->{content_type} = 'text/plain';
}

sub subject {
    my $self = shift;
    my $subject = shift;
    if($subject =~ /[a-zA-Z1-9]/) {
	$self->{subject} = $subject;
    } else {
	$self->{subject} = 'No subject';
    }
    return(1);
}

sub send {
    my $self = shift;
    if($self->{sendmail}) {
	if($self->{to}) {
	    if($self->{from}) {
		if(open(SEND, "|$self->{sendmail}")) {
		    print SEND "To: $self->{to}$self->{crlf}";
		    print SEND "From: $self->{from}$self->{crlf}";
		    print SEND "Cc: $self->{cc}$self->{crlf}" if $self->{cc};
		    print SEND "Bcc: $self->{bcc}$self->{crlf}" if $self->{bcc};
		    print SEND "Subject: $self->{subject}$self->{crlf}";
		    if($self->{boundary}) {
			print SEND "x-sender: $self->{from}$self->{crlf}";
			print SEND "x-mailer: Perl Services$self->{crlf}";
			print SEND "Mime-Version: 1.0$self->{crlf}";
			print SEND "Content-Type: multipart/mixed; boundary=\"$self->{boundary}\"$self->{crlf}";
			print SEND "--$self->{boundary}$self->{crlf}";
			print SEND "Content-Type: $self->{content_type}; charset=\"US-ASCII\"$self->{crlf}$self->{crlf}";
		    } else {
                        print SEND "Content-Type: $self->{content_type}$self->{crlf}";
			print SEND $self->{crlf};
		    }
		    print SEND "$self->{content}\n\n" if $self->{content};
		    if($self->{attachment}) {
                        my @attachments;
			if($self->{attachment} =~ /\,/) {
			    @attachments = split(/\,/, $self->{attachment});
			} else {
			    $attachments[0] = $self->{attachment};
			}
			foreach(@attachments) {
			    my $filepath = $_;
			    $filepath =~ /[\\\/:]([^\\\/:]+)$/g;
			    my $filename = $1;
			    if (-e $filepath) {
				my $encoded_content;
				if($self->{encoding} eq 'base64') {
			            print SEND "--$self->{boundary}$self->{crlf}";
				    print SEND "Content-type: application/octet-stream; name=\"$filename\"$self->{crlf}";
				    print SEND "Content-transfer-encoding: base64$self->{crlf}$self->{crlf}";
	                            if(open(ATTACHMENT, "<$filepath")) {
                                        flock(ATTACHMENT, '3');
                                        my $line;
	                                while (my $bytes = read(ATTACHMENT, $line, 57)) {
		                            my $length += $bytes;
		                            my $return = substr(pack('u57', $line), 1);
		                            chop($return);
         	                            $return =~ tr| -_`|A-Za-z0-9+/A|;
	                                    my $buff = (3 - ($length % 3)) % 3;
	                                    substr($return, -$buff, $buff) = '=' x $buff;
                                            print SEND "$return\n";
	                                }
	                                close ATTACHMENT;
	                            }
                                    print SEND "$self->{crlf}";
				} else {
                                    print SEND "Attachment:\t$filename$self->{crlf}";
                                    print SEND "Encoding:\tUUEncoded$self->{crlf}";
				    print SEND "begin 600 $filename\n";
	                            if(open(ATTACHMENT, "<$filepath")) {
	                                flock(ATTACHMENT, '2');
                                        my $line;
	                                while(read(ATTACHMENT, $line, 57)) {
		                            my $return = pack("u57", $line);
                                            print SEND "$return\n";
	                                } 
	                                close(ATTACHMENT);
                                    }
				    print SEND "`\nend\n\n";
				}
			    } else {
				$self->{error} = "The file you wanted to attach does not exist. Check the file path.";
			    }
			}
		    }
		    if($self->{boundary}) {
   		        print SEND "--$self->{boundary}--$self->{crlf}";
                    }
    		    print SEND "$self->{crlf}.$self->{crlf}";
		    close(SEND);
		    unless($self->{no_erase} == 1) {
			$self->_erase;
		    }
		    return(1);
		} else {
		    $self->{error} = "Unable to open the mail program. Check the path to sendmail that you initialized MongerMail with.";
		    return(0);
		}
	    } else {
		$self->{error} = "You did not set a \"From\" address for the mailing. All mailings must have a validly formatted email address set with the \"from\" method.";
		return(0);
	    }
	} else {
	    $self->{error} = "You did not set a \"To\" address for the mailing. All mailings must have a validly formatted email address set with the \"to\" method.";
	    return(0);
	}
    } else {
	$self->{error} = "You did not specify an absolute path to sendmail. You must specify a path to sendmail when you first create the MongerMail object reference.";
        return(0);
    }
}

sub to {
    my $self = shift;
    my @to = @_;
    my ($temp_to, $bad_to);
    foreach(@to) {
	if($_ =~ /[a-zA-Z1-9]/) {
	    if($self->_check_email($_)) {
		if($temp_to) {
		    $temp_to .= ' ';
		}
		$temp_to .= $_;
	    } else {
		if($bad_to) {
		    $bad_to .= ', ';
		}
		$bad_to .= $_;
	    }
	}
    }
    if($bad_to) {
	$self->{error} = "You attempted to send this mailing to the following address which were found to be in an invalid format: $bad_to";
    }
    if($temp_to) {
	$self->{to} = $temp_to;
	return(1);
    } else {
	if($self->{error}) {$self->{error} .= "\n\n"}
 	$self->{error} .= "You did not provide any validly formatted e-mail address to send this mailing to.";
	return(0);
    }
}

#Internal methods only. Do not access directly.

sub _attachment_size {
    my $self = shift;
    my $file = shift;
    if(-e $file) {
	my $size = -s $file;
	return($size);
    } else {
	$self->{error} = "The file you specified does not exist";
	return(0);
    }
}

sub _attachment_type {
    my $self = shift;
    my $file = shift;
    if(-B $file) {
	return('binary');
    }
    elsif(-T $file) {
	return('text');
    } else {
	return(0);
    }
}

sub _check_email {
    my $self     = shift;
    my $fe_email = shift;
    if($fe_email =~ /[a-zA-Z1-9]/) {
        if(($fe_email =~ /(@.*@)|(\.\.)|(@\.)|(\.@)|(^\.)|(\.$)/) ||
	   ($fe_email !~ /^.+@\[?(\w|[-.])+\.[a-zA-Z]{2,3}|[0-9]{1,3}\]?$/)) {
	    return(0);
        } else {
	    return(1);
	}
    } else {
    	return(0);
    }
}

sub _erase {
    my $self = shift;
    $self->{to}    = '',
    $self->{from}  = '',
    $self->{cc}    = '',
    $self->{bcc}   = '',
    $self->{content}    = '',
    $self->{attachment} = '',
    $self->{subject}    = 'No subject',
}
